#!/usr/bin/perl
use XML::Simple;
use Data::Dumper;
binmode(STDOUT, ":utf8");

sub escape {
  #to escape a string with a single ' or " SQL demands to escape it via '' or "":
  my $string = shift;
  $string =~ s/\"/\"\"/g;
  $string =~ s/---/\-\-\-/g;
  return $string;
}

print '-- DATABASE SCHEME --
create table sets (
  setid INTEGER PRIMARY KEY AUTOINCREMENT,
  name TEXT NOT NULL, -- short name for the set --
  longname TEXT NOT NULL -- complete name for the set --
);

create table cards (
  cardid INTEGER PRIMARY KEY AUTOINCREMENT,
  name TEXT NOT NULL, -- every card has a name --
  color TEXT DEFAULT NULL, -- string containing WBRGU or NULL for colorless --
  manacost TEXT DEFAULT NULL, -- string like 0 or 3BB or NULL for lands --
  type TEXT DEFAULT NULL, -- every card has a type, except tokens --
  power TEXT DEFAULT NULL, -- power of a creature like *, 0, 1, 2, 3{1/2}, ... or NULL for noncreatures--
  toughness TEXT DEFAULT NULL, -- like power --
  cardtext TEXT DEFAULT "" -- cardtext, default empty for vanilla cards --
);

create table setlist (
  cardid INTEGER REFERENCES cards,
  setid INTEGER REFERENCES sets,
  lqurl TEXT DEFAULT "", -- picture URL, can be empty --
  hqurl TEXT DEFAULT "" -- dito --
);',"\n\n";

my $xml = new XML::Simple;
my $ref = $xml->XMLin("cards.xml");
#print Dumper($ref);

# read sets
print "-- SET DATA --\nBEGIN TRANSACTION;\n";
my %sets = %{$ref->{sets}->{set}};
my @keys = keys(%sets);
for my $set (@keys) {
  my $longname = escape($sets{$set}->{longname});
  print "INSERT INTO sets(name, longname) VALUES(\"$set\", \"$longname\");\n";
}
print "COMMIT TRANSACTION;\n\n";

#SQL commands to add cards to sets will be added to this empty string
my $setstring = "";

#read cards
my $cardid=1;
print "-- CARD DATA --\nBEGIN TRANSACTION;\n";
my %cards = %{$ref->{cards}->{card}};
my @keys = keys(%cards);
for my $card (@keys) {
  #print Dumper($cards{$card});
  my $printstring = "INSERT INTO cards(name,";
  my $values = "\"".escape($card)."\",";
  my @cardkeys = keys(%{$cards{$card}});

  #print "card = $card\n";

  for my $cardkey (@cardkeys) {
    #print "key = $cardkey\n";

    if($cardkey eq "set") {
      my $setid;
      my $lqurl;
      if(ref($cards{$card}{set}) eq "ARRAY") {
	# set is an ARRAY of HASHes (-> card with wultiple set properties)
	my @setarr = @{$cards{$card}{set}};
	for my $s (@setarr) {
	  $setid = "SELECT setid FROM sets WHERE sets.name = '$s->{content}'";
	  $lqurl = "\"$s->{picURL}\"";
	  $setstring .= "INSERT INTO setlist(cardid,setid,lqurl) VALUES($cardid,($setid),$lqurl);\n";
	}
      } else {
	# card with only 1 set property
	$setid = "SELECT setid FROM sets WHERE sets.name = '$cards{$card}{set}{content}'";
	$lqurl = "\"$cards{$card}{set}->{picURL}\"";
	$setstring .= "INSERT INTO setlist(cardid,setid,lqurl) VALUES($cardid,($setid),$lqurl);\n";
      }
      next;
    }
    if($cardkey eq "color") {
      $printstring .= "color,";
      if(ref($cards{$card}{color}) eq "ARRAY") {
	my $colstr = "";
	my @colarr = @{$cards{$card}{color}};
	for my $c (@colarr) {
	  $colstr .= $c;
	}
	$values .= "\"$colstr\",";
      } else {
	$values .= "\"$cards{$card}{color}\",";
      }
      next;
    }
    if($cardkey eq "manacost") {
      $printstring .= "manacost,";
      if(ref($cards{$card}{manacost}) eq "HASH") {
	$values .= "NULL,";
      } else {
	$values .= "\"$cards{$card}{manacost}\",";
      }
      next;
    }
    if($cardkey eq "type") {
      $printstring .= "type,";
      if(ref($cards{$card}{type}) eq "HASH") {
	# if code jumps here, type is empty, aka token card
	$values .= "NULL,";
      } else {
	$values .= "\"$cards{$card}{type}\",";
      }
      next;
    }
    if($cardkey eq "pt") {
      my @pt = split("\/", $cards{$card}{pt});
      my $p;
      my $t;
      if(scalar (@pt) == 2) {
	$p = "\"$pt[0]\"";
	$t = "\"$pt[1]\"";
      } elsif(scalar (@pt) == 3) {
	if($pt[1] =~ /.*\}/ && $pt[2] !~ /.*\}/) {
	  $p = "\"$pt[0]/$pt[1]\"";
	  $t = "\"$pt[2]\"";
	} else {
	  $p = "\"$pt[0]\"";
	  $t = "\"$pt[1]/$pt[2]\"";
	}
      } elsif(scalar (@pt) == 4) {
	$p = "\"$pt[0]/$pt[1]\"";
	$t = "\"$pt[2]/$pt[3]\"";
      }
      $printstring .= "power,toughness,";
      $values .= "$p,$t,";
      next;
    }
    if($cardkey eq "text") {
      $printstring .= "cardtext,";
      if(ref($cards{$card}{text}) eq "HASH") {
	# if code jumps here, cardtext is empty
	$values .= "NULL,";
      } else {
	$values .= "\"".escape($cards{$card}{text})."\",";
      }
      next;
    }
    if($cardkey eq "tablerow") {
      next;
    }
  }
  chop $printstring;
  chop $values;
  print "$printstring) VALUES($values);\n";
  $cardid++;
}
print "COMMIT TRANSACTION;\n\n-- CARD TO SET RELATION --\n";
print "BEGIN TRANSACTION;\n";
print "$setstring\n";
print "COMMIT TRANSACTION;\n";