-- DATABASE SCHEME --
create table sets (
  setid INTEGER PRIMARY KEY AUTOINCREMENT,
  name TEXT NOT NULL, -- short name for the set --
  longname TEXT NOT NULL -- complete name for the set --
);

create table cards (
  cardid INTEGER PRIMARY KEY AUTOINCREMENT,
  name TEXT NOT NULL, -- every card has a name --
  color TEXT DEFAULT NULL, -- string containing WBRGU or NULL for colorless --
  manacost TEXT DEFAULT NULL, -- string like 0 or 3BB or NULL for lands --
  type TEXT DEFAULT NULL, -- every card has a type, except tokens --
  power TEXT DEFAULT NULL, -- power of a creature like *, 0, 1, 2, 3{1/2}, ... or NULL for noncreatures--
  toughness TEXT DEFAULT NULL, -- like power --
  cardtext TEXT DEFAULT "" -- cardtext, default empty for vanilla cards --
);

create table setlist (
  cardid INTEGER REFERENCES cards,
  setid INTEGER REFERENCES sets,
  lqurl TEXT DEFAULT "", -- picture URL, can be empty --
  hqurl TEXT DEFAULT "" -- dito --
);

-- SET DATA --
BEGIN TRANSACTION;
INSERT INTO sets(name, longname) VALUES("P2", "Portal Second Age");
INSERT INTO sets(name, longname) VALUES("SOM", "Scars of Mirrodin");
INSERT INTO sets(name, longname) VALUES("DIS", "Dissension");
INSERT INTO sets(name, longname) VALUES("M10", "Magic 2010");
INSERT INTO sets(name, longname) VALUES("DK", "The Dark");
INSERT INTO sets(name, longname) VALUES("JU", "Judgment");
INSERT INTO sets(name, longname) VALUES("PCH", "Planechase");
INSERT INTO sets(name, longname) VALUES("P3", "Portal Three Kingdoms");
INSERT INTO sets(name, longname) VALUES("UNH", "Unhinged");
INSERT INTO sets(name, longname) VALUES("ST2K", "Starter 2000");
INSERT INTO sets(name, longname) VALUES("PR", "Promo set for Gatherer");
INSERT INTO sets(name, longname) VALUES("ME4", "Masters Edition IV");
INSERT INTO sets(name, longname) VALUES("V10", "From the Vault: Relics");
INSERT INTO sets(name, longname) VALUES("SH", "Stronghold");
INSERT INTO sets(name, longname) VALUES("7E", "Seventh Edition");
INSERT INTO sets(name, longname) VALUES("DKA", "Dark Ascension");
INSERT INTO sets(name, longname) VALUES("EX", "Exodus");
INSERT INTO sets(name, longname) VALUES("COM", "Magic: The Gathering-Commander");
INSERT INTO sets(name, longname) VALUES("M12", "Magic 2012");
INSERT INTO sets(name, longname) VALUES("OD", "Odyssey");
INSERT INTO sets(name, longname) VALUES("PD2", "Premium Deck Series: Fire and Lightning");
INSERT INTO sets(name, longname) VALUES("DD2", "Duel Decks: Jace vs. Chandra");
INSERT INTO sets(name, longname) VALUES("PLC", "Planar Chaos");
INSERT INTO sets(name, longname) VALUES("LRW", "Lorwyn");
INSERT INTO sets(name, longname) VALUES("UD", "Urza's Destiny");
INSERT INTO sets(name, longname) VALUES("FUT", "Future Sight");
INSERT INTO sets(name, longname) VALUES("ST", "Starter 1999");
INSERT INTO sets(name, longname) VALUES("SHM", "Shadowmoor");
INSERT INTO sets(name, longname) VALUES("AL", "Alliances");
INSERT INTO sets(name, longname) VALUES("WWK", "Worldwake");
INSERT INTO sets(name, longname) VALUES("TSP", "Time Spiral");
INSERT INTO sets(name, longname) VALUES("EVE", "Eventide");
INSERT INTO sets(name, longname) VALUES("DDG", "Duel Decks: Knights vs. Dragons");
INSERT INTO sets(name, longname) VALUES("TSB", "Time Spiral ""Timeshifted""");
INSERT INTO sets(name, longname) VALUES("PT", "Portal");
INSERT INTO sets(name, longname) VALUES("FE", "Fallen Empires");
INSERT INTO sets(name, longname) VALUES("LG", "Legends");
INSERT INTO sets(name, longname) VALUES("DDF", "Duel Decks: Elspeth vs. Tezzeret");
INSERT INTO sets(name, longname) VALUES("AP", "Apocalypse");
INSERT INTO sets(name, longname) VALUES("FVE", "From the Vault: Exiled");
INSERT INTO sets(name, longname) VALUES("ON", "Onslaught");
INSERT INTO sets(name, longname) VALUES("ME2", "Masters Edition II");
INSERT INTO sets(name, longname) VALUES("5E", "Fifth Edition");
INSERT INTO sets(name, longname) VALUES("AQ", "Antiquities");
INSERT INTO sets(name, longname) VALUES("AN", "Arabian Nights");
INSERT INTO sets(name, longname) VALUES("DDC", "Duel Decks: Divine vs. Demonic");
INSERT INTO sets(name, longname) VALUES("WL", "Weatherlight");
INSERT INTO sets(name, longname) VALUES("ARB", "Alara Reborn");
INSERT INTO sets(name, longname) VALUES("AE", "Archenemy");
INSERT INTO sets(name, longname) VALUES("LE", "Legions");
INSERT INTO sets(name, longname) VALUES("MBS", "Mirrodin Besieged");
INSERT INTO sets(name, longname) VALUES("H09", "Premium Deck Series: Slivers");
INSERT INTO sets(name, longname) VALUES("SC", "Scourge");
INSERT INTO sets(name, longname) VALUES("UL", "Urza's Legacy");
INSERT INTO sets(name, longname) VALUES("TE", "Tempest");
INSERT INTO sets(name, longname) VALUES("DDD", "Duel Decks: Garruk vs. Liliana");
INSERT INTO sets(name, longname) VALUES("4E", "Fourth Edition");
INSERT INTO sets(name, longname) VALUES("CHK", "Champions of Kamigawa");
INSERT INTO sets(name, longname) VALUES("MM", "Mercadian Masques");
INSERT INTO sets(name, longname) VALUES("NE", "Nemesis");
INSERT INTO sets(name, longname) VALUES("DRB", "From the Vault: Dragons");
INSERT INTO sets(name, longname) VALUES("TO", "Torment");
INSERT INTO sets(name, longname) VALUES("MR", "Mirrodin");
INSERT INTO sets(name, longname) VALUES("EVG", "Duel Decks: Elves vs. Goblins");
INSERT INTO sets(name, longname) VALUES("ROE", "Rise of the Eldrazi");
INSERT INTO sets(name, longname) VALUES("MOR", "Morningtide");
INSERT INTO sets(name, longname) VALUES("PY", "Prophecy");
INSERT INTO sets(name, longname) VALUES("MED", "Masters Edition");
INSERT INTO sets(name, longname) VALUES("BOK", "Betrayers of Kamigawa");
INSERT INTO sets(name, longname) VALUES("8E", "Eighth Edition");
INSERT INTO sets(name, longname) VALUES("B", "Limited Edition Beta");
INSERT INTO sets(name, longname) VALUES("CH", "Chronicles");
INSERT INTO sets(name, longname) VALUES("PD3", "Premium Deck Series: Graveborn");
INSERT INTO sets(name, longname) VALUES("ISD", "Innistrad");
INSERT INTO sets(name, longname) VALUES("VG", "Vanguard");
INSERT INTO sets(name, longname) VALUES("DDH", "Duel Decks: Ajani vs. Nicol Bolas");
INSERT INTO sets(name, longname) VALUES("ALA", "Shards of Alara");
INSERT INTO sets(name, longname) VALUES("US", "Urza's Saga");
INSERT INTO sets(name, longname) VALUES("NPH", "New Phyrexia");
INSERT INTO sets(name, longname) VALUES("10E", "Tenth Edition");
INSERT INTO sets(name, longname) VALUES("U", "Unlimited Edition");
INSERT INTO sets(name, longname) VALUES("AVR", "Avacyn Restored");
INSERT INTO sets(name, longname) VALUES("ZEN", "Zendikar");
INSERT INTO sets(name, longname) VALUES("RAV", "Ravnica: City of Guilds");
INSERT INTO sets(name, longname) VALUES("HL", "Homelands");
INSERT INTO sets(name, longname) VALUES("SOK", "Saviors of Kamigawa");
INSERT INTO sets(name, longname) VALUES("FVL", "From the Vault: Legends");
INSERT INTO sets(name, longname) VALUES("A", "Limited Edition Alpha");
INSERT INTO sets(name, longname) VALUES("IA", "Ice Age");
INSERT INTO sets(name, longname) VALUES("DDE", "Duel Decks: Phyrexia vs. the Coalition");
INSERT INTO sets(name, longname) VALUES("ME3", "Masters Edition III");
INSERT INTO sets(name, longname) VALUES("6E", "Classic Sixth Edition");
INSERT INTO sets(name, longname) VALUES("DS", "Darksteel");
INSERT INTO sets(name, longname) VALUES("MI", "Mirage");
INSERT INTO sets(name, longname) VALUES("IN", "Invasion");
INSERT INTO sets(name, longname) VALUES("VI", "Visions");
INSERT INTO sets(name, longname) VALUES("GP", "Guildpact");
INSERT INTO sets(name, longname) VALUES("9E", "Ninth Edition");
INSERT INTO sets(name, longname) VALUES("CS", "Coldsnap");
INSERT INTO sets(name, longname) VALUES("FD", "Fifth Dawn");
INSERT INTO sets(name, longname) VALUES("M11", "Magic 2011");
INSERT INTO sets(name, longname) VALUES("BR", "Battle Royale Box Set");
INSERT INTO sets(name, longname) VALUES("R", "Revised Edition");
INSERT INTO sets(name, longname) VALUES("PS", "Planeshift");
INSERT INTO sets(name, longname) VALUES("CFX", "Conflux");
INSERT INTO sets(name, longname) VALUES("BD", "Beatdown Box Set");
INSERT INTO sets(name, longname) VALUES("UG", "Unglued");
COMMIT TRANSACTION;

-- CARD DATA --
BEGIN TRANSACTION;
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Gargantua","4","4","B","When Phyrexian Gargantua enters the battlefield, you draw two cards and you lose 2 life.","4BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Multani's Presence","G","Whenever a spell you've cast is countered, draw a card.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Marshal","3","3","R","Echo {4}{R}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Goblin Marshal enters the battlefield or dies, put two 1/1 red Goblin creature tokens onto the battlefield.","4RR","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scrapheap","Whenever an artifact or enchantment is put into your graveyard from the battlefield, you gain 1 life.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desperate Ravings","R","Draw two cards, then discard a card at random.
Flashback {2}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Builder's Bane","R","Destroy X target artifacts. Builder's Bane deals damage to each player equal to the number of artifacts he or she controlled put into a graveyard this way.","XXR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rise // Fall","BR","Target player reveals two cards at random from his or her hand, then discards each nonland card revealed this way.
---
Return target creature card from a graveyard and target creature on the battlefield to their owners' hands.","BR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magma Giant","5","5","R","When Magma Giant enters the battlefield, it deals 2 damage to each creature and each player.","5RR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wildslayer Elves","3","3","G","Wither (This deals damage to creatures in the form of -1/-1 counters.)","3G","Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sacred Nectar","W","You gain 4 life.","1W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shieldmate's Blessing","W","Prevent the next 3 damage that would be dealt to target creature or player this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sky Spirit","2","2","WU","Flying, first strike","1WU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mortician Beetle","1","1","B","Whenever a player sacrifices a creature, you may put a +1/+1 counter on Mortician Beetle.","B","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elite Inquisitor","2","2","W","First strike, vigilance
Protection from Vampires, from Werewolves, and from Zombies","WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rayne, Academy Chancellor","1","1","U","Whenever you or a permanent you control becomes the target of a spell or ability an opponent controls, you may draw a card. You may draw an additional card if Rayne, Academy Chancellor is enchanted.","2U","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unyaro Bee Sting","G","Unyaro Bee Sting deals 2 damage to target creature or player.","3G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fool's Tome","{2}, {T}: Draw a card. Activate this ability only if you have no cards in hand.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thought Prison","Imprint - When Thought Prison enters the battlefield, you may have target player reveal his or her hand. If you do, choose a nonland card from it and exile that card.
Whenever a player casts a spell that shares a color or converted mana cost with the exiled card, Thought Prison deals 2 damage to that player.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brutal Nightstalker","3","2","B","When Brutal Nightstalker enters the battlefield, you may have target opponent discard a card.","3BB","Creature - Nightstalker");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Ingot","Darksteel Ingot is indestructible. (""Destroy"" effects and lethal damage don't destroy it.)
{T}: Add one mana of any color to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitting Slug","2","4","G","Whenever Spitting Slug blocks or becomes blocked, you may pay {1}{G}. If you do, Spitting Slug gains first strike until end of turn. Otherwise, each creature blocking or blocked by Spitting Slug gains first strike until end of turn.","1GG","Creature - Slug");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellrider","3","3","R","Haste
Whenever a creature you control attacks, Hellrider deals 1 damage to defending player.","2RR","Creature - Devil");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Warp","B","Look at target player's hand and choose X cards from it. That player discards those cards.","X3B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sacred Rites","W","Discard any number of cards. Creatures you control get +0/+1 until end of turn for each card discarded this way.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Rust","R","Choose one - Destroy target artifact; or destroy target land.
Entwine {3}{R} (Choose both if you pay the entwine cost.)","3RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sorin's Vengeance","B","Sorin's Vengeance deals 10 damage to target player and you gain 10 life.","4BBB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sound the Call","G","Put a 1/1 green Wolf creature token onto the battlefield. It has ""This creature gets +1/+1 for each card named Sound the Call in each graveyard.""","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whirlpool Rider","1","1","U","When Whirlpool Rider enters the battlefield, shuffle the cards from your hand into your library, then draw that many cards.","1U","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rest for the Weary","W","Target player gains 4 life.
Landfall - If you had a land enter the battlefield under your control this turn, that player gains 8 life instead.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Walk the Aeons","U","Buyback-Sacrifice three Islands. (You may sacrifice three Islands in addition to any other costs as you cast this spell. If you do, put this card into your hand as it resolves.)
Target player takes an extra turn after this one.","4UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Capashen Standard","W","Enchant creature
Enchanted creature gets +1/+1.
{2}, Sacrifice Capashen Standard: Draw a card.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boneyard Wurm","*","*","G","Boneyard Wurm's power and toughness are each equal to the number of creature cards in your graveyard.","1G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abyssal Nocturnus","2","2","B","Whenever an opponent discards a card, Abyssal Nocturnus gets +2/+2 and gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","1BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Preordain","U","Scry 2, then draw a card. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Somnomancer","2","1","WU","When Somnomancer enters the battlefield, you may tap target creature.","1(W/U)","Creature - Kithkin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Transluminant","2","2","G","{W}, Sacrifice Transluminant: Put a 1/1 white Spirit creature token with flying onto the battlefield at the beginning of the next end step.","1G","Creature - Dryad Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Library of Leng","You have no maximum hand size.
If an effect causes you to discard a card, discard it, but you may put it on top of your library instead of into your graveyard.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kobold Taskmaster","1","2","R","Other Kobold creatures you control get +1/+0.","1R","Creature - Kobold");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Basal Thrull","1","2","B","{T}, Sacrifice Basal Thrull: Add {B}{B} to your mana pool.","BB","Creature - Thrull");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aphetto Grifter","1","1","U","Tap two untapped Wizards you control: Tap target permanent.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cranial Extraction","B","Name a nonland card. Search target player's graveyard, hand, and library for all cards with that name and exile them. Then that player shuffles his or her library.","3B","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tome Scour","U","Target player puts the top five cards of his or her library into his or her graveyard.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soratami Mirror-Mage","2","1","U","Flying
{3}, Return three lands you control to their owner's hand: Return target creature to its owner's hand.","3U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lys Alana Bowmaster","2","2","G","Reach (This can block creatures with flying.)
Whenever you cast an Elf spell, you may have Lys Alana Bowmaster deal 2 damage to target creature with flying.","2G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Errant Doomsayers","1","1","W","{T}: Tap target creature with toughness 2 or less.","1W","Creature - Human Rebel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sheltered Valley","If Sheltered Valley would enter the battlefield, instead sacrifice each other permanent named Sheltered Valley you control, then put Sheltered Valley onto the battlefield.
At the beginning of your upkeep, if you control three or fewer lands, you gain 1 life.
{T}: Add {1} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tragic Poet","1","1","W","{T}, Sacrifice Tragic Poet: Return target enchantment card from your graveyard to your hand.","W","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit of Resistance","W","As long as you control a permanent of each color, prevent all damage that would be dealt to you.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snake Umbra","G","Enchant creature
Enchanted creature gets +1/+1 and has ""Whenever this creature deals damage to an opponent, you may draw a card.""
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kaijin of the Vanishing Touch","0","3","U","Defender (This creature can't attack.)
Whenever Kaijin of the Vanishing Touch blocks a creature, return that creature to its owner's hand at end of combat. (Return it only if it's on the battlefield.)","1U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shrewd Hatchling","6","6","UR","Shrewd Hatchling enters the battlefield with four -1/-1 counters on it.
{(u/r)}: Target creature can't block Shrewd Hatchling this turn.
Whenever you cast a blue spell, remove a -1/-1 counter from Shrewd Hatchling.
Whenever you cast a red spell, remove a -1/-1 counter from Shrewd Hatchling.","3(U/R)","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Electrostatic Bolt","R","Electrostatic Bolt deals 2 damage to target creature. If it's an artifact creature, Electrostatic Bolt deals 4 damage to it instead.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tower Geist","2","2","U","Flying
When Tower Geist enters the battlefield, look at the top two cards of your library. Put one of them into your hand and the other into your graveyard.","3U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lurking Crocodile","2","2","G","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
Islandwalk (This creature is unblockable as long as defending player controls an Island.)","2G","Creature - Crocodile");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("rathi Berserker","2","4","R","Rampage 3 (Whenever this creature becomes blocked, it gets +3/+3 until end of turn for each creature blocking it beyond the first.)","2RRR","Creature - Human Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Withered Wretch","2","2","B","{1}: Exile target card from a graveyard.","BB","Creature - Zombie Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quenchable Fire","R","Quenchable Fire deals 3 damage to target player. It deals an additional 3 damage to that player at the beginning of your next upkeep step unless he or she pays {U} before that step.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodland Druid","1","2","G",NULL,"G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forfend","W","Prevent all damage that would be dealt to creatures this turn.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lava Hounds","4","4","R","Haste
When Lava Hounds enters the battlefield, it deals 4 damage to you.","2RR","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plague Spitter","2","2","B","At the beginning of your upkeep, Plague Spitter deals 1 damage to each creature and each player.
When Plague Spitter dies, Plague Spitter deals 1 damage to each creature and each player.","2B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Denying Wind","U","Search target player's library for up to seven cards and exile them. Then that player shuffles his or her library.","7UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Nocturnus","3","3","B","Play with the top card of your library revealed.
As long as the top card of your library is black, Vampire Nocturnus and other Vampire creatures you control get +2/+1 and have flying.","1BBB","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jump","U","Target creature gains flying until end of turn.","U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Leonin Bladetrap","Flash
{2}, Sacrifice Leonin Bladetrap: Leonin Bladetrap deals 2 damage to each attacking creature without flying.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Moonsilver Spear","Equipped creature has first strike.
Whenever equipped creature attacks, put a 4/4 white Angel creature token with flying onto the battlefield.
Equip {4}","4","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Archmage","0","2","U","Whenever you cast an artifact spell, draw a card.","2UU","Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gristle Grinner","3","3","B","Whenever a creature dies, Gristle Grinner gets +2/+2 until end of turn.","4B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("War Dance","G","At the beginning of your upkeep, you may put a verse counter on War Dance.
Sacrifice War Dance: Target creature gets +X/+X until end of turn, where X is the number of verse counters on War Dance.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sage of Epityr","1","1","U","When Sage of Epityr enters the battlefield, look at the top four cards of your library, then put them back in any order.","U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thraximundar","6","6","UBR","Haste
Whenever Thraximundar attacks, defending player sacrifices a creature.
Whenever a player sacrifices a creature, you may put a +1/+1 counter on Thraximundar.","4UBR","Legendary Creature - Zombie Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Selenia, Dark Angel","3","3","WB","Flying
Pay 2 life: Return Selenia, Dark Angel to its owner's hand.","3WB","Legendary Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Star Compass","Star Compass enters the battlefield tapped.
{T}: Add to your mana pool one mana of any color that a basic land you control could produce.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crossbow Infantry","1","1","W","{T}: Crossbow Infantry deals 1 damage to target attacking or blocking creature.","1W","Creature - Human Soldier Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Imp's Mischief","B","Change the target of target spell with a single target. You lose life equal to that spell's converted mana cost.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Artillerize","R","As an additional cost to cast Artillerize, sacrifice an artifact or creature.
Artillerize deals 5 damage to target creature or player.","3R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mercadian Bazaar","Mercadian Bazaar enters the battlefield tapped.
{T}: Put a storage counter on Mercadian Bazaar.
{T}, Remove any number of storage counters from Mercadian Bazaar: Add {R} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Simic Initiate","0","0","G","Graft 1 (This creature enters the battlefield with a +1/+1 counter on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)","G","Creature - Human Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Underminer","1","1","R","Whenever Akki Underminer deals combat damage to a player, that player sacrifices a permanent.","3R","Creature - Goblin Rogue Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flare","R","Flare deals 1 damage to target creature or player.
Draw a card at the beginning of the next turn's upkeep.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Fracture","U","Return two target creatures to their owners' hands.
Cycling {2}{U} ({2}{U}, Discard this card: Draw a card.)","3UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Viridian Harvest","G","Enchant artifact
When enchanted artifact is put into a graveyard, you gain 6 life.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sky Ruin Drake","2","5","U","Flying","4U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forbid","U","Buyback-Discard two cards. (You may discard two cards in addition to any other costs as you cast this spell. If you do, put this card into your hand as it resolves.)
Counter target spell.","1UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azorius Guildmage","2","2","WU","{2}{W}: Tap target creature.
{2}{U}: Counter target activated ability. (Mana abilities can't be targeted.)","(W/U)(W/U)","Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Eagle","1","1","U","Flying","1U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Presence of the Master","W","Whenever a player casts an enchantment spell, counter it.","3W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Didgeridoo","{3}: You may put a Minotaur permanent card from your hand onto the battlefield.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cairn Wanderer","4","4","B","Changeling (This card is every creature type at all times.)
As long as a creature card with flying is in a graveyard, Cairn Wanderer has flying. The same is true for fear, first strike, double strike, deathtouch, haste, landwalk, lifelink, protection, reach, trample, shroud, and vigilance.","4B","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stalker Hag","3","2","BG","Swampwalk, forestwalk","(B/G)(B/G)(B/G)","Creature - Hag");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Twigwalker","2","2","G","{1}{G}, Sacrifice Twigwalker: Two target creatures each get +2/+2 until end of turn.","2G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crimson Manticore","2","2","R","Flying
{R}, {T}: Crimson Manticore deals 1 damage to target attacking or blocking creature.","2RR","Creature - Manticore");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinderbones","1","1","B","Wither (This deals damage to creatures in the form of -1/-1 counters.)
{1}{B}: Regenerate Cinderbones.","2B","Creature - Elemental Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vivify","G","Target land becomes a 3/3 creature until end of turn. It's still a land.
Draw a card.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quicken","U","The next sorcery card you cast this turn can be cast as though it had flash.
Draw a card.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Explosive Revelation","R","Choose target creature or player. Reveal cards from the top of your library until you reveal a nonland card. Explosive Revelation deals damage equal to that card's converted mana cost to that creature or player. Put the nonland card into your hand and the rest on the bottom of your library in any order.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moon Heron","3","2","U","Flying","3U","Creature - Spirit Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Champion of Lambholt","1","1","G","Creatures with power less than Champion of Lambholt's power can't block creatures you control.
Whenever another creature enters the battlefield under your control, put a +1/+1 counter on Champion of Lambholt.","1GG","Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jolt","U","You may tap or untap target artifact, creature, or land.
Draw a card at the beginning of the next turn's upkeep.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reiver Demon","6","6","B","Flying
When Reiver Demon enters the battlefield, if you cast it from your hand, destroy all nonartifact, nonblack creatures. They can't be regenerated.","4BBBB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avian Changeling","2","2","W","Changeling (This card is every creature type at all times.)
Flying","2W","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thought Hemorrhage","BR","Name a nonland card. Target player reveals his or her hand. Thought Hemorrhage deals 3 damage to that player for each card with that name revealed this way. Search that player's graveyard, hand, and library for all cards with that name and exile them. Then that player shuffles his or her library.","2BR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Mutation","UG","Return target creature to its owner's hand. Put X 1/1 green Saproling creature tokens onto the battlefield, where X is that creature's converted mana cost.","3GU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reverse the Sands","W","Redistribute any number of players' life totals. (Each of those players gets one life total back.)","6WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hulking Ogre","3","3","R","Hulking Ogre can't block.","2R","Creature - Ogre");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Off Balance","W","Target creature can't attack or block this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Coalflinger","2","2","R","First strike
{R}, {T}: Attacking creatures gain first strike until end of turn.","1RR","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Bloodstock","3","3","B","When Phyrexian Bloodstock leaves the battlefield, destroy target white creature. It can't be regenerated.","4B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warrior en-Kor","2","2","W","{0}: The next 1 damage that would be dealt to Warrior en-Kor this turn is dealt to target creature you control instead.","WW","Creature - Kor Warrior Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corrupt Court Official","1","1","B","When Corrupt Court Official enters the battlefield, target opponent discards a card.","1B","Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightning Angel","3","4","WUR","Flying, vigilance, haste","1RWU","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Matsu-Tribe Sniper","1","1","G","{T}: Matsu-Tribe Sniper deals 1 damage to target creature with flying.
Whenever Matsu-Tribe Sniper deals damage to a creature, tap that creature and it doesn't untap during its controller's next untap step.","1G","Creature - Snake Warrior Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Allosaurus Rider","1+*","1+*","G","You may exile two green cards from your hand rather than pay Allosaurus Rider's mana cost.
Allosaurus Rider's power and toughness are each equal to 1 plus the number of lands you control.","5GG","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Izzet Guildmage","2","2","UR","({(u/r)} can be paid with either {U} or {R}.)
{2}{U}: Copy target instant spell you control with converted mana cost 2 or less. You may choose new targets for the copy.
{2}{R}: Copy target sorcery spell you control with converted mana cost 2 or less. You may choose new targets for the copy.","(U/R)(U/R)","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thrull Wizard","1","1","B","{1}{B}: Counter target black spell unless that spell's controller pays {B} or {3}.","2B","Creature - Thrull Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Child of Gaea","7","7","G","Trample
At the beginning of your upkeep, sacrifice Child of Gaea unless you pay {G}{G}.
{1}{G}: Regenerate Child of Gaea.","3GGG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightshade Schemers","3","2","B","Flying
Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Nightshade Schemers, you may reveal it. If you do, each opponent loses 2 life.","4B","Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Save Life","W","Choose one Target player gains 2{1/2} life; or prevent the next 2{1/2} damage that would be dealt to target creature this turn.
Gotcha Whenever an opponent says ""Save"" or ""Life,"" you may say ""Gotcha"" If you do, return Save Life from your graveyard to your hand.","W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rackling","2","2","At the beginning of each opponent's upkeep, Rackling deals X damage to that player, where X is 3 minus the number of cards in his or her hand.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hagra Crocodile","3","1","B","Hagra Crocodile can't block.
Landfall - Whenever a land enters the battlefield under your control, Hagra Crocodile gets +2/+2 until end of turn.","3B","Creature - Crocodile");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fleeting Distraction","U","Target creature gets -1/-0 until end of turn.
Draw a card.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rend Flesh","B","Destroy target non-Spirit creature.","2B","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forked Lightning","R","Forked Lightning deals 4 damage divided as you choose among one, two, or three target creatures.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heedless One","*","*","G","Trample
Heedless One's power and toughness are each equal to the number of Elves on the battlefield.","3G","Creature - Elf Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thundering Wurm","4","4","G","When Thundering Wurm enters the battlefield, sacrifice it unless you discard a land card.","2G","Creature - Wurm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Magistrate's Scepter","{4}, {T}: Put a charge counter on Magistrate's Scepter.
{T}, Remove three charge counters from Magistrate's Scepter: Take an extra turn after this one.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Cavaliers","3","2","R",NULL,"2R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patron of the Moon","5","4","U","Moonfolk offering (You may cast this card any time you could cast an instant by sacrificing a Moonfolk and paying the difference in mana costs between this and the sacrificed Moonfolk. Mana cost includes color.)
Flying
{1}: Put up to two land cards from your hand onto the battlefield tapped.","5UU","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Illumination","W","Counter target artifact or enchantment spell. Its controller gains life equal to its converted mana cost.","WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oust","W","Put target creature into its owner's library second from the top. Its controller gains 3 life.","W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dead // Gone","R","Dead deals 2 damage to target creature.
---
Return target creature you don't control to its owner's hand.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("The Cheese Stands Alone","W","If you control no cards in play other than The Cheese Stands Alone and have no cards in your hand, you win the game.","4WW","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Core Prowler","2","2","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
When Core Prowler dies, proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","4","Artifact Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scepter of Insight","U","{3}{U}, {T}: Draw a card.","1UU","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reaper King","6","6","WUBRG","({(2/w)} can be paid with any two mana or with {W}. This card's converted mana cost is 10.)
Other Scarecrow creatures you control get +1/+1.
Whenever another Scarecrow enters the battlefield under your control, destroy target permanent.","(2/W)(2/U)(2/B)(2/R)(2/G)","Legendary Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kinsbaile Skirmisher","2","2","W","When Kinsbaile Skirmisher enters the battlefield, target creature gets +1/+1 until end of turn.","1W","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Where Ancients Tread","R","Whenever a creature with power 5 or greater enters the battlefield under your control, you may have Where Ancients Tread deal 5 damage to target creature or player.","4R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Cutthroat","1","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
{1}{B}, {T}: Destroy target creature with shadow.","1B","Creature - Dauthi Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gate Hound","1","1","W","Creatures you control have vigilance as long as Gate Hound is enchanted.","2W","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infested Roothold","0","3","G","Defender (This creature can't attack.)
Protection from artifacts
Whenever an opponent casts an artifact spell, you may put a 1/1 green Insect creature token onto the battlefield.","4G","Creature - Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Thraben Militia","5","4","Trample",NULL,"Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Now I Know My ABC's","U","At the beginning of your upkeep, if you control permanents with names that include all twenty-six letters of the English alphabet, you win the game.","1UU","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Walking Wall","0","6","Defender
{3}: Walking Wall gets +3/-1 until end of turn and can attack this turn as though it didn't have defender. Activate this ability only once each turn.","4","Artifact Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ceremonial Guard","3","4","R","When Ceremonial Guard attacks or blocks, destroy it at end of combat.","2R","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Snarling Undorak","3","3","G","{2}{G}: Target Beast creature gets +1/+1 until end of turn.
Morph {1}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silverchase Fox","2","2","W","{1}{W}, Sacrifice Silverchase Fox: Exile target enchantment.","1W","Creature - Fox");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Vatmother","4","5","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
At the beginning of your upkeep, you get a poison counter.","2BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anavolver","3","3","G","Kicker {1}{U} and/or {B} (You may pay an additional {1}{U} and/or {B} as you cast this spell.)
If Anavolver was kicked with its {1}{U} kicker, it enters the battlefield with two +1/+1 counters on it and with flying.
If Anavolver was kicked with its {B} kicker, it enters the battlefield with a +1/+1 counter on it and with ""Pay 3 life: Regenerate Anavolver.""","3G","Creature - Volver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lifted by Clouds","U","Target creature gains flying until end of turn.
Splice onto Arcane {1}{U} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","2U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wormfang Behemoth","5","5","U","When Wormfang Behemoth enters the battlefield, exile all cards from your hand.
When Wormfang Behemoth leaves the battlefield, return the exiled cards to their owner's hand.","3UU","Creature - Nightmare Fish Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Venerated Teacher","2","2","U","When Venerated Teacher enters the battlefield, put two level counters on each creature you control with level up.","2U","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gorgon Flail","Equipped creature gets +1/+1 and has deathtouch. (Any amount of damage it deals to a creature is enough to destroy that creature.)
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nezumi Ronin","3","1","B","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","2B","Creature - Rat Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kyren Legate","1","1","R","Haste
If an opponent controls a Plains and you control a Mountain, you may cast Kyren Legate without paying its mana cost.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blightspeaker","1","1","B","{T}: Target player loses 1 life.
{4}, {T}: Search your library for a Rebel permanent card with converted mana cost 3 or less and put it onto the battlefield. Then shuffle your library.","1B","Creature - Human Rebel Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jace's Ingenuity","U","Draw three cards.","3UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plague Rats","*","*","B","Plague Rats's power and toughness are each equal to the number of creatures named Plague Rats on the battlefield.","2B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screeching Griffin","2","2","W","Flying
{R}: Target creature can't block Screeching Griffin this turn.","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nimbus Wings","W","Enchant creature
Enchanted creature gets +1/+2 and has flying.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lava Axe","R","Lava Axe deals 5 damage to target player.","4R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ashling's Prerogative","R","As Ashling's Prerogative enters the battlefield, choose odd or even. (Zero is even.)
Each creature with converted mana cost of the chosen value has haste.
Each creature without converted mana cost of the chosen value enters the battlefield tapped.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sorry","U","Before playing any spell, if a copy of that spell card is in any graveyard, the spell's caster may say ""Sorry."" If he or she does not, any other player may counter the spell by saying ""Sorry"" as it is cast.
If any player says ""Sorry"" at any other time, Sorry deals 2 damage to that player.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kemba, Kha Regent","2","4","W","At the beginning of your upkeep, put a 2/2 white Cat creature token onto the battlefield for each Equipment attached to Kemba, Kha Regent.","1WW","Legendary Creature - Cat Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thraben Purebloods","3","5","W",NULL,"4W","Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Festival of Trokin","W","You gain 2 life for each creature you control.","W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infectious Rage","R","Enchant creature
Enchanted creature gets +2/-1.
When enchanted creature dies, choose a creature at random Infectious Rage can enchant. Return Infectious Rage to the battlefield attached to that creature.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Orcish Mine","R","Enchant land
Orcish Mine enters the battlefield with three ore counters on it.
At the beginning of your upkeep or whenever enchanted land becomes tapped, remove an ore counter from Orcish Mine.
When the last ore counter is removed from Orcish Mine, destroy enchanted land and Orcish Mine deals 2 damage to that land's controller.","1RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Sundering Titan","7","10","When Sundering Titan enters the battlefield or leaves the battlefield, choose a land of each basic land type, then destroy those lands.","8","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sprout Swarm","G","Convoke (Each creature you tap while casting this spell reduces its total cost by {1} or by one mana of that creature's color.)
Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Put a 1/1 green Saproling creature token onto the battlefield.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fires of Yavimaya","RG","Creatures you control have haste.
Sacrifice Fires of Yavimaya: Target creature gets +2/+2 until end of turn.","1RG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dread Return","B","Return target creature card from your graveyard to the battlefield.
Flashback-Sacrifice three creatures. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sleeping Potion","U","Enchant creature
When Sleeping Potion enters the battlefield, tap enchanted creature.
Enchanted creature doesn't untap during its controller's untap step.
When enchanted creature becomes the target of a spell or ability, sacrifice Sleeping Potion.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screeching Bat","2","2","B","Flying
At the beginning of your upkeep, you may pay {2}{B}{B}. If you do, transform Screeching Bat.","2B","Creature - Bat");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Cobalt Golem","2","3","{1}{U}: Cobalt Golem gains flying until end of turn.","4","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tempered Steel","W","Artifact creatures you control get +2/+2.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Outlander","2","2","UB","Protection from green","UB","Creature - Zombie Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Niblis of the Breath","2","1","U","Flying
{U}, {T}: You may tap or untap target creature.","2U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faerie Noble","1","2","G","Flying
Other Faerie creatures you control get +0/+1.
{T}: Other Faerie creatures you control get +1/+0 until end of turn.","2G","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Memory Plunder","UB","You may cast target instant or sorcery card from an opponent's graveyard without paying its mana cost.","(U/B)(U/B)(U/B)(U/B)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightscape Apprentice","1","1","B","{U}, {T}: Put target creature you control on top of its owner's library.
{R}, {T}: Target creature gains first strike until end of turn.","B","Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shriekgeist","1","1","U","Flying
Whenever Shriekgeist deals combat damage to a player, that player puts the top two cards of his or her library into his or her graveyard.","1U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Rager","2","2","B","When Phyrexian Rager enters the battlefield, you draw a card and you lose 1 life.","2B","Creature - Horror");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Elf Warrior","1","1",NULL,NULL,"Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vindicate","WB","Destroy target permanent.","1WB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Apostle's Blessing","W","({(w/p)} can be paid with either {W} or 2 life.)
Target artifact or creature you control gains protection from artifacts or from the color of your choice until end of turn.","1(W/P)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frantic Salvage","W","Put any number of target artifact cards from your graveyard on top of your library.
Draw a card.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lesser Werewolf","2","4","B","{B}: If Lesser Werewolf's power is 1 or more, it gets -1/-0 until end of turn and put a -0/-1 counter on target creature blocking or blocked by Lesser Werewolf. Activate this ability only during the declare blockers step.","3B","Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zephid","3","4","U","Flying; shroud (This permanent can't be the target of spells or abilities.)","4UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Farseer","1","1","W","Flying
Whenever a permanent is turned face up, put a +1/+1 counter on Aven Farseer.","1W","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tainted Pact","B","Exile the top card of your library. You may put that card into your hand unless it has the same name as another card exiled this way. Repeat this process until you put a card into your hand or you exile two cards with the same name, whichever comes first.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Copy Artifact","U","You may have Copy Artifact enter the battlefield as a copy of any artifact on the battlefield, except it's an enchantment in addition to its other types.","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirran Spy","1","3","U","Flying
Whenever you cast an artifact spell, you may untap target creature.","2U","Creature - Drone");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nectar Faerie","1","1","B","Flying
{B}, {T}: Target Faerie or Elf gains lifelink until end of turn. (Damage dealt by the creature also causes its controller to gain that much life.)","1B","Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shallow Grave","B","Return the top creature card of your graveyard to the battlefield. That creature gains haste until end of turn. Exile it at the beginning of the next end step.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Artifact Mutation","RG","Destroy target artifact. It can't be regenerated. Put X 1/1 green Saproling creature tokens onto the battlefield, where X is that artifact's converted mana cost.","RG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("School of Piranha","3","3","U","At the beginning of your upkeep, sacrifice School of Piranha unless you pay {1}{U}.","1U","Creature - Fish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ruin Ghost","1","1","W","{W}, {T}: Exile target land you control, then return it to the battlefield under your control.","1W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seton's Desire","G","Enchant creature
Enchanted creature gets +2/+2.
Threshold - As long as seven or more cards are in your graveyard, all creatures able to block enchanted creature do so.","2G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Escape Routes","U","{2}{U}: Return target white or black creature you control to its owner's hand.","2U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urborg","{T}: Add {B} to your mana pool.
{T}: Target creature loses first strike or swampwalk until end of turn.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vagrant Plowbeasts","6","6","WG","{1}: Regenerate target creature with power 5 or greater.","5GW","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Earth Servant","4","4","R","Earth Servant gets +0/+1 for each Mountain you control.","5R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rofellos, Llanowar Emissary","2","1","G","{T}: Add {G} to your mana pool for each Forest you control.","GG","Legendary Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of the White Orchid","2","2","W","First strike
When Knight of the White Orchid enters the battlefield, if an opponent controls more lands than you, you may search your library for a Plains card, put it onto the battlefield, then shuffle your library.","WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Remove Enchantments","W","Return to your hand all enchantments you both own and control, all Auras you own attached to permanents you control, and all Auras you own attached to attacking creatures your opponents control. Then destroy all other enchantments you control, all other Auras attached to permanents you control, and all other Auras attached to attacking creatures your opponents control.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harrowing Journey","B","Target player draws three cards and loses 3 life.","4B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Meddle","U","If target spell has only one target and that target is a creature, change that spell's target to another creature.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Golgari Brownscale","2","3","G","When Golgari Brownscale is put into your hand from your graveyard, you gain 2 life.
Dredge 2 (If you would draw a card, instead you may put exactly two cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","1GG","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Treetop Defense","G","Cast Treetop Defense only during the declare attackers step and only if you've been attacked this step.
Creatures you control gain reach until end of turn. (They can block creatures with flying.)","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Evil Twin","0","0","UB","You may have Evil Twin enter the battlefield as a copy of any creature on the battlefield except it gains ""{U}{B}, {T}: Destroy target creature with the same name as this creature.""","2UB","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwater Diver","1","1","U","{T}, Sacrifice Rootwater Diver: Return target artifact card from your graveyard to your hand.","U","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Savage Silhouette","G","Enchant creature
Enchanted creature gets +2/+2 and has ""{1}{G}: Regenerate this creature.""","2G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Imps' Taunt","B","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Target creature attacks this turn if able.","1B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Livewire Lash","Equipped creature gets +2/+0 and has ""Whenever this creature becomes the target of a spell, this creature deals 2 damage to target creature or player.""
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pandemonium","R","Whenever a creature enters the battlefield, that creature's controller may have it deal damage equal to its power to target creature or player of his or her choice.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Omega Myr","1","2",NULL,"2","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reverent Mantra","W","You may exile a white card from your hand rather than pay Reverent Mantra's mana cost.
Choose a color. All creatures gain protection from the chosen color until end of turn.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Drillmaster","2","2","R","{T}: Target creature gains haste until end of turn.","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spined Fluke","5","1","B","When Spined Fluke enters the battlefield, sacrifice a creature.
{B}: Regenerate Spined Fluke.","2B","Creature - Worm Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tower of Champions","{8}, {T}: Target creature gets +6/+6 until end of turn.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Songs of the Damned","B","Add {B} to your mana pool for each creature card in your graveyard.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Consecrated Sphinx","4","6","U","Flying
Whenever an opponent draws a card, you may draw two cards.","4UU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glittering Lion","2","2","W","Prevent all damage that would be dealt to Glittering Lion.
{3}: Until end of turn, Glittering Lion loses ""Prevent all damage that would be dealt to Glittering Lion."" Any player may activate this ability.","2W","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Death Baron","2","2","B","Skeleton creatures you control and other Zombie creatures you control get +1/+1 and have deathtouch.","1BB","Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skeletal Snake","2","1","B",NULL,"1B","Creature - Snake Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emperor Crocodile","5","5","G","When you control no other creatures, sacrifice Emperor Crocodile.","3G","Creature - Crocodile");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Snitch","1","1","U","Sacrifice Cephalid Snitch: Target creature loses protection from black until end of turn.","1U","Creature - Cephalid Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sign in Blood","B","Target player draws two cards and loses 2 life.","BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Piety","W","Blocking creatures get +0/+3 until end of turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Rimerunner","2","2","R","{T}: Target creature can't block this turn.
{S}i}: Goblin Rimerunner gains haste until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","2R","Snow Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Alarmist","1","2","R","Haste
{T}: Turn target face-down creature you control face up. At the beginning of the next end step, sacrifice it.","1R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kirtar's Wrath","W","Destroy all creatures. They can't be regenerated.
Threshold - If seven or more cards are in your graveyard, instead destroy all creatures, then put two 1/1 white Spirit creature tokens with flying onto the battlefield. Creatures destroyed this way can't be regenerated.","4WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opal Acrolith","W","Whenever an opponent casts a creature spell, if Opal Acrolith is an enchantment, Opal Acrolith becomes a 2/4 Soldier creature.
{0}: Opal Acrolith becomes an enchantment.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crashing Boars","4","4","G","Whenever Crashing Boars attacks, defending player chooses an untapped creature he or she controls. That creature blocks Crashing Boars this turn if able.","3GG","Creature - Boar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boggart Forager","1","1","R","{R}, Sacrifice Boggart Forager: Target player shuffles his or her library.","R","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bounty Hunter","2","2","B","{T}: Put a bounty counter on target nonblack creature.
{T}: Destroy target creature with a bounty counter on it.","2BB","Creature - Human Archer Minion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grixis","Blue, black, and/or red creature cards in your graveyard have unearth. The unearth cost is equal to the card's mana cost. (Pay the card's mana cost: Return it to the battlefield. The creature gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)
Whenever you roll {C}, put target creature card from a graveyard onto the battlefield under your control.",NULL,"Plane - Alara");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seaside Citadel","Seaside Citadel enters the battlefield tapped.
{T}: Add {G}, {W}, or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Protean Hulk","6","6","G","When Protean Hulk dies, search your library for any number of creature cards with total converted mana cost 6 or less and put them onto the battlefield. Then shuffle your library.","5GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jackal Familiar","2","2","R","Jackal Familiar can't attack or block alone.","R","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kinscaer Harpoonist","2","2","U","Flying
Whenever Kinscaer Harpoonist attacks, you may have target creature lose flying until end of turn.","3U","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dying Wail","B","Enchant creature
When enchanted creature dies, target player discards two cards.","1B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Krosa","All creatures get +2/+2.
Whenever you roll {C}, you may add {W}{U}{B}{R}{G} to your mana pool.",NULL,"Plane - Dominaria");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yamabushi's Flame","R","Yamabushi's Flame deals 3 damage to target creature or player. If a creature dealt damage this way would die this turn, exile it instead.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master Thief","2","2","U","When Master Thief enters the battlefield, gain control of target artifact for as long as you control Master Thief.","2UU","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeneye Aven","2","3","U","Flying
Cycling {2} ({2}, Discard this card: Draw a card.)","3U","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vintara Elephant","4","3","G","Trample
{3}: Vintara Elephant loses trample until end of turn. Any player may activate this ability.","4G","Creature - Elephant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Haze of Rage","R","Buyback {2} (You may pay an additional {2} as you cast this spell. If you do, put this card into your hand as it resolves.)
Creatures you control get +1/+0 until end of turn.
Storm (When you cast this spell, copy it for each spell cast before it this turn.)","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repulse","U","Return target creature to its owner's hand.
Draw a card.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consult the Necrosages","UB","Choose one - Target player draws two cards; or target player discards two cards.","1UB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sleight of Hand","U","Look at the top two cards of your library. Put one of them into your hand and the other on the bottom of your library.","U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dark Depths","Dark Depths enters the battlefield with ten ice counters on it.
{3}: Remove an ice counter from Dark Depths.
When Dark Depths has no ice counters on it, sacrifice it. If you do, put a legendary 20/20 black Avatar creature token with flying and ""This creature is indestructible"" named Marit Lage onto the battlefield.",NULL,"Legendary Snow Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maralen of the Mornsong","2","3","B","Players can't draw cards.
At the beginning of each player's draw step, that player loses 3 life, searches his or her library for a card, puts it into his or her hand, then shuffles his or her library.","1BB","Legendary Creature - Elf Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ghoulcaller's Bell","{T}: Each player puts the top card of his or her library into his or her graveyard.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stoic Ephemera","5","5","W","Defender (This creature can't attack.)
Flying
When Stoic Ephemera blocks, sacrifice it at end of combat.","2W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Fisher","2","2","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
When Aven Fisher dies, you may draw a card.","3U","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lance","W","Enchant creature
Enchanted creature has first strike.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burgeoning","G","Whenever an opponent plays a land, you may put a land card from your hand onto the battlefield.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Protection: Shadow","W","{1}: The next time a creature of your choice with shadow would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fire Ambush","R","Fire Ambush deals 3 damage to target creature or player.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boros Swiftblade","1","2","WR","Double strike","RW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Hydra","0","0","Clockwork Hydra enters the battlefield with four +1/+1 counters on it.
Whenever Clockwork Hydra attacks or blocks, remove a +1/+1 counter from it. If you do, Clockwork Hydra deals 1 damage to target creature or player.
{T}: Put a +1/+1 counter on Clockwork Hydra.","5","Artifact Creature - Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pentarch Ward","W","Enchant creature
As Pentarch Ward enters the battlefield, choose a color.
When Pentarch Ward enters the battlefield, draw a card.
Enchanted creature has protection from the chosen color. This effect doesn't remove Pentarch Ward.","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Amugaba","6","6","U","Flying
{2}{U}, Discard a card: Return Amugaba to its owner's hand.","5UU","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chain of Plasma","R","Chain of Plasma deals 3 damage to target creature or player. Then that player or that creature's controller may discard a card. If the player does, he or she may copy this spell and may choose a new target for that copy.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boomerang","U","Return target permanent to its owner's hand.","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("King's Assassin","1","1","B","{T}: Destroy target tapped creature. Activate this ability only during your turn, before attackers are declared.","1BB","Creature - Human Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Light","3","3","W","Flying, vigilance","4W","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("River of Tears","{T}: Add {U} to your mana pool. If you played a land this turn, add {B} to your mana pool instead.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Neurok Hoversail","Equipped creature has flying.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ivy Elemental","0","0","G","Ivy Elemental enters the battlefield with X +1/+1 counters on it.","XG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exclude","U","Counter target creature spell.
Draw a card.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Helix","WR","Lightning Helix deals 3 damage to target creature or player and you gain 3 life.","RW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Windseeker Centaur","2","2","R","Vigilance","1RR","Creature - Centaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Somberwald Dryad","2","2","G","Forestwalk (This creature is unblockable as long as defending player controls a Forest.)","1G","Creature - Dryad");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ruby Medallion","Red spells you cast cost {1} less to cast.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vital Splicer","1","1","G","When Vital Splicer enters the battlefield, put a 3/3 colorless Golem artifact creature token onto the battlefield.
{1}: Regenerate target Golem you control.","3G","Creature - Human Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stave Off","W","Target creature gains protection from the color of your choice until end of turn. (It can't be blocked, targeted, dealt damage, or enchanted by anything of that color.)","W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Chainbreaker","3","3","Chainbreaker enters the battlefield with two -1/-1 counters on it.
{3}, {T}: Remove a -1/-1 counter from target creature.","2","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pride of Lions","4","4","G","You may have Pride of Lions assign its combat damage as though it weren't blocked.","3GG","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gempalm Incinerator","2","1","R","Cycling {1}{R} ({1}{R}, Discard this card: Draw a card.)
When you cycle Gempalm Incinerator, you may have it deal X damage to target creature, where X is the number of Goblins on the battlefield.","2R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sensory Deprivation","U","Enchant creature
Enchanted creature gets -3/-0.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nacatl War-Pride","3","3","G","Nacatl War-Pride must be blocked by exactly one creature if able.
Whenever Nacatl War-Pride attacks, put X tokens that are copies of Nacatl War-Pride onto the battlefield tapped and attacking, where X is the number of creatures defending player controls. Exile the tokens at the beginning of the next end step.","3GGG","Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Temporal Adept","1","1","U","{U}{U}{U}, {T}: Return target permanent to its owner's hand.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dead Reckoning","B","You may put target creature card from your graveyard on top of your library. If you do, Dead Reckoning deals damage equal to that card's power to target creature.","1BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmal Dragon","5","5","U","Flying
When Phantasmal Dragon becomes the target of a spell or ability, sacrifice it.","2UU","Creature - Dragon Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nourishing Shoal","G","You may exile a green card with converted mana cost X from your hand rather than pay Nourishing Shoal's mana cost.
You gain X life.","XGG","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Screams from Within","B","Enchant creature
Enchanted creature gets -1/-1.
When enchanted creature dies, return Screams from Within from your graveyard to the battlefield.","1BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urabrask the Hidden","4","4","R","Creatures you control have haste.
Creatures your opponents control enter the battlefield tapped.","3RR","Legendary Creature - Praetor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Fanboy","1","1","B","As Zombie Fanboy comes into play, choose an artist.
Whenever a permanent by the chosen artist is put into a graveyard, put two +1/+1 counters on Zombie Fanboy.","2B","Creature - Zombie Gamer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gempalm Sorcerer","2","2","U","Cycling {2}{U} ({2}{U}, Discard this card: Draw a card.)
When you cycle Gempalm Sorcerer, Wizard creatures gain flying until end of turn.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Caress","B","Target player loses 1 life and you gain 1 life.
Draw a card.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lionheart Maverick","1","1","W","Vigilance
{4}{W}: Lionheart Maverick gets +1/+2 until end of turn.","W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning Inquiry","R","Each player draws three cards, then discards three cards at random.","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Withdraw","U","Return target creature to its owner's hand. Then return another target creature to its owner's hand unless its controller pays {1}.","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bitterheart Witch","1","2","B","Deathtouch
When Bitterheart Witch dies, you may search your library for a Curse card, put it onto the battlefield attached to target player, then shuffle your library.","4B","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Amok","R","{1}, Discard a card at random: Put a +1/+1 counter on target creature.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Natural End","G","Destroy target artifact or enchantment. You gain 3 life.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slingbow Trap","G","If a black creature with flying is attacking, you may pay {G} rather than pay Slingbow Trap's mana cost.
Destroy target attacking creature with flying.","3G","Instant - Trap");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mise","U","Name a nonland card, then reveal the top card of your library. If that card is the named card, draw three cards.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Convolute","U","Counter target spell unless its controller pays {4}.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scent of Ivy","G","Reveal any number of green cards in your hand. Target creature gets +X/+X until end of turn, where X is the number of cards revealed this way.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Butcher's Cleaver","Equipped creature gets +3/+0.
As long as equipped creature is a Human, it has lifelink.
Equip {3}","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peace Talks","W","This turn and next turn, creatures can't attack, and players and permanents can't be the targets of spells or activated abilities.","1W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spider Climb","G","You may cast Spider Climb as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Enchant creature
Enchanted creature gets +0/+3 and has reach. (It can block creatures with flying.)","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cathartic Adept","1","1","U","{T}: Target player puts the top card of his or her library into his or her graveyard.","U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wei Ambush Force","1","1","B","Whenever Wei Ambush Force attacks, it gets +2/+0 until end of turn.","1B","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squadron Hawk","1","1","W","Flying
When Squadron Hawk enters the battlefield, you may search your library for up to three cards named Squadron Hawk, reveal them, put them into your hand, then shuffle your library.","1W","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glyph of Life","W","Until end of turn, whenever an attacking creature deals damage to target Wall creature, you gain that much life.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twiddle","U","You may tap or untap target artifact, creature, or land.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Meddling Mage","2","2","WU","As Meddling Mage enters the battlefield, name a nonland card.
The named card can't be cast.","WU","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gnarled Effigy","{4}, {T}: Put a -1/-1 counter on target creature.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Triclopean Sight","W","Flash
Enchant creature
When Triclopean Sight enters the battlefield, untap enchanted creature.
Enchanted creature gets +1/+1 and has vigilance.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Censorship","U","When Censorship comes into play, choose a CENSORED word.
Whenever any CENSORED player says the chosen CENSORED word, Censorship deals 2 CENSORED damage to him or her.","U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Haze Frog","2","1","G","Flash (You may cast this spell any time you could cast an instant.)
When Haze Frog enters the battlefield, prevent all combat damage that other creatures would deal this turn.","3GG","Creature - Frog");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grappling Hook","Equipped creature has double strike.
Whenever equipped creature attacks, you may have target creature block it this turn if able.
Equip {4}","4","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screeching Harpy","2","2","B","Flying
{1}{B}: Regenerate Screeching Harpy.","2BB","Creature - Harpy Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Freewind Equenaut","2","2","W","Flying
As long as Freewind Equenaut is enchanted, it has ""{T}: Freewind Equenaut deals 2 damage to target attacking or blocking creature.""","2W","Creature - Human Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Visara the Dreadful","5","5","B","Flying
{T}: Destroy target creature. It can't be regenerated.","3BBB","Legendary Creature - Gorgon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Strip Mine","{T}: Add {1} to your mana pool.
{T}, Sacrifice Strip Mine: Destroy target land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Demon's Jester","2","2","B","Flying
Hellbent - Demon's Jester gets +2/+1 as long as you have no cards in hand.","3B","Creature - Imp");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jandor's Saddlebags","{3}, {T}: Untap target creature.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Long-Forgotten Gohei","Arcane spells you cast cost {1} less to cast.
Spirit creatures you control get +1/+1.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wretched Anurid","3","3","B","Whenever another creature enters the battlefield, you lose 1 life.","1B","Creature - Zombie Frog Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shinen of Stars' Light","2","1","W","First strike
Channel - {1}{W}, Discard Shinen of Stars' Light: Target creature gains first strike until end of turn.","2W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drastic Revelation","UBR","Discard your hand. Draw seven cards, then discard three cards at random.","2UBR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Isolation","W","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature has shadow. (It can block or be blocked by only creatures with shadow.)
Prevent all damage that would be dealt by enchanted creature.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bull Hippo","3","3","G","Islandwalk","3G","Creature - Hippo");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mask of the Mimic","U","As an additional cost to cast Mask of the Mimic, sacrifice a creature.
Search your library for a card with the same name as target nontoken creature and put that card onto the battlefield. Then shuffle your library.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gatekeeper of Malakir","2","2","B","Kicker {B} (You may pay an additional {B} as you cast this spell.)
When Gatekeeper of Malakir enters the battlefield, if it was kicked, target player sacrifices a creature.","BB","Creature - Vampire Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Moonvessel","1","1","When Myr Moonvessel dies, add {1} to your mana pool.","1","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rampant Elephant","2","2","W","{G}: Target creature blocks Rampant Elephant this turn if able.","3W","Creature - Elephant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Refresh","G","Regenerate target creature.
Draw a card.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gang of Elk","5","4","G","Whenever Gang of Elk becomes blocked, it gets +2/+2 until end of turn for each creature blocking it.","5G","Creature - Elk Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Umbra Stalker","*","*","B","Chroma - Umbra Stalker's power and toughness are each equal to the number of black mana symbols in the mana costs of cards in your graveyard.","4BBB","Creature - Elemental");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Viseling","2","2","At the beginning of each opponent's upkeep, Viseling deals X damage to that player, where X is the number of cards in his or her hand minus 4.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wingbeat Warrior","2","1","W","Flying
Morph {2}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Wingbeat Warrior is turned face up, target creature gains first strike until end of turn.","2W","Creature - Bird Soldier Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flowering Field","W","Enchant land
Enchanted land has ""{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.""","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lifelace","G","Target spell or permanent becomes green. (Mana symbols on that permanent remain unchanged.)","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Bladewarden","1","1","W","{T}: Target creature gets +X/+X until end of turn, where X is Auriok Bladewarden's power.","1W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Cocoon","G","Enchant creature
At the beginning of your upkeep, put a +1/+1 counter on enchanted creature.
When enchanted creature attacks or blocks, sacrifice Primal Cocoon.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kurgadon","3","3","G","Whenever you cast a creature spell with converted mana cost 6 or greater, put three +1/+1 counters on Kurgadon.","4G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wind Sail","U","One or two target creatures gain flying until end of turn.","1U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cumber Stone","U","Creatures your opponents control get -1/-0.","3U","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Falkenrath Marauders","2","2","R","Flying, haste
Whenever Falkenrath Marauders deals combat damage to a player, put two +1/+1 counters on it.","3RR","Creature - Vampire Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorgon Recluse","2","4","B","Whenever Gorgon Recluse blocks or becomes blocked by a nonblack creature, destroy that creature at end of combat.
Madness {B}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","3BB","Creature - Gorgon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scion of Darkness","6","6","B","Trample
Whenever Scion of Darkness deals combat damage to a player, you may put target creature card from that player's graveyard onto the battlefield under your control.
Cycling {3} ({3}, Discard this card: Draw a card.)","5BBB","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragon Roost","R","{5}{R}{R}: Put a 5/5 red Dragon creature token with flying onto the battlefield. (It can't be blocked except by creatures with flying or reach.)","4RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gate to Phyrexia","B","Sacrifice a creature: Destroy target artifact. Activate this ability only during your upkeep and only once each turn.","BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treefolk Harbinger","0","3","G","When Treefolk Harbinger enters the battlefield, you may search your library for a Treefolk or Forest card, reveal it, then shuffle your library and put that card on top of it.","G","Creature - Treefolk Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reap the Seagraf","B","Put a 2/2 black Zombie creature token onto the battlefield.
Flashback {4}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Obelisk of Undoing","{6}, {T}: Return target permanent you both own and control to your hand.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog-Strider Ash","2","4","G","Swampwalk
Whenever a player casts a Goblin spell, you may pay {G}. If you do, you gain 2 life.","3G","Creature - Treefolk Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iname, Life Aspect","4","4","G","When Iname, Life Aspect dies, you may exile it. If you do, return any number of target Spirit cards from your graveyard to your hand.","4GG","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minion of Tevesh Szat","4","4","B","At the beginning of your upkeep, Minion of Tevesh Szat deals 2 damage to you unless you pay {B}{B}.
{T}: Target creature gets +3/-2 until end of turn.","4BBB","Creature - Demon Minion");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Minion","*","*",NULL,NULL,"Creature - Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gustcloak Savior","3","4","W","Flying
Whenever a creature you control becomes blocked, you may untap that creature and remove it from combat.","4W","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dread of Night","B","White creatures get -1/-1.","B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Maddening Wind","G","Enchant creature
Cumulative upkeep {G} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of the upkeep of enchanted creature's controller, Maddening Wind deals 2 damage to that player.","2G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fieldmist Borderpost","WU","You may pay {1} and return a basic land you control to its owner's hand rather than pay Fieldmist Borderpost's mana cost.
Fieldmist Borderpost enters the battlefield tapped.
{T}: Add {W} or {U} to your mana pool.","1WU","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Riddle of Lightning","R","Choose target creature or player. Scry 3, then reveal the top card of your library. Riddle of Lightning deals damage equal to that card's converted mana cost to that creature or player. (To scry 3, look at the top three cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","3RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spore Flower","0","1","G","At the beginning of your upkeep, put a spore counter on Spore Flower.
Remove three spore counters from Spore Flower: Prevent all combat damage that would be dealt this turn.","GG","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Entomb","B","Search your library for a card and put that card into your graveyard. Then shuffle your library.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Juggler","2","2","R","Whenever Fire Juggler becomes blocked, clash with an opponent. If you win, Fire Juggler deals 4 damage to each creature blocking it. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vindictive Mob","5","5","B","When Vindictive Mob enters the battlefield, sacrifice a creature.
Vindictive Mob can't be blocked by Saprolings.","4BB","Creature - Human Berserker");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hopping Automaton","2","2","{0}: Hopping Automaton gets -1/-1 and gains flying until end of turn.","3","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marrow Chomper","3","3","BG","Devour 2 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with twice that many +1/+1 counters on it.)
When Marrow Chomper enters the battlefield, you gain 2 life for each creature it devoured.","3BG","Creature - Zombie Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Remember the Fallen","W","Choose one or both - Return target creature card from your graveyard to your hand; and/or return target artifact card from your graveyard to your hand.","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mesmeric Trance","U","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
{U}, Discard a card: Draw a card.","1UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daggerback Basilisk","2","2","G","Deathtouch","2G","Creature - Basilisk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Etherium Astrolabe","U","Flash
{B}, {T}, Sacrifice an artifact: Draw a card.","2U","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Slasher","1","2","R","{R}: Viashino Slasher gets +1/-1 until end of turn.","1R","Creature - Viashino Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silent Attendant","0","2","W","{T}: You gain 1 life.","2W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Veil","U","You may cast Mystic Veil as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Enchant creature
Enchanted creature has shroud. (It can't be the target of spells or abilities.)","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devouring Light","W","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Exile target attacking or blocking creature.","1WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Evincar's Justice","B","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Evincar's Justice deals 2 damage to each creature and each player.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Killing Wave","B","For each creature, its controller sacrifices it unless he or she pays X life.","XB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bloodfire Infusion","R","Enchant creature you control
{R}, Sacrifice enchanted creature: Bloodfire Infusion deals damage equal to the sacrificed creature's power to each creature.","2R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Head to Head","W","You and target opponent play Seven Questions about the top card of that player's library. (That player looks at the card, then you ask up to six yes-or-no questions about the card that he or she answers truthfully. You guess the card's name-that's question seven-and the player reveals the card.) If you win, prevent all damage that would be dealt this turn by a source of your choice.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saffi Eriksdotter","2","2","WG","Sacrifice Saffi Eriksdotter: When target creature is put into your graveyard from the battlefield this turn, return that card to the battlefield.","GW","Legendary Creature - Human Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Walking Desecration","1","1","B","{B}, {T}: Creatures of the creature type of your choice attack this turn if able.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soilshaper","1","1","G","Whenever you cast a Spirit or Arcane spell, target land becomes a 3/3 creature until end of turn. It's still a land.","1G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patagia Viper","2","1","G","Flying
When Patagia Viper enters the battlefield, put two 1/1 green and blue Snake creature tokens onto the battlefield.
When Patagia Viper enters the battlefield, sacrifice it unless {U} was spent to cast it.","3G","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dark Maze","4","5","U","Defender (This creature can't attack.)
{0}: Dark Maze can attack this turn as though it didn't have defender.  Exile it at the beginning of the next end step.","4U","Creature - Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Beast","4","4",NULL,NULL,"Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thermopod","4","3","R","{S}i}: Thermopod gains haste until end of turn. ({S}i} can be paid with one mana from a snow permanent.)
Sacrifice a creature: Add {R} to your mana pool.","4R","Snow Creature - Slug");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nyxathid","7","7","B","As Nyxathid enters the battlefield, choose an opponent.
Nyxathid gets -1/-1 for each card in the chosen player's hand.","1BB","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Raider","2","2","R","Goblin Raider can't block.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gelectrode","0","1","UR","{T}: Gelectrode deals 1 damage to target creature or player.
Whenever you cast an instant or sorcery spell, you may untap Gelectrode.","1UR","Creature - Weird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rampaging Baloths","6","6","G","Trample
Landfall - Whenever a land enters the battlefield under your control, you may put a 4/4 green Beast creature token onto the battlefield.","4GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rooftop Storm","U","You may pay {0} rather than pay the mana cost for Zombie creature spells you cast.","5U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mishra's Helix","{X}, {T}: Tap X target lands.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Morgue Thrull","2","2","B","Sacrifice Morgue Thrull: Put the top three cards of your library into your graveyard.","2B","Creature - Thrull");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Giantbaiting","RG","Put a 4/4 red and green Giant Warrior creature token with haste onto the battlefield. Exile it at the beginning of the next end step.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it.)","2(R/G)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaea's Anthem","G","Creatures you control get +1/+1.","1GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gore Vassal","2","1","W","Sacrifice Gore Vassal: Put a -1/-1 counter on target creature. Then if that creature's toughness is 1 or greater, regenerate it.","2W","Creature - Hound");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Transguild Courier","3","3",NULL,"4","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Equipoise","W","At the beginning of your upkeep, for each land target player controls in excess of the number you control, choose a land he or she controls, then the chosen permanents phase out. Repeat this process for artifacts and creatures. (While they're phased out, they're treated as though they don't exist. They phase in before that player untaps during his or her next untap step.)","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Golgothian Sylex","{1}, {T}: Each nontoken permanent from the Antiquities expansion is sacrificed by its controller.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lorescale Coatl","2","2","UG","Whenever you draw a card, you may put a +1/+1 counter on Lorescale Coatl.","1GU","Creature - Snake");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Emrakul, the Aeons Torn","15","15","Emrakul, the Aeons Torn can't be countered.
When you cast Emrakul, take an extra turn after this one.
Flying, protection from colored spells, annihilator 6
When Emrakul is put into a graveyard from anywhere, its owner shuffles his or her graveyard into his or her library.","15","Legendary Creature - Eldrazi");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pilgrim of Justice","1","3","W","Protection from red
{W}, Sacrifice Pilgrim of Justice: The next time a red source of your choice would deal damage this turn, prevent that damage.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nissa's Chosen","2","3","G","If Nissa's Chosen would die, put it on the bottom of its owner's library instead.","GG","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screeching Sliver","1","1","U","All Slivers have ""{T}: Target player puts the top card of his or her library into his or her graveyard.""","U","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Cascade","U","Choose one - Each player shuffles his or her hand and graveyard into his or her library; or each player draws seven cards.
Entwine {2} (Choose both if you pay the entwine cost.)","5UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Razor Pendulum","At the beginning of each player's end step, if that player has 5 or less life, Razor Pendulum deals 2 damage to him or her.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fledgling Dragon","2","2","R","Flying
Threshold - As long as seven or more cards are in your graveyard, Fledgling Dragon gets +3/+3 and has ""{R}: Fledgling Dragon gets +1/+0 until end of turn.""","2RR","Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Starke","At the beginning of your draw step, you may draw an additional card. If you do, put a card from your hand on the bottom of your library.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grief Tyrant","8","8","BR","Grief Tyrant enters the battlefield with four -1/-1 counters on it.
When Grief Tyrant dies, put a -1/-1 counter on target creature for each -1/-1 counter on Grief Tyrant.","5(B/R)","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ali from Cairo","0","1","R","Damage that would reduce your life total to less than 1 reduces it to 1 instead.","2RR","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scourge of Kher Ridges","6","6","R","Flying
{1}{R}: Scourge of Kher Ridges deals 2 damage to each creature without flying.
{5}{R}: Scourge of Kher Ridges deals 6 damage to each other creature with flying.","6RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pyreheart Wolf","1","1","R","Whenever Pyreheart Wolf attacks, each creature you control can't be blocked this turn except by two or more creatures.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","2R","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spectral Gateguards","2","5","W","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Spectral Gateguards is paired with another creature, both creatures have vigilance.","4W","Creature - Spirit Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shimmering Efreet","2","2","U","Flying
Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)
Whenever Shimmering Efreet phases in, target creature phases out. (It phases in before its controller untaps during his or her next untap step.)","2U","Creature - Efreet");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Toy Boat","3","3","Cumulative upkeep-Say ""Toy Boat"" quickly. (At the beginning of your upkeep, put an age counter on Toy Boat, then sacrifice it unless you say ""Toy Boat"" once for each age counter on it-without pausing between or fumbling it.)","3","Artifact Creature - Ship");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Deceit","0","5","U","Defender (This creature can't attack.)
{3}: Turn Wall of Deceit face down.
Morph {U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1U","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost-Lit Nourisher","2","1","G","{2}{G}, {T}: Target creature gets +2/+2 until end of turn.
Channel - {3}{G}, Discard Ghost-Lit Nourisher: Target creature gets +4/+4 until end of turn.","2G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zur's Weirding","U","Players play with their hands revealed.
If a player would draw a card, he or she reveals it instead. Then any other player may pay 2 life. If a player does, put that card into its owner's graveyard. Otherwise, that player draws a card.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corrupt Eunuchs","2","2","R","When Corrupt Eunuchs enters the battlefield, it deals 2 damage to target creature.","3R","Creature - Human Advisor");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Liquimetal Coating","{T}: Target permanent becomes an artifact in addition to its other types until end of turn.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chandra's Outrage","R","Chandra's Outrage deals 4 damage to target creature and 2 damage to that creature's controller.","2RR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Apocalypse Chime","{2}, {T}, Sacrifice Apocalypse Chime: Destroy all nontoken permanents from the Homelands expansion. They can't be regenerated.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mark of Eviction","U","Enchant creature
At the beginning of your upkeep, return enchanted creature and all Auras attached to that creature to their owners' hands.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("White Knight","2","2","W","First strike (This creature deals combat damage before creatures without first strike.)
Protection from black (This creature can't be blocked, targeted, dealt damage, or enchanted by anything black.)","WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rage Reflection","R","Creatures you control have double strike.","4RR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Unerring Sling","{3}, {T}, Tap an untapped creature you control: Unerring Sling deals damage equal to the tapped creature's power to target attacking or blocking creature with flying.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veiled Sentry","U","When an opponent casts a spell, if Veiled Sentry is an enchantment, Veiled Sentry becomes an Illusion creature with power and toughness each equal to that spell's converted mana cost.","U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Shambler","4","3","B","Other black creatures get -1/-1.","2BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wave of Indifference","R","X target creatures can't block this turn.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Nets","0","7","W","Defender (This creature can't attack.)
At end of combat, exile all creatures blocked by Wall of Nets.
When Wall of Nets leaves the battlefield, return all cards exiled with Wall of Nets to the battlefield under their owners' control.","1WW","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Magosi, the Waterveil","Magosi, the Waterveil enters the battlefield tapped.
{T}: Add {U} to your mana pool.
{U}, {T}: Put an eon counter on Magosi, the Waterveil. Skip your next turn.
{T}, Remove an eon counter from Magosi, the Waterveil and return it to its owner's hand: Take an extra turn after this one.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cenn's Heir","1","1","W","Whenever Cenn's Heir attacks, it gets +1/+1 until end of turn for each other attacking Kithkin.","1W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mortipede","4","1","B","{2}{G}: All creatures able to block Mortipede this turn do so.","3B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaea's Balance","G","As an additional cost to cast Gaea's Balance, sacrifice five lands.
Search your library for a land card of each basic land type and put them onto the battlefield. Then shuffle your library.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of Valor","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{1}{W}: Each creature without flanking blocking Knight of Valor gets -1/-1 until end of turn. Activate this ability only once each turn.","2W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sealed Fate","UB","Look at the top X cards of target opponent's library. Exile one of those cards and put the rest back on top of that player's library in any order.","XUB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hawkeater Moth","1","2","G","Flying; shroud (This permanent can't be the target of spells or abilities.)","3G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glen Elendra Pranksters","1","3","U","Flying
Whenever you cast a spell during an opponent's turn, you may return target creature you control to its owner's hand.","3U","Creature - Faerie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saprazzan Heir","1","1","U","Whenever Saprazzan Heir becomes blocked, you may draw three cards.","1U","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pelakka Wurm","7","7","G","Trample
When Pelakka Wurm enters the battlefield, you gain 7 life.
When Pelakka Wurm dies, draw a card.","4GGG","Creature - Wurm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tower of Murmurs","{8}, {T}: Target player puts the top eight cards of his or her library into his or her graveyard.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chimeric Coils","{X}{1}: Chimeric Coils becomes an X/X Construct artifact creature. Sacrifice it at the beginning of the next end step.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goldnight Commander","2","2","W","Whenever another creature enters the battlefield under your control, creatures you control get +1/+1 until end of turn.","3W","Creature - Human Cleric Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fog Bank","0","2","U","Defender, flying
Prevent all combat damage that would be dealt to and dealt by Fog Bank.","1U","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fulminator Mage","2","2","BR","Sacrifice Fulminator Mage: Destroy target nonbasic land.","1(B/R)(B/R)","Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repel Intruders","WU","Put two 1/1 white Kithkin Soldier creature tokens onto the battlefield if {W} was spent to cast Repel Intruders. Counter up to one target creature spell if {U} was spent to cast Repel Intruders. (Do both if {W}{U} was spent.)","3(W/U)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Neurok Commando","2","1","U","Shroud
Whenever Neurok Commando deals combat damage to a player, you may draw a card.","1UU","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Infantry","1","1","W","{1}: Icatian Infantry gains first strike until end of turn.
{1}: Icatian Infantry gains banding until end of turn. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Glimmervoid Basin","Whenever a player casts an instant or sorcery spell with a single target, he or she copies that spell for each other spell, permanent, card not on the battlefield, and/or player the spell could target. Each copy targets a different one of them.
Whenever you roll {C}, choose target creature. Each player except that creature's controller puts a token that's a copy of that creature onto the battlefield.",NULL,"Plane - Mirrodin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Lumberjack","1","1","R","{T}, Sacrifice a Forest: Add three mana in any combination of {R} and/or {G} to your mana pool.","R","Creature - Orc");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Fleetwing","2","2","U","Flying
Hexproof (This creature can't be the target of spells or abilities your opponents control.)","3U","Creature - Bird Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ronin Warclub","Equipped creature gets +2/+1.
Whenever a creature enters the battlefield under your control, attach Ronin Warclub to that creature.
Equip {5} ({5}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kaervek's Hex","B","Kaervek's Hex deals 1 damage to each nonblack creature and an additional 1 damage to each green creature.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Hunter","1","1","G","{1}{G}, {T}: Target creature doesn't untap during its controller's next untap step.","1G","Creature - Elf Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin War Paint","R","Enchant creature
Enchanted creature gets +2/+2 and has haste. (It can attack and {T} no matter when it came under its controller's control.)","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hill Giant","3","3","R",NULL,"3R","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Hound","1","1","R","Whenever you're dealt damage, you may put that many +1/+1 counters on Blood Hound.
At the beginning of your end step, remove all +1/+1 counters from Blood Hound.","2R","Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Web","G","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature gets +1/+1, has reach, and can block creatures with shadow as though they didn't have shadow. (Creatures with reach can block creatures with flying.)","1G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Defense Grid","Each spell costs {3} more to cast except during its controller's turn.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infernal Tribute","B","{2}, Sacrifice a nontoken permanent: Draw a card.","BBB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Dodecapod","3","3","If a spell or ability an opponent controls causes you to discard Dodecapod, put it onto the battlefield with two +1/+1 counters on it instead of putting it into your graveyard.","4","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carnage Wurm","6","6","G","Bloodthirst 3 (If an opponent was dealt damage this turn, this creature enters the battlefield with three +1/+1 counters on it.)
Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","6G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Thran War Machine","4","5","Echo {4} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
Thran War Machine attacks each turn if able.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cockatrice","2","4","G","Flying
Whenever Cockatrice blocks or becomes blocked by a non-Wall creature, destroy that creature at end of combat.","3GG","Creature - Cockatrice");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glissa's Courier","2","3","G","Mountainwalk","1GG","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time of Need","G","Search your library for a legendary creature card, reveal it, and put it into your hand. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Filter","G","Destroy all enchantments or all nonwhite enchantments.","1GG","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Cathodion","3","3","When Cathodion dies, add {3} to your mana pool.","3","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glistener Elf","1","1","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","G","Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mine Excavation","W","Return target artifact or enchantment card from a graveyard to its owner's hand.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose a new target for the copy.)","1W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magmatic Core","R","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of your end step, Magmatic Core deals X damage divided as you choose among any number of target creatures, where X is the number of age counters on it.","2RR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kazandu Refuge","Kazandu Refuge enters the battlefield tapped.
When Kazandu Refuge enters the battlefield, you gain 1 life.
{T}: Add {R} or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Maze","U","Players can't cast spells that share a color with the spell most recently cast this turn.","1U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sarpadian Empires, Vol. VII","As Sarpadian Empires, Vol. VII enters the battlefield, choose white Citizen, blue Camarid, black Thrull, red Goblin, or green Saproling.
{3}, {T}: Put a 1/1 creature token of the chosen color and type onto the battlefield.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Accelerate","R","Target creature gains haste until end of turn.
Draw a card.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("King Cheetah","3","2","G","Flash","3G","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shaper Parasite","2","3","U","Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Shaper Parasite is turned face up, target creature gets +2/-2 or -2/+2 until end of turn.","1UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Starke of Rath","2","2","R","{T}: Destroy target artifact or creature. That permanent's controller gains control of Starke of Rath. (This effect lasts indefinitely.)","1RR","Legendary Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kill Switch","{2}, {T}: Tap all other artifacts. They don't untap during their controllers' untap steps for as long as Kill Switch remains tapped.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brown Ouphe","1","1","G","{1}{G}, {T}: Counter target activated ability from an artifact source. (Mana abilities can't be targeted.)","G","Creature - Ouphe");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakish Heir","2","2","R","Whenever a Vampire you control deals combat damage to a player, put a +1/+1 counter on it.","2R","Creature - Vampire");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hell's Caretaker Avatar","{3}, Sacrifice a creature: Return target creature card from your graveyard to the battlefield.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Augur","0","3","G","Sacrifice Llanowar Augur: Target creature gets +3/+3 and gains trample until end of turn. Activate this ability only during your upkeep.","G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cho-Arrim Alchemist","1","1","W","{1}{W}{W}, {T}, Discard a card: The next time a source of your choice would deal damage to you this turn, prevent that damage. You gain life equal to the damage prevented this way.","W","Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wing Shards","W","Target player sacrifices an attacking creature.
Storm (When you cast this spell, copy it for each spell cast before it this turn.)","1WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Killer Instinct","RG","At the beginning of your upkeep, reveal the top card of your library. If it's a creature card, put it onto the battlefield. That creature gains haste until end of turn. Sacrifice it at the beginning of the next end step.","4RG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repeal","U","Return target nonland permanent with converted mana cost X to its owner's hand.
Draw a card.","XU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Maelstrom Nexus","WUBRG","The first spell you cast each turn has cascade. (When you cast your first spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)","WUBRG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azorius AEthermage","1","1","WU","Whenever a permanent is returned to your hand, you may pay {1}. If you do, draw a card.","1WU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stampeding Rhino","4","4","G","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","4G","Creature - Rhino");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhox Charger","3","3","G","Trample
Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","3G","Creature - Rhino Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Demon's Horn","Whenever a player casts a black spell, you may gain 1 life.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakdos the Defiler","7","6","BR","Flying, trample
Whenever Rakdos the Defiler attacks, sacrifice half the non-Demon permanents you control, rounded up.
Whenever Rakdos deals combat damage to a player, that player sacrifices half the non-Demon permanents he or she controls, rounded up.","2BBRR","Legendary Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fallen Angel","3","3","B","Flying
Sacrifice a creature: Fallen Angel gets +2/+1 until end of turn.","3BB","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anaba Spirit Crafter","1","3","R","Minotaur creatures get +1/+0.","2RR","Creature - Minotaur Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Mine","{T}: Add {1} to your mana pool. If you control an Urza's Power-Plant and an Urza's Tower, add {2} to your mana pool instead.",NULL,"Land - Urza’s Mine");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incremental Growth","G","Put a +1/+1 counter on target creature, two +1/+1 counters on another target creature, and three +1/+1 counters on a third target creature.","3GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Surge","G","Exile the top card of your library. If it's a permanent card, you may put it onto the battlefield. If you do, repeat this process.","8GG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ancient Ziggurat","{T}: Add one mana of any color to your mana pool. Spend this mana only to cast a creature spell.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwater Matriarch","2","3","U","{T}: Gain control of target creature for as long as that creature is enchanted.","2UU","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nettlevine Blight","B","Enchant creature or land
Enchanted permanent has ""At the beginning of your end step, sacrifice this permanent and attach Nettlevine Blight to a creature or land you control.""","4BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Captain's Maneuver","WR","The next X damage that would be dealt to target creature or player this turn is dealt to another target creature or player instead.","XRW","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Deserted Temple","{T}: Add {1} to your mana pool.
{1}, {T}: Untap target land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Resistance","0","3","W","Defender (This creature can't attack.)
Flying
At the beginning of each end step, if Wall of Resistance was dealt damage this turn, put a +0/+1 counter on it.","1W","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benthicore","5","5","U","When Benthicore enters the battlefield, put two 1/1 blue Merfolk Wizard creature tokens onto the battlefield.
Tap two untapped Merfolk you control: Untap Benthicore. It gains shroud until end of turn. (It can't be the target of spells or abilities.)","6U","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ring of Ma'rûf","{5}, {T}, Exile Ring of Ma'rûf: The next time you would draw a card this turn, instead choose a card you own from outside the game and put it into your hand.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Valiant Guard","0","3","W",NULL,"W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Inquisitors","3","3","W","Whenever Serra Inquisitors blocks or becomes blocked by one or more black creatures, Serra Inquisitors gets +2/+0 until end of turn.","4W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aggravate","R","Aggravate deals 1 damage to each creature target player controls. Each creature dealt damage this way attacks this turn if able.","3RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turn Aside","U","Counter target spell that targets a permanent you control.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karrthus, Tyrant of Jund","7","7","BRG","Flying, haste
When Karrthus, Tyrant of Jund enters the battlefield, gain control of all Dragons, then untap all Dragons.
Other Dragon creatures you control have haste.","4BRG","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of the Golden Cricket","2","2","W","Whenever Order of the Golden Cricket attacks, you may pay {W}. If you do, it gains flying until end of turn.","1W","Creature - Kithkin Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirge Familiar","3","2","B","Flying
Discard a card: Add {B} to your mana pool.","4B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raving Oni-Slave","3","3","B","When Raving Oni-Slave enters the battlefield or leaves the battlefield, you lose 3 life if you don't control a Demon.","1B","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Blademaster","2","2","W","First strike
Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","2W","Creature - Fox Samurai");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Conjurer's Bauble","{T}, Sacrifice Conjurer's Bauble: Put up to one target card from your graveyard on the bottom of your library. Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Troll","2","1","U","{U}: Regenerate Sea Troll. Activate this ability only if Sea Troll blocked or was blocked by a blue creature this turn.","2U","Creature - Troll");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flooded Grove","{T}: Add {1} to your mana pool.
{(g/u)}, {T}: Add {G}{G}, {G}{U}, or {U}{U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Commander Greven il-Vec","7","5","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
When Commander Greven il-Vec enters the battlefield, sacrifice a creature.","3BBB","Legendary Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of the Holy Nimbus","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
If Knight of the Holy Nimbus would be destroyed, regenerate it.
{2}: Knight of the Holy Nimbus can't be regenerated this turn. Only any opponent may activate this ability.","WW","Creature - Human Rebel Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Pierce","U","Counter target noncreature spell unless its controller pays {2}.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Psychic Membrane","0","3","U","Defender (This creature can't attack.)
Whenever Psychic Membrane blocks, you may draw a card.","2U","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ogre's Cleaver","Equipped creature gets +5/+0.
Equip {5}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Syphon","U","Counter target spell unless its controller pays {1} for each blue permanent you control.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cyclopean Snare","{3}, {T}: Tap target creature, then return Cyclopean Snare to its owner's hand.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Double Cleave","WR","Target creature gains double strike until end of turn. (It deals both first-strike and regular combat damage.)","1(R/W)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serpent Skin","G","Flash
Enchant creature
Enchanted creature gets +1/+1.
{G}: Regenerate enchanted creature.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Imp","1","1","B","Flying (This creature can't be blocked except by creatures with flying or reach.)","1B","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incoming!","G","Each player searches his or her library for any number of artifacts, creatures, enchantments, and lands and puts those cards into play. Each player shuffles his or her library afterwards.","4GGGG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Innocence Kami","2","3","W","{W}, {T}: Tap target creature.
Whenever you cast a Spirit or Arcane spell, untap Innocence Kami.","3WW","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Peregrine Mask","Equipped creature has defender, flying, and first strike.
Equip {2}","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Evernight Shade","1","1","B","{B}: Evernight Shade gets +1/+1 until end of turn.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","3B","Creature - Shade");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Undermine","UB","Counter target spell. Its controller loses 3 life.","UUB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benthic Explorers","2","4","U","{T}, Untap a tapped land an opponent controls: Add one mana of any type that land could produce to your mana pool.","3U","Creature - Merfolk Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Traitorous Instinct","R","Gain control of target creature until end of turn. Untap that creature. Until end of turn, it gets +2/+0 and gains haste.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of the Ebon Hand","2","1","B","Protection from white
{B}: Order of the Ebon Hand gains first strike until end of turn.
{B}{B}: Order of the Ebon Hand gets +1/+0 until end of turn.","BB","Creature - Cleric Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ramirez DePietro","4","3","UB","First strike","3UBB","Legendary Creature - Human Pirate");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zap","R","Zap deals 1 damage to target creature or player.
Draw a card.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Menial","0","4","R","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
{R}: Ogre Menial gets +1/+0 until end of turn.","3R","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Troopers","3","3","G","{T}: Add {G} to your mana pool.","3G","Creature - Elf Druid Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ohran Yeti","3","3","R","{2}{S}i}: Target snow creature gains first strike until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","3R","Snow Creature - Yeti");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Braidwood Sextant","{2}, {T}, Sacrifice Braidwood Sextant: Search your library for a basic land card, reveal that card, and put it into your hand. Then shuffle your library.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Panacea","{X}{X}, {T}: Prevent the next X damage that would be dealt to target creature or player this turn.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recumbent Bliss","W","Enchant creature
Enchanted creature can't attack or block.
At the beginning of your upkeep, you may gain 1 life.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ice Cave","U","Whenever a player casts a spell, any other player may pay that spell's mana cost. If a player does, counter the spell. (Mana cost includes color.)","3UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Freyalise's Charm","G","Whenever an opponent casts a black spell, you may pay {G}{G}. If you do, you draw a card.
{G}{G}: Return Freyalise's Charm to its owner's hand.","GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Relic Putrescence","B","Enchant artifact
Whenever enchanted artifact becomes tapped, its controller gets a poison counter.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Counter","U","Counter target spell.
Gotcha Whenever an opponent says ""Spell"" or ""Counter,"" you may say ""Gotcha"" If you do, return Spell Counter from your graveyard to your hand.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pincer Spider","2","3","G","Kicker {3} (You may pay an additional {3} as you cast this spell.)
Reach (This creature can block creatures with flying.)
If Pincer Spider was kicked, it enters the battlefield with a +1/+1 counter on it.","2G","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Noxious Field","B","Enchant land
Enchanted land has ""{T}: This land deals 1 damage to each creature and each player.""","1BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tormented Angel","1","5","W","Flying","3W","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Portcullis","Whenever a creature enters the battlefield, if there are two or more other creatures on the battlefield, exile that creature. Return that card to the battlefield under its owner's control when Portcullis leaves the battlefield.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mana Crypt","At the beginning of your upkeep, flip a coin. If you lose the flip, Mana Crypt deals 3 damage to you.
{T}: Add {2} to your mana pool.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vigilant Martyr","1","1","W","Sacrifice Vigilant Martyr: Regenerate target creature.
{W}{W}, {T}, Sacrifice Vigilant Martyr: Counter target spell that targets an enchantment.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Funeral March","B","Enchant creature
When enchanted creature leaves the battlefield, its controller sacrifices a creature.","1BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vithian Stinger","0","1","R","{T}: Vithian Stinger deals 1 damage to target creature or player.
Unearth {1}{R} ({1}{R}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","2R","Creature - Human Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Leering Emblem","Whenever you cast a spell, equipped creature gets +2/+2 until end of turn.
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vintara Snapper","2","2","G","Vintara Snapper has shroud as long as you control no untapped lands. (It can't be the target of spells or abilities.)","GG","Creature - Turtle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kranioceros","5","2","R","{1}{W}: Kranioceros gets +0/+3 until end of turn.","4R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychotic Haze","B","Psychotic Haze deals 1 damage to each creature and each player.
Madness {1}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","2BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirror Wall","3","4","U","Defender (This creature can't attack.)
{W}: Mirror Wall can attack this turn as though it didn't have defender.","3U","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recantation","U","At the beginning of your upkeep, you may put a verse counter on Recantation.
{U}, Sacrifice Recantation: Return up to X target permanents to their owners' hands, where X is the number of verse counters on Recantation.","3UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dance of Many","U","When Dance of Many enters the battlefield, put a token that's a copy of target nontoken creature onto the battlefield.
When Dance of Many leaves the battlefield, exile the token.
When the token leaves the battlefield, sacrifice Dance of Many.
At the beginning of your upkeep, sacrifice Dance of Many unless you pay {U}{U}.","UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunscour","W","You may exile two white cards from your hand rather than pay Sunscour's mana cost.
Destroy all creatures.","5WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Punctuate","R","Punctuate deals damage to target creature equal to half the number of punctuation marks in that creature's text box. (The punctuation marks are  ? , ; : - ( ) / "" ' & .)","3R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Smoldering Crater","Smoldering Crater enters the battlefield tapped.
{T}: Add {R} to your mana pool.
Cycling {2} ({2}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parallel Lives","G","If an effect would put one or more tokens onto the battlefield under your control, it puts twice that many of those tokens onto the battlefield instead.","3G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undertaker","1","1","B","{B}, {T}, Discard a card: Return target creature card from your graveyard to your hand.","1B","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Willow Dryad","1","1","G","Forestwalk","G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teferi, Mage of Zhalfir","3","4","U","Flash (You may cast this spell any time you could cast an instant.)
Creature cards you own that aren't on the battlefield have flash.
Each opponent can cast spells only any time he or she could cast a sorcery.","2UUU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Custody Battle","R","Enchant creature
Enchanted creature has ""At the beginning of your upkeep, target opponent gains control of this creature unless you sacrifice a land.""","1R","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eye of Ramos","{T}: Add {U} to your mana pool.
Sacrifice Eye of Ramos: Add {U} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grazing Kelpie","2","3","UG","{(g/u)}, Sacrifice Grazing Kelpie: Put target card from a graveyard on the bottom of its owner's library.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","3(G/U)","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mask of Memory","Whenever equipped creature deals combat damage to a player, you may draw two cards. If you do, discard a card.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zedruu the Greathearted","2","4","WUR","At the beginning of your upkeep, you gain X life and draw X cards, where X is the number of permanents you own that your opponents control.
{R}{W}{U}: Target opponent gains control of target permanent you control.","1RWU","Legendary Creature - Minotaur Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drag Down","B","Domain - Target creature gets -1/-1 until end of turn for each basic land type among lands you control.","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plated Rootwalla","3","3","G","{2}{G}: Plated Rootwalla gets +3/+3 until end of turn. Activate this ability only once each turn.","4G","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mind Maggots","2","2","B","When Mind Maggots enters the battlefield, discard any number of creature cards. For each card discarded this way, put two +1/+1 counters on Mind Maggots.","3B","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Harbinger","3","4","R","When Giant Harbinger enters the battlefield, you may search your library for a Giant card, reveal it, then shuffle your library and put that card on top of it.","4R","Creature - Giant Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gratuitous Violence","R","If a creature you control would deal damage to a creature or player, it deals double that damage to that creature or player instead.","2RRR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grim Flowering","G","Draw a card for each creature card in your graveyard.","5G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spirit of the Night","6","5","B","Flying, trample, haste, protection from black
Spirit of the Night has first strike as long as it's attacking.","6BBB","Legendary Creature - Demon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leaden Fists","U","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature gets +3/+3 and doesn't untap during its controller's untap step.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oracle of Nectars","2","2","WG","{X}, {T}: You gain X life.","2(G/W)","Creature - Elf Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Illuminated Wings","U","Enchant creature
Enchanted creature has flying.
{2}, Sacrifice Illuminated Wings: Draw a card.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Midnight Banshee","5","5","B","Wither (This deals damage to creatures in the form of -1/-1 counters.)
At the beginning of your upkeep, put a -1/-1 counter on each nonblack creature.","3BBB","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rumbling Aftershocks","R","Whenever you cast a kicked spell, you may have Rumbling Aftershocks deal damage to target creature or player equal to the number of times that spell was kicked.","4R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of Body and Mind","Equipped creature gets +2/+2 and has protection from green and from blue.
Whenever equipped creature deals combat damage to a player, you put a 2/2 green Wolf creature token onto the battlefield and that player puts the top ten cards of his or her library into his or her graveyard.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pact of the Titan","Put a 4/4 red Giant creature token onto the battlefield.
At the beginning of your next upkeep, pay {4}{R}. If you don't, you lose the game.","0","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Oxidda Golem","3","2","Affinity for Mountains (This spell costs {1} less to cast for each Mountain you control.)
Haste","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirozel","2","3","U","Flying
When Mirozel becomes the target of a spell or ability, return Mirozel to its owner's hand.","3U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Spear","B","Target player reveals his or her hand. You choose a Spirit or Arcane card from it. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Predator Dragon","4","4","R","Flying, haste
Devour 2 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with twice that many +1/+1 counters on it.)","3RRR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stromkirk Patrol","4","3","B","Whenever Stromkirk Patrol deals combat damage to a player, put a +1/+1 counter on it.","4B","Creature - Vampire Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parapet","W","You may cast Parapet as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Creatures you control get +0/+1.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Sting","R","Whenever an opponent casts a creature spell, Æther Sting deals 1 damage to that player.","3R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Land Cap","Land Cap doesn't untap during your untap step if it has a depletion counter on it.
At the beginning of your upkeep, remove a depletion counter from Land Cap.
{T}: Add {W} or {U} to your mana pool. Put a depletion counter on Land Cap.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Helm of Obedience","{X}, {T}: Target opponent puts cards from the top of his or her library into his or her graveyard until a creature card or X cards are put into that graveyard this way, whichever comes first. If a creature card is put into that graveyard this way, sacrifice Helm of Obedience and put that card onto the battlefield under your control. X can't be 0.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Breath","W","Exile target creature with power 2 or less. Its controller gains 4 life.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Furnace of Rath","R","If a source would deal damage to a creature or player, it deals double that damage to that creature or player instead.","1RRR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lifeforce","G","{G}{G}: Counter target black spell.","GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thran Lens","All permanents are colorless.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flock of Rabid Sheep","G","Flip X coins; an opponent calls heads or tails. For each flip you win, put a Rabid Sheep token into play. Treat these tokens as 2/2 green creatures that count as Sheep.","XGG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mindstorm Crown","At the beginning of your upkeep, draw a card if you had no cards in hand at the beginning of this turn. If you had a card in hand, Mindstorm Crown deals 1 damage to you.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uktabi Wildcats","*","*","G","Uktabi Wildcats's power and toughness are each equal to the number of Forests you control.
{G}, Sacrifice a Forest: Regenerate Uktabi Wildcats.","4G","Creature - Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tainted Isle","{T}: Add {1} to your mana pool.
{T}: Add {U} or {B} to your mana pool. Activate this ability only if you control a Swamp.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Folk of the Pines","2","5","G","{1}{G}: Folk of the Pines gets +1/+0 until end of turn.","4G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corrupt Official","3","1","B","{2}{B}: Regenerate Corrupt Official.
Whenever Corrupt Official becomes blocked, defending player discards a card at random.","4B","Creature - Human Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call to Arms","W","As Call to Arms enters the battlefield, choose a color and an opponent.
White creatures get +1/+1 as long as the chosen color is the most common color among nontoken permanents the chosen player controls but isn't tied for most common.
When the chosen color isn't the most common color among nontoken permanents the chosen player controls or is tied for most common, sacrifice Call to Arms.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forget","U","Target player discards two cards, then draws as many cards as he or she discarded this way.","UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cho-Manno, Revolutionary","2","2","W","Prevent all damage that would be dealt to Cho-Manno, Revolutionary.","2WW","Legendary Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grassland Crusader","2","4","W","{T}: Target Elf or Soldier creature gets +2/+2 until end of turn.","5W","Creature - Human Cleric Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soramaro, First to Dream","*","*","U","Flying
Soramaro, First to Dream's power and toughness are each equal to the number of cards in your hand.
{4}, Return a land you control to its owner's hand: Draw a card.","4UU","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jhovall Rider","3","3","W","Trample","4W","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Abunas","2","5","W","Artifacts you control have hexproof. (They can't be the targets of spells or abilities your opponents control.)","3W","Creature - Cat Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slavering Nulls","2","1","R","Whenever Slavering Nulls deals combat damage to a player, if you control a Swamp, you may have that player discard a card.","1R","Creature - Goblin Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Oboro, Palace in the Clouds","{T}: Add {U} to your mana pool.
{1}: Return Oboro, Palace in the Clouds to its owner's hand.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bant Battlemage","2","2","W","{G}, {T}: Target creature gains trample until end of turn.
{U}, {T}: Target creature gains flying until end of turn.","2W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Initiate of Blood","4","4","R","{T}: Goka the Unjust deals 4 damage to target creature that was dealt damage this turn.","3R","Legendary Creature - Ogre Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mentor of the Meek","2","2","W","Whenever another creature with power 2 or less enters the battlefield under your control, you may pay {1}. If you do, draw a card.","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deluge","U","Tap all creatures without flying.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quagnoth","4","5","G","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Shroud (This permanent can't be the target of spells or abilities.)
When a spell or ability an opponent controls causes you to discard Quagnoth, return it to your hand.","5G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Etherium Sculptor","1","2","U","Artifact spells you cast cost {1} less to cast.","1U","Artifact Creature - Vedalken Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gush","U","You may return two Islands you control to their owner's hand rather than pay Gush's mana cost.
Draw two cards.","4U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slash Panther","4","2","R","({(r/p)} can be paid with either {R} or 2 life.)
Haste","4(R/P)","Artifact Creature - Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mossfire Valley","{1}, {T}: Add {R}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flamekin Spitfire","1","1","R","{3}{R}: Flamekin Spitfire deals 1 damage to target creature or player.","1R","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Puresight Merrow","2","2","WU","{(w/u)}, {Q}: Look at the top card of your library. You may exile that card. ({Q} is the untap symbol.)","(W/U)(W/U)","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ashes to Ashes","B","Exile two target nonartifact creatures. Ashes to Ashes deals 5 damage to you.","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pursuit of Knowledge","W","If you would draw a card, you may put a study counter on Pursuit of Knowledge instead.
Remove three study counters from Pursuit of Knowledge, Sacrifice Pursuit of Knowledge: Draw seven cards.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tropical Storm","G","Tropical Storm deals X damage to each creature with flying and 1 additional damage to each blue creature.","XG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ichorid","3","1","B","Haste
At the beginning of the end step, sacrifice Ichorid.
At the beginning of your upkeep, if Ichorid is in your graveyard, you may exile a black creature card other than Ichorid from your graveyard. If you do, return Ichorid to the battlefield.","3B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scarred Vinebreeder","1","1","B","{2}{B}, Exile an Elf card from your graveyard: Scarred Vinebreeder gets +3/+3 until end of turn.","1B","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cytoshape","UG","Choose a nonlegendary creature on the battlefield. Target creature becomes a copy of that creature until end of turn.","1GU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Spike","B","You may exile two black cards from your hand rather than pay Soul Spike's mana cost.
Soul Spike deals 4 damage to target creature or player and you gain 4 life.","5BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Turntimber Ranger","2","2","G","Whenever Turntimber Ranger or another Ally enters the battlefield under your control, you may put a 2/2 green Wolf creature token onto the battlefield. If you do, put a +1/+1 counter on Turntimber Ranger.","3GG","Creature - Elf Scout Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rishadan Footpad","2","2","U","When Rishadan Footpad enters the battlefield, each opponent sacrifices a permanent unless he or she pays {2}.","3U","Creature - Human Pirate");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Beetle","0","0","Clockwork Beetle enters the battlefield with two +1/+1 counters on it.
Whenever Clockwork Beetle attacks or blocks, remove a +1/+1 counter from it at end of combat.","1","Artifact Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Oracle","{0}: Untap target attacking creature you control and remove it from combat.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tajuru Archer","1","2","G","Whenever Tajuru Archer or another Ally enters the battlefield under your control, you may have Tajuru Archer deal damage to target creature with flying equal to the number of Allies you control.","2G","Creature - Elf Archer Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drekavac","3","3","B","When Drekavac enters the battlefield, sacrifice it unless you discard a noncreature card.","1B","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodripper","4","6","G","Fading 3 (This creature enters the battlefield with three fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
{1}, Remove a fade counter from Woodripper: Destroy target artifact.","3GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patrol Hound","2","2","W","Discard a card: Patrol Hound gains first strike until end of turn.","1W","Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grim Reminder","B","Search your library for a nonland card and reveal it. Each opponent who cast a card this turn with the same name as that card loses 6 life. Then shuffle your library.
{B}{B}: Return Grim Reminder from your graveyard to your hand. Activate this ability only during your upkeep.","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Earwig Squad","5","3","B","Prowl {2}{B} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Goblin or Rogue.)
When Earwig Squad enters the battlefield, if its prowl cost was paid, search target opponent's library for three cards and exile them. Then that player shuffles his or her library.","3BB","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soraya the Falconer","2","2","W","Bird creatures get +1/+1.
{1}{W}: Target Bird creature gains banding until end of turn. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding a player controls are blocking or being blocked by a creature, that player divides that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","1WW","Legendary Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wit's End","B","Target player discards his or her hand.","5BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Destructive Flow","BRG","At the beginning of each player's upkeep, that player sacrifices a nonbasic land.","BRG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Porcelain Legionnaire","3","1","W","({(w/p)} can be paid with either {W} or 2 life.)
First strike","2(W/P)","Artifact Creature - Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Time Vault","Time Vault enters the battlefield tapped.
Time Vault doesn't untap during your untap step.
If you would begin your turn while Time Vault is tapped, you may skip that turn instead. If you do, untap Time Vault.
{T}: Take an extra turn after this one.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Penumbra Kavu","3","3","G","When Penumbra Kavu dies, put a 3/3 black Kavu creature token onto the battlefield.","4G","Creature - Kavu");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Memnarch","4","5","{1}{U}{U}: Target permanent becomes an artifact in addition to its other types. (This effect lasts indefinitely.)
{3}{U}: Gain control of target artifact. (This effect lasts indefinitely.)","7","Legendary Artifact Creature - Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frostwielder","1","2","R","{T}: Frostwielder deals 1 damage to target creature or player.
If a creature dealt damage by Frostwielder this turn would die, exile it instead.","2RR","Creature - Human Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Obelisk of Bant","{T}: Add {G}, {W}, or {U} to your mana pool.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eureka","G","Starting with you, each player may put a permanent card from his or her hand onto the battlefield. Repeat this process until no one puts a card onto the battlefield.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screeching Drake","2","2","U","Flying
When Screeching Drake enters the battlefield, draw a card, then discard a card.","3U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sages of the Anima","3","4","UG","If you would draw a card, instead reveal the top three cards of your library. Put all creature cards revealed this way into your hand and the rest on the bottom of your library in any order.","3GU","Creature - Elf Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Merciless Predator","3","2","At the beginning of each upkeep, if a player cast two or more spells last turn, transform Merciless Predator.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sea of Sand","Players reveal each card they draw.
Whenever a player draws a land card, that player gains 3 life.
Whenever a player draws a nonland card, that player loses 3 life.
Whenever you roll {C}, put target permanent on top of its owner's library.",NULL,"Plane - Rabiah");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Murderous Redcap Avatar","Whenever a creature enters the battlefield under your control with a counter on it, you may have it deal damage equal to its power to target creature or player.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drake Umbra","U","Enchant creature
Enchanted creature gets +3/+3 and has flying.
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","4U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Bloodletting","R","Enchant player
If a source would deal damage to enchanted player, it deals double that damage to that player instead.","3RR","Enchantment - Aura Curse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hate Weaver","2","1","B","{2}: Target blue or red creature gets +1/+0 until end of turn.","1B","Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormwatch Eagle","2","1","U","Flying
Sacrifice a land: Return Stormwatch Eagle to its owner's hand.","3U","Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Portal","{3}: If your library has ten or more cards in it, target opponent looks at the top ten cards of your library and separates them into two face-down piles. Exile one of those piles. Search the other pile for a card, put it into your hand, then shuffle the rest of that pile into your library.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twincast","U","Copy target instant or sorcery spell. You may choose new targets for the copy.","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gabriel Angelfire","4","4","WG","At the beginning of your upkeep, choose flying, first strike, trample, or rampage 3. Gabriel Angelfire gains that ability until your next upkeep. (Whenever a creature with rampage 3 becomes blocked, it gets +3/+3 until end of turn for each creature blocking it beyond the first.)","3GGWW","Legendary Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razor Swine","2","1","R","First strike
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","2R","Creature - Boar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avenger of Zendikar","5","5","G","When Avenger of Zendikar enters the battlefield, put a 0/1 green Plant creature token onto the battlefield for each land you control.
Landfall - Whenever a land enters the battlefield under your control, you may put a +1/+1 counter on each Plant creature you control.","5GG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squirrel Wrangler","2","2","G","{1}{G}, Sacrifice a land: Put two 1/1 green Squirrel creature tokens onto the battlefield.
{1}{G}, Sacrifice a land: Squirrel creatures get +1/+1 until end of turn.","2GG","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Furnace Celebration","R","Whenever you sacrifice another permanent, you may pay {2}. If you do, Furnace Celebration deals 2 damage to target creature or player.","1RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Reckoning","W","Each player chooses a creature he or she controls. Destroy the rest.
Flashback {5}{W}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2WW","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Mindless Automaton","0","0","Mindless Automaton enters the battlefield with two +1/+1 counters on it.
{1}, Discard a card: Put a +1/+1 counter on Mindless Automaton.
Remove two +1/+1 counters from Mindless Automaton: Draw a card.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ambush Party","3","1","R","First strike, haste","4R","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Niall Silvain","2","2","G","{G}{G}{G}{G}, {T}: Regenerate target creature.","GGG","Creature - Ouphe");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Baton of Morale","{2}: Target creature gains banding until end of turn. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding a player controls are blocking or being blocked by a creature, that player divides that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balthor the Defiled","2","2","B","Minion creatures get +1/+1.
{B}{B}{B}, Exile Balthor the Defiled: Each player returns all black and all red creature cards from his or her graveyard to the battlefield.","2BB","Legendary Creature - Zombie Dwarf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudhoof Kirin","4","4","U","Flying
Whenever you cast a Spirit or Arcane spell, you may put the top X cards of target player's library into his or her graveyard, where X is that spell's converted mana cost.","3UU","Legendary Creature - Kirin Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eternal Dragon","5","5","W","Flying
{3}{W}{W}: Return Eternal Dragon from your graveyard to your hand. Activate this ability only during your upkeep.
Plainscycling {2} ({2}, Discard this card: Search your library for a Plains card, reveal it, and put it into your hand. Then shuffle your library.)","5WW","Creature - Dragon Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Surge Node","Surge Node enters the battlefield with six charge counters on it.
{1}, {T}, Remove a charge counter from Surge Node: Put a charge counter on target artifact.","1","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Grapeshot Catapult","2","3","{T}: Grapeshot Catapult deals 1 damage to target creature with flying.","4","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rod of Spanking","{2}, {T}: Rod of Spanking deals 1 damage to target player. Then untap Rod of Spanking unless that player says ""Thank you, sir. May I have another?""","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tiger Claws","G","Flash
Enchant creature
Enchanted creature gets +1/+1 and has trample.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plaguemaw Beast","4","3","G","{T}, Sacrifice a creature: Proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","3GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skyshroud Claim","G","Search your library for up to two Forest cards and put them onto the battlefield. Then shuffle your library.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rune-Tail, Kitsune Ascendant","2","2","W","When you have 30 or more life, flip Rune-Tail, Kitsune Ascendant.
---
Prevent all damage that would be dealt to creatures you control.","2W","Legendary Creature - Fox Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kokusho, the Evening Star","5","5","B","Flying
When Kokusho, the Evening Star dies, each opponent loses 5 life. You gain life equal to the life lost this way.","4BB","Legendary Creature - Dragon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Relearn","U","Return target instant or sorcery card from your graveyard to your hand.","1UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nettling Curse","B","Enchant creature
Whenever enchanted creature attacks or blocks, its controller loses 3 life.
{1}{R}: Enchanted creature attacks this turn if able.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Seism","R","Sacrifice any number of lands. Add {1} to your mana pool for each land sacrificed this way.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Durkwood Tracker","4","3","G","{1}{G}, {T}: If Durkwood Tracker is on the battlefield, it deals damage equal to its power to target attacking creature. That creature deals damage equal to its power to Durkwood Tracker.","4G","Creature - Giant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Misty Rainforest","{T}, Pay 1 life, Sacrifice Misty Rainforest: Search your library for a Forest or Island card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Lava","1","3","R","Defender (This creature can't attack.)
{R}: Wall of Lava gets +1/+1 until end of turn.","1RR","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ember Gale","R","Creatures target player controls can't block this turn. Ember Gale deals 1 damage to each white and/or blue creature that player controls.","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Healing Leaves","G","Choose one - Target player gains 3 life; or prevent the next 3 damage that would be dealt to target creature or player this turn.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sapphire Medallion","Blue spells you cast cost {1} less to cast.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disorient","U","Target creature gets -7/-0 until end of turn.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Catacomb Dragon","4","4","B","Flying
Whenever Catacomb Dragon becomes blocked by a nonartifact, non-Dragon creature, that creature gets -X/-0 until end of turn, where X is half the creature's power, rounded down.","4BB","Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wolfhunter's Quiver","Equipped creature has ""{T}: This creature deals 1 damage to target creature or player"" and ""{T}: This creature deals 3 damage to target Werewolf creature.""
Equip {5}","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Giant Strength","R","Enchant creature
Enchanted creature gets +2/+2.","RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashivan Dragon","4","4","RG","Flying
{R}: Viashivan Dragon gets +1/+0 until end of turn.
{G}: Viashivan Dragon gets +0/+1 until end of turn.","2RRGG","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auntie's Snitch","3","1","B","Auntie's Snitch can't block.
Prowl {1}{B} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Goblin or Rogue.)
Whenever a Goblin or Rogue you control deals combat damage to a player, if Auntie's Snitch is in your graveyard, you may return Auntie's Snitch to your hand.","2B","Creature - Goblin Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crush","R","Destroy target noncreature artifact.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blazing Torch","Equipped creature can't be blocked by Vampires or Zombies.
Equipped creature has ""{T}, Sacrifice Blazing Torch: Blazing Torch deals 2 damage to target creature or player.""
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hypergenesis","Suspend 3-{1}{G}{G} (Rather than cast this card from your hand, pay {1}{G}{G} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)
Starting with you, each player may put an artifact, creature, enchantment, or land card from his or her hand onto the battlefield. Repeat this process until no one puts a card onto the battlefield.",NULL,"Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alaborn Trooper","2","3","W",NULL,"2W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thunder Totem","{T}: Add {W} to your mana pool.
{1}{W}{W}: Thunder Totem becomes a 2/2 white Spirit artifact creature with flying and first strike until end of turn.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veiled Apparition","U","When an opponent casts a spell, if Veiled Apparition is an enchantment, Veiled Apparition becomes a 3/3 Illusion creature with flying and ""At the beginning of your upkeep, sacrifice Veiled Apparition unless you pay {1}{U}.""","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Clergy of the Holy Nimbus","1","1","W","If Clergy of the Holy Nimbus would be destroyed, regenerate it.
{1}: Clergy of the Holy Nimbus can't be regenerated this turn. Only any opponent may activate this ability.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cave Sense","R","Enchant creature
Enchanted creature gets +1/+1 and has mountainwalk.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Erg Raiders","2","3","B","At the beginning of your end step, if Erg Raiders didn't attack this turn, Erg Raiders deals 2 damage to you unless it came under your control this turn.","1B","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raging Spirit","3","3","R","{2}: Raging Spirit becomes colorless until end of turn.","3R","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lotus Blossom","At the beginning of your upkeep, you may put a petal counter on Lotus Blossom.
{T}, Sacrifice Lotus Blossom: Add X mana of any one color to your mana pool, where X is the number of petal counters on Lotus Blossom.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Viridescent Wisps","G","Target creature becomes green and gets +1/+0 until end of turn.
Draw a card.","G","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rusting Golem","*","*","Fading 5 (This creature enters the battlefield with five fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Rusting Golem's power and toughness are each equal to the number of fade counters on it.","4","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Centaur Rootcaster","2","2","G","Whenever Centaur Rootcaster deals combat damage to a player, you may search your library for a basic land card and put that card onto the battlefield tapped. If you do, shuffle your library.","3G","Creature - Centaur Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Intimidation","B","Creatures you control have fear. (They can't be blocked except by artifact creatures and/or black creatures.)","2BBB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Misfortunes","B","Enchant player
At the beginning of your upkeep, you may search your library for a Curse card that doesn't have the same name as a Curse attached to enchanted player, put it onto the battlefield attached to that player, then shuffle your library.","4B","Enchantment - Aura Curse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ondu Giant","2","4","G","When Ondu Giant enters the battlefield, you may search your library for a basic land card, put it onto the battlefield tapped, then shuffle your library.","3G","Creature - Giant Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Murk Dwellers","2","2","B","Whenever Murk Dwellers attacks and isn't blocked, it gets +2/+0 until end of combat.","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Herd","G","When an opponent plays a nonbasic land, if Hidden Herd is an enchantment, Hidden Herd becomes a 3/3 Beast creature.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Choking Vines","G","Cast Choking Vines only during the declare blockers step.
X target attacking creatures become blocked. Choking Vines deals 1 damage to each of those creatures. (This spell works on unblockable creatures.)","XG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guardian of Vitu-Ghazi","4","7","WG","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Vigilance","6GW","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tainted Well","B","Enchant land
When Tainted Well enters the battlefield, draw a card.
Enchanted land is a Swamp.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spellweaver Volute","U","Enchant instant card in a graveyard
Whenever you cast a sorcery spell, copy the enchanted instant card. You may cast the copy without paying its mana cost. If you do, exile the enchanted card and attach Spellweaver Volute to another instant card in a graveyard.","3UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angelic Curator","1","1","W","Flying, protection from artifacts","1W","Creature - Angel Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devil's Play","R","Devil's Play deals X damage to target creature or player.
Flashback {X}{R}{R}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","XR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alpha Brawl","R","Target creature an opponent controls deals damage equal to its power to each other creature that player controls, then each of those creatures deals damage equal to its power to that creature.","6RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fury of the Horde","R","You may exile two red cards from your hand rather than pay Fury of the Horde's mana cost.
Untap all creatures that attacked this turn. After this main phase, there is an additional combat phase followed by an additional main phase.","5RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Lore","G","Search your library for a Forest card and put that card onto the battlefield. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Herald of War","3","3","W","Flying
Whenever Herald of War attacks, put a +1/+1 counter on it.
Angel spells and Human spells you cast cost {1} less to cast for each +1/+1 counter on Herald of War.","3WW","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Iron Guardian Stirs","When you set this scheme in motion, put a 4/6 colorless Golem artifact creature token onto the battlefield.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sinew Sliver","1","1","W","All Sliver creatures get +1/+1.","1W","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Faith","0","5","W","Defender (This creature can't attack.)
{W}: Wall of Faith gets +0/+1 until end of turn.","3W","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reality Ripple","U","Target artifact, creature, or land phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before its controller untaps during his or her next untap step.)","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Plotter","1","1","U","When Vedalken Plotter enters the battlefield, exchange control of target land you control and target land an opponent controls.","2U","Creature - Vedalken Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Galvanic Arc","R","Enchant creature
When Galvanic Arc enters the battlefield, it deals 3 damage to target creature or player.
Enchanted creature has first strike.","2R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recoil","UB","Return target permanent to its owner's hand. Then that player discards a card.","1UB","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trepanation Blade","Whenever equipped creature attacks, defending player reveals cards from the top of his or her library until he or she reveals a land card. The creature gets +1/+0 until end of turn for each card revealed this way. That player puts the revealed cards into his or her graveyard.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx Ambassador","5","5","U","Flying
Whenever Sphinx Ambassador deals combat damage to a player, search that player's library for a card, then that player names a card. If you searched for a creature card that isn't the named card, you may put it onto the battlefield under your control. Then that player shuffles his or her library.","5UU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cunning Giant","4","4","R","If Cunning Giant is unblocked, you may have it assign its combat damage to a creature defending player controls.","5R","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prey's Vengeance","G","Target creature gets +2/+2 until end of turn.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bösium Strip","{3}, {T}: Until end of turn, if the top card of your graveyard is an instant or sorcery card, you may cast that card. If a card cast this way would be put into a graveyard this turn, exile it instead.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giltspire Avenger","2","2","WUG","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
{T}: Destroy target creature that dealt damage to you this turn.","GWU","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Griffin Guide","W","Enchant creature
Enchanted creature gets +2/+2 and has flying.
When enchanted creature dies, put a 2/2 white Griffin creature token with flying onto the battlefield.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Feast","B","Target player loses 4 life and you gain 4 life.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saltfield Recluse","1","2","W","{T}: Target creature gets -2/-0 until end of turn.","2W","Creature - Human Rebel Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tethered Skirge","2","2","B","Flying
Whenever Tethered Skirge becomes the target of a spell or ability, you lose 1 life.","2B","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pernicious Deed","BG","{X}, Sacrifice Pernicious Deed: Destroy each artifact, creature, and enchantment with converted mana cost X or less.","1BG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Gore","BR","If a spell or ability would cause its controller to gain life, that player loses that much life instead.","BR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pendrell Mists","U","All creatures have ""At the beginning of your upkeep, sacrifice this creature unless you pay {1}.""","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Qasali Pridemage","2","2","WG","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
{1}, Sacrifice Qasali Pridemage: Destroy target artifact or enchantment.","GW","Creature - Cat Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Three Tragedies","B","Target player discards three cards.","3BB","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warrior's Honor","W","Creatures you control get +1/+1 until end of turn.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spitting Image","UG","Put a token that's a copy of target creature onto the battlefield.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","4(G/U)(G/U)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soaring Hope","W","Enchant creature
When Soaring Hope enters the battlefield, you gain 3 life.
Enchanted creature has flying.
{W}: Put Soaring Hope on top of its owner's library.","4W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bash to Bits","R","Destroy target artifact.
Flashback {4}{R}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chaoslace","R","Target spell or permanent becomes red. (Its mana symbols remain unchanged.)","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blisterstick Shaman","2","1","R","When Blisterstick Shaman enters the battlefield, it deals 1 damage to target creature or player.","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Farmer","2","2","R","{T}: Target land becomes a Swamp until its controller's next untap step.","1RR","Creature - Orc");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overrule","WU","Counter target spell unless its controller pays {X}. You gain X life.","XWU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mournful Zombie","2","1","B","{W}, {T}: Target player gains 1 life.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ink-Eyes, Servant of Oni","5","4","B","Ninjutsu {3}{B}{B} ({3}{B}{B}, Return an unblocked attacker you control to hand: Put this card onto the battlefield from your hand tapped and attacking.)
Whenever Ink-Eyes, Servant of Oni deals combat damage to a player, you may put target creature card from that player's graveyard onto the battlefield under your control.
{1}{B}: Regenerate Ink-Eyes.","4BB","Legendary Creature - Rat Ninja");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sliver Queen Avatar","Whenever you cast a non-Sliver creature spell, exile that spell. If you do, put a token onto the battlefield that's a copy of a random non-Shapeshifter Sliver creature card with the same converted mana cost as that spell.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ur-Golem's Eye","{T}: Add {2} to your mana pool.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("The Ultimate Nightmare of Wizards of the Coast® Customer Service","R","The Ultimate Nightmare of Wizards of the Coast® Customer Service deals X damage to each of Y target creatures and Z target players.","XYZRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Morbid Plunder","B","Return up to two target creature cards from your graveyard to your hand.","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Second Thoughts","W","Exile target attacking creature.
Draw a card.","4W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chimeric Mass","Chimeric Mass enters the battlefield with X charge counters on it.
{1}: Until end of turn, Chimeric Mass becomes a Construct artifact creature with ""This creature's power and toughness are each equal to the number of charge counters on it.""","X","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phage the Untouchable","4","4","B","When Phage the Untouchable enters the battlefield, if you didn't cast it from your hand, you lose the game.
Whenever Phage deals combat damage to a creature, destroy that creature. It can't be regenerated.
Whenever Phage deals combat damage to a player, that player loses the game.","3BBBB","Legendary Creature - Zombie Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Timetwister","U","Each player shuffles his or her hand and graveyard into his or her library, then draws seven cards. (Then put Timetwister into its owner's graveyard.)","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spidersilk Armor","G","Creatures you control get +0/+1 and have reach. (They can block creatures with flying.)","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sandstorm Eidolon","2","2","R","{R}, Sacrifice Sandstorm Eidolon: Target creature can't block this turn.
Whenever you cast a multicolored spell, you may return Sandstorm Eidolon from your graveyard to your hand.","3R","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heightened Awareness","U","As Heightened Awareness enters the battlefield, discard your hand.
At the beginning of your draw step, draw an additional card.","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raging Minotaur","3","3","R","Haste","2RR","Creature - Minotaur Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heartwood Storyteller","2","3","G","Whenever a player casts a noncreature spell, each of that player's opponents may draw a card.","1GG","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chain Stasis","U","You may tap or untap target creature. Then that creature's controller may pay {2}{U}. If the player does, he or she may copy this spell and may choose a new target for that copy.","U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Magebane Armor","Equipped creature gets +2/+4 and loses flying.
Prevent all noncombat damage that would be dealt to equipped creature.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Treva's Ruins","When Treva's Ruins enters the battlefield, sacrifice it unless you return a non-Lair land you control to its owner's hand.
{T}: Add {G}, {W}, or {U} to your mana pool.",NULL,"Land - Lair");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pacifism","W","Enchant creature
Enchanted creature can't attack or block.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consuming Ferocity","R","Enchant non-Wall creature
Enchanted creature gets +1/+0.
At the beginning of your upkeep, put a +1/+0 counter on enchanted creature. If that creature has three or more +1/+0 counters on it, it deals damage equal to its power to its controller, then destroy that creature and it can't be regenerated.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serene Heart","G","Destroy all Auras.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cultural Exchange","U","Choose any number of creatures target player controls. Choose the same number of creatures another target player controls. Those players exchange control of those creatures. (This effect lasts indefinitely.)","4UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turn the Tables","W","All combat damage that would be dealt to you this turn is dealt to target attacking creature instead.","3WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Havoc Demon","5","5","B","Flying
When Havoc Demon dies, all creatures get -5/-5 until end of turn.","5BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin War Drums","R","Each creature you control can't be blocked except by two or more creatures.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charmed Griffin","3","3","W","Flying
When Charmed Griffin enters the battlefield, each other player may put an artifact or enchantment card onto the battlefield from his or her hand.","3W","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hanweir Watchkeep","1","5","R","Defender
At the beginning of each upkeep, if no spells were cast last turn, transform Hanweir Watchkeep.","2R","Creature - Human Warrior Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nip Gwyllion","1","1","WB","Lifelink (Damage dealt by this creature also causes you to gain that much life.)","(W/B)","Creature - Hag");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Circling Vultures","3","2","B","Flying
You may discard Circling Vultures any time you could cast an instant.
At the beginning of your upkeep, sacrifice Circling Vultures unless you exile the top creature card of your graveyard.","B","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Border Patrol","1","6","W","Vigilance","4W","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kaboom!","R","Choose any number of target players. For each of those players, reveal cards from the top of your library until you reveal a nonland card. Kaboom deals damage equal to that card's converted mana cost to that player, then you put the revealed cards on the bottom of your library in any order.","4R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nether Spirit","2","2","B","At the beginning of your upkeep, if Nether Spirit is the only creature card in your graveyard, you may return Nether Spirit to the battlefield.","1BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treva, the Renewer","6","6","WUG","Flying
Whenever Treva, the Renewer deals combat damage to a player, you may pay {2}{W}. If you do, choose a color, then you gain 1 life for each permanent of that color.","3GWU","Legendary Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darkslick Shores","Darkslick Shores enters the battlefield tapped unless you control two or fewer other lands.
{T}: Add {U} or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lust for War","R","Enchant creature
Whenever enchanted creature becomes tapped, Lust for War deals 3 damage to that creature's controller.
Enchanted creature attacks each turn if able.","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marsh Viper","1","2","G","Whenever Marsh Viper deals damage to a player, that player gets two poison counters. (A player with ten or more poison counters loses the game.)","3G","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nim Abomination","3","4","B","At the beginning of your end step, if Nim Abomination is untapped, you lose 3 life.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Gold Myr","1","1","{T}: Add {W} to your mana pool.","2","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Keldon Mantle","R","Enchant creature
{B}: Regenerate enchanted creature.
{R}: Enchanted creature gets +1/+0 until end of turn.
{G}: Enchanted creature gains trample until end of turn.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quirion Dryad","1","1","G","Whenever you cast a white, blue, black, or red spell, put a +1/+1 counter on Quirion Dryad.","1G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightbringer","2","2","W","{T}, Sacrifice Lightbringer: Exile target black creature.","2W","Creature - Kor Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Cartographer","2","2","W","When Kor Cartographer enters the battlefield, you may search your library for a Plains card, put it onto the battlefield tapped, then shuffle your library.","3W","Creature - Kor Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minotaur Explorer","3","3","R","When Minotaur Explorer enters the battlefield, sacrifice it unless you discard a card at random.","1R","Creature - Minotaur Scout");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Garruk Relentless","G","When Garruk Relentless has two or fewer loyalty counters on him, transform him.
0: Garruk Relentless deals 3 damage to target creature. That creature deals damage equal to its power to him.
0: Put a 2/2 green Wolf creature token onto the battlefield.","Planeswalker - Garruk","3G");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scion of Oona","1","1","U","Flash
Flying
Other Faerie creatures you control get +1/+1.
Other Faeries you control have shroud. (A permanent with shroud can't be the target of spells or abilities.)","2U","Creature - Faerie Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghostly Changeling","2","2","B","Changeling (This card is every creature type at all times.)
{1}{B}: Ghostly Changeling gets +1/+1 until end of turn.","2B","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Regal Unicorn","2","3","W",NULL,"2W","Creature - Unicorn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Death Cultist","1","1","B","Sacrifice Death Cultist: Target player loses 1 life and you gain 1 life.","B","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lowland Giant","4","3","R",NULL,"2RR","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Victim of Night","B","Destroy target non-Vampire, non-Werewolf, non-Zombie creature.","BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudskate","2","2","U","Flying
Fading 3 (This creature enters the battlefield with three fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reknit","WG","Regenerate target permanent.","1(G/W)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lhurgoyf","*","1+*","G","Lhurgoyf's power is equal to the number of creature cards in all graveyards and its toughness is equal to that number plus 1.","2GG","Creature - Lhurgoyf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Belbe's Armor","{X}, {T}: Target creature gets -X/+X until end of turn.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ensouled Scimitar","{3}: Ensouled Scimitar becomes a 1/5 Spirit artifact creature with flying until end of turn. (Equipment that's a creature can't equip a creature.)
Equipped creature gets +1/+5.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patron Wizard","2","2","U","Tap an untapped Wizard you control: Counter target spell unless its controller pays {1}.","UUU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Rock Sled","3","1","R","Trample
Goblin Rock Sled doesn't untap during your untap step if it attacked during your last turn.
Goblin Rock Sled can't attack unless defending player controls a Mountain.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Hydra","0","1","R","Balduvian Hydra enters the battlefield with X +1/+0 counters on it.
Remove a +1/+0 counter from Balduvian Hydra: Prevent the next 1 damage that would be dealt to Balduvian Hydra this turn.
{R}{R}{R}: Put a +1/+0 counter on Balduvian Hydra. Activate this ability only during your upkeep.","XRR","Creature - Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Schismotivate","UR","Target creature gets +4/+0 until end of turn. Another target creature gets -4/-0 until end of turn.","1UR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Psionic Entity","2","2","U","{T}: Psionic Entity deals 2 damage to target creature or player and 3 damage to itself.","4U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wrap in Flames","R","Wrap in Flames deals 1 damage to each of up to three target creatures. Those creatures can't block this turn.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keiga, the Tide Star","5","5","U","Flying
When Keiga, the Tide Star dies, gain control of target creature.","5U","Legendary Creature - Dragon Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sovereigns of Lost Alara","4","5","WU","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
Whenever a creature you control attacks alone, you may search your library for an Aura card that could enchant that creature, put it onto the battlefield attached to that creature, then shuffle your library.","4WU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Berserker","1","1","G","Whenever Elvish Berserker becomes blocked, it gets +1/+1 until end of turn for each creature blocking it.","G","Creature - Elf Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Meloku the Clouded Mirror","2","4","U","Flying
{1}, Return a land you control to its owner's hand: Put a 1/1 blue Illusion creature token with flying onto the battlefield.","4U","Legendary Creature - Moonfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eldrazi Monument","Creatures you control get +1/+1, have flying, and are indestructible.
At the beginning of your upkeep, sacrifice a creature. If you can't, sacrifice Eldrazi Monument.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dirtcowl Wurm","3","4","G","Whenever an opponent plays a land, put a +1/+1 counter on Dirtcowl Wurm.","4G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fomori Nomad","4","4","R",NULL,"4R","Creature - Nomad Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Archer","1","1","G","{T}: Target creature with flying gets -1/-1 until end of turn.","G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enigma Sphinx","5","4","WUB","Flying
When Enigma Sphinx is put into your graveyard from the battlefield, put it into your library third from the top.
Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)","4WUB","Artifact Creature - Sphinx");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lashwrithe","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +1/+1 for each Swamp you control.
Equip {(b/p){(b/p)} ({(b/p)} can be paid with either {B} or 2 life.)","4","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bladewing's Thrall","3","3","B","Bladewing's Thrall has flying as long as you control a Dragon.
When a Dragon enters the battlefield, you may return Bladewing's Thrall from your graveyard to the battlefield.","2BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elvish Fury","G","Buyback {4} (You may pay an additional {4} as you cast this spell. If you do, put this card into your hand as it resolves.)
Target creature gets +2/+2 until end of turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Marauder","3","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)","2B","Creature - Dauthi Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reclusive Wight","4","4","B","At the beginning of your upkeep, if you control another nonland permanent, sacrifice Reclusive Wight.","3B","Creature - Zombie Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Animal Magnetism","G","Reveal the top five cards of your library. An opponent chooses a creature card from among them. Put that card onto the battlefield and the rest into your graveyard.","4G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brain Pry","B","Name a nonland card. Target player reveals his or her hand. That player discards a card with that name. If he or she can't, you draw a card.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Backfire","U","Enchant creature
Whenever enchanted creature deals damage to you, Backfire deals that much damage to that creature's controller.","U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Torpor Dust","UB","Flash
Enchant creature
Enchanted creature gets -3/-0.","2(U/B)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Miss Demeanor","3","1","W","Flying, first strike
During each other player's turn, compliment that player on his or her game play or sacrifice Miss Demeanor.","3W","Summon - Lady of Proper Etiquette");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stromkirk Captain","2","2","BR","First strike
Other Vampire creatures you control get +1/+1 and have first strike.","1BR","Creature - Vampire Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shared Fate","U","If a player would draw a card, that player exiles the top card of an opponent's library face down instead.
Each player may look at and play cards he or she exiled with Shared Fate.","4U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Koth's Courier","2","3","R","Forestwalk","1RR","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azusa, Lost but Seeking","1","2","G","You may play two additional lands on each of your turns.","2G","Legendary Creature - Human Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soulless One","*","*","B","Soulless One's power and toughness are each equal to the number of Zombies on the battlefield plus the number of Zombie cards in all graveyards.","3B","Creature - Zombie Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rescue","U","Return target permanent you control to its owner's hand.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Outlander","2","2","BR","Protection from white","BR","Creature - Goblin Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shinen of Fear's Chill","3","2","B","Shinen of Fear's Chill can't block.
Channel - {1}{B}, Discard Shinen of Fear's Chill: Target creature can't block this turn.","4B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Jar","3","3","U","{T}, Sacrifice Magus of the Jar: Each player exiles all cards from his or her hand face down and draws seven cards. At the beginning of the next end step, each player discards his or her hand and returns to his or her hand each card he or she exiled this way.","3UU","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mogg Alarm","R","You may sacrifice two Mountains rather than pay Mogg Alarm's mana cost.
Put two 1/1 red Goblin creature tokens onto the battlefield.","1RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aphetto Vulture","3","2","B","Flying
When Aphetto Vulture dies, you may put target Zombie card from your graveyard on top of your library.","4BB","Creature - Zombie Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maritime Guard","1","3","U",NULL,"1U","Creature - Merfolk Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cloak and Dagger","Equipped creature gets +2/+0 and has shroud. (It can't be the target of spells or abilities.)
Whenever a Rogue creature enters the battlefield, you may attach Cloak and Dagger to it.
Equip {3}","2","Tribal Artifact - Rogue Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinder Wall","3","3","R","Defender (This creature can't attack.)
When Cinder Wall blocks, destroy it at end of combat.","R","Creature - Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Colossus","8","8","Phyrexian Colossus doesn't untap during your untap step.
Pay 8 life: Untap Phyrexian Colossus.
Phyrexian Colossus can't be blocked except by three or more creatures.","7","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scrabbling Claws","{T}: Target player exiles a card from his or her graveyard.
{1}, Sacrifice Scrabbling Claws: Exile target card from a graveyard. Draw a card.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mishra's Groundbreaker","{T}, Sacrifice Mishra's Groundbreaker: Target land becomes a 3/3 artifact creature that's still a land. (This effect lasts indefinitely.)","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunder Spirit","2","2","W","Flying, first strike","1WW","Creature - Elemental Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Planar Gate","Creature spells you cast cost up to {2} less to cast.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vile Consumption","UB","All creatures have ""At the beginning of your upkeep, sacrifice this creature unless you pay 1 life.""","1UB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Endbringer's Revel","B","{4}: Return target creature card from a graveyard to its owner's hand. Any player may activate this ability but only any time he or she could cast a sorcery.","2B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moonlit Wake","W","Whenever a creature dies, you gain 1 life.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lunk Errant","4","4","R","Whenever Lunk Errant attacks alone, it gets +1/+1 and gains trample until end of turn.","5R","Creature - Giant Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Peat Bog","Peat Bog enters the battlefield tapped with two depletion counters on it.
{T}, Remove a depletion counter from Peat Bog: Add {B}{B} to your mana pool. If there are no depletion counters on Peat Bog, sacrifice it.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Infinite Hourglass","At the beginning of your upkeep, put a time counter on Infinite Hourglass.
All creatures get +1/+0 for each time counter on Infinite Hourglass.
{3}: Remove a time counter from Infinite Hourglass. Any player may activate this ability but only during any upkeep step.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("True Conviction","W","Creatures you control have double strike and lifelink.","3WWW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Organ Harvest","B","You and your teammates may sacrifice any number of creatures. For each creature sacrificed in this way, add oBoB to your mana pool.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aspect of Wolf","G","Enchant creature
Enchanted creature gets +X/+Y, where X is half the number of Forests you control, rounded down, and Y is half the number of Forests you control, rounded up.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Epic Proportions","G","Flash
Enchant creature
Enchanted creature gets +5/+5 and has trample.","4GG","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jinxed Choker","At the beginning of your end step, target opponent gains control of Jinxed Choker and puts a charge counter on it.
At the beginning of your upkeep, Jinxed Choker deals damage to you equal to the number of charge counters on it.
{3}: Put a charge counter on Jinxed Choker or remove one from it.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kalonian Behemoth","9","9","G","Shroud (This creature can't be the target of spells or abilities.)","5GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Medics","1","1","R","Whenever Goblin Medics becomes tapped, it deals 1 damage to target creature or player.","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shuriken","Equipped creature has ""{T}, Unattach Shuriken: Shuriken deals 2 damage to target creature. That creature's controller gains control of Shuriken unless it was unattached from a Ninja.""
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Misshapen Fiend","1","1","B","Flying","1B","Creature - Horror Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trial // Error","UB","Counter target multicolored spell.
---
Return all creatures blocking or blocked by target creature to their owner's hand.","UB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Naya Sojourners","5","3","WRG","When you cycle Naya Sojourners or it dies, you may put a +1/+1 counter on target creature.
Cycling {2}{G} ({2}{G}, Discard this card: Draw a card.)","2RGW","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Obliterator","5","5","B","Trample
Whenever a source deals damage to Phyrexian Obliterator, that source's controller sacrifices that many permanents.","BBBB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Urborg Justice","B","Target opponent sacrifices a creature for each creature put into your graveyard from the battlefield this turn.","BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Wood","0","3","G","Defender (This creature can't attack.)","G","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Tusker","6","5","G","Cycling {2}{G} ({2}{G}, Discard this card: Draw a card.)
When you cycle Krosan Tusker, you may search your library for a basic land card, reveal that card, put it into your hand, then shuffle your library.","5GG","Creature - Boar Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blizzard Specter","2","3","UB","Flying
Whenever Blizzard Specter deals combat damage to a player, choose one - That player returns a permanent he or she controls to its owner's hand; or that player discards a card.","2UB","Snow Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Venerable Monk","2","2","W","When Venerable Monk enters the battlefield, you gain 2 life.","2W","Creature - Human Monk Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Approach My Molten Realm","When you set this scheme in motion, until your next turn, if a source would deal damage, it deals double that damage instead.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crown of Suspicion","B","Enchant creature
Enchanted creature gets +2/-1.
Sacrifice Crown of Suspicion: Enchanted creature and other creatures that share a creature type with it get +2/-1 until end of turn.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporary Truce","W","Each player may draw up to two cards. For each card less than two a player draws this way, that player gains 2 life.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Leveler","10","10","When Leveler enters the battlefield, exile all cards from your library.","5","Artifact Creature - Juggernaut");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ink-Eyes, Servant of Oni Avatar","At the beginning of the game, look at target opponent's hand and choose a nonland card from it. That player discards that card.
{X}, Pay X life: Put target creature card with converted mana cost X from an opponent's graveyard onto the battlefield under your control.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nether Traitor","1","1","B","Haste
Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever another creature is put into your graveyard from the battlefield, you may pay {B}. If you do, return Nether Traitor from your graveyard to the battlefield.","BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wolfbitten Captive","1","1","G","{1}{G}: Wolfbitten Captive gets +2/+2 until end of turn. Activate this ability only once each turn.
At the beginning of each upkeep, if no spells were cast last turn, transform Wolfbitten Captive.","G","Creature - Human Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sorceress Queen","1","1","B","{T}: Target creature other than Sorceress Queen becomes 0/2 until end of turn.","1BB","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin War Strike","R","Goblin War Strike deals damage equal to the number of Goblins you control to target player.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Settler","1","1","R","When Goblin Settler enters the battlefield, destroy target land.","3R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghoulflesh","B","Enchant creature
Enchanted creature gets -1/-1 and is a black Zombie in addition to its other colors and types.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Surrakar Spellblade","2","1","U","Whenever you cast an instant or sorcery spell, you may put a charge counter on Surrakar Spellblade.
Whenever Surrakar Spellblade deals combat damage to a player, you may draw X cards, where X is the number of charge counters on it.","1UU","Creature - Surrakar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Torrent of Souls","BR","Return up to one target creature card from your graveyard to the battlefield if {B} was spent to cast Torrent of Souls. Creatures target player controls get +2/+0 and gain haste until end of turn if {R} was spent to cast Torrent of Souls. (Do both if {B}{R} was spent.)","4(B/R)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gelid Shackles","W","Enchant creature
Enchanted creature can't block, and its activated abilities can't be activated.
{S}i}: Enchanted creature gains defender until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","W","Snow Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghoul's Feast","B","Target creature gets +X/+0 until end of turn, where X is the number of creature cards in your graveyard.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harvest Pyre","R","As an additional cost to cast Harvest Pyre, exile X cards from your graveyard.
Harvest Pyre deals X damage to target creature.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deadhead","3","3","B","Put Deadhead into play. Use this ability only if any opponent loses contact with his or her hand of cards and only if Deadhead is in your graveyard.","3B","Summon - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greenseeker","1","1","G","{G}, {T}, Discard a card: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.","G","Creature - Elf Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crosis's Charm","UBR","Choose one - Return target permanent to its owner's hand; or destroy target nonblack creature, and it can't be regenerated; or destroy target artifact.","UBR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jester's Mask","Jester's Mask enters the battlefield tapped.
{1}, {T}, Sacrifice Jester's Mask: Target opponent puts the cards from his or her hand on top of his or her library. Search that player's library for that many cards. That player puts those cards into his or her hand, then shuffles his or her library.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grizzled Outcasts","4","4","G","At the beginning of each upkeep, if no spells were cast last turn, transform Grizzled Outcasts.","4G","Creature - Human Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Aggressor","3","2","R","Kicker {4} (You may pay an additional {4} as you cast this spell.)
Kavu Aggressor can't block.
If Kavu Aggressor was kicked, it enters the battlefield with a +1/+1 counter on it.","2R","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Power Conduit","{T}, Remove a counter from a permanent you control: Choose one - Put a charge counter on target artifact; or put a +1/+1 counter on target creature.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gaea's Liege","*","*","G","As long as Gaea's Liege isn't attacking, its power and toughness are each equal to the number of Forests you control. As long as Gaea's Liege is attacking, its power and toughness are each equal to the number of Forests defending player controls.
{T}: Target land becomes a Forest until Gaea's Liege leaves the battlefield.","3GGG","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rumbling Crescendo","R","At the beginning of your upkeep, you may put a verse counter on Rumbling Crescendo.
{R}, Sacrifice Rumbling Crescendo: Destroy up to X target lands, where X is the number of verse counters on Rumbling Crescendo.","3RR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fallen Angel Avatar","Whenever a creature you control dies, target opponent loses 1 life and you gain 1 life.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Horizon Spellbomb","{2}, {T}, Sacrifice Horizon Spellbomb: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.
When Horizon Spellbomb is put into a graveyard from the battlefield, you may pay {G}. If you do, draw a card.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Powder Keg","At the beginning of your upkeep, you may put a fuse counter on Powder Keg.
{T}, Sacrifice Powder Keg: Destroy each artifact and creature with converted mana cost equal to the number of fuse counters on Powder Keg.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Congregation at Dawn","WG","Search your library for up to three creature cards and reveal them. Shuffle your library, then put those cards on top of it in any order.","GGW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kulrath Knight","3","3","BR","Flying
Wither (This deals damage to creatures in the form of -1/-1 counters.)
Creatures your opponents control with counters on them can't attack or block.","3(B/R)(B/R)","Creature - Elemental Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tattoo Ward","W","Enchant creature
Enchanted creature gets +1/+1 and has protection from enchantments. This effect doesn't remove Tattoo Ward.
Sacrifice Tattoo Ward: Destroy target enchantment.","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Changeling Hero","4","4","W","Changeling (This card is every creature type at all times.)
Champion a creature (When this enters the battlefield, sacrifice it unless you exile another creature you control. When this leaves the battlefield, that card returns to the battlefield.)
Lifelink (Damage dealt by this creature also causes you to gain that much life.)","4W","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodlord of Vaasgoth","3","3","B","Bloodthirst 3 (If an opponent was dealt damage this turn, this creature enters the battlefield with three +1/+1 counters on it.)
Flying
Whenever you cast a Vampire creature spell, it gains bloodthirst 3.","3BB","Creature - Vampire Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Accorder's Shield","Equipped creature gets +0/+3 and has vigilance.
Equip {3}","0","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Essence Warden","1","1","G","Whenever another creature enters the battlefield, you gain 1 life.","G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cudgel Troll","4","3","G","{G}: Regenerate Cudgel Troll. (The next time this creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","2GG","Creature - Troll");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dragon's Claw","Whenever a player casts a red spell, you may gain 1 life.","2","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Triskelavus","1","1","Flying
Triskelavus enters the battlefield with three +1/+1 counters on it.
{1}, Remove a +1/+1 counter from Triskelavus: Put a 1/1 colorless Triskelavite artifact creature token with flying onto the battlefield. It has ""Sacrifice this creature: This creature deals 1 damage to target creature or player.""","7","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skillful Lunge","W","Target creature gets +2/+0 and gains first strike until end of turn.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brawn","3","3","G","Trample
As long as Brawn is in your graveyard and you control a Forest, creatures you control have trample.","3G","Creature - Incarnation");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seize the Soul","B","Destroy target nonwhite, nonblack creature. Put a 1/1 white Spirit creature token with flying onto the battlefield.
Haunt (When this spell card is put into a graveyard after resolving, exile it haunting target creature.)
When the creature Seize the Soul haunts dies, destroy target nonwhite, nonblack creature. Put a 1/1 white Spirit creature token with flying onto the battlefield.","2BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skeletal Vampire","3","3","B","Flying
When Skeletal Vampire enters the battlefield, put two 1/1 black Bat creature tokens with flying onto the battlefield.
{3}{B}{B}, Sacrifice a Bat: Put two 1/1 black Bat creature tokens with flying onto the battlefield.
Sacrifice a Bat: Regenerate Skeletal Vampire.","4BB","Creature - Vampire Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hearthfire Hobgoblin","2","2","WR","Double strike","(R/W)(R/W)(R/W)","Creature - Goblin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mountain Goat","1","1","R","Mountainwalk","R","Creature - Goat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seize the Day","R","Untap target creature. After this main phase, there is an additional combat phase followed by an additional main phase.
Flashback {2}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necroplasm","1","1","B","At the beginning of your upkeep, put a +1/+1 counter on Necroplasm.
At the beginning of your end step, destroy each creature with converted mana cost equal to the number of +1/+1 counters on Necroplasm.
Dredge 2 (If you would draw a card, instead you may put exactly two cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","1BB","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Force of Will","U","You may pay 1 life and exile a blue card from your hand rather than pay Force of Will's mana cost.
Counter target spell.","3UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fatal Blow","B","Destroy target creature that was dealt damage this turn. It can't be regenerated.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wu Scout","1","1","U","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
When Wu Scout enters the battlefield, look at target opponent's hand.","1U","Creature - Human Soldier Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Intervention Pact","The next time a source of your choice would deal damage to you this turn, prevent that damage. You gain life equal to the damage prevented this way.
At the beginning of your next upkeep, pay {1}{W}{W}. If you don't, you lose the game.","0","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bold Defense","W","Kicker {3}{W} (You may pay an additional {3}{W} as you cast this spell.)
Creatures you control get +1/+1 until end of turn. If Bold Defense was kicked, instead creatures you control get +2/+2 and gain first strike until end of turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravecrawler","2","1","B","Gravecrawler can't block.
You may cast Gravecrawler from your graveyard as long as you control a Zombie.","B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of Yawgmoth","2","2","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Whenever Order of Yawgmoth deals damage to a player, that player discards a card.","2BB","Creature - Zombie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fungus Sliver","2","2","G","All Sliver creatures have ""Whenever this creature is dealt damage, put a +1/+1 counter on it."" (The damage is dealt before the counter is put on.)","3G","Creature - Fungus Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Markov Blademaster","1","1","R","Double strike
Whenever Markov Blademaster deals combat damage to a player, put a +1/+1 counter on it.","1RR","Creature - Vampire Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sanctum Custodian","1","2","W","{T}: Prevent the next 2 damage that would be dealt to target creature or player this turn.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thalakos Drifters","3","3","U","Discard a card: Thalakos Drifters gains shadow until end of turn. (This creature can block or be blocked by only creatures with shadow.)","2UU","Creature - Thalakos");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of the Bloody Tome","U","Enchant player
At the beginning of enchanted player's upkeep, that player puts the top two cards of his or her library into his or her graveyard.","2U","Enchantment - Aura Curse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invigorate","G","If you control a Forest, rather than pay Invigorate's mana cost, you may have an opponent gain 3 life.
Target creature gets +4/+4 until end of turn.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Distortion Strike","U","Target creature gets +1/+0 until end of turn and is unblockable this turn.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fire at Will","WR","Fire at Will deals 3 damage divided as you choose among one, two, or three target attacking or blocking creatures.","(R/W)(R/W)(R/W)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serra's Embrace","W","Enchant creature
Enchanted creature gets +2/+2 and has flying and vigilance.","2WW","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scout's Warning","W","The next creature card you play this turn can be played as though it had flash.
Draw a card.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Intangible Virtue","W","Creature tokens you control get +1/+1 and have vigilance.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Illusionist","2","2","U","{U}{U}, {T}: Target creature phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before its controller untaps during his or her next untap step.)","2U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mold Shambler","3","3","G","Kicker {1}{G} (You may pay an additional {1}{G} as you cast this spell.)
When Mold Shambler enters the battlefield, if it was kicked, destroy target noncreature permanent.","3G","Creature - Fungus Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bandage","W","Prevent the next 1 damage that would be dealt to target creature or player this turn.
Draw a card.","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vedalken Orrery","You may cast nonland cards as though they had flash.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jade Statue","{2}: Jade Statue becomes a 3/6 Golem artifact creature until end of combat. Activate this ability only during combat.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Treacherous Link","B","Enchant creature
All damage that would be dealt to enchanted creature is dealt to its controller instead.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Howling Gale","G","Howling Gale deals 1 damage to each creature with flying and each player.
Flashback {1}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Favor of the Overbeing","UG","Enchant creature
As long as enchanted creature is green, it gets +1/+1 and has vigilance.
As long as enchanted creature is blue, it gets +1/+1 and has flying.","1(G/U)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gnarlid Pack","2","2","G","Multikicker {1}{G} (You may pay an additional {1}{G} any number of times as you cast this spell.)
Gnarlid Pack enters the battlefield with a +1/+1 counter on it for each time it was kicked.","1G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veil of Birds","U","When an opponent casts a spell, if Veil of Birds is an enchantment, Veil of Birds becomes a 1/1 Bird creature with flying.","U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grave Exchange","B","Return target creature card from your graveyard to your hand. Target player sacrifices a creature.","4BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breaking Wave","U","You may cast Breaking Wave any time you could cast an instant if you pay {2} more to cast it.
Simultaneously untap all tapped creatures and tap all untapped creatures.","2UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Savage Gorilla","3","3","G","{U}{B}, {T}, Sacrifice Savage Gorilla: Target creature gets -3/-3 until end of turn. Draw a card.","4G","Creature - Ape");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Selesnya Sanctuary","Selesnya Sanctuary enters the battlefield tapped.
When Selesnya Sanctuary enters the battlefield, return a land you control to its owner's hand.
{T}: Add {G}{W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rising Waters","U","Lands don't untap during their controllers' untap steps.
At the beginning of each player's upkeep, that player untaps a land he or she controls.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Snapsail Glider","2","2","Metalcraft - Snapsail Glider has flying as long as you control three or more artifacts.","3","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Fissure","U","Return target permanent to its owner's hand.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","4U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mire Kavu","3","2","R","Mire Kavu gets +1/+1 as long as you control a Swamp.","3R","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Streetbreaker Wurm","6","4","RG",NULL,"3RG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Militant Monk","2","1","W","Vigilance
{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","1WW","Creature - Human Monk Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tresserhorn Sinks","Tresserhorn Sinks enters the battlefield tapped.
{T}: Add {B} or {R} to your mana pool.",NULL,"Snow Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scabland","Scabland enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{T}: Add {R} or {W} to your mana pool. Scabland deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reliquary Monk","2","2","W","When Reliquary Monk dies, destroy target artifact or enchantment.","2W","Creature - Human Monk Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gustcloak Cavalier","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
Whenever Gustcloak Cavalier attacks, you may tap target creature.
Whenever Gustcloak Cavalier becomes blocked, you may untap Gustcloak Cavalier and remove it from combat.","3WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stroke of Genius","U","Target player draws X cards.","X2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raksha Golden Cub","3","4","W","Vigilance
As long as Raksha Golden Cub is equipped, Cat creatures you control get +2/+2 and have double strike.","5WW","Legendary Creature - Cat Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tormentor's Trident","Equipped creature gets +3/+0 and attacks each turn if able.
Equip {3}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fact or Fiction","U","Reveal the top five cards of your library. An opponent separates those cards into two piles. Put one pile into your hand and the other into your graveyard.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sakura-Tribe Elder","1","1","G","Sacrifice Sakura-Tribe Elder: Search your library for a basic land card, put that card onto the battlefield tapped, then shuffle your library.","1G","Creature - Snake Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bribery","U","Search target opponent's library for a creature card and put that card onto the battlefield under your control. Then that player shuffles his or her library.","3UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wayward Soul","3","2","U","Flying
{U}: Put Wayward Soul on top of its owner's library.","2UU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mitotic Slime","4","4","G","When Mitotic Slime dies, put two 2/2 green Ooze creature tokens onto the battlefield. They have ""When this creature dies, put two 1/1 green Ooze creature tokens onto the battlefield.""","4G","Creature - Ooze");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steppe Lynx","0","1","W","Landfall - Whenever a land enters the battlefield under your control, Steppe Lynx gets +2/+2 until end of turn.","W","Creature - Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Al-abara's Carpet","{5}, {T}: Prevent all damage that would be dealt to you this turn by attacking creatures without flying.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Driller","2","2","R","{T}: Destroy target land unless its controller has Dwarven Driller deal 2 damage to him or her.","3R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sanity Grinding","U","Chroma - Reveal the top ten cards of your library. For each blue mana symbol in the mana costs of the revealed cards, target opponent puts the top card of his or her library into his or her graveyard. Then put the cards you revealed this way on the bottom of your library in any order.","UUU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Malicious Advice","UB","Tap X target artifacts, creatures, and/or lands. You lose X life.","XUB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brave the Elements","W","Choose a color. White creatures you control gain protection from the chosen color until end of turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Elite Guard","2","2","W","{T}: Target creature gets +2/+2 until end of turn. When that creature leaves the battlefield this turn, sacrifice Kjeldoran Elite Guard. Activate this ability only during combat.","3W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tree of Tales","(Tree of Tales isn't a spell.)
{T}: Add {G} to your mana pool.",NULL,"Artifact Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tower of Calamities","{8}, {T}: Tower of Calamities deals 12 damage to target creature.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guise of Fire","R","Enchant creature
Enchanted creature gets +1/-1 and attacks each turn if able.","R","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shadowblood Ridge","{1}, {T}: Add {B}{R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Rockspeaker","1","1","R","When Akki Rockspeaker enters the battlefield, add {R} to your mana pool.","1R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viscera Dragger","3","3","B","Cycling {2} ({2}, Discard this card: Draw a card.)
Unearth {1}{B} ({1}{B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","3B","Creature - Zombie Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Runeclaw Bear","2","2","G",NULL,"1G","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cateran Kidnappers","4","2","B","{3}, {T}: Search your library for a Mercenary permanent card with converted mana cost 3 or less and put it onto the battlefield. Then shuffle your library.","2BB","Creature - Human Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glint-Eye Nephilim","2","2","UBRG","Whenever Glint-Eye Nephilim deals combat damage to a player, draw that many cards.
{1}, Discard a card: Glint-Eye Nephilim gets +1/+1 until end of turn.","UBRG","Creature - Nephilim");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sengir Nosferatu","4","4","B","Flying
{1}{B}, Exile Sengir Nosferatu: Put a 1/2 black Bat creature token with flying onto the battlefield. It has ""{1}{B}, Sacrifice this creature: Return an exiled card named Sengir Nosferatu to the battlefield under its owner's control.""","3BB","Creature - Vampire");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gemstone Caverns","If Gemstone Caverns is in your opening hand and you're not playing first, you may begin the game with Gemstone Caverns on the battlefield with a luck counter on it. If you do, exile a card from your hand.
{T}: Add {1} to your mana pool. If Gemstone Caverns has a luck counter on it, instead add one mana of any color to your mana pool.",NULL,"Legendary Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Twilight Mire","{T}: Add {1} to your mana pool.
{(b/g)}, {T}: Add {B}{B}, {B}{G}, or {G}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exsanguinate","B","Each opponent loses X life. You gain life equal to the life lost this way.","XBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jötun Grunt","4","4","W","Cumulative upkeep-Put two cards from a single graveyard on the bottom of their owner's library. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","1W","Creature - Giant Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitting Gourna","3","4","G","Reach (This creature can block creatures with flying.)
Morph {4}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scent of Jasmine","W","Reveal any number of white cards in your hand. You gain 2 life for each card revealed this way.","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chronatog Avatar","You have no maximum hand size.
{0}: Draw three cards. You skip your next turn. Activate this ability only once each turn.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Khalni Garden","Khalni Garden enters the battlefield tapped.
When Khalni Garden enters the battlefield, put a 0/1 green Plant creature token onto the battlefield.
{T}: Add {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elven Fortress","G","{1}{G}: Target blocking creature gets +0/+1 until end of turn.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Growth","G","Kicker-Sacrifice a creature. (You may sacrifice a creature in addition to any other costs as you cast this spell.)
Search your library for a basic land card, put that card onto the battlefield, then shuffle your library. If Primal Growth was kicked, instead search your library for two basic land cards, put them onto the battlefield, then shuffle your library.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Immortal Coil","B","{T}, Exile two cards from your graveyard: Draw a card.
If damage would be dealt to you, prevent that damage. Exile a card from your graveyard for each 1 damage prevented this way.
When there are no cards in your graveyard, you lose the game.","2BB","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mouth to Mouth","U","You and target opponent have a breath-holding contest. If you win, you gain control of target creature that player controls.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undead Executioner","2","2","B","When Undead Executioner dies, you may have target creature get -2/-2 until end of turn.","3B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shinka Gatekeeper","3","2","R","Whenever Shinka Gatekeeper is dealt damage, it deals that much damage to you.","2R","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kumano, Master Yamabushi","4","4","R","{1}{R}: Kumano, Master Yamabushi deals 1 damage to target creature or player.
If a creature dealt damage by Kumano this turn would die, exile it instead.","3RR","Legendary Creature - Human Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Unholy Citadel","Black legendary creatures you control have ""bands with other legendary creatures."" (Any legendary creatures can attack in a band as long as at least one has ""bands with other legendary creatures."" Bands are blocked as a group. If at least two legendary creatures you control, one of which has ""bands with other legendary creatures,"" are blocking or being blocked by the same creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skull Collector","3","3","B","At the beginning of your upkeep, return a black creature you control to its owner's hand.
{1}{B}: Regenerate Skull Collector.","1BB","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Somberwald Spider","2","4","G","Reach (This creature can block creatures with flying.)
Morbid - Somberwald Spider enters the battlefield with two +1/+1 counters on it if a creature died this turn.","4G","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormfront Riders","4","3","W","Flying
When Stormfront Riders enters the battlefield, return two creatures you control to their owner's hand.
Whenever Stormfront Riders or another creature is returned to your hand from the battlefield, put a 1/1 white Soldier creature token onto the battlefield.","4W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunhome Enforcer","2","4","WR","Whenever Sunhome Enforcer deals combat damage, you gain that much life.
{1}{R}: Sunhome Enforcer gets +1/+0 until end of turn.","2RW","Creature - Giant Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seeds of Innocence","G","Destroy all artifacts. They can't be regenerated. The controller of each artifact destroyed this way gains life equal to its converted mana cost.","1GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Ancients","2","7","G","{G}: Yavimaya Ancients gets +1/-2 until end of turn.","3GG","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parallel Evolution","G","For each creature token on the battlefield, its controller puts a token that's a copy of that creature onto the battlefield.
Flashback {4}{G}{G}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Worthy Cause","W","Buyback {2} (You may pay an additional {2} as you cast this spell. If you do, put this card into your hand as it resolves.)
As an additional cost to cast Worthy Cause, sacrifice a creature.
You gain life equal to the sacrificed creature's toughness.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fa'adiyah Seer","1","1","G","{T}: Draw a card and reveal it. If it isn't a land card, discard it.","1G","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ridgetop Raptor","2","1","R","Double strike (This creature deals both first-strike and regular combat damage.)","3R","Creature - Lizard Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horned Cheetah","2","2","WG","Whenever Horned Cheetah deals damage, you gain that much life.","2GW","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitting Spider","3","5","G","Reach (This creature can block creatures with flying.)
Sacrifice a land: Spitting Spider deals 1 damage to each creature with flying.","3GG","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soulcatcher","1","1","W","Flying
Whenever a creature with flying dies, put a +1/+1 counter on Soulcatcher.","1W","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonecloaker","3","2","W","Flash (You may cast this spell any time you could cast an instant.)
Flying
When Stonecloaker enters the battlefield, return a creature you control to its owner's hand.
When Stonecloaker enters the battlefield, exile target card from a graveyard.","2W","Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dewdrop Spy","2","2","U","Flash
Flying
When Dewdrop Spy enters the battlefield, look at the top card of target player's library.","1UU","Creature - Faerie Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dissipation Field","U","Whenever a permanent deals damage to you, return it to its owner's hand.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gerrard's Irregulars","4","2","R","Trample, haste","4R","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Derelor","4","4","B","Black spells you cast cost {B} more to cast.","3B","Creature - Thrull");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lumengrid Drake","2","2","U","Flying
Metalcraft - When Lumengrid Drake enters the battlefield, if you control three or more artifacts, return target creature to its owner's hand.","3U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of the Painted Road","3","3","W","Whenever you cast a Spirit or Arcane spell, Kami of the Painted Road gains protection from the color of your choice until end of turn.","4W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Silent Arbiter","1","5","No more than one creature can attack each combat.
No more than one creature can block each combat.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian War Machine","0","4","U","Defender (This creature can't attack.)
Tap an untapped Merfolk you control: Vodalian War Machine can attack this turn as though it didn't have defender.
Tap an untapped Merfolk you control: Vodalian War Machine gets +2/+1 until end of turn.
When Vodalian War Machine dies, destroy all Merfolk tapped this turn to pay for its abilities.","1UU","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soulcatchers' Aerie","W","Whenever a Bird is put into your graveyard from the battlefield, put a feather counter on Soulcatchers' Aerie.
Bird creatures get +1/+1 for each feather counter on Soulcatchers' Aerie.","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Slumbering Tora","{2}, Discard a Spirit or Arcane card: Slumbering Tora becomes an X/X Cat artifact creature until end of turn, where X is the discarded card's converted mana cost.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Field Surgeon","1","1","W","Tap an untapped creature you control: Prevent the next 1 damage that would be dealt to target creature this turn.","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blightwidow","2","4","G","Reach (This creature can block creatures with flying.)
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","3G","Creature - Spider");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Antler Skulkin","3","3","{2}: Target white creature gains persist until end of turn. (When it dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","5","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brine Shaman","1","1","B","{T}, Sacrifice a creature: Target creature gets +2/+2 until end of turn.
{1}{U}{U}, Sacrifice a creature: Counter target creature spell.","1B","Creature - Human Cleric Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Emmessi Tome","{5}, {T}: Draw two cards, then discard a card.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elite Archers","3","3","W","{T}: Elite Archers deals 3 damage to target attacking or blocking creature.","5W","Creature - Human Soldier Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crib Swap","W","Changeling (This card is every creature type at all times.)
Exile target creature. Its controller puts a 1/1 colorless Shapeshifter creature token with changeling onto the battlefield.","2W","Tribal Instant - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sphere of Grace","W","If a black source would deal damage to you, prevent 2 of that damage.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heidar, Rimewind Master","3","3","U","{2}, {T}: Return target permanent to its owner's hand. Activate this ability only if you control four or more snow permanents.","4U","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crystal Seer","2","2","U","When Crystal Seer enters the battlefield, look at the top four cards of your library, then put them back in any order.
{4}{U}: Return Crystal Seer to its owner's hand.","4U","Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ebon Praetor","5","5","B","First strike, trample
At the beginning of your upkeep, put a -2/-2 counter on Ebon Praetor.
Sacrifice a creature: Remove a -2/-2 counter from Ebon Praetor. If the sacrificed creature was a Thrull, put a +1/+0 counter on Ebon Praetor. Activate this ability only during your upkeep and only once each turn.","4BB","Creature - Avatar Praetor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daru Lancer","3","4","W","First strike
Morph {2}{W}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alluring Siren","1","1","U","{T}: Target creature an opponent controls attacks you this turn if able.","1U","Creature - Siren");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reveille Squad","3","3","W","Whenever one or more creatures attack you, if Reveille Squad is untapped, you may untap all creatures you control.","2WW","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scourge Servant","3","3","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","4B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exhaustion","U","Creatures and lands target opponent controls don't untap during his or her next untap step.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shorecrasher Mimic","2","1","UG","Whenever you cast a spell that's both green and blue, Shorecrasher Mimic becomes 5/3 and gains trample until end of turn.","1(G/U)","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reverent Silence","G","If you control a Forest, rather than pay Reverent Silence's mana cost, you may have each other player gain 6 life.
Destroy all enchantments.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blackcleave Goblin","2","1","B","Haste
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","3B","Creature - Goblin Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Morale","W","Attacking creatures get +1/+1 until end of turn.","1WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planar Birth","W","Return all basic land cards from all graveyards to the battlefield tapped under their owners' control.","1W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Favorable Destiny","W","Enchant creature
Enchanted creature gets +1/+2 as long as it's white.
Enchanted creature has shroud as long as its controller controls another creature. (It can't be the target of spells or abilities.)","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Banshee","0","1","B","{X}, {T}: Banshee deals half X damage, rounded down, to target creature or player, and half X damage, rounded up, to you.","2BB","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragonstorm","R","Search your library for a Dragon permanent card and put it onto the battlefield. Then shuffle your library.
Storm (When you cast this spell, copy it for each spell cast before it this turn.)","8R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gempalm Avenger","3","5","W","Cycling {2}{W} ({2}{W}, Discard this card: Draw a card.)
When you cycle Gempalm Avenger, Soldier creatures get +1/+1 and gain first strike until end of turn.","5W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fiery Temper","R","Fiery Temper deals 3 damage to target creature or player.
Madness {R} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","1RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Choking Sands","B","Destroy target non-Swamp land. If that land was nonbasic, Choking Sands deals 2 damage to the land's controller.","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wrath of Marit Lage","U","When Wrath of Marit Lage enters the battlefield, tap all red creatures.
Red creatures don't untap during their controllers' untap steps.","3UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mortiphobia","B","{1}{B}, Discard a card: Exile target card from a graveyard.
{1}{B}, Sacrifice Mortiphobia: Exile target card from a graveyard.","1BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hoodwink","U","Return target artifact, enchantment, or land to its owner's hand.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akroma, Angel of Fury","6","6","R","Akroma, Angel of Fury can't be countered.
Flying, trample, protection from white and from blue
{R}: Akroma, Angel of Fury gets +1/+0 until end of turn.
Morph {3}{R}{R}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","5RRR","Legendary Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quicksilver Geyser","U","Return up to two target nonland permanents to their owners' hands.","4U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Samite Ministration","W","Prevent all damage that would be dealt to you this turn by a source of your choice. Whenever damage from a black or red source is prevented this way this turn, you gain that much life.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Albino Troll","3","3","G","Echo {1}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
{1}{G}: Regenerate Albino Troll.","1G","Creature - Troll");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oboro Breezecaller","1","1","U","Flying
{2}, Return a land you control to its owner's hand: Untap target land.","1U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chaosphere","R","Creatures with flying can block only creatures with flying.
Creatures without flying have reach. (They can block creatures with flying.)","2R","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Postmortem Lunge","B","({(b/p)} can be paid with either {B} or 2 life.)
Return target creature card with converted mana cost X from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step.","X(B/P)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Word of Binding","B","Tap X target creatures.","XBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arc Runner","5","1","R","Haste (This creature can attack and {T} as soon as it comes under your control.)
At the beginning of the end step, sacrifice Arc Runner.","2R","Creature - Elemental Ox");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mole Worms","1","1","B","You may choose not to untap Mole Worms during your untap step.
{T}: Tap target land. It doesn't untap during its controller's untap step for as long as Mole Worms remains tapped.","2B","Creature - Worm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Thorns","G","Choose one or more - Destroy target artifact; destroy target enchantment; and/or destroy target land.","4GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lone Revenant","4","4","U","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Whenever Lone Revenant deals combat damage to a player, if you control no other creatures, look at the top four cards of your library. Put one of them into your hand and the rest on the bottom of your library in any order.","3UU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bogardan Firefiend","2","1","R","When Bogardan Firefiend dies, it deals 2 damage to target creature.","2R","Creature - Elemental Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boar Umbra","G","Enchant creature
Enchanted creature gets +3/+3.
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","2G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Wind","R","Volcanic Wind deals X damage divided as you choose among any number of target creatures, where X is the number of creatures on the battlefield as you cast Volcanic Wind.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thraben Heretic","2","2","W","{T}: Exile target creature card from a graveyard.","1W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Ravager","0","0","Sacrifice an artifact: Put a +1/+1 counter on Arcbound Ravager.
Modular 1 (This enters the battlefield with a +1/+1 counter on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","2","Artifact Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Forsaken City","Forsaken City doesn't untap during your untap step.
At the beginning of your upkeep, you may exile a card from your hand. If you do, untap Forsaken City.
{T}: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Savaen Elves","1","1","G","{G}{G}, {T}: Destroy target Aura attached to a land.","G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost Ship","2","4","U","Flying
{U}{U}{U}: Regenerate Ghost Ship.","2UU","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelic Blessing","W","Target creature gets +3/+3 and gains flying until end of turn. (It can't be blocked except by creatures with flying or reach.)","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Traveling Plague","B","Enchant creature
At the beginning of each upkeep, put a plague counter on Traveling Plague.
Enchanted creature gets -1/-1 for each plague counter on Traveling Plague.
When enchanted creature leaves the battlefield, that creature's controller returns Traveling Plague from its owner's graveyard to the battlefield.","3BB","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mox Opal","Metalcraft - {T}: Add one mana of any color to your mana pool. Activate this ability only if you control three or more artifacts.","0","Legendary Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rootbound Crag","Rootbound Crag enters the battlefield tapped unless you control a Mountain or a Forest.
{T}: Add {R} or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Breeder","0","0","G","Spike Breeder enters the battlefield with three +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Breeder: Put a +1/+1 counter on target creature.
{2}, Remove a +1/+1 counter from Spike Breeder: Put a 1/1 green Spike creature token onto the battlefield.","3G","Creature - Spike");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Vortex","UB","Destroy target creature unless its controller pays life equal to its toughness. A creature destroyed this way can't be regenerated.","1UB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Celestial Crusader","2","2","W","Flash (You may cast this spell any time you could cast an instant.)
Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Flying
Other white creatures get +1/+1.","2WW","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hypervolt Grasp","R","Enchant creature
Enchanted creature has ""{T}: This creature deals 1 damage to target creature or player.""
{1}{U}: Return Hypervolt Grasp to its owner's hand.","2R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Liquid Fire","R","As an additional cost to cast Liquid Fire, choose a number between 0 and 5.
Liquid Fire deals X damage to target creature and 5 minus X damage to that creature's controller, where X is the chosen number.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Poisonbelly Ogre","3","3","B","Whenever another creature enters the battlefield, its controller loses 1 life.","4B","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Canyon Minotaur","3","3","R",NULL,"3R","Creature - Minotaur Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Khalni Hydra","8","8","G","Khalni Hydra costs {G} less to cast for each green creature you control.
Trample","GGGGGGGG","Creature - Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Echo","W","Soul Echo enters the battlefield with X echo counters on it.
You don't lose the game for having 0 or less life.
At the beginning of your upkeep, sacrifice Soul Echo if there are no echo counters on it. Otherwise, target opponent may choose that for each 1 damage that would be dealt to you until your next upkeep, you remove an echo counter from Soul Echo instead.","XWW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lavaclaw Reaches","Lavaclaw Reaches enters the battlefield tapped.
{T}: Add {B} or {R} to your mana pool.
{1}{B}{R}: Until end of turn, Lavaclaw Reaches becomes a 2/2 black and red Elemental creature with ""{X}: This creature gets +X/+0 until end of turn."" It's still a land.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Singe","R","Singe deals 1 damage to target creature. That creature becomes black until end of turn.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mad Auntie","2","2","B","Other Goblin creatures you control get +1/+1.
{T}: Regenerate another target Goblin.","2B","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Touch of Brilliance","U","Draw two cards.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Synchronous Sliver","3","3","U","All Sliver creatures have vigilance.","4U","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bad Ass","3{1/2}","1","B","{1B}, Growl: Regenerate Bad Ass.","2BB","Creature - Donkey Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Golem Foundry","Whenever you cast an artifact spell, you may put a charge counter on Golem Foundry.
Remove three charge counters from Golem Foundry: Put a 3/3 colorless Golem artifact creature token onto the battlefield.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leeches","W","Target player loses all poison counters. Leeches deals that much damage to that player.","1WW","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Manalith","{T}: Add one mana of any color to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ageless Entity","4","4","G","Whenever you gain life, put that many +1/+1 counters on Ageless Entity.","3GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flashfreeze","U","Counter target red or green spell.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Andradite Leech","2","2","B","Black spells you cast cost {B} more to cast.
{B}: Andradite Leech gets +1/+1 until end of turn.","2B","Creature - Leech");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Soldevi Simulacrum","2","4","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
{1}: Soldevi Simulacrum gets +1/+0 until end of turn.","4","Artifact Creature - Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Downdraft","G","{G}: Target creature loses flying until end of turn.
Sacrifice Downdraft: Downdraft deals 2 damage to each creature with flying.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dromar, the Banisher","6","6","WUB","Flying
Whenever Dromar, the Banisher deals combat damage to a player, you may pay {2}{U}. If you do, choose a color, then return all creatures of that color to their owners' hands.","3WUB","Legendary Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warrior's Charge","W","Creatures you control get +1/+1 until end of turn.","2W","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Charcoal Diamond","Charcoal Diamond enters the battlefield tapped.
{T}: Add {B} to your mana pool.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Academy Ruins","{T}: Add {1} to your mana pool.
{1}{U}, {T}: Put target artifact card from your graveyard on top of your library.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bounteous Kirin","4","4","G","Flying
Whenever you cast a Spirit or Arcane spell, you may gain life equal to that spell's converted mana cost.","5GG","Legendary Creature - Kirin Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakavolver","2","2","R","Kicker {1}{W} and/or {U} (You may pay an additional {1}{W} and/or {U} as you cast this spell.)
If Rakavolver was kicked with its {1}{W} kicker, it enters the battlefield with two +1/+1 counters on it and with ""Whenever Rakavolver deals damage, you gain that much life.""
If Rakavolver was kicked with its {U} kicker, it enters the battlefield with a +1/+1 counter on it and with flying.","2R","Creature - Volver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Barrier","U","Counter target creature spell. Its controller loses 1 life.","UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spinal Graft","B","Enchant creature
Enchanted creature gets +3/+3.
When enchanted creature becomes the target of a spell or ability, destroy that creature. It can't be regenerated.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eastern Paladin","3","3","B","{B}{B}, {T}: Destroy target green creature.","2BB","Creature - Zombie Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cloak of Mists","U","Enchant creature
Enchanted creature is unblockable.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lumberknot","1","1","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Whenever a creature dies, put a +1/+1 counter on Lumberknot.","2GG","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Aristocrat","3","3","U","Whenever Cephalid Aristocrat becomes the target of a spell or ability, put the top two cards of your library into your graveyard.","4U","Creature - Cephalid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reaping the Rewards","W","Buyback-Sacrifice a land. (You may sacrifice a land in addition to any other costs as you cast this spell. If you do, put this card into your hand as it resolves.)
You gain 2 life.","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Reflecting Mirror","{X}, {T}: Change the target of target spell with a single target if that target is you. The new target must be a player. X is twice the converted mana cost of that spell.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Refraction Trap","W","If an opponent cast a red instant or sorcery spell this turn, you may pay {W} rather than pay Refraction Trap's mana cost.
Prevent the next 3 damage that a source of your choice would deal to you and/or permanents you control this turn. If damage is prevented this way, Refraction Trap deals that much damage to target creature or player.","3W","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stillmoon Cavalier","2","1","WB","Protection from white and from black
{(w/b)}: Stillmoon Cavalier gains flying until end of turn.
{(w/b)}: Stillmoon Cavalier gains first strike until end of turn.
{(w/b){(w/b)}: Stillmoon Cavalier gets +1/+0 until end of turn.","1(W/B)(W/B)","Creature - Zombie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prickly Boggart","1","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)","B","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nameless Race","*","*","B","Trample
As Nameless Race enters the battlefield, pay any amount of life. The amount you pay can't be more than the total number of white nontoken permanents your opponents control plus the total number of white cards in their graveyards.
Nameless Race's power and toughness are each equal to the life paid as it entered the battlefield.","3B","Creature");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Slaughter Pact","Destroy target nonblack creature.
At the beginning of your next upkeep, pay {2}{B}. If you don't, you lose the game.","0","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pyknite","1","1","G","When Pyknite enters the battlefield, draw a card at the beginning of the next turn's upkeep.","2G","Creature - Ouphe");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pride of the Clouds","1","1","WU","Flying
Pride of the Clouds gets +1/+1 for each other creature with flying on the battlefield.
Forecast - {2}{W}{U}, Reveal Pride of the Clouds from your hand: Put a 1/1 white and blue Bird creature token with flying onto the battlefield. (Activate this ability only during your upkeep and only once each turn.)","WU","Creature - Elemental Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Muddle the Mixture","U","Counter target instant or sorcery spell.
Transmute {1}{U}{U} ({1}{U}{U}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charm Peddler","1","1","W","{W}, {T}, Discard a card: The next time a source of your choice would deal damage to target creature this turn, prevent that damage.","W","Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Groundswell","G","Target creature gets +2/+2 until end of turn.
Landfall - If you had a land enter the battlefield under your control this turn, that creature gets +4/+4 until end of turn instead.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghor-Clan Savage","2","3","G","Bloodthirst 3 (If an opponent was dealt damage this turn, this creature enters the battlefield with three +1/+1 counters on it.)","3GG","Creature - Centaur Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reminisce","U","Target player shuffles his or her graveyard into his or her library.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blanket of Night","B","Each land is a Swamp in addition to its other land types.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Juniper Order Druid","1","1","G","{T}: Untap target land.","2G","Creature - Human Cleric Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Strength","R","Enchant creature
Enchanted creature gets +2/+2 and has mountainwalk. (It's unblockable as long as defending player controls a Mountain.)","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Broken Dam","U","Tap one or two target creatures without horsemanship.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Noble Stand","W","Whenever a creature you control blocks, you gain 2 life.","4W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream Cache","U","Draw three cards, then put two cards from your hand both on top of your library or both on the bottom of your library.","2U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Simic Growth Chamber","Simic Growth Chamber enters the battlefield tapped.
When Simic Growth Chamber enters the battlefield, return a land you control to its owner's hand.
{T}: Add {G}{U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seize the Initiative","W","Target creature gets +1/+1 and gains first strike until end of turn.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Life from the Loam","G","Return up to three target land cards from your graveyard to your hand.
Dredge 3 (If you would draw a card, instead you may put exactly three cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horizon Drake","3","1","U","Flying, protection from lands","1UU","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Erase","W","If you control two or more white permanents that share an artist, you may play Erase (Not the Urza's Legacy One) without paying its mana cost.
Remove target enchantment from the game.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cragganwick Cremator","5","4","R","When Cragganwick Cremator enters the battlefield, discard a card at random. If you discard a creature card this way, Cragganwick Cremator deals damage equal to that card's power to target player.","2RR","Creature - Giant Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Cutthroat","5","3","R","Haste
At the beginning of the end step, return Viashino Cutthroat to its owner's hand.","2RR","Creature - Viashino");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spiraling Embers","R","Spiraling Embers deals damage to target creature or player equal to the number of cards in your hand.","3R","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coordinated Barrage","W","Choose a creature type. Coordinated Barrage deals damage to target attacking or blocking creature equal to the number of permanents you control of the chosen type.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Farrelite Priest","1","3","W","{1}: Add {W} to your mana pool. If this ability has been activated four or more times this turn, sacrifice Farrelite Priest at the beginning of the next end step.","1WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Restoration Angel","3","4","W","Flash
Flying
When Restoration Angel enters the battlefield, you may exile target non-Angel creature you control, then return that card to the battlefield under your control.","3W","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elkin Bottle","{3}, {T}: Exile the top card of your library. Until the beginning of your next upkeep, you may play that card.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Over Matter","U","Discard a card: You may tap or untap target artifact, creature, or land.","2UUUU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scab-Clan Mauler","1","1","RG","Bloodthirst 2 (If an opponent was dealt damage this turn, this creature enters the battlefield with two +1/+1 counters on it.)
Trample","RG","Creature - Human Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chromatic Armor","WU","Enchant creature
As Chromatic Armor enters the battlefield, choose a color.
Chromatic Armor enters the battlefield with a sleight counter on it.
Prevent all damage that would be dealt to enchanted creature by sources of the last chosen color.
{X}: Put a sleight counter on Chromatic Armor and choose a color. X is the number of sleight counters on Chromatic Armor.","1WU","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lyna","Creatures you control have shadow. (They can block and be blocked only by creatures with shadow.)",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Angel's Trumpet","All creatures have vigilance.
At the beginning of each player's end step, tap all untapped creatures that player controls that didn't attack this turn. Angel's Trumpet deals damage to the player equal to the number of creatures tapped this way.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Whirling Catapult","{2}, Exile the top two cards of your library: Whirling Catapult deals 1 damage to each creature with flying and each player.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flanking Troops","2","2","W","Whenever Flanking Troops attacks, you may tap target creature.","2WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Renewing Dawn","W","You gain 2 life for each Mountain target opponent controls.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quirion Trailblazer","1","2","G","When Quirion Trailblazer enters the battlefield, you may search your library for a basic land card and put that card onto the battlefield tapped. If you do, shuffle your library.","3G","Creature - Elf Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flowstone Strike","R","Target creature gets +1/-1 and gains haste until end of turn.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jaddi Lifestrider","2","8","G","When Jaddi Lifestrider enters the battlefield, you may tap any number of untapped creatures you control. You gain 2 life for each creature tapped this way.","4G","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Possessed Centaur","3","3","G","Trample
Threshold - As long as seven or more cards are in your graveyard, Possessed Centaur gets +1/+1, is black, and has ""{2}{B}, {T}: Destroy target green creature.""","2GG","Creature - Centaur Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phelddagrif","4","4","WUG","{G}: Phelddagrif gains trample until end of turn. Target opponent puts a 1/1 green Hippo creature token onto the battlefield.
{W}: Phelddagrif gains flying until end of turn. Target opponent gains 2 life.
{U}: Return Phelddagrif to its owner's hand. Target opponent may draw a card.","1GWU","Legendary Creature - Phelddagrif");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quirion Elves","1","1","G","As Quirion Elves enters the battlefield, choose a color.
{T}: Add {G} to your mana pool.
{T}: Add one mana of the chosen color to your mana pool.","1G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surging Might","G","Enchant creature
Enchanted creature gets +2/+2.
Ripple 4 (When you cast this spell, you may reveal the top four cards of your library. You may cast any revealed cards with the same name as this spell without paying their mana costs. Put the rest on the bottom of your library.)","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dryad Sophisticate","2","1","G","Nonbasic landwalk","1G","Creature - Dryad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tel-Jilad Defiance","G","Target creature gains protection from artifacts until end of turn.
Draw a card.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Merseine","U","Enchant creature
Merseine enters the battlefield with three net counters on it.
Enchanted creature doesn't untap during its controller's untap step if Merseine has a net counter on it.
Pay enchanted creature's mana cost: Remove a net counter from Merseine. Any player may activate this ability, but only if he or she controls the enchanted creature.","2UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kederekt Parasite","1","1","B","Whenever an opponent draws a card, if you control a red permanent, you may have Kederekt Parasite deal 1 damage to that player.","B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tree Monkey","1","1","G","Reach (This creature can block creatures with flying.)","G","Creature - Ape");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tormod's Crypt","{T}, Sacrifice Tormod's Crypt: Exile all cards from target player's graveyard.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kyscu Drake","2","2","G","Flying
{G}: Kyscu Drake gets +0/+1 until end of turn. Activate this ability only once each turn.
Sacrifice Kyscu Drake and a creature named Spitting Drake: Search your library for a card named Viashivan Dragon and put that card onto the battlefield. Then shuffle your library.","3G","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frightshroud Courier","2","1","B","You may choose not to untap Frightshroud Courier during your untap step.
{2}{B}, {T}: Target Zombie creature gets +2/+2 and has fear for as long as Frightshroud Courier remains tapped. (It can't be blocked except by artifact creatures and/or black creatures.)","2B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mending Hands","W","Prevent the next 4 damage that would be dealt to target creature or player this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scarblade Elite","2","2","B","{T}, Exile an Assassin card from your graveyard: Destroy target creature.","BB","Creature - Elf Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sedraxis Alchemist","2","2","B","When Sedraxis Alchemist enters the battlefield, if you control a blue permanent, return target nonland permanent to its owner's hand.","2B","Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wu Admiral","3","3","U","Wu Admiral gets +1/+1 as long as an opponent controls an Island.","4U","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wheel of Sun and Moon","WG","Enchant player
If a card would be put into enchanted player's graveyard from anywhere, instead that card is revealed and put on the bottom of that player's library.","(G/W)(G/W)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woolly Razorback","7","7","W","Woolly Razorback enters the battlefield with three ice counters on it.
As long as Woolly Razorback has an ice counter on it, prevent all combat damage it would deal and it has defender.
Whenever Woolly Razorback blocks, remove an ice counter from it.","2WW","Creature - Boar Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Battleflies","0","1","B","Flying
{B}: Phyrexian Battleflies gets +1/+0 until end of turn. Activate this ability no more than twice each turn.","B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Venom","U","Enchant land
Whenever enchanted land becomes tapped, Psychic Venom deals 2 damage to that land's controller.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talas Scout","1","2","U","Flying","1U","Creature - Human Pirate Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Necropouncer","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +3/+1 and has haste.
Equip {2}","6","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Dart","R","Lightning Dart deals 1 damage to target creature. If that creature is white or blue, Lightning Dart deals 4 damage to it instead.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trench Gorger","6","6","U","Trample
When Trench Gorger enters the battlefield, you may search your library for any number of land cards, exile them, then shuffle your library. If you do, Trench Gorger's power and toughness each become equal to the number of cards exiled this way.","6UU","Creature - Leviathan");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Speculation","U","Buyback {2} (You may pay an additional {2} as you cast this spell. If you do, put this card into your hand as it resolves.)
Scry 3. (To scry 3, look at the top three cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bant","All creatures have exalted. (Whenever a creature attacks alone, it gets +1/+1 until end of turn for each instance of exalted among permanents its controller controls.)
Whenever you roll {C}, put a divinity counter on target green, white, or blue creature. That creature is indestructible as long as it has a divinity counter on it.",NULL,"Plane - Alara");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Druid","1","2","G","{T}, Sacrifice Llanowar Druid: Untap all Forests.","1G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soothsaying","U","{3}{U}{U}: Shuffle your library.
{X}: Look at the top X cards of your library, then put them back in any order.","U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fire-Lit Thicket","{T}: Add {1} to your mana pool.
{(r/g)}, {T}: Add {R}{R}, {R}{G}, or {G}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oona, Queen of the Fae","5","5","UB","Flying
{X}{(u/b)}: Choose a color. Target opponent exiles the top X cards of his or her library. For each card of the chosen color exiled this way, put a 1/1 blue and black Faerie Rogue creature token with flying onto the battlefield.","3(U/B)(U/B)(U/B)","Legendary Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Abandon Hope","B","As an additional cost to cast Abandon Hope, discard X cards.
Look at target opponent's hand and choose X cards from it. That player discards those cards.","X1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scar","BR","Put a -1/-1 counter on target creature.","(B/R)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cursed Ronin","1","1","B","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
{B}: Cursed Ronin gets +1/+1 until end of turn.","3B","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aeon Chronicler","*","*","U","Aeon Chronicler's power and toughness are each equal to the number of cards in your hand.
Suspend X-{X}{3}{U}. X can't be 0.  (Rather than cast this card from your hand, you may pay {X}{3}{U} and exile it with X time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Whenever a time counter is removed from Aeon Chronicler while it's exiled, draw a card.","3UU","Creature - Avatar");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Opaline Bracers","Sunburst (This enters the battlefield with a charge counter on it for each color of mana spent to cast it.)
Equipped creature gets +X/+X, where X is the number of charge counters on Opaline Bracers.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","4","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sporeback Troll","0","0","G","Graft 2 (This creature enters the battlefield with two +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{1}{G}: Regenerate target creature with a +1/+1 counter on it.","3G","Creature - Troll Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thick-Skinned Goblin","2","1","R","You may pay {0} rather than pay the echo cost for permanents you control.
{R}: Thick-Skinned Goblin gains protection from red until end of turn.","1R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Furyborn Hellkite","6","6","R","Bloodthirst 6 (If an opponent was dealt damage this turn, this creature enters the battlefield with six +1/+1 counters on it.)
Flying","4RRR","Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Timesifter","At the beginning of each upkeep, each player exiles the top card of his or her library. The player who exiled the card with the highest converted mana cost takes an extra turn after this one. If two or more players' cards are tied for highest cost, the tied players repeat this process until the tie is broken.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rooting Kavu","4","3","G","When Rooting Kavu dies, you may exile it. If you do, shuffle all creature cards from your graveyard into your library.","2GG","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pariah's Shield","All damage that would be dealt to you is dealt to equipped creature instead.
Equip {3}","5","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pendrell Drake","2","3","U","Flying
Cycling {2} ({2}, Discard this card: Draw a card.)","3U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Claws of Valakut","R","Enchant creature
Enchanted creature gets +1/+0 for each Mountain you control and has first strike.","1RR","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lim-Dûl's Vault","UB","Look at the top five cards of your library. As many times as you choose, you may pay 1 life, put those cards on the bottom of your library in any order, then look at the top five cards of your library. Then shuffle your library and put the last cards you looked at this way on top of it in any order.","UB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Séance","W","At the beginning of each upkeep, you may exile target creature card from your graveyard. If you do, put a token onto the battlefield that's a copy of that card except it's a Spirit in addition to its other types. Exile it at the beginning of the next end step.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veiled Crocodile","U","When a player has no cards in hand, if Veiled Crocodile is an enchantment, Veiled Crocodile becomes a 4/4 Crocodile creature.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brassclaw Orcs","3","2","R","Brassclaw Orcs can't block creatures with power 2 or greater.","2R","Creature - Orc");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timbermare","5","5","G","Haste
Echo {5}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Timbermare enters the battlefield, tap all other creatures.","3G","Creature - Elemental Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bewilder","U","Target creature gets -3/-0 until end of turn.
Draw a card.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lumengrid Warden","1","3","U",NULL,"1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Supreme Exemplar","10","10","U","Flying
Champion an Elemental (When this enters the battlefield, sacrifice it unless you exile another Elemental you control. When this leaves the battlefield, that card returns to the battlefield.)","6U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brace for Impact","W","Prevent all damage that would be dealt to target multicolored creature this turn. For each 1 damage prevented this way, put a +1/+1 counter on that creature.","4W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Basilisk Collar","Equipped creature has deathtouch and lifelink.
Equip {2}","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Distant Melody","U","Choose a creature type. Draw a card for each permanent you control of that type.","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elemental Appeal","R","Kicker {5} (You may pay an additional {5} as you cast this spell.)
Put a 7/1 red Elemental creature token with trample and haste onto the battlefield. Exile it at the beginning of the next end step. If Elemental Appeal was kicked, that creature gets +7/+0 until end of turn.","RRRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Caribou Range","W","Enchant land you control
Enchanted land has ""{W}{W}, {T}: Put a 0/1 white Caribou creature token onto the battlefield.""
Sacrifice a Caribou token: You gain 1 life.","2WW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Royal Guard","2","5","W","{T}: All combat damage that would be dealt to you by unblocked creatures this turn is dealt to Kjeldoran Royal Guard instead.","3WW","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rootwater Depths","{T}: Add {1} to your mana pool.
{T}: Add {U} or {B} to your mana pool. Rootwater Depths doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moonveil Dragon","5","5","R","Flying
{R}: Each creature you control gets +1/+0 until end of turn.","3RRR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Marit Lage","R","When Curse of Marit Lage enters the battlefield, tap all Islands.
Islands don't untap during their controllers' untap steps.","3RR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Storage Matrix","As long as Storage Matrix is untapped, each player chooses artifact, creature, or land during his or her untap step. That player can untap only permanents of the chosen type this step.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakdos Guildmage","2","2","BR","({(b/r)} can be paid with either {B} or {R}.)
{3}{B}, Discard a card: Target creature gets -2/-2 until end of turn.
{3}{R}: Put a 2/1 red Goblin creature token with haste onto the battlefield. Exile it at the beginning of the next end step.","(B/R)(B/R)","Creature - Zombie Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pitchburn Devils","3","3","R","When Pitchburn Devils dies, it deals 3 damage to target creature or player.","4R","Creature - Devil");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cryptic Gateway","Tap two untapped creatures you control: You may put a creature card from your hand that shares a creature type with each creature tapped this way onto the battlefield.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flailing Drake","2","3","G","Flying
Whenever Flailing Drake blocks or becomes blocked by a creature, that creature gets +1/+1 until end of turn.","3G","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tarpan","1","1","G","When Tarpan dies, you gain 1 life.","G","Creature - Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ranger's Guile","G","Target creature you control gets +1/+1 and gains hexproof until end of turn. (It can't be the target of spells or abilities your opponents control.)","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Realms Befitting My Majesty","When you set this scheme in motion, search your library for up to two basic land cards, put them onto the battlefield tapped, then shuffle your library.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fissure Vent","R","Choose one or both - Destroy target artifact; and/or destroy target nonbasic land.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Palladium Myr","2","2","{T}: Add {2} to your mana pool.","3","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostly Prison","W","Creatures can't attack you unless their controller pays {2} for each creature he or she controls that's attacking you.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rotcrown Ghoul","3","3","U","When Rotcrown Ghoul dies, target player puts the top five cards of his or her library into his or her graveyard.","4U","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("AEther Membrane","0","5","R","Defender; reach (This creature can block creatures with flying.)
Whenever Æther Membrane blocks a creature, return that creature to its owner's hand at end of combat.","1RR","Creature - Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Plague Myr","1","1","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
{T}: Add {1} to your mana pool.","2","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Celestial Ancient","3","3","W","Flying
Whenever you cast an enchantment spell, put a +1/+1 counter on each creature you control.","3WW","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gathan Raiders","3","3","R","Hellbent - Gathan Raiders gets +2/+2 as long as you have no cards in hand.
Morph-Discard a card. (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3RR","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jungle Barrier","2","6","UG","Defender (This creature can't attack.)
When Jungle Barrier enters the battlefield, draw a card.","2GU","Creature - Plant Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sword Dancer","1","2","W","{W}{W}: Target attacking creature gets -1/-0 until end of turn.","1W","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crypt Creeper","2","1","B","Sacrifice Crypt Creeper: Exile target card from a graveyard.","1B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Golem Artisan","3","3","{2}: Target artifact creature gets +1/+1 until end of turn.
{2}: Target artifact creature gains your choice of flying, trample, or haste until end of turn.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Netter en-Dal","1","1","W","{W}, {T}, Discard a card: Target creature can't attack this turn.","W","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vectis Dominator","0","2","WB","{T}: Tap target creature unless its controller pays 2 life.","1WB","Artifact Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plague Wind","B","Destroy all creatures you don't control. They can't be regenerated.","7BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pitchstone Wall","2","5","R","Defender (This creature can't attack.)
Whenever you discard a card, you may sacrifice Pitchstone Wall. If you do, return the discarded card from your graveyard to your hand.","2R","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sigil Blessing","WG","Until end of turn, target creature you control gets +3/+3 and other creatures you control get +1/+1.","GW","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirri the Cursed Avatar","Creatures you control have ""{T}: Another target creature gets -1/-1 until end of turn. Put a +1/+1 counter on this creature.""",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Warlord","3","2","R","{T}: Remove target blocking creature from combat. Creatures it was blocking that hadn't become blocked by another creature this combat become unblocked, then it blocks an attacking creature of your choice. Activate this ability only during the declare blockers step.","3R","Creature - Human Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soulquake","UB","Return all creatures on the battlefield and all creature cards in graveyards to their owners' hands.","3UUBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nicol Bolas","7","7","UBR","Flying
At the beginning of your upkeep, sacrifice Nicol Bolas unless you pay {U}{B}{R}.
Whenever Nicol Bolas deals damage to an opponent, that player discards his or her hand.","2UUBBRR","Legendary Creature - Elder Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Moss Diamond","Moss Diamond enters the battlefield tapped.
{T}: Add {G} to your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunblast Angel","4","5","W","Flying
When Sunblast Angel enters the battlefield, destroy all tapped creatures.","4WW","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lotus Petal","{T}, Sacrifice Lotus Petal: Add one mana of any color to your mana pool.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necromancy","B","You may cast Necromancy as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
When Necromancy enters the battlefield, if it's on the battlefield, it becomes an Aura with ""enchant creature put onto the battlefield with Necromancy."" Put target creature card from a graveyard onto the battlefield under your control and attach Necromancy to it. When Necromancy leaves the battlefield, that creature's controller sacrifices it.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Branchbender","2","2","G","{T}: Until end of turn, target Forest becomes an X/X Treefolk creature in addition to its other types, where X is the number of Elves you control.","2G","Creature - Elf Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crypt of Agadeem","Crypt of Agadeem enters the battlefield tapped.
{T}: Add {B} to your mana pool.
{2}, {T}: Add {B} to your mana pool for each black creature card in your graveyard.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostly Visit","B","Destroy target nonblack creature.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armor Thrull","1","3","B","{T}, Sacrifice Armor Thrull: Put a +1/+2 counter on target creature.","2B","Creature - Thrull");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Light of Day","W","Black creatures can't attack or block.","3W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jester's Sombrero","o2, ocT, Sacrifice Jester's Sombrero: Look through target player's sideboard and remove any three of those cards from it for the remainder of the match.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Archer","2","2","W","Flying
{2}{W}, {T}: Aven Archer deals 2 damage to target attacking or blocking creature.","3WW","Creature - Bird Soldier Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightwind Glider","2","1","W","Flying, protection from black","2W","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coiling Oracle","1","1","UG","When Coiling Oracle enters the battlefield, reveal the top card of your library. If it's a land card, put it onto the battlefield. Otherwise, put that card into your hand.","GU","Creature - Snake Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Speaker","3","2","B","At the beginning of your upkeep, you may sacrifice Blood Speaker. If you do, search your library for a Demon card, reveal that card, and put it into your hand. Then shuffle your library.
Whenever a Demon enters the battlefield under your control, return Blood Speaker from your graveyard to your hand.","3B","Creature - Ogre Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brainspoil","B","Destroy target creature that isn't enchanted. It can't be regenerated.
Transmute {1}{B}{B} ({1}{B}{B}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zealous Guardian","1","1","WU","Flash","(W/U)","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Serpent","5","5","U","Sea Serpent can't attack unless defending player controls an Island.
When you control no Islands, sacrifice Sea Serpent.","5U","Creature - Serpent");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shade's Breath","B","Until end of turn, each creature you control becomes a black Shade and gains ""{B}: This creature gets +1/+1 until end of turn.""","1B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pentad Prism","Sunburst (This enters the battlefield with a charge counter on it for each color of mana spent to cast it.)
Remove a charge counter from Pentad Prism: Add one mana of any color to your mana pool.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Homeward Path","{T}: Add {1} to your mana pool.
{T}: Each player gains control of all creatures he or she owns.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Pilgrim's Eye","1","1","Flying
When Pilgrim's Eye enters the battlefield, you may search your library for a basic land card, reveal it, put it into your hand, then shuffle your library.","3","Artifact Creature - Thopter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunting Grounds","WG","Threshold - As long as seven or more cards are in your graveyard, Hunting Grounds has ""Whenever an opponent casts a spell, you may put a creature card from your hand onto the battlefield.""","GW","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Transmogrifying Licid","2","2","{1}, {T}: Transmogrifying Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {1} to end this effect.
Enchanted creature gets +1/+1 and is an artifact in addition to its other types.","3","Artifact Creature - Licid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Frostwaker","1","1","U","{U}, {T}: Target snow land becomes a 2/2 blue Elemental creature with flying. It's still a land.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Student of Elements","1","1","U","When Student of Elements has flying, flip it.
---
Creatures you control have flying.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spirit of the Hearth","4","5","W","Flying
You have hexproof. (You can't be the target of spells or abilities your opponents control.)","4WW","Creature - Cat Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marsh Crocodile","4","4","UB","When Marsh Crocodile enters the battlefield, return a blue or black creature you control to its owner's hand.
When Marsh Crocodile enters the battlefield, each player discards a card.","2UB","Creature - Crocodile");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Bleed","B","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, that player loses 1 life.","2B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jaya Ballard Avatar","{X}: Jaya Ballard Avatar deals an amount of damage chosen at random from 0 to X to target creature or player. Activate this ability only once each turn.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spider Umbra","G","Enchant creature
Enchanted creature gets +1/+1 and has reach. (It can block creatures with flying.)
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauntless Dourbark","*","*","G","Dauntless Dourbark's power and toughness are each equal to the number of Forests you control plus the number of Treefolk you control.
Dauntless Dourbark has trample as long as you control another Treefolk.","3G","Creature - Treefolk Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Super Secret Tech","All premium spells cost {1} less to play.
All premium creatures get +1/+1.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Haunting Echoes","B","Exile all cards from target player's graveyard other than basic land cards. For each card exiled this way, search that player's library for all cards with the same name as that card and exile them. Then that player shuffles his or her library.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Obsidian Giant","4","4","R",NULL,"4R","Creature - Giant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wand of Denial","{T}: Look at the top card of target player's library. If it's a nonland card, you may pay 2 life. If you do, put it into that player's graveyard.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lavacore Elemental","5","3","R","Vanishing 1 (This permanent enters the battlefield with a time counter on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
Whenever a creature you control deals combat damage to a player, put a time counter on Lavacore Elemental.","2R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Canopy Dragon","4","4","G","Trample
{1}{G}: Canopy Dragon gains flying and loses trample until end of turn.","4GG","Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Static Orb","As long as Static Orb is untapped, players can't untap more than two permanents during their untap steps.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost-Lit Stalker","1","1","B","{4}{B}, {T}: Target player discards two cards. Activate this ability only any time you could cast a sorcery.
Channel - {5}{B}{B}, Discard Ghost-Lit Stalker: Target player discards four cards. Activate this ability only any time you could cast a sorcery.","B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tor Giant","3","3","R",NULL,"3R","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Traitor's Clutch","B","Target creature gets +1/+0, becomes black, and gains shadow until end of turn. (It can block or be blocked by only creatures with shadow.)
Flashback {1}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","4B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Storm Front","G","{G}{G}: Tap target creature with flying.","G","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Thrull","0","1",NULL,NULL,"Creature - Thrull");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Verdant Catacombs","{T}, Pay 1 life, Sacrifice Verdant Catacombs: Search your library for a Swamp or Forest card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shauku's Minion","2","2","BR","{B}{R}, {T}: Shauku's Minion deals 2 damage to target white creature.","1BR","Creature - Human Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mystic Penitent","1","1","W","Vigilance
Threshold - As long as seven or more cards are in your graveyard, Mystic Penitent gets +1/+1 and has flying.","W","Creature - Human Nomad Mystic");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Creature Guy","3","3","G","Gotcha Whenever an opponent says ""Creature"" or ""Guy,"" you may say ""Gotcha"" If you do, return Creature Guy from your graveyard to your hand.","3G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merrow Wavebreakers","3","3","U","{1}{U}, {Q}: Merrow Wavebreakers gains flying until end of turn. ({Q} is the untap symbol.)","4U","Creature - Merfolk Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nether Shadow","1","1","B","Haste
At the beginning of your upkeep, if Nether Shadow is in your graveyard with three or more creature cards above it, you may put Nether Shadow onto the battlefield.","BB","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Veilstone Amulet","Whenever you cast a spell, creatures you control can't be the targets of spells or abilities your opponents control this turn.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Orim's Thunder","W","Kicker {R} (You may pay an additional {R} as you cast this spell.)
Destroy target artifact or enchantment. If Orim's Thunder was kicked, it deals damage equal to that permanent's converted mana cost to target creature.","2W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mortarpod","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +0/+1 and has ""Sacrifice this creature: This creature deals 1 damage to target creature or player.""
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Sorin Markov","B","+2: Sorin Markov deals 2 damage to target creature or player and you gain 2 life.
-3: Target opponent's life total becomes 10.
-7: You control target player during that player's next turn.","Planeswalker - Sorin","3BBB");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Golden Bear","4","3","G",NULL,"3G","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dong Zhou, the Tyrant","3","3","R","When Dong Zhou, the Tyrant enters the battlefield, target creature an opponent controls deals damage equal to its power to that player.","4R","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shrapnel Blast","R","As an additional cost to cast Shrapnel Blast, sacrifice an artifact.
Shrapnel Blast deals 5 damage to target creature or player.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fylamarid","1","3","U","Flying
Fylamarid can't be blocked by blue creatures.
{U}: Target creature becomes blue until end of turn.","1UU","Creature - Squid Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Font of Mythos","At the beginning of each player's draw step, that player draws two additional cards.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Torchling","3","3","R","{R}: Untap Torchling.
{R}: Target creature blocks Torchling this turn if able.
{R}: Change the target of target spell that targets only Torchling.
{1}: Torchling gets +1/-1 until end of turn.
{1}: Torchling gets -1/+1 until end of turn.","3RR","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bull Aurochs","2","1","G","Trample
Whenever Bull Aurochs attacks, it gets +1/+0 until end of turn for each other attacking Aurochs.","1G","Creature - Aurochs");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dregs of Sorrow","B","Destroy X target nonblack creatures. Draw X cards.","X4B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Marauder","3","1","B","Whenever Ogre Marauder attacks, it gains ""Ogre Marauder is unblockable"" until end of turn unless defending player sacrifices a creature.","1BB","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Sliversmith","1","1","{1}, {T}, Discard a card: Put a 1/1 colorless Sliver artifact creature token named Metallic Sliver onto the battlefield.","2","Artifact Creature - Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jushi Apprentice","1","2","U","{2}{U}, {T}: Draw a card. If you have nine or more cards in hand, flip Jushi Apprentice.
---
{3}{U}{U}, {T}: Target player draws X cards, where X is the number of cards in your hand.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raging Poltergeist","6","1","R",NULL,"4R","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seal of Fire","R","Sacrifice Seal of Fire: Seal of Fire deals 2 damage to target creature or player.","R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primeval Force","8","8","G","When Primeval Force enters the battlefield, sacrifice it unless you sacrifice three Forests.","2GGG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master of the Hunt","2","2","G","{2}{G}{G}: Put a 1/1 green Wolf creature token named Wolves of the Hunt onto the battlefield. It has ""bands with other creatures named Wolves of the Hunt."" (Any creatures named Wolves of the Hunt can attack in a band as long as at least one has ""bands with other creatures named Wolves of the Hunt."" Bands are blocked as a group. If at least two creatures named Wolves of the Hunt you control, one of which has ""bands with other creatures named Wolves of the Hunt,"" are blocking or being blocked by the same creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","2GG","Creature - Human");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Soliton","3","4","{U}: Untap Soliton.","5","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dosan's Oldest Chant","G","You gain 6 life.
Draw a card.","4G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cryoclasm","R","Destroy target Plains or Island. Cryoclasm deals 3 damage to that land's controller.","2R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Goblin Charbelcher","{3}, {T}: Reveal cards from the top of your library until you reveal a land card. Goblin Charbelcher deals damage equal to the number of nonland cards revealed this way to target creature or player. If the revealed land card was a Mountain, Goblin Charbelcher deals double that damage instead. Put the revealed cards on the bottom of your library in any order.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Browse","U","{2}{U}{U}: Look at the top five cards of your library, put one of them into your hand, and exile the rest.","2UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Endless Horizons","W","When Endless Horizons enters the battlefield, search your library for any number of Plains cards and exile them. Then shuffle your library.
At the beginning of your upkeep, you may put a card you own exiled with Endless Horizons into your hand.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Beastbreaker of Bala Ged","2","2","G","Level up {2}{G} ({2}{G}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-3
4/4
LEVEL 4+
6/6
Trample","1G","Creature - Human Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Karoo","Karoo enters the battlefield tapped.
When Karoo enters the battlefield, sacrifice it unless you return an untapped Plains you control to its owner's hand.
{T}: Add {1}{W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fist of Suns","You may pay {W}{U}{B}{R}{G} rather than pay the mana cost for spells that you cast.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crackleburr","2","2","UR","{(u/r){(u/r)}, {T}, Tap two untapped red creatures you control: Crackleburr deals 3 damage to target creature or player.
{(u/r){(u/r)}, {Q}, Untap two tapped blue creatures you control: Return target creature to its owner's hand. ({Q} is the untap symbol.)","1(U/R)(U/R)","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fanatical Devotion","W","Sacrifice a creature: Regenerate target creature.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Granulate","R","Destroy each nonland artifact with converted mana cost 4 or less.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flamebreak","R","Flamebreak deals 3 damage to each creature without flying and each player. Creatures dealt damage this way can't be regenerated this turn.","RRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greater Harvester","5","6","B","At the beginning of your upkeep, sacrifice a permanent.
Whenever Greater Harvester deals combat damage to a player, that player sacrifices two permanents.","2BBB","Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Prodigal Sorcerer Avatar","At the beginning of your upkeep, look at the top card of your library. You may put that card into your graveyard.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thraben Sentry","2","2","W","Vigilance
Whenever another creature you control dies, you may transform Thraben Sentry.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormbound Geist","2","2","U","Flying
Stormbound Geist can block only creatures with flying.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","1UU","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Black Vise","As Black Vise enters the battlefield, choose an opponent.
At the beginning of the chosen player's upkeep, Black Vise deals X damage to that player, where X is the number of cards in his or her hand minus 4.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keen-Eyed Archers","2","2","W","Reach (This creature can block creatures with flying.)","2W","Creature - Elf Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Backslide","U","Turn target creature with morph face down.
Cycling {U} ({U}, Discard this card: Draw a card.)","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fascist Art Director","2","2","W","{WW}: Fascist Art Director gains protection from the artist of your choice until end of turn..","1WW","Creature - Human Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sinkhole","B","Destroy target land.","BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guul Draz Vampire","1","1","B","As long as an opponent has 10 or less life, Guul Draz Vampire gets +2/+1 and has intimidate. (It can't be blocked except by artifact creatures and/or creatures that share a color with it.)","B","Creature - Vampire Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bequeathal","G","Enchant creature
When enchanted creature dies, you draw two cards.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Student of Warfare","1","1","W","Level up {W} ({W}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 2-6
3/3
First strike
LEVEL 7+
4/4
Double strike","W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Miscalculation","U","Counter target spell unless its controller pays {2}.
Cycling {2} ({2}, Discard this card: Draw a card.)","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Extortion","B","When you cast Temporal Extortion, any player may pay half his or her life, rounded up. If a player does, counter Temporal Extortion.
Take an extra turn after this one.","BBBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Vorine","3","2","G","Provoke (When this attacks, you may have target creature defending player controls untap and block it if able.)
Krosan Vorine can't be blocked by more than one creature.","3G","Creature - Cat Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uril, the Miststalker","5","5","WRG","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Uril, the Miststalker gets +2/+2 for each Aura attached to it.","2RGW","Legendary Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hearth Kami","2","1","R","{X}, Sacrifice Hearth Kami: Destroy target artifact with converted mana cost X.","1R","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whirlwind","G","Destroy all creatures with flying.","2GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ass Whuppin'","WB","Destroy target silver-bordered permanent in any game you can see from your seat.","1WB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ovinize","U","Target creature loses all abilities and becomes 0/1 until end of turn.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Avalanche","R","Destroy X target snow lands.","X2RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whirlpool Warrior","2","2","U","When Whirlpool Warrior enters the battlefield, shuffle the cards from your hand into your library, then draw that many cards.
{R}, Sacrifice Whirlpool Warrior: Each player shuffles the cards from his or her hand into his or her library, then draws that many cards.","2U","Creature - Merfolk Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Infiltrator","2","2","B","{2}{U}{U}: Exchange control of Phyrexian Infiltrator and target creature. (This effect lasts indefinitely.)","2B","Creature - Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Settlers","1","1","R","{X}{X}{R}, {T}, Sacrifice Orcish Settlers: Destroy X target lands.","1R","Creature - Orc");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shriekmaw","3","2","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
When Shriekmaw enters the battlefield, destroy target nonartifact, nonblack creature.
Evoke {1}{B} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","4B","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rally the Horde","R","Exile the top card of your library. Exile the top card of your library. Exile the top card of your library. If the last card exiled isn't a land, repeat this process. Put a 1/1 red Warrior creature token onto the battlefield for each nonland card exiled this way.","5R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hatching Plans","U","When Hatching Plans is put into a graveyard from the battlefield, draw three cards.","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Behemoth","10","10","G","Fading 2 (This creature enters the battlefield with two fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Skyshroud Behemoth enters the battlefield tapped.","5GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ajani's Mantra","W","At the beginning of your upkeep, you may gain 1 life.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cursebreak","W","Destroy target enchantment. You gain 2 life.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gideon's Lawkeeper","1","1","W","{W}, {T}: Tap target creature.","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mtenda Herder","1","1","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)","W","Creature - Human Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stasis","U","Players skip their untap steps.
At the beginning of your upkeep, sacrifice Stasis unless you pay {U}.","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shauku, Endbringer","5","5","B","Flying
Shauku, Endbringer can't attack if there's another creature on the battlefield.
At the beginning of your upkeep, you lose 3 life.
{T}: Exile target creature and put a +1/+1 counter on Shauku.","5BB","Legendary Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Changeling Berserker","5","3","R","Changeling (This card is every creature type at all times.)
Haste
Champion a creature (When this enters the battlefield, sacrifice it unless you exile another creature you control. When this leaves the battlefield, that card returns to the battlefield.)","3R","Creature - Shapeshifter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nature Demands an Offering","When you set this scheme in motion, target opponent chooses a creature you don't control and puts it on top of its owner's library, then repeats this process for an artifact, an enchantment, and a land. Then the owner of each permanent chosen this way shuffles his or her library.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrier Pigeons","1","1","W","Flying
When Carrier Pigeons enters the battlefield, draw a card at the beginning of the next turn's upkeep.","3W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scavenging Ooze","2","2","G","{G}: Exile target card from a graveyard. If it was a creature card, put a +1/+1 counter on Scavenging Ooze and you gain 1 life.","1G","Creature - Ooze");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Garrison","Fortified land is indestructible.
Whenever fortified land becomes tapped, target creature gets +1/+1 until end of turn.
Fortify {3} ({3}: Attach to target land you control. Fortify only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the land leaves.)","2","Artifact - Fortification");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sanguine Guard","2","2","B","First strike
{1}{B}: Regenerate Sanguine Guard.","1BB","Creature - Zombie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Old Fogey","7","7","G","Phasing, cumulative upkeep {1}, echo, fading 3, bands with other Dinosaurs, protection from Homarids, snow-covered plainswalk, flanking, rampage 2","GG","Summon - Dinosaur");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEthermage's Touch","WU","Reveal the top four cards of your library. You may put a creature card from among them onto the battlefield. It has ""At the beginning of your end step, return this creature to its owner's hand."" Then put the rest of the cards revealed this way on the bottom of your library in any order.","2WU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lowland Tracker","2","2","W","First strike
Provoke (When this attacks, you may have target creature defending player controls untap and block it if able.)","4W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Budoka Pupil","2","2","G","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Budoka Pupil.
At the beginning of the end step, if there are two or more ki counters on Budoka Pupil, you may flip it.","1GG","Creature - Human Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scars of the Veteran","W","You may exile a white card from your hand rather than pay Scars of the Veteran's mana cost.
Prevent the next 7 damage that would be dealt to target creature or player this turn. At the beginning of the next end step, put a +0/+1 counter on that creature for each 1 damage prevented this way.","4W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frontier Guide","1","1","G","{3}{G}, {T}: Search your library for a basic land card and put it onto the battlefield tapped. Then shuffle your library.","1G","Creature - Elf Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avalanche Riders","2","2","R","Haste
Echo {3}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Avalanche Riders enters the battlefield, destroy target land.","3R","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ichor Slick","B","Target creature gets -3/-3 until end of turn.
Cycling {2} ({2}, Discard this card: Draw a card.)
Madness {3}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crowd of Cinders","*","*","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Crowd of Cinders's power and toughness are each equal to the number of black permanents you control.","3B","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teferi's Drake","3","2","U","Flying
Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)","2U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boggart Birth Rite","B","Return target Goblin card from your graveyard to your hand.","B","Tribal Sorcery - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Tabernacle","2","6","W","All creatures have ""At the beginning of your upkeep, sacrifice this creature unless you pay {1}.""","3W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goldmeadow Lookout","2","2","W","{W}, {T}, Discard a card: Put a 1/1 white Kithkin Soldier creature token named Goldmeadow Harrier onto the battlefield. It has ""{W}, {T}: Tap target creature.""","3W","Creature - Kithkin Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arcum Dagsson","2","2","U","{T}: Target artifact creature's controller sacrifices it. That player may search his or her library for a noncreature artifact card, put it onto the battlefield, then shuffle his or her library.","3U","Legendary Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crashing Centaur","3","4","G","{G}, Discard a card: Crashing Centaur gains trample until end of turn.
Threshold - As long as seven or more cards are in your graveyard, Crashing Centaur gets +2/+2 and has shroud. (It can't be the target of spells or abilities.)","4GG","Creature - Centaur");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seed the Land","G","Whenever a land enters the battlefield, its controller puts a 1/1 green Snake creature token onto the battlefield.","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx's Herald","1","1","U","{2}{U}, {T}, Sacrifice a white creature, a blue creature, and a black creature: Search your library for a card named Sphinx Sovereign and put it onto the battlefield. Then shuffle your library.","U","Artifact Creature - Vedalken Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bladed Pinions","Equipped creature has flying and first strike.
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hasran Ogress","3","2","B","Whenever Hasran Ogress attacks, it deals 3 damage to you unless you pay {2}.","BB","Creature - Ogre");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Jhoira's Timebug","1","2","{T}: Choose target permanent you control or suspended card you own. If that permanent or card has a time counter on it, you may remove a time counter from it or put another time counter on it.","2","Artifact Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bonds of Faith","W","Enchant creature
Enchanted creature gets +2/+2 as long as it's a Human. Otherwise, it can't attack or block.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dromad Purebred","1","5","W","Whenever Dromad Purebred is dealt damage, you gain 1 life.","4W","Creature - Camel Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wayfaring Giant","1","3","W","Domain - Wayfaring Giant gets +1/+1 for each basic land type among lands you control.","5W","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mark of Mutiny","R","Gain control of target creature until end of turn. Put a +1/+1 counter on it and untap it. That creature gains haste until end of turn.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foxfire Oak","3","6","G","{(r/g){(r/g){(r/g)}: Foxfire Oak gets +3/+0 until end of turn.","5G","Creature - Treefolk Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Dreamer","2","1","U","Flying
{1}: Mistform Dreamer becomes the creature type of your choice until end of turn.","2U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disciple of Griselbrand","1","1","B","{1}, Sacrifice a creature: You gain life equal to the sacrificed creature's toughness.","1B","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Horn of Greed","Whenever a player plays a land, that player draws a card.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellspark Elemental","3","1","R","Trample, haste
At the beginning of the end step, sacrifice Hellspark Elemental.
Unearth {1}{R} ({1}{R}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","1R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call of the Herd","G","Put a 3/3 green Elephant creature token onto the battlefield.
Flashback {3}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blasphemous Act","R","Blasphemous Act costs {1} less to cast for each creature on the battlefield.
Blasphemous Act deals 13 damage to each creature.","8R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Salvaging Station","{T}: Return target noncreature artifact card with converted mana cost 1 or less from your graveyard to the battlefield.
Whenever a creature dies, you may untap Salvaging Station.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Covert Operative","3","2","U","Covert Operative is unblockable.","4U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Firestarter","1","1","R","Sacrifice Goblin Firestarter: Goblin Firestarter deals 1 damage to target creature or player. Activate this ability only during your turn, before attackers are declared.","R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Living Lands","G","All Forests are 1/1 creatures that are still lands.","3G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Offering","W","Destroy target artifact. You gain life equal to its converted mana cost.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drowned Rusalka","1","1","U","{U}, Sacrifice a creature: Discard a card, then draw a card.","U","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Which of You Burns Brightest?","When you set this scheme in motion, you may pay {X}. If you do, this scheme deals X damage to target opponent and each creature he or she controls.",NULL,"Scheme");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hankyu","Equipped creature has ""{T}: Put an aim counter on Hankyu"" and ""{T}, Remove all aim counters from Hankyu: This creature deals damage to target creature or player equal to the number of aim counters removed this way.""
Equip {4} ({4}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lumengrid Sentinel","1","2","U","Flying
Whenever an artifact enters the battlefield under your control, you may tap target permanent.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bonfire of the Damned","R","Bonfire of the Damned deals X damage to target player and each creature he or she controls.
Miracle {X}{R} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","XXR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Liege of the Pit","7","7","B","Flying, trample
At the beginning of your upkeep, sacrifice a creature other than Liege of the Pit. If you can't, Liege of the Pit deals 7 damage to you.
Morph {B}{B}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","5BBB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devout Lightcaster","2","2","W","Protection from black
When Devout Lightcaster enters the battlefield, exile target black permanent.","WWW","Creature - Kor Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grave Robbers","1","1","B","{B}, {T}: Exile target artifact card from a graveyard. You gain 2 life.","1BB","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gang of Devils","3","3","R","When Gang of Devils dies, it deals 3 damage divided as you choose among one, two, or three target creatures and/or players.","5R","Creature - Devil");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spectral Searchlight","{T}: Choose a player. That player adds one mana of any color he or she chooses to his or her mana pool.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grim Affliction","B","Put a -1/-1 counter on target creature, then proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thought Reflection","U","If you would draw a card, draw two cards instead.","4UUU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Patriarch's Desire","B","Enchant creature
Enchanted creature gets +2/-2.
Threshold - Enchanted creature gets an additional +2/-2 as long as seven or more cards are in your graveyard.","3B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crash Landing","G","Target creature with flying loses flying until end of turn. Crash Landing deals damage to that creature equal to the number of Forests you control.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Artful Dodge","U","Target creature is unblockable this turn.
Flashback {U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Colossal Might","RG","Target creature gets +4/+2 and gains trample until end of turn.","RG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Intruder Alarm","U","Creatures don't untap during their controllers' untap steps.
Whenever a creature enters the battlefield, untap all creatures.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ice Cage","U","Enchant creature
Enchanted creature can't attack or block, and its activated abilities can't be activated.
When enchanted creature becomes the target of a spell or ability, destroy Ice Cage.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tomorrow, Azami's Familiar","1","5","U","If you would draw a card, look at the top three cards of your library instead. Put one of those cards into your hand and the rest on the bottom of your library in any order.","5U","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Bellow","G","Target creature gets +1/+1 until end of turn for each Forest you control.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shimmer","U","As Shimmer enters the battlefield, choose a land type.
Each land of the chosen type has phasing. (It phases in or out before its controller untaps during each of his or her untap steps. While it's phased out, it's treated as though it doesn't exist.)","2UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Leak","U","Counter target spell unless its controller pays {3}.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Power Leak","U","Enchant enchantment
At the beginning of the upkeep of enchanted enchantment's controller, that player may pay any amount of mana. Power Leak deals 2 damage to that player. Prevent X of that damage, where X is the amount of mana that player paid this way.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Shell Skulkin","3","2","{3}: Target blue creature gains shroud until end of turn.","4","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ancient Spider","2","5","WG","First strike; reach (This creature can block creatures with flying.)","2GW","Creature - Spider");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gargoyle Castle","{T}: Add {1} to your mana pool.
{5}, {T}, Sacrifice Gargoyle Castle: Put a 3/4 colorless Gargoyle artifact creature token with flying onto the battlefield.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sulfuric Vapors","R","If a red spell would deal damage to a permanent or player, it deals that much damage plus 1 to that permanent or player instead.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daru Mender","1","1","W","Morph {W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Daru Mender is turned face up, regenerate target creature.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mist Raven","2","2","U","Flying
When Mist Raven enters the battlefield, return target creature to its owner's hand.","2UU","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Kavu","*","*","RG","Yavimaya Kavu's power is equal to the number of red creatures on the battlefield.
Yavimaya Kavu's toughness is equal to the number of green creatures on the battlefield.","2RG","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horseshoe Crab","1","3","U","{U}: Untap Horseshoe Crab.","2U","Creature - Crab");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zanam Djinn","5","6","U","Flying
Zanam Djinn gets -2/-2 as long as blue is the most common color among all permanents or is tied for most common.","5U","Creature - Djinn");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Liar's Pendulum","{2}, {T}: Name a card. Target opponent guesses whether a card with that name is in your hand. You may reveal your hand. If you do and your opponent guessed wrong, draw a card.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shield of Duty and Reason","W","Enchant creature
Enchanted creature has protection from green and from blue.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunter of Eyeblights","3","3","B","When Hunter of Eyeblights enters the battlefield, put a +1/+1 counter on target creature you don't control.
{2}{B}, {T}: Destroy target creature with a counter on it.","3BB","Creature - Elf Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glory","3","3","W","Flying
{2}{W}: Choose a color. Creatures you control gain protection from the chosen color until end of turn. Activate this ability only if Glory is in your graveyard.","3WW","Creature - Incarnation");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirrorwood Treefolk","2","4","G","{2}{R}{W}: The next time damage would be dealt to Mirrorwood Treefolk this turn, that damage is dealt to target creature or player instead.","3G","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rust Elemental","4","4","Flying
At the beginning of your upkeep, sacrifice an artifact other than Rust Elemental. If you can't, tap Rust Elemental and you lose 4 life.","4","Artifact Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Root Maze","G","Artifacts and lands enter the battlefield tapped.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wormfang Crab","3","6","U","Wormfang Crab is unblockable.
When Wormfang Crab enters the battlefield, an opponent chooses a permanent you control other than Wormfang Crab and exiles it.
When Wormfang Crab leaves the battlefield, return the exiled card to the battlefield under its owner's control.","3U","Creature - Nightmare Crab");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonic Collusion","B","Buyback-Discard two cards. (You may discard two cards in addition to any other costs as you cast this spell. If you do, put this card into your hand as it resolves.)
Search your library for a card and put that card into your hand. Then shuffle your library.","3BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grasslands","Grasslands enters the battlefield tapped.
{T}, Sacrifice Grasslands: Search your library for a Forest or Plains card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Morselhoarder","6","4","RG","Morselhoarder enters the battlefield with two -1/-1 counters on it.
Remove a -1/-1 counter from Morselhoarder: Add one mana of any color to your mana pool.","4(R/G)(R/G)","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Black Lotus","{T}, Sacrifice Black Lotus: Add three mana of any one color to your mana pool.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slinking Skirge","2","1","B","Flying
{2}, Sacrifice Slinking Skirge: Draw a card.","3B","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stolen Grain","B","Stolen Grain deals 5 damage to target opponent. You gain 5 life.","4BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Catapult Master","3","3","W","Tap five untapped Soldiers you control: Exile target creature.","3WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wu Longbowman","1","1","U","{T}: Wu Longbowman deals 1 damage to target creature or player. Activate this ability only during your turn, before attackers are declared.","2U","Creature - Human Soldier Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moonglove Winnower","2","3","B","Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","3B","Creature - Elf Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mask of Law and Grace","W","Enchant creature
Enchanted creature has protection from black and from red.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Birds of Paradise","0","1","G","Flying
{T}: Add one mana of any color to your mana pool.","G","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Gaveleer","1","1","R","Trample
Goblin Gaveleer gets +2/+0 for each Equipment attached to it.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Korlash, Heir to Blackblade","*","*","B","Korlash, Heir to Blackblade's power and toughness are each equal to the number of Swamps you control.
{1}{B}: Regenerate Korlash.
Grandeur - Discard another card named Korlash, Heir to Blackblade: Search your library for up to two Swamp cards, put them onto the battlefield tapped, then shuffle your library.","2BB","Legendary Creature - Zombie Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Excavation","U","{1}, Sacrifice a land: Draw a card. Any player may activate this ability.","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragon Whelp","2","3","R","Flying
{R}: Dragon Whelp gets +1/+0 until end of turn. If this ability has been activated four or more times this turn, sacrifice Dragon Whelp at the beginning of the next end step.","2RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragon Broodmother","4","4","RG","Flying
At the beginning of each upkeep, put a 1/1 red and green Dragon creature token with flying and devour 2 onto the battlefield. (As the token enters the battlefield, you may sacrifice any number of creatures. It enters the battlefield with twice that many +1/+1 counters on it.)","2RRRG","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stinkdrinker Daredevil","1","3","R","Giant spells you cast cost {2} less to cast.","2R","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Masako the Humorless","2","1","W","Flash
Tapped creatures you control can block as though they were untapped.","2W","Legendary Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Future Sight","U","Play with the top card of your library revealed.
You may play the top card of your library.","2UUU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spikeshot Goblin","1","2","R","{R}, {T}: Spikeshot Goblin deals damage equal to its power to target creature or player.","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Resounding Wave","U","Return target permanent to its owner's hand.
Cycling {5}{W}{U}{B} ({5}{W}{U}{B}, Discard this card: Draw a card.)
When you cycle Resounding Wave, return two target permanents to their owners' hands.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heavy Fog","G","Cast Heavy Fog only during the declare attackers step and only if you've been attacked this step.
Prevent all damage that would be dealt to you this turn by attacking creatures.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Soul Net","Whenever a creature dies, you may pay {1}. If you do, you gain 1 life.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faceless Devourer","2","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
When Faceless Devourer enters the battlefield, exile another target creature with shadow.
When Faceless Devourer leaves the battlefield, return the exiled card to the battlefield under its owner's control.","2B","Creature - Nightmare Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Baton of Courage","Flash
Sunburst (This enters the battlefield with a charge counter on it for each color of mana spent to cast it.)
Remove a charge counter from Baton of Courage: Target creature gets +1/+1 until end of turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sutured Ghoul","*","*","B","Trample
As Sutured Ghoul enters the battlefield, exile any number of creature cards from your graveyard.
Sutured Ghoul's power is equal to the total power of the exiled cards and its toughness is equal to their total toughness.","4BBB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Watchwolf","3","3","WG",NULL,"GW","Creature - Wolf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Desolate Lighthouse","{T}: Add {1} to your mana pool.
{1}{U}{R}, {T}: Draw a card, then discard a card.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Llanowar Reborn","Llanowar Reborn enters the battlefield tapped.
{T}: Add {G} to your mana pool.
Graft 1 (This land enters the battlefield with a +1/+1 counter on it. Whenever a creature enters the battlefield, you may move a +1/+1 counter from this land onto it.)",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Icy Manipulator","{1}, {T}: Tap target artifact, creature, or land.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skeleton Ship","0","3","UB","When you control no Islands, sacrifice Skeleton Ship.
{T}: Put a -1/-1 counter on target creature.","3UB","Legendary Creature - Skeleton");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crystal Ball","{1}, {T}: Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Smoke","R","Players can't untap more than one creature during their untap steps.","RR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashnod's Coupon","ocT, Sacrifice Ashnod's Coupon: Target player gets you target drink.
Errata: You pay any costs for the drink.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reality Anchor","G","Target creature loses shadow until end of turn.
Draw a card.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Dead Shall Serve","When you set this scheme in motion, for each opponent, put up to one target creature card from that player's graveyard onto the battlefield under your control. Each of those creatures attacks its owner each combat if able.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vigilante Justice","R","Whenever a Human enters the battlefield under your control, Vigilante Justice deals 1 damage to target creature or player.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seasoned Tactician","1","3","W","{3}, Exile the top four cards of your library: The next time a source of your choice would deal damage to you this turn, prevent that damage.","2W","Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hold the Line","W","Blocking creatures get +7/+7 until end of turn.","1WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Titan's Revenge","R","Titan's Revenge deals X damage to target creature or player. Clash with an opponent. If you win, return Titan's Revenge to its owner's hand. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","XRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Typhoon","G","Typhoon deals damage to each opponent equal to the number of Islands that player controls.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyhunter Patrol","2","3","W","Flying, first strike (This creature can't be blocked except by creatures with flying or reach, and it deals combat damage before creatures without first strike.)","2WW","Creature - Cat Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Festering Wound","B","Enchant creature
At the beginning of your upkeep, you may put an infection counter on Festering Wound.
At the beginning of the upkeep of enchanted creature's controller, Festering Wound deals X damage to that player, where X is the number of infection counters on Festering Wound.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Chandra, the Firebrand","R","+1: Chandra, the Firebrand deals 1 damage to target creature or player.
-2: When you cast your next instant or sorcery spell this turn, copy that spell. You may choose new targets for the copy.
-6: Chandra, the Firebrand deals 6 damage to each of up to six target creatures and/or players.","Planeswalker - Chandra","3R");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drooling Groodion","4","3","BG","{2}{B}{G}, Sacrifice a creature: Target creature gets +2/+2 until end of turn. Another target creature gets -2/-2 until end of turn.","3BBG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flame Rift","R","Flame Rift deals 4 damage to each player.","1R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Coastal Tower","Coastal Tower enters the battlefield tapped.
{T}: Add {W} or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leering Gargoyle","2","2","WU","Flying
{T}: Leering Gargoyle gets -2/+2 and loses flying until end of turn.","1WU","Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angelic Overseer","5","3","W","Flying
As long as you control a Human, Angelic Overseer has hexproof and is indestructible.","3WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Troubled Healer","1","2","W","Sacrifice a land: Prevent the next 2 damage that would be dealt to target creature or player this turn.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Divining Witch","1","1","B","{1}{B}, {T}, Discard a card: Name a card. Exile the top six cards of your library. Reveal cards from the top of your library until you reveal the named card, then put that card into your hand. Exile all other cards revealed this way.","1B","Creature - Human Spellshaper");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Conjurer's Closet","At the beginning of your end step, you may exile target creature you control, then return that card to the battlefield under your control.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Skyreaver","6","6","U","Flying
{1}: Mistform Skyreaver becomes the creature type of your choice until end of turn.","5UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rusted Slasher","4","1","Sacrifice an artifact: Regenerate Rusted Slasher.","4","Artifact Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Obelisk of Jund","{T}: Add {B}, {R}, or {G} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plated Sliver","1","1","W","All Sliver creatures get +0/+1.","W","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charmbreaker Devils","4","4","R","At the beginning of your upkeep, return an instant or sorcery card at random from your graveyard to your hand.
Whenever you cast an instant or sorcery spell, Charmbreaker Devils gets +4/+0 until end of turn.","5R","Creature - Devil");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Catalog","U","Draw two cards, then discard a card.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Black Scarab","W","Enchant creature
Enchanted creature can't be blocked by black creatures.
Enchanted creature gets +2/+2 as long as an opponent controls a black permanent.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iname as One","8","8","BG","When Iname as One enters the battlefield, if you cast it from your hand, you may search your library for a Spirit permanent card, put it onto the battlefield, then shuffle your library.
When Iname as One dies, you may exile it. If you do, return target Spirit permanent card from your graveyard to the battlefield.","8BBGG","Legendary Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wurm's Tooth","Whenever a player casts a green spell, you may gain 1 life.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Purge","W","Destroy target artifact creature or black creature. It can't be regenerated.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Banewasp Affliction","B","Enchant creature
When enchanted creature dies, that creature's controller loses life equal to its toughness.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unscythe, Killer of Kings","UBR","Equipped creature gets +3/+3 and has first strike.
Whenever a creature dealt damage by equipped creature this turn dies, you may exile that card. If you do, put a 2/2 black Zombie creature token onto the battlefield.
Equip {2}","UBBR","Legendary Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbed-Back Wurm","4","3","B","{B}: Target green creature blocking Barbed-Back Wurm gets -1/-1 until end of turn.","4B","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Undying Flames","R","Exile cards from the top of your library until you exile a nonland card. Undying Flames deals damage to target creature or player equal to that card's converted mana cost.
Epic (For the rest of the game, you can't cast spells. At the beginning of each of your upkeeps, copy this spell except for its epic ability. You may choose a new target for the copy.)","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Arena","5","5","R","{3}, {T}: Tap target creature you control and target creature of an opponent's choice he or she controls. Those creatures fight each other. (Each deals damage equal to its power to the other.)","4RR","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steam Frigate","3","3","U","Steam Frigate can't attack unless defending player controls an Island.","2U","Creature - Human Pirate");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sinstriker's Will","W","Enchant creature
Enchanted creature has ""{T}: This creature deals damage equal to its power to target attacking or blocking creature.""","3W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devastation","R","Destroy all creatures and lands.","5RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nezumi Shortfang","1","1","B","{1}{B}, {T}: Target opponent discards a card. Then if that player has no cards in hand, flip Nezumi Shortfang.
---
At the beginning of each opponent's upkeep, that player loses 1 life for each card fewer than three in his or her hand.","1B","Creature - Rat Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blurred Mongoose","2","1","G","Blurred Mongoose can't be countered.
Shroud (This permanent can't be the target of spells or abilities.)","1G","Creature - Mongoose");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirsdag Flayer","1","1","B","{3}{B}, {T}, Sacrifice a Human: Destroy target creature.","1B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cessation","W","Enchant creature
Enchanted creature can't attack.
When Cessation is put into a graveyard from the battlefield, return Cessation to its owner's hand.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demolish","R","Destroy target artifact or land.","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Nova","W","Exile target attacking creature and all Equipment attached to it.","3WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Evaporate","R","Evaporate deals 1 damage to each white and/or blue creature.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reviving Vapors","WU","Reveal the top three cards of your library and put one of them into your hand. You gain life equal to that card's converted mana cost. Put all other cards revealed this way into your graveyard.","2WU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crown of Empires","{3}, {T}: Tap target creature. Gain control of that creature instead if you control artifacts named Scepter of Empires and Throne of Empires.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu Elite Companions","3","3","W","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","4W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skeletonize","R","Skeletonize deals 3 damage to target creature. When a creature dealt damage this way dies this turn, put a 1/1 black Skeleton creature token onto the battlefield with ""{B}: Regenerate this creature.""","4R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diabolic Edict","B","Target player sacrifices a creature.","1B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Life Chisel","Sacrifice a creature: You gain life equal to the sacrificed creature's toughness. Activate this ability only during your upkeep.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stampede Driver","1","1","G","{1}{G}, {T}, Discard a card: Creatures you control get +1/+1 and gain trample until end of turn.","G","Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cauldron Dance","BR","Cast Cauldron Dance only during combat.
Return target creature card from your graveyard to the battlefield. That creature gains haste. Return it to your hand at the beginning of the next end step.
You may put a creature card from your hand onto the battlefield. That creature gains haste. Its controller sacrifices it at the beginning of the next end step.","4BR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Hypnotist","1","1","U","{2}{B}, {T}: Target player discards a card. Activate this ability only any time you could cast a sorcery.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Wind","B","Target creature gets -X/-X until end of turn.","XB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Caustic Tar","B","Enchant land
Enchanted land has ""{T}: Target player loses 3 life.""","4BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pus Kami","3","3","B","{B}, Sacrifice Pus Kami: Destroy target nonblack creature.
Soulshift 6 (When this creature dies, you may return target Spirit card with converted mana cost 6 or less from your graveyard to your hand.)","5BB","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dakmor Plague","B","Dakmor Plague deals 3 damage to each creature and each player.","3BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sylvan Might","G","Target creature gets +2/+2 and gains trample until end of turn.
Flashback {2}{G}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1G","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Stonework Puma","2","2",NULL,"3","Artifact Creature - Cat Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ritual of Restoration","W","Return target artifact card from your graveyard to your hand.","W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwalla","2","2","G","{1}{G}: Rootwalla gets +2/+2 until end of turn. Activate this ability only once each turn.","2G","Creature - Lizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rofellos","Whenever a creature you control dies, draw a card.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swift Maneuver","W","Prevent the next 2 damage that would be dealt to target creature or player this turn.
Draw a card at the beginning of the next turn's upkeep.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ancestor's Prophet","1","5","W","Tap five untapped Clerics you control: You gain 10 life.","4W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cruel Edict","B","Target opponent sacrifices a creature.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cloven Casting","UR","Whenever you cast a multicolored instant or sorcery spell, you may pay {1}. If you do, copy that spell. You may choose new targets for the copy.","5UR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wipe Away","U","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Return target permanent to its owner's hand.","1UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nomad Decoy","1","2","W","{W}, {T}: Tap target creature.
Threshold - {W}{W}, {T}: Tap two target creatures. Activate this ability only if seven or more cards are in your graveyard.","2W","Creature - Human Nomad");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("My Undead Horde Awakens","(An ongoing scheme remains face up until it's abandoned.)
At the beginning of your end step, you may put target creature card from an opponent's graveyard onto the battlefield under your control.
When a creature put onto the battlefield with this scheme dies, abandon this scheme.",NULL,"Ongoing Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Latulla, Keldon Overseer","3","3","R","{X}{R}, {T}, Discard two cards: Latulla, Keldon Overseer deals X damage to target creature or player.","3RR","Legendary Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Natural Selection","G","Look at the top three cards of target player's library, then put them back in any order. You may have that player shuffle his or her library.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tremor","R","Tremor deals 1 damage to each creature without flying.","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bear Umbra","G","Enchant creature
Enchanted creature gets +2/+2 and has ""Whenever this creature attacks, untap all lands you control.""
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","2GG","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Floodwater Dam","{X}{X}{1}, {T}: Tap X target lands.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Higure, the Still Wind","3","4","U","Ninjutsu {2}{U}{U} ({2}{U}{U}, Return an unblocked attacker you control to hand: Put this card onto the battlefield from your hand tapped and attacking.)
Whenever Higure, the Still Wind deals combat damage to a player, you may search your library for a Ninja card, reveal it, and put it into your hand. If you do, shuffle your library.
{2}: Target Ninja creature is unblockable this turn.","3UU","Legendary Creature - Human Ninja");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Day of Judgment","W","Destroy all creatures.","2WW","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Windswept Heath","{T}, Pay 1 life, Sacrifice Windswept Heath: Search your library for a Forest or Plains card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Splinterfright","*","*","G","Trample
Splinterfright's power and toughness are each equal to the number of creature cards in your graveyard.
At the beginning of your upkeep, put the top two cards of your library into your graveyard.","2G","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Looming Shade","1","1","B","{B}: Looming Shade gets +1/+1 until end of turn.","2B","Creature - Shade");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sway of the Stars","U","Each player shuffles his or her hand, graveyard, and permanents he or she owns into his or her library, then draws seven cards. Each player's life total becomes 7.","8UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Legions of Lim-Dûl","2","3","B","Snow swampwalk","1BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merrow Grimeblotter","2","2","UB","{1}{(u/b)}, {Q}: Target creature gets -2/-0 until end of turn. ({Q} is the untap symbol.)","3(U/B)","Creature - Merfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sungrass Egg","{2}, {T}, Sacrifice Sungrass Egg: Add {G}{W} to your mana pool. Draw a card.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dakmor Salvage","Dakmor Salvage enters the battlefield tapped.
{T}: Add {B} to your mana pool.
Dredge 2 (If you would draw a card, instead you may put exactly two cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curiosity","U","Enchant creature
Whenever enchanted creature deals damage to an opponent, you may draw a card.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tamanoa","2","4","WRG","Whenever a noncreature source you control deals damage, you gain that much life.","RGW","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skyshroud Blessing","G","Lands gain shroud until end of turn. (They can't be the targets of spells or abilities.)
Draw a card.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gauntlet of Might","Red creatures get +1/+1.
Whenever a Mountain is tapped for mana, its controller adds {R} to his or her mana pool (in addition to the mana the land produces).","4","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Demon","*","*","Flying",NULL,"Creature - Demon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Golden Urn","At the beginning of your upkeep, you may put a charge counter on Golden Urn.
{T}, Sacrifice Golden Urn: You gain life equal to the number of charge counters on Golden Urn.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Vandal","1","1","U","At the beginning of your upkeep, put a shred counter on Cephalid Vandal. Then put the top card of your library into your graveyard for each shred counter on Cephalid Vandal.","1U","Creature - Cephalid Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bubbling Muck","B","Until end of turn, whenever a player taps a Swamp for mana, that player adds {B} to his or her mana pool (in addition to the mana the land produces).","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Salvage Scout","1","1","W","{W}, Sacrifice Salvage Scout: Return target artifact card from your graveyard to your hand.","W","Creature - Human Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burst of Energy","W","Untap target permanent.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thoughtbind","U","Counter target spell with converted mana cost 4 or less.","2U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arcanis, the Omnipotent Avatar","{X}, Return a creature you control with converted mana cost X to its owner's hand: Draw a number of cards chosen at random between 0 and X. X can't be 0.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Samite Sanctuary","W","{2}: Prevent the next 1 damage that would be dealt to target creature this turn. Any player may activate this ability.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tears of Rage","R","Cast Tears of Rage only during the declare attackers step.
Attacking creatures you control get +X/+0 until end of turn, where X is the number of attacking creatures. Sacrifice those creatures at the beginning of the next end step.","2RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Shikari","2","2","W","You may activate equip abilities any time you could cast an instant.","1W","Creature - Cat Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shard Volley","R","As an additional cost to cast Shard Volley, sacrifice a land.
Shard Volley deals 3 damage to target creature or player.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tooth and Claw","R","Sacrifice two creatures: Put a 3/1 red Beast creature token named Carnivore onto the battlefield.","3R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wax // Wane","W","Destroy target enchantment.
---
Target creature gets +2/+2 until end of turn.","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crosis's Catacombs","When Crosis's Catacombs enters the battlefield, sacrifice it unless you return a non-Lair land you control to its owner's hand.
{T}: Add {U}, {B}, or {R} to your mana pool.",NULL,"Land - Lair");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gilt-Leaf Ambush","G","Put two 1/1 green Elf Warrior creature tokens onto the battlefield. Clash with an opponent. If you win, those creatures gain deathtouch until end of turn. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost. Any amount of damage a creature with deathtouch deals to a creature is enough to destroy it.)","2G","Tribal Instant - Elf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elven Lyre","{1}, {T}, Sacrifice Elven Lyre: Target creature gets +2/+2 until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wolfir Avenger","3","3","G","Flash (You may cast this spell any time you could cast an instant.)
{1}{G}: Regenerate Wolfir Avenger.","1GG","Creature - Wolf Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Slayers' Stronghold","{T}: Add {1} to your mana pool.
{R}{W}, {T}: Target creature gets +2/+0 and gains vigilance and haste until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dumb Ass","3{1/2}","2","R","At the beginning of your upkeep, flip a coin. If you lose the flip, target opponent chooses whether Dumb Ass attacks this turn.","2R","Creature - Donkey Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Verdict","W","Destroy target attacking or blocking creature.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Elder","1","1","W","Whenever an artifact enters the battlefield, you may gain 1 life.","W","Creature - Cat Cleric");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Jace, Memory Adept","U","+1: Draw a card. Target player puts the top card of his or her library into his or her graveyard.
0: Target player puts the top ten cards of his or her library into his or her graveyard.
-7: Any number of target players each draw twenty cards.","Planeswalker - Jace","3UU");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbed Shocker","2","2","R","Trample, haste
Whenever Barbed Shocker deals damage to a player, that player discards all the cards in his or her hand, then draws that many cards.","3R","Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Akoum Refuge","Akoum Refuge enters the battlefield tapped.
When Akoum Refuge enters the battlefield, you gain 1 life.
{T}: Add {B} or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rhystic Cave","{T}: Choose a color. Add one mana of that color to your mana pool unless any player pays {1}. Activate this ability only any time you could cast an instant.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mindblaze","R","Name a nonland card and choose a number greater than 0. Target player reveals his or her library. If that library contains exactly the chosen number of the named card, Mindblaze deals 8 damage to that player. Then that player shuffles his or her library.","5R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Nighthawk","2","3","B","Flying
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)
Lifelink (Damage dealt by this creature also causes you to gain that much life.)","1BB","Creature - Vampire Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oxidda Scrapmelter","3","3","R","When Oxidda Scrapmelter enters the battlefield, destroy target artifact.","3R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dispel","U","Counter target instant spell.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fugitive Druid","3","2","G","Whenever Fugitive Druid becomes the target of an Aura spell, you draw a card.","3G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Artillery","1","3","R","{T}: Goblin Artillery deals 2 damage to target creature or player and 3 damage to you.","1RR","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Barren Moor","Barren Moor enters the battlefield tapped.
{T}: Add {B} to your mana pool.
Cycling {B} ({B}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Takeno's Cavalry","1","1","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
{T}: Takeno's Cavalry deals 1 damage to target attacking or blocking Spirit.","3W","Creature - Human Samurai Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sanity Gnawers","1","1","BR","When Sanity Gnawers enters the battlefield, target player discards a card at random.","1BR","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undead Slayer","2","2","W","{W}, {T}: Exile target Skeleton, Vampire, or Zombie.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sarcatog","1","2","BR","Exile two cards from your graveyard: Sarcatog gets +1/+1 until end of turn.
Sacrifice an artifact: Sarcatog gets +1/+1 until end of turn.","1BR","Creature - Atog");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rix Maadi, Dungeon Palace","{T}: Add {1} to your mana pool.
{1}{B}{R}, {T}: Each player discards a card. Activate this ability only any time you could cast a sorcery.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earth Rift","R","Destroy target land.
Flashback {5}{R}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ostracize","B","Target opponent reveals his or her hand. You choose a creature card from it. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sensation Gorger","2","2","R","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Sensation Gorger, you may reveal it. If you do, each player discards his or her hand and draws four cards.","1RR","Creature - Goblin Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Undiscovered Paradise","{T}: Add one mana of any color to your mana pool. During your next untap step, as you untap your permanents, return Undiscovered Paradise to its owner's hand.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prismatic Wardrobe","W","Destroy target card that does not share a color with clothing worn by its controller. You cannot choose an artifact or land card.","W","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Reliquary Tower","You have no maximum hand size.
{T}: Add {1} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plow Under","G","Put two target lands on top of their owners' libraries.","3GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sabertooth Alley Cat","2","1","R","Sabertooth Alley Cat attacks each turn if able.
{1}{R}: Creatures without defender can't block Sabertooth Alley Cat this turn.","1RR","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tinder Wall","0","3","G","Defender (This creature can't attack.)
Sacrifice Tinder Wall: Add {R}{R} to your mana pool.
{R}, Sacrifice Tinder Wall: Tinder Wall deals 2 damage to target creature it's blocking.","G","Creature - Plant Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Afterlife","W","Destroy target creature. It can't be regenerated. Its controller puts a 1/1 white Spirit creature token with flying onto the battlefield.","2W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Adaptive Automaton","2","2","As Adaptive Automaton enters the battlefield, choose a creature type.
Adaptive Automaton is the chosen type in addition to its other types.
Other creatures you control of the chosen type get +1/+1.","3","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thran Weaponry","Echo {4} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
You may choose not to untap Thran Weaponry during your untap step.
{2}, {T}: All creatures get +2/+2 for as long as Thran Weaponry remains tapped.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghastly Remains","0","0","B","Amplify 1 (As this creature enters the battlefield, put a +1/+1 counter on it for each Zombie card you reveal in your hand.)
At the beginning of your upkeep, if Ghastly Remains is in your graveyard, you may pay {B}{B}{B}. If you do, return Ghastly Remains to your hand.","BBB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vanishing","U","Enchant creature
{U}{U}: Enchanted creature phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before its controller untaps during his or her next untap step.)","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rockslide Elemental","1","1","R","First strike
Whenever another creature dies, you may put a +1/+1 counter on Rockslide Elemental.","2R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incendiary Command","R","Choose two - Incendiary Command deals 4 damage to target player; or Incendiary Command deals 2 damage to each creature; or destroy target nonbasic land; or each player discards all the cards in his or her hand, then draws that many cards.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Elite","1","1","G","Skyshroud Elite gets +1/+2 as long as an opponent controls a nonbasic land.","G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pattern of Rebirth","G","Enchant creature
When enchanted creature dies, that creature's controller may search his or her library for a creature card and put that card onto the battlefield. If that player does, he or she shuffles his or her library.","3G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tolarian Entrancer","1","1","U","Whenever Tolarian Entrancer becomes blocked by a creature, gain control of that creature at end of combat.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Unholy Fiend","3","3","At the beginning of your end step, you lose 1 life.",NULL,"Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steal Strength","B","Target creature gets +1/+1 until end of turn. Another target creature gets -1/-1 until end of turn.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Clam-I-Am","2","2","U","Whenever you roll a 3 on a six-sided die, you may reroll that die.","2U","Summon - Clamfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jedit Ojanen","5","5","WU",NULL,"4WWU","Legendary Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plague Beetle","1","1","B","Swampwalk (This creature is unblockable as long as defending player controls a Swamp.)","B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barkshell Blessing","WG","Target creature gets +2/+2 until end of turn.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose a new target for the copy.)","(G/W)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Genesis Wave","G","Reveal the top X cards of your library. You may put any number of permanent cards with converted mana cost X or less from among them onto the battlefield. Then put all cards revealed this way that weren't put onto the battlefield into your graveyard.","XGGG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Alchemist's Refuge","{T}: Add {1} to your mana pool.
{G}{U}, {T}: You may cast nonland cards this turn as though they had flash.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skulking Fugitive","3","4","B","When Skulking Fugitive becomes the target of a spell or ability, sacrifice it.","2B","Creature - Horror Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uthden Troll","2","2","R","{R}: Regenerate Uthden Troll.","2R","Creature - Troll");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Zoetic Cavern","{T}: Add {1} to your mana pool.
Morph {2} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Suq'Ata Lancer","2","2","R","Haste
Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)","2R","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strange Inversion","R","Switch target creature's power and toughness until end of turn.
Splice onto Arcane {1}{R} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","2R","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Big Game Hunter","1","1","B","When Big Game Hunter enters the battlefield, destroy target creature with power 4 or greater. It can't be regenerated.
Madness {B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","1BB","Creature - Human Rebel Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planar Overlay","U","Each player chooses a land he or she controls of each basic land type. Return those lands to their owners' hands.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chain Reaction","R","Chain Reaction deals X damage to each creature, where X is the number of creatures on the battlefield.","2RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tormented Pariah","3","2","R","At the beginning of each upkeep, if no spells were cast last turn, transform Tormented Pariah.","3R","Creature - Human Warrior Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Covetous Dragon","6","5","R","Flying
When you control no artifacts, sacrifice Covetous Dragon.","4R","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Valeron Outlander","2","2","WG","Protection from black","GW","Creature - Human Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx Summoner","3","3","UB","Flying
When Sphinx Summoner enters the battlefield, you may search your library for an artifact creature card, reveal it, and put it into your hand. If you do, shuffle your library.","3UB","Artifact Creature - Sphinx");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Glint Hawk Idol","Whenever another artifact enters the battlefield under your control, you may have Glint Hawk Idol become a 2/2 Bird artifact creature with flying until end of turn.
{W}: Glint Hawk Idol becomes a 2/2 Bird artifact creature with flying until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thornwind Faeries","1","1","U","Flying
{T}: Thornwind Faeries deals 1 damage to target creature or player.","1UU","Creature - Faerie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trusty Machete","Equipped creature gets +2/+1.
Equip {2}","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonehewer Giant","4","4","W","Vigilance
{1}{W}, {T}: Search your library for an Equipment card and put it onto the battlefield. Attach it to a creature you control. Then shuffle your library.","3WW","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Laccolith Whelp","1","1","R","Whenever Laccolith Whelp becomes blocked, you may have it deal damage equal to its power to target creature. If you do, Laccolith Whelp assigns no combat damage this turn.","R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bringer of the Blue Dawn","5","5","U","You may pay {W}{U}{B}{R}{G} rather than pay Bringer of the Blue Dawn's mana cost.
Trample
At the beginning of your upkeep, you may draw two cards.","7UU","Creature - Bringer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rocky Tar Pit","Rocky Tar Pit enters the battlefield tapped.
{T}, Sacrifice Rocky Tar Pit: Search your library for a Swamp or Mountain card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heart Warden","1","1","G","{T}: Add {G} to your mana pool.
{2}, Sacrifice Heart Warden: Draw a card.","1G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Metalworker","1","2","{T}: Reveal any number of artifact cards in your hand. Add {2} to your mana pool for each card revealed this way.","3","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Global Ruin","W","Each player chooses from the lands he or she controls a land of each basic land type, then sacrifices the rest.","4W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glacial Plating","W","Enchant creature
Cumulative upkeep {S}i} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it. {S}i} can be paid with one mana from a snow permanent.)
Enchanted creature gets +3/+3 for each age counter on Glacial Plating.","2WW","Snow Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Warlord","*","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
Dauthi Warlord's power is equal to the number of creatures with shadow on the battlefield.","1B","Creature - Dauthi Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yamabushi's Storm","R","Yamabushi's Storm deals 1 damage to each creature. If a creature dealt damage this way would die this turn, exile it instead.","1R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kite Shield","Equipped creature gets +0/+3.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery.)","0","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Slug","1","1","B","{5}: At the beginning of your next upkeep, choose a basic land type. Giant Slug gains landwalk of the chosen type until the end of that turn.","1B","Creature - Slug");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invigorating Falls","G","You gain life equal to the number of creature cards in all graveyards.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scorched Rusalka","1","1","R","{R}, Sacrifice a creature: Scorched Rusalka deals 1 damage to target player.","R","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sedge Sliver","2","2","R","All Sliver creatures have ""This creature gets +1/+1 as long as you control a Swamp.""
All Slivers have ""{B}: Regenerate this permanent.""","2R","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Snip","U","Counter target spell unless its controller pays {1}.
Cycling {2} ({2}, Discard this card: Draw a card.)","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jace's Erasure","U","Whenever you draw a card, you may have target player put the top card of his or her library into his or her graveyard.","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mephidross Vampire","3","4","B","Flying
Each creature you control is a Vampire in addition to its other creature types and has ""Whenever this creature deals damage to a creature, put a +1/+1 counter on this creature.""","4BB","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Orim's Prayer","W","Whenever one or more creatures attack you, you gain 1 life for each attacking creature.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brine Hag","2","2","U","When Brine Hag dies, all creatures that dealt damage to it this turn become 0/2. (This effect lasts indefinitely.)","2UU","Creature - Hag");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goldenglow Moth","0","1","W","Flying
Whenever Goldenglow Moth blocks, you may gain 4 life.","W","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defensive Formation","W","Rather than the attacking player, you assign the combat damage of each creature attacking you. You can divide that creature's combat damage as you choose among any of the creatures blocking it.","W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undead Warchief","1","1","B","Zombie spells you cast cost {1} less to cast.
Zombie creatures you control get +2/+1.","2BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brainbite","UB","Target opponent reveals his or her hand. You choose a card from it. That player discards that card.
Draw a card.","2UB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Profane Command","B","Choose two - Target player loses X life; or return target creature card with converted mana cost X or less from your graveyard to the battlefield; or target creature gets -X/-X until end of turn; or up to X target creatures gain fear until end of turn. (They can't be blocked except by artifact creatures and/or black creatures.)","XBB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scent of Nightshade","B","Reveal any number of black cards in your hand. Target creature gets -X/-X until end of turn, where X is the number of cards revealed this way.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bulwark","R","At the beginning of your upkeep, Bulwark deals X damage to target opponent, where X is the number of cards in your hand minus the number of cards in that player's hand.","3RR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gilt-Leaf Palace","As Gilt-Leaf Palace enters the battlefield, you may reveal an Elf card from your hand. If you don't, Gilt-Leaf Palace enters the battlefield tapped.
{T}: Add {B} or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Shaman","3","2","G","When Nantuko Shaman enters the battlefield, if you control no tapped lands, draw a card.
Suspend 1-{2}{G}{G} (Rather than cast this card from your hand, you may pay {2}{G}{G} and exile it with a time counter on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","2G","Creature - Insect Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chain of Acid","G","Destroy target noncreature permanent. Then that permanent's controller may copy this spell and may choose a new target for that copy.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Metallic Mastery","R","Gain control of target artifact until end of turn. Untap that artifact. It gains haste until end of turn.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shield of the Oversoul","WG","Enchant creature
As long as enchanted creature is green, it gets +1/+1 and is indestructible. (Lethal damage and effects that say ""destroy"" don't destroy it. If its toughness is 0 or less, it's still put into its owner's graveyard.)
As long as enchanted creature is white, it gets +1/+1 and has flying.","2(G/W)","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chains of Mephistopheles","B","If a player would draw a card except the first one he or she draws in his or her draw step each turn, that player discards a card instead. If the player discards a card this way, he or she draws a card. If the player doesn't discard a card this way, he or she puts the top card of his or her library into his or her graveyard.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brothers of Fire","2","2","R","{1}{R}{R}: Brothers of Fire deals 1 damage to target creature or player and 1 damage to you.","1RR","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Mantis","2","4","G","Reach (This creature can block creatures with flying.)","3G","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Acid Rain","U","Destroy all Forests.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mayor of Avabruck","1","1","G","Other Human creatures you control get +1/+1.
At the beginning of each upkeep, if no spells were cast last turn, transform Mayor of Avabruck.","1G","Creature - Human Advisor Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Voidslime","UG","Counter target spell, activated ability, or triggered ability. (Mana abilities can't be targeted.)","GUU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Golgari Germination","BG","Whenever a nontoken creature you control dies, put a 1/1 green Saproling creature token onto the battlefield.","1BG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodcrazed Goblin","2","2","R","Bloodcrazed Goblin can't attack unless an opponent has been dealt damage this turn.","R","Creature - Goblin Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whipcorder","2","2","W","{W}, {T}: Tap target creature.
Morph {W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","WW","Creature - Human Soldier Rebel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Talisman of Dominance","{T}: Add {1} to your mana pool.
{T}: Add {U} or {B} to your mana pool. Talisman of Dominance deals 1 damage to you.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Auntie's Hovel","As Auntie's Hovel enters the battlefield, you may reveal a Goblin card from your hand. If you don't, Auntie's Hovel enters the battlefield tapped.
{T}: Add {B} or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Immolation","R","Enchant creature
Enchanted creature gets +2/-2.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Saproling Symbiosis","G","You may cast Saproling Symbiosis any time you could cast an instant if you pay {2} more to cast it.
Put a 1/1 green Saproling creature token onto the battlefield for each creature you control.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Celestial Convergence","W","Celestial Convergence enters the battlefield with seven omen counters on it.
At the beginning of your upkeep, remove an omen counter from Celestial Convergence. If there are no omen counters on Celestial Convergence, the player with the highest life total wins the game. If two or more players are tied for highest life total, the game is a draw.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surprise Deployment","W","Cast Surprise Deployment only during combat.
You may put a nonwhite creature card from your hand onto the battlefield. At the beginning of the next end step, return that creature to your hand. (Return it only if it's on the battlefield.)","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time Stretch","U","Target player takes two extra turns after this one.","8UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashnod's Altar","Sacrifice a creature: Add {2} to your mana pool.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reprocess","B","Sacrifice any number of artifacts, creatures, and/or lands. Draw a card for each permanent sacrificed this way.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Manaforge Cinder","1","1","BR","{1}: Add {B} or {R} to your mana pool. Activate this ability no more than three times each turn.","(B/R)","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voice of Law","2","2","W","Flying, protection from red","3W","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Graceful Adept","1","3","U","You have no maximum hand size.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Suffer the Past","B","Exile X target cards from target player's graveyard. For each card exiled this way, that player loses 1 life and you gain 1 life.","XB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mountain Yeti","3","3","R","Mountainwalk, protection from white","2RR","Creature - Yeti");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volition Reins","U","Enchant permanent
When Volition Reins enters the battlefield, if enchanted permanent is tapped, untap it.
You control enchanted permanent.","3UUU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Khalni Heart Expedition","G","Landfall - Whenever a land enters the battlefield under your control, you may put a quest counter on Khalni Heart Expedition.
Remove three quest counters from Khalni Heart Expedition and sacrifice it: Search your library for up to two basic land cards, put them onto the battlefield tapped, then shuffle your library.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Martyr's Cry","W","Exile all white creatures. For each creature exiled this way, its controller draws a card.","WW","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Insatiable Souleater","5","1","{(g/p)}: Insatiable Souleater gains trample until end of turn. ({(g/p)} can be paid with either {G} or 2 life.)","4","Artifact Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scragnoth","3","4","G","Scragnoth can't be countered.
Protection from blue","4G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kingfisher","2","2","U","Flying
When Kingfisher dies, draw a card.","3U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Adarkar Valkyrie","4","5","W","Flying, vigilance
{T}: When target creature other than Adarkar Valkyrie dies this turn, return that card to the battlefield under your control.","4WW","Snow Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Epicenter","R","Target player sacrifices a land.
Threshold - Each player sacrifices all lands he or she controls instead if seven or more cards are in your graveyard.","4R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Naya Battlemage","2","2","G","{R}, {T}: Target creature gets +2/+0 until end of turn.
{W}, {T}: Tap target creature.","2G","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reckless Embermage","2","2","R","{1}{R}: Reckless Embermage deals 1 damage to target creature or player and 1 damage to itself.","3R","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goldmeadow Harrier","1","1","W","{W}, {T}: Tap target creature.","W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Telepathic Spies","2","2","U","When Telepathic Spies enters the battlefield, look at target opponent's hand.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Conjurer","0","2","U","{T}: Target snow land becomes a 2/2 creature until end of turn. It's still a land.","1U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Night Soil","G","{1}, Exile two creature cards from a single graveyard: Put a 1/1 green Saproling creature token onto the battlefield.","GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ravaged Highlands","Ravaged Highlands enters the battlefield tapped.
{T}: Add {R} to your mana pool.
{T}, Sacrifice Ravaged Highlands: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Fiend","0","0","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
At the beginning of your upkeep, you may move a +1/+1 counter from target creature onto Arcbound Fiend.
Modular 3 (This enters the battlefield with three +1/+1 counters on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","6","Artifact Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Imposing Visage","R","Enchant creature
Enchanted creature can't be blocked except by two or more creatures.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crimson Mage","2","1","R","{R}: Target creature you control gains haste until end of turn. (It can attack and {T} this turn.)","1R","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Creeping Mold","G","Destroy target artifact, enchantment, or land.","2GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragon Scales","W","Enchant creature
Enchanted creature gets +1/+2 and has vigilance.
When a creature with converted mana cost 6 or greater enters the battlefield, you may return Dragon Scales from your graveyard to the battlefield attached to that creature.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitting Sliver","3","3","B","All Sliver creatures have first strike.","4B","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reincarnation","G","Choose target creature. When that creature dies this turn, return a creature card from its owner's graveyard to the battlefield under the control of that creature's owner.","1GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wicked Akuba","2","2","B","{B}: Target player dealt damage by Wicked Akuba this turn loses 1 life.","BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pith Driller","2","4","B","({(b/p)} can be paid with either {B} or 2 life.)
When Pith Driller enters the battlefield, put a -1/-1 counter on target creature.","4(B/P)","Artifact Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harvester of Souls","5","5","B","Deathtouch
Whenever another nontoken creature dies, you may draw a card.","4BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bedlam","R","Creatures can't block.","2RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hand of Justice","2","6","W","{T}, Tap three untapped white creatures you control: Destroy target creature.","5W","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sen Triplets","3","3","WUB","At the beginning of your upkeep, choose target opponent. This turn, that player can't cast spells or activate abilities and plays with his or her hand revealed. You may play cards from that player's hand this turn.","2WUB","Legendary Artifact Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pay No Heed","W","Prevent all damage a source of your choice would deal this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Killer Whale","3","5","U","{U}: Killer Whale gains flying until end of turn.","3UU","Creature - Whale");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stingerfling Spider","2","5","G","Reach (This creature can block creatures with flying.)
When Stingerfling Spider enters the battlefield, you may destroy target creature with flying.","4G","Creature - Spider");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pyrite Spellbomb","{R}, Sacrifice Pyrite Spellbomb: Pyrite Spellbomb deals 2 damage to target creature or player.
{1}, Sacrifice Pyrite Spellbomb: Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ronin Cavekeeper","4","3","R","Bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)","5R","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raven Guild Initiate","1","4","U","Morph-Return a Bird you control to its owner's hand. (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Venomous Fangs","G","Enchant creature
Whenever enchanted creature deals damage to a creature, destroy the other creature.","2G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lifespark Spellbomb","{G}, Sacrifice Lifespark Spellbomb: Until end of turn, target land becomes a 3/3 creature that's still a land.
{1}, Sacrifice Lifespark Spellbomb: Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marshdrinker Giant","4","3","G","When Marshdrinker Giant enters the battlefield, destroy target Island or Swamp an opponent controls.","3GG","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harmattan Efreet","2","2","U","Flying
{1}{U}{U}: Target creature gains flying until end of turn.","2UU","Creature - Efreet");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Envelop","U","Counter target sorcery spell.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arcane Laboratory","U","Each player can't cast more than one spell each turn.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("No Quarter","R","Whenever a creature becomes blocked by a creature with lesser power, destroy the blocking creature.
Whenever a creature blocks a creature with lesser power, destroy the attacking creature.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oracle of Mul Daya","2","2","G","You may play an additional land on each of your turns.
Play with the top card of your library revealed.
You may play the top card of your library if it's a land card.","3G","Creature - Elf Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lightning Greaves","Equipped creature has haste and shroud. (It can't be the target of spells or abilities.)
Equip {0}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rouse","B","If you control a Swamp, you may pay 2 life rather than pay Rouse's mana cost.
Target creature gets +2/+0 until end of turn.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sedge Troll","2","2","R","Sedge Troll gets +1/+1 as long as you control a Swamp.
{B}: Regenerate Sedge Troll.","2R","Creature - Troll");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flame Slash","R","Flame Slash deals 4 damage to target creature.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oona's Prowler","3","1","B","Flying
Discard a card: Oona's Prowler gets -2/-0 until end of turn. Any player may activate this ability.","1B","Creature - Faerie Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Sacrament","W","White creatures get +1/+1.
Threshold - White creatures get an additional +1/+1 as long as seven or more cards are in your graveyard.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dungeon Geists","3","3","U","Flying
When Dungeon Geists enters the battlefield, tap target creature an opponent controls. That creature doesn't untap during its controller's untap step for as long as you control Dungeon Geists.","2UU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Our Market Research Shows That Players Like Really Long Card Names So We Made this Card to Have the Absolute Longest Card Name Ever Elemental","2","2","G","Art rampage 2 (Whenever this becomes blocked by a creature, it gets +2/+2 for each creature in the blocker's art beyond the first.)","1GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fade Away","U","For each creature, its controller sacrifices a permanent unless he or she pays {1}.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dusk Urchins","4","3","B","Whenever Dusk Urchins attacks or blocks, put a -1/-1 counter on it.
When Dusk Urchins dies, draw a card for each -1/-1 counter on it.","2B","Creature - Ouphe");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Honden of Infinite Rage","R","At the beginning of your upkeep, Honden of Infinite Rage deals damage to target creature or player equal to the number of Shrines you control.","2R","Legendary Enchantment - Shrine");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hornet Harasser","2","2","B","When Hornet Harasser dies, target creature gets -2/-2 until end of turn.","2BB","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cromat","5","5","WUBRG","{W}{B}: Destroy target creature blocking or blocked by Cromat.
{U}{R}: Cromat gains flying until end of turn.
{B}{G}: Regenerate Cromat.
{R}{W}: Cromat gets +1/+1 until end of turn.
{G}{U}: Put Cromat on top of its owner's library.","WUBRG","Legendary Creature - Illusion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jayemdae Tome","{4}, {T}: Draw a card.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Carapace","G","Enchant creature
Enchanted creature gets +0/+2.
Sacrifice Carapace: Regenerate enchanted creature.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soldevi Adnate","1","2","B","{T}, Sacrifice a black or artifact creature: Add to your mana pool an amount of {B} equal to the sacrificed creature's converted mana cost.","1B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vertigo","R","Vertigo deals 2 damage to target creature with flying. That creature loses flying until end of turn.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flying Carpet","{2}, {T}: Target creature gains flying until end of turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crypt Cobra","3","3","B","Whenever Crypt Cobra attacks and isn't blocked, defending player gets a poison counter. (A player with ten or more poison counters loses the game.)","3B","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Halam Djinn","6","5","R","Haste
Halam Djinn gets -2/-2 as long as red is the most common color among all permanents or is tied for most common.","5R","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Luminesce","W","Prevent all damage that black sources and red sources would deal this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rustmouth Ogre","5","4","R","Whenever Rustmouth Ogre deals combat damage to a player, you may destroy target artifact that player controls.","4RR","Creature - Ogre");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plumes of Peace","WU","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.
Forecast - {W}{U}, Reveal Plumes of Peace from your hand: Tap target creature. (Activate this ability only during your upkeep and only once each turn.)","1WU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Raiders","2","2","B","Swampwalk (This creature is unblockable as long as defending player controls a Swamp.)","2B","Creature - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Salt Marsh","Salt Marsh enters the battlefield tapped.
{T}: Add {U} or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of the Palace Fields","3","2","W","Flying, first strike
Soulshift 5 (When this creature dies, you may return target Spirit card with converted mana cost 5 or less from your graveyard to your hand.)","5W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kalitas, Bloodchief of Ghet","5","5","B","{B}{B}{B}, {T}: Destroy target creature. If that creature dies this way, put a black Vampire creature token onto the battlefield. Its power is equal to that creature's power and its toughness is equal to that creature's toughness.","5BB","Legendary Creature - Vampire Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vigilant Sentry","2","2","W","Threshold - As long as seven or more cards are in your graveyard, Vigilant Sentry gets +1/+1 and has ""{T}: Target attacking or blocking creature gets +3/+3 until end of turn.""","1WW","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crystallization","WUG","Enchant creature
Enchanted creature can't attack or block.
When enchanted creature becomes the target of a spell or ability, exile that creature.","(G/U)W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of Stromgald","2","1","B","Protection from white
{B}: Knight of Stromgald gains first strike until end of turn.
{B}{B}: Knight of Stromgald gets +1/+0 until end of turn.","BB","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bathe in Light","W","Radiance - Choose a color. Target creature and each other creature that shares a color with it gain protection from the chosen color until end of turn.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brine Seer","1","1","U","{2}{U}, {T}: Reveal any number of blue cards in your hand. Counter target spell unless its controller pays {1} for each card revealed this way.","3U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flayed Nim","2","2","B","Whenever Flayed Nim deals combat damage to a creature, that creature's controller loses that much life.
{2}{B}: Regenerate Flayed Nim.","3B","Creature - Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wrack with Madness","R","Target creature deals damage to itself equal to its power.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darigaaz, the Igniter","6","6","BRG","Flying
Whenever Darigaaz, the Igniter deals combat damage to a player, you may pay {2}{R}. If you do, choose a color, then that player reveals his or her hand and Darigaaz deals damage to the player equal to the number of cards of that color revealed this way.","3BRG","Legendary Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sonic Seizure","R","As an additional cost to cast Sonic Seizure, discard a card at random.
Sonic Seizure deals 3 damage to target creature or player.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancient Grudge","R","Destroy target artifact.
Flashback {G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Summoning","U","Counter target creature spell.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overwhelming Intellect","U","Counter target creature spell. Draw cards equal to that spell's converted mana cost.","4UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jedit Ojanen of Efrava","5","5","G","Forestwalk
Whenever Jedit Ojanen of Efrava attacks or blocks, put a 2/2 green Cat Warrior creature token with forestwalk onto the battlefield.","3GGG","Legendary Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Surrakar Banisher","3","3","U","When Surrakar Banisher enters the battlefield, you may return target tapped creature to its owner's hand.","4U","Creature - Surrakar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Solar Blast","R","Solar Blast deals 3 damage to target creature or player.
Cycling {1}{R}{R} ({1}{R}{R}, Discard this card: Draw a card.)
When you cycle Solar Blast, you may have it deal 1 damage to target creature or player.","3R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cowardice","U","Whenever a creature becomes the target of a spell or ability, return that creature to its owner's hand. (It won't be affected by the spell or ability.)","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Calciderm","5","5","W","Shroud (This permanent can't be the target of spells or abilities.)
Vanishing 4 (This permanent enters the battlefield with four time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)","2WW","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blitz Hellion","7","7","RG","Trample, haste
At the beginning of the end step, Blitz Hellion's owner shuffles it into his or her library.","3RG","Creature - Hellion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunscape Battlemage","2","2","W","Kicker {1}{G} and/or {2}{U} (You may pay an additional {1}{G} and/or {2}{U} as you cast this spell.)
When Sunscape Battlemage enters the battlefield, if it was kicked with its {1}{G} kicker, destroy target creature with flying.
When Sunscape Battlemage enters the battlefield, if it was kicked with its {2}{U} kicker, draw two cards.","2W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Circu, Dimir Lobotomist","2","3","UB","Whenever you cast a blue spell, exile the top card of target library.
Whenever you cast a black spell, exile the top card of target library.
Your opponents can't cast nonland cards with the same name as a card exiled with Circu, Dimir Lobotomist.","2UB","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancient Runes","R","At the beginning of each player's upkeep, Ancient Runes deals damage to that player equal to the number of artifacts he or she controls.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magistrate's Veto","R","White creatures and blue creatures can't block.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ardent Soldier","1","2","W","Kicker {2} (You may pay an additional {2} as you cast this spell.)
Vigilance
If Ardent Soldier was kicked, it enters the battlefield with a +1/+1 counter on it.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Runed Servitor","2","2","When Runed Servitor dies, each player draws a card.","2","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gerrard Capashen","3","4","W","At the beginning of your upkeep, you gain 1 life for each card in target opponent's hand.
{3}{W}: Tap target creature. Activate this ability only if Gerrard Capashen is attacking.","3WW","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Battle Screech","W","Put two 1/1 white Bird creature tokens with flying onto the battlefield.
Flashback-Tap three untapped white creatures you control. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Split-Tail Miko","1","1","W","{W}, {T}: Prevent the next 2 damage that would be dealt to target creature or player this turn.","1W","Creature - Fox Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gallowbraid","5","5","B","Trample
Cumulative upkeep-Pay 1 life. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","3BB","Legendary Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seek the Horizon","G","Search your library for up to three basic land cards, reveal them, and put them into your hand. Then shuffle your library.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drawn Together","W","As Drawn Together comes into play, choose an artist.
Creatures by the chosen artist get +2/+2.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kamahl, Fist of Krosa","4","3","G","{G}: Target land becomes a 1/1 creature until end of turn. It's still a land.
{2}{G}{G}{G}: Creatures you control get +3/+3 and gain trample until end of turn.","4GG","Legendary Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Cockroach","4","2","B",NULL,"3B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Camouflage","G","Cast Camouflage only during your declare attackers step.
This turn, instead of declaring blockers, each defending player chooses any number of creatures he or she controls and divides them into a number of piles equal to the number of attacking creatures for whom that player is the defending player. Creatures he or she controls that can block additional creatures may likewise be put into additional piles. Assign each pile to a different one of those attacking creatures at random. Each creature in a pile that can block the creature that pile is assigned to does so. (Piles can be empty.)","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nalathni Dragon","1","1","R","Flying; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
{R}: Nalathni Dragon gets +1/+0 until end of turn. If this ability has been activated four or more times this turn, sacrifice Nalathni Dragon at the beginning of the next end step.","2RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Veteran","2","2","R","Orcish Veteran can't block white creatures with power 2 or greater.
{R}: Orcish Veteran gains first strike until end of turn.","2R","Creature - Orc");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhuge Jin, Wu Strategist","1","1","U","{T}: Target creature is unblockable this turn. Activate this ability only during your turn, before attackers are declared.","1UU","Legendary Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defiant Stand","W","Cast Defiant Stand only during the declare attackers step and only if you've been attacked this step.
Target creature gets +1/+3 until end of turn. Untap that creature.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devout Monk","1","1","W","When Devout Monk enters the battlefield, you gain 1 life.","W","Creature - Human Monk Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bloodscent","G","All creatures able to block target creature this turn do so.","3G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seething Anger","R","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Target creature gets +3/+0 until end of turn.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Harbinger","1","3","W","When Kithkin Harbinger enters the battlefield, you may search your library for a Kithkin card, reveal it, then shuffle your library and put that card on top of it.","2W","Creature - Kithkin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spellbound Dragon","3","5","UR","Flying
Whenever Spellbound Dragon attacks, draw a card, then discard a card. Spellbound Dragon gets +X/+0 until end of turn, where X is the discarded card's converted mana cost.","3UR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ill-Gotten Gains","B","Exile Ill-Gotten Gains. Each player discards his or her hand, then returns up to three cards from his or her graveyard to his or her hand.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Spring","UG","Put target permanent on top of its owner's library.","1GU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screeching Skaab","2","1","U","When Screeching Skaab enters the battlefield, put the top two cards of your library into your graveyard.","1U","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enslave","B","Enchant creature
You control enchanted creature.
At the beginning of your upkeep, enchanted creature deals 1 damage to its owner.","4BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spectral Prison","U","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.
When enchanted creature becomes the target of a spell, sacrifice Spectral Prison.","1U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Coral Atoll","Coral Atoll enters the battlefield tapped.
When Coral Atoll enters the battlefield, sacrifice it unless you return an untapped Island you control to its owner's hand.
{T}: Add {1}{U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Upwelling","G","Mana pools don't empty as steps and phases end.","3G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Déjà Vu","U","Return target sorcery card from your graveyard to your hand.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mystic Zealot","2","4","W","Threshold - As long as seven or more cards are in your graveyard, Mystic Zealot gets +1/+1 and has flying.","3W","Creature - Human Nomad Mystic");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mire Shade","1","1","B","{B}, Sacrifice a Swamp: Put a +1/+1 counter on Mire Shade. Activate this ability only any time you could cast a sorcery.","1B","Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Ridge Exhumer","1","1","B","{B}, {T}, Discard a card: Put a 1/1 black Zombie Goblin creature token named Festering Goblin onto the battlefield. It has ""When Festering Goblin dies, target creature gets -1/-1 until end of turn.""","1B","Creature - Zombie Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Casting of Bones","B","Enchant creature
When enchanted creature dies, draw three cards, then discard one of them.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Royal Herbalist","1","1","W","{2}, Exile the top card of your library: You gain 1 life.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Mammoth","3","4","G","At the beginning of your upkeep, if a player controls more creatures than each other player, the player who controls the most creatures gains control of Wild Mammoth.","2G","Creature - Elephant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sphere of Truth","W","If a white source would deal damage to you, prevent 2 of that damage.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Acidic Slime","2","2","G","Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)
When Acidic Slime enters the battlefield, destroy target artifact, enchantment, or land.","3GG","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Traumatic Visions","U","Counter target spell.
Basic landcycling {1}{U} ({1}{U}, Discard this card: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.)","3UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argent Sphinx","4","3","U","Flying
Metalcraft - {U}: Exile Argent Sphinx. Return it to the battlefield under your control at the beginning of the next end step. Activate this ability only if you control three or more artifacts.","2UU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clone Shell","2","2","Imprint - When Clone Shell enters the battlefield, look at the top four cards of your library, exile one face down, then put the rest on the bottom of your library in any order.
When Clone Shell dies, turn the exiled card face up. If it's a creature card, put it onto the battlefield under your control.","5","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sootstoke Kindler","1","1","BR","Haste
{T}: Target black or red creature gains haste until end of turn.","1(B/R)","Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sterling Grove","WG","Other enchantments you control have shroud. (They can't be the targets of spells or abilities.)
{1}, Sacrifice Sterling Grove: Search your library for an enchantment card and reveal that card. Shuffle your library, then put the card on top of it.","GW","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Ironfoot","3","4","Phyrexian Ironfoot doesn't untap during your untap step.
{1}{S}i}: Untap Phyrexian Ironfoot. ({S}i} can be paid with one mana from a snow permanent.)","3","Snow Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frankenstein's Monster","0","1","B","As Frankenstein's Monster enters the battlefield, exile X creature cards from your graveyard. If you can't, put Frankenstein's Monster into its owner's graveyard instead of onto the battlefield. For each creature card exiled this way, Frankenstein's Monster enters the battlefield with a +2/+0, +1/+1, or +0/+2 counter on it.","XBB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flight","U","Enchant creature
Enchanted creature has flying.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Village Survivors","4","5","G","Vigilance
Fateful hour - As long as you have 5 or less life, other creatures you control have vigilance.","4G","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reweave","U","Target permanent's controller sacrifices it. If he or she does, that player reveals cards from the top of his or her library until he or she reveals a permanent card that shares a card type with the sacrificed permanent, puts that card onto the battlefield, then shuffles his or her library.
Splice onto Arcane {2}{U}{U} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","5U","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul's Might","G","Put X +1/+1 counters on target creature, where X is that creature's power.","4G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arrogant Vampire","4","3","B","Flying","3BB","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oculus","1","1","U","When Oculus dies, you may draw a card.","1U","Creature - Homunculus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyknight Legionnaire","2","2","WR","Flying, haste","1RW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grizzly Bears","2","2","G",NULL,"1G","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhox Meditant","2","4","W","When Rhox Meditant enters the battlefield, if you control a green permanent, draw a card.","3W","Creature - Rhino Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Streambed Aquitects","2","3","U","{T}: Target Merfolk creature gets +1/+1 and gains islandwalk until end of turn.
{T}: Target land becomes an Island until end of turn.","1UU","Creature - Merfolk Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Creature Bond","U","Enchant creature
When enchanted creature dies, Creature Bond deals damage equal to that creature's toughness to the creature's controller.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blazing Shoal","R","You may exile a red card with converted mana cost X from your hand rather than pay Blazing Shoal's mana cost.
Target creature gets +X/+0 until end of turn.","XRR","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snake Pit","G","Whenever an opponent casts a blue or black spell, you may put a 1/1 green Snake creature token onto the battlefield.","3G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guan Yu's 1,000-Li March","W","Destroy all tapped creatures.","4WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tahngarth's Rage","R","Enchant creature
Enchanted creature gets +3/+0 as long as it's attacking. Otherwise, it gets -2/-1.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primordial Ooze","1","1","R","Primordial Ooze attacks each turn if able.
At the beginning of your upkeep, put a +1/+1 counter on Primordial Ooze. Then you may pay {X}, where X is the number of +1/+1 counters on it. If you don't, tap Primordial Ooze and it deals X damage to you.","R","Creature - Ooze");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rusted Sentinel","3","4","Rusted Sentinel enters the battlefield tapped.","4","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stuffy Doll Avatar","{0}: If damage would be dealt to you this turn by a source of your choice, prevent that damage. Stuffy Doll Avatar deals damage to you and target opponent equal to half the damage prevented this way, rounded up. Activate this ability only once each turn.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jade Idol","Whenever you cast a Spirit or Arcane spell, Jade Idol becomes a 4/4 Spirit artifact creature until end of turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Roterothopter","0","2","Flying
{2}: Roterothopter gets +1/+0 until end of turn. Activate this ability no more than twice each turn.","1","Artifact Creature - Thopter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Working Stiff","2","2","B","As Working Stiff comes into play, straighten your arms.
When you bend an elbow, sacrifice Working Stiff.","1B","Creature - Mummy");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oxidda Daredevil","2","1","R","Sacrifice an artifact: Oxidda Daredevil gains haste until end of turn.","1R","Creature - Goblin Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coffin Queen","1","1","B","You may choose not to untap Coffin Queen during your untap step.
{2}{B}, {T}: Put target creature card from a graveyard onto the battlefield under your control. When Coffin Queen becomes untapped or you lose control of Coffin Queen, exile that creature.","2B","Creature - Zombie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pulse of the Dross","B","Target player reveals three cards from his or her hand and you choose one of them. That player discards that card. Then if that player has more cards in hand than you, return Pulse of the Dross to its owner's hand.","1BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghave, Guru of Spores","0","0","WBG","Ghave, Guru of Spores enters the battlefield with five +1/+1 counters on it.
{1}, Remove a +1/+1 counter from a creature you control: Put a 1/1 green Saproling creature token onto the battlefield.
{1}, Sacrifice a creature: Put a +1/+1 counter on target creature.","2BGW","Legendary Creature - Fungus Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cartographer","2","2","G","When Cartographer enters the battlefield, you may return target land card from your graveyard to your hand.","2G","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Captive Flame","R","{R}: Target creature gets +1/+0 until end of turn.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Katabatic Winds","G","Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)
Creatures with flying can't attack or block, and their activated abilities with {T} in their costs can't be activated.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Halo Hunter","6","3","B","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)
When Halo Hunter enters the battlefield, destroy target Angel.","2BBB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Builder's Blessing","W","Untapped creatures you control get +0/+2.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stronghold Machinist","1","1","U","{U}{U}, {T}, Discard a card: Counter target noncreature spell.","2U","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Invader Parasite","3","2","R","Imprint - When Invader Parasite enters the battlefield, exile target land.
Whenever a land with the same name as the exiled card enters the battlefield under an opponent's control, Invader Parasite deals 2 damage to that player.","3RR","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silver Wyvern","4","3","U","Flying
{U}: Change the target of target spell or ability that targets only Silver Wyvern. The new target must be a creature.","3UU","Creature - Drake");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Markov's Servant","4","4",NULL,NULL,"Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Delirium","BR","Cast Delirium only during an opponent's turn.
Tap target creature that player controls. That creature deals damage equal to its power to the player. Prevent all combat damage that would be dealt to and dealt by the creature this turn.","1BR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timmerian Fiends","1","1","B","Remove Timmerian Fiends from your deck before playing if you're not playing for ante.
{B}{B}{B}, Sacrifice Timmerian Fiends: The owner of target artifact may ante the top card of his or her library. If that player doesn't, exchange ownership of that artifact and Timmerian Fiends. Put the artifact card into your graveyard and Timmerian Fiends from anywhere into that player's graveyard. This change in ownership is permanent.","1BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Limestone Golem","3","4","{2}, Sacrifice Limestone Golem: Target player draws a card.","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Callous Oppressor","1","2","U","You may choose not to untap Callous Oppressor during your untap step.
As Callous Oppressor enters the battlefield, an opponent chooses a creature type.
{T}: Gain control of target creature that isn't of the chosen type for as long as Callous Oppressor remains tapped.","1UU","Creature - Cephalid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Worldheart Phoenix","2","2","R","Flying
You may cast Worldheart Phoenix from your graveyard by paying {W}{U}{B}{R}{G} rather than paying its mana cost. If you do, it enters the battlefield with two +1/+1 counters on it.","3R","Creature - Phoenix");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pulse of the Tangle","G","Put a 3/3 green Beast creature token onto the battlefield. Then if an opponent controls more creatures than you, return Pulse of the Tangle to its owner's hand.","1GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leech Bonder","3","3","U","Leech Bonder enters the battlefield with two -1/-1 counters on it.
{U}, {Q}: Move a counter from target creature onto another target creature. ({Q} is the untap symbol.)","2U","Creature - Merfolk Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Tears","B","Destroy target land.","1BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armor Sliver","2","2","W","All Sliver creatures have ""{2}: This creature gets +0/+1 until end of turn.""","2W","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Chirurgeon","0","2","R","Sacrifice a Goblin: Regenerate target creature.","R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sleeper's Robe","UB","Enchant creature
Enchanted creature has fear. (It can't be blocked except by artifact creatures and/or black creatures.)
Whenever enchanted creature deals combat damage to an opponent, you may draw a card.","UB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blaze of Glory","W","Cast Blaze of Glory only during combat before blockers are declared.
Target creature defending player controls can block any number of creatures this turn. It blocks each attacking creature this turn if able.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Life's Finale","B","Destroy all creatures, then search target opponent's library for up to three creature cards and put them into his or her graveyard. Then that player shuffles his or her library.","4BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reckless Spite","B","Destroy two target nonblack creatures. You lose 5 life.","1BB","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Adventuring Gear","Landfall - Whenever a land enters the battlefield under your control, equipped creature gets +2/+2 until end of turn.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Traumatize","U","Target player puts the top half of his or her library, rounded down, into his or her graveyard.","3UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scalding Tongs","At the beginning of your upkeep, if you have three or fewer cards in hand, Scalding Tongs deals 1 damage to target opponent.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancestral Tribute","W","You gain 2 life for each card in your graveyard.
Flashback {9}{W}{W}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","5WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Solidarity","W","Creatures you control get +0/+5 until end of turn.","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sanctimony","W","Whenever an opponent taps a Mountain for mana, you may gain 1 life.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stromgald Cabal","2","2","B","{T}, Pay 1 life: Counter target white spell.","1BB","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Haakon, Stromgald Scourge","3","3","B","You may cast Haakon, Stromgald Scourge from your graveyard, but not from anywhere else.
As long as Haakon is on the battlefield, you may play Knight cards from your graveyard.
When Haakon dies, you lose 2 life.","1BB","Legendary Creature - Zombie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nearheath Stalker","4","1","R","Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","4R","Creature - Vampire Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hex Parasite","1","1","{X}{(b/p)}: Remove up to X counters from target permanent. For each counter removed this way, Hex Parasite gets +1/+0 until end of turn. ({(b/p)} can be paid with either {B} or 2 life.)","1","Artifact Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrion Wurm","6","5","B","Whenever Carrion Wurm attacks or blocks, any player may exile three cards from his or her graveyard. If a player does, Carrion Wurm assigns no combat damage this turn.","3BB","Creature - Zombie Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Humble","W","Target creature loses all abilities and becomes 0/1 until end of turn.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rune of Protection: Lands","W","{W}: The next time a land source of your choice would deal damage to you this turn, prevent that damage.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Town Sentry","2","2","W","Whenever Town Sentry blocks, it gets +0/+2 until end of turn.","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Misinformation","B","Put up to three target cards from an opponent's graveyard on top of his or her library in any order.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cackling Fiend","2","1","B","When Cackling Fiend enters the battlefield, each opponent discards a card.","2BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conviction","W","Enchant creature
Enchanted creature gets +1/+3.
{W}: Return Conviction to its owner's hand.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alabaster Leech","1","3","W","White spells you cast cost {W} more to cast.","W","Creature - Leech");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inundate","U","Return all nonblue creatures to their owners' hands.","3UUU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Vandal","1","1","R","Whenever Goblin Vandal attacks and isn't blocked, you may pay {R}. If you do, destroy target artifact defending player controls and Goblin Vandal assigns no combat damage this turn.","R","Creature - Goblin Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Living Armor","{T}, Sacrifice Living Armor: Put X +0/+1 counters on target creature, where X is that creature's converted mana cost.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akroma's Devoted","2","4","W","Cleric creatures have vigilance.","3W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tolarian Winds","U","Discard all the cards in your hand, then draw that many cards.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arcane Teachings","R","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature gets +2/+2 and has ""{T}: This creature deals 1 damage to target creature or player.""","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daru Stinger","1","1","W","Amplify 1 (As this creature enters the battlefield, put a +1/+1 counter on it for each Human and/or Soldier card you reveal in your hand.)
{T}: Daru Stinger deals damage equal to the number of +1/+1 counters on it to target attacking or blocking creature.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squeaking Pie Sneak","2","2","B","As an additional cost to cast Squeaking Pie Sneak, reveal a Goblin card from your hand or pay {3}.
Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)","1B","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudreach Cavalry","1","1","W","As long as you control a Bird, Cloudreach Cavalry gets +2/+2 and has flying.","1W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Diamond Faerie Avatar","{S}i}: Target creature you control gets +1/+1 until end of turn.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Splinter Twin","R","Enchant creature
Enchanted creature has ""{T}: Put a token that's a copy of this creature onto the battlefield. That token has haste. Exile it at the beginning of the next end step.""","2RR","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Controlled Instincts","U","Enchant red or green creature
Enchanted creature doesn't untap during its controller's untap step.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plated Pegasus","1","2","W","Flash (You may cast this spell any time you could cast an instant.)
Flying
If a spell would deal damage to a creature or player, prevent 1 damage that spell would deal to that creature or player.","2W","Creature - Pegasus");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Unholy Grotto","{T}: Add {1} to your mana pool.
{B}, {T}: Put target Zombie card from your graveyard on top of your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hundred-Talon Kami","2","3","W","Flying
Soulshift 4 (When this creature dies, you may return target Spirit card with converted mana cost 4 or less from your graveyard to your hand.)","4W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Levitation","U","Creatures you control have flying.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Soldiers","1","2","U",NULL,"1U","Creature - Merfolk Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rhystic Syphon","B","Unless target player pays {3}, he or she loses 5 life and you gain 5 life.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ostiary Thrull","2","2","B","{W}, {T}: Tap target creature.","3B","Creature - Thrull");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zuberi, Golden Feather","3","3","W","Flying
Other Griffin creatures get +1/+1.","4W","Legendary Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodland Sleuth","2","3","G","Morbid - When Woodland Sleuth enters the battlefield, if a creature died this turn, return a creature card at random from your graveyard to your hand.","3G","Creature - Human Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Illusionary Wall","7","4","U","Defender, flying, first strike
Cumulative upkeep {U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","4U","Creature - Illusion Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horde of Boggarts","*","*","R","Horde of Boggarts's power and toughness are each equal to the number of red permanents you control.
Horde of Boggarts can't be blocked except by two or more creatures.","3R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mishra, Artificer Prodigy","4","4","UBR","Whenever you cast an artifact spell, you may search your graveyard, hand, and/or library for a card with the same name as that spell and put it onto the battlefield. If you search your library this way, shuffle it.","1UBR","Legendary Creature - Human Artificer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Amulet of Quoz","Remove Amulet of Quoz from your deck before playing if you're not playing for ante.
{T}, Sacrifice Amulet of Quoz: Target opponent may add the top card of his or her library to the ante. If he or she doesn't, you flip a coin. If you win the flip, that player loses the game. If you lose the flip, you lose the game. Activate this ability only during your upkeep.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul Collector","3","4","B","Flying
Whenever a creature dealt damage by Soul Collector this turn dies, return that card to the battlefield under your control.
Morph {B}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3BB","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dire Wolves","2","2","G","Dire Wolves has banding as long as you control a Plains. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","2G","Creature - Wolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dystopia","B","Cumulative upkeep-Pay 1 life. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of each player's upkeep, that player sacrifices a green or white permanent.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wistful Selkie","2","2","UG","When Wistful Selkie enters the battlefield, draw a card.","(G/U)(G/U)(G/U)","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiritmonger","6","6","BG","Whenever Spiritmonger deals damage to a creature, put a +1/+1 counter on Spiritmonger.
{B}: Regenerate Spiritmonger.
{G}: Spiritmonger becomes the color of your choice until end of turn.","3BG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trench Wurm","3","3","B","{2}{R}, {T}: Destroy target nonbasic land.","3B","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boggart Mob","5","5","B","Champion a Goblin (When this enters the battlefield, sacrifice it unless you exile another Goblin you control. When this leaves the battlefield, that card returns to the battlefield.)
Whenever a Goblin you control deals combat damage to a player, you may put a 1/1 black Goblin Rogue creature token onto the battlefield.","3B","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surging AEther","U","Ripple 4 (When you cast this spell, you may reveal the top four cards of your library. You may cast any revealed cards with the same name as this spell without paying their mana costs. Put the rest on the bottom of your library.)
Return target permanent to its owner's hand.","3U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fortify","W","Choose one - Creatures you control get +2/+0 until end of turn; or creatures you control get +0/+2 until end of turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pit Spawn","6","4","B","First strike
At the beginning of your upkeep, sacrifice Pit Spawn unless you pay {B}{B}.
Whenever Pit Spawn deals damage to a creature, exile that creature.","4BBB","Creature - Demon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirari","Whenever you cast an instant or sorcery spell, you may pay {3}. If you do, copy that spell. You may choose new targets for the copy.","5","Legendary Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pact of Negation","Counter target spell.
At the beginning of your next upkeep, pay {3}{U}{U}. If you don't, you lose the game.","0","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inkfathom Infiltrator","2","1","UB","Inkfathom Infiltrator can't block and is unblockable.","(U/B)(U/B)","Creature - Merfolk Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crovax","Whenever a creature you control deals damage to a creature or player, you gain 1 life.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lichenthrope","5","5","G","If damage would be dealt to Lichenthrope, put that many -1/-1 counters on it instead.
At the beginning of your upkeep, remove a -1/-1 counter from Lichenthrope.","3GG","Creature - Plant Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wanderlust","G","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, Wanderlust deals 1 damage to that player.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Atalya, Samite Master","2","3","W","{X}, {T}: Choose one - Prevent the next X damage that would be dealt to target creature this turn; or you gain X life. Spend only white mana this way.","3WW","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sabertooth Wyvern","3","2","R","Flying, first strike","4R","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duct Crawler","1","1","R","{1}{R}: Target creature can't block Duct Crawler this turn.","R","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flaccify","U","Counter target spell unless its controller pays {o3o{1/2}}.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whip Vine","1","4","G","Defender; reach (This creature can block creatures with flying.)
You may choose not to untap Whip Vine during your untap step.
{T}: Tap target creature with flying blocked by Whip Vine. That creature doesn't untap during its controller's untap step for as long as Whip Vine remains tapped.","2G","Creature - Plant Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Branchsnap Lorian","4","1","G","Trample
Morph {G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oracle en-Vec","1","1","W","{T}: Target opponent chooses any number of creatures he or she controls. During that player's next turn, the chosen creatures attack if able, and other creatures can't attack. At the beginning of that turn's end step, destroy each of the chosen creatures that didn't attack. Activate this ability only during your turn.","1W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yellow Scarves Troops","2","2","R","Yellow Scarves Troops can't block.","1R","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aggravated Assault","R","{3}{R}{R}: Untap all creatures you control. After this main phase, there is an additional combat phase followed by an additional main phase. Activate this ability only any time you could cast a sorcery.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Penance","W","Put a card from your hand on top of your library: The next time a black or red source of your choice would deal damage this turn, prevent that damage.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raging Bull","2","2","R",NULL,"2R","Creature - Ox");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Highway Robber","2","2","B","When Highway Robber enters the battlefield, target opponent loses 2 life and you gain 2 life.","2BB","Creature - Human Rogue Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Graypelt Hunter","2","2","G","Trample
Whenever Graypelt Hunter or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Graypelt Hunter.","3G","Creature - Human Warrior Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trace of Abundance","WRG","Enchant land
Enchanted land has shroud. (It can't be the target of spells or abilities.)
Whenever enchanted land is tapped for mana, its controller adds one mana of any color to his or her mana pool (in addition to the mana the land produces).","(R/W)G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nafs Asp","1","1","G","Whenever Nafs Asp deals damage to a player, that player loses 1 life at the beginning of his or her next draw step unless he or she pays {1} before that draw step.","G","Creature - Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burnout","R","Counter target instant spell if it's blue.
Draw a card at the beginning of the next turn's upkeep.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Secluded Glen","As Secluded Glen enters the battlefield, you may reveal a Faerie card from your hand. If you don't, Secluded Glen enters the battlefield tapped.
{T}: Add {U} or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidewalker","*","*","U","Tidewalker enters the battlefield with a time counter on it for each Island you control.
Vanishing (At the beginning of your upkeep, remove a time counter from this permanent. When the last is removed, sacrifice it.)
Tidewalker's power and toughness are each equal to the number of time counters on it.","2U","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Molting Harpy","2","1","B","Flying
At the beginning of your upkeep, sacrifice Molting Harpy unless you pay {2}.","B","Creature - Harpy Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbtooth Wurm","6","4","G",NULL,"5G","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Break Asunder","G","Destroy target artifact or enchantment.
Cycling {2} ({2}, Discard this card: Draw a card.)","2GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Predatory Hunger","G","Enchant creature
Whenever an opponent casts a creature spell, put a +1/+1 counter on enchanted creature.","G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rapid Fire","W","Cast Rapid Fire only before blockers are declared.
Target creature gains first strike until end of turn. If it doesn't have rampage, that creature gains rampage 2 until end of turn. (Whenever this creature becomes blocked, it gets +2/+2 until end of turn for each creature blocking it beyond the first.)","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elder Land Wurm","5","5","W","Defender, trample
When Elder Land Wurm blocks, it loses defender.","4WWW","Creature - Dragon Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rift Elemental","1","1","R","{1}{R}, Remove a time counter from a permanent you control or suspended card you own: Rift Elemental gets +2/+0 until end of turn.","R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Merchant Scroll","U","Search your library for a blue instant card, reveal that card, and put it into your hand. Then shuffle your library.","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Steelshaper","1","1","W","Equip costs you pay cost {1} less.
As long as Auriok Steelshaper is equipped, Soldier and/or Knight creatures you control get +1/+1.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Peregrine Griffin","2","4","W","Flying
First strike (This creature deals combat damage before creatures without first strike.)","4W","Creature - Griffin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Swamp","B",NULL,"Basic Land - Swamp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Humility","W","All creatures lose all abilities and are 1/1.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guided Passage","URG","Reveal the cards in your library. An opponent chooses from among them a creature card, a land card, and a noncreature, nonland card. You put the chosen cards into your hand. Then shuffle your library.","URG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Silver-Inlaid Dagger","Equipped creature gets +2/+0.
As long as equipped creature is a Human, it gets an additional +1/+0.
Equip {2}","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sigarda, Host of Herons","5","5","WG","Flying, hexproof
Spells and abilities your opponents control can't cause you to sacrifice permanents.","2GWW","Legendary Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jalum Grifter","3","5","R","o1oR, ocT: Put Jalum Grifter and two lands you control face down in front of target opponent after revealing each card to him or her. Then, rearrange the order of the three cards as often as you wish, keeping them on the table at all times. That opponent then chooses one of those cards. If a land is chosen, destroy target card in play. Otherwise, sacrifice Jalum Grifter.","3RR","Summon - Legend");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sunken Ruins","{T}: Add {1} to your mana pool.
{(u/b)}, {T}: Add {U}{U}, {U}{B}, or {B}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Transmutation","B","Switch target creature's power and toughness until end of turn.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mirror Strike","W","All combat damage that would be dealt to you this turn by target unblocked creature is dealt to its controller instead.","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Noble Purpose","W","Whenever a creature you control deals combat damage, you gain that much life.","3WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sidar Kondo","{3}: Target creature gets +3/+3 until end of turn.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zektar Shrine Expedition","R","Landfall - Whenever a land enters the battlefield under your control, you may put a quest counter on Zektar Shrine Expedition.
Remove three quest counters from Zektar Shrine Expedition and sacrifice it: Put a 7/1 red Elemental creature token with trample and haste onto the battlefield. Exile it at the beginning of the next end step.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aesthetic Consultation","B","Name an artist. Remove the top six cards of your library from the game, then reveal cards from the top of your library until you reveal a card by the named artist. Put that card in your hand, then remove all the other cards revealed this way from the game.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ember Weaver","2","3","G","Reach (This can block creatures with flying.)
As long as you control a red permanent, Ember Weaver gets +1/+0 and has first strike.","2G","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Endure","W","Prevent all damage that would be dealt to you and permanents you control this turn.","3WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nearheath Pilgrim","2","1","W","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Nearheath Pilgrim is paired with another creature, both creatures have lifelink.","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Walking Corpse","2","2","B",NULL,"1B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pavel Maliki","5","3","BR","{B}{R}: Pavel Maliki gets +1/+0 until end of turn.","4BR","Legendary Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of the White Shield","2","1","W","Protection from black
{W}: Order of the White Shield gains first strike until end of turn.
{W}{W}: Order of the White Shield gets +1/+0 until end of turn.","WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jungle Troll","2","1","RG","{R}: Regenerate Jungle Troll.
{G}: Regenerate Jungle Troll.","1RG","Creature - Troll");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Morgue Theft","B","Return target creature card from your graveyard to your hand.
Flashback {4}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skinrender","3","3","B","When Skinrender enters the battlefield, put three -1/-1 counters on target creature.","2BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Honden of Life's Web","G","At the beginning of your upkeep, put a 1/1 colorless Spirit creature token onto the battlefield for each Shrine you control.","4G","Legendary Enchantment - Shrine");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lava Storm","R","Lava Storm deals 2 damage to each attacking creature or Lava Storm deals 2 damage to each blocking creature.","3RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devouring Swarm","2","1","B","Flying
Sacrifice a creature: Devouring Swarm gets +1/+1 until end of turn.","1BB","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Sliver","1","1","U","All Slivers have ""{1}: This permanent becomes the creature type of your choice in addition to its other types until end of turn.""","1U","Creature - Illusion Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ancient Den","(Ancient Den isn't a spell.)
{T}: Add {W} to your mana pool.",NULL,"Artifact Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Collective Voyage","G","Join forces - Starting with you, each player may pay any amount of mana. Each player searches his or her library for up to X basic land cards, where X is the total amount of mana paid this way, puts them onto the battlefield tapped, then shuffles his or her library.","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Changeling Sentinel","3","2","W","Changeling (This card is every creature type at all times.)
Vigilance","3W","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flame Burst","R","Flame Burst deals X damage to target creature or player, where X is 2 plus the number of cards named Flame Burst in all graveyards.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faerie Macabre","2","2","B","Flying
Discard Faerie Macabre: Exile up to two target cards from graveyards.","1BB","Creature - Faerie Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wickerbough Elder","4","4","G","Wickerbough Elder enters the battlefield with a -1/-1 counter on it.
{G}, Remove a -1/-1 counter from Wickerbough Elder: Destroy target artifact or enchantment.","3G","Creature - Treefolk Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unstable Hulk","2","2","R","Morph {3}{R}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Unstable Hulk is turned face up, it gets +6/+6 and gains trample until end of turn. You skip your next turn.","1RR","Creature - Goblin Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Salvage Slasher","1","1","B","Salvage Slasher gets +1/+0 for each artifact card in your graveyard.","1B","Artifact Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heartseeker","Equipped creature gets +2/+1 and has ""{T}, Unattach Heartseeker: Destroy target creature.""
Equip {5} ({5}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","4","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Melting","G","Destroy target artifact or enchantment.
Draw a card at the beginning of the next turn's upkeep.","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drifter il-Dal","2","1","U","Shadow (This creature can block or be blocked by only creatures with shadow.)
At the beginning of your upkeep, sacrifice Drifter il-Dal unless you pay {U}.","U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunted Ghoul","1","2","B","Hunted Ghoul can't block Humans.","B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Psychosis Crawler","*","*","Psychosis Crawler's power and toughness are each equal to the number of cards in your hand.
Whenever you draw a card, each opponent loses 1 life.","5","Artifact Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rolling Stones","W","Wall creatures can attack as though they didn't have defender.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Carbonize","R","Carbonize deals 3 damage to target creature or player. That creature can't be regenerated this turn. If the creature would die this turn, exile it instead.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trained Orgg","6","6","R",NULL,"6R","Creature - Orgg");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Greater Auramancy","W","Other enchantments you control have shroud.
Enchanted creatures you control have shroud.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chapel Geist","2","3","W","Flying","1WW","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudthresher","7","7","G","Flash
Reach (This can block creatures with flying.)
When Cloudthresher enters the battlefield, it deals 2 damage to each creature with flying and each player.
Evoke {2}{G}{G} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","2GGGG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heartstabber Mosquito","2","2","B","Kicker {2}{B} (You may pay an additional {2}{B} as you cast this spell.)
Flying
When Heartstabber Mosquito enters the battlefield, if it was kicked, destroy target creature.","3B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Power Artifact","U","Enchant artifact
Enchanted artifact's activated abilities cost {2} less to activate. This effect can't reduce the amount of mana an ability costs to activate to less than one mana.","UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vassal's Duty","W","{1}: The next 1 damage that would be dealt to target legendary creature you control this turn is dealt to you instead.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Marble Chalice","W","{T}: You gain 1 life.","2W","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nuisance Engine","{2}, {T}: Put a 0/1 colorless Pest artifact creature token onto the battlefield.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Notorious Assassin","2","2","B","{2}{B}, {T}, Discard a card: Destroy target nonblack creature. It can't be regenerated.","3B","Creature - Human Spellshaper Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Double Header","2","3","U","Flying
When Double Header comes into play, you may return target permanent with a two-word name to its owner's hand.","3UU","Creature - Drake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Great Furnace","(Great Furnace isn't a spell.)
{T}: Add {R} to your mana pool.",NULL,"Artifact Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thousand-legged Kami","6","6","G","Soulshift 7 (When this creature dies, you may return target Spirit card with converted mana cost 7 or less from your graveyard to your hand.)","6GG","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Denial","0","8","WU","Defender, flying
Shroud (This creature can't be the target of spells or abilities.)","1WU","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Windstorm","G","Windstorm deals X damage to each creature with flying.","XG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divergent Growth","G","Until end of turn, lands you control gain ""{T}: Add one mana of any color to your mana pool.""","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grand Coliseum","Grand Coliseum enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{T}: Add one mana of any color to your mana pool. Grand Coliseum deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Giant's Ire","R","Giant's Ire deals 4 damage to target player. If you control a Giant, draw a card.","3R","Tribal Sorcery - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tooth and Nail","G","Choose one - Search your library for up to two creature cards, reveal them, put them into your hand, then shuffle your library; or put up to two creature cards from your hand onto the battlefield.
Entwine {2} (Choose both if you pay the entwine cost.)","5GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mudbutton Torchrunner","1","1","R","When Mudbutton Torchrunner dies, it deals 3 damage to target creature or player.","2R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aphetto Exterminator","3","1","B","Morph {3}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Aphetto Exterminator is turned face up, target creature gets -3/-3 until end of turn.","2B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brand of Ill Omen","R","Enchant creature
Cumulative upkeep {R} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Enchanted creature's controller can't cast creature spells.","3R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cultivate","G","Search your library for up to two basic land cards, reveal those cards, and put one onto the battlefield tapped and the other into your hand. Then shuffle your library.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dakmor Sorceress","*","4","B","Dakmor Sorceress's power is equal to the number of Swamps you control.","5B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defensive Stance","U","Enchant creature
Enchanted creature gets -1/+1.","U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seismic Strike","R","Seismic Strike deals damage to target creature equal to the number of Mountains you control.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Drover","2","2","G","Creature spells you cast with converted mana cost 6 or greater cost {2} less to cast.","3G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whim of Volrath","U","Buyback {2} (You may pay an additional {2} as you cast this spell. If you do, put this card into your hand as it resolves.)
Change the text of target permanent by replacing all instances of one color word or basic land type with another until end of turn. (For example, you may change ""nonred creature"" to ""nongreen creature"" or ""plainswalk"" to ""swampwalk."")","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("An-Havva Constable","2","1+*","G","An-Havva Constable's toughness is equal to 1 plus the number of green creatures on the battlefield.","1GG","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ribbons of the Reikai","U","Draw a card for each Spirit you control.","4U","Sorcery - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Diviner","0","1","W","{T}: Tap target Spirit.","W","Creature - Fox Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Starlight","W","You gain 3 life for each black creature target opponent controls.","1W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Artificial Evolution","U","Change the text of target spell or permanent by replacing all instances of one creature type with another. The new creature type can't be Wall. (This effect lasts indefinitely.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rafiq of the Many","3","3","WUG","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
Whenever a creature you control attacks alone, it gains double strike until end of turn.","1GWU","Legendary Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blind Zealot","2","2","B","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)
Whenever Blind Zealot deals combat damage to a player, you may sacrifice it. If you do, destroy target creature that player controls.","1BB","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shapeshifter's Marrow","U","At the beginning of each opponent's upkeep, that player reveals the top card of his or her library. If it's a creature card, the player puts the card into his or her graveyard and Shapeshifter's Marrow becomes a copy of that card. (If it does, it loses this ability.)","2UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Torch Song","R","At the beginning of your upkeep, you may put a verse counter on Torch Song.
{2}{R}, Sacrifice Torch Song: Torch Song deals X damage to target creature or player, where X is the number of verse counters on Torch Song.","2R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Saprazzan Cove","Saprazzan Cove enters the battlefield tapped.
{T}: Put a storage counter on Saprazzan Cove.
{T}, Remove any number of storage counters from Saprazzan Cove: Add {U} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master of Etherium","*","*","U","Master of Etherium's power and toughness are each equal to the number of artifacts you control.
Other artifact creatures you control get +1/+1.","2U","Artifact Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Captain of the Watch","3","3","W","Vigilance (Attacking doesn't cause this creature to tap.)
Other Soldier creatures you control get +1/+1 and have vigilance.
When Captain of the Watch enters the battlefield, put three 1/1 white Soldier creature tokens onto the battlefield.","4WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cautery Sliver","2","2","WR","All Slivers have ""{1}, Sacrifice this permanent: This permanent deals 1 damage to target creature or player.""
All Slivers have ""{1}, Sacrifice this permanent: Prevent the next 1 damage that would be dealt to target Sliver creature or player this turn.""","RW","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Butcher Ghoul","1","1","B","Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","1B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cosmic Larva","7","6","R","Trample
At the beginning of your upkeep, sacrifice Cosmic Larva unless you sacrifice two lands.","1RR","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gemhide Sliver","1","1","G","All Slivers have ""{T}: Add one mana of any color to your mana pool.""","1G","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Winding Canyons","{T}: Add {1} to your mana pool.
{2}, {T}: Until end of turn, you may play creature cards as though they had flash.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eladamri, Lord of Leaves","2","2","G","Other Elf creatures have forestwalk.
Other Elves have shroud. (They can't be the targets of spells or abilities.)","GG","Legendary Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stand Firm","W","Target creature gets +1/+1 until end of turn.
Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faceless Butcher","2","3","B","When Faceless Butcher enters the battlefield, exile target creature other than Faceless Butcher.
When Faceless Butcher leaves the battlefield, return the exiled card to the battlefield under its owner's control.","2BB","Creature - Nightmare Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Archon of Redemption","3","4","W","Flying
Whenever Archon of Redemption or another creature with flying enters the battlefield under your control, you may gain life equal to that creature's power.","3WW","Creature - Archon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thundersong Trumpeter","2","1","WR","{T}: Target creature can't attack or block this turn.","RW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Uncontrolled Infestation","R","Enchant nonbasic land
When enchanted land becomes tapped, destroy it.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Baloth Woodcrasher","4","4","G","Landfall - Whenever a land enters the battlefield under your control, Baloth Woodcrasher gets +4/+4 and gains trample until end of turn.","4GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Splitting Headache","B","Choose one - Target player discards two cards; or target player reveals his or her hand, you choose a card from it, then that player discards that card.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Snapping Thragg","3","3","R","Whenever Snapping Thragg deals combat damage to a player, you may have it deal 3 damage to target creature that player controls.
Morph {4}{R}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Somber Hoverguard","3","2","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Flying","5U","Creature - Drone");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mesmeric Orb","Whenever a permanent becomes untapped, that permanent's controller puts the top card of his or her library into his or her graveyard.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cerulean Wisps","U","Target creature becomes blue until end of turn. Untap that creature.
Draw a card.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inviolability","W","Enchant creature
Prevent all damage that would be dealt to enchanted creature.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Interdict","U","Counter target activated ability from an artifact, creature, enchantment, or land. That permanent's activated abilities can't be activated this turn. (Mana abilities can't be targeted.)
Draw a card.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Purge","BR","Destroy any number of target creatures.
Phyrexian Purge costs 3 life more to cast for each target.","2BR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Feroz's Ban","Creature spells cost {2} more to cast.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Saltblast","W","Destroy target nonwhite permanent.","3WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Second Guess","U","Counter target spell that's the second spell cast this turn.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cruel Revival","B","Destroy target non-Zombie creature. It can't be regenerated. Return up to one target Zombie card from your graveyard to your hand.","4B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blessed Orator","1","4","W","Other creatures you control get +0/+1.","3W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Heralds","2","4","W","{3}{U}, {T}: Draw a card.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tower Drake","2","1","U","Flying
{W}: Tower Drake gets +0/+1 until end of turn.","2U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Piper","1","1","G","{G}, {T}: You may put a creature card from your hand onto the battlefield.","3G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dralnu's Pet","2","2","U","Kicker-{2}{B}, Discard a creature card. (You may pay {2}{B} and discard a creature card in addition to any other costs as you cast this spell.)
If Dralnu's Pet was kicked, it enters the battlefield with flying and with X +1/+1 counters on it, where X is the discarded card's converted mana cost.","1UU","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flames of the Blood Hand","R","Flames of the Blood Hand deals 4 damage to target player. The damage can't be prevented. If that player would gain life this turn, that player gains no life instead.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Mystic","1","1","U","{T}: Target instant or sorcery spell becomes the color of your choice.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drogskol Captain","2","2","WU","Flying
Other Spirit creatures you control get +1/+1 and have hexproof. (They can't be the targets of spells or abilities your opponents control.)","1WU","Creature - Spirit Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fervent Cathar","2","1","R","Haste
When Fervent Cathar enters the battlefield, target creature can't block this turn.","2R","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seer's Vision","UB","Your opponents play with their hands revealed.
Sacrifice Seer's Vision: Look at target player's hand and choose a card from it. That player discards that card. Activate this ability only any time you could cast a sorcery.","2UB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Mentor","1","1","G","{2}{G}, {T}: Target creature gets +X/+X until end of turn, where X is that creature's power.","2G","Creature - Insect Druid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Spinal Parasite","-1","-1","Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it.)
Remove two +1/+1 counters from Spinal Parasite: Remove a counter from target permanent.","5","Artifact Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kinsbaile Borderguard","1","1","W","Kinsbaile Borderguard enters the battlefield with a +1/+1 counter on it for each other Kithkin you control.
When Kinsbaile Borderguard dies, put a 1/1 white Kithkin Soldier creature token onto the battlefield for each counter on it.","1WW","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Blast","R","Lightning Blast deals 4 damage to target creature or player.","3R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhys the Redeemed","1","1","WG","{2}{(g/w)}, {T}: Put a 1/1 green and white Elf Warrior creature token onto the battlefield.
{4}{(g/w){(g/w)}, {T}: For each creature token you control, put a token that's a copy of that creature onto the battlefield.","(G/W)","Legendary Creature - Elf Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mortal Flesh Is Weak","When you set this scheme in motion, each opponent's life total becomes the lowest life total among your opponents.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cleansing Beam","R","Radiance - Cleansing Beam deals 2 damage to target creature and each other creature that shares a color with it.","4R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Grasp","WB","Death Grasp deals X damage to target creature or player. You gain X life.","XWB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reflex Sliver","2","2","G","All Sliver creatures have haste.","3G","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quillmane Baku","3","3","U","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Quillmane Baku.
{1}, {T}, Remove X ki counters from Quillmane Baku: Return target creature with converted mana cost X or less to its owner's hand.","4U","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Barl's Cage","{3}: Target creature doesn't untap during its controller's next untap step.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skulking Ghost","2","1","B","Flying
When Skulking Ghost becomes the target of a spell or ability, sacrifice it.","1B","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Diamond Kaleidoscope","{3}, {T}: Put a 0/1 colorless Prism artifact creature token onto the battlefield.
Sacrifice a Prism token: Add one mana of any color to your mana pool.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Testament of Faith","W","{X}: Testament of Faith becomes an X/X Wall creature with defender until end of turn. It's still an enchantment.","W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spinneret Sliver","2","2","G","All Sliver creatures have reach. (They can block creatures with flying.)","1G","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pyre Zombie","2","1","BR","At the beginning of your upkeep, if Pyre Zombie is in your graveyard, you may pay {1}{B}{B}. If you do, return Pyre Zombie to your hand.
{1}{R}{R}, Sacrifice Pyre Zombie: Pyre Zombie deals 2 damage to target creature or player.","1BR","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lava Burst","R","Lava Burst deals X damage to target creature or player. If Lava Burst would deal damage to a creature, that damage can't be prevented or dealt instead to another creature or player.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rank and File","3","3","B","When Rank and File enters the battlefield, green creatures get -1/-1 until end of turn.","2BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistmeadow Witch","1","1","WU","{2}{W}{U}: Exile target creature. Return that card to the battlefield under its owner's control at the beginning of the next end step.","1(W/U)","Creature - Kithkin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Homing Sliver","2","2","R","Each Sliver card in each player's hand has slivercycling {3}.
Slivercycling {3} ({3}, Discard this card: Search your library for a Sliver card, reveal it, and put it into your hand. Then shuffle your library.)","2R","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slay","B","Destroy target green creature. It can't be regenerated.
Draw a card.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turn to Frog","U","Target creature loses all abilities and becomes a 1/1 blue Frog until end of turn.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benevolent Unicorn","1","2","W","If a spell would deal damage to a creature or player, it deals that much damage minus 1 to that creature or player instead.","1W","Creature - Unicorn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stream of Consciousness","U","Target player shuffles up to four target cards from his or her graveyard into his or her library.","1U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boros Recruit","1","1","WR","({(r/w)} can be paid with either {R} or {W}.)
First strike","(R/W)","Creature - Goblin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Cavalry","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)","1W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Armorer","0","2","R","{R}, {T}, Discard a card: Put a +0/+1 counter or a +1/+0 counter on target creature.","R","Creature - Dwarf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Sentinel","2","3","G","When Llanowar Sentinel enters the battlefield, you may pay {1}{G}. If you do, search your library for a card named Llanowar Sentinel and put that card onto the battlefield. Then shuffle your library.","2G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Reflection","G","If you tap a permanent for mana, it produces twice as much of that mana instead.","4GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unstoppable Ash","5","5","G","Trample
Champion a Treefolk or Warrior (When this enters the battlefield, sacrifice it unless you exile another Treefolk or Warrior you control. When this leaves the battlefield, that card returns to the battlefield.)
Whenever a creature you control becomes blocked, it gets +0/+5 until end of turn.","3G","Creature - Treefolk Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of the Reliquary","2","2","WG","Knight of the Reliquary gets +1/+1 for each land card in your graveyard.
{T}, Sacrifice a Forest or Plains: Search your library for a land card, put it onto the battlefield, then shuffle your library.","1GW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sawback Manticore","2","4","RG","{4}: Sawback Manticore gains flying until end of turn.
{1}: Sawback Manticore deals 2 damage to target attacking or blocking creature. Activate this ability only if Sawback Manticore is attacking or blocking and only once each turn.","3RG","Creature - Manticore");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Monkey Monkey Monkey","1","1","G","As Monkey Monkey Monkey comes into play, choose a letter.
Monkey Monkey Monkey gets +1/+1 for each nonland permanent whose name begins with the chosen letter.","3G","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trained Armodon","3","3","G",NULL,"1GG","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Sandscout","2","1","R","Haste (This creature can attack and {T} as soon as it comes under your control.)
At the beginning of the end step, return Viashino Sandscout to its owner's hand. (Return it only if it's on the battlefield.)","1R","Creature - Viashino Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oathsworn Giant","3","4","W","Vigilance
Other creatures you control get +0/+2 and have vigilance.","4WW","Creature - Giant Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thatcher Revolt","R","Put three 1/1 red Human creature tokens with haste onto the battlefield. Sacrifice those tokens at the beginning of the next end step.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Protection: Black","W","{1}: The next time a black source of your choice would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seedling Charm","G","Choose one - Return target Aura attached to a creature to its owner's hand; or regenerate target green creature; or target creature gains trample until end of turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Shortcutter","2","1","R","When Goblin Shortcutter enters the battlefield, target creature can't block this turn.","1R","Creature - Goblin Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arbiter of Knollridge","5","5","W","Vigilance
When Arbiter of Knollridge enters the battlefield, each player's life total becomes the highest life total among all players.","6W","Creature - Giant Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Final Revels","B","Choose one - All creatures get +2/+0 until end of turn; or all creatures get -0/-2 until end of turn.","4B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Greatsword","Equipped creature gets +3/+0.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trap Digger","1","3","W","{2}{W}, {T}: Put a trap counter on target land you control.
Sacrifice a land with a trap counter on it: Trap Digger deals 3 damage to target attacking creature without flying.","3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Land Aid '04","G","Search your library for a basic land card, put that card into play tapped, then shuffle your library. If you sang a song the whole time you were searching and shuffling, you may untap that land.","GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trokin High Guard","3","3","W",NULL,"3W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scathe Zombies","2","2","B",NULL,"2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nest Invader","2","2","G","When Nest Invader enters the battlefield, put a 0/1 colorless Eldrazi Spawn creature token onto the battlefield. It has ""Sacrifice this creature: Add {1} to your mana pool.""","1G","Creature - Eldrazi Drone");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bee Sting","G","Bee Sting deals 2 damage to target creature or player.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seed Spark","W","Destroy target artifact or enchantment. If {G} was spent to cast Seed Spark, put two 1/1 green Saproling creature tokens onto the battlefield.","3W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lake of the Dead","If Lake of the Dead would enter the battlefield, sacrifice a Swamp instead. If you do, put Lake of the Dead onto the battlefield. If you don't, put it into its owner's graveyard.
{T}: Add {B} to your mana pool.
{T}, Sacrifice a Swamp: Add {B}{B}{B}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ethereal Usher","2","3","U","{U}, {T}: Target creature is unblockable this turn.
Transmute {1}{U}{U} ({1}{U}{U}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","5U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Promise of Bunrei","W","When a creature you control dies, sacrifice Promise of Bunrei. If you do, put four 1/1 colorless Spirit creature tokens onto the battlefield.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vivisection","U","As an additional cost to cast Vivisection, sacrifice a creature.
Draw three cards.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undying Beast","3","2","B","When Undying Beast dies, put it on top of its owner's library.","3B","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daughter of Autumn","2","4","G","{W}: The next 1 damage that would be dealt to target white creature this turn is dealt to Daughter of Autumn instead.","2GG","Legendary Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashen Ghoul","3","1","B","Haste
{B}: Return Ashen Ghoul from your graveyard to the battlefield. Activate this ability only during your upkeep and only if three or more creature cards are above Ashen Ghoul.","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oblation","W","The owner of target nonland permanent shuffles it into his or her library, then draws two cards.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kresh the Bloodbraided","3","3","BRG","Whenever another creature dies, you may put X +1/+1 counters on Kresh the Bloodbraided, where X is that creature's power.","2BRG","Legendary Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("An-Zerrin Ruins","R","As An-Zerrin Ruins enters the battlefield, choose a creature type.
Creatures of the chosen type don't untap during their controllers' untap steps.","2RR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trailblazer's Boots","Equipped creature has nonbasic landwalk. (It's unblockable as long as defending player controls a nonbasic land.)
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crown of Ascension","U","Enchant creature
Enchanted creature has flying.
Sacrifice Crown of Ascension: Enchanted creature and other creatures that share a creature type with it gain flying until end of turn.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning Sands","R","Whenever a creature dies, that creature's controller sacrifices a land.","3RR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Manriki-Gusari","Equipped creature gets +1/+2 and has ""{T}: Destroy target Equipment.""
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Warrior","1","1","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","W","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Ingester","3","3","U","Imprint - When Phyrexian Ingester enters the battlefield, you may exile target nontoken creature.
Phyrexian Ingester gets +X/+Y, where X is the exiled creature card's power and Y is its toughness.","6U","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakka Mar","2","2","R","Haste
{R}, {T}: Put a 3/1 red Elemental creature token with haste onto the battlefield.","2RR","Legendary Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stolen Goods","U","Target opponent exiles cards from the top of his or her library until he or she exiles a nonland card. Until end of turn, you may cast that card without paying its mana cost.","3U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Celestial Sword","{3}, {T}: Target creature you control gets +3/+3 until end of turn. Its controller sacrifices it at the beginning of the next end step.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Repentant Vampire","3","3","B","Flying
Whenever a creature dealt damage by Repentant Vampire this turn dies, put a +1/+1 counter on Repentant Vampire.
Threshold - As long as seven or more cards are in your graveyard, Repentant Vampire is white and has ""{T}: Destroy target black creature.""","3BB","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Extinction","B","Destroy all creatures of the creature type of your choice.","4B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Artifact Blast","R","Counter target artifact spell.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gemstone Array","{2}: Put a charge counter on Gemstone Array.
Remove a charge counter from Gemstone Array: Add one mana of any color to your mana pool.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cast Through Time","U","Instant and sorcery spells you control have rebound. (Exile the spell as it resolves if you cast it from your hand. At the beginning of your next upkeep, you may cast that card from exile without paying its mana cost.)","4UUU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time Spiral","U","Exile Time Spiral. Each player shuffles his or her graveyard and hand into his or her library, then draws seven cards. You untap up to six lands.","4UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mounted Archers","2","3","W","Reach (This creature can block creatures with flying.)
{W}: Mounted Archers can block an additional creature this turn.","3W","Creature - Human Soldier Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abyssal Horror","2","2","B","Flying
When Abyssal Horror enters the battlefield, target player discards two cards.","4BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bosk Banneret","1","3","G","Treefolk spells and Shaman spells you cast cost {1} less to cast.","1G","Creature - Treefolk Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barrin's Spite","UB","Choose two target creatures controlled by the same player. Their controller chooses and sacrifices one of them. Return the other to its owner's hand.","2UB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Null Profusion","B","Skip your draw step.
Whenever you play a card, draw a card.
Your maximum hand size is two.","4BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disease Carriers","2","2","B","When Disease Carriers dies, target creature gets -2/-2 until end of turn.","2BB","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alpha Kavu","2","2","G","{1}{G}: Target Kavu creature gets -1/+1 until end of turn.","2G","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Murasa Pyromancer","3","2","R","Whenever Murasa Pyromancer or another Ally enters the battlefield under your control, you may have Murasa Pyromancer deal damage to target creature equal to the number of Allies you control.","4RR","Creature - Human Shaman Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lost Leonin","2","1","W","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","1W","Creature - Cat Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quagmire","B","Creatures with swampwalk can be blocked as though they didn't have swampwalk.","2B","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lotus Bloom","Suspend 3-{0} (Rather than cast this card from your hand, pay {0} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)
{T}, Sacrifice Lotus Bloom: Add three mana of any one color to your mana pool.",NULL,"Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Freyalise's Radiance","G","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Snow permanents don't untap during their controllers' untap steps.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindlash Sliver","1","1","B","All Slivers have ""{1}, Sacrifice this permanent: Each player discards a card.""","B","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Natural Order","G","As an additional cost to cast Natural Order, sacrifice a green creature.
Search your library for a green creature card and put it onto the battlefield. Then shuffle your library.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Defender of the Order","2","4","W","Morph {W}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Defender of the Order is turned face up, creatures you control get +0/+2 until end of turn.","3W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wildsize","G","Target creature gets +2/+2 and gains trample until end of turn.
Draw a card.","2G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Evolving Wilds","{T}, Sacrifice Evolving Wilds: Search your library for a basic land card and put it onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thicket Elemental","4","4","G","Kicker {1}{G} (You may pay an additional {1}{G} as you cast this spell.)
When Thicket Elemental enters the battlefield, if it was kicked, you may reveal cards from the top of your library until you reveal a creature card. If you do, put that card onto the battlefield and shuffle all other cards revealed this way into your library.","3GG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blistering Barrier","5","2","R","Defender (This creature can't attack.)","2R","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Essence Sliver","3","3","W","Whenever a Sliver deals damage, its controller gains that much life.","3W","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Talisman of Unity","{T}: Add {1} to your mana pool.
{T}: Add {G} or {W} to your mana pool. Talisman of Unity deals 1 damage to you.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Opportunist","2","2","R","{T}: Opportunist deals 1 damage to target creature that was dealt damage this turn.","2R","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flickerform","W","Enchant creature
{2}{W}{W}: Exile enchanted creature and all Auras attached to it. At the beginning of the next end step, return that card to the battlefield under its owner's control. If you do, return those Auras to the battlefield under their owners' control attached to that creature.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Painful Memories","B","Look at target opponent's hand and choose a card from it. Put that card on top of that player's library.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Knights","2","2","U","First strike
Vodalian Knights can't attack unless defending player controls an Island.
When you control no Islands, sacrifice Vodalian Knights.
{U}: Vodalian Knights gains flying until end of turn.","1UU","Creature - Merfolk Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vigorous Charge","G","Kicker {W} (You may pay an additional {W} as you cast this spell.)
Target creature gains trample until end of turn. Whenever that creature deals combat damage this turn, if Vigorous Charge was kicked, you gain life equal to that damage.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magefire Wings","UR","Enchant creature
Enchanted creature gets +2/+0 and has flying.","UR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tresserhorn Skyknight","5","3","B","Flying
Prevent all damage that would be dealt to Tresserhorn Skyknight by creatures with first strike.","5BB","Creature - Zombie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squirming Mass","1","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)","1B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Galepowder Mage","3","3","W","Flying
Whenever Galepowder Mage attacks, exile another target creature. Return that card to the battlefield under its owner's control at the beginning of the next end step.","3W","Creature - Kithkin Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tectonic Instability","R","Whenever a land enters the battlefield, tap all lands its controller controls.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lingering Souls","W","Put two 1/1 white Spirit creature tokens with flying onto the battlefield.
Flashback {1}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Midnight Haunting","W","Put two 1/1 white Spirit creature tokens with flying onto the battlefield.","2W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thumbscrews","At the beginning of your upkeep, if you have five or more cards in hand, Thumbscrews deals 1 damage to target opponent.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inside Out","UR","Switch target creature's power and toughness until end of turn.
Draw a card.","1(U/R)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bayou Dragonfly","1","1","G","Flying, swampwalk","1G","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hymn of Rebirth","WG","Put target creature card from a graveyard onto the battlefield under your control.","3GW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Puca's Mischief","U","At the beginning of your upkeep, you may exchange control of target nonland permanent you control and target nonland permanent an opponent controls with an equal or lesser converted mana cost.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vizzerdrix","6","6","U",NULL,"6U","Creature - Rabbit Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alexi, Zephyr Mage","3","3","U","{X}{U}, {T}, Discard two cards: Return X target creatures to their owners' hands.","3UU","Legendary Creature - Human Spellshaper");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crystal Vein","{T}: Add {1} to your mana pool.
{T}, Sacrifice Crystal Vein: Add {2} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armored Griffin","2","3","W","Flying, vigilance","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Cure","B","Until end of turn, whenever a player gains life, that player loses 2 life for each 1 life he or she gained.","BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grave Peril","B","When a nonblack creature enters the battlefield, sacrifice Grave Peril. If you do, destroy that creature.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightning Reaver","3","3","BR","Haste; fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Whenever Lightning Reaver deals combat damage to a player, put a charge counter on it.
At the beginning of your end step, Lightning Reaver deals damage equal to the number of charge counters on it to each opponent.","3BR","Creature - Zombie Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Apothecary Initiate","1","1","W","Whenever a player casts a white spell, you may pay {1}. If you do, you gain 1 life.","W","Creature - Kithkin Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Insidious Dreams","B","As an additional cost to cast Insidious Dreams, discard X cards.
Search your library for X cards. Then shuffle your library and put those cards on top of it in any order.","3B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frontline Strategist","1","1","W","Morph {W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Frontline Strategist is turned face up, prevent all combat damage non-Soldier creatures would deal this turn.","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fauna Shaman","2","2","G","{G}, {T}, Discard a creature card: Search your library for a creature card, reveal it, and put it into your hand. Then shuffle your library.","1G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rush of Blood","R","Target creature gets +X/+0 until end of turn, where X is its power.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prismatic Lace","U","Target permanent becomes the color or colors of your choice. (This effect lasts indefinitely.)","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Choice of Damnations","B","Target opponent chooses a number. You may have that player lose that much life. If you don't, that player sacrifices all but that many permanents.","5B","Sorcery - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("My First Tome","{1}, {T}: Say the flavor text on a card in your hand. Target opponent guesses that card's name. You may reveal that card. If you do and your opponent guessed wrong, draw a card.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Embargo","U","Nonland permanents don't untap during their controllers' untap steps.
At the beginning of your upkeep, you lose 2 life.","3U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rancor","G","Enchant creature
Enchanted creature gets +2/+0 and has trample.
When Rancor is put into a graveyard from the battlefield, return Rancor to its owner's hand.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rag Dealer","1","1","B","{2}{B}, {T}: Exile up to three target cards from a single graveyard.","B","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Manipulation","U","Take an extra turn after this one.","3UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Turtle","2","4","G","Giant Turtle can't attack if it attacked during your last turn.","1GG","Creature - Turtle");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enfeeblement","B","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature gets -2/-2.","BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Healing Salve","W","Choose one - Target player gains 3 life; or prevent the next 3 damage that would be dealt to target creature or player this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightstalker Engine","*","3","B","Nightstalker Engine's power is equal to the number of creature cards in your graveyard.","4B","Creature - Nightstalker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Omen of Fire","R","Return all Islands to their owners' hands.
Each player sacrifices a Plains or a white permanent for each white permanent he or she controls.","3RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crazed Armodon","3","3","G","{G}: Crazed Armodon gets +3/+0 and gains trample until end of turn. Destroy Crazed Armodon at the beginning of the next end step. Activate this ability only once each turn.","2GG","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Brigadier","3","5","W","Flying
Other Bird creatures get +1/+1.
Other Soldier creatures get +1/+1.","3WWW","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shadow Guildmage","1","1","B","{U}, {T}: Put target creature you control on top of its owner's library.
{R}, {T}: Shadow Guildmage deals 1 damage to target creature or player and 1 damage to you.","B","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whiptongue Frog","1","3","U","{U}: Whiptongue Frog gains flying until end of turn.","2U","Creature - Frog");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Solace","W","As Circle of Solace enters the battlefield, choose a creature type.
{1}{W}: The next time a creature of the chosen type would deal damage to you this turn, prevent that damage.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snake Cult Initiation","B","Enchant creature
Enchanted creature has poisonous 3. (Whenever it deals combat damage to a player, that player gets three poison counters. A player with ten or more poison counters loses the game.)","3B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Skirmishers","1","1","W","First strike; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
Whenever Icatian Skirmishers attacks, all creatures banded with it gain first strike until end of turn.","3W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Genesis Chamber","Whenever a nontoken creature enters the battlefield, if Genesis Chamber is untapped, that creature's controller puts a 1/1 colorless Myr artifact creature token onto the battlefield.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shriveling Rot","B","Choose one - Until end of turn, whenever a creature is dealt damage, destroy it; or until end of turn, whenever a creature dies, that creature's controller loses life equal to its toughness.
Entwine {2}{B} (Choose both if you pay the entwine cost.)","2BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Organ Grinder","3","1","B","{T}, Exile three cards from your graveyard: Target player loses 3 life.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Briarpack Alpha","3","3","G","Flash (You may cast this spell any time you could cast an instant.)
When Briarpack Alpha enters the battlefield, target creature gets +2/+2 until end of turn.","3G","Creature - Wolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devour in Shadow","B","Destroy target creature. It can't be regenerated. You lose life equal to that creature's toughness.","BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin S.W.A.T. Team","2","2","R","Say ""Goblin S.W.A.T. Team"": Put a +1/+1 counter on Goblin S.W.A.T. Team unless an opponent swats the table within five seconds. Play this ability only once each turn.","3R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Iron Maiden","At the beginning of each opponent's upkeep, Iron Maiden deals X damage to that player, where X is the number of cards in his or her hand minus 4.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krovikan Elementalist","1","1","B","{2}{R}: Target creature gets +1/+0 until end of turn.
{U}{U}: Target creature you control gains flying until end of turn. Sacrifice it at the beginning of the next end step.","BB","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Water Wurm","1","1","U","Water Wurm gets +0/+1 as long as an opponent controls an Island.","U","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inspired Sprite","2","2","U","Flash
Flying
Whenever you cast a Wizard spell, you may untap Inspired Sprite.
{T}: Draw a card, then discard a card.","3U","Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mindstab","B","Target player discards three cards.
Suspend 4-{B} (Rather than cast this card from your hand, you may pay {B} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","5B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skinthinner","2","1","B","Morph {3}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Skinthinner is turned face up, destroy target nonblack creature. It can't be regenerated.","1B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blessed Wine","W","You gain 1 life.
Draw a card at the beginning of the next turn's upkeep.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nevermaker","2","3","U","Flying
When Nevermaker leaves the battlefield, put target nonland permanent on top of its owner's library.
Evoke {3}{U} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","3U","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grim Backwoods","{T}: Add {1} to your mana pool.
{2}{B}{G}, {T}, Sacrifice a creature: Draw a card.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Orders","R","Cast False Orders only during the declare blockers step.
Remove target creature defending player controls from combat. Creatures it was blocking that had become blocked by only that creature this combat become unblocked. You may have it block an attacking creature of your choice.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marsh Lurker","3","2","B","Sacrifice a Swamp: Marsh Lurker gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","3B","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Increasing Savagery","G","Put five +1/+1 counters on target creature. If Increasing Savagery was cast from a graveyard, put ten +1/+1 counters on that creature instead.
Flashback {5}{G}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Camel","0","1","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
As long as Camel is attacking, prevent all damage Deserts would deal to Camel and to creatures banded with Camel.","W","Creature - Camel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heal","W","Prevent the next 1 damage that would be dealt to target creature or player this turn.
Draw a card at the beginning of the next turn's upkeep.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tempest Owl","1","2","U","Kicker {4}{U} (You may pay an additional {4}{U} as you cast this spell.)
Flying
When Tempest Owl enters the battlefield, if it was kicked, tap up to three target permanents.","1U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cantivore","*","*","W","Vigilance
Cantivore's power and toughness are each equal to the number of enchantment cards in all graveyards.","1WW","Creature - Lhurgoyf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grizzled Leotau","1","5","WG",NULL,"GW","Creature - Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Brushland","{T}: Add {1} to your mana pool.
{T}: Add {G} or {W} to your mana pool. Brushland deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lead Golem","3","5","Whenever Lead Golem attacks, it doesn't untap during its controller's next untap step.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragon's Herald","1","1","R","{2}{R}, {T}, Sacrifice a black creature, a red creature, and a green creature: Search your library for a card named Hellkite Overlord and put it onto the battlefield. Then shuffle your library.","R","Creature - Goblin Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kher Keep","{T}: Add {1} to your mana pool.
{1}{R}, {T}: Put a 0/1 red Kobold creature token named Kobolds of Kher Keep onto the battlefield.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diminishing Returns","U","Each player shuffles his or her hand and graveyard into his or her library. You exile the top ten cards of your library. Then each player draws up to seven cards.","2UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beastmaster's Magemark","G","Enchant creature
Creatures you control that are enchanted get +1/+1.
Whenever a creature you control that's enchanted becomes blocked, it gets +1/+1 until end of turn for each creature blocking it.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Symbiotic Elf","2","2","G","When Symbiotic Elf dies, put two 1/1 green Insect creature tokens onto the battlefield.","3G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Piker","2","1","R",NULL,"1R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Spirit","1","1","Flying",NULL,"Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timberwatch Elf","1","2","G","{T}: Target creature gets +X/+X until end of turn, where X is the number of Elves on the battlefield.","2G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Laccolith Grunt","2","2","R","Whenever Laccolith Grunt becomes blocked, you may have it deal damage equal to its power to target creature. If you do, Laccolith Grunt assigns no combat damage this turn.","2R","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sunhome, Fortress of the Legion","{T}: Add {1} to your mana pool.
{2}{R}{W}, {T}: Target creature gains double strike until end of turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Refreshing Rain","G","If an opponent controls a Swamp and you control a Forest, you may cast Refreshing Rain without paying its mana cost.
Target player gains 6 life.","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prince of Thralls","7","7","UBR","Whenever a permanent an opponent controls is put into a graveyard, put that card onto the battlefield under your control unless that opponent pays 3 life.","4UBBR","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Geralf's Messenger","3","2","B","Geralf's Messenger enters the battlefield tapped.
When Geralf's Messenger enters the battlefield, target opponent loses 2 life.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","BBB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiketail Drakeling","2","2","U","Flying
Sacrifice Spiketail Drakeling: Counter target spell unless its controller pays {2}.","1UU","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cardpecker","1{1/2}","1","W","Flying
Gotcha Whenever an opponent touches the table with his or her hand, you may say ""Gotcha"" If you do, return Cardpecker from your graveyard to your hand.","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Weaver of Lies","4","4","U","Morph {4}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Weaver of Lies is turned face up, turn any number of target creatures with morph other than Weaver of Lies face down.","5UU","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shifting Borders","U","Exchange control of two target lands.
Splice onto Arcane {3}{U} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","3U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glimmering Angel","2","2","W","Flying
{U}: Glimmering Angel gains shroud until end of turn. (It can't be the target of spells or abilities.)","3W","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scrapdiver Serpent","5","5","U","Scrapdiver Serpent is unblockable as long as defending player controls an artifact.","5UU","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cao Cao, Lord of Wei","3","3","B","{T}: Target opponent discards two cards. Activate this ability only during your turn, before attackers are declared.","3BB","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Airborne Aid","U","Draw a card for each Bird on the battlefield.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lone Wolf","2","2","G","You may have Lone Wolf assign its combat damage as though it weren't blocked.","2G","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vesper Ghoul","1","1","B","{T}, Pay 1 life: Add one mana of any color to your mana pool.","2B","Creature - Zombie Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Orb of Dreams","Permanents enter the battlefield tapped.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ice Floe","You may choose not to untap Ice Floe during your untap step.
{T}: Tap target creature without flying that's attacking you. It doesn't untap during its controller's untap step for as long as Ice Floe remains tapped.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for the Gemblades","G","Whenever a creature you control deals combat damage to a creature, you may put a quest counter on Quest for the Gemblades.
Remove a quest counter from Quest for the Gemblades and sacrifice it: Put four +1/+1 counters on target creature.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rib Cage Spider","1","4","G","Reach (This creature can block creatures with flying.)","2G","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Extra Arms","R","Enchant creature
Whenever enchanted creature attacks, it deals 2 damage to target creature or player.","4R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Realms Uncharted","G","Search your library for four land cards with different names and reveal them. An opponent chooses two of those cards. Put the chosen cards into your graveyard and the rest into your hand. Then shuffle your library.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Mutant","5","3","R","Trample
Goblin Mutant can't attack if defending player controls an untapped creature with power 3 or greater.
Goblin Mutant can't block creatures with power 3 or greater.","2RR","Creature - Goblin Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightscape Battlemage","2","2","B","Kicker {2}{U} and/or {2}{R} (You may pay an additional {2}{U} and/or {2}{R} as you cast this spell.)
When Nightscape Battlemage enters the battlefield, if it was kicked with its {2}{U} kicker, return up to two target nonblack creatures to their owners' hands.
When Nightscape Battlemage enters the battlefield, if it was kicked with its {2}{R} kicker, destroy target land.","2B","Creature - Zombie Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Throne of Geth","{T}, Sacrifice an artifact: Proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lashknife Barrier","W","When Lashknife Barrier enters the battlefield, draw a card.
If a source would deal damage to a creature you control, it deals that much damage minus 1 to that creature instead.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desperate Ritual","R","Add {R}{R}{R} to your mana pool.
Splice onto Arcane {1}{R} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1R","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Elder","1","2","G","{T}: Add {1}{G} to your mana pool.","2G","Creature - Insect Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Roughrider","3","2","R",NULL,"2R","Creature - Goblin Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Restless Dreams","B","As an additional cost to cast Restless Dreams, discard X cards.
Return X target creature cards from your graveyard to your hand.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warren-Scourge Elf","1","1","G","Protection from Goblins","1G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Swans of Bryn Argoll","4","3","WU","Flying
If a source would deal damage to Swans of Bryn Argoll, prevent that damage. The source's controller draws cards equal to the damage prevented this way.","2(W/U)(W/U)","Creature - Bird Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trap Runner","2","3","W","{T}: Target unblocked attacking creature becomes blocked. Activate this ability only during combat after blockers are declared. (This ability works on unblockable creatures.)","2WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Ward","W","Regenerate target creature.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("River Merfolk","2","1","U","{U}: River Merfolk gains mountainwalk until end of turn.","UU","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nim Lasher","1","1","B","Nim Lasher gets +1/+0 for each artifact you control.","2B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sudden Shock","R","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Sudden Shock deals 2 damage to target creature or player.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reborn Hope","WG","Return target multicolored card from your graveyard to your hand.","GW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circular Logic","U","Counter target spell unless its controller pays {1} for each card in your graveyard.
Madness {U} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Utvara Scalper","1","2","R","Flying
Utvara Scalper attacks each turn if able.","1R","Creature - Goblin Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Charger","2","5","WR","Whenever Flowstone Charger attacks, it gets +3/-3 until end of turn.","2RW","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alchemist's Apprentice","1","1","U","Sacrifice Alchemist's Apprentice: Draw a card.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wirewood Elf","1","2","G","{T}: Add {G} to your mana pool.","1G","Creature - Elf Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Horn of Deafening","{2}, {T}: Prevent all combat damage that would be dealt by target creature this turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Acolyte","1","1","G","{1}, {T}: Add one mana of any color to your mana pool.","G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reach of Branches","G","Put a 2/5 green Treefolk Shaman creature token onto the battlefield.
Whenever a Forest enters the battlefield under your control, you may return Reach of Branches from your graveyard to your hand.","4G","Tribal Instant - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Condor","2","2","U","Flying
Cast Skyshroud Condor only if you've cast another spell this turn.","1U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sandbar Merfolk","1","1","U","Cycling {2} ({2}, Discard this card: Draw a card.)","U","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dracoplasm","0","0","UR","Flying
As Dracoplasm enters the battlefield, sacrifice any number of creatures. Dracoplasm's power becomes the total power of those creatures and its toughness becomes their total toughness.
{R}: Dracoplasm gets +1/+0 until end of turn.","UR","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moor Fiend","3","3","B","Swampwalk","3B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dreamborn Muse","2","2","U","At the beginning of each player's upkeep, that player puts the top X cards of his or her library into his or her graveyard, where X is the number of cards in his or her hand.","2UU","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time Stop","U","End the turn. (Exile all spells and abilities on the stack, including this card. The player whose turn it is discards down to his or her maximum hand size. Damage wears off, and ""this turn"" and ""until end of turn"" effects end.)","4UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Revered Elder","1","2","W","{1}: Prevent the next 1 damage that would be dealt to Revered Elder this turn.","2W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Double Cross","B","Choose another player. Look at that player's hand and choose one of those cards other than a basic land. He or she discards that card. At the beginning of the next game with the player, look at the player's hand and choose one of those cards other than a basic land. He or she discards that card.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ageless Sentinels","4","4","W","Defender (This creature can't attack.)
Flying
When Ageless Sentinels blocks, it becomes a Bird Giant, and it loses defender. (It's no longer a Wall. This effect lasts indefinitely.)","3W","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spoils of the Vault","B","Name a card. Reveal cards from the top of your library until you reveal the named card, then put that card into your hand. Exile all other cards revealed this way, and you lose 1 life for each of the exiled cards.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Retribution","W","Divine Retribution deals damage to target attacking creature equal to the number of attacking creatures.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Puppetry","U","You may tap or untap target permanent.
Splice onto Arcane {U} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voidmage Prodigy","2","1","U","{U}{U}, Sacrifice a Wizard: Counter target spell.
Morph {U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","UU","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Prophetic Prism","When Prophetic Prism enters the battlefield, draw a card.
{1}, {T}: Add one mana of any color to your mana pool.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("First Volley","R","First Volley deals 1 damage to target creature and 1 damage to that creature's controller.","1R","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shinen of Fury's Fire","2","1","R","Haste
Channel - {R}, Discard Shinen of Fury's Fire: Target creature gains haste until end of turn.","2R","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heavy Ballista","2","3","W","{T}: Heavy Ballista deals 2 damage to target attacking or blocking creature.","3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nix","U","Counter target spell if no mana was spent to cast it.","U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Zelyon Sword","You may choose not to untap Zelyon Sword during your untap step.
{3}, {T}: Target creature gets +2/+0 for as long as Zelyon Sword remains tapped.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warrior Angel","3","4","W","Flying
Whenever Warrior Angel deals damage, you gain that much life.","4WW","Creature - Angel Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ivory Tower","At the beginning of your upkeep, you gain X life, where X is the number of cards in your hand minus 4.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eye of the Storm","U","Whenever a player casts an instant or sorcery card, exile it. Then that player copies each instant or sorcery card exiled with Eye of the Storm. For each copy, the player may cast the copy without paying its mana cost.","5UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Illusionary Servant","3","4","U","Flying
When Illusionary Servant becomes the target of a spell or ability, sacrifice it.","1UU","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soulscour","W","Destroy all nonartifact permanents.","7WWW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Killer Bees","0","1","G","Flying
{G}: Killer Bees gets +1/+1 until end of turn.","1GG","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coercion","B","Target opponent reveals his or her hand. You choose a card from it. That player discards that card.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flame Elemental","3","2","R","{R}, {T}, Sacrifice Flame Elemental: Flame Elemental deals damage equal to its power to target creature.","2RR","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Battlefield Forge","{T}: Add {1} to your mana pool.
{T}: Add {R} or {W} to your mana pool. Battlefield Forge deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of the Fire Penguin","R","Curse of the Fire Penquin consumes and confuses enchanted creature.
-----
Creature Penguin
Trample
When this creature is put into a graveyard from play, return Curse of the Fire Penguin from your graveyard to play.
6/5","4RR","Enchant Creature");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stand // Deliver","U","Return target permanent to its owner's hand.
---
Prevent the next 2 damage that would be dealt to target creature this turn.","2U","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Stone Golem","4","4",NULL,"5","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whitesun's Passage","W","You gain 5 life.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashling the Pilgrim","1","1","R","{1}{R}: Put a +1/+1 counter on Ashling the Pilgrim. If this is the third time this ability has resolved this turn, remove all +1/+1 counters from Ashling the Pilgrim, and it deals that much damage to each creature and each player.","1R","Legendary Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surge of Zeal","R","Radiance - Target creature and each other creature that shares a color with it gain haste until end of turn.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serra Aviary","W","Creatures with flying get +1/+1.","3W","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Winged Coatl","1","1","UG","Flash
Flying
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","1GU","Creature - Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Storm Herd","W","Put X 1/1 white Pegasus creature tokens with flying onto the battlefield, where X is your life total.","8WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Living Plane","G","All lands are 1/1 creatures that are still lands.","2GG","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Writ of Passage","U","Enchant creature
Whenever enchanted creature attacks, if its power is 2 or less, it's unblockable this turn.
Forecast - {1}{U}, Reveal Writ of Passage from your hand: Target creature with power 2 or less is unblockable this turn. (Activate this ability only during your upkeep and only once each turn.)","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Sage","2","3","U","Threshold - As long as seven or more cards are in your graveyard, Cephalid Sage has ""When Cephalid Sage enters the battlefield, draw three cards, then discard two cards.""","3U","Creature - Cephalid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Daggers","B","Destroy all creatures target opponent controls. You lose 2 life for each creature destroyed this way.","4BB","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lockjaw Snapper","2","2","Wither (This deals damage to creatures in the form of -1/-1 counters.)
When Lockjaw Snapper dies, put a -1/-1 counter on each creature with a -1/-1 counter on it.","4","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Arsonist","1","1","R","When Goblin Arsonist dies, you may have it deal 1 damage to target creature or player.","R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Nishoba","0","0","WG","Trample
Phantom Nishoba enters the battlefield with seven +1/+1 counters on it.
Whenever Phantom Nishoba deals damage, you gain that much life.
If damage would be dealt to Phantom Nishoba, prevent that damage. Remove a +1/+1 counter from Phantom Nishoba.","5GW","Creature - Cat Beast Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whip-Spine Drake","3","3","U","Flying
Morph {2}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3UU","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rapacious One","5","4","R","Trample
Whenever Rapacious One deals combat damage to a player, put that many 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool.""","5R","Creature - Eldrazi Drone");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unspeakable Symbol","B","Pay 3 life: Put a +1/+1 counter on target creature.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Revoker","2","1","As Phyrexian Revoker enters the battlefield, name a nonland card.
Activated abilities of sources with the chosen name can't be activated.","2","Artifact Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rotted Ones, Lay Siege","When you set this scheme in motion, for each opponent, put a 2/2 black Zombie creature token onto the battlefield that attacks that player each combat if able.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for the Holy Relic","W","Whenever you cast a creature spell, you may put a quest counter on Quest for the Holy Relic.
Remove five quest counters from Quest for the Holy Relic and sacrifice it: Search your library for an Equipment card, put it onto the battlefield, and attach it to a creature you control. Then shuffle your library.","W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Escaped Null","1","2","B","Lifelink
Whenever Escaped Null blocks or becomes blocked, it gets +5/+0 until end of turn.","3B","Creature - Zombie Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bonded Fetch","0","2","U","Defender, haste
{T}: Draw a card, then discard a card.","2U","Creature - Homunculus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greater Mossdog","3","3","G","Dredge 3 (If you would draw a card, instead you may put exactly three cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","3G","Creature - Plant Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Aberration","4","5","G","{T}: Add {G}{G}{G} to your mana pool.
Forestcycling {2} ({2}, Discard this card: Search your library for a Forest card, reveal it, and put it into your hand. Then shuffle your library.)","5G","Creature - Elf Mutant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Riptide Replicator","As Riptide Replicator enters the battlefield, choose a color and a creature type.
Riptide Replicator enters the battlefield with X charge counters on it.
{4}, {T}: Put an X/X creature token of the chosen color and type onto the battlefield, where X is the number of charge counters on Riptide Replicator.","X4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Konda's Hatamoto","1","2","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
As long as you control a legendary Samurai, Konda's Hatamoto gets +1/+2 and has vigilance. (Attacking doesn't cause this creature to tap.)","1W","Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twist Allegiance","R","You and target opponent each gain control of all creatures the other controls until end of turn. Untap those creatures. Those creatures gain haste until end of turn.","6R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rhystic Shield","W","Creatures you control get +0/+1 until end of turn. They get an additional +0/+2 until end of turn unless any player pays {2}.","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Weatherseed Totem","{T}: Add {G} to your mana pool.
{2}{G}{G}{G}: Weatherseed Totem becomes a 5/3 green Treefolk artifact creature with trample until end of turn.
When Weatherseed Totem is put into a graveyard from the battlefield, if it was a creature, return this card to its owner's hand.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Colfenor's Urn","Whenever a creature with toughness 4 or greater is put into your graveyard from the battlefield, you may exile it.
At the beginning of the end step, if three or more cards have been exiled with Colfenor's Urn, sacrifice it. If you do, return those cards to the battlefield under their owner's control.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ricochet Trap","R","If an opponent cast a blue spell this turn, you may pay {R} rather than pay Ricochet Trap's mana cost.
Change the target of target spell with a single target.","3R","Instant - Trap");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Leashling","3","3","Put a card from your hand on top of your library: Return Leashling to its owner's hand.","6","Artifact Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unmake","WB","Exile target creature.","(W/B)(W/B)(W/B)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soulsworn Jury","1","4","W","Defender (This creature can't attack.)
{1}{U}, Sacrifice Soulsworn Jury: Counter target creature spell.","2W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ceta Disciple","1","1","U","{R}, {T}: Target creature gets +2/+0 until end of turn.
{G}, {T}: Add one mana of any color to your mana pool.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ironclaw Buzzardiers","2","2","R","Ironclaw Buzzardiers can't block creatures with power 2 or greater.
{R}: Ironclaw Buzzardiers gains flying until end of turn.","2R","Creature - Orc Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Edgewright","2","2","W","Metalcraft - Auriok Edgewright has double strike as long as you control three or more artifacts.","WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Pierce Strider","3","3","When Pierce Strider enters the battlefield, target opponent loses 3 life.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Leaden Myr","1","1","{T}: Add {B} to your mana pool.","2","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Confound","U","Counter target spell that targets one or more creatures.
Draw a card.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caldera Kavu","2","2","R","{1}{B}: Caldera Kavu gets +1/+1 until end of turn.
{G}: Caldera Kavu becomes the color of your choice until end of turn.","2R","Creature - Kavu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Apocalypse","R","Exile all permanents. You discard your hand.","2RRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Liu Bei, Lord of Shu","2","4","W","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Liu Bei, Lord of Shu gets +2/+2 as long as you control a permanent named Guan Yu, Sainted Warrior or a permanent named Zhang Fei, Fierce Warrior.","3WW","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rainbow Efreet","3","1","U","Flying
{U}{U}: Rainbow Efreet phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before you untap during your next untap step.)","3U","Creature - Efreet");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nebuchadnezzar","3","3","UB","{X}, {T}: Name a card. Target opponent reveals X cards at random from his or her hand. Then that player discards all cards with that name revealed this way. Activate this ability only during your turn.","3UB","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Griselbrand","7","7","B","Flying, lifelink
Pay 7 life: Draw seven cards.","4BBBB","Legendary Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Look at Me, I'm the DCI","W","Ban one card, other than a basic land, for the remainder of the match. (For the remainder of the match, each player removes from the game all copies of that card in play or in any graveyard, hand, library, or sideboard.)","5WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stampeding Serow","5","4","G","Trample
At the beginning of your upkeep, return a green creature you control to its owner's hand.","2GG","Creature - Antelope Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ragnar","2","2","WUG","{G}{W}{U}, {T}: Regenerate target creature.","GWU","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spirit Weaver","2","1","W","{2}: Target green or blue creature gets +0/+1 until end of turn.","1W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tel-Jilad Outrider","3","1","G","Protection from artifacts","3G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mystic Crusader","2","1","W","Protection from black and from red
Threshold - As long as seven or more cards are in your graveyard, Mystic Crusader gets +1/+1 and has flying.","1WW","Creature - Human Nomad Mystic");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Illuminate","R","Kicker {2}{R} and/or {3}{U} (You may pay an additional {2}{R} and/or {3}{U} as you cast this spell.)
Illuminate deals X damage to target creature. If Illuminate was kicked with its {2}{R} kicker, it deals X damage to that creature's controller. If Illuminate was kicked with its {3}{U} kicker, you draw X cards.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Watchwing Scarecrow","2","4","Watchwing Scarecrow has vigilance as long as you control a white creature.
Watchwing Scarecrow has flying as long as you control a blue creature.","4","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sangrophage","3","3","B","At the beginning of your upkeep, tap Sangrophage unless you pay 2 life.","BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Endrek Sahr, Master Breeder","2","2","B","Whenever you cast a creature spell, put X 1/1 black Thrull creature tokens onto the battlefield, where X is that spell's converted mana cost.
When you control seven or more Thrulls, sacrifice Endrek Sahr, Master Breeder.","4B","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kemba's Legion","4","6","W","Vigilance
Kemba's Legion can block an additional creature for each Equipment attached to Kemba's Legion.","5WW","Creature - Cat Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Honorable Passage","W","The next time a source of your choice would deal damage to target creature or player this turn, prevent that damage. If damage from a red source is prevented this way, Honorable Passage deals that much damage to the source's controller.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Doubling Season","G","If an effect would put one or more tokens onto the battlefield under your control, it puts twice that many of those tokens onto the battlefield instead.
If an effect would place one or more counters on a permanent you control, it places twice that many of those counters on that permanent instead.","4G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sewers of Estark","B","Choose target creature. If it's attacking, it's unblockable this turn. If it's blocking, prevent all combat damage that would be dealt this turn by it and each creature it's blocking.","2BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kessig Wolf","3","1","R","{1}{R}: Kessig Wolf gains first strike until end of turn.","2R","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirsdag Cultist","2","2","R","{R}, {T}, Sacrifice a creature: Skirsdag Cultist deals 2 damage to target creature or player.","2RR","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snag","G","You may discard a Forest card rather than pay Snag's mana cost.
Prevent all combat damage that would be dealt by unblocked creatures this turn.","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silver Drake","3","3","WU","Flying
When Silver Drake enters the battlefield, return a white or blue creature you control to its owner's hand.","1WU","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Purify","W","Destroy all artifacts and enchantments.","3WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arc Trail","R","Arc Trail deals 2 damage to target creature or player and 1 damage to another target creature or player.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Acidic Soil","R","Acidic Soil deals damage to each player equal to the number of lands he or she controls.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Search for Survivors","R","Reorder your graveyard at random. An opponent chooses a card at random in your graveyard. If it's a creature card, put it onto the battlefield. Otherwise, exile it.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darkthicket Wolf","2","2","G","{2}{G}: Darkthicket Wolf gets +2/+2 until end of turn. Activate this ability only once each turn.","1G","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duergar Mine-Captain","2","1","WR","{1}{(r/w)}, {Q}: Attacking creatures get +1/+0 until end of turn. ({Q} is the untap symbol.)","2(R/W)","Creature - Dwarf Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time Sieve","UB","{T}, Sacrifice five artifacts: Take an extra turn after this one.","UB","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bola Warrior","1","1","R","{R}, {T}, Discard a card: Target creature can't block this turn.","1R","Creature - Human Spellshaper Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Firebreathing","R","Enchant creature
{R}: Enchanted creature gets +1/+0 until end of turn.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heartless Hidetsugu","4","3","R","{T}: Heartless Hidetsugu deals damage to each player equal to half that player's life total, rounded down.","3RR","Legendary Creature - Ogre Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spined Thopter","2","1","U","({(u/p)} can be paid with either {U} or 2 life.)
Flying","2(U/P)","Artifact Creature - Thopter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gwafa Hazid, Profiteer","2","2","WU","{W}{U}, {T}: Put a bribery counter on target creature you don't control. Its controller draws a card.
Creatures with bribery counters on them can't attack or block.","1WU","Legendary Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silvos, Rogue Elemental","8","5","G","Trample
{G}: Regenerate Silvos, Rogue Elemental.","3GGG","Legendary Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wicked Pact","B","Destroy two target nonblack creatures. You lose 5 life.","1BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warped Researcher","3","4","U","Whenever a player cycles a card, Warped Researcher gains flying and shroud until end of turn. (It can't be the target of spells or abilities.)","4U","Creature - Human Wizard Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pale Recluse","4","5","WG","Reach (This can block creatures with flying.)
Forestcycling {2}, plainscycling {2} ({2}, Discard this card: Search your library for a Forest or Plains card, reveal it, and put it into your hand. Then shuffle your library.)","4GW","Creature - Spider");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Obsianus Golem","4","6",NULL,"6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horned Sliver","2","2","G","All Sliver creatures have trample.","2G","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Wurm","2","0","G","Phantom Wurm enters the battlefield with four +1/+1 counters on it.
If damage would be dealt to Phantom Wurm, prevent that damage. Remove a +1/+1 counter from Phantom Wurm.","4GG","Creature - Wurm Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("AEtherflame Wall","0","4","R","Defender
Ætherflame Wall can block creatures with shadow as though they didn't have shadow.
{R}: Ætherflame Wall gets +1/+0 until end of turn.","1R","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thundercloud Elemental","3","4","U","Flying
{3}{U}: Tap all creatures with toughness 2 or less.
{3}{U}: All other creatures lose flying until end of turn.","5UU","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Uproot","G","Put target land on top of its owner's library.","3G","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desertion","U","Counter target spell. If an artifact or creature spell is countered this way, put that card onto the battlefield under your control instead of into its owner's graveyard.","3UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pearlspear Courier","2","1","W","You may choose not to untap Pearlspear Courier during your untap step.
{2}{W}, {T}: Target Soldier creature gets +2/+2 and has vigilance for as long as Pearlspear Courier remains tapped.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shattered Angel","3","3","W","Flying
Whenever a land enters the battlefield under an opponent's control, you may gain 3 life.","3WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tarfire","R","Tarfire deals 2 damage to target creature or player.","R","Tribal Instant - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Glider","1","1","R","Flying
Goblin Glider can't block.","1R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spoils of War","B","X is the number of artifact and/or creature cards in an opponent's graveyard as you cast Spoils of War.
Distribute X +1/+1 counters among any number of target creatures.","XB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Liability","B","Whenever a nontoken permanent is put into a player's graveyard from the battlefield, that player loses 1 life.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Karn, Silver Golem","4","4","Whenever Karn, Silver Golem blocks or becomes blocked, it gets -4/+4 until end of turn.
{1}: Target noncreature artifact becomes an artifact creature with power and toughness each equal to its converted mana cost until end of turn.","5","Legendary Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Witchbane Orb","When Witchbane Orb enters the battlefield, destroy all Curses attached to you.
You have hexproof. (You can't be the target of spells or abilities your opponents control, including Aura spells.)","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harsh Judgment","W","As Harsh Judgment enters the battlefield, choose a color.
If an instant or sorcery spell of the chosen color would deal damage to you, it deals that damage to its controller instead.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunder-Thrash Elder","1","1","R","Devour 3 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with three times that many +1/+1 counters on it.)","2R","Creature - Viashino Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arc Lightning","R","Arc Lightning deals 3 damage divided as you choose among one, two, or three target creatures and/or players.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitesail Apprentice","1","1","W","As long as Kitesail Apprentice is equipped, it gets +1/+1 and has flying.","W","Creature - Kor Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Roc Hatchling","0","1","R","Roc Hatchling enters the battlefield with four shell counters on it.
At the beginning of your upkeep, remove a shell counter from Roc Hatchling.
As long as Roc Hatchling has no shell counters on it, it gets +3/+2 and has flying.","R","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diamond Faerie","3","3","WUG","Flying
{1}{S}i}: Snow creatures you control get +1/+1 until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","2GWU","Snow Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudseeder","1","1","U","Flying
{U}, {T}, Discard a card: Put a 1/1 blue Faerie creature token named Cloud Sprite onto the battlefield. It has flying and ""Cloud Sprite can block only creatures with flying.""","1U","Creature - Faerie Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dowsing Shaman","3","4","G","{2}{G}, {T}: Return target enchantment card from your graveyard to your hand.","4G","Creature - Centaur Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sanguine Bond","B","Whenever you gain life, target opponent loses that much life.","3BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tel-Jilad Exile","2","3","G","{1}{G}: Regenerate Tel-Jilad Exile.","3G","Creature - Troll Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakdos Ickspitter","1","1","BR","{T}: Rakdos Ickspitter deals 1 damage to target creature and that creature's controller loses 1 life.","1BR","Creature - Thrull");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Foil","U","You may discard an Island card and another card rather than pay Foil's mana cost.
Counter target spell.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dungrove Elder","*","*","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Dungrove Elder's power and toughness are each equal to the number of Forests you control.","2G","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Coffers","4","4","B","{2}, {T}: Add {B} to your mana pool for each Swamp you control.","4B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Word","U","Last Word can't be countered by spells or abilities.
Counter target spell.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Clone","0","0","U","You may have Clone enter the battlefield as a copy of any creature on the battlefield.","3U","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloom Tender","1","1","G","{T}: For each color among permanents you control, add one mana of that color to your mana pool.","1G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disruptive Pitmage","1","1","U","{T}: Counter target spell unless its controller pays {1}.
Morph {U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindbender Spores","0","1","G","Defender (This creature can't attack.)
Flying
Whenever Mindbender Spores blocks a creature, put four fungus counters on that creature. The creature gains ""This creature doesn't untap during your untap step if it has a fungus counter on it"" and ""At the beginning of your upkeep, remove a fungus counter from this creature.""","2G","Creature - Fungus Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kinsbaile Balloonist","2","2","W","Flying
Whenever Kinsbaile Balloonist attacks, you may have target creature gain flying until end of turn.","3W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Defiler","3","3","B","{T}, Sacrifice Phyrexian Defiler: Target creature gets -3/-3 until end of turn.","2BB","Creature - Carrier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vow of Duty","W","Enchant creature
Enchanted creature gets +2/+2, has vigilance, and can't attack you or a planeswalker you control.","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Goblin War Wagon","3","3","Goblin War Wagon doesn't untap during your untap step.
At the beginning of your upkeep, you may pay {2}. If you do, untap Goblin War Wagon.","4","Artifact Creature - Juggernaut");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Laquatus's Champion","6","3","B","When Laquatus's Champion enters the battlefield, target player loses 6 life.
When Laquatus's Champion leaves the battlefield, that player gains 6 life.
{B}: Regenerate Laquatus's Champion.","4BB","Creature - Nightmare Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rishadan Airship","3","1","U","Flying
Rishadan Airship can block only creatures with flying.","2U","Creature - Human Pirate");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harbor Serpent","5","5","U","Islandwalk (This creature is unblockable as long as defending player controls an Island.)
Harbor Serpent can't attack unless there are five or more Islands on the battlefield.","4UU","Creature - Serpent");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hive Mind","U","Whenever a player casts an instant or sorcery spell, each other player copies that spell. Each of those players may choose new targets for his or her copy.","5U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reality Strobe","U","Return target permanent to its owner's hand. Exile Reality Strobe with three time counters on it.
Suspend 3-{2}{U} (Rather than cast this card from your hand, you may pay {2}{U} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","4UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fleshbag Marauder","3","1","B","When Fleshbag Marauder enters the battlefield, each player sacrifices a creature.","2B","Creature - Zombie Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mijae Djinn","6","3","R","Whenever Mijae Djinn attacks, flip a coin. If you lose the flip, remove Mijae Djinn from combat and tap it.","RRR","Creature - Djinn");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Creepy Doll","1","1","Creepy Doll is indestructible.
Whenever Creepy Doll deals combat damage to a creature, flip a coin. If you win the flip, destroy that creature.","5","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barrow Ghoul","4","4","B","At the beginning of your upkeep, sacrifice Barrow Ghoul unless you exile the top creature card of your graveyard.","1B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Statecraft","U","Prevent all combat damage that would be dealt to and dealt by creatures you control.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Odylic Wraith","2","2","B","Swampwalk
Whenever Odylic Wraith deals damage to a player, that player discards a card.","3B","Creature - Wraith");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serra's Liturgy","W","At the beginning of your upkeep, you may put a verse counter on Serra's Liturgy.
{W}, Sacrifice Serra's Liturgy: Destroy up to X target artifacts and/or enchantments, where X is the number of verse counters on Serra's Liturgy.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Path of Anger's Flame","R","Creatures you control get +2/+0 until end of turn.","2R","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mad Dog","2","2","R","At the beginning of your end step, if Mad Dog didn't attack or come under your control this turn, sacrifice it.","1R","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wandering Wolf","2","1","G","Creatures with power less than Wandering Wolf's power can't block it.","1G","Creature - Wolf");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Nullstone Gargoyle","4","5","Flying
Whenever the first noncreature spell of a turn is cast, counter that spell.","9","Artifact Creature - Gargoyle");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dwarven Catapult","R","Dwarven Catapult deals X damage divided evenly, rounded down, among all creatures target opponent controls.","XR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dune-Brood Nephilim","3","3","WBRG","Whenever Dune-Brood Nephilim deals combat damage to a player, put a 1/1 colorless Sand creature token onto the battlefield for each land you control.","BRGW","Creature - Nephilim");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Labyrinth Minotaur","1","4","U","Whenever Labyrinth Minotaur blocks a creature, that creature doesn't untap during its controller's next untap step.","3U","Creature - Minotaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Farbog Explorer","2","3","W","Swampwalk (This creature is unblockable as long as defending player controls a Swamp.)","2W","Creature - Human Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Armistice","W","{3}{W}{W}: You draw a card and target opponent gains 3 life.","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cyclopean Tomb","{2}, {T}: Put a mire counter on target non-Swamp land. That land is a Swamp for as long as it has a mire counter on it. Activate this ability only during your upkeep.
When Cyclopean Tomb is put into a graveyard from the battlefield, at the beginning of each of your upkeeps for the rest of the game, remove all mire counters from a land that a mire counter was put onto with Cyclopean Tomb but that a mire counter has not been removed from with Cyclopean Tomb.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirri, Cat Warrior","2","3","G","First strike, forestwalk, vigilance (This creature deals combat damage before creatures without first strike, it's unblockable as long as defending player controls a Forest, and attacking doesn't cause this creature to tap.)","1GG","Legendary Creature - Cat Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Fourth Sphere","At the beginning of your upkeep, sacrifice a nonblack creature.
Whenever you roll {C}, put a 2/2 black Zombie creature token onto the battlefield.",NULL,"Plane - Phyrexia");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of Fire and Ice","Equipped creature gets +2/+2 and has protection from red and from blue.
Whenever equipped creature deals combat damage to a player, Sword of Fire and Ice deals 2 damage to target creature or player and you draw a card.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fatespinner","1","2","U","At the beginning of each opponent's upkeep, that player chooses draw step, main phase, or combat phase. The player skips each instance of the chosen step or phase this turn.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Warchief","2","2","G","Beast spells you cast cost {1} less to cast.
{1}{G}: Regenerate target Beast.","2G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lava Blister","R","Destroy target nonbasic land unless its controller has Lava Blister deal 6 damage to him or her.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tidal Bore","U","You may return an Island you control to its owner's hand rather than pay Tidal Bore's mana cost.
You may tap or untap target creature.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rolling Earthquake","R","Rolling Earthquake deals X damage to each creature without horsemanship and each player.","XR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jund Charm","BRG","Choose one - Exile all cards from target player's graveyard; or Jund Charm deals 2 damage to each creature; or put two +1/+1 counters on target creature.","BRG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barkhide Mauler","4","4","G","Cycling {2} ({2}, Discard this card: Draw a card.)","4G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Doomgape","10","10","BG","Trample
At the beginning of your upkeep, sacrifice a creature. You gain life equal to that creature's toughness.","4(B/G)(B/G)(B/G)","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Snapping Drake","3","2","U","Flying","3U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Somnophore","2","2","U","Flying
Whenever Somnophore deals damage to a player, tap target creature that player controls. That creature doesn't untap during its controller's untap step for as long as Somnophore remains on the battlefield.","2UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Phalanx","2","5","W","First strike; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","5W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karplusan Strider","3","4","G","Karplusan Strider can't be the target of blue or black spells.","3G","Creature - Yeti");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Seeker","1","1","B","Whenever a creature enters the battlefield under an opponent's control, you may have that player lose 1 life.","1B","Creature - Vampire Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Serpent","2","2","U","Kicker {2} (You may pay an additional {2} as you cast this spell.)
Vodalian Serpent can't attack unless defending player controls an Island.
If Vodalian Serpent was kicked, it enters the battlefield with four +1/+1 counters on it.","3U","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zulaport Enforcer","1","1","B","Level up {4} ({4}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-2
3/3
LEVEL 3+
5/5
Zulaport Enforcer can't be blocked except by black creatures.","B","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chisei, Heart of Oceans","4","4","U","Flying
At the beginning of your upkeep, sacrifice Chisei, Heart of Oceans unless you remove a counter from a permanent you control.","2UU","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Earthblighter","1","1","B","{2}{B}, {T}, Sacrifice a Goblin: Destroy target land.","1B","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fiend of the Shadows","3","3","B","Flying
Whenever Fiend of the Shadows deals combat damage to a player, that player exiles a card from his or her hand. You may play that card for as long as it remains exiled.
Sacrifice a Human: Regenerate Fiend of the Shadows.","3BB","Creature - Vampire Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sabertooth Cobra","2","2","G","Whenever Sabertooth Cobra deals damage to a player, he or she gets a poison counter. That player gets another poison counter at the beginning of his or her next upkeep unless he or she pays {2} before that turn. (A player with ten or more poison counters loses the game.)","2G","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("True Believer","2","2","W","You have shroud. (You can't be the target of spells or abilities.)","WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nesting Wurm","4","3","G","Trample
When Nesting Wurm enters the battlefield, you may search your library for up to three cards named Nesting Wurm, reveal them, and put them into your hand. If you do, shuffle your library.","4GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Squire","2","2","W","When Leonin Squire enters the battlefield, return target artifact card with converted mana cost 1 or less from your graveyard to your hand.","1W","Creature - Cat Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Chosen","G","Enchant creature you control
{0}: Untap enchanted creature. Activate this ability only during your turn and only once each turn.
Tap enchanted creature: Untap target artifact, creature, or land. Activate this ability only if enchanted creature is white and is untapped and only once each turn.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wilt-Leaf Liege","4","4","WG","Other green creatures you control get +1/+1.
Other white creatures you control get +1/+1.
If a spell or ability an opponent controls causes you to discard Wilt-Leaf Liege, put it onto the battlefield instead of putting it into your graveyard.","1(G/W)(G/W)(G/W)","Creature - Elf Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invincible Hymn","W","Count the number of cards in your library. Your life total becomes that number.","6WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Skycutter","2","1","R","Sacrifice Goblin Skycutter: Goblin Skycutter deals 2 damage to target creature with flying. That creature loses flying until end of turn.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tempest Efreet","3","3","R","Remove Tempest Efreet from your deck before playing if you're not playing for ante.
{T}, Sacrifice Tempest Efreet: Target opponent may pay 10 life. If that player doesn't, he or she reveals a card at random from his or her hand. Exchange ownership of the revealed card and Tempest Efreet. Put the revealed card into your hand and Tempest Efreet from anywhere into that player's graveyard.","1RRR","Creature - Efreet");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Erayo, Soratami Ascendant","1","1","U","Flying
Whenever the fourth spell of a turn is cast, flip Erayo, Soratami Ascendant.
---
Whenever an opponent casts a spell for the first time in a turn, counter that spell.","1U","Legendary Creature - Moonfolk Monk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shriekhorn","Shriekhorn enters the battlefield with three charge counters on it.
{T}, Remove a charge counter from Shriekhorn: Target player puts the top two cards of his or her library into his or her graveyard.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hana Kami","1","1","G","{1}{G}, Sacrifice Hana Kami: Return target Arcane card from your graveyard to your hand.","G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dreamcatcher","1","1","U","Whenever you cast a Spirit or Arcane spell, you may sacrifice Dreamcatcher. If you do, draw a card.","U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lava Spike","R","Lava Spike deals 3 damage to target player.","R","Sorcery - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devout Chaplain","2","2","W","{T}, Tap two untapped Humans you control: Exile target artifact or enchantment.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moonlight Geist","2","1","W","Flying
{3}{W}: Prevent all combat damage that would be dealt to and dealt by Moonlight Geist this turn.","2W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Bully","2","1","R",NULL,"1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aurora Eidolon","2","2","W","{W}, Sacrifice Aurora Eidolon: Prevent the next 3 damage that would be dealt to target creature or player this turn.
Whenever you cast a multicolored spell, you may return Aurora Eidolon from your graveyard to your hand.","3W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heart of Light","W","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Prevent all damage that would be dealt to and dealt by enchanted creature.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Temper","R","Dark Temper deals 2 damage to target creature. If you control a black permanent, destroy the creature instead.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lumbering Satyr","5","4","G","All creatures have forestwalk.","2GG","Creature - Satyr Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Free-for-All","U","When Free-for-All comes into play, set aside all creatures in play, face down.
During each player's upkeep, that player chooses a creature card at random from those set aside in this way and puts that creature into play under his or her control.
If Free-for-All leaves play, put each creature still set aside this way into its owner's graveyard.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Temple Acolyte","1","3","W","When Temple Acolyte enters the battlefield, you gain 3 life.","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Air Servant","4","3","U","Flying
{2}{U}: Tap target creature with flying.","4U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wound Reflection","B","At the beginning of each end step, each opponent loses life equal to the life he or she lost this turn. (Damage causes loss of life.)","5B","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Executioner's Hood","Equipped creature has intimidate. (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Huntmaster of the Fells","2","2","RG","Whenever this creature enters the battlefield or transforms into Huntmaster of the Fells, put a 2/2 green Wolf creature token onto the battlefield and you gain 2 life.
At the beginning of each upkeep, if no spells were cast last turn, transform Huntmaster of the Fells.","2RG","Creature - Human Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quickening Licid","1","1","W","{1}{W}, {T}: Quickening Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {W} to end this effect.
Enchanted creature has first strike.","1W","Creature - Licid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fountain of Youth","{2}, {T}: You gain 1 life.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foul Imp","2","2","B","Flying
When Foul Imp enters the battlefield, you lose 2 life.","BB","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread Drone","4","1","B","When Dread Drone enters the battlefield, put two 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool.""","4B","Creature - Eldrazi Drone");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Valleymaker","5","5","RG","{T}, Sacrifice a Mountain: Valleymaker deals 3 damage to target creature.
{T}, Sacrifice a Forest: Choose a player. That player adds {G}{G}{G} to his or her mana pool.","5(R/G)","Creature - Giant Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fodder Launch","B","As an additional cost to cast Fodder Launch, sacrifice a Goblin.
Target creature gets -5/-5 until end of turn. Fodder Launch deals 5 damage to that creature's controller.","3B","Tribal Sorcery - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Putrid Cyclops","3","3","B","When Putrid Cyclops enters the battlefield, scry 1, then reveal the top card of your library. Putrid Cyclops gets -X/-X until end of turn, where X is that card's converted mana cost. (To scry 1, look at the top card of your library, then you may put that card on the bottom of your library.)","2B","Creature - Zombie Cyclops");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Equal Treatment","W","If any source would deal 1 or more damage to a creature or player this turn, it deals 2 damage to that creature or player instead.
Draw a card.","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Citanul Flute","{X}, {T}: Search your library for a creature card with converted mana cost X or less, reveal it, and put it into your hand. Then shuffle your library.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Patriarch","5","5","B","{2}{B}, Sacrifice a creature: Target creature gets -2/-2 until end of turn.
{2}{B}, Exile a creature card from your graveyard: Target creature gets -2/-2 until end of turn.","3BBB","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Immolating Souleater","1","1","{(r/p)}: Immolating Souleater gets +1/+0 until end of turn. ({(r/p)} can be paid with either {R} or 2 life.)","2","Artifact Creature - Hound");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sandstone Deadfall","{T}, Sacrifice two lands and Sandstone Deadfall: Destroy target attacking creature.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Makeshift Mauler","4","5","U","As an additional cost to cast Makeshift Mauler, exile a creature card from your graveyard.","3U","Creature - Zombie Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dralnu's Crusade","BR","Goblin creatures get +1/+1.
All Goblins are black and are Zombies in addition to their other creature types.","1BR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Architects of Will","3","3","UB","When Architects of Will enters the battlefield, look at the top three cards of target player's library, then put them back in any order.
Cycling {(u/b)} ({(u/b)}, Discard this card: Draw a card.)","2UB","Artifact Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Snake","3","1","R","When Fire Snake dies, destroy target land.","4R","Creature - Snake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tradewind Rider Avatar","{3}: Each player returns a permanent he or she controls to its owner's hand unless he or she pays 2 life.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Living Death","B","Each player exiles all creature cards from his or her graveyard, then sacrifices all creatures he or she controls, then puts all cards he or she exiled this way onto the battlefield.","3BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Evolution Vat","{3}, {T}: Tap target creature and put a +1/+1 counter on it. Until end of turn, that creature gains ""{2}{G}{U}: Double the number of +1/+1 counters on this creature.""","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gloryscale Viashino","3","3","WRG","Whenever you cast a multicolored spell, Gloryscale Viashino gets +3/+3 until end of turn.","1RGW","Creature - Viashino Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Panther Warriors","6","3","G",NULL,"4G","Creature - Cat Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Teferi's Isle","Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)
Teferi's Isle enters the battlefield tapped.
{T}: Add {U}{U} to your mana pool.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feast of the Unicorn","B","Enchant creature
Enchanted creature gets +4/+0.","3B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bramblewood Paragon","2","2","G","Each other Warrior creature you control enters the battlefield with an additional +1/+1 counter on it.
Each creature you control with a +1/+1 counter on it has trample.","1G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hydra Omnivore","8","8","G","Whenever Hydra Omnivore deals combat damage to an opponent, it deals that much damage to each other opponent.","4GG","Creature - Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Short","U","Tap all lands target player controls and empty his or her mana pool.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Withering Gaze","U","Target opponent reveals his or her hand. You draw a card for each Forest and green card in it.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Icefall","R","Destroy target artifact or land.
Recover {R}{R} (When a creature is put into your graveyard from the battlefield, you may pay {R}{R}. If you do, return this card from your graveyard to your hand. Otherwise, exile this card.)","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sacrifice","B","As an additional cost to cast Sacrifice, sacrifice a creature.
Add to your mana pool an amount of {B} equal to the sacrificed creature's converted mana cost.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raven Familiar","1","2","U","Flying
Echo {2}{U} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Raven Familiar enters the battlefield, look at the top three cards of your library. Put one of them into your hand and the rest on the bottom of your library in any order.","2U","Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thran Forge","{2}: Until end of turn, target nonartifact creature gets +1/+0 and becomes an artifact in addition to its other types.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream Fracture","U","Counter target spell. Its controller draws a card.
Draw a card.","1UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Altar of Bone","WG","As an additional cost to cast Altar of Bone, sacrifice a creature.
Search your library for a creature card, reveal that card, and put it into your hand. Then shuffle your library.","GW","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Behold the Power of Destruction","When you set this scheme in motion, destroy all nonland permanents target opponent controls.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orochi Sustainer","1","2","G","{T}: Add {G} to your mana pool.","1G","Creature - Snake Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lingering Tormentor","2","2","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","3B","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragon Shadow","B","Enchant creature
Enchanted creature gets +1/+0 and has fear. (It can't be blocked except by artifact creatures and/or black creatures.)
When a creature with converted mana cost 6 or greater enters the battlefield, you may return Dragon Shadow from your graveyard to the battlefield attached to that creature.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx of Magosi","6","6","U","Flying
{2}{U}: Draw a card, then put a +1/+1 counter on Sphinx of Magosi.","3UUU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Paratroopers","4","4","R","When Orcish Paratroopers comes into play, flip it from a height of at least one foot. Sacrifice Orcish Paratroopers unless it lands face up after turning over completely.","2R","Creature - Orc Paratrooper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Dismisser","2","2","U","When Vedalken Dismisser enters the battlefield, put target creature on top of its owner's library.","5U","Creature - Vedalken Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Illusions of Grandeur","U","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When Illusions of Grandeur enters the battlefield, you gain 20 life.
When Illusions of Grandeur leaves the battlefield, you lose 20 life.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("War-Spike Changeling","3","3","R","Changeling (This card is every creature type at all times.)
{R}: War-Spike Changeling gains first strike until end of turn.","3R","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Command","G","Choose two - Target player gains 7 life; or put target noncreature permanent on top of its owner's library; or target player shuffles his or her graveyard into his or her library; or search your library for a creature card, reveal it, put it into your hand, then shuffle your library.","3GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pygmy Razorback","2","1","G","Trample","1G","Creature - Boar");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Axe","Darksteel Axe is indestructible. (Effects that say ""destroy"" don't destroy it.)
Equipped creature gets +2/+0.
Equip {2}","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudchaser Eagle","2","2","W","Flying
When Cloudchaser Eagle enters the battlefield, destroy target enchantment.","3W","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seeds of Strength","WG","Target creature gets +1/+1 until end of turn.
Target creature gets +1/+1 until end of turn.
Target creature gets +1/+1 until end of turn.","GW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiderwig Boggart","2","2","B","When Spiderwig Boggart enters the battlefield, target creature gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","2B","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx of Jwar Isle","5","5","U","Flying, shroud
You may look at the top card of your library. (You may do this at any time.)","4UU","Creature - Sphinx");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slice and Dice","R","Slice and Dice deals 4 damage to each creature.
Cycling {2}{R} ({2}{R}, Discard this card: Draw a card.)
When you cycle Slice and Dice, you may have it deal 1 damage to each creature.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Fangtail","3","3","R","{T}: Viashino Fangtail deals 1 damage to target creature or player.","2RR","Creature - Viashino Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cerulean Sphinx","5","5","U","Flying
{U}: Cerulean Sphinx's owner shuffles it into his or her library.","4UU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cemetery Puca","1","2","UB","Whenever a creature dies, you may pay {1}. If you do, Cemetery Puca becomes a copy of that creature and gains this ability.","1(U/B)(U/B)","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Balloon Brigade","1","1","R","{R}: Goblin Balloon Brigade gains flying until end of turn.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arc Blade","R","Arc Blade deals 2 damage to target creature or player. Exile Arc Blade with three time counters on it.
Suspend 3-{2}{R} (Rather than cast this card from your hand, you may pay {2}{R} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Escaped Shapeshifter","3","4","U","As long as an opponent controls a creature with flying not named Escaped Shapeshifter, Escaped Shapeshifter has flying. The same is true for first strike, trample, and protection from any color.","3UU","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaea's Bounty","G","Search your library for up to two Forest cards, reveal those cards, and put them into your hand. Then shuffle your library.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fledgling Osprey","1","1","U","Fledgling Osprey has flying as long as it's enchanted.","U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Impending Disaster","R","At the beginning of your upkeep, if there are seven or more lands on the battlefield, sacrifice Impending Disaster and destroy all lands.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heroism","W","Sacrifice a white creature: For each attacking red creature, prevent all combat damage that would be dealt by that creature this turn unless its controller pays {2}{R}.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chartooth Cougar","4","4","R","{R}: Chartooth Cougar gets +1/+0 until end of turn.
Mountaincycling {2} ({2}, Discard this card: Search your library for a Mountain card, reveal it, and put it into your hand. Then shuffle your library.)","5R","Creature - Cat Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Millikin","0","1","{T}, Put the top card of your library into your graveyard: Add {1} to your mana pool.","2","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Null Rod","Activated abilities of artifacts can't be activated.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Detainment Spell","W","Enchant creature
Enchanted creature's activated abilities can't be activated.
{1}{W}: Attach Detainment Spell to target creature.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feedback","U","Enchant enchantment
At the beginning of the upkeep of enchanted enchantment's controller, Feedback deals 1 damage to that player.","2U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("In the Web of War","R","Whenever a creature enters the battlefield under your control, it gets +2/+0 and gains haste until end of turn.","3RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonfire","R","Demonfire deals X damage to target creature or player. If a creature dealt damage this way would die this turn, exile it instead.
Hellbent - If you have no cards in hand, Demonfire can't be countered by spells or abilities and the damage can't be prevented.","XR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raking Canopy","G","Whenever a creature with flying attacks you, Raking Canopy deals 4 damage to it.","1GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chimeric Staff","{X}: Chimeric Staff becomes an X/X Construct artifact creature until end of turn.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magnetic Mountain","R","Blue creatures don't untap during their controllers' untap steps.
At the beginning of each player's upkeep, that player may choose any number of tapped blue creatures he or she controls and pay {4} for each creature chosen this way. If the player does, untap those creatures.","1RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strength of the Tajuru","G","Multikicker {1} (You may pay an additional {1} any number of times as you cast this spell.)
Choose target creature, then choose another target creature for each time Strength of the Tajuru was kicked. Put X +1/+1 counters on each of them.","XGG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathbringer Liege","3","4","WB","Other white creatures you control get +1/+1.
Other black creatures you control get +1/+1.
Whenever you cast a white spell, you may tap target creature.
Whenever you cast a black spell, you may destroy target creature if it's tapped.","2(W/B)(W/B)(W/B)","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Tribute","B","As an additional cost to cast Phyrexian Tribute, sacrifice two creatures.
Destroy target artifact.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moat","W","Creatures without flying can't attack.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Havengul Runebinder","2","2","U","{2}{U}, {T}, Exile a creature card from your graveyard: Put a 2/2 black Zombie creature token onto the battlefield, then put a +1/+1 counter on each Zombie creature you control.","2UU","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Daily Regimen","W","Enchant creature
{1}{W}: Put a +1/+1 counter on enchanted creature.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Energy Bolt","WR","Choose one - Energy Bolt deals X damage to target player; or target player gains X life.","XRW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gluttonous Zombie","3","3","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)","4B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Centaur Archer","3","2","RG","{T}: Centaur Archer deals 1 damage to target creature with flying.","1RG","Creature - Centaur Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crippling Chill","U","Tap target creature. It doesn't untap during its controller's next untap step.
Draw a card.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dredge","B","Sacrifice a creature or land.
Draw a card.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepwood Ghoul","2","1","B","Pay 2 life: Regenerate Deepwood Ghoul.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ronin Cliffrider","2","2","R","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
Whenever Ronin Cliffrider attacks, you may have it deal 1 damage to each creature defending player controls.","3RR","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Executioner","2","2","B","Whenever Cabal Executioner deals combat damage to a player, that player sacrifices a creature.
Morph {3}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2BB","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Summoner's Pact","Search your library for a green creature card, reveal it, and put it into your hand. Then shuffle your library.
At the beginning of your next upkeep, pay {2}{G}{G}. If you don't, you lose the game.","0","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shield Bearer","0","3","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stone-Seeder Hierophant","1","1","G","Whenever a land enters the battlefield under your control, untap Stone-Seeder Hierophant.
{T}: Untap target land.","2GG","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yellow Scarves General","2","2","R","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Yellow Scarves General can't block.","3R","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Simic Signet","{1}, {T}: Add {G}{U} to your mana pool.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sudden Spoiling","B","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Creatures target player controls become 0/2 and lose all abilities until end of turn.","1BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Bomb","B","As an additional cost to cast Death Bomb, sacrifice a creature.
Destroy target nonblack creature. It can't be regenerated. Its controller loses 2 life.","3B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Maggot Therapy","B","Flash
Enchant creature
Enchanted creature gets +2/-2.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lunar Avenger","2","2","Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it.)
Remove a +1/+1 counter from Lunar Avenger: Lunar Avenger gains your choice of flying, first strike, or haste until end of turn.","7","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Samite Pilgrim","1","1","W","Domain - {T}: Prevent the next X damage that would be dealt to target creature this turn, where X is the number of basic land types among lands you control.","1W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirror Gallery","The ""legend rule"" doesn't apply.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shirei, Shizo's Caretaker","2","2","B","Whenever a creature with power 1 or less is put into your graveyard from the battlefield, you may return that card to the battlefield under your control at the beginning of the next end step if Shirei, Shizo's Caretaker is still on the battlefield.","4B","Legendary Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Llanowar","All creatures have ""{T}: Add {G}{G} to your mana pool.""
Whenever you roll {C}, untap all creatures you control.",NULL,"Plane - Dominaria");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Riding the Dilu Horse","G","Target creature gets +2/+2 and gains horsemanship. (It can't be blocked except by creatures with horsemanship. This effect lasts indefinitely.)","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Belltower Sphinx","2","5","U","Flying
Whenever a source deals damage to Belltower Sphinx, that source's controller puts that many cards from the top of his or her library into his or her graveyard.","4U","Creature - Sphinx");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Iona's Judgment","W","Exile target creature or enchantment.","4W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shaleskin Plower","3","2","R","Morph {4}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Shaleskin Plower is turned face up, destroy target land.","3R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zuo Ci, the Mocking Sage","1","2","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Zuo Ci, the Mocking Sage can't be blocked by creatures with horsemanship.","1GG","Legendary Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Terra Eternal","W","All lands are indestructible.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rune of Protection: Red","W","{W}: The next time a red source of your choice would deal damage to you this turn, prevent that damage.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Marshal's Anthem","W","Multikicker {1}{W} (You may pay an additional {1}{W} any number of times as you cast this spell.)
Creatures you control get +1/+1.
When Marshal's Anthem enters the battlefield, return up to X target creature cards from your graveyard to the battlefield, where X is the number of times Marshal's Anthem was kicked.","2WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vivid Crag","Vivid Crag enters the battlefield tapped with two charge counters on it.
{T}: Add {R} to your mana pool.
{T}, Remove a charge counter from Vivid Crag: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duskwalker","1","1","B","Kicker {3}{B} (You may pay an additional {3}{B} as you cast this spell.)
If Duskwalker was kicked, it enters the battlefield with two +1/+1 counters on it and with fear. (It can't be blocked except by artifact creatures and/or black creatures.)","B","Creature - Human Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sightless Ghoul","2","2","B","Sightless Ghoul can't block.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","3B","Creature - Zombie Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yuan Shao's Infantry","2","2","R","Whenever Yuan Shao's Infantry attacks alone, Yuan Shao's Infantry is unblockable this combat.","3R","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Punisher","2","2","W","Loxodon Punisher gets +2/+2 for each Equipment attached to it.","3W","Creature - Elephant Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divert","U","Change the target of target spell with a single target unless that spell's controller pays {2}.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kessig Recluse","2","3","G","Reach (This creature can block creatures with flying.)
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","2GG","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Power of Fire","R","Enchant creature
Enchanted creature has ""{T}: This creature deals 1 damage to target creature or player.""","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aphetto Dredging","B","Return up to three target creature cards of the creature type of your choice from your graveyard to your hand.","3B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vernal Bloom","G","Whenever a Forest is tapped for mana, its controller adds {G} to his or her mana pool (in addition to the mana the land produces).","3G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sivitri Scarzam","6","4","UB",NULL,"5UB","Legendary Creature - Human");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Magewright's Stone","{1}, {T}: Untap target creature that has an activated ability with {T} in its cost.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Painwracker Oni","5","4","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
At the beginning of your upkeep, sacrifice a creature if you don't control an Ogre.","3BB","Creature - Demon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chaotic Strike","R","Cast Chaotic Strike only during combat after blockers are declared.
Flip a coin. If you win the flip, target creature gets +1/+1 until end of turn.
Draw a card.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Full Moon's Rise","G","Werewolf creatures you control get +1/+0 and have trample.
Sacrifice Full Moon's Rise: Regenerate all Werewolf creatures you control.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Militia's Pride","W","Whenever a nontoken creature you control attacks, you may pay {W}. If you do, put a 1/1 white Kithkin Soldier creature token onto the battlefield tapped and attacking.","1W","Tribal Enchantment - Kithkin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Demonlord of Ashmouth","5","4","B","Flying
When Demonlord of Ashmouth enters the battlefield, exile it unless you sacrifice another creature.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","2BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Manabarbs","R","Whenever a player taps a land for mana, Manabarbs deals 1 damage to that player.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skeletal Wurm","7","6","B","{B}: Regenerate Skeletal Wurm.","7B","Creature - Skeleton Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cavalry Master","3","3","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
Other creatures you control with flanking have flanking. (Each instance of flanking triggers separately.)","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistfolk","1","2","U","{U}: Counter target spell that targets Mistfolk.","UU","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Natural Spring","G","Target player gains 8 life.","3GG","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Coiled Tinviper","2","1","First strike","3","Artifact Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Evil Eye of Orms-by-Gore","3","6","B","Non-Eye creatures you control can't attack.
Evil Eye of Orms-by-Gore can't be blocked except by Walls.","4B","Creature - Eye");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pain Kami","2","2","R","{X}{R}, Sacrifice Pain Kami: Pain Kami deals X damage to target creature.","2R","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seeker","W","Enchant creature
Enchanted creature can't be blocked except by artifact creatures and/or white creatures.","2WW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightkeeper of Emeria","2","4","W","Multikicker {W} (You may pay an additional {W} any number of times as you cast this spell.)
Flying
When Lightkeeper of Emeria enters the battlefield, you gain 2 life for each time it was kicked.","3W","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Umezawa's Jitte","Whenever equipped creature deals combat damage, put two charge counters on Umezawa's Jitte.
Remove a charge counter from Umezawa's Jitte: Choose one -  Equipped creature gets +2/+2 until end of turn; or target creature gets -1/-1 until end of turn; or you gain 2 life.
Equip {2}","2","Legendary Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cho-Manno's Blessing","W","Flash
Enchant creature
As Cho-Manno's Blessing enters the battlefield, choose a color.
Enchanted creature has protection from the chosen color. This effect doesn't remove Cho-Manno's Blessing.","WW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of the Beasts","1","2","G","{G}, {T}: Choose target opponent who controlled more creatures than you did as you activated this ability. Put a 2/2 green Beast creature token onto the battlefield.","GG","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pillage","R","Destroy target artifact or land. It can't be regenerated.","1RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vengeful Firebrand","5","2","R","Vengeful Firebrand has haste as long as a Warrior card is in your graveyard.
{R}: Vengeful Firebrand gets +1/+0 until end of turn.","3R","Creature - Elemental Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Helm of Possession","You may choose not to untap Helm of Possession during your untap step.
{2}, {T}, Sacrifice a creature: Gain control of target creature for as long as you control Helm of Possession and Helm of Possession remains tapped.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Suppress","B","Target player exiles all cards from his or her hand face down. At the beginning of the end step of that player's next turn, that player returns those cards to his or her hand.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Carnival of Souls","B","Whenever a creature enters the battlefield, you lose 1 life and add {B} to your mana pool.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duskdale Wurm","7","7","G","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","5GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Broker","2","2","U","{T}: Target player draws two cards, then discards two cards.","3U","Creature - Cephalid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("When Fluffy Bunnies Attack","B","Target creature gets -X/-X until end of turn, where X is the number of times the letter of your choice appears in that creature's name.","3B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yomiji, Who Bars the Way","4","4","W","Whenever a legendary permanent other than Yomiji, Who Bars the Way is put into a graveyard from the battlefield, return that card to its owner's hand.","5WW","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Braingeyser","U","Target player draws X cards.","XUU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Homarid Shaman","2","1","U","{U}: Tap target green creature.","2UU","Creature - Homarid Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scute Mob","1","1","G","At the beginning of your upkeep, if you control five or more lands, put four +1/+1 counters on Scute Mob.","G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flame-Kin Zealot","2","2","WR","When Flame-Kin Zealot enters the battlefield, creatures you control get +1/+1 and gain haste until end of turn.","1RRW","Creature - Elemental Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Mauler","4","4","G","Trample
Whenever Kavu Mauler attacks, it gets +1/+1 until end of turn for each other attacking Kavu.","4GG","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razor Hippogriff","3","3","W","Flying
When Razor Hippogriff enters the battlefield, return target artifact card from your graveyard to your hand. You gain life equal to that card's converted mana cost.","3WW","Creature - Hippogriff");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunt Down","G","Target creature blocks target creature this turn if able.","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Razors","4","1","R","Defender (This creature can't attack.)
First strike","1R","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Control","U","Enchant creature
You control enchanted creature.","3UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emerald Dragonfly","1","1","G","Flying
{G}{G}: Emerald Dragonfly gains first strike until end of turn.","1G","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forgotten Lore","G","Target opponent chooses a card in your graveyard. You may pay {G}. If you do, repeat this process except that opponent can't choose a card already chosen for Forgotten Lore. Then put the last chosen card into your hand.","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akron Legionnaire","8","4","W","Except for creatures named Akron Legionnaire and artifact creatures, creatures you control can't attack.","6WW","Creature - Giant Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aphetto Runecaster","2","3","U","Whenever a permanent is turned face up, you may draw a card.","3U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Delver","3","2","B","When Phyrexian Delver enters the battlefield, return target creature card from your graveyard to the battlefield. You lose life equal to that card's converted mana cost.","3BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Task Force","1","3","W","Whenever Task Force becomes the target of a spell or ability, it gets +0/+3 until end of turn.","2W","Creature - Human Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Path to Exile","W","Exile target creature. Its controller may search his or her library for a basic land card, put that card onto the battlefield tapped, then shuffle his or her library.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Perimeter Captain","0","4","W","Defender
Whenever a creature you control with defender blocks, you may gain 2 life.","W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oath of Druids","G","At the beginning of each player's upkeep, that player chooses target player who controls more creatures than he or she does and is his or her opponent. The first player may reveal cards from the top of his or her library until he or she reveals a creature card. If he or she does, that player puts that card onto the battlefield and all other cards revealed this way into his or her graveyard.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul's Attendant","1","1","W","Whenever another creature enters the battlefield, you may gain 1 life.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hum of the Radix","G","Each artifact spell costs {1} more to cast for each artifact its controller controls.","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kei Takahashi","2","2","WG","{T}: Prevent the next 2 damage that would be dealt to target creature this turn.","2GW","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thrull Champion","2","2","B","Thrull creatures get +1/+1.
{T}: Gain control of target Thrull for as long as you control Thrull Champion.","4B","Creature - Thrull");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Concerted Effort","W","At the beginning of each upkeep, all creatures you control gain flying until end of turn if a creature you control has flying. The same is true for fear, first strike, double strike, landwalk, protection, trample, and vigilance.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitchen Finks","3","2","WG","When Kitchen Finks enters the battlefield, you gain 2 life.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","1(G/W)(G/W)","Creature - Ouphe");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scarwood Hag","1","1","G","{G}{G}{G}{G}, {T}: Target creature gains forestwalk until end of turn.
{T}: Target creature loses forestwalk until end of turn.","1G","Creature - Hag");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hail Storm","G","Hail Storm deals 2 damage to each attacking creature and 1 damage to you and each creature you control.","1GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Affa Guard Hound","2","2","W","Flash
When Affa Guard Hound enters the battlefield, target creature gets +0/+3 until end of turn.","2W","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nova Cleric","1","2","W","{2}{W}, {T}, Sacrifice Nova Cleric: Destroy all enchantments.","W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Halimar Depths","Halimar Depths enters the battlefield tapped.
When Halimar Depths enters the battlefield, look at the top three cards of your library, then put them back in any order.
{T}: Add {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scarwood Goblins","2","2","RG",NULL,"RG","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cauldron Haze","WB","Choose any number of target creatures. Each of those creatures gains persist until end of turn. (When it dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","1(W/B)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aquastrand Spider","0","0","G","Graft 2 (This creature enters the battlefield with two +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{G}: Target creature with a +1/+1 counter on it gains reach until end of turn. (It can block creatures with flying.)","1G","Creature - Spider Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Glare","0","5","W","Defender (This creature can't attack.)
Wall of Glare can block any number of creatures.","1W","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Quietus Spike","Equipped creature has deathtouch.
Whenever equipped creature deals combat damage to a player, that player loses half his or her life, rounded up.
Equip {3}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quillspike","1","1","BG","{(b/g)}, Remove a -1/-1 counter from a creature you control: Quillspike gets +3/+3 until end of turn.","2(B/G)","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vampiric Link","B","Enchant creature
Whenever enchanted creature deals damage, you gain that much life.","B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("City of Ass","City of Ass comes into play tapped.
{T}: Add one and one-half mana of any one color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Centaur Glade","G","{2}{G}{G}: Put a 3/3 green Centaur creature token onto the battlefield.","3GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Praetor's Grasp","B","Search target opponent's library for a card and exile it face down. Then that player shuffles his or her library. You may look at and play that card for as long as it remains exiled.","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beacon of Destruction","R","Beacon of Destruction deals 5 damage to target creature or player. Shuffle Beacon of Destruction into its owner's library.","3RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodfire Kavu","2","2","R","{R}, Sacrifice Bloodfire Kavu: Bloodfire Kavu deals 2 damage to each creature.","2RR","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stern Marshal","2","2","W","{T}: Target creature gets +2/+2 until end of turn. Activate this ability only during your turn, before attackers are declared.","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tyrannize","BR","Target player discards his or her hand unless he or she pays 7 life.","3(B/R)(B/R)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aluren","G","Any player may play creature cards with converted mana cost 3 or less without paying their mana cost and as though they had flash.","2GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cremate","B","Exile target card from a graveyard.
Draw a card.","B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Altar","Sacrifice a creature: Add one mana of any color to your mana pool.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Overgrown Tomb","({T}: Add {B} or {G} to your mana pool.)
As Overgrown Tomb enters the battlefield, you may pay 2 life. If you don't, Overgrown Tomb enters the battlefield tapped.",NULL,"Land - Swamp Forest");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enslaved Scout","2","2","R","{2}: Enslaved Scout gains mountainwalk until end of turn.","2R","Creature - Goblin Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Petradon","5","6","R","When Petradon enters the battlefield, exile two target lands.
When Petradon leaves the battlefield, return the exiled cards to the battlefield under their owners' control.
{R}: Petradon gets +1/+0 until end of turn.","6RR","Creature - Nightmare Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master Decoy","1","2","W","{W}, {T}: Tap target creature.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Child of Night","2","1","B","Lifelink (Damage dealt by this creature also causes you to gain that much life.)","1B","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dromar's Charm","WUB","Choose one - You gain 5 life; or counter target spell; or target creature gets -2/-2 until end of turn.","WUB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ironfist Crusher","2","4","W","Ironfist Crusher can block any number of creatures.
Morph {3}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daunting Defender","3","3","W","If a source would deal damage to a Cleric creature you control, prevent 1 of that damage.","4W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Groffskithur","3","3","G","Whenever Groffskithur becomes blocked, you may return target card named Groffskithur from your graveyard to your hand.","5G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Segovian Leviathan","3","3","U","Islandwalk","4U","Creature - Leviathan");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Violent Ultimatum","BRG","Destroy three target permanents.","BBRRRGG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("The Unspeakable","6","7","U","Flying, trample
Whenever The Unspeakable deals combat damage to a player, you may return target Arcane card from your graveyard to your hand.","6UUU","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sacred Ground","W","Whenever a spell or ability an opponent controls causes a land to be put into your graveyard from the battlefield, return that card to the battlefield.","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Damping Matrix","Activated abilities of artifacts and creatures can't be activated unless they're mana abilities.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Roughshod Mentor","5","4","G","Green creatures you control have trample.","5G","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Overbeing of Myth","*","*","UG","Overbeing of Myth's power and toughness are each equal to the number of cards in your hand.
At the beginning of your draw step, draw an additional card.","(G/U)(G/U)(G/U)(G/U)(G/U)","Creature - Spirit Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frazzled Editor","2","2","R","Protection from wordy (Something is wordy if it has four or more lines of text in its text box.)","1R","Creature - Human Bureaucrat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mistform Mask","U","Enchant creature
{1}: Enchanted creature becomes the creature type of your choice until end of turn.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peel from Reality","U","Return target creature you control and target creature you don't control to their owners' hands.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tempest Drake","2","2","WU","Flying, vigilance","1WU","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brawl","R","Until end of turn, all creatures gain ""{T}: This creature deals damage equal to its power to target creature.""","3RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cackling Witch","1","1","B","{X}{B}, {T}, Discard a card: Target creature gets +X/+0 until end of turn.","1B","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seraph","4","4","W","Flying
Whenever a creature dealt damage by Seraph this turn dies, put that card onto the battlefield under your control at the beginning of the next end step. Sacrifice the creature when you lose control of Seraph.","6W","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lesser Gargadon","6","4","R","Whenever Lesser Gargadon attacks or blocks, sacrifice a land.","2RR","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aftershock","R","Destroy target artifact, creature, or land. Aftershock deals 3 damage to you.","2RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lighthouse Chronologist","1","3","U","Level up {U} ({U}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 4-6
2/4
LEVEL 7+
3/5
At the beginning of each end step, if it's not your turn, take an extra turn after this one.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pilgrim of Virtue","1","3","W","Protection from black
{W}, Sacrifice Pilgrim of Virtue: The next time a black source of your choice would deal damage this turn, prevent that damage.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Lacerator","2","2","B","At the beginning of your upkeep, you lose 1 life unless an opponent has 10 or less life.","B","Creature - Vampire Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corpsehatch","B","Destroy target nonblack creature. Put two 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool.""","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Citanul Hierophants","3","2","G","Creatures you control have ""{T}: Add {G} to your mana pool.""","3G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Slash","B","{B}, Sacrifice a creature: Target opponent reveals his or her hand. You choose a card from it. That player discards that card. Activate this ability only any time you could cast a sorcery.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Assassin","2","1","R","{T}: You choose target creature an opponent controls, and that opponent chooses target creature. Flip a coin. If you win the flip, destroy the creature you chose. If you lose the flip, destroy the creature your opponent chose.","2R","Creature - Goblin Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eye of Singularity","W","When Eye of Singularity enters the battlefield, destroy each permanent with the same name as another permanent, except for basic lands. They can't be regenerated.
Whenever a permanent other than a basic land enters the battlefield, destroy all other permanents with that name. They can't be regenerated.","3W","World Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grimoire of the Dead","{1}, {T}, Discard a card: Put a study counter on Grimoire of the Dead.
{T}, Remove three study counters from Grimoire of the Dead and sacrifice it: Put all creature cards from all graveyards onto the battlefield under your control. They're black Zombies in addition to their other colors and types.","4","Legendary Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rushwood Herbalist","2","2","G","{G}, {T}, Discard a card: Regenerate target creature.","2G","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Savra, Queen of the Golgari","2","2","BG","Whenever you sacrifice a black creature, you may pay 2 life. If you do, each other player sacrifices a creature.
Whenever you sacrifice a green creature, you may gain 2 life.","2BG","Legendary Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rathi Assassin","2","2","B","{1}{B}{B}, {T}: Destroy target tapped nonblack creature.
{3}, {T}: Search your library for a Mercenary permanent card with converted mana cost 3 or less and put it onto the battlefield. Then shuffle your library.","2BB","Creature - Zombie Mercenary Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunderscape Master","2","2","R","{B}{B}, {T}: Target player loses 2 life and you gain 2 life.
{G}{G}, {T}: Creatures you control get +2/+2 until end of turn.","2RR","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tuktuk Grunts","2","2","R","Haste
Whenever Tuktuk Grunts or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Tuktuk Grunts.","4R","Creature - Goblin Warrior Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Conundrum Sphinx","4","4","U","Flying
Whenever Conundrum Sphinx attacks, each player names a card. Then each player reveals the top card of his or her library. If the card a player revealed is the card he or she named, that player puts it into his or her hand. If it's not, that player puts it on the bottom of his or her library.","2UU","Creature - Sphinx");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sisay's Ring","{T}: Add {2} to your mana pool.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Purgatory","WB","Whenever a nontoken creature is put into your graveyard from the battlefield, exile that card.
At the beginning of your upkeep, you may pay {4} and 2 life. If you do, return a card exiled with Purgatory to the battlefield.","2WB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lifespinner","3","3","G","{T}, Sacrifice three Spirits: Search your library for a legendary Spirit permanent card and put it onto the battlefield. Then shuffle your library.","3G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ricochet","R","Whenever any spell targets a single player, each player rolls a six-sided die. That spell is redirected to the player or players with the lowest die roll. If two or more players tie for the lowest die roll, they reroll until there is no tie.","R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Resister","4","3","R",NULL,"2RR","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elite Javelineer","2","2","W","Whenever Elite Javelineer blocks, it deals 1 damage to target attacking creature.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread Specter","2","2","B","Whenever Dread Specter blocks or becomes blocked by a nonblack creature, destroy that creature at end of combat.","3B","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phytohydra","1","1","WG","If damage would be dealt to Phytohydra, put that many +1/+1 counters on it instead.","2GWW","Creature - Plant Hydra");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oboro Envoy","1","3","U","Flying
{2}, Return a land you control to its owner's hand: Target creature gets -X/-0 until end of turn, where X is the number of cards in your hand.","3U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Black Carriage","4","4","B","Trample
Black Carriage doesn't untap during your untap step.
Sacrifice a creature: Untap Black Carriage. Activate this ability only during your upkeep.","3BB","Creature - Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cradle to Grave","B","Destroy target nonblack creature that entered the battlefield this turn.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lapse of Certainty","W","Counter target spell. If that spell is countered this way, put it on top of its owner's library instead of into that player's graveyard.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Indomitable Will","W","Flash
Enchant creature
Enchanted creature gets +1/+2.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread Slaver","3","5","B","Whenever a creature dealt damage by Dread Slaver this turn dies, return it to the battlefield under your control. That creature is a black Zombie in addition to its other colors and types.","3BB","Creature - Zombie Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crumbling Sanctuary","If damage would be dealt to a player, that player exiles that many cards from the top of his or her library instead.","5","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Grimoire","{4}, {T}: Target opponent chooses one of the top two cards of your graveyard. Exile that card and put the other one into your hand.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shriek Raptor","2","3","W","Flying
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","3WW","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jaws of Stone","R","Jaws of Stone deals X damage divided as you choose among any number of target creatures and/or players, where X is the number of Mountains you control as you cast Jaws of Stone.","5R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swerve","UR","Change the target of target spell with a single target.","UR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stir the Grave","B","Return target creature card with converted mana cost X or less from your graveyard to the battlefield.","XB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Dragon","6","6","R","Flying
When Fire Dragon enters the battlefield, it deals damage equal to the number of Mountains you control to target creature.","6RRR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Gnomes","2","2","{3}, {T}: Regenerate target artifact creature.","4","Artifact Creature - Gnome");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cao Ren, Wei Commander","3","3","B","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
When Cao Ren, Wei Commander enters the battlefield, you lose 3 life.","2BB","Legendary Creature - Human Soldier Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Scout","0","2","R","Domain - Kavu Scout gets +1/+0 for each basic land type among lands you control.","2R","Creature - Kavu Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Amrou Seekers","2","2","W","Amrou Seekers can't be blocked except by artifact creatures and/or white creatures.","2W","Creature - Kithkin Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glint Hawk","2","2","W","Flying
When Glint Hawk enters the battlefield, sacrifice it unless you return an artifact you control to its owner's hand.","W","Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seshiro the Anointed Avatar","At the beginning of the game, choose a creature type. Creatures you control, creature spells you control, and creature cards you own in any zone other than  the battlefield or the stack have the chosen type in addition to their other types.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("White Shield Crusader","2","1","W","Protection from black
{W}: White Shield Crusader gains flying until end of turn.
{W}{W}: White Shield Crusader gets +1/+0 until end of turn.","WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plague Fiend","1","1","B","Whenever Plague Fiend deals combat damage to a creature, destroy that creature unless its controller pays {2}.","1B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serene Offering","W","Destroy target enchantment. You gain life equal to its converted mana cost.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nightmare Void","B","Target player reveals his or her hand. You choose a card from it. That player discards that card.
Dredge 2 (If you would draw a card, instead you may put exactly two cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hisoka, Minamo Sensei","1","3","U","{2}{U}, Discard a card: Counter target spell if it has the same converted mana cost as the discarded card.","2UU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scuzzback Marauders","5","2","RG","Trample
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","4(R/G)","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pegasus Stampede","W","Buyback-Sacrifice a land. (You may sacrifice a land in addition to any other costs as you cast this spell. If you do, put this card into your hand as it resolves.)
Put a 1/1 white Pegasus creature token with flying onto the battlefield.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yosei, the Morning Star","5","5","W","Flying
When Yosei, the Morning Star dies, target player skips his or her next untap step. Tap up to five target permanents that player controls.","4WW","Legendary Creature - Dragon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Freed from the Real","U","Enchant creature
{U}: Tap enchanted creature.
{U}: Untap enchanted creature.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu Grain Caravan","2","2","W","When Shu Grain Caravan enters the battlefield, you gain 2 life.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Anchorite","2","3","W","{T}: Prevent the next 2 damage that would be dealt to target creature or player this turn.","2WW","Creature - Elephant Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hypnotic Specter","2","2","B","Flying
Whenever Hypnotic Specter deals damage to an opponent, that player discards a card at random.","1BB","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glissa Sunseeker","3","2","G","First strike
{T}: Destroy target artifact if its converted mana cost is equal to the amount of mana in your mana pool.","2GG","Legendary Creature - Elf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kor Haven","{T}: Add {1} to your mana pool.
{1}{W}, {T}: Prevent all combat damage that would be dealt by target attacking creature this turn.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Radiant's Judgment","W","Destroy target creature with power 4 or greater.
Cycling {2} ({2}, Discard this card: Draw a card.)","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enervate","U","Tap target artifact, creature, or land.
Draw a card at the beginning of the next turn's upkeep.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sky Weaver","2","1","U","{2}: Target white or black creature gains flying until end of turn. (It can't be blocked except by creatures with flying or reach.)","1U","Creature - Metathran Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crimson Acolyte","1","1","W","Protection from red
{W}: Target creature gains protection from red until end of turn.","1W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("An-Havva Inn","G","You gain X plus 1 life, where X is the number of green creatures on the battlefield.","1GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of the Pierced Heart","R","Enchant player
At the beginning of enchanted player's upkeep, Curse of the Pierced Heart deals 1 damage to that player.","1R","Enchantment - Aura Curse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Presence","W","If a source would deal 4 or more damage to a creature or player, that source deals 3 damage to that creature or player instead.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Commander Eesha","2","4","W","Flying, protection from creatures","2WW","Legendary Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blinkmoth Infusion","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Untap all artifacts.","12UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Owl Familiar","1","1","U","Flying
When Owl Familiar enters the battlefield, draw a card, then discard a card.","1U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gitaxian Probe","U","({(u/p)} can be paid with either {U} or 2 life.)
Look at target player's hand.
Draw a card.","(U/P)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mageta the Lion","3","3","W","{2}{W}{W}, {T}, Discard two cards: Destroy all creatures except for Mageta the Lion. Those creatures can't be regenerated.","3WW","Legendary Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lairwatch Giant","5","3","W","Lairwatch Giant can block an additional creature.
Whenever Lairwatch Giant blocks two or more creatures, it gains first strike until end of turn.","5W","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bamboozle","U","Target player reveals the top four cards of his or her library. You choose two of those cards and put them into his or her graveyard. Put the rest on top of his or her library in any order.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Waylay","W","Put three 2/2 white Knight creature tokens onto the battlefield. Exile them at the beginning of the next cleanup step.","2W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Razormane Masticore","5","5","First strike (This creature deals combat damage before creatures without first strike.)
At the beginning of your upkeep, sacrifice Razormane Masticore unless you discard a card.
At the beginning of your draw step, you may have Razormane Masticore deal 3 damage to target creature.","5","Artifact Creature - Masticore");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Onyx Talisman","Whenever a player casts a black spell, you may pay {3}. If you do, untap target permanent.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Final Fortune","R","Take an extra turn after this one. At the beginning of that turn's end step, you lose the game.","RR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Triangle of War","{2}, Sacrifice Triangle of War: Choose target creature you control and target creature an opponent controls. Those creatures fight each other. (Each deals damage equal to its power to the other.)","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harmless Assault","W","Prevent all combat damage that would be dealt this turn by attacking creatures.","2WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Starlight Invoker","1","3","W","{7}{W}: You gain 5 life.","1W","Creature - Human Cleric Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reveka, Wizard Savant","0","1","U","{T}: Reveka, Wizard Savant deals 2 damage to target creature or player and doesn't untap during your next untap step.","2UU","Legendary Creature - Dwarf Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vorinclex, Voice of Hunger","7","6","G","Trample
Whenever you tap a land for mana, add one mana to your mana pool of any type that land produced.
Whenever an opponent taps a land for mana, that land doesn't untap during its controller's next untap step.","6GG","Legendary Creature - Praetor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stronghold Rats","2","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever Stronghold Rats deals combat damage to a player, each player discards a card.","2B","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sudden Strength","G","Target creature gets +3/+3 until end of turn.
Draw a card.","3G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time of Heroes","W","Each creature you control with a level counter on it gets +2/+2.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vampiric Embrace","B","Enchant creature
Enchanted creature gets +2/+2 and has flying.
Whenever a creature dealt damage by enchanted creature this turn dies, put a +1/+1 counter on that creature.","2BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vampiric Touch","B","Vampiric Touch deals 2 damage to target opponent and you gain 2 life.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disaster Radius","R","As an additional cost to cast Disaster Radius, reveal a creature card from your hand.
Disaster Radius deals X damage to each creature your opponents control, where X is the revealed card's converted mana cost.","5RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Roofstalker Wight","2","1","B","{1}{U}: Roofstalker Wight gains flying until end of turn.","1B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stone Catapult","1","2","B","{T}: Destroy target tapped nonblack creature. Activate this ability only during your turn, before attackers are declared.","4B","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bloodstained Mire","{T}, Pay 1 life, Sacrifice Bloodstained Mire: Search your library for a Swamp or Mountain card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodletter","2","3","B","When the names of three or more nonland permanents begin with the same letter, sacrifice Bloodletter. If you do, it deals 2 damage to each creature and each player.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Trespassing Souleater","2","2","{(u/p)}: Trespassing Souleater is unblockable this turn. ({(u/p)} can be paid with either {U} or 2 life.)","3","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forced Fruition","U","Whenever an opponent casts a spell, that player draws seven cards.","4UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Absolver Thrull","2","3","W","Haunt (When this creature dies, exile it haunting target creature.)
When Absolver Thrull enters the battlefield or the creature it haunts dies, destroy target enchantment.","3W","Creature - Thrull Cleric");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Krallenhorde Wantons","7","7","At the beginning of each upkeep, if a player cast two or more spells last turn, transform Krallenhorde Wantons.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Snapping Creeper","2","3","G","Landfall - Whenever a land enters the battlefield under your control, Snapping Creeper gains vigilance until end of turn.","2G","Creature - Plant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Keldon Twilight","BR","At the beginning of each player's end step, if no creatures attacked this turn, that player sacrifices a creature he or she controlled since the beginning of the turn.","1BR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silverclaw Griffin","3","2","W","Flying, first strike","3WW","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sandstorm","G","Sandstorm deals 1 damage to each attacking creature.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lin Sivvi, Defiant Hero","1","3","W","{X}, {T}: Search your library for a Rebel permanent card with converted mana cost X or less and put it onto the battlefield. Then shuffle your library.
{3}: Put target Rebel card from your graveyard on the bottom of your library.","1WW","Legendary Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scalpelexis","1","5","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
Whenever Scalpelexis deals combat damage to a player, that player exiles the top four cards of his or her library. If two or more of those cards have the same name, repeat this process.","4U","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Galvanic Key","Flash
{3}, {T}: Untap target artifact.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hisoka's Guard","1","1","U","You may choose not to untap Hisoka's Guard during your untap step.
{1}{U}, {T}: Target creature you control other than Hisoka's Guard has shroud for as long as Hisoka's Guard remains tapped. (It can't be the target of spells or abilities.)","1U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wave of Terror","B","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of your draw step, destroy each creature with converted mana cost equal to the number of age counters on Wave of Terror. They can't be regenerated.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cadaver Imp","1","1","B","Flying
When Cadaver Imp enters the battlefield, you may return target creature card from your graveyard to your hand.","1BB","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Burn","B","Spend only black and/or red mana on X.
Soul Burn deals X damage to target creature or player. You gain life equal to the damage dealt, but not more than the amount of {B} spent on X, the player's life total before Soul Burn dealt damage, or the creature's toughness.","X2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Husk","2","2","B","Sacrifice a creature: Nantuko Husk gets +2/+2 until end of turn.","2B","Creature - Zombie Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soulbound Guardians","4","5","W","Defender, flying","4W","Creature - Kor Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Memory Sluice","UB","Target player puts the top four cards of his or her library into his or her graveyard.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose a new target for the copy.)","(U/B)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Swarm of Rats","*","1","B","Swarm of Rats's power is equal to the number of Rats you control.","1B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Famished Ghoul","3","2","B","{1}{B}, Sacrifice Famished Ghoul: Exile up to two target cards from a single graveyard.","3B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Overseer","4","4","R","{R}{R}: Target creature gets +1/-1 until end of turn.","2RRR","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Underling","2","1","R","As long as you have seven or more cards in hand, Akki Underling gets +2/+1 and has first strike.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leap of Faith","W","Target creature gains flying until end of turn. Prevent all damage that would be dealt to that creature this turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scryb Ranger","1","1","G","Flash (You may cast this spell any time you could cast an instant.)
Flying, protection from blue
Return a Forest you control to its owner's hand: Untap target creature. Activate this ability only once each turn.","1G","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earthbind","R","Enchant creature
When Earthbind enters the battlefield, if enchanted creature has flying, Earthbind deals 2 damage to that creature and Earthbind gains ""Enchanted creature loses flying.""","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ben-Ben, Akki Hermit","1","1","R","{T}: Ben-Ben, Akki Hermit deals damage to target attacking creature equal to the number of untapped Mountains you control.","2RR","Legendary Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plunge into Darkness","B","Choose one - Sacrifice any number of creatures, then you gain 3 life for each sacrificed creature; or pay X life, then look at the top X cards of your library, put one of those cards into your hand, and exile the rest.
Entwine {B} (Choose both if you pay the entwine cost.)","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elemental Augury","UBR","{3}: Look at the top three cards of target player's library, then put them back in any order.","UBR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ground Rift","R","Target creature without flying can't block this turn.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Snapcaster Mage","2","1","U","Flash
When Snapcaster Mage enters the battlefield, target instant or sorcery card in your graveyard gains flashback until end of turn. The flashback cost is equal to its mana cost. (You may cast that card from your graveyard for its flashback cost. Then exile it.)","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Patchwork Gnomes","2","1","Discard a card: Regenerate Patchwork Gnomes.","3","Artifact Creature - Gnome");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plague of Vermin","B","Starting with you, each player may pay any amount of life. Repeat this process until no one pays life. Each player puts a 1/1 black Rat creature token onto the battlefield for each 1 life he or she paid this way.","6B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merrow Bonegnawer","1","1","B","{T}: Target player exiles a card from his or her graveyard.
Whenever you cast a black spell, you may untap Merrow Bonegnawer.","B","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Snare","W","{W}, Sacrifice Soul Snare: Exile target creature that's attacking you or a planeswalker you control.","W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Saprazzan Skerry","Saprazzan Skerry enters the battlefield tapped with two depletion counters on it.
{T}, Remove a depletion counter from Saprazzan Skerry: Add {U}{U} to your mana pool. If there are no depletion counters on Saprazzan Skerry, sacrifice it.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Eruption","U","Destroy X target Mountains. Volcanic Eruption deals damage to each creature and each player equal to the number of Mountains put into a graveyard this way.","XUUU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Endless Swarm","G","Put a 1/1 green Snake creature token onto the battlefield for each card in your hand.
Epic (For the rest of the game, you can't cast spells. At the beginning of each of your upkeeps, copy this spell except for its epic ability.)","5GGG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spellstutter Sprite","1","1","U","Flash
Flying
When Spellstutter Sprite enters the battlefield, counter target spell with converted mana cost X or less, where X is the number of Faeries you control.","1U","Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sickening Shoal","B","You may exile a black card with converted mana cost X from your hand rather than pay Sickening Shoal's mana cost.
Target creature gets -X/-X until end of turn.","XBB","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coffin Puppets","3","3","B","Sacrifice two lands: Return Coffin Puppets from your graveyard to the battlefield. Activate this ability only during your upkeep and only if you control a Swamp.","3BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Granite","0","7","R","Defender (This creature can't attack.)","2R","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Living Artifact","G","Enchant artifact
Whenever you're dealt damage, put that many vitality counters on Living Artifact.
At the beginning of your upkeep, you may remove a vitality counter from Living Artifact. If you do, you gain 1 life.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Starved Rusalka","1","1","G","{G}, Sacrifice a creature: You gain 1 life.","G","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Paupers' Cage","At the beginning of each opponent's upkeep, if that player has two or fewer cards in hand, Paupers' Cage deals 2 damage to him or her.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Guardian Idol","Guardian Idol enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{2}: Guardian Idol becomes a 2/2 Golem artifact creature until end of turn.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spinerock Knoll","Hideaway (This land enters the battlefield tapped. When it does, look at the top four cards of your library, exile one face down, then put the rest on the bottom of your library.)
{T}: Add {R} to your mana pool.
{R}, {T}: You may play the exiled card without paying its mana cost if an opponent was dealt 7 or more damage this turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soulless Revival","B","Return target creature card from your graveyard to your hand.
Splice onto Arcane {1}{B} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1B","Instant - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Myr Turbine","{T}: Put a 1/1 colorless Myr artifact creature token onto the battlefield.
{T}, Tap five untapped Myr you control: Search your library for a Myr creature card, put it onto the battlefield, then shuffle your library.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tendrils of Agony","B","Target player loses 2 life and you gain 2 life.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kindle","R","Kindle deals X damage to target creature or player, where X is 2 plus the number of cards named Kindle in all graveyards.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vacuumelt","U","Replicate {2}{U} (When you cast this spell, copy it for each time you paid its replicate cost. You may choose new targets for the copies.)
Return target creature to its owner's hand.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Induce Despair","B","As an additional cost to cast Induce Despair, reveal a creature card from your hand.
Target creature gets -X/-X until end of turn, where X is the revealed card's converted mana cost.","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Furnace Spirit","1","1","R","Haste
{R}: Furnace Spirit gets +1/+0 until end of turn.","2R","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disenchant","W","Destroy target artifact or enchantment.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avacyn's Pilgrim","1","1","G","{T}: Add {W} to your mana pool.","G","Creature - Human Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gleeful Sabotage","G","Destroy target artifact or enchantment.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose a new target for the copy.)","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Predator","2","2","G","Trample
Whenever an opponent gains life, put that many +1/+1 counters on Kavu Predator.","1G","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tyrranax","5","4","G","{1}{G}: Tyrranax gets -1/+1 until end of turn.","4GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Pathcutter","1","2","G","{2}{G}: Target Elf creature gains forestwalk until end of turn.","3G","Creature - Elf Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Spider","G","When an opponent casts a creature spell with flying, if Hidden Spider is an enchantment, Hidden Spider becomes a 3/5 Spider creature with reach. (It can block creatures with flying.)","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gnathosaur","5","4","R","Sacrifice an artifact: Gnathosaur gains trample until end of turn.","4RR","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Longhorn Firebeast","3","2","R","When Longhorn Firebeast enters the battlefield, any opponent may have it deal 5 damage to him or her. If a player does, sacrifice Longhorn Firebeast.","2R","Creature - Elemental Ox Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devout Witness","2","2","W","{1}{W}, {T}, Discard a card: Destroy target artifact or enchantment.","2W","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Copper Gnomes","1","1","{4}, Sacrifice Copper Gnomes: You may put an artifact card from your hand onto the battlefield.","2","Artifact Creature - Gnome");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Pits of Rath","B","Whenever a creature is dealt damage, destroy it. It can't be regenerated.","3BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mysteries of the Deep","U","Draw two cards.
Landfall - If you had a land enter the battlefield under your control this turn, draw three cards instead.","4U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grappler Spider","2","1","G","Reach (This creature can block creatures with flying.)","1G","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Crab","1","3","WU","Vigilance
When Riptide Crab dies, draw a card.","1WU","Creature - Crab");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master Transmuter","1","2","U","{U}, {T}, Return an artifact you control to its owner's hand: You may put an artifact card from your hand onto the battlefield.","3U","Artifact Creature - Human Artificer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Morinfen Avatar","At the beginning of your upkeep, you lose 1 life for each permanent you control.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fangren Hunter","4","4","G","Trample","3GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Paradigm Shift","U","Exile all cards from your library. Then shuffle your graveyard into your library.","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seasoned Marshal","2","2","W","Whenever Seasoned Marshal attacks, you may tap target creature.","2WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Titania's Boon","G","Put a +1/+1 counter on each creature you control.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Peregrine Drake","2","3","U","Flying
When Peregrine Drake enters the battlefield, untap up to five lands.","4U","Creature - Drake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Temple of the False God","{T}: Add {2} to your mana pool. Activate this ability only if you control five or more lands.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Buoyancy","U","Flash
Enchant creature
Enchanted creature has flying.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overmaster","R","The next instant or sorcery spell you cast this turn can't be countered by spells or abilities.
Draw a card.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nim Shambler","2","1","B","Nim Shambler gets +1/+0 for each artifact you control.
Sacrifice a creature: Regenerate Nim Shambler.","2BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inquisition of Kozilek","B","Target player reveals his or her hand. You choose a nonland card from it with converted mana cost 3 or less. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Will-o'-the-Wisp","0","1","B","Flying (This creature can't be blocked except by creatures with flying or reach.)
{B}: Regenerate Will-o'-the-Wisp. (The next time this creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talruum Minotaur","3","3","R","Haste","2RR","Creature - Minotaur Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ydwen Efreet","3","6","R","Whenever Ydwen Efreet blocks, flip a coin. If you lose the flip, remove Ydwen Efreet from combat and it can't block this turn. Creatures it was blocking that had become blocked by only Ydwen Efreet this combat become unblocked.","RRR","Creature - Efreet");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Haunted Guardian","2","1","Defender, first strike","2","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wall of Junk","0","7","Defender (This creature can't attack.)
Whenever Wall of Junk blocks, return it to its owner's hand at end of combat. (Return it only if it's on the battlefield.)","2","Artifact Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Taste of Blood","B","Taste of Blood deals 1 damage to target player and you gain 1 life.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blossoming Wreath","G","You gain life equal to the number of creature cards in your graveyard.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surging Dementia","B","Ripple 4 (When you cast this spell, you may reveal the top four cards of your library. You may cast any revealed cards with the same name as this spell without paying their mana costs. Put the rest on the bottom of your library.)
Target player discards a card.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Woodland Guidance","G","Return target card from your graveyard to your hand. Clash with an opponent. If you win, untap all Forests you control. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)
Exile Woodland Guidance.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quirion Druid","1","2","G","{G}, {T}: Target land becomes a 2/2 green creature that's still a land. (This effect lasts indefinitely.)","2G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Avian","0","4","Flying
Clockwork Avian enters the battlefield with four +1/+0 counters on it.
At end of combat, if Clockwork Avian attacked or blocked this combat, remove a +1/+0 counter from it.
{X}, {T}: Put up to X +1/+0 counters on Clockwork Avian. This ability can't cause the total number of +1/+0 counters on Clockwork Avian to be greater than four. Activate this ability only during your upkeep.","5","Artifact Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ebony Horse","{2}, {T}: Untap target attacking creature you control. Prevent all combat damage that would be dealt to and dealt by that creature this turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Medicine Runner","2","1","WG","When Medicine Runner enters the battlefield, you may remove a counter from target permanent.","1(G/W)","Creature - Elf Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forced March","B","Destroy all creatures with converted mana cost X or less.","XBBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chaos Lord","7","7","R","First strike
At the beginning of your upkeep, target opponent gains control of Chaos Lord if the number of permanents is even.
Chaos Lord can attack as though it had haste unless it entered the battlefield this turn.","4RRR","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Solfatara","R","Target player can't play land cards this turn.
Draw a card at the beginning of the next turn's upkeep.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Red Scarab","W","Enchant creature
Enchanted creature can't be blocked by red creatures.
Enchanted creature gets +2/+2 as long as an opponent controls a red permanent.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voiceless Spirit","2","1","W","Flying, first strike","2W","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sculpting Steel","You may have Sculpting Steel enter the battlefield as a copy of any artifact on the battlefield.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Riptide Laboratory","{T}: Add {1} to your mana pool.
{1}{U}, {T}: Return target Wizard you control to its owner's hand.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Storm","R","Lightning Storm deals X damage to target creature or player, where X is 3 plus the number of charge counters on it.
Discard a land card: Put two charge counters on Lightning Storm. You may choose a new target for it. Any player may activate this ability but only if Lightning Storm is on the stack.","1RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elemental Mastery","R","Enchant creature
Enchanted creature has ""{T}: Put X 1/1 red Elemental creature tokens with haste onto the battlefield, where X is this creature's power. Exile them at the beginning of the next end step.""","3R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rewards of Diversity","W","Whenever an opponent casts a multicolored spell, you gain 4 life.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mudhole","R","Target player exiles all land cards from his or her graveyard.","2R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Moonglove Extract","Sacrifice Moonglove Extract: Moonglove Extract deals 2 damage to target creature or player.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grove of the Burnwillows","{T}: Add {1} to your mana pool.
{T}: Add {R} or {G} to your mana pool. Each opponent gains 1 life.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tolarian Serpent","7","7","U","At the beginning of your upkeep, put the top seven cards of your library into your graveyard.","5UU","Creature - Serpent");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Flare","R","Whenever a player taps a land for mana, that player adds one mana to his or her mana pool of any type that land produced.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skizzik","5","3","R","Kicker {R} (You may pay an additional {R} as you cast this spell.)
Trample, haste
At the beginning of the end step, sacrifice Skizzik unless it was kicked.","3R","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stirring Wildwood","Stirring Wildwood enters the battlefield tapped.
{T}: Add {G} or {W} to your mana pool.
{1}{G}{W}: Until end of turn, Stirring Wildwood becomes a 3/4 green and white Elemental creature with reach. It's still a land.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Null Brooch","{2}, {T}, Discard your hand: Counter target noncreature spell.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psionic Gift","U","Enchant creature
Enchanted creature has ""{T}: This creature deals 1 damage to target creature or player.""","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Driver of the Dead","3","2","B","When Driver of the Dead dies, return target creature card with converted mana cost 2 or less from your graveyard to the battlefield.","3B","Creature - Vampire");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Book of Rass","{2}, Pay 2 life: Draw a card.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kindled Fury","R","Target creature gets +1/+0 and gains first strike until end of turn. (It deals combat damage before creatures without first strike.)","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Surge","R","Lightning Surge deals 4 damage to target creature or player.
Threshold - If seven or more cards are in your graveyard, instead Lightning Surge deals 6 damage to that creature or player and the damage can't be prevented.
Flashback {5}{R}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Conscripts","2","2","R","Mogg Conscripts can't attack unless you've cast a creature spell this turn.","R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Godtoucher","2","2","G","{1}{W}, {T}: Prevent all damage that would be dealt to target creature with power 5 or greater this turn.","3G","Creature - Elf Cleric");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Condor","0","0","Flying
Clockwork Condor enters the battlefield with three +1/+1 counters on it.
Whenever Clockwork Condor attacks or blocks, remove a +1/+1 counter from it at end of combat.","4","Artifact Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inner-Flame Acolyte","2","2","R","When Inner-Flame Acolyte enters the battlefield, target creature gets +2/+0 and gains haste until end of turn.
Evoke {R} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","1RR","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chromeshell Crab","3","3","U","Morph {4}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Chromeshell Crab is turned face up, you may exchange control of target creature you control and target creature an opponent controls.","4U","Creature - Crab Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hag Hedge-Mage","2","2","BG","When Hag Hedge-Mage enters the battlefield, if you control two or more Swamps, you may have target player discard a card.
When Hag Hedge-Mage enters the battlefield, if you control two or more Forests, you may put target card from your graveyard on top of your library.","2(B/G)","Creature - Hag Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greater Gargadon","9","7","R","Suspend 10-{R} (Rather than cast this card from your hand, you may pay {R} and exile it with ten time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Sacrifice an artifact, creature, or land: Remove a time counter from Greater Gargadon. Activate this ability only if Greater Gargadon is suspended.","9R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thrill of the Hunt","G","Target creature gets +1/+2 until end of turn.
Flashback {W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Halimar Wavewatch","0","3","U","Level up {2} ({2}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-4
0/6
LEVEL 5+
6/6
Islandwalk","1U","Creature - Merfolk Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grifter's Blade","Flash
As Grifter's Blade enters the battlefield, choose a creature you control it could be attached to. If you do, it enters the battlefield attached to that creature.
Equipped creature gets +1/+1.
Equip {1}","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nighthaze","B","Target creature gains swampwalk until end of turn.
Draw a card.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unnerve","B","Each opponent discards two cards.","3B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call of the Wild","G","{2}{G}{G}: Reveal the top card of your library. If it's a creature card, put it onto the battlefield. Otherwise, put it into your graveyard.","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Purity","6","6","W","Flying
If noncombat damage would be dealt to you, prevent that damage. You gain life equal to the damage prevented this way.
When Purity is put into a graveyard from anywhere, shuffle it into its owner's library.","3WWW","Creature - Elemental Incarnation");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flowstone Slide","R","All creatures get +X/-X until end of turn.","X2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flash of Defiance","R","Green creatures and white creatures can't block this turn.
Flashback-{1}{R}, Pay 3 life. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shrieking Mogg","1","1","R","Haste
When Shrieking Mogg enters the battlefield, tap all other creatures.","1R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dash Hopes","B","When you cast Dash Hopes, any player may pay 5 life. If a player does, counter Dash Hopes.
Counter target spell.","BB","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Needlebug","2","2","Flash
Protection from artifacts","4","Artifact Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Granite Shard","{3}, {T} or {R}, {T}: Granite Shard deals 1 damage to target creature or player.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancestral Mask","G","Enchant creature
Enchanted creature gets +2/+2 for each other enchantment on the battlefield.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Warchief","2","2","R","Goblin spells you cast cost {1} less to cast.
Goblin creatures you control have haste.","1RR","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rendclaw Trow","2","2","BG","Wither (This deals damage to creatures in the form of -1/-1 counters.)
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","2(B/G)","Creature - Troll");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lunge","R","Lunge deals 2 damage to target creature and 2 damage to target player.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prophetic Bolt","UR","Prophetic Bolt deals 4 damage to target creature or player. Look at the top four cards of your library. Put one of those cards into your hand and the rest on the bottom of your library in any order.","3UR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pinecrest Ridge","{T}: Add {1} to your mana pool.
{T}: Add {R} or {G} to your mana pool. Pinecrest Ridge doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kelinore Bat","2","1","B","Flying","2B","Creature - Bat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Affliction","B","As Circle of Affliction enters the battlefield, choose a color.
Whenever a source of the chosen color deals damage to you, you may pay {1}. If you do, target player loses 1 life and you gain 1 life.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heartwood Treefolk","3","4","G","Forestwalk","2GG","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tin Street Hooligan","2","1","R","When Tin Street Hooligan enters the battlefield, if {G} was spent to cast Tin Street Hooligan, destroy target artifact.","1R","Creature - Goblin Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arcanum Wings","U","Enchant creature
Enchanted creature has flying.
Aura swap {2}{U} ({2}{U}: Exchange this Aura with an Aura card in your hand.)","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Hero","2","2","R",NULL,"2R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mammoth Umbra","W","Enchant creature
Enchanted creature gets +3/+3 and has vigilance.
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","4W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squall","G","Squall deals 2 damage to each creature with flying.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Descendant of Masumaro","1","1","G","At the beginning of your upkeep, put a +1/+1 counter on Descendant of Masumaro for each card in your hand, then remove a +1/+1 counter from Descendant of Masumaro for each card in target opponent's hand.","2G","Creature - Human Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Resize","G","Target creature gets +3/+3 until end of turn.
Recover {1}{G} (When a creature is put into your graveyard from the battlefield, you may pay {1}{G}. If you do, return this card from your graveyard to your hand. Otherwise, exile this card.)","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Myr Matrix","Myr Matrix is indestructible. (""Destroy"" effects and lethal damage don't destroy it.)
Myr creatures get +1/+1.
{5}: Put a 1/1 colorless Myr artifact creature token onto the battlefield.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flamestick Courier","2","1","R","You may choose not to untap Flamestick Courier during your untap step.
{2}{R}, {T}: Target Goblin creature gets +2/+2 and has haste for as long as Flamestick Courier remains tapped.","2R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bird Maiden","1","2","R","Flying","2R","Creature - Human Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidespout Tyrant","5","5","U","Flying
Whenever you cast a spell, return target permanent to its owner's hand.","5UUU","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barbed Field","R","Enchant land
Enchanted land has ""{T}: This land deals 1 damage to target creature or player.""","2RR","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Donate","U","Target player gains control of target permanent you control.","2U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Iron Star","Whenever a player casts a red spell, you may pay {1}. If you do, you gain 1 life.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Castigate","WB","Target opponent reveals his or her hand. You choose a nonland card from it and exile that card.","WB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crosswinds","G","Creatures with flying get -2/-0.","1G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scepter of Empires","{T}: Scepter of Empires deals 1 damage to target player. It deals 3 damage to that player instead if you control artifacts named Crown of Empires and Throne of Empires.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Make a Wish","G","Return two cards at random from your graveyard to your hand.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fatal Attraction","R","Enchant creature
When Fatal Attraction enters the battlefield, it deals 2 damage to enchanted creature.
At the beginning of your upkeep, Fatal Attraction deals 4 damage to enchanted creature.","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keldon Firebombers","3","3","R","When Keldon Firebombers enters the battlefield, each player sacrifices all lands he or she controls except for three.","3RR","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Farmstead","W","Enchant land
Enchanted land has ""At the beginning of your upkeep, you may pay {W}{W}. If you do, you gain 1 life.""","WWW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Slayer","2","2","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
Dauthi Slayer attacks each turn if able.","BB","Creature - Dauthi Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Niblis of the Mist","2","1","W","Flying
When Niblis of the Mist enters the battlefield, you may tap target creature.","2W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seraph of Dawn","2","4","W","Flying
Lifelink (Damage dealt by this creature also causes you to gain that much life.)","2WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stone Kavu","3","3","G","{R}: Stone Kavu gets +1/+0 until end of turn.
{W}: Stone Kavu gets +0/+1 until end of turn.","4G","Creature - Kavu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lobotomy","UB","Target player reveals his or her hand, then you choose a card other than a basic land card from it. Search that player's graveyard, hand, and library for all cards with the same name as the chosen card and exile them. Then that player shuffles his or her library.","2UB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Agonizing Memories","B","Look at target player's hand and choose two cards from it. Put them on top of that player's library in any order.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Tezzeret, Agent of Bolas","UB","+1: Look at the top five cards of your library. You may reveal an artifact card from among them and put it into your hand. Put the rest on the bottom of your library in any order.
-1: Target artifact becomes a 5/5 artifact creature.
-4: Target player loses X life and you gain X life, where X is twice the number of artifacts you control.","Planeswalker - Tezzeret","2UB");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vectis Silencers","1","2","U","{2}{B}: Vectis Silencers gains deathtouch until end of turn. (Any amount of damage it deals to a creature is enough to destroy that creature.)","2U","Artifact Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Obsidian Battle-Axe","Equipped creature gets +2/+1 and has haste.
Whenever a Warrior creature enters the battlefield, you may attach Obsidian Battle-Axe to it.
Equip {3}","3","Tribal Artifact - Warrior Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Violent Outburst","RG","Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)
Creatures you control get +1/+0 until end of turn.","1RG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hecatomb","B","When Hecatomb enters the battlefield, sacrifice Hecatomb unless you sacrifice four creatures.
Tap an untapped Swamp you control: Hecatomb deals 1 damage to target creature or player.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Infantry","1","3","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cannibalize","B","Choose two target creatures controlled by the same player. Exile one of those creatures and put two +1/+1 counters on the other.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Copy Enchantment","U","You may have Copy Enchantment enter the battlefield as a copy of any enchantment on the battlefield.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crusading Knight","2","2","W","Protection from black
Crusading Knight gets +1/+1 for each Swamp your opponents control.","2WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tourach's Chant","B","At the beginning of your upkeep, sacrifice Tourach's Chant unless you pay {B}.
Whenever a player puts a Forest onto the battlefield, Tourach's Chant deals 3 damage to that player unless he or she puts a -1/-1 counter on a creature he or she controls.","1BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Into the Core","R","Exile two target artifacts.","2RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Capsize","U","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Return target permanent to its owner's hand.","1UU","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Dragon","0","0","Flying
Clockwork Dragon enters the battlefield with six +1/+1 counters on it.
Whenever Clockwork Dragon attacks or blocks, remove a +1/+1 counter from it at end of combat.
{3}: Put a +1/+1 counter on Clockwork Dragon.","7","Artifact Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vithian Renegades","3","2","RG","When Vithian Renegades enters the battlefield, destroy target artifact.","1RG","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scavenging Ghoul","2","2","B","At the beginning of each end step, put a corpse counter on Scavenging Ghoul for each creature that died this turn.
Remove a corpse counter from Scavenging Ghoul: Regenerate Scavenging Ghoul.","3B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Bronze Bombshell","4","1","When a player other than Bronze Bombshell's owner controls it, that player sacrifices it. If the player does, Bronze Bombshell deals 7 damage to him or her.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serpent Warrior","3","3","B","When Serpent Warrior enters the battlefield, you lose 3 life.","2B","Creature - Snake Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Geist Snatch","U","Counter target creature spell. Put a 1/1 blue Spirit creature token with flying onto the battlefield.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barrin, Master Wizard","1","1","U","{2}, Sacrifice a permanent: Return target creature to its owner's hand.","1UU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Life and Limb","G","All Forests and all Saprolings are 1/1 green Saproling creatures and Forest lands in addition to their other types.","3G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Strings","B","Return two target creature cards from your graveyard to your hand unless any player pays {X}.","XB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goatnapper","2","2","R","When Goatnapper enters the battlefield, untap target Goat and gain control of it until end of turn. It gains haste until end of turn.","2R","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hornet Queen","2","2","G","Flying, deathtouch
When Hornet Queen enters the battlefield, put four 1/1 green Insect creature tokens with flying and deathtouch onto the battlefield.","4GGG","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Purraj of Urborg","2","3","B","Purraj of Urborg has first strike as long as it's attacking.
Whenever a player casts a black spell, you may pay {B}. If you do, put a +1/+1 counter on Purraj of Urborg.","3BB","Legendary Creature - Cat Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boom // Bust","R","Destroy target land you control and target land you don't control.
---
Destroy all lands.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flamekin Bladewhirl","2","1","R","As an additional cost to cast Flamekin Bladewhirl, reveal an Elemental card from your hand or pay {3}.","R","Creature - Elemental Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stoneshaker Shaman","1","1","R","At the beginning of each player's end step, that player sacrifices an untapped land.","2R","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("War's Toll","R","Whenever an opponent taps a land for mana, tap all lands that player controls.
If a creature an opponent controls attacks, all creatures that opponent controls attack if able.","3R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cabal Pit","{T}: Add {B} to your mana pool. Cabal Pit deals 1 damage to you.
Threshold - {B}, {T}, Sacrifice Cabal Pit: Target creature gets -2/-2 until end of turn. Activate this ability only if seven or more cards are in your graveyard.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Grid Monitor","4","6","You can't cast creature spells.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orzhov Guildmage","2","2","WB","{2}{W}: Target player gains 1 life.
{2}{B}: Each player loses 1 life.","(W/B)(W/B)","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Channel the Suns","G","Add {W}{U}{B}{R}{G} to your mana pool.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Walking Sponge","1","1","U","{T}: Target creature loses your choice of flying, first strike, or trample until end of turn.","1U","Creature - Sponge");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Part the Veil","U","Return all creatures you control to their owner's hand.","3U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ascending Aven","3","2","U","Flying
Ascending Aven can block only creatures with flying.
Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2UU","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jumbo Imp","0","0","B","Flying
When you play Jumbo Imp, roll a six-sided die. Jumbo Imp comes into play with a number of +1/+1 counters on it equal to the die roll.
During your upkeep, roll a six-sided die and put on Jumbo Imp a number of +1/+1 counters equal to the die roll.
At the end of your turn, roll a six-sided die and remove from Jumbo Imp a number of +1/+1 counters equal to the die roll.","2B","Summon - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Honorable Scout","1","1","W","When Honorable Scout enters the battlefield, you gain 2 life for each black and/or red creature target opponent controls.","W","Creature - Human Soldier Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("War Tax","U","{X}{U}: This turn, creatures can't attack unless their controller pays {X} for each attacking creature he or she controls.","2U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spidersilk Net","Equipped creature gets +0/+2 and has reach. (It can block creatures with flying.)
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","0","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cosi's Trickster","1","1","U","Whenever an opponent shuffles his or her library, you may put a +1/+1 counter on Cosi's Trickster.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yawgmoth's Will","B","Until end of turn, you may play cards from your graveyard.
If a card would be put into your graveyard from anywhere this turn, exile that card instead.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talas Air Ship","3","2","U","Flying","3U","Creature - Human Pirate");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Torrent of Fire","R","Torrent of Fire deals damage equal to the highest converted mana cost among permanents you control to target creature or player.","3RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zombify","B","Return target creature card from your graveyard to the battlefield.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gigapede","6","1","G","Shroud (This permanent can't be the target of spells or abilities.)
At the beginning of your upkeep, if Gigapede is in your graveyard, you may discard a card. If you do, return Gigapede to your hand.","3GG","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of the Nine Gales","1","2","U","Flying
{T}, Tap two untapped Birds you control: Return target permanent to its owner's hand.","2U","Creature - Bird Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mountain Stronghold","Red legendary creatures you control have ""bands with other legendary creatures."" (Any legendary creatures can attack in a band as long as at least one has ""bands with other legendary creatures."" Bands are blocked as a group. If at least two legendary creatures you control, one of which has ""bands with other legendary creatures,"" are blocking or being blocked by the same creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deep-Sea Serpent","5","5","U","Deep-Sea Serpent can't attack unless defending player controls an Island.","4UU","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightning Elemental","4","1","R","Haste (This creature can attack and {T} as soon as it comes under your control.)","3R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Festival","W","Cast Festival only during an opponent's upkeep.
Creatures can't attack this turn.","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Plains","W",NULL,"Basic Land - Plains");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Caves","R","Enchant land
As long as enchanted land is a basic Mountain, Goblin creatures get +0/+2.","1RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kasimir the Lone Wolf","5","3","WU",NULL,"4WU","Legendary Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fathom Seer","1","3","U","Morph-Return two Islands you control to their owner's hand. (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Fathom Seer is turned face up, draw two cards.","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lifegift","G","Whenever a land enters the battlefield, you may gain 1 life.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Simic Sky Swallower","6","6","UG","Flying, trample
Shroud (This creature can't be the target of spells or abilities.)","5GU","Creature - Leviathan");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shard Phoenix","2","2","R","Flying (This creature can't be blocked except by creatures with flying or reach.)
Sacrifice Shard Phoenix: Shard Phoenix deals 2 damage to each creature without flying.
{R}{R}{R}: Return Shard Phoenix from your graveyard to your hand. Activate this ability only during your upkeep.","4R","Creature - Phoenix");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brain Weevil","1","1","B","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)
Sacrifice Brain Weevil: Target player discards two cards. Activate this ability only any time you could cast a sorcery.","3B","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Paladin en-Vec","2","2","W","First strike, protection from black and from red (This creature deals combat damage before creatures without first strike. It can't be blocked, targeted, dealt damage, or enchanted by anything black or red.)","1WW","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mana Cylix","{1}, {T}: Add one mana of any color to your mana pool.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spellshock","R","Whenever a player casts a spell, Spellshock deals 2 damage to that player.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elite Vanguard","2","1","W",NULL,"W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Touch of Invisibility","U","Target creature is unblockable this turn.
Draw a card.","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Face to Face","R","You and target opponent play a best two-out-of-three Rock, Paper, Scissors match. If you win, Face to Face deals 5 damage to that opponent.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curtain of Light","W","Cast Curtain of Light only during combat after blockers are declared.
Target unblocked attacking creature becomes blocked. (This spell works on unblockable creatures.)
Draw a card.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wolfbriar Elemental","4","4","G","Multikicker {G} (You may pay an additional {G} any number of times as you cast this spell.)
When Wolfbriar Elemental enters the battlefield, put a 2/2 green Wolf creature token onto the battlefield for each time it was kicked.","2GG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gristleback","2","2","G","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
Sacrifice Gristleback: You gain life equal to Gristleback's power.","2G","Creature - Boar Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balshan Griffin","3","2","U","Flying
{1}{U}, Discard a card: Return Balshan Griffin to its owner's hand.","3UU","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shrivel","B","All creatures get -1/-1 until end of turn.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primeval Shambler","3","3","B","{B}: Primeval Shambler gets +1/+1 until end of turn.","4B","Creature - Horror Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Faerie Trickery","U","Counter target non-Faerie spell. If that spell is countered this way, exile it instead of putting it into its owner's graveyard.","1UU","Tribal Instant - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skulking Knight","3","3","B","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
When Skulking Knight becomes the target of a spell or ability, sacrifice it.","2B","Creature - Zombie Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necrotic Plague","B","Enchant creature
Enchanted creature has ""At the beginning of your upkeep, sacrifice this creature.""
When enchanted creature dies, its controller chooses target creature one of his or her opponents controls. Return Necrotic Plague from its owner's graveyard to the battlefield attached to that creature.","2BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Signal Pest","0","1","Battle cry (Whenever this creature attacks, each other attacking creature gets +1/+0 until end of turn.)
Signal Pest can't be blocked except by creatures with flying or reach.","1","Artifact Creature - Pest");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anurid Scavenger","3","3","G","Protection from black
At the beginning of your upkeep, sacrifice Anurid Scavenger unless you put a card from your graveyard on the bottom of your library.","2G","Creature - Frog Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crushing Vines","G","Choose one - Destroy target creature with flying; or destroy target artifact.","2G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Tower","{T}: Add {1} to your mana pool.
{T}, Sacrifice a creature: Add {B}{B} to your mana pool.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Knives","B","Target opponent discards a card at random.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skizzik Surger","6","4","R","Haste
Echo-Sacrifice two lands. (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","4RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Razorfield Thresher","6","4",NULL,"7","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chilling Shade","1","1","B","Flying
{S}i}: Chilling Shade gets +1/+1 until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","2B","Snow Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sprouting Thrinax","3","3","BRG","When Sprouting Thrinax dies, put three 1/1 green Saproling creature tokens onto the battlefield.","BRG","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Groundbreaker","6","1","G","Trample, haste
At the beginning of the end step, sacrifice Groundbreaker.","GGG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Richard Garfield, Ph.D.","2","2","U","You may play cards as though they were other Magic cards of your choice with the same mana cost. (Mana cost includes color.) You can't choose the same card twice.","3UU","Legendary Creature - Human Designer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dwarven Song","R","Any number of target creatures become red until end of turn.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Omen","U","Look at the top three cards of your library, then put them back in any order. You may shuffle your library.
Draw a card.","1U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sprout","G","Put a 1/1 green Saproling creature token onto the battlefield.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chain of Vapor","U","Return target nonland permanent to its owner's hand. Then that permanent's controller may sacrifice a land. If the player does, he or she may copy this spell and may choose a new target for that copy.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Comet Storm","R","Multikicker {1} (You may pay an additional {1} any number of times as you cast this spell.)
Choose target creature or player, then choose another target creature or player for each time Comet Storm was kicked. Comet Storm deals X damage to each of them.","XRR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vernal Equinox","G","Any player may cast creature and enchantment cards as though they had flash.","3G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sabretooth Tiger","2","1","R","First strike","2R","Creature - Cat");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Kuldotha Forgemaster","3","5","{T}, Sacrifice three artifacts: Search your library for an artifact card and put it onto the battlefield. Then shuffle your library.","5","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bonethorn Valesk","4","2","R","Whenever a permanent is turned face up, Bonethorn Valesk deals 1 damage to target creature or player.","4R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight Exemplar","2","2","W","First strike (This creature deals combat damage before creatures without first strike.)
Other Knight creatures you control get +1/+1 and are indestructible. (Lethal damage and effects that say ""destroy"" don't destroy them.)","1WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Mycosynth Golem","4","5","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Artifact creature spells you cast have affinity for artifacts. (They cost {1} less to cast for each artifact you control.)","11","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flame Jab","R","Flame Jab deals 1 damage to target creature or player.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ronom Unicorn","2","2","W","Sacrifice Ronom Unicorn: Destroy target enchantment.","1W","Creature - Unicorn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teroh's Vanguard","2","3","W","Flash
Threshold - As long as seven or more cards are in your graveyard, Teroh's Vanguard has ""When Teroh's Vanguard enters the battlefield, creatures you control gain protection from black until end of turn.""","3W","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Syphon Life","B","Target player loses 2 life and you gain 2 life.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream Salvage","UB","Draw cards equal to the number of cards target opponent discarded this turn.","(U/B)","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Panic Spellbomb","{T}, Sacrifice Panic Spellbomb: Target creature can't block this turn.
When Panic Spellbomb is put into a graveyard from the battlefield, you may pay {R}. If you do, draw a card.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Raksha Golden Cub Avatar","Creatures you control get +0/+1.
Equipped creatures you control get +1/+0 and have first strike.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flaring Pain","R","Damage can't be prevented this turn.
Flashback {R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kuon, Ogre Ascendant","2","4","B","At the beginning of the end step, if three or more creatures died this turn, flip Kuon, Ogre Ascendant.
---
At the beginning of each player's upkeep, that player sacrifices a creature.","BBB","Legendary Creature - Ogre Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skittering Horror","4","3","B","When you cast a creature spell, sacrifice Skittering Horror.","2B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stingscourger","2","2","R","Echo {3}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Stingscourger enters the battlefield, return target creature an opponent controls to its owner's hand.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Know Naught but Fire","When you set this scheme in motion, it deals damage to each opponent equal to the number of cards in that player's hand.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earthlore","G","Enchant land you control
Tap enchanted land: Target blocking creature gets +1/+2 until end of turn. Activate this ability only if enchanted land is untapped.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kyren Glider","1","1","R","Flying
Kyren Glider can't block.","1R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vow of Lightning","R","Enchant creature
Enchanted creature gets +2/+2, has first strike, and can't attack you or a planeswalker you control.","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Zephyrnaut","2","2","W","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Kithkin Zephyrnaut, you may reveal it. If you do, Kithkin Zephyrnaut gets +2/+2 and gains flying and vigilance until end of turn.","2W","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Homicidal Seclusion","B","As long as you control exactly one creature, that creature gets +3/+1 and has lifelink.","4B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Arbiter","2","2","W","Players can't search libraries. Any player may pay {2} for that player to ignore this effect until end of turn.","1W","Creature - Cat Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Paradise Mantle","Equipped creature has ""{T}: Add one mana of any color to your mana pool.""
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","0","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boon Reflection","W","If you would gain life, you gain twice that much life instead.","4W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel's Herald","1","1","W","{2}{W}, {T}, Sacrifice a green creature, a white creature, and a blue creature: Search your library for a card named Empyrial Archangel and put it onto the battlefield. Then shuffle your library.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scorned Villager","1","1","G","{T}: Add {G} to your mana pool.
At the beginning of each upkeep, if no spells were cast last turn, transform Scorned Villager.","1G","Creature - Human Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Farewell to Arms","B","As Farewell to Arms comes into play, choose a hand attached to an opponent's arm.
When the chosen hand isn't behind its owner's back, sacrifice Farewell to Arms. If you do, that player discards his or her hand . . . of cards. (The lawyers wouldn't let us do it the other way.)","1BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Evasive Action","U","Domain - Counter target spell unless its controller pays {1} for each basic land type among lands you control.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Air","1","5","U","Defender, flying (This creature can't attack, and it can block creatures with flying.)","1UU","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Drowned Catacomb","Drowned Catacomb enters the battlefield tapped unless you control an Island or a Swamp.
{T}: Add {U} or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Windfall","U","Each player discards his or her hand, then draws cards equal to the greatest number of cards a player discarded this way.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wormwood Treefolk","4","4","G","{G}{G}: Wormwood Treefolk gains forestwalk until end of turn and deals 2 damage to you.
{B}{B}: Wormwood Treefolk gains swampwalk until end of turn and deals 2 damage to you.","3GG","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Truth or Tale","U","Reveal the top five cards of your library and separate them into two piles. An opponent chooses one of those piles. Put a card from the chosen pile into your hand, then put all other cards revealed this way on the bottom of your library in any order.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Ox","3","3","G","Swampwalk","3G","Creature - Ox");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Subterranean Spirit","3","3","R","Protection from red
{T}: Subterranean Spirit deals 1 damage to each creature without flying.","3RR","Creature - Elemental Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nomadic Elf","2","2","G","{1}{G}: Add one mana of any color to your mana pool.","1G","Creature - Elf Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Darkblast","B","Target creature gets -1/-1 until end of turn.
Dredge 3 (If you would draw a card, instead you may put exactly three cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blessing","W","Enchant creature
{W}: Enchanted creature gets +1/+1 until end of turn.","WW","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Horizon Canopy","{T}, Pay 1 life: Add {G} or {W} to your mana pool.
{1}, {T}, Sacrifice Horizon Canopy: Draw a card.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glarecaster","3","3","W","Flying
{5}{W}: The next time damage would be dealt to Glarecaster and/or you this turn, that damage is dealt to target creature or player instead.","4WW","Creature - Bird Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Protective Sphere","W","{1}, Pay 1 life: Prevent all damage that would be dealt to you this turn by a source of your choice that shares a color with the mana spent on this activation cost. (Colorless mana prevents no damage.)","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hindering Touch","U","Counter target spell unless its controller pays {2}.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","3U","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clay Statue","3","1","{2}: Regenerate Clay Statue.","4","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistral Charger","2","1","W","Flying","1W","Creature - Pegasus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crime // Punishment","WB","Put target creature or enchantment card from an opponent's graveyard onto the battlefield under your control.
---
Destroy each artifact, creature, and enchantment with converted mana cost X.","3WB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Heart of the Wood","BG","Sacrifice a Forest: You gain 3 life.","BG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorilla Chieftain","3","3","G","{1}{G}: Regenerate Gorilla Chieftain.","2GG","Creature - Ape");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sunbeam Spellbomb","{W}, Sacrifice Sunbeam Spellbomb: You gain 5 life.
{1}, Sacrifice Sunbeam Spellbomb: Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quagmire Lamprey","1","1","B","Whenever Quagmire Lamprey becomes blocked by a creature, put a -1/-1 counter on that creature.","2B","Creature - Fish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mantle of Leadership","W","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Whenever a creature enters the battlefield, enchanted creature gets +2/+2 until end of turn.","1W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Primal Beyond","As Primal Beyond enters the battlefield, you may reveal an Elemental card from your hand. If you don't, Primal Beyond enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{T}: Add one mana of any color to your mana pool. Spend this mana only to cast an Elemental spell or activate an ability of an Elemental.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veteran Explorer","1","1","G","When Veteran Explorer dies, each player may search his or her library for up to two basic land cards and put them onto the battlefield. Then each player who searched his or her library this way shuffles it.","G","Creature - Human Soldier Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragon Mage","5","5","R","Flying
Whenever Dragon Mage deals combat damage to a player, each player discards his or her hand and draws seven cards.","5RR","Creature - Dragon Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Omnath, Locus of Mana","1","1","G","Green mana doesn't empty from your mana pool as steps and phases end.
Omnath, Locus of Mana gets +1/+1 for each green mana in your mana pool.","2G","Legendary Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mogg Hollows","{T}: Add {1} to your mana pool.
{T}: Add {R} or {G} to your mana pool. Mogg Hollows doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Duskworker","2","2","Whenever Duskworker becomes blocked, regenerate it.
{3}: Duskworker gets +1/+0 until end of turn.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orim, Samite Healer","1","3","W","{T}: Prevent the next 3 damage that would be dealt to target creature or player this turn.","1WW","Legendary Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crucible of Worlds","You may play land cards from your graveyard.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cruel Ultimatum","UBR","Target opponent sacrifices a creature, discards three cards, then loses 5 life. You return a creature card from your graveyard to your hand, draw three cards, then gain 5 life.","UUBBBRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonic Dread","BR","Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)
Target creature can't block this turn.","1BR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grinning Demon","6","6","B","At the beginning of your upkeep, you lose 2 life.
Morph {2}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2BB","Creature - Demon");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Keldon Battlewagon","0","3","Trample
Keldon Battlewagon can't block.
When Keldon Battlewagon attacks, sacrifice it at end of combat.
Tap an untapped creature you control: Keldon Battlewagon gets +X/+0 until end of turn, where X is the power of the creature tapped this way.","5","Artifact Creature - Juggernaut");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Agony Warp","UB","Target creature gets -3/-0 until end of turn.
Target creature gets -0/-3 until end of turn.","UB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Increasing Ambition","B","Search your library for a card and put that card into your hand. If Increasing Ambition was cast from a graveyard, instead search your library for two cards and put those cards into your hand. Then shuffle your library.
Flashback {7}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","4B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Privileged Position","WG","({(g/w)} can be paid with either {G} or {W}.)
Other permanents you control have hexproof. (They can't be the targets of spells or abilities your opponents control.)","2(G/W)(G/W)(G/W)","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Restrain","W","Prevent all combat damage that would be dealt by target attacking creature this turn.
Draw a card.","2W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Astrolabe","{1}, {T}, Sacrifice Astrolabe: Add two mana of any one color to your mana pool. Draw a card at the beginning of the next turn's upkeep.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sylvan Library","G","At the beginning of your draw step, you may draw two additional cards. If you do, choose two cards in your hand drawn this turn. For each of those cards, pay 4 life or put the card on top of your library.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhao Zilong, Tiger General","3","3","W","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Whenever Zhao Zilong, Tiger General blocks, it gets +1/+1 until end of turn.","3WW","Legendary Creature - Human Soldier Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fowl Play","U","Enchanted creature loses all abilities and is a 1/1 creature that counts as a Chicken.","2U","Enchant Creature");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fierce Empath","1","1","G","When Fierce Empath enters the battlefield, you may search your library for a creature card with converted mana cost 6 or greater, reveal it, put it into your hand, then shuffle your library.","2G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhalfirin Commander","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{1}{W}{W}: Target Knight creature gets +1/+1 until end of turn.","2W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inquisitor's Snare","W","Prevent all damage target attacking or blocking creature would deal this turn. If that creature is black or red, destroy it.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fanatical Fever","G","Target creature gets +3/+0 and gains trample until end of turn.","2GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zoologist","1","2","G","{3}{G}, {T}: Reveal the top card of your library. If it's a creature card, put it onto the battlefield. Otherwise, put it into your graveyard.","3G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quirion Sentinel","2","1","G","When Quirion Sentinel enters the battlefield, add one mana of any color to your mana pool.","1G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chill","U","Red spells cost {2} more to cast.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Buried Alive","B","Search your library for up to three creature cards and put them into your graveyard. Then shuffle your library.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ulamog, the Infinite Gyre","10","10","When you cast Ulamog, the Infinite Gyre, destroy target permanent.
Annihilator 4 (Whenever this creature attacks, defending player sacrifices four permanents.)
Ulamog is indestructible.
When Ulamog is put into a graveyard from anywhere, its owner shuffles his or her graveyard into his or her library.","11","Legendary Creature - Eldrazi");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Taunting Challenge","G","All creatures able to block target creature this turn do so.","1GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinder Shade","1","1","BR","{B}: Cinder Shade gets +1/+1 until end of turn.
{R}, Sacrifice Cinder Shade: Cinder Shade deals damage equal to its power to target creature.","1BR","Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Surveilling Sprite","1","1","U","Flying
When Surveilling Sprite dies, you may draw a card.","1U","Creature - Faerie Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armored Skaab","1","4","U","When Armored Skaab enters the battlefield, put the top four cards of your library into your graveyard.","2U","Creature - Zombie Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Matca Rioters","*","*","G","Domain - Matca Rioters's power and toughness are each equal to the number of basic land types among lands you control.","2G","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veteran Armorer","2","2","W","Other creatures you control get +0/+1.","1W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cease-Fire","W","Target player can't cast creature spells this turn.
Draw a card.","2W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hand of Emrakul","7","7","You may sacrifice four Eldrazi Spawn rather than pay Hand of Emrakul's mana cost.
Annihilator 1 (Whenever this creature attacks, defending player sacrifices a permanent.)","9","Creature - Eldrazi");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rath's Edge","{T}: Add {1} to your mana pool.
{4}, {T}, Sacrifice a land: Rath's Edge deals 1 damage to target creature or player.",NULL,"Legendary Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jwar Isle Refuge","Jwar Isle Refuge enters the battlefield tapped.
When Jwar Isle Refuge enters the battlefield, you gain 1 life.
{T}: Add {U} or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Locust Swarm","1","1","G","Flying
{G}: Regenerate Locust Swarm.
{G}: Untap Locust Swarm. Activate this ability only once each turn.","3G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Negator","5","5","B","Trample
Whenever Phyrexian Negator is dealt damage, sacrifice that many permanents.","2B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skullbriar, the Walking Grave","1","1","BG","Haste
Whenever Skullbriar, the Walking Grave deals combat damage to a player, put a +1/+1 counter on it.
Counters remain on Skullbriar as it moves to any zone other than a player's hand or library.","BG","Legendary Creature - Zombie Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rend Spirit","B","Destroy target Spirit.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thunderous Wrath","R","Thunderous Wrath deals 5 damage to target creature or player.
Miracle {R} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","4RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Telim'Tor's Edict","R","Exile target permanent you own or control.
Draw a card at the beginning of the next turn's upkeep.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wakestone Gargoyle","3","4","W","Defender (This creature can't attack.)
Flying
{1}{W}: Creatures you control with defender can attack this turn as though they didn't have defender.","3W","Creature - Gargoyle");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decree of Annihilation","R","Exile all artifacts, creatures, and lands from the battlefield, all cards from all graveyards, and all cards from all hands.
Cycling {5}{R}{R} ({5}{R}{R}, Discard this card: Draw a card.)
When you cycle Decree of Annihilation, destroy all lands.","8RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cathars' Crusade","W","Whenever a creature enters the battlefield under your control, put a +1/+1 counter on each creature you control.","3WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Electryte","3","3","R","Whenever Electryte deals combat damage to defending player, it deals damage equal to its power to each blocking creature.","3RR","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seafarer's Quay","Blue legendary creatures you control have ""bands with other legendary creatures."" (Any legendary creatures can attack in a band as long as at least one has ""bands with other legendary creatures."" Bands are blocked as a group. If at least two legendary creatures you control, one of which has ""bands with other legendary creatures,"" are blocking or being blocked by the same creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Falkenrath Exterminator","1","1","R","Whenever Falkenrath Exterminator deals combat damage to a player, put a +1/+1 counter on it.
{2}{R}: Falkenrath Exterminator deals damage to target creature equal to the number of +1/+1 counters on Falkenrath Exterminator.","1R","Creature - Vampire Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anurid Swarmsnapper","1","4","G","Reach (This creature can block creatures with flying.)
{1}{G}: Anurid Swarmsnapper can block an additional creature this turn.","2G","Creature - Frog Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Strata Scythe","Imprint - When Strata Scythe enters the battlefield, search your library for a land card, exile it, then shuffle your library.
Equipped creature gets +1/+1 for each land on the battlefield with the same name as the exiled card.
Equip {3}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Geist Trappers","3","5","G","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Geist Trappers is paired with another creature, both creatures have reach.","4G","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talas Researcher","1","1","U","{T}: Draw a card. Activate this ability only during your turn, before attackers are declared.","4U","Creature - Human Pirate Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Combat Medic","0","2","W","{1}{W}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","2W","Creature - Human Cleric Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hipparion","1","3","W","Hipparion can't block creatures with power 3 or greater unless you pay {1}.","1W","Creature - Horse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vitaspore Thallid","1","1","G","At the beginning of your upkeep, put a spore counter on Vitaspore Thallid.
Remove three spore counters from Vitaspore Thallid: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: Target creature gains haste until end of turn.","1G","Creature - Fungus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shocker","1","1","R","Whenever Shocker deals damage to a player, that player discards all the cards in his or her hand, then draws that many cards.","1R","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of Sursi","2","2","W","Flying; flanking  (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
Suspend 3-{W} (Rather than cast this card from your hand, you may pay {W} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","3W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kill Destroy","B","Destroy target nonblack creature.
Gotcha Whenever an opponent says ""Kill"" or ""Destroy,"" you may say ""Gotcha"" If you do, return Kill Destroy from your graveyard to your hand.","1BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormscape Familiar","1","1","U","Flying
White spells and black spells you cast cost {1} less to cast.","1U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunken Hope","U","At the beginning of each player's upkeep, that player returns a creature he or she controls to its owner's hand.","3UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hermetic Study","U","Enchant creature
Enchanted creature has ""{T}: This creature deals 1 damage to target creature or player.""","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vish Kal, Blood Arbiter","5","5","WB","Flying, lifelink
Sacrifice a creature: Put X +1/+1 counters on Vish Kal, Blood Arbiter, where X is the sacrificed creature's power.
Remove all +1/+1 counters from Vish Kal: Target creature gets -1/-1 until end of turn for each +1/+1 counter removed this way.","4WBB","Legendary Creature - Vampire");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Leechridden Swamp","({T}: Add {B} to your mana pool.)
Leechridden Swamp enters the battlefield tapped.
{B}, {T}: Each opponent loses 1 life. Activate this ability only if you control two or more black permanents.",NULL,"Land - Swamp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wind Dancer","1","1","U","Flying
{T}: Target creature gains flying until end of turn.","1U","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Precognition","U","At the beginning of your upkeep, you may look at the top card of target opponent's library. If you do, you may put that card on the bottom of that player's library.","4U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jester's Cap","{2}, {T}, Sacrifice Jester's Cap: Search target player's library for three cards and exile them. Then that player shuffles his or her library.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nameless Inversion","B","Changeling (This card is every creature type at all times.)
Target creature gets +3/-3 and loses all creature types until end of turn.","1B","Tribal Instant - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Embersmith","2","1","R","Whenever you cast an artifact spell, you may pay {1}. If you do, Embersmith deals 1 damage to target creature or player.","1R","Creature - Human Artificer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Charmed Pendant","{T}, Put the top card of your library into your graveyard: For each colored mana symbol in that card's mana cost, add one mana of that color to your mana pool. Activate this ability only any time you could cast an instant. (For example, if the card's mana cost is {3}{U}{U}{B}, you add {U}{U}{B} to your mana pool.)","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of Twisted Reflection","2","2","U","Sacrifice Kami of Twisted Reflection: Return target creature you control to its owner's hand.","1UU","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Novijen, Heart of Progress","{T}: Add {1} to your mana pool.
{G}{U}, {T}: Put a +1/+1 counter on each creature that entered the battlefield this turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mold Adder","1","1","G","Whenever an opponent casts a blue or black spell, you may put a +1/+1 counter on Mold Adder.","G","Creature - Fungus Snake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Woodland Cemetery","Woodland Cemetery enters the battlefield tapped unless you control a Swamp or a Forest.
{T}: Add {B} or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spectral Force","8","8","G","Trample
Whenever Spectral Force attacks, if defending player controls no black permanents, it doesn't untap during your next untap step.","3GG","Creature - Elemental Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Imagecrafter","1","1","U","{T}: Choose a creature type other than Wall. Target creature becomes that type until end of turn.","U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diseased Vermin","1","1","B","Whenever Diseased Vermin deals combat damage to a player, put an infection counter on it.
At the beginning of your upkeep, Diseased Vermin deals X damage to target opponent previously dealt damage by it, where X is the number of infection counters on it.","2B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Penumbra Wurm","6","6","G","Trample
When Penumbra Wurm dies, put a 6/6 black Wurm creature token with trample onto the battlefield.","5GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prohibit","U","Kicker {2} (You may pay an additional {2} as you cast this spell.)
Counter target spell if its converted mana cost is 2 or less. If Prohibit was kicked, counter that spell if its converted mana cost is 4 or less instead.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Naked Singularity","Cumulative upkeep {3} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
If tapped for mana, Plains produce {R}, Islands produce {G}, Swamps produce {W}, Mountains produce {U}, and Forests produce {B} instead of any other type.","5","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pine Barrens","Pine Barrens enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{T}: Add {B} or {G} to your mana pool. Pine Barrens deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thermal Glider","2","1","W","Flying, protection from red","2W","Creature - Human Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bloodbond March","BG","Whenever a player casts a creature spell, each player returns all cards with the same name as that spell from his or her graveyard to the battlefield.","2BG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Beast","4","5","U","When Phantom Beast becomes the target of a spell or ability, sacrifice it.","3U","Creature - Illusion Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scour","W","Exile target enchantment. Search its controller's graveyard, hand, and library for all cards with the same name as that enchantment and exile them. Then that player shuffles his or her library.","2WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Baloth Cage Trap","G","If an opponent had an artifact enter the battlefield under his or her control this turn, you may pay {1}{G} rather than pay Baloth Cage Trap's mana cost.
Put a 4/4 green Beast creature token onto the battlefield.","3GG","Instant - Trap");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Perilous Forays","G","{1}, Sacrifice a creature: Search your library for a land card with a basic land type and put it onto the battlefield tapped. Then shuffle your library.","3GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cryptoplasm","2","2","U","At the beginning of your upkeep, you may have Cryptoplasm become a copy of another target creature. If you do, Cryptoplasm gains this ability.","1UU","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fleeting Aven","2","2","U","Flying
Whenever a player cycles a card, return Fleeting Aven to its owner's hand.","1UU","Creature - Bird Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ezuri's Brigade","4","4","G","Metalcraft - As long as you control three or more artifacts, Ezuri's Brigade gets +4/+4 and has trample.","2GG","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Endless Cockroaches","1","1","B","When Endless Cockroaches dies, return it to its owner's hand.","1BB","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cateran Summons","B","Search your library for a Mercenary card, reveal that card, and put it into your hand. Then shuffle your library.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Southern Elephant","3","4","G",NULL,"3G","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellkite Charger","5","5","R","Flying, haste
Whenever Hellkite Charger attacks, you may pay {5}{R}{R}. If you do, untap all attacking creatures and after this phase, there is an additional combat phase.","4RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Zombie","2","2","UB","Protection from green","UB","Creature - Merfolk Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oblivion Ring","W","When Oblivion Ring enters the battlefield, exile another target nonland permanent.
When Oblivion Ring leaves the battlefield, return the exiled card to the battlefield under its owner's control.","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Keldon Megaliths","Keldon Megaliths enters the battlefield tapped.
{T}: Add {R} to your mana pool.
Hellbent - {1}{R}, {T}: Keldon Megaliths deals 1 damage to target creature or player. Activate this ability only if you have no cards in hand.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harsh Mercy","W","Each player chooses a creature type. Destroy all creatures that aren't of a type chosen this way. They can't be regenerated.","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beastmaster Ascension","G","Whenever a creature you control attacks, you may put a quest counter on Beastmaster Ascension.
As long as Beastmaster Ascension has seven or more quest counters on it, creatures you control get +5/+5.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thieving Sprite","1","1","B","Flying
When Thieving Sprite enters the battlefield, target player reveals X cards from his or her hand, where X is the number of Faeries you control. You choose one of those cards. That player discards that card.","2B","Creature - Faerie Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Despoil","B","Destroy target land. Its controller loses 2 life.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Gust-Skimmer","2","1","{U}: Gust-Skimmer gains flying until end of turn.","2","Artifact Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hornet Cobra","2","1","G","First strike","1GG","Creature - Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Auramancer's Guise","U","Enchant creature
Enchanted creature gets +2/+2 for each Aura attached to it and has vigilance.","2UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foothill Guide","1","1","W","Protection from Goblins
Morph {W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Champion","2","2","W","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever Soltari Champion attacks, all other creatures you control get +1/+1 until end of turn.","2W","Creature - Soltari Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("The Lady of the Mountain","5","5","RG",NULL,"4RG","Legendary Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Charisma","U","Enchant creature
Whenever enchanted creature deals damage to a creature, gain control of the other creature for as long as Charisma remains on the battlefield.","UUU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Torrent of Stone","R","Torrent of Stone deals 4 damage to target creature.
Splice onto Arcane-Sacrifice two Mountains. (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","3R","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gravebind","B","Target creature can't be regenerated this turn.
Draw a card at the beginning of the next turn's upkeep.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argothian Treefolk","3","5","G","Prevent all damage that would be dealt to Argothian Treefolk by artifact sources.","3GG","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lifelink","W","Enchant creature
Enchanted creature has lifelink. (Damage dealt by the creature also causes its controller to gain that much life.)","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vermiculos","1","1","B","Whenever an artifact enters the battlefield, Vermiculos gets +4/+4 until end of turn.","4B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Stag","G","Whenever an opponent plays a land, if Hidden Stag is an enchantment, Hidden Stag becomes a 3/2 Elk Beast creature.
Whenever you play a land, if Hidden Stag is a creature, Hidden Stag becomes an enchantment.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghosts of the Innocent","4","5","W","If a source would deal damage to a creature or player, it deals half that damage, rounded down, to that creature or player instead.","5WW","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Azorius Chancery","Azorius Chancery enters the battlefield tapped.
When Azorius Chancery enters the battlefield, return a land you control to its owner's hand.
{T}: Add {W}{U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chainer's Edict","B","Target player sacrifices a creature.
Flashback {5}{B}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blademane Baku","1","1","R","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Blademane Baku.
{1}, Remove X ki counters from Blademane Baku: For each counter removed, Blademane Baku gets +2/+0 until end of turn.","1R","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Soultiller","5","4","G","When Elvish Soultiller dies, choose a creature type. Shuffle all creature cards of that type from your graveyard into your library.","3GG","Creature - Elf Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glowering Rogon","4","4","G","Amplify 1 (As this creature enters the battlefield, put a +1/+1 counter on it for each Beast card you reveal in your hand.)","5G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wail of the Nim","B","Choose one - Regenerate each creature you control; or Wail of the Nim deals 1 damage to each creature and each player.
Entwine {B} (Choose both if you pay the entwine cost.)","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Emissary","3","1","B","Kicker {1}{U} (You may pay an additional {1}{U} as you cast this spell.)
When Urborg Emissary enters the battlefield, if it was kicked, return target permanent to its owner's hand.","2B","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grimclaw Bats","1","1","B","Flying
{B}, Pay 1 life: Grimclaw Bats gets +1/+1 until end of turn.","1B","Creature - Bat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Laboratory Maniac","2","2","U","If you would draw a card while your library has no cards in it, you win the game instead.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Banefire","R","Banefire deals X damage to target creature or player.
If X is 5 or more, Banefire can't be countered by spells or abilities and the damage can't be prevented.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Femeref Archers","2","2","G","{T}: Femeref Archers deals 4 damage to target attacking creature with flying.","2G","Creature - Human Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teysa, Orzhov Scion","2","3","WB","Sacrifice three white creatures: Exile target creature.
Whenever another black creature you control dies, put a 1/1 white Spirit creature token with flying onto the battlefield.","1WB","Legendary Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thornscape Master","2","2","G","{R}{R}, {T}: Thornscape Master deals 2 damage to target creature.
{W}{W}, {T}: Target creature gains protection from the color of your choice until end of turn.","2GG","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quirion Ranger","1","1","G","Return a Forest you control to its owner's hand: Untap target creature. Activate this ability only once each turn.","G","Creature - Elf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nomad Stadium","{T}: Add {W} to your mana pool. Nomad Stadium deals 1 damage to you.
Threshold - {W}, {T}, Sacrifice Nomad Stadium: You gain 4 life. Activate this ability only if seven or more cards are in your graveyard.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lose Hope","B","Target creature gets -1/-1 until end of turn.
Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shimian Specter","2","2","B","Flying
Whenever Shimian Specter deals combat damage to a player, that player reveals his or her hand. You choose a nonland card from it. Search that player's graveyard, hand, and library for all cards with the same name as that card and exile them. Then that player shuffles his or her library.","2BB","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Vineyard","1","1","G","At the beginning of each player's precombat main phase, add {G}{G} to that player's mana pool.","G","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Afflicted Deserter","3","2","R","At the beginning of each upkeep, if no spells were cast last turn, transform Afflicted Deserter.","3R","Creature - Human Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flamecore Elemental","5","4","R","Echo {2}{R}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","2RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mundungu","1","1","UB","{T}: Counter target spell unless its controller pays {1} and 1 life.","1UB","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mox Ruby","{T}: Add {R} to your mana pool.","0","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Geistcatcher's Rig","4","5","When Geistcatcher's Rig enters the battlefield, you may have it deal 4 damage to target creature with flying.","6","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bump in the Night","B","Target opponent loses 3 life.
Flashback {5}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exquisite Blood","B","Whenever an opponent loses life, you gain that much life.","4B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Framed!","U","Tap or untap all permanents by the artist of your choice.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devastating Summons","R","As an additional cost to cast Devastating Summons, sacrifice X lands.
Put two X/X red Elemental creature tokens onto the battlefield.","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Air Bladder","U","Enchant creature
Enchanted creature has flying.
Enchanted creature can block only creatures with flying.","U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seal of Doom","B","Sacrifice Seal of Doom: Destroy target nonblack creature. It can't be regenerated.","2B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Predatory Advantage","RG","At the beginning of each opponent's end step, if that player didn't cast a creature spell this turn, put a 2/2 green Lizard creature token onto the battlefield.","3RG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slipstream Serpent","6","6","U","Slipstream Serpent can't attack unless defending player controls an Island.
When you control no Islands, sacrifice Slipstream Serpent.
Morph {5}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","7U","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Neurok Invisimancer","2","1","U","Neurok Invisimancer is unblockable.
When Neurok Invisimancer enters the battlefield, target creature is unblockable this turn.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Omens","0","4","W","Defender
When Wall of Omens enters the battlefield, draw a card.","1W","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wildheart Invoker","4","3","G","{8}: Target creature gets +5/+5 and gains trample until end of turn.","2GG","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Midnight Duelist","1","2","W","Protection from Vampires","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidal Warrior","1","1","U","{T}: Target land becomes an Island until end of turn.","U","Creature - Merfolk Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cystbearer","2","3","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","2G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talas Merchant","1","3","U",NULL,"1U","Creature - Human Pirate");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cloak of Feathers","U","Target creature gains flying until end of turn.
Draw a card.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodlurker Mimic","2","1","BG","Whenever you cast a spell that's both black and green, Woodlurker Mimic becomes 4/5 and gains wither until end of turn. (It deals damage to creatures in the form of -1/-1 counters.)","1(B/G)","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darkling Stalker","1","1","B","{B}: Regenerate Darkling Stalker.
{B}: Darkling Stalker gets +1/+1 until end of turn.","3B","Creature - Shade Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Join the Ranks","W","Put two 1/1 white Soldier Ally creature tokens onto the battlefield.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiny Starfish","0","1","U","{U}: Regenerate Spiny Starfish.
At the beginning of each end step, if Spiny Starfish regenerated this turn, put a 0/1 blue Starfish creature token onto the battlefield for each time it regenerated this turn.","2U","Creature - Starfish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cabal Therapy","B","Name a nonland card. Target player reveals his or her hand and discards all cards with that name.
Flashback-Sacrifice a creature. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodline Keeper","3","3","B","Flying
{T}: Put a 2/2 black Vampire creature token with flying onto the battlefield.
{B}: Transform Bloodline Keeper. Activate this ability only if you control five or more Vampires.","2BB","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Electrolyze","UR","Electrolyze deals 2 damage divided as you choose among one or two target creatures and/or players.
Draw a card.","1UR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Waterspout Elemental","3","4","U","Kicker {U} (You may pay an additional {U} as you cast this spell.)
Flying
When Waterspout Elemental enters the battlefield, if it was kicked, return all other creatures to their owners' hands and you skip your next turn.","3UU","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sphere of Resistance","Spells cost {1} more to cast.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nemesis Mask","All creatures able to block equipped creature do so.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abbey Matron","1","3","W","{W}, {T}: Abbey Matron gets +0/+3 until end of turn.","2W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chimeric Sphere","{2}: Until end of turn, Chimeric Sphere becomes a 2/1 Construct artifact creature with flying.
{2}: Until end of turn, Chimeric Sphere becomes a 3/2 Construct artifact creature without flying.","3","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Pentavus","0","0","Pentavus enters the battlefield with five +1/+1 counters on it.
{1}, Remove a +1/+1 counter from Pentavus: Put a 1/1 colorless Pentavite artifact creature token with flying onto the battlefield.
{1}, Sacrifice a Pentavite: Put a +1/+1 counter on Pentavus.","7","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Predator Ooze","1","1","G","Predator Ooze is indestructible.
Whenever Predator Ooze attacks, put a +1/+1 counter on it.
Whenever a creature dealt damage by Predator Ooze this turn dies, put a +1/+1 counter on Predator Ooze.","GGG","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Distorting Wake","U","Return X target nonland permanents to their owners' hands.","XUUU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Honor the Fallen","W","Exile all creature cards from all graveyards. You gain 1 life for each card exiled this way.","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sol Grail","As Sol Grail enters the battlefield, choose a color.
{T}: Add one mana of the chosen color to your mana pool.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Virtuous Charge","W","Creatures you control get +1/+1 until end of turn.","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Valor Made Real","W","Target creature can block any number of creatures this turn.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wrench Mind","B","Target player discards two cards unless he or she discards an artifact card.","BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hanweir Lancer","2","2","R","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Hanweir Lancer is paired with another creature, both creatures have first strike.","2R","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scapeshift","G","Sacrifice any number of lands. Search your library for that many land cards, put them onto the battlefield tapped, then shuffle your library.","2GG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shuko","Equipped creature gets +1/+0.
Equip {0} ({0}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Barbed Wire","At the beginning of each player's upkeep, Barbed Wire deals 1 damage to that player.
{2}: Prevent the next 1 damage that would be dealt by Barbed Wire this turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tempting Wurm","5","5","G","When Tempting Wurm enters the battlefield, each opponent may put any number of artifact, creature, enchantment, and/or land cards from his or her hand onto the battlefield.","1G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Stuffy Doll","0","1","As Stuffy Doll enters the battlefield, choose a player.
Stuffy Doll is indestructible.
Whenever Stuffy Doll is dealt damage, it deals that much damage to the chosen player.
{T}: Stuffy Doll deals 1 damage to itself.","5","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squeeze","U","Sorcery spells cost {3} more to cast.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ludevic's Abomination","13","13","Trample",NULL,"Creature - Lizard Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashling, the Extinguisher","4","4","B","Whenever Ashling, the Extinguisher deals combat damage to a player, choose target creature that player controls. He or she sacrifices that creature.","2BB","Legendary Creature - Elemental Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Feed the Machine","When you set this scheme in motion, target opponent chooses self or others. If that player chooses self, he or she sacrifices two creatures. If the player chooses others, each of your other opponents sacrifices a creature.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inexorable Tide","U","Whenever you cast a spell, proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("B-I-N-G-O","1","1","G","Trample
Whenever a player plays a spell, put a chip counter on its converted mana cost.
B-I-N-G-O gets +9/+9 for each set of three numbers in a row with chip counters on them.","1G","Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Salvage","G","Put target card from your graveyard on top of your library.","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pegasus Charger","2","1","W","Flying (This creature can't be blocked except by creatures with flying or reach.)
First strike (This creature deals combat damage before creatures without first strike.)","2W","Creature - Pegasus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Johan","5","4","WRG","At the beginning of combat on your turn, you may have Johan gain ""Johan can't attack"" until end of combat. If you do, attacking doesn't cause creatures you control to tap this combat if Johan is untapped.","3RGW","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corrosive Gale","G","({(g/p)} can be paid with either {G} or 2 life.)
Corrosive Gale deals X damage to each creature with flying.","X(G/P)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Illicit Auction","R","Each player may bid life for control of target creature. You start the bidding with a bid of 0. In turn order, each player may top the high bid. The bidding ends if the high bid stands. The high bidder loses life equal to the high bid and gains control of the creature. (This effect lasts indefinitely.)","3RR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Forest","G",NULL,"Basic Land - Forest");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stupor","B","Target opponent discards a card at random, then discards a card.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Windborn Muse","2","3","W","Flying
Creatures can't attack you unless their controller pays {2} for each creature he or she controls that's attacking you.","3W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chimney Imp","1","2","B","Flying
When Chimney Imp dies, target opponent puts a card from his or her hand on top of his or her library.","4B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crazed Goblin","1","1","R","Crazed Goblin attacks each turn if able.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thought Eater","2","2","U","Flying
Your maximum hand size is reduced by three.","1U","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timber Protector","4","6","G","Other Treefolk creatures you control get +1/+1.
Other Treefolk and Forests you control are indestructible.","4G","Creature - Treefolk Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Regrowth","G","Return target card from your graveyard to your hand.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pack Hunt","G","Search your library for up to three cards with the same name as target creature, reveal them, and put them into your hand. Then shuffle your library.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alabaster Potion","W","Choose one - Target player gains X life; or prevent the next X damage that would be dealt to target creature or player this turn.","XWW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avizoa","2","2","U","Flying
{0}: Avizoa gets +2/+2 until end of turn. You skip your next untap step. Activate this ability only once each turn.","3U","Creature - Jellyfish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chill of Foreboding","U","Each player puts the top five cards of his or her library into his or her graveyard.
Flashback {7}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lotus Vale","If Lotus Vale would enter the battlefield, sacrifice two untapped lands instead. If you do, put Lotus Vale onto the battlefield. If you don't, put it into its owner's graveyard.
{T}: Add three mana of any one color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Putrefy","BG","Destroy target artifact or creature. It can't be regenerated.","1BG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silverglade Pathfinder","1","1","G","{1}{G}, {T}, Discard a card: Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.","1G","Creature - Dryad Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Psychotrope Thallid","1","1","G","At the beginning of your upkeep, put a spore counter on Psychotrope Thallid.
Remove three spore counters from Psychotrope Thallid: Put a 1/1 green Saproling creature token onto the battlefield.
{1}, Sacrifice a Saproling: Draw a card.","2G","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sacred Boon","W","Prevent the next 3 damage that would be dealt to target creature this turn. At the beginning of the next end step, put a +0/+1 counter on that creature for each 1 damage prevented this way.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sensor Splicer","1","1","W","When Sensor Splicer enters the battlefield, put a 3/3 colorless Golem artifact creature token onto the battlefield.
Golem creatures you control have vigilance.","4W","Creature - Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pulling Teeth","B","Clash with an opponent. If you win, target player discards two cards. Otherwise, that player discards a card. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shard Convergence","G","Search your library for a Plains card, an Island card, a Swamp card, and a Mountain card. Reveal those cards and put them into your hand. Then shuffle your library.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghoultree","10","10","G","Ghoultree costs {1} less to cast for each creature card in your graveyard.","7G","Creature - Zombie Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Delraich","6","6","B","Trample
You may sacrifice three black creatures rather than pay Delraich's mana cost.","6B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daru Warchief","1","1","W","Soldier spells you cast cost {1} less to cast.
Soldier creatures you control get +1/+2.","2WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Mindripper","2","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever Dauthi Mindripper attacks and isn't blocked, you may sacrifice it. If you do, defending player discards three cards.","3B","Creature - Dauthi Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wizened Snitches","1","3","U","Flying
Players play with the top card of their libraries revealed.","3U","Creature - Faerie Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tortured Existence","B","{B}, Discard a creature card: Return target creature card from your graveyard to your hand.","B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit Shackle","B","Enchant creature
Whenever enchanted creature becomes tapped, put a -0/-2 counter on it.","BB","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sorcerer's Strongbox","{2}, {T}: Flip a coin. If you win the flip, sacrifice Sorcerer's Strongbox and draw three cards.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Midnight Guard","2","3","W","Whenever another creature enters the battlefield, untap Midnight Guard.","2W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wizards' School","{T}: Add {1} to your mana pool.
{1}, {T}: Add {U} to your mana pool.
{2}, {T}: Add {W} or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dominating Licid","1","1","U","{1}{U}{U}, {T}: Dominating Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {U} to end this effect.
You control enchanted creature.","1UU","Creature - Licid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thunder of Hooves","R","Thunder of Hooves deals X damage to each creature without flying and each player, where X is the number of Beasts on the battlefield.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boreal Druid","1","1","G","{T}: Add {1} to your mana pool.","G","Snow Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Black Cat","1","1","B","When Black Cat dies, target opponent discards a card at random.","1B","Creature - Zombie Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Murmurs from Beyond","U","Reveal the top three cards of your library. An opponent chooses one of them. Put that card into your graveyard and the rest into your hand.","2U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Valor","2","2","W","First strike
As long as Valor is in your graveyard and you control a Plains, creatures you control have first strike.","3W","Creature - Incarnation");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nimana Sell-Sword","2","2","B","Whenever Nimana Sell-Sword or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Nimana Sell-Sword.","3B","Creature - Human Warrior Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Geth, Lord of the Vault","5","5","B","Intimidate
{X}{B}: Put target artifact or creature card with converted mana cost X from an opponent's graveyard onto the battlefield under your control tapped. Then that player puts the top X cards of his or her library into his or her graveyard.","4BB","Legendary Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Barrier","U","Whenever a player casts a creature spell, that player sacrifices a permanent unless he or she pays {1}.","2U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("All in Good Time","When you set this scheme in motion, take an extra turn after this one. Schemes can't be set in motion that turn.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ulamog's Crusher","8","8","Annihilator 2 (Whenever this creature attacks, defending player sacrifices two permanents.)
Ulamog's Crusher attacks each turn if able.","8","Creature - Eldrazi");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wave Elemental","2","3","U","{U}, {T}, Sacrifice Wave Elemental: Tap up to three target creatures without flying.","2UU","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bant Charm","WUG","Choose one - Destroy target artifact; or put target creature on the bottom of its owner's library; or counter target instant spell.","GWU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zerapa Minotaur","3","3","R","First strike
{2}: Zerapa Minotaur loses first strike until end of turn. Any player may activate this ability.","2RR","Creature - Minotaur");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death's Caress","B","Destroy target creature. If that creature was a Human, you gain life equal to its toughness.","3BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Miraculous Recovery","W","Return target creature card from your graveyard to the battlefield. Put a +1/+1 counter on it.","4W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jasmine Seer","1","1","W","{2}{W}, {T}: Reveal any number of white cards in your hand. You gain 2 life for each card revealed this way.","3W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Fanatic","1","1","R","Sacrifice Mogg Fanatic: Mogg Fanatic deals 1 damage to target creature or player.","R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deadly Allure","B","Target creature gains deathtouch until end of turn and must be blocked this turn if able.
Flashback {G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Cloud","B","Each player loses X life, then discards X cards, then sacrifices X creatures, then sacrifices X lands.","XBBB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Descendants' Path","G","At the beginning of your upkeep, reveal the top card of your library. If it's a creature card that shares a creature type with a creature you control, you may cast that card without paying its mana cost. Otherwise, put that card on the bottom of your library.","2G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Memory Lapse","U","Counter target spell. If that spell is countered this way, put it on top of its owner's library instead of into that player's graveyard.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chant of the Skifsang","U","Enchant creature
Enchanted creature gets -13/-0.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Matopi Golem","3","3","{1}: Regenerate Matopi Golem. When it regenerates this way, put a -1/-1 counter on it.","5","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Knollspine Invocation","R","{X}, Discard a card with converted mana cost X: Knollspine Invocation deals X damage to target creature or player.","1RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stinkdrinker Bandit","2","1","B","Prowl {1}{B} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Goblin or Rogue.)
Whenever a Rogue you control attacks and isn't blocked, it gets +2/+1 until end of turn.","3B","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Shield Sphere","0","6","Defender
Whenever Shield Sphere blocks, put a -0/-1 counter on it.","0","Artifact Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reki, the History of Kamigawa","1","2","G","Whenever you cast a legendary spell, draw a card.","2G","Legendary Creature - Human Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dwarven Ruins","Dwarven Ruins enters the battlefield tapped.
{T}: Add {R} to your mana pool.
{T}, Sacrifice Dwarven Ruins: Add {R}{R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noggle Bridgebreaker","4","3","UR","When Noggle Bridgebreaker enters the battlefield, return a land you control to its owner's hand.","2(U/R)(U/R)","Creature - Noggle Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Master Warcraft","WR","Cast Master Warcraft only before attackers are declared.
You choose which creatures attack this turn.
You choose which creatures block this turn and how those creatures block.","2(R/W)(R/W)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loam Dweller","2","2","G","Whenever you cast a Spirit or Arcane spell, you may put a land card from your hand onto the battlefield tapped.","1G","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Draconian Cylix","{2}, {T}, Discard a card at random: Regenerate target creature.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragonsoul Knight","2","2","R","First strike
{W}{U}{B}{R}{G}: Until end of turn, Dragonsoul Knight becomes a Dragon, gets +5/+3, and gains flying and trample.","2R","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arms Dealer","1","1","R","{1}{R}, Sacrifice a Goblin: Arms Dealer deals 4 damage to target creature.","2R","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leaping Lizard","2","3","G","{1}{G}: Leaping Lizard gets -0/-1 and gains flying until end of turn.","1GG","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Foresee","U","Scry 4, then draw two cards. (To scry 4, look at the top four cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Icy Prison","U","When Icy Prison enters the battlefield, exile target creature.
At the beginning of your upkeep, sacrifice Icy Prison unless any player pays {3}.
When Icy Prison leaves the battlefield, return the exiled card to the battlefield under its owner's control.","UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Treasure Trove","U","{2}{U}{U}: Draw a card.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anowon, the Ruin Sage","4","3","B","At the beginning of your upkeep, each player sacrifices a non-Vampire creature.","3BB","Legendary Creature - Vampire Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mana Web","Whenever a land an opponent controls is tapped for mana, tap all lands that player controls that could produce any type of mana that land could produce.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Djinn Illuminatus","3","5","UR","({(u/r)} can be paid with either {U} or {R}.)
Flying
Each instant and sorcery spell you cast has replicate. The replicate cost is equal to its mana cost. (When you cast it, copy it for each time you paid its replicate cost. You may choose new targets for the copies.)","5(U/R)(U/R)","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reviving Dose","W","You gain 3 life.
Draw a card.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pearled Unicorn","2","2","W",NULL,"2W","Creature - Unicorn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caller of the Claw","2","2","G","Flash
When Caller of the Claw enters the battlefield, put a 2/2 green Bear creature token onto the battlefield for each nontoken creature put into your graveyard from the battlefield this turn.","2G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Clam Session","2","5","U","When Clam Session comes into play, choose a word.
During your upkeep, sing at least six words of a song, one of which must be the chosen word, or sacrifice Clam Session. You cannot repeat a song.","1UU","Summon - Clamfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emancipation Angel","3","3","W","Flying
When Emancipation Angel enters the battlefield, return a permanent you control to its owner's hand.","1WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Even the Odds","W","Cast Even the Odds only if you control fewer creatures than each opponent.
Put three 1/1 white Soldier creature tokens onto the battlefield.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vendetta","B","Destroy target nonblack creature. It can't be regenerated. You lose life equal to that creature's toughness.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jetting Glasskite","4","4","U","Flying
Whenever Jetting Glasskite becomes the target of a spell or ability for the first time in a turn, counter that spell or ability.","4UU","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arctic Flats","Arctic Flats enters the battlefield tapped.
{T}: Add {G} or {W} to your mana pool.",NULL,"Snow Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Information Dealer","1","1","U","{T}: Look at the top X cards of your library, where X is the number of Wizards on the battlefield, then put them back in any order.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Putrid Imp","1","1","B","Discard a card: Putrid Imp gains flying until end of turn.
Threshold - As long as seven or more cards are in your graveyard, Putrid Imp gets +1/+1 and can't block.","B","Creature - Zombie Imp");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Saproling","1","1",NULL,NULL,"Creature - Saproling");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moggcatcher","2","2","R","{3}, {T}: Search your library for a Goblin permanent card and put it onto the battlefield. Then shuffle your library.","2RR","Creature - Human Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pride Guardian","0","3","W","Defender (This creature can't attack.)
Whenever Pride Guardian blocks, you gain 3 life.","W","Creature - Cat Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call to the Netherworld","B","Return target black creature card from your graveyard to your hand.
Madness {0} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pegasus Refuge","W","{2}, Discard a card: Put a 1/1 white Pegasus creature token with flying onto the battlefield.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Penumbra Bobcat","2","1","G","When Penumbra Bobcat dies, put a 2/1 black Cat creature token onto the battlefield.","2G","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feebleness","B","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature gets -2/-1.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Changeling Titan","7","7","G","Changeling (This card is every creature type at all times.)
Champion a creature (When this enters the battlefield, sacrifice it unless you exile another creature you control. When this leaves the battlefield, that card returns to the battlefield.)","4G","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Apprentice Wizard","0","1","U","{U}, {T}: Add {3} to your mana pool.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Healer's Headdress","Equipped creature gets +0/+2 and has ""{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.""
{W}{W}: Attach Healer's Headdress to target creature you control.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Phantom","3","1","B","Urborg Phantom can't block.
{U}: Prevent all combat damage that would be dealt to and dealt by Urborg Phantom this turn.","2B","Creature - Spirit Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slith Predator","1","1","G","Trample
Whenever Slith Predator deals combat damage to a player, put a +1/+1 counter on it.","GG","Creature - Slith");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Silverpelt Werewolf","4","5","Whenever Silverpelt Werewolf deals combat damage to a player, draw a card.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Silverpelt Werewolf.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Villainous Ogre","3","2","B","Villainous Ogre can't block.
As long as you control a Demon, Villainous Ogre has ""{B}: Regenerate Villainous Ogre.""","2B","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Volcano Hellion","6","5","R","Volcano Hellion has echo {X}, where X is your life total. (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Volcano Hellion enters the battlefield, it deals an amount of damage of your choice to you and target creature. The damage can't be prevented.","2RR","Creature - Hellion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infernal Harvest","B","As an additional cost to cast Infernal Harvest, return X Swamps you control to their owner's hand.
Infernal Harvest deals X damage divided as you choose among any number of target creatures.","1B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Contested Cliffs","{T}: Add {1} to your mana pool.
{R}{G}, {T}: Choose target Beast creature you control and target creature an opponent controls. Those creatures fight each other. (Each deals damage equal to its power to the other.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vital Surge","G","You gain 3 life.
Splice onto Arcane {1}{G} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1G","Instant - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cellar Door","{3}, {T}: Target player puts the bottom card of his or her library into his or her graveyard. If it's a creature card, you put a 2/2 black Zombie creature token onto the battlefield.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caldera Hellion","3","3","R","Devour 1 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with that many +1/+1 counters on it.)
When Caldera Hellion enters the battlefield, it deals 3 damage to each creature.","3RR","Creature - Hellion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Horned Helm","Equipped creature gets +1/+1 and has trample.
{G}{G}: Attach Horned Helm to target creature you control.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Totem","{T}: Add {B} to your mana pool.
{2}{B}: Phyrexian Totem becomes a 5/5 black Horror artifact creature with trample until end of turn.
Whenever Phyrexian Totem is dealt damage, if it's a creature, sacrifice that many permanents.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ixidor's Will","U","Counter target spell unless its controller pays {2} for each Wizard on the battlefield.","2U","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Masticore","4","4","At the beginning of your upkeep, sacrifice Masticore unless you discard a card.
{2}: Masticore deals 1 damage to target creature.
{2}: Regenerate Masticore.","4","Artifact Creature - Masticore");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prison Barricade","1","3","W","Defender (This creature can't attack.)
Kicker {1}{W} (You may pay an additional {1}{W} as you cast this spell.)
If Prison Barricade was kicked, it enters the battlefield with a +1/+1 counter on it and with ""Prison Barricade can attack as though it didn't have defender.""","1W","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fend Off","W","Prevent all combat damage that would be dealt by target creature this turn.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Corrupter","2","2","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
When Viridian Corrupter enters the battlefield, destroy target artifact.","1GG","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slaughter","B","Buyback-Pay 4 life. (You may pay 4 life in addition to any other costs as you cast this spell. If you do, put this card into your hand as it resolves.)
Destroy target nonblack creature. It can't be regenerated.","2BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Radiate","R","Choose target instant or sorcery spell that targets only a single permanent or player. Copy that spell for each other permanent or player the spell could target. Each copy targets a different one of those permanents and players.","3RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unhinge","B","Target player discards a card.
Draw a card.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Constricting Tendrils","U","Target creature gets -3/-0 until end of turn.
Cycling {2} ({2}, Discard this card: Draw a card.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darkwatch Elves","2","2","G","Protection from black
Cycling {2} ({2}, Discard this card: Draw a card.)","2G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cunning Advisor","1","1","B","{T}: Target opponent discards a card. Activate this ability only during your turn, before attackers are declared.","3B","Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enchanted Being","2","2","W","Prevent all combat damage that would be dealt to Enchanted Being by enchanted creatures.","1WW","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exploration","G","You may play an additional land on each of your turns.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Razerunners","3","4","R","{1}{R}, Sacrifice a land: Put a +1/+1 counter on Goblin Razerunners.
At the beginning of your end step, you may have Goblin Razerunners deal damage equal to the number of +1/+1 counters on it to target player.","2RR","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flickering Ward","W","Enchant creature
As Flickering Ward enters the battlefield, choose a color.
Enchanted creature has protection from the chosen color. This effect doesn't remove Flickering Ward.
{W}: Return Flickering Ward to its owner's hand.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Utopia Tree","0","2","G","{T}: Add one mana of any color to your mana pool.","1G","Creature - Plant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Karn Liberated","+4: Target player exiles a card from his or her hand.
-3: Exile target permanent.
-14: Restart the game, leaving in exile all non-Aura permanent cards exiled with Karn Liberated. Then put those cards onto the battlefield under your control.","7","Planeswalker - Karn");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Deathrender","Equipped creature gets +2/+2.
Whenever equipped creature dies, you may put a creature card from your hand onto the battlefield and attach Deathrender to it.
Equip {2}","4","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Isochron Scepter","Imprint - When Isochron Scepter enters the battlefield, you may exile an instant card with converted mana cost 2 or less from your hand.
{2}, {T}: You may copy the exiled card. If you do, you may cast the copy without paying its mana cost.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hazduhr the Abbot","2","5","W","{X}, {T}: The next X damage that would be dealt this turn to target white creature you control is dealt to Hazduhr the Abbot instead.","3WW","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corrosive Mentor","1","3","B","Black creatures you control have wither. (They deal damage to creatures in the form of -1/-1 counters.)","2B","Creature - Elemental Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Greel's Caress","B","Flash
Enchant creature
Enchanted creature gets -3/-0.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Offalsnout","2","2","B","Flash
When Offalsnout leaves the battlefield, exile target card from a graveyard.
Evoke {B} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","2B","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tel-Jilad Justice","G","Destroy target artifact.
Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parasitic Implant","B","Enchant creature
At the beginning of your upkeep, enchanted creature's controller sacrifices it and you put a 1/1 colorless Myr artifact creature token onto the battlefield.","3B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elves of Deep Shadow","1","1","G","{T}: Add {B} to your mana pool. Elves of Deep Shadow deals 1 damage to you.","G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Gnarr","2","2","G","Whenever a player casts a black spell, Bog Gnarr gets +2/+2 until end of turn.","4G","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("School of the Unseen","{T}: Add {1} to your mana pool.
{2}, {T}: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Echoes","U","Enchant player
Whenever enchanted player casts an instant or sorcery spell, each other player may copy that spell and may choose new targets for the copy he or she controls.","4U","Enchantment - Aura Curse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anaba Ancestor","1","1","R","{T}: Another target Minotaur creature gets +1/+1 until end of turn.","1R","Creature - Minotaur Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heart of Ramos","{T}: Add {R} to your mana pool.
Sacrifice Heart of Ramos: Add {R} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arc Mage","2","2","R","{2}{R}, {T}, Discard a card: Arc Mage deals 2 damage divided as you choose among one or two target creatures and/or players.","2R","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorehorn Minotaurs","3","3","R","Bloodthirst 2 (If an opponent was dealt damage this turn, this creature enters the battlefield with two +1/+1 counters on it.)","2RR","Creature - Minotaur Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Taunting Elf","0","1","G","All creatures able to block Taunting Elf do so.","G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gus","2","2","G","Gus comes into play with one +1/+1 counter on it for each game you have lost to your opponent since you last won a Magic game against him or her.","2G","Summon - Gus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Manipulate Fate","U","Search your library for three cards, exile them, then shuffle your library.
Draw a card.","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daggerclaw Imp","3","1","B","Flying
Daggerclaw Imp can't block.","2B","Creature - Imp");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Memory Jar","{T}, Sacrifice Memory Jar: Each player exiles all cards from his or her hand face down and draws seven cards. At the beginning of the next end step, each player discards his or her hand and returns to his or her hand each card he or she exiled this way.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Knight","2","2","WG","Protection from black","GW","Creature - Elf Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Galvanoth","3","3","R","At the beginning of your upkeep, you may look at the top card of your library. If it's an instant or sorcery card, you may cast it without paying its mana cost.","3RR","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Warrior","4","2","R",NULL,"3R","Creature - Viashino Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Simian Brawler","3","3","G","Discard a land card: Simian Brawler gets +1/+1 until end of turn.","3G","Creature - Ape Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nezumi Cutthroat","2","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Nezumi Cutthroat can't block.","1B","Creature - Rat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slipstream Eel","6","6","U","Slipstream Eel can't attack unless defending player controls an Island.
Cycling {1}{U} ({1}{U}, Discard this card: Draw a card.)","5UU","Creature - Fish Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orochi Ranger","2","1","G","Whenever Orochi Ranger deals combat damage to a creature, tap that creature and it doesn't untap during its controller's next untap step.","1G","Creature - Snake Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noggle Hedge-Mage","2","2","UR","When Noggle Hedge-Mage enters the battlefield, if you control two or more Islands, you may tap two target permanents.
When Noggle Hedge-Mage enters the battlefield, if you control two or more Mountains, you may have Noggle Hedge-Mage deal 2 damage to target player.","2(U/R)","Creature - Noggle Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balshan Beguiler","1","1","U","Whenever Balshan Beguiler deals combat damage to a player, that player reveals the top two cards of his or her library. You choose one of those cards and put it into his or her graveyard.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Etched Champion","2","2","Metalcraft - Etched Champion has protection from all colors as long as you control three or more artifacts.","3","Artifact Creature - Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drain the Well","BG","Destroy target land. You gain 2 life.","2(B/G)(B/G)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overwhelming Stampede","G","Until end of turn, creatures you control gain trample and get +X/+X, where X is the greatest power among creatures you control. (If a creature you control would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","3GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Revered Unicorn","2","3","W","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When Revered Unicorn leaves the battlefield, you gain life equal to the number of age counters on it.","1W","Creature - Unicorn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravel Slinger","1","3","W","{T}: Gravel Slinger deals 1 damage to target attacking or blocking creature.
Morph {1}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ghost Quarter","{T}: Add {1} to your mana pool.
{T}, Sacrifice Ghost Quarter: Destroy target land. Its controller may search his or her library for a basic land card, put it onto the battlefield, then shuffle his or her library.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Maelstrom Archangel Avatar","At end of combat, for each creature you controlled at the time it dealt combat damage to a player this turn, copy a random card with the same mana cost as that creature. You may pay {3}. If you do, choose one of those copies. If a copy of a permanent card is chosen, you may put a token onto the battlefield that's a copy of that card. If a copy of an instant or sorcery card is chosen, you may cast the copy without paying its mana cost.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Suicidal Charge","BR","Sacrifice Suicidal Charge: Creatures your opponents control get -1/-1 until end of turn. Those creatures attack this turn if able.","3BR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ruby Leech","2","2","R","First strike
Red spells you cast cost {R} more to cast.","1R","Creature - Leech");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shield Wall","W","Creatures you control get +0/+2 until end of turn.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Somberwald Sage","0","1","G","{T}: Add three mana of any one color to your mana pool. Spend this mana only to cast creature spells.","2G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moonlight Bargain","B","Look at the top five cards of your library. For each card, put that card into your graveyard unless you pay 2 life. Then put the rest into your hand.","3BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fungal Shambler","6","4","UBG","Trample
Whenever Fungal Shambler deals damage to an opponent, you draw a card and that opponent discards a card.","4GUB","Creature - Fungus Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Adarkar Unicorn","2","2","W","{T}: Add {U} or {1}{U} to your mana pool. Spend this mana only to pay cumulative upkeep costs.","1WW","Creature - Unicorn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emerald Oryx","2","3","G","Forestwalk (This creature is unblockable as long as defending player controls a Forest.)","3G","Creature - Antelope");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marble Titan","3","3","W","Creatures with power 3 or greater don't untap during their controllers' untap steps.","3W","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Morbid Bloom","BG","Exile target creature card from a graveyard, then put X 1/1 green Saproling creature tokens onto the battlefield, where X is the exiled card's toughness.","4BG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunder Wall","0","2","U","Defender (This creature can't attack.)
Flying
{U}: Thunder Wall gets +1/+1 until end of turn.","1UU","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Night Revelers","4","4","R","Night Revelers has haste as long as an opponent controls a Human.","4R","Creature - Vampire");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mask of Avacyn","Equipped creature gets +1/+2 and has hexproof. (It can't be the target of spells or abilities your opponents control.)
Equip {3}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Exorcist","1","1","W","{1}{W}, {T}: Destroy target black creature.","WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reaper of Sheoldred","2","5","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Whenever a source deals damage to Reaper of Sheoldred, that source's controller gets a poison counter.","4B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cursecatcher","1","1","U","Sacrifice Cursecatcher: Counter target instant or sorcery spell unless its controller pays {1}.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infernal Genesis","B","At the beginning of each player's upkeep, that player puts the top card of his or her library into his or her graveyard. Then he or she puts X 1/1 black Minion creature tokens onto the battlefield, where X is that card's converted mana cost.","4BB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Suntouched Myr","0","0","Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it.)","3","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clinging Darkness","B","Enchant creature
Enchanted creature gets -4/-1.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Spy","1","1","R","{T}: Look at the top three cards of target player's library.","R","Creature - Orc Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Shrieker","3","3","U","Flying
{1}: Mistform Shrieker becomes the creature type of your choice until end of turn.
Morph {3}{U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stoic Angel","3","4","WUG","Flying, vigilance
Players can't untap more than one creature during their untap steps.","1GWU","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Avacyn's Collar","Equipped creature gets +1/+0 and has vigilance.
Whenever equipped creature dies, if it was a Human, put a 1/1 white Spirit creature token with flying onto the battlefield.
Equip {2}","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sacred Prey","1","1","G","Whenever Sacred Prey becomes blocked, you gain 1 life.","G","Creature - Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yoke of the Damned","B","Enchant creature
When a creature dies, destroy enchanted creature.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sporoloth Ancient","4","4","G","At the beginning of your upkeep, put a spore counter on Sporoloth Ancient.
Creatures you control have ""Remove two spore counters from this creature: Put a 1/1 green Saproling creature token onto the battlefield.""","3GG","Creature - Fungus");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Viridian Zealot Avatar","{2}, Sacrifice a creature: Destroy target artifact or enchantment. Search your library for a card with the same name as the sacrificed creature, reveal that card, and put it into your hand. Then shuffle your library.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mercurial Kite","2","2","U","Flying
Whenever Mercurial Kite deals combat damage to a creature, tap that creature. That creature doesn't untap during its controller's next untap step.","3U","Creature - Bird");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hinterland Scourge","3","2","Hinterland Scourge must be blocked if able.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Hinterland Scourge.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hatred","B","As an additional cost to cast Hatred, pay X life.
Target creature gets +X/+0 until end of turn.","3BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dregscape Zombie","2","1","B","Unearth {B} ({B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","1B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breath of Fury","R","Enchant creature you control
When enchanted creature deals combat damage to a player, sacrifice it and attach Breath of Fury to a creature you control. If you do, untap all creatures you control and after this phase, there is an additional combat phase.","2RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Elemental","5","4","R",NULL,"3RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karona's Zealot","2","5","W","Morph {3}{W}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Karona's Zealot is turned face up, all damage that would be dealt to it this turn is dealt to target creature instead.","4W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chronosavant","5","5","W","{1}{W}: Return Chronosavant from your graveyard to the battlefield tapped. You skip your next turn.","5W","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spontaneous Combustion","BR","As an additional cost to cast Spontaneous Combustion, sacrifice a creature.
Spontaneous Combustion deals 3 damage to each creature.","1BR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Topsy Turvy","U","The phases of each player's turn are reversed. (The phases are, in reverse order, end, postcombat main, combat, precombat main, and beginning.)
If there are more than two players in the game, the turn order is reversed.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Taunt","U","During target player's next turn, creatures that player controls attack you if able.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudcrown Oak","3","4","G","Reach (This can block creatures with flying.)","2GG","Creature - Treefolk Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jerrard of the Closed Fist","6","5","RG",NULL,"3RGG","Legendary Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tor Wauki","3","3","BR","{T}: Tor Wauki deals 2 damage to target attacking or blocking creature.","2BBR","Legendary Creature - Human Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nightbird's Clutches","R","Up to two target creatures can't block this turn.
Flashback {3}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancient Stirrings","G","Look at the top five cards of your library. You may reveal a colorless card from among them and put it into your hand. Then put the rest on the bottom of your library in any order. (Cards with no colored mana in their mana costs are colorless. Lands are also colorless.)","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ambiguity","U","Whenever a player plays a spell that counters a spell that has been played or a player plays a spell that comes into play with counters, that player may counter the next spell played or put an additional counter on a permanent that has already been played, but not countered.","2UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Multani's Decree","G","Destroy all enchantments. You gain 2 life for each enchantment destroyed this way.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talas Explorer","1","1","U","Flying
When Talas Explorer enters the battlefield, look at target opponent's hand.","1U","Creature - Human Pirate Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Dead","2","2","BG","{T}: Add {B} to your mana pool.","BG","Creature - Zombie Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Graceful Antelope","1","4","W","Plainswalk
Whenever Graceful Antelope deals combat damage to a player, you may have target land become a Plains until Graceful Antelope leaves the battlefield.","2WW","Creature - Antelope");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mortify","WB","Destroy target creature or enchantment.","1WB","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blasting Station","{T}, Sacrifice a creature: Blasting Station deals 1 damage to target creature or player.
Whenever a creature enters the battlefield, you may untap Blasting Station.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drifting Djinn","5","5","U","Flying
At the beginning of your upkeep, sacrifice Drifting Djinn unless you pay {1}{U}.
Cycling {2} ({2}, Discard this card: Draw a card.)","4UU","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disruptive Student","1","1","U","{T}: Counter target spell unless its controller pays {1}.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bruna, Light of Alabaster","5","5","WU","Flying, vigilance
Whenever Bruna, Light of Alabaster attacks or blocks, you may attach to it any number of Auras on the battlefield and you may put onto the battlefield attached to it any number of Aura cards that could enchant it from your graveyard and/or hand.","3WWU","Legendary Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trophy Hunter","2","3","G","{1}{G}: Trophy Hunter deals 1 damage to target creature with flying.
Whenever a creature with flying dealt damage by Trophy Hunter this turn dies, put a +1/+1 counter on Trophy Hunter.","2G","Creature - Human Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necro-Impotence","B","Skip your untap step.
At the beginning of your upkeep, you may pay X life. If you do, untap X permanents.
Pay {1/2} life: Remove the top card of your library from the game face down. Put that card into your hand at end of turn.","BBB","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grafted Wargear","Equipped creature gets +3/+2.
Whenever Grafted Wargear becomes unattached from a permanent, sacrifice that permanent.
Equip {0} ({0}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maul Splicer","1","1","G","When Maul Splicer enters the battlefield, put two 3/3 colorless Golem artifact creature tokens onto the battlefield.
Golem creatures you control have trample.","6G","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rappelling Scouts","1","4","W","Flying
{2}{W}: Rappelling Scouts gains protection from the color of your choice until end of turn.","2WW","Creature - Human Rebel Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thunderclap","R","You may sacrifice a Mountain rather than pay Thunderclap's mana cost.
Thunderclap deals 3 damage to target creature.","2R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Orcish Squatters Avatar","At the beginning of your precombat main phase, add {X} to your mana pool, where X is the number of lands target opponent controls.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Venomous Breath","G","Choose target creature. At end of combat, destroy all creatures that blocked or were blocked by it this turn.","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodshot Trainee","2","3","R","{T}: Bloodshot Trainee deals 4 damage to target creature. Activate this ability only if Bloodshot Trainee's power is 4 or greater.","3R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu General","2","2","W","Vigilance; horsemanship (This creature can't be blocked except by creatures with horsemanship.)","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Sovereign","2","2","U","Other Merfolk creatures you control get +1/+1.
{T}: Target Merfolk creature is unblockable this turn.","1UU","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Offensive","R","Put X 1/1 red Goblin creature tokens onto the battlefield.","X1RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Green Scarab","W","Enchant creature
Enchanted creature can't be blocked by green creatures.
Enchanted creature gets +2/+2 as long as an opponent controls a green permanent.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sulfur Elemental","3","2","R","Flash (You may cast this spell any time you could cast an instant.)
Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
White creatures get +1/-1.","2R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elder of Laurels","2","3","G","{3}{G}: Target creature gets +X/+X until end of turn, where X is the number of creatures you control.","2G","Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stranglehold","R","Your opponents can't search libraries.
If an opponent would begin an extra turn, that player skips that turn instead.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Urza's Avenger","4","4","{0}: Urza's Avenger gets -1/-1 and gains your choice of banding, flying, first strike, or trample until end of turn. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","6","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fresh Meat","G","Put a 3/3 green Beast creature token onto the battlefield for each creature put into your graveyard from the battlefield this turn.","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ivy Dancer","1","2","G","{T}: Target creature gains forestwalk until end of turn.","2G","Creature - Dryad Shaman");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Overseer","0","0","At the beginning of your upkeep, put a +1/+1 counter on each creature with modular you control.
Modular 6 (This enters the battlefield with six +1/+1 counters on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","8","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blinding Powder","Equipped creature has ""Unattach Blinding Powder: Prevent all combat damage that would be dealt to this creature this turn.""
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sphere of Law","W","If a red source would deal damage to you, prevent 2 of that damage.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crystalline Sliver","2","2","WU","All Slivers have shroud. (They can't be the targets of spells or abilities.)","WU","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Warhawk","2","2","W","Amplify 1 (As this creature enters the battlefield, put a +1/+1 counter on it for each Bird and/or Soldier card you reveal in your hand.)
Flying","4W","Creature - Bird Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shell of the Last Kappa","{3}, {T}: Exile target instant or sorcery spell that targets you. (The spell has no effect.)
{3}, {T}, Sacrifice Shell of the Last Kappa: You may cast a card exiled with Shell of the Last Kappa without paying its mana cost.","3","Legendary Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Caterpillar","3","3","G","{G}, Sacrifice Giant Caterpillar: Put a 1/1 green Insect creature token with flying named Butterfly onto the battlefield at the beginning of the next end step.","3G","Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trigon of Mending","Trigon of Mending enters the battlefield with three charge counters on it.
{W}{W}, {T}: Put a charge counter on Trigon of Mending.
{2}, {T}, Remove a charge counter from Trigon of Mending: Target player gains 3 life.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knights of Thorn","2","2","W","Protection from red; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","3W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repel the Darkness","W","Tap up to two target creatures.
Draw a card.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moriok Rigger","2","2","B","Whenever an artifact is put into a graveyard from the battlefield, you may put a +1/+1 counter on Moriok Rigger.","2B","Creature - Human Rogue Rigger");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flash Foliage","G","Cast Flash Foliage only during combat after blockers are declared.
Put a 1/1 green Saproling creature token onto the battlefield blocking target creature attacking you.
Draw a card.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hollowsage","2","2","B","Whenever Hollowsage becomes untapped, you may have target player discard a card.","3B","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chameleon Spirit","*","*","U","As Chameleon Spirit enters the battlefield, choose a color.
Chameleon Spirit's power and toughness are each equal to the number of permanents of the chosen color your opponents control.","3U","Creature - Illusion Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raging Goblin","1","1","R","Haste (This creature can attack and {T} as soon as it comes under your control.)","R","Creature - Goblin Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Green Ward","W","Enchant creature
Enchanted creature has protection from green. This effect doesn't remove Green Ward.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Tyranny","UBR","Whenever a player draws a card, that player loses 2 life unless he or she pays {2}.","UBR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battle Hurda","3","3","W","First strike","4W","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elephant Resurgence","G","Each player puts a green Elephant creature token onto the battlefield. Those creatures have ""This creature's power and toughness are each equal to the number of creature cards in its controller's graveyard.""","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beseech the Queen","B","({(2/b)} can be paid with any two mana or with {B}. This card's converted mana cost is 6.)
Search your library for a card with converted mana cost less than or equal to the number of lands you control, reveal it, and put it into your hand. Then shuffle your library.","(2/B)(2/B)(2/B)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gibbering Hyenas","3","2","G","Gibbering Hyenas can't block black creatures.","2G","Creature - Hyena");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Carry Away","U","Enchant Equipment
When Carry Away enters the battlefield, unattach enchanted Equipment.
You control enchanted Equipment.","UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hull Breach","RG","Choose one - Destroy target artifact; or destroy target enchantment; or destroy target artifact and target enchantment.","RG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Predator, Flagship","{2}: Target creature gains flying until end of turn.
{5}, {T}: Destroy target creature with flying.","5","Legendary Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura of Silence","W","Artifact and enchantment spells your opponents cast cost {2} more to cast.
Sacrifice Aura of Silence: Destroy target artifact or enchantment.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inner-Chamber Guard","0","2","W","Bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)","1W","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zephyr Falcon","1","1","U","Flying, vigilance","1U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disperse","U","Return target nonland permanent to its owner's hand.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ring of Gix","Echo {3} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
{1}, {T}: Tap target artifact, creature, or land.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mage Slayer","RG","Whenever equipped creature attacks, it deals damage equal to its power to defending player.
Equip {3}","1RG","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Impromptu Raid","RG","{2}{(r/g)}: Reveal the top card of your library. If it isn't a creature card, put it into your graveyard. Otherwise, put that card onto the battlefield. That creature gains haste. Sacrifice it at the beginning of the next end step.","3(R/G)","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sun Quan, Lord of Wu","4","4","U","Creatures you control have horsemanship. (They can't be blocked except by creatures with horsemanship.)","4UU","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Teleport","U","Cast Teleport only during the declare attackers step.
Target creature is unblockable this turn.","UUU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prismatic Strands","W","Prevent all damage that sources of the color of your choice would deal this turn.
Flashback-Tap an untapped white creature you control. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slinking Serpent","2","3","UB","Forestwalk","2UB","Creature - Serpent");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Colossus of Sardia","9","9","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)
Colossus of Sardia doesn't untap during your untap step.
{9}: Untap Colossus of Sardia. Activate this ability only during your upkeep.","9","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Diviner's Wand","Equipped creature has ""Whenever you draw a card, this creature gets +1/+1 and gains flying until end of turn"" and ""{4}: Draw a card.""
Whenever a Wizard creature enters the battlefield, you may attach Diviner's Wand to it.
Equip {3}","3","Tribal Artifact - Wizard Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Credit Voucher","{2}, {T}, Sacrifice Credit Voucher: Shuffle any number of cards from your hand into your library, then draw that many cards.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hua Tuo, Honored Physician","1","2","G","{T}: Put target creature card from your graveyard on top of your library. Activate this ability only during your turn, before attackers are declared.","1GG","Legendary Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sonic Burst","R","As an additional cost to cast Sonic Burst, discard a card at random.
Sonic Burst deals 4 damage to target creature or player.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tideforce Elemental","2","1","U","{U}, {T}: You may tap or untap another target creature.
Landfall - Whenever a land enters the battlefield under your control, you may untap Tideforce Elemental.","2U","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wu Light Cavalry","1","2","U","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","1U","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fleet-Footed Monk","1","1","W","Fleet-Footed Monk can't be blocked by creatures with power 2 or greater.","1W","Creature - Human Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tukatongue Thallid","1","1","G","When Tukatongue Thallid dies, put a 1/1 green Saproling creature token onto the battlefield.","G","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conflagrate","R","Conflagrate deals X damage divided as you choose among any number of target creatures and/or players.
Flashback-{R}{R}, Discard X cards. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","XXR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tephraderm","4","5","R","Whenever a creature deals damage to Tephraderm, Tephraderm deals that much damage to that creature.
Whenever a spell deals damage to Tephraderm, Tephraderm deals that much damage to that spell's controller.","4R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tangle","G","Prevent all combat damage that would be dealt this turn.
Each attacking creature doesn't untap during its controller's next untap step.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelic Destiny","W","Enchant creature
Enchanted creature gets +4/+4, has flying and first strike, and is an Angel in addition to its other types.
When enchanted creature dies, return Angelic Destiny to its owner's hand.","2WW","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trumpet Blast","R","Attacking creatures get +2/+0 until end of turn.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Garruk's Horde","7","7","G","Trample
Play with the top card of your library revealed.
You may cast the top card of your library if it's a creature card. (Do this only any time you could cast that creature card. You still pay the spell's costs.)","5GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Solitary Confinement","W","At the beginning of your upkeep, sacrifice Solitary Confinement unless you discard a card.
Skip your draw step.
You have shroud. (You can't be the target of spells or abilities.)
Prevent all damage that would be dealt to you.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flash Conscription","R","Untap target creature and gain control of it until end of turn.  That creature gains haste until end of turn. If {W} was spent to cast Flash Conscription, the creature gains ""Whenever this creature deals combat damage, you gain that much life"" until end of turn.","5R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Academy Researchers","2","2","U","When Academy Researchers enters the battlefield, you may put an Aura card from your hand onto the battlefield attached to Academy Researchers.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Demonmail Hauberk","Equipped creature gets +4/+2.
Equip-Sacrifice a creature.","4","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yawgmoth's Bargain","B","Skip your draw step.
Pay 1 life: Draw a card.","4BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inheritance","W","Whenever a creature dies, you may pay {3}. If you do, draw a card.","W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blue Mana Battery","{2}, {T}: Put a charge counter on Blue Mana Battery.
{T}, Remove any number of charge counters from Blue Mana Battery: Add {U} to your mana pool, then add an additional {U} to your mana pool for each charge counter removed this way.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jungle Shrine","Jungle Shrine enters the battlefield tapped.
{T}: Add {R}, {G}, or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Totem-Guide Hartebeest","2","5","W","When Totem-Guide Hartebeest enters the battlefield, you may search your library for an Aura card, reveal it, put it into your hand, then shuffle your library.","4W","Creature - Antelope");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellcarver Demon","6","6","B","Flying
Whenever Hellcarver Demon deals combat damage to a player, sacrifice all other permanents you control and discard your hand. Exile the top six cards of your library. You may cast any number of nonland cards exiled this way without paying their mana costs.","3BBB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Survivor","2","1","U","Morph {1}{U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Riptide Survivor is turned face up, discard two cards, then draw three cards.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Sky Raider","1","2","R","Flying (This creature can't be blocked except by creatures with flying or reach.)","2R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anarchist","2","2","R","When Anarchist enters the battlefield, you may return target sorcery card from your graveyard to your hand.","4R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Staying Power","W","As long as Staying Power is in play, ""until end of turn"" and ""this turn"" effects don't end.","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arena","{3}, {T}: Tap target creature you control and target creature of an opponent's choice he or she controls. Those creatures fight each other. (Each deals damage equal to its power to the other.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Furnace Brood","3","3","R","{R}: Target creature can't be regenerated this turn.","3R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Takeno, Samurai General","3","3","W","Bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)
Each other Samurai creature you control gets +1/+1 for each point of bushido it has.","5W","Legendary Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necrosavant","5","5","B","{3}{B}{B}, Sacrifice a creature: Return Necrosavant from your graveyard to the battlefield. Activate this ability only during your upkeep.","3BBB","Creature - Zombie Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Twist","B","Target player discards X cards at random.","XB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scoria Cat","3","3","R","Scoria Cat gets +3/+3 as long as you control no untapped lands.","3RR","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vesuvan Shapeshifter","0","0","U","As Vesuvan Shapeshifter enters the battlefield or is turned face up, you may choose another creature on the battlefield. If you do, until Vesuvan Shapeshifter is turned face down, it becomes a copy of that creature and gains ""At the beginning of your upkeep, you may turn this creature face down.""
Morph {1}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3UU","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightsky Mimic","2","1","WB","Whenever you cast a spell that's both white and black, Nightsky Mimic becomes 4/4 and gains flying until end of turn.","1(W/B)","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Cadets","2","1","R","Whenever Goblin Cadets blocks or becomes blocked, target opponent gains control of it. (This removes Goblin Cadets from combat.)","R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Neurok Spy","2","2","U","Neurok Spy is unblockable as long as defending player controls an artifact.","2U","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lumengrid Gargoyle","4","4","Flying","6","Artifact Creature - Gargoyle");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Withering","B","Destroy target nonblack creature.
Madness {B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","4BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Runic Repetition","U","Return target exiled card with flashback you own to your hand.","2U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Coat of Arms","Each creature gets +1/+1 for each other creature on the battlefield that shares at least one creature type with it. (For example, if two Goblin Warriors and a Goblin Shaman are on the battlefield, each gets +2/+2.)","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Martyr's Cause","W","Sacrifice a creature: The next time a source of your choice would deal damage to target creature or player this turn, prevent that damage.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathcurse Ogre","3","3","B","When Deathcurse Ogre dies, each player loses 3 life.","5B","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anaconda","3","3","G","Swampwalk (This creature is unblockable as long as defending player controls a Swamp.)","3G","Creature - Snake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mogg Cannon","{T}: Target creature you control gets +1/+0 and gains flying until end of turn. Destroy that creature at the beginning of the next end step.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sangromancer","3","3","B","Flying
Whenever a creature an opponent controls dies, you may gain 3 life.
Whenever an opponent discards a card, you may gain 3 life.","2BB","Creature - Vampire Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Xanthic Statue","{5}: Until end of turn, Xanthic Statue becomes an 8/8 Golem artifact creature with trample.","8","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Angelheart Vial","Whenever you're dealt damage, you may put that many charge counters on Angelheart Vial.
{2}, {T}, Remove four charge counters from Angelheart Vial: You gain 2 life and draw a card.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glyph of Delusion","U","Put X glyph counters on target creature that target Wall blocked this turn, where X is the power of that blocked creature. The creature gains ""This creature doesn't untap during your untap step if it has a glyph counter on it"" and ""At the beginning of your upkeep, remove a glyph counter from this creature.""","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bitterblossom","B","At the beginning of your upkeep, you lose 1 life and put a 1/1 black Faerie Rogue creature token with flying onto the battlefield.","1B","Tribal Enchantment - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flooded Shoreline","U","{U}{U}, Return two Islands you control to their owner's hand: Return target creature to its owner's hand.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Steel Hellkite","5","5","Flying
{2}: Steel Hellkite gets +1/+0 until end of turn.
{X}: Destroy each nonland permanent with converted mana cost X whose controller was dealt combat damage by Steel Hellkite this turn. Activate this ability only once each turn.","6","Artifact Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("The Brute","R","Enchant creature
Enchanted creature gets +1/+0.
{R}{R}{R}: Regenerate enchanted creature.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodrage Vampire","3","1","B","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)","2B","Creature - Vampire");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("High Market","{T}: Add {1} to your mana pool.
{T}, Sacrifice a creature: You gain 1 life.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eye for an Eye","W","The next time a source of your choice would deal damage to you this turn, instead that source deals that much damage to you and Eye for an Eye deals that much damage to that source's controller.","WW","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sunpetal Grove","Sunpetal Grove enters the battlefield tapped unless you control a Forest or a Plains.
{T}: Add {G} or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Stroke","B","Destroy target tapped creature.","BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rune Snag","U","Counter target spell unless its controller pays {2} plus an additional {2} for each card named Rune Snag in each graveyard.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ensnaring Bridge","Creatures with power greater than the number of cards in your hand can't attack.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("New Frontiers","G","Each player may search his or her library for up to X basic land cards and put them onto the battlefield tapped. Then each player who searched his or her library this way shuffles it.","XG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rainbow Vale","{T}: Add one mana of any color to your mana pool. An opponent gains control of Rainbow Vale at the beginning of the next end step.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jungle Weaver","5","6","G","Reach (This can block creatures with flying.)
Cycling {2} ({2}, Discard this card: Draw a card.)","5GG","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("D'Avenant Healer","1","2","W","{T}: D'Avenant Healer deals 1 damage to target attacking or blocking creature.
{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","1WW","Creature - Human Cleric Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunspring Expedition","W","Landfall - Whenever a land enters the battlefield under your control, you may put a quest counter on Sunspring Expedition.
Remove three quest counters from Sunspring Expedition and sacrifice it: You gain 8 life.","W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soratami Mindsweeper","1","4","U","Flying
{2}, Return a land you control to its owner's hand: Target player puts the top two cards of his or her library into his or her graveyard.","3U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bottomless Vault","Bottomless Vault enters the battlefield tapped.
You may choose not to untap Bottomless Vault during your untap step.
At the beginning of your upkeep, if Bottomless Vault is tapped, put a storage counter on it.
{T}, Remove any number of storage counters from Bottomless Vault: Add {B} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rustspore Ram","1","3","When Rustspore Ram enters the battlefield, destroy target Equipment.","4","Artifact Creature - Sheep");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Warrior","2","3","G",NULL,"GG","Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rune of Protection: Black","W","{W}: The next time a black source of your choice would deal damage to you this turn, prevent that damage.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vigean Hydropon","0","0","UG","Graft 5 (This creature enters the battlefield with five +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
Vigean Hydropon can't attack or block.","1GU","Creature - Plant Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Entropic Specter","*","*","B","Flying
As Entropic Specter enters the battlefield, choose an opponent.
Entropic Specter's power and toughness are each equal to the number of cards in the chosen player's hand.
Whenever Entropic Specter deals damage to a player, that player discards a card.","3BB","Creature - Specter Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Battle Hymn","R","Add {R} to your mana pool for each creature you control.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sickle Ripper","2","1","B","Wither (This deals damage to creatures in the form of -1/-1 counters.)","1B","Creature - Elemental Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pygmy Troll","1","1","G","Whenever Pygmy Troll becomes blocked by a creature, Pygmy Troll gets +1/+1 until end of turn.
{G}: Regenerate Pygmy Troll.","1G","Creature - Troll");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbarian Riftcutter","3","3","R","{R}, Sacrifice Barbarian Riftcutter: Destroy target land.","4R","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sakura-Tribe Springcaller","2","4","G","At the beginning of your upkeep, add {G} to your mana pool. Until end of turn, this mana doesn't empty from your mana pool as steps and phases end.","3G","Creature - Snake Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Izzet Steam Maze","Whenever a player casts an instant or sorcery spell, that player copies it. The player may choose new targets for the copy.
Whenever you roll {C}, instant and sorcery spells you cast this turn cost {3} less to cast.",NULL,"Plane - Ravnica");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Geistflame","R","Geistflame deals 1 damage to target creature or player.
Flashback {3}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zealous Persecution","WB","Until end of turn, creatures you control get +1/+1 and creatures your opponents control get -1/-1.","WB","Instant");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Liliana Vess","B","+1: Target player discards a card.
-2: Search your library for a card, then shuffle your library and put that card on top of it.
-8: Put all creature cards from all graveyards onto the battlefield under your control.","Planeswalker - Liliana","3BB");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stinging Licid","1","1","U","{1}{U}, {T}: Stinging Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {U} to end this effect.
Whenever enchanted creature becomes tapped, Stinging Licid deals 2 damage to that creature's controller.","1U","Creature - Licid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eerie Procession","U","Search your library for an Arcane card, reveal that card, and put it into your hand. Then shuffle your library.","2U","Sorcery - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thundering Tanadon","5","4","G","({(g/p)} can be paid with either {G} or 2 life.)
Trample","4(G/P)(G/P)","Artifact Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sigil of the New Dawn","W","Whenever a creature is put into your graveyard from the battlefield, you may pay {1}{W}. If you do, return that card to your hand.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Commandeer","U","You may exile two blue cards from your hand rather than pay Commandeer's mana cost.
Gain control of target noncreature spell. You may choose new targets for it. (If that spell is an artifact, enchantment, or planeswalker, the permanent enters the battlefield under your control.)","5UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cyclical Evolution","G","Target creature gets +3/+3 until end of turn. Exile Cyclical Evolution with three time counters on it.
Suspend 3-{2}{G} (Rather than cast this card from your hand, you may pay {2}{G} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","3GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Beacon of Destiny","1","3","W","{T}: The next time a source of your choice would deal damage to you this turn, that damage is dealt to Beacon of Destiny instead.","1W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sosuke's Summons","G","Put two 1/1 green Snake creature tokens onto the battlefield.
Whenever a nontoken Snake enters the battlefield under your control, you may return Sosuke's Summons from your graveyard to your hand.","2G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Graven Cairns","{T}: Add {1} to your mana pool.
{(b/r)}, {T}: Add {B}{B}, {B}{R}, or {R}{R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Beast","1","1","G","Threshold - Krosan Beast gets +7/+7 as long as seven or more cards are in your graveyard.","3G","Creature - Squirrel Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Smoldering Butcher","4","2","B","Wither (This deals damage to creatures in the form of -1/-1 counters.)","3B","Creature - Elemental Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nightmare Incursion","B","Search target player's library for up to X cards, where X is the number of Swamps you control, and exile them. Then that player shuffles his or her library.","5B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Macetail Hystrodon","4","4","R","First strike, haste
Cycling {3} ({3}, Discard this card: Draw a card.)","6R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dovescape","WU","({(w/u)} can be paid with either {W} or {U}.)
Whenever a player casts a noncreature spell, counter that spell. That player puts X 1/1 white and blue Bird creature tokens with flying onto the battlefield, where X is the spell's converted mana cost.","3(W/U)(W/U)(W/U)","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Roots","0","5","G","Defender
Put a -0/-1 counter on Wall of Roots: Add {G} to your mana pool. Activate this ability only once each turn.","1G","Creature - Plant Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormscape Master","2","2","U","{W}{W}, {T}: Target creature gains protection from the color of your choice until end of turn.
{B}{B}, {T}: Target player loses 2 life and you gain 2 life.","2UU","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steady Progress","U","Proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)
Draw a card.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Centaur Veteran","3","3","G","Trample
{G}, Discard a card: Regenerate Centaur Veteran.","5G","Creature - Centaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harbinger of Spring","2","1","G","Protection from non-Spirit creatures
Soulshift 4 (When this creature dies, you may return target Spirit card with converted mana cost 4 or less from your graveyard to your hand.)","4G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infinite Authority","W","Enchant creature
Whenever enchanted creature blocks or becomes blocked by a creature with toughness 3 or less, destroy the other creature at end of combat. At the beginning of the next end step, if that creature was destroyed this way, put a +1/+1 counter on the first creature.","WWW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phobian Phantasm","3","3","B","Flying; fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Cumulative upkeep {B} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","1BB","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Polar Kraken","11","11","U","Trample
Polar Kraken enters the battlefield tapped.
Cumulative upkeep-Sacrifice a land. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","8UUU","Creature - Kraken");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gigantiform","G","Kicker {4}
Enchant creature
Enchanted creature is 8/8 and has trample.
When Gigantiform enters the battlefield, if it was kicked, you may search your library for a card named Gigantiform, put it onto the battlefield, then shuffle your library.","3GG","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dwarven Shrine","R","Whenever a player casts a spell, Dwarven Shrine deals X damage to that player, where X is twice the number of cards in all graveyards with the same name as that spell.","1RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight-Captain of Eos","2","2","W","When Knight-Captain of Eos enters the battlefield, put two 1/1 white Soldier creature tokens onto the battlefield.
{W}, Sacrifice a Soldier: Prevent all combat damage that would be dealt this turn.","4W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fervor","R","Creatures you control have haste.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Distortion","1","3","B","Defender (This creature can't attack.)
{2}{B}, {T}: Target player discards a card. Activate this ability only any time you could cast a sorcery.","2BB","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armored Galleon","5","4","U","Armored Galleon can't attack unless defending player controls an Island.","4U","Creature - Human Pirate");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seizan, Perverter of Truth","6","5","B","At the beginning of each player's upkeep, that player loses 2 life and draws two cards.","3BB","Legendary Creature - Demon Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Atogatog","5","5","WUBRG","Sacrifice an Atog creature: Atogatog gets +X/+X until end of turn, where X is the sacrificed creature's power.","WUBRG","Legendary Creature - Atog");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mox Sapphire","{T}: Add {U} to your mana pool.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quicksilver Wall","1","6","U","Defender (This creature can't attack.)
{4}: Return Quicksilver Wall to its owner's hand. Any player may activate this ability.","2U","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Runner","3","3","R","Kavu Runner has haste as long as no opponent controls a white or blue creature.","3R","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spinal Villain","1","2","R","{T}: Destroy target blue creature.","2R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dimensional Breach","W","Exile all permanents. For as long as any of those cards remain exiled, at the beginning of each player's upkeep, that player returns one of the exiled cards he or she owns to the battlefield.","5WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Belfry Spirit","1","1","W","Flying
Haunt (When this creature dies, exile it haunting target creature.)
When Belfry Spirit enters the battlefield or the creature it haunts dies, put two 1/1 black Bat creature tokens with flying onto the battlefield.","3WW","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ruhan of the Fomori","7","7","WUR","At the beginning of combat on your turn, choose an opponent at random. Ruhan of the Fomori attacks that player this combat if able.","1RWU","Legendary Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harabaz Druid","0","1","G","{T}: Add X mana of any one color to your mana pool, where X is the number of Allies you control.","1G","Creature - Human Druid Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Norwood Priestess","1","1","G","{T}: You may put a green creature card from your hand onto the battlefield. Activate this ability only during your turn, before attackers are declared.","2GG","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lashknife","W","If you control a Plains, you may tap an untapped creature you control rather than pay Lashknife's mana cost.
Enchant creature
Enchanted creature has first strike.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exile into Darkness","B","Target player sacrifices a creature with converted mana cost 3 or less.
At the beginning of your upkeep, if you have more cards in hand than each opponent, you may return Exile into Darkness from your graveyard to your hand.","4B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squirrel Nest","G","Enchant land
Enchanted land has ""{T}: Put a 1/1 green Squirrel creature token onto the battlefield.""","1GG","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azimaet Drake","1","3","U","Flying
{U}: Azimaet Drake gets +1/+0 until end of turn. Activate this ability only once each turn.","2U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Fracture","W","Sacrifice a land: Destroy target enchantment.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harsh Justice","W","Cast Harsh Justice only during the declare attackers step and only if you've been attacked this step.
This turn, whenever an attacking creature deals combat damage to you, it deals that much damage to its controller.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Major Teroh","2","3","W","Flying
{3}{W}{W}, Sacrifice Major Teroh: Exile all black creatures.","3W","Legendary Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wretched Banquet","B","Destroy target creature if it has the least power or is tied for least power among creatures on the battlefield.","B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirrorworks","Whenever another nontoken artifact enters the battlefield under your control, you may pay {2}. If you do, put a token that's a copy of that artifact onto the battlefield.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Young Wolf","1","1","G","Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","G","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flamewave Invoker","2","2","R","{7}{R}: Flamewave Invoker deals 5 damage to target player.","2R","Creature - Goblin Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slashing Tiger","3","3","G","Whenever Slashing Tiger becomes blocked, it gets +2/+2 until end of turn.","2GG","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Intervention","W","Divine Intervention enters the battlefield with two intervention counters on it.
At the beginning of your upkeep, remove an intervention counter from Divine Intervention.
When you remove the last intervention counter from Divine Intervention, the game is a draw.","6WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heart Sliver","1","1","R","All Sliver creatures have haste.","1R","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cabal Ritual","B","Add {B}{B}{B} to your mana pool.
Threshold - Add {B}{B}{B}{B}{B} to your mana pool instead if seven or more cards are in your graveyard.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Link","WB","Enchant creature
Whenever enchanted creature deals damage, you gain that much life.
Whenever enchanted creature is dealt damage, you gain that much life.","1WB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Flash","R","Whenever a creature enters the battlefield, Æther Flash deals 2 damage to it.","2RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skeleton Scavengers","0","0","B","Skeleton Scavengers enters the battlefield with a +1/+1 counter on it.
Pay {1} for each +1/+1 counter on Skeleton Scavengers: Regenerate Skeleton Scavengers. When it regenerates this way, put a +1/+1 counter on it.","2B","Creature - Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cyclopean Giant","4","2","B","When Cyclopean Giant dies, target land becomes a Swamp. Exile Cyclopean Giant.","2BB","Creature - Zombie Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wizened Cenn","2","2","W","Other Kithkin creatures you control get +1/+1.","WW","Creature - Kithkin Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shattering Spree","R","Replicate {R} (When you cast this spell, copy it for each time you paid its replicate cost. You may choose new targets for the copies.)
Destroy target artifact.","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Verdant Field","G","Enchant land
Enchanted land has ""{T}: Target creature gets +1/+1 until end of turn.""","2G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cobbled Wings","Equipped creature has flying.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Advanced Hoverguard","2","2","U","Flying
{U}: Advanced Hoverguard gains shroud until end of turn. (It can't be the target of spells or abilities.)","3U","Creature - Drone");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crag Puca","2","4","UR","{(u/r)}: Switch Crag Puca's power and toughness until end of turn.","(U/R)(U/R)(U/R)","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Consumptive Goo","1","1","B","{2}{B}{B}: Target creature gets -1/-1 until end of turn. Put a +1/+1 counter on Consumptive Goo.","BB","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Act of Aggression","R","({(r/p)} can be paid with either {R} or 2 life.)
Gain control of target creature an opponent controls until end of turn. Untap that creature. It gains haste until end of turn.","3(R/P)(R/P)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aphetto Alchemist","1","2","U","{T}: Untap target artifact or creature.
Morph {U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hazerider Drake","2","3","WU","Flying, protection from red","2WU","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Basalt Gargoyle","3","2","R","Flying
Echo {2}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
{R}: Basalt Gargoyle gets +0/+1 until end of turn.","2R","Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Archangel of Strife","6","6","W","Flying
As Archangel of Strife enters the battlefield, each player chooses war or peace.
Creatures controlled by players who chose war get +3/+0.
Creatures controlled by players who chose peace get +0/+3.","5WW","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Batterskull","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +4/+4 and has vigilance and lifelink.
{3}: Return Batterskull to its owner's hand.
Equip {5}","5","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mossbridge Troll","5","5","G","If Mossbridge Troll would be destroyed, regenerate it.
Tap any number of untapped creatures you control other than Mossbridge Troll with total power 10 or greater: Mossbridge Troll gets +20/+20 until end of turn.","5GG","Creature - Troll");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diaochan, Artful Beauty","1","1","R","{T}: Destroy target creature of your choice, then destroy target creature of an opponent's choice. Activate this ability only during your turn, before attackers are declared.","3R","Legendary Creature - Human Advisor");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vanguard's Shield","Equipped creature gets +0/+3 and can block an additional creature.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hada Spy Patrol","1","1","U","Level up {2}{U} ({2}{U}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-2
2/2
Hada Spy Patrol is unblockable.
LEVEL 3+
3/3
Shroud
Hada Spy Patrol is unblockable.","1U","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infernal Caretaker","2","2","B","Morph {3}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Infernal Caretaker is turned face up, return all Zombie cards from all graveyards to their owners' hands.","3B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scattering Stroke","U","Counter target spell. Clash with an opponent. If you win, at the beginning of your next main phase, you may add {X} to your mana pool, where X is that spell's converted mana cost. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","2UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stamina","G","Enchant creature
Enchanted creature has vigilance.
Sacrifice Stamina: Regenerate enchanted creature.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crowd Favorites","4","4","W","{3}{W}: Tap target creature.
{3}{W}: Crowd Favorites gets +0/+5 until end of turn.","6W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Null Chamber","W","As Null Chamber enters the battlefield, you and an opponent each name a card other than a basic land card.
The named cards can't be played.","3W","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corpse Dance","B","Buyback {2} (You may pay an additional {2} as you cast this spell. If you do, put this card into your hand as it resolves.)
Return the top creature card of your graveyard to the battlefield. That creature gains haste until end of turn. Exile it at the beginning of the next end step.","2B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tawnos","You may cast artifact, creature, and enchantment spells as though they had flash.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arcbound Overseer Avatar","At the beginning of your upkeep, you may put a +1/+1 counter on target creature you control.
At the beginning of your upkeep, you may put a charge counter on target permanent you control.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treasure Mage","2","2","U","When Treasure Mage enters the battlefield, you may search your library for an artifact card with converted mana cost 6 or greater, reveal that card, and put it into your hand. If you do, shuffle your library.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heart Wolf","2","2","R","First strike
{T}: Target Dwarf creature gets +2/+0 and gains first strike until end of turn. When that creature leaves the battlefield this turn, sacrifice Heart Wolf. Activate this ability only during combat.","3R","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spawnbroker","1","1","U","When Spawnbroker enters the battlefield, you may exchange control of target creature you control and target creature with power less than or equal to that creature's power an opponent controls.","2U","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skarrg, the Rage Pits","{T}: Add {1} to your mana pool.
{R}{G}, {T}: Target creature gets +1/+1 and gains trample until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timberland Guide","1","1","G","When Timberland Guide enters the battlefield, put a +1/+1 counter on target creature.","1G","Creature - Human Scout");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Nicol Bolas, Planeswalker","UBR","+3: Destroy target noncreature permanent.
-2: Gain control of target creature.
-9: Nicol Bolas, Planeswalker deals 7 damage to target player. That player discards seven cards, then sacrifices seven permanents.","Planeswalker - Bolas","4UBBR");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Territorial Baloth","4","4","G","Landfall - Whenever a land enters the battlefield under your control, Territorial Baloth gets +2/+2 until end of turn.","4G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Craw Wurm","6","4","G",NULL,"4GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tree of Redemption","0","13","G","Defender
{T}: Exchange your life total with Tree of Redemption's toughness.","3G","Creature - Plant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vile Requiem","B","At the beginning of your upkeep, you may put a verse counter on Vile Requiem.
{1}{B}, Sacrifice Vile Requiem: Destroy up to X target nonblack creatures, where X is the number of verse counters on Vile Requiem. They can't be regenerated.","2BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squee's Revenge","UR","Choose a number. Flip a coin that many times or until you lose a flip, whichever comes first. If you win all the flips, draw two cards for each flip.","1UR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Maw of the Mire","B","Destroy target land. You gain 4 life.","4B","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Otarian Juggernaut","2","3","Otarian Juggernaut can't be blocked by Walls.
Threshold - As long as seven or more cards are in your graveyard, Otarian Juggernaut gets +3/+0 and attacks each turn if able.","4","Artifact Creature - Juggernaut");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trolls of Tel-Jilad","5","6","G","{1}{G}: Regenerate target green creature.","5GG","Creature - Troll Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Double Dip","W","Choose another player. Gain 5 life now and an additional 5 life at the beginning of the next game with that player.","4W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maalfeld Twins","4","4","B","When Maalfeld Twins dies, put two 2/2 black Zombie creature tokens onto the battlefield.","5B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stronghold Zeppelin","3","3","U","Flying
Stronghold Zeppelin can block only creatures with flying.","2UU","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Virulent Swipe","B","Target creature gets +2/+0 and gains deathtouch until end of turn.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cave-In","R","You may exile a red card from your hand rather than pay Cave-In's mana cost.
Cave-In deals 2 damage to each creature and each player.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Tunneler","1","1","R","{T}: Target creature with power 2 or less is unblockable this turn.","1R","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Griffin Rider","1","1","W","As long as you control a Griffin creature, Griffin Rider gets +3/+3 and has flying.","1W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mangara's Equity","W","As Mangara's Equity enters the battlefield, choose black or red.
At the beginning of your upkeep, sacrifice Mangara's Equity unless you pay {1}{W}.
Whenever a creature of the chosen color deals damage to you or a white creature you control, Mangara's Equity deals that much damage to that creature.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Empath","2","2","G","When Llanowar Empath enters the battlefield, scry 2, then reveal the top card of your library. If it's a creature card, put it into your hand. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","3G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Gideon Jura","W","+2: During target opponent's next turn, creatures that player controls attack Gideon Jura if able.
-2: Destroy target tapped creature.
0: Until end of turn, Gideon Jura becomes a 6/6 Human Soldier creature that's still a planeswalker. Prevent all damage that would be dealt to him this turn.","Planeswalker - Gideon","3WW");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhang Liao, Hero of Hefei","3","3","B","Whenever Zhang Liao, Hero of Hefei deals damage to an opponent, that opponent discards a card.","4BB","Legendary Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jabari's Banner","{1}, {T}: Target creature gains flanking until end of turn. (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wonder","2","2","U","Flying
As long as Wonder is in your graveyard and you control an Island, creatures you control have flying.","3U","Creature - Incarnation");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vebulid","0","0","B","Vebulid enters the battlefield with a +1/+1 counter on it.
At the beginning of your upkeep, you may put a +1/+1 counter on Vebulid.
When Vebulid attacks or blocks, destroy it at end of combat.","B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cleansing","W","For each land, destroy that land unless any player pays 1 life.","WWW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Erosion","U","Enchant land
At the beginning of the upkeep of enchanted land's controller, destroy that land unless that player pays {1} or 1 life.","UUU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Delver of Secrets","1","1","U","At the beginning of your upkeep, look at the top card of your library. You may reveal that card. If an instant or sorcery card is revealed this way, transform Delver of Secrets.","U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glorious Anthem","W","Creatures you control get +1/+1.","1WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Throne of Empires","{1}, {T}: Put a 1/1 white Soldier creature token onto the battlefield. Put five of those tokens onto the battlefield instead if you control artifacts named Crown of Empires and Scepter of Empires.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Research the Deep","U","Draw a card. Clash with an opponent. If you win, return Research the Deep to its owner's hand. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloud of Faeries","1","1","U","Flying
When Cloud of Faeries enters the battlefield, untap up to two lands.
Cycling {2} ({2}, Discard this card: Draw a card.)","1U","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Word of Command","B","Look at target opponent's hand and choose a card from it. You control that player until Word of Command finishes resolving. The player plays that card if able. While doing so, the player can activate mana abilities only if they're from lands he or she controls and only if mana they produce is spent to activate other mana abilities of lands he or she controls and/or play that card. If the chosen card is cast as a spell, you control the player while that spell is resolving.","BB","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Hulk","5","4",NULL,"6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Squatters","2","3","R","Whenever Orcish Squatters attacks and isn't blocked, you may gain control of target land defending player controls for as long as you control Orcish Squatters. If you do, Orcish Squatters assigns no combat damage this turn.","4R","Creature - Orc");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hideous Laughter","B","All creatures get -2/-2 until end of turn.
Splice onto Arcane {3}{B}{B} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","2BB","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saprazzan Raider","1","2","U","When Saprazzan Raider becomes blocked, return it to its owner's hand.","2U","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blinding Beam","W","Choose one - Tap two target creatures; or creatures don't untap during target player's next untap step.
Entwine {1} (Choose both if you pay the entwine cost.)","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plagiarize","U","Until end of turn, if target player would draw a card, instead that player skips that draw and you draw a card.","3U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reckless Abandon","R","As an additional cost to cast Reckless Abandon, sacrifice a creature.
Reckless Abandon deals 4 damage to target creature or player.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rith, the Awakener","6","6","WRG","Flying
Whenever Rith, the Awakener deals combat damage to a player, you may pay {2}{G}. If you do, choose a color, then put a 1/1 green Saproling creature token onto the battlefield for each permanent of that color.","3RGW","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Exalted Angel","4","5","W","Flying
Whenever Exalted Angel deals damage, you gain that much life.
Morph {2}{W}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hide // Seek","WR","Put target artifact or enchantment on the bottom of its owner's library.
---
Search target opponent's library for a card and exile it. You gain life equal to its converted mana cost. Then that player shuffles his or her library.","RW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepwood Elder","2","2","G","{X}{G}{G}, {T}, Discard a card: X target lands become Forests until end of turn.","GG","Creature - Dryad Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Metathran Soldier","1","1","U","Metathran Soldier is unblockable.","1U","Creature - Metathran Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necra Disciple","1","1","B","{G}, {T}: Add one mana of any color to your mana pool.
{W}, {T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sirocco","R","Target player reveals his or her hand. For each blue instant card revealed this way, that player discards that card unless he or she pays 4 life.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ichneumon Druid","1","1","G","Whenever an opponent casts an instant spell other than the first instant spell that player casts each turn, Ichneumon Druid deals 4 damage to him or her.","1GG","Creature - Human Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Well of Life","At the beginning of your end step, if you control no untapped lands, you gain 2 life.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshooter","1","2","G","Reach (This creature can block creatures with flying.)
{T}, Sacrifice Skyshooter: Destroy target attacking or blocking creature with flying.","1G","Creature - Centaur Archer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Evermind","(Nonexistent mana costs can't be paid.)
Draw a card.
Splice onto Arcane {1}{U} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)",NULL,"Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scourge of Geier Reach","3","3","R","Scourge of Geier Reach gets +1/+1 for each creature your opponents control.","3RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plague Dogs","3","3","B","When Plague Dogs dies, all creatures get -1/-1 until end of turn.
{2}, Sacrifice Plague Dogs: Draw a card.","4B","Creature - Zombie Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Pulse","B","Target creature gets -4/-4 until end of turn.
Cycling {1}{B}{B} ({1}{B}{B}, Discard this card: Draw a card.)
When you cycle Death Pulse, you may have target creature get -1/-1 until end of turn.","2BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Price of Glory","R","Whenever a player taps a land for mana, if it's not that player's turn, destroy that land.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leafdrake Roost","UG","Enchant land
Enchanted land has ""{G}{U}, {T}: Put a 2/2 green and blue Drake creature token with flying onto the battlefield.""","3GU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dega Sanctuary","W","At the beginning of your upkeep, if you control a black or red permanent, you gain 2 life. If you control a black permanent and a red permanent, you gain 4 life instead.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodthrone Vampire","1","1","B","Sacrifice a creature: Bloodthrone Vampire gets +2/+2 until end of turn.","1B","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tidal Control","U","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Pay 2 life or {2}: Counter target red or green spell. Any player may activate this ability.","1UU","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blight Sickle","Equipped creature gets +1/+0 and has wither. (It deals damage to creatures in the form of -1/-1 counters.)
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crypt Rats","1","1","B","{X}: Crypt Rats deals X damage to each creature and each player. Spend only black mana this way.","2B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("River Kaijin","1","4","U",NULL,"2U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scrapskin Drake","2","3","U","Flying
Scrapskin Drake can block only creatures with flying.","2U","Creature - Zombie Drake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bog Wreckage","Bog Wreckage enters the battlefield tapped.
{T}: Add {B} to your mana pool.
{T}, Sacrifice Bog Wreckage: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chainflinger","2","2","R","{1}{R}, {T}: Chainflinger deals 1 damage to target creature or player.
Threshold - {2}{R}, {T}: Chainflinger deals 2 damage to target creature or player. Activate this ability only if seven or more cards are in your graveyard.","3R","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Paradise Plume","As Paradise Plume enters the battlefield, choose a color.
Whenever a player casts a spell of the chosen color, you may gain 1 life.
{T}: Add one mana of the chosen color to your mana pool.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Winged Sliver","1","1","U","All Sliver creatures have flying.","1U","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unifying Theory","U","Whenever a player casts a spell, that player may pay {2}. If the player does, he or she draws a card.","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Patrol","4","2","R","Dwarven Patrol doesn't untap during your untap step.
Whenever you cast a nonred spell, untap Dwarven Patrol.","2R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turbulent Dreams","U","As an additional cost to cast Turbulent Dreams, discard X cards.
Return X target nonland permanents to their owners' hands.","UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Protective Bubble","U","Enchant creature
Enchanted creature is unblockable and has shroud. (It can't be the target of spells or abilities.)","3U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hedron Scrabbler","1","1","Landfall - Whenever a land enters the battlefield under your control, Hedron Scrabbler gets +1/+1 until end of turn.","2","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Well of Lost Dreams","Whenever you gain life, you may pay {X}, where X is less than or equal to the amount of life you gained. If you do, draw X cards.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burr Grafter","2","2","G","Sacrifice Burr Grafter: Target creature gets +2/+2 until end of turn.
Soulshift 3 (When this creature dies, you may return target Spirit card with converted mana cost 3 or less from your graveyard to your hand.)","3G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rescind","U","Return target permanent to its owner's hand.
Cycling {2} ({2}, Discard this card: Draw a card.)","1UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mortal Wound","G","Enchant creature
When enchanted creature is dealt damage, destroy it.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nim Devourer","4","1","B","Nim Devourer gets +1/+0 for each artifact you control.
{B}{B}: Return Nim Devourer from your graveyard to the battlefield, then sacrifice a creature. Activate this ability only during your upkeep.","3BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guul Draz Specter","2","2","B","Flying
Guul Draz Specter gets +3/+3 as long as an opponent has no cards in hand.
Whenever Guul Draz Specter deals combat damage to a player, that player discards a card.","2BB","Creature - Specter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mikokoro, Center of the Sea","{T}: Add {1} to your mana pool.
{2}, {T}: Each player draws a card.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seal of Cleansing","W","Sacrifice Seal of Cleansing: Destroy target artifact or enchantment.","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jinxed Ring","Whenever a nontoken permanent is put into your graveyard from the battlefield, Jinxed Ring deals 1 damage to you.
Sacrifice a creature: Target opponent gains control of Jinxed Ring. (This effect lasts indefinitely.)","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sensei Golden-Tail","2","1","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
{1}{W}, {T}: Put a training counter on target creature. That creature gains bushido 1 and becomes a Samurai in addition to its other creature types. Activate this ability only any time you could cast a sorcery.","1W","Legendary Creature - Fox Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ramses Overdark","4","3","UB","{T}: Destroy target enchanted creature.","2UUBB","Legendary Creature - Human Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cut the Earthly Bond","U","Return target enchanted permanent to its owner's hand.","U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashmouth Hound","2","1","R","Whenever Ashmouth Hound blocks or becomes blocked by a creature, Ashmouth Hound deals 1 damage to that creature.","1R","Creature - Elemental Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Welkin Tern","2","1","U","Flying
Welkin Tern can block only creatures with flying.","1U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Psionic Sliver","2","2","U","All Sliver creatures have ""{T}: This creature deals 2 damage to target creature or player and 3 damage to itself.""","4U","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ideas Unbound","U","Draw three cards. Discard three cards at the beginning of the next end step.","UU","Sorcery - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Karakas","{T}: Add {W} to your mana pool.
{T}: Return target legendary creature to its owner's hand.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("First Come, First Served","W","The attacking or blocking creature with the lowest collector number has first strike. If two or more creatures are tied, they all have first strike.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chronozoa","3","3","U","Flying
Vanishing 3 (This permanent enters the battlefield with three time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
When Chronozoa dies, if it had no time counters on it, put two tokens that are copies of it onto the battlefield.","3U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Night of Souls' Betrayal","B","All creatures get -1/-1.","2BB","Legendary Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Concordant Crossroads","G","All creatures have haste.","G","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Howling Banshee","3","3","B","Flying
When Howling Banshee enters the battlefield, each player loses 3 life.","2BB","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ancient Amphitheater","As Ancient Amphitheater enters the battlefield, you may reveal a Giant card from your hand. If you don't, Ancient Amphitheater enters the battlefield tapped.
{T}: Add {R} or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Song of Serenity","G","Creatures that are enchanted can't attack or block.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sporecap Spider","1","5","G","Reach (This creature can block creatures with flying.)","2G","Creature - Spider");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Braids, Conjurer Adept Avatar","{2}: Each player may put a land card from his or her hand onto the battlefield tapped.
{3}: Each player may put a noncreature artifact card from his or her hand onto the battlefield.
{4}: Each player may put a creature card from his or her hand onto the battlefield. Activate this ability only any time you could cast a sorcery.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oona's Blackguard","1","1","B","Flying
Each other Rogue creature you control enters the battlefield with an additional +1/+1 counter on it.
Whenever a creature you control with a +1/+1 counter on it deals combat damage to a player, that player discards a card.","1B","Creature - Faerie Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Haakon, Stromgald Scourge Avatar","Pay 1 life: You may play target creature card in your graveyard this turn.
Whenever you play a creature card from your graveyard, it becomes a black Zombie Knight.
If a Zombie Knight would be put into your graveyard from the battlefield, exile it instead.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Toymaker","1","1","{1}, {T}, Discard a card: Target noncreature artifact becomes an artifact creature with power and toughness each equal to its converted mana cost until end of turn. (It retains its abilities.)","2","Artifact Creature - Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fastbond","G","You may play any number of additional lands on each of your turns.
Whenever you play a land, if it wasn't the first land you played this turn, Fastbond deals 1 damage to you.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghazbán Ogre","2","2","G","At the beginning of your upkeep, if a player has more life than each other player, the player with the most life gains control of Ghazbán Ogre.","G","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Djinn of the Lamp","5","6","U","Flying","5UU","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magma Burst","R","Kicker-Sacrifice two lands. (You may sacrifice two lands in addition to any other costs as you cast this spell.)
Magma Burst deals 3 damage to target creature or player. If Magma Burst was kicked, it deals 3 damage to another target creature or player.","3R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Archon of Justice","4","4","W","Flying
When Archon of Justice dies, exile target permanent.","3WW","Creature - Archon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Traitor's Roar","BR","Tap target untapped creature. It deals damage equal to its power to its controller.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose a new target for the copy.)","4(B/R)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Rabble","*","*","W","Vigilance
Kithkin Rabble's power and toughness are each equal to the number of white permanents you control.","3W","Creature - Kithkin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karplusan Wolverine","1","1","R","Whenever Karplusan Wolverine becomes blocked, you may have it deal 1 damage to target creature or player.","R","Snow Creature - Wolverine Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Remand","U","Counter target spell. If that spell is countered this way, put it into its owner's hand instead of into that player's graveyard.
Draw a card.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tidal Wave","U","Put a 5/5 blue Wall creature token with defender onto the battlefield. Sacrifice it at the beginning of the next end step.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Land Leeches","2","2","G","First strike","1GG","Creature - Leech");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wirewood Pride","G","Target creature gets +X/+X until end of turn, where X is the number of Elves on the battlefield.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Wayfinder","1","2","U","Flying
When Merfolk Wayfinder enters the battlefield, reveal the top three cards of your library. Put all Island cards revealed this way into your hand and the rest on the bottom of your library in any order.","2U","Creature - Merfolk Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eloren Wilds","Whenever a player taps a permanent for mana, that player adds one mana to his or her mana pool of any type that permanent produced.
Whenever you roll {C}, target player can't cast spells until a player planeswalks.",NULL,"Plane - Shandalar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of the Dead","1","2","B","{B}, {T}: Choose target opponent who had at least two fewer creature cards in his or her graveyard than you did as you activated this ability. Destroy target nonblack creature he or she controls.","BB","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wrap in Vigor","G","Regenerate each creature you control.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master Healer","1","4","W","{T}: Prevent the next 4 damage that would be dealt to target creature or player this turn.","4W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwater Thief","1","2","U","{U}: Rootwater Thief gains flying until end of turn.
Whenever Rootwater Thief deals combat damage to a player, you may pay {2}. If you do, search that player's library for a card and exile it, then the player shuffles his or her library.","1U","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diabolic Tutor","B","Search your library for a card and put that card into your hand. Then shuffle your library.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ana Sanctuary","G","At the beginning of your upkeep, if you control a blue or black permanent, target creature gets +1/+1 until end of turn. If you control a blue permanent and a black permanent, that creature gets +5/+5 until end of turn instead.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pangosaur","6","6","G","Whenever a player plays a land, return Pangosaur to its owner's hand.","2GG","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shrieking Specter","2","2","B","Flying
Whenever Shrieking Specter attacks, defending player discards a card.","5B","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vastwood Gorger","5","6","G",NULL,"5G","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sivvi's Ruse","W","If an opponent controls a Mountain and you control a Plains, you may cast Sivvi's Ruse without paying its mana cost.
Prevent all damage that would be dealt this turn to creatures you control.","2WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Standing Army","2","4","W","As long as you're standing, Standing Army has vigilance. (Attacking doesn't cause it to tap.)","2WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rogue Kavu","1","1","R","Whenever Rogue Kavu attacks alone, it gets +2/+0 until end of turn.","1R","Creature - Kavu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Night's Whisper","B","You draw two cards and you lose 2 life.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Misfortune","BRG","An opponent chooses one - You put a +1/+1 counter on each creature you control and gain 4 life; or you put a -1/-1 counter on each creature that player controls and Misfortune deals 4 damage to him or her.","1BRG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maggot Carrier","1","1","B","When Maggot Carrier enters the battlefield, each player loses 1 life.","B","Creature - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Birds of Paradise Avatar","Lands you control have ""{T}: Add one mana of any color to your mana pool.""",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dread Statuary","{T}: Add {1} to your mana pool.
{4}: Dread Statuary becomes a 4/2 Golem artifact creature until end of turn. It's still a land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ivory Giant","3","4","W","When Ivory Giant enters the battlefield, tap all nonwhite creatures.
Suspend 5-{W} (Rather than cast this card from your hand, you may pay {W} and exile it with five time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","5WW","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream Coat","U","Enchant creature
{0}: Enchanted creature becomes the color or colors of your choice. Activate this ability only once each turn.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dementia Bat","2","2","B","Flying
{4}{B}, Sacrifice Dementia Bat: Target player discards two cards.","4B","Creature - Bat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Bladescout","2","1","R","Flash (You may cast this spell any time you could cast an instant.)
When Viashino Bladescout enters the battlefield, target creature gains first strike until end of turn.","1RR","Creature - Viashino Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Umbral Mantle","Equipped creature has ""{3}, {Q}: This creature gets +2/+2 until end of turn."" ({Q} is the untap symbol.)
Equip {0}","3","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Memory Crystal","Buyback costs cost {2} less.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wallop","G","Destroy target blue or black creature with flying.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Red Sun's Zenith","R","Red Sun's Zenith deals X damage to target creature or player. If a creature dealt damage this way would die this turn, exile it instead. Shuffle Red Sun's Zenith into its owner's library.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argothian Wurm","6","6","G","Trample
When Argothian Wurm enters the battlefield, any player may sacrifice a land. If a player does, put Argothian Wurm on top of its owner's library.","3G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Indentured Oaf","4","3","R","Prevent all damage that Indentured Oaf would deal to red creatures.","3R","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Striped Bears","2","2","G","When Striped Bears enters the battlefield, draw a card.","3G","Creature - Bear");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peer Through Depths","U","Look at the top five cards of your library. You may reveal an instant or sorcery card from among them and put it into your hand. Put the rest on the bottom of your library in any order.","1U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chlorophant","1","1","G","At the beginning of your upkeep, you may put a +1/+1 counter on Chlorophant.
Threshold - As long as seven or more cards are in your graveyard, Chlorophant has ""At the beginning of your upkeep, you may put another +1/+1 counter on Chlorophant.""","GGG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reverence","W","Creatures with power 2 or less can't attack you.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cavern Crawler","0","3","R","Mountainwalk
{R}: Cavern Crawler gets +1/-1 until end of turn.","2R","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crimson Hellkite","6","6","R","Flying
{X}, {T}: Crimson Hellkite deals X damage to target creature. Spend only red mana this way.","6RRR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cavern Thoctar","5","5","G","{1}{R}: Cavern Thoctar gets +1/+0 until end of turn.","5G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of the Cabal","B","Target player sacrifices half the permanents he or she controls, rounded down.
Suspend 2-{2}{B}{B} (Rather than cast this card from your hand, you may pay {2}{B}{B} and exile it with two time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)
At the beginning of each player's upkeep, if Curse of the Cabal is suspended, that player may sacrifice a permanent. If he or she does, put two time counters on Curse of the Cabal.","9B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raven's Run Dragoon","3","3","WG","Raven's Run Dragoon can't be blocked by black creatures.","2(G/W)(G/W)","Creature - Elf Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Possessed Aven","3","3","U","Flying
Threshold - As long as seven or more cards are in your graveyard, Possessed Aven gets +1/+1, is black, and has ""{2}{B}, {T}: Destroy target blue creature.""","2UU","Creature - Bird Soldier Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scryb Sprites","1","1","G","Flying","G","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dust of Moments","W","Choose one - Remove two time counters from each permanent and each suspended card; or put two time counters on each permanent with a time counter on it and each suspended card.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earth Surge","G","Each land gets +2/+2 as long as it's a creature.","3G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crystal Rod","Whenever a player casts a blue spell, you may pay {1}. If you do, you gain 1 life.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pious Warrior","2","3","W","Whenever Pious Warrior is dealt combat damage, you gain that much life.","3W","Creature - Human Rebel Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hydroblast","U","Choose one - Counter target spell if it's red; or destroy target permanent if it's red.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wood Elemental","*","*","G","As Wood Elemental enters the battlefield, sacrifice any number of untapped Forests.
Wood Elemental's power and toughness are each equal to the number of Forests sacrificed as it entered the battlefield.","3G","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mesa Enchantress","0","2","W","Whenever you cast an enchantment spell, you may draw a card.","1WW","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Seaswift","3","1","U","Flying
{1}: Mistform Seaswift becomes the creature type of your choice until end of turn.
Morph {1}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Might of Old Krosa","G","Target creature gets +2/+2 until end of turn. If you cast this spell during your main phase, that creature gets +4/+4 until end of turn instead.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kukemssa Serpent","4","3","U","Kukemssa Serpent can't attack unless defending player controls an Island.
{U}, Sacrifice an Island: Target land an opponent controls becomes an Island until end of turn.
When you control no Islands, sacrifice Kukemssa Serpent.","3U","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Candelabra","1","2","G","{X}, {T}: Untap X target lands.","G","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Forcefield","{1}: The next time an unblocked creature of your choice would deal combat damage to you this turn, prevent all but 1 of that damage.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Word of Blasting","R","Destroy target Wall. It can't be regenerated. Word of Blasting deals damage equal to that Wall's converted mana cost to the Wall's controller.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhang He, Wei General","4","2","B","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Whenever Zhang He, Wei General attacks, each other creature you control gets +1/+0 until end of turn.","3BB","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Archery Training","W","Enchant creature
At the beginning of your upkeep, you may put an arrow counter on Archery Training.
Enchanted creature has ""{T}: This creature deals X damage to target attacking or blocking creature, where X is the number of arrow counters on Archery Training.""","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gaea's Skyfolk","2","2","UG","Flying","GU","Creature - Elf Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azorius First-Wing","2","2","WU","Flying, protection from enchantments","WU","Creature - Griffin");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Heap Doll","1","1","Sacrifice Heap Doll: Exile target card from a graveyard.","1","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steam Vines","R","Enchant land
When enchanted land becomes tapped, destroy it and Steam Vines deals 1 damage to that land's controller. That player attaches Steam Vines to a land of his or her choice.","1RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serendib Djinn","5","6","U","Flying
At the beginning of your upkeep, sacrifice a land. If you sacrifice an Island this way, Serendib Djinn deals 3 damage to you.
When you control no lands, sacrifice Serendib Djinn.","2UU","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glacial Crevasses","R","Sacrifice a snow Mountain: Prevent all combat damage that would be dealt this turn.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arc-Slogger","4","5","R","{R}, Exile the top ten cards of your library: Arc-Slogger deals 2 damage to target creature or player.","3RR","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx of Uthuun","5","6","U","Flying
When Sphinx of Uthuun enters the battlefield, reveal the top five cards of your library. An opponent separates those cards into two piles. Put one pile into your hand and the other into your graveyard.","5UU","Creature - Sphinx");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lux Cannon","{T}: Put a charge counter on Lux Cannon.
{T}, Remove three charge counters from Lux Cannon: Destroy target permanent.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dimir Infiltrator","1","3","UB","Dimir Infiltrator is unblockable.
Transmute {1}{U}{B} ({1}{U}{B}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","UB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Farbog Boneflinger","2","2","B","When Farbog Boneflinger enters the battlefield, target creature gets -2/-2 until end of turn.","4B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Monster","3","3","U","Flying","3U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread","6","6","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Whenever a creature deals damage to you, destroy it.
When Dread is put into a graveyard from anywhere, shuffle it into its owner's library.","3BBB","Creature - Elemental Incarnation");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inspired Charge","W","Creatures you control get +2/+1 until end of turn.","2WW","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Monkey Cage","When a creature enters the battlefield, sacrifice Monkey Cage and put X 2/2 green Ape creature tokens onto the battlefield, where X is that creature's converted mana cost.","5","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Great Forest","Each creature assigns combat damage equal to its toughness rather than its power.
Whenever you roll {C}, creatures you control get +0/+2 and gain trample until end of turn.",NULL,"Plane - Lorwyn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rathi Intimidator","2","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
{2}, {T}: Search your library for a Mercenary permanent card with converted mana cost 2 or less and put it onto the battlefield. Then shuffle your library.","1BB","Creature - Horror Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostway","W","Exile each creature you control. Return those cards to the battlefield under their owner's control at the beginning of the next end step.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Outrider","2","2","R","As long as you control a Beast, Skirk Outrider gets +2/+2 and has trample.","3R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rumbling Slum","5","5","RG","At the beginning of your upkeep, Rumbling Slum deals 1 damage to each player.","1RGG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corrupting Licid","2","2","B","{B}, {T}: Corrupting Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {B} to end this effect.
Enchanted creature has fear. (It can't be blocked except by artifact creatures and/or black creatures.)","2B","Creature - Licid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vitu-Ghazi, the City-Tree","{T}: Add {1} to your mana pool.
{2}{G}{W}, {T}: Put a 1/1 green Saproling creature token onto the battlefield.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Honor-Worn Shaku","{T}: Add {1} to your mana pool.
Tap an untapped legendary permanent you control: Untap Honor-Worn Shaku.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wormfang Drake","3","4","U","Flying
When Wormfang Drake enters the battlefield, sacrifice it unless you exile a creature you control other than Wormfang Drake.
When Wormfang Drake leaves the battlefield, return the exiled card to the battlefield under its owner's control.","2U","Creature - Nightmare Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cloak of Confusion","B","Enchant creature you control
Whenever enchanted creature attacks and isn't blocked, you may have it assign no combat damage this turn. If you do, defending player discards a card at random.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voidstone Gargoyle","3","3","W","Flying
As Voidstone Gargoyle enters the battlefield, name a nonland card.
The named card can't be cast.
Activated abilities of sources with the chosen name can't be activated.","3WW","Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampiric Sliver","3","3","B","All Sliver creatures have ""Whenever a creature dealt damage by this creature this turn dies, put a +1/+1 counter on this creature.""","3B","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deathmark","B","Destroy target green or white creature.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gustrider Exuberant","1","2","W","Flying
Sacrifice Gustrider Exuberant: Creatures you control with power 5 or greater gain flying until end of turn.","2W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burrenton Forge-Tender","1","1","W","Protection from red
Sacrifice Burrenton Forge-Tender: Prevent all damage a red source of your choice would deal this turn.","W","Creature - Kithkin Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Team Spirit","G","All creatures controlled by target player and his or her teammates get +1/+1 until end of turn.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magma Vein","R","{R}, Sacrifice a land: Magma Vein deals 1 damage to each creature without flying.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Paralyze","B","Enchant creature
When Paralyze enters the battlefield, tap enchanted creature.
Enchanted creature doesn't untap during its controller's untap step.
At the beginning of the upkeep of enchanted creature's controller, that player may pay {4}. If he or she does, untap the creature.","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Honor of the Pure","W","White creatures you control get +1/+1.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stitcher's Apprentice","1","2","U","{1}{U}, {T}: Put a 2/2 blue Homunculus creature token onto the battlefield, then sacrifice a creature.","1U","Creature - Homunculus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flickering Spirit","2","2","W","Flying
{3}{W}: Exile Flickering Spirit, then return it to the battlefield under its owner's control.","3W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Clash","R","You and target opponent each flip a coin. Mana Clash deals 1 damage to each player whose coin comes up tails. Repeat this process until both players' coins come up heads on the same flip.","R","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ulvenwald Primordials","5","5","{G}: Regenerate Ulvenwald Primordials.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Ulvenwald Primordials.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Savannah",NULL,NULL,"Land - Forest Plains");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thoughtleech","G","Whenever an Island an opponent controls becomes tapped, you may gain 1 life.","GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Broken Ambitions","U","Counter target spell unless its controller pays {X}. Clash with an opponent. If you win, that spell's controller puts the top four cards of his or her library into his or her graveyard. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","XU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lost in the Mist","U","Counter target spell. Return target permanent to its owner's hand.","3UU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ankh of Mishra","Whenever a land enters the battlefield, Ankh of Mishra deals 2 damage to that land's controller.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tel-Jilad Lifebreather","3","2","G","{G}, {T}, Sacrifice a Forest: Regenerate target creature.","4G","Creature - Troll Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nihil Spellbomb","{T}, Sacrifice Nihil Spellbomb: Exile all cards from target player's graveyard.
When Nihil Spellbomb is put into a graveyard from the battlefield, you may pay {B}. If you do, draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Weaponsmith","2","2","R","Whenever Viashino Weaponsmith becomes blocked by a creature, Viashino Weaponsmith gets +2/+2 until end of turn.","3R","Creature - Viashino");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cetavolver","1","1","U","Kicker {1}{R} and/or {G} (You may pay an additional {1}{R} and/or {G} as you cast this spell.)
If Cetavolver was kicked with its {1}{R} kicker, it enters the battlefield with two +1/+1 counters on it and with first strike.
If Cetavolver was kicked with its {G} kicker, it enters the battlefield with a +1/+1 counter on it and with trample.","1U","Creature - Volver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Crusader","2","2","B","First strike, protection from red and from white
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","1BB","Creature - Zombie Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flame Fusillade","R","Until end of turn, permanents you control gain ""{T}: This permanent deals 1 damage to target creature or player.""","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Utopia Mycon","0","2","G","At the beginning of your upkeep, put a spore counter on Utopia Mycon.
Remove three spore counters from Utopia Mycon: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: Add one mana of any color to your mana pool.","G","Creature - Fungus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mephitic Ooze","0","5","B","Mephitic Ooze gets +1/+0 for each artifact you control.
Whenever Mephitic Ooze deals combat damage to a creature, destroy that creature. The creature can't be regenerated.","4B","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surge of Thoughtweft","W","Creatures you control get +1/+1 until end of turn. If you control a Kithkin, draw a card.","1W","Tribal Instant - Kithkin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fable of Wolf and Owl","UG","Whenever you cast a green spell, you may put a 2/2 green Wolf creature token onto the battlefield.
Whenever you cast a blue spell, you may put a 1/1 blue Bird creature token with flying onto the battlefield.","3(G/U)(G/U)(G/U)","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incite","R","Target creature becomes red until end of turn and attacks this turn if able.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbarian Bully","2","2","R","Discard a card at random: Barbarian Bully gets +2/+2 until end of turn unless a player has Barbarian Bully deal 4 damage to him or her. Activate this ability only once each turn.","2R","Creature - Human Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fires of Undeath","R","Fires of Undeath deals 2 damage to target creature or player.
Flashback {5}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Academy at Tolaria West","At the beginning of your end step, if you have no cards in hand, draw seven cards.
Whenever you roll {C}, discard your hand.",NULL,"Plane - Dominaria");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Past in Flames","R","Each instant and sorcery card in your graveyard gains flashback until end of turn. The flashback cost is equal to its mana cost.
Flashback {4}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Boon","B","Enchant creature
Enchanted creature gets +2/+1 as long as it's black. Otherwise, it gets -1/-2.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rohgahh of Kher Keep","5","5","BR","At the beginning of your upkeep, you may pay {R}{R}{R}. If you don't, tap Rohgahh of Kher Keep and all creatures named Kobolds of Kher Keep, then an opponent gains control of them.
Creatures you control named Kobolds of Kher Keep get +2/+2.","2BBRR","Legendary Creature - Kobold");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Celestial Gatekeeper","2","2","W","Flying
When Celestial Gatekeeper dies, exile it, then return up to two target Bird and/or Cleric permanent cards from your graveyard to the battlefield.","3WW","Creature - Bird Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Altar's Reap","B","As an additional cost to cast Altar's Reap, sacrifice a creature.
Draw two cards.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jund Sojourners","3","2","BRG","When you cycle Jund Sojourners or it dies, you may have it deal 1 damage to target creature or player.
Cycling {2}{R} ({2}{R}, Discard this card: Draw a card.)","BRG","Creature - Viashino Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Peacekeeper Avatar","{3}: For each opponent who controls a creature, put a token onto the battlefield that's a copy of a card named Arrest and attach it to a creature that player controls chosen at random.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mother of Goons","3","2","B","Whenever a creature an opponent controls is put into a graveyard from play, sacrifice Mother of Goons unless you insult that creature.","2B","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Momir Vig, Simic Visionary Avatar","{X}, Discard a card: Put a token onto the battlefield that's a copy of a creature card with converted mana cost X chosen at random. Activate this ability only any time you could cast a sorcery and only once each turn.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disciple of the Vault","1","1","B","Whenever an artifact is put into a graveyard from the battlefield, you may have target opponent lose 1 life.","B","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chittering Rats","2","2","B","When Chittering Rats enters the battlefield, target opponent puts a card from his or her hand on top of his or her library.","1BB","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Hexmage","2","1","B","First strike
Sacrifice Vampire Hexmage: Remove all counters from target permanent.","BB","Creature - Vampire Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Groundskeeper","1","1","G","{1}{G}: Return target basic land card from your graveyard to your hand.","G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Accelerated Mutation","G","Target creature gets +X/+X until end of turn, where X is the highest converted mana cost among permanents you control.","3GG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bind","G","Counter target activated ability. (Mana abilities can't be targeted.)
Draw a card.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dingus Staff","Whenever a creature dies, Dingus Staff deals 2 damage to that creature's controller.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elder Spawn","6","6","U","At the beginning of your upkeep, unless you sacrifice an Island, sacrifice Elder Spawn and it deals 6 damage to you.
Elder Spawn can't be blocked by red creatures.","4UUU","Creature - Spawn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sparkcaster","5","3","RG","When Sparkcaster enters the battlefield, return a red or green creature you control to its owner's hand.
When Sparkcaster enters the battlefield, it deals 1 damage to target player.","2RG","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Volrath","Whenever a creature you control is put into your graveyard from the battlefield, you may put it on top of your library.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ember Hauler","2","2","R","{1}, Sacrifice Ember Hauler: Ember Hauler deals 2 damage to target creature or player.","RR","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Festering Evil","B","At the beginning of your upkeep, Festering Evil deals 1 damage to each creature and each player.
{B}{B}, Sacrifice Festering Evil: Festering Evil deals 3 damage to each creature and each player.","3BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mages' Contest","R","You and target spell's controller bid life. You start the bidding with a bid of 1. In turn order, each player may top the high bid. The bidding ends if the high bid stands. The high bidder loses life equal to the high bid. If you win the bidding, counter that spell.","1RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lizard Warrior","4","2","R",NULL,"3R","Creature - Lizard Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wildfire","R","Each player sacrifices four lands. Wildfire deals 4 damage to each creature.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plated Seastrider","1","4","U",NULL,"UU","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyrokinesis","R","You may exile a red card from your hand rather than pay Pyrokinesis's mana cost.
Pyrokinesis deals 4 damage divided as you choose among any number of target creatures.","4RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Careful Study","U","Draw two cards, then discard two cards.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balthor the Stout","2","2","R","Other Barbarian creatures get +1/+1.
{R}: Another target Barbarian creature gets +1/+0 until end of turn.","1RR","Legendary Creature - Dwarf Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mobilize","G","Untap all creatures you control.","G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fountain of Cho","Fountain of Cho enters the battlefield tapped.
{T}: Put a storage counter on Fountain of Cho.
{T}, Remove any number of storage counters from Fountain of Cho: Add {W} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cleanfall","W","Destroy all enchantments.","2W","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Endless Ranks of the Dead","B","At the beginning of your upkeep, put X 2/2 black Zombie creature tokens onto the battlefield, where X is half the number of Zombies you control, rounded down.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("King Suleiman","1","1","W","{T}: Destroy target Djinn or Efreet.","1W","Creature - Human");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sheep token card",NULL,NULL,NULL);
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grim Tutor","B","Search your library for a card and put that card into your hand, then shuffle your library. You lose 3 life.","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ocular Halo","U","Enchant creature
Enchanted creature has ""{T}: Draw a card.""
{W}: Enchanted creature gains vigilance until end of turn.","3U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Festerhide Boar","3","3","G","Trample
Morbid - Festerhide Boar enters the battlefield with two +1/+1 counters on it if a creature died this turn.","3G","Creature - Boar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spoils of Evil","B","For each artifact or creature card in target opponent's graveyard, add {1} to your mana pool and you gain 1 life.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ravages of War","W","Destroy all lands.","3W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormscape Apprentice","1","1","U","{W}, {T}: Tap target creature.
{B}, {T}: Target player loses 1 life.","U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kilnmouth Dragon","5","5","R","Amplify 3 (As this creature enters the battlefield, put three +1/+1 counters on it for each Dragon card you reveal in your hand.)
Flying
{T}: Kilnmouth Dragon deals damage equal to the number of +1/+1 counters on it to target creature or player.","5RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Avoid Fate","G","Counter target instant or Aura spell that targets a permanent you control.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Simulacrum","B","You gain life equal to the damage dealt to you this turn. Simulacrum deals damage to target creature you control equal to the damage dealt to you this turn.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iridescent Angel","4","4","WU","Flying, protection from all colors","5WU","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Johnny, Combo Player","1","1","U","{4}: Search your library for a card and put that card into your hand. Then shuffle your library.","2UU","Legendary Creature - Human Gamer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parallax Tide","U","Fading 5 (This enchantment enters the battlefield with five fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Remove a fade counter from Parallax Tide: Exile target land.
When Parallax Tide leaves the battlefield, each player returns to the battlefield all cards he or she owns exiled with Parallax Tide.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Torch Fiend","2","1","R","{R}, Sacrifice Torch Fiend: Destroy target artifact.","1R","Creature - Devil");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Lackey","1","1","R","Whenever Goblin Lackey deals damage to a player, you may put a Goblin permanent card from your hand onto the battlefield.","R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exclusion Ritual","W","Imprint - When Exclusion Ritual enters the battlefield, exile target nonland permanent.
Players can't cast spells with the same name as the exiled card.","4WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spawning Pool","Spawning Pool enters the battlefield tapped.
{T}: Add {B} to your mana pool.
{1}{B}: Spawning Pool becomes a 1/1 black Skeleton creature with ""{B}: Regenerate this creature"" until end of turn. It's still a land. (If it regenerates, the next time it would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Standard Bearer","1","1","W","While choosing targets as part of casting a spell or activating an ability, your opponents must choose at least one Flagbearer on the battlefield if able.","1W","Creature - Human Flagbearer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scrib Nibblers","1","1","B","{T}: Exile the top card of target player's library. If it's a land card, you gain 1 life.
Landfall - Whenever a land enters the battlefield under your control, you may untap Scrib Nibblers.","2B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Luminous Guardian","1","4","W","{W}: Luminous Guardian gets +0/+1 until end of turn.
{2}: Luminous Guardian can block an additional creature this turn.","3W","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dry Spell","B","Dry Spell deals 1 damage to each creature and each player.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Beasts of Bogardan","3","3","R","Protection from red
Beasts of Bogardan gets +1/+1 as long as an opponent controls a white permanent.","4R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thaumatog","1","2","WG","Sacrifice a land: Thaumatog gets +1/+1 until end of turn.
Sacrifice an enchantment: Thaumatog gets +1/+1 until end of turn.","1GW","Creature - Atog");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cloudpost","Cloudpost enters the battlefield tapped.
{T}: Add {1} to your mana pool for each Locus on the battlefield.",NULL,"Land - Locus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consume the Meek","B","Destroy each creature with converted mana cost 3 or less. They can't be regenerated.","3BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Winnow","W","Destroy target nonland permanent if another permanent with the same name is on the battlefield.
Draw a card.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kederekt Creeper","2","3","UBR","Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)
Kederekt Creeper can't be blocked except by two or more creatures.","UBR","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Blessing","WG","{G}{W}, Discard a card: Put a +1/+1 counter on target creature or that creature gains banding, first strike, or trample. (This effect lasts indefinitely. Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding a player controls are blocking or being blocked by a creature, that player divides that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","2GW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wicked Reward","B","As an additional cost to cast Wicked Reward, sacrifice a creature.
Target creature gets +4/+2 until end of turn.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nefashu","5","3","B","Whenever Nefashu attacks, up to five target creatures each get -1/-1 until end of turn.","4BB","Creature - Zombie Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akoum Boulderfoot","4","5","R","When Akoum Boulderfoot enters the battlefield, it deals 1 damage to target creature or player.","4RR","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drain Life","B","Spend only black mana on X.
Drain Life deals X damage to target creature or player. You gain life equal to the damage dealt, but not more life than the player's life total before Drain Life dealt damage or the creature's toughness.","X1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orchard Warden","4","6","G","Whenever another Treefolk creature enters the battlefield under your control, you may gain life equal to that creature's toughness.","4GG","Creature - Treefolk Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Numai Outcast","1","1","B","Bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)
{B}, Pay 5 life: Regenerate Numai Outcast.","3B","Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blue Sun's Zenith","U","Target player draws X cards. Shuffle Blue Sun's Zenith into its owner's library.","XUUU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sheltering Prayers","W","Basic lands each player controls have shroud as long as that player controls three or fewer lands. (They can't be the targets of spells or abilities.)","W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Malevolent Awakening","B","{1}{B}{B}, Sacrifice a creature: Return target creature card from your graveyard to your hand.","1BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brimstone Volley","R","Brimstone Volley deals 3 damage to target creature or player.
Morbid - Brimstone Volley deals 5 damage to that creature or player instead if a creature died this turn.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Filthy Cur","2","2","B","Whenever Filthy Cur is dealt damage, you lose that much life.","1B","Creature - Hound");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Carnage Altar","{3}, Sacrifice a creature: Draw a card.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nath of the Gilt-Leaf","4","4","BG","At the beginning of your upkeep, you may have target opponent discard a card at random.
Whenever an opponent discards a card, you may put a 1/1 green Elf Warrior creature token onto the battlefield.","3BG","Legendary Creature - Elf Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Two-Headed Giant of Foriys Avatar","Each creature you control can block an additional creature each combat.
Each creature you control can't be blocked except by two or more creatures.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Time Elemental","0","2","U","When Time Elemental attacks or blocks, at end of combat, sacrifice it and it deals 5 damage to you.
{2}{U}{U}, {T}: Return target permanent that isn't enchanted to its owner's hand.","2U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shriek of Dread","B","Target creature gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arctic Foxes","1","1","W","Creatures with power 2 or greater can't block Arctic Foxes as long as defending player controls a snow land.","1W","Creature - Fox");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Shielddare","1","1","W","{W}, {T}: Target blocking creature gets +2/+2 until end of turn.","1W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tarox Bladewing","4","3","R","Flying, haste
Grandeur - Discard another card named Tarox Bladewing: Tarox Bladewing gets +X/+X until end of turn, where X is its power.","2RRR","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kyoki, Sanity's Eclipse","6","4","B","Whenever you cast a Spirit or Arcane spell, target opponent exiles a card from his or her hand.","4BB","Legendary Creature - Demon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dispense Justice","W","Target player sacrifices an attacking creature.
Metalcraft - That player sacrifices two attacking creatures instead if you control three or more artifacts.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dirge of Dread","B","All creatures gain fear until end of turn. (They can't be blocked except by artifact creatures and/or black creatures.)
Cycling {1}{B} ({1}{B}, Discard this card: Draw a card.)
When you cycle Dirge of Dread, you may have target creature gain fear until end of turn.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Chains","U","Enchant creature
Enchanted creature has ""Cumulative upkeep {1}."" (At the beginning of its controller's upkeep, that player puts an age counter on it, then sacrifices it unless he or she pays its upkeep cost for each age counter on it.)","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lady Orca","7","4","BR",NULL,"5BR","Legendary Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fallow Earth","G","Put target land on top of its owner's library.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warren Pilferers","3","3","B","When Warren Pilferers enters the battlefield, return target creature card from your graveyard to your hand. If that card is a Goblin card, Warren Pilferers gains haste until end of turn.","4B","Creature - Goblin Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cathedral of Serra","White legendary creatures you control have ""bands with other legendary creatures."" (Any legendary creatures can attack in a band as long as at least one has ""bands with other legendary creatures."" Bands are blocked as a group. If at least two legendary creatures you control, one of which has ""bands with other legendary creatures,"" are blocking or being blocked by the same creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("March of Souls","W","Destroy all creatures. They can't be regenerated. For each creature destroyed this way, its controller puts a 1/1 white Spirit creature token with flying onto the battlefield.","4W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Monstrous Growth","G","Target creature gets +4/+4 until end of turn.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fiery Justice","WRG","Fiery Justice deals 5 damage divided as you choose among any number of target creatures and/or players. Target opponent gains 5 life.","RGW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Force of Savagery","8","0","G","Trample","2G","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blistering Firecat","7","1","R","Trample, haste
At the beginning of the end step, sacrifice Blistering Firecat.
Morph {R}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1RRR","Creature - Elemental Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mycoid Shepherd","5","4","WG","Whenever Mycoid Shepherd or another creature you control with power 5 or greater dies, you may gain 5 life.","1GGW","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thieves' Auction","R","Exile all nontoken permanents. Starting with you, each player chooses one of the exiled cards and puts it onto the battlefield tapped under his or her control. Repeat this process until all cards exiled this way have been chosen.","4RRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sandsower","1","3","W","Tap three untapped creatures you control: Tap target creature.","3W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Storm Entity","1","1","R","Haste
Storm Entity enters the battlefield with a +1/+1 counter on it for each other spell cast this turn.","1R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nephalia Seakite","2","3","U","Flash (You may cast this spell any time you could cast an instant.)
Flying","3U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Luminarch Ascension","W","At the beginning of each opponent's end step, if you didn't lose life this turn, you may put a quest counter on Luminarch Ascension. (Damage causes loss of life.)
{1}{W}: Put a 4/4 white Angel creature token with flying onto the battlefield. Activate this ability only if Luminarch Ascension has four or more quest counters on it.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hedron Rover","2","2","Landfall - Whenever a land enters the battlefield under your control, Hedron Rover gets +2/+2 until end of turn.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Slith","0","0","Whenever Arcbound Slith deals combat damage to a player, put a +1/+1 counter on it.
Modular 1 (This enters the battlefield with a +1/+1 counter on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","2","Artifact Creature - Slith");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("AEther Vial","At the beginning of your upkeep, you may put a charge counter on Æther Vial.
{T}: You may put a creature card with converted mana cost equal to the number of charge counters on Æther Vial from your hand onto the battlefield.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Selkie Hedge-Mage","2","2","UG","When Selkie Hedge-Mage enters the battlefield, if you control two or more Forests, you may gain 3 life.
When Selkie Hedge-Mage enters the battlefield, if you control two or more Islands, you may return target tapped creature to its owner's hand.","2(G/U)","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lurking Evil","B","Pay half your life, rounded up: Lurking Evil becomes a 4/4 Horror creature with flying.","BBB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Superion","5","6","Spend only mana produced by creatures to cast Myr Superion.","2","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drought","W","At the beginning of your upkeep, sacrifice Drought unless you pay {W}{W}.
Spells cost an additional ""Sacrifice a Swamp"" to cast for each black mana symbol in their mana costs.
Activated abilities cost an additional ""Sacrifice a Swamp"" to activate for each black mana symbol in their activation costs.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decompose","B","Exile up to three target cards from a single graveyard.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iridescent Drake","2","2","U","Flying
When Iridescent Drake enters the battlefield, put target Aura card from a graveyard onto the battlefield under your control attached to Iridescent Drake.","3U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hissing Iguanar","3","1","R","Whenever another creature dies, you may have Hissing Iguanar deal 1 damage to target player.","2R","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Haru-Onna","2","1","G","When Haru-Onna enters the battlefield, draw a card.
Whenever you cast a Spirit or Arcane spell, you may return Haru-Onna to its owner's hand.","3G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lord of Lineage","5","5","Flying
Other Vampire creatures you control get +2/+2.
{T}: Put a 2/2 black Vampire creature token with flying onto the battlefield.",NULL,"Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit Flare","W","Tap target untapped creature you control. If you do, it deals damage equal to its power to target attacking or blocking creature an opponent controls.
Flashback-{1}{W}, Pay 3 life. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Withering Boon","B","As an additional cost to cast Withering Boon, pay 3 life.
Counter target creature spell.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cultbrand Cinder","3","3","BR","When Cultbrand Cinder enters the battlefield, put a -1/-1 counter on target creature.","4(B/R)","Creature - Elemental Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ventifact Bottle","{X}{1}, {T}: Put X charge counters on Ventifact Bottle. Activate this ability only any time you could cast a sorcery.
At the beginning of your precombat main phase, if Ventifact Bottle has a charge counter on it, tap it and remove all charge counters from it. Add {1} to your mana pool for each charge counter removed this way.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riddlesmith","2","1","U","Whenever you cast an artifact spell, you may draw a card. If you do, discard a card.","1U","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bioplasm","4","4","G","Whenever Bioplasm attacks, exile the top card of your library. If it's a creature card, Bioplasm gets +X/+Y until end of turn, where X is the exiled creature card's power and Y is its toughness.","3GG","Creature - Ooze");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thought Courier","1","1","U","{T}: Draw a card, then discard a card.","1U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heat Ray","R","Heat Ray deals X damage to target creature.","XR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hand of Honor","2","2","W","Protection from black
Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","WW","Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Release the Ants","R","Release the Ants deals 1 damage to target creature or player. Clash with an opponent. If you win, return Release the Ants to its owner's hand. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spellshift","U","Counter target instant or sorcery spell. Its controller reveals cards from the top of his or her library until he or she reveals an instant or sorcery card. That player may cast that card without paying its mana cost. Then he or she shuffles his or her library.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hystrodon","3","4","G","Trample
Whenever Hystrodon deals combat damage to a player, you may draw a card.
Morph {1}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Firecat Blitz","R","Put X 1/1 red Elemental Cat creature tokens with haste onto the battlefield. Exile them at the beginning of the next end step.
Flashback-{R}{R}, Sacrifice X Mountains. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","XRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Annex","U","Enchant land (Target a land as you cast this. This card enters the battlefield attached to that land.)
You control enchanted land.","2UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Irresistible Prey","G","Target creature must be blocked this turn if able.
Draw a card.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tanglesap","G","Prevent all combat damage that would be dealt this turn by creatures without trample.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Halimar Excavator","1","3","U","Whenever Halimar Excavator or another Ally enters the battlefield under your control, target player puts the top X cards of his or her library into his or her graveyard, where X is the number of Allies you control.","1U","Creature - Human Wizard Ally");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Beast of Burden","*","*","Beast of Burden's power and toughness are each equal to the number of creatures on the battlefield.","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brooding Saurian","4","4","G","At the beginning of each end step, each player gains control of all nontoken permanents he or she owns.","2GG","Creature - Lizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Forbidden Orchard","{T}: Add one mana of any color to your mana pool.
Whenever you tap Forbidden Orchard for mana, put a 1/1 colorless Spirit creature token onto the battlefield under target opponent's control.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("He Who Hungers","3","2","B","Flying
{1}, Sacrifice a Spirit: Target opponent reveals his or her hand. You choose a card from it. That player discards that card. Activate this ability only any time you could cast a sorcery.
Soulshift 4 (When this creature dies, you may return target Spirit card with converted mana cost 4 or less from your graveyard to your hand.)","4B","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lost Soul","2","1","B","Swampwalk","1BB","Creature - Spirit Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sowing Salt","R","Exile target nonbasic land. Search its controller's graveyard, hand, and library for all cards with the same name as that land and exile them. Then that player shuffles his or her library.","2RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Liberated Dwarf","1","1","R","{R}, Sacrifice Liberated Dwarf: Target green creature gets +1/+0 and gains first strike until end of turn.","R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Manamorphose","RG","Add two mana in any combination of colors to your mana pool.
Draw a card.","1(R/G)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trickster Mage","1","1","U","{U}, {T}, Discard a card: You may tap or untap target artifact, creature, or land.","U","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thriss, Nantuko Primus","5","5","G","{G}, {T}: Target creature gets +5/+5 until end of turn.","5GG","Legendary Creature - Insect Druid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Narstad Scrapper","3","3","{2}: Narstad Scrapper gets +1/+0 until end of turn.","5","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wasp Lancer","3","2","UB","Flying","(U/B)(U/B)(U/B)","Creature - Faerie Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Copper Tablet","At the beginning of each player's upkeep, Copper Tablet deals 1 damage to that player.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Imperious Perfect","2","2","G","Other Elf creatures you control get +1/+1.
{G}, {T}: Put a 1/1 green Elf Warrior creature token onto the battlefield.","2G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razorfin Abolisher","2","2","U","{1}{U}, {T}: Return target creature with a counter on it to its owner's hand.","2U","Creature - Merfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Runed Stalactite","Equipped creature gets +1/+1 and is every creature type.
Equip {2}","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Instill Energy","G","Enchant creature
Enchanted creature can attack as though it had haste.
{0}: Untap enchanted creature. Activate this ability only during your turn and only once each turn.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Temp of the Damned","3","3","B","When you play Temp of the Damned, roll a six-sided die. Temp of the Damned comes into play with a number of funk counters on it equal to the die roll.
During your upkeep, remove a funk counter from Temp of the Damned or sacrifice Temp of the Damned.","2B","Summon - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eye of Ugin","Colorless Eldrazi spells you cast cost {2} less to cast.
{7}, {T}: Search your library for a colorless creature card, reveal it, and put it into your hand. Then shuffle your library.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Metathran Transport","1","3","U","Flying
Metathran Transport can't be blocked by blue creatures.
{U}: Target creature becomes blue until end of turn.","1UU","Creature - Metathran");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reforge the Soul","R","Each player discards his or her hand and draws seven cards.
Miracle {1}{R} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Childhood Horror","2","2","B","Flying
Threshold - As long as seven or more cards are in your graveyard, Childhood Horror gets +2/+2 and can't block.","3B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sylvan Yeti","*","4","G","Sylvan Yeti's power is equal to the number of cards in your hand.","2GG","Creature - Yeti");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razortooth Rats","2","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)","2B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Blightcutter","2","2","G","Protection from black
Threshold - Nantuko Blightcutter gets +1/+1 for each black permanent your opponents control as long as seven or more cards are in your graveyard.","2G","Creature - Insect Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Into the Maw of Hell","R","Destroy target land. Into the Maw of Hell deals 13 damage to target creature.","4RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mental Note","U","Put the top two cards of your library into your graveyard.
Draw a card.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Tiller","0","0","G","Spike Tiller enters the battlefield with three +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Tiller: Put a +1/+1 counter on target creature.
{2}, Remove a +1/+1 counter from Spike Tiller: Target land becomes a 2/2 creature that's still a land. Put a +1/+1 counter on it.","3GG","Creature - Spike");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oath of Scholars","U","At the beginning of each player's upkeep, that player chooses target player who has more cards in hand than he or she does and is his or her opponent. The first player may discard his or her hand and draw three cards.","3U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gilded Light","W","You gain shroud until end of turn. (You can't be the target of spells or abilities.)
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gale Force","G","Gale Force deals 5 damage to each creature with flying.","4G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Personal Tutor","U","Search your library for a sorcery card and reveal that card. Shuffle your library, then put the card on top of it.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harm's Way","W","The next 2 damage that a source of your choice would deal to you and/or permanents you control this turn is dealt to target creature or player instead.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steam Catapult","2","3","W","{T}: Destroy target tapped creature. Activate this ability only during your turn, before attackers are declared.","3WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noggle Bandit","2","2","UR","Noggle Bandit can't be blocked except by creatures with defender.","1(U/R)(U/R)","Creature - Noggle Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ignite the Cloneforge!","When you set this scheme in motion, put a token onto the battlefield that's a copy of target permanent an opponent controls.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frightcrawler","1","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Threshold - As long as seven or more cards are in your graveyard, Frightcrawler gets +2/+2 and can't block.","1B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avenging Druid","1","3","G","Whenever Avenging Druid deals damage to an opponent, you may reveal cards from the top of your library until you reveal a land card. If you do, put that card onto the battlefield and put all other cards revealed this way into your graveyard.","2G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devastate","R","Destroy target land. Devastate deals 1 damage to each creature and each player.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jeska, Warrior Adept","3","1","R","First strike, haste
{T}: Jeska, Warrior Adept deals 1 damage to target creature or player.","2RR","Legendary Creature - Human Barbarian Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Salvation","5","5","W","Flash; convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Flying
When Angel of Salvation enters the battlefield, prevent the next 5 damage that would be dealt this turn to any number of target creatures and/or players, divided as you choose.","6WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Battle Cry","W","Untap all white creatures you control.
Whenever a creature blocks this turn, it gets +0/+1 until end of turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rishadan Cutpurse","1","1","U","When Rishadan Cutpurse enters the battlefield, each opponent sacrifices a permanent unless he or she pays {1}.","2U","Creature - Human Pirate");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pulse Tracker","1","1","B","Whenever Pulse Tracker attacks, each opponent loses 1 life.","B","Creature - Vampire Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tenza, Godo's Maul","Equipped creature gets +1/+1. As long as it's legendary, it gets an additional +2/+2. As long as it's red, it has trample.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","3","Legendary Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight Of Dawn","2","2","W","First strike
{W}{W}: Knight of Dawn gains protection from the color of your choice until end of turn.","1WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootrunner","3","3","G","{G}{G}, Sacrifice Rootrunner: Put target land on top of its owner's library.
Soulshift 3 (When this creature dies, you may return target Spirit card with converted mana cost 3 or less from your graveyard to your hand.)","2GG","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spinning Darkness","B","You may exile the top three black cards of your graveyard rather than pay Spinning Darkness's mana cost.
Spinning Darkness deals 3 damage to target nonblack creature. You gain 3 life.","4BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Samite Blessing","W","Enchant creature
Enchanted creature has ""{T}: The next time a source of your choice would deal damage to target creature this turn, prevent that damage.""","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rock Basilisk","4","5","RG","Whenever Rock Basilisk blocks or becomes blocked by a non-Wall creature, destroy that creature at end of combat.","4RG","Creature - Basilisk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seething Song","R","Add {R}{R}{R}{R}{R} to your mana pool.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jhessian Balmgiver","1","1","WU","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.
{T}: Target creature is unblockable this turn.","1WU","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Double Take","U","Choose another player. Draw two cards now and draw an additional two cards at the beginning of the next game with that player.","3UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leatherback Baloth","4","5","G",NULL,"GGG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Youthful Knight","2","1","W","First strike (This creature deals combat damage before creatures without first strike.)","1W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lu Meng, Wu General","4","4","U","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","3UU","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fists of Ironwood","G","Enchant creature
When Fists of Ironwood enters the battlefield, put two 1/1 green Saproling creature tokens onto the battlefield.
Enchanted creature has trample.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rockslide Ambush","R","Rockslide Ambush deals damage to target creature equal to the number of Mountains you control.","1R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mangara's Tome","When Mangara's Tome enters the battlefield, search your library for five cards, exile them in a face-down pile, and shuffle that pile. Then shuffle your library.
{2}: The next time you would draw a card this turn, instead put the top card of the exiled pile into its owner's hand.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Leadfoot","3","3","R","Whenever Ogre Leadfoot becomes blocked by an artifact creature, destroy that creature.","4R","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Archwing Dragon","4","4","R","Flying, haste
At the beginning of the end step, return Archwing Dragon to its owner's hand.","2RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skywinder Drake","3","1","U","Flying
Skywinder Drake can block only creatures with flying.","2U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Argent Mutation","U","Target permanent becomes an artifact in addition to its other types until end of turn.
Draw a card.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scarwood Bandits","2","2","G","Forestwalk
{2}{G}, {T}: Unless an opponent pays {2}, gain control of target artifact for as long as Scarwood Bandits remains on the battlefield.","2GG","Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Engineered Explosives","Sunburst (This enters the battlefield with a charge counter on it for each color of mana spent to cast it.)
{2}, Sacrifice Engineered Explosives: Destroy each nonland permanent with converted mana cost equal to the number of charge counters on Engineered Explosives.","X","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Restless Apparition","2","2","WB","{(w/b){(w/b){(w/b)}: Restless Apparition gets +3/+3 until end of turn.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","(W/B)(W/B)(W/B)","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Filth","2","2","B","Swampwalk
As long as Filth is in your graveyard and you control a Swamp, creatures you control have swampwalk.","3B","Creature - Incarnation");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Osai Vultures","1","1","W","Flying
At the beginning of each end step, if a creature died this turn, put a carrion counter on Osai Vultures.
Remove two carrion counters from Osai Vultures: Osai Vultures gets +1/+1 until end of turn.","1W","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serra's Hymn","W","At the beginning of your upkeep, you may put a verse counter on Serra's Hymn.
Sacrifice Serra's Hymn: Prevent the next X damage that would be dealt this turn to any number of target creatures and/or players, divided as you choose, where X is the number of verse counters on Serra's Hymn.","W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Salvagers","2","4","W","{1}{W}: Return target artifact card with converted mana cost 1 or less from your graveyard to your hand.","3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rime Transfusion","B","Enchant creature
Enchanted creature gets +2/+1 and has ""{S}i}: This creature can't be blocked this turn except by snow creatures."" ({S}i} can be paid with one mana from a snow permanent.)","1B","Snow Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Terashi's Cry","W","Tap up to three target creatures.","3W","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exile","W","Exile target nonwhite attacking creature. You gain life equal to its toughness.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Metamorph","0","0","U","({(u/p)} can be paid with either {U} or 2 life.)
You may have Phyrexian Metamorph enter the battlefield as a copy of any artifact or creature on the battlefield, except it's an artifact in addition to its other types.","3(U/P)","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Edge of the Divinity","WB","Enchant creature
As long as enchanted creature is white, it gets +1/+2.
As long as enchanted creature is black, it gets +2/+1.","(W/B)","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Treasure Hunt","U","Reveal cards from the top of your library until you reveal a nonland card, then put all cards revealed this way into your hand.","1U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flooded Strand","{T}, Pay 1 life, Sacrifice Flooded Strand: Search your library for a Plains or Island card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Roiling Horror","*","*","B","Roiling Horror's power and toughness are each equal to your life total minus the life total of an opponent with the most life.
Suspend X-{X}{B}{B}{B}. X can't be 0. (Rather than cast this card from your hand, you may pay {X}{B}{B}{B} and exile it with X time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Whenever a time counter is removed from Roiling Horror while it's exiled, target player loses 1 life and you gain 1 life.","3BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Grunt","1","1","R","Mountainwalk","R","Creature - Dwarf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squallmonger","3","3","G","{2}: Squallmonger deals 1 damage to each creature with flying and each player. Any player may activate this ability.","3G","Creature - Monger");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Windwalker","2","3","W","Flying
{T}: Attach target Equipment you control to target creature you control.","3W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wizard Replica","1","3","Flying
{U}, Sacrifice Wizard Replica: Counter target spell unless its controller pays {2}.","3","Artifact Creature - Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Ants","5","1","G","Trample, haste
Cumulative upkeep {G}{G} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","2GG","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Melesse Spirit","3","3","W","Flying, protection from black","3WW","Creature - Angel Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Trance","U","Until end of turn, Wizards you control gain ""{T}: Counter target spell.""","2UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gibbering Descent","B","At the beginning of each player's upkeep, that player loses 1 life and discards a card.
Hellbent - Skip your upkeep step if you have no cards in hand.
Madness {2}{B}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","4BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu Soldier-Farmers","2","4","W","When Shu Soldier-Farmers enters the battlefield, you gain 4 life.","4W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kiri-Onna","2","2","U","When Kiri-Onna enters the battlefield, return target creature to its owner's hand.
Whenever you cast a Spirit or Arcane spell, you may return Kiri-Onna to its owner's hand.","4U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Rend","B","Destroy target creature if it's white. A creature destroyed this way can't be regenerated.
Draw a card at the beginning of the next turn's upkeep.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ethereal Champion","3","4","W","Pay 1 life: Prevent the next 1 damage that would be dealt to Ethereal Champion this turn.","2WWW","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eye Spy","U","Look at the top card of target player's library. You may put that card into his or her graveyard.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Protomatter Powder","U","{4}{W}, {T}, Sacrifice Protomatter Powder: Return target artifact card from your graveyard to the battlefield.","2U","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of the Crescent Moon","1","3","U","At the beginning of each player's draw step, that player draws an additional card.","UU","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bounty of the Hunt","G","You may exile a green card from your hand rather than pay Bounty of the Hunt's mana cost.
Distribute three +1/+1 counters among one, two, or three target creatures. For each +1/+1 counter you put on a creature this way, remove a +1/+1 counter from that creature at the beginning of the next cleanup step.","3GG","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rings of Brighthearth","Whenever you activate an ability, if it isn't a mana ability, you may pay {2}. If you do, copy that ability. You may choose new targets for the copy.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battering Wurm","4","3","G","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
Creatures with power less than Battering Wurm's power can't block it.","6G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Javelineers","1","1","W","Icatian Javelineers enters the battlefield with a javelin counter on it.
{T}, Remove a javelin counter from Icatian Javelineers: Icatian Javelineers deals 1 damage to target creature or player.","W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sundial of the Infinite","{1}, {T}: End the turn. Activate this ability only during your turn. (Exile all spells and abilities on the stack. Discard down to your maximum hand size. Damage wears off, and ""this turn"" and ""until end of turn"" effects end.)","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Witch Engine","4","4","B","Swampwalk
{T}: Add {B}{B}{B}{B} to your mana pool. Target opponent gains control of Witch Engine. (Activate this ability only any time you could cast an instant.)","5B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Retreat","W","Put a card from your hand on top of your library: Prevent all damage that would be dealt by target instant or sorcery spell this turn.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wojek Embermage","1","2","R","Radiance - {T}: Wojek Embermage deals 1 damage to target creature and each other creature that shares a color with it.","3R","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guul Draz Assassin","1","1","B","Level up {1}{B} ({1}{B}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 2-3
2/2
{B}, {T}: Target creature gets -2/-2 until end of turn.
LEVEL 4+
4/4
{B}, {T}: Target creature gets -4/-4 until end of turn.","B","Creature - Vampire Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mutagenic Growth","G","({(g/p)} can be paid with either {G} or 2 life.)
Target creature gets +2/+2 until end of turn.","(G/P)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Runes of the Deus","RG","Enchant creature
As long as enchanted creature is red, it gets +1/+1 and has double strike. (It deals both first-strike and regular combat damage.)
As long as enchanted creature is green, it gets +1/+1 and has trample.","4(R/G)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flaring Flame-Kin","2","2","R","As long as Flaring Flame-Kin is enchanted, it gets +2/+2, has trample, and has ""{R}: Flaring Flame-Kin gets +1/+0 until end of turn.""","2R","Creature - Elemental Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fireball","R","Fireball deals X damage divided evenly, rounded down, among any number of target creatures and/or players.
Fireball costs {1} more to cast for each target beyond the first.","XR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of Punishment","R","If Leyline of Punishment is in your opening hand, you may begin the game with it on the battlefield.
Players can't gain life.
Damage can't be prevented.","2RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boartusk Liege","3","4","RG","Trample
Other red creatures you control get +1/+1.
Other green creatures you control get +1/+1.","1(R/G)(R/G)(R/G)","Creature - Goblin Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Geyser Glider","4","4","R","Landfall - Whenever a land enters the battlefield under your control, Geyser Glider gains flying until end of turn.","3RR","Creature - Elemental Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Suture Spirit","1","1","W","Flying
{(w/b){(w/b){(w/b)}: Regenerate target creature.","1W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Willow Priestess","2","2","G","{T}: You may put a Faerie permanent card from your hand onto the battlefield.
{2}{G}: Target green creature gains protection from black until end of turn.","2GG","Creature - Faerie Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eladamri, Lord of Leaves Avatar","At the beginning of each player's precombat main phase, that player adds {G}{G} to his or her mana pool.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mnemonic Nexus","U","Each player shuffles his or her graveyard into his or her library.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Priest of Titania","1","1","G","{T}: Add {G} to your mana pool for each Elf on the battlefield.","1G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seeker of Skybreak","2","1","G","{T}: Untap target creature.","1G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Resplendent Mentor","2","2","W","White creatures you control have ""{T}: You gain 1 life.""","4W","Creature - Kithkin Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wargate","WUG","Search your library for a permanent card with converted mana cost X or less, put it onto the battlefield, then shuffle your library.","XGWU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Champion","2","2","G","Other Elf creatures get +1/+1 and have forestwalk. (They're unblockable as long as defending player controls a Forest.)","1GG","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Three Dreams","W","Search your library for up to three Aura cards with different names, reveal them, and put them into your hand. Then shuffle your library.","4W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Proclamation of Rebirth","W","Return up to three target creature cards with converted mana cost 1 or less from your graveyard to the battlefield.
Forecast - {5}{W}, Reveal Proclamation of Rebirth from your hand: Return target creature card with converted mana cost 1 or less from your graveyard to the battlefield. (Activate this ability only during your upkeep and only once each turn.)","2W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shambling Swarm","3","3","B","When Shambling Swarm dies, distribute three -1/-1 counters among one, two, or three target creatures. For each -1/-1 counter you put on a creature this way, remove a -1/-1 counter from that creature at the beginning of the next end step.","1BBB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earthcraft","G","Tap an untapped creature you control: Untap target basic land.","1G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mutavault","{T}: Add {1} to your mana pool.
{1}: Mutavault becomes a 2/2 creature with all creature types until end of turn. It's still a land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Vulshok Replica","3","1","{1}{R}, Sacrifice Vulshok Replica: Vulshok Replica deals 3 damage to target player.","3","Artifact Creature - Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scorch the Fields","R","Destroy target land. Scorch the Fields deals 1 damage to each Human creature.","4R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disruption Aura","U","Enchant artifact
Enchanted artifact has ""At the beginning of your upkeep, sacrifice this artifact unless you pay its mana cost.""","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dominus of Fealty","4","4","UR","Flying
At the beginning of your upkeep, you may gain control of target permanent until end of turn. If you do, untap it and it gains haste until end of turn.","(U/R)(U/R)(U/R)(U/R)(U/R)","Creature - Spirit Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fade from Memory","B","Exile target card from a graveyard.
Cycling {B} ({B}, Discard this card: Draw a card.)","B","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Steel Overseer","1","1","{T}: Put a +1/+1 counter on each artifact creature you control.","2","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stomper Cub","5","3","G","Trample","3GG","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bladed Bracers","Equipped creature gets +1/+1.
As long as equipped creature is a Human or an Angel, it has vigilance.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Pair","G","Whenever a creature enters the battlefield, if you cast it from your hand, you may search your library for a creature card with the same total power and toughness and put it onto the battlefield. If you do, shuffle your library.","4GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vampirism","B","Enchant creature
When Vampirism enters the battlefield, draw a card at the beginning of the next turn's upkeep.
Enchanted creature gets +1/+1 for each other creature you control.
Other creatures you control get -1/-1.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ulvenwald Tracker","1","1","G","{1}{G}, {T}: Target creature you control fights another target creature. (Each deals damage equal to its power to the other.)","G","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reflexes","R","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature has first strike. (It deals combat damage before creatures without first strike.)","R","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Champion's Helm","Equipped creature gets +2/+2.
As long as equipped creature is legendary, it has hexproof. (It can't be the target of spells or abilities your opponents control.)
Equip {1}","3","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Contagion Clasp","When Contagion Clasp enters the battlefield, put a -1/-1 counter on target creature.
{4}, {T}: Proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krovikan Horror","2","2","B","At the beginning of the end step, if Krovikan Horror is in your graveyard with a creature card directly above it, you may return Krovikan Horror to your hand.
{1}, Sacrifice a creature: Krovikan Horror deals 1 damage to target creature or player.","3B","Creature - Horror Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Homarid Spawning Bed","U","{1}{U}{U}, Sacrifice a blue creature: Put X 1/1 blue Camarid creature tokens onto the battlefield, where X is the sacrificed creature's converted mana cost.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Telekinetic Sliver","2","2","U","All Slivers have ""{T}: Tap target permanent.""","2UU","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Chronologist","1","3","U","{U}, Sacrifice Riptide Chronologist: Untap all creatures of the creature type of your choice.","3UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hell's Thunder","4","4","R","Flying, haste
At the beginning of the end step, sacrifice Hell's Thunder.
Unearth {4}{R} ({4}{R}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","1RR","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Irradiate","B","Target creature gets -1/-1 until end of turn for each artifact you control.","3B","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Fang Skulkin","2","1","{2}: Target black creature gains wither until end of turn. (It deals damage to creatures in the form of -1/-1 counters.)","2","Artifact Creature - Scarecrow");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gauntlets of Chaos","{5}, Sacrifice Gauntlets of Chaos: Exchange control of target artifact, creature, or land you control and target permanent an opponent controls that shares one of those types with it. If those permanents are exchanged this way, destroy all Auras attached to them.","5","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Undercity Reaches","Whenever a creature deals combat damage to a player, its controller may draw a card.
Whenever you roll {C}, you have no maximum hand size for the rest of the game.",NULL,"Plane - Ravnica");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orochi Eggwatcher","1","1","G","{2}{G}, {T}: Put a 1/1 green Snake creature token onto the battlefield. If you control ten or more creatures, flip Orochi Eggwatcher.
---
{G}, Sacrifice a creature: Target creature gets +3/+3 until end of turn.","2G","Creature - Snake Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lady Caleria","3","6","WG","{T}: Lady Caleria deals 3 damage to target attacking or blocking creature.","3GGWW","Legendary Creature - Human Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sasaya, Orochi Ascendant","2","3","G","Reveal your hand: If you have seven or more land cards in your hand, flip Sasaya, Orochi Ascendant.
---
Whenever a land you control is tapped for mana, for each other land you control with the same name, add one mana to your mana pool of any type that land produced.","1GG","Legendary Creature - Snake Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magmaw","4","4","R","{1}, Sacrifice a nonland permanent: Magmaw deals 1 damage to target creature or player.","3RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Broodlings","2","2","B","{1}, Sacrifice a creature: Put a +1/+1 counter on Phyrexian Broodlings.","1BB","Creature - Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghosts of the Damned","0","2","B","{T}: Target creature gets -1/-0 until end of turn.","1BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lady Sun","1","1","U","{T}: Return Lady Sun and another target creature to their owners' hands. Activate this ability only during your turn, before attackers are declared.","1UU","Legendary Creature - Human Advisor");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("North Star","{4}, {T}: For one spell this turn, you may spend mana as though it were mana of any color to pay that spell's mana cost. (Additional costs are still paid normally.)","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Power Matrix","{T}: Target creature gets +1/+1 and gains flying, first strike, and trample until end of turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Death-Hood Cobra","2","2","G","{1}{G}: Death-Hood Cobra gains reach until end of turn. (It can block creatures with flying.)
{1}{G}: Death-Hood Cobra gains deathtouch until end of turn. (Any amount of damage it deals to a creature is enough to destroy it.)","1G","Creature - Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gather Courage","G","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Target creature gets +2/+2 until end of turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jamuraan Lion","3","1","W","{W}, {T}: Target creature can't block this turn.","2W","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fling","R","As an additional cost to cast Fling, sacrifice a creature.
Fling deals damage equal to the sacrificed creature's power to target creature or player.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mycosynth Lattice","All permanents are artifacts in addition to their other types.
All cards that aren't on the battlefield, spells, and permanents are colorless.
Players may spend mana as though it were mana of any color.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barbed Foliage","G","Whenever a creature attacks you, it loses flanking until end of turn.
Whenever a creature without flying attacks you, Barbed Foliage deals 1 damage to it.","2GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Echoing Ruin","R","Destroy target artifact and all other artifacts with the same name as that artifact.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fyndhorn Elder","1","1","G","{T}: Add {G}{G} to your mana pool.","2G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunder","U","Return all lands to their owners' hands.","3UU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Everglades","Everglades enters the battlefield tapped.
When Everglades enters the battlefield, sacrifice it unless you return an untapped Swamp you control to its owner's hand.
{T}: Add {1}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Glimmerpost","When Glimmerpost enters the battlefield, you gain 1 life for each Locus on the battlefield.
{T}: Add {1} to your mana pool.",NULL,"Land - Locus");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Bladed Sentinel","2","4","{W}: Bladed Sentinel gains vigilance until end of turn.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seht's Tiger","3","3","W","Flash (You may cast this spell any time you could cast an instant.)
When Seht's Tiger enters the battlefield, you gain protection from the color of your choice until end of turn. (You can't be targeted, dealt damage, or enchanted by anything of the chosen color.)","2WW","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cloak of Invisibility","U","Enchant creature
Enchanted creature has phasing and can't be blocked except by Walls. (It phases in or out before its controller untaps during each of his or her untap steps. While it's phased out, it's treated as though it doesn't exist.)","U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enduring Ideal","W","Search your library for an enchantment card and put it onto the battlefield. Then shuffle your library.
Epic (For the rest of the game, you can't cast spells. At the beginning of each of your upkeeps, copy this spell except for its epic ability.)","5WW","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Brass Secretary","2","1","{2}, Sacrifice Brass Secretary: Draw a card.","3","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sylvan Messenger","2","2","G","Trample
When Sylvan Messenger enters the battlefield, reveal the top four cards of your library. Put all Elf cards revealed this way into your hand and the rest on the bottom of your library in any order.","3G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning Vengeance","R","Whenever you cast a spell from your graveyard, Burning Vengeance deals 2 damage to target creature or player.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nomads en-Kor","1","1","W","{0}: The next 1 damage that would be dealt to Nomads en-Kor this turn is dealt to target creature you control instead.","W","Creature - Kor Nomad Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Bottle Gnomes","1","3","Sacrifice Bottle Gnomes: You gain 3 life.","3","Artifact Creature - Gnome");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Steam Vents","({T}: Add {U} or {R} to your mana pool.)
As Steam Vents enters the battlefield, you may pay 2 life. If you don't, Steam Vents enters the battlefield tapped.",NULL,"Land - Island Mountain");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whispering Specter","1","1","B","Flying
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Whenever Whispering Specter deals combat damage to a player, you may sacrifice it. If you do, that player discards a card for each poison counter he or she has.","1BB","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plague Sliver","5","5","B","All Slivers have ""At the beginning of your upkeep, this permanent deals 1 damage to you.""","2BB","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Who/What/When/Where/Why","Who
oXoW
Instant
Target player gains X life.
-----
What
o2oR
Instant
Destroy target artifact.
-----
When
o2oU
Instant
Counter target creature spell.
-----
Where
o3oB
Instant
Destroy target land.
-----
Why
o1oG
Instant
Destroy target enchantment.",NULL,"Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heart of Yavimaya","If Heart of Yavimaya would enter the battlefield, sacrifice a Forest instead. If you do, put Heart of Yavimaya onto the battlefield. If you don't, put it into its owner's graveyard.
{T}: Add {G} to your mana pool.
{T}: Target creature gets +1/+1 until end of turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mourning","B","Enchant creature
Enchanted creature gets -2/-0.
{B}: Return Mourning to its owner's hand.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chicken à la King","2","2","U","Whenever a 6 is rolled on a six-sided die, put a +1/+1 counter on each Chicken in play. (You may roll dice only when a card instructs you to.)
Tap a Chicken you control: Roll a six-sided die.","1UU","Summon - Chicken");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sejiri Steppe","Sejiri Steppe enters the battlefield tapped.
When Sejiri Steppe enters the battlefield, target creature you control gains protection from the color of your choice until end of turn.
{T}: Add {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bay Falcon","1","1","U","Flying, vigilance","1U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spiritual Visit","W","Put a 1/1 colorless Spirit creature token onto the battlefield.
Splice onto Arcane {W} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","W","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rhystic Circle","W","{1}: Any player may pay {1}. If no one does, the next time a source of your choice would deal damage to you this turn, prevent that damage.","2WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mossfire Egg","{2}, {T}, Sacrifice Mossfire Egg: Add {R}{G} to your mana pool. Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Scryer","1","1","U","{T}: Add {1} to your mana pool.
{1}, {T}: Add {U} to your mana pool.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flowstone Surge","R","Creatures you control get +1/-1.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clairvoyance","U","Look at target player's hand.
Draw a card at the beginning of the next turn's upkeep.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sidewinder Sliver","1","1","W","All Sliver creatures have flanking. (Whenever a creature without flanking blocks a Sliver, the blocking creature gets -1/-1 until end of turn.)","W","Creature - Sliver");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Frogmite","2","2","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)","4","Artifact Creature - Frog");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nacre Talisman","Whenever a player casts a white spell, you may pay {3}. If you do, untap target permanent.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hurly-Burly","R","Choose one - Hurly-Burly deals 1 damage to each creature without flying; or Hurly-Burly deals 1 damage to each creature with flying.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Endemic Plague","B","As an additional cost to cast Endemic Plague, sacrifice a creature.
Destroy all creatures that share a creature type with the sacrificed creature. They can't be regenerated.","3B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Venser's Diffusion","U","Return target nonland permanent or suspended card to its owner's hand.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of the Nightly Hunt","R","Enchant player
Creatures enchanted player controls attack each turn if able.","2R","Enchantment - Aura Curse");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Oblivion Stone","{4}, {T}: Put a fate counter on target permanent.
{5}, {T}, Sacrifice Oblivion Stone: Destroy each nonland permanent without a fate counter on it, then remove all fate counters from all permanents.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrion Feeder","1","1","B","Carrion Feeder can't block.
Sacrifice a creature: Put a +1/+1 counter on Carrion Feeder.","B","Creature - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dormant Volcano","Dormant Volcano enters the battlefield tapped.
When Dormant Volcano enters the battlefield, sacrifice it unless you return an untapped Mountain you control to its owner's hand.
{T}: Add {1}{R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shield Mate","1","1","W","Sacrifice Shield Mate: Target creature gets +0/+4 until end of turn.","W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barreling Attack","R","Target creature gains trample until end of turn. When that creature becomes blocked this turn, it gets +1/+1 until end of turn for each creature blocking it.","2RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Carrion Call","G","Put two 1/1 green Insect creature tokens with infect onto the battlefield. (They deal damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lavafume Invoker","2","2","R","{8}: Creatures you control get +3/+0 until end of turn.","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crypt Angel","3","3","B","Flying, protection from white
When Crypt Angel enters the battlefield, return target blue or red creature card from your graveyard to your hand.","4B","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Outwit","U","Counter target spell that targets a player.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disarm","U","Unattach all Equipment from target creature.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cytospawn Shambler","0","0","G","Graft 6 (This creature enters the battlefield with six +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{G}: Target creature with a +1/+1 counter on it gains trample until end of turn.","6G","Creature - Elemental Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Man-o'-War","2","2","U","When Man-o'-War enters the battlefield, return target creature to its owner's hand.","2U","Creature - Jellyfish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dripping Dead","4","1","B","Dripping Dead can't block.
Whenever Dripping Dead deals combat damage to a creature, destroy that creature. It can't be regenerated.","4BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mark of Fury","R","Enchant creature
Enchanted creature has haste.
At the beginning of the end step, return Mark of Fury to its owner's hand.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Explosive Vegetation","G","Search your library for up to two basic land cards and put them onto the battlefield tapped. Then shuffle your library.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unnatural Hunger","B","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, Unnatural Hunger deals damage to that player equal to that creature's power unless he or she sacrifices another creature.","3BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faerie Swarm","*","*","U","Flying
Faerie Swarm's power and toughness are each equal to the number of blue permanents you control.","3U","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saprazzan Bailiff","2","2","U","When Saprazzan Bailiff enters the battlefield, exile all artifact and enchantment cards from all graveyards.
When Saprazzan Bailiff leaves the battlefield, return all artifact and enchantment cards from all graveyards to their owners' hands.","3UU","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Needlepeak Spider","4","2","R","Reach (This creature can block creatures with flying.)","3R","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thundercloud Shaman","4","4","R","When Thundercloud Shaman enters the battlefield, it deals damage equal to the number of Giants you control to each non-Giant creature.","3RR","Creature - Giant Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primitive Justice","R","As an additional cost to cast Primitive Justice, you may pay {1}{R} and/or {1}{G} any number of times.
Destroy target artifact. For each additional {1}{R} you paid, destroy another target artifact. For each additional {1}{G} you paid, destroy another target artifact, and you gain 1 life.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frost Giant","4","4","R","Rampage 2 (Whenever this creature becomes blocked, it gets +2/+2 until end of turn for each creature blocking it beyond the first.)","3RRR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blister Beetle","1","1","B","When Blister Beetle enters the battlefield, target creature gets -1/-1 until end of turn.","1B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recross the Paths","G","Reveal cards from the top of your library until you reveal a land card. Put that card onto the battlefield and the rest on the bottom of your library in any order. Clash with an opponent. If you win, return Recross the Paths to its owner's hand. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Divinity of Pride","4","4","WB","Flying, lifelink
Divinity of Pride gets +4/+4 as long as you have 25 or more life.","(W/B)(W/B)(W/B)(W/B)(W/B)","Creature - Spirit Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Xenic Poltergeist","1","1","B","{T}: Until your next upkeep, target noncreature artifact becomes an artifact creature with power and toughness each equal to its converted mana cost.","1BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ridge Rannet","6","4","R","Cycling {2} ({2}, Discard this card: Draw a card.)","5RR","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragon Fangs","G","Enchant creature
Enchanted creature gets +1/+1 and has trample.
When a creature with converted mana cost 6 or greater enters the battlefield, you may return Dragon Fangs from your graveyard to the battlefield attached to that creature.","1G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mons's Goblin Raiders","1","1","R",NULL,"R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pit Raptor","4","3","B","Flying, first strike
At the beginning of your upkeep, sacrifice Pit Raptor unless you pay {2}{B}{B}.","2BB","Creature - Bird Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moonmist","G","Transform all Humans. Prevent all combat damage that would be dealt this turn by creatures other than Werewolves and Wolves. (Only double-faced cards can be transformed.)","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ley Druid","1","1","G","{T}: Untap target land.","2G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Narwhal","2","2","U","First strike, protection from red","2UU","Creature - Whale");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angelic Wall","0","4","W","Defender, flying","1W","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Coalition Relic","{T}: Add one mana of any color to your mana pool.
{T}: Put a charge counter on Coalition Relic.
At the beginning of your precombat main phase, remove all charge counters from Coalition Relic. Add one mana of any color to your mana pool for each charge counter removed this way.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shimmering Glasskite","2","3","U","Flying
Whenever Shimmering Glasskite becomes the target of a spell or ability for the first time in a turn, counter that spell or ability.","3U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Improvised Armor","W","Enchant creature
Enchanted creature gets +2/+5.
Cycling {3} ({3}, Discard this card: Draw a card.)","3W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drowner Initiate","1","1","U","Whenever a player casts a blue spell, you may pay {1}. If you do, target player puts the top two cards of his or her library into his or her graveyard.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wurmcalling","G","Buyback {2}{G} (You may pay an additional {2}{G} as you cast this spell. If you do, put this card into your hand as it resolves.)
Put an X/X green Wurm creature token onto the battlefield.","XG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stromkirk Noble","1","1","R","Stromkirk Noble can't be blocked by Humans.
Whenever Stromkirk Noble deals combat damage to a player, put a +1/+1 counter on it.","R","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Salamander","3","4","R","{R}: Flowstone Salamander deals 1 damage to target creature blocking it.","3RR","Creature - Salamander");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Updraft","U","Target creature gains flying until end of turn.
Draw a card at the beginning of the next turn's upkeep.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thrive","G","Put a +1/+1 counter on each of X target creatures.","XG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Berserk","G","Cast Berserk only before the combat damage step.
Target creature gains trample and gets +X/+0 until end of turn, where X is its power. At the beginning of the next end step, destroy that creature if it attacked this turn.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Saltcrusted Steppe","{T}: Add {1} to your mana pool.
{1}, {T}: Put a storage counter on Saltcrusted Steppe.
{1}, Remove X storage counters from Saltcrusted Steppe: Add X mana in any combination of {G} and/or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magical Hacker","1","2","U","{U}: Change the text of target spell or permanent by replacing all instances of + with -, and vice versa, until end of turn.","1U","Creature - Human Gamer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pestermite","2","1","U","Flash
Flying
When Pestermite enters the battlefield, you may tap or untap target permanent.","2U","Creature - Faerie Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deep Spawn","6","6","U","Trample
At the beginning of your upkeep, sacrifice Deep Spawn unless you put the top two cards of your library into your graveyard.
{U}: Deep Spawn gains shroud until end of turn and doesn't untap during your next untap step. Tap Deep Spawn. (A permanent with shroud can't be the target of spells or abilities.)","5UUU","Creature - Homarid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seshiro the Anointed","3","4","G","Other Snake creatures you control get +2/+2.
Whenever a Snake you control deals combat damage to a player, you may draw a card.","4GG","Legendary Creature - Snake Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lancers en-Kor","3","3","W","Trample
{0}: The next 1 damage that would be dealt to Lancers en-Kor this turn is dealt to target creature you control instead.","3WW","Creature - Kor Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Counterspell","U","Counter target spell.","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warpath Ghoul","3","2","B",NULL,"2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gamekeeper","2","2","G","When Gamekeeper dies, you may exile it. If you do, reveal cards from the top of your library until you reveal a creature card. Put that card onto the battlefield and put all other cards revealed this way into your graveyard.","3G","Creature - Elf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Howling Mine","At the beginning of each player's draw step, if Howling Mine is untapped, that player draws an additional card.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stoic Champion","2","2","W","Whenever a player cycles a card, Stoic Champion gets +2/+2 until end of turn.","WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thwart","U","You may return three Islands you control to their owner's hand rather than pay Thwart's mana cost.
Counter target spell.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sneaky Homunculus","1","1","U","Sneaky Homunculus can't block or be blocked by creatures with power 2 or greater.","1U","Creature - Homunculus Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Odious Trow","1","1","BG","{1}{(b/g)}: Regenerate Odious Trow.","(B/G)","Creature - Troll");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Southern Paladin","3","3","W","{W}{W}, {T}: Destroy target red permanent.","2WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Noble Steeds","W","{1}{W}: Target creature gains first strike until end of turn.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Armorguard","3","3","WG","When Leonin Armorguard enters the battlefield, creatures you control get +1/+1 until end of turn.","2GW","Creature - Cat Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Entrancer","1","1","U","Whenever Riptide Entrancer deals combat damage to a player, you may sacrifice it. If you do, gain control of target creature that player controls. (This effect lasts indefinitely.)
Morph {U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbarian Lunatic","2","1","R","{2}{R}, Sacrifice Barbarian Lunatic: Barbarian Lunatic deals 2 damage to target creature.","2R","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Darksteel Juggernaut","*","*","Darksteel Juggernaut's power and toughness are each equal to the number of artifacts you control.
Darksteel Juggernaut is indestructible and attacks each turn if able.","5","Artifact Creature - Juggernaut");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Axegrinder Giant","6","4","R",NULL,"4RR","Creature - Giant Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Viridian Claw","Equipped creature gets +1/+0 and has first strike.
Equip {1}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flesh Reaver","4","4","B","Whenever Flesh Reaver deals damage to a creature or opponent, Flesh Reaver deals that much damage to you.","1B","Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seraph Sanctuary","When Seraph Sanctuary enters the battlefield, you gain 1 life.
Whenever an Angel enters the battlefield under your control, you gain 1 life.
{T}: Add {1} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Claws of Wirewood","G","Claws of Wirewood deals 3 damage to each creature with flying and each player.
Cycling {2} ({2}, Discard this card: Draw a card.)","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormscape Battlemage","2","2","U","Kicker {W} and/or {2}{B} (You may pay an additional {W} and/or {2}{B} as you cast this spell.)
When Stormscape Battlemage enters the battlefield, if it was kicked with its {W} kicker, you gain 3 life.
When Stormscape Battlemage enters the battlefield, if it was kicked with its {2}{B} kicker, destroy target nonblack creature. That creature can't be regenerated.","2U","Creature - Metathran Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sparksmith","1","1","R","{T}: Sparksmith deals X damage to target creature and X damage to you, where X is the number of Goblins on the battlefield.","1R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elixir of Immortality","{2}, {T}: You gain 5 life. Shuffle Elixir of Immortality and your graveyard into their owner's library.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Surrakar Marauder","2","1","B","Landfall - Whenever a land enters the battlefield under your control, Surrakar Marauder gains intimidate until end of turn. (It can't be blocked except by artifact creatures and/or creatures that share a color with it.)","1B","Creature - Surrakar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inspirit","W","Untap target creature. It gets +2/+4 until end of turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vexing Shusher","2","2","RG","Vexing Shusher can't be countered.
{(r/g)}: Target spell can't be countered by spells or abilities.","(R/G)(R/G)","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undead Alchemist","4","2","U","If a Zombie you control would deal combat damage to a player, instead that player puts that many cards from the top of his or her library into his or her graveyard.
Whenever a creature card is put into an opponent's graveyard from his or her library, exile that card and put a 2/2 black Zombie creature token onto the battlefield.","3U","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skulltap","B","As an additional cost to cast Skulltap, sacrifice a creature.
Draw two cards.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Nantuko","0","0","G","Trample
Phantom Nantuko enters the battlefield with two +1/+1 counters on it.
If damage would be dealt to Phantom Nantuko, prevent that damage. Remove a +1/+1 counter from Phantom Nantuko.
{T}: Put a +1/+1 counter on Phantom Nantuko.","2G","Creature - Insect Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scarscale Ritual","UB","As an additional cost to cast Scarscale Ritual, put a -1/-1 counter on a creature you control.
Draw two cards.","1(U/B)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flicker","W","Exile target nontoken permanent, then return it to the battlefield under its owner's control.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhox","5","5","G","You may have Rhox assign its combat damage as though it weren't blocked.
{2}{G}: Regenerate Rhox. (The next time this creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","4GG","Creature - Rhino Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lava Zombie","4","3","BR","When Lava Zombie enters the battlefield, return a black or red creature you control to its owner's hand.
{2}: Lava Zombie gets +1/+0 until end of turn.","1BR","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drafna's Restoration","U","Return any number of target artifact cards from target player's graveyard to the top of his or her library in any order.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gift of Estates","W","If an opponent controls more lands than you, search your library for up to three Plains cards, reveal them, and put them into your hand. Then shuffle your library.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jasmine Boreal","4","5","WG",NULL,"3GW","Legendary Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frost Ogre","5","3","R",NULL,"3RR","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mothrider Samurai","2","2","W","Flying
Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","3W","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Psychopath","5","5","R","Whenever Goblin Psychopath attacks or blocks, flip a coin. If you lose the flip, the next time it would deal combat damage this turn, it deals that damage to you instead.","3R","Creature - Goblin Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disciple of Malice","1","2","B","Protection from white
Cycling {2} ({2}, Discard this card: Draw a card.)","1B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burn the Impure","R","Burn the Impure deals 3 damage to target creature. If that creature has infect, Burn the Impure deals 3 damage to that creature's controller.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Terashi's Grasp","W","Destroy target artifact or enchantment. You gain life equal to its converted mana cost.","2W","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Piracy Charm","U","Choose one - Target creature gains islandwalk until end of turn; or target creature gets +2/-1 until end of turn; or target player discards a card.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychotic Episode","B","Target player reveals his or her hand and the top card of his or her library. You choose a card revealed this way. That player puts the chosen card on the bottom of his or her library.
Madness {1}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dawn Charm","W","Choose one - Prevent all combat damage that would be dealt this turn; or regenerate target creature; or counter target spell that targets you.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("All Suns' Dawn","G","For each color, return up to one target card of that color from your graveyard to your hand. Exile All Suns' Dawn.","4G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Entangler","W","Enchant creature
Enchanted creature can block any number of creatures.","2WW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Mechanics","1","1","R","{T}, Sacrifice an artifact: Orcish Mechanics deals 2 damage to target creature or player.","2R","Creature - Orc");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sporesower Thallid","4","4","G","At the beginning of your upkeep, put a spore counter on each Fungus you control.
Remove three spore counters from Sporesower Thallid: Put a 1/1 green Saproling creature token onto the battlefield.","2GG","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strongarm Tactics","B","Each player discards a card. Then each player who didn't discard a creature card this way loses 4 life.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Catastrophe","W","Destroy all lands or all creatures. Creatures destroyed this way can't be regenerated.","4WW","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bottled Cloister","At the beginning of each opponent's upkeep, exile all cards from your hand face down.
At the beginning of your upkeep, return all cards you own exiled with Bottled Cloister to your hand, then draw a card.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guilty Conscience","W","Enchant creature
Whenever enchanted creature deals damage, Guilty Conscience deals that much damage to that creature.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodied Ghost","3","3","WB","Flying
Bloodied Ghost enters the battlefield with a -1/-1 counter on it.","1(W/B)(W/B)","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Celestial Kirin","3","3","W","Flying
Whenever you cast a Spirit or Arcane spell, destroy all permanents with that spell's converted mana cost.","2WW","Legendary Creature - Kirin Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sage's Knowledge","U","Return target sorcery card from your graveyard to your hand.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("War-Torch Goblin","1","1","R","{R}, Sacrifice War-Torch Goblin: War-Torch Goblin deals 2 damage to target blocking creature.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brilliant Ultimatum","WUB","Exile the top five cards of your library. An opponent separates those cards into two piles. You may play any number of cards from one of those piles without paying their mana costs.","WWUUUBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Crab","3","3","U","{U}: Giant Crab gains shroud until end of turn. (It can't be the target of spells or abilities.)","4U","Creature - Crab");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mercadian Lift","{1}, {T}: Put a winch counter on Mercadian Lift.
{T}, Remove X winch counters from Mercadian Lift: You may put a creature card with converted mana cost X from your hand onto the battlefield.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellkite Igniter","5","5","R","Flying, haste
{1}{R}: Hellkite Igniter gets +X/+0 until end of turn, where X is the number of artifacts you control.","5RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Atinlay Igpay","3","3","W","Oubleday ikestray
Eneverwhay Atinlay Igpay's ontrollercay eaksspay ay onnay-Igpay-Atinlay ordway, acrificesay Atinlay Igpay.","5W","Eaturecray - Igpay");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bringer of the Red Dawn","5","5","R","You may pay {W}{U}{B}{R}{G} rather than pay Bringer of the Red Dawn's mana cost.
Trample
At the beginning of your upkeep, you may untap target creature and gain control of it until end of turn. That creature gains haste until end of turn.","7RR","Creature - Bringer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Take Possession","U","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Enchant permanent
You control enchanted permanent.","5UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jackal Pup","2","1","R","Whenever Jackal Pup is dealt damage, it deals that much damage to you.","R","Creature - Hound");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Miter","Whenever an artifact you control is put into a graveyard from the battlefield, if it wasn't sacrificed, you may pay {3}. If you do, draw a card.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Desert Drake","2","2","R","Flying","3R","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coral Merfolk","2","1","U",NULL,"1U","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serenity","W","At the beginning of your upkeep, destroy all artifacts and enchantments. They can't be regenerated.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bushi Tenderfoot","1","1","W","When a creature dealt damage by Bushi Tenderfoot this turn dies, flip Bushi Tenderfoot.
---
Double strike; bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Drake","2","3","UB","Flying
Urborg Drake attacks each turn if able.","1UB","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orochi Leafcaller","1","1","G","{G}: Add one mana of any color to your mana pool.","G","Creature - Snake Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Samurai of the Pale Curtain","2","2","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
If a permanent would be put into a graveyard, exile it instead.","WW","Creature - Fox Samurai");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of Kaldra","Equipped creature gets +5/+5.
Whenever equipped creature deals damage to a creature, exile that creature.
Equip {4} ({4}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","4","Legendary Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blastoderm","5","5","G","Shroud (This permanent can't be the target of spells or abilities.)
Fading 3 (This creature enters the battlefield with three fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)","2GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noxious Ghoul","3","3","B","Whenever Noxious Ghoul or another Zombie enters the battlefield, all non-Zombie creatures get -1/-1 until end of turn.","3BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ohran Viper","1","3","G","Whenever Ohran Viper deals combat damage to a creature, destroy that creature at end of combat.
Whenever Ohran Viper deals combat damage to a player, you may draw a card.","1GG","Snow Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spellwild Ouphe","1","3","G","Spells that target Spellwild Ouphe cost {2} less to cast.","1G","Creature - Ouphe");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Evil Eye of Urborg","6","3","B","Non-Eye creatures you control can't attack.
Whenever Evil Eye of Urborg becomes blocked by a creature, destroy that creature.","4B","Creature - Eye");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Retaliate","W","Destroy all creatures that dealt damage to you this turn.","2WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodwraith Corrupter","3","6","BG","{1}{B}{G}, {T}: Target Forest becomes a 4/4 black and green Elemental Horror creature. It's still a land.","3BBG","Creature - Elemental Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ad Nauseam","B","Reveal the top card of your library and put that card into your hand. You lose life equal to its converted mana cost. You may repeat this process any number of times.","3BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Belbe's Percher","2","2","B","Flying
Belbe's Percher can block only creatures with flying.","2B","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Stalwart","3","3","W","Vigilance
{W}: Loxodon Stalwart gets +0/+1 until end of turn.","3WW","Creature - Elephant Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hulking Goblin","2","2","R","Hulking Goblin can't block.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rathi Dragon","5","5","R","Flying (This creature can't be blocked except by creatures with flying or reach.)
When Rathi Dragon enters the battlefield, sacrifice it unless you sacrifice two Mountains.","2RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Idle Thoughts","U","{2}: Draw a card if you have no cards in hand.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burrenton Bombardier","2","2","W","Flying
Reinforce 2-{2}{W} ({2}{W}, Discard this card: Put two +1/+1 counters on target creature.)","2W","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shining Shoal","W","You may exile a white card with converted mana cost X from your hand rather than pay Shining Shoal's mana cost.
The next X damage that a source of your choice would deal to you and/or creatures you control this turn is dealt to target creature or player instead.","XWW","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magnify","G","All creatures get +1/+1 until end of turn.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stronghold Furnace","If a source would deal damage to a creature or player, it deals double that damage to that creature or player instead.
Whenever you roll {C}, Stronghold Furnace deals 1 damage to target creature or player.",NULL,"Plane - Rath");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cackling Imp","2","2","B","Flying
{T}: Target player loses 1 life.","2BB","Creature - Imp");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fluctuator","Cycling abilities you activate cost you up to {2} less to activate.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightshade Peddler","1","1","G","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Nightshade Peddler is paired with another creature, both creatures have deathtouch.","1G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ophidian","1","3","U","Whenever Ophidian attacks and isn't blocked, you may draw a card. If you do, Ophidian assigns no combat damage this turn.","2U","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Intimidator Initiate","1","1","R","Whenever a player casts a red spell, you may pay {1}. If you do, target creature can't block this turn.","R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stalking Vengeance","5","5","R","Haste
Whenever another creature you control dies, it deals damage equal to its power to target player.","5RR","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zephyr Sprite","1","1","U","Flying","U","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leshrac's Rite","B","Enchant creature
Enchanted creature has swampwalk.","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Marshaling the Troops","G","Tap any number of untapped creatures you control. You gain 4 life for each creature tapped this way.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elder Pine of Jukai","2","1","G","Whenever you cast a Spirit or Arcane spell, reveal the top three cards of your library. Put all land cards revealed this way into your hand and the rest on the bottom of your library in any order.
Soulshift 2 (When this creature dies, you may return target Spirit card with converted mana cost 2 or less from your graveyard to your hand.)","2G","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rith, the Awakener Avatar","Whenever a creature you control deals combat damage to a player, you may pay {5}. If you do, put a 5/5 red Dragon creature token with flying onto the battlefield.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trusted Advisor","1","2","U","Your maximum hand size is increased by two.
At the beginning of your upkeep, return a blue creature you control to its owner's hand.","U","Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Bazaar","0","1","U","{T}: Draw two cards, then discard three cards.","1U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("High Ground","W","Each creature you control can block an additional creature.","W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whip Silk","G","Enchant creature
Enchanted creature has reach. (It can block creatures with flying.)
{G}: Return Whip Silk to its owner's hand.","G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Polluted Delta","{T}, Pay 1 life, Sacrifice Polluted Delta: Search your library for an Island or Swamp card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anaba Bodyguard","2","3","R","First strike (This creature deals combat damage before creatures without first strike.)","3R","Creature - Minotaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plague Stinger","1","1","B","Flying
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","1B","Creature - Insect Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silver Seraph","6","6","W","Flying
Threshold - Other creatures you control get +2/+2 as long as seven or more cards are in your graveyard.","5WWW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pollen Remedy","W","Kicker-Sacrifice a land. (You may sacrifice a land in addition to any other costs as you cast this spell.)
Prevent the next 3 damage that would be dealt this turn to any number of target creatures and/or players, divided as you choose. If Pollen Remedy was kicked, prevent the next 6 damage this way instead.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eviscerator","5","5","B","Protection from white
When Eviscerator enters the battlefield, you lose 5 life.","3BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Abjure","U","As an additional cost to cast Abjure, sacrifice a blue permanent.
Counter target spell.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vow of Flight","U","Enchant creature
Enchanted creature gets +2/+2, has flying, and can't attack you or a planeswalker you control.","2U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Selective Memory","U","Search your library for any number of nonland cards and exile them. Then shuffle your library.","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vitalizing Wind","G","Creatures you control get +7/+7 until end of turn.","8G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Granny's Payback","G","You gain life equal to your age.","7G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Meddling Kids","2","3","WU","As Meddling Kids comes into play, choose a word with four or more letters.
Nonland cards with the chosen word in their text box can't be played.","2WU","Creature - Human Child");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lash Out","R","Lash Out deals 3 damage to target creature. Clash with an opponent. If you win, Lash Out deals 3 damage to that creature's controller. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Squee, Goblin Nabob Avatar","{1}: Prevent the next 1 damage that would be dealt to target creature you control this turn.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Covenant of Minds","U","Reveal the top three cards of your library. Target opponent may choose to put those cards into your hand. If he or she doesn't, put those cards into your graveyard and draw five cards.","4U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Reclamation","B","{1}{B}, Pay 2 life: Return target creature card from your graveyard to your hand.","B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opal Guardian","W","When an opponent casts a creature spell, if Opal Guardian is an enchantment, Opal Guardian becomes a 3/4 Gargoyle creature with flying and protection from red.","WWW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Waves of Aggression","WR","Untap all creatures that attacked this turn. After this main phase, there is an additional combat phase followed by an additional main phase.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","3(R/W)(R/W)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Hunger","G","Target creature gets +3/+1 and gains trample until end of turn.
Flashback {3}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nausea","B","All creatures get -1/-1 until end of turn.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iwamori of the Open Fist","5","5","G","Trample
When Iwamori of the Open Fist enters the battlefield, each opponent may put a legendary creature card from his or her hand onto the battlefield.","2GG","Legendary Creature - Human Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flowstone Blade","R","Enchant creature
{R}: Enchanted creature gets +1/-1 until end of turn.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Junk Golem","0","0","Junk Golem enters the battlefield with three +1/+1 counters on it.
At the beginning of your upkeep, sacrifice Junk Golem unless you remove a +1/+1 counter from it.
{1}, Discard a card: Put a +1/+1 counter on Junk Golem.","4","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Untaidake, the Cloud Keeper","Untaidake, the Cloud Keeper enters the battlefield tapped.
{T}, Pay 2 life: Add {2} to your mana pool. Spend this mana only to cast legendary spells.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Night Dealings","B","Whenever a source you control deals damage to another player, put that many theft counters on Night Dealings.
{2}{B}{B}, Remove X theft counters from Night Dealings: Search your library for a nonland card with converted mana cost X, reveal it, and put it into your hand. Then shuffle your library.","2BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Game of Chaos","R","Flip a coin. If you win the flip, you gain 1 life and target opponent loses 1 life, and you decide whether to flip again. If you lose the flip, you lose 1 life and that opponent gains 1 life, and that player decides whether to flip again. Double the life stakes with each flip.","RRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Primarch","3","3","G","Convoke (Each creature you tap while casting this spell reduces its total cost by {1} or by one mana of that creature's color.)
Kicker {4} (You may pay an additional {4} as you cast this spell.)
If Kavu Primarch was kicked, it enters the battlefield with four +1/+1 counters on it.","3G","Creature - Kavu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Land Equilibrium","U","If an opponent who controls at least as many lands as you do would put a land onto the battlefield, that player instead puts that land onto the battlefield then sacrifices a land.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Radjan Spirit","3","2","G","{T}: Target creature loses flying until end of turn.","3G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pawn of Ulamog","2","2","B","Whenever Pawn of Ulamog or another nontoken creature you control dies, you may put a 0/1 colorless Eldrazi Spawn creature token onto the battlefield. It has ""Sacrifice this creature: Add {1} to your mana pool.""","1BB","Creature - Vampire Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Samurai Enforcers","4","4","W","Bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)","4WW","Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancestral Memories","U","Look at the top seven cards of your library. Put two of them into your hand and the rest into your graveyard.","2UUU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathforge Shaman","4","3","R","Multikicker {R} (You may pay an additional {R} any number of times as you cast this spell.)
When Deathforge Shaman enters the battlefield, it deals damage to target player equal to twice the number of times it was kicked.","4R","Creature - Ogre Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sungrass Prairie","{1}, {T}: Add {G}{W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Takara","Sacrifice a creature: Takara deals 1 damage to target creature or player.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Midnight Covenant","B","Enchant creature
Enchanted creature has ""{B}: This creature gets +1/+1 until end of turn.""","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alexi's Cloak","U","Flash
Enchant creature
Enchanted creature has shroud. (It can't be the target of spells or abilities.)","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Immerwolf","2","2","RG","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)
Other Wolf and Werewolf creatures you control get +1/+1.
Non-Human Werewolves you control can't transform.","1RG","Creature - Wolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seismic Shudder","R","Seismic Shudder deals 1 damage to each creature without flying.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Verdigris","G","Destroy target artifact.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dawnstrider","1","1","G","{G}, {T}, Discard a card: Prevent all combat damage that would be dealt this turn.","1G","Creature - Dryad Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sanguine Praetor","7","5","B","{B}, Sacrifice a creature: Destroy each creature with the same converted mana cost as the sacrificed creature.","6BB","Creature - Avatar Praetor");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Golem's Heart","Whenever a player casts an artifact spell, you may gain 1 life.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shattered Dreams","B","Target opponent reveals his or her hand. You choose an artifact card from it. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Indigo Faerie","1","1","U","Flying
{U}: Target permanent becomes blue in addition to its other colors until end of turn.","1U","Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ferocity","G","Enchant creature
Whenever enchanted creature blocks or becomes blocked, you may put a +1/+1 counter on it.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of the Void","B","If Leyline of the Void is in your opening hand, you may begin the game with it on the battlefield.
If a card would be put into an opponent's graveyard from anywhere, exile it instead.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Machinist","0","5","R","{2}{R}: Reveal cards from the top of your library until you reveal a nonland card. Goblin Machinist gets +X/+0 until end of turn, where X is that card's converted mana cost. Put the revealed cards on the bottom of your library in any order.","4R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gray Ogre","2","2","R",NULL,"2R","Creature - Ogre");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Common Courtesy","U","Counter any spell unless its caster asks your permission to play that spell. If you refuse permission, Sacrifice Common Courtesy and counter the spell.","2UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Metal Fatigue","W","Tap all artifacts.","2W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wildblood Pack","5","5","Trample
Attacking creatures you control get +3/+0.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Wildblood Pack.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sewn-Eye Drake","3","1","UBR","Flying, haste","2(U/R)B","Creature - Zombie Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invasion Plans","R","All creatures block each turn if able.
The attacking player chooses how each creature blocks each turn.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blind Phantasm","2","3","U",NULL,"2U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fire and Brimstone","W","Fire and Brimstone deals 4 damage to target player who declared an attacking creature this turn and 4 damage to you.","3WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vexing Devil","4","3","R","When Vexing Devil enters the battlefield, any opponent may have it deal 4 damage to him or her. If a player does, sacrifice Vexing Devil.","R","Creature - Devil");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ryusei, the Falling Star","5","5","R","Flying
When Ryusei, the Falling Star dies, it deals 5 damage to each creature without flying.","5R","Legendary Creature - Dragon Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Righteous Avengers","3","1","W","Plainswalk","4W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wanderwine Prophets","4","4","U","Champion a Merfolk (When this enters the battlefield, sacrifice it unless you exile another Merfolk you control. When this leaves the battlefield, that card returns to the battlefield.)
Whenever Wanderwine Prophets deals combat damage to a player, you may sacrifice a Merfolk. If you do, take an extra turn after this one.","4UU","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moorish Cavalry","3","3","W","Trample","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lambholt Elder","1","2","G","At the beginning of each upkeep, if no spells were cast last turn, transform Lambholt Elder.","2G","Creature - Human Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Red Elemental Blast","R","Choose one - Counter target blue spell; or destroy target blue permanent.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Biomantic Mastery","UG","({(g/u)} can be paid with either {G} or {U}.)
Draw a card for each creature target player controls, then draw a card for each creature another target player controls.","4(G/U)(G/U)(G/U)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Joven's Ferrets","1","1","G","Whenever Joven's Ferrets attacks, it gets +0/+2 until end of turn.
At end of combat, tap all creatures that blocked Joven's Ferrets this turn. They don't untap during their controller's next untap step.","G","Creature - Ferret");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Izzet Boilerworks","Izzet Boilerworks enters the battlefield tapped.
When Izzet Boilerworks enters the battlefield, return a land you control to its owner's hand.
{T}: Add {U}{R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Evershrike","2","2","WB","Flying
Evershrike gets +2/+2 for each Aura attached to it.
{X}{(w/b){(w/b)}: Return Evershrike from your graveyard to the battlefield. You may put an Aura card with converted mana cost X or less from your hand onto the battlefield attached to it. If you don't, exile Evershrike.","3(W/B)(W/B)","Creature - Elemental Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Searing Spear Askari","2","2","R","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{1}{R}: Searing Spear Askari can't be blocked except by two or more creatures this turn.","2R","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sorin's Thirst","B","Sorin's Thirst deals 2 damage to target creature and you gain 2 life.","BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cunning Wish","U","You may choose an instant card you own from outside the game, reveal that card, and put it into your hand. Exile Cunning Wish.","2U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stonehewer Giant Avatar","Whenever a creature enters the battlefield under your control, put a token onto the battlefield that's a copy of a random Equipment card with converted mana cost less than or equal to that creature's converted mana cost. Attach that Equipment to that creature.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hanna","Spells you cast cost {1} less to cast.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Springleaf Drum","{T}, Tap an untapped creature you control: Add one mana of any color to your mana pool.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rabid Rats","1","1","B","{T}: Target blocking creature gets -1/-1 until end of turn.","1B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Priests of Norn","1","4","W","Vigilance
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","2W","Creature - Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eladamri","{0}: The next 1 damage that would be dealt to target creature you control is dealt to you instead.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keepers of the Faith","2","3","W",NULL,"1WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skarrgan Skybreaker","3","3","RG","Bloodthirst 3 (If an opponent was dealt damage this turn, this creature enters the battlefield with three +1/+1 counters on it.)
{1}, Sacrifice Skarrgan Skybreaker: Skarrgan Skybreaker deals damage equal to its power to target creature or player.","4RRG","Creature - Giant Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lagac Lizard","3","3","R",NULL,"3R","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Radiant Kavu","3","3","WRG","{R}{G}{W}: Prevent all combat damage blue creatures and black creatures would deal this turn.","RGW","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krakilin","0","0","G","Krakilin enters the battlefield with X +1/+1 counters on it.
{1}{G}: Regenerate Krakilin.","XGG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Praetor's Counsel","G","Return all cards from your graveyard to your hand. Exile Praetor's Counsel. You have no maximum hand size for the rest of the game.","5GGG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Eulogist","1","1","G","Sacrifice Elvish Eulogist: You gain 1 life for each Elf card in your graveyard.","G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Autumn's Veil","G","Spells you control can't be countered by blue or black spells this turn, and creatures you control can't be the targets of blue or black spells this turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aboroth","9","9","G","Cumulative upkeep-Put a -1/-1 counter on Aboroth. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","4GG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bellowing Tanglewurm","4","4","G","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)
Other green creatures you control have intimidate.","3GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brink of Disaster","B","Enchant creature or land
When enchanted permanent becomes tapped, destroy it.","2BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbarian Outcast","2","2","R","When you control no Swamps, sacrifice Barbarian Outcast.","1R","Creature - Human Barbarian Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lieutenant Kirtar","2","2","W","Flying
{1}{W}, Sacrifice Lieutenant Kirtar: Exile target attacking creature.","1WW","Legendary Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Toxic Nim","4","1","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
{B}: Regenerate Toxic Nim.","4BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Graveborn Muse","3","3","B","At the beginning of your upkeep, you draw X cards and you lose X life, where X is the number of Zombies you control.","2BB","Creature - Zombie Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flight of Fancy","U","Enchant creature
When Flight of Fancy enters the battlefield, draw two cards.
Enchanted creature has flying.","3U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sleeper Agent","3","3","B","When Sleeper Agent enters the battlefield, target opponent gains control of it.
At the beginning of your upkeep, Sleeper Agent deals 2 damage to you.","B","Creature - Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fiend Hunter","1","3","W","When Fiend Hunter enters the battlefield, you may exile another target creature.
When Fiend Hunter leaves the battlefield, return the exiled card to the battlefield under its owner's control.","1WW","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Delif's Cube","{2}, {T}: This turn, when target creature you control attacks and isn't blocked, it assigns no combat damage this turn and you put a cube counter on Delif's Cube.
{2}, Remove a cube counter from Delif's Cube: Regenerate target creature.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Yavimaya Coast","{T}: Add {1} to your mana pool.
{T}: Add {G} or {U} to your mana pool. Yavimaya Coast deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tattermunge Maniac","2","1","RG","Tattermunge Maniac attacks each turn if able.","(R/G)","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sosuke, Son of Seshiro","3","4","G","Other Snake creatures you control get +1/+0.
Whenever a Warrior you control deals combat damage to a creature, destroy that creature at end of combat.","2GG","Legendary Creature - Snake Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moonlace","U","Target spell or permanent becomes colorless.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("River Kelpie","3","3","U","Whenever River Kelpie or another permanent is put onto the battlefield from a graveyard, draw a card.
Whenever a player casts a spell from a graveyard, draw a card.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","3UU","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grip of Amnesia","U","Counter target spell unless its controller exiles all cards from his or her graveyard.
Draw a card.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Borrowing the East Wind","G","Borrowing the East Wind deals X damage to each creature with horsemanship and each player.","XGG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blessing of the Nephilim","W","Enchant creature
Enchanted creature gets +1/+1 for each of its colors.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Spelunkers","2","2","R","Mountainwalk","2R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dragon Mask","{3}, {T}: Target creature you control gets +2/+2 until end of turn. Return it to its owner's hand at the beginning of the next end step. (Return it only if it's on the battlefield.)","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talon Trooper","2","3","WU","Flying","1WU","Creature - Bird Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Azorius Signet","{1}, {T}: Add {W}{U} to your mana pool.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hypochondria","W","{W}, Discard a card: Prevent the next 3 damage that would be dealt to target creature or player this turn.
{W}, Sacrifice Hypochondria: Prevent the next 3 damage that would be dealt to target creature or player this turn.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Warchief","1","3","U","Creature spells you cast that share a creature type with Mistform Warchief cost {1} less to cast.
{T}: Mistform Warchief becomes the creature type of your choice until end of turn.","2U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oakgnarl Warrior","5","7","G","Vigilance, trample","5GG","Creature - Treefolk Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Jhoira's Toolbox","1","1","{2}: Regenerate target artifact creature.","2","Artifact Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brink of Madness","B","At the beginning of your upkeep, if you have no cards in hand, sacrifice Brink of Madness and target opponent discards his or her hand.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stinging Barrier","0","4","U","Defender (This creature can't attack.)
{U}, {T}: Stinging Barrier deals 1 damage to target creature or player.","2UU","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riot Devils","2","3","R",NULL,"2R","Creature - Devil");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seaside Haven","{T}: Add {1} to your mana pool.
{W}{U}, {T}, Sacrifice a Bird: Draw a card.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Teeka's Dragon","5","5","Flying; trample; rampage 4 (Whenever this creature becomes blocked, it gets +4/+4 until end of turn for each creature blocking it beyond the first.)","9","Artifact Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twisted Experiment","B","Enchant creature
Enchanted creature gets +3/-1.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rubinia Soulsinger","2","3","WUG","You may choose not to untap Rubinia Soulsinger during your untap step.
{T}: Gain control of target creature for as long as you control Rubinia and Rubinia remains tapped.","2GWU","Legendary Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armored Warhorse","2","3","W",NULL,"WW","Creature - Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Balance of Power","U","If target opponent has more cards in hand than you, draw cards equal to the difference.","3UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Otherworldly Journey","W","Exile target creature. At the beginning of the next end step, return that card to the battlefield under its owner's control with a +1/+1 counter on it.","1W","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hazy Homunculus","1","1","U","Hazy Homunculus is unblockable as long as defending player controls an untapped land.","1U","Creature - Homunculus Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sorcerous Sight","U","Look at target opponent's hand.
Draw a card.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Colos Yearling","1","1","R","Mountainwalk
{R}: Colos Yearling gets +1/+0 until end of turn.","2R","Creature - Goat Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sanctum Plowbeast","3","6","WU","Defender
Plainscycling {2}, islandcycling {2} ({2}, Discard this card: Search your library for a Plains or Island card, reveal it, and put it into your hand. Then shuffle your library.)","4WU","Artifact Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overburden","U","Whenever a player puts a nontoken creature onto the battlefield, that player returns a land he or she controls to its owner's hand.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mighty Emergence","G","Whenever a creature with power 5 or greater enters the battlefield under your control, you may put two +1/+1 counters on it.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Preacher","1","1","W","You may choose not to untap Preacher during your untap step.
{T}: Gain control of target creature of an opponent's choice that he or she controls for as long as Preacher remains tapped.","1WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rotting Legion","4","5","B","Rotting Legion enters the battlefield tapped.","4B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Retrieval","U","Return target instant or sorcery card from your graveyard to your hand.
Flashback {2}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Saproling Burst","G","Fading 7 (This enchantment enters the battlefield with seven fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Remove a fade counter from Saproling Burst: Put a green Saproling creature token onto the battlefield. It has ""This creature's power and toughness are each equal to the number of fade counters on Saproling Burst.""
When Saproling Burst leaves the battlefield, destroy all tokens put onto the battlefield with Saproling Burst. They can't be regenerated.","4G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stabilizer","Players can't cycle cards.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fallen Ferromancer","1","1","R","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
{1}{R}, {T}: Fallen Ferromancer deals 1 damage to target creature or player.","3R","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Facevaulter","1","1","B","{B}, Sacrifice a Goblin: Facevaulter gets +2/+2 until end of turn.","B","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thieving Magpie","1","3","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
Whenever Thieving Magpie deals damage to an opponent, draw a card.","2UU","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raiding Nightstalker","2","2","B","Swampwalk","2B","Creature - Nightstalker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Strike Force","4","3","R","Discard a card at random: Dwarven Strike Force gains first strike and haste until end of turn.","4R","Creature - Dwarf Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Incurable Ogre","5","1","R",NULL,"3R","Creature - Ogre Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Indrik Stomphowler","4","4","G","When Indrik Stomphowler enters the battlefield, destroy target artifact or enchantment.","4G","Creature - Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Bronze Horse","4","4","Trample
As long as you control another creature, prevent all damage that would be dealt to Bronze Horse by spells that target it.","7","Artifact Creature - Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Krovikan Fetish","B","Enchant creature
When Krovikan Fetish enters the battlefield, draw a card at the beginning of the next turn's upkeep.
Enchanted creature gets +1/+1.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Entangling Vines","G","Enchant tapped creature
Enchanted creature doesn't untap during its controller's untap step.","3G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rampaging Werewolf","6","4","At the beginning of each upkeep, if a player cast two or more spells last turn, transform Rampaging Werewolf.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forced Worship","W","Enchant creature
Enchanted creature can't attack.
{2}{W}: Return Forced Worship to its owner's hand.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Theft","U","Target player reveals his or her hand. You choose an instant or sorcery card from it and exile that card. You may cast that card for as long as it remains exiled. At the beginning of the next end step, if you haven't cast the card, return it to its owner's hand.","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necrotic Sliver","2","2","WB","All Slivers have ""{3}, Sacrifice this permanent: Destroy target permanent.""","1WB","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Bears","2","2","G",NULL,"1G","Creature - Bear");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Teetering Peaks","Teetering Peaks enters the battlefield tapped.
When Teetering Peaks enters the battlefield, target creature gets +2/+0 until end of turn.
{T}: Add {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Araba Mothrider","1","1","W","Flying
Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","1W","Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Venarian Gold","U","Enchant creature
When Venarian Gold enters the battlefield, tap enchanted creature and put X sleep counters on it.
Enchanted creature doesn't untap during its controller's untap step if it has a sleep counter on it.
At the beginning of the upkeep of enchanted creature's controller, remove a sleep counter from that creature.","XUU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mystic Familiar","1","2","W","Flying
Threshold - As long as seven or more cards are in your graveyard, Mystic Familiar gets +1/+1 and has protection from black.","1W","Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Forbidding Watchtower","Forbidding Watchtower enters the battlefield tapped.
{T}: Add {W} to your mana pool.
{1}{W}: Forbidding Watchtower becomes a 1/5 white Soldier creature until end of turn. It's still a land.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nim Deathmantle","Equipped creature gets +2/+2, has intimidate, and is a black Zombie.
Whenever a nontoken creature is put into your graveyard from the battlefield, you may pay {4}. If you do, return that card to the battlefield and attach Nim Deathmantle to it.
Equip {4}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feast of Worms","G","Destroy target land. If that land was legendary, its controller sacrifices another land.","3GG","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drain Power","U","Target player activates a mana ability of each land he or she controls. Then put all mana from that player's mana pool into yours.","UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treetop Sentinel","2","3","U","Flying, protection from green","2UU","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arenson's Aura","W","{W}, Sacrifice an enchantment: Destroy target enchantment.
{3}{U}{U}: Counter target enchantment spell.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Capashen Unicorn","1","2","W","{1}{W}, {T}, Sacrifice Capashen Unicorn: Destroy target artifact or enchantment.","1W","Creature - Unicorn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantatog","1","2","WU","Sacrifice an enchantment: Phantatog gets +1/+1 until end of turn.
Discard a card: Phantatog gets +1/+1 until end of turn.","1WU","Creature - Atog");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Extortion","B","Look at target player's hand and choose up to two cards from it. That player discards those cards.","3BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time Reversal","U","Each player shuffles his or her hand and graveyard into his or her library, then draws seven cards. Exile Time Reversal.","3UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tigereye Cameo","{T}: Add {G} or {W} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Myrsmith","2","1","W","Whenever you cast an artifact spell, you may pay {1}. If you do, put a 1/1 colorless Myr artifact creature token onto the battlefield.","1W","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sheltering Ancient","5","5","G","Trample
Cumulative upkeep-Put a +1/+1 counter on a creature an opponent controls. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","1G","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Princess Lucrezia","5","4","UB","{T}: Add {U} to your mana pool.","3UUB","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Piety Charm","W","Choose one - Destroy target Aura attached to a creature; or target Soldier creature gets +2/+2 until end of turn; or creatures you control gain vigilance until end of turn.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fill with Fright","B","Target player discards two cards.
Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Sphinx","4","4","U","Flying, vigilance","3UU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Capashen Knight","1","1","W","First strike
{1}{W}: Capashen Knight gets +1/+0 until end of turn.","1W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Artisans","1","1","R","{T}: Flip a coin. If you win the flip, draw a card. If you lose the flip, counter target artifact spell you control that isn't the target of an ability from another creature named Goblin Artisans.","R","Creature - Goblin Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Joint Assault","G","Target creature gets +2/+2 until end of turn. If it's paired with a creature, that creature also gets +2/+2 until end of turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Restorer","1","2","G","{T}: Untap target land.
Threshold - {T}: Untap up to three target lands. Activate this ability only if seven or more cards are in your graveyard.","2G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Skeleton","0","1","B","Kicker {3} (You may pay an additional {3} as you cast this spell.)
{B}: Regenerate Urborg Skeleton.
If Urborg Skeleton was kicked, it enters the battlefield with a +1/+1 counter on it.","B","Creature - Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gloomwidow's Feast","G","Destroy target creature with flying. If that creature was blue or black, put a 1/2 green Spider creature token with reach onto the battlefield. (It can block creatures with flying.)","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fatestitcher","1","2","U","{T}: You may tap or untap another target permanent.
Unearth {U} ({U}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","3U","Creature - Zombie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dreams of the Dead","U","{1}{U}: Return target white or black creature card from your graveyard to the battlefield. That creature gains ""Cumulative upkeep {2}."" If the creature would leave the battlefield, exile it instead of putting it anywhere else. (At the beginning of its controller's upkeep, that player puts an age counter on it, then sacrifices it unless he or she pays its upkeep cost for each age counter on it.)","3U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Assassinate","B","Destroy target tapped creature.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oath of Lieges","W","At the beginning of each player's upkeep, that player chooses target player who controls more lands than he or she does and is his or her opponent. The first player may search his or her library for a basic land card, put that card onto the battlefield, then shuffle his or her library.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veteran Brawlers","4","4","R","Veteran Brawlers can't attack if defending player controls an untapped land.
Veteran Brawlers can't block if you control an untapped land.","1R","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Air Elemental","4","4","U","Flying","3UU","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uktabi Drake","2","1","G","Flying, haste
Echo {1}{G}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","G","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Winter's Grasp","G","Destroy target land.","1GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Redeem","W","Prevent all damage that would be dealt this turn to up to two target creatures.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Illusionary Terrain","U","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
As Illusionary Terrain enters the battlefield, choose two basic land types.
Basic lands of the first chosen type are the second chosen type.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silvercoat Lion","2","2","W",NULL,"1W","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Order","G","At the beginning of each player's upkeep, Primal Order deals damage to that player equal to the number of nonbasic lands he or she controls.","2GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Remote Isle","Remote Isle enters the battlefield tapped.
{T}: Add {U} to your mana pool.
Cycling {2} ({2}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kor Chant","W","All damage that would be dealt this turn to target creature you control by a source of your choice is dealt to another target creature instead.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Behemoth Sledge","WG","Equipped creature gets +2/+2 and has lifelink and trample.
Equip {3}","1GW","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nezumi Graverobber","2","1","B","{1}{B}: Exile target card from an opponent's graveyard. If no cards are in that graveyard, flip Nezumi Graverobber.
---
{4}{B}: Put target creature card from a graveyard onto the battlefield under your control.","1B","Creature - Rat Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stern Proctor","1","2","U","When Stern Proctor enters the battlefield, return target artifact or enchantment to its owner's hand.","UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Platinum Angel","4","4","Flying
You can't lose the game and your opponents can't win the game.","7","Artifact Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning Cloak","R","Target creature gets +2/+0 until end of turn. Burning Cloak deals 2 damage to that creature.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thornscape Apprentice","1","1","G","{R}, {T}: Target creature gains first strike until end of turn.
{W}, {T}: Tap target creature.","G","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Prototype Portal","Imprint - When Prototype Portal enters the battlefield, you may exile an artifact card from your hand.
{X}, {T}: Put a token that's a copy of the exiled card onto the battlefield. X is the converted mana cost of that card.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spark Elemental","3","1","R","Trample, haste (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker. This creature can attack and {T} as soon as it comes under your control.)
At the beginning of the end step, sacrifice Spark Elemental.","R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thornscape Battlemage","2","2","G","Kicker {R} and/or {W} (You may pay an additional {R} and/or {W} as you cast this spell.)
When Thornscape Battlemage enters the battlefield, if it was kicked with its {R} kicker, it deals 2 damage to target creature or player.
When Thornscape Battlemage enters the battlefield, if it was kicked with its {W} kicker, destroy target artifact.","2G","Creature - Elf Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sylvan Scrying","G","Search your library for a land card, reveal it, and put it into your hand. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Avenger","3","3","W","You can't cast Serra Avenger during your first, second, or third turns of the game.
Flying, vigilance","WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vexing Beetle","3","3","G","Vexing Beetle can't be countered.
Vexing Beetle gets +3/+3 as long as no opponent controls a creature.","4G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kathari Bomber","2","2","BR","Flying
When Kathari Bomber deals combat damage to a player, put two 1/1 red Goblin creature tokens onto the battlefield and sacrifice Kathari Bomber.
Unearth {3}{B}{R} ({3}{B}{R}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","1BR","Creature - Bird Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Morbid Hunger","B","Morbid Hunger deals 3 damage to target creature or player. You gain 3 life.
Flashback {7}{B}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","4BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hada Freeblade","0","1","W","Whenever Hada Freeblade or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Hada Freeblade.","W","Creature - Human Soldier Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("White Scarab","W","Enchant creature
Enchanted creature can't be blocked by white creatures.
Enchanted creature gets +2/+2 as long as an opponent controls a white permanent.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shifting Sliver","2","2","U","Slivers can't be blocked except by Slivers.","3U","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cunning Sparkmage","0","1","R","Haste
{T}: Cunning Sparkmage deals 1 damage to target creature or player.","2R","Creature - Human Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Obelisk of Alara","{1}{W}, {T}: You gain 5 life.
{1}{U}, {T}: Draw a card, then discard a card.
{1}{B}, {T}: Target creature gets -2/-2 until end of turn.
{1}{R}, {T}: Obelisk of Alara deals 3 damage to target player.
{1}{G}, {T}: Target creature gets +4/+4 until end of turn.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Witch Hunter","1","1","W","{T}: Witch Hunter deals 1 damage to target player.
{1}{W}{W}, {T}: Return target creature an opponent controls to its owner's hand.","2WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("El-Hajjâj","1","1","B","Whenever El-Hajjâj deals damage, you gain that much life.","1BB","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reap","G","Return up to X target cards from your graveyard to your hand, where X is the number of black permanents target opponent controls as you cast Reap.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oyobi, Who Split the Heavens","3","6","W","Flying
Whenever you cast a Spirit or Arcane spell, put a 3/3 white Spirit creature token with flying onto the battlefield.","6W","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Battering Ram","1","1","At the beginning of combat on your turn, Battering Ram gains banding until end of combat. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's blocking.)
Whenever Battering Ram becomes blocked by a Wall, destroy that Wall at end of combat.","2","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Brute","Darksteel Brute is indestructible. (""Destroy"" effects and lethal damage don't destroy it.)
{3}: Darksteel Brute becomes a 2/2 Beast artifact creature until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jhessian Infiltrator","2","2","UG","Jhessian Infiltrator is unblockable.","GU","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Saproling Cluster","G","{1}, Discard a card: Put a 1/1 green Saproling creature token onto the battlefield. Any player may activate this ability.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nameless One","*","*","U","Nameless One's power and toughness are each equal to the number of Wizards on the battlefield.
Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3U","Creature - Wizard Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tracker's Instincts","G","Reveal the top four cards of your library. Put a creature card from among them into your hand and the rest into your graveyard.
Flashback {2}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crusher Zendikon","R","Enchant land
Enchanted land is a 4/2 red Beast creature with trample. It's still a land.
When enchanted land dies, return that card to its owner's hand.","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thornling","4","4","G","{G}: Thornling gains haste until end of turn.
{G}: Thornling gains trample until end of turn.
{G}: Thornling is indestructible this turn.
{1}: Thornling gets +1/-1 until end of turn.
{1}: Thornling gets -1/+1 until end of turn.","3GG","Creature - Elemental Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thelon's Chant","G","At the beginning of your upkeep, sacrifice Thelon's Chant unless you pay {G}.
Whenever a player puts a Swamp onto the battlefield, Thelon's Chant deals 3 damage to that player unless he or she puts a -1/-1 counter on a creature he or she controls.","1GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vision Charm","U","Choose one - Target player puts the top four cards of his or her library into his or her graveyard; or choose a land type and a basic land type, and each land of the first chosen type becomes the second chosen type until end of turn; or target artifact phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before its controller untaps during his or her next untap step.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cho-Arrim Bruiser","3","4","W","Whenever Cho-Arrim Bruiser attacks, you may tap up to two target creatures.","5W","Creature - Ogre Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cabal Shrine","B","Whenever a player casts a spell, that player discards X cards, where X is the number of cards in all graveyards with the same name as that spell.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Haunted Cadaver","2","2","B","Whenever Haunted Cadaver deals combat damage to a player, you may sacrifice it. If you do, that player discards three cards.
Morph {1}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Field of Souls","W","Whenever a nontoken creature is put into your graveyard from the battlefield, put a 1/1 white Spirit creature token with flying onto the battlefield.","2WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dragon Arch","{2}, {T}: You may put a multicolored creature card from your hand onto the battlefield.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Amoeboid Changeling","1","1","U","Changeling (This card is every creature type at all times.)
{T}: Target creature gains all creature types until end of turn.
{T}: Target creature loses all creature types until end of turn.","1U","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of the Tended Garden","4","4","G","At the beginning of your upkeep, sacrifice Kami of the Tended Garden unless you pay {G}.
Soulshift 3 (When this creature dies, you may return target Spirit card with converted mana cost 3 or less from your graveyard to your hand.)","3G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unified Will","U","Counter target spell if you control more creatures than that spell's controller.","1U","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Workhorse","0","0","Workhorse enters the battlefield with four +1/+1 counters on it.
Remove a +1/+1 counter from Workhorse: Add {1} to your mana pool.","6","Artifact Creature - Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Melancholy","B","Enchant creature
When Melancholy enters the battlefield, tap enchanted creature.
Enchanted creature doesn't untap during its controller's untap step.
At the beginning of your upkeep, sacrifice Melancholy unless you pay {B}.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Torrent of Lava","R","Torrent of Lava deals X damage to each creature without flying.
As long as Torrent of Lava is on the stack, each creature has ""{T}: Prevent the next 1 damage that would be dealt to this creature by Torrent of Lava this turn.""","XRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dreadwing","1","1","B","{1}{U}{R}: Dreadwing gets +3/+0 and gains flying until end of turn.","B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duergar Assailant","1","1","WR","Sacrifice Duergar Assailant: Duergar Assailant deals 1 damage to target attacking or blocking creature.","(R/W)","Creature - Dwarf Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sighted-Caste Sorcerer","1","1","W","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
{U}: Sighted-Caste Sorcerer gains shroud until end of turn. (It can't be the target of spells or abilities.)","1W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reflect Damage","WR","The next time a source of your choice would deal damage this turn, that damage is dealt to that source's controller instead.","3RW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Triumph of the Hordes","G","Until end of turn, creatures you control get +1/+1 and gain trample and infect. (Creatures with infect deal damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","2GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ruthless Invasion","R","({(r/p)} can be paid with either {R} or 2 life.)
Nonartifact creatures can't block this turn.","3(R/P)","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Badlands",NULL,NULL,"Land - Swamp Mountain");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shattered Perception","R","Discard all the cards in your hand, then draw that many cards.
Flashback {5}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for Pure Flame","R","Whenever a source you control deals damage to an opponent, you may put a quest counter on Quest for Pure Flame.
Remove four quest counters from Quest for Pure Flame and sacrifice it: If any source you control would deal damage to a creature or player this turn, it deals double that damage to that creature or player instead.","R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shaman's Trance","R","Until end of turn, other players can't play cards from their graveyards, and you may play cards from other players' graveyards as though they were in your graveyard.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Man of Measure","2","2","W","As long as you're shorter than an opponent, Man of Measure has first strike and gets +0/+1.
As long as you're taller than an opponent, Man of Measure gets +1/+0.","1WW","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Serra's Sanctum","{T}: Add {W} to your mana pool for each enchantment you control.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rotlung Reanimator","2","2","B","Whenever Rotlung Reanimator or another Cleric dies, put a 2/2 black Zombie creature token onto the battlefield.","2B","Creature - Zombie Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Grappler","3","1","R","{G}: Viashino Grappler gains trample until end of turn.","2R","Creature - Viashino");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Alloy Myr","2","2","{T}: Add one mana of any color to your mana pool.","3","Artifact Creature - Myr");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scale of Chiss-Goria","Flash
Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
{T}: Target creature gets +0/+1 until end of turn.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Murmuring Bosk","({T}: Add {G} to your mana pool.)
As Murmuring Bosk enters the battlefield, you may reveal a Treefolk card from your hand. If you don't, Murmuring Bosk enters the battlefield tapped.
{T}: Add {W} or {B} to your mana pool. Murmuring Bosk deals 1 damage to you.",NULL,"Land - Forest");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firefright Mage","1","1","R","{1}{R}, {T}, Discard a card: Target creature can't be blocked this turn except by artifact creatures and/or red creatures.","R","Creature - Goblin Spellshaper");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rakdos Carnarium","Rakdos Carnarium enters the battlefield tapped.
When Rakdos Carnarium enters the battlefield, return a land you control to its owner's hand.
{T}: Add {B}{R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jhoira of the Ghitu Avatar","{3}, Discard a card: Copy three instant cards chosen at random. You may cast one of the copies without paying its mana cost.
{3}, Discard a card: Copy three sorcery cards chosen at random. You may cast one of the copies without paying its mana cost. Activate this ability only any time you could cast a sorcery.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trapmaker's Snare","U","Search your library for a Trap card, reveal it, and put it into your hand. Then shuffle your library.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Inquisitor","1","1","B","Threshold - {1}{B}, {T}, Exile two cards from your graveyard: Target player discards a card. Activate this ability only any time you could cast a sorcery, and only if seven or more cards are in your graveyard.","1B","Creature - Human Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Outfitter","2","2","W","When Kor Outfitter enters the battlefield, you may attach target Equipment you control to target creature you control.","WW","Creature - Kor Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hearthcage Giant","5","5","R","When Hearthcage Giant enters the battlefield, put two 3/1 red Elemental Shaman creature tokens onto the battlefield.
Sacrifice an Elemental: Target Giant creature gets +3/+1 until end of turn.","6RR","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stream of Life","G","Target player gains X life.","XG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Revered Dead","1","1","W","{W}: Regenerate Revered Dead.","1W","Creature - Spirit Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arid Mesa","{T}, Pay 1 life, Sacrifice Arid Mesa: Search your library for a Mountain or Plains card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Damnation","B","Destroy all creatures. They can't be regenerated.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gruesome Discovery","B","Target player discards two cards.
Morbid - If a creature died this turn, instead that player reveals his or her hand, you choose two cards from it, then that player discards those cards.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Vassal","2","2","B","Sacrifice Blood Vassal: Add {B}{B} to your mana pool.","2B","Creature - Thrull");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mesmeric Sliver","2","2","U","All Slivers have ""When this permanent enters the battlefield, you may fateseal 1."" (To fateseal 1, its controller looks at the top card of an opponent's library, then he or she may put that card on the bottom of that library.)","3U","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crypt Champion","2","2","B","Double strike
When Crypt Champion enters the battlefield, each player puts a creature card with converted mana cost 3 or less from his or her graveyard onto the battlefield.
When Crypt Champion enters the battlefield, sacrifice it unless {R} was spent to cast it.","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breeding Pit","B","At the beginning of your upkeep, sacrifice Breeding Pit unless you pay {B}{B}.
At the beginning of your end step, put a 0/1 black Thrull creature token onto the battlefield.","3B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Farrel's Zealot","2","2","W","Whenever Farrel's Zealot attacks and isn't blocked, you may have it deal 3 damage to target creature. If you do, Farrel's Zealot assigns no combat damage this turn.","1WW","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Waterspout Djinn","4","4","U","Flying
At the beginning of your upkeep, sacrifice Waterspout Djinn unless you return an untapped Island you control to its owner's hand.","2UU","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rorix Bladewing","6","5","R","Flying, haste","3RRR","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hero of Oxid Ridge","4","2","R","Haste
Battle cry (Whenever this creature attacks, each other attacking creature gets +1/+0 until end of turn.)
Whenever Hero of Oxid Ridge attacks, creatures with power 1 or less can't block this turn.","2RR","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dawnfluke","0","3","W","Flash
When Dawnfluke enters the battlefield, prevent the next 3 damage that would be dealt to target creature or player this turn.
Evoke {W} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","3W","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Civic Guildmage","1","1","W","{G}, {T}: Target creature gets +0/+1 until end of turn.
{U}, {T}: Put target creature you control on top of its owner's library.","W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Political Trickery","U","Exchange control of target land you control and target land an opponent controls. (This effect lasts indefinitely.)","2U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Foriysian Totem","{T}: Add {R} to your mana pool.
{4}{R}: Foriysian Totem becomes a 4/4 red Giant artifact creature with trample until end of turn.
As long as Foriysian Totem is a creature, it can block an additional creature.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uktabi Efreet","5","4","G","Cumulative upkeep {G} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","2GG","Creature - Efreet");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Liliana's Specter","2","1","B","Flying
When Liliana's Specter enters the battlefield, each opponent discards a card.","1BB","Creature - Specter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dwarven Hold","Dwarven Hold enters the battlefield tapped.
You may choose not to untap Dwarven Hold during your untap step.
At the beginning of your upkeep, if Dwarven Hold is tapped, put a storage counter on it.
{T}, Remove any number of storage counters from Dwarven Hold: Add {R} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guerrilla Tactics","R","Guerrilla Tactics deals 2 damage to target creature or player.
When a spell or ability an opponent controls causes you to discard Guerrilla Tactics, Guerrilla Tactics deals 4 damage to target creature or player.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pouncing Wurm","3","3","G","Kicker {2}{G} (You may pay an additional {2}{G} as you cast this spell.)
If Pouncing Wurm was kicked, it enters the battlefield with three +1/+1 counters on it and with haste.","3G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Ringleader","2","2","R","Haste
When Goblin Ringleader enters the battlefield, reveal the top four cards of your library. Put all Goblin cards revealed this way into your hand and the rest on the bottom of your library in any order.","3R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Pig","3","3","B","Swampwalk","3B","Creature - Boar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmagorian","6","6","B","When you cast Phantasmagorian, any player may discard three cards. If a player does, counter Phantasmagorian.
Discard three cards: Return Phantasmagorian from your graveyard to your hand.","5BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Garza's Assassin","2","2","B","Sacrifice Garza's Assassin: Destroy target nonblack creature.
Recover-Pay half your life, rounded up. (When another creature is put into your graveyard from the battlefield, you may pay half your life, rounded up. If you do, return this card from your graveyard to your hand. Otherwise, exile this card.)","BBB","Creature - Human Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Smugglers","2","2","B","Swampwalk","1BB","Creature - Human Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prowling Nightstalker","2","2","B","Prowling Nightstalker can't be blocked except by black creatures.","3B","Creature - Nightstalker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riddlekeeper","1","4","U","Whenever a creature attacks you or a planeswalker you control, that creature's controller puts the top two cards of his or her library into his or her graveyard.","2U","Creature - Homunculus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Remedy","W","Prevent the next 5 damage that would be dealt this turn to any number of target creatures and/or players, divided as you choose.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veiled Serpent","U","When an opponent casts a spell, if Veiled Serpent is an enchantment, Veiled Serpent becomes a 4/4 Serpent creature that can't attack unless defending player controls an Island.
Cycling {2} ({2}, Discard this card: Draw a card.)","2U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Warmonger's Chariot","Equipped creature gets +2/+2.
As long as equipped creature has defender, it can attack as though it didn't have defender.
Equip {3}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shaper Guildmage","1","1","U","{W}, {T}: Target creature gains first strike until end of turn.
{B}, {T}: Target creature gets +1/+0 until end of turn.","U","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lightning Coils","Whenever a nontoken creature you control dies, put a charge counter on Lightning Coils.
At the beginning of your upkeep, if Lightning Coils has five or more charge counters on it, remove all of them from it and put that many 3/1 red Elemental creature tokens with haste onto the battlefield. Exile them at the beginning of the next end step.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vitality Charm","G","Choose one - Put a 1/1 green Insect creature token onto the battlefield; or target creature gets +1/+1 and gains trample until end of turn; or regenerate target Beast.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Forethought Amulet","At the beginning of your upkeep, sacrifice Forethought Amulet unless you pay {3}.
If an instant or sorcery source would deal 3 or more damage to you, it deals 2 damage to you instead.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Elephant","3","3","G","Trample","3G","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Two-Headed Dragon","4","4","R","Flying
{1}{R}: Two-Headed Dragon gets +2/+0 until end of turn.
Two-Headed Dragon can't be blocked except by two or more creatures.
Two-Headed Dragon can block an additional creature.","4RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plaxcaster Frogling","0","0","UG","Graft 3 (This creature enters the battlefield with three +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{2}: Target creature with a +1/+1 counter on it gains shroud until end of turn. (It can't be the target of spells or abilities.)","1GU","Creature - Frog Mutant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Hybrid","0","0","Haste
Modular 2 (This enters the battlefield with two +1/+1 counters on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","4","Artifact Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Razor Boomerang","Equipped creature has ""{T}, Unattach Razor Boomerang: Razor Boomerang deals 1 damage to target creature or player. Return Razor Boomerang to its owner's hand.""
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knollspine Dragon","7","5","R","Flying
When Knollspine Dragon enters the battlefield, you may discard your hand and draw cards equal to the damage dealt to target opponent this turn.","5RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Defiler of Souls","5","5","BR","Flying
At the beginning of each player's upkeep, that player sacrifices a monocolored creature.","3BBR","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steadfastness","W","Creatures you control get +0/+3 until end of turn.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravelgill Duo","2","1","UB","Whenever you cast a blue spell, Gravelgill Duo gets +1/+1 until end of turn.
Whenever you cast a black spell, Gravelgill Duo gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","2(U/B)","Creature - Merfolk Rogue Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cardboard Carapace","G","For each other Cardboard Carapace card you have with you, enchanted creature gets +1/+1.
Errata: This does not count any Cardboard Carapace cards in play that you control or in your graveyard, hand, or library.","5G","Enchant Creature");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Manaforce Mace","Domain - Equipped creature gets +1/+1 for each basic land type among lands you control.
Equip {3}","4","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Footbottom Feast","B","Put any number of target creature cards from your graveyard on top of your library.
Draw a card.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warning","W","Prevent all combat damage that would be dealt by target attacking creature this turn.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coffin Purge","B","Exile target card from a graveyard.
Flashback {B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desperate Charge","B","Creatures you control get +2/+0 until end of turn.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Archivist","1","1","U","{T}: Draw a card.","2UU","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crown of Vigor","G","Enchant creature
Enchanted creature gets +1/+1.
Sacrifice Crown of Vigor: Enchanted creature and other creatures that share a creature type with it get +1/+1 until end of turn.","1G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Snowman","1","1","R","Whenever Goblin Snowman blocks, prevent all combat damage that would be dealt to and dealt by it this turn.
{T}: Goblin Snowman deals 1 damage to target creature it's blocking.","3R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dolmen Gate","Prevent all combat damage that would be dealt to attacking creatures you control.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mawcor","3","3","U","Flying
{T}: Mawcor deals 1 damage to target creature or player.","3UU","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fungal Reaches","{T}: Add {1} to your mana pool.
{1}, {T}: Put a storage counter on Fungal Reaches.
{1}, Remove X storage counters from Fungal Reaches: Add X mana in any combination of {R} and/or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("General Jarkeld","1","2","W","{T}: Switch the blocking creatures of two target attacking creatures. Activate this ability only during the declare blockers step.","3W","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wei Scout","1","1","B","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","1B","Creature - Human Soldier Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spatial Binding","UB","Pay 1 life: Until your next upkeep, target permanent can't phase out.","UB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jace's Archivist","2","2","U","{U}, {T}: Each player discards his or her hand, then draws cards equal to the greatest number of cards a player discarded this way.","1UU","Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ur-Drago","4","4","UB","First strike
Creatures with swampwalk can be blocked as though they didn't have swampwalk.","3UUBB","Legendary Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shadowmage Infiltrator","1","3","UB","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Whenever Shadowmage Infiltrator deals combat damage to a player, you may draw a card.","1UB","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oaken Brawler","2","4","W","When Oaken Brawler enters the battlefield, clash with an opponent. If you win, put a +1/+1 counter on Oaken Brawler. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","3W","Creature - Treefolk Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charging Paladin","2","2","W","Whenever Charging Paladin attacks, it gets +0/+3 until end of turn.","2W","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mindcrank","Whenever an opponent loses life, that player puts that many cards from the top of his or her library into his or her graveyard. (Damage dealt by sources without infect causes loss of life.)","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hellion Eruption","R","Sacrifice all creatures you control, then put that many 4/4 red Hellion creature tokens onto the battlefield.","5R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sneak Attack","R","{R}: You may put a creature card from your hand onto the battlefield. That creature gains haste. Sacrifice the creature at the beginning of the next end step.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hoverguard Observer","3","3","U","Flying
Hoverguard Observer can block only creatures with flying.","2UU","Creature - Drone");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Final Strike","B","As an additional cost to cast Final Strike, sacrifice a creature.
Final Strike deals damage to target opponent equal to the sacrificed creature's power.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enigma Eidolon","2","2","U","{U}, Sacrifice Enigma Eidolon: Target player puts the top three cards of his or her library into his or her graveyard.
Whenever you cast a multicolored spell, you may return Enigma Eidolon from your graveyard to your hand.","3U","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mana Vault","Mana Vault doesn't untap during your untap step.
At the beginning of your upkeep, you may pay {4}. If you do, untap Mana Vault.
At the beginning of your draw step, if Mana Vault is tapped, it deals 1 damage to you.
{T}: Add {3} to your mana pool.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Icequake","B","Destroy target land. If that land was a snow land, Icequake deals 1 damage to that land's controller.","1BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spellbinder","Imprint - When Spellbinder enters the battlefield, you may exile an instant card from your hand.
Whenever equipped creature deals combat damage to a player, you may copy the exiled card. If you do, you may cast the copy without paying its mana cost.
Equip {4}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slith Bloodletter","1","1","B","Whenever Slith Bloodletter deals combat damage to a player, put a +1/+1 counter on it.
{1}{B}: Regenerate Slith Bloodletter.","BB","Creature - Slith");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Infiltration Lens","Whenever equipped creature becomes blocked by a creature, you may draw two cards.
Equip {1}","1","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shadowblood Egg","{2}, {T}, Sacrifice Shadowblood Egg: Add {B}{R} to your mana pool. Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Needle Specter","1","1","B","Flying
Wither (This deals damage to creatures in the form of -1/-1 counters.)
Whenever Needle Specter deals combat damage to a player, that player discards that many cards.","1BB","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harbinger of Night","2","3","B","At the beginning of your upkeep, put a -1/-1 counter on each creature.","2BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dosan the Falling Leaf","2","2","G","Players can cast spells only during their own turns.","1GG","Legendary Creature - Human Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelic Favor","W","If you control a Plains, you may tap an untapped creature you control rather than pay Angelic Favor's mana cost.
Cast Angelic Favor only during combat.
Put a 4/4 white Angel creature token with flying onto the battlefield. Exile it at the beginning of the next end step.","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serum Visions","U","Draw a card.
Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","U","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Gargoyle Sentinel","3","3","Defender (This creature can't attack.)
{3}: Until end of turn, Gargoyle Sentinel loses defender and gains flying.","3","Artifact Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("One-Eyed Scarecrow","2","3","Defender
Creatures with flying your opponents control get -1/-0.","3","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ceta Sanctuary","U","At the beginning of your upkeep, if you control a red or green permanent, draw a card, then discard a card. If you control a red permanent and a green permanent, instead draw two cards, then discard a card.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiraling Duelist","3","1","R","Metalcraft - Spiraling Duelist has double strike as long as you control three or more artifacts.","2RR","Creature - Human Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Telemin Performance","U","Target opponent reveals cards from the top of his or her library until he or she reveals a creature card. That player puts all noncreature cards revealed this way into his or her graveyard, then you put the creature card onto the battlefield under your control.","3UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Letter Bomb","When Letter Bomb comes into play, sign it and shuffle it into target player's library. That player reveals each card he or she draws until Letter Bomb is drawn. When that player draws Letter Bomb, it deals 19{1/2} damage to him or her.","6","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Runechanter's Pike","Equipped creature has first strike and gets +X/+0, where X is the number of instant and sorcery cards in your graveyard.
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zealous Strike","W","Target creature gets +2/+2 and gains first strike until end of turn.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tangle Kelp","U","Enchant creature
When Tangle Kelp enters the battlefield, tap enchanted creature.
Enchanted creature doesn't untap during its controller's untap step if it attacked during its controller's last turn.","U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heed the Mists","U","Put the top card of your library into your graveyard, then draw cards equal to that card's converted mana cost.","3UU","Sorcery - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hillcomber Giant","3","3","W","Mountainwalk","2WW","Creature - Giant Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Incubator","As Urza's Incubator enters the battlefield, choose a creature type.
Creature spells of the chosen type cost {2} less to cast.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Betrayal","U","Enchant creature an opponent controls
Whenever enchanted creature becomes tapped, you draw a card.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tangle Spider","3","4","G","Flash (You may cast this spell any time you could cast an instant.)
Reach (This creature can block creatures with flying.)","4GG","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jötun Owl Keeper","3","3","W","Cumulative upkeep {W} or {U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When Jötun Owl Keeper dies, put a 1/1 white Bird creature token with flying onto the battlefield for each age counter on it.","2W","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fumiko the Lowblood","3","2","R","Fumiko the Lowblood has bushido X, where X is the number of attacking creatures. (When this blocks or becomes blocked, it gets +X/+X until end of turn.)
Creatures your opponents control attack each turn if able.","2RR","Legendary Creature - Human Samurai");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Whalebone Glider","{2}, {T}: Target creature with power 3 or less gains flying until end of turn.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Syphon Mind","B","Each other player discards a card. You draw a card for each card discarded this way.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Ascendant","1","1","W","Lifelink (Damage dealt by this creature also causes you to gain that much life.)
As long as you have 30 or more life, Serra Ascendant gets +5/+5 and has flying.","W","Creature - Human Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Corpses","0","2","B","Defender (This creature can't attack.)
{B}, Sacrifice Wall of Corpses: Destroy target creature Wall of Corpses is blocking.","1B","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Water Elemental","5","4","U",NULL,"3UU","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Commander's Authority","W","Enchant creature
Enchanted creature has ""At the beginning of your upkeep, put a 1/1 white Human creature token onto the battlefield.""","4W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yawgmoth's Edict","B","Whenever an opponent casts a white spell, that player loses 1 life and you gain 1 life.","1B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hibernation's End","G","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Whenever you pay Hibernation's End's cumulative upkeep, you may search your library for a creature card with converted mana cost equal to the number of age counters on Hibernation's End and put it onto the battlefield. If you do, shuffle your library.","4G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathknell Kami","0","1","B","Flying
{2}: Deathknell Kami gets +1/+1 until end of turn. Sacrifice it at the beginning of the next end step.
Soulshift 1 (When this creature dies, you may return target Spirit card with converted mana cost 1 or less from your graveyard to your hand.)","1B","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thran Quarry","At the beginning of the end step, if you control no creatures, sacrifice Thran Quarry.
{T}: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("All Hallow's Eve","B","Exile All Hallow's Eve with two scream counters on it.
At the beginning of your upkeep, if All Hallow's Eve is exiled with a scream counter on it, remove a scream counter from it. If there are no more scream counters on it, put it into your graveyard and each player returns all creature cards from his or her graveyard to the battlefield.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frenzied Tilling","RG","Destroy target land. Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.","3RG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scroll Thief","1","3","U","Whenever Scroll Thief deals combat damage to a player, draw a card.","2U","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drop of Honey","G","At the beginning of your upkeep, destroy the creature with the least power. It can't be regenerated. If two or more creatures are tied for least power, you choose one of them.
When there are no creatures on the battlefield, sacrifice Drop of Honey.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Transcendence","W","You don't lose the game for having 0 or less life.
When you have 20 or more life, you lose the game.
Whenever you lose life, you gain 2 life for each 1 life you lost. (Damage dealt to you causes you to lose life.)","3WWW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stromgald Crusader","2","1","B","Protection from white
{B}: Stromgald Crusader gains flying until end of turn.
{B}{B}: Stromgald Crusader gets +1/+0 until end of turn.","BB","Creature - Zombie Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Worship","W","If you control a creature, damage that would reduce your life total to less than 1 reduces it to 1 instead.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyhunter Cub","2","2","W","As long as Skyhunter Cub is equipped, it gets +1/+1 and has flying.","2W","Creature - Cat Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ivory Gargoyle","2","2","W","Flying
When Ivory Gargoyle dies, return it to the battlefield under its owner's control at the beginning of the next end step and you skip your next draw step.
{4}{W}: Exile Ivory Gargoyle.","4W","Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grave Scrabbler","2","2","B","Madness {1}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)
When Grave Scrabbler enters the battlefield, if its madness cost was paid, you may return target creature card from a graveyard to its owner's hand.","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cleansing Meditation","W","Destroy all enchantments.
Threshold - If seven or more cards are in your graveyard, instead destroy all enchantments, then return all cards in your graveyard destroyed this way to the battlefield.","1WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warstorm Surge","R","Whenever a creature enters the battlefield under your control, it deals damage equal to its power to target creature or player.","5R","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Training Drone","4","4","Training Drone can't attack or block unless it's equipped.","3","Artifact Creature - Drone");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Hammer","R","Volcanic Hammer deals 3 damage to target creature or player.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kiss of Death","B","Kiss of Death deals 4 damage to target opponent. You gain 4 life.","4BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glissa's Scorn","G","Destroy target artifact. Its controller loses 1 life.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Two-Headed Sliver","1","1","R","All Sliver creatures have ""This creature can't be blocked except by two or more creatures.""","1R","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crypt Ripper","2","2","B","Haste
{B}: Crypt Ripper gets +1/+1 until end of turn.","2BB","Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marauding Knight","2","2","B","Protection from white
Marauding Knight gets +1/+1 for each Plains your opponents control.","2BB","Creature - Zombie Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thermal Blast","R","Thermal Blast deals 3 damage to target creature.
Threshold - Thermal Blast deals 5 damage to that creature instead if seven or more cards are in your graveyard.","4R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Gatecrasher","3","3","R","When Ogre Gatecrasher enters the battlefield, destroy target creature with defender.","3R","Creature - Ogre Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shisato, Whispering Hunter","2","2","G","At the beginning of your upkeep, sacrifice a Snake.
Whenever Shisato, Whispering Hunter deals combat damage to a player, that player skips his or her next untap step.","3G","Legendary Creature - Snake Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fireslinger","1","1","R","{T}: Fireslinger deals 1 damage to target creature or player and 1 damage to you.","1R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Security Detail","W","{W}{W}: Put a 1/1 white Soldier creature token onto the battlefield. Activate this ability only if you control no creatures and only once each turn.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kodama of the South Tree","4","4","G","Whenever you cast a Spirit or Arcane spell, each other creature you control gets +1/+1 and gains trample until end of turn.","2GG","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daru Sanctifier","1","4","W","Morph {1}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Daru Sanctifier is turned face up, destroy target enchantment.","3W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spined Sliver","2","2","RG","Whenever a Sliver becomes blocked, that Sliver gets +1/+1 until end of turn for each creature blocking it.","RG","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sigiled Paladin","2","2","W","First strike
Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mtenda Griffin","2","2","W","Flying
{W}, {T}: Return Mtenda Griffin to its owner's hand and return target Griffin card from your graveyard to your hand. Activate this ability only during your upkeep.","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heartbeat of Spring","G","Whenever a player taps a land for mana, that player adds one mana to his or her mana pool of any type that land produced.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arrogant Wurm","4","4","G","Trample
Madness {2}{G} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","3GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Chandra Nalaar","R","+1: Chandra Nalaar deals 1 damage to target player.
-X: Chandra Nalaar deals X damage to target creature.
-8: Chandra Nalaar deals 10 damage to target player and each creature he or she controls.","Planeswalker - Chandra","3RR");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Moon","R","Nonbasic lands are Mountains.","2R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pristine Talisman","{T}: Add {1} to your mana pool. You gain 1 life.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crash","R","You may sacrifice a Mountain rather than pay Crash's mana cost.
Destroy target artifact.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Suppression Field","W","Activated abilities cost {2} more to activate unless they're mana abilities.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostfire","R","Ghostfire is colorless.
Ghostfire deals 3 damage to target creature or player.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scythe Specter","4","4","B","Flying
Whenever Scythe Specter deals combat damage to a player, each opponent discards a card. Each player who discarded a card with the highest converted mana cost among cards discarded this way loses life equal to that converted mana cost.","4BB","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yuan Shao, the Indecisive","2","3","R","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Each creature you control can't be blocked by more than one creature.","4R","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enduring Renewal","W","Play with your hand revealed.
If you would draw a card, reveal the top card of your library instead. If it's a creature card, put it into your graveyard. Otherwise, draw a card.
Whenever a creature is put into your graveyard from the battlefield, return it to your hand.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vigil for the Lost","W","Whenever a creature you control dies, you may pay {X}. If you do, you gain X life.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marsh Threader","2","1","W","Swampwalk","1W","Creature - Kor Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinder Giant","5","3","R","At the beginning of your upkeep, Cinder Giant deals 2 damage to each other creature you control.","3R","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reclamation","WG","Black creatures can't attack unless their controller sacrifices a land for each black creature he or she controls that's attacking.","2GW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grotag Siege-Runner","2","1","R","{R}, Sacrifice Grotag Siege-Runner: Destroy target creature with defender. Grotag Siege-Runner deals 2 damage to that creature's controller.","1R","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Elemental Shaman","3","1",NULL,NULL,"Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enslaved Horror","4","4","B","When Enslaved Horror enters the battlefield, each other player may return a creature card from his or her graveyard to the battlefield.","3B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hallowed Healer","1","1","W","{T}: Prevent the next 2 damage that would be dealt to target creature or player this turn.
Threshold - {T}: Prevent the next 4 damage that would be dealt to target creature or player this turn. Activate this ability only if seven or more cards are in your graveyard.","2W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit Mantle","W","Enchant creature
Enchanted creature gets +1/+1 and has protection from creatures. (It can't be blocked, targeted, or dealt damage by creatures.)","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Death-Mask Duplicant","5","5","Imprint - {1}: Exile target creature card from your graveyard.
As long as a card exiled with Death-Mask Duplicant has flying, Death-Mask Duplicant has flying. The same is true for fear, first strike, double strike, haste, landwalk, protection, and trample.","7","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planar Despair","B","Domain - All creatures get -1/-1 until end of turn for each basic land type among lands you control.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx of the Steel Wind","6","6","WUB","Flying, first strike, vigilance, lifelink, protection from red and from green","5WUB","Artifact Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Colossus","9","9","G","Morph {6}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","6GGG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fight to the Death","WR","Destroy all blocking creatures and all blocked creatures.","RW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spectral Rider","2","2","W","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)","WW","Creature - Spirit Knight");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lodestone Myr","2","2","Trample
Tap an untapped artifact you control: Lodestone Myr gets +1/+1 until end of turn.","4","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Dreadnought","12","12","Trample
When Phyrexian Dreadnought enters the battlefield, sacrifice it unless you sacrifice any number of creatures with total power 12 or greater.","1","Artifact Creature - Dreadnought");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodland Changeling","2","2","G","Changeling (This card is every creature type at all times.)","1G","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sacred Mesa","W","At the beginning of your upkeep, sacrifice Sacred Mesa unless you sacrifice a Pegasus.
{1}{W}: Put a 1/1 white Pegasus creature token with flying onto the battlefield.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lat-Nam's Legacy","U","Shuffle a card from your hand into your library. If you do, draw two cards at the beginning of the next turn's upkeep.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vanish into Memory","WU","Exile target creature. You draw cards equal to that creature's power. At the beginning of your next upkeep, return that card to the battlefield under its owner's control. If you do, discard cards equal to its toughness.","2WU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Daybreak Coronet","W","Enchant creature with another Aura attached to it
Enchanted creature gets +3/+3 and has first strike, vigilance, and lifelink. (Damage dealt by the creature also causes its controller to gain that much life.)","WW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Septic Rats","2","2","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Whenever Septic Rats attacks, if defending player is poisoned, it gets +1/+1 until end of turn.","1BB","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forgotten Harvest","G","At the beginning of your upkeep, you may exile a land card from your graveyard. If you do, put a +1/+1 counter on target creature.","1G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sigil of Distinction","Sigil of Distinction enters the battlefield with X charge counters on it.
Equipped creature gets +1/+1 for each charge counter on Sigil of Distinction.
Equip-Remove a charge counter from Sigil of Distinction.","X","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soratami Seer","2","3","U","Flying
{4}, Return two lands you control to their owner's hand: Discard all the cards in your hand, then draw that many cards.","4U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Prahv, Spires of Order","{T}: Add {1} to your mana pool.
{4}{W}{U}, {T}: Prevent all damage a source of your choice would deal this turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Spincrusher","0","2","Whenever Spincrusher blocks, put a +1/+1 counter on it.
Remove a +1/+1 counter from Spincrusher: Spincrusher is unblockable this turn.","2","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostform","U","Up to two target creatures are unblockable this turn.","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Archers","2","1","G","First strike","1G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karstoderm","0","0","G","Karstoderm enters the battlefield with five +1/+1 counters on it.
Whenever an artifact enters the battlefield, remove a +1/+1 counter from Karstoderm.","2GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kabuto Moth","1","2","W","Flying
{T}: Target creature gets +1/+2 until end of turn.","2W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sleight of Mind","U","Change the text of target spell or permanent by replacing all instances of one color word with another. (For example, you may change ""target black spell"" to ""target blue spell."" This effect lasts indefinitely.)","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heal the Scars","G","Regenerate target creature. You gain life equal to that creature's toughness.","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Feeder","0","0","G","Spike Feeder enters the battlefield with two +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Feeder: Put a +1/+1 counter on target creature.
Remove a +1/+1 counter from Spike Feeder: You gain 2 life.","1GG","Creature - Spike");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fever Charm","R","Choose one - Target creature gains haste until end of turn; or target creature gets +2/+0 until end of turn; or Fever Charm deals 3 damage to target Wizard creature.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Utopia Sprawl","G","Enchant Forest
As Utopia Sprawl enters the battlefield, choose a color.
Whenever enchanted Forest is tapped for mana, its controller adds one mana of the chosen color to his or her mana pool (in addition to the mana the land produces).","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gilded Drake","3","3","U","Flying
When Gilded Drake enters the battlefield, exchange control of Gilded Drake and up to one target creature an opponent controls. If you don't make an exchange, sacrifice Gilded Drake. This ability can't be countered except by spells and abilities. (This effect lasts indefinitely.)","1U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jenara, Asura of War","3","3","WUG","Flying
{1}{W}: Put a +1/+1 counter on Jenara, Asura of War.","GWU","Legendary Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plover Knights","3","3","W","Flying, first strike","3WW","Creature - Kithkin Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Conclave Phalanx","2","4","W","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
When Conclave Phalanx enters the battlefield, you gain 1 life for each creature you control.","4W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blazing Effigy","0","3","R","When Blazing Effigy dies, it deals X damage to target creature, where X is 3 plus the amount of damage dealt to Blazing Effigy this turn by other sources named Blazing Effigy.","1R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ink Dissolver","2","1","U","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Ink Dissolver, you may reveal it. If you do, each opponent puts the top three cards of his or her library into his or her graveyard.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keldon Marauders","3","3","R","Vanishing 2 (This permanent enters the battlefield with two time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
When Keldon Marauders enters the battlefield or leaves the battlefield, it deals 1 damage to target player.","1R","Creature - Human Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Island of Wak-Wak","{T}: The power of target creature with flying becomes 0 until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gustcloak Runner","1","1","W","Whenever Gustcloak Runner becomes blocked, you may untap it and remove it from combat.","W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stir the Pride","W","Choose one - Creatures you control get +2/+2 until end of turn; or until end of turn, creatures you control gain ""Whenever this creature deals damage, you gain that much life.""
Entwine {1}{W} (Choose both if you pay the entwine cost.)","4W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unburial Rites","B","Return target creature card from your graveyard to the battlefield.
Flashback {3}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","4B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consuming Bonfire","R","Choose one - Consuming Bonfire deals 4 damage to target non-Elemental creature; or Consuming Bonfire deals 7 damage to target Treefolk creature.","3RR","Tribal Sorcery - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vulshok Morningstar","Equipped creature gets +2/+2.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nurturer Initiate","1","1","G","Whenever a player casts a green spell, you may pay {1}. If you do, target creature gets +1/+1 until end of turn.","G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Tutelage","B","At the beginning of your upkeep, reveal the top card of your library and put that card into your hand. You lose life equal to its converted mana cost.","2B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Near-Death Experience","W","At the beginning of your upkeep, if you have exactly 1 life, you win the game.","2WWW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gravity Sphere","R","All creatures lose flying.","2R","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Weathered Bodyguards","2","5","W","As long as Weathered Bodyguards is untapped, all combat damage that would be dealt to you by unblocked creatures is dealt to Weathered Bodyguards instead.
Morph {3}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","5W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repopulate","G","Shuffle all creature cards from target player's graveyard into that player's library.
Cycling {2} ({2}, Discard this card: Draw a card.)","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Helm of Chatzuk","{1}, {T}: Target creature gains banding until end of turn. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding a player controls are blocking or being blocked by a creature, that player divides that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pitfall Trap","W","If exactly one creature is attacking, you may pay {W} rather than pay Pitfall Trap's mana cost.
Destroy target attacking creature without flying.","2W","Instant - Trap");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tolaria","{T}: Add {U} to your mana pool.
{T}: Target creature loses banding and all ""bands with other"" abilities until end of turn. Activate this ability only during any upkeep step.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mob Mentality","R","Enchant creature
Enchanted creature has trample.
Whenever all non-Wall creatures you control attack, enchanted creature gets +X/+0 until end of turn, where X is the number of attacking creatures.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sky-Eel School","3","3","U","Flying
When Sky-Eel School enters the battlefield, draw a card, then discard a card.","3UU","Creature - Fish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin King","2","2","R","Other Goblin creatures get +1/+1 and have mountainwalk. (They're unblockable as long as defending player controls a Mountain.)","1RR","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Clergy en-Vec","1","1","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","1W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Foster","G","Whenever a creature you control dies, you may pay {1}. If you do, reveal cards from the top of your library until you reveal a creature card. Put that card into your hand and the rest into your graveyard.","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiritual Guardian","3","4","W","When Spiritual Guardian enters the battlefield, you gain 4 life.","3WW","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sima Yi, Wei Field Marshal","*","4","B","Sima Yi, Wei Field Marshal's power is equal to the number of Swamps you control.","5B","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whirlpool Drake","2","2","U","Flying
When Whirlpool Drake enters the battlefield, shuffle the cards from your hand into your library, then draw that many cards.
When Whirlpool Drake dies, shuffle the cards from your hand into your library, then draw that many cards.","3U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Galina's Knight","2","2","WU","Protection from red","WU","Creature - Merfolk Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tendrils of Despair","B","As an additional cost to cast Tendrils of Despair, sacrifice a creature.
Target opponent discards two cards.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skywing Aven","2","1","U","Flying
Discard a card: Return Skywing Aven to its owner's hand.","2U","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunderscape Familiar","1","1","R","First strike
Black spells and green spells you cast cost {1} less to cast.","1R","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Excavator","{T}, Sacrifice a basic land: Target creature gains landwalk of each of the land types of the sacrificed land until end of turn.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moldervine Cloak","G","Enchant creature
Enchanted creature gets +3/+3.
Dredge 2 (If you would draw a card, instead you may put exactly two cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost-Lit Redeemer","1","1","W","{W}, {T}: You gain 2 life.
Channel - {1}{W}, Discard Ghost-Lit Redeemer: You gain 4 life.","W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Exhaustion","W","Enchant player
Enchanted player can't cast more than one spell each turn.","2WW","Enchantment - Aura Curse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tunneler Wurm","6","6","G","Discard a card: Regenerate Tunneler Wurm.","6GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyromania","R","{1}{R}, Discard a card at random: Pyromania deals 1 damage to target creature or player.
{1}{R}, Sacrifice Pyromania: Pyromania deals 1 damage to target creature or player.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Telepathy","U","Your opponents play with their hands revealed.","U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Awakening","G","At the beginning of each upkeep, untap all creatures and lands.","2GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Drudge Spell","B","{B}, Exile two creature cards from your graveyard: Put a 1/1 black Skeleton creature token onto the battlefield. It has ""{B}: Regenerate this creature.""
When Drudge Spell leaves the battlefield, destroy all Skeleton tokens. They can't be regenerated.","BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Swarmlord","4","4","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
At the beginning of your upkeep, put a 1/1 green Insect creature token with infect onto the battlefield for each poison counter your opponents have.","4GG","Creature - Insect Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Titan","2","2","G","Kicker {2}{G} (You may pay an additional {2}{G} as you cast this spell.)
If Kavu Titan was kicked, it enters the battlefield with three +1/+1 counters on it and with trample.","1G","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slippery Bogle","1","1","UG","Hexproof (This creature can't be the target of spells or abilities your opponents control.)","(G/U)","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Goliath","4","3","B",NULL,"4B","Creature - Zombie Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bone Harvest","B","Put any number of target creature cards from your graveyard on top of your library.
Draw a card at the beginning of the next turn's upkeep.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Untamed Might","G","Target creature gets +X/+X until end of turn.","XG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Remodel","G","If you control two or more green permanents that share an artist, you may play Remodel without paying its mana cost.
Remove target artifact from the game.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Righteous Charge","W","Creatures you control get +2/+2 until end of turn.","1WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Life // Death","B","Return target creature card from your graveyard to the battlefield. You lose life equal to its converted mana cost.
---
All lands you control become 1/1 creatures until end of turn. They're still lands.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Broken Fall","G","Return Broken Fall to its owner's hand: Regenerate target creature.","2G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Creeping Tar Pit","Creeping Tar Pit enters the battlefield tapped.
{T}: Add {U} or {B} to your mana pool.
{1}{U}{B}: Until end of turn, Creeping Tar Pit becomes a 3/2 blue and black Elemental creature and is unblockable. It's still a land.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Brittle Effigy","{4}, {T}, Exile Brittle Effigy: Exile target creature.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chambered Nautilus","2","2","U","Whenever Chambered Nautilus becomes blocked, you may draw a card.","2U","Creature - Nautilus Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glimpse of Nature","G","Whenever you cast a creature spell this turn, draw a card.","G","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Tin-Wing Chimera","2","2","Flying
Sacrifice Tin-Wing Chimera: Put a +2/+2 counter on target Chimera creature. It gains flying. (This effect lasts indefinitely.)","4","Artifact Creature - Chimera");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Falcon","1","1","W","Flying, vigilance","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bala Ged Scorpion","2","3","B","When Bala Ged Scorpion enters the battlefield, you may destroy target creature with power 1 or less.","3B","Creature - Scorpion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cage of Hands","W","Enchant creature
Enchanted creature can't attack or block.
{1}{W}: Return Cage of Hands to its owner's hand.","2W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elvish Champion Avatar","You begin the game with a 1/1 green Elf creature token on the battlefield. It has ""{T}: Add {G} to your mana pool.""",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for the Gravelord","B","Whenever a creature dies, you may put a quest counter on Quest for the Gravelord.
Remove three quest counters from Quest for the Gravelord and sacrifice it: Put a 5/5 black Zombie Giant creature token onto the battlefield.","B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Guide","2","2","R","Haste
Whenever Goblin Guide attacks, defending player reveals the top card of his or her library. If it's a land card, that player puts it into his or her hand.","R","Creature - Goblin Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invulnerability","W","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
The next time a source of your choice would deal damage to you this turn, prevent that damage.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raise the Alarm","W","Put two 1/1 white Soldier creature tokens onto the battlefield.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Missionary","1","1","W","{1}{W}, {T}: Prevent all combat damage that would be dealt by target blocked creature this turn.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire-Belly Changeling","1","1","R","Changeling (This card is every creature type at all times.)
{R}: Fire-Belly Changeling gets +1/+0 until end of turn. Activate this ability no more than twice each turn.","1R","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nettle Swine","4","3","G",NULL,"3G","Creature - Boar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Callow Jushi","2","2","U","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Callow Jushi.
At the beginning of the end step, if there are two or more ki counters on Callow Jushi, you may flip it.
----
Jaraku the Interloper
Legendary Creature - Spirit
3/4
Remove a ki counter from Jaraku the Interloper: Counter target spell unless its controller pays {2}.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Melira, Sylvok Outcast","2","2","G","You can't get poison counters.
Creatures you control can't have -1/-1 counters placed on them.
Creatures your opponents control lose infect.","1G","Legendary Creature - Human Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lord of Tresserhorn","10","4","UBR","When Lord of Tresserhorn enters the battlefield, you lose 2 life, you sacrifice two creatures, and target opponent draws two cards.
{B}: Regenerate Lord of Tresserhorn.","1UBR","Legendary Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ferocious Charge","G","Target creature gets +4/+4 until end of turn.
Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quag Sickness","B","Enchant creature
Enchanted creature gets -1/-1 for each Swamp you control.","2B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Squirrel token card",NULL,NULL,NULL);
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voracious Hatchling","6","6","WB","Lifelink
Voracious Hatchling enters the battlefield with four -1/-1 counters on it.
Whenever you cast a white spell, remove a -1/-1 counter from Voracious Hatchling.
Whenever you cast a black spell, remove a -1/-1 counter from Voracious Hatchling.","3(W/B)","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plague Witch","1","1","B","{B}, {T}, Discard a card: Target creature gets -1/-1 until end of turn.","1B","Creature - Elf Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodgift Demon","5","4","B","Flying
At the beginning of your upkeep, target player draws a card and loses 1 life.","3BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tribute to the Wild","G","Each opponent sacrifices an artifact or enchantment.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Granite Grip","R","Enchant creature
Enchanted creature gets +1/+0 for each Mountain you control.","2R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rebirth","G","Remove Rebirth from your deck before playing if you're not playing for ante.
Each player may put the top card of his or her library into the ante. If a player does, his or her life total becomes 20.","3GGG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spiritual Asylum","W","Creatures and lands you control have shroud. (They can't be the targets of spells or abilities.)
When a creature you control attacks, sacrifice Spiritual Asylum.","2WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Telim'Tor's Darts","{2}, {T}: Telim'Tor's Darts deals 1 damage to target player.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Monster","6","6","U","Sea Monster can't attack unless defending player controls an Island.","4UU","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Partisan","3","4","W","Battle cry (Whenever this creature attacks, each other attacking creature gets +1/+0 until end of turn.)","4W","Creature - Elephant Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Despair","5","5","WB","Flying
When Angel of Despair enters the battlefield, destroy target permanent.","3WWBB","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Second Sunrise","W","Each player returns to the battlefield all artifact, creature, enchantment, and land cards in his or her graveyard that were put there from the battlefield this turn.","1WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hex","B","Destroy six target creatures.","4BB","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Spawnsire of Ulamog","7","11","Annihilator 1 (Whenever this creature attacks, defending player sacrifices a permanent.)
{4}: Put two 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool.""
{2}0}: Cast any number of Eldrazi cards you own from outside the game without paying their mana costs.","10","Creature - Eldrazi");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Paragon of the Amesha","2","2","W","First strike
{W}{U}{B}{R}{G}: Until end of turn, Paragon of the Amesha becomes an Angel, gets +3/+3, and gains flying and lifelink.","2W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decimate","RG","Destroy target artifact, target creature, target enchantment, and target land.","2RG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ertai, Wizard Adept","1","1","U","{2}{U}{U}, {T}: Counter target spell.","2U","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sphere of Duty","W","If a green source would deal damage to you, prevent 2 of that damage.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shoe Tree","3","3","G","Shoe Tree comes into play with up to two shoe counters on it. Use your shoes as counters.
Shoe Tree gets +1/+1 for each shoe counter on it.","4G","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battering Sliver","4","4","R","All Sliver creatures have trample.","5R","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Springing Tiger","3","3","G","Threshold - Springing Tiger gets +2/+2 as long as seven or more cards are in your graveyard.","3G","Creature - Cat");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Beast","0","4","Clockwork Beast enters the battlefield with seven +1/+0 counters on it.
At end of combat, if Clockwork Beast attacked or blocked this combat, remove a +1/+0 counter from it.
{X}, {T}: Put up to X +1/+0 counters on Clockwork Beast. This ability can't cause the total number of +1/+0 counters on Clockwork Beast to be greater than seven. Activate this ability only during your upkeep.","6","Artifact Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stalking Stones","{T}: Add {1} to your mana pool.
{6}: Stalking Stones becomes a 3/3 Elemental artifact creature that's still a land. (This effect lasts indefinitely.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Hierarch","4","4","WG","When Loxodon Hierarch enters the battlefield, you gain 4 life.
{G}{W}, Sacrifice Loxodon Hierarch: Regenerate each creature you control.","2GW","Creature - Elephant Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Descent into Madness","B","At the beginning of your upkeep, put a despair counter on Descent into Madness, then each player exiles X permanents he or she controls and/or cards from his or her hand, where X is the number of despair counters on Descent into Madness.","3BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Chains","WU","Enchant creature
At the beginning of each upkeep, tap enchanted creature.","1(W/U)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gallows Warden","3","3","W","Flying
Other Spirit creatures you control get +0/+1.","4W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Souls","0","4","B","Defender (This creature can't attack.)
Whenever Wall of Souls is dealt combat damage, it deals that much damage to target opponent.","1B","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gruul Nodorog","4","4","G","{R}: Gruul Nodorog can't be blocked this turn except by two or more creatures.","4GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mongrel Pack","4","1","G","When Mongrel Pack dies during combat, put four 1/1 green Hound creature tokens onto the battlefield.","3G","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Rat","1","1","B","Swampwalk","B","Creature - Rat");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Insectile Aberration","3","2","Flying",NULL,"Creature - Human Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream Halls","U","Rather than pay the mana cost for a spell, its controller may discard a card that shares a color with that spell.","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pyroclast Consul","3","3","R","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Pyroclast Consul, you may reveal it. If you do, Pyroclast Consul deals 2 damage to each creature.","3RR","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodhall Ooze","1","1","R","At the beginning of your upkeep, if you control a black permanent, you may put a +1/+1 counter on Bloodhall Ooze.
At the beginning of your upkeep, if you control a green permanent, you may put a +1/+1 counter on Bloodhall Ooze.","R","Creature - Ooze");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Crusher","4","4","R","{R}: Flowstone Crusher gets +1/-1 until end of turn.","3RR","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Branded Brawlers","2","2","R","Branded Brawlers can't attack if defending player controls an untapped land.
Branded Brawlers can't block if you control an untapped land.","R","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Order // Chaos","R","Creatures can't block this turn.
---
Exile target attacking creature.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Great Defender","W","Target creature gets +0/+X until end of turn, where X is its converted mana cost.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thalakos Seer","1","1","U","Shadow (This creature can block or be blocked by only creatures with shadow.)
When Thalakos Seer leaves the battlefield, draw a card.","UU","Creature - Thalakos Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wing Storm","G","Wing Storm deals damage to each player equal to twice the number of creatures with flying that player controls.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Rooster","2","1","G","Plainswalk","1G","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desperate Gambit","R","Choose a source you control and flip a coin. If you win the flip, the next time that source would deal damage this turn, it deals double that damage instead. If you lose the flip, the next time it would deal damage this turn, prevent that damage.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Empty-Shrine Kannushi","1","1","W","Empty-Shrine Kannushi has protection from the colors of permanents you control.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kukemssa Pirates","2","2","U","Whenever Kukemssa Pirates attacks and isn't blocked, you may gain control of target artifact defending player controls. If you do, Kukemssa Pirates assigns no combat damage this turn.","3U","Creature - Human Pirate");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alabaster Mage","2","1","W","{1}{W}: Target creature you control gains lifelink until end of turn. (Damage dealt by the creature also causes its controller to gain that much life.)","1W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dega Disciple","1","1","W","{B}, {T}: Target creature gets -2/-0 until end of turn.
{R}, {T}: Target creature gets +2/+0 until end of turn.","W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gruul Guildmage","2","2","RG","({(r/g)} can be paid with either {R} or {G}.)
{3}{R}, Sacrifice a land: Gruul Guildmage deals 2 damage to target player.
{3}{G}: Target creature gets +2/+2 until end of turn.","(R/G)(R/G)","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hateflayer","5","5","R","Wither (This deals damage to creatures in the form of -1/-1 counters.)
{2}{R}, {Q}: Hateflayer deals damage equal to its power to target creature or player. ({Q} is the untap symbol.)","5RR","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Noxious Revival","G","({(g/p)} can be paid with either {G} or 2 life.)
Put target card from a graveyard on top of its owner's library.","(G/P)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cathedral Membrane","0","3","W","({(w/p)} can be paid with either {W} or 2 life.)
Defender
When Cathedral Membrane dies during combat, it deals 6 damage to each creature it blocked this combat.","1(W/P)","Artifact Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keldon Berserker","2","3","R","Whenever Keldon Berserker attacks, if you control no untapped lands, it gets +3/+0 until end of turn.","3R","Creature - Human Soldier Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emberwilde Caliph","4","4","UR","Flying, trample
Emberwilde Caliph attacks each turn if able.
Whenever Emberwilde Caliph deals damage, you lose that much life.","2UR","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steel of the Godhead","WU","Enchant creature
As long as enchanted creature is white, it gets +1/+1 and has lifelink. (Damage dealt by the creature also causes its controller to gain that much life.)
As long as enchanted creature is blue, it gets +1/+1 and is unblockable.","2(W/U)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primal Whisperer","2","2","G","Primal Whisperer gets +2/+2 for each face-down creature on the battlefield.
Morph {3}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4G","Creature - Elf Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orc General","2","2","R","{T}, Sacrifice another Orc or Goblin: Other Orc creatures get +1/+1 until end of turn.","2R","Creature - Orc Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magical Hack","U","Change the text of target spell or permanent by replacing all instances of one basic land type with another. (For example, you may change ""swampwalk"" to ""plainswalk."" This effect lasts indefinitely.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battle Rampart","1","3","R","Defender
{T}: Target creature gains haste until end of turn.","2R","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Reaper King Avatar","Each creature you control gets +1/+1 for each of its colors.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragon Breath","R","Enchant creature
Enchanted creature has haste.
{R}: Enchanted creature gets +1/+0 until end of turn.
When a creature with converted mana cost 6 or greater enters the battlefield, you may return Dragon Breath from your graveyard to the battlefield attached to that creature.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spring Cleaning","G","Destroy target enchantment. Clash with an opponent. If you win, destroy all enchantments your opponents control. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Purging Scythe","At the beginning of your upkeep, Purging Scythe deals 2 damage to the creature with the least toughness. If two or more creatures are tied for least toughness, you choose one of them.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhalfirin Knight","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{W}{W}: Zhalfirin Knight gains first strike until end of turn.","2W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Dross Ripper","3","3","{2}{B}: Dross Ripper gets +1/+1 until end of turn.","4","Artifact Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Retraced Image","U","Reveal a card in your hand, then put that card onto the battlefield if it has the same name as a permanent.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corpulent Corpse","3","3","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Suspend 5-{B} (Rather than cast this card from your hand, you may pay {B} and exile it with five time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","5B","Creature - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Power Plant","{T}: Add {1} to your mana pool. If you control an Urza's Mine and an Urza's Tower, add {2} to your mana pool instead.",NULL,"Land - Urza’s Power-Plant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arboria","G","Creatures can't attack a player unless that player cast a spell or put a nontoken permanent onto the battlefield during his or her last turn.","2GG","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lammastide Weave","G","Name a card, then target player puts the top card of his or her library into his or her graveyard. If that card is the named card, you gain life equal to its converted mana cost.
Draw a card.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dead Weight","B","Enchant creature
Enchanted creature gets -2/-2.","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nether Void","B","Whenever a player casts a spell, counter it unless its controller pays {3}.","3B","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nekrataal","2","1","B","First strike (This creature deals combat damage before creatures without first strike.)
When Nekrataal enters the battlefield, destroy target nonartifact, nonblack creature. That creature can't be regenerated.","2BB","Creature - Human Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enslaved Dwarf","1","1","R","{R}, Sacrifice Enslaved Dwarf: Target black creature gets +1/+0 and gains first strike until end of turn.","R","Creature - Dwarf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindleech Mass","6","6","UB","Trample
Whenever Mindleech Mass deals combat damage to a player, you may look at that player's hand. If you do, you may cast a nonland card in it without paying that card's mana cost.","5UBB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quickchange","U","Target creature becomes the color or colors of your choice until end of turn.
Draw a card.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Multani","Creatures you control get +X/+0, where X is the number of cards in your hand.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Geargrabber","4","4","R","Whenever Ogre Geargrabber attacks, gain control of target Equipment an opponent controls until end of turn. Attach it to Ogre Geargrabber. When you lose control of that Equipment, unattach it.","4RR","Creature - Ogre Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Allied Strategies","U","Domain - Target player draws a card for each basic land type among lands he or she controls.","4U","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wicker Warcrawler","6","6","Whenever Wicker Warcrawler attacks or blocks, put a -1/-1 counter on it at end of combat.","5","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lair Delve","G","Reveal the top two cards of your library. Put all creature and land cards revealed this way into your hand and the rest on the bottom of your library in any order.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Perilous Myr","1","1","When Perilous Myr dies, it deals 2 damage to target creature or player.","2","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cleanse","W","Destroy all black creatures.","2WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Formation","W","Target creature gains banding until end of turn. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding a player controls are blocking or being blocked by a creature, that player divides that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
Draw a card at the beginning of the next turn's upkeep.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Karmic Justice","W","Whenever a spell or ability an opponent controls destroys a noncreature permanent you control, you may destroy target permanent that opponent controls.","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Basalt Monolith","Basalt Monolith doesn't untap during your untap step.
{T}: Add {3} to your mana pool.
{3}: Untap Basalt Monolith.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Prowler","3","3","B","Fading 3 (This creature enters the battlefield with three fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Remove a fade counter from Phyrexian Prowler: Phyrexian Prowler gets +1/+1 until end of turn.","3B","Creature - Zombie Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skullmulcher","3","3","G","Devour 1 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with that many +1/+1 counters on it.)
When Skullmulcher enters the battlefield, draw a card for each creature it devoured.","4G","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Equinox","W","Enchant land
Enchanted land has ""{T}: Counter target spell if it would destroy a land you control.""","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viscerid Drone","1","2","U","{T}, Sacrifice a creature and a Swamp: Destroy target nonartifact creature. It can't be regenerated.
{T}, Sacrifice a creature and a snow Swamp: Destroy target creature. It can't be regenerated.","1U","Creature - Homarid Drone");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Victual Sliver","2","2","WG","All Slivers have ""{2}, Sacrifice this permanent: You gain 4 life.""","GW","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Energy Chamber","At the beginning of your upkeep, choose one - Put a +1/+1 counter on target artifact creature; or put a charge counter on target noncreature artifact.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathgazer","2","2","B","Whenever Deathgazer blocks or becomes blocked by a nonblack creature, destroy that creature at end of combat.","3B","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moment's Peace","G","Prevent all combat damage that would be dealt this turn.
Flashback {2}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gangrenous Goliath","4","4","B","Tap three untapped Clerics you control: Return Gangrenous Goliath from your graveyard to your hand.","3BB","Creature - Zombie Giant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vulshok Battlegear","Equipped creature gets +3/+3.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emrakul's Hatcher","3","3","R","When Emrakul's Hatcher enters the battlefield, put three 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool.""","4R","Creature - Eldrazi Drone");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Morningtide","W","Exile all cards from all graveyards.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunspear Shikari","2","2","W","As long as Sunspear Shikari is equipped, it has first strike and lifelink.","1W","Creature - Cat Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heckling Fiends","2","2","R","{2}{R}: Target creature attacks this turn if able.","2R","Creature - Devil");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feral Contest","G","Put a +1/+1 counter on target creature you control. Another target creature blocks it this turn if able.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oversoul of Dusk","5","5","WG","Protection from blue, from black, and from red","(G/W)(G/W)(G/W)(G/W)(G/W)","Creature - Spirit Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Old Ghastbark","3","6","WG",NULL,"3(G/W)(G/W)","Creature - Treefolk Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glare of Subdual","WG","Tap an untapped creature you control: Tap target artifact or creature.","2GW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Nomad","1","1","R","{T}: Target creature with power 2 or less is unblockable this turn.","2R","Creature - Dwarf Nomad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silvergill Adept","2","1","U","As an additional cost to cast Silvergill Adept, reveal a Merfolk card from your hand or pay {3}.
When Silvergill Adept enters the battlefield, draw a card.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Meteor Storm","RG","{2}{R}{G}, Discard two cards at random: Meteor Storm deals 4 damage to target creature or player.","RG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rishadan Pawnshop","{2}, {T}: Shuffle target nontoken permanent you control into its owner's library.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shivan Harvest","R","{1}{R}, Sacrifice a creature: Destroy target nonbasic land.","1R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodhusk Ritualist","2","2","B","Multikicker {B} (You may pay an additional {B} any number of times as you cast this spell.)
When Bloodhusk Ritualist enters the battlefield, target opponent discards a card for each time it was kicked.","2B","Creature - Vampire Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Restless Bones","1","1","B","{3}{B}, {T}: Target creature gains swampwalk until end of turn.
{1}{B}: Regenerate Restless Bones.","2B","Creature - Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eaten by Spiders","G","Destroy target creature with flying and all Equipment attached to that creature.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keldon Warlord","*","*","R","Keldon Warlord's power and toughness are each equal to the number of non-Wall creatures you control.","2RR","Creature - Human Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Three Visits","G","Search your library for a Forest card and put that card onto the battlefield. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thalakos Mistfolk","2","1","U","Shadow (This creature can block or be blocked by only creatures with shadow.)
{U}: Put Thalakos Mistfolk on top of its owner's library.","2U","Creature - Thalakos Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wolverine Pack","2","4","G","Rampage 2 (Whenever this creature becomes blocked, it gets +2/+2 until end of turn for each creature blocking it beyond the first.)","2GG","Creature - Wolverine");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Churning Eddy","U","Return target creature and target land to their owners' hands.","3U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pentagram of the Ages","{4}, {T}: The next time a source of your choice would deal damage to you this turn, prevent that damage.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lurking Nightstalker","1","1","B","Whenever Lurking Nightstalker attacks, it gets +2/+0 until end of turn.","BB","Creature - Nightstalker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Remove Soul","U","Counter target creature spell.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Utopia Vow","G","Enchant creature
Enchanted creature can't attack or block.
Enchanted creature has ""{T}: Add one mana of any color to your mana pool.""","1G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Reflecting Pool","{T}: Add to your mana pool one mana of any type that a land you control could produce.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scorched Ruins","If Scorched Ruins would enter the battlefield, sacrifice two untapped lands instead. If you do, put Scorched Ruins onto the battlefield. If you don't, put it into its owner's graveyard.
{T}: Add {4} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fellwar Stone","{T}: Add to your mana pool one mana of any color that a land an opponent controls could produce.","2","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Crusher","0","0","Trample
Whenever another artifact enters the battlefield, put a +1/+1 counter on Arcbound Crusher.
Modular 1 (This enters the battlefield with a +1/+1 counter on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","4","Artifact Creature - Juggernaut");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jiwari, the Earth Aflame","3","3","R","{X}{R}, {T}: Jiwari, the Earth Aflame deals X damage to target creature without flying.
Channel - {X}{R}{R}{R}, Discard Jiwari: Jiwari deals X damage to each creature without flying.","3RR","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dive Bomber","2","2","W","Flying
{T}, Sacrifice Dive Bomber: Dive Bomber deals 2 damage to target attacking or blocking creature.","3W","Creature - Bird Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Summoning Station","{T}: Put a 2/2 colorless Pincher creature token onto the battlefield.
Whenever an artifact is put into a graveyard from the battlefield, you may untap Summoning Station.","7","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Denouncer","1","1","B","{T}, Sacrifice Phyrexian Denouncer: Target creature gets -1/-1 until end of turn.","1B","Creature - Carrier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Piper's Melody","G","Shuffle any number of target creature cards from your graveyard into your library.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shunt","R","Change the target of target spell with a single target.","1RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wirewood Hivemaster","1","1","G","Whenever another nontoken Elf enters the battlefield, you may put a 1/1 green Insect creature token onto the battlefield.","1G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jor Kadeen, the Prevailer","5","4","WR","First strike
Metalcraft - Creatures you control get +3/+0 as long as you control three or more artifacts.","3RW","Legendary Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rushwood Elemental","4","4","G","Trample
At the beginning of your upkeep, you may put a +1/+1 counter on Rushwood Elemental.","GGGGG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tendrils of Corruption","B","Tendrils of Corruption deals X damage to target creature and you gain X life, where X is the number of Swamps you control.","3B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mesmeric Fiend","1","1","B","When Mesmeric Fiend enters the battlefield, target opponent reveals his or her hand and you choose a nonland card from it. Exile that card.
When Mesmeric Fiend leaves the battlefield, return the exiled card to its owner's hand.","1B","Creature - Nightmare Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caravan Escort","1","1","W","Level up {2} ({2}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-4
2/2
LEVEL 5+
5/5
First strike","W","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Illuminated Folio","{1}, {T}, Reveal two cards from your hand that share a color: Draw a card.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noble Benefactor","2","2","U","When Noble Benefactor dies, each player may search his or her library for a card and put that card into his or her hand. Then each player who searched his or her library this way shuffles it.","2U","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ghost Town","{T}: Add {1} to your mana pool.
{0}: Return Ghost Town to its owner's hand. Activate this ability only if it's not your turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Untamed Wilds","G","Search your library for a basic land card and put that card onto the battlefield. Then shuffle your library.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abyssal Hunter","1","1","B","{B}, {T}: Tap target creature. Abyssal Hunter deals damage equal to Abyssal Hunter's power to that creature.","3B","Creature - Human Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sanctum Guardian","1","4","W","Sacrifice Sanctum Guardian: The next time a source of your choice would deal damage to target creature or player this turn, prevent that damage.","1WW","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Panoptic Mirror","Imprint - {X}, {T}: You may exile an instant or sorcery card with converted mana cost X from your hand.
At the beginning of your upkeep, you may copy a card exiled with Panoptic Mirror. If you do, you may cast the copy without paying its mana cost.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bower Passage","G","Creatures with flying can't block creatures you control.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silkbind Faerie","1","3","WU","Flying
{1}{(w/u)}, {Q}: Tap target creature. ({Q} is the untap symbol.)","2(W/U)","Creature - Faerie Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunstrike Legionnaire","1","2","W","Sunstrike Legionnaire doesn't untap during your untap step.
Whenever another creature enters the battlefield, untap Sunstrike Legionnaire.
{T}: Tap target creature with converted mana cost 3 or less.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pardic Arsonist","3","3","R","Threshold - As long as seven or more cards are in your graveyard, Pardic Arsonist has ""When Pardic Arsonist enters the battlefield, it deals 3 damage to target creature or player.""","2RR","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Spy","1","1","U","Islandwalk (This creature is unblockable as long as defending player controls an Island.)
Whenever Merfolk Spy deals combat damage to a player, that player reveals a card at random from his or her hand.","U","Creature - Merfolk Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Earth","0","6","R","Defender (This creature can't attack.)","1R","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seal of Primordium","G","Sacrifice Seal of Primordium: Destroy target artifact or enchantment.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Allay","W","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Destroy target enchantment.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashenmoor Gouger","4","4","BR","Ashenmoor Gouger can't block.","(B/R)(B/R)(B/R)","Creature - Elemental Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sick and Tired","B","Two target creatures each get -1/-1 until end of turn.","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spirit en-Kor","2","2","W","Flying
{0}: The next 1 damage that would be dealt to Spirit en-Kor this turn is dealt to target creature you control instead.","3W","Creature - Kor Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aquitect's Will","U","Put a flood counter on target land. That land is an Island in addition to its other types for as long as it has a flood counter on it. If you control a Merfolk, draw a card.","U","Tribal Sorcery - Merfolk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Barbed Sextant","{1}, {T}, Sacrifice Barbed Sextant: Add one mana of any color to your mana pool. Draw a card at the beginning of the next turn's upkeep.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blue Ward","W","Enchant creature
Enchanted creature has protection from blue. This effect doesn't remove Blue Ward.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yore-Tiller Nephilim","2","2","WUBR","Whenever Yore-Tiller Nephilim attacks, return target creature card from your graveyard to the battlefield tapped and attacking.","WUBR","Creature - Nephilim");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Denied!","U","Play Denied only as any opponent casts target spell. Name a card, then look at all cards in that player's hand. If the named card is in the player's hand, counter target spell.","U","Interrupt");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Skysweeper","1","1","G","{4}{G}, Sacrifice a creature: Destroy target creature with flying.","G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bull Cerodon","5","5","WR","Vigilance, haste","4RW","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spreading Seas","U","Enchant land
When Spreading Seas enters the battlefield, draw a card.
Enchanted land is an Island.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Betrayers","3","1","G","Viridian Betrayers has infect as long as an opponent is poisoned. (It deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","1GG","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gosta Dirk","4","4","WU","First strike
Creatures with islandwalk can be blocked as though they didn't have islandwalk.","3WWUU","Legendary Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Black Knight","2","2","B","First strike (This creature deals combat damage before creatures without first strike.)
Protection from white (This creature can't be blocked, targeted, dealt damage, or enchanted by anything white.)","BB","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hinder","U","Counter target spell. If that spell is countered this way, put that card on the top or bottom of its owner's library instead of into that player's graveyard.","1UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Concussive Bolt","R","Concussive Bolt deals 4 damage to target player.
Metalcraft - If you control three or more artifacts, creatures that player controls can't block this turn.","3RR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Frost Marsh","Frost Marsh enters the battlefield tapped.
{T}: Add {U} or {B} to your mana pool.",NULL,"Snow Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaea's Might","G","Domain - Target creature gets +1/+1 until end of turn for each basic land type among lands you control.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arbalest Elite","2","3","W","{2}{W}, {T}: Arbalest Elite deals 3 damage to target attacking or blocking creature. Arbalest Elite doesn't untap during your next untap step.","2WW","Creature - Human Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nullmage Shepherd","2","4","G","Tap four untapped creatures you control: Destroy target artifact or enchantment.","3G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mwonvuli Ooze","1+*","1+*","G","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay {2} for each age counter on it.)
Mwonvuli Ooze's power and toughness are each equal to 1 plus twice the number of age counters on it.","G","Creature - Ooze");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veldrane of Sengir","5","5","B","{1}{B}{B}: Veldrane of Sengir gets -3/-0 and gains forestwalk until end of turn.","5BB","Legendary Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hand of Cruelty","2","2","B","Protection from white
Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","BB","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thought Devourer","4","4","U","Flying
Your maximum hand size is reduced by four.","2UU","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Serum Tank","Whenever Serum Tank or another artifact enters the battlefield, put a charge counter on Serum Tank.
{3}, {T}, Remove a charge counter from Serum Tank: Draw a card.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Secluded Steppe","Secluded Steppe enters the battlefield tapped.
{T}: Add {W} to your mana pool.
Cycling {W} ({W}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turn the Tide","U","Creatures your opponents control get -2/-0 until end of turn.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guardians' Pledge","W","White creatures you control get +2/+2 until end of turn.","1WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Weatherseed Treefolk","5","3","G","Trample
When Weatherseed Treefolk dies, return it to its owner's hand.","2GGG","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Daggerdare","1","1","G","{G}, {T}: Target attacking creature gets +2/+2 until end of turn.","1G","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Honor Guard","1","1","W","{W}: Honor Guard gets +0/+1 until end of turn.","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caustic Crawler","4","3","B","Landfall - Whenever a land enters the battlefield under your control, you may have target creature get -1/-1 until end of turn.","3BB","Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Silverskin Armor","Equipped creature gets +1/+1 and is an artifact in addition to its other types.
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Torture","B","Enchant creature
{1}{B}: Put a -1/-1 counter on enchanted creature.","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reanimate","B","Put target creature card from a graveyard onto the battlefield under your control. You lose life equal to its converted mana cost.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mark of Asylum","W","Prevent all noncombat damage that would be dealt to creatures you control.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Rage","G","Creatures you control have trample. (If a creature you control would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firefly","1","1","R","Flying
{R}: Firefly gets +1/+0 until end of turn.","3R","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kazandu Blademaster","1","1","W","First strike, vigilance
Whenever Kazandu Blademaster or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Kazandu Blademaster.","WW","Creature - Human Soldier Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surging Flame","R","Ripple 4 (When you cast this spell, you may reveal the top four cards of your library. You may cast any revealed cards with the same name as this spell without paying their mana costs. Put the rest on the bottom of your library.)
Surging Flame deals 2 damage to target creature or player.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Storm","U","Creature spells can't be cast.
Pay 4 life: Destroy Æther Storm. It can't be regenerated. Any player may activate this ability.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auspicious Ancestor","2","3","W","When Auspicious Ancestor dies, you gain 3 life.
Whenever a player casts a white spell, you may pay {1}. If you do, you gain 1 life.","3W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Scatter","U","Counter target creature spell.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Treachery","U","Enchant creature
When Treachery enters the battlefield, untap up to five lands.
You control enchanted creature.","3UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pillaging Horde","5","5","R","When Pillaging Horde enters the battlefield, sacrifice it unless you discard a card at random.","2RR","Creature - Human Barbarian");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Windbrisk Heights","Hideaway (This land enters the battlefield tapped. When it does, look at the top four cards of your library, exile one face down, then put the rest on the bottom of your library.)
{T}: Add {W} to your mana pool.
{W}, {T}: You may play the exiled card without paying its mana cost if you attacked with three or more creatures this turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guardians of Akrasa","0","4","W","Defender
Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of Sanctity","W","If Leyline of Sanctity is in your opening hand, you may begin the game with it on the battlefield.
You have hexproof. (You can't be the target of spells or abilities your opponents control.)","2WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shrine of Piercing Vision","At the beginning of your upkeep or whenever you cast a blue spell, put a charge counter on Shrine of Piercing Vision.
{T}, Sacrifice Shrine of Piercing Vision: Look at the top X cards of your library, where X is the number of charge counters on Shrine of Piercing Vision. Put one of those cards into your hand and the rest on the bottom of your library in any order.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Titanic Growth","G","Target creature gets +4/+4 until end of turn.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirror Mirror","Mirror Mirror comes into play tapped.
o7, ocT, Sacrifice Mirror Mirror: At end of turn, exchange life totals with target player and exchange all cards in play that you control, and all cards in your hand, library, and graveyard, with that player until end of game.","7","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Familiar Ground","G","Each creature you control can't be blocked by more than one creature.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Howler","4","5","G","When Kavu Howler enters the battlefield, reveal the top four cards of your library. Put all Kavu cards revealed this way into your hand and the rest on the bottom of your library in any order.","4GG","Creature - Kavu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bottomless Pit","B","At the beginning of each player's upkeep, that player discards a card at random.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carnivorous Plant","4","5","G","Defender","3G","Creature - Plant Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Life Matrix","{4}, {T}: Put a matrix counter on target creature and that creature gains ""Remove a matrix counter from this creature: Regenerate this creature."" Activate this ability only during your upkeep.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duskrider Falcon","1","1","W","Flying, protection from black","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mystic Enforcer","3","3","WG","Protection from black
Threshold - As long as seven or more cards are in your graveyard, Mystic Enforcer gets +3/+3 and has flying.","2GW","Creature - Human Nomad Mystic");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Purelace","W","Target spell or permanent becomes white. (Mana symbols on that permanent remain unchanged.)","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jackalope Herd","4","5","G","When you cast a spell, return Jackalope Herd to its owner's hand.","3G","Creature - Rabbit Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stench of Decay","B","Nonartifact creatures get -1/-1 until end of turn.","1BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wojek Apothecary","1","1","W","Radiance - {T}: Prevent the next 1 damage that would be dealt to target creature and each other creature that shares a color with it this turn.","2WW","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gix","{3}: Return target creature card from your graveyard to your hand.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planeswalker's Scorn","B","{3}{B}: Target opponent reveals a card at random from his or her hand. Target creature gets -X/-X until end of turn, where X is the revealed card's converted mana cost. Activate this ability only any time you could cast a sorcery.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Crystal Golem","3","3","At the beginning of your end step, Crystal Golem phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before you untap during your next untap step.)","4","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Pieces Are Coming Together","When you set this scheme in motion, draw two cards. Artifact spells you cast this turn cost {2} less to cast.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Saproling Infestation","G","Whenever a player kicks a spell, you put a 1/1 green Saproling creature token onto the battlefield.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Champion's Victory","U","Cast Champion's Victory only during the declare attackers step and only if you've been attacked this step.
Return target attacking creature to its owner's hand.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cagemail","W","Enchant creature
Enchanted creature gets +2/+2 and can't attack.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scornful Egotist","1","1","U","Morph {U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","7U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Network","U","Each player reveals the top card of his or her library to all other players by continuously holding it against his or her forehead. This does not allow a player to look at his or her own card. (That card still counts as the top card of your library. Whenever you draw a card, draw that one and replace it with the next card of your library.)","U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrionette","1","1","B","{2}{B}{B}: Exile Carrionette and target creature unless that creature's controller pays {2}. Activate this ability only if Carrionette is in your graveyard.","1B","Creature - Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Longbow Archer","2","2","W","First strike; reach (This creature can block creatures with flying.)","WW","Creature - Human Soldier Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pallimud","*","3","R","As Pallimud enters the battlefield, choose an opponent.
Pallimud's power is equal to the number of tapped lands the chosen player controls.","2R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ray of Revelation","W","Destroy target enchantment.
Flashback {G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trade Routes","U","{1}: Return target land you control to its owner's hand.
{1}, Discard a land card: Draw a card.","1U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blinkmoth Nexus","{T}: Add {1} to your mana pool.
{1}: Blinkmoth Nexus becomes a 1/1 Blinkmoth artifact creature with flying until end of turn. It's still a land.
{1}, {T}: Target Blinkmoth creature gets +1/+1 until end of turn.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sandals of Abdallah","{2}, {T}: Target creature gains islandwalk until end of turn. When that creature dies this turn, destroy Sandals of Abdallah.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Higure, the Still Wind Avatar","Whenever a nontoken creature you control deals combat damage to an opponent, choose a creature card at random from your library, reveal that card, and put it into your hand. Then shuffle your library.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Soothsayer","1","1","R","{R}, {T}, Sacrifice a Goblin: Red creatures get +1/+1 until end of turn.","R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repay in Kind","B","Each player's life total becomes the lowest life total among all players.","5BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deglamer","G","Choose target artifact or enchantment. Its owner shuffles it into his or her library.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chainer, Dementia Master","3","3","B","Nightmare creatures get +1/+1.
{B}{B}{B}, Pay 3 life: Put target creature card from a graveyard onto the battlefield under your control. That creature is black and is a Nightmare in addition to its other creature types.
When Chainer, Dementia Master leaves the battlefield, exile all Nightmares.","3BB","Legendary Creature - Human Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infernal Kirin","3","3","B","Flying
Whenever you cast a Spirit or Arcane spell, target player reveals his or her hand and discards all cards with that spell's converted mana cost.","2BB","Legendary Creature - Kirin Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rabid Wombat","0","1","G","Vigilance
Rabid Wombat gets +2/+2 for each Aura attached to it.","2GG","Creature - Wombat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swallowing Plague","B","Swallowing Plague deals X damage to target creature and you gain X life.","XBB","Sorcery - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blinding Angel","2","4","W","Flying (This creature can't be blocked except by creatures with flying or reach.)
Whenever Blinding Angel deals combat damage to a player, that player skips his or her next combat phase.","3WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Soulgorger","8","8","Cumulative upkeep-Sacrifice a creature. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","3","Snow Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vapor Snag","U","Return target creature to its owner's hand. Its controller loses 1 life.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Souls of the Faultless","0","4","WB","Defender (This creature can't attack.)
Whenever Souls of the Faultless is dealt combat damage, you gain that much life and attacking player loses that much life.","WBB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brontotherium","5","3","G","Trample
Provoke (When this attacks, you may have target creature defending player controls untap and block it if able.)","4GG","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cloudcrest Lake","{T}: Add {1} to your mana pool.
{T}: Add {W} or {U} to your mana pool. Cloudcrest Lake doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Opal-Eye, Konda's Yojimbo","1","4","W","Defender (This creature can't attack.)
Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
{T}: The next time a source of your choice would deal damage this turn, that damage is dealt to Opal-Eye, Konda's Yojimbo instead.
{1}{W}: Prevent the next 1 damage that would be dealt to Opal-Eye this turn.","1WW","Legendary Creature - Fox Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pyric Salamander","1","1","R","{R}: Pyric Salamander gets +1/+0 until end of turn. Sacrifice Pyric Salamander at the beginning of the next end step.","1R","Creature - Salamander");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Smallpox","B","Each player loses 1 life, discards a card, sacrifices a creature, then sacrifices a land.","BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kinsbaile Cavalier","2","2","W","Knight creatures you control have double strike.","3W","Creature - Kithkin Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Paroxysm","R","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, that player reveals the top card of his or her library. If that card is a land card, destroy that creature. Otherwise, it gets +3/+3 until end of turn.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Puppet's Verdict","R","Flip a coin. If you win the flip, destroy all creatures with power 2 or less. If you lose the flip, destroy all creatures with power 3 or greater.","1RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepchannel Mentor","2","2","U","Blue creatures you control are unblockable.","5U","Creature - Merfolk Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flame Spirit","2","3","R","{R}: Flame Spirit gets +1/+0 until end of turn.","4R","Creature - Elemental Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grim Lavamancer","1","1","R","{R}, {T}, Exile two cards from your graveyard: Grim Lavamancer deals 2 damage to target creature or player.","R","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avatar of Slaughter","8","8","R","All creatures have double strike and attack each turn if able.","6RR","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ward of Piety","W","Enchant creature
{1}{W}: The next 1 damage that would be dealt to enchanted creature this turn is dealt to target creature or player instead.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Exchange","B","As an additional cost to cast Soul Exchange, exile a creature you control.
Return target creature card from your graveyard to the battlefield. Put a +2/+2 counter on that creature if the exiled creature was a Thrull.","BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Swing","R","Choose three target nonenchantment permanents. Destroy one of them at random.","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Food Chain","G","Exile a creature you control: Add X mana of any one color to your mana pool, where X is the exiled creature's converted mana cost plus one. Spend this mana only to cast creature spells.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soldevi Sage","1","1","U","{T}, Sacrifice two lands: Draw three cards, then discard one of them.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seedcradle Witch","1","1","WG","{2}{G}{W}: Target creature gets +3/+3 until end of turn. Untap that creature.","(G/W)","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slayer of the Wicked","3","2","W","When Slayer of the Wicked enters the battlefield, you may destroy target Vampire, Werewolf, or Zombie.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Admonition Angel","6","6","W","Flying
Landfall - Whenever a land enters the battlefield under your control, you may exile target nonland permanent other than Admonition Angel.
When Admonition Angel leaves the battlefield, return all cards exiled with it to the battlefield under their owners' control.","3WWW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burning Shield Askari","2","2","R","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{R}{R}: Burning Shield Askari gains first strike until end of turn.","2R","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Havengul Lich","4","4","UB","{1}: You may cast target creature card in a graveyard this turn. When you cast that card this turn, Havengul Lich gains all activated abilities of that card until end of turn.","3UB","Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Mastermind","1","2","U","{U}, {T}: Return target permanent you control to its owner's hand.","UU","Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Might Sliver","2","2","G","All Sliver creatures get +2/+2.","4G","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Ice","0","7","G","Defender (This creature can't attack.)","2G","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chaos Confetti","o4, ocT: Tear Chaos Confetti into pieces. Throw the pieces onto the playing area from a distance of at least five feet. Destroy each card in play that a piece touches. Remove the pieces from the game afterwards.","4","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Lancer","0","0","First strike
Modular 4 (This enters the battlefield with four +1/+1 counters on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","7","Artifact Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Figure of Destiny","1","1","WR","{(r/w)}: Figure of Destiny becomes a 2/2 Kithkin Spirit.
{(r/w){(r/w){(r/w)}: If Figure of Destiny is a Spirit, it becomes a 4/4 Kithkin Spirit Warrior.
{(r/w){(r/w){(r/w){(r/w){(r/w){(r/w)}: If Figure of Destiny is a Warrior, it becomes an 8/8 Kithkin Spirit Warrior Avatar with flying and first strike.","(R/W)","Creature - Kithkin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ardent Plea","WU","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)","1WU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Replenish","W","Return all enchantment cards from your graveyard to the battlefield. (Auras with nothing to enchant remain in your graveyard.)","3W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Riptide","U","Tap all blue creatures.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karplusan Giant","3","3","R","Tap an untapped snow land you control: Karplusan Giant gets +1/+1 until end of turn.","6R","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Akroma's Vengeance","W","Destroy all artifacts, creatures, and enchantments.
Cycling {3} ({3}, Discard this card: Draw a card.)","4WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Compulsion","U","{1}{U}, Discard a card: Draw a card.
{1}{U}, Sacrifice Compulsion: Draw a card.","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gnat Miser","1","1","B","Each opponent's maximum hand size is reduced by one.","B","Creature - Rat Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Molten Firebird","2","2","R","Flying
When Molten Firebird dies, return it to the battlefield under its owner's control at the beginning of the next end step and you skip your next draw step.
{4}{R}: Exile Molten Firebird.","4R","Creature - Phoenix");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marrow Bats","4","1","B","Flying
Pay 4 life: Regenerate Marrow Bats.","4B","Creature - Bat Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tobias Andrion","4","4","WU",NULL,"3WU","Legendary Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boggart Ram-Gang","3","3","RG","Haste
Wither (This deals damage to creatures in the form of -1/-1 counters.)","(R/G)(R/G)(R/G)","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Igneous Pouncer","5","1","BR","Haste
Swampcycling {2}, mountaincycling {2} ({2}, Discard this card: Search your library for a Swamp or Mountain card, reveal it, and put it into your hand. Then shuffle your library.)","4BR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Moneychanger","0","2","W","Icatian Moneychanger enters the battlefield with three credit counters on it.
When Icatian Moneychanger enters the battlefield, it deals 3 damage to you.
At the beginning of your upkeep, put a credit counter on Icatian Moneychanger.
Sacrifice Icatian Moneychanger: You gain 1 life for each credit counter on Icatian Moneychanger. Activate this ability only during your upkeep.","W","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Haunting Hymn","B","Target player discards two cards. If you cast this spell during your main phase, that player discards four cards instead.","4BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Somberwald Vigilante","1","1","R","Whenever Somberwald Vigilante becomes blocked by a creature, Somberwald Vigilante deals 1 damage to that creature.","R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lich Lord of Unx","2","2","UB","{U}{B}, {T}: Put a 1/1 blue and black Zombie Wizard creature token onto the battlefield.
{U}{U}{B}{B}: Target player loses X life and puts the top X cards of his or her library into his or her graveyard, where X is the number of Zombies you control.","1UB","Creature - Zombie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancient Craving","B","You draw three cards and you lose 3 life.","3B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Land Grant","G","If you have no land cards in hand, you may reveal your hand rather than pay Land Grant's mana cost.
Search your library for a Forest card, reveal that card, and put it into your hand. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dimir Machinations","B","Look at the top three cards of target player's library. Exile any number of those cards, then put the rest back in any order.
Transmute {1}{B}{B} ({1}{B}{B}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sylvan Hierophant","1","2","G","When Sylvan Hierophant dies, exile Sylvan Hierophant, then return another target creature card from your graveyard to your hand.","1G","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phantasmal Terrain","U","Enchant land
As Phantasmal Terrain enters the battlefield, choose a basic land type.
Enchanted land is the chosen type.","UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serpentine Basilisk","2","3","G","Whenever Serpentine Basilisk deals combat damage to a creature, destroy that creature at end of combat.
Morph {1}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2GG","Creature - Basilisk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shadow Rift","U","Target creature gains shadow until end of turn. (It can block or be blocked by only creatures with shadow.)
Draw a card.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Thaumaturgist","1","2","R","{T}: Switch target creature's power and toughness until end of turn.","2R","Creature - Dwarf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Distress","B","Target player reveals his or her hand. You choose a nonland card from it. That player discards that card.","BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Living End","Suspend 3-{2}{B}{B} (Rather than cast this card from your hand, pay {2}{B}{B} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)
Each player exiles all creature cards from his or her graveyard, then sacrifices all creatures he or she controls, then puts all cards he or she exiled this way onto the battlefield.",NULL,"Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Minions' Murmurs","B","You draw X cards and you lose X life, where X is the number of creatures you control.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Summoning Trap","G","If a creature spell you cast this turn was countered by a spell or ability an opponent controlled, you may pay {0} rather than pay Summoning Trap's mana cost.
Look at the top seven cards of your library. You may put a creature card from among them onto the battlefield. Put the rest on the bottom of your library in any order.","4GG","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ravaging Riftwurm","6","6","G","Kicker {4} (You may pay an additional {4} as you cast this spell.)
Vanishing 2 (This permanent enters the battlefield with two time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
If Ravaging Riftwurm was kicked, it enters the battlefield with three additional time counters on it.","1GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beacon of Creation","G","Put a 1/1 green Insect creature token onto the battlefield for each Forest you control. Shuffle Beacon of Creation into its owner's library.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magma Rift","R","As an additional cost to cast Magma Rift, sacrifice a land.
Magma Rift deals 5 damage to target creature.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trailblazer","G","Target creature is unblockable this turn.","2GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glittering Lynx","1","1","W","Prevent all damage that would be dealt to Glittering Lynx.
{2}: Until end of turn, Glittering Lynx loses ""Prevent all damage that would be dealt to Glittering Lynx."" Any player may activate this ability.","W","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Panther","2","2","B","{B}, Sacrifice Urborg Panther: Destroy target creature blocking Urborg Panther.
Sacrifice a creature named Feral Shadow, a creature named Breathstealer, and Urborg Panther: Search your library for a card named Spirit of the Night and put that card onto the battlefield. Then shuffle your library.","2B","Creature - Nightstalker Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghalma's Warden","2","4","W","Metalcraft - Ghalma's Warden gets +2/+2 as long as you control three or more artifacts.","3W","Creature - Elephant Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tsabo's Assassin","1","1","B","{T}: Destroy target creature if it shares a color with the most common color among all permanents or a color tied for most common. A creature destroyed this way can't be regenerated.","2BB","Creature - Zombie Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drudge Reavers","2","1","B","Flash (You may cast this spell any time you could cast an instant.)
{B}: Regenerate Drudge Reavers.","3B","Creature - Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Palladia-Mors","7","7","WRG","Flying, trample
At the beginning of your upkeep, sacrifice Palladia-Mors unless you pay {R}{G}{W}.","2RRGGWW","Legendary Creature - Elder Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chariot of the Sun","{2}, {T}: Until end of turn, target creature you control gains flying and its toughness becomes 1.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Memories","U","Put the top seven cards of your library into your graveyard. At the beginning of the next end step, exile seven cards from your graveyard.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("All Shall Smolder in My Wake","When you set this scheme in motion, destroy up to one target artifact, up to one target enchantment, and up to one target nonbasic land.",NULL,"Scheme");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bonesplitter","Equipped creature gets +2/+0.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Renewing Touch","G","Shuffle any number of target creature cards from your graveyard into your library.","G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Knowledge Vault","{2}, {T}: Exile the top card of your library face down.
{0}: Sacrifice Knowledge Vault. If you do, discard your hand, then put all cards exiled with Knowledge Vault into their owner's hand.
When Knowledge Vault leaves the battlefield, put all cards exiled with Knowledge Vault into their owner's graveyard.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Doran, the Siege Tower","0","5","WBG","Each creature assigns combat damage equal to its toughness rather than its power.","BGW","Legendary Creature - Treefolk Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shelter","W","Target creature you control gains protection from the color of your choice until end of turn.
Draw a card.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Village Ironsmith","1","1","R","First strike
At the beginning of each upkeep, if no spells were cast last turn, transform Village Ironsmith.","1R","Creature - Human Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trygon Predator","2","3","UG","Flying
Whenever Trygon Predator deals combat damage to a player, you may destroy target artifact or enchantment that player controls.","1GU","Creature - Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Reclaimer","0","0","Remove a +1/+1 counter from Arcbound Reclaimer: Put target artifact card from your graveyard on top of your library.
Modular 2 (This enters the battlefield with two +1/+1 counters on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","4","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dwarven Landslide","R","Kicker-{2}{R}, Sacrifice a land. (You may pay {2}{R} and sacrifice a land in addition to any other costs as you cast this spell.)
Destroy target land. If Dwarven Landslide was kicked, destroy another target land.","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rites of Flourishing","G","At the beginning of each player's draw step, that player draws an additional card.
Each player may play an additional land on each of his or her turns.","2G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Instill Furor","R","Enchant creature
Enchanted creature has ""At the beginning of your end step, sacrifice this creature unless it attacked this turn.""","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mighty Leap","W","Target creature gets +2/+2 and gains flying until end of turn.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Cache","R","At the beginning of each player's end step, put a charge counter on Mana Cache for each untapped land that player controls.
Remove a charge counter from Mana Cache: Add {1} to your mana pool. Any player may activate this ability but only during his or her turn before the end step.","1RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inner Sanctum","W","Cumulative upkeep-Pay 2 life. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Prevent all damage that would be dealt to creatures you control.","1WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crown of the Ages","{4}, {T}: Attach target Aura attached to a creature to another creature.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Runesword","{3}, {T}: Target attacking creature gets +2/+0 until end of turn. When that creature leaves the battlefield this turn, sacrifice Runesword. If the creature deals damage to a creature this turn, the creature dealt damage can't be regenerated this turn. If a creature dealt damage by the targeted creature would die this turn, exile that creature instead.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cursed Land","B","Enchant land
At the beginning of the upkeep of enchanted land's controller, Cursed Land deals 1 damage to that player.","2BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("One Dozen Eyes","G","Choose one - Put a 5/5 green Beast creature token onto the battlefield; or put five 1/1 green Insect creature tokens onto the battlefield.
Entwine {G}{G}{G} (Choose both if you pay the entwine cost.)","5G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sunglasses of Urza","You may spend white mana as though it were red mana.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Counsel of the Soratami","U","Draw two cards.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Swarm","0","3","Clockwork Swarm enters the battlefield with four +1/+0 counters on it.
Clockwork Swarm can't be blocked by Walls.
At end of combat, if Clockwork Swarm attacked or blocked this combat, remove a +1/+0 counter from it.
{X}, {T}: Put up to X +1/+0 counters on Clockwork Swarm. This ability can't cause the total number of +1/+0 counters on Clockwork Swarm to be greater than four. Activate this ability only during your upkeep.","4","Artifact Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Talons of Falkenrath","R","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature has ""{1}{R}: This creature gets +2/+0 until end of turn.""","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seton's Scout","2","1","G","Reach (This creature can block creatures with flying.)
Threshold - Seton's Scout gets +2/+2 as long as seven or more cards are in your graveyard.","1G","Creature - Centaur Druid Scout Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Wish","B","You may choose a card you own from outside the game and put it into your hand. You lose half your life, rounded up. Exile Death Wish.","1BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Basal Sliver","2","2","B","All Slivers have ""Sacrifice this permanent: Add {B}{B} to your mana pool.""","2B","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Antagonism","R","At the beginning of each player's end step, Antagonism deals 2 damage to that player unless one of his or her opponents was dealt damage this turn.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Linessa, Zephyr Mage","3","3","U","{X}{U}{U}, {T}: Return target creature with converted mana cost X to its owner's hand.
Grandeur - Discard another card named Linessa, Zephyr Mage: Target player returns a creature he or she controls to its owner's hand, then repeats this process for an artifact, an enchantment, and a land.","3U","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warbreak Trumpeter","1","1","R","Morph {X}{X}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Warbreak Trumpeter is turned face up, put X 1/1 red Goblin creature tokens onto the battlefield.","R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Retriever","1","1","When Myr Retriever dies, return another target artifact card from your graveyard to your hand.","2","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Servant of Volrath","3","3","B","When Servant of Volrath leaves the battlefield, sacrifice a creature.","2B","Creature - Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diabolic Servitude","B","When Diabolic Servitude enters the battlefield, return target creature card from your graveyard to the battlefield.
When the creature put onto the battlefield with Diabolic Servitude dies, exile it and return Diabolic Servitude to its owner's hand.
When Diabolic Servitude leaves the battlefield, exile the creature put onto the battlefield with Diabolic Servitude.","3B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silent-Chant Zubera","1","2","W","When Silent-Chant Zubera dies, you gain 2 life for each Zubera that died this turn.","1W","Creature - Zubera Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu Elite Infantry","3","3","W",NULL,"3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Linvala, Keeper of Silence","3","4","W","Flying
Activated abilities of creatures your opponents control can't be activated.","2WW","Legendary Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wistful Thinking","U","Target player draws two cards, then discards four cards.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin General","1","1","R","Whenever Goblin General attacks, Goblin creatures you control get +1/+1 until end of turn.","1RR","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Momentous Fall","G","As an additional cost to cast Momentous Fall, sacrifice a creature.
You draw cards equal to the sacrificed creature's power, then you gain life equal to its toughness.","2GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thelonite Druid","1","1","G","{1}{G}, {T}, Sacrifice a creature: Forests you control become 2/3 creatures until end of turn. They're still lands.","2G","Creature - Human Cleric Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Heretic","1","3","R","{1}{R}, {T}: Destroy target artifact. Viashino Heretic deals damage to that artifact's controller equal to the artifact's converted mana cost.","2R","Creature - Viashino");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Hounds","2","2","B","Discard a creature card: Vampire Hounds gets +2/+2 until end of turn.","2B","Creature - Vampire Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiketail Hatchling","1","1","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
Sacrifice Spiketail Hatchling: Counter target spell unless its controller pays {1}.","1U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Windwright Mage","2","2","WUB","Lifelink (Damage dealt by this creature also causes you to gain that much life.)
Windwright Mage has flying as long as an artifact card is in your graveyard.","WUB","Artifact Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Demigod of Revenge","5","4","BR","Flying, haste
When you cast Demigod of Revenge, return all cards named Demigod of Revenge from your graveyard to the battlefield.","(B/R)(B/R)(B/R)(B/R)(B/R)","Creature - Spirit Avatar");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sulfur Vent","Sulfur Vent enters the battlefield tapped.
{T}: Add {B} to your mana pool.
{T}, Sacrifice Sulfur Vent: Add {U}{R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Monstrous Carabid","4","4","BR","Monstrous Carabid attacks each turn if able.
Cycling {(b/r)} ({(b/r)}, Discard this card: Draw a card.)","3BR","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rite of Passage","G","Whenever a creature you control is dealt damage, put a +1/+1 counter on it. (The damage is dealt before the counter is put on.)","2G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of the Meek","Equipped creature gets +1/+2.
Equip {2}
Whenever a 1/1 creature enters the battlefield under your control, you may return Sword of the Meek from your graveyard to the battlefield, then attach it to that creature.","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dawnray Archer","1","1","U","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
{W}, {T}: Dawnray Archer deals 1 damage to target attacking or blocking creature.","2U","Creature - Human Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shock Troops","2","2","R","Sacrifice Shock Troops: Shock Troops deals 2 damage to target creature or player.","3R","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonebrow, Krosan Hero","4","4","RG","Trample
Whenever a creature you control with trample attacks, it gets +2/+2 until end of turn.","3RG","Legendary Creature - Centaur Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Malignus","*","*","R","Malignus's power and toughness are each equal to half the highest life total among your opponents, rounded up.
Damage that would be dealt by Malignus can't be prevented.","3RR","Creature - Elemental Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kaalia of the Vast","2","2","WBR","Flying
Whenever Kaalia of the Vast attacks an opponent, you may put an Angel, Demon, or Dragon creature card from your hand onto the battlefield tapped and attacking that opponent.","1WBR","Legendary Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chalice of the Void","Chalice of the Void enters the battlefield with X charge counters on it.
Whenever a player casts a spell with converted mana cost equal to the number of charge counters on Chalice of the Void, counter that spell.","XX","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frog Tongue","G","Enchant creature
When Frog Tongue enters the battlefield, draw a card.
Enchanted creature has reach. (It can block creatures with flying.)","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Intrepid Hero","1","1","W","{T}: Destroy target creature with power 4 or greater.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coastal Wizard","1","1","U","{T}: Return Coastal Wizard and another target creature to their owners' hands. Activate this ability only during your turn, before attackers are declared.","2UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skittish Valesk","5","5","R","At the beginning of your upkeep, flip a coin. If you lose the flip, turn Skittish Valesk face down.
Morph {5}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","6R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Havengul Skaab","4","5","U","Whenever Havengul Skaab attacks, return another creature you control to its owner's hand.","5U","Creature - Zombie Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Famine","B","Famine deals 3 damage to each creature and each player.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Captain Sisay","2","2","WG","{T}: Search your library for a legendary card, reveal that card, and put it into your hand. Then shuffle your library.","2GW","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pouncing Jaguar","2","2","G","Echo {G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","G","Creature - Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Terrain Generator","{T}: Add {1} to your mana pool.
{2}, {T}: You may put a basic land card from your hand onto the battlefield tapped.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rout","W","You may cast Rout any time you could cast an instant if you pay {2} more to cast it.
Destroy all creatures. They can't be regenerated.","3WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Destructive Urge","R","Enchant creature
Whenever enchanted creature deals combat damage to a player, that player sacrifices a land.","1RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ajani's Pridemate","2","2","W","Whenever you gain life, you may put a +1/+1 counter on Ajani's Pridemate. (For example, if an effect causes you to gain 3 life, you may put one +1/+1 counter on this creature.)","1W","Creature - Cat Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Autumn Willow","4","4","G","Shroud
{G}: Until end of turn, Autumn Willow can be the target of spells and abilities controlled by target player as though it didn't have shroud.","4GG","Legendary Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jagwasp Swarm","3","2","B","Flying","3B","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lord of the Undead","2","2","B","Other Zombie creatures get +1/+1.
{1}{B}, {T}: Return target Zombie card from your graveyard to your hand.","1BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghostflame Sliver","2","2","BR","All Slivers are colorless.","BR","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Toils of Night and Day","U","You may tap or untap target permanent, then you may tap or untap another target permanent.","2U","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gloomwidow","3","3","G","Reach
Gloomwidow can block only creatures with flying.","2G","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nihilistic Glee","B","{2}{B}, Discard a card: Target opponent loses 1 life and you gain 1 life.
Hellbent - {1}, Pay 2 life: Draw a card. Activate this ability only if you have no cards in hand.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dark Impostor","2","2","B","{4}{B}{B}: Exile target creature and put a +1/+1 counter on Dark Impostor.
Dark Impostor has all activated abilities of all creature cards exiled with it.","2B","Creature - Vampire Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bountiful Harvest","G","You gain 1 life for each land you control.","4G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Taste of Paradise","G","As an additional cost to cast Taste of Paradise, you may pay {1}{G} any number of times.
You gain 3 life plus an additional 3 life for each additional {1}{G} you paid.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Imp","2","1","R","When Fire Imp enters the battlefield, it deals 2 damage to target creature.","2R","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Desolation Angel","5","4","B","Kicker {W}{W} (You may pay an additional {W}{W} as you cast this spell.)
Flying
When Desolation Angel enters the battlefield, destroy all lands you control. If it was kicked, destroy all lands instead.","3BB","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rust","G","Counter target activated ability from an artifact source. (Mana abilities can't be targeted.)","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glintwing Invoker","3","3","U","{7}{U}: Glintwing Invoker gets +3/+3 and gains flying until end of turn.","4U","Creature - Human Wizard Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Ultimus","3","3","U","Mistform Ultimus is every creature type (even if this card isn't on the battlefield).","3U","Legendary Creature - Illusion");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Tribal Golem","4","4","Tribal Golem has trample as long as you control a Beast, haste as long as you control a Goblin, first strike as long as you control a Soldier, flying as long as you control a Wizard, and ""{B}: Regenerate Tribal Golem"" as long as you control a Zombie.","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bile Urchin","1","1","B","Sacrifice Bile Urchin: Target player loses 1 life.","B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Surging Sentinels","2","1","W","First strike
Ripple 4 (When you cast this spell, you may reveal the top four cards of your library. You may cast any revealed cards with the same name as this spell without paying their mana costs. Put the rest on the bottom of your library.)","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ersatz Gnomes","1","1","{T}: Target spell becomes colorless.
{T}: Target permanent becomes colorless until end of turn.","3","Artifact Creature - Gnome");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shrine of Boundless Growth","At the beginning of your upkeep or whenever you cast a green spell, put a charge counter on Shrine of Boundless Growth.
{T}, Sacrifice Shrine of Boundless Growth: Add {1} to your mana pool for each charge counter on Shrine of Boundless Growth.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planar Cleansing","W","Destroy all nonland permanents.","3WWW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guardian Zendikon","W","Enchant land
Enchanted land is a 2/6 white Wall creature with defender. It's still a land.
When enchanted land dies, return that card to its owner's hand.","2W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Breeding Pool","({T}: Add {G} or {U} to your mana pool.)
As Breeding Pool enters the battlefield, you may pay 2 life. If you don't, Breeding Pool enters the battlefield tapped.",NULL,"Land - Forest Island");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Muzzle","W","Enchant creature
Prevent all damage that would be dealt by enchanted creature.","1W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Knowledge Pool","Imprint - When Knowledge Pool enters the battlefield, each player exiles the top three cards of his or her library.
Whenever a player casts a spell from his or her hand, that player exiles it. If the player does, he or she may cast another nonland card exiled with Knowledge Pool without paying that card's mana cost.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Gate Oracle","1","3","U","When Sea Gate Oracle enters the battlefield, look at the top two cards of your library. Put one of them into your hand and the other on the bottom of your library.","2U","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Obelisk of Grixis","{T}: Add {U}, {B}, or {R} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinder Seer","1","1","R","{2}{R}, {T}: Reveal any number of red cards in your hand. Cinder Seer deals X damage to target creature or player, where X is the number of cards revealed this way.","3R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gift of the Deity","BG","Enchant creature
As long as enchanted creature is black, it gets +1/+1 and has deathtouch. (Any amount of damage it deals to a creature is enough to destroy that creature.)
As long as enchanted creature is green, it gets +1/+1 and all creatures able to block it do so.","4(B/G)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guiltfeeder","0","4","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Whenever Guiltfeeder attacks and isn't blocked, defending player loses 1 life for each card in his or her graveyard.","3BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clutch of the Undercity","UB","Return target permanent to its owner's hand. Its controller loses 3 life.
Transmute {1}{U}{B} ({1}{U}{B}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","1UUB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Captain","1","1","R","{1}: Flip a coin. If you win the flip, target Orc creature gets +2/+0 until end of turn. If you lose the flip, it gets -0/-2 until end of turn.","R","Creature - Orc Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Harvest","B","Target player loses X life and you gain X life, where X is the greatest power among creatures you control.","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Endoskeleton","You may choose not to untap Endoskeleton during your untap step.
{2}, {T}: Target creature gets +0/+3 for as long as Endoskeleton remains tapped.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Femeref Enchantress","1","2","WG","Whenever an enchantment is put into a graveyard from the battlefield, draw a card.","GW","Creature - Human Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mask of Intolerance","At the beginning of each player's upkeep, if there are four or more basic land types among lands that player controls, Mask of Intolerance deals 3 damage to him or her.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancestral Recall","U","Target player draws three cards.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Dawn","W","Until end of turn, spells and abilities you control that would add colored mana to your mana pool add that much white mana instead. Until end of turn, you may spend white mana as though it were mana of any color.
Draw a card.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Engulfing Slagwurm","7","7","G","Whenever Engulfing Slagwurm blocks or becomes blocked by a creature, destroy that creature. You gain life equal to that creature's toughness.","5GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tormentor Exarch","2","2","R","When Tormentor Exarch enters the battlefield, choose one - Target creature gets +2/+0 until end of turn; or target creature gets -0/-2 until end of turn.","3R","Creature - Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Maniac","1","1","R","Whenever Mogg Maniac is dealt damage, it deals that much damage to target opponent.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowering Lumberknot","5","5","G","Flowering Lumberknot can't attack or block unless it's paired with a creature with soulbond.","3G","Creature - Treefolk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lonely Sandbar","Lonely Sandbar enters the battlefield tapped.
{T}: Add {U} to your mana pool.
Cycling {U} ({U}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moonwing Moth","2","1","W","Flying
{W}: Moonwing Moth gets +0/+1 until end of turn.","1WW","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Panic Attack","R","Up to three target creatures can't block this turn.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelic Chorus","W","Whenever a creature enters the battlefield under your control, you gain life equal to its toughness.","3WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wingcrafter","1","1","U","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Wingcrafter is paired with another creature, both creatures have flying.","U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sudden Disappearance","W","Exile all nonland permanents target player controls. Return the exiled cards to the battlefield under their owner's control at the beginning of the next end step.","5W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Visionary","1","1","G","When Elvish Visionary enters the battlefield, draw a card.","1G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Battle Strain","R","Whenever a creature blocks, Battle Strain deals 1 damage to that creature's controller.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Head Games","B","Target opponent puts the cards from his or her hand on top of his or her library. Search that player's library for that many cards. The player puts those cards into his or her hand, then shuffles his or her library.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Miner","1","2","R","{2}{R}, {T}: Destroy target nonbasic land.","1R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bend or Break","R","Each player separates all nontoken lands he or she controls into two piles. For each player, one of his or her piles is chosen by one of his or her opponents of his or her choice. Destroy all lands in the chosen piles. Tap all lands in the other piles.","3R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cinder Marsh","{T}: Add {1} to your mana pool.
{T}: Add {B} or {R} to your mana pool. Cinder Marsh doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Banshee's Blade","Equipped creature gets +1/+1 for each charge counter on Banshee's Blade.
Whenever equipped creature deals combat damage, put a charge counter on Banshee's Blade.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scrambleverse","R","For each nonland permanent, choose a player at random. Then each player gains control of each permanent for which he or she was chosen. Untap those permanents.","6RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Earthshaker","4","5","R","Whenever you cast a Spirit or Arcane spell, Earthshaker deals 2 damage to each creature without flying.","4RR","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Waiting in the Weeds","G","Each player puts a 1/1 green Cat creature token onto the battlefield for each untapped Forest he or she controls.","1GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viral Drake","1","4","U","Flying
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
{3}{U}: Proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","3U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sell-Sword Brute","2","2","R","When Sell-Sword Brute dies, it deals 2 damage to you.","1R","Creature - Human Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shadow Lance","W","Enchant creature
Enchanted creature has first strike.
{1}{B}: Enchanted creature gets +2/+2 until end of turn.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Echoing Truth","U","Return target nonland permanent and all other permanents with the same name as that permanent to their owners' hands.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cho-Arrim Legate","1","2","W","Protection from black
If an opponent controls a Swamp and you control a Plains, you may cast Cho-Arrim Legate without paying its mana cost.","2W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Door to Nothingness","Door to Nothingness enters the battlefield tapped.
{W}{W}{U}{U}{B}{B}{R}{R}{G}{G}, {T}, Sacrifice Door to Nothingness: Target player loses the game.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quicksilver Behemoth","4","5","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
When Quicksilver Behemoth attacks or blocks, return it to its owner's hand at end of combat. (Return it only if it's on the battlefield.)","6U","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Territorial Dispute","R","At the beginning of your upkeep, sacrifice Territorial Dispute unless you sacrifice a land.
Players can't play lands.","4RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aggression","R","Enchant non-Wall creature
Enchanted creature has first strike and trample.
At the beginning of the end step of enchanted creature's controller, destroy that creature if it didn't attack this turn.","2R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rack and Ruin","R","Destroy two target artifacts.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reroute","R","Change the target of target activated ability with a single target. (Mana abilities can't be targeted.)
Draw a card.","1R","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Dross Golem","3","2","Affinity for Swamps (This spell costs {1} less to cast for each Swamp you control.)
Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grazing Gladehart","2","2","G","Landfall - Whenever a land enters the battlefield under your control, you may gain 2 life.","2G","Creature - Antelope");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pemmin's Aura","U","Enchant creature
{U}: Untap enchanted creature.
{U}: Enchanted creature gains flying until end of turn.
{U}: Enchanted creature gains shroud until end of turn. (It can't be the target of spells or abilities.)
{1}: Enchanted creature gets +1/-1 or -1/+1 until end of turn.","1UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inkfathom Witch","1","1","UB","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
{2}{U}{B}: Each unblocked creature becomes 4/1 until end of turn.","1(U/B)","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screaming Seahawk","2","2","U","Flying
When Screaming Seahawk enters the battlefield, you may search your library for a card named Screaming Seahawk, reveal it, and put it into your hand. If you do, shuffle your library.","4U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gutless Ghoul","2","2","B","{1}, Sacrifice a creature: You gain 2 life.","2B","Snow Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infernal Denizen","5","7","B","At the beginning of your upkeep, sacrifice two Swamps. If you can't, tap Infernal Denizen, and an opponent may gain control of a creature you control of his or her choice for as long as Infernal Denizen remains on the battlefield.
{T}: Gain control of target creature for as long as Infernal Denizen remains on the battlefield.","7B","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ramosian Lieutenant","1","2","W","{4}, {T}: Search your library for a Rebel permanent card with converted mana cost 3 or less and put it onto the battlefield. Then shuffle your library.","1W","Creature - Human Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Greener Pastures","G","At the beginning of each player's upkeep, if that player controls more lands than each other player, the player puts a 1/1 green Saproling creature token onto the battlefield.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frenetic Ogre","2","3","R","{R}, Discard a card at random: Frenetic Ogre gets +3/+0 until end of turn.","4R","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magma Sliver","3","3","R","All Slivers have ""{T}: Target Sliver creature gets +X/+0 until end of turn, where X is the number of Slivers on the battlefield.""","3R","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nimble Mongoose","1","1","G","Shroud (This permanent can't be the target of spells or abilities.)
Threshold - Nimble Mongoose gets +2/+2 as long as seven or more cards are in your graveyard.","G","Creature - Mongoose");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Weird Harvest","G","Each player may search his or her library for up to X creature cards, reveal those cards, and put them into his or her hand. Then each player who searched his or her library this way shuffles it.","XGG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jolrael, Empress of Beasts","3","3","G","{2}{G}, {T}, Discard two cards: All lands target player controls become 3/3 creatures until end of turn. They're still lands.","3GG","Legendary Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Caress of Phyrexia","B","Target player draws three cards, loses 3 life, and gets three poison counters.","3BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fractured Loyalty","R","Enchant creature
Whenever enchanted creature becomes the target of a spell or ability, that spell or ability's controller gains control of that creature. (This effect lasts indefinitely.)","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overabundance","RG","Whenever a player taps a land for mana, that player adds one mana to his or her mana pool of any type that land produced, and Overabundance deals 1 damage to him or her.","1RG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Darkness","B","Prevent all combat damage that would be dealt this turn.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Cannibal","1","1","B","Whenever Zombie Cannibal deals combat damage to a player, you may exile target card from that player's graveyard.","B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mannichi, the Fevered Dream","1","2","R","{1}{R}: Switch each creature's power and toughness until end of turn.","2R","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sliver Queen","7","7","WUBRG","{2}: Put a 1/1 colorless Sliver creature token onto the battlefield.","WUBRG","Legendary Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crazed Skirge","2","2","B","Flying, haste","3B","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shoving Match","U","Until end of turn, all creatures gain ""{T}: Tap target creature.""","2U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bloodletter Quill","{2}, {T}, Put a blood counter on Bloodletter Quill: Draw a card, then lose 1 life for each blood counter on Bloodletter Quill.
{U}{B}: Remove a blood counter from Bloodletter Quill.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Regeneration","G","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
{G}: Regenerate enchanted creature. (The next time that creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","1G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Straw Soldiers","1","3","U",NULL,"1U","Creature - Scarecrow Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heartstone","Activated abilities of creatures cost {1} less to activate. This effect can't reduce the amount of mana an ability costs to activate to less than one mana.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Battle of Wits","U","At the beginning of your upkeep, if you have 200 or more cards in your library, you win the game.","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Konda, Lord of Eiganjo","3","3","W","Vigilance (Attacking doesn't cause this creature to tap.)
Bushido 5 (When this blocks or becomes blocked, it gets +5/+5 until end of turn.)
Konda, Lord of Eiganjo is indestructible.","5WW","Legendary Creature - Human Samurai");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mox Jet","{T}: Add {B} to your mana pool.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Resounding Silence","W","Exile target attacking creature.
Cycling {5}{G}{W}{U} ({5}{G}{W}{U}, Discard this card: Draw a card.)
When you cycle Resounding Silence, exile up to two target attacking creatures.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dust Elemental","6","6","W","Flash (You may cast this spell any time you could cast an instant.)
Flying; fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
When Dust Elemental enters the battlefield, return three creatures you control to their owner's hand.","2WW","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phthisis","B","Destroy target creature. Its controller loses life equal to its power plus its toughness.
Suspend 5-{1}{B} (Rather than cast this card from your hand, you may pay {1}{B} and exile it with five time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","3BBBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jaya Ballard, Task Mage","2","2","R","{R}, {T}, Discard a card: Destroy target blue permanent.
{1}{R}, {T}, Discard a card: Jaya Ballard, Task Mage deals 3 damage to target creature or player. A creature dealt damage this way can't be regenerated this turn.
{5}{R}{R}, {T}, Discard a card: Jaya Ballard deals 6 damage to each creature and each player.","1RR","Legendary Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Aeronaut","2","2","W","Kicker {1}{W} (You may pay an additional {1}{W} as you cast this spell.)
Flying
When Kor Aeronaut enters the battlefield, if it was kicked, target creature gains flying until end of turn.","WW","Creature - Kor Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Hippodrome","All creatures get -5/-0.
Whenever you roll {C}, you may destroy target creature if its power is 0 or less.",NULL,"Plane - Segovia");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Recruiter","2","2","R","When Dwarven Recruiter enters the battlefield, search your library for any number of Dwarf cards and reveal those cards. Shuffle your library, then put them on top of it in any order.","2R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mayael's Aria","WRG","At the beginning of your upkeep, put a +1/+1 counter on each creature you control if you control a creature with power 5 or greater. Then you gain 10 life if you control a creature with power 10 or greater. Then you win the game if you control a creature with power 20 or greater.","RGW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rally the Forces","R","Attacking creatures get +1/+0 and gain first strike until end of turn.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vampiric Tutor","B","Search your library for a card, then shuffle your library and put that card on top of it. You lose 2 life.","B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seafloor Debris","Seafloor Debris enters the battlefield tapped.
{T}: Add {U} to your mana pool.
{T}, Sacrifice Seafloor Debris: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Siege Wurm","5","5","G","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Trample","5GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shrouded Lore","B","Target opponent chooses a card in your graveyard. You may pay {B}. If you do, repeat this process except that opponent can't choose a card already chosen for Shrouded Lore. Then put the last chosen card into your hand.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Sludge","B","Target player discards a card for each Swamp you control.","4B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Metathran Elite","2","3","U","Metathran Elite is unblockable as long as it's enchanted.","1UU","Creature - Metathran Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trickbind","U","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Counter target activated or triggered ability. If a permanent's ability is countered this way, activated abilities of that permanent can't be activated this turn. (Mana abilities can't be targeted.)","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Early Frost","U","Tap up to three target lands.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashling, the Extinguisher Avatar","{5}: Destroy all nonland permanents. Activate this ability only once and only during your turn.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vec Townships","{T}: Add {1} to your mana pool.
{T}: Add {G} or {W} to your mana pool. Vec Townships doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Cohort","2","2","R","Goblin Cohort can't attack unless you've cast a creature spell this turn.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ancestral Knowledge","U","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When Ancestral Knowledge enters the battlefield, look at the top ten cards of your library, then exile any number of them and put the rest back on top of your library in any order.
When Ancestral Knowledge leaves the battlefield, shuffle your library.","1U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blackcleave Cliffs","Blackcleave Cliffs enters the battlefield tapped unless you control two or fewer other lands.
{T}: Add {B} or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mana Skimmer","2","2","B","Flying
Whenever Mana Skimmer deals damage to a player, tap target land that player controls. That land doesn't untap during its controller's next untap step.","3B","Creature - Leech");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Haunted Crossroads","B","{B}: Put target creature card from your graveyard on top of your library.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Erhnam Djinn","4","5","G","At the beginning of your upkeep, target non-Wall creature an opponent controls gains forestwalk until your next upkeep.","3G","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Death's Hold","B","Enchant player
Creatures enchanted player controls get -1/-1.","3BB","Enchantment - Aura Curse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Petrified Plating","G","Enchant creature
Enchanted creature gets +2/+2.
Suspend 2-{G} (Rather than cast this card from your hand, you may pay {G} and exile it with two time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","2G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trip Wire","G","Destroy target creature with horsemanship.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veil of Secrecy","U","Target creature gains shroud until end of turn and is unblockable this turn.
Splice onto Arcane-Return a blue creature you control to its owner's hand. (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1U","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deadshot","R","Tap target creature. It deals damage equal to its power to another target creature.","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kiku's Shadow","B","Target creature deals damage to itself equal to its power.","BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spearbreaker Behemoth","5","5","G","Spearbreaker Behemoth is indestructible.
{1}: Target creature with power 5 or greater is indestructible this turn.","5GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coral Eel","2","1","U",NULL,"1U","Creature - Fish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Animate Artifact","U","Enchant artifact
As long as enchanted artifact isn't a creature, it's an artifact creature with power and toughness each equal to its converted mana cost.","3U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Possession","U","Enchant opponent
Skip your draw step.
Whenever enchanted opponent draws a card, you may draw a card.","2UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Adun Oakenshield","1","2","BRG","{B}{R}{G}, {T}: Return target creature card from your graveyard to your hand.","BRG","Legendary Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Echo Mage","2","3","U","Level up {1}{U} ({1}{U}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 2-3
2/4
{U}{U}, {T}: Copy target instant or sorcery spell. You may choose new targets for the copy.
LEVEL 4+
2/5
{U}{U}, {T}: Copy target instant or sorcery spell twice. You may choose new targets for the copies.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lay of the Land","G","Search your library for a basic land card, reveal that card, and put it into your hand. Then shuffle your library.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Amnesia","U","Target player reveals his or her hand and discards all nonland cards.","3UUU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strands of Undeath","B","Enchant creature
When Strands of Undeath enters the battlefield, target player discards two cards.
{B}: Regenerate enchanted creature.","3B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blood Clock","At the beginning of each player's upkeep, that player returns a permanent he or she controls to its owner's hand unless he or she pays 2 life.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jugan, the Rising Star","5","5","G","Flying
When Jugan, the Rising Star dies, you may distribute five +1/+1 counters among any number of target creatures.","3GGG","Legendary Creature - Dragon Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Trap Door Spider","2","3","RG","{1}{R}{G}, {T}: Exile Giant Trap Door Spider and target creature without flying that's attacking you.","1RG","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fevered Strength","B","Target creature gets +2/+0 until end of turn.
Draw a card at the beginning of the next turn's upkeep.","2B","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Welder","1","4","Imprint - {T}: Exile target artifact card from a graveyard.
Myr Welder has all activated abilities of all cards exiled with it.","3","Artifact Creature - Myr");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Island","U",NULL,"Basic Land - Island");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wanderbrine Rootcutters","3","3","UB","Wanderbrine Rootcutters can't be blocked by green creatures.","2(U/B)(U/B)","Creature - Merfolk Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tolarian Emissary","1","2","U","Kicker {1}{W} (You may pay an additional {1}{W} as you cast this spell.)
Flying
When Tolarian Emissary enters the battlefield, if it was kicked, destroy target enchantment.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thraben Doomsayer","2","2","W","{T}: Put a 1/1 white Human creature token onto the battlefield.
Fateful hour - As long as you have 5 or less life, other creatures you control get +2/+2.","1WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu Farmer","1","1","W","{T}: You gain 1 life. Activate this ability only during your turn, before attackers are declared.","1W","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reveillark","4","3","W","Flying
When Reveillark leaves the battlefield, return up to two target creature cards with power 2 or less from your graveyard to the battlefield.
Evoke {5}{W} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","4W","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spoils of Victory","G","Search your library for a Plains, Island, Swamp, Mountain, or Forest card and put that card onto the battlefield. Then shuffle your library.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deadly Grub","3","1","B","Vanishing 3 (This permanent enters the battlefield with three time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
When Deadly Grub dies, if it had no time counters on it, put a 6/1 green Insect creature token with shroud onto the battlefield. (It can't be the target of spells or abilities.)","2B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Symbol of Unsummoning","U","Return target creature to its owner's hand.
Draw a card.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sleep","U","Tap all creatures target player controls. Those creatures don't untap during that player's next untap step.","2UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Natural Balance","G","Each player who controls six or more lands chooses five lands he or she controls and sacrifices the rest. Each player who controls four or fewer lands may search his or her library for X basic land cards and put them onto the battlefield, where X is five minus the number of lands he or she controls. Then each player who searched his or her library this way shuffles it.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lord of the Pit","7","7","B","Flying, trample
At the beginning of your upkeep, sacrifice a creature other than Lord of the Pit. If you can't, Lord of the Pit deals 7 damage to you.","4BBB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keldon Champion","3","2","R","Haste
Echo {2}{R}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Keldon Champion enters the battlefield, it deals 3 damage to target player.","2RR","Creature - Human Barbarian");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Expedition Map","{2}, {T}, Sacrifice Expedition Map: Search your library for a land card, reveal it, and put it into your hand. Then shuffle your library.","1","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Worker","0","0","Modular 1 (This enters the battlefield with a +1/+1 counter on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","1","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flailing Soldier","2","2","R","{1}: Flailing Soldier gets +1/+1 until end of turn. Any player may activate this ability.
{1}: Flailing Soldier gets -1/-1 until end of turn. Any player may activate this ability.","R","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rage Extractor","R","({(r/p)} can be paid with either {R} or 2 life.)
Whenever you cast a spell with {P} in its mana cost, Rage Extractor deals damage equal to that spell's converted mana cost to target creature or player.","4(R/P)","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Outbreak","B","You may discard a Swamp card rather than pay Outbreak's mana cost.
Choose a creature type. All creatures of that type get -1/-1 until end of turn.","3B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reins of the Vinesteed","G","Enchant creature
Enchanted creature gets +2/+2.
When enchanted creature dies, you may return Reins of the Vinesteed from your graveyard to the battlefield attached to a creature that shares a creature type with that creature.","3G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sliptide Serpent","4","4","U","{3}{U}: Return Sliptide Serpent to its owner's hand.","4UU","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skullmane Baku","2","1","B","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Skullmane Baku.
{1}, {T}, Remove X ki counters from Skullmane Baku: Target creature gets -X/-X until end of turn.","3BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Emissary","1","1","R","Kicker {1}{B} (You may pay an additional {1}{B} as you cast this spell.)
When Shivan Emissary enters the battlefield, if it was kicked, destroy target nonblack creature. It can't be regenerated.","2R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Flair","R","Add {R} to your mana pool for each nonland permanent by the artist of your choice.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sea's Claim","U","Enchant land (Target a land as you cast this. This card enters the battlefield attached to that land.)
Enchanted land is an Island.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alaborn Veteran","2","2","W","{T}: Target creature gets +2/+2 until end of turn. Activate this ability only during your turn, before attackers are declared.","2W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Glass Golem","6","2",NULL,"5","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for the Goblin Lord","R","Whenever a Goblin enters the battlefield under your control, you may put a quest counter on Quest for the Goblin Lord.
As long as Quest for the Goblin Lord has five or more quest counters on it, creatures you control get +2/+0.","R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Think Twice","U","Draw a card.
Flashback {2}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of the Sacred Word","2","3","G","As Keeper of the Sacred Word comes into play, choose a word.
Whenever an opponent says the chosen word, Keeper of the Sacred Word gets +3/+3 until end of turn.","2G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Veteran","2","2","W","Whenever Benalish Veteran attacks, it gets +1/+1 until end of turn.","2W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gilded Lotus","{T}: Add three mana of any one color to your mana pool.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kuro, Pitlord","9","9","B","At the beginning of your upkeep, sacrifice Kuro, Pitlord unless you pay {B}{B}{B}{B}.
Pay 1 life: Target creature gets -1/-1 until end of turn.","6BBB","Legendary Creature - Demon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squelch","U","Counter target activated ability. (Mana abilities can't be targeted.)
Draw a card.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hammer of Bogardan","R","Hammer of Bogardan deals 3 damage to target creature or player.
{2}{R}{R}{R}: Return Hammer of Bogardan from your graveyard to your hand. Activate this ability only during your upkeep.","1RR","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Kobolds of Kher Keep","0","1",NULL,"0","Creature - Kobold");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Calming Licid","2","2","W","{W}, {T}: Calming Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {W} to end this effect.
Enchanted creature can't attack.","2W","Creature - Licid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moaning Spirit","2","1","B","Flying","2B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Behemoth","4","4","G","Tap an untapped creature you control: Llanowar Behemoth gets +1/+1 until end of turn.","3GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mammoth Harness","G","Enchant creature
Enchanted creature loses flying.
Whenever enchanted creature blocks or becomes blocked by a creature, the other creature gains first strike until end of turn.","3G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Control Magic","U","Enchant creature
You control enchanted creature.","2UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tomb Hex","B","Target creature gets -2/-2 until end of turn.
Landfall - If you had a land enter the battlefield under your control this turn, that creature gets -4/-4 until end of turn instead.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaze of the Gorgon","BG","({(b/g)} can be paid with either {B} or {G}.)
Regenerate target creature. At end of combat, destroy all creatures that blocked or were blocked by that creature this turn.","3(B/G)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Minamo's Meddling","U","Counter target spell. That spell's controller reveals his or her hand, then discards each card with the same name as a card spliced onto that spell.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Ranger","1","1","G","{T}: You may put a land card from your hand onto the battlefield. Activate this ability only any time you could cast a sorcery.","G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magmatic Force","7","7","R","At the beginning of each upkeep, Magmatic Force deals 3 damage to target creature or player.","5RRR","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sparring Collar","Equipped creature has first strike.
{R}{R}: Attach Sparring Collar to target creature you control.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread Warlock","2","2","B","Dread Warlock can't be blocked except by black creatures.","1BB","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Supersize","G","Target creature gets +3{1/2}/+3{1/2} until end of turn.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Night // Day","W","Creatures target player controls get +1/+1 until end of turn.
---
Target creature gets -1/-1 until end of turn.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exotic Disease","B","Domain - Target player loses X life and you gain X life, where X is the number of basic land types among lands you control.","4B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Awakening","R","Destroy target land.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","4RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crescendo of War","W","At the beginning of each upkeep, put a strife counter on Crescendo of War.
Attacking creatures get +1/+0 for each strife counter on Crescendo of War.
Blocking creatures you control get +1/+0 for each strife counter on Crescendo of War.","3W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thalakos Lowlands","{T}: Add {1} to your mana pool.
{T}: Add {W} or {U} to your mana pool. Thalakos Lowlands doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Murderous Spoils","B","Destroy target nonblack creature. It can't be regenerated. You gain control of all Equipment that was attached to it. (This effect lasts indefinitely.)","5B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwater Alligator","3","2","G","Sacrifice a Forest: Regenerate Rootwater Alligator.","3G","Creature - Crocodile");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Safeguard","W","{2}{W}: Prevent all combat damage that would be dealt by target creature this turn.","3WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exoskeletal Armor","G","Enchant creature
Enchanted creature gets +X/+X, where X is the number of creature cards in all graveyards.","1G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyhunter Prowler","1","3","W","Flying, vigilance (This creature can't be blocked except by creatures with flying or reach, and attacking doesn't cause this creature to tap.)","2W","Creature - Cat Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sharding Sphinx","4","4","U","Flying
Whenever an artifact creature you control deals combat damage to a player, you may put a 1/1 blue Thopter artifact creature token with flying onto the battlefield.","4UU","Artifact Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Wall","0","6","R","Defender (This creature can't attack.)
{R}: Flowstone Wall gets +1/-1 until end of turn.","2R","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Touch of Vitae","G","Until end of turn, target creature gains haste and ""{0}: Untap this creature. Activate this ability only once.""
Draw a card at the beginning of the next turn's upkeep.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stromgald Spy","2","4","B","Whenever Stromgald Spy attacks and isn't blocked, you may have defending player play with his or her hand revealed for as long as Stromgald Spy remains on the battlefield. If you do, Stromgald Spy assigns no combat damage this turn.","3B","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Clickslither","3","3","R","Haste
Sacrifice a Goblin: Clickslither gets +2/+2 and gains trample until end of turn.","1RRR","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Bend","U","Change the text of target permanent by replacing all instances of one color word with another or one basic land type with another. (For example, you may change ""nonblack creature"" to ""nongreen creature"" or ""forestwalk"" to ""islandwalk."" This effect lasts indefinitely.)","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Web of Inertia","U","At the beginning of each opponent's combat, that player may exile a card from his or her graveyard. If the player doesn't, creatures he or she controls can't attack you this turn.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Laquatus's Disdain","U","Counter target spell cast from a graveyard.
Draw a card.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Isao, Enlightened Bushi","2","1","G","Isao, Enlightened Bushi can't be countered.
Bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)
{2}: Regenerate target Samurai.","2G","Legendary Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul's Grace","W","You gain life equal to target creature's power.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swat","B","Destroy target creature with power 2 or less.
Cycling {2} ({2}, Discard this card: Draw a card.)","1BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soldevi Heretic","2","2","U","{W}, {T}: Prevent the next 2 damage that would be dealt to target creature this turn. Target opponent may draw a card.","2U","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Svyelunite Priest","1","1","U","{U}{U}, {T}: Target creature gains shroud until end of turn. Activate this ability only during your upkeep. (It can't be the target of spells or abilities.)","1U","Creature - Merfolk Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nacatl Outlander","2","2","RG","Protection from blue","RG","Creature - Cat Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Manipulation","UB","Choose one or both - Counter target creature spell; and/or return target creature card from your graveyard to your hand.","1UB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Olivia Voldaren","3","3","BR","Flying
{1}{R}: Olivia Voldaren deals 1 damage to another target creature. That creature becomes a Vampire in addition to its other types. Put a +1/+1 counter on Olivia Voldaren.
{3}{B}{B}: Gain control of target Vampire for as long as you control Olivia Voldaren.","2BR","Legendary Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Copperhorn Scout","1","1","G","Whenever Copperhorn Scout attacks, untap each other creature you control.","G","Creature - Elf Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scepter of Dominance","W","{W}, {T}: Tap target permanent.","1WW","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fields of Summer","Whenever a player casts a spell, that player may gain 2 life.
Whenever you roll {C}, you may gain 10 life.",NULL,"Plane - Moag");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hypnotic Cloud","B","Kicker {4} (You may pay an additional {4} as you cast this spell.)
Target player discards a card. If Hypnotic Cloud was kicked, that player discards three cards instead.","1B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirror Entity Avatar","{X}: Choose a creature type. Until end of turn, creatures you control of the chosen type become X/X and gain all creature types.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cocoon","G","Enchant creature you control
Enchanted creature doesn't untap during your untap step if Cocoon has a pupa counter on it.
When Cocoon enters the battlefield, tap enchanted creature and put three pupa counters on Cocoon.
At the beginning of your upkeep, remove a pupa counter from Cocoon. If you can't, sacrifice it, put a +1/+1 counter on enchanted creature, and that creature gains flying. (This effect lasts indefinitely.)","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gryff Vanguard","3","2","U","Flying
When Gryff Vanguard enters the battlefield, draw a card.","4U","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mwonvuli Acid-Moss","G","Destroy target land. Search your library for a Forest card and put that card onto the battlefield tapped. Then shuffle your library.","2GG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Quicksilver Amulet","{4}, {T}: You may put a creature card from your hand onto the battlefield.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aysen Highway","W","White creatures have plainswalk.","3WWW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vile Deacon","2","2","B","Whenever Vile Deacon attacks, it gets +X/+X until end of turn, where X is the number of Clerics on the battlefield.","2BB","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("The Fallen","2","3","B","At the beginning of your upkeep, The Fallen deals 1 damage to each opponent it has dealt damage to this game.","1BBB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Promise of Power","B","Choose one - You draw five cards and you lose 5 life; or put an X/X black Demon creature token with flying onto the battlefield, where X is the number of cards in your hand as the token enters the battlefield.
Entwine {4} (Choose both if you pay the entwine cost.)","2BBB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Soul Foundry","Imprint - When Soul Foundry enters the battlefield, you may exile a creature card from your hand.
{X}, {T}: Put a token that's a copy of the exiled card onto the battlefield. X is the converted mana cost of that card.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Liege of the Hollows","3","4","G","When Liege of the Hollows dies, each player may pay any amount of mana. Then each player who paid mana this way puts that many 1/1 green Squirrel creature tokens onto the battlefield.","2GG","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mirran Mettle","G","Target creature gets +2/+2 until end of turn.
Metalcraft - That creature gets +4/+4 until end of turn instead if you control three or more artifacts.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Puppet Master","U","Enchant creature
When enchanted creature dies, return that card to its owner's hand. If that card is returned to its owner's hand this way, you may pay {U}{U}{U}. If you do, return Puppet Master to its owner's hand.","UUU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Flowstone Sculpture","4","4","{2}, Discard a card: Put a +1/+1 counter on Flowstone Sculpture or Flowstone Sculpture gains flying, first strike, or trample. (This effect lasts indefinitely.)","6","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krark-Clan Ogre","3","3","R","{R}, Sacrifice an artifact: Target creature can't block this turn.","3RR","Creature - Ogre");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Splintering Wind","G","{2}{G}: Splintering Wind deals 1 damage to target creature. Put a 1/1 green Splinter creature token onto the battlefield. It has flying and ""Cumulative upkeep {G}."" When it leaves the battlefield, it deals 1 damage to you and each creature you control. (At the beginning of its controller's upkeep, that player puts an age counter on it, then sacrifices it unless he or she pays its upkeep cost for each age counter on it.)","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alaborn Zealot","1","1","W","When Alaborn Zealot blocks a creature, destroy that creature and Alaborn Zealot.","W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Martyrs' Tomb","WB","Pay 2 life: Prevent the next 1 damage that would be dealt to target creature this turn.","2WB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Verdant Succession","G","Whenever a green nontoken creature dies, that creature's controller may search his or her library for a card with the same name as that creature and put it onto the battlefield. If that player does, he or she shuffles his or her library.","4G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Archaeologist","1","2","R","{R}, {T}: Flip a coin. If you win the flip, destroy target artifact and untap Goblin Archaeologist. If you lose the flip, sacrifice Goblin Archaeologist.","1R","Creature - Goblin Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Virulent Sliver","1","1","G","All Sliver creatures have poisonous 1. (Whenever a Sliver deals combat damage to a player, that player gets a poison counter. A player with ten or more poison counters loses the game.)","G","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tumble Magnet","Tumble Magnet enters the battlefield with three charge counters on it.
{T}, Remove a charge counter from Tumble Magnet: Tap target artifact or creature.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shivan Reef","{T}: Add {1} to your mana pool.
{T}: Add {U} or {R} to your mana pool. Shivan Reef deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frostweb Spider","1","3","G","Reach (This creature can block creatures with flying.)
Whenever Frostweb Spider blocks a creature with flying, put a +1/+1 counter on Frostweb Spider at end of combat.","2G","Snow Creature - Spider");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scalding Tarn","{T}, Pay 1 life, Sacrifice Scalding Tarn: Search your library for an Island or Mountain card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gangrenous Zombies","2","2","B","{T}, Sacrifice Gangrenous Zombies: Gangrenous Zombies deals 1 damage to each creature and each player. If you control a snow Swamp, Gangrenous Zombies deals 2 damage to each creature and each player instead.","1BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oraxid","2","3","U","Protection from red","3U","Creature - Crab Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Ruinblaster","2","1","R","Kicker {R} (You may pay an additional {R} as you cast this spell.)
Haste
When Goblin Ruinblaster enters the battlefield, if it was kicked, destroy target nonbasic land.","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dismantling Blow","W","Kicker {2}{U} (You may pay an additional {2}{U} as you cast this spell.)
Destroy target artifact or enchantment.
If Dismantling Blow was kicked, draw two cards.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Pilferer","1","1","U","Whenever Riptide Pilferer deals combat damage to a player, that player discards a card.
Morph {U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1U","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Muraganda Petroglyphs","G","Creatures with no abilities get +2/+2.","3G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Celestial Force","7","7","W","At the beginning of each upkeep, you gain 3 life.","5WWW","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lone Missionary","2","1","W","When Lone Missionary enters the battlefield, you gain 4 life.","1W","Creature - Kor Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Empty the Catacombs","B","Each player returns all creature cards from his or her graveyard to his or her hand.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Sprites","1","1","G","Flying
{G}, {T}: Add {R} to your mana pool.","1G","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Geist-Honored Monk","*","*","W","Vigilance
Geist-Honored Monk's power and toughness are each equal to the number of creatures you control.
When Geist-Honored Monk enters the battlefield, put two 1/1 white Spirit creature tokens with flying onto the battlefield.","3WW","Creature - Human Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noxious Hatchling","6","6","BG","Noxious Hatchling enters the battlefield with four -1/-1 counters on it.
Wither (This deals damage to creatures in the form of -1/-1 counters.)
Whenever you cast a black spell, remove a -1/-1 counter from Noxious Hatchling.
Whenever you cast a green spell, remove a -1/-1 counter from Noxious Hatchling.","3(B/G)","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grip of Chaos","R","Whenever a spell or ability is put onto the stack, if it has a single target, reselect its target at random. (Select from among all legal targets.)","4RR","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Titanium Golem","3","3","{1}{W}: Titanium Golem gains first strike until end of turn.","5","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("I'm Rubber, You're Glue","W","Speak only in rhyming sentences. If you do not, sacrifice I'm Rubber, You're Glue.
Say ""I'm rubber, you're glue. Everything bounces off me and sticks to you"": Target spell or ability, which targets only you, targets another player of your choice instead. (The new target must be legal.)","WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reef Shaman","0","2","U","{T}: Target land becomes the basic land type of your choice until end of turn.","U","Creature - Merfolk Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Marsh Casualties","B","Kicker {3} (You may pay an additional {3} as you cast this spell.)
Creatures target player controls get -1/-1 until end of turn. If Marsh Casualties was kicked, those creatures get -2/-2 until end of turn instead.","BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Banners Raised","R","Creatures you control get +1/+0 until end of turn.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Black Poplar Shaman","1","3","B","{2}{B}: Regenerate target Treefolk.","2B","Creature - Treefolk Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Molten Rain","R","Destroy target land. If that land was nonbasic, Molten Rain deals 2 damage to the land's controller.","1RR","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Gatstaf Howler","3","3","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Gatstaf Howler.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infuse","U","Untap target artifact, creature, or land.
Draw a card at the beginning of the next turn's upkeep.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Granger Guildmage","1","1","G","{R}, {T}: Granger Guildmage deals 1 damage to target creature or player and 1 damage to you.
{W}, {T}: Target creature gains first strike until end of turn.","G","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Sentry","1","1","R","Whenever an opponent casts a spell, Mogg Sentry gets +2/+2 until end of turn.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oppression","B","Whenever a player casts a spell, that player discards a card.","1BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glory of Warfare","WR","As long as it's your turn, creatures you control get +2/+0.
As long as it's not your turn, creatures you control get +0/+2.","2RW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Glider","2","1","R","{W}: Kavu Glider gets +0/+1 until end of turn.
{U}: Kavu Glider gains flying until end of turn.","2R","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dementia Sliver","3","3","UB","All Slivers have ""{T}: Name a card. Target opponent reveals a card at random from his or her hand. If it's the named card, that player discards it. Activate this ability only during your turn.""","3UB","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Guerrillas","3","2","WR","Shadow (This creature can block or be blocked by only creatures with shadow.)
{0}: The next time Soltari Guerrillas would deal combat damage to an opponent this turn, it deals that damage to target creature instead.","2WR","Creature - Soltari Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hakim, Loreweaver","2","4","U","Flying
{U}{U}: Return target Aura card from your graveyard to the battlefield attached to Hakim, Loreweaver. Activate this ability only during your upkeep and only if Hakim isn't enchanted.
{U}{U}, {T}: Destroy all Auras attached to Hakim.","3UU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vastwood Animist","1","1","G","{T}: Target land you control becomes an X/X Elemental creature until end of turn, where X is the number of Allies you control. It's still a land.","2G","Creature - Elf Shaman Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cemetery Gate","0","5","B","Defender (This creature can't attack.)
Protection from black","2B","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bonesplitter Sliver","2","2","R","All Sliver creatures get +2/+0.","3R","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gibbering Kami","2","2","B","Flying
Soulshift 3 (When this creature dies, you may return target Spirit card with converted mana cost 3 or less from your graveyard to your hand.)","3B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kargan Dragonlord","2","2","R","Level up {R} ({R}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 4-7
4/4
Flying
LEVEL 8+
8/8
Flying, trample
{R}: Kargan Dragonlord gets +1/+0 until end of turn.","RR","Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunting Triad","G","Put three 1/1 green Elf Warrior creature tokens onto the battlefield.
Reinforce 3-{3}{G} ({3}{G}, Discard this card: Put three +1/+1 counters on target creature.)","3G","Tribal Sorcery - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whippoorwill","1","1","G","{G}{G}, {T}: Target creature can't be regenerated this turn. Damage that would be dealt to that creature this turn can't be prevented or dealt instead to another creature or player. When the creature dies this turn, exile the creature.","G","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hyalopterous Lemure","4","3","B","{0}: Hyalopterous Lemure gets -1/-0 and gains flying until end of turn.","4B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shoreline Raider","2","2","U","Protection from Kavu","2U","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vaporous Djinn","3","4","U","Flying
At the beginning of your upkeep, Vaporous Djinn phases out unless you pay {U}{U}. (While it's phased out, it's treated as though it doesn't exist. It phases in before you untap during your next untap step.)","2UU","Creature - Djinn");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Savage Lands","Savage Lands enters the battlefield tapped.
{T}: Add {B}, {R}, or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Cutter","2","2","G","If you control a Forest, rather than pay Skyshroud Cutter's mana cost, you may have each other player gain 5 life.","3G","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vexing Arcanix","{3}, {T}: Target player names a card, then reveals the top card of his or her library. If it's the named card, the player puts it into his or her hand. Otherwise, the player puts it into his or her graveyard and Vexing Arcanix deals 2 damage to him or her.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Skeleton","2","1","R","{1}{B}, Discard a card: Regenerate Viashino Skeleton.","3R","Creature - Viashino Skeleton");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Thermal Navigator","2","2","Sacrifice an artifact: Thermal Navigator gains flying until end of turn.","3","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rebound","U","Change the target of target spell that targets only a player. The new target must be a player.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sunstone","{2}, Sacrifice a snow land: Prevent all combat damage that would be dealt this turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Nacatl","1","1","G","Wild Nacatl gets +1/+1 as long as you control a Mountain.
Wild Nacatl gets +1/+1 as long as you control a Plains.","G","Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lexivore","2","3","W","If Lexivore damages any player, destroy target card in play, other than Lexivore, with the most lines of text in its text box. (If more than one card has the most lines of text, you choose which of those cards to destroy.)","3W","Summon - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Return to Dust","W","Exile target artifact or enchantment. If you cast this spell during your main phase, you may exile up to one other target artifact or enchantment.","2WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abyssal Persecutor","6","6","B","Flying, trample
You can't win the game and your opponents can't lose the game.","2BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hope and Glory","W","Untap two target creatures. Each of them gets +1/+1 until end of turn.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flowstone Flood","R","Buyback-Pay 3 life, Discard a card at random. (You may pay 3 life and discard a card at random in addition to any other costs as you cast this spell. If you do, put this card into your hand as it resolves.)
Destroy target land.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lurker","2","3","G","Lurker can't be the target of spells unless it attacked or blocked this turn.","2G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jihad","W","As Jihad enters the battlefield, choose a color and an opponent.
White creatures get +2/+1 as long as the chosen player controls a nontoken permanent of the chosen color.
When the chosen player controls no nontoken permanents of the chosen color, sacrifice Jihad.","WWW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shanodin Dryads","1","1","G","Forestwalk","G","Creature - Dryad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Bolt","R","Lightning Bolt deals 3 damage to target creature or player.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Catalyst Stone","Flashback costs you pay cost up to {2} less.
Flashback costs your opponents pay cost {2} more.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Traitorous Blood","R","Gain control of target creature until end of turn. Untap it. It gains trample and haste until end of turn.","1RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fertile Imagination","G","Choose a card type. Target opponent reveals his or her hand. Put two 1/1 green Saproling creature tokens onto the battlefield for each card of the chosen type revealed this way. (Artifact, creature, enchantment, instant, land, planeswalker, sorcery, and tribal are card types.)","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nurturing Licid","1","1","G","{G}, {T}: Nurturing Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {G} to end this effect.
{G}: Regenerate enchanted creature.","1G","Creature - Licid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skyshroud Forest","Skyshroud Forest enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{T}: Add {G} or {U} to your mana pool. Skyshroud Forest deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Snare","U","Counter target spell with converted mana cost 2.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Swine-Rider","1","1","R","Whenever Goblin Swine-Rider becomes blocked, it deals 2 damage to each attacking creature and each blocking creature.","R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tooth of Chiss-Goria","Flash
Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
{T}: Target creature gets +1/+0 until end of turn.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Presence of Gond","G","Enchant creature
Enchanted creature has ""{T}: Put a 1/1 green Elf Warrior creature token onto the battlefield.""","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunscape Familiar","0","3","W","Defender (This creature can't attack.)
Green spells and blue spells you cast cost {1} less to cast.","1W","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drake Hatchling","1","3","U","Flying
{U}: Drake Hatchling gets +1/+0 until end of turn. Activate this ability only once each turn.","2U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wu Warship","3","3","U","Wu Warship can't attack unless defending player controls an Island.","2U","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Dust","1","4","R","Defender (This creature can't attack.)
Whenever Wall of Dust blocks a creature, that creature can't attack during its controller's next turn.","2R","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Worldly Counsel","U","Domain - Look at the top X cards of your library, where X is the number of basic land types among lands you control. Put one of those cards into your hand and the rest on the bottom of your library in any order.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lich","B","As Lich enters the battlefield, you lose life equal to your life total.
You don't lose the game for having 0 or less life.
If you would gain life, draw that many cards instead.
Whenever you're dealt damage, sacrifice that many nontoken permanents. If you can't, you lose the game.
When Lich is put into a graveyard from the battlefield, you lose the game.","BBBB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dunerider Outlaw","1","1","B","Protection from green
At the beginning of each end step, if Dunerider Outlaw dealt damage to an opponent this turn, put a +1/+1 counter on it.","BB","Creature - Human Rebel Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seething Pathblazer","2","2","R","Sacrifice an Elemental: Seething Pathblazer gets +2/+0 and gains first strike until end of turn.","2R","Creature - Elemental Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Trapper","1","1","B","{T}: Target creature gains shadow until end of turn. (It can block or be blocked by only creatures with shadow.)","2B","Creature - Dauthi Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dance of the Dead","B","Enchant creature card in a graveyard
When Dance of the Dead enters the battlefield, if it's on the battlefield, it loses ""enchant creature card in a graveyard"" and gains ""enchant creature put onto the battlefield with Dance of the Dead."" Return enchanted creature card to the battlefield tapped under your control and attach Dance of the Dead to it. When Dance of the Dead leaves the battlefield, that creature's controller sacrifices it.
Enchanted creature gets +1/+1 and doesn't untap during its controller's untap step.
At the beginning of the upkeep of enchanted creature's controller, that player may pay {1}{B}. If he or she does, untap that creature.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Journey to Nowhere","W","When Journey to Nowhere enters the battlefield, exile target creature.
When Journey to Nowhere leaves the battlefield, return the exiled card to the battlefield under its owner's control.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lingering Mirage","U","Enchant land
Enchanted land is an Island.
Cycling {2} ({2}, Discard this card: Draw a card.)","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Baron Sengir","5","5","B","Flying
Whenever a creature dealt damage by Baron Sengir this turn dies, put a +2/+2 counter on Baron Sengir.
{T}: Regenerate another target Vampire.","5BBB","Legendary Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whipflare","R","Whipflare deals 2 damage to each nonartifact creature.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Abundant Growth","G","Enchant land
When Abundant Growth enters the battlefield, draw a card.
Enchanted land has ""{T}: Add one mana of any color to your mana pool.""","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noble Templar","3","6","W","Vigilance
Plainscycling {2} ({2}, Discard this card: Search your library for a Plains card, reveal it, and put it into your hand. Then shuffle your library.)","5W","Creature - Human Cleric Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spine of Ish Sah","When Spine of Ish Sah enters the battlefield, destroy target permanent.
When Spine of Ish Sah is put into a graveyard from the battlefield, return Spine of Ish Sah to its owner's hand.","7","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chatter of the Squirrel","G","Put a 1/1 green Squirrel creature token onto the battlefield.
Flashback {1}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desert Sandstorm","R","Desert Sandstorm deals 1 damage to each creature.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carnifex Demon","6","6","B","Flying
Carnifex Demon enters the battlefield with two -1/-1 counters on it.
{B}, Remove a -1/-1 counter from Carnifex Demon: Put a -1/-1 counter on each other creature.","4BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unmask","B","You may exile a black card from your hand rather than pay Unmask's mana cost.
Target player reveals his or her hand. You choose a nonland card from it. That player discards that card.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maga, Traitor to Mortals","0","0","B","Maga, Traitor to Mortals enters the battlefield with X +1/+1 counters on it.
When Maga enters the battlefield, target player loses life equal to the number of +1/+1 counters on it.","XBBB","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wirewood Herald","1","1","G","When Wirewood Herald dies, you may search your library for an Elf card, reveal that card, put it into your hand, then shuffle your library.","1G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shimatsu the Bloodcloaked","0","0","R","As Shimatsu the Bloodcloaked enters the battlefield, sacrifice any number of permanents. Shimatsu enters the battlefield with that many +1/+1 counters on it.","3R","Legendary Creature - Demon Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heartwood Giant","4","4","G","{T}, Sacrifice a Forest: Heartwood Giant deals 2 damage to target player.","3GG","Creature - Giant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Leonin Bola","Equipped creature has ""{T}, Unattach Leonin Bola: Tap target creature.""
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conspiracy","B","As Conspiracy enters the battlefield, choose a creature type.
Creature cards you own that aren't on the battlefield, creature spells you control, and creatures you control are the chosen type.","3BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call to the Grave","B","At the beginning of each player's upkeep, that player sacrifices a non-Zombie creature.
At the beginning of the end step, if no creatures are on the battlefield, sacrifice Call to the Grave.","4B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Errant Ephemeron","4","4","U","Flying
Suspend 4-{1}{U} (Rather than cast this card from your hand, you may pay {1}{U} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","6U","Creature - Illusion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Soldier token card",NULL,NULL,NULL);
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thoughtweft Gambit","WU","Tap all creatures your opponents control and untap all creatures you control.","4(W/U)(W/U)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Hatcher","0","0","G","Spike Hatcher enters the battlefield with six +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Hatcher: Put a +1/+1 counter on target creature.
{1}, Remove a +1/+1 counter from Spike Hatcher: Regenerate Spike Hatcher.","6G","Creature - Spike");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Army Ants","1","1","BR","{T}, Sacrifice a land: Destroy target land.","1BR","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Knight","1","1","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
{1}{W}: Kjeldoran Knight gets +1/+0 until end of turn.
{W}{W}: Kjeldoran Knight gets +0/+2 until end of turn.","WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiteful Bully","3","3","B","At the beginning of your upkeep, Spiteful Bully deals 3 damage to target creature you control.","1B","Creature - Zombie Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caterwauling Boggart","2","2","R","Each Goblin you control can't be blocked except by two or more creatures.
Each Elemental you control can't be blocked except by two or more creatures.","3R","Creature - Goblin Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skycloud Egg","{2}, {T}, Sacrifice Skycloud Egg: Add {W}{U} to your mana pool. Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deadly Recluse","1","2","G","Reach (This creature can block creatures with flying.)
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","1G","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stronghold Overseer","5","5","B","Flying
Shadow (This creature can block or be blocked by only creatures with shadow.)
{B}{B}: Creatures with shadow get +1/+0 until end of turn and creatures without shadow get -1/-0 until end of turn.","3BBB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jagged-Scar Archers","*","*","G","Jagged-Scar Archers's power and toughness are each equal to the number of Elves you control.
{T}: Jagged-Scar Archers deals damage equal to its power to target creature with flying.","1GG","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glissa, the Traitor","3","3","BG","First strike, deathtouch
Whenever a creature an opponent controls dies, you may return target artifact card from your graveyard to your hand.","BGG","Legendary Creature - Zombie Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Go for the Throat","B","Destroy target nonartifact creature.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oakenform","G","Enchant creature
Enchanted creature gets +3/+3.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidehollow Strix","2","1","UB","Flying
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","UB","Artifact Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blight Mamba","1","1","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
{1}{G}: Regenerate Blight Mamba.","1G","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Sledder","1","1","R","Sacrifice a Goblin: Target creature gets +1/+1 until end of turn.","R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trigon of Infestation","Trigon of Infestation enters the battlefield with three charge counters on it.
{G}{G}, {T}: Put a charge counter on Trigon of Infestation.
{2}, {T}, Remove a charge counter from Trigon of Infestation: Put a 1/1 green Insect creature token with infect onto the battlefield.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Caves of Koilos","{T}: Add {1} to your mana pool.
{T}: Add {W} or {B} to your mana pool. Caves of Koilos deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fervent Denial","U","Counter target spell.
Flashback {5}{U}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flux","U","Each player discards any number of cards, then draws that many cards.
Draw a card.","2U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wanderwine Hub","As Wanderwine Hub enters the battlefield, you may reveal a Merfolk card from your hand. If you don't, Wanderwine Hub enters the battlefield tapped.
{T}: Add {W} or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deception","B","Target opponent discards two cards.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Imaginary Pet","4","4","U","At the beginning of your upkeep, if you have a card in hand, return Imaginary Pet to its owner's hand.","1U","Creature - Illusion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Teferi's Puzzle Box","At the beginning of each player's draw step, that player puts the cards in his or her hand on the bottom of his or her library in any order, then draws that many cards.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Hookmaster","2","2","W","When Kor Hookmaster enters the battlefield, tap target creature an opponent controls. That creature doesn't untap during its controller's next untap step.","2W","Creature - Kor Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Interloper","2","1","B","Flying
Vampire Interloper can't block.","1B","Creature - Vampire Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scuzzback Scrapper","1","1","RG","Wither (This deals damage to creatures in the form of -1/-1 counters.)","(R/G)","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul Warden","1","1","W","Whenever another creature enters the battlefield, you gain 1 life.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chastise","W","Destroy target attacking creature. You gain life equal to its power.","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ward of Lights","W","You may cast Ward of Lights as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Enchant creature
As Ward of Lights enters the battlefield, choose a color.
Enchanted creature has protection from the chosen color. This effect doesn't remove Ward of Lights.","WW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veteran Bodyguard","2","5","W","As long as Veteran Bodyguard is untapped, all damage that would be dealt to you by unblocked creatures is dealt to Veteran Bodyguard instead.","3WW","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Needleshot Gourna","3","6","G","Reach (This creature can block creatures with flying.)","4GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Searing Touch","R","Buyback {4} (You may pay an additional {4} as you cast this spell. If you do, put this card into your hand as it resolves.)
Searing Touch deals 1 damage to target creature or player.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treacherous Pit-Dweller","4","3","B","When Treacherous Pit-Dweller enters the battlefield from a graveyard, target opponent gains control of it.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Assault // Battery","R","Assault deals 2 damage to target creature or player.
---
Put a 3/3 green Elephant creature token onto the battlefield.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Young Wei Recruits","2","2","B","Young Wei Recruits can't block.","1B","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza","{3}: Urza deals 1 damage to target creature or player.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Craw Giant","6","4","G","Trample
Rampage 2 (Whenever this creature becomes blocked, it gets +2/+2 until end of turn for each creature blocking it beyond the first.)","3GGGG","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Punish Ignorance","WUB","Counter target spell. Its controller loses 3 life and you gain 3 life.","WUUB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stream Hopper","1","1","UR","{(u/r)}: Stream Hopper gains flying until end of turn.","(U/R)","Creature - Goblin");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Soultether Golem","3","3","Vanishing 1 (This permanent enters the battlefield with a time counter on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
Whenever another creature enters the battlefield under your control, put a time counter on Soultether Golem.","2","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elder Cathar","2","2","W","When Elder Cathar dies, put a +1/+1 counter on target creature you control. If that creature is a Human, put two +1/+1 counters on it instead.","2W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Serra Angel Avatar","Whenever you cast a spell, you gain 2 life.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Twilight Shepherd","5","5","W","Flying, vigilance
When Twilight Shepherd enters the battlefield, return to your hand all cards in your graveyard that were put there from the battlefield this turn.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","3WWW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diminish","U","Target creature becomes 1/1 until end of turn.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fertilid","0","0","G","Fertilid enters the battlefield with two +1/+1 counters on it.
{1}{G}, Remove a +1/+1 counter from Fertilid: Target player searches his or her library for a basic land card and puts it onto the battlefield tapped. Then that player shuffles his or her library.","2G","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vault Skyward","U","Target creature gains flying until end of turn. Untap it.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goham Djinn","5","5","B","{1}{B}: Regenerate Goham Djinn.
Goham Djinn gets -2/-2 as long as black is the most common color among all permanents or is tied for most common.","5B","Creature - Djinn");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kormus Bell","All Swamps are 1/1 black creatures that are still lands.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Orim","Creatures you control have reach.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breakthrough","U","Draw four cards, then choose X cards in your hand and discard the rest.","XU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rupture Spire","Rupture Spire enters the battlefield tapped.
When Rupture Spire enters the battlefield, sacrifice it unless you pay {1}.
{T}: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death of a Thousand Stings","B","Target player loses 1 life and you gain 1 life.
At the beginning of your upkeep, if you have more cards in hand than each opponent, you may return Death of a Thousand Stings from your graveyard to your hand.","4B","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Moon","2","2","R","Nonbasic lands are Mountains.","2R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opal Gargoyle","W","When an opponent casts a creature spell, if Opal Gargoyle is an enchantment, Opal Gargoyle becomes a 2/2 Gargoyle creature with flying.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistbind Clique","4","4","U","Flash
Flying
Champion a Faerie (When this enters the battlefield, sacrifice it unless you exile another Faerie you control. When this leaves the battlefield, that card returns to the battlefield.)
When a Faerie is championed with Mistbind Clique, tap all lands target player controls.","3U","Creature - Faerie Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Krark's Thumb","If you would flip a coin, instead flip two coins and ignore one.","2","Legendary Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wasteland","{T}: Add {1} to your mana pool.
{T}, Sacrifice Wasteland: Destroy target nonbasic land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diregraf Captain","2","2","UB","Deathtouch
Other Zombie creatures you control get +1/+1.
Whenever another Zombie you control dies, target opponent loses 1 life.","1UB","Creature - Zombie Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pinpoint Avalanche","R","Pinpoint Avalanche deals 4 damage to target creature. The damage can't be prevented.","3RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rebuff the Wicked","W","Counter target spell that targets a permanent you control.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ranger of Eos","3","2","W","When Ranger of Eos enters the battlefield, you may search your library for up to two creature cards with converted mana cost 1 or less, reveal them, and put them into your hand. If you do, shuffle your library.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Venerable Kumo","2","3","G","Reach (This creature can block creatures with flying.)
Soulshift 4 (When this creature dies, you may return target Spirit card with converted mana cost 4 or less from your graveyard to your hand.)","4G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brighthearth Banneret","1","1","R","Elemental spells and Warrior spells you cast cost {1} less to cast.
Reinforce 1-{1}{R} ({1}{R}, Discard this card: Put a +1/+1 counter on target creature.)","1R","Creature - Elemental Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tainted Wood","{T}: Add {1} to your mana pool.
{T}: Add {B} or {G} to your mana pool. Activate this ability only if you control a Swamp.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Demonic Taskmaster","4","3","B","Flying
At the beginning of your upkeep, sacrifice a creature other than Demonic Taskmaster.","2B","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sarcomite Myr","2","1","U","{2}: Sarcomite Myr gains flying until end of turn.
{2}, Sacrifice Sarcomite Myr: Draw a card.","2U","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Resilient Wanderer","2","3","W","First strike
Discard a card: Resilient Wanderer gains protection from the color of your choice until end of turn.","2WW","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Undertow","U","Creatures with islandwalk can be blocked as though they didn't have islandwalk.","2U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grinding Station","{T}, Sacrifice an artifact: Target player puts the top three cards of his or her library into his or her graveyard.
Whenever an artifact enters the battlefield, you may untap Grinding Station.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jodah's Avenger","4","4","U","{0}: Until end of turn, Jodah's Avenger gets -1/-1 and gains your choice of double strike, protection from red, vigilance, or shadow. (A creature with shadow can block or be blocked by only creatures with shadow.)","5U","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fevered Convulsions","B","{2}{B}{B}: Put a -1/-1 counter on target creature.","BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vitalize","G","Untap all creatures you control.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Windreader","3","3","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
{1}{U}: Target player reveals the top card of his or her library.","3UU","Creature - Bird Soldier Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Truce","W","Each player may draw up to two cards. For each card less than two a player draws this way, that player gains 2 life.","2W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grinning Demon Avatar","Whenever a nontoken creature you control dies, target opponent discards a card.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aphotic Wisps","B","Target creature becomes black and gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)
Draw a card.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skittering Skirge","3","2","B","Flying
When you cast a creature spell, sacrifice Skittering Skirge.","BB","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lifesmith","2","1","G","Whenever you cast an artifact spell, you may pay {1}. If you do, you gain 3 life.","1G","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chandra's Spitfire","1","3","R","Flying
Whenever an opponent is dealt noncombat damage, Chandra's Spitfire gets +3/+0 until end of turn.","2R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feedback Bolt","R","Feedback Bolt deals damage to target player equal to the number of artifacts you control.","4R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stabbing Pain","B","Target creature gets -1/-1 until end of turn. Tap that creature.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Safehold Duo","2","4","WG","Whenever you cast a green spell, Safehold Duo gets +1/+1 until end of turn.
Whenever you cast a white spell, Safehold Duo gains vigilance until end of turn.","3(G/W)","Creature - Elf Warrior Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Wrath","G","At the beginning of your upkeep, sacrifice Nature's Wrath unless you pay {G}.
Whenever a player puts an Island or blue permanent onto the battlefield, he or she sacrifices an Island or blue permanent.
Whenever a player puts a Swamp or black permanent onto the battlefield, he or she sacrifices a Swamp or black permanent.","4GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hivis of the Scale","3","4","R","You may choose not to untap Hivis of the Scale during your untap step.
{T}: Gain control of target Dragon for as long as you control Hivis and Hivis remains tapped.","3RR","Legendary Creature - Viashino Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blade of the Bloodchief","Whenever a creature dies, put a +1/+1 counter on equipped creature. If equipped creature is a Vampire, put two +1/+1 counters on it instead.
Equip {1}","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sewer Rats","1","1","B","{B}, Pay 1 life: Sewer Rats gets +1/+0 until end of turn. Activate this ability no more than three times each turn.","B","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sinking Feeling","U","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.
Enchanted creature has ""{1}, Put a -1/-1 counter on this creature: Untap this creature.""","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minamo Sightbender","1","2","U","{X}, {T}: Target creature with power X or less is unblockable this turn.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infiltrator il-Kor","3","1","U","Shadow (This creature can block or be blocked by only creatures with shadow.)
Suspend 2-{1}{U} (Rather than cast this card from your hand, you may pay {1}{U} and exile it with two time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","4U","Creature - Kor Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Druid's Familiar","2","2","G","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Druid's Familiar is paired with another creature, each of those creatures gets +2/+2.","3G","Creature - Bear");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mox Emerald","{T}: Add {G} to your mana pool.","0","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tangleroot","Whenever a player casts a creature spell, that player adds {G} to his or her mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kindercatch","6","6","G",NULL,"3GGG","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pariah","W","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
All damage that would be dealt to you is dealt to enchanted creature instead.","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crimson Roc","2","2","R","Flying
Whenever Crimson Roc blocks a creature without flying, Crimson Roc gets +1/+0 and gains first strike until end of turn.","4R","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soothing Balm","W","Target player gains 5 life.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Archangel's Light","W","You gain 2 life for each card in your graveyard, then shuffle your graveyard into your library.","7W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strands of Night","B","{B}{B}, Pay 2 life, Sacrifice a Swamp: Return target creature card from your graveyard to the battlefield.","2BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Triumph of Ferocity","G","At the beginning of your upkeep, draw a card if you control the creature with the greatest power or tied for the greatest power.","2G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bazaar of Baghdad","{T}: Draw two cards, then discard three cards.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tradewind Rider","1","4","U","Flying
{T}, Tap two untapped creatures you control: Return target permanent to its owner's hand.","3U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alaborn Grenadier","2","2","W","Vigilance","WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oona's Grace","U","Target player draws a card.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Golgari Rotwurm","5","4","BG","{B}, Sacrifice a creature: Target player loses 1 life.","3BG","Creature - Zombie Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prison Term","W","Enchant creature
Enchanted creature can't attack or block, and its activated abilities can't be activated.
Whenever a creature enters the battlefield under an opponent's control, you may attach Prison Term to that creature.","1WW","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strength of Unity","W","Enchant creature
Domain - Enchanted creature gets +1/+1 for each basic land type among lands you control.","3W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of Empty Graves","4","1","B","Soulshift 3 (When this creature dies, you may return target Spirit card with converted mana cost 3 or less from your graveyard to your hand.)","3B","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jund Panorama","{T}: Add {1} to your mana pool.
{1}, {T}, Sacrifice Jund Panorama: Search your library for a basic Swamp, Mountain, or Forest card and put it onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shielding Plax","UG","({(g/u)} can be paid with either {G} or {U}.)
Enchant creature
When Shielding Plax enters the battlefield, draw a card.
Enchanted creature can't be the target of spells or abilities your opponents control.","2(G/U)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("The Fallen Apart","4","4","B","The Fallen Apart comes into play with two arms and two legs.
Whenever damage is dealt to The Fallen Apart, remove an arm or a leg from it.
The Fallen Apart can't attack if it has no legs and can't block if it has no arms.","2BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Treacherous Urge","B","Target opponent reveals his or her hand. You may put a creature card from it onto the battlefield under your control. That creature gains haste. Sacrifice it at the beginning of the next end step.","4B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sandstone Needle","Sandstone Needle enters the battlefield tapped with two depletion counters on it.
{T}, Remove a depletion counter from Sandstone Needle: Add {R}{R} to your mana pool. If there are no depletion counters on Sandstone Needle, sacrifice it.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kagemaro, First to Suffer","*","*","B","Kagemaro, First to Suffer's power and toughness are each equal to the number of cards in your hand.
{B}, Sacrifice Kagemaro: All creatures get -X/-X until end of turn, where X is the number of cards in your hand.","3BB","Legendary Creature - Demon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spread the Sickness","B","Destroy target creature, then proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","4B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("I Bask in Your Silent Awe","(An ongoing scheme remains face up until it's abandoned.)
Each opponent can't cast more than one spell each turn.
At the beginning of your upkeep, if no opponent cast a spell since your last turn ended, abandon this scheme.",NULL,"Ongoing Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Monarch","3","3","R","Kavu creatures have trample.
Whenever another Kavu enters the battlefield, put a +1/+1 counter on Kavu Monarch.","2RR","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guardian Beast","2","4","B","As long as Guardian Beast is untapped, noncreature artifacts you control can't be enchanted, they're indestructible, and other players can't gain control of them. This effect doesn't remove Auras already attached to those artifacts.","3B","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hyperion Blacksmith","2","2","R","{T}: You may tap or untap target artifact an opponent controls.","1RR","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Archdemon of Unx","6","6","B","Flying, trample
At the beginning of your upkeep, sacrifice a non-Zombie creature, then put a 2/2 black Zombie creature token onto the battlefield.","5BB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul Seizer","1","3","U","Flying
When Soul Seizer deals combat damage to a player, you may transform it. If you do, attach it to target creature that player controls.","3UU","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Venom","G","Enchant creature
Whenever enchanted creature blocks or becomes blocked by a non-Wall creature, destroy the other creature at end of combat.","1GG","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overwhelm","G","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Creatures you control get +3/+3 until end of turn.","5GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of the Flame","1","2","R","{R}, {T}: Choose target opponent who had more life than you did as you activated this ability. Keeper of the Flame deals 2 damage to him or her.","RR","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grimoire Thief","2","2","U","Whenever Grimoire Thief becomes tapped, exile the top three cards of target opponent's library face down.
You may look at cards exiled with Grimoire Thief.
{U}, Sacrifice Grimoire Thief: Turn all cards exiled with Grimoire Thief face up. Counter all spells with those names.","UU","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feeding Frenzy","B","Target creature gets -X/-X until end of turn, where X is the number of Zombies on the battlefield.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cephalid Shrine","U","Whenever a player casts a spell, counter that spell unless that player pays {X}, where X is the number of cards in all graveyards with the same name as the spell.","1UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Willbender","1","2","U","Morph {1}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Willbender is turned face up, change the target of target spell or ability with a single target.","1U","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Swiftfoot Boots","Equipped creature has hexproof and haste. (It can't be the target of spells or abilities your opponents control, and it can attack and {T} as soon as it comes under your control.)
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Silent Departure","U","Return target creature to its owner's hand.
Flashback {4}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","U","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Duplicant","2","4","Imprint - When Duplicant enters the battlefield, you may exile target nontoken creature.
As long as the exiled card is a creature card, Duplicant has that card's power, toughness, and creature types. It's still a Shapeshifter.","6","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Helium Squirter","0","0","U","Graft 3 (This creature enters the battlefield with three +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{1}: Target creature with a +1/+1 counter on it gains flying until end of turn.","4U","Creature - Beast Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Bloodpainter","2","1","R","{T}, Sacrifice a creature: Orcish Bloodpainter deals 1 damage to target creature or player.","2R","Creature - Orc Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chromescale Drake","3","4","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Flying
When Chromescale Drake enters the battlefield, reveal the top three cards of your library. Put all artifact cards revealed this way into your hand and the rest into your graveyard.","6UUU","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yoked Plowbeast","5","5","W","Cycling {2} ({2}, Discard this card: Draw a card.)","5WW","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blind with Anger","R","Untap target nonlegendary creature and gain control of it until end of turn. That creature gains haste until end of turn.","3R","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tranquil Path","G","Destroy all enchantments.
Draw a card.","4G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firestorm Phoenix","3","2","R","Flying
If Firestorm Phoenix would die, return Firestorm Phoenix to its owner's hand instead. Until that player's next turn, that player plays with that card revealed in his or her hand and can't play it.","4RR","Creature - Phoenix");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cover of Winter","W","Cumulative upkeep {S}i} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it. {S}i} can be paid with one mana from a snow permanent.)
If a creature would deal combat damage to you and/or one or more creatures you control, prevent X of that damage, where X is the number of age counters on Cover of Winter.
{S}i}: Put an age counter on Cover of Winter.","2W","Snow Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Outcasts","2","2","B","Bloodthirst 2 (If an opponent was dealt damage this turn, this creature enters the battlefield with two +1/+1 counters on it.)
Lifelink (Damage dealt by this creature also causes you to gain that much life.)","2BB","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soar","U","You may cast Soar as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Enchant creature
Enchanted creature gets +0/+1 and has flying.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scorching Missile","R","Scorching Missile deals 4 damage to target player.
Flashback {9}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Master of the Wild Hunt Avatar","{2}{G}: Put a green creature token onto the battlefield that's a 2/2 Wolf, a 2/3 Antelope with forestwalk, a 3/2 Cat with shroud, or a 4/4 green Rhino with trample, chosen at random.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("I Call on the Ancient Magics","When you set this scheme in motion, each other player searches his or her library for a card, reveals it, and puts it into his or her hand. Then you search your library for two cards and put them into your hand. Each player shuffles his or her library.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Warthog","5","5","G","Trample","5G","Creature - Boar Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Altar Golem","*","*","Trample
Altar Golem's power and toughness are each equal to the number of creatures on the battlefield.
Altar Golem doesn't untap during your untap step.
Tap five untapped creatures you control: Untap Altar Golem.","7","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Stalked Prey","R","Enchant player
Whenever a creature deals combat damage to enchanted player, put a +1/+1 counter on that creature.","1R","Enchantment - Aura Curse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prismwake Merrow","2","1","U","Flash
When Prismwake Merrow enters the battlefield, target permanent becomes the color or colors of your choice until end of turn.","2U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peppersmoke","B","Target creature gets -1/-1 until end of turn. If you control a Faerie, draw a card.","B","Tribal Instant - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thresher Beast","4","4","G","Whenever Thresher Beast becomes blocked, defending player sacrifices a land.","3GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plaxmanta","2","2","U","Flash
When Plaxmanta enters the battlefield, creatures you control gain shroud until end of turn. (They can't be the targets of spells or abilities.)
When Plaxmanta enters the battlefield, sacrifice it unless {G} was spent to cast it.","1U","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Close Quarters","R","Whenever a creature you control becomes blocked, Close Quarters deals 1 damage to target creature or player.","2RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Horde","5","5","R","When Balduvian Horde enters the battlefield, sacrifice it unless you discard a card at random.","2RR","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spectral Bears","3","3","G","Whenever Spectral Bears attacks, if defending player controls no black nontoken permanents, it doesn't untap during your next untap step.","1G","Creature - Bear Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudgoat Ranger","3","3","W","When Cloudgoat Ranger enters the battlefield, put three 1/1 white Kithkin Soldier creature tokens onto the battlefield.
Tap three untapped Kithkin you control: Cloudgoat Ranger gets +2/+0 and gains flying until end of turn.","3WW","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Serpent","5","5","B","Bog Serpent can't attack unless defending player controls a Swamp.
When you control no Swamps, sacrifice Bog Serpent.","5B","Creature - Serpent");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pit Trap","{2}, {T}, Sacrifice Pit Trap: Destroy target attacking creature without flying. It can't be regenerated.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recurring Insight","U","Draw cards equal to the number of cards in target opponent's hand.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","4UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sacred Wolf","3","1","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)","2G","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aquamorph Entity","*","*","U","As Aquamorph Entity enters the battlefield or is turned face up, it becomes your choice of 5/1 or 1/5.
Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2UU","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Blightsteel Colossus","11","11","Trample, infect
Blightsteel Colossus is indestructible.
If Blightsteel Colossus would be put into a graveyard from anywhere, reveal Blightsteel Colossus and shuffle it into its owner's library instead.","12","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Caustic Rain","B","Exile target land.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yuki-Onna","3","1","R","When Yuki-Onna enters the battlefield, destroy target artifact.
Whenever you cast a Spirit or Arcane spell, you may return Yuki-Onna to its owner's hand.","3R","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zephyr Net","U","Enchant creature
Enchanted creature has defender and flying.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mistvein Borderpost","UB","You may pay {1} and return a basic land you control to its owner's hand rather than pay Mistvein Borderpost's mana cost.
Mistvein Borderpost enters the battlefield tapped.
{T}: Add {U} or {B} to your mana pool.","1UB","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reign of Terror","B","Destroy all green creatures or all white creatures. They can't be regenerated. You lose 2 life for each creature that died this way.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prodigal Pyromancer","1","1","R","{T}: Prodigal Pyromancer deals 1 damage to target creature or player.","2R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oath of Lim-Dûl","B","Whenever you lose life, for each 1 life you lost, sacrifice a permanent other than Oath of Lim-Dûl unless you discard a card. (Damage dealt to you causes you to lose life.)
{B}{B}: Draw a card.","3B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grixis Grimblade","2","1","UBR","As long as you control another multicolored permanent, Grixis Grimblade gets +1/+1 and has deathtouch. (Any amount of damage it deals to a creature is enough to destroy that creature.)","(U/R)B","Creature - Zombie Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sage of Fables","2","2","U","Each other Wizard creature you control enters the battlefield with an additional +1/+1 counter on it.
{2}, Remove a +1/+1 counter from a creature you control: Draw a card.","2U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crumbling Ashes","B","At the beginning of your upkeep, destroy target creature with a -1/-1 counter on it.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pygmy Kavu","1","2","G","When Pygmy Kavu enters the battlefield, draw a card for each black creature your opponents control.","3G","Creature - Kavu");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Digester","2","1","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","3","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Farhaven Elf","1","1","G","When Farhaven Elf enters the battlefield, you may search your library for a basic land card and put it onto the battlefield tapped. If you do, shuffle your library.","2G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pongify","U","Destroy target creature. It can't be regenerated. That creature's controller puts a 3/3 green Ape creature token onto the battlefield.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Geyser","R","Add {R} to your mana pool for each tapped land your opponents control.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diplomatic Escort","1","1","U","{U}, {T}, Discard a card: Counter target spell or ability that targets a creature.","1U","Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reins of Power","U","Untap all creatures you control and all creatures target opponent controls. You and that opponent each gain control of all creatures the other controls until end of turn. Those creatures gain haste until end of turn.","2UU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Staff of Domination","{1}: Untap Staff of Domination.
{2}, {T}: You gain 1 life.
{3}, {T}: Untap target creature.
{4}, {T}: Tap target creature.
{5}, {T}: Draw a card.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scorching Winds","R","Cast Scorching Winds only during the declare attackers step and only if you've been attacked this step.
Scorching Winds deals 1 damage to each attacking creature.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cinder Storm","R","Cinder Storm deals 7 damage to target creature or player.","6R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guiding Spirit","1","2","WU","Flying
{T}: If the top card of target player's graveyard is a creature card, put that card on top of that player's library.","1WU","Creature - Angel Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Meng Huo's Horde","4","5","G",NULL,"4G","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Entrancer","1","4","U","{U}, {T}: Target player puts the top two cards of his or her library into his or her graveyard.","3U","Creature - Vedalken Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death by Dragons","R","Each player other than target player puts a 5/5 red Dragon creature token with flying onto the battlefield.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wood Elves","1","1","G","When Wood Elves enters the battlefield, search your library for a Forest card and put that card onto the battlefield. Then shuffle your library.","2G","Creature - Elf Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Paradox Haze","U","Enchant player
At the beginning of enchanted player's first upkeep each turn, that player gets an additional upkeep step after this step.","2U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Howling Fury","B","Target creature gets +4/+0 until end of turn.","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gods' Eye, Gate to the Reikai","{T}: Add {1} to your mana pool.
When Gods' Eye, Gate to the Reikai is put into a graveyard from the battlefield, put a 1/1 colorless Spirit creature token onto the battlefield.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mirrorweave","WU","Each other creature becomes a copy of target nonlegendary creature until end of turn.","2(W/U)(W/U)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Realm Razer","4","2","WRG","When Realm Razer enters the battlefield, exile all lands.
When Realm Razer leaves the battlefield, return the exiled cards to the battlefield tapped under their owners' control.","3RGW","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disembowel","B","Destroy target creature with converted mana cost X.","XB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Foot Soldier","1","1","W","Shadow (This creature can block or be blocked by only creatures with shadow.)","W","Creature - Soltari Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harrow","G","As an additional cost to cast Harrow, sacrifice a land.
Search your library for up to two basic land cards and put them onto the battlefield. Then shuffle your library.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quicksilver Elemental","3","4","U","{U}: Quicksilver Elemental gains all activated abilities of target creature until end of turn. (If any of the abilities use that creature's name, use this creature's name instead.)
You may spend blue mana as though it were mana of any color to pay the activation costs of Quicksilver Elemental's abilities.","3UU","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Martyred Rusalka","1","1","W","{W}, Sacrifice a creature: Target creature can't attack this turn.","W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dross Hopper","2","1","B","Sacrifice a creature: Dross Hopper gains flying until end of turn.","1B","Creature - Insect Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fault Line","R","Fault Line deals X damage to each creature without flying and each player.","XRR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Booster Tutor","B","Open a sealed Magic booster pack, reveal the cards, and put one of those cards into your hand. (Remove that card from your deck before beginning a new game.)","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hoar Shade","1","2","B","{B}: Hoar Shade gets +1/+1 until end of turn.","3B","Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorilla Warrior","3","2","G",NULL,"2G","Creature - Ape Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twilight's Call","B","You may cast Twilight's Call any time you could cast an instant if you pay {2} more to cast it.
Each player returns all creature cards from his or her graveyard to the battlefield.","4BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Briar Shield","G","Enchant creature
Enchanted creature gets +1/+1.
Sacrifice Briar Shield: Enchanted creature gets +3/+3 until end of turn.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blade Sliver","2","2","R","All Sliver creatures get +1/+0.","2R","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Violent Eruption","R","Violent Eruption deals 4 damage divided as you choose among any number of target creatures and/or players.
Madness {1}{R}{R} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","1RRR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soulblast","R","As an additional cost to cast Soulblast, sacrifice all creatures you control.
Soulblast deals damage to target creature or player equal to the total power of the sacrificed creatures.","3RRR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treetop Rangers","2","2","G","Treetop Rangers can't be blocked except by creatures with flying.","2G","Creature - Elf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vulshok Gauntlets","Equipped creature gets +4/+2 and doesn't untap during its controller's untap step.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Tiger","3","4","G","Forestwalk","2GG","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Calming Verse","G","Destroy all enchantments you don't control. Then, if you control an untapped land, destroy all enchantments you control.","3G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urborg Volcano","Urborg Volcano enters the battlefield tapped.
{T}: Add {B} or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Palliation Accord","WU","Whenever a creature an opponent controls becomes tapped, put a shield counter on Palliation Accord.
Remove a shield counter from Palliation Accord: Prevent the next 1 damage that would be dealt to you this turn.","3WU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Betrayal of Flesh","B","Choose one - Destroy target creature; or return target creature card from your graveyard to the battlefield.
Entwine-Sacrifice three lands. (Choose both if you pay the entwine cost.)","5B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Form of the Squirrel","G","As Form of the Squirrel comes into play, put a 1/1 green Squirrel creature token into play. You lose the game when it leaves play.
Creatures can't attack you.
You can't be the target of spells or abilities.
You can't play spells.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infantry Veteran","1","1","W","{T}: Target attacking creature gets +1/+1 until end of turn.","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brigid, Hero of Kinsbaile","2","3","W","First strike
{T}: Brigid, Hero of Kinsbaile deals 2 damage to each attacking or blocking creature target player controls.","2WW","Legendary Creature - Kithkin Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greenhilt Trainee","2","3","G","{T}: Target creature gets +4/+4 until end of turn. Activate this ability only if Greenhilt Trainee's power is 4 or greater.","3G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Transcendent Master","3","3","W","Level up {1} ({1}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 6-11
6/6
Lifelink
LEVEL 12+
9/9
Lifelink
Transcendent Master is indestructible.","1WW","Creature - Human Cleric Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spire Barrage","R","Spire Barrage deals damage to target creature or player equal to the number of Mountains you control.","4R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Machinate","U","Look at the top X cards of your library, where X is the number of artifacts you control. Put one of those cards into your hand and the rest on the bottom of your library in any order.","1UU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Conversion Chamber","{2}, {T}: Exile target artifact card from a graveyard. Put a charge counter on Conversion Chamber.
{2}, {T}, Remove a charge counter from Conversion Chamber: Put a 3/3 colorless Golem artifact creature token onto the battlefield.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hallowed Burial","W","Put all creatures on the bottom of their owners' libraries.","3WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quilled Slagwurm","8","8","G",NULL,"4GGG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feral Lightning","R","Put three 3/1 red Elemental creature tokens with haste onto the battlefield. Exile them at the beginning of the next end step.","3RRR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cloud Key","As Cloud Key enters the battlefield, choose artifact, creature, enchantment, instant, or sorcery.
Spells you cast of the chosen type cost {1} less to cast.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sharpened Pitchfork","Equipped creature has first strike.
As long as equipped creature is a Human, it gets +1/+1.
Equip {1}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ant Queen","5","5","G","{1}{G}: Put a 1/1 green Insect creature token onto the battlefield.","3GG","Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sky Diamond","Sky Diamond enters the battlefield tapped.
{T}: Add {U} to your mana pool.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gauntlet of Power","As Gauntlet of Power enters the battlefield, choose a color.
Creatures of the chosen color get +1/+1.
Whenever a basic land is tapped for mana of the chosen color, its controller adds one mana of that color to his or her mana pool (in addition to the mana the land produces).","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Defiant Vanguard","2","2","W","When Defiant Vanguard blocks, at end of combat, destroy it and all creatures it blocked this turn.
{5}, {T}: Search your library for a Rebel permanent card with converted mana cost 4 or less and put it onto the battlefield. Then shuffle your library.","2W","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tribal Forcemage","1","1","G","Morph {1}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Tribal Forcemage is turned face up, creatures of the creature type of your choice get +2/+2 and gain trample until end of turn.","1G","Creature - Elf Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Natural Affinity","G","All lands become 2/2 creatures until end of turn. They're still lands.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Assault Zeppelid","3","3","UG","Flying, trample","2GU","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Animal Boneyard","W","Enchant land
Enchanted land has ""{T}, Sacrifice a creature: You gain life equal to that creature's toughness.""","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Battlesphere","4","7","When Myr Battlesphere enters the battlefield, put four 1/1 colorless Myr artifact creature tokens onto the battlefield.
Whenever Myr Battlesphere attacks, you may tap X untapped Myr you control. If you do, Myr Battlesphere gets +X/+0 until end of turn and deals X damage to defending player.","7","Artifact Creature - Myr Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Street Wraith","3","4","B","Swampwalk
Cycling-Pay 2 life. (Pay 2 life, Discard this card: Draw a card.)","3BB","Creature - Wraith");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opalescence","W","Each other non-Aura enchantment is a creature with power and toughness each equal to its converted mana cost. It's still an enchantment.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gerrard's Verdict","WB","Target player discards two cards. You gain 3 life for each land card discarded this way.","WB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aven Warcraft","W","Creatures you control get +0/+2 until end of turn.
Threshold - If seven or more cards are in your graveyard, choose a color. Creatures you control also gain protection from the chosen color until end of turn.","2W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Celestial Colonnade","Celestial Colonnade enters the battlefield tapped.
{T}: Add {W} or {U} to your mana pool.
{3}{W}{U}: Until end of turn, Celestial Colonnade becomes a 4/4 white and blue Elemental creature with flying and vigilance. It's still a land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horned Turtle","1","4","U",NULL,"2U","Creature - Turtle");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Intimidation Bolt","WR","Intimidation Bolt deals 3 damage to target creature. Other creatures can't attack this turn.","1RW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unbreathing Horde","0","0","B","Unbreathing Horde enters the battlefield with a +1/+1 counter on it for each other Zombie you control and each Zombie card in your graveyard.
If Unbreathing Horde would be dealt damage, prevent that damage and remove a +1/+1 counter from it.","2B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leshrac's Sigil","B","Whenever an opponent casts a green spell, you may pay {B}{B}. If you do, look at that player's hand and choose a card from it. The player discards that card.
{B}{B}: Return Leshrac's Sigil to its owner's hand.","BB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Urza's Science Fair Project","4","4","o2: Roll a six-sided die for Urza's Science Fair Project.
1 It gets -2/-2 until end of turn.
2 It deals no combat damage this turn.
3 Attacking does not cause it to tap this turn.
4 It gains first strike until end of turn.
5 It gains flying until end of turn.
6 It gets +2/+2 until end of turn.","6","Artifact Creature");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dead-Iron Sledge","Whenever equipped creature blocks or becomes blocked by a creature, destroy both creatures.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trapfinder's Trick","U","Target player reveals his or her hand and discards all Trap cards.","1U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Convalescent Care","W","At the beginning of your upkeep, if you have 5 or less life, you gain 3 life and draw a card.","1WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reach Through Mists","U","Draw a card.","U","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warrior's Stand","W","Cast Warrior's Stand only during the declare attackers step and only if you've been attacked this step.
Creatures you control get +2/+2 until end of turn.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Forcemage Advocate","2","1","G","{T}: Return target card from an opponent's graveyard to his or her hand. Put a +1/+1 counter on target creature.","1G","Creature - Centaur Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sparkmage Apprentice","1","1","R","When Sparkmage Apprentice enters the battlefield, it deals 1 damage to target creature or player.","1R","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Western Paladin","3","3","B","{B}{B}, {T}: Destroy target white creature.","2BB","Creature - Zombie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Lyrist","1","1","G","{G}, {T}, Sacrifice Elvish Lyrist: Destroy target enchantment.","G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darkheart Sliver","2","2","BG","All Slivers have ""Sacrifice this permanent: You gain 3 life.""","BG","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit Away","U","Enchant creature
You control enchanted creature.
Enchanted creature gets +2/+2 and has flying.","5UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of the Stars","0","1","W","Defender (This creature can't attack.)
As Order of the Stars enters the battlefield, choose a color.
Order of the Stars has protection from the chosen color.","W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Soaring Seacliff","Soaring Seacliff enters the battlefield tapped.
When Soaring Seacliff enters the battlefield, target creature gains flying until end of turn.
{T}: Add {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hearth Charm","R","Choose one - Destroy target artifact creature; or attacking creatures get +1/+0 until end of turn; or target creature with power 2 or less is unblockable this turn.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moment of Heroism","W","Target creature gets +2/+2 and gains lifelink until end of turn. (Damage dealt by the creature also causes its controller to gain that much life.)","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("War Mammoth","3","3","G","Trample","3G","Creature - Elephant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flametongue Kavu Avatar","Whenever a nontoken creature enters the battlefield under your control, that creature deals X damage to target creature, where X is a number chosen at random from 0 to 4.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gaea's Herald","1","1","G","Creature spells can't be countered.","1G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mountain Bandit","1","1","R","Haste","R","Creature - Human Soldier Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bone Saw","Equipped creature gets +1/+0.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","0","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Searing Blaze","R","Searing Blaze deals 1 damage to target player and 1 damage to target creature that player controls.
Landfall - If you had a land enter the battlefield under your control this turn, Searing Blaze deals 3 damage to that player and 3 damage to that creature instead.","RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sworn Defender","1","3","W","{1}: Sworn Defender's power becomes the toughness of target creature blocking or being blocked by Sworn Defender minus 1 until end of turn, and Sworn Defender's toughness becomes 1 plus the power of that creature until end of turn.","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pincher Beetles","3","1","G","Shroud (This creature can't be the target of spells or abilities.)","2G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Pioneer","1","1","G","When Elvish Pioneer enters the battlefield, you may put a basic land card from your hand onto the battlefield tapped.","G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Defender en-Vec","2","4","W","Fading 4 (This creature enters the battlefield with four fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Remove a fade counter from Defender en-Vec: Prevent the next 2 damage that would be dealt to target creature or player this turn.","3W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stampede","G","Attacking creatures get +1/+0 and gain trample until end of turn.","1GG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Victimize","B","Choose two target creature cards in your graveyard. Sacrifice a creature. If you do, return the chosen cards to the battlefield tapped.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firescreamer","2","2","B","{R}: Firescreamer gets +1/+0 until end of turn.","3B","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmal Sphere","0","1","U","Flying
At the beginning of your upkeep, put a +1/+1 counter on Phantasmal Sphere, then sacrifice Phantasmal Sphere unless you pay {1} for each +1/+1 counter on it.
When Phantasmal Sphere leaves the battlefield, put a blue Orb creature token with flying onto the battlefield under target opponent's control. That creature's power and toughness are each equal to the number of +1/+1 counters on Phantasmal Sphere.","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Revive the Fallen","B","Return target creature card from a graveyard to its owner's hand. Clash with an opponent. If you win, return Revive the Fallen to its owner's hand. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leviathan","10","10","U","Trample
Leviathan enters the battlefield tapped and doesn't untap during your untap step.
At the beginning of your upkeep, you may sacrifice two Islands. If you do, untap Leviathan.
Leviathan can't attack unless you sacrifice two Islands.","5UUUU","Creature - Leviathan");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kezzerdrix","4","4","B","First strike
At the beginning of your upkeep, if your opponents control no creatures, Kezzerdrix deals 4 damage to you.","2BB","Creature - Rabbit Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Taj-Nar Swordsmith","2","3","W","When Taj-Nar Swordsmith enters the battlefield, you may pay {X}. If you do, search your library for an Equipment card with converted mana cost X or less and put that card onto the battlefield. Then shuffle your library.","3W","Creature - Cat Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dispatch","W","Tap target creature.
Metalcraft - If you control three or more artifacts, exile that creature.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snowfall","U","Cumulative upkeep {U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Whenever an Island is tapped for mana, its controller may add {U} to his or her mana pool (in addition to the mana the land produces). If that Island is snow, its controller may add {U}{U} to his or her mana pool instead. Spend this mana only to pay cumulative upkeep costs.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Darksteel Sentinel","3","3","Flash (You may cast this spell any time you could cast an instant.)
Vigilance
Darksteel Sentinel is indestructible. (Lethal damage and effects that say ""destroy"" don't destroy it. If its toughness is 0 or less, it's still put into its owner's graveyard.)","6","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Contract from Below","B","Remove Contract from Below from your deck before playing if you're not playing for ante.
Discard your hand, add the top card of your library to the ante, then draw seven cards.","B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nantuko Monastery","{T}: Add {1} to your mana pool.
Threshold - {G}{W}: Nantuko Monastery becomes a 4/4 green and white Insect Monk creature with first strike until end of turn. It's still a land. Activate this ability only if seven or more cards are in your graveyard.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Luminous Angel","4","4","W","Flying
At the beginning of your upkeep, you may put a 1/1 white Spirit creature token with flying onto the battlefield.","4WWW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Touch of Death","B","Touch of Death deals 1 damage to target player. You gain 1 life.
Draw a card at the beginning of the next turn's upkeep.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghastly Discovery","U","Draw two cards, then discard a card.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it.)","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knotvine Paladin","2","2","WG","Whenever Knotvine Paladin attacks, it gets +1/+1 until end of turn for each untapped creature you control.","GW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grizzly Fate","G","Put two 2/2 green Bear creature tokens onto the battlefield.
Threshold - Put four 2/2 green Bear creature tokens onto the battlefield instead if seven or more cards are in your graveyard.
Flashback {5}{G}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blizzard Elemental","5","5","U","Flying
{3}{U}: Untap Blizzard Elemental.","5UU","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Armageddon","W","Destroy all lands.","3W","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexia's Core","{T}: Add {1} to your mana pool.
{1}, {T}, Sacrifice an artifact: You gain 1 life.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Delirium Skeins","B","Each player discards three cards.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tolsimir Wolfblood","3","4","WG","Other green creatures you control get +1/+1.
Other white creatures you control get +1/+1.
{T}: Put a legendary 2/2 green and white Wolf creature token named Voja onto the battlefield.","4GW","Legendary Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emeria Angel","3","3","W","Flying
Landfall - Whenever a land enters the battlefield under your control, you may put a 1/1 white Bird creature token with flying onto the battlefield.","2WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Melt Terrain","R","Destroy target land. Melt Terrain deals 2 damage to that land's controller.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snow Devil","U","Enchant creature
Enchanted creature has flying.
Enchanted creature has first strike as long as it's blocking and you control a snow land.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Blizzard-Herder","1","1","R","When Akki Blizzard-Herder dies, each player sacrifices a land.","1R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Amass the Components","U","Draw three cards, then put a card from your hand on the bottom of your library.","3U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Salt Flats","Salt Flats enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{T}: Add {W} or {B} to your mana pool. Salt Flats deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battlewise Aven","2","2","W","Flying
Threshold - As long as seven or more cards are in your graveyard, Battlewise Aven gets +1/+1 and has first strike.","3W","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Diabolic Machine","4","4","{3}: Regenerate Diabolic Machine.","7","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread Cacodemon","8","8","B","When Dread Cacodemon enters the battlefield, if you cast it from your hand, destroy all creatures your opponents control, then tap all other creatures you control.","7BBB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fangren Marauder","5","5","G","Whenever an artifact is put into a graveyard from the battlefield, you may gain 5 life.","5G","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("I Know All, I See All","(An ongoing scheme remains face up until it's abandoned.)
Untap all permanents you control during each opponent's untap step.
At the beginning of each end step, if three or more cards were put into your graveyard this turn from anywhere, abandon this scheme.",NULL,"Ongoing Scheme");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jinxed Idol","At the beginning of your upkeep, Jinxed Idol deals 2 damage to you.
Sacrifice a creature: Target opponent gains control of Jinxed Idol.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shatterskull Giant","4","3","R",NULL,"2RR","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crown of Awe","W","Enchant creature
Enchanted creature has protection from black and from red.
Sacrifice Crown of Awe: Enchanted creature and other creatures that share a creature type with it gain protection from black and from red until end of turn.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Look at Me, I'm R&D","W","As Look at Me, I'm R&D comes into play, choose a number and a second number one higher or one lower than that number.
All instances of the first chosen number on permanents, spells, and cards in any zone are the second chosen number.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Archmage Ascension","U","At the beginning of each end step, if you drew two or more cards this turn, you may put a quest counter on Archmage Ascension.
As long as Archmage Ascension has six or more quest counters on it, if you would draw a card, you may instead search your library for a card, put that card into your hand, then shuffle your library.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Drake","1","2","R","Flying
{R}: Fire Drake gets +1/+0 until end of turn. Activate this ability only once each turn.","1RR","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mungha Wurm","6","5","G","You can't untap more than one land during your untap step.","2GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Extruder","4","3","Echo {4} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
Sacrifice an artifact: Put a +1/+1 counter on target creature.","4","Artifact Creature - Juggernaut");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scrapyard Salvo","R","Scrapyard Salvo deals damage to target player equal to the number of artifact cards in your graveyard.","1RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Journey of Discovery","G","Choose one - Search your library for up to two basic land cards, reveal them, put them into your hand, then shuffle your library; or you may play up to two additional lands this turn.
Entwine {2}{G} (Choose both if you pay the entwine cost.)","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Strongarm Thug","1","1","B","When Strongarm Thug enters the battlefield, you may return target Mercenary card from your graveyard to your hand.","2B","Creature - Human Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ashen Powder","B","Put target creature card from an opponent's graveyard onto the battlefield under your control.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hit // Run","BR","Target player sacrifices an artifact or creature. Hit deals damage to that player equal to that permanent's converted mana cost.
---
Attacking creatures you control get +1/+0 until end of turn for each other attacking creature.","1BR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boggart Harbinger","2","1","B","When Boggart Harbinger enters the battlefield, you may search your library for a Goblin card, reveal it, then shuffle your library and put that card on top of it.","2B","Creature - Goblin Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Celestial Prism","{2}, {T}: Add one mana of any color to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Red Cliffs Armada","5","4","U","Red Cliffs Armada can't attack unless defending player controls an Island.","4U","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demoralize","R","Each creature can't be blocked this turn except by two or more creatures.
Threshold - If seven or more cards are in your graveyard, creatures can't block this turn.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frazzle","U","Counter target nonblue spell.","3U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prismatic Omen","G","Lands you control are every basic land type in addition to their other types.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cyclone","G","At the beginning of your upkeep, put a wind counter on Cyclone, then sacrifice Cyclone unless you pay {G} for each wind counter on it. If you pay, Cyclone deals damage equal to the number of wind counters on it to each creature and each player.","2GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Presence of the Wise","W","You gain 2 life for each card in your hand.","2WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunding Gjornersen","5","4","WU","Rampage 1 (Whenever this creature becomes blocked, it gets +1/+1 until end of turn for each creature blocking it beyond the first.)","3WUU","Legendary Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sidar Jabari","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
Whenever Sidar Jabari attacks, tap target creature defending player controls.","3W","Legendary Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goldmeadow Stalwart","2","2","W","As an additional cost to cast Goldmeadow Stalwart, reveal a Kithkin card from your hand or pay {3}.","W","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overgrown Estate","WBG","Sacrifice a land: You gain 3 life.","BGW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vengevine","4","3","G","Haste
Whenever you cast a spell, if it's the second creature spell you cast this turn, you may return Vengevine from your graveyard to the battlefield.","2GG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Inkshrouder","2","1","U","Discard a card: Cephalid Inkshrouder gains shroud until end of turn and is unblockable this turn. (A permanent with shroud can't be the target of spells or abilities.)","2U","Creature - Cephalid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kookus","3","5","R","Trample
At the beginning of your upkeep, if you don't control a creature named Keeper of Kookus, Kookus deals 3 damage to you and attacks this turn if able.
{R}: Kookus gets +1/+0 until end of turn.","3RR","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Revenge of the Hunted","G","Until end of turn, target creature gets +6/+6 and gains trample, and all creatures able to block it this turn do so.
Miracle {G} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","4GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marker Beetles","2","3","G","When Marker Beetles dies, target creature gets +1/+1 until end of turn.
{2}, Sacrifice Marker Beetles: Draw a card.","1GG","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Feral Shadow","2","1","B","Flying","2B","Creature - Nightstalker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decree of Justice","W","Put X 4/4 white Angel creature tokens with flying onto the battlefield.
Cycling {2}{W} ({2}{W}, Discard this card: Draw a card.)
When you cycle Decree of Justice, you may pay {X}. If you do, put X 1/1 white Soldier creature tokens onto the battlefield.","XX2WW","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("River Delta","River Delta doesn't untap during your untap step if it has a depletion counter on it.
At the beginning of your upkeep, remove a depletion counter from River Delta.
{T}: Add {U} or {B} to your mana pool. Put a depletion counter on River Delta.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brushwagg","3","2","G","Whenever Brushwagg blocks or becomes blocked, it gets -2/+2 until end of turn.","1GG","Creature - Brushwagg");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pillarfield Ox","2","4","W",NULL,"3W","Creature - Ox");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skaab Ruinator","5","6","U","As an additional cost to cast Skaab Ruinator, exile three creature cards from your graveyard.
Flying
You may cast Skaab Ruinator from your graveyard.","1UU","Creature - Zombie Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit Cairn","W","Whenever a player discards a card, you may pay {W}. If you do, put a 1/1 white Spirit creature token with flying onto the battlefield.","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kyren Archive","At the beginning of your upkeep, you may exile the top card of your library face down.
{5}, Discard your hand, Sacrifice Kyren Archive: Put all cards exiled with Kyren Archive into their owner's hand.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opal Caryatid","W","When an opponent casts a creature spell, if Opal Caryatid is an enchantment, Opal Caryatid becomes a 2/2 Soldier creature.","W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prismatic Boon","WU","Choose a color. X target creatures gain protection from the chosen color until end of turn.","XWU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Abandoned Outpost","Abandoned Outpost enters the battlefield tapped.
{T}: Add {W} to your mana pool.
{T}, Sacrifice Abandoned Outpost: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirri the Cursed","3","2","B","Flying, first strike, haste
Whenever Mirri the Cursed deals combat damage to a creature, put a +1/+1 counter on Mirri the Cursed.","2BB","Legendary Creature - Vampire Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spontaneous Generation","G","Put a 1/1 green Saproling creature token onto the battlefield for each card in your hand.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Protean Hydra","0","0","G","Protean Hydra enters the battlefield with X +1/+1 counters on it.
If damage would be dealt to Protean Hydra, prevent that damage and remove that many +1/+1 counters from it.
Whenever a +1/+1 counter is removed from Protean Hydra, put two +1/+1 counters on it at the beginning of the next end step.","XG","Creature - Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frozen Solid","U","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.
When enchanted creature is dealt damage, destroy it.","1UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voracious Cobra","2","2","RG","First strike
Whenever Voracious Cobra deals combat damage to a creature, destroy that creature.","2RG","Creature - Snake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jester's Scepter","When Jester's Scepter enters the battlefield, exile the top five cards of target player's library face down. You may look at those cards for as long as they remain exiled.
{2}, {T}, Put a card exiled with Jester's Scepter into its owner's graveyard: Counter target spell if it has the same name as that card.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Liege of the Tangle","8","8","G","Trample
Whenever Liege of the Tangle deals combat damage to a player, you may choose any number of target lands you control and put an awakening counter on each of them. Each of those lands is an 8/8 green Elemental creature for as long as it has an awakening counter on it. They're still lands.","6GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Resounding Roar","G","Target creature gets +3/+3 until end of turn.
Cycling {5}{R}{G}{W} ({5}{R}{G}{W}, Discard this card: Draw a card.)
When you cycle Resounding Roar, target creature gets +6/+6 until end of turn.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fallen Cleric","4","2","B","Protection from Clerics
Morph {4}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4B","Creature - Zombie Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enemy of the Guildpact","4","2","B","Protection from multicolored","4B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Amphin Cutthroat","2","4","U",NULL,"3U","Creature - Salamander Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fracturing Gust","WG","Destroy all artifacts and enchantments. You gain 2 life for each permanent destroyed this way.","2(G/W)(G/W)(G/W)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oppressive Will","U","Counter target spell unless its controller pays {1} for each card in your hand.","2U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Xantcha","Sacrifice a permanent: Regenerate target creature.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seedtime","G","Cast Seedtime only during your turn.
Take an extra turn after this one if an opponent cast a blue spell this turn.","1G","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Steel Golem","3","4","You can't cast creature spells.","3","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Back to Basics","U","Nonbasic lands don't untap during their controllers' untap steps.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathgreeter","1","1","B","Whenever another creature dies, you may gain 1 life.","B","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Numbing Dose","U","Enchant artifact or creature
Enchanted permanent doesn't untap during its controller's untap step.
At the beginning of the upkeep of enchanted permanent's controller, that player loses 1 life.","3UU","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wanderer's Twig","{1}, Sacrifice Wanderer's Twig: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treetop Scout","1","1","G","Treetop Scout can't be blocked except by creatures with flying.","G","Creature - Elf Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trollhide","G","Enchant creature
Enchanted creature gets +2/+2 and has ""{1}{G}: Regenerate this creature."" (The next time the creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","2G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jet Medallion","Black spells you cast cost {1} less to cast.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tooth, Claw, and Tail","When you set this scheme in motion, destroy up to three target nonland permanents.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mercenary Informer","2","1","W","Mercenary Informer can't be the target of black spells or abilities from black sources.
{2}{W}: Put target nontoken Mercenary on the bottom of its owner's library.","2W","Creature - Human Rebel Mercenary");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Goblin Warchief Avatar","Attacking creatures you control get +1/+0.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mist Leopard","3","2","G","Shroud (This creature can't be the target of spells or abilities.)","3G","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghoulraiser","2","2","B","When Ghoulraiser enters the battlefield, return a Zombie card at random from your graveyard to your hand.","1BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fortitude","G","Enchant creature
Sacrifice a Forest: Regenerate enchanted creature.
When Fortitude is put into a graveyard from the battlefield, return Fortitude to its owner's hand.","1G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hamletback Goliath","6","6","R","Whenever another creature enters the battlefield, you may put X +1/+1 counters on Hamletback Goliath, where X is that creature's power.","6R","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jokulhaups","R","Destroy all artifacts, creatures, and lands. They can't be regenerated.","4RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raze","R","As an additional cost to cast Raze, sacrifice a land.
Destroy target land.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Floodgate","0","5","U","Defender (This creature can't attack.)
When Floodgate has flying, sacrifice it.
When Floodgate leaves the battlefield, it deals damage equal to half the number of Islands you control, rounded down, to each nonblue creature without flying.","3U","Creature - Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Propagator","1","1","{3}, {T}: Put a token that's a copy of Myr Propagator onto the battlefield.","3","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harbor Guardian","3","4","WU","Reach (This creature can block creatures with flying.)
Whenever Harbor Guardian attacks, defending player may draw a card.","2WU","Creature - Gargoyle");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shrine of Loyal Legions","At the beginning of your upkeep or whenever you cast a white spell, put a charge counter on Shrine of Loyal Legions.
{3}, {T}, Sacrifice Shrine of Loyal Legions: Put a 1/1 colorless Myr artifact creature token onto the battlefield for each charge counter on Shrine of Loyal Legions.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Distant Memories","U","Search your library for a card, exile it, then shuffle your library. Any opponent may have you put that card into your hand. If no player does, you draw three cards.","2UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmal Forces","4","1","U","Flying
At the beginning of your upkeep, sacrifice Phantasmal Forces unless you pay {U}.","3U","Creature - Illusion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Volrath's Stronghold","{T}: Add {1} to your mana pool.
{1}{B}, {T}: Put target creature card from your graveyard on top of your library.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enlistment Officer","2","3","W","First strike
When Enlistment Officer enters the battlefield, reveal the top four cards of your library. Put all Soldier cards revealed this way into your hand and the rest on the bottom of your library in any order.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rotting Giant","3","3","B","Whenever Rotting Giant attacks or blocks, sacrifice it unless you exile a card from your graveyard.","1B","Creature - Zombie Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volrath's Motion Sensor","B","When Volrath's Motion Sensor comes into play, choose target hand controlled by an opponent. Enchanted player balances Volrath's Motion Sensor on the back of that hand.
If Volrath's Motion Sensor falls off the hand, sacrifice Volrath's Motion Sensor and that player loses 3 life.","B","Enchant Player");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Sandswimmer","3","2","R","{R}: Flip a coin. If you win the flip, return Viashino Sandswimmer to its owner's hand. If you lose the flip, sacrifice Viashino Sandswimmer.","2RR","Creature - Viashino");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mystic Visionary","2","1","W","Threshold - Mystic Visionary has flying as long as seven or more cards are in your graveyard.","1W","Creature - Human Nomad Mystic");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("AEther Adept","2","2","U","When Æther Adept enters the battlefield, return target creature to its owner's hand.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angelfire Crusader","2","3","W","{R}: Angelfire Crusader gets +1/+0 until end of turn.","3W","Creature - Human Soldier Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of the Hokey Pokey","2","2","W","First strike
o1oW, Do the Hokey Pokey (Stand up, wiggle your butt, raise your hands above your head, and shake them wildly as you rotate 360 degrees): Prevent all damage to Knight of the Hokey Pokey from any one source.","WW","Summon - Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quagmire Druid","2","2","B","{G}, {T}, Sacrifice a creature: Destroy target enchantment.","2B","Creature - Zombie Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squandered Resources","BG","Sacrifice a land: Add to your mana pool one mana of any type the sacrificed land could produce.","BG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragonstalker","3","3","W","Flying, protection from Dragons","4W","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hindering Light","WU","Counter target spell that targets you or a permanent you control.
Draw a card.","WU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Windreaver","1","3","WU","Flying
{W}: Windreaver gains vigilance until end of turn.
{W}: Windreaver gets +0/+1 until end of turn.
{U}: Switch Windreaver's power and toughness until end of turn.
{U}: Return Windreaver to its owner's hand.","3WU","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Victory's Herald","4","4","W","Flying
Whenever Victory's Herald attacks, attacking creatures gain flying and lifelink until end of turn.","3WWW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coastal Piracy","U","Whenever a creature you control deals combat damage to an opponent, you may draw a card.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravetiller Wurm","4","4","G","Trample
Morbid - Gravetiller Wurm enters the battlefield with four +1/+1 counters on it if a creature died this turn.","5G","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Relentless Assault","R","Untap all creatures that attacked this turn. After this main phase, there is an additional combat phase followed by an additional main phase.","2RR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Coral Helm","{3}, Discard a card at random: Target creature gets +2/+2 until end of turn.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Velis Vel","Each creature gets +1/+1 for each other creature on the battlefield that shares at least one creature type with it. (For example, if two Elemental Shamans and an Elemental Spirit are on the battlefield, each gets +2/+2.)
Whenever you roll {C}, target creature gains all creature types until end of turn.",NULL,"Plane - Lorwyn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Premature Burial","B","Destroy target nonblack creature that entered the battlefield since your last turn ended.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Misguided Rage","R","Target player sacrifices a permanent.","2R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gaea's Cradle","{T}: Add {G} to your mana pool for each creature you control.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Festercreep","0","0","B","Festercreep enters the battlefield with a +1/+1 counter on it.
{1}{B}, Remove a +1/+1 counter from Festercreep: All other creatures get -1/-1 until end of turn.","1B","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cliffrunner Behemoth","5","3","G","Cliffrunner Behemoth has haste as long as you control a red permanent.
Cliffrunner Behemoth has lifelink as long as you control a white permanent.","3G","Creature - Rhino Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Livonya Silone","4","4","RG","First strike, legendary landwalk","2RRGG","Legendary Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jhessian Zombies","2","4","UB","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Islandcycling {2}, swampcycling {2} ({2}, Discard this card: Search your library for an Island or Swamp card, reveal it, and put it into your hand. Then shuffle your library.)","4UB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master Splicer","1","1","W","When Master Splicer enters the battlefield, put a 3/3 colorless Golem artifact creature token onto the battlefield.
Golem creatures you control get +1/+1.","3W","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Poultrygeist","1","1","B","Flying
Whenever a creature is put into any graveyard from play, you may roll a six-sided die. On a 1, sacrifice Poultrygeist. Otherwise, put a +1/+1 counter on Poultrygeist.","2B","Summon - Chicken");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Summoner's Bane","U","Counter target creature spell. Put a 2/2 blue Illusion creature token onto the battlefield.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noble Elephant","2","2","W","Trample; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","3W","Creature - Elephant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inquisition","B","Target player reveals his or her hand. Inquisition deals damage to that player equal to the number of white cards in his or her hand.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cadaverous Bloom","BG","Exile a card from your hand: Add {B}{B} or {G}{G} to your mana pool.","3BG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harvest Wurm","3","2","G","When Harvest Wurm enters the battlefield, sacrifice it unless you return a basic land card from your graveyard to your hand.","1G","Creature - Wurm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grim Monolith","Grim Monolith doesn't untap during your untap step.
{T}: Add {3} to your mana pool.
{4}: Untap Grim Monolith.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hibernation","U","Return all green permanents to their owners' hands.","2U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lyzolda, the Blood Witch Avatar","Hellbent - As long as you have no cards in hand, if a source you control would deal damage to a creature or player, it deals double that damage to that creature or player instead.
Hellbent - At the beginning of your end step, if you have no cards in hand, each of your opponents discards a card.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shieldmage Advocate","1","3","W","{T}: Return target card from an opponent's graveyard to his or her hand. Prevent all damage that would be dealt to target creature or player this turn by a source of your choice.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blazing Blade Askari","2","2","R","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{2}: Blazing Blade Askari becomes colorless until end of turn.","2R","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dormant Gomazoa","5","5","U","Flying
Dormant Gomazoa enters the battlefield tapped.
Dormant Gomazoa doesn't untap during your untap step.
Whenever you become the target of a spell, you may untap Dormant Gomazoa.","1UU","Creature - Jellyfish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recoup","R","Target sorcery card in your graveyard gains flashback until end of turn. The flashback cost is equal to its mana cost. (Mana cost includes color.)
Flashback {3}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Wyvern","3","3","R","Flying
{R}: Flowstone Wyvern gets +2/-2 until end of turn.","3RR","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Lavarunner","1","1","R","Haste
Whenever Akki Lavarunner deals damage to an opponent, flip it.
---
Protection from red
If a red source would deal damage to a player, it deals that much damage plus 1 to that player instead.","3R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmal Image","0","0","U","You may have Phantasmal Image enter the battlefield as a copy of any creature on the battlefield, except it's an Illusion in addition to its other types and it gains ""When this creature becomes the target of a spell or ability, sacrifice it.""","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tainted Strike","B","Target creature gets +1/+0 and gains infect until end of turn. (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tide of War","R","Whenever one or more creatures block, flip a coin. If you win the flip, each blocking creature is sacrificed by its controller. If you lose the flip, each blocked creature is sacrificed by its controller.","4RR","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ornithopter","0","2","Flying","0","Artifact Creature - Thopter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gomazoa","0","3","U","Defender, flying
{T}: Put Gomazoa and each creature it's blocking on top of their owners' libraries, then those players shuffle their libraries.","2U","Creature - Jellyfish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karplusan Minotaur","3","3","R","Cumulative upkeep-Flip a coin. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Whenever you win a coin flip, Karplusan Minotaur deals 1 damage to target creature or player.
Whenever you lose a coin flip, Karplusan Minotaur deals 1 damage to target creature or player of an opponent's choice.","2RR","Creature - Minotaur Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angus Mackenzie","2","2","WUG","{G}{W}{U}, {T}: Prevent all combat damage that would be dealt this turn. Activate this ability only before the combat damage step.","GWU","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Galvanic Alchemist","1","4","U","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Galvanic Alchemist is paired with another creature, each of those creatures has ""{2}{U}: Untap this creature.""","2U","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skull of Ramos","{T}: Add {B} to your mana pool.
Sacrifice Skull of Ramos: Add {B} to your mana pool.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bargaining Table","{X}, {T}: Draw a card. X is the number of cards in an opponent's hand.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Norn's Annex","W","({(w/p)} can be paid with either {W} or 2 life.)
Creatures can't attack you or a planeswalker you control unless their controller pays {(w/p)} for each of those creatures.","3(W/P)(W/P)","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Brass Gnat","1","1","Flying
Brass Gnat doesn't untap during your untap step.
At the beginning of your upkeep, you may pay {1}. If you do, untap Brass Gnat.","1","Artifact Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Controvert","U","Counter target spell.
Recover {2}{U}{U} (When a creature is put into your graveyard from the battlefield, you may pay {2}{U}{U}. If you do, return this card from your graveyard to your hand. Otherwise, exile this card.)","2UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frantic Purification","W","Destroy target enchantment.
Madness {W} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ulvenwald Bear","2","2","G","Morbid - When Ulvenwald Bear enters the battlefield, if a creature died this turn, put two +1/+1 counters on target creature.","2G","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ixidron","*","*","U","As Ixidron enters the battlefield, turn all other nontoken creatures face down. (They're 2/2 creatures.)
Ixidron's power and toughness are each equal to the number of face-down creatures on the battlefield.","3UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rotting Fensnake","5","1","B",NULL,"3B","Creature - Zombie Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Backdraft","R","Choose a player who cast one or more sorcery spells this turn. Backdraft deals damage to that player equal to half the damage dealt by one of those sorcery spells this turn, rounded down.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plated Slagwurm","8","8","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)","4GGG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Long-Term Plans","U","Search your library for a card, shuffle your library, then put that card third from the top.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lush Growth","G","Enchant land
Enchanted land is a Mountain, Forest, and Plains.","G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mightstone","Attacking creatures get +1/+0.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stone Rain","R","Destroy target land.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bladetusk Boar","3","2","R","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)","3R","Creature - Boar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cursed Flesh","B","Enchant creature
Enchanted creature gets -1/-1 and has fear. (It can't be blocked except by artifact creatures and/or black creatures.)","B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scrying Sheets","{T}: Add {1} to your mana pool.
{1}{S}i}, {T}: Look at the top card of your library. If that card is snow, you may reveal it and put it into your hand. ({S}i} can be paid with one mana from a snow permanent.)",NULL,"Snow Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood of the Martyr","W","Until end of turn, if damage would be dealt to any creature, you may have that damage dealt to you instead.","WWW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Castle","W","Untapped creatures you control get +0/+2.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dizzying Gaze","R","Enchant creature you control
{R}: Enchanted creature deals 1 damage to target creature with flying.","R","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Torpor Orb","Creatures entering the battlefield don't cause abilities to trigger.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Greven il-Vec","Whenever a creature you control deals damage to a creature, destroy the other creature. It can't be regenerated.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argothian Swine","3","3","G","Trample","3G","Creature - Boar");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Titania","You may play an additional land on each of your turns.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Broodwarden","4","4","G","Eldrazi Spawn creatures you control get +2/+1.","3GG","Creature - Eldrazi Drone");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forsaken Wastes","B","Players can't gain life.
At the beginning of each player's upkeep, that player loses 1 life.
Whenever Forsaken Wastes becomes the target of a spell, that spell's controller loses 5 life.","2B","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Searing Flesh","R","Searing Flesh deals 7 damage to target opponent.","6R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Accumulated Knowledge","U","Draw a card, then draw cards equal to the number of cards named Accumulated Knowledge in all graveyards.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Defender of Chaos","2","1","R","Flash
Protection from white","2R","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Voyager Staff","{2}, Sacrifice Voyager Staff: Exile target creature. Return the exiled card to the battlefield under its owner's control at the beginning of the next end step.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disintegrate","R","Disintegrate deals X damage to target creature or player. That creature can't be regenerated this turn. If the creature would die this turn, exile it instead.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boreal Griffin","3","2","W","Flying
{S}i}: Boreal Griffin gains first strike until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","3WW","Snow Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mire's Toll","B","Target player reveals a number of cards from his or her hand equal to the number of Swamps you control. You choose one of them. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Titania's Song","G","Each noncreature artifact loses all abilities and becomes an artifact creature with power and toughness each equal to its converted mana cost. If Titania's Song leaves the battlefield, this effect continues until end of turn.","3G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Subdue","G","Prevent all combat damage that would be dealt by target creature this turn. That creature gets +0/+X until end of turn, where X is its converted mana cost.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Demon of Death's Gate","9","9","B","You may pay 6 life and sacrifice three black creatures rather than pay Demon of Death's Gate's mana cost.
Flying, trample","6BBB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tectonic Break","R","Each player sacrifices X lands.","XRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minamo Scrollkeeper","2","3","U","Defender (This creature can't attack.)
Your maximum hand size is increased by one.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirror-Sigil Sergeant","4","4","W","Trample
At the beginning of your upkeep, if you control a blue permanent, you may put a token that's a copy of Mirror-Sigil Sergeant onto the battlefield.","5W","Creature - Rhino Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Increasing Confusion","U","Target player puts the top X cards of his or her library into his or her graveyard. If Increasing Confusion was cast from a graveyard, that player puts twice that many cards into his or her graveyard instead.
Flashback {X}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","XU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serpent Assassin","2","2","B","When Serpent Assassin enters the battlefield, you may destroy target nonblack creature.","3BB","Creature - Snake Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flame Javelin","R","({(2/r)} can be paid with any two mana or with {R}. This card's converted mana cost is 6.)
Flame Javelin deals 4 damage to target creature or player.","(2/R)(2/R)(2/R)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Abduction","U","Enchant creature
When Abduction enters the battlefield, untap enchanted creature.
You control enchanted creature.
When enchanted creature dies, return that card to the battlefield under its owner's control.","2UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Windbrisk Raptor","5","7","W","Flying
Attacking creatures you control have lifelink.","5WW","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wielding the Green Dragon","G","Target creature gets +4/+4 until end of turn.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Withering Hex","B","Enchant creature
Whenever a player cycles a card, put a plague counter on Withering Hex.
Enchanted creature gets -1/-1 for each plague counter on Withering Hex.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Living Wall","0","6","Defender (This creature can't attack.)
{1}: Regenerate Living Wall.","4","Artifact Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ebony Charm","B","Choose one - Target opponent loses 1 life and you gain 1 life; or exile up to three target cards from a single graveyard; or target creature gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spidery Grasp","G","Untap target creature. It gets +2/+4 and gains reach until end of turn. (It can block creatures with flying.)","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ember Shot","R","Ember Shot deals 3 damage to target creature or player.
Draw a card.","6R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temper","W","Prevent the next X damage that would be dealt to target creature this turn. For each 1 damage prevented this way, put a +1/+1 counter on that creature.","X1W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Alloy Golem","4","4","As Alloy Golem enters the battlefield, choose a color.
Alloy Golem is the chosen color. (It's still an artifact.)","6","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arcum's Whistle","{3}, {T}: Choose target non-Wall creature the active player has controlled continuously since the beginning of the turn. That player may pay {X}, where X is that creature's converted mana cost. If he or she doesn't, the creature attacks this turn if able, and at the beginning of the next end step, destroy it if it didn't attack. Activate this ability only before attackers are declared.","3","Artifact");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Sarkhan Vol","RG","+1: Creatures you control get +1/+1 and gain haste until end of turn.
-2: Gain control of target creature until end of turn. Untap that creature. It gains haste until end of turn.
-6: Put five 4/4 red Dragon creature tokens with flying onto the battlefield.","Planeswalker - Sarkhan","2RG");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Centaur Courser","3","3","G",NULL,"2G","Creature - Centaur Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinder Crawler","1","2","R","{R}: Cinder Crawler gets +1/+0 until end of turn. Activate this ability only if Cinder Crawler is blocked.","1R","Creature - Salamander");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("My Genius Knows No Bounds","When you set this scheme in motion, you may pay {X}. If you do, you gain X life and draw X cards.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nath's Elite","4","2","G","All creatures able to block Nath's Elite do so.
When Nath's Elite enters the battlefield, clash with an opponent. If you win, put a +1/+1 counter on Nath's Elite. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","4G","Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chaos Warp","R","The owner of target permanent shuffles it into his or her library, then reveals the top card of his or her library. If it's a permanent card, he or she puts it onto the battlefield.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soldier of Fortune","1","1","R","{R}, {T}: Target player shuffles his or her library.","R","Creature - Human Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prophecy","W","Reveal the top card of target opponent's library. If it's a land, you gain 1 life. Then that player shuffles his or her library.
Draw a card at the beginning of the next turn's upkeep.","W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Encroach","B","Target player reveals his or her hand. You choose a nonbasic land card from it. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Golgari Thug","1","1","B","When Golgari Thug dies, put target creature card from your graveyard on top of your library.
Dredge 4 (If you would draw a card, instead you may put exactly four cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","1B","Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Witch's Mist","B","{2}{B}, {T}: Destroy target creature that was dealt damage this turn.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Arsonist","3","3","R","When Ogre Arsonist enters the battlefield, destroy target land.","4R","Creature - Ogre");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Flowstone Thopter","4","4","{1}: Flowstone Thopter gets +1/-1 and gains flying until end of turn.","7","Artifact Creature - Thopter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Uba Mask","If a player would draw a card, that player exiles that card face up instead.
Each player may play cards he or she exiled with Uba Mask this turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gaea's Avenger","1+*","1+*","G","Gaea's Avenger's power and toughness are each equal to 1 plus the number of artifacts your opponents control.","1GG","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ophidian Eye","U","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Whenever enchanted creature deals damage to an opponent, you may draw a card.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Cultist","1","1","BR","{T}: Blood Cultist deals 1 damage to target creature.
Whenever a creature dealt damage by Blood Cultist this turn dies, put a +1/+1 counter on Blood Cultist.","1BR","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quiet Speculation","U","Search target player's library for up to three cards with flashback and put them into that player's graveyard. Then the player shuffles his or her library.","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Barbarian","2","2","RG","Protection from blue","RG","Creature - Elf Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shah of Naar Isle","6","6","R","Trample
Echo {0} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Shah of Naar Isle's echo cost is paid, each opponent may draw up to three cards.","3R","Creature - Efreet");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodfall Primus","6","6","G","Trample
When Woodfall Primus enters the battlefield, destroy target noncreature permanent.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","5GGG","Creature - Treefolk Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Door of Destinies","As Door of Destinies enters the battlefield, choose a creature type.
Whenever you cast a spell of the chosen type, put a charge counter on Door of Destinies.
Creatures you control of the chosen type get +1/+1 for each charge counter on Door of Destinies.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Herald of Serra","3","4","W","Flying, vigilance
Echo {2}{W}{W} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","2WW","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Quicksand","{T}: Add {1} to your mana pool.
{T}, Sacrifice Quicksand: Target attacking creature without flying gets -1/-2 until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Great Sable Stag","3","3","G","Great Sable Stag can't be countered.
Protection from blue and from black (This creature can't be blocked, targeted, dealt damage, or enchanted by anything blue or black.)","1GG","Creature - Elk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Warlord's Axe","Equipped creature gets +3/+1.
Equip {4} ({4}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lumithread Field","W","Creatures you control get +0/+1.
Morph {1}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("About Face","R","Switch target creature's power and toughness until end of turn.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ravaging Horde","3","3","R","When Ravaging Horde enters the battlefield, destroy target land.","3RR","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wellspring","WG","Enchant land
When Wellspring enters the battlefield, gain control of enchanted land until end of turn.
At the beginning of your upkeep, untap enchanted land. You gain control of that land until end of turn.","1GW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Desecration Elemental","8","8","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Whenever a player casts a spell, sacrifice a creature.","3B","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bonehoard","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +X/+X, where X is the number of creature cards in all graveyards.
Equip {2}","4","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eldrazi Temple","{T}: Add {1} to your mana pool.
{T}: Add {2} to your mana pool. Spend this mana only to cast colorless Eldrazi spells or activate abilities of colorless Eldrazi.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time Ebb","U","Put target creature on top of its owner's library.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vhati il-Dal","3","3","BG","{T}: Target creature's power or toughness becomes 1 until end of turn.","2BG","Legendary Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deprive","U","As an additional cost to cast Deprive, return a land you control to its owner's hand.
Counter target spell.","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Berserker","1","1","R","Whenever Dwarven Berserker becomes blocked, it gets +3/+0 and gains trample until end of turn.","1R","Creature - Dwarf Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lu Bu, Master-at-Arms","4","3","R","Haste; horsemanship (This creature can't be blocked except by creatures with horsemanship.)","5R","Legendary Creature - Human Soldier Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Verdeloth the Ancient","4","7","G","Kicker {X} (You may pay an additional {X} as you cast this spell.)
Saproling creatures and other Treefolk creatures get +1/+1.
When Verdeloth the Ancient enters the battlefield, if it was kicked, put X 1/1 green Saproling creature tokens onto the battlefield.","4GG","Legendary Creature - Treefolk");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Corpse Cur","2","2","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
When Corpse Cur enters the battlefield, you may return target creature card with infect from your graveyard to your hand.","4","Artifact Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blessings of Nature","G","Distribute four +1/+1 counters among any number of target creatures.
Miracle {G} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","4G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Winter Sky","R","Flip a coin. If you win the flip, Winter Sky deals 1 damage to each creature and each player. If you lose the flip, each player draws a card.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soratami Savant","2","2","U","Flying
{3}, Return a land you control to its owner's hand: Counter target spell unless its controller pays {3}.","2UU","Creature - Moonfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crawling Filth","2","2","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Soulshift 5 (When this creature dies, you may return target Spirit card with converted mana cost 5 or less from your graveyard to your hand.)","5B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Badger","2","2","G","Whenever Giant Badger blocks, it gets +2/+2 until end of turn.","1GG","Creature - Badger");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fangren Firstborn","4","2","G","Whenever Fangren Firstborn attacks, put a +1/+1 counter on each attacking creature.","1GGG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pistus Strike","G","Destroy target creature with flying. Its controller gets a poison counter.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Trooper","1","1","W","Flying
{2}{W}, Discard a card: Aven Trooper gets +1/+2 until end of turn.","3W","Creature - Bird Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirror Universe","{T}, Sacrifice Mirror Universe: Exchange life totals with target opponent. Activate this ability only during your upkeep.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shoreline Ranger","3","4","U","Flying
Islandcycling {2} ({2}, Discard this card: Search your library for an Island card, reveal it, and put it into your hand. Then shuffle your library.)","5U","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aura Thief","2","2","U","Flying
When Aura Thief dies, you gain control of all enchantments. (You don't get to move Auras.)","3U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prowling Pangolin","6","5","B","When Prowling Pangolin enters the battlefield, any player may sacrifice two creatures. If a player does, sacrifice Prowling Pangolin.","3BB","Creature - Anteater Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sigil of the Nayan Gods","WG","Enchant creature
Enchanted creature gets +1/+1 for each creature you control.
Cycling {(g/w)} ({(g/w)}, Discard this card: Draw a card.)","1GW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mondronen Shaman","3","2","R","At the beginning of each upkeep, if no spells were cast last turn, transform Mondronen Shaman.","3R","Creature - Human Werewolf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arcades Sabboth","7","7","WUG","Flying
At the beginning of your upkeep, sacrifice Arcades Sabboth unless you pay {G}{W}{U}.
Each untapped creature you control gets +0/+2 as long as it's not attacking.
{W}: Arcades Sabboth gets +0/+1 until end of turn.","2GGWWUU","Legendary Creature - Elder Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elbrus, the Binding Blade","Equipped creature gets +1/+0.
When equipped creature deals combat damage to a player, unattach Elbrus, the Binding Blade, then transform it.
Equip {1}","7","Legendary Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ethercaste Knight","1","3","WU","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","WU","Artifact Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nosy Goblin","2","1","R","{T}, Sacrifice Nosy Goblin: Destroy target face-down creature.","2R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of Whiteclay","1","4","W","{1}{W}{W}, {Q}: Return target creature card with converted mana cost 3 or less from your graveyard to the battlefield. ({Q} is the untap symbol.)","1WW","Creature - Kithkin Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mind Stone","{T}: Add {1} to your mana pool.
{1}, {T}, Sacrifice Mind Stone: Draw a card.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ancient Hydra","5","1","R","Fading 5 (This creature enters the battlefield with five fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
{1}, Remove a fade counter from Ancient Hydra: Ancient Hydra deals 1 damage to target creature or player.","4R","Creature - Hydra");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caller of the Hunt","*","*","G","As an additional cost to cast Caller of the Hunt, choose a creature type.
Caller of the Hunt's power and toughness are each equal to the number of creatures of the chosen type on the battlefield.","2G","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infectious Host","1","1","B","When Infectious Host dies, target player loses 2 life.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Avatar","*","*","W","Serra Avatar's power and toughness are each equal to your life total.
When Serra Avatar is put into a graveyard from anywhere, shuffle it into its owner's library.","4WWW","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vampiric Feast","B","Vampiric Feast deals 4 damage to target creature or player and you gain 4 life.","5BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("The Wretched","2","5","B","At end of combat, gain control of all creatures blocking The Wretched for as long as you control The Wretched.","3BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning of Xinye","R","Choose four lands you control and destroy those lands. Then target opponent chooses four lands he or she controls. Destroy those lands. Then Burning of Xinye deals 4 damage to each creature.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diregraf Escort","1","1","G","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Diregraf Escort is paired with another creature, both creatures have protection from Zombies.","G","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Orcish Oriflamme","R","Attacking creatures you control get +1/+0.","3R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Illusion // Reality","U","Target spell or permanent becomes the color of your choice until end of turn.
---
Destroy target artifact.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Kiss","G","Enchant creature
{1}, Exile the top card of your graveyard: Enchanted creature gets +1/+1 until end of turn.","1G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spellweaver Helix","Imprint - When Spellweaver Helix enters the battlefield, you may exile two target sorcery cards from a single graveyard.
Whenever a player casts a card, if it has the same name as one of the cards exiled with Spellweaver Helix, you may copy the other. If you do, you may cast the copy without paying its mana cost.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Second Chance","U","At the beginning of your upkeep, if you have 5 or less life, sacrifice Second Chance and take an extra turn after this one.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mark of Sakiko","G","Enchant creature
Enchanted creature has ""Whenever this creature deals combat damage to a player, add that much {G} to your mana pool. Until end of turn, this mana doesn't empty from your mana pool as steps and phases end.""","1G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Volrath's Laboratory","As Volrath's Laboratory enters the battlefield, choose a color and a creature type.
{5}, {T}: Put a 2/2 creature token of the chosen color and type onto the battlefield.","5","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Panopticon","When you planeswalk to Panopticon, draw a card.
At the beginning of your draw step, draw an additional card.
Whenever you roll {C}, draw a card.",NULL,"Plane - Mirrodin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quiet Purity","W","Destroy target enchantment.","W","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Gibbons","G","When an opponent casts an instant spell, if Hidden Gibbons is an enchantment, Hidden Gibbons becomes a 4/4 Ape creature.","G","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Pili-Pala","1","1","Flying
{2}, {Q}: Add one mana of any color to your mana pool. ({Q} is the untap symbol.)","2","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scaled Wurm","7","6","G",NULL,"7G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gilder Bairn","1","3","UG","{2}{(g/u)}, {Q}: For each counter on target permanent, put another of those counters on that permanent. ({Q} is the untap symbol.)","1(G/U)(G/U)","Creature - Ouphe");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blatant Thievery","U","For each opponent, gain control of target permanent that player controls. (This effect lasts indefinitely.)","4UUU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plumeveil","4","4","WU","Flash (You may cast this spell any time you could cast an instant.)
Defender, flying","(W/U)(W/U)(W/U)","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Riftwalker","2","1","W","Protection from Spirits and from Arcane","1WW","Creature - Fox Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mindbreak Trap","U","If an opponent cast three or more spells this turn, you may pay {0} rather than pay Mindbreak Trap's mana cost.
Exile any number of target spells.","2UU","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diligent Farmhand","1","1","G","{1}{G}, Sacrifice Diligent Farmhand: Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.
If Diligent Farmhand is in a graveyard, effects from spells named Muscle Burst count it as a card named Muscle Burst.","G","Creature - Human Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Umbilicus","At the beginning of each player's upkeep, that player returns a permanent he or she controls to its owner's hand unless he or she pays 2 life.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Power Armor","Domain - {3}, {T}: Target creature gets +1/+1 until end of turn for each basic land type among lands you control.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ring of Immortals","{3}, {T}: Counter target instant or Aura spell that targets a permanent you control.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Root Cage","G","Mercenaries don't untap during their controllers' untap steps.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Compost","G","Whenever a black card is put into an opponent's graveyard from anywhere, you may draw a card.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ronin Houndmaster","2","2","R","Haste
Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","2R","Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whiteout","G","All creatures lose flying until end of turn.
Sacrifice a snow land: Return Whiteout from your graveyard to your hand.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Michiko Konda, Truth Seeker","2","2","W","Whenever a source an opponent controls deals damage to you, that player sacrifices a permanent.","3W","Legendary Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelic Shield","WU","Creatures you control get +0/+1.
Sacrifice Angelic Shield: Return target creature to its owner's hand.","WU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Ogre","2","2","R","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
First strike (This creature deals combat damage before creatures without first strike.)","2R","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spindrift Drake","2","1","U","Flying
At the beginning of your upkeep, sacrifice Spindrift Drake unless you pay {U}.","U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Graft","U","Gain control of target Aura that's attached to a permanent. Attach it to another permanent it can enchant.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steelshaper Apprentice","1","3","W","{W}, {T}, Return Steelshaper Apprentice to its owner's hand: Search your library for an Equipment card, reveal that card, and put it into your hand. Then shuffle your library.","2WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Reinforced Bulwark","0","4","Defender
{T}: Prevent the next 1 damage that would be dealt to you this turn.","3","Artifact Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampiric Spirit","4","3","B","Flying
When Vampiric Spirit enters the battlefield, you lose 4 life.","2BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Priest of Urabrask","2","1","R","When Priest of Urabrask enters the battlefield, add {R}{R}{R} to your mana pool.","2R","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pestilence Rats","*","3","B","Pestilence Rats's power is equal to the number of other Rats on the battlefield. (For example, as long as there are two other Rats on the battlefield, Pestilence Rats's power and toughness are 2/3.)","2B","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Creeping Corrosion","G","Destroy all artifacts.","2GG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nivix, Aerie of the Firemind","{T}: Add {1} to your mana pool.
{2}{U}{R}, {T}: Exile the top card of your library. Until your next turn, you may cast that card if it's an instant or sorcery.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Temporal Aperture","{5}, {T}: Shuffle your library, then reveal the top card. Until end of turn, for as long as that card remains on top of your library, play with the top card of your library revealed and you may play that card without paying its mana cost. (If it has X in its mana cost, X is 0.)","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Putrefaction","B","Whenever a player casts a green or white spell, that player discards a card.","4B","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lurebound Scarecrow","4","4","As Lurebound Scarecrow enters the battlefield, choose a color.
When you control no permanents of the chosen color, sacrifice Lurebound Scarecrow.","3","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Steel Wall","0","4","Defender (This creature can't attack.)","1","Artifact Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Martial Coup","W","Put X 1/1 white Soldier creature tokens onto the battlefield. If X is 5 or more, destroy all other creatures.","XWW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Groundshaker","6","6","G","{G}: Target Beast creature gains trample until end of turn.","4GGG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Megatog","3","4","R","Sacrifice an artifact: Megatog gets +3/+3 and gains trample until end of turn.","4RR","Creature - Atog");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Tatterkite","2","1","Flying
Tatterkite can't have counters placed on it.","3","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Shrine","R","Enchant land
As long as enchanted land is a basic Mountain, Goblin creatures get +1/+0.
When Goblin Shrine leaves the battlefield, it deals 1 damage to each Goblin creature.","1RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Turncoat","2","1","B","Sacrifice a Goblin: Regenerate Goblin Turncoat.","1B","Creature - Goblin Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brutal Suppression","R","Activated abilities of nontoken Rebels cost an additional ""Sacrifice a land"" to activate.","R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Advance Scout","1","1","W","First strike
{W}: Target creature gains first strike until end of turn.","1W","Creature - Human Soldier Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning Oil","R","Burning Oil deals 3 damage to target attacking or blocking creature.
Flashback {3}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Akroma's Blessing","W","Choose a color. Creatures you control gain protection from the chosen color until end of turn.
Cycling {W} ({W}, Discard this card: Draw a card.)","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disharmony","R","Cast Disharmony only during combat before blockers are declared.
Untap target attacking creature and remove it from combat. Gain control of that creature until end of turn.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bartel Runeaxe","6","5","BRG","Vigilance
Bartel Runeaxe can't be the target of Aura spells.","3BRG","Legendary Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of Lunacy","4","1","B","Flying
Soulshift 5 (When this creature dies, you may return target Spirit card with converted mana cost 5 or less from your graveyard to your hand.)","4BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marhault Elsdragon","4","6","RG","Rampage 1 (Whenever this creature becomes blocked, it gets +1/+1 until end of turn for each creature blocking it beyond the first.)","3RRG","Legendary Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elephant Guide","G","Enchant creature
Enchanted creature gets +3/+3.
When enchanted creature dies, put a 3/3 green Elephant creature token onto the battlefield.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Outlander","2","2","WU","Protection from red","WU","Artifact Creature - Vedalken Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Kiss","B","Enchant creature
{B}, Pay 1 life: Enchanted creature gets +2/+2 until end of turn. Activate this ability no more than three times each turn.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Minds Aglow","U","Join forces - Starting with you, each player may pay any amount of mana. Each player draws X cards, where X is the total amount of mana paid this way.","U","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Mantis Engine","3","3","{2}: Mantis Engine gains flying until end of turn. (It can't be blocked except by creatures with flying or reach.)
{2}: Mantis Engine gains first strike until end of turn. (It deals combat damage before creatures without first strike.)","5","Artifact Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Snow-Covered Plains","W",NULL,"Basic Snow Land - Plains");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Cloud","R","Whenever a player casts a red spell, you may pay {R}. If you do, Lightning Cloud deals 1 damage to target creature or player.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veteran Armorsmith","2","3","W","Other Soldier creatures you control get +0/+1.","WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moriok Scavenger","2","3","B","When Moriok Scavenger enters the battlefield, you may return target artifact creature card from your graveyard to your hand.","3B","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Spray","R","Volcanic Spray deals 1 damage to each creature without flying and each player.
Flashback {1}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Delif's Cone","{T}, Sacrifice Delif's Cone: This turn, when target creature you control attacks and isn't blocked, you may gain life equal to its power. If you do, it assigns no combat damage this turn.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lim-Dûl's Cohort","2","3","B","Whenever Lim-Dûl's Cohort blocks or becomes blocked by a creature, that creature can't be regenerated this turn.","1BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decaying Soil","B","At the beginning of your upkeep, exile a card from your graveyard.
Threshold - As long as seven or more cards are in your graveyard, Decaying Soil has ""Whenever a nontoken creature is put into your graveyard from the battlefield, you may pay {1}. If you do, return that card to your hand.""","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravelgill Axeshark","3","3","UB","Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","4(U/B)","Creature - Merfolk Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mirror Sheen","UR","{1}{(u/r){(u/r)}: Copy target instant or sorcery spell that targets you. You may choose new targets for the copy.","1(U/R)(U/R)","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Geosurge","R","Add {R}{R}{R}{R}{R}{R}{R} to your mana pool. Spend this mana only to cast artifact or creature spells.","RRRR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sol Ring","{T}: Add {2} to your mana pool.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Mangler","0","3","U","{1}{U}: Change Riptide Mangler's power to target creature's power. (This effect lasts indefinitely.)","1U","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cranial Plating","Equipped creature gets +1/+0 for each artifact you control.
{B}{B}: Attach Cranial Plating to target creature you control.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Avenger","3","1","G","Trample
Threshold - {1}{G}: Regenerate Krosan Avenger. Activate this ability only if seven or more cards are in your graveyard.","2G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planar Chaos","R","At the beginning of your upkeep, flip a coin. If you lose the flip, sacrifice Planar Chaos.
Whenever a player casts a spell, that player flips a coin. If he or she loses the flip, counter that spell.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Channeler","2","2","R","{1}{R}, {T}, Discard a card: Target creature gets +1/-1 and gains haste until end of turn.","2R","Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chain of Smog","B","Target player discards two cards. That player may copy this spell and may choose a new target for that copy.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Intuition","U","Search your library for any three cards and reveal them. Target opponent chooses one. Put that card into your hand and the rest into your graveyard. Then shuffle your library.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wordmail","W","Enchanted creature gets +1/+1 for each word in its name.","W","Enchant Creature");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crown of Flames","R","Enchant creature
{R}: Enchanted creature gets +1/+0 until end of turn.
{R}: Return Crown of Flames to its owner's hand.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Standardize","U","Choose a creature type other than Wall. Each creature becomes that type until end of turn.","UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaea's Touch","G","You may play an additional land during your turn if that land is a basic Forest.
Sacrifice Gaea's Touch: Add {G}{G} to your mana pool.","GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hammerheim Deadeye","3","3","R","Echo {5}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Hammerheim Deadeye enters the battlefield, destroy target creature with flying.","3R","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flow of Ideas","U","Draw a card for each Island you control.","5U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wildfield Borderpost","WG","You may pay {1} and return a basic land you control to its owner's hand rather than pay Wildfield Borderpost's mana cost.
Wildfield Borderpost enters the battlefield tapped.
{T}: Add {G} or {W} to your mana pool.","1GW","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disrupt","U","Counter target instant or sorcery spell unless its controller pays {1}.
Draw a card.","U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Snow-Covered Mountain","R",NULL,"Basic Snow Land - Mountain");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blowfly Infestation","B","Whenever a creature dies, if it had a -1/-1 counter on it, put a -1/-1 counter on target creature.","2B","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mouth of Ronom","{T}: Add {1} to your mana pool.
{4}{S}i}, {T}, Sacrifice Mouth of Ronom: Mouth of Ronom deals 4 damage to target creature. ({S}i} can be paid with one mana from a snow permanent.)",NULL,"Snow Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Homicidal Brute","5","1","At the beginning of your end step, if Homicidal Brute didn't attack this turn, tap Homicidal Brute, then transform it.",NULL,"Creature - Human Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Heretic","1","1","UG","Whenever Vedalken Heretic deals damage to an opponent, you may draw a card.","GU","Creature - Vedalken Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Farsight Mask","Whenever a source an opponent controls deals damage to you, if Farsight Mask is untapped, you may draw a card.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul's Majesty","G","Draw cards equal to the power of target creature you control.","4G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boneshard Slasher","1","1","B","Flying
Threshold - As long as seven or more cards are in your graveyard, Boneshard Slasher gets +2/+2 and has ""When Boneshard Slasher becomes the target of a spell or ability, sacrifice it.""","1B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hallowed Ground","W","{W}{W}: Return target nonsnow land you control to its owner's hand.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tormented Soul","1","1","B","Tormented Soul can't block and is unblockable.","B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vine Kami","4","4","G","Vine Kami can't be blocked except by two or more creatures.
Soulshift 6 (When this creature dies, you may return target Spirit card with converted mana cost 6 or less from your graveyard to your hand.)","6G","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scroll of Avacyn","{1}, Sacrifice Scroll of Avacyn: Draw a card. If you control an Angel, you gain 5 life.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sylvan Ranger","1","1","G","When Sylvan Ranger enters the battlefield, you may search your library for a basic land card, reveal it, put it into your hand, then shuffle your library.","1G","Creature - Elf Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tibor and Lumia","3","3","UR","Whenever you cast a blue spell, target creature gains flying until end of turn.
Whenever you cast a red spell, Tibor and Lumia deals 1 damage to each creature without flying.","2UR","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cytoplast Root-Kin","0","0","G","Graft 4 (This creature enters the battlefield with four +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
When Cytoplast Root-Kin enters the battlefield, put a +1/+1 counter on each other creature you control that has a +1/+1 counter on it.
{2}: Move a +1/+1 counter from target creature you control onto Cytoplast Root-Kin.","2GG","Creature - Elemental Mutant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mental Agony","B","Target player discards two cards and loses 2 life.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Iron-Heart Chimera","2","2","Vigilance
Sacrifice Iron-Heart Chimera: Put a +2/+2 counter on target Chimera creature. It gains vigilance. (This effect lasts indefinitely.)","4","Artifact Creature - Chimera");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mad Prophet","2","2","R","Haste
{T}, Discard a card: Draw a card.","3R","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Edge of Autumn","G","If you control four or fewer lands, search your library for a basic land card, put it onto the battlefield tapped, then shuffle your library.
Cycling-Sacrifice a land. (Sacrifice a land, Discard this card: Draw a card.)","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spin into Myth","U","Put target creature on top of its owner's library, then fateseal 2. (To fateseal 2, look at the top two cards of an opponent's library, then put any number of them on the bottom of that player's library and the rest on top in any order.)","4U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leeching Bite","G","Target creature gets +1/+1 until end of turn. Another target creature gets -1/-1 until end of turn.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coal Stoker","3","3","R","When Coal Stoker enters the battlefield, if you cast it from your hand, add {R}{R}{R} to your mana pool.","3R","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of the Ages","Sword of the Ages enters the battlefield tapped.
{T}, Sacrifice Sword of the Ages and any number of creatures you control: Sword of the Ages deals X damage to target creature or player, where X is the total power of the creatures sacrificed this way, then exile Sword of the Ages and those creature cards.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elkin Lair","R","At the beginning of each player's upkeep, that player exiles a card at random from his or her hand. The player may play that card this turn. At the beginning of the next end step, if the player hasn't played the card, he or she puts it into his or her graveyard.","3R","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Experiment Kraj","4","6","UG","Experiment Kraj has all activated abilities of each other creature with a +1/+1 counter on it.
{T}: Put a +1/+1 counter on target creature.","2GGUU","Legendary Creature - Ooze Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grand Architect","1","3","U","Other blue creatures you control get +1/+1.
{U}: Target artifact creature becomes blue until end of turn.
Tap an untapped blue creature you control: Add {2} to your mana pool. Spend this mana only to cast artifact spells or activate abilities of artifacts.","1UU","Creature - Vedalken Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nettling Imp","1","1","B","{T}: Choose target non-Wall creature the active player has controlled continuously since the beginning of the turn. That creature attacks this turn if able. If it doesn't, destroy it at the beginning of the next end step. Activate this ability only during an opponent's turn, before attackers are declared.","2B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moon Sprite","1","1","G","Flying","1G","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bala Ged Thief","2","2","B","Whenever Bala Ged Thief or another Ally enters the battlefield under your control, target player reveals a number of cards from his or her hand equal to the number of Allies you control. You choose one of them. That player discards that card.","3B","Creature - Human Rogue Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tragic Slip","B","Target creature gets -1/-1 until end of turn.
Morbid - That creature gets -13/-13 until end of turn instead if a creature died this turn.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bound in Silence","W","Enchant creature
Enchanted creature can't attack or block.","2W","Tribal Enchantment - Rebel Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Drill Sergeant","2","1","R","Whenever Skirk Drill Sergeant or another Goblin dies, you may pay {2}{R}. If you do, reveal the top card of your library. If it's a Goblin permanent card, put it onto the battlefield. Otherwise, put it into your graveyard.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trained Jackal","1","2","G",NULL,"G","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Augur","2","2","U","Flying
Sacrifice Aven Augur: Return up to two target creatures to their owners' hands. Activate this ability only during your upkeep.","3U","Creature - Bird Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Terraformer","2","2","U","{1}: Choose a basic land type. Each land you control becomes that type until end of turn.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hero's Demise","B","Destroy target legendary creature.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Cloak","G","Green creatures you control gain forestwalk until end of turn.","2G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nothing Can Stop Me Now","(An ongoing scheme remains face up until it's abandoned.)
If a source an opponent controls would deal damage to you, prevent 1 of that damage.
At the beginning of each end step, if you've been dealt 5 or more damage this turn, abandon this scheme.",NULL,"Ongoing Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Johtull Wurm","6","6","G","Whenever Johtull Wurm becomes blocked, it gets -2/-1 until end of turn for each creature blocking it beyond the first.","5G","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Garruk, Primal Hunter","G","+1: Put a 3/3 green Beast creature token onto the battlefield.
-3: Draw cards equal to the greatest power among creatures you control.
-6: Put a 6/6 green Wurm creature token onto the battlefield for each land you control.","Planeswalker - Garruk","2GGG");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Blinding Souleater","1","3","{(w/p)}, {T}: Tap target creature. ({(w/p)} can be paid with either {W} or 2 life.)","3","Artifact Creature - Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shadowbane","W","The next time a source of your choice would deal damage to you and/or creatures you control this turn, prevent that damage. If damage from a black source is prevented this way, you gain that much life.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merrow Witsniper","1","1","U","When Merrow Witsniper enters the battlefield, target player puts the top card of his or her library into his or her graveyard.","U","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Graceful Reprieve","W","When target creature dies this turn, return that card to the battlefield under its owner's control.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burden of Greed","B","Target player loses 1 life for each tapped artifact he or she controls.","3B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tidal Influence","U","Cast Tidal Influence only if no permanents named Tidal Influence are on the battlefield.
Tidal Influence enters the battlefield with a tide counter on it.
At the beginning of your upkeep, put a tide counter on Tidal Influence.
As long as there is exactly one tide counter on Tidal Influence, all blue creatures get -2/-0.
As long as there are exactly three tide counters on Tidal Influence, all blue creatures get +2/+0.
Whenever there are four tide counters on Tidal Influence, remove all tide counters from it.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glen Elendra Archmage","2","2","U","Flying
{U}, Sacrifice Glen Elendra Archmage: Counter target noncreature spell.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","3U","Creature - Faerie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spellbane Centaur","3","2","G","Creatures you control can't be the targets of blue spells or abilities from blue sources.","2G","Creature - Centaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nettle Sentinel","2","2","G","Nettle Sentinel doesn't untap during your untap step.
Whenever you cast a green spell, you may untap Nettle Sentinel.","G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Bard","2","4","G","All creatures able to block Elvish Bard do so.","3GG","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stinkweed Imp","1","2","B","Flying
Whenever Stinkweed Imp deals combat damage to a creature, destroy that creature.
Dredge 5 (If you would draw a card, instead you may put exactly five cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","2B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Torii Watchward","3","3","W","Vigilance (Attacking doesn't cause this creature to tap.)
Soulshift 4 (When this creature dies, you may return target Spirit card with converted mana cost 4 or less from your graveyard to your hand.)","4W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deceiver Exarch","1","4","U","Flash (You may cast this spell any time you could cast an instant.)
When Deceiver Exarch enters the battlefield, choose one - Untap target permanent you control; or tap target permanent an opponent controls.","2U","Creature - Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parallax Nexus","B","Fading 5 (This enchantment enters the battlefield with five fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Remove a fade counter from Parallax Nexus: Target opponent exiles a card from his or her hand. Activate this ability only any time you could cast a sorcery.
When Parallax Nexus leaves the battlefield, each player returns to his or her hand all cards he or she owns exiled with Parallax Nexus.","2B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vex","U","Counter target spell. That spell's controller may draw a card.","2U","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lodestone Golem","5","3","Nonartifact spells cost {1} more to cast.","4","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Indestructibility","W","Enchant permanent
Enchanted permanent is indestructible. (Effects that say ""destroy"" don't destroy that permanent. An indestructible creature can't be destroyed by damage.)","3W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Weatherseed Faeries","2","1","U","Flying, protection from red","2U","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thada Adel, Acquisitor","2","2","U","Islandwalk
Whenever Thada Adel, Acquisitor deals combat damage to a player, search that player's library for an artifact card and exile it. Then that player shuffles his or her library. Until end of turn, you may play that card.","1UU","Legendary Creature - Merfolk Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Solemn Simulacrum","2","2","When Solemn Simulacrum enters the battlefield, you may search your library for a basic land card, put that card onto the battlefield tapped, then shuffle your library.
When Solemn Simulacrum dies, you may draw a card.","4","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aisling Leprechaun","1","1","G","Whenever Aisling Leprechaun blocks or becomes blocked by a creature, that creature becomes green. (This effect lasts indefinitely.)","G","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ambassador Oak","3","3","G","When Ambassador Oak enters the battlefield, put a 1/1 green Elf Warrior creature token onto the battlefield.","3G","Creature - Treefolk Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Momir Vig, Simic Visionary","2","2","UG","Whenever you cast a green creature spell, you may search your library for a creature card and reveal it. If you do, shuffle your library and put that card on top of it.
Whenever you cast a blue creature spell, reveal the top card of your library. If it's a creature card, put that card into your hand.","3GU","Legendary Creature - Elf Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hurloon Minotaur","2","3","R",NULL,"1RR","Creature - Minotaur");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deflection","U","Change the target of target spell with a single target.","3U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyresis","B","Enchant creature
Enchanted creature has infect. (It deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","1B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Slagwurm Armor","Equipped creature gets +0/+6.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conclave's Blessing","W","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Enchant creature
Enchanted creature gets +0/+2 for each other creature you control.","3W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marsh Boa","1","1","G","Swampwalk","G","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stenchskipper","6","5","B","Flying
At the beginning of the end step, if you control no Goblins, sacrifice Stenchskipper.","3B","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rage Forger","2","2","R","When Rage Forger enters the battlefield, put a +1/+1 counter on each other Shaman creature you control.
Whenever a creature you control with a +1/+1 counter on it attacks, you may have that creature deal 1 damage to target player.","2R","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tel-Jilad Wolf","2","2","G","Whenever Tel-Jilad Wolf becomes blocked by an artifact creature, Tel-Jilad Wolf gets +3/+3 until end of turn.","2G","Creature - Wolf");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Brass-Talon Chimera","2","2","First strike
Sacrifice Brass-Talon Chimera: Put a +2/+2 counter on target Chimera creature. It gains first strike. (This effect lasts indefinitely.)","4","Artifact Creature - Chimera");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Lorebearers","3","3","G","{3}{G}, {T}: Target creature gets +X/+X until end of turn, where X is the number of artifacts your opponents control.","3G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Poison Arrow","B","Destroy target nonblack creature. You gain 3 life.","4BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Bomb","U","Each player may discard up to three cards. Mind Bomb deals damage to each player equal to 3 minus the number of cards he or she discarded this way.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swift Silence","WU","Counter all other spells. Draw a card for each spell countered this way.","2WUU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raging Gorilla","2","3","R","Whenever Raging Gorilla blocks or becomes blocked, it gets +2/-2 until end of turn.","2R","Creature - Ape");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reparations","WU","Whenever an opponent casts a spell that targets you or a creature you control, you may draw a card.","1WU","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Scuttlemutt","2","2","{T}: Add one mana of any color to your mana pool.
{T}: Target creature becomes the color or colors of your choice until end of turn.","3","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cackling Flames","R","Cackling Flames deals 3 damage to target creature or player.
Hellbent - Cackling Flames deals 5 damage to that creature or player instead if you have no cards in hand.","3R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rhystic Tutor","B","Unless any player pays {2}, search your library for a card, put that card into your hand, then shuffle your library.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nemesis of Reason","3","7","UB","Whenever Nemesis of Reason attacks, defending player puts the top ten cards of his or her library into his or her graveyard.","3UB","Creature - Leviathan Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Laugh","B","Whenever a permanent other than Last Laugh is put into a graveyard from the battlefield, Last Laugh deals 1 damage to each creature and each player.
When no creatures are on the battlefield, sacrifice Last Laugh.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frostling","1","1","R","Sacrifice Frostling: Frostling deals 1 damage to target creature.","R","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Triassic Egg","{3}, {T}: Put a hatchling counter on Triassic Egg.
Sacrifice Triassic Egg: Choose one - You may put a creature card from your hand onto the battlefield; or return target creature card from your graveyard to the battlefield. Activate this ability only if two or more hatchling counters are on Triassic Egg.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Groundling Pouncer","2","1","UG","{(g/u)}: Groundling Pouncer gets +1/+3 and gains flying until end of turn. Activate this ability only once each turn and only if an opponent controls a creature with flying.","1(G/U)","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arsenal Thresher","2","2","WUB","As Arsenal Thresher enters the battlefield, you may reveal any number of other artifact cards from your hand. Arsenal Thresher enters the battlefield with a +1/+1 counter on it for each card revealed this way.","2(W/B)U","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glistening Oil","B","Enchant creature
Enchanted creature has infect.
At the beginning of your upkeep, put a -1/-1 counter on enchanted creature.
When Glistening Oil is put into a graveyard from the battlefield, return Glistening Oil to its owner's hand.","BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tundra Wolves","1","1","W","First strike (This creature deals combat damage before creatures without first strike.)","W","Creature - Wolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infernal Darkness","B","Cumulative upkeep-Pay {B} and 1 life. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
If a land is tapped for mana, it produces {B} instead of any other type.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Freewind Falcon","1","1","W","Flying, protection from red","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Retribution","5","5","W","Flying, first strike","6W","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pardic Lancer","3","2","R","Discard a card at random: Pardic Lancer gets +1/+0 and gains first strike until end of turn.","4R","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Doomed Necromancer","2","2","B","{B}, {T}, Sacrifice Doomed Necromancer: Return target creature card from your graveyard to the battlefield.","2B","Creature - Human Cleric Mercenary");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Marble Diamond","Marble Diamond enters the battlefield tapped.
{T}: Add {W} to your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Walker of Secret Ways","1","2","U","Ninjutsu {1}{U} ({1}{U}, Return an unblocked attacker you control to hand: Put this card onto the battlefield from your hand tapped and attacking.)
Whenever Walker of Secret Ways deals combat damage to a player, look at that player's hand.
{1}{U}: Return target Ninja you control to its owner's hand. Activate this ability only during your turn.","2U","Creature - Human Ninja");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Body of Jukai","8","5","G","Trample
Soulshift 8 (When this creature dies, you may return target Spirit card with converted mana cost 8 or less from your graveyard to your hand.)","7GG","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Earth Elemental","4","5","R",NULL,"3RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lunar Mystic","2","2","U","Whenever you cast an instant spell, you may pay {1}. If you do, draw a card.","2UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Initiate","1","1","B","{1}: Add {B} to your mana pool.","1B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disturbed Burial","B","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Return target creature card from your graveyard to your hand.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Flectomancer","2","2","UR","Sacrifice Goblin Flectomancer: You may change the targets of target instant or sorcery spell.","URR","Creature - Goblin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Interrogator","1","1","B","{X}{B}, {T}: Target player reveals X cards from his or her hand and you choose one of them. That player discards that card. Activate this ability only any time you could cast a sorcery.","1B","Creature - Zombie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Golden Wish","W","You may choose an artifact or enchantment card you own from outside the game, reveal that card, and put it into your hand. Exile Golden Wish.","3WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Pit Offering","B","When Death Pit Offering enters the battlefield, sacrifice all creatures you control.
Creatures you control get +2/+2.","2BB","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Throne of Bone","Whenever a player casts a black spell, you may pay {1}. If you do, you gain 1 life.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mordant Dragon","5","5","R","Flying
{1}{R}: Mordant Dragon gets +1/+0 until end of turn.
Whenever Mordant Dragon deals combat damage to a player, you may have it deal that much damage to target creature that player controls.","3RRR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akoum Battlesinger","1","1","R","Haste
Whenever Akoum Battlesinger or another Ally enters the battlefield under your control, you may have Ally creatures you control get +1/+0 until end of turn.","1R","Creature - Human Berserker Ally");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trip Noose","{2}, {T}: Tap target creature.","2","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Telethopter","3","1","Tap an untapped creature you control: Telethopter gains flying until end of turn.","4","Artifact Creature - Thopter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brightflame","WR","Radiance - Brightflame deals X damage to target creature and each other creature that shares a color with it. You gain life equal to the damage dealt this way.","XRRWW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Teferi's Curse","U","Enchant artifact or creature
Enchanted permanent has phasing. (It phases in or out before its controller untaps during each of his or her untap steps. While it's phased out, it's treated as though it doesn't exist.)","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Puncturing Light","W","Destroy target attacking or blocking creature with power 3 or less.","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gallows at Willow Hill","{3}, {T}, Tap three untapped Humans you control: Destroy target creature. Its controller puts a 1/1 white Spirit creature token with flying onto the battlefield.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Memory Erosion","U","Whenever an opponent casts a spell, that player puts the top two cards of his or her library into his or her graveyard.","1UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rock Slide","R","Rock Slide deals X damage divided as you choose among any number of target attacking or blocking creatures without flying.","XR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daru Healer","1","2","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.
Morph {W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faerie Harbinger","2","2","U","Flash
Flying
When Faerie Harbinger enters the battlefield, you may search your library for a Faerie card, reveal it, then shuffle your library and put that card on top of it.","3U","Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hall of Gemstone","G","At the beginning of each player's upkeep, that player chooses a color. Until end of turn, lands tapped for mana produce mana of the chosen color instead of any other color.","1GG","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glyph of Destruction","R","Target blocking Wall you control gets +10/+0 until end of combat. Prevent all damage that would be dealt to it this turn. Destroy it at the beginning of the next end step.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avatar of Discord","5","3","BR","({(b/r)} can be paid with either {B} or {R}.)
Flying
When Avatar of Discord enters the battlefield, sacrifice it unless you discard two cards.","(B/R)(B/R)(B/R)","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ambition's Cost","B","You draw three cards and you lose 3 life.","3B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heat Stroke","R","At end of combat, destroy each creature that blocked or was blocked this turn.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steal Artifact","U","Enchant artifact
You control enchanted artifact.","2UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lavamancer's Skill","R","Enchant creature
Enchanted creature has ""{T}: This creature deals 1 damage to target creature.""
As long as enchanted creature is a Wizard, it has ""{T}: This creature deals 2 damage to target creature.""","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Demonic Hordes","5","5","B","{T}: Destroy target land.
At the beginning of your upkeep, unless you pay {B}{B}{B}, tap Demonic Hordes and sacrifice a land of an opponent's choice.","3BBB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rude Awakening","G","Choose one - Untap all lands you control; or until end of turn, lands you control become 2/2 creatures that are still lands.
Entwine {2}{G} (Choose both if you pay the entwine cost.)","4G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Northern Paladin","3","3","W","{W}{W}, {T}: Destroy target black permanent.","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Accorder Paladin","3","1","W","Battle cry (Whenever this creature attacks, each other attacking creature gets +1/+0 until end of turn.)","1W","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Fate of the Flammable","When you set this scheme in motion, target opponent chooses self or others. If that player chooses self, this scheme deals 6 damage to him or her. If the player chooses others, this scheme deals 3 damage to each of your other opponents.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wu Elite Cavalry","2","3","U","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","3U","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hulking Cyclops","5","5","R","Hulking Cyclops can't block.","3RR","Creature - Cyclops");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrion Thrash","4","4","BRG","When Carrion Thrash dies, you may pay {2}. If you do, return another target creature card from your graveyard to your hand.","2BRG","Creature - Viashino Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fog","G","Prevent all combat damage that would be dealt this turn.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ratchet Bomb","{T}: Put a charge counter on Ratchet Bomb.
{T}, Sacrifice Ratchet Bomb: Destroy each nonland permanent with converted mana cost equal to the number of charge counters on Ratchet Bomb.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cephalid Coliseum","{T}: Add {U} to your mana pool. Cephalid Coliseum deals 1 damage to you.
Threshold - {U}, {T}, Sacrifice Cephalid Coliseum: Target player draws three cards, then discards three cards. Activate this ability only if seven or more cards are in your graveyard.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Norwood Riders","3","3","G","Norwood Riders can't be blocked by more than one creature.","3G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hoarder's Greed","B","You lose 2 life and draw two cards, then clash with an opponent. If you win, repeat this process. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sakura-Tribe Scout","1","1","G","{T}: You may put a land card from your hand onto the battlefield.","G","Creature - Snake Shaman Scout");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Jawbone Skulkin","1","1","{2}: Target red creature gains haste until end of turn.","1","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Counterintelligence","U","Return one or two target creatures to their owners' hands.","2UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kaho, Minamo Historian","2","2","U","When Kaho, Minamo Historian enters the battlefield, search your library for up to three instant cards and exile them. Then shuffle your library.
{X}, {T}: You may cast a card with converted mana cost X exiled with Kaho without paying its mana cost.","2UU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chilling Apparition","1","1","B","{B}: Regenerate Chilling Apparition.
Whenever Chilling Apparition deals combat damage to a player, that player discards a card.","2B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Breezekeeper","4","4","U","Flying
Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)","3U","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alabaster Dragon","4","4","W","Flying
When Alabaster Dragon dies, shuffle it into its owner's library.","4WW","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drowner of Secrets","1","3","U","Tap an untapped Merfolk you control: Target player puts the top card of his or her library into his or her graveyard.","2U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Albatross","1","1","U","Flying
When Giant Albatross dies, you may pay {1}{U}. If you do, for each creature that dealt damage to Giant Albatross this turn, destroy that creature unless its controller pays 2 life. A creature destroyed this way can't be regenerated.","1U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incite War","R","Choose one - Creatures target player controls attack this turn if able; or creatures you control gain first strike until end of turn.
Entwine {2} (Choose both if you pay the entwine cost.)","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hurricane","G","Hurricane deals X damage to each creature with flying and each player.","XG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battlefield Percher","2","2","B","Flying
Battlefield Percher can block only creatures with flying.
{1}{B}: Battlefield Percher gets +1/+1 until end of turn.","3BB","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brightstone Ritual","R","Add {R} to your mana pool for each Goblin on the battlefield.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mark of the Oni","B","Enchant creature
You control enchanted creature.
At the beginning of the end step, if you control no Demons, sacrifice Mark of the Oni.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vitalizing Cascade","WG","You gain X plus 3 life.","XGW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noble Panther","3","3","WG","{1}: Noble Panther gains first strike until end of turn.","1GW","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragon Appeasement","BRG","Skip your draw step.
Whenever you sacrifice a creature, you may draw a card.","3BRG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Animate Wall","W","Enchant Wall
Enchanted Wall can attack as though it didn't have defender.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ivy Seer","1","1","G","{2}{G}, {T}: Reveal any number of green cards in your hand. Target creature gets +X/+X until end of turn, where X is the number of cards revealed this way.","3G","Creature - Elf Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hotheaded Giant","4","4","R","Haste
Hotheaded Giant enters the battlefield with two -1/-1 counters on it unless you've cast another red spell this turn.","3R","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Complex Automaton","4","4","At the beginning of your upkeep, if you control seven or more permanents, return Complex Automaton to its owner's hand.","4","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turn to Slag","R","Turn to Slag deals 5 damage to target creature. Destroy all Equipment attached to that creature.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vigilant Drake","3","3","U","Flying
{2}{U}: Untap Vigilant Drake.","4U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Budoka Gardener","2","1","G","{T}: You may put a land card from your hand onto the battlefield. If you control ten or more lands, flip Budoka Gardener.
---
{4}{G}{G}, {T}: Put an X/X green Elemental creature token onto the battlefield, where X is the number of lands you control.","1G","Creature - Human Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brilliant Halo","W","Enchant creature
Enchanted creature gets +1/+2.
When Brilliant Halo is put into a graveyard from the battlefield, return Brilliant Halo to its owner's hand.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sekki, Seasons' Guide","0","0","G","Sekki, Seasons' Guide enters the battlefield with eight +1/+1 counters on it.
If damage would be dealt to Sekki, prevent that damage, remove that many +1/+1 counters from Sekki, and put that many 1/1 colorless Spirit creature tokens onto the battlefield.
Sacrifice eight Spirits: Return Sekki from your graveyard to the battlefield.","5GGG","Legendary Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirri","If a basic land you control is tapped for mana, it produces mana of a color of your choice instead of any other type.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kangee, Aerie Keeper","2","2","WU","Kicker {X}{2} (You may pay an additional {X}{2} as you cast this spell.)
Flying
When Kangee, Aerie Keeper enters the battlefield, if it was kicked, put X feather counters on it.
Other Bird creatures get +1/+1 for each feather counter on Kangee, Aerie Keeper.","2WU","Legendary Creature - Bird Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Blazethorn Scarecrow","3","3","Blazethorn Scarecrow has haste as long as you control a red creature.
Blazethorn Scarecrow has wither as long as you control a green creature. (It deals damage to creatures in the form of -1/-1 counters.)","5","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sharuum the Hegemon","5","5","WUB","Flying
When Sharuum the Hegemon enters the battlefield, you may return target artifact card from your graveyard to the battlefield.","3WUB","Legendary Artifact Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mountain Titan","2","2","BR","{1}{R}{R}: Until end of turn, whenever you cast a black spell, put a +1/+1 counter on Mountain Titan.","2BR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seascape Aerialist","2","3","U","Whenever Seascape Aerialist or another Ally enters the battlefield under your control, you may have Ally creatures you control gain flying until end of turn.","4U","Creature - Merfolk Wizard Ally");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The AEther Flues","When you planeswalk to The Æther Flues or at the beginning of your upkeep, you may sacrifice a creature. If you do, reveal cards from the top of your library until you reveal a creature card, put that card onto the battlefield, then shuffle all other cards revealed this way into your library.
Whenever you roll {C}, you may put a creature card from your hand onto the battlefield.",NULL,"Plane - Iquatana");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Insight","U","Whenever an opponent casts a green spell, you draw a card.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Binding Agony","B","Enchant creature
Whenever enchanted creature is dealt damage, Binding Agony deals that much damage to that creature's controller.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scalding Salamander","2","1","R","Whenever Scalding Salamander attacks, you may have it deal 1 damage to each creature without flying defending player controls.","2R","Creature - Salamander");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dimir Doppelganger","0","2","UB","{1}{U}{B}: Exile target creature card from a graveyard. Dimir Doppelganger becomes a copy of that card and gains this ability.","1UB","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Test of Endurance","W","At the beginning of your upkeep, if you have 50 or more life, you win the game.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rith's Attendant","3","3","{1}, Sacrifice Rith's Attendant: Add {R}{G}{W} to your mana pool.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nakaya Shade","1","1","B","{B}: Nakaya Shade gets +1/+1 until end of turn unless any player pays {2}.","1B","Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wirewood Savage","2","2","G","Whenever a Beast enters the battlefield, you may draw a card.","2G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Lookout","1","1","G","Shroud (This permanent can't be the target of spells or abilities.)","G","Creature - Elf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Armageddon Clock","At the beginning of your upkeep, put a doom counter on Armageddon Clock.
At the beginning of your draw step, Armageddon Clock deals damage equal to the number of doom counters on it to each player.
{4}: Remove a doom counter from Armageddon Clock. Any player may activate this ability but only during any upkeep step.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Certarch","1","1","U","Metalcraft - {T}: Tap target artifact, creature, or land. Activate this ability only if you control three or more artifacts.","U","Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rage Nimbus","5","3","R","Defender, flying
{1}{R}: Target creature attacks this turn if able.","2R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Cannoneers","1","3","R","{T}: Orcish Cannoneers deals 2 damage to target creature or player and 3 damage to you.","1RR","Creature - Orc Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunastian Falconer","4","4","RG","{T}: Add {2} to your mana pool.","3RG","Legendary Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Jhovall","3","3","R",NULL,"3R","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oros, the Avenger","6","6","WBR","Flying
Whenever Oros, the Avenger deals combat damage to a player, you may pay {2}{W}. If you do, Oros deals 3 damage to each nonwhite creature.","3WBR","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Esper Battlemage","2","2","U","{W}, {T}: Prevent the next 2 damage that would be dealt to you this turn.
{B}, {T}: Target creature gets -1/-1 until end of turn.","2U","Artifact Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nacatl Savage","2","1","G","Protection from artifacts","1G","Creature - Cat Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Everflowing Chalice","Multikicker {2} (You may pay an additional {2} any number of times as you cast this spell.)
Everflowing Chalice enters the battlefield with a charge counter on it for each time it was kicked.
{T}: Add {1} to your mana pool for each charge counter on Everflowing Chalice.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elfhame Sanctuary","G","At the beginning of your upkeep, you may search your library for a basic land card, reveal that card, and put it into your hand. If you do, you skip your draw step this turn and shuffle your library.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Teferi's Moat","WU","As Teferi's Moat enters the battlefield, choose a color.
Creatures of the chosen color without flying can't attack you.","3WU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Biologist","1","2","U","Protection from Beasts
Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Duh","B","Destroy target creature with reminder text. (Reminder text is any italicized text in parentheses that explains rules you already know.)","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Show and Tell","U","Each player may put an artifact, creature, enchantment, or land card from his or her hand onto the battlefield.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Battle Mastery","W","Enchant creature
Enchanted creature has double strike.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arcane Melee","U","Instant and sorcery spells cost {2} less to cast.","4U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Misstep","U","Creatures target player controls don't untap during that player's next untap step.","1U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skybreen","Players play with the top card of their libraries revealed.
Spells that share a card type with the top card of a library can't be cast.
Whenever you roll {C}, target player loses life equal to the number of cards in his or her hand.",NULL,"Plane - Kaldheim");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lost Auramancers","3","3","W","Vanishing 3 (This permanent enters the battlefield with three time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
When Lost Auramancers dies, if it had no time counters on it, you may search your library for an enchantment card and put it onto the battlefield. If you do, shuffle your library.","2WW","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thought Nibbler","1","1","U","Flying
Your maximum hand size is reduced by two.","U","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodthorn Taunter","1","1","R","Haste
{T}: Target creature with power 5 or greater gains haste until end of turn.","1R","Creature - Human Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sigil of the Empty Throne","W","Whenever you cast an enchantment spell, put a 4/4 white Angel creature token with flying onto the battlefield.","3WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ulvenwald Mystics","3","3","G","At the beginning of each upkeep, if no spells were cast last turn, transform Ulvenwald Mystics.","2GG","Creature - Human Shaman Werewolf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Claws of Gix","{1}, Sacrifice a permanent: You gain 1 life.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dungeon Shade","1","1","B","Flying
{B}: Dungeon Shade gets +1/+1 until end of turn.","3B","Creature - Shade Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Weakstone","Attacking creatures get -1/-0.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaea's Embrace","G","Enchant creature
Enchanted creature gets +3/+3 and has trample.
{G}: Regenerate enchanted creature.","2GG","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Spellduster","2","3","W","Flying
{1}{W}, Sacrifice Kithkin Spellduster: Destroy target enchantment.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","4W","Creature - Kithkin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Riftwatcher","2","3","W","Flying
Vanishing 3 (This permanent enters the battlefield with three time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
When Aven Riftwatcher enters the battlefield or leaves the battlefield, you gain 2 life.","2W","Creature - Bird Rebel Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kagemaro's Clutch","B","Enchant creature
Enchanted creature gets -X/-X, where X is the number of cards in your hand.","3B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Explorer's Scope","Whenever equipped creature attacks, look at the top card of your library. If it's a land card, you may put it onto the battlefield tapped.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dakmor Scorpion","2","1","B",NULL,"1B","Creature - Scorpion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tireless Missionaries","2","3","W","When Tireless Missionaries enters the battlefield, you gain 3 life.","4W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seedborn Muse","2","4","G","Untap all permanents you control during each other player's untap step.","3GG","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Tradewinds","U","Return target permanent you control and target permanent you don't control to their owners' hands.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blistering Dieflyn","0","1","R","Flying
{(b/r)}: Blistering Dieflyn gets +1/+0 until end of turn.","3R","Creature - Imp");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Evil Comes to Fruition","When you set this scheme in motion, put seven 0/1 green Plant creature tokens onto the battlefield. If you control ten or more lands, put seven 3/3 green Elemental creature tokens onto the battlefield instead.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wood Sage","1","1","UG","{T}: Name a creature card. Reveal the top four cards of your library and put all of them with that name into your hand. Put the rest into your graveyard.","GU","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Smash to Smithereens","R","Destroy target artifact. Smash to Smithereens deals 3 damage to that artifact's controller.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alarum","W","Untap target nonattacking creature. It gets +1/+3 until end of turn.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warped Devotion","B","Whenever a permanent is returned to a player's hand, that player discards a card.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragon Tyrant","6","6","R","Flying, trample
Double strike (This creature deals both first-strike and regular combat damage.)
At the beginning of your upkeep, sacrifice Dragon Tyrant unless you pay {R}{R}{R}{R}.
{R}: Dragon Tyrant gets +1/+0 until end of turn.","8RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Indentured Djinn","4","4","U","Flying
When Indentured Djinn enters the battlefield, each other player may draw up to three cards.","1UU","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angelic Arbiter","5","6","W","Flying
Each opponent who cast a spell this turn can't attack with creatures.
Each opponent who attacked with a creature this turn can't cast spells.","5WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrion Beetles","1","1","B","{2}{B}, {T}: Exile up to three target cards from a single graveyard.","B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lilting Refrain","U","At the beginning of your upkeep, you may put a verse counter on Lilting Refrain.
Sacrifice Lilting Refrain: Counter target spell unless its controller pays {X}, where X is the number of verse counters on Lilting Refrain.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unforge","R","Destroy target Equipment. If that Equipment was attached to a creature, Unforge deals 2 damage to that creature.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("No Rest for the Wicked","B","Sacrifice No Rest for the Wicked: Return to your hand all creature cards in your graveyard that were put there from the battlefield this turn.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gluetius Maximus","5","5","G","As Gluetius Maximus comes into play, an opponent chooses one of your fingers. (Thumbs are fingers, too.)
When the chosen finger isn't touching Gluetius Maximus, sacrifice Gluetius Maximus.","3GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brood Sliver","3","3","G","Whenever a Sliver deals combat damage to a player, its controller may put a 1/1 colorless Sliver creature token onto the battlefield.","4G","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quicksilver Gargantuan","7","7","U","You may have Quicksilver Gargantuan enter the battlefield as a copy of any creature on the battlefield, except it's still 7/7.","5UU","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silvergill Douser","1","1","U","{T}: Target creature gets -X/-0 until end of turn, where X is the number of Merfolk and/or Faeries you control.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cathedral Sanctifier","1","1","W","When Cathedral Sanctifier enters the battlefield, you gain 3 life.","W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sunforger","Equipped creature gets +4/+0.
{R}{W}, Unattach Sunforger: Search your library for a red or white instant card with converted mana cost 4 or less and cast that card without paying its mana cost. Then shuffle your library.
Equip {3}","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nemesis Trap","B","If a white creature is attacking, you may pay {B}{B} rather than pay Nemesis Trap's mana cost.
Exile target attacking creature. Put a token that's a copy of that creature onto the battlefield. Exile it at the beginning of the next end step.","4BB","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Howling Wolf","2","2","G","When Howling Wolf enters the battlefield, you may search your library for up to three cards named Howling Wolf, reveal them, and put them into your hand. If you do, shuffle your library.","2GG","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Exiled Boggart","2","2","B","When Exiled Boggart dies, discard a card.","1B","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunderscape Battlemage","2","2","R","Kicker {1}{B} and/or {G} (You may pay an additional {1}{B} and/or {G} as you cast this spell.)
When Thunderscape Battlemage enters the battlefield, if it was kicked with its {1}{B} kicker, target player discards two cards.
When Thunderscape Battlemage enters the battlefield, if it was kicked with its {G} kicker, destroy target enchantment.","2R","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fettergeist","3","4","U","Flying
At the beginning of your upkeep, sacrifice Fettergeist unless you pay {1} for each other creature you control.","2U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snapback","U","You may exile a blue card from your hand rather than pay Snapback's mana cost.
Return target creature to its owner's hand.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reprisal","W","Destroy target creature with power 4 or greater. It can't be regenerated.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rustrazor Butcher","1","2","R","First strike
Wither (This deals damage to creatures in the form of -1/-1 counters.)","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Triumph","B","If you control a Swamp, you may sacrifice a creature rather than pay Dark Triumph's mana cost.
Creatures you control get +2/+0 until end of turn.","4B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Priest of Yawgmoth","1","2","B","{T}, Sacrifice an artifact: Add to your mana pool an amount of {B} equal to the sacrificed artifact's converted mana cost.","1B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wring Flesh","B","Target creature gets -3/-1 until end of turn.","B","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Coal Golem","3","3","{3}, Sacrifice Coal Golem: Add {R}{R}{R} to your mana pool.","5","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Krosan Grip","G","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Destroy target artifact or enchantment.","2G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tawnos's Coffin","You may choose not to untap Tawnos's Coffin during your untap step.
{3}, {T}: Exile target creature and all Auras attached to it. Note the number and kind of counters that were on that creature. When Tawnos's Coffin leaves the battlefield or becomes untapped, return the exiled card to the battlefield under its owner's control tapped with the noted number and kind of counters on it, and if you do, return the exiled Aura cards to the battlefield under their owner's control attached to that permanent.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mage il-Vec","2","2","R","{T}, Discard a card at random: Mage il-Vec deals 1 damage to target creature or player.","2R","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Expendable Troops","2","1","W","{T}, Sacrifice Expendable Troops: Expendable Troops deals 2 damage to target attacking or blocking creature.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sturdy Hatchling","6","6","UG","Sturdy Hatchling enters the battlefield with four -1/-1 counters on it.
{(g/u)}: Sturdy Hatchling gains shroud until end of turn.
Whenever you cast a green spell, remove a -1/-1 counter from Sturdy Hatchling.
Whenever you cast a blue spell, remove a -1/-1 counter from Sturdy Hatchling.","3(G/U)","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woolly Spider","2","3","G","Reach (This creature can block creatures with flying.)
Whenever Woolly Spider blocks a creature with flying, Woolly Spider gets +0/+2 until end of turn.","1GG","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cliff Threader","2","1","W","Mountainwalk","1W","Creature - Kor Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tidal Flats","U","{U}{U}: For each attacking creature without flying, its controller may pay {1}. If he or she doesn't, creatures you control blocking that creature gain first strike until end of turn.","U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skarrgan Firebird","3","3","R","Bloodthirst 3 (If an opponent was dealt damage this turn, this creature enters the battlefield with three +1/+1 counters on it.)
Flying
{R}{R}{R}: Return Skarrgan Firebird from your graveyard to your hand. Activate this ability only if an opponent was dealt damage this turn.","4RR","Creature - Phoenix");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Filigree Fracture","G","Destroy target artifact or enchantment. If that permanent was blue or black, draw a card.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necrogen Scudder","3","3","B","Flying
When Necrogen Scudder enters the battlefield, you lose 3 life.","2B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yare","W","Target creature defending player controls gets +3/+0 until end of turn. That creature can block up to two additional creatures this turn.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Stand","WUBRG","Target opponent loses 2 life for each Swamp you control. Last Stand deals damage equal to the number of Mountains you control to target creature. Put a 1/1 green Saproling creature token onto the battlefield for each Forest you control. You gain 2 life for each Plains you control. Draw a card for each Island you control, then discard that many cards.","WUBRG","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Elf Replica","2","2","{1}{G}, Sacrifice Elf Replica: Destroy target enchantment.","3","Artifact Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Meng Huo, Barbarian King","4","4","G","Other green creatures you control get +1/+1.","3GG","Legendary Creature - Human Barbarian Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lurking Jackals","B","When an opponent has 10 or less life, if Lurking Jackals is an enchantment, it becomes a 3/2 Hound creature.","B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lim-Dûl the Necromancer","4","4","B","Whenever a creature an opponent controls dies, you may pay {1}{B}. If you do, return that card to the battlefield under your control. If it's a creature, it's a Zombie in addition to its other creature types.
{1}{B}: Regenerate target Zombie.","5BB","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Godtracker of Jund","2","2","RG","Whenever a creature with power 5 or greater enters the battlefield under your control, you may put a +1/+1 counter on Godtracker of Jund.","1RG","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fraction Jackson","1","1{1/2}","G","{G}, {T}: Return target card with a {1/2} on it from your graveyard to your hand.","2G","Creature - Human Hero");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arcane Sanctum","Arcane Sanctum enters the battlefield tapped.
{T}: Add {W}, {U}, or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greater Stone Spirit","4","4","R","Greater Stone Spirit can't be blocked by creatures with flying.
{2}{R}: Until end of turn, target creature gets +0/+2 and gains ""{R}: This creature gets +1/+0 until end of turn.""","4RR","Creature - Elemental Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rime Dryad","1","2","G","Snow forestwalk","G","Creature - Dryad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Symbol Status","G","Put a 1/1 colorless Expansion-Symbol creature token into play for each different expansion symbol among permanents you control.","2GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Battle Frenzy","R","Green creatures you control get +1/+1 until end of turn.
Nongreen creatures you control get +1/+0 until end of turn.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volley of Boulders","R","Volley of Boulders deals 6 damage divided as you choose among any number of target creatures and/or players.
Flashback {R}{R}{R}{R}{R}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","8R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frost Breath","U","Tap up to two target creatures. Those creatures don't untap during their controller's next untap step.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merrow Reejerey","2","2","U","Other Merfolk creatures you control get +1/+1.
Whenever you cast a Merfolk spell, you may tap or untap target permanent.","2U","Creature - Merfolk Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Highland Berserker","2","1","R","Whenever Highland Berserker or another Ally enters the battlefield under your control, you may have Ally creatures you control gain first strike until end of turn.","1R","Creature - Human Berserker Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Violet Pall","B","Destroy target nonblack creature. Put a 1/1 black Faerie Rogue creature token with flying onto the battlefield.","4B","Tribal Instant - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battle-Mad Ronin","1","1","R","Bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)
Battle-Mad Ronin attacks each turn if able.","1R","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spire Owl","1","1","U","Flying
When Spire Owl enters the battlefield, look at the top four cards of your library, then put them back in any order.","1U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghitu Fire","R","You may cast Ghitu Fire any time you could cast an instant if you pay {2} more to cast it.
Ghitu Fire deals X damage to target creature or player.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Parasitic Strix","2","2","U","Flying
When Parasitic Strix enters the battlefield, if you control a black permanent, target player loses 2 life and you gain 2 life.","2U","Artifact Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Merchant","1","2","U","When Vodalian Merchant enters the battlefield, draw a card, then discard a card.","1U","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peace of Mind","W","{W}, Discard a card: You gain 3 life.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Martyr of Bones","1","1","B","{1}, Reveal X black cards from your hand, Sacrifice Martyr of Bones: Exile up to X target cards from a single graveyard.","B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conjurer's Ban","WB","Name a card. Until your next turn, the named card can't be played.
Draw a card.","WB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cursed Totem","Activated abilities of creatures can't be activated.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Barrier","U","Whenever an opponent casts a creature spell, Soul Barrier deals 2 damage to that player unless he or she pays {2}.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beast Attack","G","Put a 4/4 green Beast creature token onto the battlefield.
Flashback {2}{G}{G}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2GGG","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Planar Portal","{6}, {T}: Search your library for a card and put that card into your hand. Then shuffle your library.","6","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grindclock","{T}: Put a charge counter on Grindclock.
{T}: Target player puts the top X cards of his or her library into his or her graveyard, where X is the number of charge counters on Grindclock.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krark-Clan Stoker","2","2","R","{T}, Sacrifice an artifact: Add {R}{R} to your mana pool.","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shaman en-Kor","1","2","W","{0}: The next 1 damage that would be dealt to Shaman en-Kor this turn is dealt to target creature you control instead.
{1}{W}: The next time a source of your choice would deal damage to target creature this turn, that damage is dealt to Shaman en-Kor instead.","1W","Creature - Kor Cleric Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necrite","2","2","B","Whenever Necrite attacks and isn't blocked, you may sacrifice it. If you do, destroy target creature defending player controls. It can't be regenerated.","1BB","Creature - Thrull");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tempest of Light","W","Destroy all enchantments.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Muck Drubb","3","3","B","Flash (You may cast this spell any time you could cast an instant.)
When Muck Drubb enters the battlefield, change the target of target spell that targets only a single creature to Muck Drubb.
Madness {2}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","3BB","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Centaur","2","0","G","Protection from black
Phantom Centaur enters the battlefield with three +1/+1 counters on it.
If damage would be dealt to Phantom Centaur, prevent that damage. Remove a +1/+1 counter from Phantom Centaur.","2GG","Creature - Centaur Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shadow of Doubt","UB","({(u/b)} can be paid with either {U} or {B}.)
Players can't search libraries this turn.
Draw a card.","(U/B)(U/B)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Grenade","R","As an additional cost to cast Goblin Grenade, sacrifice a Goblin.
Goblin Grenade deals 5 damage to target creature or player.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spur Grappler","2","1","R","Spur Grappler gets +2/+1 as long as you control no untapped lands.","2R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Accursed Centaur","2","2","B","When Accursed Centaur enters the battlefield, sacrifice a creature.","B","Creature - Zombie Centaur");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turn to Mist","WU","Exile target creature. Return that card to the battlefield under its owner's control at the beginning of the next end step.","1(W/U)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prized Unicorn","2","2","G","All creatures able to block Prized Unicorn do so.","3G","Creature - Unicorn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warthog","3","2","G","Swampwalk","1GG","Creature - Boar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vug Lizard","3","4","R","Mountainwalk
Echo {1}{R}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","1RR","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Enchantress","2","2","G","Yavimaya Enchantress gets +1/+1 for each enchantment on the battlefield.","2G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scoria Elemental","6","1","R",NULL,"4R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Revenant","*","*","B","Flying
Revenant's power and toughness are each equal to the number of creature cards in your graveyard.","4B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Puppeteer","1","2","U","{U}, {T}: You may tap or untap target creature.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ridged Kusite","1","1","B","{1}{B}, {T}, Discard a card: Target creature gets +1/+0 and gains first strike until end of turn.","B","Creature - Horror Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trapjaw Kelpie","3","3","UG","Flash
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","4(G/U)(G/U)","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Vines","0","3","G","Defender (This creature can't attack.)
Reach (This creature can block creatures with flying.)","G","Creature - Plant Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmal Abomination","5","5","U","Defender
When Phantasmal Abomination becomes the target of a spell or ability, sacrifice it.","1UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Zealot","1","1","W","First strike","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Isamaru, Hound of Konda","2","2","W",NULL,"W","Legendary Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fyndhorn Brownie","1","1","G","{2}{G}, {T}: Untap target creature.","2G","Creature - Ouphe");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slithery Stalker","1","1","B","Swampwalk
When Slithery Stalker enters the battlefield, exile target green or white creature an opponent controls.
When Slithery Stalker leaves the battlefield, return the exiled card to the battlefield under its owner's control.","1BB","Creature - Nightmare Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Magnetic Mine","Whenever another artifact is put into a graveyard from the battlefield, Magnetic Mine deals 2 damage to that artifact's controller.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Weight of Spires","R","Weight of Spires deals damage to target creature equal to the number of nonbasic lands that creature's controller controls.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pradesh Gypsies","1","1","G","{1}{G}, {T}: Target creature gets -2/-0 until end of turn.","2G","Creature - Human Nomad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Selesnya Guildmage","2","2","WG","{3}{G}: Put a 1/1 green Saproling creature token onto the battlefield.
{3}{W}: Creatures you control get +1/+1 until end of turn.","(G/W)(G/W)","Creature - Elf Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feast of Blood","B","Cast Feast of Blood only if you control two or more Vampires.
Destroy target creature. You gain 4 life.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mirri's Guile","G","At the beginning of your upkeep, you may look at the top three cards of your library, then put them back in any order.","G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sejiri Refuge","Sejiri Refuge enters the battlefield tapped.
When Sejiri Refuge enters the battlefield, you gain 1 life.
{T}: Add {W} or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Shimmer Myr","2","2","Flash
You may cast artifact cards as though they had flash.","3","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glimmerdust Nap","U","Enchant tapped creature
Enchanted creature doesn't untap during its controller's untap step.","2U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Attrition","B","{B}, Sacrifice a creature: Destroy target nonblack creature.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glarewielder","3","1","R","Haste
When Glarewielder enters the battlefield, up to two target creatures can't block this turn.
Evoke {1}{R} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","4R","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ironhoof Ox","4","4","G","Ironhoof Ox can't be blocked by more than one creature.","3GG","Creature - Ox");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spore Frog","1","1","G","Sacrifice Spore Frog: Prevent all combat damage that would be dealt this turn.","G","Creature - Frog");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Monomania","B","Target player chooses a card in his or her hand and discards the rest.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elven Warhounds","2","2","G","Whenever Elven Warhounds becomes blocked by a creature, put that creature on top of its owner's library.","3G","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thalia, Guardian of Thraben","2","1","W","First strike
Noncreature spells cost {1} more to cast.","1W","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Windrider Eel","2","2","U","Flying
Landfall - Whenever a land enters the battlefield under your control, Windrider Eel gets +2/+2 until end of turn.","3U","Creature - Fish");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Little Girl","{1/2}","{1/2}",NULL,"500","Creature - Human Child");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloud Crusader","2","3","W","Flying
First strike (This creature deals combat damage before creatures without first strike.)","2WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bearscape","G","{1}{G}, Exile two cards from your graveyard: Put a 2/2 green Bear creature token onto the battlefield.","1GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Spark","R","Death Spark deals 1 damage to target creature or player.
At the beginning of your upkeep, if Death Spark is in your graveyard with a creature card directly above it, you may pay {1}. If you do, return Death Spark to your hand.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lion's Eye Diamond","Sacrifice Lion's Eye Diamond, Discard your hand: Add three mana of any one color to your mana pool. Activate this ability only any time you could cast an instant.","0","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sulfur Falls","Sulfur Falls enters the battlefield tapped unless you control an Island or a Mountain.
{T}: Add {U} or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burning-Tree Shaman","3","4","RG","Whenever a player activates an ability that isn't a mana ability, Burning-Tree Shaman deals 1 damage to that player.","1RG","Creature - Centaur Shaman");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Manor Gargoyle","4","4","Defender
Manor Gargoyle is indestructible as long as it has defender.
{1}: Until end of turn, Manor Gargoyle loses defender and gains flying.","5","Artifact Creature - Gargoyle");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Echoing Decay","B","Target creature and all other creatures with the same name as that creature get -2/-2 until end of turn.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Takenuma Bleeder","3","3","B","Whenever Takenuma Bleeder attacks or blocks, you lose 1 life if you don't control a Demon.","2B","Creature - Ogre Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Beacon Hawk","1","1","W","Flying
Whenever Beacon Hawk deals combat damage to a player, you may untap target creature.
{W}: Beacon Hawk gets +0/+1 until end of turn.","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grinning Ignus","2","2","R","{R}, Return Grinning Ignus to its owner's hand: Add {2}{R} to your mana pool. Activate this ability only any time you could cast a sorcery.","2R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Liquify","U","Counter target spell with converted mana cost 3 or less. If that spell is countered this way, exile it instead of putting it into its owner's graveyard.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sek'Kuar, Deathkeeper","4","3","BRG","Whenever another nontoken creature you control dies, put a 3/1 black and red Graveborn creature token with haste onto the battlefield.","2BRG","Legendary Creature - Orc Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diversionary Tactics","W","Tap two untapped creatures you control: Tap target creature.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Artist","0","1","B","Whenever Blood Artist or another creature dies, target player loses 1 life and you gain 1 life.","1B","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flamekin Brawler","0","2","R","{R}: Flamekin Brawler gets +1/+0 until end of turn.","R","Creature - Elemental Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mindslaver","{4}, {T}, Sacrifice Mindslaver: You control target player during that player's next turn. (You see all cards that player could see and make all decisions for the player.)","6","Legendary Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Panglacial Wurm","9","5","G","Trample
While you're searching your library, you may cast Panglacial Wurm from your library.","5GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Privilege","B","Enchant creature
Enchanted creature gets +1/+1.
Sacrifice a creature: Regenerate enchanted creature.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Orcish Cannonade","R","Orcish Cannonade deals 2 damage to target creature or player and 3 damage to you.
Draw a card.","1RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud War Beast","*","*","G","Trample
As Skyshroud War Beast enters the battlefield, choose an opponent.
Skyshroud War Beast's power and toughness are each equal to the number of nonbasic lands the chosen player controls.","1G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infernal Plunge","R","As an additional cost to cast Infernal Plunge, sacrifice a creature.
Add {R}{R}{R} to your mana pool.","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Branching Bolt","RG","Choose one or both - Branching Bolt deals 3 damage to target creature with flying; and/or Branching Bolt deals 3 damage to target creature without flying.","1RG","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Aysen Abbey","{T}: Add {1} to your mana pool.
{1}, {T}: Add {W} to your mana pool.
{2}, {T}: Add {G} or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scandalmonger","3","3","B","{2}: Target player discards a card. Any player may activate this ability but only any time he or she could cast a sorcery.","3B","Creature - Boar Monger");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Marble Priest","3","3","All Walls able to block Marble Priest do so.
Prevent all combat damage that would be dealt to Marble Priest by Walls.","5","Artifact Creature - Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scaled Hulk","4","4","G","Whenever you cast a Spirit or Arcane spell, Scaled Hulk gets +2/+2 until end of turn.","5G","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Rack","As The Rack enters the battlefield, choose an opponent.
At the beginning of the chosen player's upkeep, The Rack deals X damage to that player, where X is 3 minus the number of cards in his or her hand.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Back from the Brink","U","Exile a creature card from your graveyard and pay its mana cost: Put a token onto the battlefield that's a copy of that card. Activate this ability only any time you could cast a sorcery.","4UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("World at War","R","After the first postcombat main phase this turn, there's an additional combat phase followed by an additional main phase. At the beginning of that combat, untap all creatures that attacked this turn.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","3RR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grafted Skullcap","At the beginning of your draw step, draw an additional card.
At the beginning of your end step, discard your hand.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mental Discipline","U","{1}{U}, Discard a card: Draw a card.","1UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breath of Malfegor","BR","Breath of Malfegor deals 5 damage to each opponent.","3BR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silent Assassin","2","1","B","{3}{B}: Destroy target blocking creature at end of combat.","BB","Creature - Human Mercenary Assassin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Inkmoth Nexus","{T}: Add {1} to your mana pool.
{1}: Inkmoth Nexus becomes a 1/1 Blinkmoth artifact creature with flying and infect until end of turn. It's still a land. (It deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Parapet Watchers","2","2","U","{(w/u)}: Parapet Watchers gets +0/+1 until end of turn.","2U","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Touch and Go","R","Destroy target land.
Gotcha Whenever an opponent touches his or her face, you may say ""Gotcha"" If you do, return Touch and Go from your graveyard to your hand.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dripping-Tongue Zubera","1","2","G","When Dripping-Tongue Zubera dies, put a 1/1 colorless Spirit creature token onto the battlefield for each Zubera that died this turn.","1G","Creature - Zubera Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ward Sliver","2","2","W","As Ward Sliver enters the battlefield, choose a color.
All Slivers have protection from the chosen color.","4W","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Reito Lantern","{3}: Put target card from a graveyard on the bottom of its owner's library.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Foreshadow","U","Name a card, then put the top card of target opponent's library into his or her graveyard. If that card is the named card, you draw a card.
Draw a card at the beginning of the next turn's upkeep.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost-Lit Warder","1","1","U","{3}{U}, {T}: Counter target spell unless its controller pays {2}.
Channel - {3}{U}, Discard Ghost-Lit Warder: Counter target spell unless its controller pays {4}.","1U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pulmonic Sliver","3","3","W","All Sliver creatures have flying.
All Slivers have ""If this permanent would be put into a graveyard, you may put it on top of its owner's library instead.""","3WW","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sphere of Reason","W","If a blue source would deal damage to you, prevent 2 of that damage.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Terastodon","9","9","G","When Terastodon enters the battlefield, you may destroy up to three target noncreature permanents. For each permanent put into a graveyard this way, its controller puts a 3/3 green Elephant creature token onto the battlefield.","6GG","Creature - Elephant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heartless Summoning","B","Creature spells you cast cost {2} less to cast.
Creatures you control get -1/-1.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidewater Minion","4","4","U","Defender (This creature can't attack.)
{4}: Tidewater Minion loses defender until end of turn.
{T}: Untap target permanent.","3UU","Creature - Elemental Minion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Furnace","{T}: Exile the bottom card of target player's graveyard.
{1}, Sacrifice Phyrexian Furnace: Exile target card from a graveyard. Draw a card.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mist of Stagnation","U","Permanents don't untap during their controllers' untap steps.
At the beginning of each player's upkeep, that player chooses a permanent for each card in his or her graveyard, then untaps those permanents.","3UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Power Taint","U","Enchant enchantment
At the beginning of the upkeep of enchanted enchantment's controller, that player loses 2 life unless he or she pays {2}.
Cycling {2} ({2}, Discard this card: Draw a card.)","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sedris, the Traitor King","5","5","UBR","Each creature card in your graveyard has unearth {2}{B}. ({2}{B}: Return the card to the battlefield. The creature gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","3UBR","Legendary Creature - Zombie Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Petrahydrox","3","3","UR","({(u/r)} can be paid with either {U} or {R}.)
When Petrahydrox becomes the target of a spell or ability, return Petrahydrox to its owner's hand.","3(U/R)","Creature - Weird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grave Titan","6","6","B","Deathtouch
Whenever Grave Titan enters the battlefield or attacks, put two 2/2 black Zombie creature tokens onto the battlefield.","4BB","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Genju of the Fens","B","Enchant Swamp
{2}: Until end of turn, enchanted Swamp becomes a 2/2 black Spirit creature with ""{B}: This creature gets +1/+1 until end of turn."" It's still a land.
When enchanted Swamp is put into a graveyard, you may return Genju of the Fens from your graveyard to your hand.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Molder Beast","5","3","G","Trample
Whenever an artifact is put into a graveyard from the battlefield, Molder Beast gets +2/+0 until end of turn.","4G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tower Above","G","({(2/g)} can be paid with any two mana or with {G}. This card's converted mana cost is 6.)
Until end of turn, target creature gets +4/+4 and gains trample, wither, and ""When this creature attacks, target creature blocks it this turn if able."" (It deals damage to creatures in the form of -1/-1 counters.)","(2/G)(2/G)(2/G)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("People of the Woods","1","*","G","People of the Woods's toughness is equal to the number of Forests you control.","GG","Creature - Human");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mystic Gate","{T}: Add {1} to your mana pool.
{(w/u)}, {T}: Add {W}{W}, {W}{U}, or {U}{U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Sarkhan the Mad","BR","0: Reveal the top card of your library and put it into your hand. Sarkhan the Mad deals damage to himself equal to that card's converted mana cost.
-2: Target creature's controller sacrifices it, then that player puts a 5/5 red Dragon creature token with flying onto the battlefield.
-4: Each Dragon creature you control deals damage equal to its power to target player.","Planeswalker - Sarkhan","3BR");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Ridgeback","2","3","G","Fading 2 (This creature enters the battlefield with two fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)","G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Complicate","U","Counter target spell unless its controller pays {3}.
Cycling {2}{U} ({2}{U}, Discard this card: Draw a card.)
When you cycle Complicate, you may counter target spell unless its controller pays {1}.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Annul","U","Counter target artifact or enchantment spell.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pixie Queen","1","1","G","Flying
{G}{G}{G}, {T}: Target creature gains flying until end of turn.","2GG","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghazbán Ogress","2","2","G","When Ghazbán Ogress comes into play, the player who has won the most Magic games that day gains control of it. If more than one player has won the same number of games, you retain control of Ghazbán Ogress.","G","Summon - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Acid Web Spider","3","5","G","Reach
When Acid Web Spider enters the battlefield, you may destroy target Equipment.","3GG","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pale Moon","U","Until end of turn, if a player taps a nonbasic land for mana, it produces colorless mana instead of any other type.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tideshaper Mystic","1","1","U","{T}: Target land becomes the basic land type of your choice until end of turn. Activate this ability only during your turn.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mox Diamond","If Mox Diamond would enter the battlefield, you may discard a land card instead. If you do, put Mox Diamond onto the battlefield. If you don't, put it into its owner's graveyard.
{T}: Add one mana of any color to your mana pool.","0","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lifeline","Whenever a creature dies, if another creature is on the battlefield, return the first card to the battlefield under its owner's control at the beginning of the next end step.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arctic Merfolk","1","1","U","Kicker-Return a creature you control to its owner's hand. (You may return a creature you control to its owner's hand in addition to any other costs as you cast this spell.)
If Arctic Merfolk was kicked, it enters the battlefield with a +1/+1 counter on it.","1U","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Dross Scorpion","3","1","Whenever Dross Scorpion or another artifact creature dies, you may untap target artifact.","4","Artifact Creature - Scorpion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barishi","4","3","G","When Barishi dies, exile Barishi, then shuffle all creature cards from your graveyard into your library.","2GG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Lieutenant","1","2","R","{1}{R}: Target Dwarf creature gets +1/+0 until end of turn.","RR","Creature - Dwarf Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cloudstone Curio","Whenever a nonartifact permanent enters the battlefield under your control, you may return another permanent you control that shares a card type with it to its owner's hand.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wintermoon Mesa","Wintermoon Mesa enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{2}, {T}, Sacrifice Wintermoon Mesa: Tap two target lands.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tivadar of Thorn","2","2","W","First strike, protection from red
When Tivadar of Thorn enters the battlefield, destroy target Goblin.","1WW","Legendary Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pulse of the Fields","W","You gain 4 life. Then if an opponent has more life than you, return Pulse of the Fields to its owner's hand.","1WW","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wheel of Fate","Suspend 4-{1}{R} (Rather than cast this card from your hand, pay {1}{R} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)
Each player discards his or her hand, then draws seven cards.",NULL,"Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathmask Nezumi","2","2","B","As long as you have seven or more cards in hand, Deathmask Nezumi gets +2/+1 and has fear. (It can't be blocked except by artifact creatures and/or black creatures.)","2B","Creature - Rat Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Deflection","W","Prevent the next X damage that would be dealt to you and/or permanents you control this turn. If damage is prevented this way, Divine Deflection deals that much damage to target creature or player.","XW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nether Horror","4","2","B",NULL,"3B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sphere of Purity","W","If an artifact would deal damage to you, prevent 1 of that damage.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Hexhunter","1","1","WG","{(g/w)}, {T}, Sacrifice Elvish Hexhunter: Destroy target enchantment.","(G/W)","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kill-Suit Cultist","1","1","R","Kill-Suit Cultist attacks each turn if able.
{B}, Sacrifice Kill-Suit Cultist: The next time damage would be dealt to target creature this turn, destroy that creature instead.","R","Creature - Goblin Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marisi's Twinclaws","2","4","WRG","Double strike","2(R/W)G","Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rimefeather Owl","*","*","U","Flying
Rimefeather Owl's power and toughness are each equal to the number of snow permanents on the battlefield.
{1}{S}i}: Put an ice counter on target permanent.
Permanents with ice counters on them are snow.","5UU","Snow Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heavy Arbalest","Equipped creature doesn't untap during its controller's untap step.
Equipped creature has ""{T}: This creature deals 2 damage to target creature or player.""
Equip {4}","3","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fetid Heath","{T}: Add {1} to your mana pool.
{(w/b)}, {T}: Add {W}{W}, {W}{B}, or {B}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Root Greevil","2","3","G","{2}{G}, {T}, Sacrifice Root Greevil: Destroy all enchantments of the color of your choice.","3G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zephid's Embrace","U","Enchant creature
Enchanted creature gets +2/+2 and has flying and shroud. (It can't be the target of spells or abilities.)","2UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Welder","1","1","R","{T}: Choose target artifact a player controls and target artifact card in that player's graveyard. If both targets are still legal as this ability resolves, that player simultaneously sacrifices the artifact and returns the artifact card to the battlefield.","R","Creature - Goblin Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Roar of the Crowd","R","Choose a creature type. Roar of the Crowd deals damage to target creature or player equal to the number of permanents you control of the chosen type.","3R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ark of Blight","{3}, {T}, Sacrifice Ark of Blight: Destroy target land.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caller of Gales","1","1","U","{1}{U}, {T}: Target creature gains flying until end of turn.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gift of the Gargantuan","G","Look at the top four cards of your library. You may reveal a creature card and/or a land card from among them and put the revealed cards into your hand. Put the rest on the bottom of your library in any order.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barrel Down Sokenzan","R","Sweep - Return any number of Mountains you control to their owner's hand. Barrel Down Sokenzan deals damage to target creature equal to twice the number of Mountains returned this way.","2R","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Contagion","B","You may pay 1 life and exile a black card from your hand rather than pay Contagion's mana cost.
Distribute two -2/-1 counters among one or two target creatures.","3BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragonmaster Outcast","1","1","R","At the beginning of your upkeep, if you control six or more lands, put a 5/5 red Dragon creature token with flying onto the battlefield.","R","Creature - Human Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Talisman of Indulgence","{T}: Add {1} to your mana pool.
{T}: Add {B} or {R} to your mana pool. Talisman of Indulgence deals 1 damage to you.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Spirit Guide","2","2","G","Exile Elvish Spirit Guide from your hand: Add {G} to your mana pool.","2G","Creature - Elf Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cadaverous Knight","2","2","B","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{1}{B}{B}: Regenerate Cadaverous Knight.","2B","Creature - Zombie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stone-Throwing Devils","1","1","B","First strike","B","Creature - Devil");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mimic Vat","Imprint - Whenever a nontoken creature dies, you may exile that card. If you do, return each other card exiled with Mimic Vat to its owner's graveyard.
{3}, {T}: Put a token onto the battlefield that's a copy of the exiled card. It gains haste. Exile it at the beginning of the next end step.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Barbs","R","Each enchantment deals 2 damage to its controller, then each Aura attached to a creature deals 2 damage to the creature it's attached to.","2R","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seahunter","2","2","U","{3}, {T}: Search your library for a Merfolk permanent card and put it onto the battlefield. Then shuffle your library.","2UU","Creature - Human Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kindle the Carnage","R","Discard a card at random. If you do, Kindle the Carnage deals damage equal to that card's converted mana cost to each creature. You may repeat this process any number of times.","1RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seton, Krosan Protector","2","2","G","Tap an untapped Druid you control: Add {G} to your mana pool.","GGG","Legendary Creature - Centaur Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of Lifeforce","G","If Leyline of Lifeforce is in your opening hand, you may begin the game with it on the battlefield.
Creature spells can't be countered.","2GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Twinning Glass","{1}, {T}: You may cast a nonland card from your hand without paying its mana cost if it has the same name as a spell that was cast this turn.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devastating Dreams","R","As an additional cost to cast Devastating Dreams, discard X cards at random.
Each player sacrifices X lands. Devastating Dreams deals X damage to each creature.","RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vengeful Vampire","3","2","B","Flying
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","4BB","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Far Wanderings","G","Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.
Threshold - If seven or more cards are in your graveyard, instead search your library for three basic land cards and put them onto the battlefield tapped. Then shuffle your library.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Natural Emergence","RG","When Natural Emergence enters the battlefield, return a red or green enchantment you control to its owner's hand.
Lands you control are 2/2 creatures with first strike. They're still lands.","2RG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Skyknight","1","1","W","Flying; first strike; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","2W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boldwyr Intimidator","5","5","R","Cowards can't block Warriors.
{R}: Target creature becomes a Coward until end of turn.
{2}{R}: Target creature becomes a Warrior until end of turn.","5RR","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glassdust Hulk","3","4","WU","Whenever another artifact enters the battlefield under your control, Glassdust Hulk gets +1/+1 until end of turn and is unblockable this turn.
Cycling {(w/u)} ({(w/u)}, Discard this card: Draw a card.)","3WU","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Tetravus","1","1","Flying
Tetravus enters the battlefield with three +1/+1 counters on it.
At the beginning of your upkeep, you may remove any number of +1/+1 counters from Tetravus. If you do, put that many 1/1 colorless Tetravite artifact creature tokens onto the battlefield. They each have flying and ""This creature can't be enchanted.""
At the beginning of your upkeep, you may exile any number of tokens put onto the battlefield with Tetravus. If you do, put that many +1/+1 counters on Tetravus.","6","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moss Monster","3","6","G",NULL,"3GG","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Splicer","{2}, {T}: Choose one - flying; first strike; shadow; or trample. Target creature with that ability loses it until end of turn and another target creature gains it until end of turn.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Predators","G","When an opponent controls a creature with power 4 or greater, if Hidden Predators is an enchantment, Hidden Predators becomes a 4/4 Beast creature.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Riding Red Hare","W","Target creature gets +3/+3 and gains horsemanship until end of turn. (It can't be blocked except by creatures with horsemanship.)","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dust to Dust","W","Exile two target artifacts.","1WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avatar of Me","*","*","U","Avatar of Me costs {1} more to play for each ten years you've been alive.
Avatar of Me's power is equal to your height in feet and its toughness is equal to your American shoe size. Round to the nearest {1/2}.
Avatar of Me's color is the color of your eyes.","2UU","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skywatcher Adept","1","1","U","Level up {3} ({3}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-2
2/2
Flying
LEVEL 3+
4/2
Flying","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ayesha Tanaka","2","2","WU","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
{T}: Counter target activated ability from an artifact source unless that ability's controller pays {W}. (Mana abilities can't be targeted.)","WWUU","Legendary Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enormous Baloth","7","7","G",NULL,"6G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of Ancient Law","2","2","W","Sacrifice Kami of Ancient Law: Destroy target enchantment.","1W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Gate Loremaster","1","3","U","{T}: Draw a card for each Ally you control.","4U","Creature - Merfolk Wizard Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cognivore","*","*","U","Flying
Cognivore's power and toughness are each equal to the number of instant cards in all graveyards.","6UU","Creature - Lhurgoyf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sol'kanar the Swamp King","5","5","UBR","Swampwalk
Whenever a player casts a black spell, you gain 1 life.","2UBR","Legendary Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spelltithe Enforcer","3","3","W","Whenever an opponent casts a spell, that player sacrifices a permanent unless he or she pays {1}.","3WW","Creature - Elephant Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("High Tide","U","Until end of turn, whenever a player taps an Island for mana, that player adds {U} to his or her mana pool (in addition to the mana the land produces).","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Typhoid Rats","1","1","B","Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pang Tong, ""Young Phoenix""","1","2","W","{T}: Target creature gets +0/+2 until end of turn. Activate this ability only during your turn, before attackers are declared.","1WW","Legendary Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spiritual Sanctuary","W","At the beginning of each player's upkeep, if that player controls a Plains, he or she gains 1 life.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Denial","U","Counter target creature or sorcery spell.","1UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Acorn Harvest","G","Put two 1/1 green Squirrel creature tokens onto the battlefield.
Flashback-{1}{G}, Pay 3 life. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wirewood Symbiote","1","1","G","Return an Elf you control to its owner's hand: Untap target creature. Activate this ability only once each turn.","G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shadow Rider","3","3","B","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)","2BB","Creature - Zombie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Huang Zhong, Shu General","2","3","W","Huang Zhong, Shu General can't be blocked by more than one creature.","2WW","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Taste for Mayhem","R","Enchant creature
Enchanted creature gets +2/+0.
Hellbent - Enchanted creature gets an additional +2/+0 as long as you have no cards in hand.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse Artifact","B","Enchant artifact
At the beginning of the upkeep of enchanted artifact's controller, Curse Artifact deals 2 damage to that player unless he or she sacrifices that artifact.","2BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hundroog","4","7","G","Cycling {3} ({3}, Discard this card: Draw a card.)","6G","Creature - Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Pewter Golem","4","2","{1}{B}: Regenerate Pewter Golem.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Wurm","7","7","RG","Trample
When Shivan Wurm enters the battlefield, return a red or green creature you control to its owner's hand.","3RG","Creature - Wurm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Khalni Gem","When Khalni Gem enters the battlefield, return two lands you control to their owner's hand.
{T}: Add two mana of any one color to your mana pool.","4","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Adarkar Sentinel","3","3","{1}: Adarkar Sentinel gets +0/+1 until end of turn.","5","Artifact Creature - Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Artifact Possession","B","Enchant artifact
Whenever enchanted artifact becomes tapped or a player activates an ability of enchanted artifact without {T} in its activation cost, Artifact Possession deals 2 damage to that artifact's controller.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blade of the Sixth Pride","3","1","W",NULL,"1W","Creature - Cat Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necrotic Ooze","4","3","B","As long as Necrotic Ooze is on the battlefield, it has all activated abilities of all creature cards in all graveyards.","2BB","Creature - Ooze");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pulsating Illusion","0","1","U","Flying
Discard a card: Pulsating Illusion gets +4/+4 until end of turn. Activate this ability only once each turn.","4U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodthirsty Ogre","3","1","B","{T}: Put a devotion counter on Bloodthirsty Ogre.
{T}: Target creature gets -X/-X until end of turn, where X is the number of devotion counters on Bloodthirsty Ogre. Activate this ability only if you control a Demon.","2B","Creature - Ogre Warrior Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu Defender","2","2","W","Whenever Shu Defender blocks, it gets +0/+2 until end of turn.","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recover","B","Return target creature card from your graveyard to your hand.
Draw a card.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Searing Meditation","WR","Whenever you gain life, you may pay {2}. If you do, Searing Meditation deals 2 damage to target creature or player.","1RW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Patrol","2","1","R","Echo {R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sink into Takenuma","B","Sweep - Return any number of Swamps you control to their owner's hand. Target player discards a card for each Swamp returned this way.","3B","Sorcery - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Glaivemaster","1","1","W","As long as Auriok Glaivemaster is equipped, it gets +1/+1 and has first strike.","W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jolrael's Favor","G","Flash
Enchant creature
{1}{G}: Regenerate enchanted creature.","1G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Forgotten Cave","Forgotten Cave enters the battlefield tapped.
{T}: Add {R} to your mana pool.
Cycling {R} ({R}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wellgabber Apothecary","2","3","W","{1}{W}: Prevent all damage that would be dealt to target tapped Merfolk or Kithkin creature this turn.","4W","Creature - Merfolk Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Active Volcano","R","Choose one - Destroy target blue permanent; or return target Island to its owner's hand.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("City of Solitude","G","Players can cast spells and activate abilities only during their own turns.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ichor Rats","2","1","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
When Ichor Rats enters the battlefield, each player gets a poison counter.","1BB","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forbidden Alchemy","U","Look at the top four cards of your library. Put one of them into your hand and the rest into your graveyard.
Flashback {6}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Handservant","1","1","G","Whenever a player casts a Giant spell, you may put a +1/+1 counter on Elvish Handservant.","G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Desert Nomads","2","2","R","Desertwalk
Prevent all damage that would be dealt to Desert Nomads by Deserts.","2R","Creature - Human Nomad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jacques le Vert","3","2","WRG","Green creatures you control get +0/+2.","1RGW","Legendary Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Amrou Scout","2","1","W","{4}, {T}: Search your library for a Rebel permanent card with converted mana cost 3 or less and put it onto the battlefield. Then shuffle your library.","1W","Creature - Kithkin Rebel Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gustha's Scepter","{T}: Exile a card from your hand face down. You may look at it for as long as it remains exiled.
{T}: Return a card you own exiled with Gustha's Scepter to your hand.
When you lose control of Gustha's Scepter, put all cards exiled with Gustha's Scepter into their owner's graveyard.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporary Insanity","R","Untap target creature with power less than the number of cards in your graveyard and gain control of it until end of turn. That creature gains haste until end of turn.","3R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silver Erne","2","2","U","Flying, trample","3U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infernal Tutor","B","Reveal a card from your hand. Search your library for a card with the same name as that card, reveal it, put it into your hand, then shuffle your library.
Hellbent - If you have no cards in hand, instead search your library for a card, put it into your hand, then shuffle your library.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gideon's Avenger","2","2","W","Whenever a creature an opponent controls becomes tapped, put a +1/+1 counter on Gideon's Avenger.","1WW","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashnod's Transmogrant","{T}, Sacrifice Ashnod's Transmogrant: Put a +1/+1 counter on target nonartifact creature. That creature becomes an artifact in addition to its other types.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wildwood Geist","3","3","G","Wildwood Geist gets +2/+2 as long as it's your turn.","4G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guardian Angel","W","Prevent the next X damage that would be dealt to target creature or player this turn. Until end of turn, you may pay {1} any time you could cast an instant. If you do, prevent the next 1 damage that would be dealt to that creature or player this turn.","XW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dismantle","R","Destroy target artifact. If that artifact had counters on it, put that many +1/+1 counters or charge counters on an artifact you control.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blind Fury","R","Cast Blind Fury only before the combat damage step.
All creatures lose trample until end of turn. If a creature would deal combat damage to a creature this turn, it deals double that damage to that creature instead.","2RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necrobite","B","Target creature gains deathtouch until end of turn. Regenerate it.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Runeflare Trap","R","If an opponent drew three or more cards this turn, you may pay {R} rather than pay Runeflare Trap's mana cost.
Runeflare Trap deals damage to target player equal to the number of cards in that player's hand.","4RR","Instant - Trap");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fuel for the Cause","U","Counter target spell, then proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","2UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Specter's Wail","B","Target player discards a card at random.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shape of the Wiitigo","G","Enchant creature
When Shape of the Wiitigo enters the battlefield, put six +1/+1 counters on enchanted creature.
At the beginning of your upkeep, put a +1/+1 counter on enchanted creature if it attacked or blocked since your last upkeep. Otherwise, remove a +1/+1 counter from it.","3GGG","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fathom Trawl","U","Reveal cards from the top of your library until you reveal three nonland cards. Put the nonland cards revealed this way into your hand, then put the rest of the revealed cards on the bottom of your library in any order.","3UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Vault","{2}, {T}, Sacrifice a creature: Draw a card.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brush with Death","B","Buyback {2}{B}{B} (You may pay an additional {2}{B}{B} as you cast this spell. If you do, put this card into your hand as it resolves.)
Target opponent loses 2 life. You gain 2 life.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Elf","1","1","G","{T}: Add {G} to your mana pool.
{1}: Add {R} or {W} to your mana pool.","1G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shade's Form","B","Enchant creature
Enchanted creature has ""{B}: This creature gets +1/+1 until end of turn.""
When enchanted creature dies, return that card to the battlefield under your control.","1BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vulshok Berserker","3","2","R","Haste (This creature can attack and {T} as soon as it comes under your control.)","3R","Creature - Human Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avenging Angel","3","3","W","Flying
When Avenging Angel dies, you may put it on top of its owner's library.","3WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sindbad","1","1","U","{T}: Draw a card and reveal it. If it isn't a land card, discard it.","1U","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drana, Kalastria Bloodchief","4","4","B","Flying
{X}{B}{B}: Target creature gets -0/-X until end of turn and Drana, Kalastria Bloodchief gets +X/+0 until end of turn.","3BB","Legendary Creature - Vampire Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pledge of Loyalty","W","Enchant creature
Enchanted creature has protection from the colors of permanents you control. This effect doesn't remove Pledge of Loyalty.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Limited Resources","W","When Limited Resources enters the battlefield, each player chooses five lands he or she controls and sacrifices the rest.
Players can't play lands as long as ten or more lands are on the battlefield.","W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heat Wave","R","Cumulative upkeep {R} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Blue creatures can't block creatures you control.
Nonblue creatures can't block creatures you control unless their controller pays 1 life for each blocking creature he or she controls.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Garruk's Packleader","4","4","G","Whenever another creature with power 3 or greater enters the battlefield under your control, you may draw a card.","4G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fyndhorn Elves","1","1","G","{T}: Add {G} to your mana pool.","G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Baleful Stare","U","Target opponent reveals his or her hand. You draw a card for each Mountain and red card in it.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ritual of the Machine","B","As an additional cost to cast Ritual of the Machine, sacrifice a creature.
Gain control of target nonartifact, nonblack creature.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Outrider","4","3","R","Echo {2}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","2R","Creature - Viashino");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lullmage Mentor","2","2","U","Whenever a spell or ability you control counters a spell, you may put a 1/1 blue Merfolk creature token onto the battlefield.
Tap seven untapped Merfolk you control: Counter target spell.","1UU","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindwarper","0","0","B","Mindwarper enters the battlefield with three +1/+1 counters on it.
{2}{B}, Remove a +1/+1 counter from Mindwarper: Target player discards a card. Activate this ability only any time you could cast a sorcery.","2BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Commando","2","1","R","Whenever Skirk Commando deals combat damage to a player, you may have it deal 2 damage to target creature that player controls.
Morph {2}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1RR","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hidden Horror","4","4","B","When Hidden Horror enters the battlefield, sacrifice it unless you discard a creature card.","1BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx Sovereign","6","6","WUB","Flying
At the beginning of your end step, you gain 3 life if Sphinx Sovereign is untapped. Otherwise, each opponent loses 3 life.","4WUUB","Artifact Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master of Arms","2","2","W","First strike
{1}{W}: Tap target creature blocking Master of Arms.","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("War Report","W","You gain life equal to the number of creatures on the battlefield plus the number of artifacts on the battlefield.","3W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Solarion","0","0","Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it.)
{T}: Double the number of +1/+1 counters on Solarion.","7","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sliver Queen, Brood Mother","{3}: Put a 1/1 colorless Sliver creature token onto the battlefield.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragon Fodder","R","Put two 1/1 red Goblin creature tokens onto the battlefield.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Monk Idealist","2","2","W","When Monk Idealist enters the battlefield, return target enchantment card from your graveyard to your hand.","2W","Creature - Human Monk Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Favor","W","Enchant creature
When Divine Favor enters the battlefield, you gain 3 life.
Enchanted creature gets +1/+3.","1W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Boros Garrison","Boros Garrison enters the battlefield tapped.
When Boros Garrison enters the battlefield, return a land you control to its owner's hand.
{T}: Add {R}{W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Duress","B","Target opponent reveals his or her hand. You choose a noncreature, nonland card from it. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Okiba-Gang Shinobi","3","2","B","Ninjutsu {3}{B} ({3}{B}, Return an unblocked attacker you control to hand: Put this card onto the battlefield from your hand tapped and attacking.)
Whenever Okiba-Gang Shinobi deals combat damage to a player, that player discards two cards.","3BB","Creature - Rat Ninja");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul Scourge","3","2","B","Flying
When Soul Scourge enters the battlefield, target player loses 3 life.
When Soul Scourge leaves the battlefield, that player gains 3 life.","4B","Creature - Nightmare Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Soldevi Digger","{2}: Put the top card of your graveyard on the bottom of your library.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pain // Suffering","B","Target player discards a card.
---
Destroy target land.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Simoon","RG","Simoon deals 1 damage to each creature target opponent controls.","RG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call to Glory","W","Untap all creatures you control. Samurai creatures you control get +1/+1 until end of turn.","1W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Brass Man","1","3","Brass Man doesn't untap during your untap step.
At the beginning of your upkeep, you may pay {1}. If you do, untap Brass Man.","1","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cryptic Command","U","Choose two - Counter target spell; or return target permanent to its owner's hand; or tap all creatures your opponents control; or draw a card.","1UUU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kuldotha Ringleader","4","4","R","Battle cry (Whenever this creature attacks, each other attacking creature gets +1/+0 until end of turn.)
Kuldotha Ringleader attacks each turn if able.","4R","Creature - Giant Berserker");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lapis Lazuli Talisman","Whenever a player casts a blue spell, you may pay {3}. If you do, untap target permanent.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jilt","U","Kicker {1}{R} (You may pay an additional {1}{R} as you cast this spell.)
Return target creature to its owner's hand. If Jilt was kicked, it deals 2 damage to another target creature.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blood Crypt","({T}: Add {B} or {R} to your mana pool.)
As Blood Crypt enters the battlefield, you may pay 2 life. If you don't, Blood Crypt enters the battlefield tapped.",NULL,"Land - Swamp Mountain");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Garruk, the Veil-Cursed","+1: Put a 1/1 black Wolf creature token with deathtouch onto the battlefield.
-1: Sacrifice a creature. If you do, search your library for a creature card, reveal it, put it into your hand, then shuffle your library.
-3: Creatures you control gain trample and get +X/+X until end of turn, where X is the number of creature cards in your graveyard.",NULL,"Planeswalker - Garruk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pretender's Claim","B","Enchant creature
Whenever enchanted creature becomes blocked, tap all lands defending player controls.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hideous Visage","B","Creatures you control gain intimidate until end of turn. (Each of those creatures can't be blocked except by artifact creatures and/or creatures that share a color with it.)","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magma Phoenix","3","3","R","Flying
When Magma Phoenix dies, it deals 3 damage to each creature and each player.
{3}{R}{R}: Return Magma Phoenix from your graveyard to your hand.","3RR","Creature - Phoenix");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ovinomancer","0","1","U","When Ovinomancer enters the battlefield, sacrifice it unless you return three basic lands you control to their owner's hand.
{T}, Return Ovinomancer to its owner's hand: Destroy target creature. It can't be regenerated. That creature's controller puts a 0/1 green Sheep creature token onto the battlefield.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Raiders","2","3","U","Islandwalk
Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)","1U","Creature - Merfolk Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spined Basher","3","1","B","Morph {2}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2B","Creature - Zombie Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jukai Messenger","1","1","G","Forestwalk","G","Creature - Human Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Genju of the Spires","R","Enchant Mountain
{2}: Enchanted Mountain becomes a 6/1 red Spirit creature until end of turn. It's still a land.
When enchanted Mountain is put into a graveyard, you may return Genju of the Spires from your graveyard to your hand.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deep Reconnaissance","G","Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.
Flashback {4}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loyal Gyrfalcon","3","3","W","Defender, flying
Whenever you cast a white spell, Loyal Gyrfalcon loses defender until end of turn.","3W","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Upheaval","U","Return all permanents to their owners' hands.","4UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spiritual Focus","W","Whenever a spell or ability an opponent controls causes you to discard a card, you gain 2 life and you may draw a card.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dream Stalker","1","5","U","When Dream Stalker enters the battlefield, return a permanent you control to its owner's hand.","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Emerge Unscathed","W","Target creature you control gains protection from the color of your choice until end of turn.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boldwyr Heavyweights","8","8","R","Trample
When Boldwyr Heavyweights enters the battlefield, each opponent may search his or her library for a creature card and put it onto the battlefield. Then each player who searched his or her library this way shuffles it.","2RR","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepcavern Imp","2","2","B","Flying, haste
Echo-Discard a card. (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","2B","Creature - Imp Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guard Duty","W","Enchant creature
Enchanted creature has defender.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exploding Borders","RG","Domain - Search your library for a basic land card, put that card onto the battlefield tapped, then shuffle your library. Exploding Borders deals X damage to target player, where X is the number of basic land types among lands you control.","2RG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Samite Archer","1","1","WU","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.
{T}: Samite Archer deals 1 damage to target creature or player.","1WU","Creature - Human Cleric Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rabid Elephant","3","4","G","Whenever Rabid Elephant becomes blocked, it gets +2/+2 until end of turn for each creature blocking it.","4G","Creature - Elephant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tremble","R","Each player sacrifices a land.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Scroll","1","1","R","{3}, {T}: Name a card. Reveal a card at random from your hand. If it's the named card, Magus of the Scroll deals 2 damage to target creature or player.","R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magma Spray","R","Magma Spray deals 2 damage to target creature. If that creature would die this turn, exile it instead.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ray of Command","U","Untap target creature an opponent controls and gain control of it until end of turn. That creature gains haste until end of turn. When you lose control of the creature, tap it.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Bloodboiler","2","2","R","Tap an untapped Dwarf you control: Target creature gets +2/+0 until end of turn.","RRR","Creature - Dwarf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Barbed Battlegear","Equipped creature gets +4/-1.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nova Pentacle","{3}, {T}: The next time a source of your choice would deal damage to you this turn, that damage is dealt to target creature of an opponent's choice instead.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fallow Wurm","4","4","G","When Fallow Wurm enters the battlefield, sacrifice it unless you discard a land card.","2G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dross Crocodile","5","1","B",NULL,"3B","Creature - Zombie Crocodile");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goldnight Redeemer","4","4","W","Flying
When Goldnight Redeemer enters the battlefield, you gain 2 life for each other creature you control.","4WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argothian Elder","2","2","G","{T}: Untap two target lands.","3G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Empyrial Armor","W","Enchant creature
Enchanted creature gets +1/+1 for each card in your hand.","1WW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauntless Escort","3","3","WG","Sacrifice Dauntless Escort: Creatures you control are indestructible this turn.","1GW","Creature - Rhino Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ertai's Meddling","U","X can't be 0.
Target spell's controller exiles it with X delay counters on it.
At the beginning of each of that player's upkeeps, if that card is exiled, remove a delay counter from it. If the card has no delay counters on it, he or she puts it onto the stack as a copy of the original spell.","XU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Your Fate Is Thrice Sealed","When you set this scheme in motion, reveal the top three cards of your library. Put all land cards revealed this way onto the battlefield and the rest into your hand.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gnarled Mass","3","3","G",NULL,"1GG","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hematite Talisman","Whenever a player casts a red spell, you may pay {3}. If you do, untap target permanent.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wurmskin Forger","2","2","G","When Wurmskin Forger enters the battlefield, distribute three +1/+1 counters among one, two, or three target creatures.","5GG","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Griffin","2","2","W","Flying","2W","Creature - Griffin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grixis Panorama","{T}: Add {1} to your mana pool.
{1}, {T}, Sacrifice Grixis Panorama: Search your library for a basic Island, Swamp, or Mountain card and put it onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Krark-Clan Ironworks","Sacrifice an artifact: Add {2} to your mana pool.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Wizard","1","1","R","{T}: You may put a Goblin permanent card from your hand onto the battlefield.
{R}: Target Goblin gains protection from white until end of turn.","2RR","Creature - Goblin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pardic Collaborator","2","2","R","First strike
{B}: Pardic Collaborator gets +1/+1 until end of turn.","3R","Creature - Human Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hisoka's Defiance","U","Counter target Spirit or Arcane spell.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Terravore","*","*","G","Trample
Terravore's power and toughness are each equal to the number of land cards in all graveyards.","1GG","Creature - Lhurgoyf");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arachnoid","2","6","Reach (This creature can block creatures with flying.)","6","Artifact Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kazuul Warlord","3","3","R","Whenever Kazuul Warlord or another Ally enters the battlefield under your control, you may put a +1/+1 counter on each Ally creature you control.","4R","Creature - Minotaur Warrior Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Creeping Renaissance","G","Choose a permanent type. Return all cards of the chosen type from your graveyard to your hand.
Flashback {5}{G}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Reverence","1","6","W","Defender, flying
At the beginning of your end step, you may gain life equal to the power of target creature you control.","3W","Creature - Spirit Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overwhelming Instinct","G","Whenever you attack with three or more creatures, draw a card.","2G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trinisphere","As long as Trinisphere is untapped, each spell that would cost less than three mana to cast costs three mana to cast. (Additional mana in the cost may be paid with any color of mana or colorless mana. For example, a spell that would cost {1}{B} to cast costs {2}{B} to cast instead.)","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wreak Havoc","RG","Wreak Havoc can't be countered by spells or abilities.
Destroy target artifact or land.","2RG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Locust Miser","2","2","B","Each opponent's maximum hand size is reduced by two.","2BB","Creature - Rat Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silkenfist Fighter","1","3","W","Whenever Silkenfist Fighter becomes blocked, untap it.","1W","Creature - Kor Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rakalite","{2}: Prevent the next 1 damage that would be dealt to target creature or player this turn. Return Rakalite to its owner's hand at the beginning of the next end step.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ravenous Demon","4","4","B","Sacrifice a Human: Transform Ravenous Demon. Activate this ability only any time you could cast a sorcery.","3BB","Creature - Demon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jeweled Amulet","{1}, {T}: Put a charge counter on Jeweled Amulet. Note the type of mana spent to pay this activation cost. Activate this ability only if there are no charge counters on Jeweled Amulet.
{T}, Remove a charge counter from Jeweled Amulet: Add one mana of Jeweled Amulet's last noted type to your mana pool.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Immaculate Magistrate","2","2","G","{T}: Put a +1/+1 counter on target creature for each Elf you control.","3G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fishliver Oil","U","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature has islandwalk. (This creature is unblockable as long as defending player controls an Island.)","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thunderheads","U","Replicate {2}{U} (When you cast this spell, copy it for each time you paid its replicate cost.)
Put a 3/3 blue Weird creature token with defender and flying onto the battlefield. Exile it at the beginning of the next end step.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thalakos Deceiver","1","1","U","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever Thalakos Deceiver attacks and isn't blocked, you may sacrifice it. If you do, gain control of target creature. (This effect lasts indefinitely.)","3U","Creature - Thalakos Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Meadowboon","3","3","W","When Meadowboon leaves the battlefield, put a +1/+1 counter on each creature target player controls.
Evoke {3}{W} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","2WW","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spire Serpent","3","5","U","Defender
Metalcraft - As long as you control three or more artifacts, Spire Serpent gets +2/+2 and can attack as though it didn't have defender.","4U","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dermoplasm","1","1","U","Flying
Morph {2}{U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Dermoplasm is turned face up, you may put a creature card with morph from your hand onto the battlefield face up. If you do, return Dermoplasm to its owner's hand.","2U","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Countryside Crusher","3","3","R","At the beginning of your upkeep, reveal the top card of your library. If it's a land card, put it into your graveyard and repeat this process.
Whenever a land card is put into your graveyard from anywhere, put a +1/+1 counter on Countryside Crusher.","1RR","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhys the Exiled","3","2","G","Whenever Rhys the Exiled attacks, you gain 1 life for each Elf you control.
{B}, Sacrifice an Elf: Regenerate Rhys the Exiled.","2G","Legendary Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Purify the Grave","W","Exile target card from a graveyard.
Flashback {W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stalking Assassin","1","1","UB","{3}{U}, {T}: Tap target creature.
{3}{B}, {T}: Destroy target tapped creature.","1UB","Creature - Human Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drift of the Dead","*","*","B","Defender (This creature can't attack.)
Drift of the Dead's power and toughness are each equal to the number of snow lands you control.","3B","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shizo, Death's Storehouse","{T}: Add {B} to your mana pool.
{B}, {T}: Target legendary creature gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Toshiro Umezawa","2","2","B","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
Whenever a creature an opponent controls dies, you may cast target instant card from your graveyard. If that card would be put into a graveyard this turn, exile it instead.","1BB","Legendary Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gravitational Shift","U","Creatures with flying get +2/+0.
Creatures without flying get -2/-0.","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ebon Drake","3","3","B","Flying
Whenever a player casts a spell, you lose 1 life.","2B","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Colfenor's Plans","B","When Colfenor's Plans enters the battlefield, exile the top seven cards of your library face down.
You may look at and play cards exiled with Colfenor's Plans.
Skip your draw step.
You can't cast more than one spell each turn.","2BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Altar's Light","W","Exile target artifact or enchantment.","2WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Worldly Tutor","G","Search your library for a creature card and reveal that card. Shuffle your library, then put the card on top of it.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("War Priest of Thune","2","2","W","When War Priest of Thune enters the battlefield, you may destroy target enchantment.","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burrenton Shield-Bearers","3","3","W","Whenever Burrenton Shield-Bearers attacks, target creature gets +0/+3 until end of turn.","4W","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wipe Clean","W","Exile target enchantment.
Cycling {3} ({3}, Discard this card: Draw a card.)","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Persecute Artist","B","Choose an artist other than Rebecca Guay. Target player reveals his or her hand and discards all nonland cards by the chosen artist.","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dwarven Armory","R","{2}, Sacrifice a land: Put a +2/+2 counter on target creature. Activate this ability only during any upkeep step.","2RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frightful Delusion","U","Counter target spell unless its controller pays {1}. That player discards a card.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Misthollow Griffin","3","3","U","Flying
You may cast Misthollow Griffin from exile.","2UU","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fledgling Mawcor","2","2","U","Flying
{T}: Fledgling Mawcor deals 1 damage to target creature or player.
Morph {U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3U","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leery Fogbeast","4","2","G","Whenever Leery Fogbeast becomes blocked, prevent all combat damage that would be dealt this turn.","2G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chord of Calling","G","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Search your library for a creature card with converted mana cost X or less and put it onto the battlefield. Then shuffle your library.","XGGG","Instant");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Tamiyo, the Moon Sage","U","+1: Tap target permanent. It doesn't untap during its controller's next untap step.
-2: Draw a card for each tapped creature target player controls.
-8: You get an emblem with ""You have no maximum hand size"" and ""Whenever a card is put into your graveyard from anywhere, you may return it to your hand.""","Planeswalker - Tamiyo","3UU");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Resurrection","W","Return target creature card from your graveyard to the battlefield.","2WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tracker","2","2","G","{G}{G}, {T}: Tracker deals damage equal to its power to target creature. That creature deals damage equal to its power to Tracker.","2G","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Troll","3","3","G","{1}{G}: Regenerate Skyshroud Troll.","2GG","Creature - Troll Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Storm Elemental","3","4","U","Flying
{U}, Exile the top card of your library: Tap target creature with flying.
{U}, Exile the top card of your library: If the exiled card is a snow land, Storm Elemental gets +1/+1 until end of turn.","5U","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Juvenile Gloomwidow","1","3","G","Reach (This can block creatures with flying.)
Wither (This deals damage to creatures in the form of -1/-1 counters.)","GG","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eland Umbra","W","Enchant creature
Enchanted creature gets +0/+4.
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","1W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Magma Mine","{4}: Put a pressure counter on Magma Mine.
{T}, Sacrifice Magma Mine: Magma Mine deals damage equal to the number of pressure counters on it to target creature or player.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angry Mob","2+*","2+*","W","Trample
As long as it's your turn, Angry Mob's power and toughness are each equal to 2 plus the number of Swamps your opponents control. As long as it's not your turn, Angry Mob's power and toughness are each 2.","2WW","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tel-Jilad Chosen","2","1","G","Protection from artifacts","1G","Creature - Elf Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Echo Circlet","Equipped creature can block an additional creature.
Equip {1}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Pine Needles","3","3","G","Defender (This creature can't attack.)
{G}: Regenerate Wall of Pine Needles.","3G","Creature - Plant Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wingrattle Scarecrow","2","2","Wingrattle Scarecrow has flying as long as you control a blue creature.
Wingrattle Scarecrow has persist as long as you control a black creature. (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","3","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Druid of the Anima","1","1","G","{T}: Add {R}, {G}, or {W} to your mana pool.","1G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Miracle Worker","1","1","W","{T}: Destroy target Aura attached to a creature you control.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Chariot","2","2","R","Haste (This creature can attack the turn it comes under your control.)","2R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Void","BR","Choose a number. Destroy all artifacts and creatures with converted mana cost equal to that number. Then target player reveals his or her hand and discards all nonland cards with converted mana cost equal to the number.","3BR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cone of Flame","R","Cone of Flame deals 1 damage to target creature or player, 2 damage to another target creature or player, and 3 damage to a third target creature or player.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Molten Sentry","*","*","R","As Molten Sentry enters the battlefield, flip a coin. If the coin comes up heads, Molten Sentry enters the battlefield as a 5/2 creature with haste. If it comes up tails, Molten Sentry enters the battlefield as a 2/5 creature with defender.","3R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Desecrator Hag","2","2","BG","When Desecrator Hag enters the battlefield, return to your hand the creature card in your graveyard with the greatest power. If two or more cards are tied for greatest power, you choose one of them.","2(B/G)(B/G)","Creature - Hag");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soulshriek","B","Target creature you control gets +X/+0 until end of turn, where X is the number of creature cards in your graveyard. Sacrifice that creature at the beginning of the next end step.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nath's Buffoon","1","1","B","Protection from Elves","1B","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moonglove Changeling","2","2","B","Changeling (This card is every creature type at all times.)
{B}: Moonglove Changeling gains deathtouch until end of turn. (Any amount of damage it deals to a creature is enough to destroy that creature.)","2B","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barrin's Unmaking","U","Return target permanent to its owner's hand if that permanent shares a color with the most common color among all permanents or a color tied for most common.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AWOL","W","Remove target attacking creature from the game. Then remove it from the removed-from-game zone and put it into the absolutely-removed-from-the-freaking-game-forever zone.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Greater Realm of Preservation","W","{1}{W}: The next time a black or red source of your choice would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peach Garden Oath","W","You gain 2 life for each creature you control.","W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Restraints","U","Flash
Enchant creature
When Mystic Restraints enters the battlefield, tap enchanted creature.
Enchanted creature doesn't untap during its controller's untap step.","2UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lu Su, Wu Advisor","1","2","U","{T}: Draw a card. Activate this ability only during your turn, before attackers are declared.","3UU","Legendary Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrion Rats","2","1","B","Whenever Carrion Rats attacks or blocks, any player may exile a card from his or her graveyard. If a player does, Carrion Rats assigns no combat damage this turn.","B","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necropotence","B","Skip your draw step.
Whenever you discard a card, exile that card from your graveyard.
Pay 1 life: Exile the top card of your library face down. Put that card into your hand at the beginning of your next end step.","BBB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Willow Faerie","1","2","G","Flying","1G","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kird Ape","1","1","R","Kird Ape gets +1/+2 as long as you control a Forest.","R","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marsh Goblins","1","1","BR","Swampwalk","BR","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steward of Valeron","2","2","WG","Vigilance
{T}: Add {G} to your mana pool.","GW","Creature - Human Druid Knight");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Wanderer","0","0","Modular-Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it. When it dies, you may put its +1/+1 counters on target artifact creature.)","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Scion","4","4","G","Protection from artifacts","4G","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Regenerate","G","Regenerate target creature. (The next time that creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("City of Brass","Whenever City of Brass becomes tapped, it deals 1 damage to you.
{T}: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bloodshed Fever","R","Enchant creature
Enchanted creature attacks each turn if able.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thought Lash","U","Cumulative upkeep-Exile the top card of your library. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When a player doesn't pay Thought Lash's cumulative upkeep, that player exiles all cards from his or her library.
Exile the top card of your library: Prevent the next 1 damage that would be dealt to you this turn.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Safehold Sentry","2","2","W","{2}{W}, {Q}: Safehold Sentry gets +0/+2 until end of turn. ({Q} is the untap symbol.)","1W","Creature - Elf Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bone Flute","{2}, {T}: All creatures get -1/-0 until end of turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frost Raptor","2","2","U","Flying
{S}i{S}i}: Frost Raptor gains shroud until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","2U","Snow Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phylactery Lich","5","5","B","As Phylactery Lich enters the battlefield, put a phylactery counter on an artifact you control.
Phylactery Lich is indestructible.
When you control no permanents with phylactery counters on them, sacrifice Phylactery Lich.","BBB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Human Frailty","B","Destroy target Human creature.","B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Altar of the Lost","Altar of the Lost enters the battlefield tapped.
{T}: Add two mana in any combination of colors to your mana pool. Spend this mana only to cast spells with flashback from a graveyard.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veinfire Borderpost","BR","You may pay {1} and return a basic land you control to its owner's hand rather than pay Veinfire Borderpost's mana cost.
Veinfire Borderpost enters the battlefield tapped.
{T}: Add {B} or {R} to your mana pool.","1BR","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unwilling Recruit","R","Gain control of target creature until end of turn. Untap that creature. It gets +X/+0 and gains haste until end of turn.","XRRR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Geth's Grimoire","Whenever an opponent discards a card, you may draw a card.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swell of Courage","W","Creatures you control get +2/+2 until end of turn.
Reinforce X-{X}{W}{W} ({X}{W}{W}, Discard this card: Put X +1/+1 counters on target creature.)","3WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vigean Intuition","UG","Choose a card type, then reveal the top four cards of your library. Put all cards of the chosen type revealed this way into your hand and the rest into your graveyard. (Artifact, creature, enchantment, instant, land, planeswalker, sorcery, and tribal are card types.)","3GU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Read the Runes","U","Draw X cards. For each card drawn this way, discard a card unless you sacrifice a permanent.","XU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Maze of Shadows","{T}: Add {1} to your mana pool.
{T}: Untap target attacking creature with shadow. Prevent all combat damage that would be dealt to and dealt by that creature this turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slice in Twain","G","Destroy target artifact or enchantment.
Draw a card.","2GG","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirrodin's Core","{T}: Add {1} to your mana pool.
{T}: Put a charge counter on Mirrodin's Core.
{T}, Remove a charge counter from Mirrodin's Core: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Landbind Ritual","W","You gain 2 life for each Plains you control.","3WW","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grafdigger's Cage","Creature cards can't enter the battlefield from graveyards or libraries.
Players can't cast cards in graveyards or libraries.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Metathran Zombie","1","1","U","{B}: Regenerate Metathran Zombie.","1U","Creature - Metathran Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Lookout","1","2","R","{T}, Sacrifice a Goblin: Goblin creatures get +2/+0 until end of turn.","1R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blue Scarab","W","Enchant creature
Enchanted creature can't be blocked by blue creatures.
Enchanted creature gets +2/+2 as long as an opponent controls a blue permanent.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblins of the Flarg","1","1","R","Mountainwalk
When you control a Dwarf, sacrifice Goblins of the Flarg.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("World-Bottling Kit","{5}, Sacrifice World-Bottling Kit: Choose a Magic set. Remove from the game all permanents with that set's expansion symbol except for basic lands.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of the Sacred Bell","4","3","G",NULL,"3G","Creature - Human Monk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Verdant Touch","G","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Target land becomes a 2/2 creature that's still a land. (This effect lasts indefinitely.)","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Zombie","2","2","BR","Protection from white","BR","Creature - Barbarian Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Ghoul","1","1","UB","Whenever Vedalken Ghoul becomes blocked, defending player loses 4 life.","UB","Creature - Vedalken Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Xiahou Dun, the One-Eyed","3","2","B","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Sacrifice Xiahou Dun, the One-Eyed: Return target black card from your graveyard to your hand. Activate this ability only during your turn, before attackers are declared.","2BB","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Synod Centurion","4","4","When you control no other artifacts, sacrifice Synod Centurion.","4","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noxious Toad","1","1","B","When Noxious Toad dies, each opponent discards a card.","2B","Creature - Frog");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loafing Giant","4","6","R","Whenever Loafing Giant attacks or blocks, put the top card of your library into your graveyard. If that card is a land card, prevent all combat damage Loafing Giant would deal this turn.","4R","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rivalry","R","At the beginning of each player's upkeep, if that player controls more lands than each other player, Rivalry deals 2 damage to him or her.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cave People","1","4","R","Whenever Cave People attacks, it gets +1/-2 until end of turn.
{1}{R}{R}, {T}: Target creature gains mountainwalk until end of turn.","1RR","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frenetic Raptor","6","6","R","Beasts can't block.","5R","Creature - Lizard Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Ritual","B","Add {B}{B}{B} to your mana pool.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Shaman","3","3","R","{2}, Discard a card at random: Ogre Shaman deals 2 damage to target creature or player.","3RR","Creature - Ogre Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deadwood Treefolk","3","6","G","Vanishing 3 (This permanent enters the battlefield with three time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
When Deadwood Treefolk enters the battlefield or leaves the battlefield, return another target creature card from your graveyard to your hand.","5G","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("War Cadence","R","{X}{R}: This turn, creatures can't block unless their controller pays {X} for each blocking creature he or she controls.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Lust","R","If target creature has toughness 5 or greater, it gets +4/-4 until end of turn. Otherwise, it gets +4/-X until end of turn, where X is its toughness minus 1.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tanglebloom","{1}, {T}: You gain 1 life.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of Meadowgrain","2","2","W","First strike
Lifelink (Damage dealt by this creature also causes you to gain that much life.)","WW","Creature - Kithkin Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Staff of the Ages","Creatures with landwalk abilities can be blocked as though they didn't have those abilities.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Armor of Faith","W","Enchant creature
Enchanted creature gets +1/+1.
{W}: Enchanted creature gets +0/+1 until end of turn.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pikemen","1","1","W","First strike; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","1W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Agonizing Demise","B","Kicker {1}{R} (You may pay an additional {1}{R} as you cast this spell.)
Destroy target nonblack creature. It can't be regenerated. If Agonizing Demise was kicked, it deals damage equal to that creature's power to the creature's controller.","3B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Echo Tracer","2","2","U","Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Echo Tracer is turned face up, return target creature to its owner's hand.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shared Discovery","U","As an additional cost to cast Shared Discovery, tap four untapped creatures you control.
Draw three cards.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Festival","R","{2}: Goblin Festival deals 1 damage to target creature or player. Flip a coin. If you lose the flip, choose one of your opponents. That player gains control of Goblin Festival.","1R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Imperial Hellkite","6","6","R","Flying
Morph {6}{R}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Imperial Hellkite is turned face up, you may search your library for a Dragon card, reveal it, and put it into your hand. If you do, shuffle your library.","5RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vinelasher Kudzu","1","1","G","Whenever a land enters the battlefield under your control, put a +1/+1 counter on Vinelasher Kudzu.","1G","Creature - Plant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stitched Drake","3","4","U","As an additional cost to cast Stitched Drake, exile a creature card from your graveyard.
Flying","1UU","Creature - Zombie Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Do or Die","B","Separate all creatures target player controls into two piles. Destroy all creatures in the pile of that player's choice. They can't be regenerated.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rock Badger","3","3","R","Mountainwalk (This creature is unblockable as long as defending player controls a Mountain.)","4R","Creature - Badger Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heroes' Reunion","WG","Target player gains 7 life.","GW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightshade Stinger","1","1","B","Flying
Nightshade Stinger can't block.","B","Creature - Faerie Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ticking Gnomes","3","3","Echo {3} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
Sacrifice Ticking Gnomes: Ticking Gnomes deals 1 damage to target creature or player.","3","Artifact Creature - Gnome");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Army of Allah","W","Attacking creatures get +2/+0 until end of turn.","1WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tourach's Gate","B","Enchant land you control
Sacrifice a Thrull: Put three time counters on Tourach's Gate.
At the beginning of your upkeep, remove a time counter from Tourach's Gate. If there are no time counters on Tourach's Gate, sacrifice it.
Tap enchanted land: Attacking creatures you control get +2/-1 until end of turn. Activate this ability only if enchanted land is untapped.","1BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unyaro Bees","0","1","G","Flying
{G}: Unyaro Bees gets +1/+1 until end of turn.
{3}{G}, Sacrifice Unyaro Bees: Unyaro Bees deals 2 damage to target creature or player.","GGG","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fickle Efreet","5","2","R","Whenever Fickle Efreet attacks or blocks, flip a coin at end of combat. If you lose the flip, an opponent gains control of Fickle Efreet.","3R","Creature - Efreet");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Narrow Escape","W","Return target permanent you control to its owner's hand. You gain 4 life.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gerrard's Battle Cry","W","{2}{W}: Creatures you control get +1/+1 until end of turn.","W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Butcher Orgg","6","6","R","You may assign Butcher Orgg's combat damage divided as you choose among defending player and/or any number of creatures he or she controls.","4RRR","Creature - Orgg");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inferno","R","Inferno deals 6 damage to each creature and each player.","5RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wayward Angel","4","4","W","Flying, vigilance
Threshold - As long as seven or more cards are in your graveyard, Wayward Angel gets +3/+3, is black, has trample, and has ""At the beginning of your upkeep, sacrifice a creature.""","4WW","Creature - Angel Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Apprentice Necromancer","1","1","B","{B}, {T}, Sacrifice Apprentice Necromancer: Return target creature card from your graveyard to the battlefield. That creature gains haste. At the beginning of the next end step, sacrifice it.","1B","Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Aristocrat","2","2","B","Sacrifice a creature: Vampire Aristocrat gets +2/+2 until end of turn.","2B","Creature - Vampire Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Umbra Mystic","2","2","W","Auras attached to permanents you control have totem armor. (If an enchanted permanent you control would be destroyed, instead remove all damage from it and destroy an Aura attached to it.)","2W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Terminate","BR","Destroy target creature. It can't be regenerated.","BR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serum Raker","3","2","U","Flying
When Serum Raker dies, each player discards a card.","2UU","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serendib Sorcerer","1","1","U","{T}: Target creature other than Serendib Sorcerer becomes 0/2 until end of turn.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Righteous Blow","W","Righteous Blow deals 2 damage to target attacking or blocking creature.","W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Walking Atlas","1","1","{T}: You may put a land card from your hand onto the battlefield.","2","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vein Drinker","4","4","B","Flying
{R}, {T}: Vein Drinker deals damage equal to its power to target creature. That creature deals damage equal to its power to Vein Drinker.
Whenever a creature dealt damage by Vein Drinker this turn dies, put a +1/+1 counter on Vein Drinker.","4BB","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noggle Ransacker","2","1","UR","When Noggle Ransacker enters the battlefield, each player draws two cards, then discards a card at random.","2(U/R)","Creature - Noggle Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Body Snatcher","2","2","B","When Body Snatcher enters the battlefield, exile it unless you discard a creature card.
When Body Snatcher dies, exile Body Snatcher and return target creature card from your graveyard to the battlefield.","2BB","Creature - Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ruthless Cullblade","2","1","B","Ruthless Cullblade gets +2/+1 as long as an opponent has 10 or less life.","1B","Creature - Vampire Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kentaro, the Smiling Cat","2","1","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
You may pay {X} rather than pay the mana cost for Samurai spells you cast, where X is that spell's converted mana cost.","1W","Legendary Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talruum Piper","3","3","R","All creatures with flying able to block Talruum Piper do so.","4R","Creature - Minotaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Champion of the Parish","1","1","W","Whenever another Human enters the battlefield under your control, put a +1/+1 counter on Champion of the Parish.","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hand of the Praetors","3","2","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Other creatures you control with infect get +1/+1.
Whenever you cast a creature spell with infect, target player gets a poison counter.","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Respite","G","Prevent all combat damage that would be dealt this turn. You gain 1 life for each attacking creature.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gruul Turf","Gruul Turf enters the battlefield tapped.
When Gruul Turf enters the battlefield, return a land you control to its owner's hand.
{T}: Add {R}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Divebomber Griffin","3","2","W","Flying
{T}, Sacrifice Divebomber Griffin: Divebomber Griffin deals 3 damage to target attacking or blocking creature.","3WW","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Mob","2","0","B","Zombie Mob enters the battlefield with a +1/+1 counter on it for each creature card in your graveyard.
When Zombie Mob enters the battlefield, exile all creature cards from your graveyard.","2BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coalhauler Swine","4","4","R","Whenever Coalhauler Swine is dealt damage, it deals that much damage to each player.","4RR","Creature - Boar Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Taiga",NULL,NULL,"Land - Mountain Forest");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inertia Bubble","U","Enchant artifact
Enchanted artifact doesn't untap during its controller's untap step.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infiltrator's Magemark","U","Enchant creature
Creatures you control that are enchanted get +1/+1 and can't be blocked except by creatures with defender.","2U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Onslaught","R","Whenever you cast a creature spell, tap target creature.","R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corpse Lunge","B","As an additional cost to cast Corpse Lunge, exile a creature card from your graveyard.
Corpse Lunge deals damage equal to the exiled card's power to target creature.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kodama's Reach","G","Search your library for two basic land cards, reveal those cards, and put one onto the battlefield tapped and the other into your hand. Then shuffle your library.","2G","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scald","R","Whenever a player taps an Island for mana, Scald deals 1 damage to that player.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thoughtseize","B","Target player reveals his or her hand. You choose a nonland card from it. That player discards that card. You lose 2 life.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Commune with Nature","G","Look at the top five cards of your library. You may reveal a creature card from among them and put it into your hand. Put the rest on the bottom of your library in any order.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hysterical Blindness","U","Creatures your opponents control get -4/-0 until end of turn.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overgrowth","G","Enchant land (Target a land as you cast this. This card enters the battlefield attached to that land.)
Whenever enchanted land is tapped for mana, its controller adds {G}{G} to his or her mana pool (in addition to the mana the land produces).","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitfire Handler","1","1","R","Spitfire Handler can't block creatures with power greater than Spitfire Handler's power.
{R}: Spitfire Handler gets +1/+0 until end of turn.","1R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Every Last Vestige Shall Rot","When you set this scheme in motion, you may pay {X}. If you do, put each nonland permanent target player controls with converted mana cost X or less on the bottom of its owner's library.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sleeper's Guile","B","Enchant creature
Enchanted creature has fear. (It can't be blocked except by artifact creatures and/or black creatures.)
When Sleeper's Guile is put into a graveyard from the battlefield, return Sleeper's Guile to its owner's hand.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bogardan Phoenix","3","3","R","Flying
When Bogardan Phoenix dies, exile it if it had a death counter on it. Otherwise, return it to the battlefield under your control and put a death counter on it.","2RRR","Creature - Phoenix");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Restless Dead","1","1","B","{B}: Regenerate Restless Dead.","1B","Creature - Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Survival of the Fittest","G","{G}, Discard a creature card: Search your library for a creature card, reveal that card, and put it into your hand. Then shuffle your library.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Falkenrath Torturer","2","1","B","Sacrifice a creature: Falkenrath Torturer gains flying until end of turn. If the sacrificed creature was a Human, put a +1/+1 counter on Falkenrath Torturer.","2B","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ephemeron","4","4","U","Flying
Discard a card: Return Ephemeron to its owner's hand.","4UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Death's Shadow","13","13","B","Death's Shadow gets -X/-X, where X is your life total.","B","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sandstone Warrior","1","3","R","First strike (This creature deals combat damage before creatures without first strike.)
{R}: Sandstone Warrior gets +1/+0 until end of turn.","2RR","Creature - Human Soldier Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hydromorph Guardian","2","2","U","{U}, Sacrifice Hydromorph Guardian: Counter target spell that targets one or more creatures you control.","2U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vampiric Fury","R","Vampire creatures you control get +2/+0 and gain first strike until end of turn.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Metamorphic Wurm","3","3","G","Threshold - Metamorphic Wurm gets +4/+4 as long as seven or more cards are in your graveyard.","3GG","Creature - Elephant Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Gnats","0","1","G","Flying
{G}: Regenerate Yavimaya Gnats.","2G","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rhystic Study","U","Whenever an opponent casts a spell, you may draw a card unless that player pays {1}.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fortress Crab","1","6","U",NULL,"3U","Creature - Crab");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darkwater Egg","{2}, {T}, Sacrifice Darkwater Egg: Add {U}{B} to your mana pool. Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Six-y Beast","0","0","R","As Six-y Beast comes into play, you secretly put six or fewer +1/+1 counters on it, then an opponent guesses the number of counters. If that player guesses right, sacrifice Six-y Beast.","3R","Creature - Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Moriok Replica","2","2","{1}{B}, Sacrifice Moriok Replica: You draw two cards and lose 2 life.","3","Artifact Creature - Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Polluted Dead","3","3","B","When Polluted Dead dies, destroy target land.","4B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Defiant Elf","1","1","G","Trample","G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scoria Wurm","7","7","R","At the beginning of your upkeep, flip a coin. If you lose the flip, return Scoria Wurm to its owner's hand.","4R","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Caravan Vigil","G","Search your library for a basic land card, reveal it, put it into your hand, then shuffle your library.
Morbid - You may put that card onto the battlefield instead of putting it into your hand if a creature died this turn.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reckless Charge","R","Target creature gets +3/+0 and gains haste until end of turn.
Flashback {2}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Caltrops","Whenever a creature attacks, Caltrops deals 1 damage to it.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ramosian Sky Marshal","3","3","W","Flying
{7}, {T}: Search your library for a Rebel permanent card with converted mana cost 6 or less and put it onto the battlefield. Then shuffle your library.","3WW","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armored Cancrix","2","5","U",NULL,"4U","Creature - Crab");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shinewend","0","0","W","Flying
Shinewend enters the battlefield with a +1/+1 counter on it.
{1}{W}, Remove a +1/+1 counter from Shinewend: Destroy target enchantment.","1W","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Nomad","0","0","W","Phantom Nomad enters the battlefield with two +1/+1 counters on it.
If damage would be dealt to Phantom Nomad, prevent that damage. Remove a +1/+1 counter from Phantom Nomad.","1W","Creature - Spirit Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disrupting Shoal","U","You may exile a blue card with converted mana cost X from your hand rather than pay Disrupting Shoal's mana cost.
Counter target spell if its converted mana cost is X.","XUU","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Simic Basilisk","0","0","G","Graft 3 (This creature enters the battlefield with three +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{1}{G}: Until end of turn, target creature with a +1/+1 counter on it gains ""Whenever this creature deals combat damage to a creature, destroy that creature at end of combat.""","4GG","Creature - Basilisk Mutant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ichorclaw Myr","1","1","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Whenever Ichorclaw Myr becomes blocked, it gets +2/+2 until end of turn.","2","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Addle","B","Choose a color. Target player reveals his or her hand and you choose a card of that color from it. That player discards that card.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chronomantic Escape","W","Until your next turn, creatures can't attack you. Exile Chronomantic Escape with three time counters on it.
Suspend 3-{2}{W} (Rather than cast this card from your hand, you may pay {2}{W} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","4WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Holy Light","W","Nonwhite creatures get -1/-1 until end of turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorilla Titan","4","4","G","Trample
Gorilla Titan gets +4/+4 as long as there are no cards in your graveyard.","3GG","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Canopy Spider","1","3","G","Reach (This creature can block creatures with flying.)","1G","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Umara Raptor","1","1","U","Flying
Whenever Umara Raptor or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Umara Raptor.","2U","Creature - Bird Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dark Supplicant","1","1","B","{T}, Sacrifice three Clerics: Search your graveyard, hand, and/or library for a card named Scion of Darkness and put it onto the battlefield. If you search your library this way, shuffle it.","B","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tolarian Sentinel","1","3","U","Flying
{U}, {T}, Discard a card: Return target permanent you control to its owner's hand.","3U","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skithiryx, the Blight Dragon","4","4","B","Flying
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
{B}: Skithiryx, the Blight Dragon gains haste until end of turn.
{B}{B}: Regenerate Skithiryx.","3BB","Legendary Creature - Dragon Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmal Mount","1","1","U","Flying
{T}: Target creature you control with toughness 2 or less gets +1/+1 and gains flying until end of turn. When Phantasmal Mount leaves the battlefield this turn, sacrifice that creature. When the creature leaves the battlefield this turn, sacrifice Phantasmal Mount.","1U","Creature - Illusion Horse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Scout","1","1","G","{G}, {T}: Untap target attacking creature you control. Prevent all combat damage that would be dealt to and dealt by it this turn.","G","Creature - Elf Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashenmoor Cohort","4","3","B","Ashenmoor Cohort gets +1/+1 as long as you control another black creature.","5B","Creature - Elemental Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chancellor of the Dross","6","6","B","You may reveal this card from your opening hand. If you do, at the beginning of the first upkeep, each opponent loses 3 life, then you gain life equal to the life lost this way.
Flying, lifelink","4BBB","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightwing Shade","2","2","B","Flying
{1}{B}: Nightwing Shade gets +1/+1 until end of turn.","4B","Creature - Shade");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flowstone Embrace","R","Enchant creature
{T}: Enchanted creature gets +2/-2 until end of turn.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kumano's Blessing","R","Flash
Enchant creature
If a creature dealt damage by enchanted creature this turn would die, exile it instead.","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riftmarked Knight","2","2","W","Protection from black; flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
Suspend 3-{1}{W}{W} (Rather than cast this card from your hand, you may pay {1}{W}{W} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
When the last time counter is removed from Riftmarked Knight while it's exiled, put a 2/2 black Knight creature token with flanking, protection from white, and haste onto the battlefield.","1WW","Creature - Human Rebel Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodbraid Elf","3","2","RG","Haste
Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)","2RG","Creature - Elf Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Armor of Thorns","G","You may cast Armor of Thorns as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Enchant nonblack creature
Enchanted creature gets +2/+2.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dire Undercurrents","UB","Whenever a blue creature enters the battlefield under your control, you may have target player draw a card.
Whenever a black creature enters the battlefield under your control, you may have target player discard a card.","3(U/B)(U/B)","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reward the Faithful","W","Any number of target players each gain life equal to the highest converted mana cost among permanents you control.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stag Beetle","0","0","G","Stag Beetle enters the battlefield with X +1/+1 counters on it, where X is the number of other creatures on the battlefield.","3GG","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Worldpurge","WU","Return all permanents to their owners' hands. Each player chooses up to seven cards in his or her hand, then shuffles the rest into his or her library. Empty all mana pools.","4(W/U)(W/U)(W/U)(W/U)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Relic Ward","W","You may cast Relic Ward as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Enchant artifact
Enchanted artifact has shroud. (It can't be the target of spells or abilities.)","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frozen AEther","U","Artifacts, creatures, and lands your opponents control enter the battlefield tapped.","3U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("War Barge","{3}: Target creature gains islandwalk until end of turn. When War Barge leaves the battlefield this turn, destroy that creature. A creature destroyed this way can't be regenerated.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("One with Nothing","B","Discard your hand.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Misfortune's Gain","W","Destroy target creature. Its owner gains 4 life.","3W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ith, High Arcanist","3","5","WU","Vigilance
{T}: Untap target attacking creature. Prevent all combat damage that would be dealt to and dealt by that creature this turn.
Suspend 4-{W}{U} (Rather than cast this card from your hand, you may pay {W}{U} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","5WU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consuming Vapors","B","Target player sacrifices a creature. You gain life equal to that creature's toughness.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","3B","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Etched Oracle","0","0","Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it.)
{1}, Remove four +1/+1 counters from Etched Oracle: Target player draws three cards.","4","Artifact Creature - Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tireless Tribe","1","1","W","Discard a card: Tireless Tribe gets +0/+4 until end of turn.","W","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hand of Death","B","Destroy target nonblack creature.","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dauntless Escort Avatar","Creatures you control have exalted. (Whenever a creature you control attacks alone, it gets +1/+1 until end of turn for each instance of exalted among permanents you control.)",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Tatters","4","2","B","Swampwalk","4B","Creature - Wraith");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Defender of Law","2","1","W","Flash
Protection from red","2W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ironroot Treefolk","3","5","G",NULL,"4G","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Uncanny Speed","R","Target creature gets +3/+0 and gains haste until end of turn.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Abyss","4","3","B","At the beginning of each player's upkeep, destroy target nonartifact creature that player controls of his or her choice. It can't be regenerated.","3B","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Orzhov Signet","{1}, {T}: Add {W}{B} to your mana pool.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Match","B","Whenever a creature enters the battlefield, that creature's controller may have target creature of his or her choice get -3/-3 until end of turn.","3B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kaervek the Merciless","5","4","BR","Whenever an opponent casts a spell, Kaervek the Merciless deals damage to target creature or player equal to that spell's converted mana cost.","5BR","Legendary Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Dead","2","3","B","{2}{R}, Exile a creature card from your graveyard: Put a 3/1 black and red Graveborn creature token with haste onto the battlefield. Sacrifice it at the beginning of the next end step.","3B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Bruiser","0","0","Modular 3 (This enters the battlefield with three +1/+1 counters on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daraja Griffin","2","2","W","Flying
Sacrifice Daraja Griffin: Destroy target black creature.","3W","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Great Whale","5","5","U","When Great Whale enters the battlefield, untap up to seven lands.","5UU","Creature - Whale");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Zealot","1","3","W","When Kithkin Zealot enters the battlefield, you gain 1 life for each black and/or red permanent target opponent controls.","1W","Creature - Kithkin Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Urza's Rage","R","Kicker {8}{R} (You may pay an additional {8}{R} as you cast this spell.)
Urza's Rage can't be countered by spells or abilities.
Urza's Rage deals 3 damage to target creature or player. If Urza's Rage was kicked, instead it deals 10 damage to that creature or player and the damage can't be prevented.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glaciers","WU","At the beginning of your upkeep, sacrifice Glaciers unless you pay {W}{U}.
All Mountains are Plains.","2WU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Valorous Charge","W","White creatures get +2/+0 until end of turn.","1WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vine Dryad","1","3","G","Flash
Forestwalk
You may exile a green card from your hand rather than pay Vine Dryad's mana cost.","3G","Creature - Dryad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dawn's Reflection","G","Enchant land
Whenever enchanted land is tapped for mana, its controller adds two mana in any combination of colors to his or her mana pool (in addition to the mana the land produces).","3G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gainsay","U","Counter target blue spell.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bringer of the Black Dawn","5","5","B","You may pay {W}{U}{B}{R}{G} rather than pay Bringer of the Black Dawn's mana cost.
Trample
At the beginning of your upkeep, you may pay 2 life. If you do, search your library for a card, then shuffle your library and put that card on top of it.","7BB","Creature - Bringer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charging Troll","3","3","WG","Vigilance
{G}: Regenerate Charging Troll.","2GW","Creature - Troll");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ancestor's Chosen","4","4","W","First strike (This creature deals combat damage before creatures without first strike.)
When Ancestor's Chosen enters the battlefield, you gain 1 life for each card in your graveyard.","5WW","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beckon Apparition","WB","Exile target card from a graveyard. Put a 1/1 white and black Spirit creature token with flying onto the battlefield.","(W/B)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glacial Wall","0","7","U","Defender (This creature can't attack.)","2U","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stoneforge Mystic","1","2","W","When Stoneforge Mystic enters the battlefield, you may search your library for an Equipment card, reveal it, put it into your hand, then shuffle your library.
{1}{W}, {T}: You may put an Equipment card from your hand onto the battlefield.","1W","Creature - Kor Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Shaman","2","2","R","Skirk Shaman can't be blocked except by artifact creatures and/or red creatures.","1RR","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Healer","2","2","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.
{T}: Prevent all damage that would be dealt to target legendary creature this turn.","3W","Creature - Fox Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Living Wish","G","You may choose a creature or land card you own from outside the game, reveal that card, and put it into your hand. Exile Living Wish.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deep Analysis","U","Target player draws two cards.
Flashback-{1}{U}, Pay 3 life. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Finest Hour","WUG","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
Whenever a creature you control attacks alone, if it's the first combat phase of the turn, untap that creature. After this phase, there is an additional combat phase.","2GWU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Animar, Soul of Elements","1","1","URG","Protection from white and from black
Whenever you cast a creature spell, put a +1/+1 counter on Animar, Soul of Elements.
Creature spells you cast cost {1} less to cast for each +1/+1 counter on Animar.","URG","Legendary Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Lancer","2","2","W","Shadow (This creature can block or be blocked by only creatures with shadow.)
Soltari Lancer has first strike as long as it's attacking.","2W","Creature - Soltari Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steam Blast","R","Steam Blast deals 2 damage to each creature and each player.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Legacy's Allure","U","At the beginning of your upkeep, you may put a treasure counter on Legacy's Allure.
Sacrifice Legacy's Allure: Gain control of target creature with power less than or equal to the number of treasure counters on Legacy's Allure. (This effect lasts indefinitely.)","UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Well-Laid Plans","U","Prevent all damage that would be dealt to a creature by another creature if they share a color.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oriss, Samite Guardian","1","3","W","{T}: Prevent all damage that would be dealt to target creature this turn.
Grandeur - Discard another card named Oriss, Samite Guardian: Target player can't cast spells this turn, and creatures that player controls can't attack this turn.","1WW","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Juniper Order Ranger","2","4","WG","Whenever another creature enters the battlefield under your control, put a +1/+1 counter on that creature and a +1/+1 counter on Juniper Order Ranger.","3GW","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Relic Barrier","{T}: Tap target artifact.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dematerialize","U","Return target permanent to its owner's hand.
Flashback {5}{U}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tarmogoyf","*","1+*","G","Tarmogoyf's power is equal to the number of card types among cards in all graveyards and its toughness is equal to that number plus 1. (Artifact, creature, enchantment, instant, land, planeswalker, sorcery, and tribal are card types.)","1G","Creature - Lhurgoyf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alpha Tyrranax","6","5","G",NULL,"4GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coast Watcher","1","1","U","Flying, protection from green","1U","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormcloud Djinn","3","3","U","Flying
Stormcloud Djinn can block only creatures with flying.
{R}{R}: Stormcloud Djinn gets +2/+0 until end of turn and deals 1 damage to you.","4U","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Manic Vandal","2","2","R","When Manic Vandal enters the battlefield, destroy target artifact.","2R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Triskelion","1","1","Triskelion enters the battlefield with three +1/+1 counters on it.
Remove a +1/+1 counter from Triskelion: Triskelion deals 1 damage to target creature or player.","6","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Survivor of the Unseen","2","1","U","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
{T}: Draw two cards, then put a card from your hand on top of your library.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Agent of Shauku","1","1","B","{1}{B}, Sacrifice a land: Target creature gets +2/+0 until end of turn.","1B","Creature - Human Mercenary");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Factory","{T}: Add {1} to your mana pool.
{7}, {T}: Put a 2/2 colorless Assembly-Worker artifact creature token onto the battlefield.",NULL,"Land - Urza’s");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Smother","B","Destroy target creature with converted mana cost 3 or less. It can't be regenerated.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dehydration","U","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature doesn't untap during its controller's untap step.","3U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bad Moon","B","Black creatures get +1/+1.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Broodstar","*","*","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Flying
Broodstar's power and toughness are each equal to the number of artifacts you control.","8UU","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Godsire","8","8","WRG","Vigilance
{T}: Put an 8/8 Beast creature token that's red, green, and white onto the battlefield.","4RGGW","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Esper Sojourners","2","3","WUB","When you cycle Esper Sojourners or it dies, you may tap or untap target permanent.
Cycling {2}{U} ({2}{U}, Discard this card: Draw a card.)","WUB","Artifact Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cateran Enforcer","4","3","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
{4}, {T}: Search your library for a Mercenary permanent card with converted mana cost 4 or less and put it onto the battlefield. Then shuffle your library.","3BB","Creature - Horror Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Canker Abomination","6","6","BG","As Canker Abomination enters the battlefield, choose an opponent. Canker Abomination enters the battlefield with a -1/-1 counter on it for each creature that player controls.","2(B/G)(B/G)","Creature - Treefolk Horror");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Brass Herald","2","2","As Brass Herald enters the battlefield, choose a creature type.
When Brass Herald enters the battlefield, reveal the top four cards of your library. Put all creature cards of the chosen type revealed this way into your hand and the rest on the bottom of your library in any order.
Creatures of the chosen type get +1/+1.","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tainted Monkey","1","1","B","{T}: Choose a word. Target player puts the top card of his or her library into his or her graveyard. If that card has the chosen word in its text box, that player loses 3 life.","1B","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zendikar Farguide","3","3","G","Forestwalk","4G","Creature - Elemental");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Archdemon of Greed","9","9","Flying, trample
At the beginning of your upkeep, sacrifice a Human. If you can't, tap Archdemon of Greed and it deals 9 damage to you.",NULL,"Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sacellum Archers","2","3","G","{R}{W}, {T}: Sacellum Archers deals 2 damage to target attacking or blocking creature.","2G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Bone","1","4","B","Defender (This creature can't attack.)
{B}: Regenerate Wall of Bone. (The next time this creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","2B","Creature - Skeleton Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arcum's Weathervane","{2}, {T}: Target snow land is no longer snow.
{2}, {T}: Target nonsnow basic land becomes snow.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Council of Advisors","1","1","U","When Council of Advisors enters the battlefield, draw a card.","2U","Creature - Human Advisor");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cascade Bluffs","{T}: Add {1} to your mana pool.
{(u/r)}, {T}: Add {U}{U}, {U}{R}, or {R}{R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dawn Elemental","3","3","W","Flying
Prevent all damage that would be dealt to Dawn Elemental.","WWWW","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Oblivion","B","Enchant player
At the beginning of enchanted player's upkeep, that player exiles two cards from his or her graveyard.","3B","Enchantment - Aura Curse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Bangchuckers","2","2","R","{T}: Flip a coin. If you win the flip, Goblin Bangchuckers deals 2 damage to target creature or player. If you lose the flip, Goblin Bangchuckers deals 2 damage to itself.","2RR","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Light","1","5","W","Defender (This creature can't attack.)
Protection from black","2W","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Thirst","B","Enchant player
At the beginning of enchanted player's upkeep, Curse of Thirst deals damage to that player equal to the number of Curses attached to him or her.","4B","Enchantment - Aura Curse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Emblem of the Warmind","R","Enchant creature you control
Creatures you control have haste.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Chameleon","4","4","G","Kavu Chameleon can't be countered.
{G}: Kavu Chameleon becomes the color of your choice until end of turn.","3GG","Creature - Kavu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bond of Agony","B","As an additional cost to cast Bond of Agony, pay X life.
Each other player loses X life.","XB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Battlegrowth","G","Put a +1/+1 counter on target creature.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Indebted Samurai","2","3","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
Whenever a Samurai you control dies, you may put a +1/+1 counter on Indebted Samurai.","3W","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ethereal Whiskergill","4","3","U","Flying
Ethereal Whiskergill can't attack unless defending player controls an Island.","3U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strategic Planning","U","Look at the top three cards of your library. Put one of them into your hand and the rest into your graveyard.","1U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Astral Steel","W","Target creature gets +1/+2 until end of turn.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","2W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Memnite","1","1",NULL,"0","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ravenous Baloth","4","4","G","Sacrifice a Beast: You gain 4 life.","2GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moniker Mage","2","2","U","{U}, Say your middle name: Moniker Mage can't be the target of spells or abilities this turn.
{U}, Say an opponent's middle name: Moniker Mage gains flying until end of turn.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ore Gorger","3","1","R","Whenever you cast a Spirit or Arcane spell, you may destroy target nonbasic land.","3RR","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stifle","U","Counter target activated or triggered ability. (Mana abilities can't be targeted.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crag Saurian","4","4","R","Whenever a source deals damage to Crag Saurian, that source's controller gains control of Crag Saurian.","RRR","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arachnus Web","G","Enchant creature
Enchanted creature can't attack or block, and its activated abilities can't be activated.
At the beginning of the end step, if enchanted creature's power is 4 or greater, destroy Arachnus Web.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abu Ja'far","0","1","W","When Abu Ja'far dies, destroy all creatures blocking or blocked by it. They can't be regenerated.","W","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carnophage","2","2","B","At the beginning of your upkeep, tap Carnophage unless you pay 1 life.","B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Floating-Dream Zubera","1","2","U","When Floating-Dream Zubera dies, draw a card for each Zubera that died this turn.","1U","Creature - Zubera Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wilt-Leaf Cavaliers","3","4","WG","Vigilance","(G/W)(G/W)(G/W)","Creature - Elf Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zephyr Spirit","0","6","U","When Zephyr Spirit blocks, return it to its owner's hand.","5U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Ancients","G","When an opponent casts an enchantment spell, if Hidden Ancients is an enchantment, Hidden Ancients becomes a 5/5 Treefolk creature.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treespring Lorian","5","4","G","Morph {5}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","5G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Embers","R","Rain of Embers deals 1 damage to each creature and each player.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ardent Recruit","1","1","W","Metalcraft - Ardent Recruit gets +2/+2 as long as you control three or more artifacts.","W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elephant Graveyard","{T}: Add {1} to your mana pool.
{T}: Regenerate target Elephant.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Elite Infantry","2","2","R","Whenever Goblin Elite Infantry blocks or becomes blocked, it gets -1/-1 until end of turn.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stone Calendar","Spells you cast cost up to {1} less to cast.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glorious Charge","W","Creatures you control get +1/+1 until end of turn.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Contaminated Bond","B","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Whenever enchanted creature attacks or blocks, its controller loses 3 life.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Predatory Nightstalker","3","2","B","When Predatory Nightstalker enters the battlefield, you may have target opponent sacrifice a creature.","3BB","Creature - Nightstalker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sever the Bloodline","B","Exile target creature and all other creatures with the same name as that creature.
Flashback {5}{B}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battlefield Scrounger","3","3","G","Threshold - Put three cards from your graveyard on the bottom of your library: Battlefield Scrounger gets +3/+3 until end of turn. Activate this ability only once each turn, and only if seven or more cards are in your graveyard.","3GG","Creature - Centaur");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaseous Form","U","Enchant creature
Prevent all combat damage that would be dealt to and dealt by enchanted creature.","2U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hallowed Fountain","({T}: Add {W} or {U} to your mana pool.)
As Hallowed Fountain enters the battlefield, you may pay 2 life. If you don't, Hallowed Fountain enters the battlefield tapped.",NULL,"Land - Plains Island");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heaven's Gate","W","Any number of target creatures become white until end of turn.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heat Shimmer","R","Put a token that's a copy of target creature onto the battlefield. That token has haste and ""At the beginning of the end step, exile this permanent.""","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maddening Imp","1","1","B","Flying
{T}: Non-Wall creatures the active player controls attack this turn if able. At the beginning of the next end step, destroy each of those creatures that didn't attack this turn. Activate this ability only during an opponent's turn and only before combat.","2B","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Waste Away","B","As an additional cost to cast Waste Away, discard a card.
Target creature gets -5/-5 until end of turn.","4B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prosperity","U","Each player draws X cards.","XU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of the Hunt","2","2","G","Whenever you cast a Spirit or Arcane spell, Kami of the Hunt gets +1/+1 until end of turn.","2G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unearthly Blizzard","R","Up to three target creatures can't block this turn.","2R","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dismiss","U","Counter target spell.
Draw a card.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hungry Hungry Heifer","3","3","G","During your upkeep, remove a counter from any card you control or sacrifice Hungry Hungry Heifer.","2G","Summon - Cow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mutilate","B","All creatures get -1/-1 until end of turn for each Swamp you control.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fen Stalker","3","2","B","Fen Stalker has fear as long as you control no untapped lands. (It can't be blocked except by artifact creatures and/or black creatures.)","3B","Creature - Nightstalker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Flare","U","Enchant creature
Enchanted creature gets +2/+0.
At the beginning of the upkeep of enchanted creature's controller, put a -0/-1 counter on that creature.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Muscle Sliver","1","1","G","All Sliver creatures get +1/+1.","1G","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moroii","4","4","UB","Flying
At the beginning of your upkeep, you lose 1 life.","2UB","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mortal Combat","B","At the beginning of your upkeep, if twenty or more creature cards are in your graveyard, you win the game.","2BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primitive Etchings","G","Reveal the first card you draw each turn. Whenever you reveal a creature card this way, draw a card.","2GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spawning Pit","Sacrifice a creature: Put a charge counter on Spawning Pit.
{1}, Remove two charge counters from Spawning Pit: Put a 2/2 colorless Spawn artifact creature token onto the battlefield.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Word of Seizing","R","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Untap target permanent and gain control of it until end of turn. It gains haste until end of turn.","3RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sift","U","Draw three cards, then discard a card.","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clear","W","Destroy target enchantment.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glitterfang","1","1","R","Haste
At the beginning of the end step, return Glitterfang to its owner's hand.","R","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Research // Development","UR","Put a 3/1 red Elemental creature token onto the battlefield unless any opponent has you draw a card. Repeat this process two more times.
---
Choose up to four cards you own from outside the game and shuffle them into your library.","3UR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vengeful Archon","7","7","W","Flying
{X}: Prevent the next X damage that would be dealt to you this turn. If damage is prevented this way, Vengeful Archon deals that much damage to target player.","4WWW","Creature - Archon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Sharpshooter","1","1","R","Goblin Sharpshooter doesn't untap during your untap step.
Whenever a creature dies, untap Goblin Sharpshooter.
{T}: Goblin Sharpshooter deals 1 damage to target creature or player.","2R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strength in Numbers","G","Until end of turn, target creature gains trample and gets +X/+X, where X is the number of attacking creatures.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rysorian Badger","2","2","G","Whenever Rysorian Badger attacks and isn't blocked, you may exile up to two target creature cards from defending player's graveyard. If you do, you gain 1 life for each card exiled this way and Rysorian Badger assigns no combat damage this turn.","2G","Creature - Badger");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fistful of Force","G","Target creature gets +2/+2 until end of turn. Clash with an opponent. If you win, that creature gets an additional +2/+2 and gains trample until end of turn. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Cloudchaser","2","2","W","Flying (This creature can't be blocked except by creatures with flying or reach.)
When Aven Cloudchaser enters the battlefield, destroy target enchantment.","3W","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Godhead of Awe","4","4","WU","Flying
Other creatures are 1/1.","(W/U)(W/U)(W/U)(W/U)(W/U)","Creature - Spirit Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bane of the Living","4","3","B","Morph {X}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Bane of the Living is turned face up, all creatures get -X/-X until end of turn.","2BB","Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dreadship Reef","{T}: Add {1} to your mana pool.
{1}, {T}: Put a storage counter on Dreadship Reef.
{1}, Remove X storage counters from Dreadship Reef: Add X mana in any combination of {U} and/or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Domineer","U","Enchant artifact creature
You control enchanted artifact creature.","1UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vines of Vastwood","G","Kicker {G} (You may pay an additional {G} as you cast this spell.)
Target creature can't be the target of spells or abilities your opponents control this turn. If Vines of Vastwood was kicked, that creature gets +4/+4 until end of turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Scorpion","1","3","B","Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","2B","Creature - Scorpion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tablet of Epityr","Whenever an artifact you control is put into a graveyard from the battlefield, you may pay {1}. If you do, you gain 1 life.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minotaur Tactician","1","1","R","Haste
Minotaur Tactician gets +1/+1 as long as you control a white creature.
Minotaur Tactician gets +1/+1 as long as you control a blue creature.","3R","Creature - Minotaur");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Game Preserve","G","At the beginning of your upkeep, each player reveals the top card of his or her library. If all cards revealed this way are creature cards, put those cards onto the battlefield under their owners' control.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Capashen Templar","2","2","W","{W}: Capashen Templar gets +0/+1 until end of turn.","2W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Remove","U","Cast Remove only during the declare attackers step and only if you've been attacked this step.
Return target attacking creature to its owner's hand.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Renewal","G","As an additional cost to cast Renewal, sacrifice a land.
Search your library for a basic land card and put that card onto the battlefield. Then shuffle your library.
Draw a card at the beginning of the next turn's upkeep.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Celestial Mantle","W","Enchant creature
Enchanted creature gets +3/+3.
Whenever enchanted creature deals combat damage to a player, double its controller's life total.","3WWW","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feast of Flesh","B","Feast of Flesh deals X damage to target creature and you gain X life, where X is 1 plus the number of cards named Feast of Flesh in all graveyards.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Conclave Equenaut","3","3","W","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Flying","4WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flameshot","R","You may discard a Mountain card rather than pay Flameshot's mana cost.
Flameshot deals 3 damage divided as you choose among one, two, or three target creatures.","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fortified Area","W","Wall creatures you control get +1/+0 and have banding. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","1WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Possessed Portal","If a player would draw a card, that player skips that draw instead.
At the beginning of each end step, each player sacrifices a permanent unless he or she discards a card.","8","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Soldier Replica","1","3","{1}{W}, Sacrifice Soldier Replica: Soldier Replica deals 3 damage to target attacking or blocking creature.","3","Artifact Creature - Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foriysian Interceptor","0","5","W","Flash (You may cast this spell any time you could cast an instant.)
Defender
Foriysian Interceptor can block an additional creature.","3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tunnel","R","Destroy target Wall. It can't be regenerated.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lignify","G","Enchant creature
Enchanted creature is a 0/4 Treefolk with no abilities.","1G","Tribal Enchantment - Treefolk Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tainted AEther","B","Whenever a creature enters the battlefield, its controller sacrifices a creature or land.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Molimo, Maro-Sorcerer","*","*","G","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)
Molimo, Maro-Sorcerer's power and toughness are each equal to the number of lands you control.","4GGG","Legendary Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Taskmaster","4","3","R","Ogre Taskmaster can't block.","3R","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vaevictis Asmadi","7","7","BRG","Flying
At the beginning of your upkeep, sacrifice Vaevictis Asmadi unless you pay {B}{R}{G}.
{B}: Vaevictis Asmadi gets +1/+0 until end of turn.
{R}: Vaevictis Asmadi gets +1/+0 until end of turn.
{G}: Vaevictis Asmadi gets +1/+0 until end of turn.","2BBRRGG","Legendary Creature - Elder Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Gatekeeper","2","3","W","Artifacts, creatures, and lands your opponents control enter the battlefield tapped.","2WW","Creature - Elephant Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rishadan Brigand","3","2","U","Flying
When Rishadan Brigand enters the battlefield, each opponent sacrifices a permanent unless he or she pays {3}.
Rishadan Brigand can block only creatures with flying.","4U","Creature - Human Pirate");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hostile Realm","R","Enchant land
Enchanted land has ""{T}: Target creature can't block this turn.""","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mons's Goblin Waiters","1","1","R","Sacrifice a creature or land: Add {o{1/2*R}} to your mana pool.","R","Creature - Goblin Waiter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patron of the Wild","1","1","G","Morph {2}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Patron of the Wild is turned face up, target creature gets +3/+3 until end of turn.","G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moonhold","WR","Target player can't play land cards this turn if {R} was spent to cast Moonhold and can't play creature cards this turn if {W} was spent to cast it. (Do both if {R}{W} was spent.)","2(R/W)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slithermuse","3","3","U","When Slithermuse leaves the battlefield, choose an opponent. If that player has more cards in hand than you, draw cards equal to the difference.
Evoke {3}{U} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","2UU","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Figure of Destiny Avatar","{X}: Put a +1/+1 counter on target creature with fewer than X +1/+1 counters on it.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ire of Kaminari","R","Ire of Kaminari deals damage to target creature or player equal to the number of Arcane cards in your graveyard.","3R","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death or Glory","W","Separate all creature cards in your graveyard into two piles. Exile the pile of an opponent's choice and return the other to the battlefield.","4W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Craven Giant","4","1","R","Craven Giant can't block.","2R","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repel","U","Put target creature on top of its owner's library.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rimewind Cryomancer","2","3","U","{1}, {T}: Counter target activated ability. Activate this ability only if you control four or more snow permanents. (Mana abilities can't be targeted.)","3U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quag Vampires","1","1","B","Multikicker {1}{B} (You may pay an additional {1}{B} any number of times as you cast this spell.)
Swampwalk
Quag Vampires enters the battlefield with a +1/+1 counter on it for each time it was kicked.","B","Creature - Vampire Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exhume","B","Each player puts a creature card from his or her graveyard onto the battlefield.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quirion Explorer","1","1","G","{T}: Add to your mana pool one mana of any color that a land an opponent controls could produce.","1G","Creature - Elf Druid Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shoal Serpent","5","5","U","Defender
Landfall - Whenever a land enters the battlefield under your control, Shoal Serpent loses defender until end of turn.","5U","Creature - Serpent");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forced Retreat","U","Put target creature on top of its owner's library.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hope Charm","W","Choose one - Target creature gains first strike until end of turn; or target player gains 2 life; or destroy target Aura.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oath of Ghouls","B","At the beginning of each player's upkeep, that player chooses target player whose graveyard has fewer creature cards in it than his or her graveyard does and is his or her opponent. The first player may return a creature card from his or her graveyard to his or her hand.","1B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warren Weirding","B","Target player sacrifices a creature. If a Goblin is sacrificed this way, that player puts two 1/1 black Goblin Rogue creature tokens onto the battlefield, and those tokens gain haste until end of turn.","1B","Tribal Sorcery - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Processor","As Phyrexian Processor enters the battlefield, pay any amount of life.
{4}, {T}: Put an X/X black Minion creature token onto the battlefield, where X is the life paid as Phyrexian Processor entered the battlefield.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Discordant Spirit","2","2","BR","At the beginning of each end step, if it's an opponent's turn, put a +1/+1 counter on Discordant Spirit for each 1 damage dealt to you this turn.
At the beginning of your end step, remove all +1/+1 counters from Discordant Spirit.","2BR","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Distortion","U","Whenever a creature or land becomes tapped, put an hourglass counter on it.
Permanents with hourglass counters on them don't untap during their controllers' untap steps.
At the beginning of each player's upkeep, remove all hourglass counters from permanents that player controls.","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bellowing Fiend","3","3","B","Flying
Whenever Bellowing Fiend deals damage to a creature, Bellowing Fiend deals 3 damage to that creature's controller and 3 damage to you.","4B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Lieutenant","1","2","W","{1}{W}: Target Soldier creature gets +1/+0 until end of turn.","WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tromp the Domains","G","Domain - Until end of turn, creatures you control gain trample and get +1/+1 for each basic land type among lands you control.","5G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aspect of Mongoose","G","Enchant creature
Enchanted creature has shroud. (It can't be the target of spells or abilities.)
When Aspect of Mongoose is put into a graveyard from the battlefield, return Aspect of Mongoose to its owner's hand.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Midnight Charm","B","Choose one - Midnight Charm deals 1 damage to target creature and you gain 1 life; or target creature gains first strike until end of turn; or tap target creature.","B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Maelstrom","When you planeswalk to The Maelstrom or at the beginning of your upkeep, you may reveal the top card of your library. If it's a permanent card, you may put it onto the battlefield. If you revealed a card but didn't put it onto the battlefield, put it on the bottom of your library.
Whenever you roll {C}, return target permanent card from your graveyard to the battlefield.",NULL,"Plane - Alara");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Borderland Behemoth","4","4","R","Trample
Borderland Behemoth gets +4/+4 for each other Giant you control.","5RR","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Tithe","W","Counter target spell unless its controller pays {1}.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stormcaller's Boon","WU","Sacrifice Stormcaller's Boon: Creatures you control gain flying until end of turn.
Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)","2WU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llawan, Cephalid Empress","2","3","U","When Llawan, Cephalid Empress enters the battlefield, return all blue creatures your opponents control to their owners' hands.
Your opponents can't cast blue creature spells.","3U","Legendary Creature - Cephalid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Razorgrass Screen","2","1","Defender (This creature can't attack.)
Razorgrass Screen blocks each turn if able.","1","Artifact Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cankerous Thirst","BG","If {B} was spent to cast Cankerous Thirst, you may have target creature get -3/-3 until end of turn. If {G} was spent to cast Cankerous Thirst, you may have target creature get +3/+3 until end of turn. (Do both if {B}{G} was spent.)","3(B/G)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wandering Graybeard","4","4","W","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Wandering Graybeard, you may reveal it. If you do, you gain 4 life.","3WW","Creature - Giant Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shred Memory","B","Exile up to four target cards from a single graveyard.
Transmute {1}{B}{B} ({1}{B}{B}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","1B","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Howlpack Alpha","3","3","Other Werewolf and Wolf creatures you control get +1/+1.
At the beginning of your end step, put a 2/2 green Wolf creature token onto the battlefield.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Howlpack Alpha.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heartmender","2","2","WG","At the beginning of your upkeep, remove a -1/-1 counter from each creature you control.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","2(G/W)(G/W)","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ragged Veins","B","Flash
Enchant creature
Whenever enchanted creature is dealt damage, its controller loses that much life.","1B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bad River","Bad River enters the battlefield tapped.
{T}, Sacrifice Bad River: Search your library for an Island or Swamp card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skull of Orm","{5}, {T}: Return target enchantment card from your graveyard to your hand.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Hope","0","3","W","Defender (This creature can't attack.)
Whenever Wall of Hope is dealt damage, you gain that much life.","W","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merrow Harbinger","2","3","U","Islandwalk
When Merrow Harbinger enters the battlefield, you may search your library for a Merfolk card, reveal it, then shuffle your library and put that card on top of it.","3U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grand Arbiter Augustin IV","2","3","WU","White spells you cast cost {1} less to cast.
Blue spells you cast cost {1} less to cast.
Spells your opponents cast cost {1} more to cast.","2WU","Legendary Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reap and Sow","G","Choose one - Destroy target land; or search your library for a land card, put that card onto the battlefield, then shuffle your library.
Entwine {1}{G} (Choose both if you pay the entwine cost.)","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashenmoor Liege","4","1","BR","Other black creatures you control get +1/+1.
Other red creatures you control get +1/+1.
Whenever Ashenmoor Liege becomes the target of a spell or ability an opponent controls, that player loses 4 life.","1(B/R)(B/R)(B/R)","Creature - Elemental Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nourish","G","You gain 6 life.","GG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fury Charm","R","Choose one - Destroy target artifact; or target creature gets +1/+1 and gains trample until end of turn; or remove two time counters from target permanent or suspended card.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elsewhere Flask","When Elsewhere Flask enters the battlefield, draw a card.
Sacrifice Elsewhere Flask: Choose a basic land type. Each land you control becomes that type until end of turn.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Research","R","{1}{W}: Search your library for an enchantment card and reveal that card. Put it into your hand, then discard a card at random. Then shuffle your library.
{1}{U}: Search your library for an instant card and reveal that card. Put it into your hand, then discard a card at random. Then shuffle your library.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whip Sergeant","2","1","R","{R}: Target creature gains haste until end of turn. (It can attack this turn.)","2R","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ignite Disorder","R","Ignite Disorder deals 3 damage divided as you choose among one, two, or three target white and/or blue creatures.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fire Whip","R","Enchant creature you control
Enchanted creature has ""{T}: This creature deals 1 damage to target creature or player.""
Sacrifice Fire Whip: Fire Whip deals 1 damage to target creature or player.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spawnwrithe","2","2","G","Trample
Whenever Spawnwrithe deals combat damage to a player, put a token that's a copy of Spawnwrithe onto the battlefield.","2G","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaze of Pain","B","Until end of turn, whenever a creature you control attacks and isn't blocked, you may choose to have it deal damage equal to its power to a target creature. If you do, it assigns no combat damage this turn.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Home Guard","1","6","W","At end of combat, if Kjeldoran Home Guard attacked or blocked this combat, put a -0/-1 counter on Kjeldoran Home Guard and put a 0/1 white Deserter creature token onto the battlefield.","3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for Ancient Secrets","U","Whenever a card is put into your graveyard from anywhere, you may put a quest counter on Quest for Ancient Secrets.
Remove five quest counters from Quest for Ancient Secrets and sacrifice it: Target player shuffles his or her graveyard into his or her library.","U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skull Fracture","B","Target player discards a card.
Flashback {3}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thalakos Sentry","1","2","U","Shadow (This creature can block or be blocked by only creatures with shadow)","1U","Creature - Thalakos Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blinkmoth Urn","At the beginning of each player's precombat main phase, if Blinkmoth Urn is untapped, that player adds {1} to his or her mana pool for each artifact he or she controls.","5","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rejuvenation Chamber","Fading 2 (This artifact enters the battlefield with two fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
{T}: You gain 2 life.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merieke Ri Berit","1","1","WUB","Merieke Ri Berit doesn't untap during your untap step.
{T}: Gain control of target creature for as long as you control Merieke Ri Berit. When Merieke Ri Berit leaves the battlefield or becomes untapped, destroy that creature. It can't be regenerated.","WUB","Legendary Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Giant Growth","G","Target creature gets +3/+3 until end of turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wake of Vultures","3","1","B","Flying
{1}{B}, Sacrifice a creature: Regenerate Wake of Vultures.","3B","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reality Acid","U","Enchant permanent
Vanishing 3 (This permanent enters the battlefield with three time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)
When Reality Acid leaves the battlefield, enchanted permanent's controller sacrifices it.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riot Ringleader","2","2","R","Whenever Riot Ringleader attacks, Human creatures you control get +1/+0 until end of turn.","2R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sapling of Colfenor","2","5","BG","Sapling of Colfenor is indestructible.
Whenever Sapling of Colfenor attacks, reveal the top card of your library. If it's a creature card, you gain life equal to that card's toughness, lose life equal to its power, then put it into your hand.","3(B/G)(B/G)","Legendary Creature - Treefolk Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chaotic Backlash","R","Chaotic Backlash deals damage to target player equal to twice the number of white and/or blue permanents he or she controls.","4R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dense Canopy","G","Creatures with flying can block only creatures with flying.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harmonic Convergence","G","Put all enchantments on top of their owners' libraries.","2G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cold Storage","{3}: Exile target creature you control.
Sacrifice Cold Storage: Return each creature card exiled with Cold Storage to the battlefield under your control.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ion Storm","R","{1}{R}, Remove a +1/+1 counter or a charge counter from a permanent you control: Ion Storm deals 2 damage to target creature or player.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Sire","1","1","When Myr Sire dies, put a 1/1 colorless Myr artifact creature token onto the battlefield.","2","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Tortoise","1","1","U","Giant Tortoise gets +0/+3 as long as it's untapped.","1U","Creature - Turtle");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bant Panorama","{T}: Add {1} to your mana pool.
{1}, {T}, Sacrifice Bant Panorama: Search your library for a basic Forest, Plains, or Island card and put it onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silverglade Elemental","4","4","G","When Silverglade Elemental enters the battlefield, you may search your library for a Forest card and put that card onto the battlefield. If you do, shuffle your library.","4G","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Trader","1","1","R",NULL,"R","Creature - Dwarf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uktabi Orangutan","2","2","G","When Uktabi Orangutan enters the battlefield, destroy target artifact.","2G","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spirit en-Dal","2","1","W","Shadow (This creature can block or be blocked by only creatures with shadow.)
Forecast - {1}{W}, Reveal Spirit en-Dal from your hand: Target creature gains shadow until end of turn. (Activate this ability only during your upkeep and only once each turn.)","2W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Timely Reinforcements","W","If you have less life than an opponent, you gain 6 life. If you control fewer creatures than an opponent, put three 1/1 white Soldier creature tokens onto the battlefield.","2W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Stalker","1","1","U","{1}: Mistform Stalker becomes the creature type of your choice until end of turn.
{2}{U}{U}: Mistform Stalker gets +2/+2 and gains flying until end of turn.","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Razor Barrier","W","Target permanent you control gains protection from artifacts or from the color of your choice until end of turn.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trade Secrets","U","Target opponent draws two cards, then you draw up to four cards. That opponent may repeat this process as many times as he or she chooses.","1UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cylian Sunsinger","2","2","G","{R}{G}{W}: Cylian Sunsinger and each other creature with the same name as it get +3/+3 until end of turn.","1G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barrenton Medic","0","4","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.
Put a -1/-1 counter on Barrenton Medic: Untap Barrenton Medic.","4W","Creature - Kithkin Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wiitigo","0","0","G","Wiitigo enters the battlefield with six +1/+1 counters on it.
At the beginning of your upkeep, put a +1/+1 counter on Wiitigo if it has blocked or been blocked since your last upkeep. Otherwise, remove a +1/+1 counter from it.","3GGG","Creature - Yeti");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mycosynth Fiend","2","2","G","Mycosynth Fiend gets +1/+1 for each poison counter your opponents have.","2G","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Aesthir","1","1","W","Flying, first strike
{W}{W}: Wild Aesthir gets +2/+0 until end of turn. Activate this ability only once each turn.","2W","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incendiary","R","Enchant creature
At the beginning of your upkeep, you may put a fuse counter on Incendiary.
When enchanted creature dies, Incendiary deals X damage to target creature or player, where X is the number of fuse counters on Incendiary.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heritage Druid","1","1","G","Tap three untapped Elves you control: Add {G}{G}{G} to your mana pool.","G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Recruiter","1","1","R","When Goblin Recruiter enters the battlefield, search your library for any number of Goblin cards and reveal those cards. Shuffle your library, then put them on top of it in any order.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Assault Griffin","3","2","W","Flying","3W","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shimian Night Stalker","4","4","B","{B}, {T}: All damage that would be dealt to you this turn by target attacking creature is dealt to Shimian Night Stalker instead.","3BB","Creature - Nightstalker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Disk","2","4","W","Magus of the Disk enters the battlefield tapped.
{1}, {T}: Destroy all artifacts, creatures, and enchantments.","2WW","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Martyrs of Korlis","1","6","W","As long as Martyrs of Korlis is untapped, all damage that would be dealt to you by artifacts is dealt to Martyrs of Korlis instead.","3WW","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twisted Justice","UB","Target player sacrifices a creature. You draw cards equal to that creature's power.","4UB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blessed Wind","W","Target player's life total becomes 20.","7WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hurkyl's Recall","U","Return all artifacts target player owns to his or her hand.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dragonspeaker Shaman","2","2","R","Dragon spells you cast cost {2} less to cast.","1RR","Creature - Human Barbarian Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Concentrate","U","Draw three cards.","2UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chameleon Colossus","4","4","G","Changeling (This card is every creature type at all times.)
Protection from black
{2}{G}{G}: Chameleon Colossus gets +X/+X until end of turn, where X is its power.","2GG","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Border Guard","1","4","W",NULL,"2W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Viridian Longbow","Equipped creature has ""{T}: This creature deals 1 damage to target creature or player.""
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Talons","R","Enchant creature
Enchanted creature gets +3/+0 and has first strike.","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undead Gladiator","3","1","B","{1}{B}, Discard a card: Return Undead Gladiator from your graveyard to your hand. Activate this ability only during your upkeep.
Cycling {1}{B} ({1}{B}, Discard this card: Draw a card.)","1BB","Creature - Zombie Barbarian");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dance, Pathetic Marionette","When you set this scheme in motion, each opponent reveals cards from the top of his or her library until he or she reveals a creature card. Choose one of the revealed creature cards and put it onto the battlefield under your control. Put all other cards revealed this way into their owners' graveyards.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chancellor of the Annex","5","6","W","You may reveal this card from your opening hand. If you do, when each opponent casts his or her first spell of the game, counter that spell unless that player pays {1}.
Flying
Whenever an opponent casts a spell, counter it unless that player pays {1}.","4WWW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sentinels of Glen Elendra","2","3","U","Flash
Flying","3U","Creature - Faerie Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corpse Connoisseur","3","3","B","When Corpse Connoisseur enters the battlefield, you may search your library for a creature card and put that card into your graveyard. If you do, shuffle your library.
Unearth {3}{B} ({3}{B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","4B","Creature - Zombie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Frantic Search","U","Draw two cards, then discard two cards. Untap up to three lands.","2U","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Epochrasite","1","1","Epochrasite enters the battlefield with three +1/+1 counters on it if you didn't cast it from your hand.
When Epochrasite dies, exile it with three time counters on it and it gains suspend. (At the beginning of your upkeep, remove a time counter. When the last is removed, cast this card without paying its mana cost. It has haste.)","2","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hollow Dogs","3","3","B","Whenever Hollow Dogs attacks, it gets +2/+0 until end of turn.","4B","Creature - Zombie Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balshan Collaborator","2","2","U","Flying
{B}: Balshan Collaborator gets +1/+1 until end of turn.","3U","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Surgespanner","2","2","U","Whenever Surgespanner becomes tapped, you may pay {1}{U}. If you do, return target permanent to its owner's hand.","2UU","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Torch Drake","2","2","U","Flying
{1}{R}: Torch Drake gets +1/+0 until end of turn.","3U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rune of Protection: Artifacts","W","{W}: The next time an artifact source of your choice would deal damage to you this turn, prevent that damage.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Old Man of the Sea","2","3","U","You may choose not to untap Old Man of the Sea during your untap step.
{T}: Gain control of target creature with power less than or equal to Old Man of the Sea's power for as long as Old Man of the Sea remains tapped and that creature's power remains less than or equal to Old Man of the Sea's power.","1UU","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lymph Sliver","3","3","W","All Sliver creatures have absorb 1. (If a source would deal damage to a Sliver, prevent 1 of that damage.)","4W","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sanctum of Serra","When you planeswalk away from Sanctum of Serra, destroy all nonland permanents.
Whenever you roll {C}, you may have your life total become 20.",NULL,"Plane - Serra’s Realm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Adder-Staff Boggart","2","1","R","When Adder-Staff Boggart enters the battlefield, clash with an opponent. If you win, put a +1/+1 counter on Adder-Staff Boggart. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vault of the Archangel","{T}: Add {1} to your mana pool.
{2}{W}{B}, {T}: Creatures you control gain deathtouch and lifelink until end of turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chamber of Manipulation","U","Enchant land
Enchanted land has ""{T}, Discard a card: Gain control of target creature until end of turn.""","2UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Congregation","W","You gain 2 life for each creature target player controls.
Suspend 5-{1}{W} (Rather than cast this card from your hand, you may pay {1}{W} and exile it with five time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","3W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carapace Forger","2","2","G","Metalcraft - Carapace Forger gets +2/+2 as long as you control three or more artifacts.","1G","Creature - Elf Artificer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kresh the Bloodbraided Avatar","Whenever a creature you control is devoured, put an X/X green Ooze creature token onto the battlefield, where X is the devoured creature's power.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Junkyo Bell","At the beginning of your upkeep, you may have target creature you control get +X/+X until end of turn, where X is the number of creatures you control. If you do, sacrifice that creature at the beginning of the next end step.","4","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hematite Golem","1","4","{1}{R}: Hematite Golem gets +2/+0 until end of turn.","4","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Red Ward","W","Enchant creature
Enchanted creature has protection from red. This effect doesn't remove Red Ward.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prowess of the Fair","B","Whenever another nontoken Elf is put into your graveyard from the battlefield, you may put a 1/1 green Elf Warrior creature token onto the battlefield.","1B","Tribal Enchantment - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Din of the Fireherd","BR","Put a 5/5 black and red Elemental creature token onto the battlefield. Target opponent sacrifices a creature for each black creature you control, then sacrifices a land for each red creature you control.","5(B/R)(B/R)(B/R)","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grafted Exoskeleton","Equipped creature gets +2/+2 and has infect. (It deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Whenever Grafted Exoskeleton becomes unattached from a permanent, sacrifice that permanent.
Equip {2}","4","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veiling Oddity","2","3","U","Suspend 4-{1}{U} (Rather than cast this card from your hand, you may pay {1}{U} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
When the last time counter is removed from Veiling Oddity while it's exiled, creatures are unblockable this turn.","3U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Impersonators","*","*","G","When you play Elvish Impersonators, roll two six-sided dice one after the other. Elvish Impersonators comes into play with power equal to the first die roll and toughness equal to the second.","3G","Summon - Elves");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wooded Foothills","{T}, Pay 1 life, Sacrifice Wooded Foothills: Search your library for a Mountain or Forest card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nucklavee","4","4","UR","When Nucklavee enters the battlefield, you may return target red sorcery card from your graveyard to your hand.
When Nucklavee enters the battlefield, you may return target blue instant card from your graveyard to your hand.","4(U/R)(U/R)","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Swords","3","5","W","Defender, flying (This creature can't attack, and it can block creatures with flying.)","3W","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dragonskull Summit","Dragonskull Summit enters the battlefield tapped unless you control a Swamp or a Mountain.
{T}: Add {B} or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Petrified Wood-Kin","3","3","G","Petrified Wood-Kin can't be countered.
Bloodthirst X (This creature enters the battlefield with X +1/+1 counters on it, where X is the damage dealt to your opponents this turn.)
Protection from instants","6G","Creature - Elemental Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devouring Rage","R","As an additional cost to cast Devouring Rage, you may sacrifice any number of Spirits.
Target creature gets +3/+0 until end of turn. For each Spirit sacrificed this way, that creature gets an additional +3/+0 until end of turn.","4R","Instant - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mana Screw","{1}: Flip a coin. If you win the flip, add {2} to your mana pool. Play this ability only any time you could play an instant.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stoic Rebuttal","U","Metalcraft - Stoic Rebuttal costs {1} less to cast if you control three or more artifacts.
Counter target spell.","1UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deal Damage","R","Deal Damage deals 4 damage to target creature or player.
Gotcha Whenever an opponent says ""Deal"" or ""Damage,"" you may say ""Gotcha"" If you do, return Deal Damage from your graveyard to your hand.","2RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heartwood Dryad","2","1","G","Heartwood Dryad can block creatures with shadow as though Heartwood Dryad had shadow.","1G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voice of Reason","2","2","W","Flying, protection from blue","3W","Creature - Angel");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Spellskite","0","4","{(u/p)}: Change a target of target spell or ability to Spellskite. ({(u/p)} can be paid with either {U} or 2 life.)","2","Artifact Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Terramorphic Expanse","{T}, Sacrifice Terramorphic Expanse: Search your library for a basic land card and put it onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Thopter Squadron","0","0","Flying
Thopter Squadron enters the battlefield with three +1/+1 counters on it.
{1}, Remove a +1/+1 counter from Thopter Squadron: Put a 1/1 colorless Thopter artifact creature token with flying onto the battlefield. Activate this ability only any time you could cast a sorcery.
{1}, Sacrifice another Thopter: Put a +1/+1 counter on Thopter Squadron. Activate this ability only any time you could cast a sorcery.","5","Artifact Creature - Thopter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Godo's Irregulars","1","1","R","{R}: Godo's Irregulars deals 1 damage to target creature blocking it.","R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Putrid Flesh","2","4","B","Defender (This creature can't attack.)
Protection from white
Prevent all damage that would be dealt to Wall of Putrid Flesh by enchanted creatures.","2B","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirran Crusader","2","2","W","Double strike, protection from black and from green","1WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Silver Myr","1","1","{T}: Add {U} to your mana pool.","2","Artifact Creature - Myr");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Argentum Armor","Equipped creature gets +6/+6.
Whenever equipped creature attacks, destroy target permanent.
Equip {6}","6","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plated Spider","4","4","G","Reach (This creature can block creatures with flying.)","4G","Creature - Spider");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Stratadon","5","5","Domain - Stratadon costs {1} less to cast for each basic land type among lands you control.
Trample","10","Artifact Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talas Warrior","2","2","U","Talas Warrior is unblockable.","1UU","Creature - Human Pirate Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whirling Dervish","1","1","G","Protection from black
At the beginning of each end step, if Whirling Dervish dealt damage to an opponent this turn, put a +1/+1 counter on it.","GG","Creature - Human Monk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Echo Chamber","{4}, {T}: An opponent chooses target creature he or she controls. Put a token that's a copy of that creature onto the battlefield. That token gains haste until end of turn. Exile the token at the beginning of the next end step. Activate this ability only any time you could cast a sorcery.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Haunting Wind","B","Whenever an artifact becomes tapped or a player activates an artifact's ability without {T} in its activation cost, Haunting Wind deals 1 damage to that artifact's controller.","3B","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pegasus token card",NULL,NULL,NULL);
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saprazzan Legate","1","3","U","Flying
If an opponent controls a Mountain and you control an Island, you may cast Saprazzan Legate without paying its mana cost.","3U","Creature - Merfolk Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daru Spiritualist","1","1","W","Whenever a Cleric creature you control becomes the target of a spell or ability, it gets +0/+2 until end of turn.","1W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidetsugu's Second Rite","R","If target player has exactly 10 life, Hidetsugu's Second Rite deals 10 damage to that player.","3R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Cavalry","1","4","G","{W}: Llanowar Cavalry gains vigilance until end of turn.","2G","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Towering Baloth","7","6","G","Morph {6}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","6GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jhovall Queen","4","7","W","Vigilance","4WW","Creature - Cat Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootbreaker Wurm","6","6","G","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","5GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devout Harpist","1","1","W","{T}: Destroy target Aura attached to a creature.","W","Creature - Human");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skull Catapult","{1}, {T}, Sacrifice a creature: Skull Catapult deals 2 damage to target creature or player.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Esper Panorama","{T}: Add {1} to your mana pool.
{1}, {T}, Sacrifice Esper Panorama: Search your library for a basic Plains, Island, or Swamp card and put it onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Droning Bureaucrats","1","4","W","{X}, {T}: Each creature with converted mana cost X can't attack or block this turn.","3W","Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dazzling Beauty","W","Cast Dazzling Beauty only during the declare blockers step.
Target unblocked attacking creature becomes blocked. (This spell works on unblockable creatures.)
Draw a card at the beginning of the next turn's upkeep.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Vampire","3","3","B","Flying
Discard a creature card: Skyshroud Vampire gets +2/+2 until end of turn.","3BB","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thoughtcast","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Draw two cards.","4U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swirl the Mists","U","As Swirl the Mists enters the battlefield, choose a color word.
All instances of color words in the text of spells and permanents are changed to the chosen color word.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightning Mauler","2","1","R","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Lightning Mauler is paired with another creature, both creatures have haste.","1R","Creature - Human Berserker");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Scrapbasket","3","2","{1}: Scrapbasket becomes all colors until end of turn.","4","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Molder Slug","4","6","G","At the beginning of each player's upkeep, that player sacrifices an artifact.","3GG","Creature - Slug Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ursapine","3","3","G","{G}: Target creature gets +1/+1 until end of turn.","3GG","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cliffside Market","When you planeswalk to Cliffside Market or at the beginning of your upkeep, you may exchange life totals with target player.
Whenever you roll {C}, exchange control of two target permanents that share a card type.",NULL,"Plane - Mercadia");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Graypelt Refuge","Graypelt Refuge enters the battlefield tapped.
When Graypelt Refuge enters the battlefield, you gain 1 life.
{T}: Add {G} or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Forgotten Ancient","0","3","G","Whenever a player casts a spell, you may put a +1/+1 counter on Forgotten Ancient.
At the beginning of your upkeep, you may move any number of +1/+1 counters from Forgotten Ancient onto other creatures.","3G","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dogpile","R","Dogpile deals damage to target creature or player equal to the number of attacking creatures you control.","3R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voice of Duty","2","2","W","Flying, protection from green","3W","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Necrogen Spellbomb","{B}, Sacrifice Necrogen Spellbomb: Target player discards a card.
{1}, Sacrifice Necrogen Spellbomb: Draw a card.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Weed Strangle","B","Destroy target creature. Clash with an opponent. If you win, you gain life equal to that creature's toughness. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Metallurgeon","1","2","W","{W}, {T}: Regenerate target artifact.","1W","Artifact Creature - Human Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sea Kings' Blessing","U","Any number of target creatures become blue until end of turn.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sylvan Paradise","G","Any number of target creatures become green until end of turn.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Anvil of Bogardan","Players have no maximum hand size.
At the beginning of each player's draw step, that player draws an additional card, then discards a card.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of the Waning Moon","1","1","B","Flying
Whenever you cast a Spirit or Arcane spell, target creature gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","2B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ardent Militia","2","5","W","Vigilance","4W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Craterhoof Behemoth","5","5","G","Haste
When Craterhoof Behemoth enters the battlefield, creatures you control gain trample and get +X/+X until end of turn, where X is the number of creatures you control.","5GGG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Batwing Brume","WB","Prevent all combat damage that would be dealt this turn if {W} was spent to cast Batwing Brume. Each player loses 1 life for each attacking creature he or she controls if {B} was spent to cast Batwing Brume. (Do both if {W}{B} was spent.)","1(W/B)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Free-Range Chicken","3","3","G","o1oG: Roll two six-sided dice. If both die rolls are the same, Free-Range Chicken gets +X/+X until end of turn, where X is the number rolled on each die. Otherwise, if the total rolled is equal to any other total you have rolled this turn for Free-Range Chicken, sacrifice it. (For example, if you roll two 3s, Free-Range Chicken gets +3/+3. If you roll a total of 6 for Free-Range Chicken later in that turn, sacrifice it.)","3G","Summon - Chicken");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shield of Kaldra","Equipment named Sword of Kaldra, Shield of Kaldra, and Helm of Kaldra are indestructible.
Equipped creature is indestructible. (""Destroy"" effects and lethal damage don't destroy it.)
Equip {4}","4","Legendary Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mahamoti Djinn","5","6","U","Flying (This creature can't be blocked except by creatures with flying or reach.)","4UU","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wrecking Ball","BR","Destroy target creature or land.","2BR","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Juggernaut","5","5","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Phyrexian Juggernaut attacks each turn if able.","6","Artifact Creature - Juggernaut");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("White Sun's Zenith","W","Put X 2/2 white Cat creature tokens onto the battlefield. Shuffle White Sun's Zenith into its owner's library.","XWWW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Anti-Magic Aura","U","Enchant creature
Enchanted creature can't be the target of spells and can't be enchanted. This effect doesn't remove Anti-Magic Aura.","2U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Orzhov Basilica","Orzhov Basilica enters the battlefield tapped.
When Orzhov Basilica enters the battlefield, return a land you control to its owner's hand.
{T}: Add {W}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Callous Giant","4","4","R","If a source would deal 3 or less damage to Callous Giant, prevent that damage.","4RR","Creature - Giant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Maralen of the Mornsong Avatar","At the beginning of the game, you may pay any amount of life.
You can't draw cards.
At the beginning of your draw step, look at the top X cards of your library, where X is the amount of life paid with Maralen of the Mornsong Avatar. Put one of them into your hand, then shuffle your library.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonic Torment","B","Enchant creature
Enchanted creature can't attack.
Prevent all combat damage that would be dealt by enchanted creature.","2B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Dark Barony","Whenever a nonblack card is put into a player's graveyard from anywhere, that player loses 1 life.
Whenever you roll {C}, each opponent discards a card.",NULL,"Plane - Ulgrotha");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guttural Response","RG","Counter target blue instant spell.","(R/G)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consume Strength","BG","Target creature gets +2/+2 until end of turn. Another target creature gets -2/-2 until end of turn.","1BG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charnelhoard Wurm","6","6","BRG","Trample
Whenever Charnelhoard Wurm deals damage to an opponent, you may return target card from your graveyard to your hand.","4BRG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aku Djinn","5","6","B","Trample
At the beginning of your upkeep, put a +1/+1 counter on each creature each opponent controls.","3BB","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vault Skirge","1","1","B","({(b/p)} can be paid with either {B} or 2 life.)
Flying
Lifelink (Damage dealt by this creature also causes you to gain that much life.)","1(B/P)","Artifact Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Champion Lancer","3","3","W","Prevent all damage that would be dealt to Champion Lancer by creatures.","4WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Secretkeeper","2","2","U","As long as you have more cards in hand than each opponent, Secretkeeper gets +2/+2 and has flying.","3U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Crookshank Kobolds","0","1",NULL,"0","Creature - Kobold");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbarian General","3","2","R","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","4R","Creature - Human Barbarian Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skittish Kavu","1","1","R","Skittish Kavu gets +1/+1 as long as no opponent controls a white or blue creature.","1R","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Feral Throwback","3","3","G","Amplify 2 (As this creature enters the battlefield, put two +1/+1 counters on it for each Beast card you reveal in your hand.)
Provoke (When this attacks, you may have target creature defending player controls untap and block it if able.)","4GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alpha Status","G","Enchant creature
Enchanted creature gets +2/+2 for each other creature on the battlefield that shares a creature type with it.","2G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Noggin Whack","B","Prowl {1}{B} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Rogue.)
Target player reveals three cards from his or her hand. You choose two of them. That player discards those cards.","2BB","Tribal Sorcery - Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Excise","W","Exile target attacking creature unless its controller pays {X}.","XW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Murderous Betrayal","B","{B}{B}, Pay half your life, rounded up: Destroy target nonblack creature. It can't be regenerated.","BBB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moorland Inquisitor","2","2","W","{2}{W}: Moorland Inquisitor gains first strike until end of turn.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inferno Titan","6","6","R","{R}: Inferno Titan gets +1/+0 until end of turn.
Whenever Inferno Titan enters the battlefield or attacks, it deals 3 damage divided as you choose among one, two, or three target creatures and/or players.","4RR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knucklebone Witch","1","1","B","Whenever a Goblin you control dies, you may put a +1/+1 counter on Knucklebone Witch.","B","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Rift","RG","At the beginning of your upkeep, discard a card at random. If you discard a creature card this way, return it from your graveyard to the battlefield unless any player pays 5 life.","1RG","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Arcbound Stinger","0","0","Flying
Modular 1 (This enters the battlefield with a +1/+1 counter on it. When it dies, you may put its +1/+1 counters on target artifact creature.)","2","Artifact Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Femeref Knight","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{W}: Femeref Knight gains vigilance until end of turn.","2W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("World Queller","4","4","W","At the beginning of your upkeep, you may choose a card type. If you do, each player sacrifices a permanent of that type.","3WW","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Charge Across the Araba","W","Sweep - Return any number of Plains you control to their owner's hand. Creatures you control get +1/+1 until end of turn for each Plains returned this way.","4W","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Decree","U","All creatures lose flying and islandwalk.","2UU","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breath of Dreams","U","Cumulative upkeep {U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Green creatures have ""Cumulative upkeep {1}.""","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ishi-Ishi, Akki Crackshot","1","1","R","Whenever an opponent casts a Spirit or Arcane spell, Ishi-Ishi, Akki Crackshot deals 2 damage to that player.","1R","Legendary Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghosthelm Courier","2","1","U","You may choose not to untap Ghosthelm Courier during your untap step.
{2}{U}, {T}: Target Wizard creature gets +2/+2 and has shroud for as long as Ghosthelm Courier remains tapped. (It can't be the target of spells or abilities.)","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Forest Bear","2","2","G",NULL,"1G","Creature - Bear");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shared Triumph","W","As Shared Triumph enters the battlefield, choose a creature type.
Creatures of the chosen type get +1/+1.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rock Hydra","0","0","R","Rock Hydra enters the battlefield with X +1/+1 counters on it.
For each 1 damage that would be dealt to Rock Hydra, if it has a +1/+1 counter on it, remove a +1/+1 counter from it and prevent that 1 damage.
{R}: Prevent the next 1 damage that would be dealt to Rock Hydra this turn.
{R}{R}{R}: Put a +1/+1 counter on Rock Hydra. Activate this ability only during your upkeep.","XRR","Creature - Hydra");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sadistic Augermage","3","1","B","When Sadistic Augermage dies, each player puts a card from his or her hand on top of his or her library.","2B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time Walk","U","Take an extra turn after this one.","1U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Roar of Reclamation","W","Each player returns all artifact cards from his or her graveyard to the battlefield.","5WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necromantic Thirst","B","Enchant creature
Whenever enchanted creature deals combat damage to a player, you may return target creature card from your graveyard to your hand.","2BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time and Tide","U","Simultaneously, all phased-out creatures phase in and all creatures with phasing phase out.","UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Kiss","B","Last Kiss deals 2 damage to target creature and you gain 2 life.","2B","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Platinum Emperion","8","8","Your life total can't change. (You can't gain or lose life. You can't pay any amount of life except 0.)","8","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grasp of Darkness","B","Target creature gets -4/-4 until end of turn.","BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invoke Prejudice","U","Whenever an opponent casts a creature spell that doesn't share a color with a creature you control, counter that spell unless its controller pays {X}, where X is its converted mana cost.","UUUU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scalebane's Elite","4","4","WG","Protection from black","3GW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Paraselene","W","Destroy all enchantments. You gain 1 life for each enchantment destroyed this way.","2W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Erdwal Ripper","2","1","R","Haste
Whenever Erdwal Ripper deals combat damage to a player, put a +1/+1 counter on it.","1RR","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ignoble Soldier","3","1","W","Whenever Ignoble Soldier becomes blocked, prevent all combat damage that would be dealt by it this turn.","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disfigure","B","Target creature gets -2/-2 until end of turn.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rite of Consumption","B","As an additional cost to cast Rite of Consumption, sacrifice a creature.
Rite of Consumption deals damage equal to the sacrificed creature's power to target player. You gain life equal to the damage dealt this way.","1B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Staff of Zegon","{3}, {T}: Target creature gets -2/-0 until end of turn.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Goblin Lyre","Sacrifice Goblin Lyre: Flip a coin. If you win the flip, Goblin Lyre deals damage to target opponent equal to the number of creatures you control. If you lose the flip, Goblin Lyre deals damage to you equal to the number of creatures that opponent controls.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vigor","6","6","G","Trample
If damage would be dealt to a creature you control other than Vigor, prevent that damage. Put a +1/+1 counter on that creature for each 1 damage prevented this way.
When Vigor is put into a graveyard from anywhere, shuffle it into its owner's library.","3GGG","Creature - Elemental Incarnation");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightscape Familiar","1","1","B","Blue spells and red spells you cast cost {1} less to cast.
{1}{B}: Regenerate Nightscape Familiar.","1B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Restock","G","Return two target cards from your graveyard to your hand. Exile Restock.","3GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloud Spirit","3","1","U","Flying
Cloud Spirit can block only creatures with flying.","2U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Knowledge Exploitation","U","Prowl {3}{U} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Rogue.)
Search target opponent's library for an instant or sorcery card. You may cast that card without paying its mana cost. Then that player shuffles his or her library.","5UU","Tribal Sorcery - Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Season of the Witch","B","At the beginning of your upkeep, sacrifice Season of the Witch unless you pay 2 life.
At the beginning of the end step, destroy all untapped creatures that didn't attack this turn, except for creatures that couldn't attack.","BBB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faith Healer","1","1","W","Sacrifice an enchantment: You gain life equal to the sacrificed enchantment's converted mana cost.","1W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Imperial Mask","W","When Imperial Mask enters the battlefield, if it's not a token, each of your teammates puts a token that's a copy of Imperial Mask onto the battlefield.
You have hexproof. (You can't be the target of spells or abilities your opponents control.)","4W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warmth","W","Whenever an opponent casts a red spell, you gain 2 life.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frogtosser Banneret","1","1","B","Haste
Goblin spells and Rogue spells you cast cost {1} less to cast.","1B","Creature - Goblin Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thunderstaff","As long as Thunderstaff is untapped, if a creature would deal combat damage to you, prevent 1 of that damage.
{2}, {T}: Attacking creatures get +1/+0 until end of turn.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corrupt","B","Corrupt deals damage equal to the number of Swamps you control to target creature or player. You gain life equal to the damage dealt this way.","5B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darigaaz's Caldera","When Darigaaz's Caldera enters the battlefield, sacrifice it unless you return a non-Lair land you control to its owner's hand.
{T}: Add {B}, {R}, or {G} to your mana pool.",NULL,"Land - Lair");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Entangling Trap","W","Whenever you clash, tap target creature an opponent controls. If you won, that creature doesn't untap during its controller's next untap step. (This ability triggers after the clash ends.)","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Land's Edge","R","Discard a card: If the discarded card is a land card, Land's Edge deals 2 damage to target player. Any player may activate this ability.","1RR","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Matron","1","1","R","When Goblin Matron enters the battlefield, you may search your library for a Goblin card, reveal that card, and put it into your hand. If you do, shuffle your library.","2R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chaotic Goo","0","0","R","Chaotic Goo enters the battlefield with three +1/+1 counters on it.
At the beginning of your upkeep, you may flip a coin. If you win the flip, put a +1/+1 counter on Chaotic Goo. If you lose the flip, remove a +1/+1 counter from Chaotic Goo.","2RR","Creature - Ooze");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Gemini Engine","3","4","Whenever Gemini Engine attacks, put a colorless Construct artifact creature token named Twin onto the battlefield attacking. Its power is equal to Gemini Engine's power and its toughness is equal to Gemini Engine's toughness. Sacrifice the token at end of combat.","6","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Pyromancer","2","2","R","When Goblin Pyromancer enters the battlefield, Goblin creatures get +3/+0 until end of turn.
At the beginning of the end step, destroy all Goblins.","3R","Creature - Goblin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Soulgazer","3","3","W","Flying
{2}{W}: Look at target face-down creature.","3WW","Creature - Bird Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grave-Shell Scarab","4","4","BG","{1}, Sacrifice Grave-Shell Scarab: Draw a card.
Dredge 1 (If you would draw a card, instead you may put exactly one card from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","2BGG","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gustcloak Skirmisher","2","3","W","Flying
Whenever Gustcloak Skirmisher becomes blocked, you may untap it and remove it from combat.","3W","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vorapede","5","4","G","Vigilance, trample
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","2GGG","Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mind's Eye","Whenever an opponent draws a card, you may pay {1}. If you do, draw a card.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mobilization","W","Soldier creatures have vigilance. (Attacking doesn't cause them to tap.)
{2}{W}: Put a 1/1 white Soldier creature token onto the battlefield.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voice of the Provinces","3","3","W","Flying
When Voice of the Provinces enters the battlefield, put a 1/1 white Human creature token onto the battlefield.","4WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blinding Light","W","Tap all nonwhite creatures.","2W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Healer","1","1","R","{R}{R}, {T}: Target creature can't be regenerated this turn.
{B}{B}{R}, {T}: Regenerate target black or green creature.
{R}{G}{G}, {T}: Regenerate target black or green creature.","RR","Creature - Orc Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Entrails Feaster","1","1","B","At the beginning of your upkeep, you may exile a creature card from a graveyard. If you do, put a +1/+1 counter on Entrails Feaster. If you don't, tap Entrails Feaster.","B","Creature - Zombie Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enclave Cryptologist","0","1","U","Level up {1}{U} ({1}{U}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-2
0/1
{T}: Draw a card, then discard a card.
LEVEL 3+
0/1
{T}: Draw a card.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rimescale Dragon","5","5","R","Flying
{2}{S}i}: Tap target creature and put an ice counter on it. ({S}i} can be paid with one mana from a snow permanent.)
Creatures with ice counters on them don't untap during their controllers' untap steps.","5RR","Snow Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Irrigation Ditch","Irrigation Ditch enters the battlefield tapped.
{T}: Add {W} to your mana pool.
{T}, Sacrifice Irrigation Ditch: Add {G}{U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Search for Tomorrow","G","Search your library for a basic land card and put it onto the battlefield. Then shuffle your library.
Suspend 2-{G} (Rather than cast this card from your hand, you may pay {G} and exile it with two time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zealots en-Dal","2","4","W","At the beginning of your upkeep, if all nonland permanents you control are white, you gain 1 life.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coastal Drake","2","1","U","Flying
{1}{U}, {T}: Return target Kavu to its owner's hand.","2U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tidal Surge","U","Tap up to three target creatures without flying.","1U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Worm Harvest","BG","Put a 1/1 black and green Worm creature token onto the battlefield for each land card in your graveyard.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","2(B/G)(B/G)(B/G)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faithful Squire","2","2","W","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Faithful Squire.
At the beginning of the end step, if there are two or more ki counters on Faithful Squire, you may flip it.
---
Flying
Remove a ki counter from Kaiso, Memory of Loyalty: Prevent all damage that would be dealt to target creature this turn.","1WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodflow Connoisseur","1","1","B","Sacrifice a creature: Put a +1/+1 counter on Bloodflow Connoisseur.","2B","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steely Resolve","G","As Steely Resolve enters the battlefield, choose a creature type.
Creatures of the chosen type have shroud. (They can't be the targets of spells or abilities.)","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coiling Woodworm","*","1","G","Coiling Woodworm's power is equal to the number of Forests on the battlefield.","2G","Creature - Insect Worm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Barbarian Ring","{T}: Add {R} to your mana pool. Barbarian Ring deals 1 damage to you.
Threshold - {R}, {T}, Sacrifice Barbarian Ring: Barbarian Ring deals 2 damage to target creature or player. Activate this ability only if seven or more cards are in your graveyard.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repercussion","R","Whenever a creature is dealt damage, Repercussion deals that much damage to that creature's controller.","1RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of the Meek","W","If Leyline of the Meek is in your opening hand, you may begin the game with it on the battlefield.
Creature tokens get +1/+1.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonic Attorney","B","Remove Demonic Attorney from your deck before playing if you're not playing for ante.
Each player antes the top card of his or her library.","1BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mycosynth Wellspring","When Mycosynth Wellspring enters the battlefield or is put into a graveyard from the battlefield, you may search your library for a basic land card, reveal it, put it into your hand, then shuffle your library.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Due Respect","W","Permanents enter the battlefield tapped this turn.
Draw a card.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidehollow Sculler","2","2","WB","When Tidehollow Sculler enters the battlefield, target opponent reveals his or her hand and you choose a nonland card from it. Exile that card.
When Tidehollow Sculler leaves the battlefield, return the exiled card to its owner's hand.","WB","Artifact Creature - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sapseep Forest","({T}: Add {G} to your mana pool.)
Sapseep Forest enters the battlefield tapped.
{G}, {T}: You gain 1 life. Activate this ability only if you control two or more green permanents.",NULL,"Land - Forest");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Watery Grave","({T}: Add {U} or {B} to your mana pool.)
As Watery Grave enters the battlefield, you may pay 2 life. If you don't, Watery Grave enters the battlefield tapped.",NULL,"Land - Island Swamp");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Graveyard Shovel","{2}, {T}: Target player exiles a card from his or her graveyard. If it's a creature card, you gain 2 life.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Perish","B","Destroy all green creatures. They can't be regenerated.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surgical Extraction","B","({(b/p)} can be paid with either {B} or 2 life.)
Choose target card in a graveyard other than a basic land card. Search its owner's graveyard, hand, and library for any number of cards with the same name as that card and exile them. Then that player shuffles his or her library.","(B/P)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spotted Griffin","2","3","W","Flying","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vicious Betrayal","B","As an additional cost to cast Vicious Betrayal, sacrifice any number of creatures.
Target creature gets +2/+2 until end of turn for each creature sacrificed this way.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dream Thrush","1","1","U","Flying
{T}: Target land becomes the basic land type of your choice until end of turn.","1U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of Tattered Shoji","2","5","W","Whenever you cast a Spirit or Arcane spell, Kami of Tattered Shoji gains flying until end of turn.","4W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Confessor","1","1","W","Whenever a player discards a card, you may gain 1 life.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seismic Mage","1","1","R","{2}{R}, {T}, Discard a card: Destroy target land.","3R","Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cry of Contrition","B","Target player discards a card.
Haunt (When this spell card is put into a graveyard after resolving, exile it haunting target creature.)
When the creature Cry of Contrition haunts dies, target player discards a card.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tranquil Grove","G","{1}{G}{G}: Destroy all other enchantments.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Commander","*","*","W","Benalish Commander's power and toughness are each equal to the number of Soldiers you control.
Suspend X-{X}{W}{W}. X can't be 0. (Rather than cast this card from your hand, you may pay {X}{W}{W} and exile it with X time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Whenever a time counter is removed from Benalish Commander while it's exiled, put a 1/1 white Soldier creature token onto the battlefield.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bogardan Lancer","1","1","R","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)","1R","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Growth Spurt","G","Roll a six-sided die. Target creature gets +X/+X until end of turn, where X is equal to the die roll.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rakdos Riteknife","Equipped creature gets +1/+0 for each blood counter on Rakdos Riteknife and has ""{T}, Sacrifice a creature: Put a blood counter on Rakdos Riteknife.""
{B}{R}, Sacrifice Rakdos Riteknife: Target player sacrifices a permanent for each blood counter on Rakdos Riteknife.
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Madblind Mountain","({T}: Add {R} to your mana pool.)
Madblind Mountain enters the battlefield tapped.
{R}, {T}: Shuffle your library. Activate this ability only if you control two or more red permanents.",NULL,"Land - Mountain");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rebel Informer","1","2","B","Rebel Informer can't be the target of white spells or abilities from white sources.
{3}: Put target nontoken Rebel on the bottom of its owner's library.","2B","Creature - Human Mercenary Rebel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arcum's Sleigh","{2}, {T}: Target creature gains vigilance until end of turn. Activate this ability only during combat and only if defending player controls a snow land.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thrun, the Last Troll","4","4","G","Thrun, the Last Troll can't be countered.
Hexproof (This creature can't be the target of spells or abilities your opponents control.)
{1}{G}: Regenerate Thrun.","2GG","Legendary Creature - Troll Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Glacial Fortress","Glacial Fortress enters the battlefield tapped unless you control a Plains or an Island.
{T}: Add {W} or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Strangleroot Geist","2","1","G","Haste
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","GG","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tower of Fortunes","{8}, {T}: Draw four cards.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Constant Mists","G","Buyback-Sacrifice a land. (You may sacrifice a land in addition to any other costs as you cast this spell. If you do, put this card into your hand as it resolves.)
Prevent all combat damage that would be dealt this turn.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Betrothed of Fire","R","Enchant creature
Sacrifice an untapped creature: Enchanted creature gets +2/+0 until end of turn.
Sacrifice enchanted creature: Creatures you control get +2/+0 until end of turn.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gifts Ungiven","U","Search your library for four cards with different names and reveal them. Target opponent chooses two of those cards. Put the chosen cards into your graveyard and the rest into your hand. Then shuffle your library.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aysen Bureaucrats","1","1","W","{T}: Tap target creature with power 2 or less.","1W","Creature - Human Advisor");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Bauble","{T}, Sacrifice Urza's Bauble: Look at a card at random in target player's hand. You draw a card at the beginning of the next turn's upkeep.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Debaser","2","2","B","Flying
{T}, Sacrifice Phyrexian Debaser: Target creature gets -2/-2 until end of turn.","3B","Creature - Carrier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bitter Ordeal","B","Search target player's library for a card and exile it. Then that player shuffles his or her library.
Gravestorm (When you cast this spell, copy it for each permanent put into a graveyard this turn. You may choose new targets for the copies.)","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kazandu Tuskcaller","1","1","G","Level up {1}{G} ({1}{G}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 2-5
1/1
{T}: Put a 3/3 green Elephant creature token onto the battlefield.
LEVEL 6+
1/1
{T}: Put two 3/3 green Elephant creature tokens onto the battlefield.","1G","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aboshan's Desire","U","Enchant creature
Enchanted creature has flying.
Threshold - Enchanted creature has shroud as long as seven or more cards are in your graveyard. (It can't be the target of spells or abilities.)","U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elemental Resonance","G","Enchant permanent
At the beginning of your precombat main phase, add mana equal to enchanted permanent's mana cost to your mana pool. (Mana cost includes color. If a mana symbol has multiple colors, choose one.)","2GG","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scalding Devil","1","1","R","{2}{R}: Scalding Devil deals 1 damage to target player.","1R","Creature - Devil");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Erithizon","4","4","G","Whenever Erithizon attacks, put a +1/+1 counter on target creature of defending player's choice.","2GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clear the Land","G","Each player reveals the top five cards of his or her library, puts all land cards revealed this way onto the battlefield tapped, and exiles the rest.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shrouded Serpent","4","4","U","Whenever Shrouded Serpent attacks, defending player may pay {4}. If he or she doesn't, Shrouded Serpent is unblockable this turn.","4UUU","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flickerwisp","3","1","W","Flying
When Flickerwisp enters the battlefield, exile another target permanent. Return that card to the battlefield under its owner's control at the beginning of the next end step.","1WW","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghastlord of Fugue","4","4","UB","Ghastlord of Fugue is unblockable.
Whenever Ghastlord of Fugue deals combat damage to a player, that player reveals his or her hand. You choose a card from it. That player exiles that card.","(U/B)(U/B)(U/B)(U/B)(U/B)","Creature - Spirit Avatar");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Enatu Golem","3","5","When Enatu Golem dies, you gain 4 life.","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Court Hussar","1","3","U","Vigilance
When Court Hussar enters the battlefield, look at the top three cards of your library, then put one of them into your hand and the rest on the bottom of your library in any order.
When Court Hussar enters the battlefield, sacrifice it unless {W} was spent to cast it.","2U","Creature - Vedalken Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Stalker","2","4","B","At the beginning of each player's upkeep, if that player controls a nonblack, nonland permanent, Urborg Stalker deals 1 damage to that player.","3B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Morgue Toad","2","2","B","Sacrifice Morgue Toad: Add {U}{R} to your mana pool.","2B","Creature - Frog");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Promised Kannushi","1","1","G","Soulshift 7 (When this creature dies, you may return target Spirit card with converted mana cost 7 or less from your graveyard to your hand.)","G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rashka the Slayer","3","3","W","Reach (This creature can block creatures with flying.)
Whenever Rashka the Slayer blocks one or more black creatures, Rashka gets +1/+2 until end of turn.","3WW","Legendary Creature - Human Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defang","W","Enchant creature
Prevent all damage that would be dealt by enchanted creature.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Valley Rannet","6","3","RG","Mountaincycling {2}, forestcycling {2} ({2}, Discard this card: Search your library for a Mountain or Forest card, reveal it, and put it into your hand. Then shuffle your library.)","4RG","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Minamo, School at Water's Edge","{T}: Add {U} to your mana pool.
{U}, {T}: Untap target legendary permanent.",NULL,"Legendary Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Filter","Multicolored spells cost up to {2} less to cast.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Biting Tether","U","Enchant creature
You control enchanted creature.
At the beginning of your upkeep, put a -1/-1 counter on enchanted creature.","4U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Life Burst","W","Target player gains 4 life, then gains 4 life for each card named Life Burst in each graveyard.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bant Sureblade","2","1","WUG","As long as you control another multicolored permanent, Bant Sureblade gets +1/+1 and has first strike.","(G/U)W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Larceny","B","Whenever a creature you control deals combat damage to a player, that player discards a card.","3BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Courier Hawk","1","2","W","Flying, vigilance","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Questing Phelddagrif","4","4","WUG","{G}: Questing Phelddagrif gets +1/+1 until end of turn. Target opponent puts a 1/1 green Hippo creature token onto the battlefield.
{W}: Questing Phelddagrif gains protection from black and from red until end of turn. Target opponent gains 2 life.
{U}: Questing Phelddagrif gains flying until end of turn. Target opponent may draw a card.","1GWU","Creature - Phelddagrif");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Dawnblade","2","3","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
When Kitsune Dawnblade enters the battlefield, you may tap target creature.","4W","Creature - Fox Samurai");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Pathrazer of Ulamog","9","9","Annihilator 3 (Whenever this creature attacks, defending player sacrifices three permanents.)
Pathrazer of Ulamog can't be blocked except by three or more creatures.","11","Creature - Eldrazi");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firemane Angel","4","3","WR","Flying, first strike
At the beginning of your upkeep, if Firemane Angel is in your graveyard or on the battlefield, you may gain 1 life.
{6}{R}{R}{W}{W}: Return Firemane Angel from your graveyard to the battlefield. Activate this ability only during your upkeep.","3RWW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mtenda Lion","2","1","G","Whenever Mtenda Lion attacks, defending player may pay {U}. If that player does, prevent all combat damage that would be dealt by Mtenda Lion this turn.","G","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mikaeus, the Lunarch","0","0","W","Mikaeus, the Lunarch enters the battlefield with X +1/+1 counters on it.
{T}: Put a +1/+1 counter on Mikaeus.
{T}, Remove a +1/+1 counter from Mikaeus: Put a +1/+1 counter on each other creature you control.","XW","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nirkana Cutthroat","3","2","B","Level up {2}{B} ({2}{B}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-2
4/3
Deathtouch
LEVEL 3+
5/4
First strike, deathtouch","2B","Creature - Vampire Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scion of the Ur-Dragon","4","4","WUBRG","Flying
{2}: Search your library for a Dragon permanent card and put it into your graveyard. If you do, Scion of the Ur-Dragon becomes a copy of that card until end of turn. Then shuffle your library.","WUBRG","Legendary Creature - Dragon Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Augur il-Vec","1","3","W","Shadow (This creature can block or be blocked by only creatures with shadow.)
Sacrifice Augur il-Vec: You gain 4 life. Activate this ability only during your upkeep.","1W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tower of Eons","{8}, {T}: You gain 10 life.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Evocation","R","At the beginning of each player's upkeep, that player reveals a card at random from his or her hand. If it's a land card, the player puts it onto the battlefield. Otherwise, the player casts it without paying its mana cost if able.","5R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Hoodlums","4","1","B","Bog Hoodlums can't block.
When Bog Hoodlums enters the battlefield, clash with an opponent. If you win, put a +1/+1 counter on Bog Hoodlums. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","5B","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Witch","1","1","B","{B}, {T}, Discard a card: Add {B}{B}{B} to your mana pool.","2B","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jhoira of the Ghitu","2","2","UR","{2}, Exile a nonland card from your hand: Put four time counters on the exiled card. If it doesn't have suspend, it gains suspend. (At the beginning of your upkeep, remove a time counter from that card. When the last is removed, cast it without paying its mana cost. If it's a creature, it has haste.)","1UR","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nullmage Advocate","2","3","G","{T}: Return two target cards from an opponent's graveyard to his or her hand. Destroy target artifact or enchantment.","2G","Creature - Insect Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stonewood Invocation","G","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Target creature gets +5/+5 and gains shroud until end of turn. (It can't be the target of spells or abilities.)","3G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Transfer","U","If the difference between your life total and target player's life total is 5 or less, exchange life totals with that player.","4U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elven Palisade","G","Sacrifice a Forest: Target attacking creature gets -3/-0 until end of turn.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whipgrass Entangler","1","3","W","{1}{W}: Until end of turn, target creature gains ""This creature can't attack or block unless its controller pays {1} for each Cleric on the battlefield.""","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitemare","3","3","WR","Whenever Spitemare is dealt damage, it deals that much damage to target creature or player.","2(R/W)(R/W)","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dirtwater Wraith","1","3","B","Swampwalk
{B}: Dirtwater Wraith gets +1/+0 until end of turn.","3B","Creature - Wraith");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Chance","R","Take an extra turn after this one. At the beginning of that turn's end step, you lose the game.","RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grapeshot","R","Grapeshot deals 1 damage to target creature or player.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leave No Trace","W","Radiance - Destroy target enchantment and each other enchantment that shares a color with it.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pulsemage Advocate","1","3","W","{T}: Return three target cards from an opponent's graveyard to his or her hand. Return target creature card from your graveyard to the battlefield.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindless Null","2","2","B","Mindless Null can't block unless you control a Vampire.","2B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bog Down","B","Kicker-Sacrifice two lands. (You may sacrifice two lands in addition to any other costs as you cast this spell.)
Target player discards two cards. If Bog Down was kicked, that player discards three cards instead.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Cantor","1","1","RG","({(r/g)} can be paid with either {R} or {G}.)
Sacrifice Wild Cantor: Add one mana of any color to your mana pool.","(R/G)","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Will","G","Whenever one or more creatures you control deal combat damage to a player, tap all lands that player controls and untap all lands you control.","2GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunger of the Nim","B","Target creature gets +1/+0 until end of turn for each artifact you control.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benthic Behemoth","7","6","U","Islandwalk","5UUU","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ragamuffyn","2","2","B","Hellbent - {T}, Sacrifice a creature or land: Draw a card. Activate this ability only if you have no cards in hand.","2B","Creature - Zombie Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Advice from the Fae","U","({(2/u)} can be paid with any two mana or with {U}. This card's converted mana cost is 6.)
Look at the top five cards of your library. If you control more creatures than each other player, put two of those cards into your hand. Otherwise, put one of them into your hand. Then put the rest on the bottom of your library in any order.","(2/U)(2/U)(2/U)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rapid Decay","B","Exile up to three target cards from a single graveyard.
Cycling {2} ({2}, Discard this card: Draw a card.)","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chorus of Woe","B","Creatures you control get +1/+0 until end of turn.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Izzet Chronarch","2","2","UR","When Izzet Chronarch enters the battlefield, return target instant or sorcery card from your graveyard to your hand.","3UR","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lost in Thought","U","Enchant creature
Enchanted creature can't attack or block, and its activated abilities can't be activated. Its controller may exile three cards from his or her graveyard for that player to ignore this effect until end of turn.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leaf Gilder","2","1","G","{T}: Add {G} to your mana pool.","1G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beast Hunt","G","Reveal the top three cards of your library. Put all creature cards revealed this way into your hand and the rest into your graveyard.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tombstalker","5","5","B","Flying
Delve (You may exile any number of cards from your graveyard as you cast this spell. It costs {1} less to cast for each card exiled this way.)","6BB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pardic Firecat","2","3","R","Haste
If Pardic Firecat is in a graveyard, effects from spells named Flame Burst count it as a card named Flame Burst.","3R","Creature - Elemental Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Omen Machine","Players can't draw cards.
At the beginning of each player's draw step, that player exiles the top card of his or her library. If it's a land card, the player puts it onto the battlefield. Otherwise, the player casts it without paying its mana cost if able.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Revive","G","Return target green card from your graveyard to your hand.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dimir House Guard","2","3","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Sacrifice a creature: Regenerate Dimir House Guard.
Transmute {1}{B}{B} ({1}{B}{B}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","3B","Creature - Skeleton");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shelldock Isle","Hideaway (This land enters the battlefield tapped. When it does, look at the top four cards of your library, exile one face down, then put the rest on the bottom of your library.)
{T}: Add {U} to your mana pool.
{U}, {T}: You may play the exiled card without paying its mana cost if a library has twenty or fewer cards in it.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tatsumasa, the Dragon's Fang","Equipped creature gets +5/+5.
{6}, Exile Tatsumasa, the Dragon's Fang: Put a 5/5 blue Dragon Spirit creature token with flying onto the battlefield. Return Tatsumasa to the battlefield under its owner's control when that token dies.
Equip {3}","6","Legendary Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thoughtbound Primoc","2","3","R","Flying
At the beginning of your upkeep, if a player controls more Wizards than each other player, the player who controls the most Wizards gains control of Thoughtbound Primoc.","2R","Creature - Bird Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thalakos Scout","2","1","U","Shadow (This creature can block or be blocked by only creatures with shadow.)
Discard a card: Return Thalakos Scout to its owner's hand.","2U","Creature - Thalakos Soldier Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ebon Stronghold","Ebon Stronghold enters the battlefield tapped.
{T}: Add {B} to your mana pool.
{T}, Sacrifice Ebon Stronghold: Add {B}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul of the Harvest","6","6","G","Trample
Whenever another nontoken creature enters the battlefield under your control, you may draw a card.","4GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Side to Side","G","You and target opponent arm-wrestle. If you win, put a 3/3 green Ape creature token into play.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overlaid Terrain","G","As Overlaid Terrain enters the battlefield, sacrifice all lands you control.
Lands you control have ""{T}: Add two mana of any one color to your mana pool.""","2GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Compulsive Research","U","Target player draws three cards. Then that player discards two cards unless he or she discards a land card.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woolly Mammoths","3","2","G","Woolly Mammoths has trample as long as you control a snow land.","1GG","Creature - Elephant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Amber Prison","You may choose not to untap Amber Prison during your untap step.
{4}, {T}: Tap target artifact, creature, or land. That permanent doesn't untap during its controller's untap step for as long as Amber Prison remains tapped.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Feral Thallid","6","3","G","At the beginning of your upkeep, put a spore counter on Feral Thallid.
Remove three spore counters from Feral Thallid: Regenerate Feral Thallid.","3GGG","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mythic Proportions","G","Enchant creature
Enchanted creature gets +8/+8 and has trample.","4GGG","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Amphibious Kavu","2","2","G","Whenever Amphibious Kavu blocks or becomes blocked by one or more blue and/or black creatures, Amphibious Kavu gets +3/+3 until end of turn.","2G","Creature - Kavu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warp World","R","Each player shuffles all permanents he or she owns into his or her library, then reveals that many cards from the top of his or her library. Each player puts all artifact, creature, and land cards revealed this way onto the battlefield, then does the same for enchantment cards, then puts all cards revealed this way that weren't put onto the battlefield on the bottom of his or her library.","5RRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nut Collector","1","1","G","At the beginning of your upkeep, you may put a 1/1 green Squirrel creature token onto the battlefield.
Threshold - Squirrel creatures get +2/+2 as long as seven or more cards are in your graveyard.","5G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Latch Seeker","3","1","U","Latch Seeker is unblockable.","1UU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wildfire Emissary","2","4","R","Protection from white
{1}{R}: Wildfire Emissary gets +1/+0 until end of turn.","3R","Creature - Efreet");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consume Spirit","B","Spend only black mana on X.
Consume Spirit deals X damage to target creature or player and you gain X life.","X1B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Henge of Ramos","{T}: Add {1} to your mana pool.
{2}, {T}: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devouring Strossus","9","9","B","Flying, trample
At the beginning of your upkeep, sacrifice a creature.
Sacrifice a creature: Regenerate Devouring Strossus.","5BBB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minion of Leshrac","5","5","B","Protection from black
At the beginning of your upkeep, Minion of Leshrac deals 5 damage to you unless you sacrifice a creature other than Minion of Leshrac. If Minion of Leshrac deals damage to you this way, tap it.
{T}: Destroy target creature or land.","4BBB","Creature - Demon Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Junún Efreet","3","3","B","Flying
At the beginning of your upkeep, sacrifice Junún Efreet unless you pay {B}{B}.","1BB","Creature - Efreet");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of Vengeance","Equipped creature gets +2/+0 and has first strike, vigilance, trample, and haste.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Root Sliver","2","2","G","Root Sliver can't be countered.
Sliver spells can't be countered by spells or abilities.","3G","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fendeep Summoner","3","5","B","{T}: Up to two target Swamps each become 3/5 Treefolk Warrior creatures in addition to their other types until end of turn.","4B","Creature - Treefolk Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chandra's Phoenix","2","2","R","Flying
Haste (This creature can attack and {T} as soon as it comes under your control.)
Whenever an opponent is dealt damage by a red instant or sorcery spell you control or by a red planeswalker you control, return Chandra's Phoenix from your graveyard to your hand.","1RR","Creature - Phoenix");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sprite Noble","2","2","U","Flying
Other creatures you control with flying get +0/+1.
{T}: Other creatures you control with flying get +1/+0 until end of turn.","1UU","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ichor Explosion","B","As an additional cost to cast Ichor Explosion, sacrifice a creature.
All creatures get -X/-X until end of turn, where X is the sacrificed creature's power.","5BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drumhunter","2","2","G","At the beginning of your end step, if you control a creature with power 5 or greater, you may draw a card.
{T}: Add {1} to your mana pool.","3G","Creature - Human Druid Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wormfang Turtle","2","4","U","When Wormfang Turtle enters the battlefield, exile a land you control.
When Wormfang Turtle leaves the battlefield, return the exiled card to the battlefield under its owner's control.","2U","Creature - Nightmare Turtle Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yellow Scarves Cavalry","1","1","R","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Yellow Scarves Cavalry can't block.","1R","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Calcite Snapper","1","4","U","Shroud (This creature can't be the target of spells or abilities.)
Landfall - Whenever a land enters the battlefield under your control, you may switch Calcite Snapper's power and toughness until end of turn.","1UU","Creature - Turtle");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Silk Net","G","Target creature gets +1/+1 and gains reach until end of turn. (It can block creatures with flying.)","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Turntimber Grove","Turntimber Grove enters the battlefield tapped.
When Turntimber Grove enters the battlefield, target creature gets +1/+1 until end of turn.
{T}: Add {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ikiral Outrider","1","2","W","Level up {4} ({4}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-3
2/6
Vigilance
LEVEL 4+
3/10
Vigilance","1W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Contested War Zone","Whenever a creature deals combat damage to you, that creature's controller gains control of Contested War Zone.
{T}: Add {1} to your mana pool.
{1}, {T}: Attacking creatures get +1/+0 until end of turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Needle Storm","G","Needle Storm deals 4 damage to each creature with flying.","2G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("AEther Spellbomb","{U}, Sacrifice Æther Spellbomb: Return target creature to its owner's hand.
{1}, Sacrifice Æther Spellbomb: Draw a card.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mishra","If a creature you control would deal damage, it deals double that damage instead.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Musician","1","3","U","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
{T}: Put a music counter on target creature. If it doesn't have ""At the beginning of your upkeep, destroy this creature unless you pay {1} for each music counter on it,"" it gains that ability.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Martyr of Ashes","1","1","R","{2}, Reveal X red cards from your hand, Sacrifice Martyr of Ashes: Martyr of Ashes deals X damage to each creature without flying.","R","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of Lightning","R","If Leyline of Lightning is in your opening hand, you may begin the game with it on the battlefield.
Whenever you cast a spell, you may pay {1}. If you do, Leyline of Lightning deals 1 damage to target player.","2RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Briarknit Kami","3","3","G","Whenever you cast a Spirit or Arcane spell, put a +1/+1 counter on target creature.","3GG","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Screams of the Damned","B","{1}{B}, Exile a card from your graveyard: Screams of the Damned deals 1 damage to each creature and each player.","3BB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Grim Poppet","4","4","Grim Poppet enters the battlefield with three -1/-1 counters on it.
Remove a -1/-1 counter from Grim Poppet: Put a -1/-1 counter on another target creature.","7","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orchard Spirit","2","2","G","Orchard Spirit can't be blocked except by creatures with flying or reach.","2G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tranquil Domain","G","Destroy all non-Aura enchantments.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soulcage Fiend","3","2","B","When Soulcage Fiend dies, each player loses 3 life.","1BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Redirect","U","You may choose new targets for target spell.","UU","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Sawtooth Thresher","1","1","Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it.)
Remove two +1/+1 counters from Sawtooth Thresher: Sawtooth Thresher gets +4/+4 until end of turn.","6","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shadow Sliver","1","1","U","All Sliver creatures have shadow. (They can block or be blocked by only creatures with shadow.)","2U","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jin-Gitaxias, Core Augur","5","4","U","Flash
At the beginning of your end step, draw seven cards.
Each opponent's maximum hand size is reduced by seven.","8UU","Legendary Creature - Praetor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Parry","W","Prevent all damage one or two target creatures would deal this turn.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Firewild Borderpost","RG","You may pay {1} and return a basic land you control to its owner's hand rather than pay Firewild Borderpost's mana cost.
Firewild Borderpost enters the battlefield tapped.
{T}: Add {R} or {G} to your mana pool.","1RG","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravedigger","2","2","B","When Gravedigger enters the battlefield, you may return target creature card from your graveyard to your hand.","3B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Wakecaster","2","3","U","Flying
{1}: Mistform Wakecaster becomes the creature type of your choice until end of turn.
{2}{U}{U}, {T}: Choose a creature type. Each creature you control becomes that type until end of turn.","4U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Contemplation","W","Whenever you cast a spell, you gain 1 life.","1WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heavy Mattock","Equipped creature gets +1/+1.
As long as equipped creature is a Human, it gets an additional +1/+1.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rite of Ruin","R","Choose an order for artifacts, creatures, and lands. Each player sacrifices one permanent of the first type, sacrifices two of the second type, then sacrifices three of the third type.","5RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rise of the Hobgoblins","WR","When Rise of the Hobgoblins enters the battlefield, you may pay {X}. If you do, put X 1/1 red and white Goblin Soldier creature tokens onto the battlefield.
{(r/w)}: Red creatures and white creatures you control gain first strike until end of turn.","(R/W)(R/W)","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nefarious Lich","B","If damage would be dealt to you, exile that many cards from your graveyard instead. If you can't, you lose the game.
If you would gain life, draw that many cards instead.
When Nefarious Lich leaves the battlefield, you lose the game.","BBBB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Into the Roil","U","Kicker {1}{U} (You may pay an additional {1}{U} as you cast this spell.)
Return target nonland permanent to its owner's hand. If Into the Roil was kicked, draw a card.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cytoplast Manipulator","0","0","U","Graft 2 (This creature enters the battlefield with two +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{U}, {T}: Gain control of target creature with a +1/+1 counter on it for as long as Cytoplast Manipulator remains on the battlefield.","2UU","Creature - Human Wizard Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crafty Pathmage","1","1","U","{T}: Target creature with power 2 or less is unblockable this turn.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shadowfeed","B","Exile target card from a graveyard. You gain 3 life.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spellgorger Barbarian","3","1","R","When Spellgorger Barbarian enters the battlefield, discard a card at random.
When Spellgorger Barbarian leaves the battlefield, draw a card.","3R","Creature - Human Nightmare Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leeching Licid","1","1","B","{B}, {T}: Leeching Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {B} to end this effect.
At the beginning of the upkeep of enchanted creature's controller, Leeching Licid deals 1 damage to that player.","1B","Creature - Licid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Razorfield Rhino","4","4","Metalcraft - Razorfield Rhino gets +2/+2 as long as you control three or more artifacts.","6","Artifact Creature - Rhino");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fiery Conclusion","R","As an additional cost to cast Fiery Conclusion, sacrifice a creature.
Fiery Conclusion deals 5 damage to target creature.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhox Pikemaster","3","3","W","First strike (This creature deals combat damage before creatures without first strike.)
Other Soldier creatures you control have first strike.","2WW","Creature - Rhino Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corrupted Conscience","U","Enchant creature
You control enchanted creature.
Enchanted creature has infect. (It deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","3UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magnetic Flux","U","Artifact creatures you control gain flying until end of turn.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rise from the Grave","B","Put target creature card from a graveyard onto the battlefield under your control. That creature is a black Zombie in addition to its other colors and types.","4B","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ebony Rhino","4","5","Trample","7","Artifact Creature - Rhino");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Midvast Protector","2","3","W","When Midvast Protector enters the battlefield, target creature you control gains protection from the color of your choice until end of turn.","3W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cataclysm","W","Each player chooses from the permanents he or she controls an artifact, a creature, an enchantment, and a land, then sacrifices the rest.","2WW","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wand of the Elements","{T}, Sacrifice an Island: Put a 2/2 blue Elemental creature token with flying onto the battlefield.
{T}, Sacrifice a Mountain: Put a 3/3 red Elemental creature token onto the battlefield.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Homarid Warrior","3","3","U","{U}: Homarid Warrior gains shroud until end of turn and doesn't untap during your next untap step. Tap Homarid Warrior. (A permanent with shroud can't be the target of spells or abilities.)","4U","Creature - Homarid Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oni Possession","B","Enchant creature
At the beginning of your upkeep, sacrifice a creature.
Enchanted creature gets +3/+3 and has trample.
Enchanted creature is a Demon Spirit.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ordruun Commando","4","1","R","{W}: Prevent the next 1 damage that would be dealt to Ordruun Commando this turn.","3R","Creature - Minotaur Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shore Snapper","2","2","B","{U}: Shore Snapper gains islandwalk until end of turn.","2B","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brood of Cockroaches","1","1","B","When Brood of Cockroaches is put into your graveyard from the battlefield, at the beginning of the next end step, you lose 1 life and return Brood of Cockroaches to your hand.","1B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Tide","U","As an additional cost to cast Æther Tide, discard X creature cards.
Return X target creatures to their owners' hands.","XU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coral Net","U","Enchant green or white creature
Enchanted creature has ""At the beginning of your upkeep, sacrifice this creature unless you discard a card.""","U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("White Mana Battery","{2}, {T}: Put a charge counter on White Mana Battery.
{T}, Remove any number of charge counters from White Mana Battery: Add {W} to your mana pool, then add an additional {W} to your mana pool for each charge counter removed this way.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gruesome Deformity","B","Enchant creature
Enchanted creature has intimidate. (It can't be blocked except by artifact creatures and/or creatures that share a color with it.)","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Light of Sanction","W","Prevent all damage that would be dealt to creatures you control by sources you control.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caregiver","1","1","W","{W}, Sacrifice a creature: Prevent the next 1 damage that would be dealt to target creature or player this turn.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zo-Zu the Punisher","2","2","R","Whenever a land enters the battlefield, Zo-Zu the Punisher deals 2 damage to that land's controller.","1RR","Legendary Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunfire Balm","W","Prevent the next 4 damage that would be dealt to target creature or player this turn.
Cycling {1}{W} ({1}{W}, Discard this card: Draw a card.)
When you cycle Sunfire Balm, you may prevent the next 1 damage that would be dealt to target creature or player this turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ember-Fist Zubera","1","2","R","When Ember-Fist Zubera dies, it deals damage to target creature or player equal to the number of Zubera that died this turn.","1R","Creature - Zubera Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghitu Firebreathing","R","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
{R}: Enchanted creature gets +1/+0 until end of turn.
{R}: Return Ghitu Firebreathing to its owner's hand.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wear Away","G","Destroy target artifact or enchantment.
Splice onto Arcane {3}{G} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","GG","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Ricochet","R","You may choose new targets for target instant or sorcery spell. Then copy that spell. You may choose new targets for the copy.","2RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Welkin Hawk","1","1","W","Flying
When Welkin Hawk dies, you may search your library for a card named Welkin Hawk, reveal that card, put it into your hand, then shuffle your library.","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hell-Bent Raider","2","2","R","First strike, haste
Discard a card at random: Hell-Bent Raider gains protection from white until end of turn.","1RR","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lotus Cobra","2","1","G","Landfall - Whenever a land enters the battlefield under your control, you may add one mana of any color to your mana pool.","1G","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmal Fiend","1","5","B","{B}: Phantasmal Fiend gets +1/-1 until end of turn.
{1}{U}: Switch Phantasmal Fiend's power and toughness until end of turn.","3B","Creature - Illusion");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Eater of Days","9","8","Flying, trample
When Eater of Days enters the battlefield, you skip your next two turns.","4","Artifact Creature - Leviathan");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashen Firebeast","6","6","R","{1}{R}: Ashen Firebeast deals 1 damage to each creature without flying.","6RR","Creature - Elemental Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Worry Beads","At the beginning of each player's upkeep, that player puts the top card of his or her library into his or her graveyard.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Singing Tree","0","3","G","{T}: Target attacking creature's power becomes 0 until end of turn.","3G","Creature - Plant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Absolute Grace","W","All creatures have protection from black.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barrenton Cragtreads","3","3","WU","Barrenton Cragtreads can't be blocked by red creatures.","2(W/U)(W/U)","Creature - Kithkin Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battle Squadron","*","*","R","Flying
Battle Squadron's power and toughness are each equal to the number of creatures you control.","3RR","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deeptread Merrow","2","1","U","{U}: Deeptread Merrow gains islandwalk until end of turn.","1U","Creature - Merfolk Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Serrated Arrows","Serrated Arrows enters the battlefield with three arrowhead counters on it.
At the beginning of your upkeep, if there are no arrowhead counters on Serrated Arrows, sacrifice it.
{T}, Remove an arrowhead counter from Serrated Arrows: Put a -1/-1 counter on target creature.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fury Sliver","3","3","R","All Sliver creatures have double strike.","5R","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("May Civilization Collapse","When you set this scheme in motion, target opponent chooses self or others. If that player chooses self, he or she sacrifices two lands. If the player chooses others, each of your other opponents sacrifices a land.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patron of the Nezumi","6","6","B","Rat offering (You may cast this card any time you could cast an instant by sacrificing a Rat and paying the difference in mana costs between this and the sacrificed Rat. Mana cost includes color.)
Whenever a permanent is put into an opponent's graveyard, that player loses 1 life.","5BB","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deny Reality","UB","Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)
Return target permanent to its owner's hand.","3UB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angelic Page","1","1","W","Flying
{T}: Target attacking or blocking creature gets +1/+1 until end of turn.","1W","Creature - Angel Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Uphill Battle","R","Creatures played by your opponents enter the battlefield tapped.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Mindsucker","2","2","B","{B}, Sacrifice Urborg Mindsucker: Target opponent discards a card at random. Activate this ability only any time you could cast a sorcery.","2B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crown of Fury","R","Enchant creature
Enchanted creature gets +1/+0 and has first strike.
Sacrifice Crown of Fury: Enchanted creature and other creatures that share a creature type with it get +1/+0 and gain first strike until end of turn.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Meglonoth","6","6","WRG","Vigilance, trample
Whenever Meglonoth blocks a creature, Meglonoth deals damage to that creature's controller equal to Meglonoth's power.","3RGW","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tangle Asp","1","2","G","Whenever Tangle Asp blocks or becomes blocked by a creature, destroy that creature at end of combat.","1G","Creature - Snake");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Jangling Automaton","3","2","Whenever Jangling Automaton attacks, untap all creatures defending player controls.","3","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kessig Wolf Run","{T}: Add {1} to your mana pool.
{X}{R}{G}, {T}: Target creature gets +X/+0 and gains trample until end of turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curse of Wizardry","B","As Curse of Wizardry enters the battlefield, choose a color.
Whenever a player casts a spell of the chosen color, that player loses 1 life.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Spin Engine","3","1","{R}: Target creature can't block Spin Engine this turn.","3","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mayael the Anima","2","3","WRG","{3}{R}{G}{W}, {T}: Look at the top five cards of your library. You may put a creature card with power 5 or greater from among them onto the battlefield. Put the rest on the bottom of your library in any order.","RGW","Legendary Creature - Elf Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hollow Trees","Hollow Trees enters the battlefield tapped.
You may choose not to untap Hollow Trees during your untap step.
At the beginning of your upkeep, if Hollow Trees is tapped, put a storage counter on it.
{T}, Remove any number of storage counters from Hollow Trees: Add {G} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eight-and-a-Half-Tails","2","2","W","{1}{W}: Target permanent you control gains protection from white until end of turn.
{1}: Target spell or permanent becomes white until end of turn.","WW","Legendary Creature - Fox Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eyes of the Wisent","G","Whenever an opponent casts a blue spell during your turn, you may put a 4/4 green Elemental creature token onto the battlefield.","1G","Tribal Enchantment - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armament Master","2","2","W","Other Kor creatures you control get +2/+2 for each Equipment attached to Armament Master.","WW","Creature - Kor Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Living Terrain","G","Enchant land
Enchanted land is a 5/6 green Treefolk creature that's still a land.","2GG","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sustainer of the Realm","2","3","W","Flying
Whenever Sustainer of the Realm blocks, it gets +0/+2 until end of turn.","2WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("D'Avenant Archer","1","2","W","{T}: D'Avenant Archer deals 1 damage to target attacking or blocking creature.","2W","Creature - Human Soldier Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Drain","B","Essence Drain deals 3 damage to target creature or player and you gain 3 life.","4B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Discordant Dirge","B","At the beginning of your upkeep, you may put a verse counter on Discordant Dirge.
{B}, Sacrifice Discordant Dirge: Look at target opponent's hand and choose up to X cards from it, where X is the number of verse counters on Discordant Dirge. That player discards those cards.","3BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rite of Flame","R","Add {R}{R} to your mana pool, then add {R} to your mana pool for each card named Rite of Flame in each graveyard.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flailing Ogre","3","3","R","{1}: Flailing Ogre gets +1/+1 until end of turn. Any player may activate this ability.
{1}: Flailing Ogre gets -1/-1 until end of turn. Any player may activate this ability.","2R","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crypt Sliver","1","1","B","All Slivers have ""{T}: Regenerate target Sliver.""","1B","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rage Thrower","4","2","R","Whenever another creature dies, Rage Thrower deals 2 damage to target player.","5R","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Edgewalker","2","2","WB","Cleric spells you cast cost {W}{B} less to cast. This effect reduces only the amount of colored mana you pay. (For example, if you cast a Cleric spell with mana cost {1}{W}, it costs {1} to cast.)","1WB","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moment of Silence","W","Target player skips his or her next combat phase this turn.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dryad's Caress","G","You gain 1 life for each creature on the battlefield. If {W} was spent to cast Dryad's Caress, untap all creatures you control.","4GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eternal Witness","2","1","G","When Eternal Witness enters the battlefield, you may return target card from your graveyard to your hand.","1GG","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turn to Dust","G","Destroy target Equipment. Add {G} to your mana pool.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shrink","G","Target creature gets -5/-0 until end of turn.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tranquil Garden","{T}: Add {1} to your mana pool.
{T}: Add {G} or {W} to your mana pool. Tranquil Garden doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rith's Charm","WRG","Choose one - Destroy target nonbasic land; or put three 1/1 green Saproling creature tokens onto the battlefield; or prevent all damage a source of your choice would deal this turn.","RGW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fume Spitter","1","1","B","Sacrifice Fume Spitter: Put a -1/-1 counter on target creature.","B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dream Prowler","1","5","U","Dream Prowler is unblockable as long as it's attacking alone.","2UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shambling Strider","5","5","G","{R}{G}: Shambling Strider gets +1/-1 until end of turn.","4GG","Creature - Yeti");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lantern of Insight","Each player plays with the top card of his or her library revealed.
{T}, Sacrifice Lantern of Insight: Target player shuffles his or her library.","1","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Adapter","1","1","Myr Adapter gets +1/+1 for each Equipment attached to it.","3","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raka Disciple","1","1","R","{W}, {T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.
{U}, {T}: Target creature gains flying until end of turn.","R","Creature - Minotaur Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bant Sojourners","2","4","WUG","When you cycle Bant Sojourners or it dies, you may put a 1/1 white Soldier creature token onto the battlefield.
Cycling {2}{W} ({2}{W}, Discard this card: Draw a card.)","1GWU","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lethal Vapors","B","Whenever a creature enters the battlefield, destroy it.
{0}: Destroy Lethal Vapors. You skip your next turn. Any player may activate this ability.","2BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shower of Coals","R","Shower of Coals deals 2 damage to each of up to three target creatures and/or players.
Threshold - Shower of Coals deals 4 damage to each of those creatures and/or players instead if seven or more cards are in your graveyard.","3RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Might of Oaks","G","Target creature gets +7/+7 until end of turn.","3G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Swords","B","If you control a Swamp, you may sacrifice a creature rather than pay Mind Swords's mana cost.
Each player exiles two cards from his or her hand.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riku of Two Reflections","2","2","URG","Whenever you cast an instant or sorcery spell, you may pay {U}{R}. If you do, copy that spell. You may choose new targets for the copy.
Whenever another nontoken creature enters the battlefield under your control, you may pay {G}{U}. If you do, put a token that's a copy of that creature onto the battlefield.","2URG","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Resurgence","G","Each player draws a card for each creature card in his or her graveyard.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cave Tiger","2","2","G","Whenever Cave Tiger becomes blocked by a creature, Cave Tiger gets +1/+1 until end of turn.","2G","Creature - Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Erratic Portal","{1}, {T}: Return target creature to its owner's hand unless its controller pays {1}.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Laccolith Rig","R","Enchant creature
Whenever enchanted creature becomes blocked, you may have it deal damage equal to its power to target creature. If you do, the first creature assigns no combat damage this turn.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Footsteps of the Goryo","B","Return target creature card from your graveyard to the battlefield. Sacrifice that creature at the beginning of the next end step.","2B","Sorcery - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Armillary Sphere","{2}, {T}, Sacrifice Armillary Sphere: Search your library for up to two basic land cards, reveal them, and put them into your hand. Then shuffle your library.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aura Gnarlid","2","2","G","Creatures with power less than Aura Gnarlid's power can't block it.
Aura Gnarlid gets +1/+1 for each Aura on the battlefield.","2G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barony Vampire","3","2","B",NULL,"2B","Creature - Vampire");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Patagia Golem","2","3","{3}: Patagia Golem gains flying until end of turn.","4","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scarwood Treefolk","3","5","G","Scarwood Treefolk enters the battlefield tapped.","3G","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strength of Lunacy","B","Enchant creature
Enchanted creature gets +2/+1 and has protection from white.
Madness {B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Symbiotic Deployment","G","Skip your draw step.
{1}, Tap two untapped creatures you control: Draw a card.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Liege of the Axe","2","3","W","Vigilance
Morph {1}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Liege of the Axe is turned face up, untap it.","3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Might of the Nephilim","G","Target creature gets +2/+2 until end of turn for each of its colors.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistmeadow Skulk","1","1","W","Lifelink, protection from converted mana cost 3 or greater","1W","Creature - Kithkin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dreamwinder","4","3","U","Dreamwinder can't attack unless defending player controls an Island.
{U}, Sacrifice an Island: Target land becomes an Island until end of turn.","3U","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vesuvan Doppelganger","0","0","U","You may have Vesuvan Doppelganger enter the battlefield as a copy of any creature on the battlefield except it doesn't copy that creature's color and it gains ""At the beginning of your upkeep, you may have this creature become a copy of target creature except it doesn't copy that creature's color. If you do, this creature gains this ability.""","3UU","Creature - Shapeshifter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Noetic Scales","At the beginning of each player's upkeep, return to its owner's hand each creature that player controls with power greater than the number of cards in his or her hand.","4","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lotus Guardian","4","4","Flying
{T}: Add one mana of any color to your mana pool.","7","Artifact Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hammerfist Giant","5","4","R","{T}: Hammerfist Giant deals 4 damage to each creature without flying and each player.","4RR","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Acceptable Losses","R","As an additional cost to cast Acceptable Losses, discard a card at random.
Acceptable Losses deals 5 damage to target creature.","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Predatory Focus","G","You may have creatures you control assign their combat damage this turn as though they weren't blocked.","3GG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Svyelunite Temple","Svyelunite Temple enters the battlefield tapped.
{T}: Add {U} to your mana pool.
{T}, Sacrifice Svyelunite Temple: Add {U}{U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Augury Adept","2","2","WU","Whenever Augury Adept deals combat damage to a player, reveal the top card of your library and put that card into your hand. You gain life equal to its converted mana cost.","1(W/U)(W/U)","Creature - Kithkin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voyager Drake","3","3","U","Multikicker {U} (You may pay an additional {U} any number of times as you cast this spell.)
Flying
When Voyager Drake enters the battlefield, up to X target creatures gain flying until end of turn, where X is the number of times Voyager Drake was kicked.","3U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of Old Stone","1","7","W",NULL,"3W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guardian of Cloverdell","4","5","G","When Guardian of Cloverdell enters the battlefield, put three 1/1 white Kithkin Soldier creature tokens onto the battlefield.
{G}, Sacrifice a Kithkin: You gain 1 life.","5GG","Creature - Treefolk Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sengir Bats","1","2","B","Flying
Whenever a creature dealt damage by Sengir Bats this turn dies, put a +1/+1 counter on Sengir Bats.","1BB","Creature - Bat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Hydra","7","7","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
If damage would be dealt to Phyrexian Hydra, prevent that damage. Put a -1/-1 counter on Phyrexian Hydra for each 1 damage prevented this way.","3GG","Creature - Hydra");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakdos Pit Dragon","3","3","R","{R}{R}: Rakdos Pit Dragon gains flying until end of turn.
{R}: Rakdos Pit Dragon gets +1/+0 until end of turn.
Hellbent - Rakdos Pit Dragon has double strike as long as you have no cards in hand.","2RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guma","2","2","R","Protection from blue","2R","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ambush","R","Blocking creatures gain first strike until end of turn.","3R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rainbow Crow","2","2","U","Flying
{1}: Rainbow Crow becomes the color of your choice until end of turn.","3U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Hellkite","5","5","R","Flying (This creature can't be blocked except by creatures with flying or reach.)
{1}{R}: Shivan Hellkite deals 1 damage to target creature or player.","5RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flesh-Eater Imp","2","2","B","Flying
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Sacrifice a creature: Flesh-Eater Imp gets +1/+1 until end of turn.","3B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Feral Ridgewolf","1","2","R","Trample
{1}{R}: Feral Ridgewolf gets +2/+0 until end of turn.","2R","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Mutant","3","4","U","{1}{U}: Choose a creature type other than Wall. Target creature becomes that type until end of turn.","4UU","Creature - Illusion Mutant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wirecat","4","3","Wirecat can't attack or block if an enchantment is on the battlefield.","4","Artifact Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charging Slateback","4","3","R","Charging Slateback can't block.
Morph {4}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Felidar Sovereign","4","6","W","Vigilance, lifelink
At the beginning of your upkeep, if you have 40 or more life, you win the game.","4WW","Creature - Cat Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunting Pack","G","Put a 4/4 green Beast creature token onto the battlefield.
Storm (When you cast this spell, copy it for each spell cast before it this turn.)","5GG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thrull Retainer","B","Enchant creature
Enchanted creature gets +1/+1.
Sacrifice Thrull Retainer: Regenerate enchanted creature.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunderblust","7","2","R","Haste
Thunderblust has trample as long as it has a -1/-1 counter on it.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","2RRR","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dual Nature","G","Whenever a nontoken creature enters the battlefield, its controller puts a token that's a copy of that creature onto the battlefield.
Whenever a nontoken creature leaves the battlefield, exile all tokens with the same name as that creature.
When Dual Nature leaves the battlefield, exile all tokens put onto the battlefield with Dual Nature.","4GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Exotic Curse","B","Enchant creature
Domain - Enchanted creature gets -1/-1 for each basic land type among lands you control.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cunning","U","Enchant creature
Enchanted creature gets +3/+3.
When enchanted creature attacks or blocks, sacrifice Cunning at the beginning of the next cleanup step.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Flotilla","2","2","R","Islandwalk
At the beginning of combat, unless you pay {R}, whenever Goblin Flotilla blocks or becomes blocked by a creature this combat, that creature gains first strike until end of turn.","2R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Bey","4","3","R","If Viashino Bey attacks, all creatures you control attack if able.","2RR","Creature - Viashino");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Swamp Mosquito","0","1","B","Flying
Whenever Swamp Mosquito attacks and isn't blocked, defending player gets a poison counter. (A player with ten or more poison counters loses the game.)","1B","Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Relic of Progenitus","{T}: Target player exiles a card from his or her graveyard.
{1}, Exile Relic of Progenitus: Exile all cards from all graveyards. Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Acolyte of Xathrid","0","1","B","{1}{B}, {T}: Target player loses 1 life.","B","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Henchfiend of Ukor","3","2","R","Haste
Echo {1}{B} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice this permanent unless you pay its echo cost.)
{(b/r)}: Henchfiend of Ukor gets +1/+0 until end of turn.","3R","Creature - Ogre");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mercadia's Downfall","R","Each attacking creature gets +1/+0 until end of turn for each nonbasic land defending player controls.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deranged Assistant","1","1","U","{T}, Put the top card of your library into your graveyard: Add {1} to your mana pool.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("River Bear","3","3","G","Islandwalk (This creature is unblockable as long as defending player controls an Island.)","3G","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scythe Tiger","3","2","G","Shroud (This creature can't be the target of spells or abilities.)
When Scythe Tiger enters the battlefield, sacrifice it unless you sacrifice a land.","G","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recycle","G","Skip your draw step.
Whenever you play a card, draw a card.
Your maximum hand size is two.","4GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thrumming Stone","Spells you cast have ripple 4. (Whenever you cast a spell, you may reveal the top four cards of your library. You may cast any revealed cards with the same name as the spell without paying their mana costs. Put the rest on the bottom of your library.)","5","Legendary Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stomping Slabs","R","Reveal the top seven cards of your library, then put those cards on the bottom of your library in any order. If a card named Stomping Slabs was revealed this way, Stomping Slabs deals 7 damage to target creature or player.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gleam of Resistance","W","Creatures you control get +1/+2 until end of turn. Untap those creatures.
Basic landcycling {1}{W} ({1}{W}, Discard this card: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.)","4W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Musher","2","3","B","Snow landwalk
{S}i}: Regenerate Zombie Musher. ({S}i} can be paid with one mana from a snow permanent.)","3B","Snow Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fervent Charge","WBR","Whenever a creature you control attacks, it gets +2/+2 until end of turn.","1WBR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Necrogen Censer","Necrogen Censer enters the battlefield with two charge counters on it.
{T}, Remove a charge counter from Necrogen Censer: Target player loses 2 life.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Boost","G","Target creature gets +4/+4 until end of turn.
Cycling {2}{G} ({2}{G}, Discard this card: Draw a card.)
When you cycle Primal Boost, you may have target creature get +1/+1 until end of turn.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infected Vermin","1","1","B","{2}{B}: Infected Vermin deals 1 damage to each creature and each player.
Threshold - {3}{B}: Infected Vermin deals 3 damage to each creature and each player. Activate this ability only if seven or more cards are in your graveyard.","2B","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overblaze","R","Each time target permanent would deal damage to a creature or player this turn, it deals double that damage to that creature or player instead.
Splice onto Arcane {2}{R}{R} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","3R","Instant - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pointy Finger of Doom","{3}, {T}: Spin Pointy Finger of Doom in the middle of the table so that it rotates completely at least once, then destroy the closest permanent the finger points to.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grindstone","{3}, {T}: Put the top two cards of target player's library into that player's graveyard. If both cards share a color, repeat this process.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Marauder","2","1","R","Morph {2}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Skirk Marauder is turned face up, it deals 2 damage to target creature or player.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karona, False God","5","5","WUBRG","Haste
At the beginning of each player's upkeep, that player untaps Karona, False God and gains control of it.
Whenever Karona attacks, creatures of the creature type of your choice get +3/+3 until end of turn.","1WUBRG","Legendary Creature - Avatar");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Myr Incubator","{6}, {T}, Sacrifice Myr Incubator: Search your library for any number of artifact cards, exile them, then put that many 1/1 colorless Myr artifact creature tokens onto the battlefield. Then shuffle your library.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Extraction","B","As an additional cost to cast Mind Extraction, sacrifice a creature.
Target player reveals his or her hand and discards all cards of each of the sacrificed creature's colors.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Neurok Replica","1","4","{1}{U}, Sacrifice Neurok Replica: Return target creature to its owner's hand.","3","Artifact Creature - Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hivestone","Creatures you control are Slivers in addition to their other creature types.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oubliette","B","When Oubliette enters the battlefield, exile target creature and all Auras attached to it. Note the number and kind of counters that were on that creature.
When Oubliette leaves the battlefield, return the exiled card to the battlefield under its owner's control tapped with the noted number and kind of counters on it. If you do, return the exiled Aura cards to the battlefield under their owner's control attached to that permanent.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frenetic Efreet","2","1","UR","Flying
{0}: Flip a coin. If you win the flip, Frenetic Efreet phases out. If you lose the flip, sacrifice Frenetic Efreet. (While it's phased out, it's treated as though it doesn't exist. It phases in before you untap during your next untap step.)","1UR","Creature - Efreet");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sun Titan","6","6","W","Vigilance
Whenever Sun Titan enters the battlefield or attacks, you may return target permanent card with converted mana cost 3 or less from your graveyard to the battlefield.","4WW","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lure","G","Enchant creature
All creatures able to block enchanted creature do so.","1GG","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chain Lightning","R","Chain Lightning deals 3 damage to target creature or player. Then that player or that creature's controller may pay {R}{R}. If the player does, he or she may copy this spell and may choose a new target for that copy.","R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashes of the Fallen","As Ashes of the Fallen enters the battlefield, choose a creature type.
Each creature card in your graveyard has the chosen creature type in addition to its other types.","2","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Tek","2","2","Tek gets +0/+2 as long as you control a Plains, has flying as long as you control an Island, gets +2/+0 as long as you control a Swamp, has first strike as long as you control a Mountain, and has trample as long as you control a Forest.","5","Artifact Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bogardan Rager","3","4","R","Flash (You may cast this spell any time you could cast an instant.)
When Bogardan Rager enters the battlefield, target creature gets +4/+0 until end of turn.","5R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boros Guildmage","2","2","WR","{1}{R}: Target creature gains haste until end of turn.
{1}{W}: Target creature gains first strike until end of turn.","(R/W)(R/W)","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azami, Lady of Scrolls","0","2","U","Tap an untapped Wizard you control: Draw a card.","2UUU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Undergrowth","G","As an additional cost to cast Undergrowth, you may pay {2}{R}.
Prevent all combat damage that would be dealt this turn. If its additional cost was paid, Undergrowth doesn't affect combat damage that would be dealt by red creatures.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Isleback Spawn","4","8","U","Shroud
Isleback Spawn gets +4/+8 as long as a library has twenty or fewer cards in it.","5UU","Creature - Kraken");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timmy, Power Gamer","1","1","G","o4: Put a creature into play from your hand.","2GG","Summon - Legend");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soldevi Machinist","1","1","U","{T}: Add {2} to your mana pool. Spend this mana only to activate abilities of artifacts.","1U","Creature - Human Wizard Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Day of Destiny","W","Legendary creatures you control get +2/+2.","3W","Legendary Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mourner's Shield","Imprint - When Mourner's Shield enters the battlefield, you may exile target card from a graveyard.
{2}, {T}: Prevent all damage that would be dealt this turn by a source of your choice that shares a color with the exiled card.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Behemoth's Herald","1","1","G","{2}{G}, {T}, Sacrifice a red creature, a green creature, and a white creature: Search your library for a card named Godsire and put it onto the battlefield. Then shuffle your library.","G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Tombstones","0","1","B","Defender (This creature can't attack.)
At the beginning of your upkeep, Wall of Tombstones's toughness becomes 1 plus the number of creature cards in your graveyard. (This effect lasts indefinitely.)","1B","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clutch of Undeath","B","Enchant creature
Enchanted creature gets +3/+3 as long as it's a Zombie. Otherwise, it gets -3/-3.","3BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodrite Invoker","3","1","B","{8}: Target player loses 3 life and you gain 3 life.","2B","Creature - Vampire Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Buried Ruin","{T}: Add {1} to your mana pool.
{2}, {T}, Sacrifice Buried Ruin: Return target artifact card from your graveyard to your hand.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Brigand","2","2","R","Goblin Brigand attacks each turn if able.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blind-Spot Giant","4","3","R","Blind-Spot Giant can't attack or block unless you control another Giant.","2R","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Austere Command","W","Choose two - Destroy all artifacts; or destroy all enchantments; or destroy all creatures with converted mana cost 3 or less; or destroy all creatures with converted mana cost 4 or greater.","4WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Scout","1","1","U","Flying
{2}{U}, Sacrifice a land: Draw a card.","1U","Creature - Cephalid Wizard Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pygmy Pyrosaur","1","1","R","Pygmy Pyrosaur can't block.
{R}: Pygmy Pyrosaur gets +1/+0 until end of turn.","1R","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Retromancer","3","3","R","Whenever Retromancer becomes the target of a spell or ability, Retromancer deals 3 damage to that spell or ability's controller.","2RR","Creature - Viashino Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tendo Ice Bridge","Tendo Ice Bridge enters the battlefield with a charge counter on it.
{T}: Add {1} to your mana pool.
{T}, Remove a charge counter from Tendo Ice Bridge: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyrohemia","R","At the beginning of the end step, if no creatures are on the battlefield, sacrifice Pyrohemia.
{R}: Pyrohemia deals 1 damage to each creature and each player.","2RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of the Light","1","2","W","{W}, {T}: Choose target opponent who had more life than you did as you activated this ability. You gain 3 life.","WW","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Griptide","U","Put target creature on top of its owner's library.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Soldier","0","0","G","Spike Soldier enters the battlefield with three +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Soldier: Put a +1/+1 counter on target creature.
Remove a +1/+1 counter from Spike Soldier: Spike Soldier gets +2/+2 until end of turn.","2GG","Creature - Spike Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Genesis","4","4","G","At the beginning of your upkeep, if Genesis is in your graveyard, you may pay {2}{G}. If you do, return target creature card from your graveyard to your hand.","4G","Creature - Incarnation");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pennon Blade","Equipped creature gets +1/+1 for each creature you control.
Equip {4}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Feral Animist","2","1","RG","{3}: Feral Animist gets +X/+0 until end of turn, where X is its power.","1RG","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Entropic Eidolon","2","2","B","{B}, Sacrifice Entropic Eidolon: Target player loses 1 life and you gain 1 life.
Whenever you cast a multicolored spell, you may return Entropic Eidolon from your graveyard to your hand.","3B","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyrrhic Revival","WB","Each player returns each creature card from his or her graveyard to the battlefield with an additional -1/-1 counter on it.","3(W/B)(W/B)(W/B)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sedraxis Specter","3","2","UBR","Flying
Whenever Sedraxis Specter deals combat damage to a player, that player discards a card.
Unearth {1}{B} ({1}{B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","UBR","Creature - Specter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit Mirror","W","At the beginning of your upkeep, if there are no Reflection tokens on the battlefield, put a 2/2 white Reflection creature token onto the battlefield.
{0}: Destroy target Reflection.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Charge","R","Whenever a Beast enters the battlefield under your control, you may have it deal 4 damage to target opponent.","4R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nemata, Grove Guardian","4","5","G","{2}{G}: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: Saproling creatures get +1/+1 until end of turn.","4GG","Legendary Creature - Treefolk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bottle of Suleiman","{1}, Sacrifice Bottle of Suleiman: Flip a coin. If you lose the flip, Bottle of Suleiman deals 5 damage to you. If you win the flip, put a 5/5 colorless Djinn artifact creature token with flying onto the battlefield.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sengir Vampire","4","4","B","Flying
Whenever a creature dealt damage by Sengir Vampire this turn dies, put a +1/+1 counter on Sengir Vampire.","3BB","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Malfegor","6","6","BR","Flying
When Malfegor enters the battlefield, discard your hand. Each opponent sacrifices a creature for each card discarded this way.","2BBRR","Legendary Creature - Demon Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Terrifying Presence","G","Prevent all combat damage that would be dealt by creatures other than target creature this turn.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Subterranean Hangar","Subterranean Hangar enters the battlefield tapped.
{T}: Put a storage counter on Subterranean Hangar.
{T}, Remove any number of storage counters from Subterranean Hangar: Add {B} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zur the Enchanter","1","4","WUB","Flying
Whenever Zur the Enchanter attacks, you may search your library for an enchantment card with converted mana cost 3 or less and put it onto the battlefield. If you do, shuffle your library.","1WUB","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Homarid","2","2","U","Homarid enters the battlefield with a tide counter on it.
At the beginning of your upkeep, put a tide counter on Homarid.
As long as there is exactly one tide counter on Homarid, it gets -1/-1.
As long as there are exactly three tide counters on Homarid, it gets +1/+1.
Whenever there are four tide counters on Homarid, remove all tide counters from it.","2U","Creature - Homarid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Melting","R","All lands are no longer snow.","3R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaze of Adamaro","R","Gaze of Adamaro deals damage to target player equal to the number of cards in that player's hand.","2RR","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wei Assassins","3","2","B","When Wei Assassins enters the battlefield, target opponent chooses a creature he or she controls. Destroy it.","3BB","Creature - Human Soldier Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Vapors","U","Lands target player controls don't untap during his or her next untap step.","1U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skullmead Cauldron","{T}: You gain 1 life.
{T}, Discard a card: You gain 3 life.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fatal Mutation","B","Enchant creature
When enchanted creature is turned face up, destroy it. It can't be regenerated.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Headless Horseman","2","2","B",NULL,"2B","Creature - Zombie Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Open the Vaults","W","Return all artifact and enchantment cards from all graveyards to the battlefield under their owners' control. (Auras with nothing to enchant remain in graveyards.)","4WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lowland Basilisk","1","3","G","Whenever Lowland Basilisk deals damage to a creature, destroy that creature at end of combat.","2G","Creature - Basilisk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necravolver","2","2","B","Kicker {1}{G} and/or {W} (You may pay an additional {1}{G} and/or {W} as you cast this spell.)
If Necravolver was kicked with its {1}{G} kicker, it enters the battlefield with two +1/+1 counters on it and with trample.
If Necravolver was kicked with its {W} kicker, it enters the battlefield with a +1/+1 counter on it and with ""Whenever Necravolver deals damage, you gain that much life.""","2B","Creature - Volver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reality Spasm","U","Choose one - Tap X target permanents; or untap X target permanents.","XUU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Weed-Pruner Poplar","3","3","B","At the beginning of your upkeep, target creature other than Weed-Pruner Poplar gets -1/-1 until end of turn.","4B","Creature - Treefolk Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diving Griffin","2","2","W","Flying, vigilance","1WW","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Erratic Mutation","U","Choose target creature. Reveal cards from the top of your library until you reveal a nonland card. That creature gets +X/-X until end of turn, where X is that card's converted mana cost. Put all cards revealed this way on the bottom of your library in any order.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Keep Watch","U","Draw a card for each attacking creature.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistform Wall","1","4","U","Mistform Wall has defender as long as it's a Wall.
{1}: Mistform Wall becomes the creature type of your choice until end of turn.","2U","Creature - Illusion Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gwyllion Hedge-Mage","2","2","WB","When Gwyllion Hedge-Mage enters the battlefield, if you control two or more Plains, you may put a 1/1 white Kithkin Soldier creature token onto the battlefield.
When Gwyllion Hedge-Mage enters the battlefield, if you control two or more Swamps, you may put a -1/-1 counter on target creature.","2(W/B)","Creature - Hag Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Armor","If a source would deal damage to you, prevent 1 of that damage.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Island Fish Jasconius","6","8","U","Island Fish Jasconius doesn't untap during your untap step.
At the beginning of your upkeep, you may pay {U}{U}{U}. If you do, untap Island Fish Jasconius.
Island Fish Jasconius can't attack unless defending player controls an Island.
When you control no Islands, sacrifice Island Fish Jasconius.","4UUU","Creature - Fish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leaf Arrow","G","Leaf Arrow deals 3 damage to target creature with flying.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iname, Death Aspect","4","4","B","When Iname, Death Aspect enters the battlefield, you may search your library for any number of Spirit cards and put them into your graveyard. If you do, shuffle your library.","4BB","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Number Crunch","U","Return target permanent to its owner's hand.
Gotcha Whenever an opponent says a number, you may say ""Gotcha"" If you do, return Number Crunch from your graveyard to your hand.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit Loop","W","Enchant creature you control
Whenever enchanted creature deals damage, you gain that much life.
When Spirit Loop is put into a graveyard from the battlefield, return Spirit Loop to its owner's hand.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wash Out","U","Return all permanents of the color of your choice to their owners' hands.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vicious Kavu","2","2","BR","Whenever Vicious Kavu attacks, it gets +2/+0 until end of turn.","1BR","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sisters of Stone Death Avatar","Each creature you control must be blocked if able.
{4}: Exile target creature that's blocking a creature you control.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greel, Mind Raker","3","3","B","{X}{B}, {T}, Discard two cards: Target player discards X cards at random.","3BB","Legendary Creature - Horror Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelic Voices","W","Creatures you control get +1/+1 as long as you control no nonartifact, nonwhite creatures.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Draining Whelk","1","1","U","Flash (You may cast this spell any time you could cast an instant.)
Flying
When Draining Whelk enters the battlefield, counter target spell. Put X +1/+1 counters on Draining Whelk, where X is that spell's converted mana cost.","4UU","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Teferi's Realm","U","At the beginning of each player's upkeep, that player chooses artifact, creature, land, or non-Aura enchantment. All nontoken permanents of that type phase out. (While they're phased out, they're treated as though they don't exist. Each one phases in before its controller untaps during his or her next untap step.)","1UU","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Borderland Ranger","2","2","G","When Borderland Ranger enters the battlefield, you may search your library for a basic land card, reveal it, and put it into your hand. If you do, shuffle your library.","2G","Creature - Human Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Scuta","3","3","B","Kicker-Pay 3 life. (You may pay 3 life in addition to any other costs as you cast this spell.)
If Phyrexian Scuta was kicked, it enters the battlefield with two +1/+1 counters on it.","3B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Nim Replica","3","1","{2}{B}, Sacrifice Nim Replica: Target creature gets -1/-1 until end of turn.","3","Artifact Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Obsidian Acolyte","1","1","W","Protection from black
{W}: Target creature gains protection from black until end of turn.","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abyssal Nightstalker","2","2","B","Whenever Abyssal Nightstalker attacks and isn't blocked, defending player discards a card.","3B","Creature - Nightstalker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ana Disciple","1","1","G","{U}, {T}: Target creature gains flying until end of turn.
{B}, {T}: Target creature gets -2/-0 until end of turn.","G","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parallax Dementia","B","Enchant creature
Fading 1 (This enchantment enters the battlefield with one fade counter on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Enchanted creature gets +3/+2.
When Parallax Dementia leaves the battlefield, destroy enchanted creature. That creature can't be regenerated.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benevolent Bodyguard","1","1","W","Sacrifice Benevolent Bodyguard: Target creature you control gains protection from the color of your choice until end of turn.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wheel and Deal","U","Any number of target opponents each discards his or her hand and draws seven cards.
Draw a card.","3U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Righteous Indignation","W","Whenever a creature blocks a black or red creature, the blocking creature gets +1/+1 until end of turn.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cemetery Reaper","2","2","B","Other Zombie creatures you control get +1/+1.
{2}{B}, {T}: Exile target creature card from a graveyard. Put a 2/2 black Zombie creature token onto the battlefield.","1BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drelnoch","3","3","U","Whenever Drelnoch becomes blocked, you may draw two cards.","4U","Creature - Yeti Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Putrid Raptor","4","4","B","Morph-Discard a Zombie card. (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4BB","Creature - Zombie Lizard Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duergar Hedge-Mage","2","2","WR","When Duergar Hedge-Mage enters the battlefield, if you control two or more Mountains, you may destroy target artifact.
When Duergar Hedge-Mage enters the battlefield, if you control two or more Plains, you may destroy target enchantment.","2(R/W)","Creature - Dwarf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Return of the Nightstalkers","B","Return all Nightstalker permanent cards from your graveyard to the battlefield. Then destroy all Swamps you control.","5BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Griffin Canyon","{T}: Add {1} to your mana pool.
{T}: Untap target Griffin. If it's a creature, it gets +1/+1 until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Soldevi Steam Beast","4","2","Whenever Soldevi Steam Beast becomes tapped, target opponent gains 2 life.
{2}: Regenerate Soldevi Steam Beast.","5","Artifact Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reset","U","Cast Reset only during an opponent's turn after his or her upkeep step.
Untap all lands you control.","UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bargain","W","Target opponent draws a card.
You gain 7 life.","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pure Intentions","W","Whenever a spell or ability an opponent controls causes you to discard cards this turn, return those cards from your graveyard to your hand.
When a spell or ability an opponent controls causes you to discard Pure Intentions, return Pure Intentions from your graveyard to your hand at the beginning of the next end step.","W","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Murder of Crows","4","4","U","Flying
Whenever another creature dies, you may draw a card. If you do, discard a card.","3UU","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Willow Elf","1","1","G",NULL,"G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burning-Eye Zubera","3","3","R","When Burning-Eye Zubera dies, if 4 or more damage was dealt to it this turn, Burning-Eye Zubera deals 3 damage to target creature or player.","2RR","Creature - Zubera Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serra's Boon","W","Enchant creature
Enchanted creature gets +1/+2 as long as it's white. Otherwise, it gets -2/-1.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flame Jet","R","Flame Jet deals 3 damage to target player.
Cycling {2} ({2}, Discard this card: Draw a card.)","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cooperation","W","Enchant creature
Enchanted creature has banding. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding a player controls are blocking or being blocked by a creature, that player divides that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moonlit Strider","1","4","W","Sacrifice Moonlit Strider: Target creature you control gains protection from the color of your choice until end of turn.
Soulshift 3 (When this creature dies, you may return target Spirit card with converted mana cost 3 or less from your graveyard to your hand.)","3W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Possessed Barbarian","3","3","R","First strike
Threshold - As long as seven or more cards are in your graveyard, Possessed Barbarian gets +1/+1, is black, and has ""{2}{B}, {T}: Destroy target red creature.""","2RR","Creature - Human Barbarian Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Acquire","U","Search target opponent's library for an artifact card and put that card onto the battlefield under your control. Then that player shuffles his or her library.","3UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Poison the Well","BR","Destroy target land. Poison the Well deals 2 damage to that land's controller.","2(B/R)(B/R)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Digging Team","1","1","R","{T}, Sacrifice Goblin Digging Team: Destroy target Wall.","R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stream of Acid","B","Destroy target land or nonblack creature.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warpath","R","Warpath deals 3 damage to each blocking creature and each blocked creature.","3R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rimewind Taskmage","1","2","U","{1}, {T}: You may tap or untap target permanent. Activate this ability only if you control four or more snow permanents.","1U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earnest Fellowship","W","Each creature has protection from its colors.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crippling Fatigue","B","Target creature gets -2/-2 until end of turn.
Flashback-{1}{B}, Pay 3 life. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deranged Hermit","1","1","G","Echo {3}{G}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Deranged Hermit enters the battlefield, put four 1/1 green Squirrel creature tokens onto the battlefield.
Squirrel creatures get +1/+1.","3GG","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonic Tutor","B","Search your library for a card and put that card into your hand. Then shuffle your library.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Diffusion","0","5","R","Defender (This creature can't attack.)
Wall of Diffusion can block creatures with shadow as though Wall of Diffusion had shadow.","1R","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Relentless Skaabs","4","4","U","As an additional cost to cast Relentless Skaabs, exile a creature card from your graveyard.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","3UU","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Claustrophobia","U","Enchant creature
When Claustrophobia enters the battlefield, tap enchanted creature.
Enchanted creature doesn't untap during its controller's untap step.","1UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Briarberry Cohort","1","1","U","Flying
Briarberry Cohort gets +1/+1 as long as you control another blue creature.","1U","Creature - Faerie Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Koskun Keep","{T}: Add {1} to your mana pool.
{1}, {T}: Add {R} to your mana pool.
{2}, {T}: Add {B} or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Akroma, Angel of Wrath Avatar","Whenever a creature enters the battlefield under your control, it gains two abilities chosen at random from flying, first strike, trample, haste, protection from black, protection from red, and vigilance.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Castle Sengir","{T}: Add {1} to your mana pool.
{1}, {T}: Add {B} to your mana pool.
{2}, {T}: Add {U} or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Tracer","2","1","G","When Nantuko Tracer enters the battlefield, you may put target card from a graveyard on the bottom of its owner's library.","1G","Creature - Insect Druid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Pestilent Souleater","3","3","{(b/p)}: Pestilent Souleater gains infect until end of turn. ({(b/p)} can be paid with either {B} or 2 life. A creature with infect deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","5","Artifact Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enchanted Evening","WU","All permanents are enchantments in addition to their other types.","3(W/U)(W/U)","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Genju of the Fields","W","Enchant Plains
{2}: Until end of turn, enchanted Plains becomes a 2/5 white Spirit creature with ""Whenever this creature deals damage, its controller gains that much life."" It's still a land.
When enchanted Plains is put into a graveyard, you may return Genju of the Fields from your graveyard to your hand.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind's Desire","U","Shuffle your library. Then exile the top card of your library. Until end of turn, you may play that card without paying its mana cost. (If it has X in its mana cost, X is 0.)
Storm (When you cast this spell, copy it for each spell cast before it this turn.)","4UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Molting Skin","G","Return Molting Skin to its owner's hand: Regenerate target creature.","2G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cowed by Wisdom","W","Enchant creature
Enchanted creature can't attack or block unless its controller pays {1} for each card in your hand.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Waking Nightmare","B","Target player discards two cards.","2B","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Regress","U","Return target permanent to its owner's hand.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thraben Valiant","2","1","W","Vigilance","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Painter's Servant","1","3","As Painter's Servant enters the battlefield, choose a color.
All cards that aren't on the battlefield, spells, and permanents are the chosen color in addition to their other colors.","2","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Extinguish","U","Counter target sorcery spell.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disturbing Plot","B","Return target creature card from a graveyard to its owner's hand.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose a new target for the copy.)","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Confiscate","U","Enchant permanent (Target a permanent as you cast this. This card enters the battlefield attached to that permanent.)
You control enchanted permanent.","4UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Daze","U","You may return an Island you control to its owner's hand rather than pay Daze's mana cost.
Counter target spell unless its controller pays {1}.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Razorverge Thicket","Razorverge Thicket enters the battlefield tapped unless you control two or fewer other lands.
{T}: Add {G} or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Banishment Decree","W","Put target artifact, creature, or enchantment on top of its owner's library.","3WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Skyfisher","2","3","W","Flying
When Kor Skyfisher enters the battlefield, return a permanent you control to its owner's hand.","1W","Creature - Kor Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rootgrapple","G","Destroy target noncreature permanent. If you control a Treefolk, draw a card.","4G","Tribal Instant - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warp Artifact","B","Enchant artifact
At the beginning of the upkeep of enchanted artifact's controller, Warp Artifact deals 1 damage to that player.","BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skullsnatcher","2","1","B","Ninjutsu {B} ({B}, Return an unblocked attacker you control to hand: Put this card onto the battlefield from your hand tapped and attacking.)
Whenever Skullsnatcher deals combat damage to a player, exile up to two target cards from that player's graveyard.","1B","Creature - Rat Ninja");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Indestructible Aura","W","Prevent all damage that would be dealt to target creature this turn.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Renounce","W","Sacrifice any number of permanents. You gain 2 life for each permanent sacrificed this way.","1W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Artisan of Kozilek","10","9","When you cast Artisan of Kozilek, you may return target creature card from your graveyard to the battlefield.
Annihilator 2 (Whenever this creature attacks, defending player sacrifices two permanents.)","9","Creature - Eldrazi");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nomads' Assembly","W","Put a 1/1 white Kor Soldier creature token onto the battlefield for each creature you control.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","4WW","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Thornwatch Scarecrow","4","4","Thornwatch Scarecrow has wither as long as you control a green creature. (It deals damage to creatures in the form of -1/-1 counters.)
Thornwatch Scarecrow has vigilance as long as you control a white creature.","6","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Revoke Existence","W","Exile target artifact or enchantment.","1W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fog Patch","G","Cast Fog Patch only during the declare blockers step.
Attacking creatures become blocked. (This spell works on unblockable creatures.)","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Overgrown Battlement","0","4","G","Defender
{T}: Add {G} to your mana pool for each creature with defender you control.","1G","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spectral Guardian","2","3","W","As long as Spectral Guardian is untapped, noncreature artifacts have shroud. (They can't be the targets of spells or abilities.)","2WW","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treacherous Vampire","4","4","B","Flying
Whenever Treacherous Vampire attacks or blocks, sacrifice it unless you exile a card from your graveyard.
Threshold - As long as seven or more cards are in your graveyard, Treacherous Vampire gets +2/+2 and has ""When Treacherous Vampire dies, you lose 6 life.""","4B","Creature - Vampire");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Junk Diver","1","1","Flying
When Junk Diver dies, return another target artifact card from your graveyard to your hand.","3","Artifact Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rift Bolt","R","Rift Bolt deals 3 damage to target creature or player.
Suspend 1-{R} (Rather than cast this card from your hand, you may pay {R} and exile it with a time counter on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lay Bare","U","Counter target spell. Look at its controller's hand.","2UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rebuild","U","Return all artifacts to their owners' hands.
Cycling {2} ({2}, Discard this card: Draw a card.)","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kuldotha Rebirth","R","As an additional cost to cast Kuldotha Rebirth, sacrifice an artifact.
Put three 1/1 red Goblin creature tokens onto the battlefield.","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rites of Initiation","R","Discard any number of cards at random. Creatures you control get +1/+0 until end of turn for each card discarded this way.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zealot il-Vec","1","1","W","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever Zealot il-Vec attacks and isn't blocked, you may have it deal 1 damage to target creature. If you do, prevent all combat damage Zealot il-Vec would deal this turn.","2W","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sprouting Phytohydra","0","2","G","Defender (This creature can't attack.)
Whenever Sprouting Phytohydra is dealt damage, you may put a token that's a copy of Sprouting Phytohydra onto the battlefield.","4G","Creature - Plant Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Customs Depot","U","Whenever you cast a creature spell, you may pay {1}. If you do, draw a card, then discard a card.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Headlong Rush","R","Attacking creatures gain first strike until end of turn.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Countersquall","UB","Counter target noncreature spell. Its controller loses 2 life.","UB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Menacing Ogre","3","3","R","Trample, haste
When Menacing Ogre enters the battlefield, each player secretly chooses a number. Then those numbers are revealed. Each player with the highest number loses that much life. If you are one of those players, put two +1/+1 counters on Menacing Ogre.","3RR","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Prodigal Sorcerer","1","1","U","{T}: Prodigal Sorcerer deals 1 damage to target creature or player.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lead the Stampede","G","Look at the top five cards of your library. You may reveal any number of creature cards from among them and put the revealed cards into your hand. Put the rest on the bottom of your library in any order.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nantuko Shrine","G","Whenever a player casts a spell, that player puts X 1/1 green Squirrel creature tokens onto the battlefield, where X is the number of cards in all graveyards with the same name as that spell.","1GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eternal Dominion","U","Search target opponent's library for an artifact, creature, enchantment, or land card. Put that card onto the battlefield under your control. Then that player shuffles his or her library.
Epic (For the rest of the game, you can't cast spells. At the beginning of each of your upkeeps, copy this spell except for its epic ability. You may choose a new target for the copy.)","7UUU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Might of the Masses","G","Target creature gets +1/+1 until end of turn for each creature you control.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blue Elemental Blast","U","Choose one - Counter target red spell; or destroy target red permanent.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wu Infantry","2","1","U",NULL,"1U","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Bushwhacker","1","1","R","Kicker {R} (You may pay an additional {R} as you cast this spell.)
When Goblin Bushwhacker enters the battlefield, if it was kicked, creatures you control get +1/+0 and gain haste until end of turn.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Despotic Scepter","{T}: Destroy target permanent you own. It can't be regenerated.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Ghoul","2","2","B","Sacrifice a creature: Phyrexian Ghoul gets +2/+2 until end of turn.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Taskmaster","1","1","R","{1}{R}: Target Goblin creature gets +1/+0 until end of turn.
Morph {R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Origin Spellbomb","{1}, {T}, Sacrifice Origin Spellbomb: Put a 1/1 colorless Myr artifact creature token onto the battlefield.
When Origin Spellbomb is put into a graveyard from the battlefield, you may pay {W}. If you do, draw a card.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corrupted Resolve","U","Counter target spell if its controller is poisoned.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightwielder Paladin","4","4","W","First strike (This creature deals combat damage before creatures without first strike.)
Whenever Lightwielder Paladin deals combat damage to a player, you may exile target black or red permanent that player controls.","3WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lab Rats","B","Buyback {4} (You may pay an additional {4} as you cast this spell. If you do, put this card into your hand as it resolves.)
Put a 1/1 black Rat creature token onto the battlefield.","B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Acidic Dagger","{4}, {T}: Whenever target creature deals combat damage to a non-Wall creature this turn, destroy that non-Wall creature. When the targeted creature leaves the battlefield this turn, sacrifice Acidic Dagger. Activate this ability only before blockers are declared.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Firestorm","R","As an additional cost to cast Firestorm, discard X cards.
Firestorm deals X damage to each of X target creatures and/or players.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arcanis the Omnipotent","3","4","U","{T}: Draw three cards.
{2}{U}{U}: Return Arcanis the Omnipotent to its owner's hand.","3UUU","Legendary Creature - Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minotaur Illusionist","3","4","UR","{1}{U}: Minotaur Illusionist gains shroud until end of turn. (It can't be the target of spells or abilities.)
{R}, Sacrifice Minotaur Illusionist: Minotaur Illusionist deals damage equal to its power to target creature.","3UR","Creature - Minotaur Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Puffer Extract","{X}, {T}: Target creature you control gets +X/+X until end of turn. Destroy it at the beginning of the next end step.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Dogs","2","1","G","At the beginning of your upkeep, if a player has more life than each other player, the player with the most life gains control of Wild Dogs.
Cycling {2} ({2}, Discard this card: Draw a card.)","G","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Runner","3","2","R","Viashino Runner can't be blocked except by two or more creatures.","3R","Creature - Viashino");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Banishing Stroke","W","Put target artifact, creature, or enchantment on the bottom of its owner's library.
Miracle {W} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","5W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thrashing Mudspawn","4","4","B","Whenever Thrashing Mudspawn is dealt damage, you lose that much life.
Morph {1}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3BB","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rod of Ruin","{3}, {T}: Rod of Ruin deals 1 damage to target creature or player.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squall Line","G","Squall Line deals X damage to each creature with flying and each player.","XGG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Erratic Explosion","R","Choose target creature or player. Reveal cards from the top of your library until you reveal a nonland card. Erratic Explosion deals damage equal to that card's converted mana cost to that creature or player. Put the revealed cards on the bottom of your library in any order.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ow","B","Whenever any creature damages a player, for each Ow card in play, that player says ""Ow"" once or Ow deals 1 damage to him or her.","B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elgaud Inquisitor","2","2","W","Lifelink (Damage dealt by this creature also causes you to gain that much life.)
When Elgaud Inquisitor dies, put a 1/1 white Spirit creature token with flying onto the battlefield.","3W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Underground Sea",NULL,NULL,"Land - Island Swamp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Desolation Giant","3","3","R","Kicker {W}{W} (You may pay an additional {W}{W} as you cast this spell.)
When Desolation Giant enters the battlefield, destroy all other creatures you control. If it was kicked, destroy all other creatures instead.","2RR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ekundu Griffin","2","2","W","Flying, first strike","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Retaliation","G","Creatures you control have ""Whenever this creature becomes blocked by a creature, this creature gets +1/+1 until end of turn.""","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cerodon Yearling","2","2","WR","Vigilance, haste","RW","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blistergrub","2","2","B","Swampwalk
When Blistergrub dies, each opponent loses 2 life.","2B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lingering Death","B","Enchant creature
At the beginning of the end step of enchanted creature's controller, that player sacrifices that creature.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trickery Charm","U","Choose one - Target creature gains flying until end of turn; or target creature becomes the creature type of your choice until end of turn; or look at the top four cards of your library, then put them back in any order.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tel-Jilad Fallen","3","1","G","Protection from artifacts
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","2GG","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quick Sliver","1","1","G","Flash
Any player may play Sliver cards as though they had flash.","1G","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Juniper Order Advocate","1","2","W","As long as Juniper Order Advocate is untapped, green creatures you control get +1/+1.","2W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raise Dead","B","Return target creature card from your graveyard to your hand.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gulf Squid","2","2","U","When Gulf Squid enters the battlefield, tap all lands target player controls.","3U","Creature - Squid Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fire Diamond","Fire Diamond enters the battlefield tapped.
{T}: Add {R} to your mana pool.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cinder Cloud","R","Destroy target creature. If a white creature dies this way, Cinder Cloud deals damage to that creature's controller equal to the creature's power.","3RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spinal Embrace","UB","Cast Spinal Embrace only during combat.
Untap target creature you don't control and gain control of it. It gains haste until end of turn. At the beginning of the next end step, sacrifice it. If you do, you gain life equal to its toughness.","3UUB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dedicated Martyr","1","1","W","{W}, Sacrifice Dedicated Martyr: You gain 3 life.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stonehands","R","Enchant creature
Enchanted creature gets +0/+2.
{R}: Enchanted creature gets +1/+0 until end of turn.","2R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Geth's Verdict","B","Target player sacrifices a creature and loses 1 life.","BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Relic Bane","B","Enchant artifact
Enchanted artifact has ""At the beginning of your upkeep, you lose 2 life.""","1BB","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vivid Creek","Vivid Creek enters the battlefield tapped with two charge counters on it.
{T}: Add {U} to your mana pool.
{T}, Remove a charge counter from Vivid Creek: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bojuka Brigand","1","1","B","Bojuka Brigand can't block.
Whenever Bojuka Brigand or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Bojuka Brigand.","1B","Creature - Human Warrior Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Leotau","5","4","G","At the beginning of your upkeep, sacrifice Wild Leotau unless you pay {G}.","2GG","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uncle Istvan","1","3","B","Prevent all damage that would be dealt to Uncle Istvan by creatures.","1BBB","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Apocalypse Hydra","0","0","RG","Apocalypse Hydra enters the battlefield with X +1/+1 counters on it. If X is 5 or more, it enters the battlefield with an additional X +1/+1 counters on it.
{1}{R}, Remove a +1/+1 counter from Apocalypse Hydra: Apocalypse Hydra deals 1 damage to target creature or player.","XRG","Creature - Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Extract","U","Search target player's library for a card and exile it. Then that player shuffles his or her library.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Haazda Shield Mate","1","1","W","At the beginning of your upkeep, sacrifice Haazda Shield Mate unless you pay {W}{W}.
{W}: The next time a source of your choice would deal damage to you this turn, prevent that damage.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Preeminent Captain","2","2","W","First strike
Whenever Preeminent Captain attacks, you may put a Soldier creature card from your hand onto the battlefield tapped and attacking.","2W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abyssal Gatekeeper","1","1","B","When Abyssal Gatekeeper dies, each player sacrifices a creature.","1B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Annihilate","B","Destroy target nonblack creature. It can't be regenerated.
Draw a card.","3BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vulshok Heartstoker","2","2","R","When Vulshok Heartstoker enters the battlefield, target creature gets +2/+0 until end of turn.","2R","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skullscorch","R","Target player discards two cards at random unless that player has Skullscorch deal 4 damage to him or her.","RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talara's Battalion","4","3","G","Trample
Cast Talara's Battalion only if you've cast another green spell this turn.","1G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nirkana Revenant","4","4","B","Whenever you tap a Swamp for mana, add {B} to your mana pool (in addition to the mana the land produces).
{B}: Nirkana Revenant gets +1/+1 until end of turn.","4BB","Creature - Vampire Shade");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Elspeth, Knight-Errant","W","+1: Put a 1/1 white Soldier creature token onto the battlefield.
+1: Target creature gets +3/+3 and gains flying until end of turn.
-8: You get an emblem with ""Artifacts, creatures, enchantments, and lands you control are indestructible.""","Planeswalker - Elspeth","2WW");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloud Dragon","5","4","U","Flying
Cloud Dragon can block only creatures with flying.","5U","Creature - Illusion Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Disciple","2","2","G","{G}, {T}: Target creature gets +2/+2 until end of turn.","3G","Creature - Insect Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Venomspout Brackus","5","5","G","{1}{G}, {T}: Venomspout Brackus deals 5 damage to target attacking or blocking creature with flying.
Morph {3}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","6G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magister Sphinx","5","5","WUB","Flying
When Magister Sphinx enters the battlefield, target player's life total becomes 10.","4WUB","Artifact Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Phoenix","3","4","R","Flying
When Shivan Phoenix dies, return Shivan Phoenix to its owner's hand.","4RR","Creature - Phoenix");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Retribution of the Meek","W","Destroy all creatures with power 4 or greater. They can't be regenerated.","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Counterbalance","U","Whenever an opponent casts a spell, you may reveal the top card of your library. If you do, counter that spell if it has the same converted mana cost as the revealed card.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wolfir Silverheart","4","4","G","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Wolfir Silverheart is paired with another creature, each of those creatures gets +4/+4.","3GG","Creature - Wolf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darien, King of Kjeldor","3","3","W","Whenever you're dealt damage, you may put that many 1/1 white Soldier creature tokens onto the battlefield.","4WW","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Peacekeeper","4","4","W","At the beginning of your upkeep, the player with the lowest life total gains control of Loxodon Peacekeeper. If two or more players are tied for lowest life total, you choose one of them, and that player gains control of Loxodon Peacekeeper.","1W","Creature - Elephant Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thallid","1","1","G","At the beginning of your upkeep, put a spore counter on Thallid.
Remove three spore counters from Thallid: Put a 1/1 green Saproling creature token onto the battlefield.","G","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of Anticipation","U","If Leyline of Anticipation is in your opening hand, you may begin the game with it on the battlefield.
You may cast nonland cards as though they had flash. (You may cast them any time you could cast an instant.)","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathbringer Thoctar","3","3","BR","Whenever another creature dies, you may put a +1/+1 counter on Deathbringer Thoctar.
Remove a +1/+1 counter from Deathbringer Thoctar: Deathbringer Thoctar deals 1 damage to target creature or player.","4BR","Creature - Zombie Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ringskipper","1","1","U","Flying
When Ringskipper dies, clash with an opponent. If you win, return Ringskipper to its owner's hand. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1U","Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earthlink","BRG","At the beginning of your upkeep, sacrifice Earthlink unless you pay {2}.
Whenever a creature dies, that creature's controller sacrifices a land.","3BRG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invigorating Boon","G","Whenever a player cycles a card, you may put a +1/+1 counter on target creature.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diabolic Intent","B","As an additional cost to cast Diabolic Intent, sacrifice a creature.
Search your library for a card and put that card into your hand. Then shuffle your library.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Catapult Squad","2","1","W","Tap two untapped Soldiers you control: Catapult Squad deals 2 damage to target attacking or blocking creature.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Suq'Ata Assassin","1","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Whenever Suq'Ata Assassin attacks and isn't blocked, defending player gets a poison counter. (A player with ten or more poison counters loses the game.)","1BB","Creature - Human Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Descendant of Kiyomaro","2","3","W","As long as you have more cards in hand than each opponent, Descendant of Kiyomaro gets +1/+2 and has ""Whenever this creature deals combat damage, you gain 3 life.""","1WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snap","U","Return target creature to its owner's hand. Untap up to two lands.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coma Veil","U","Enchant artifact or creature
Enchanted permanent doesn't untap during its controller's untap step.","4U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Saving Grasp","U","Return target creature you own to your hand.
Flashback {W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kodama's Might","G","Target creature gets +2/+2 until end of turn.
Splice onto Arcane {G} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","G","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Foxfire","G","Untap target attacking creature. Prevent all combat damage that would be dealt to and dealt by that creature this turn.
Draw a card at the beginning of the next turn's upkeep.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Games","U","Buyback {2}{U} (You may pay an additional {2}{U} as you cast this spell. If you do, put this card into your hand as it resolves.)
Tap target artifact, creature, or land.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hollowhenge Spirit","2","2","W","Flash (You may cast this spell any time you could cast an instant.)
Flying
When Hollowhenge Spirit enters the battlefield, remove target attacking or blocking creature from combat.","3W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Savor the Moment","U","Take an extra turn after this one. Skip the untap step of that turn.","1UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Starlit Angel","3","4","W","Flying","3WW","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skeleton Shard","{3}, {T} or {B}, {T}: Return target artifact creature card from your graveyard to your hand.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Symbiotic Beast","4","4","G","When Symbiotic Beast dies, put four 1/1 green Insect creature tokens onto the battlefield.","4GG","Creature - Insect Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wilderness Hypnotist","1","3","U","{T}: Target red or green creature gets -2/-0 until end of turn.","2UU","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Basandra, Battle Seraph","4","4","WR","Flying
Players can't cast spells during combat.
{R}: Target creature attacks this turn if able.","3RW","Legendary Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Copperhoof Vorrac","2","2","G","Copperhoof Vorrac gets +1/+1 for each untapped permanent your opponents control.","3GG","Creature - Boar Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spiteful Visions","BR","At the beginning of each player's draw step, that player draws an additional card.
Whenever a player draws a card, Spiteful Visions deals 1 damage to that player.","2(B/R)(B/R)","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gnat Alley Creeper","3","1","R","Gnat Alley Creeper can't be blocked by creatures with flying.","2R","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aladdin","1","1","R","{1}{R}{R}, {T}: Gain control of target artifact for as long as you control Aladdin.","2RR","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Possessed Nomad","3","3","W","Vigilance
Threshold - As long as seven or more cards are in your graveyard, Possessed Nomad gets +1/+1, is black, and has ""{2}{B}, {T}: Destroy target white creature.""","2WW","Creature - Human Nomad Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stand Together","G","Put two +1/+1 counters on target creature and two +1/+1 counters on another target creature.","3GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Advocate","2","2","W","Flying
{T}: Target attacking or blocking creature gets +2/+2 until end of turn.","3W","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deadeye Navigator","5","5","U","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Deadeye Navigator is paired with another creature, each of those creatures has ""{1}{U}: Exile this creature, then return it to the battlefield under your control.""","4UU","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aven Shrine","W","Whenever a player casts a spell, that player gains X life, where X is the number of cards in all graveyards with the same name as that spell.","1WW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Relic","Darksteel Relic is indestructible. (Effects that say ""destroy"" don't destroy it.)","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Femeref Healer","1","1","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sun Ce, Young Conquerer","3","3","U","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
When Sun Ce, Young Conquerer enters the battlefield, you may return target creature to its owner's hand.","3UU","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call to Serve","W","Enchant nonblack creature
Enchanted creature gets +1/+2, has flying, and is an Angel in addition to its other types.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Thopter Assembly","5","5","Flying
At the beginning of your upkeep, if you control no Thopters other than Thopter Assembly, return Thopter Assembly to its owner's hand and put five 1/1 colorless Thopter artifact creature tokens with flying onto the battlefield.","6","Artifact Creature - Thopter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Court Homunculus","1","1","W","Court Homunculus gets +1/+1 as long as you control another artifact.","W","Artifact Creature - Homunculus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Basking Rootwalla","1","1","G","{1}{G}: Basking Rootwalla gets +2/+2 until end of turn. Activate this ability only once each turn.
Madness {0} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","G","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kaervek's Spite","B","As an additional cost to cast Kaervek's Spite, sacrifice all permanents you control and discard your hand.
Target player loses 5 life.","BBB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Afflict","B","Target creature gets -1/-1 until end of turn.
Draw a card.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Collective Unconscious","G","Draw a card for each creature you control.","4GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elder Mastery","UBR","Enchant creature
Enchanted creature gets +3/+3 and has flying.
Whenever enchanted creature deals damage to a player, that player discards two cards.","3UBR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Obstinate Baloth","4","4","G","When Obstinate Baloth enters the battlefield, you gain 4 life.
If a spell or ability an opponent controls causes you to discard Obstinate Baloth, put it onto the battlefield instead of putting it into your graveyard.","2GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Eddy","U","Put target creature or land on top of its owner's library.","2UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heirs of Stromkirk","2","2","R","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)
Whenever Heirs of Stromkirk deals combat damage to a player, put a +1/+1 counter on it.","2RR","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("River's Grasp","UB","If {U} was spent to cast River's Grasp, return up to one target creature to its owner's hand. If {B} was spent to cast River's Grasp, target player reveals his or her hand, you choose a nonland card from it, then that player discards that card. (Do both if {U}{B} was spent.)","3(U/B)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hideous End","B","Destroy target nonblack creature. Its controller loses 2 life.","1BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Zealot","2","1","G","{1}{G}, Sacrifice Viridian Zealot: Destroy target artifact or enchantment.","GG","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Homura, Human Ascendant","4","4","R","Homura, Human Ascendant can't block.
When Homura dies, return it to the battlefield flipped.","4RR","Legendary Creature - Human Monk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Voodoo Doll","At the beginning of your upkeep, put a pin counter on Voodoo Doll.
At the beginning of your end step, if Voodoo Doll is untapped, destroy Voodoo Doll and it deals damage to you equal to the number of pin counters on it.
{X}{X}, {T}: Voodoo Doll deals damage equal to the number of pin counters on it to target creature or player. X is the number of pin counters on Voodoo Doll.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Endless Wurm","9","9","G","Trample
At the beginning of your upkeep, sacrifice Endless Wurm unless you sacrifice an enchantment.","3GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inner Fire","R","Add {R} to your mana pool for each card in your hand.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silhana Ledgewalker","1","1","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Silhana Ledgewalker can't be blocked except by creatures with flying.","1G","Creature - Elf Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elder Druid","2","2","G","{3}{G}, {T}: You may tap or untap target artifact, creature, or land.","3G","Creature - Elf Cleric Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Darkest Hour","B","All creatures are black.","B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chub Toad","1","1","G","Whenever Chub Toad blocks or becomes blocked, it gets +2/+2 until end of turn.","2G","Creature - Frog");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hurloon Wrangler","2","2","R","Denimwalk (If defending player is wearing any clothing made of denim, this creature is unblockable.)","2R","Summon - Minotaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sandbar Serpent","3","4","U","Cycling {2} ({2}, Discard this card: Draw a card.)","4U","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cateran Brute","2","2","B","{2}, {T}: Search your library for a Mercenary permanent card with converted mana cost 2 or less and put it onto the battlefield. Then shuffle your library.","2B","Creature - Horror Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decree of Silence","U","Whenever an opponent casts a spell, counter that spell and put a depletion counter on Decree of Silence. If there are three or more depletion counters on Decree of Silence, sacrifice it.
Cycling {4}{U}{U} ({4}{U}{U}, Discard this card: Draw a card.)
When you cycle Decree of Silence, you may counter target spell.","6UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Whip","B","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, that player may pay {3}. If he or she doesn't, Mind Whip deals 2 damage to that player and you tap that creature.","2BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Krosan Reclamation","G","Target player shuffles up to two target cards from his or her graveyard into his or her library.
Flashback {1}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Downhill Charge","R","You may sacrifice a Mountain rather than pay Downhill Charge's mana cost.
Target creature gets +X/+0 until end of turn, where X is the number of Mountains you control.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of Tresserhorn","6","6","B","Whenever Keeper of Tresserhorn attacks and isn't blocked, it assigns no combat damage this turn and defending player loses 2 life.","5B","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crop Rotation","G","As an additional cost to cast Crop Rotation, sacrifice a land.
Search your library for a land card and put that card onto the battlefield. Then shuffle your library.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sicken","B","Enchant creature
Enchanted creature gets -1/-1.
Cycling {2} ({2}, Discard this card: Draw a card.)","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tuktuk Scrapper","2","2","R","Whenever Tuktuk Scrapper or another Ally enters the battlefield under your control, you may destroy target artifact. If that artifact is put into a graveyard this way, Tuktuk Scrapper deals damage to that artifact's controller equal to the number of Allies you control.","3R","Creature - Goblin Artificer Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Agility","R","Enchant creature
Enchanted creature gets +1/+1 and has flanking. (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coalition Flag","W","Enchant creature you control
Enchanted creature is a Flagbearer.
While choosing targets as part of casting a spell or activating an ability, your opponents must choose at least one Flagbearer on the battlefield if able.","W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Desert","{T}: Add {1} to your mana pool.
{T}: Desert deals 1 damage to target attacking creature. Activate this ability only during the end of combat step.",NULL,"Land - Desert");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gatstaf Shepherd","2","2","G","At the beginning of each upkeep, if no spells were cast last turn, transform Gatstaf Shepherd.","1G","Creature - Human Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emberstrike Duo","1","1","BR","Whenever you cast a black spell, Emberstrike Duo gets +1/+1 until end of turn.
Whenever you cast a red spell, Emberstrike Duo gains first strike until end of turn.","1(B/R)","Creature - Elemental Warrior Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sizzle","R","Sizzle deals 3 damage to each opponent.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frontline Sage","0","1","U","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
{U}, {T}: Draw a card, then discard a card.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tar Pit Warrior","3","4","B","When Tar Pit Warrior becomes the target of a spell or ability, sacrifice it.","2B","Creature - Cyclops Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dream Chisel","Face-down creature spells you cast cost {1} less to cast.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Galvanic Blast","R","Galvanic Blast deals 2 damage to target creature or player.
Metalcraft - Galvanic Blast deals 4 damage to that creature or player instead if you control three or more artifacts.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spite // Malice","B","Destroy target nonblack creature. It can't be regenerated.
---
Counter target noncreature spell.","3B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glen Elendra Liege","2","3","UB","Flying
Other blue creatures you control get +1/+1.
Other black creatures you control get +1/+1.","1(U/B)(U/B)(U/B)","Creature - Faerie Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Laccolith Titan","6","6","R","Whenever Laccolith Titan becomes blocked, you may have it deal damage equal to its power to target creature. If you do, Laccolith Titan assigns no combat damage this turn.","5RR","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heliophial","Sunburst (This enters the battlefield with a charge counter on it for each color of mana spent to cast it.)
{2}, Sacrifice Heliophial: Heliophial deals damage equal to the number of charge counters on it to target creature or player.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Condescend","U","Counter target spell unless its controller pays {X}.
Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","XU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nobilis of War","3","4","WR","Flying
Attacking creatures you control get +2/+0.","(R/W)(R/W)(R/W)(R/W)(R/W)","Creature - Spirit Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pain Magnification","BR","Whenever an opponent is dealt 3 or more damage by a single source, that player discards a card.","1BR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Eon Fog","Players skip their untap steps.
Whenever you roll {C}, untap all permanents you control.",NULL,"Plane - Equilor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Villagers of Estwald","2","3","G","At the beginning of each upkeep, if no spells were cast last turn, transform Villagers of Estwald.","2G","Creature - Human Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fallen Askari","2","2","B","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
Fallen Askari can't block.","1B","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Balancing Act","W","Each player chooses a number of permanents he or she controls equal to the number of permanents controlled by the player who controls the fewest, then sacrifices the rest. Each player discards cards the same way.","2WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crystal Spray","U","Change the text of target spell or permanent by replacing all instances of one color word or basic land type with another until end of turn.
Draw a card.","2U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Booby Trap","As Booby Trap enters the battlefield, name a card other than a basic land card and choose an opponent.
The chosen player reveals each card he or she draws.
When the chosen player draws the named card, sacrifice Booby Trap. If you do, Booby Trap deals 10 damage to that player.","6","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arena of the Ancients","Legendary creatures don't untap during their controllers' untap steps.
When Arena of the Ancients enters the battlefield, tap all legendary creatures.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Helionaut","1","2","W","Flying
{1}, {T}: Add one mana of any color to your mana pool.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guardian of Solitude","1","2","U","Whenever you cast a Spirit or Arcane spell, target creature gains flying until end of turn.","1U","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Hive","{5}, {T}: Put a 1/1 colorless Insect artifact creature token with flying named Wasp onto the battlefield. (It can't be blocked except by creatures with flying or reach.)","5","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Candles of Leng","{4}, {T}: Reveal the top card of your library. If it has the same name as a card in your graveyard, put it into your graveyard. Otherwise, draw a card.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightning Crafter","3","3","R","Champion a Goblin or Shaman (When this enters the battlefield, sacrifice it unless you exile another Goblin or Shaman you control. When this leaves the battlefield, that card returns to the battlefield.)
{T}: Lightning Crafter deals 3 damage to target creature or player.","3R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unbender Tine","WU","{T}: Untap another target permanent.","2WU","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grixis Battlemage","2","2","B","{U}, {T}: Draw a card, then discard a card.
{R}, {T}: Target creature can't block this turn.","2B","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veteran Cavalier","2","2","W","Vigilance (Attacking doesn't cause this creature to tap.)","WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gorilla War Cry","R","Cast Gorilla War Cry only during combat before blockers are declared.
Creatures can't be blocked this turn except by two or more creatures.
Draw a card at the beginning of the next turn's upkeep.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Renegade Warlord","3","3","R","First strike
Whenever Renegade Warlord attacks, each other attacking creature gets +1/+0 until end of turn.","4R","Creature - Human Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Juntu Stakes","Creatures with power 1 or less don't untap during their controllers' untap steps.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mana Matrix","Instant and enchantment spells you cast cost up to {2} less to cast.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rune of Protection: White","W","{W}: The next time a white source of your choice would deal damage to you this turn, prevent that damage.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noble Hierarch","0","1","G","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
{T}: Add {G}, {W}, or {U} to your mana pool.","G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pestilence Demon","7","6","B","Flying
{B}: Pestilence Demon deals 1 damage to each creature and each player.","5BBB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pillar of Flame","R","Pillar of Flame deals 2 damage to target creature or player. If a creature dealt damage this way would die this turn, exile it instead.","R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bojuka Bog","Bojuka Bog enters the battlefield tapped.
When Bojuka Bog enters the battlefield, exile all cards from target player's graveyard.
{T}: Add {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Convalescence","W","At the beginning of your upkeep, if you have 10 or less life, you gain 1 life.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Angel","4","4","W","Flying
Vigilance (Attacking doesn't cause this creature to tap.)","3WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twinstrike","BR","Twinstrike deals 2 damage to each of two target creatures.
Hellbent - Destroy those creatures instead if you have no cards in hand.","3BR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spined Wurm","5","4","G",NULL,"4G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Belligerent Hatchling","6","6","WR","First strike
Belligerent Hatchling enters the battlefield with four -1/-1 counters on it.
Whenever you cast a red spell, remove a -1/-1 counter from Belligerent Hatchling.
Whenever you cast a white spell, remove a -1/-1 counter from Belligerent Hatchling.","3(R/W)","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rushwood Grove","Rushwood Grove enters the battlefield tapped.
{T}: Put a storage counter on Rushwood Grove.
{T}, Remove any number of storage counters from Rushwood Grove: Add {G} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dimir Guildmage","2","2","UB","({(u/b)} can be paid with either {U} or {B}.)
{3}{U}: Target player draws a card. Activate this ability only any time you could cast a sorcery.
{3}{B}: Target player discards a card. Activate this ability only any time you could cast a sorcery.","(U/B)(U/B)","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sire of the Storm","3","3","U","Flying
Whenever you cast a Spirit or Arcane spell, you may draw a card.","4UU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vexing Sphinx","4","4","U","Flying
Cumulative upkeep-Discard a card. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When Vexing Sphinx dies, draw a card for each age counter on it.","1UU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merchant of Secrets","1","1","U","When Merchant of Secrets enters the battlefield, draw a card.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Words of Wind","U","{1}: The next time you would draw a card this turn, each player returns a permanent he or she controls to its owner's hand instead.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scatter the Seeds","G","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Put three 1/1 green Saproling creature tokens onto the battlefield.","3GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glade Gnarr","4","4","G","Whenever a player casts a blue spell, Glade Gnarr gets +2/+2 until end of turn.","5G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Predatory Urge","G","Enchant creature
Enchanted creature has ""{T}: This creature deals damage equal to its power to target creature. That creature deals damage equal to its power to this creature.""","3G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shinen of Flight's Wings","3","3","U","Flying
Channel - {U}, Discard Shinen of Flight's Wings: Target creature gains flying until end of turn.","4U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chancellor of the Tangle","6","7","G","You may reveal this card from your opening hand. If you do, at the beginning of your first main phase, add {G} to your mana pool.
Vigilance, reach","4GGG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stop That","B","Target player discards a card.
Gotcha Whenever an opponent audibly flicks the cards in his or her hand, you may say ""Gotcha"" If you do, return Stop That from your graveyard to your hand.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abomination","2","6","B","Whenever Abomination blocks or becomes blocked by a green or white creature, destroy that creature at end of combat.","3BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gerrard's Command","WG","Untap target creature. It gets +3/+3 until end of turn.","GW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Planar Guide","1","1","W","{3}{W}, Exile Planar Guide: Exile all creatures. At the beginning of the next end step, return those cards to the battlefield under their owners' control.","W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thran Tome","{5}, {T}: Reveal the top three cards of your library. Target opponent chooses one of those cards. Put that card into your graveyard, then draw two cards.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jaded Response","U","Counter target spell if it shares a color with a creature you control.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Scouts","R","Put three 1/1 red Goblin Scout creature tokens with mountainwalk onto the battlefield.","3RR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thran Turbine","At the beginning of your upkeep, you may add {1} or {2} to your mana pool. You can't spend this mana to cast spells.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Illusionary Presence","2","2","U","Cumulative upkeep {U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of your upkeep, choose a land type. Illusionary Presence gains landwalk of the chosen type until end of turn.","1UU","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burnt Offering","B","As an additional cost to cast Burnt Offering, sacrifice a creature.
Add X mana in any combination of {B} and/or {R} to your mana pool, where X is the sacrificed creature's converted mana cost.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Horror of Horrors","B","Sacrifice a Swamp: Regenerate target black creature. (The next time that creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","3BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Choking Tethers","U","Tap up to four target creatures.
Cycling {1}{U} ({1}{U}, Discard this card: Draw a card.)
When you cycle Choking Tethers, you may tap target creature.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steamflogger Boss","3","3","R","Other Rigger creatures you control get +1/+0 and have haste.
If a Rigger you control would assemble a Contraption, it assembles two Contraptions instead.","3R","Creature - Goblin Rigger");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eye of Yawgmoth","{3}, {T}, Sacrifice a creature: Reveal a number of cards from the top of your library equal to the sacrificed creature's power. Put one into your hand and exile the rest.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lys Alana Scarblade","1","1","B","{T}, Discard an Elf card: Target creature gets -X/-X until end of turn, where X is the number of Elves you control.","2B","Creature - Elf Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hobgoblin Dragoon","1","2","WR","Flying, first strike","2(R/W)","Creature - Goblin Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunting Kavu","2","3","RG","{1}{R}{G}, {T}: Exile Hunting Kavu and target creature without flying that's attacking you.","1RG","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hollow Specter","2","2","B","Flying
Whenever Hollow Specter deals combat damage to a player, you may pay {X}. If you do, that player reveals X cards from his or her hand and you choose one of them. That player discards that card.","1BB","Creature - Specter");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Malachite Golem","5","3","{1}{G}: Malachite Golem gains trample until end of turn.","6","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Master's Call","W","Put two 1/1 colorless Myr artifact creature tokens onto the battlefield.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crush Underfoot","R","Choose a Giant creature you control. It deals damage equal to its power to target creature.","1R","Tribal Instant - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Solifuge","4","1","RG","({(r/g)} can be paid with either {R} or {G}.)
Trample; haste; shroud (This permanent can't be the target of spells or abilities.)","2(R/G)(R/G)","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Waxmane Baku","2","2","W","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Waxmane Baku.
{1}, Remove X ki counters from Waxmane Baku: Tap X target creatures.","2W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spreading Algae","G","Enchant Swamp
When enchanted land becomes tapped, destroy it.
When Spreading Algae is put into a graveyard from the battlefield, return Spreading Algae to its owner's hand.","G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sarcomancy","B","When Sarcomancy enters the battlefield, put a 2/2 black Zombie creature token onto the battlefield.
At the beginning of your upkeep, if there are no Zombies on the battlefield, Sarcomancy deals 1 damage to you.","B","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ertai","Creatures you control have hexproof. (They can't be the targets of spells or abilities your opponents control.)",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Khabál Ghoul","1","1","B","At the beginning of each end step, put a +1/+1 counter on Khabál Ghoul for each creature that died this turn.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Sunchaser","1","1","W","Metalcraft - As long as you control three or more artifacts, Auriok Sunchaser gets +2/+2 and has flying.","1W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Welding Jar","Sacrifice Welding Jar: Regenerate target artifact.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Weakness","B","Enchant creature
Enchanted creature gets -2/-1.","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Familiar's Ruse","U","As an additional cost to cast Familiar's Ruse, return a creature you control to its owner's hand.
Counter target spell.","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dakmor Lancer","3","3","B","When Dakmor Lancer enters the battlefield, destroy target nonblack creature.","4BB","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lethe Lake","At the beginning of your upkeep, put the top ten cards of your library into your graveyard.
Whenever you roll {C}, target player puts the top ten cards of his or her library into his or her graveyard.",NULL,"Plane - Arkhos");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blaze","R","Blaze deals X damage to target creature or player.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hollowhenge Scavenger","4","5","G","Morbid - When Hollowhenge Scavenger enters the battlefield, if a creature died this turn, you gain 5 life.","3GG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gluttonous Slime","2","2","G","Flash
Devour 1 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with that many +1/+1 counters on it.)","2G","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cabal Conditioning","B","Any number of target players each discard a number of cards equal to the highest converted mana cost among permanents you control.","6B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ravenous Vampire","3","3","B","Flying
At the beginning of your upkeep, you may sacrifice a nonartifact creature. If you do, put a +1/+1 counter on Ravenous Vampire. If you don't, tap Ravenous Vampire.","3BB","Creature - Vampire");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Moonring Island","({T}: Add {U} to your mana pool.)
Moonring Island enters the battlefield tapped.
{U}, {T}: Look at the top card of target player's library. Activate this ability only if you control two or more blue permanents.",NULL,"Land - Island");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sand Squid","2","2","U","Islandwalk
You may choose not to untap Sand Squid during your untap step.
{T}: Tap target creature. That creature doesn't untap during its controller's untap step for as long as Sand Squid remains tapped.","3U","Creature - Squid Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unstable Shapeshifter","0","1","U","Whenever another creature enters the battlefield, Unstable Shapeshifter becomes a copy of that creature and gains this ability.","3U","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silent Specter","4","4","B","Flying
Whenever Silent Specter deals combat damage to a player, that player discards two cards.
Morph {3}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4BB","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battlegrace Angel","4","4","W","Flying
Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
Whenever a creature you control attacks alone, it gains lifelink until end of turn.","3WW","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Iron Lance","{3}, {T}: Target creature gains first strike until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Trailblazer","2","2","B","Tap an untapped Zombie you control: Target land becomes a Swamp until end of turn.
Tap an untapped Zombie you control: Target creature gains swampwalk until end of turn.","BBB","Creature - Zombie Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decree of Savagery","G","Put four +1/+1 counters on each creature you control.
Cycling {4}{G}{G} ({4}{G}{G}, Discard this card: Draw a card.)
When you cycle Decree of Savagery, you may put four +1/+1 counters on target creature.","7GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deadly Insect","6","1","G","Shroud (This permanent can't be the target of spells or abilities.)","4G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harrier Griffin","3","3","W","Flying
At the beginning of your upkeep, tap target creature.","5W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Armored Ascension","W","Enchant creature
Enchanted creature gets +1/+1 for each Plains you control and has flying.","3W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unearth","B","Return target creature card with converted mana cost 3 or less from your graveyard to the battlefield.
Cycling {2} ({2}, Discard this card: Draw a card.)","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mystic Snake","2","2","UG","Flash (You may cast this spell any time you could cast an instant.)
When Mystic Snake enters the battlefield, counter target spell.","1GUU","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kobold Drill Sergeant","1","2","R","Other Kobold creatures you control get +0/+1 and have trample.","1R","Creature - Kobold Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minister of Impediments","1","1","WU","({(w/u)} can be paid with either {W} or {U}.)
{T}: Tap target creature.","2(W/U)","Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sylvok Explorer","1","1","G","{T}: Add to your mana pool one mana of any color that a land an opponent controls could produce.","1G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Adamaro, First to Desire","*","*","R","Adamaro, First to Desire's power and toughness are each equal to the number of cards in the hand of the opponent with the most cards in hand.","1RR","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Emissary","2","1","W","{W}: Soltari Emissary gains shadow until end of turn. (This creature can block or be blocked by only creatures with shadow.)","1W","Creature - Soltari Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voice of Truth","2","2","W","Flying, protection from white","3W","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scapegoat","W","As an additional cost to cast Scapegoat, sacrifice a creature.
Return any number of target creatures you control to their owner's hand.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Persecute","B","Choose a color. Target player reveals his or her hand and discards all cards of that color.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feint","R","Tap all creatures blocking target attacking creature. Prevent all combat damage that would be dealt this turn by that creature and each creature blocking it.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sigil Captain","3","3","WG","Whenever a creature enters the battlefield under your control, if that creature is 1/1, put two +1/+1 counters on it.","1GWW","Creature - Rhino Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mesa Pegasus","1","1","W","Flying; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","1W","Creature - Pegasus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Penumbra Spider","2","4","G","Reach (This creature can block creatures with flying.)
When Penumbra Spider dies, put a 2/4 black Spider creature token with reach onto the battlefield.","2GG","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Question Elemental?","3","4","U","Flying
Are you aware that when you say something that isn't a question, the player who first points out this fact gains control of Question Elemental?","2UU","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Krovikan Whispers","U","Enchant creature
Cumulative upkeep {U} or {B} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
You control enchanted creature.
When Krovikan Whispers is put into a graveyard from the battlefield, you lose 2 life for each age counter on it.","3U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Apprentice Sorcerer","1","1","U","{T}: Apprentice Sorcerer deals 1 damage to target creature or player. Activate this ability only during your turn, before attackers are declared.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guided Strike","W","Target creature gets +1/+0 and gains first strike until end of turn.
Draw a card.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hurloon Shaman","2","3","R","When Hurloon Shaman dies, each player sacrifices a land.","1RR","Creature - Minotaur Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Astral Slide","W","Whenever a player cycles a card, you may exile target creature. If you do, return that card to the battlefield under its owner's control at the beginning of the next end step.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raging Cougar","2","2","R","Haste","2R","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Braids, Conjurer Adept","2","2","U","At the beginning of each player's upkeep, that player may put an artifact, creature, or land card from his or her hand onto the battlefield.","2UU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dandân","4","1","U","Dandân can't attack unless defending player controls an Island.
When you control no Islands, sacrifice Dandân.","UU","Creature - Fish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Apathy","U","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.
At the beginning of the upkeep of enchanted creature's controller, that player may discard a card at random. If he or she does, untap that creature.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skaab Goliath","6","9","U","As an additional cost to cast Skaab Goliath, exile two creature cards from your graveyard.
Trample","5U","Creature - Zombie Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Insidious Bookworms","1","1","B","When Insidious Bookworms dies, you may pay {1}{B}. If you do, target player discards a card at random.","B","Creature - Worm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Might","G","Target creature gets +1/+1 until end of turn. That creature gets an additional +4/+4 until end of turn unless any player pays {2}.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Assassin","2","2","R","Whenever Goblin Assassin or another Goblin enters the battlefield, each player flips a coin. Each player whose coin comes up tails sacrifices a creature.","3RR","Creature - Goblin Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Captivating Glance","U","Enchant creature
At the beginning of your end step, clash with an opponent. If you win, gain control of enchanted creature. Otherwise, that player gains control of enchanted creature. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorilla Shaman","1","1","R","{X}{X}{1}: Destroy target noncreature artifact with converted mana cost X.","R","Creature - Ape Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Painsmith","2","1","B","Whenever you cast an artifact spell, you may have target creature get +2/+0 and gain deathtouch until end of turn.","1B","Creature - Human Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Undying Rage","R","Enchant creature
Enchanted creature gets +2/+2 and can't block.
When Undying Rage is put into a graveyard from the battlefield, return Undying Rage to its owner's hand.","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disciple of Tevesh Szat","3","1","B","{T}: Target creature gets -1/-1 until end of turn.
{4}{B}{B}, {T}, Sacrifice Disciple of Tevesh Szat: Target creature gets -6/-6 until end of turn.","2BB","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Breathstealer","2","2","B","{B}: Breathstealer gets +1/-1 until end of turn.","2B","Creature - Nightstalker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tundra Kavu","2","2","R","{T}: Target land becomes a Plains or an Island until end of turn.","2R","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wirewood Guardian","6","6","G","Forestcycling {2} ({2}, Discard this card: Search your library for a Forest card, reveal it, and put it into your hand. Then shuffle your library.)","5GG","Creature - Elf Mutant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Allergy","U","As Psychic Allergy enters the battlefield, choose a color.
At the beginning of each opponent's upkeep, Psychic Allergy deals X damage to that player, where X is the number of nontoken permanents of the chosen color he or she controls.
At the beginning of your upkeep, destroy Psychic Allergy unless you sacrifice two Islands.","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Kozilek, Butcher of Truth","12","12","When you cast Kozilek, Butcher of Truth, draw four cards.
Annihilator 4 (Whenever this creature attacks, defending player sacrifices four permanents.)
When Kozilek is put into a graveyard from anywhere, its owner shuffles his or her graveyard into his or her library.","10","Legendary Creature - Eldrazi");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Book Burning","R","Any player may have Book Burning deal 6 damage to him or her. If no one does, put the top six cards of target player's library into his or her graveyard.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rebellion of the Flamekin","R","Whenever you clash, you may pay {1}. If you do, put a 3/1 red Elemental Shaman creature token onto the battlefield. If you won, that token gains haste until end of turn. (This ability triggers after the clash ends.)","3R","Tribal Enchantment - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turnabout","U","Choose artifact, creature, or land. Tap all untapped permanents of the chosen type target player controls, or untap all tapped permanents of that type that player controls.","2UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Double Play","G","Choose another player. Search your library for a basic land and put that land into play. At the beginning of the next game with that player, search your library for an additional basic land and put that land into play. In both cases, shuffle your library afterwards.","3GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bifurcate","G","Search your library for a permanent card with the same name as target nontoken creature and put that card onto the battlefield. Then shuffle your library.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Debtors' Knell","WB","({(w/b)} can be paid with either {W} or {B}.)
At the beginning of your upkeep, put target creature card from a graveyard onto the battlefield under your control.","4(W/B)(W/B)(W/B)","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodcrazed Neonate","2","1","R","Bloodcrazed Neonate attacks each turn if able.
Whenever Bloodcrazed Neonate deals combat damage to a player, put a +1/+1 counter on it.","1R","Creature - Vampire");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darkwater Catacombs","{1}, {T}: Add {U}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diabolic Vision","UB","Look at the top five cards of your library. Put one of them into your hand and the rest on top of your library in any order.","UB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Birchlore Rangers","1","1","G","Tap two untapped Elves you control: Add one mana of any color to your mana pool.
Morph {G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Rabbit","1","1","G","Forestwalk","G","Creature - Rabbit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sandbar Crocodile","6","5","U","Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)","4U","Creature - Crocodile");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treefolk Healer","2","3","G","{2}{W}, {T}: Prevent the next 2 damage that would be dealt to target creature or player this turn.","4G","Creature - Treefolk Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost Council of Orzhova","4","4","WB","When Ghost Council of Orzhova enters the battlefield, target opponent loses 1 life and you gain 1 life.
{1}, Sacrifice a creature: Exile Ghost Council of Orzhova. Return it to the battlefield under its owner's control at the beginning of the next end step.","WWBB","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fulgent Distraction","W","Choose two target creatures. Tap those creatures, then unattach all Equipment from them.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Virulent Wound","B","Put a -1/-1 counter on target creature. When that creature dies this turn, its controller gets a poison counter.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Forked-Branch Garami","4","4","G","Soulshift 4, soulshift 4 (When this creature dies, you may return up to two target Spirit cards with converted mana cost 4 or less from your graveyard to your hand.)","3GG","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lifeblood","W","Whenever a Mountain an opponent controls becomes tapped, you gain 1 life.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hapless Researcher","1","1","U","Sacrifice Hapless Researcher: Draw a card, then discard a card.","U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindslicer","4","3","B","When Mindslicer dies, each player discards his or her hand.","2BB","Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Proteus Staff","{2}{U}, {T}: Put target creature on the bottom of its owner's library. That creature's controller reveals cards from the top of his or her library until he or she reveals a creature card. The player puts that card onto the battlefield and the rest on the bottom of his or her library in any order. Activate this ability only any time you could cast a sorcery.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kemuri-Onna","3","3","B","When Kemuri-Onna enters the battlefield, target player discards a card.
Whenever you cast a Spirit or Arcane spell, you may return Kemuri-Onna to its owner's hand.","4B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Brambles","2","3","G","Defender (This creature can't attack.)
{G}: Regenerate Wall of Brambles.","2G","Creature - Plant Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Composite Golem","4","4","Sacrifice Composite Golem: Add {W}{U}{B}{R}{G} to your mana pool.","6","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Whetwheel","{X}{X}, {T}: Target player puts the top X cards of his or her library into his or her graveyard.
Morph {3} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uyo, Silent Prophet","4","4","U","Flying
{2}, Return two lands you control to their owner's hand: Copy target instant or sorcery spell. You may choose new targets for the copy.","4UU","Legendary Creature - Moonfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arm with AEther","U","Until end of turn, creatures you control gain ""Whenever this creature deals damage to an opponent, you may return target creature that player controls to its owner's hand.""","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Savage Offensive","R","Kicker {G} (You may pay an additional {G} as you cast this spell.)
Creatures you control gain first strike until end of turn. If Savage Offensive was kicked, they get +1/+1 until end of turn.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Void Maw","4","5","B","Trample
If another creature would die, exile it instead.
Put a card exiled with Void Maw into its owner's graveyard: Void Maw gets +2/+2 until end of turn.","4BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Leak","U","Enchant permanent
As long as enchanted permanent is red or green, it has ""At the beginning of your upkeep, sacrifice this permanent unless you pay its mana cost.""","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gloom Surgeon","2","1","B","If combat damage would be dealt to Gloom Surgeon, prevent that damage and exile that many cards from the top of your library.","1B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Skycaptain","2","2","W","Flying; first strike; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","4W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("City in a Bottle","Whenever a nontoken permanent from the Arabian Nights expansion other than City in a Bottle is on the battlefield, its controller sacrifices it.
Players can't play cards from the Arabian Nights expansion.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drowned","1","1","U","{B}: Regenerate Drowned.","1U","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Battle","U","Whenever a player chooses one or more targets, each player reveals the top card of his or her library. The player who reveals the card with the highest converted mana cost may change the target or targets. If two or more cards are tied for highest cost, the target or targets remain unchanged. Changing targets this way doesn't trigger this ability.","3UU","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vivid Marsh","Vivid Marsh enters the battlefield tapped with two charge counters on it.
{T}: Add {B} to your mana pool.
{T}, Remove a charge counter from Vivid Marsh: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Mystic","3","3","W","{W}, {T}: Tap target creature.","3WW","Creature - Elephant Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Contact Lenses","Urza's Contact Lenses comes into play tapped and does not untap during its controller's untap phase.
All players play with their hands face up.
Clap your hands twice: Tap or untap Urza's Contact Lenses.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Halt Order","U","Counter target artifact spell.
Draw a card.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thoughtpicker Witch","1","1","B","{1}, Sacrifice a creature: Look at the top two cards of target opponent's library, then exile one of them.","B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Defeat","W","Return target creature card from your graveyard to the battlefield.","3W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Court Archers","1","3","G","Reach (This can block creatures with flying.)
Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","2G","Creature - Human Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lavaborn Muse","3","3","R","At the beginning of each opponent's upkeep, if that player has two or fewer cards in hand, Lavaborn Muse deals 3 damage to him or her.","3R","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Executioner's Capsule","B","{1}{B}, {T}, Sacrifice Executioner's Capsule: Destroy target nonblack creature.","B","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dawnglow Infusion","WG","You gain X life if {G} was spent to cast Dawnglow Infusion and X life if {W} was spent to cast it. (Do both if {G}{W} was spent.)","X(G/W)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Juzám Djinn","5","5","B","At the beginning of your upkeep, Juzám Djinn deals 1 damage to you.","2BB","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightshade Seer","1","1","B","{2}{B}, {T}: Reveal any number of black cards in your hand. Target creature gets -X/-X until end of turn, where X is the number of cards revealed this way.","3B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necromancer's Magemark","B","Enchant creature
Creatures you control that are enchanted get +1/+1.
If a creature you control that's enchanted would die, return it to its owner's hand instead.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alley Grifters","2","2","B","Whenever Alley Grifters becomes blocked, defending player discards a card.","1BB","Creature - Human Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slith Ascendant","1","1","W","Flying
Whenever Slith Ascendant deals combat damage to a player, put a +1/+1 counter on it.","1WW","Creature - Slith");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gift of Granite","W","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature gets +0/+2.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Retribution","R","Choose two target creatures an opponent controls. That player chooses and sacrifices one of those creatures. Put a -1/-1 counter on the other.","2RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Totem Speaker","3","3","G","Whenever a Beast enters the battlefield, you may gain 3 life.","4G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scavenger Drake","1","1","B","Flying
Whenever another creature dies, you may put a +1/+1 counter on Scavenger Drake.","3B","Creature - Drake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Aegis of the Meek","{1}, {T}: Target 1/1 creature gets +1/+2 until end of turn.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ray of Distortion","W","Destroy target artifact or enchantment.
Flashback {4}{W}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Masked Admirers","3","2","G","When Masked Admirers enters the battlefield, draw a card.
Whenever you cast a creature spell, you may pay {G}{G}. If you do, return Masked Admirers from your graveyard to your hand.","2GG","Creature - Elf Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Halls of Mist","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Creatures that attacked during their controller's last turn can't attack.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blades of Velis Vel","R","Changeling (This card is every creature type at all times.)
Up to two target creatures each get +2/+0 and gain all creature types until end of turn.","1R","Tribal Instant - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Roar of the Kha","W","Choose one - Creatures you control get +1/+1 until end of turn; or untap all creatures you control.
Entwine {1}{W} (Choose both if you pay the entwine cost.)","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talus Paladin","2","3","W","Whenever Talus Paladin or another Ally enters the battlefield under your control, you may have Allies you control gain lifelink until end of turn, and you may put a +1/+1 counter on Talus Paladin.","3W","Creature - Human Knight Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brindle Boar","2","2","G","Sacrifice Brindle Boar: You gain 4 life.","2G","Creature - Boar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Retainer","2","3","U","{U}{U}: Tap target creature without flying.","2UU","Creature - Cephalid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tithe","W","Search your library for a Plains card. If target opponent controls more lands than you, you may search your library for an additional Plains card. Reveal those cards and put them into your hand. Then shuffle your library.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defiling Tears","B","Until end of turn, target creature becomes black, gets +1/-1, and gains ""{B}: Regenerate this creature.""","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nagao, Bound by Honor","3","3","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
Whenever Nagao, Bound by Honor attacks, Samurai creatures you control get +1/+1 until end of turn.","3W","Legendary Creature - Human Samurai");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gallantry","W","Target blocking creature gets +4/+4 until end of turn.
Draw a card.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Second Wind","U","Enchant creature
{T}: Tap enchanted creature.
{T}: Untap enchanted creature.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Masons","2","1","R","When Goblin Masons dies, destroy target Wall.","1R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Caldera Lake","Caldera Lake enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{T}: Add {U} or {R} to your mana pool. Caldera Lake deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hell Swarm","B","All creatures get -1/-0 until end of turn.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hyena Umbra","W","Enchant creature
Enchanted creature gets +1/+1 and has first strike.
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grab the Reins","R","Choose one - Until end of turn, you gain control of target creature and it gains haste; or sacrifice a creature, then Grab the Reins deals damage equal to that creature's power to target creature or player.
Entwine {2}{R} (Choose both if you pay the entwine cost.)","3R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disempower","W","Put target artifact or enchantment on top of its owner's library.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wispmare","1","3","W","Flying
When Wispmare enters the battlefield, destroy target enchantment.
Evoke {W} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","2W","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infiltrate","U","Target creature is unblockable this turn.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avenger en-Dal","1","1","W","{2}{W}, {T}, Discard a card: Exile target attacking creature. Its controller gains life equal to its toughness.","1W","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wall of Tanglecord","0","6","Defender
{G}: Wall of Tanglecord gains reach until end of turn. (It can block creatures with flying.)","2","Artifact Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burden of Guilt","W","Enchant creature
{1}: Tap enchanted creature.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Norwood Archers","3","3","G","Reach (This creature can block creatures with flying.)","3G","Creature - Elf Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Rites","B","Discard any number of cards. Target player reveals his or her hand, then you choose a nonland card from it for each card discarded this way. That player discards those cards.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eye to Eye","B","You and target creature's controller have a staring contest. If you win, destroy that creature.","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Spearhunter","2","1","R","First strike (This creature deals combat damage before creatures without first strike.)","2R","Creature - Viashino Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Initiates of the Ebon Hand","1","1","B","{1}: Add {B} to your mana pool. If this ability has been activated four or more times this turn, sacrifice Initiates of the Ebon Hand at the beginning of the next end step.","B","Creature - Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Trainee","1","1","B","Sacrifice Cabal Trainee: Target creature gets -2/-0 until end of turn.","B","Creature - Human Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mangara of Corondor","1","1","W","{T}: Exile Mangara of Corondor and target permanent.","1WW","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drooling Ogre","3","3","R","Whenever a player casts an artifact spell, that player gains control of Drooling Ogre. (This effect lasts indefinitely.)","1R","Creature - Ogre");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burrowing","R","Enchant creature
Enchanted creature has mountainwalk.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necroskitter","1","4","B","Wither (This deals damage to creatures in the form of -1/-1 counters.)
Whenever a creature an opponent controls with a -1/-1 counter on it dies, you may return that card to the battlefield under your control.","1BB","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vision Skeins","U","Each player draws two cards.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spelljack","U","Counter target spell. If that spell is countered this way, exile it instead of putting it into its owner's graveyard. You may play it without paying its mana cost for as long as it remains exiled. (If it has X in its mana cost, X is 0.)","3UUU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riftwing Cloudskate","2","2","U","Flying
When Riftwing Cloudskate enters the battlefield, return target permanent to its owner's hand.
Suspend 3-{1}{U} (Rather than cast this card from your hand, you may pay {1}{U} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","3UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Staunch Defenders","3","4","W","When Staunch Defenders enters the battlefield, you gain 4 life.","3WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cerebral Vortex","UR","Target player draws two cards, then Cerebral Vortex deals damage to that player equal to the number of cards he or she has drawn this turn.","1UR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Storm World","R","At the beginning of each player's upkeep, Storm World deals X damage to that player, where X is 4 minus the number of cards in his or her hand.","R","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quarum Trench Gnomes","1","1","R","{T}: If target Plains is tapped for mana, it produces colorless mana instead of white mana. (This effect lasts indefinitely.)","3R","Creature - Gnome");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thorn Thallid","2","2","G","At the beginning of your upkeep, put a spore counter on Thorn Thallid.
Remove three spore counters from Thorn Thallid: Thorn Thallid deals 1 damage to target creature or player.","1GG","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Silence","W","Your opponents can't cast spells this turn. (Spells cast before this resolves are unaffected.)","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peace and Quiet","W","Destroy two target enchantments.","1W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Horrible Hordes","2","2","Rampage 1 (Whenever this creature becomes blocked, it gets +1/+1 until end of turn for each creature blocking it beyond the first.)","3","Artifact Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Safehold Elite","2","2","WG","Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","1(G/W)","Creature - Elf Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rebuking Ceremony","G","Put two target artifacts on top of their owners' libraries.","3GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Macabre Waltz","B","Return up to two target creature cards from your graveyard to your hand, then discard a card.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reckless Scholar","2","1","U","{T}: Target player draws a card, then discards a card.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Painful Quandary","B","Whenever an opponent casts a spell, that player loses 5 life unless he or she discards a card.","3BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Siege of Towers","R","Replicate {1}{R} (When you cast this spell, copy it for each time you paid its replicate cost. You may choose new targets for the copies.)
Target Mountain becomes a 3/1 creature. It's still a land.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drove of Elves","*","*","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Drove of Elves's power and toughness are each equal to the number of green permanents you control.","3G","Creature - Elf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nekrataal Avatar","Creature spells you cast cost {B} less to cast. This effect reduces only the amount of colored mana you pay.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Resounding Thunder","R","Resounding Thunder deals 3 damage to target creature or player.
Cycling {5}{B}{R}{G} ({5}{B}{R}{G}, Discard this card: Draw a card.)
When you cycle Resounding Thunder, it deals 6 damage to target creature or player.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flay","B","Target player discards a card at random. Then that player discards another card at random unless he or she pays {1}.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lost Order of Jarkeld","1+*","1+*","W","As Lost Order of Jarkeld enters the battlefield, choose an opponent.
Lost Order of Jarkeld's power and toughness are each equal to 1 plus the number of creatures the chosen player controls.","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vigean Graftmage","0","0","U","Graft 2 (This creature enters the battlefield with two +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{1}{U}: Untap target creature with a +1/+1 counter on it.","2U","Creature - Vedalken Wizard Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Naya Hushblade","2","1","WRG","As long as you control another multicolored permanent, Naya Hushblade gets +1/+1 and has shroud. (It can't be the target of spells or abilities.)","(R/W)G","Creature - Elf Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wolf Pack","7","6","G","You may have Wolf Pack assign its combat damage as though it weren't blocked.","6GG","Creature - Wolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stupefying Touch","U","Enchant creature
When Stupefying Touch enters the battlefield, draw a card.
Enchanted creature's activated abilities can't be activated.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Roar of Jukai","G","If you control a Forest, each blocked creature gets +2/+2 until end of turn.
Splice onto Arcane-An opponent gains 5 life. (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","2G","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primoc Escapee","4","4","U","Flying
Cycling {2} ({2}, Discard this card: Draw a card.)","6U","Creature - Bird Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Algae Gharial","1","1","G","Shroud
Whenever another creature dies, you may put a +1/+1 counter on Algae Gharial.","3G","Creature - Crocodile");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Yotian Soldier","1","4","Vigilance","3","Artifact Creature - Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rusted Relic","Metalcraft - Rusted Relic is a 5/5 Golem artifact creature as long as you control three or more artifacts.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greater Forgeling","3","4","R","{1}{R}: Greater Forgeling gets +3/-3 until end of turn.","3RR","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Venarian Glimmer","U","Target player reveals his or her hand. You choose a nonland card with converted mana cost X or less from it. That player discards that card.","XU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spark Mage","1","1","R","Whenever Spark Mage deals combat damage to a player, you may have Spark Mage deal 1 damage to target creature that player controls.","R","Creature - Dwarf Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Diamond Valley","{T}, Sacrifice a creature: You gain life equal to the sacrificed creature's toughness.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("O-Naginata","O-Naginata can be attached only to a creature with 3 or more power.
Equipped creature gets +3/+0 and has trample.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lord of Extinction","*","*","BG","Lord of Extinction's power and toughness are each equal to the number of cards in all graveyards.","3BG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Meditate","U","Draw four cards. You skip your next turn.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slaughterhouse Bouncer","3","3","B","Hellbent - When Slaughterhouse Bouncer dies, if you have no cards in hand, target creature gets -3/-3 until end of turn.","4B","Creature - Ogre Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Muse Vessel","{3}, {T}: Target player exiles a card from his or her hand. Activate this ability only any time you could cast a sorcery.
{1}: Choose a card exiled with Muse Vessel. You may play that card this turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magnivore","*","*","R","Haste (This creature can attack the turn it comes under your control.)
Magnivore's power and toughness are each equal to the number of sorcery cards in all graveyards.","2RR","Creature - Lhurgoyf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Temple Garden","({T}: Add {G} or {W} to your mana pool.)
As Temple Garden enters the battlefield, you may pay 2 life. If you don't, Temple Garden enters the battlefield tapped.",NULL,"Land - Forest Plains");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scorching Spear","R","Scorching Spear deals 1 damage to target creature or player.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Godo, Bandit Warlord","3","3","R","When Godo, Bandit Warlord enters the battlefield, you may search your library for an Equipment card and put it onto the battlefield. If you do, shuffle your library.
Whenever Godo attacks for the first time each turn, untap it and all Samurai you control. After this phase, there is an additional combat phase.","5R","Legendary Creature - Human Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recall","U","Discard X cards, then return a card from your graveyard to your hand for each card discarded this way. Exile Recall.","XXU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Radiant, Archangel","3","3","W","Flying, vigilance
Radiant, Archangel gets +1/+1 for each other creature with flying on the battlefield.","3WW","Legendary Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fleeting Image","2","1","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
{1}{U}: Return Fleeting Image to its owner's hand.","2U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brimstone Mage","2","2","R","Level up {3}{R} ({3}{R}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-2
2/3
{T}: Brimstone Mage deals 1 damage to target creature or player.
LEVEL 3+
2/4
{T}: Brimstone Mage deals 3 damage to target creature or player.","2R","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grayscaled Gharial","1","1","U","Islandwalk","U","Creature - Crocodile");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ronom Serpent","5","6","U","Ronom Serpent can't attack unless defending player controls a snow land.
When you control no snow lands, sacrifice Ronom Serpent.","5U","Snow Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Sand-Mage","3","2","R","When Shivan Sand-Mage enters the battlefield, choose one - Remove two time counters from target permanent or suspended card; or put two time counters on target permanent with a time counter on it or suspended card.
Suspend 4-{R} (Rather than cast this card from your hand, you may pay {R} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","2RR","Creature - Viashino Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inescapable Brute","3","3","R","Wither (This deals damage to creatures in the form of -1/-1 counters.)
Inescapable Brute must be blocked if able.","5R","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tunnel Vision","U","Name a card. Target player reveals cards from the top of his or her library until the named card is revealed. If it is, that player puts the rest of the revealed cards into his or her graveyard and puts the named card on top of his or her library. Otherwise, the player shuffles his or her library.","5U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Havenwood Wurm","5","6","G","Flash (You may cast this spell any time you could cast an instant.)
Trample","6G","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lucent Liminid","3","3","W","Flying","3WW","Enchantment Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Molten Influence","R","Counter target instant or sorcery spell unless its controller has Molten Influence deal 4 damage to him or her.","1R","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Saberclaw Golem","4","2","{R}: Saberclaw Golem gains first strike until end of turn.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormscale Anarch","2","2","R","{2}{R}, Discard a card at random: Stormscale Anarch deals 2 damage to target creature or player. If the discarded card was multicolored, Stormscale Anarch deals 4 damage to that creature or player instead.","2RR","Creature - Viashino Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Izzet Signet","{1}, {T}: Add {U}{R} to your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chaos Harlequin","2","4","R","{R}: Exile the top card of your library. If that card is a land card, Chaos Harlequin gets -4/-0 until end of turn. Otherwise, Chaos Harlequin gets +2/+0 until end of turn.","2RR","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Elder","2","1","G","When Yavimaya Elder dies, you may search your library for up to two basic land cards, reveal them, and put them into your hand. If you do, shuffle your library.
{2}, Sacrifice Yavimaya Elder: Draw a card.","1GG","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Collapsing Borders","R","Domain - At the beginning of each player's upkeep, that player gains 1 life for each basic land type among lands he or she controls. Then Collapsing Borders deals 3 damage to him or her.","3R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sage's Dousing","U","Counter target spell unless its controller pays {3}. If you control a Wizard, draw a card.","2U","Tribal Instant - Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk of the Pearl Trident","1","1","U",NULL,"U","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Laquatus's Creativity","U","Target player draws cards equal to the number of cards in his or her hand, then discards that many cards.","4U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brothers Yamazaki","2","1","R","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
If there are exactly two permanents named Brothers Yamazaki on the battlefield, the ""legend rule"" doesn't apply to them.
Each other creature named Brothers Yamazaki gets +2/+2 and has haste.","2R","Legendary Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Markov Patrician","3","1","B","Lifelink (Damage dealt by this creature also causes you to gain that much life.)","2B","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Faces of the Past","U","Whenever a creature dies, tap all untapped creatures that share a creature type with it or untap all tapped creatures that share a creature type with it.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sustenance","G","{1}, Sacrifice a land: Target creature gets +1/+1 until end of turn.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Alpha Myr","2","1",NULL,"2","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pirate Ship","4","3","U","Pirate Ship can't attack unless defending player controls an Island.
{T}: Pirate Ship deals 1 damage to target creature or player.
When you control no Islands, sacrifice Pirate Ship.","4U","Creature - Human Pirate");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Topple","W","Exile target creature with the greatest power. (If two or more creatures are tied for greatest power, target any one of them.)","2W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cyclopean Mummy","2","1","B","When Cyclopean Mummy dies, exile it.","1B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("_____","1","1","U","{1}: This card's name becomes the name of your choice. Play this ability anywhere, anytime.","1U","Creature - Shapeshifter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sylvok Lifestaff","Equipped creature gets +1/+0.
Whenever equipped creature dies, you gain 3 life.
Equip {1}","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Winnower Patrol","3","2","G","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Winnower Patrol, you may reveal it. If you do, put a +1/+1 counter on Winnower Patrol.","2G","Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vengeance","W","Destroy target tapped creature.","3W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arrest","W","Enchant creature
Enchanted creature can't attack or block, and its activated abilities can't be activated.","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serpentine Kavu","4","4","G","{R}: Serpentine Kavu gains haste until end of turn.","4G","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Observer","2","1","U","When Merfolk Observer enters the battlefield, look at the top card of target player's library.","1U","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dispeller's Capsule","W","{2}{W}, {T}, Sacrifice Dispeller's Capsule: Destroy target artifact or enchantment.","W","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Surgery","U","Whenever an opponent shuffles his or her library, you may look at the top two cards of that library. You may exile one of those cards. Then put the rest on top of that library in any order.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beacon of Immortality","W","Double target player's life total. Shuffle Beacon of Immortality into its owner's library.","5W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Insolence","R","Enchant creature
Whenever enchanted creature becomes tapped, Insolence deals 2 damage to that creature's controller.","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Serrated Biskelion","2","2","{T}: Put a -1/-1 counter on Serrated Biskelion and a -1/-1 counter on target creature.","3","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Highland Weald","Highland Weald enters the battlefield tapped.
{T}: Add {R} or {G} to your mana pool.",NULL,"Snow Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Plaguelord","4","4","B","{T}, Sacrifice Phyrexian Plaguelord: Target creature gets -4/-4 until end of turn.
Sacrifice a creature: Target creature gets -1/-1 until end of turn.","3BB","Creature - Carrier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fylgja","W","Enchant creature
Fylgja enters the battlefield with four healing counters on it.
Remove a healing counter from Fylgja: Prevent the next 1 damage that would be dealt to enchanted creature this turn.
{2}{W}: Put a healing counter on Fylgja.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yukora, the Prisoner","5","5","B","When Yukora, the Prisoner leaves the battlefield, sacrifice all non-Ogre creatures you control.","2BB","Legendary Creature - Demon Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Airdrop Condor","2","2","R","Flying
{1}{R}, Sacrifice a Goblin creature: Airdrop Condor deals damage equal to the sacrificed creature's power to target creature or player.","4R","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nostalgic Dreams","G","As an additional cost to cast Nostalgic Dreams, discard X cards.
Return X target cards from your graveyard to your hand. Exile Nostalgic Dreams.","GG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shinka, the Bloodsoaked Keep","{T}: Add {R} to your mana pool.
{R}, {T}: Target legendary creature gains first strike until end of turn.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stony Silence","W","Activated abilities of artifacts can't be activated.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spurnmage Advocate","1","1","W","{T}: Return two target cards from an opponent's graveyard to his or her hand. Destroy target attacking creature.","W","Creature - Human Nomad");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tropical Island",NULL,NULL,"Land - Forest Island");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Megrim","B","Whenever an opponent discards a card, Megrim deals 2 damage to that player.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Garruk's Companion","3","2","G","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reckless One","*","*","R","Haste
Reckless One's power and toughness are each equal to the number of Goblins on the battlefield.","3R","Creature - Goblin Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Librarian","1","1","R","{R}, {T}: Look at the top eight cards of your library. Exile four of them at random, then put the rest on top of your library in any order.","1R","Creature - Orc");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patron of the Akki","5","5","R","Goblin offering (You may cast this card any time you could cast an instant by sacrificing a Goblin and paying the difference in mana costs between this and the sacrificed Goblin. Mana cost includes color.)
Whenever Patron of the Akki attacks, creatures you control get +2/+0 until end of turn.","4RR","Legendary Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shield of the Ages","{2}: Prevent the next 1 damage that would be dealt to you this turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Thaumaturgist","1","2","U","{T}: Switch target creature's power and toughness until end of turn.","2U","Creature - Merfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vault of Whispers","(Vault of Whispers isn't a spell.)
{T}: Add {B} to your mana pool.",NULL,"Artifact Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Form of the Dragon","R","At the beginning of your upkeep, Form of the Dragon deals 5 damage to target creature or player.
At the beginning of each end step, your life total becomes 5.
Creatures without flying can't attack you.","4RRR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Taoist Mystic","2","2","G","Taoist Mystic can't be blocked by creatures with horsemanship.","2G","Creature - Human Mystic");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coalition Victory","WUBRG","You win the game if you control a land of each basic land type and a creature of each color.","3WUBRG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deathlace","B","Target spell or permanent becomes black. (Mana symbols on that permanent remain unchanged.)","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Topan Ascetic","2","2","G","Tap an untapped creature you control: Topan Ascetic gets +1/+1 until end of turn.","2G","Creature - Human Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Imperial Recruiter","1","1","R","When Imperial Recruiter enters the battlefield, search your library for a creature card with power 2 or less, reveal it, and put it into your hand. Then shuffle your library.","2R","Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampire Bats","0","1","B","Flying (This creature can't be blocked except by creatures with flying or reach.)
{B}: Vampire Bats gets +1/+0 until end of turn. Activate this ability no more than twice each turn.","B","Creature - Bat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Golgari Signet","{1}, {T}: Add {B}{G} to your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krovikan Vampire","3","3","B","At the beginning of each end step, if a creature dealt damage by Krovikan Vampire this turn died, put that card onto the battlefield under your control. Sacrifice it when you lose control of Krovikan Vampire.","3BB","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Death Speakers","1","1","W","Protection from black","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Overeager Apprentice","1","2","B","Discard a card, Sacrifice Overeager Apprentice: Add {B}{B}{B} to your mana pool.","2B","Creature - Human Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwater Shaman","2","2","U","You may cast Aura cards with enchant creature as though they had flash.","2U","Creature - Merfolk Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aegis of Honor","W","{1}: The next time an instant or sorcery spell would deal damage to you this turn, that spell deals that damage to its controller instead.","W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crawlspace","No more than two creatures can attack you each combat.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lava Runner","2","2","R","Haste
Whenever Lava Runner becomes the target of a spell or ability, that spell or ability's controller sacrifices a land.","1RR","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wings of Velis Vel","U","Changeling (This card is every creature type at all times.)
Target creature becomes 4/4, gains all creature types, and gains flying until end of turn.","1U","Tribal Instant - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Words of Worship","W","{1}: The next time you would draw a card this turn, you gain 5 life instead.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grixis Slavedriver","4","4","B","When Grixis Slavedriver leaves the battlefield, put a 2/2 black Zombie creature token onto the battlefield.
Unearth {3}{B} ({3}{B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","5B","Creature - Zombie Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Patriarch's Bidding","B","Each player chooses a creature type. Each player returns all creature cards of a type chosen this way from his or her graveyard to the battlefield.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Tovolar's Magehunter","5","5","Whenever an opponent casts a spell, Tovolar's Magehunter deals 2 damage to that player.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Tovolar's Magehunter.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dauthi Embrace","B","{B}{B}: Target creature gains shadow until end of turn. (It can block or be blocked by only creatures with shadow.)","2B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wheel of Fortune","R","Each player discards his or her hand and draws seven cards.","2R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spirit Shield","You may choose not to untap Spirit Shield during your untap step.
{2}, {T}: Target creature gets +0/+2 for as long as Spirit Shield remains tapped.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talruum Champion","3","3","R","First strike
Whenever Talruum Champion blocks or becomes blocked by a creature, that creature loses first strike until end of turn.","4R","Creature - Minotaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harmonic Sliver","1","1","WG","All Slivers have ""When this permanent enters the battlefield, destroy target artifact or enchantment.""","1GW","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Artful Looter","1","2","U","{T}: Draw a card, then discard a card.
Whenever a permanent comes into play that shares an artist with another permanent you control, untap Artful Looter.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonic Rising","B","At the beginning of your end step, if you control exactly one creature, put a 5/5 black Demon creature token with flying onto the battlefield.","3BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghitu Fire-Eater","2","2","R","{T}, Sacrifice Ghitu Fire-Eater: Ghitu Fire-Eater deals damage equal to its power to target creature or player.","2R","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rekindled Flame","R","Rekindled Flame deals 4 damage to target creature or player.
At the beginning of your upkeep, if an opponent has no cards in hand, you may return Rekindled Flame from your graveyard to your hand.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tezzeret's Gambit","U","({(u/p)} can be paid with either {U} or 2 life.)
Draw two cards, then proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","3(U/P)","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flood Plain","Flood Plain enters the battlefield tapped.
{T}, Sacrifice Flood Plain: Search your library for a Plains or Island card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Act of Treason","R","Gain control of target creature until end of turn. Untap that creature. It gains haste until end of turn. (It can attack and {T} this turn.)","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deft Duelist","2","1","WU","First strike
Shroud (This creature can't be the target of spells or abilities.)","WU","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Suncrusher","3","3","Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it.)
{4}, {T}, Remove a +1/+1 counter from Suncrusher: Destroy target creature.
{2}, Remove a +1/+1 counter from Suncrusher: Return Suncrusher to its owner's hand.","9","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spellbook","You have no maximum hand size.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Toxin Sliver","3","3","B","Whenever a Sliver deals combat damage to a creature, destroy that creature. It can't be regenerated.","3B","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Blow","W","Target creature gains first strike until end of turn.
Draw a card at the beginning of the next turn's upkeep.","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dark Sphere","{T}, Sacrifice Dark Sphere: The next time a source of your choice would deal damage to you this turn, it deals half that damage, rounded up, to you instead.","0","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Leonin Sun Standard","{1}{W}: Creatures you control get +1/+1 until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghor-Clan Bloodscale","2","1","R","First strike
{3}{G}: Ghor-Clan Bloodscale gets +2/+2 until end of turn. Activate this ability only once each turn.","3R","Creature - Viashino Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostly Possession","W","Enchant creature
Enchanted creature has flying.
Prevent all combat damage that would be dealt to and dealt by enchanted creature.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eternal Flame","R","Eternal Flame deals X damage to target opponent, where X is the number of Mountains you control. It deals half X damage, rounded up, to you.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parallel Thoughts","U","When Parallel Thoughts enters the battlefield, search your library for seven cards, exile them in a face-down pile, and shuffle that pile. Then shuffle your library.
If you would draw a card, you may instead put the top card of the pile you exiled into your hand.","3UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Burst","B","Target player discards X cards, where X is one plus the number of cards named Mind Burst in all graveyards.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tandem Lookout","2","1","U","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Tandem Lookout is paired with another creature, each of those creatures has ""Whenever this creature deals damage to an opponent, draw a card.""","2U","Creature - Human Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woolly Thoctar","5","4","WRG",NULL,"RGW","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Shards","WG","Whenever a creature enters the battlefield under your control, you may destroy target artifact or enchantment.","1GW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blanchwood Treefolk","4","5","G",NULL,"4G","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Constable","1","1","U","Whenever Cephalid Constable deals combat damage to a player, return up to that many target permanents that player controls to their owners' hands.","1UU","Creature - Cephalid Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cruel Fate","U","Look at the top five cards of target opponent's library. Put one of those cards into that player's graveyard, and the rest on top of his or her library in any order.","4U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jinx","U","Target land becomes the basic land type of your choice until end of turn.
Draw a card at the beginning of the next turn's upkeep.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Wumpus","6","6","R","Trample
When Shivan Wumpus enters the battlefield, any player may sacrifice a land. If a player does, put Shivan Wumpus on top of its owner's library.","3R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Roil Elemental","3","2","U","Flying
Landfall - Whenever a land enters the battlefield under your control, you may gain control of target creature for as long as you control Roil Elemental.","3UUU","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brushstroke Paintermage","2","3","U","{T}: Target permanent's artist becomes the artist of your choice until end of turn.","3U","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sorrow's Path","{T}: Choose two target blocking creatures an opponent controls. If each of those creatures could block all creatures that the other is blocking, remove both of them from combat. Each one then blocks all creatures the other was blocking.
Whenever Sorrow's Path becomes tapped, it deals 2 damage to you and each creature you control.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Synod Sanctum","{2}, {T}: Exile target permanent you control.
{2}, Sacrifice Synod Sanctum: Return all cards exiled with Synod Sanctum to the battlefield under your control.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swords to Plowshares","W","Exile target creature. Its controller gains life equal to its power.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flameborn Hellion","5","4","R","Haste
Flameborn Hellion attacks each turn if able.","5R","Creature - Hellion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Reap","B","Destroy target nongreen creature. Its controller loses 3 life if you've cast another black spell this turn.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Training Grounds","U","Activated abilities of creatures you control cost up to {2} less to activate. This effect can't reduce the amount of mana an ability costs to activate to less than one mana.","U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Portent","U","Look at the top three cards of target player's library, then put them back in any order. You may have that player shuffle his or her library.
Draw a card at the beginning of the next turn's upkeep.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devastation Tide","U","Return all nonland permanents to their owners' hands.
Miracle {1}{U} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","3UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rushwood Dryad","2","1","G","Forestwalk (This creature is unblockable as long as defending player controls a Forest.)","1G","Creature - Dryad");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pools of Becoming","At the beginning of your end step, put the cards in your hand on the bottom of your library in any order, then draw that many cards.
Whenever you roll {C}, reveal the top three cards of your planar deck. Each of the revealed cards' {C} abilities triggers. Then put the revealed cards on the bottom of your planar deck in any order.",NULL,"Plane - Bolas’s Meditation Realm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Raider","2","1","R","Whenever a land is put into a graveyard from the battlefield, Akki Raider gets +1/+0 until end of turn.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dakkon Blackblade Avatar","You may play any colored card from your hand as a copy of a basic land card chosen at random that can produce mana of one of the card's colors.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Travel Preparations","G","Put a +1/+1 counter on each of up to two target creatures.
Flashback {1}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boil","R","Destroy all Islands.","3R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treefolk Mystic","2","4","G","Whenever Treefolk Mystic blocks or becomes blocked by a creature, destroy all Auras attached to that creature.","3G","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepfire Elemental","4","4","BR","{X}{X}{1}: Destroy target artifact or creature with converted mana cost X.","4BR","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fatal Lore","B","An opponent chooses one - You draw three cards; or you destroy up to two target creatures that opponent controls and that player draws up to three cards. Those creatures can't be regenerated.","2BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Polluted Mire","Polluted Mire enters the battlefield tapped.
{T}: Add {B} to your mana pool.
Cycling {2} ({2}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twitch","U","You may tap or untap target artifact, creature, or land.
Draw a card.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightning Dragon","4","4","R","Flying
Echo {2}{R}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
{R}: Lightning Dragon gets +1/+0 until end of turn.","2RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loyal Cathar","2","2","W","Vigilance
When Loyal Cathar dies, return it to the battlefield transformed under your control at the beginning of the next end step.","WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Manakin","1","1","{T}: Add {1} to your mana pool.","2","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fetid Horror","1","2","B","{B}: Fetid Horror gets +1/+1 until end of turn.","3B","Creature - Shade Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tectonic Fiend","7","7","R","Echo {4}{R}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
Tectonic Fiend attacks each turn if able.","4RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Scrapper","1","1","G","{G}, {T}, Sacrifice Elvish Scrapper: Destroy target artifact.","G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ballynock Trapper","2","2","W","{T}: Tap target creature.
Whenever you cast a white spell, you may untap Ballynock Trapper.","3W","Creature - Kithkin Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Spatula of the Ages","o4, ocT, Sacrifice Spatula of the Ages: Put into play from your hand any card from an Unglued supplement.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hooded Kavu","2","2","R","{B}: Hooded Kavu gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","2R","Creature - Kavu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infest","B","All creatures get -2/-2 until end of turn.","1BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Your Will Is Not Your Own","When you set this scheme in motion, gain control of target creature an opponent controls until end of turn. Untap that creature. It gets +3/+3 and gains haste and trample until end of turn.",NULL,"Scheme");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Otaria","Instant and sorcery cards in graveyards have flashback. The flashback cost is equal to the card's mana cost. (Its owner may cast the card from his or her graveyard for its mana cost. Then he or she exiles it.)
Whenever you roll {C}, take an extra turn after this one.",NULL,"Plane - Dominaria");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fledgling Djinn","2","2","B","Flying
At the beginning of your upkeep, Fledgling Djinn deals 1 damage to you.","1B","Creature - Djinn");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Touchstone","{T}: Tap target artifact you don't control.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Holy Day","W","Prevent all combat damage that would be dealt this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ramosian Commander","2","4","W","{6}, {T}: Search your library for a Rebel permanent card with converted mana cost 5 or less and put it onto the battlefield. Then shuffle your library.","2WW","Creature - Human Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seizures","B","Enchant creature
Whenever enchanted creature becomes tapped, Seizures deals 3 damage to that creature's controller unless that player pays {3}.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunken Field","U","Enchant land
Enchanted land has ""{T}: Counter target spell unless its controller pays {1}.""","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Anoint","W","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Prevent the next 3 damage that would be dealt to target creature this turn.","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Oni of Wild Places Avatar","Creatures you control have haste.
At the beginning of your upkeep, return a creature you control to its owner's hand.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Talon of Pain","Whenever a source you control other than Talon of Pain deals damage to an opponent, put a charge counter on Talon of Pain.
{X}, {T}, Remove X charge counters from Talon of Pain: Talon of Pain deals X damage to target creature or player.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blade-Tribe Berserkers","3","3","R","Metalcraft - When Blade-Tribe Berserkers enters the battlefield, if you control three or more artifacts, Blade-Tribe Berserkers gets +3/+3 and gains haste until end of turn.","3R","Creature - Human Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Detritivore","*","*","R","Detritivore's power and toughness are each equal to the number of nonbasic land cards in your opponents' graveyards.
Suspend X-{X}{3}{R}. X can't be 0. (Rather than cast this card from your hand, you may pay {X}{3}{R} and exile it with X time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Whenever a time counter is removed from Detritivore while it's exiled, destroy target nonbasic land.","2RR","Creature - Lhurgoyf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Greatheart","2","1","W","As long as you control a Giant, Kithkin Greatheart gets +1/+1 and has first strike.","1W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Demolition Team","1","1","R","{T}: Destroy target Wall.","2R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyromatics","R","Replicate {1}{R} (When you cast this spell, copy it for each time you paid its replicate cost. You may choose new targets for the copies.)
Pyromatics deals 1 damage to target creature or player.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Attunement","U","Return Attunement to its owner's hand: Draw three cards, then discard four cards.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Choke","G","Islands don't untap during their controllers' untap steps.","2G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opal Champion","W","When an opponent casts a creature spell, if Opal Champion is an enchantment, Opal Champion becomes a 3/3 Knight creature with first strike.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steeling Stance","W","Creatures you control get +1/+1 until end of turn.
Forecast - {W}, Reveal Steeling Stance from your hand: Target creature gets +1/+1 until end of turn. (Activate this ability only during your upkeep and only once each turn.)","1WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindshrieker","1","1","U","Flying
{2}: Target player puts the top card of his or her library into his or her graveyard. Mindshrieker gets +X/+X until end of turn, where X is that card's converted mana cost.","1U","Creature - Spirit Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Maze of Ith","{T}: Untap target attacking creature. Prevent all combat damage that would be dealt to and dealt by that creature this turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mausoleum Turnkey","3","2","B","When Mausoleum Turnkey enters the battlefield, return target creature card of an opponent's choice from your graveyard to your hand.","3B","Creature - Ogre Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teneb, the Harvester","6","6","WBG","Flying
Whenever Teneb, the Harvester deals combat damage to a player, you may pay {2}{B}. If you do, put target creature card from a graveyard onto the battlefield under your control.","3BGW","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deus of Calamity","6","6","RG","Trample
Whenever Deus of Calamity deals 6 or more damage to an opponent, destroy target land that player controls.","(R/G)(R/G)(R/G)(R/G)(R/G)","Creature - Spirit Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ray of Erasure","U","Target player puts the top card of his or her library into his or her graveyard.
Draw a card at the beginning of the next turn's upkeep.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cradle of Vitality","W","Whenever you gain life, you may pay {1}{W}. If you do, put a +1/+1 counter on target creature for each 1 life you gained.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Words of War","R","{1}: The next time you would draw a card this turn, Words of War deals 2 damage to target creature or player instead.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Puncture Blast","R","Wither (This deals damage to creatures in the form of -1/-1 counters.)
Puncture Blast deals 3 damage to target creature or player.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pooling Venom","B","Enchant land
Whenever enchanted land becomes tapped, its controller loses 2 life.
{3}{B}: Destroy enchanted land.","1B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Howltooth Hollow","Hideaway (This land enters the battlefield tapped. When it does, look at the top four cards of your library, exile one face down, then put the rest on the bottom of your library.)
{T}: Add {B} to your mana pool.
{B}, {T}: You may play the exiled card without paying its mana cost if each player has no cards in hand.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mental Misstep","U","({(u/p)} can be paid with either {U} or 2 life.)
Counter target spell with converted mana cost 1.","(U/P)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudchaser Kestrel","2","2","W","Flying
When Cloudchaser Kestrel enters the battlefield, destroy target enchantment.
{W}: Target permanent becomes white until end of turn.","1WW","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Induce Paranoia","U","Counter target spell. If {B} was spent to cast Induce Paranoia, that spell's controller puts the top X cards of his or her library into his or her graveyard, where X is the spell's converted mana cost.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Joraga Warcaller","1","1","G","Multikicker {1}{G} (You may pay an additional {1}{G} any number of times as you cast this spell.)
Joraga Warcaller enters the battlefield with a +1/+1 counter on it for each time it was kicked.
Other Elf creatures you control get +1/+1 for each +1/+1 counter on Joraga Warcaller.","G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Elemental","5","4","B","Protection from white
At the beginning of your upkeep, sacrifice Bog Elemental unless you sacrifice a land.","3BB","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("March of the Machines","U","Each noncreature artifact is an artifact creature with power and toughness each equal to its converted mana cost. (Equipment that's a creature can't equip a creature.)","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blockade Runner","2","2","U","{U}: Blockade Runner is unblockable this turn.","3U","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormtide Leviathan","8","8","U","Islandwalk (This creature is unblockable as long as defending player controls an Island.)
All lands are Islands in addition to their other types.
Creatures without flying or islandwalk can't attack.","5UUU","Creature - Leviathan");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enter the Dungeon","B","Players play a Magic subgame under the table starting at 5 life, using their libraries as their decks. After the subgame ends, the winner searches his or her library for two cards, puts those cards into his or her hand, then shuffles his or her library.","BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Suq'Ata Firewalker","0","1","U","Suq'Ata Firewalker can't be the target of red spells or abilities from red sources.
{T}: Suq'Ata Firewalker deals 1 damage to target creature or player.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Joraga Treespeaker","1","1","G","Level up {1}{G} ({1}{G}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-4
1/2
{T}: Add {G}{G} to your mana pool.
LEVEL 5+
1/4
Elves you control have ""{T}: Add {G}{G} to your mana pool.""","G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorger Wurm","5","5","RG","Devour 1 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with that many +1/+1 counters on it.)","3RG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barrage Ogre","3","3","R","{T}, Sacrifice an artifact: Barrage Ogre deals 2 damage to target creature or player.","3RR","Creature - Ogre Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mangara's Blessing","W","You gain 5 life.
When a spell or ability an opponent controls causes you to discard Mangara's Blessing, you gain 2 life, and you return Mangara's Blessing from your graveyard to your hand at the beginning of the next end step.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of New Alara","2","2","WG","Each other multicolored creature you control gets +1/+1 for each of its colors.","2GW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Shapeshifter","3","3","U","{2}{U}{U}, Sacrifice Riptide Shapeshifter: Choose a creature type. Reveal cards from the top of your library until you reveal a creature card of that type. Put that card onto the battlefield and shuffle the rest into your library.","3UU","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invert the Skies","UG","Creatures your opponents control lose flying until end of turn if {G} was spent to cast Invert the Skies, and creatures you control gain flying until end of turn if {U} was spent to cast it. (Do both if {G}{U} was spent.)","3(G/U)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ertai's Trickery","U","Counter target spell if it was kicked.","U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cauldron of Souls","{T}: Choose any number of target creatures. Each of those creatures gains persist until end of turn. (When it dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","5","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Coretapper","1","1","{T}: Put a charge counter on target artifact.
Sacrifice Coretapper: Put two charge counters on target artifact.","2","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invoke the Firemind","UR","Choose one - Draw X cards; or Invoke the Firemind deals X damage to target creature or player.","XUUR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fruition","G","You gain 1 life for each Forest on the battlefield.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Suleiman's Legacy","WR","When Suleiman's Legacy enters the battlefield, destroy all Djinns and Efreets. They can't be regenerated.
Whenever a Djinn or Efreet enters the battlefield, destroy it. It can't be regenerated.","RW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Scout","1","1","W","{1}, {T}: Target creature gains first strike until end of turn.","W","Creature - Human Soldier Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Monsoon","RG","At the beginning of each player's end step, tap all untapped Islands that player controls and Monsoon deals X damage to the player, where X is the number of Islands tapped this way.","2RG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Smolder Initiate","1","1","B","Whenever a player casts a black spell, you may pay {1}. If you do, target player loses 1 life.","B","Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volrath's Curse","U","Enchant creature
Enchanted creature can't attack or block, and its activated abilities can't be activated. That creature's controller may sacrifice a permanent for that player to ignore this effect until end of turn.
{1}{U}: Return Volrath's Curse to its owner's hand.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Abuna's Chant","W","Choose one - You gain 5 life; or prevent the next 5 damage that would be dealt to target creature this turn.
Entwine {2} (Choose both if you pay the entwine cost.)","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chicken Egg","0","1","R","During your upkeep, roll a six-sided die. On a 6, sacrifice Chicken Egg and put a Giant Chicken token into play. Treat this token as a 4/4 red creature that counts as a Chicken.","1R","Summon - Egg");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spectral Shift","U","Choose one - Change the text of target spell or permanent by replacing all instances of one basic land type with another; or change the text of target spell or permanent by replacing all instances of one color word with another. (These effects last indefinitely.)
Entwine {2} (Choose both if you pay the entwine cost.)","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sapphire Leech","2","2","U","Flying
Blue spells you cast cost {U} more to cast.","1U","Creature - Leech");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gaea's Revenge","8","5","G","Gaea's Revenge can't be countered.
Haste
Gaea's Revenge can't be the target of nongreen spells or abilities from nongreen sources.","5GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glimpse the Unthinkable","UB","Target player puts the top ten cards of his or her library into his or her graveyard.","UB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magnigoth Treefolk","2","6","G","Domain - For each basic land type among lands you control, Magnigoth Treefolk has landwalk of that type.","4G","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lace with Moonglove","G","Target creature gains deathtouch until end of turn. (Any amount of damage it deals to a creature is enough to destroy that creature.)
Draw a card.","2G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mayael the Anima Avatar","At the beginning of your upkeep, reveal the top card of your library. If it's a creature card with power 5 or greater, put it into your hand. Otherwise, you may put it on the bottom of your library.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lady Evangela","1","2","WUB","{W}{B}, {T}: Prevent all combat damage that would be dealt by target creature this turn.","WUB","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ebony Treefolk","3","3","BG","{B}{G}: Ebony Treefolk gets +1/+1 until end of turn.","1BG","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scorching Lava","R","Kicker {R} (You may pay an additional {R} as you cast this spell.)
Scorching Lava deals 2 damage to target creature or player. If Scorching Lava was kicked, that creature can't be regenerated this turn and if it would die this turn, exile it instead.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treasure Hunter","2","2","W","When Treasure Hunter enters the battlefield, you may return target artifact card from your graveyard to your hand.","2W","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of Cliffhaven","2","2","W","Level up {3} ({3}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-3
2/3
Flying
LEVEL 4+
4/4
Flying, vigilance","1W","Creature - Kor Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kamahl, Pit Fighter","6","1","R","Haste (This creature can attack and {T} as soon as it comes under your control.)
{T}: Kamahl, Pit Fighter deals 3 damage to target creature or player.","4RR","Legendary Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Martyr of Frost","1","1","U","{2}, Reveal X blue cards from your hand, Sacrifice Martyr of Frost: Counter target spell unless its controller pays {X}.","U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Briar Patch","G","Whenever a creature attacks you, it gets -1/-0 until end of turn.","1GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nulltread Gargantuan","5","6","UG","When Nulltread Gargantuan enters the battlefield, put a creature you control on top of its owner's library.","1GU","Creature - Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Metallic Sliver","1","1",NULL,"1","Artifact Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teller of Tales","3","3","U","Flying
Whenever you cast a Spirit or Arcane spell, you may tap or untap target creature.","3UU","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Faithless Looting","R","Draw two cards, then discard two cards.
Flashback {2}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","R","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Chrome Steed","2","2","Metalcraft - Chrome Steed gets +2/+2 as long as you control three or more artifacts.","4","Artifact Creature - Horse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Blastminer","1","1","R","{2}{R}, {T}: Destroy target nonbasic land.
Morph {R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1R","Creature - Dwarf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alert Shu Infantry","2","2","W","Vigilance","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Kelp","0","3","U","Defender (This creature can't attack.)
{U}{U}, {T}: Put a 0/1 blue Plant Wall creature token with defender named Kelp onto the battlefield.","UU","Creature - Plant Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Judge Unworthy","W","Choose target attacking or blocking creature. Scry 3, then reveal the top card of your library. Judge Unworthy deals damage equal to that card's converted mana cost to that creature. (To scry 3, look at the top three cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Konda's Banner","Konda's Banner can be attached only to a legendary creature.
Creatures that share a color with equipped creature get +1/+1.
Creatures that share a creature type with equipped creature get +1/+1.
Equip {2}","2","Legendary Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quilled Sliver","1","1","W","All Slivers have ""{T}: This permanent deals 1 damage to target attacking or blocking creature.""","1W","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fate Transfer","UB","Move all counters from target creature onto another target creature.","1(U/B)","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cursed Rack","As Cursed Rack enters the battlefield, choose an opponent.
The chosen player's maximum hand size is four.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Time Warp","U","Target player takes an extra turn after this one.","3UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Murasa","Whenever a nontoken creature enters the battlefield, its controller may search his or her library for a basic land card, put it onto the battlefield tapped, then shuffle his or her library.
Whenever you roll {C}, target land becomes a 4/4 creature that's still a land.",NULL,"Plane - Zendikar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shahrazad","W","Players play a MAGIC subgame, using their libraries as their decks. Each player who doesn't win the subgame loses half his or her life, rounded up.","WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Fateshaper","4","5","U","Flying
When Aven Fateshaper enters the battlefield, look at the top four cards of your library, then put them back in any order.
{4}{U}: Look at the top four cards of your library, then put them back in any order.","6U","Creature - Bird Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Terror","B","Destroy target nonartifact, nonblack creature. It can't be regenerated.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Orim's Cure","W","If you control a Plains, you may tap an untapped creature you control rather than pay Orim's Cure's mana cost.
Prevent the next 4 damage that would be dealt to target creature or player this turn.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wandering Goblins","0","3","R","Domain - {3}: Wandering Goblins gets +1/+0 until end of turn for each basic land type among lands you control.","2R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screeching Silcaw","1","2","U","Flying
Metalcraft - Whenever Screeching Silcaw deals combat damage to a player, if you control three or more artifacts, that player puts the top four cards of his or her library into his or her graveyard.","1U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alabaster Wall","0","4","W","Defender (This creature can't attack.)
{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","2W","Creature - Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Semblance Anvil","Imprint - When Semblance Anvil enters the battlefield, you may exile a nonland card from your hand.
Spells you cast that share a card type with the exiled card cost {2} less to cast.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Powerleech","G","Whenever an artifact an opponent controls becomes tapped or an opponent activates an artifact's ability without {T} in its activation cost, you gain 1 life.","GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thunder Strike","R","Target creature gets +2/+0 and gains first strike until end of turn. (It deals combat damage before creatures without first strike.)","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Durkwood Boars","4","4","G",NULL,"4G","Creature - Boar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cerebral Eruption","R","Target opponent reveals the top card of his or her library. Cerebral Eruption deals damage equal to the revealed card's converted mana cost to that player and each creature he or she controls. If a land card is revealed this way, return Cerebral Eruption to its owner's hand.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flooded Woodlands","UB","Green creatures can't attack unless their controller sacrifices a land for each green creature he or she controls that's attacking.","2UB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Celestial Purge","W","Exile target black or red permanent.","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stomping Ground","({T}: Add {R} or {G} to your mana pool.)
As Stomping Ground enters the battlefield, you may pay 2 life. If you don't, Stomping Ground enters the battlefield tapped.",NULL,"Land - Mountain Forest");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lorthos, the Tidemaker","8","8","U","Whenever Lorthos, the Tidemaker attacks, you may pay {8}. If you do, tap up to eight target permanents. Those permanents don't untap during their controllers' next untap steps.","5UUU","Legendary Creature - Octopus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Delaying Shield","W","If damage would be dealt to you, put that many delay counters on Delaying Shield instead.
At the beginning of your upkeep, remove all delay counters from Delaying Shield. For each delay counter removed this way, you lose 1 life unless you pay {1}{W}.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Votary of the Conclave","1","1","W","{2}{G}: Regenerate Votary of the Conclave.","W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Drain","U","Counter target spell. At the beginning of your next main phase, add {X} to your mana pool, where X is that spell's converted mana cost.","UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Private Research","U","Enchant creature
At the beginning of your upkeep, you may put a page counter on Private Research.
When enchanted creature dies, draw a card for each page counter on Private Research.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Esper Stormblade","2","1","WUB","As long as you control another multicolored permanent, Esper Stormblade gets +1/+1 and has flying.","(W/B)U","Artifact Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silkenfist Order","3","5","W","Whenever Silkenfist Order becomes blocked, untap it.","3WW","Creature - Kor Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sigil of Sleep","U","Enchant creature
Whenever enchanted creature deals damage to a player, return target creature that player controls to its owner's hand.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ravenous Rats","1","1","B","When Ravenous Rats enters the battlefield, target opponent discards a card.","1B","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Starstorm","R","Starstorm deals X damage to each creature.
Cycling {3} ({3}, Discard this card: Draw a card.)","XRR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Melee","R","Cast Melee only during your turn and only during combat before blockers are declared.
You choose which creatures block this combat and how those creatures block.
Whenever a creature attacks and isn't blocked this combat, untap it and remove it from combat.","4R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Searing Wind","R","Searing Wind deals 10 damage to target creature or player.","8R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Oathkeeper, Takeno's Daisho","Equipped creature gets +3/+1.
Whenever equipped creature dies, return that card to the battlefield under your control if it's a Samurai card.
When Oathkeeper, Takeno's Daisho is put into a graveyard from the battlefield, exile equipped creature.
Equip {2}","3","Legendary Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("General's Kabuto","Equipped creature has shroud. (It can't be the target of spells or abilities.)
Prevent all combat damage that would be dealt to equipped creature.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","4","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Hot Tub","{2}, Discard a card: Search your library for a card that shares a complete word in its name with the discarded card, reveal it, and put it into your hand. Then shuffle your library.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vorstclaw","7","7","G",NULL,"4GG","Creature - Elemental Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stone Giant","3","4","R","{T}: Target creature you control with toughness less than Stone Giant's power gains flying until end of turn. Destroy that creature at the beginning of the next end step.","2RR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fungus Elemental","3","3","G","{G}, Sacrifice a Forest: Put a +2/+2 counter on Fungus Elemental. Activate this ability only if Fungus Elemental entered the battlefield this turn.","3G","Creature - Fungus Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lost Hours","B","Target player reveals his or her hand. You choose a nonland card from it. That player puts that card into his or her library third from the top.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Selfless Exorcist","3","4","W","{T}: Exile target creature card from a graveyard. That card deals damage equal to its power to Selfless Exorcist.","3WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stratozeppelid","4","4","U","Flying
Stratozeppelid can block only creatures with flying.","4U","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brion Stoutarm","4","4","WR","Lifelink
{R}, {T}, Sacrifice a creature other than Brion Stoutarm: Brion Stoutarm deals damage equal to the sacrificed creature's power to target player.","2RW","Legendary Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Civilized Scholar","0","1","U","{T}: Draw a card, then discard a card. If a creature card is discarded this way, untap Civilized Scholar, then transform it.","2U","Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Watcher Sliver","2","2","W","All Sliver creatures get +0/+2.","3W","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Boros Signet","{1}, {T}: Add {R}{W} to your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Root Elemental","6","5","G","Morph {5}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Root Elemental is turned face up, you may put a creature card from your hand onto the battlefield.","4GG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hammer Mage","1","1","R","{X}{R}, {T}, Discard a card: Destroy all artifacts with converted mana cost X or less.","1R","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindwhip Sliver","2","2","B","All Slivers have ""{2}, Sacrifice this permanent: Target player discards a card at random. Activate this ability only any time you could cast a sorcery.""","2B","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bronze Calendar","Your spells cost o1 less to play as long as you speak in a voice other than your normal voice.
If you speak in your normal voice, sacrifice Bronze Calendar.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eradicate","B","Exile target nonblack creature. Search its controller's graveyard, hand, and library for all cards with the same name as that creature and exile them. Then that player shuffles his or her library.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Mercy","3","3","W","Flying
When Angel of Mercy enters the battlefield, you gain 3 life.","4W","Creature - Angel");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Kiln Walker","0","3","Whenever Kiln Walker attacks, it gets +3/+0 until end of turn.","3","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Niblis of the Urn","1","1","W","Flying
Whenever Niblis of the Urn attacks, you may tap target creature.","1W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Savage Conception","G","Put a 3/3 green Beast creature token onto the battlefield.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","3GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thieves' Fortune","U","Prowl {U} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Rogue.)
Look at the top four cards of your library. Put one of them into your hand and the rest on the bottom of your library in any order.","2U","Tribal Instant - Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Piston Sledge","When Piston Sledge enters the battlefield, attach it to target creature you control.
Equipped creature gets +3/+1.
Equip-Sacrifice an artifact.","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunting Wilds","G","Kicker {3}{G} (You may pay an additional {3}{G} as you cast this spell.)
Search your library for up to two Forest cards and put them onto the battlefield tapped. Then shuffle your library.
If Hunting Wilds was kicked, untap all Forests put onto the battlefield this way. They become 3/3 green creatures with haste that are still lands.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bone Splinters","B","As an additional cost to cast Bone Splinters, sacrifice a creature.
Destroy target creature.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cosmic Horror","7","7","B","First strike
At the beginning of your upkeep, destroy Cosmic Horror unless you pay {3}{B}{B}{B}. If Cosmic Horror is destroyed this way, it deals 7 damage to you.","3BBB","Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("R&D's Secret Lair","Play cards as written. Ignore all errata.
{T}: Add {1} to your mana pool.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arctic Wolves","4","5","G","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When Arctic Wolves enters the battlefield, draw a card.","3GG","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Mimeomancer","3","1","WU","Flying
At the beginning of your upkeep, you may put a feather counter on target creature. If you do, that creature is 3/1 and has flying for as long as it has a feather counter on it.","1WU","Creature - Bird Wizard");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Ajani Goldmane","W","+1: You gain 2 life.
-1: Put a +1/+1 counter on each creature you control. Those creatures gain vigilance until end of turn.
-6: Put a white Avatar creature token onto the battlefield. It has ""This creature's power and toughness are each equal to your life total.""","Planeswalker - Ajani","2WW");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Myr Reservoir","{T}: Add {2} to your mana pool. Spend this mana only to cast Myr spells or activate abilities of Myr.
{3}, {T}: Return target Myr card from your graveyard to your hand.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("S.N.O.T.","*{^2}","*{^2}","G","As S.N.O.T. comes into play, you may stick it onto another creature named S.N.O.T. in play. If you do, all those creatures form a single creature.
S.N.O.T.'s power and toughness are equal to the square of the number of S.N.O.T.s stuck together. (One is a 1/1, two are a 4/4, three are a 9/9, and four are a 16/16.)","G","Creature - Ooze");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silverblade Paladin","2","2","W","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Silverblade Paladin is paired with another creature, both creatures have double strike.","1WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last-Ditch Effort","R","Sacrifice any number of creatures. Last-Ditch Effort deals that much damage to target creature or player.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonybrook Banneret","1","1","U","Islandwalk
Merfolk spells and Wizard spells you cast cost {1} less to cast.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Phalanx","2","4","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","4W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunflare Shaman","2","1","R","{1}{R}, {T}: Sunflare Shaman deals X damage to target creature or player and X damage to itself, where X is the number of Elemental cards in your graveyard.","1R","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inner-Flame Igniter","2","2","R","{2}{R}: Creatures you control get +1/+0 until end of turn. If this is the third time this ability has resolved this turn, creatures you control gain first strike until end of turn.","2R","Creature - Elemental Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashnod's Battle Gear","You may choose not to untap Ashnod's Battle Gear during your untap step.
{2}, {T}: Target creature you control gets +2/-2 for as long as Ashnod's Battle Gear remains tapped.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Simic Guildmage","2","2","UG","({(g/u)} can be paid with either {G} or {U}.)
{1}{G}: Move a +1/+1 counter from target creature onto another target creature with the same controller.
{1}{U}: Attach target Aura enchanting a permanent to another permanent with the same controller.","(G/U)(G/U)","Creature - Elf Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunweb","5","6","W","Defender (This creature can't attack.)
Flying
Sunweb can't block creatures with power 2 or less.","3W","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Enforcer","4","4","R","Ogre Enforcer can't be destroyed by lethal damage unless lethal damage dealt by a single source is marked on it.","3RR","Creature - Ogre");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Suspicions","B","At the beginning of each opponent's upkeep, that player loses X life, where X is the number of cards in that player's hand minus the number of cards in your hand.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maelstrom Archangel","5","5","WUBRG","Flying
Whenever Maelstrom Archangel deals combat damage to a player, you may cast a nonland card from your hand without paying its mana cost.","WUBRG","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maro","*","*","G","Maro's power and toughness are each equal to the number of cards in your hand.","2GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eel Umbra","U","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature gets +1/+1.
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voracious Dragon","4","4","R","Flying
Devour 1 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with that many +1/+1 counters on it.)
When Voracious Dragon enters the battlefield, it deals damage to target creature or player equal to twice the number of Goblins it devoured.","3RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Canyon Drake","1","2","R","Flying
{1}, Discard a card at random: Canyon Drake gets +2/+0 until end of turn.","2RR","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conflux","WUBRG","Search your library for a white card, a blue card, a black card, a red card, and a green card. Reveal those cards and put them into your hand. Then shuffle your library.","3WUBRG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nessian Courser","3","3","G",NULL,"2G","Creature - Centaur Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Expunge","B","Destroy target nonartifact, nonblack creature. It can't be regenerated.
Cycling {2} ({2}, Discard this card: Draw a card.)","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fallen Ideal","B","Enchant creature
Enchanted creature has flying and ""Sacrifice a creature: This creature gets +2/+1 until end of turn.""
When Fallen Ideal is put into a graveyard from the battlefield, return Fallen Ideal to its owner's hand.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Koth of the Hammer","R","+1: Untap target Mountain. It becomes a 4/4 red Elemental creature until end of turn. It's still a land.
-2: Add {R} to your mana pool for each Mountain you control.
-5: You get an emblem with ""Mountains you control have ‘{T}: This land deals 1 damage to target creature or player.'""","Planeswalker - Koth","2RR");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Explore","G","You may play an additional land this turn.
Draw a card.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bituminous Blast","BR","Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)
Bituminous Blast deals 4 damage to target creature.","3BR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Heat","2","6","R","Defender (This creature can't attack.)","2R","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Assassin","1","2","U","{T}: Destroy target creature with islandwalk.","UU","Creature - Merfolk Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scavenger Folk","1","1","G","{G}, {T}, Sacrifice Scavenger Folk: Destroy target artifact.","G","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stitch Together","B","Return target creature card from your graveyard to your hand.
Threshold - Return that card from your graveyard to the battlefield instead if seven or more cards are in your graveyard.","BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merrow Levitator","2","3","U","{T}: Target creature gains flying until end of turn.
Whenever you cast a blue spell, you may untap Merrow Levitator.","3U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sun's Bounty","W","You gain 4 life.
Recover {1}{W} (When a creature is put into your graveyard from the battlefield, you may pay {1}{W}. If you do, return this card from your graveyard to your hand. Otherwise, exile this card.)","1W","Instant");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Elspeth Tirel","W","+2: You gain 1 life for each creature you control.
-2: Put three 1/1 white Soldier creature tokens onto the battlefield.
-5: Destroy all other permanents except for lands and tokens.","Planeswalker - Elspeth","3WW");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lifetap","U","Whenever a Forest an opponent controls becomes tapped, you gain 1 life.","UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Confusion in the Ranks","R","Whenever an artifact, creature, or enchantment enters the battlefield, its controller chooses target permanent another player controls that shares a card type with it. Exchange control of those permanents.","3RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonybrook Angler","1","2","U","{1}{U}, {T}: You may tap or untap target creature.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Captured Sunlight","WG","Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)
You gain 4 life.","2GW","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Roots of All Evil","When you set this scheme in motion, put five 1/1 green Saproling creature tokens onto the battlefield.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swelter","R","Swelter deals 2 damage to each of two target creatures.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Tyrant","5","5","UBR","Flying, trample
At the beginning of your upkeep, each player loses 1 life. Put a +1/+1 counter on Blood Tyrant for each 1 life lost this way.
Whenever a player loses the game, put five +1/+1 counters on Blood Tyrant.","4UBR","Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alter Reality","U","Change the text of target spell or permanent by replacing all instances of one color word with another. (This effect lasts indefinitely.)
Flashback {1}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ransack","U","Look at the top five cards of target player's library. Put any number of them on the bottom of that library in any order and the rest on top of the library in any order.","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Nissa Revane","G","+1: Search your library for a card named Nissa's Chosen and put it onto the battlefield. Then shuffle your library.
+1: You gain 2 life for each Elf you control.
-7: Search your library for any number of Elf creature cards and put them onto the battlefield. Then shuffle your library.","Planeswalker - Nissa","2GG");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Titanic Bulvox","7","4","G","Trample
Morph {4}{G}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","6GG","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sand Silos","Sand Silos enters the battlefield tapped.
You may choose not to untap Sand Silos during your untap step.
At the beginning of your upkeep, if Sand Silos is tapped, put a storage counter on it.
{T}, Remove any number of storage counters from Sand Silos: Add {U} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nezumi Bone-Reader","1","1","B","{B}, Sacrifice a creature: Target player discards a card. Activate this ability only any time you could cast a sorcery.","1B","Creature - Rat Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Tower","{T}: Add {1} to your mana pool. If you control an Urza's Mine and an Urza's Power-Plant, add {3} to your mana pool instead.",NULL,"Land - Urza’s Tower");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kaervek's Purge","BR","Destroy target creature with converted mana cost X. If that creature dies this way, Kaervek's Purge deals damage equal to the creature's power to the creature's controller.","XBR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Arcane Spyglass","{2}, {T}, Sacrifice a land: Draw a card and put a charge counter on Arcane Spyglass.
Remove three charge counters from Arcane Spyglass: Draw a card.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saltskitter","3","4","W","Whenever another creature enters the battlefield, exile Saltskitter. Return Saltskitter to the battlefield under its owner's control at the beginning of the next end step.","3W","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Toil to Renown","G","You gain 1 life for each tapped artifact, creature, and land you control.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Vipers","2","4","B","Defender (This creature can't attack.)
{3}: Destroy Wall of Vipers and target creature it's blocking. Any player may activate this ability.","2B","Creature - Snake Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jade Monolith","{1}: The next time a source of your choice would deal damage to target creature this turn, that source deals that damage to you instead.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ichor Wellspring","When Ichor Wellspring enters the battlefield or is put into a graveyard from the battlefield, draw a card.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Spider","2","4","G","Reach (This creature can block creatures with flying.)","3G","Creature - Spider");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thunderbolt","R","Choose one - Thunderbolt deals 3 damage to target player; or Thunderbolt deals 4 damage to target creature with flying.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Doomsday Specter","2","3","UB","Flying
When Doomsday Specter enters the battlefield, return a blue or black creature you control to its owner's hand.
Whenever Doomsday Specter deals combat damage to a player, look at that player's hand and choose a card from it. The player discards that card.","2UB","Creature - Specter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wooden Sphere","Whenever a player casts a green spell, you may pay {1}. If you do, you gain 1 life.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kitesail","Equipped creature gets +1/+0 and has flying.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Havenwood Battleground","Havenwood Battleground enters the battlefield tapped.
{T}: Add {G} to your mana pool.
{T}, Sacrifice Havenwood Battleground: Add {G}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Web","G","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature gets +0/+2 and has reach. (It can block creatures with flying.)","G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyroblast","R","Choose one - Counter target spell if it's blue; or destroy target permanent if it's blue.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Command Tower","{T}: Add to your mana pool one mana of any color in your commander's color identity.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Snowcrusher","6","5","Phyrexian Snowcrusher attacks each turn if able.
{1}{S}i}: Phyrexian Snowcrusher gets +1/+0 until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","6","Snow Artifact Creature - Juggernaut");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloudheath Drake","3","3","U","Flying
{1}{W}: Cloudheath Drake gains vigilance until end of turn.","4U","Artifact Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Village Elder","1","1","G","{G}, {T}, Sacrifice a Forest: Regenerate target creature.","G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Watchdog","1","2","Watchdog blocks each turn if able.
As long as Watchdog is untapped, all creatures attacking you get -1/-0.","3","Artifact Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arashi, the Sky Asunder","5","5","G","{X}{G}, {T}: Arashi, the Sky Asunder deals X damage to target creature with flying.
Channel - {X}{G}{G}, Discard Arashi: Arashi deals X damage to each creature with flying.","3GG","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cold Snap","W","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of each player's upkeep, Cold Snap deals damage to that player equal to the number of snow lands he or she controls.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Peace","W","Target player skips all combat phases of his or her next turn.","W","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mystifying Maze","{T}: Add {1} to your mana pool.
{4}, {T}: Exile target attacking creature an opponent controls. At the beginning of the next end step, return it to the battlefield tapped under its owner's control.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Neurok Prodigy","2","1","U","Flying
Discard an artifact card: Return Neurok Prodigy to its owner's hand.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Tears","0","4","U","Defender (This creature can't attack.)
Whenever Wall of Tears blocks a creature, return that creature to its owner's hand at end of combat.","1U","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Genju of the Falls","U","Enchant Island
{2}: Enchanted Island becomes a 3/2 blue Spirit creature with flying until end of turn. It's still a land.
When enchanted Island is put into a graveyard, you may return Genju of the Falls from your graveyard to your hand.","U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Frenzy","R","Cast Blood Frenzy only before the combat damage step.
Target attacking or blocking creature gets +4/+0 until end of turn. Destroy that creature at the beginning of the next end step.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of the Paruns","As long as equipped creature is tapped, tapped creatures you control get +2/+0.
As long as equipped creature is untapped, untapped creatures you control get +0/+2.
{3}: You may tap or untap equipped creature.
Equip {3}","4","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gliding Licid","2","2","U","{U}, {T}: Gliding Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {U} to end this effect.
Enchanted creature has flying.","2U","Creature - Licid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Persuasion","U","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
You control enchanted creature.","3UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battle-Rattle Shaman","2","2","R","At the beginning of combat on your turn, you may have target creature get +2/+0 until end of turn.","3R","Creature - Goblin Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Alchor's Tomb","{2}, {T}: Target permanent you control becomes the color of your choice. (This effect lasts indefinitely.)","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fireshrieker","Equipped creature has double strike. (It deals both first-strike and regular combat damage.)
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fireblast","R","You may sacrifice two Mountains rather than pay Fireblast's mana cost.
Fireblast deals 4 damage to target creature or player.","4RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skeletal Grimace","B","Enchant creature
Enchanted creature gets +1/+1 and has ""{B}: Regenerate this creature.""","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Outrider of Jhess","2","2","U","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","3U","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloistered Youth","1","1","W","At the beginning of your upkeep, you may transform Cloistered Youth.","1W","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unfulfilled Desires","UB","{1}, Pay 1 life: Draw a card, then discard a card.","1UB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nettletooth Djinn","4","4","G","At the beginning of your upkeep, Nettletooth Djinn deals 1 damage to you.","3G","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Browbeat","R","Any player may have Browbeat deal 5 damage to him or her. If no one does, target player draws three cards.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Army of the Damned","B","Put thirteen 2/2 black Zombie creature tokens onto the battlefield tapped.
Flashback {7}{B}{B}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","5BBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aerie Mystics","3","3","W","Flying
{1}{G}{U}: Creatures you control gain shroud until end of turn.","4W","Creature - Bird Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stunted Growth","G","Target player chooses three cards from his or her hand and puts them on top of his or her library in any order.","3GG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Not of This World","Counter target spell or ability that targets a permanent you control.
Not of This World costs {7} less to cast if it targets a spell or ability that targets a creature you control with power 7 or greater.","7","Tribal Instant - Eldrazi");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mishra's Bauble","{T}, Sacrifice Mishra's Bauble: Look at the top card of target player's library. Draw a card at the beginning of the next turn's upkeep.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darklit Gargoyle","1","2","W","Flying
{B}: Darklit Gargoyle gets +2/-1 until end of turn.","1W","Artifact Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hermit Druid","1","1","G","{G}, {T}: Reveal cards from the top of your library until you reveal a basic land card. Put that card into your hand and all other cards revealed this way into your graveyard.","1G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashen-Skin Zubera","1","2","B","When Ashen-Skin Zubera dies, target opponent discards a card for each Zubera that died this turn.","1B","Creature - Zubera Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Magnetic Web","If a creature with a magnet counter on it attacks, all creatures with magnet counters on them attack if able.
Whenever a creature with a magnet counter on it attacks, all creatures with magnet counters on them block that creature this turn if able.
{1}, {T}: Put a magnet counter on target creature.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Blueprints","Echo {6} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
{T}: Draw a card.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Endangered Armodon","4","5","G","When you control a creature with toughness 2 or less, sacrifice Endangered Armodon.","2GG","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fledgling Griffin","2","2","W","Landfall - Whenever a land enters the battlefield under your control, Fledgling Griffin gains flying until end of turn.","1W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Xenograft","U","As Xenograft enters the battlefield, choose a creature type.
Each creature you control is the chosen type in addition to its other types.","4U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wirewood Channeler","2","2","G","{T}: Add X mana of any one color to your mana pool, where X is the number of Elves on the battlefield.","3G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Safe Passage","W","Prevent all damage that would be dealt to you and creatures you control this turn.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corpse Traders","3","3","B","{2}{B}, Sacrifice a creature: Target opponent reveals his or her hand. You choose a card from it. That player discards that card. Activate this ability only any time you could cast a sorcery.","3B","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chameleon Blur","G","Prevent all damage that creatures would deal to players this turn.","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("King Crab","4","5","U","{1}{U}, {T}: Put target green creature on top of its owner's library.","4UU","Creature - Crab");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream Leash","U","Enchant permanent
You can't choose an untapped permanent as Dream Leash's target as you cast Dream Leash.
You control enchanted permanent.","3UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opal Titan","W","When an opponent casts a creature spell, if Opal Titan is an enchantment, Opal Titan becomes a 4/4 Giant creature with protection from each of that spell's colors.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wurmcoil Engine","6","6","Deathtouch, lifelink
When Wurmcoil Engine dies, put a 3/3 colorless Wurm artifact creature token with deathtouch and a 3/3 colorless Wurm artifact creature token with lifelink onto the battlefield.","6","Artifact Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ink-Treader Nephilim","3","3","WURG","Whenever a player casts an instant or sorcery spell, if Ink-Treader Nephilim is the only target of that spell, copy the spell for each other creature that spell could target. Each copy targets a different one of those creatures.","RGWU","Creature - Nephilim");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foot Soldiers","2","4","W",NULL,"3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Blades","W","Rain of Blades deals 1 damage to each attacking creature.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flameborn Viron","6","4","R",NULL,"4RR","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eladamri's Call","WG","Search your library for a creature card, reveal that card, and put it into your hand. Then shuffle your library.","GW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Dragon","5","5","R","Flying
{R}: Shivan Dragon gets +1/+0 until end of turn.","4RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Might of Alara","G","Domain - Target creature gets +1/+1 until end of turn for each basic land type among lands you control.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chronatog Totem","{T}: Add {U} to your mana pool.
{1}{U}: Chronatog Totem becomes a 1/2 blue Atog artifact creature until end of turn.
{0}: Chronatog Totem gets +3/+3 until end of turn. You skip your next turn. Activate this ability only once each turn and only if Chronatog Totem is a creature.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Teferi's Veil","U","Whenever a creature you control attacks, it phases out at end of combat. (While it's phased out, it's treated as though it doesn't exist. It phases in before you untap during your next untap step.)","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Suffocating Blast","UR","Counter target spell and Suffocating Blast deals 3 damage to target creature.","1UUR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Highland Giant","3","4","R",NULL,"2RR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Palliator","0","2","W","{T}: Prevent the next 1 damage that would be dealt to each creature and each player this turn.","2W","Creature - Fox Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kiki-Jiki, Mirror Breaker","2","2","R","Haste
{T}: Put a token that's a copy of target nonlegendary creature you control onto the battlefield. That token has haste. Sacrifice it at the beginning of the next end step.","2RRR","Legendary Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Suffocation","U","Cast Suffocation only if you were dealt damage this turn by a red instant or sorcery spell.
Suffocation deals 4 damage to the controller of the last red instant or sorcery spell that dealt damage to you this turn.
Draw a card at the beginning of the next turn's upkeep.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream's Grip","U","Choose one - Tap target permanent; or untap target permanent.
Entwine {1} (Choose both if you pay the entwine cost.)","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fiery Fall","R","Fiery Fall deals 5 damage to target creature.
Basic landcycling {1}{R} ({1}{R}, Discard this card: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.)","5R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moss Kami","5","5","G","Trample","5G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jungle Lion","2","1","G","Jungle Lion can't block.","G","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barktooth Warbeard","6","5","BR",NULL,"4BRR","Legendary Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bloodchief Ascension","B","At the beginning of each end step, if an opponent lost 2 or more life this turn, you may put a quest counter on Bloodchief Ascension. (Damage causes loss of life.)
Whenever a card is put into an opponent's graveyard from anywhere, if Bloodchief Ascension has three or more quest counters on it, you may have that player lose 2 life. If you do, you gain 2 life.","B","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ironfang","3","1","First strike
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Ironfang.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Righteousness","W","Target blocking creature gets +7/+7 until end of turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Royal Assassin","1","1","B","{T}: Destroy target tapped creature.","1BB","Creature - Human Assassin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Herbal Poultice","{3}, Sacrifice Herbal Poultice: Regenerate target creature.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Storm Seeker","G","Storm Seeker deals damage to target player equal to the number of cards in that player's hand.","3G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angelic Protector","2","2","W","Flying
Whenever Angelic Protector becomes the target of a spell or ability, Angelic Protector gets +0/+3 until end of turn.","3W","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Poacher","2","2","G","{3}, {T}: Search your library for an Elf permanent card and put it onto the battlefield. Then shuffle your library.","2GG","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duskrider Peregrine","3","3","W","Flying, protection from black
Suspend 3-{1}{W} (Rather than cast this card from your hand, you may pay {1}{W} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","5W","Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skyship Weatherlight","When Skyship Weatherlight enters the battlefield, search your library for any number of artifact and/or creature cards and exile them. Then shuffle your library.
{4}, {T}: Choose a card at random that was exiled with Skyship Weatherlight. Put that card into its owner's hand.","4","Legendary Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sejiri Merfolk","2","1","U","As long as you control a Plains, Sejiri Merfolk has first strike and lifelink. (Damage dealt by a creature with lifelink also causes its controller to gain that much life.)","1U","Creature - Merfolk Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fatal Frenzy","R","Until end of turn, target creature you control gains trample and gets +X/+0, where X is its power. Sacrifice it at the beginning of the next end step.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Grenadiers","2","2","R","Whenever Goblin Grenadiers attacks and isn't blocked, you may sacrifice it. If you do, destroy target creature and target land.","3R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zealous Inquisitor","2","2","W","{1}{W}: The next 1 damage that would be dealt to Zealous Inquisitor this turn is dealt to target creature instead.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinder Pyromancer","0","1","R","{T}: Cinder Pyromancer deals 1 damage to target player.
Whenever you cast a red spell, you may untap Cinder Pyromancer.","2R","Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eternity Snare","U","Enchant creature
When Eternity Snare enters the battlefield, draw a card.
Enchanted creature doesn't untap during its controller's untap step.","5U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thallid Shell-Dweller","0","5","G","Defender
At the beginning of your upkeep, put a spore counter on Thallid Shell-Dweller.
Remove three spore counters from Thallid Shell-Dweller: Put a 1/1 green Saproling creature token onto the battlefield.","1G","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tribal Unity","G","Creatures of the creature type of your choice get +X/+X until end of turn.","X2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Library","1","1","G","{T}: Add {1} to your mana pool.
{T}: Draw a card. Activate this ability only if you have exactly seven cards in hand.","GG","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woebearer","2","3","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Whenever Woebearer deals combat damage to a player, you may return target creature card from your graveyard to your hand.","4B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flailing Manticore","3","3","R","Flying, first strike
{1}: Flailing Manticore gets +1/+1 until end of turn. Any player may activate this ability.
{1}: Flailing Manticore gets -1/-1 until end of turn. Any player may activate this ability.","3R","Creature - Manticore");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whipstitched Zombie","2","2","B","At the beginning of your upkeep, sacrifice Whipstitched Zombie unless you pay {B}.","1B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saprazzan Outrigger","5","5","U","When Saprazzan Outrigger attacks or blocks, put it on top of its owner's library at end of combat.","3U","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Meishin, the Mind Cage","U","All creatures get -X/-0, where X is the number of cards in your hand.","4UUU","Legendary Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Platinum Angel Avatar","If you control an artifact, a creature, an enchantment, and a land, you can't lose the game and your opponents can't win the game.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unchecked Growth","G","Target creature gets +4/+4 until end of turn. If it's a Spirit, it gains trample until end of turn.","2G","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tornado","G","Cumulative upkeep {G} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
{2}{G}, Pay 3 life for each velocity counter on Tornado: Destroy target permanent and put a velocity counter on Tornado. Activate this ability only once each turn.","4G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brain Freeze","U","Target player puts the top three cards of his or her library into his or her graveyard.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Baneful Omen","B","At the beginning of your end step, you may reveal the top card of your library. If you do, each opponent loses life equal to that card's converted mana cost.","4BBB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pure // Simple","RG","Destroy target multicolored permanent.
---
Destroy all Auras and Equipment.","1RG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Waning Wurm","7","6","B","Vanishing 2 (This permanent enters the battlefield with two time counters on it. At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)","3B","Creature - Zombie Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Ski Patrol","1","1","R","{1}{R}: Goblin Ski Patrol gets +2/+0 and gains flying. Its controller sacrifices it at the beginning of the next end step. Activate this ability only once and only if you control a snow Mountain.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stronghold Assassin","2","1","B","{T}, Sacrifice a creature: Destroy target nonblack creature.","1BB","Creature - Zombie Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Shadows","0","1","B","Defender (This creature can't attack.)
Prevent all damage that would be dealt to Wall of Shadows by creatures it's blocking.
Wall of Shadows can't be the target of spells that can target only Walls or of abilities that can target only Walls.","1BB","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Face of Fear","3","4","B","{2}{B}, Discard a card: Face of Fear gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","5B","Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jalum Tome","{2}, {T}: Draw a card, then discard a card.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thing from the Deep","9","9","U","Whenever Thing from the Deep attacks, sacrifice it unless you sacrifice an Island.","6UUU","Creature - Leviathan");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Energizer","2","2","{2}, {T}: Put a +1/+1 counter on Energizer.","4","Artifact Creature - Juggernaut");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Solar Tide","W","Choose one - Destroy all creatures with power 2 or less; or destroy all creatures with power 3 or greater.
Entwine-Sacrifice two lands. (Choose both if you pay the entwine cost.)","4WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jareth, Leonine Titan","4","7","W","Whenever Jareth, Leonine Titan blocks, it gets +7/+7 until end of turn.
{W}: Jareth gains protection from the color of your choice until end of turn.","3WWW","Legendary Creature - Cat Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cruel Tutor","B","Search your library for a card, then shuffle your library and put that card on top of it. You lose 2 life.","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Helm of Kaldra","Equipped creature has first strike, trample, and haste.
{1}: If you control Equipment named Helm of Kaldra, Sword of Kaldra, and Shield of Kaldra, put a legendary 4/4 colorless Avatar creature token named Kaldra onto the battlefield and attach those Equipment to it.
Equip {2}","3","Legendary Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fear","B","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature has fear. (It can't be blocked except by artifact creatures and/or black creatures.)","BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vengeful Dreams","W","As an additional cost to cast Vengeful Dreams, discard X cards.
Exile X target attacking creatures.","WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Artifact Ward","W","Enchant creature
Enchanted creature can't be blocked by artifact creatures.
Prevent all damage that would be dealt to enchanted creature by artifact sources.
Enchanted creature can't be the target of abilities from artifact sources.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hundred-Talon Strike","W","Target creature gets +1/+0 and gains first strike until end of turn.
Splice onto Arcane-Tap an untapped white creature you control. (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","W","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stasis Cocoon","W","Enchant artifact
Enchanted artifact's activated abilities can't be activated. If enchanted artifact is a creature, it can't attack or block.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Scout","1","2","G","{2}{G}, Sacrifice Viridian Scout: Viridian Scout deals 2 damage to target creature with flying.","3G","Creature - Elf Warrior Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flayer Husk","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +1/+1.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gruul War Plow","Creatures you control have trample.
{1}{R}{G}: Gruul War Plow becomes a 4/4 Juggernaut artifact creature until end of turn.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Explosive Growth","G","Kicker {5} (You may pay an additional {5} as you cast this spell.)
Target creature gets +2/+2 until end of turn. If Explosive Growth was kicked, that creature gets +5/+5 until end of turn instead.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Assault Strobe","R","Target creature gains double strike until end of turn. (It deals both first-strike and regular combat damage.)","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Argivian Restoration","U","Return target artifact card from your graveyard to the battlefield.","2UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steel Sabotage","U","Choose one - Counter target artifact spell; or return target artifact to its owner's hand.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pardic Swordsmith","1","1","R","{R}, Discard a card at random: Pardic Swordsmith gets +2/+0 until end of turn.","2R","Creature - Dwarf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Soldevi Excavations","If Soldevi Excavations would enter the battlefield, sacrifice an untapped Island instead. If you do, put Soldevi Excavations onto the battlefield. If you don't, put it into its owner's graveyard.
{T}: Add {1}{U} to your mana pool.
{1}, {T}: Look at the top card of your library. You may put that card on the bottom of your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anurid Brushhopper","3","4","WG","Discard two cards: Exile Anurid Brushhopper. Return it to the battlefield under its owner's control at the beginning of the next end step.","1GW","Creature - Frog Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kyren Sniper","1","1","R","At the beginning of your upkeep, you may have Kyren Sniper deal 1 damage to target player.","2R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riftsweeper","2","2","G","When Riftsweeper enters the battlefield, choose target face-up exiled card. Its owner shuffles it into his or her library.","1G","Creature - Elf Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ice Cauldron","{X}, {T}: Put a charge counter on Ice Cauldron and exile a nonland card from your hand. You may cast that card for as long as it remains exiled. Note the type and amount of mana spent to pay this activation cost. Activate this ability only if there are no charge counters on Ice Cauldron.
{T}, Remove a charge counter from Ice Cauldron: Add Ice Cauldron's last noted type and amount of mana to your mana pool. Spend this mana only to cast the last card exiled with Ice Cauldron.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kiku, Night's Flower","1","1","B","{2}{B}{B}, {T}: Target creature deals damage to itself equal to its power.","BB","Legendary Creature - Human Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ego Erasure","U","Changeling (This card is every creature type at all times.)
Creatures target player controls get -2/-0 and lose all creature types until end of turn.","2U","Tribal Instant - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lead Astray","W","Tap up to two target creatures.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caravan Hurda","1","5","W","Lifelink (Damage dealt by this creature also causes you to gain that much life.)","4W","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shelkin Brownie","1","1","G","{T}: Target creature loses all ""bands with other"" abilities until end of turn.","1G","Creature - Ouphe");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mana Prism","{T}: Add {1} to your mana pool.
{1}, {T}: Add one mana of any color to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Selesnya Sagittars","2","5","WG","Reach (This creature can block creatures with flying.)
Selesnya Sagittars can block an additional creature.","3GW","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Niv-Mizzet, the Firemind","4","4","UR","Flying
Whenever you draw a card, Niv-Mizzet, the Firemind deals 1 damage to target creature or player.
{T}: Draw a card.","2UURR","Legendary Creature - Dragon Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rite of Replication","U","Kicker {5} (You may pay an additional {5} as you cast this spell.)
Put a token that's a copy of target creature onto the battlefield. If Rite of Replication was kicked, put five of those tokens onto the battlefield instead.","2UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Onyx Mage","2","1","B","{1}{B}: Target creature you control gains deathtouch until end of turn. (Any amount of damage it deals to a creature is enough to destroy it.)","1B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Karma","W","At the beginning of each player's upkeep, Karma deals damage to that player equal to the number of Swamps he or she controls.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Vanguard","1","1","G","{T}: Llanowar Vanguard gets +0/+4 until end of turn.","2G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Judge of Currents","1","1","W","Whenever a Merfolk you control becomes tapped, you may gain 1 life.","1W","Creature - Merfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("That Which Was Taken","{4}, {T}: Put a divinity counter on target permanent other than That Which Was Taken.
Each permanent with a divinity counter on it is indestructible.","5","Legendary Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Petalmane Baku","1","2","G","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Petalmane Baku.
{1}, Remove X ki counters from Petalmane Baku: Add X mana of any one color to your mana pool.","1G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Molten Hydra","1","1","R","{1}{R}{R}: Put a +1/+1 counter on Molten Hydra.
{T}, Remove all +1/+1 counters from Molten Hydra: Molten Hydra deals damage to target creature or player equal to the number of +1/+1 counters removed this way.","1R","Creature - Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skeletal Scrying","B","As an additional cost to cast Skeletal Scrying, exile X cards from your graveyard.
You draw X cards and you lose X life.","XB","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Nightfall Predator","4","4","{R}, {T}: Nightfall Predator fights target creature. (Each deals damage equal to its power to the other.)
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Nightfall Predator.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Benediction of Moons","W","You gain 1 life for each player.
Haunt (When this spell card is put into a graveyard after resolving, exile it haunting target creature.)
When the creature Benediction of Moons haunts dies, you gain 1 life for each player.","W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hesitation","U","When a player casts a spell, sacrifice Hesitation and counter that spell.","1U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zof Shade","2","2","B","{2}{B}: Zof Shade gets +2/+2 until end of turn.","3B","Creature - Shade");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incite Hysteria","R","Radiance - Until end of turn, target creature and each other creature that shares a color with it gain ""This creature can't block.""","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Priest","1","1","W","{1}{W}{W}: Target creature gets +1/+1 until end of turn.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brine Elemental","5","4","U","Morph {5}{U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Brine Elemental is turned face up, each opponent skips his or her next untap step.","4UU","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Suture Priest","1","1","W","Whenever another creature enters the battlefield under your control, you may gain 1 life.
Whenever a creature enters the battlefield under an opponent's control, you may have that player lose 1 life.","1W","Creature - Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lawbringer","2","2","W","{T}, Sacrifice Lawbringer: Exile target red creature.","2W","Creature - Kor Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tar Pitcher","2","2","R","{T}, Sacrifice a Goblin: Tar Pitcher deals 2 damage to target creature or player.","3R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glowrider","2","1","W","Noncreature spells cost {1} more to cast.","2W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Funeral","UB","Target opponent reveals cards from the top of his or her library until four land cards are revealed. That player puts all cards revealed this way into his or her graveyard.","1UB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pteron Ghost","1","1","W","Flying
Sacrifice Pteron Ghost: Regenerate target artifact.","1W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiketail Drake","3","3","U","Flying
Sacrifice Spiketail Drake: Counter target spell unless its controller pays {3}.","3UU","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vulshok Refugee","3","2","R","Protection from red","1RR","Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sylvan Echoes","G","Whenever you clash and win, you may draw a card. (This ability triggers after the clash ends.)","G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Underground River","{T}: Add {1} to your mana pool.
{T}: Add {U} or {B} to your mana pool. Underground River deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blightsoil Druid","1","2","B","{T}, Pay 1 life: Add {G} to your mana pool.","1B","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mother of Runes","1","1","W","{T}: Target creature you control gains protection from the color of your choice until end of turn.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Femeref Scouts","1","4","W",NULL,"2W","Creature - Human Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostly Wings","U","Enchant creature
Enchanted creature gets +1/+1 and has flying.
Discard a card: Return enchanted creature to its owner's hand.","1U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashnod's Cylix","{3}, {T}: Target player looks at the top three cards of his or her library, puts one of them back on top of his or her library, then exiles the rest.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Falter","R","Creatures without flying can't block this turn.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Remote Farm","Remote Farm enters the battlefield tapped with two depletion counters on it.
{T}, Remove a depletion counter from Remote Farm: Add {W}{W} to your mana pool. If there are no depletion counters on Remote Farm, sacrifice it.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Millstone","{2}, {T}: Target player puts the top two cards of his or her library into his or her graveyard.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rosheen Meanderer","4","4","RG","{T}: Add {4} to your mana pool. Spend this mana only on costs that contain {X}.","3(R/G)","Legendary Creature - Giant Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carnivorous Death-Parrot","2","2","U","Flying
At the beginning of your upkeep, sacrifice Carnivorous Death-Parrot unless you say its flavor text.","1U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ancient Silverback","6","5","G","{G}: Regenerate Ancient Silverback. (The next time this creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","4GG","Creature - Ape");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Polluted Bonds","B","Whenever a land enters the battlefield under an opponent's control, that player loses 2 life and you gain 2 life.","3BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scourge of Numai","4","4","B","At the beginning of your upkeep, you lose 2 life if you don't control an Ogre.","3B","Creature - Demon Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Independent Troops","2","1","R",NULL,"1R","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Paper Tiger","4","3","Rock Lobsters cannot attack or block.","4","Artifact Creature");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mob Justice","R","Mob Justice deals damage to target player equal to the number of creatures you control.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Progenitus","10","10","WUBRG","Protection from everything
If Progenitus would be put into a graveyard from anywhere, reveal Progenitus and shuffle it into its owner's library instead.","WWUUBBRRGG","Legendary Creature - Hydra Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Punishing Fire","R","Punishing Fire deals 2 damage to target creature or player.
Whenever an opponent gains life, you may pay {R}. If you do, return Punishing Fire from your graveyard to your hand.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arctic Nishoba","6","6","G","Trample
Cumulative upkeep {G} or {W} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When Arctic Nishoba dies, you gain 2 life for each age counter on it.","5G","Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Flint Golem","2","3","Whenever Flint Golem becomes blocked, defending player puts the top three cards of his or her library into his or her graveyard.","4","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hydromorph Gull","3","3","U","Flying
{U}, Sacrifice Hydromorph Gull: Counter target spell that targets one or more creatures you control.","3UU","Creature - Elemental Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("War Elephant","2","2","W","Trample; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","3W","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Titania's Chosen","1","1","G","Whenever a player casts a green spell, put a +1/+1 counter on Titania's Chosen.","2G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coalition Honor Guard","2","4","W","While choosing targets as part of casting a spell or activating an ability, your opponents must choose at least one Flagbearer on the battlefield if able.","3W","Creature - Human Flagbearer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rush of Knowledge","U","Draw cards equal to the highest converted mana cost among permanents you control.","4U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scavenging Scarab","3","3","B","Scavenging Scarab can't block.","3B","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wandering Stream","G","Domain - You gain 2 life for each basic land type among lands you control.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dralnu, Lich Lord","3","3","UB","If damage would be dealt to Dralnu, Lich Lord, sacrifice that many permanents instead.
{T}: Target instant or sorcery card in your graveyard gains flashback until end of turn. The flashback cost is equal to its mana cost. (You may cast that card from your graveyard for its flashback cost. Then exile it.)","3UB","Legendary Creature - Zombie Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("An-Havva Township","{T}: Add {1} to your mana pool.
{1}, {T}: Add {G} to your mana pool.
{2}, {T}: Add {R} or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rally","W","Blocking creatures get +1/+1 until end of turn.","WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chasm Drake","3","3","U","Flying
Whenever Chasm Drake attacks, target creature you control gains flying until end of turn.","4U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Notorious Throng","U","Prowl {5}{U} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Rogue.)
Put X 1/1 black Faerie Rogue creature tokens with flying onto the battlefield, where X is the damage dealt to your opponents this turn. If Notorious Throng's prowl cost was paid, take an extra turn after this one.","3U","Tribal Sorcery - Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Soul Conduit","{6}, {T}: Two target players exchange life totals.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tonic Peddler","1","1","W","{W}, {T}, Discard a card: Target player gains 3 life.","1W","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vampiric Dragon","5","5","BR","Flying
Whenever a creature dealt damage by Vampiric Dragon this turn dies, put a +1/+1 counter on Vampiric Dragon.
{1}{R}: Vampiric Dragon deals 1 damage to target creature.","6BR","Creature - Vampire Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unnatural Speed","R","Target creature gains haste until end of turn.","R","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inner Calm, Outer Strength","G","Target creature gets +X/+X until end of turn, where X is the number of cards in your hand.","2G","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fugue","B","Target player discards three cards.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Verduran Enchantress","0","2","G","Whenever you cast an enchantment spell, you may draw a card.","1GG","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Molten-Tail Masticore","4","4","At the beginning of your upkeep, sacrifice Molten-Tail Masticore unless you discard a card.
{4}, Exile a creature card from your graveyard: Molten-Tail Masticore deals 4 damage to target creature or player.
{2}: Regenerate Molten-Tail Masticore.","4","Artifact Creature - Masticore");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Venser, the Sojourner","WU","+2: Exile target permanent you own. Return it to the battlefield under your control at the beginning of the next end step.
-1: Creatures are unblockable this turn.
-8: You get an emblem with ""Whenever you cast a spell, exile target permanent.""","Planeswalker - Venser","3WU");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urborg, Tomb of Yawgmoth","Each land is a Swamp in addition to its other land types.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pox","B","Each player loses a third of his or her life, then discards a third of the cards in his or her hand, then sacrifices a third of the creatures he or she controls, then sacrifices a third of the lands he or she controls. Round up each time.","BBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anthroplasm","0","0","U","Anthroplasm enters the battlefield with two +1/+1 counters on it.
{X}, {T}: Remove all +1/+1 counters from Anthroplasm and put X +1/+1 counters on it.","2UU","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Summer Bloom","G","You may play up to three additional lands this turn.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunder from Within","R","Destroy target artifact or land.","2RR","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elven Cache","G","Return target card from your graveyard to your hand.","2GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonic Consultation","B","Name a card. Exile the top six cards of your library, then reveal cards from the top of your library until you reveal the named card. Put that card into your hand and exile all other cards revealed this way.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Tutor","R","Roll a six-sided die for Goblin Tutor. On a 1, Goblin Tutor has no effect. Otherwise, search your library for the indicated card, reveal that card to all players, and put it into your hand. Shuffle your library afterwards.
2 Any Goblin Tutor
3 Any enchantment
4 Any artifact
5 Any creature
6 Any sorcery, instant, or interrupt","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oni of Wild Places","6","5","R","Haste
At the beginning of your upkeep, return a red creature you control to its owner's hand.","5R","Creature - Demon Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Troll Ascetic","3","2","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
{1}{G}: Regenerate Troll Ascetic.","1GG","Creature - Troll Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teferi's Imp","1","1","U","Flying
Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)
Whenever Teferi's Imp phases out, discard a card.
Whenever Teferi's Imp phases in, draw a card.","2U","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Declaration of Naught","U","As Declaration of Naught enters the battlefield, name a card.
{U}: Counter target spell with the chosen name.","UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Relic Crush","G","Destroy target artifact or enchantment and up to one other target artifact or enchantment.","4G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hallow","W","Prevent all damage target spell would deal this turn. You gain life equal to the damage prevented this way.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shimmering Wings","U","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature has flying. (It can't be blocked except by creatures with flying or reach.)
{U}: Return Shimmering Wings to its owner's hand.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blind Creeper","3","3","B","Whenever a player casts a spell, Blind Creeper gets -1/-1 until end of turn.","1B","Creature - Zombie Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Assembly Hall","{4}, {T}: Reveal a creature card in your hand. Search your library for a card with the same name as that card, reveal it, and put it into your hand. Then shuffle your library.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reclaim","G","Put target card from your graveyard on top of your library.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Naya","You may play any number of additional lands on each of your turns.
Whenever you roll {C}, target red, green, or white creature you control gets +1/+1 until end of turn for each land you control.",NULL,"Plane - Alara");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reciprocate","W","Exile target creature that dealt damage to you this turn.","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Naya Panorama","{T}: Add {1} to your mana pool.
{1}, {T}, Sacrifice Naya Panorama: Search your library for a basic Mountain, Forest, or Plains card and put it onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orbweaver Kumo","3","4","G","Reach (This creature can block creatures with flying.)
Whenever you cast a Spirit or Arcane spell, Orbweaver Kumo gains forestwalk until end of turn.","4GG","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of Leitbur","2","1","W","Protection from black
{W}: Order of Leitbur gains first strike until end of turn.
{W}{W}: Order of Leitbur gets +1/+0 until end of turn.","WW","Creature - Human Cleric Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fencer's Magemark","R","Enchant creature
Creatures you control that are enchanted get +1/+1 and have first strike.","2R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grim Discovery","B","Choose one or both - Return target creature card from your graveyard to your hand; and/or return target land card from your graveyard to your hand.","1B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Parallax Inhibitor","{1}, {T}, Sacrifice Parallax Inhibitor: Put a fade counter on each permanent with fading you control.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grave Consequences","B","Each player may exile any number of cards from his or her graveyard. Then each player loses 1 life for each card in his or her graveyard.
Draw a card.","1B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Amulet of Kroog","{2}, {T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lim-Dûl's Hex","B","At the beginning of your upkeep, for each player, Lim-Dûl's Hex deals 1 damage to that player unless he or she pays {B} or {3}.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Empress Galina","1","3","U","{U}{U}, {T}: Gain control of target legendary permanent. (This effect lasts indefinitely.)","3UU","Legendary Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Night Terrors","B","Target player reveals his or her hand. You choose a nonland card from it. Exile that card.","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hermit Druid Avatar","At the beginning of your upkeep, put a land card from your library chosen at random onto the battlefield.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rampant Growth","G","Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sprouting Vines","G","Search your library for a basic land card, reveal that card, and put it into your hand. Then shuffle your library.
Storm (When you cast this spell, copy it for each spell cast before it this turn.)","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boiling Seas","R","Destroy all Islands.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goliath Sphinx","8","7","U","Flying","5UU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blinking Spirit","2","2","W","{0}: Return Blinking Spirit to its owner's hand.","3W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brand","R","Gain control of all permanents you own. (This effect lasts indefinitely.)
Cycling {2} ({2}, Discard this card: Draw a card.)","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Reflexes","R","You may cast Lightning Reflexes as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Enchant creature
Enchanted creature gets +1/+0 and has first strike.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elephant Grass","G","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Black creatures can't attack you.
Nonblack creatures can't attack you unless their controller pays {2} for each creature he or she controls that's attacking you.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angel's Grace","W","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
You can't lose the game this turn and your opponents can't win the game this turn. Until end of turn, damage that would reduce your life total to less than 1 reduces it to 1 instead.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Assault","R","At the beginning of your upkeep, put a 1/1 red Goblin creature token with haste onto the battlefield.
Goblin creatures attack each turn if able.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ingenious Thief","1","1","U","Flying
When Ingenious Thief enters the battlefield, look at target player's hand.","1U","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kozilek's Predator","3","3","G","When Kozilek's Predator enters the battlefield, put two 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool.""","3G","Creature - Eldrazi Drone");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Borrowing 100,000 Arrows","U","Draw a card for each tapped creature target opponent controls.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minotaur Warrior","2","3","R",NULL,"2R","Creature - Minotaur Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enraging Licid","1","1","R","{R}, {T}: Enraging Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {R} to end this effect.
Enchanted creature has haste.","1R","Creature - Licid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steelclad Serpent","4","5","U","Steelclad Serpent can't attack unless you control another artifact.","5U","Artifact Creature - Serpent");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bayou",NULL,NULL,"Land - Swamp Forest");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gnaw to the Bone","G","You gain 2 life for each creature card in your graveyard.
Flashback {2}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2G","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Withengar Unbound","13","13","Flying, intimidate, trample
Whenever a player loses the game, put thirteen +1/+1 counters on Withengar Unbound.",NULL,"Legendary Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jedit's Dragoons","2","5","W","Vigilance
When Jedit's Dragoons enters the battlefield, you gain 4 life.","5W","Creature - Cat Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chance Encounter","R","Whenever you win a coin flip, put a luck counter on Chance Encounter.
At the beginning of your upkeep, if Chance Encounter has ten or more luck counters on it, you win the game.","2RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Falkenrath Aristocrat","4","1","BR","Flying, haste
Sacrifice a creature: Falkenrath Aristocrat is indestructible this turn. If the sacrificed creature was a Human, put a +1/+1 counter on Falkenrath Aristocrat.","2BR","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thundermare","5","5","R","Haste (This creature can attack the turn it comes under your control.)
When Thundermare enters the battlefield, tap all other creatures.","5R","Creature - Elemental Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Loose Lips","U","As Loose Lips comes into play, choose a sentence with eight or fewer words.
Enchanted creature has flying.
Whenever enchanted creature deals damage to an opponent, you draw two cards unless that player says the chosen sentence.","U","Enchant Creature");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wee Dragonauts","1","3","UR","Flying
Whenever you cast an instant or sorcery spell, Wee Dragonauts gets +2/+0 until end of turn.","1UR","Creature - Faerie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodshot Cyclops","4","4","R","{T}, Sacrifice a creature: Bloodshot Cyclops deals damage equal to the sacrificed creature's power to target creature or player.","5R","Creature - Cyclops Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Bookie","1","1","R","oR, ocT: Reflip any coin or reroll any die.","R","Summon - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Angel's Feather","Whenever a player casts a white spell, you may gain 1 life.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Tribute","B","Kicker-Tap an untapped Vampire you control. (You may tap a Vampire you control in addition to any other costs as you cast this spell.)
Target opponent loses half his or her life, rounded up. If Blood Tribute was kicked, you gain life equal to the life lost this way.","4BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Omnibian","3","3","UG","{T}: Target creature becomes a 3/3 Frog until end of turn.","1GGU","Creature - Frog");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Offering to Asha","WU","Counter target spell unless its controller pays {4}. You gain 4 life.","2WU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Tiger","1","0","G","Phantom Tiger enters the battlefield with two +1/+1 counters on it.
If damage would be dealt to Phantom Tiger, prevent that damage. Remove a +1/+1 counter from Phantom Tiger.","2G","Creature - Cat Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Arena","B","At the beginning of your upkeep, you draw a card and you lose 1 life.","1BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Word of Undoing","U","Return target creature and all white Auras you own attached to it to their owners' hands.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Day of the Dragons","U","When Day of the Dragons enters the battlefield, exile all creatures you control. Then put that many 5/5 red Dragon creature tokens with flying onto the battlefield.
When Day of the Dragons leaves the battlefield, sacrifice all Dragons you control. Then return the exiled cards to the battlefield under your control.","4UUU","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Unhallowed Cathar","2","1","Unhallowed Cathar can't block.",NULL,"Creature - Zombie Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Searing Rays","R","Choose a color. Searing Rays deals damage to each player equal to the number of creatures of that color that player controls.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cavern Harpy","2","1","UB","Flying
When Cavern Harpy enters the battlefield, return a blue or black creature you control to its owner's hand.
Pay 1 life: Return Cavern Harpy to its owner's hand.","UB","Creature - Harpy Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vibrating Sphere","As long as it's your turn, creatures you control get +2/+0.
As long as it's not your turn, creatures you control get -0/-2.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Molten Frame","R","Destroy target artifact creature.
Cycling {2} ({2}, Discard this card: Draw a card.)","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blightning","BR","Blightning deals 3 damage to target player. That player discards two cards.","1BR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rune of Protection: Blue","W","{W}: The next time a blue source of your choice would deal damage to you this turn, prevent that damage.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tectonic Rift","R","Destroy target land. Creatures without flying can't block this turn.","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pestilence","B","At the beginning of the end step, if no creatures are on the battlefield, sacrifice Pestilence.
{B}: Pestilence deals 1 damage to each creature and each player.","2BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boros Fury-Shield","W","Prevent all combat damage that would be dealt by target attacking or blocking creature this turn. If {R} was spent to cast Boros Fury-Shield, it deals damage to that creature's controller equal to the creature's power.","2W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wayfarer's Bauble","{2}, {T}, Sacrifice Wayfarer's Bauble: Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veteran's Voice","R","Enchant creature you control
Tap enchanted creature: Target creature other than the creature tapped this way gets +2/+1 until end of turn. Activate this ability only if enchanted creature is untapped.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thought Gorger","2","2","B","Trample
When Thought Gorger enters the battlefield, put a +1/+1 counter on it for each card in your hand. If you do, discard your hand.
When Thought Gorger leaves the battlefield, draw a card for each +1/+1 counter on it.","2BB","Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Goldmeadow","Whenever a land enters the battlefield, that land's controller puts three 0/1 white Goat creature tokens onto the battlefield.
Whenever you roll {C}, put a 0/1 white Goat creature token onto the battlefield.",NULL,"Plane - Lorwyn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Carrion","B","As an additional cost to cast Carrion, sacrifice a creature.
Put X 0/1 black Insect creature tokens onto the battlefield, where X is the sacrificed creature's power.","1BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Greed","B","{B}, Pay 2 life: Draw a card.","3B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planeswalker's Mirth","W","{3}{W}: Target opponent reveals a card at random from his or her hand. You gain life equal to that card's converted mana cost.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kavu Lair","G","Whenever a creature with power 4 or greater enters the battlefield, its controller draws a card.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clay Pigeon","1","1","Flying
o1, Throw Clay Pigeon into the air at least two feet above your head while seated, Attempt to catch it with one hand: If you catch Clay Pigeon, prevent all damage to you from any one source and return Clay Pigeon to play, tapped. Otherwise, sacrifice it.","3","Artifact Creature");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Drake-Skull Cameo","{T}: Add {U} or {B} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wort, Boggart Auntie","3","3","BR","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
At the beginning of your upkeep, you may return target Goblin card from your graveyard to your hand.","2BR","Legendary Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elvish Promenade","G","Put a 1/1 green Elf Warrior creature token onto the battlefield for each Elf you control.","3G","Tribal Sorcery - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reborn Hero","2","2","W","Vigilance
Threshold - As long as seven or more cards are in your graveyard, Reborn Hero has ""When Reborn Hero dies, you may pay {W}{W}. If you do, return Reborn Hero to the battlefield under your control.""","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pillar Tombs of Aku","B","At the beginning of each player's upkeep, that player may sacrifice a creature. If that player doesn't, he or she loses 5 life and you sacrifice Pillar Tombs of Aku.","2BB","World Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("My Crushing Masterstroke","When you set this scheme in motion, gain control of all nonland permanents your opponents control until end of turn. Untap those permanents. They gain haste until end of turn. Each of them attacks its owner this turn if able.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu Foot Soldiers","2","3","W",NULL,"2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mine Bearer","1","1","W","{T}, Sacrifice Mine Bearer: Destroy target attacking creature.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ludevic's Test Subject","0","3","U","Defender
{1}{U}: Put a hatchling counter on Ludevic's Test Subject. Then if there are five or more hatchling counters on it, remove all of them and transform it.","1U","Creature - Lizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dimir Signet","{1}, {T}: Add {U}{B} to your mana pool.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rustic Clachan","As Rustic Clachan enters the battlefield, you may reveal a Kithkin card from your hand. If you don't, Rustic Clachan enters the battlefield tapped.
{T}: Add {W} to your mana pool.
Reinforce 1-{1}{W} ({1}{W}, Discard this card: Put a +1/+1 counter on target creature.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Diplomatic Immunity","U","Enchant creature
Shroud (This permanent can't be the target of spells or abilities.)
Enchanted creature has shroud.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hornet","1","1","Flying, haste",NULL,"Artifact Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shapesharer","1","1","U","Changeling (This card is every creature type at all times.)
{2}{U}: Target Shapeshifter becomes a copy of target creature until your next turn.","1U","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravebane Zombie","3","2","B","If Gravebane Zombie would die, put Gravebane Zombie on top of its owner's library instead.","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sway of Illusion","U","Any number of target creatures become the color of your choice until end of turn.
Draw a card.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patron of the Orochi","7","7","G","Snake offering (You may cast this card any time you could cast an instant by sacrificing a Snake and paying the difference in mana costs between this and the sacrificed Snake. Mana cost includes color.)
{T}: Untap all Forests and all green creatures. Activate this ability only once each turn.","6GG","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Invisible Stalker","1","1","U","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Invisible Stalker is unblockable.","1U","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakdos Augermage","3","2","BR","First strike
{T}: Reveal your hand and discard a card of target opponent's choice. Then that player reveals his or her hand and discards a card of your choice. Activate this ability only any time you could cast a sorcery.","BBR","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sift Through Sands","U","Draw two cards, then discard a card.
If you've cast a spell named Peer Through Depths and a spell named Reach Through Mists this turn, you may search your library for a card named The Unspeakable, put it onto the battlefield, then shuffle your library.","1UU","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Entomber Exarch","2","2","B","When Entomber Exarch enters the battlefield, choose one - Return target creature card from your graveyard to your hand; or target opponent reveals his or her hand, you choose a noncreature card from it, then that player discards that card.","2BB","Creature - Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dreamspoiler Witches","2","2","B","Flying
Whenever you cast a spell during an opponent's turn, you may have target creature get -1/-1 until end of turn.","3B","Creature - Faerie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aurochs","2","3","G","Trample
Whenever Aurochs attacks, it gets +1/+0 until end of turn for each other attacking Aurochs.","3G","Creature - Aurochs");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glyph of Reincarnation","G","Cast Glyph of Reincarnation only after combat.
Destroy all creatures that were blocked by target Wall this turn. They can't be regenerated. For each creature that died this way, put a creature card from the graveyard of the player who controlled that creature the last time it became blocked by that Wall onto the battlefield under its owner's control.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fangren Pathcutter","4","6","G","Whenever Fangren Pathcutter attacks, attacking creatures gain trample until end of turn.","4GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rot Wolf","2","2","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Whenever a creature dealt damage by Rot Wolf this turn dies, you may draw a card.","2G","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stalking Tiger","3","3","G","Stalking Tiger can't be blocked by more than one creature.","3G","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Finesse","U","Attach target Aura you control to target creature.
Draw a card.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glittering Wish","WG","You may choose a multicolored card you own from outside the game, reveal that card, and put it into your hand. Exile Glittering Wish.","GW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Goon","6","6","R","Goblin Goon can't attack unless you control more creatures than defending player.
Goblin Goon can't block unless you control more creatures than attacking player.","3R","Creature - Goblin Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dimir Cutpurse","2","2","UB","Whenever Dimir Cutpurse deals combat damage to a player, that player discards a card and you draw a card.","1UB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pentarch Paladin","3","3","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
As Pentarch Paladin enters the battlefield, choose a color.
{W}{W}, {T}: Destroy target permanent of the chosen color.","2WWW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Psychatog","1","2","UB","Discard a card: Psychatog gets +1/+1 until end of turn.
Exile two cards from your graveyard: Psychatog gets +1/+1 until end of turn.","1UB","Creature - Atog");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Errand of Duty","W","Put a 1/1 white Knight creature token with banding onto the battlefield. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Joyous Respite","G","You gain 1 life for each land you control.","3G","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dominate","U","Gain control of target creature with converted mana cost X or less. (This effect lasts indefinitely.)","X1UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sustaining Spirit","0","3","W","Cumulative upkeep {1}{W} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Damage that would reduce your life total to less than 1 reduces it to 1 instead.","1W","Creature - Angel Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tar Fiend","4","4","B","Devour 2 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with twice that many +1/+1 counters on it.)
When Tar Fiend enters the battlefield, target player discards a card for each creature it devoured.","5B","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Calmer","2","3","G","{G}, {T}, Sacrifice Nantuko Calmer: Destroy target enchantment.
Threshold - Nantuko Calmer gets +1/+1 as long as seven or more cards are in your graveyard.","2GG","Creature - Insect Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sulfurous Springs","{T}: Add {1} to your mana pool.
{T}: Add {B} or {R} to your mana pool. Sulfurous Springs deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Festering Goblin","1","1","B","When Festering Goblin dies, target creature gets -1/-1 until end of turn.","B","Creature - Zombie Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boris Devilboon","2","2","BR","{2}{B}{R}, {T}: Put a 1/1 black and red Demon creature token named Minor Demon onto the battlefield.","3BR","Legendary Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greater Werewolf","2","4","B","At end of combat, put a -0/-2 counter on each creature blocking or blocked by Greater Werewolf.","4B","Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Peacekeeper","1","1","W","At the beginning of your upkeep, sacrifice Peacekeeper unless you pay {1}{W}.
Creatures can't attack.","2W","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Xantid Swarm","0","1","G","Flying
Whenever Xantid Swarm attacks, defending player can't cast spells this turn.","G","Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lantern-Lit Graveyard","{T}: Add {1} to your mana pool.
{T}: Add {B} or {R} to your mana pool. Lantern-Lit Graveyard doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Animate Dead","B","Enchant creature card in a graveyard
When Animate Dead enters the battlefield, if it's on the battlefield, it loses ""enchant creature card in a graveyard"" and gains ""enchant creature put onto the battlefield with Animate Dead."" Return enchanted creature card to the battlefield under your control and attach Animate Dead to it. When Animate Dead leaves the battlefield, that creature's controller sacrifices it.
Enchanted creature gets -1/-0.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Wayfarer","1","1","G","Sacrifice Krosan Wayfarer: You may put a land card from your hand onto the battlefield.","G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firewake Sliver","1","1","RG","All Sliver creatures have haste.
All Slivers have ""{1}, Sacrifice this permanent: Target Sliver creature gets +2/+2 until end of turn.""","1RG","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vulturous Zombie","3","3","BG","Flying
Whenever a card is put into an opponent's graveyard from anywhere, put a +1/+1 counter on Vulturous Zombie.","3BG","Creature - Plant Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Vigilante","3","2","G","Morph {1}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Nantuko Vigilante is turned face up, destroy target artifact or enchantment.","3G","Creature - Insect Druid Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Samite Elder","1","2","W","{T}: Creatures you control gain protection from the colors of target permanent you control until end of turn.","2W","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("City of Traitors","When you play another land, sacrifice City of Traitors.
{T}: Add {2} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lavaball Trap","R","If an opponent had two or more lands enter the battlefield under his or her control this turn, you may pay {3}{R}{R} rather than pay Lavaball Trap's mana cost.
Destroy two target lands. Lavaball Trap deals 4 damage to each creature.","6RR","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Doubtless One","*","*","W","Doubtless One's power and toughness are each equal to the number of Clerics on the battlefield.
Whenever Doubtless One deals damage, you gain that much life.","3W","Creature - Cleric Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elite Cat Warrior","2","3","G","Forestwalk","2G","Creature - Cat Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Snow-Covered Swamp","B",NULL,"Basic Snow Land - Swamp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sludge Strider","3","3","WUB","Whenever another artifact enters the battlefield under your control or another artifact you control leaves the battlefield, you may pay {1}. If you do, target player loses 1 life and you gain 1 life.","1WUB","Artifact Creature - Insect");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Conch Horn","{1}, {T}, Sacrifice Conch Horn: Draw two cards, then put a card from your hand on top of your library.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earthbrawn","G","Target creature gets +3/+3 until end of turn.
Reinforce 1-{1}{G} ({1}{G}, Discard this card: Put a +1/+1 counter on target creature.)","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dross Prowler","2","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Moat","0","3","W","Creatures without flying can't attack.","2WW","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Obliterate","R","Obliterate can't be countered.
Destroy all artifacts, creatures, and lands. They can't be regenerated.","6RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Black Ward","W","Enchant creature
Enchanted creature has protection from black. This effect doesn't remove Black Ward.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shimmering Mirage","U","Target land becomes the basic land type of your choice until end of turn.
Draw a card.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Warrens","R","{2}{R}, Sacrifice two Goblins: Put three 1/1 red Goblin creature tokens onto the battlefield.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armored Guardian","2","5","WU","{1}{W}{W}: Target creature you control gains protection from the color of your choice until end of turn.
{1}{U}{U}: Armored Guardian gains shroud until end of turn. (It can't be the target of spells or abilities.)","3WU","Creature - Cat Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Torches","4","1","R","Defender (This creature can't attack.)","1R","Creature - Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Anodet Lurker","3","3","When Anodet Lurker dies, you gain 3 life.","5","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Glider","1","1","B","Flying
{T}, Sacrifice a land: Search your library for a Mercenary permanent card with converted mana cost 2 or less and put it onto the battlefield. Then shuffle your library.","2B","Creature - Human Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Graphic Violence","G","All creatures by the artist of your choice get +2/+2 and gain trample until end of turn.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunlance","W","Sunlance deals 3 damage to target nonwhite creature.","W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Venomous Dragonfly","1","1","G","Flying
Whenever Venomous Dragonfly blocks or becomes blocked by a creature, destroy that creature at end of combat.","3G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soulgorger Orgg","6","6","R","Trample
When Soulgorger Orgg enters the battlefield, you lose all but 1 life.
When Soulgorger Orgg leaves the battlefield, you gain life equal to the life you lost when it entered the battlefield.","3RR","Creature - Nightmare Orgg");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burst of Speed","R","Creatures you control gain haste until end of turn. (They can attack and {T} even if they just came under your control.)","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horizon Seed","2","1","W","Whenever you cast a Spirit or Arcane spell, regenerate target creature.","4W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vicious Shadows","R","Whenever a creature dies, you may have Vicious Shadows deal damage to target player equal to the number of cards in that player's hand.","6R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flowstone Armor","You may choose not to untap Flowstone Armor during your untap step.
{3}, {T}: Target creature gets +1/-1 for as long as Flowstone Armor remains tapped.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Paleoloth","5","5","G","Whenever another creature with power 5 or greater enters the battlefield under your control, you may return target creature card from your graveyard to your hand.","4GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Autochthon Wurm","9","14","WG","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Trample","10GGGWW","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Subversion","B","At the beginning of your upkeep, each opponent loses 1 life. You gain life equal to the life lost this way.","3BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Struggle for Sanity","B","Target opponent reveals his or her hand. That player exiles a card from it, then you exile a card from it. Repeat this process until all cards in that hand have been exiled. That player returns the cards he or she exiled this way to his or her hand and puts the rest into his or her graveyard.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elgaud Shieldmate","2","3","U","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Elgaud Shieldmate is paired with another creature, both creatures have hexproof. (They can't be the targets of spells or abilities your opponents control.)","3U","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Llanowar Wastes","{T}: Add {1} to your mana pool.
{T}: Add {B} or {G} to your mana pool. Llanowar Wastes deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hobble","W","Enchant creature
When Hobble enters the battlefield, draw a card.
Enchanted creature can't attack.
Enchanted creature can't block if it's black.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Despondency","B","Enchant creature
Enchanted creature gets -2/-0.
When Despondency is put into a graveyard from the battlefield, return Despondency to its owner's hand.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Borborygmos","6","7","RG","Trample
Whenever Borborygmos deals combat damage to a player, put a +1/+1 counter on each creature you control.","3RRGG","Legendary Creature - Cyclops");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Ghoul","1","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever a creature with shadow dies, put a +1/+1 counter on Dauthi Ghoul.","1B","Creature - Dauthi Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rushing-Tide Zubera","3","3","U","When Rushing-Tide Zubera dies, if 4 or more damage was dealt to it this turn, draw three cards.","2UU","Creature - Zubera Spirit");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Scarecrone","1","2","{1}, Sacrifice a Scarecrow: Draw a card.
{4}, {T}: Return target artifact creature card from your graveyard to the battlefield.","3","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boiling Blood","R","Target creature attacks this turn if able.
Draw a card.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vengeful Dead","3","2","B","Whenever Vengeful Dead or another Zombie dies, each opponent loses 1 life.","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyrotechnics","R","Pyrotechnics deals 4 damage divided as you choose among any number of target creatures and/or players.","4R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flagstones of Trokair","{T}: Add {W} to your mana pool.
When Flagstones of Trokair is put into a graveyard from the battlefield, you may search your library for a Plains card and put it onto the battlefield tapped. If you do, shuffle your library.",NULL,"Legendary Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shiv","All creatures have ""{R}: This creature gets +1/+0 until end of turn.""
Whenever you roll {C}, put a 5/5 red Dragon creature token with flying onto the battlefield.",NULL,"Plane - Dominaria");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lord of Atlantis","2","2","U","Other Merfolk creatures get +1/+1 and have islandwalk.","UU","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goretusk Firebeast","2","2","R","When Goretusk Firebeast enters the battlefield, it deals 4 damage to target player.","5R","Creature - Elemental Boar Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guan Yu, Sainted Warrior","3","5","W","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
When Guan Yu, Sainted Warrior is put into your graveyard from the battlefield, you may shuffle Guan Yu into your library.","3WW","Legendary Creature - Human Soldier Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jagged Poppet","3","4","BR","Whenever Jagged Poppet is dealt damage, discard that many cards.
Hellbent - Whenever Jagged Poppet deals combat damage to a player, if you have no cards in hand, that player discards cards equal to the damage.","1BR","Creature - Ogre Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spiteful Shadows","B","Enchant creature
Whenever enchanted creature is dealt damage, it deals that much damage to its controller.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spectral Shield","WU","Enchant creature
Enchanted creature gets +0/+2 and can't be the target of spells.","1WU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Holy Armor","W","Enchant creature
Enchanted creature gets +0/+2.
{W}: Enchanted creature gets +0/+1 until end of turn.","W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Smokestack","At the beginning of your upkeep, you may put a soot counter on Smokestack.
At the beginning of each player's upkeep, that player sacrifices a permanent for each soot counter on Smokestack.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Words of Waste","B","{1}: The next time you would draw a card this turn, each opponent discards a card instead.","2B","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hair-Strung Koto","Tap an untapped creature you control: Target player puts the top card of his or her library into his or her graveyard.","6","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Raging Ravine","Raging Ravine enters the battlefield tapped.
{T}: Add {R} or {G} to your mana pool.
{2}{R}{G}: Until end of turn, Raging Ravine becomes a 3/3 red and green Elemental creature with ""Whenever this creature attacks, put a +1/+1 counter on it."" It's still a land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Elves","1","1","G","{T}: Add {G} to your mana pool.","G","Creature - Elf Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Zuran Orb","Sacrifice a land: You gain 2 life.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bramble Creeper","0","3","G","Whenever Bramble Creeper attacks, it gets +5/+0 until end of turn.","4G","Creature - Elemental");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Primal Clay","*","*","As Primal Clay enters the battlefield, it becomes your choice of a 3/3 artifact creature, a 2/2 artifact creature with flying, or a 1/6 Wall artifact creature with defender in addition to its other types.","4","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Luminescent Rain","G","Choose a creature type. You gain 2 life for each permanent you control of that type.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shattered Crypt","B","Return X target creature cards from your graveyard to your hand. You lose X life.","XBB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tangle Wire","Fading 4 (This artifact enters the battlefield with four fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
At the beginning of each player's upkeep, that player taps an untapped artifact, creature, or land he or she controls for each fade counter on Tangle Wire.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mirror of Fate","{T}, Sacrifice Mirror of Fate: Choose up to seven face-up exiled cards you own. Exile all the cards from your library, then put the chosen cards on top of your library.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boreal Centaur","2","2","G","{S}i}: Boreal Centaur gets +1/+1 until end of turn. Activate this ability only once each turn. ({S}i} can be paid with one mana from a snow permanent.)","1G","Snow Creature - Centaur Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunted Phantasm","4","6","U","Hunted Phantasm is unblockable.
When Hunted Phantasm enters the battlefield, put five 1/1 red Goblin creature tokens onto the battlefield under target opponent's control.","1UU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathspore Thallid","1","1","B","At the beginning of your upkeep, put a spore counter on Deathspore Thallid.
Remove three spore counters from Deathspore Thallid: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: Target creature gets -1/-1 until end of turn.","1B","Creature - Zombie Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shared Animosity","R","Whenever a creature you control attacks, it gets +1/+0 until end of turn for each other attacking creature that shares a creature type with it.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devoted Retainer","1","1","W","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","W","Creature - Human Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mourning Thrull","1","1","WB","({(w/b)} can be paid with either {W} or {B}.)
Flying
Whenever Mourning Thrull deals damage, you gain that much life.","1(W/B)","Creature - Thrull");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battleground Geist","3","3","U","Flying
Other Spirit creatures you control get +1/+0.","4U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chaos Moon","R","At the beginning of each upkeep, count the number of permanents. If the number is odd, until end of turn, red creatures get +1/+1 and whenever a player taps a Mountain for mana, that player adds {R} to his or her mana pool (in addition to the mana the land produces). If the number is even, until end of turn, red creatures get -1/-1 and if a player taps a Mountain for mana, that Mountain produces colorless mana instead of any other type.","3R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Commando Raid","R","Until end of turn, target creature you control gains ""When this creature deals combat damage to a player, you may have it deal damage equal to its power to target creature that player controls.""","2R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Agyrem","Whenever a white creature dies, return it to the battlefield under its owner's control at the beginning of the next end step.
Whenever a nonwhite creature dies, return it to its owner's hand at the beginning of the next end step.
Whenever you roll {C}, creatures can't attack you until a player planeswalks.",NULL,"Plane - Ravnica");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Minion of the Wastes","*","*","B","Trample
As Minion of the Wastes enters the battlefield, pay any amount of life.
Minion of the Wastes's power and toughness are each equal to the life paid as it entered the battlefield.","3BBB","Creature - Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Soldier","2","1","R","Whenever Dwarven Soldier blocks or becomes blocked by one or more Orcs, Dwarven Soldier gets +0/+2 until end of turn.","1R","Creature - Dwarf Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boggart Arsonists","2","1","R","Plainswalk
{2}{R}, Sacrifice Boggart Arsonists: Destroy target Scarecrow or Plains.","2R","Creature - Goblin Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Imprison","B","Enchant creature
Whenever a player activates an ability of enchanted creature with {T} in its activation cost that isn't a mana ability, you may pay {1}. If you do, counter that ability. If you don't, destroy Imprison.
Whenever enchanted creature attacks or blocks, you may pay {1}. If you do, tap the creature, remove it from combat, and creatures it was blocking that had become blocked by only that creature this combat become unblocked. If you don't, destroy Imprison.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razorfin Hunter","1","1","UR","{T}: Razorfin Hunter deals 1 damage to target creature or player.","UR","Creature - Merfolk Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Char-Rumbler","-1","3","R","Double strike
{R}: Char-Rumbler gets +1/+0 until end of turn.","2RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fleshgrafter","2","2","B","Discard an artifact card: Fleshgrafter gets +2/+2 until end of turn.","2B","Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tahngarth's Glare","R","Look at the top three cards of target opponent's library, then put them back in any order. That player looks at the top three cards of your library, then puts them back in any order.","R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trigon of Thought","Trigon of Thought enters the battlefield with three charge counters on it.
{U}{U}, {T}: Put a charge counter on Trigon of Thought.
{2}, {T}, Remove a charge counter from Trigon of Thought: Draw a card.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodghast","2","1","B","Bloodghast can't block.
Bloodghast has haste as long as an opponent has 10 or less life.
Landfall - Whenever a land enters the battlefield under your control, you may return Bloodghast from your graveyard to the battlefield.","BB","Creature - Vampire Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Final-Sting Faerie","2","2","B","Flying
When Final-Sting Faerie enters the battlefield, destroy target creature that was dealt damage this turn.","3B","Creature - Faerie Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mosstodon","5","3","G","{1}: Target creature with power 5 or greater gains trample until end of turn.","4G","Creature - Plant Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rakeclaw Gargantuan","5","3","WRG","{1}: Target creature with power 5 or greater gains first strike until end of turn.","2RGW","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overrun","G","Creatures you control get +3/+3 and gain trample until end of turn. (If a creature you control would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","2GGG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gut Shot","R","({(r/p)} can be paid with either {R} or 2 life.)
Gut Shot deals 1 damage to target creature or player.","(R/P)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Anatomist","1","2","U","{2}{U}, {T}: Put a -1/-1 counter on target creature. You may tap or untap that creature.","2U","Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Mourncaller","2","2","G","Whenever an attacking Kithkin or Elf is put into your graveyard from the battlefield, you may draw a card.","2G","Creature - Kithkin Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Braidwood Cup","{T}: You gain 1 life.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tazeem","Creatures can't block.
Whenever you roll {C}, draw a card for each land you control.",NULL,"Plane - Zendikar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sokenzan Renegade","3","3","R","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
At the beginning of your upkeep, if a player has more cards in hand than each other player, the player who has the most cards in hand gains control of Sokenzan Renegade.","2R","Creature - Ogre Samurai Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Torment","B","Enchant creature
Enchanted creature gets -3/-0.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Naturalize","G","Destroy target artifact or enchantment.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gisela, Blade of Goldnight","5","5","WR","Flying, first strike
If a source would deal damage to an opponent or a permanent an opponent controls, that source deals double that damage to that player or permanent instead.
If a source would deal damage to you or a permanent you control, prevent half that damage, rounded up.","4RWW","Legendary Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flourishing Defenses","G","Whenever a -1/-1 counter is placed on a creature, you may put a 1/1 green Elf Warrior creature token onto the battlefield.","4G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wing Splicer","1","1","U","When Wing Splicer enters the battlefield, put a 3/3 colorless Golem artifact creature token onto the battlefield.
Golem creatures you control have flying.","3U","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tajuru Preserver","2","1","G","Spells and abilities your opponents control can't cause you to sacrifice permanents.","1G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Fury","3","5","W","Flying
When Angel of Fury is put into your graveyard from the battlefield, you may shuffle it into your library.","4WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Necropede","1","1","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
When Necropede dies, you may put a -1/-1 counter on target creature.","2","Artifact Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grimgrin, Corpse-Born","5","5","UB","Grimgrin, Corpse-Born enters the battlefield tapped and doesn't untap during your untap step.
Sacrifice another creature: Untap Grimgrin and put a +1/+1 counter on it.
Whenever Grimgrin attacks, destroy target creature defending player controls, then put a +1/+1 counter on Grimgrin.","3UB","Legendary Creature - Zombie Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost Tactician","2","5","W","{W}, {T}, Discard a card: Creatures you control get +1/+0 until end of turn.","4W","Creature - Spirit Spellshaper");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phage the Untouchable Avatar","Pay 1 life: Until end of turn, whenever a creature deals combat damage to you, destroy that creature.
Pay 1 life: Until end of turn, whenever target creature you control deals combat damage to a creature, destroy the damaged creature.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pendrell Flux","U","Enchant creature
Enchanted creature has ""At the beginning of your upkeep, sacrifice this creature unless you pay its mana cost.""","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razorfoot Griffin","2","2","W","Flying
First strike (This creature deals combat damage before creatures without first strike.)","3W","Creature - Griffin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Winter Orb","Players can't untap more than one land during their untap steps.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Godless Shrine","({T}: Add {W} or {B} to your mana pool.)
As Godless Shrine enters the battlefield, you may pay 2 life. If you don't, Godless Shrine enters the battlefield tapped.",NULL,"Land - Plains Swamp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Supply // Demand","WU","Search your library for a multicolored card, reveal it, and put it into your hand. Then shuffle your library.
---
Put X 1/1 green Saproling creature tokens onto the battlefield.","1WU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sadistic Glee","B","Enchant creature
Whenever a creature dies, put a +1/+1 counter on enchanted creature.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Water","0","5","U","Defender (This creature can't attack.)
{U}: Wall of Water gets +1/+0 until end of turn.","1UU","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Might Weaver","2","1","G","{2}: Target red or white creature gains trample until end of turn. (If the creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","1G","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frenzy Sliver","1","1","B","All Sliver creatures have frenzy 1. (Whenever a Sliver attacks and isn't blocked, it gets +1/+0 until end of turn.)","1B","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crater Hellion","6","6","R","Echo {4}{R}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Crater Hellion enters the battlefield, it deals 4 damage to each other creature.","4RR","Creature - Hellion Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Narcomoeba","1","1","U","Flying
When Narcomoeba is put into your graveyard from your library, you may put it onto the battlefield.","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Etherwrought Page","WUB","At the beginning of your upkeep, choose one - You gain 2 life; or look at the top card of your library, then you may put that card into your graveyard; or each opponent loses 1 life.","1WUB","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Insubordination","B","Enchant creature
At the beginning of the end step of enchanted creature's controller, Insubordination deals 2 damage to that player unless that creature attacked this turn.","BB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Colony","0","0","G","Spike Colony enters the battlefield with four +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Colony: Put a +1/+1 counter on target creature.","4G","Creature - Spike");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dramatic Entrance","G","You may put a green creature card from your hand onto the battlefield.","3GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbarian Horde","3","3","R",NULL,"3R","Creature - Human Barbarian Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonic Appetite","B","Enchant creature you control
Enchanted creature gets +3/+3.
At the beginning of your upkeep, sacrifice a creature.","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Horrifying Revelation","B","Target player discards a card, then puts the top card of his or her library into his or her graveyard.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sulam Djinn","6","6","G","Trample
Sulam Djinn gets -2/-2 as long as green is the most common color among all permanents or is tied for most common.","5G","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mothdust Changeling","1","1","U","Changeling (This card is every creature type at all times.)
Tap an untapped creature you control: Mothdust Changeling gains flying until end of turn.","U","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ferropede","1","1","Ferropede is unblockable.
Whenever Ferropede deals combat damage to a player, you may remove a counter from target permanent.","3","Artifact Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Berserk Murlodont","3","3","G","Whenever a Beast becomes blocked, it gets +1/+1 until end of turn for each creature blocking it.","4G","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Choose Your Champion","When you set this scheme in motion, target opponent chooses a player. Until your next turn, only you and the chosen player can cast spells and attack with creatures.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul Charmer","2","2","W","Whenever Soul Charmer deals combat damage to a creature, you gain 2 life unless that creature's controller pays {2}.","2W","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krovikan Mist","*","*","U","Flying
Krovikan Mist's power and toughness are each equal to the number of Illusions on the battlefield.","1U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anaba Shaman","2","2","R","{R}, {T}: Anaba Shaman deals 1 damage to target creature or player.","3R","Creature - Minotaur Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bone Shredder","1","1","B","Flying
Echo {2}{B} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Bone Shredder enters the battlefield, destroy target nonartifact, nonblack creature.","2B","Creature - Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Shaman","1","1","U","{T}: Change the text of target white enchantment you control that doesn't have cumulative upkeep by replacing all instances of one color word with another. (For example, you may change ""black creatures can't attack"" to ""blue creatures can't attack."") That enchantment gains ""Cumulative upkeep {1}."" (At the beginning of its controller's upkeep, that player puts an age counter on it, then sacrifices it unless he or she pays its upkeep cost for each age counter on it.)","U","Creature - Human Cleric Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Medicine Bag","{1}, {T}, Discard a card: Regenerate target creature.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("No Mercy","B","Whenever a creature deals damage to you, destroy it.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merchant Ship","0","2","U","Merchant Ship can't attack unless defending player controls an Island.
Whenever Merchant Ship attacks and isn't blocked, you gain 2 life.
When you control no Islands, sacrifice Merchant Ship.","U","Creature - Human");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Mindservant","1","1","{2}, {T}: Shuffle your library.","1","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Drain","UB","Target player puts the top X cards of his or her library into his or her graveyard and you gain X life.","XUB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sigiled Behemoth","5","4","WG","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","4GW","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Task Mage Assembly","R","When there are no creatures on the battlefield, sacrifice Task Mage Assembly.
{2}: Task Mage Assembly deals 1 damage to target creature. Any player may activate this ability but only any time he or she could cast a sorcery.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Damping Field","W","Players can't untap more than one artifact during their untap steps.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Offering","B","Destroy target nonblack creature. You gain 3 life.","4BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nim Shrieker","0","1","B","Flying
Nim Shrieker gets +1/+0 for each artifact you control.","3B","Creature - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Unwinding Clock","Untap all artifacts you control during each other player's untap step.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Luminous Wake","W","Enchant creature
Whenever enchanted creature attacks or blocks, you gain 4 life.","2W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Look Skyward and Despair","When you set this scheme in motion, put a 5/5 red Dragon creature token with flying onto the battlefield.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Graven Dominator","4","4","W","Flying
Haunt (When this creature dies, exile it haunting target creature.)
When Graven Dominator enters the battlefield or the creature it haunts dies, each other creature becomes 1/1 until end of turn.","4WW","Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Okina Nightwatch","4","3","G","As long as you have more cards in hand than each opponent, Okina Nightwatch gets +3/+3.","4G","Creature - Human Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Earsplitting Rats","2","1","B","When Earsplitting Rats enters the battlefield, each player discards a card.
Discard a card: Regenerate Earsplitting Rats.","3B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonybrook Schoolmaster","1","2","W","Whenever Stonybrook Schoolmaster becomes tapped, you may put a 1/1 blue Merfolk Wizard creature token onto the battlefield.","2W","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Righteous Cause","W","Whenever a creature attacks, you gain 1 life.","3WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Roaring Slagwurm","6","4","G","Whenever Roaring Slagwurm attacks, tap all artifacts.","5GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trumpeting Armodon","3","3","G","{1}{G}: Target creature blocks Trumpeting Armodon this turn if able.","3G","Creature - Elephant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shimmering Grotto","{T}: Add {1} to your mana pool.
{1}, {T}: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Retaliator Griffin","2","2","WRG","Flying
Whenever a source an opponent controls deals damage to you, you may put that many +1/+1 counters on Retaliator Griffin.","1RGW","Creature - Griffin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lodestone Bauble","{1}, {T}, Sacrifice Lodestone Bauble: Put up to four target basic land cards from a player's graveyard on top of his or her library in any order. That player draws a card at the beginning of the next turn's upkeep.","0","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Okk","4","4","R","Okk can't attack unless a creature with greater power also attacks.
Okk can't block unless a creature with greater power also blocks.","1R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Empyrial Plate","Equipped creature gets +1/+1 for each card in your hand.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravegouger","2","2","B","When Gravegouger enters the battlefield, exile up to two target cards from a single graveyard.
When Gravegouger leaves the battlefield, return the exiled cards to their owner's graveyard.","2B","Creature - Nightmare Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Domestication","U","Enchant creature
You control enchanted creature.
At the beginning of your end step, if enchanted creature's power is 4 or greater, sacrifice Domestication.","2UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Songstitcher","1","1","W","{1}{W}: Prevent all combat damage that would be dealt this turn by target attacking creature with flying.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Grappler","1","1","R","Provoke (When this attacks, you may have target creature defending player controls untap and block it if able.)","R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for the Nihil Stone","B","Whenever an opponent discards a card, you may put a quest counter on Quest for the Nihil Stone.
At the beginning of each opponent's upkeep, if that player has no cards in hand and Quest for the Nihil Stone has two or more quest counters on it, you may have that player lose 5 life.","B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flame Wave","R","Flame Wave deals 4 damage to target player and each creature he or she controls.","3RRRR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hall of the Bandit Lord","Hall of the Bandit Lord enters the battlefield tapped.
{T}, Pay 3 life: Add {1} to your mana pool. If that mana is spent on a creature spell, it gains haste.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sir Shandlar of Eberyn","4","7","WG",NULL,"4GW","Legendary Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snakeform","UG","Target creature loses all abilities and becomes a 1/1 green Snake until end of turn.
Draw a card.","2(G/U)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reshape","U","As an additional cost to cast Reshape, sacrifice an artifact.
Search your library for an artifact card with converted mana cost X or less and put it onto the battlefield. Then shuffle your library.","XUU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sage Owl","1","1","U","Flying
When Sage Owl enters the battlefield, look at the top four cards of your library, then put them back in any order.","1U","Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sokenzan","All creatures get +1/+1 and have haste.
Whenever you roll {C}, untap all creatures that attacked this turn. After this main phase, there is an additional combat phase followed by an additional main phase.",NULL,"Plane - Kamigawa");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Warriors","1","1","R","{T}: Target creature with power 2 or less is unblockable this turn.","2R","Creature - Dwarf Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mystic Compass","{1}, {T}: Target land becomes the basic land type of your choice until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brain Gorgers","4","2","B","When you cast Brain Gorgers, any player may sacrifice a creature. If a player does, counter Brain Gorgers.
Madness {1}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","3B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Balance","W","Each player chooses a number of lands he or she controls equal to the number of lands controlled by the player who controls the fewest, then sacrifices the rest. Players discard cards and sacrifice creatures the same way.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greater Morphling","5","5","U","{2}: Greater Morphling gains your choice of banding, bushido 1, double strike, fear, flying, first strike, haste, landwalk of your choice, protection from a color of your choice, provoke, rampage 1, shadow, or trample until end of turn.
{2}: Greater Morphling becomes the colors of your choice until end of turn.
{2}: Greater Morphling's type becomes the creature type of your choice until end of turn.
{2}: Greater Morphling's expansion symbol becomes the symbol of your choice until end of turn.
{2}: Greater Morphling's artist becomes the artist of your choice until end of turn.
{2}: Greater Morphling gets +2/-2 or -2/+2 until end of turn.
{2}: Untap Greater Morphling.","6UU","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quash","U","Counter target instant or sorcery spell. Search its controller's graveyard, hand, and library for all cards with the same name as that spell and exile them. Then that player shuffles his or her library.","2UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unhallowed Pact","B","Enchant creature
When enchanted creature dies, return that card to the battlefield under your control.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Put Away","U","Counter target spell. You may shuffle up to one target card from your graveyard into your library.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spurred Wolverine","3","2","R","Tap two untapped Beasts you control: Target creature gains first strike until end of turn.","4R","Creature - Wolverine Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Green Mana Battery","{2}, {T}: Put a charge counter on Green Mana Battery.
{T}, Remove any number of charge counters from Green Mana Battery: Add {G} to your mana pool, then add an additional {G} to your mana pool for each charge counter removed this way.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Obsessive Search","U","Draw a card.
Madness {U} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deadshot Minotaur","3","4","RG","When Deadshot Minotaur enters the battlefield, it deals 3 damage to target creature with flying.
Cycling {(r/g)} ({(r/g)}, Discard this card: Draw a card.)","3RG","Creature - Minotaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ob Nixilis, the Fallen","3","3","B","Landfall - Whenever a land enters the battlefield under your control, you may have target player lose 3 life. If you do, put three +1/+1 counters on Ob Nixilis, the Fallen.","3BB","Legendary Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kirtar's Desire","W","Enchant creature
Enchanted creature can't attack.
Threshold - Enchanted creature can't block as long as seven or more cards are in your graveyard.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hedron Crab","0","2","U","Landfall - Whenever a land enters the battlefield under your control, target player puts the top three cards of his or her library into his or her graveyard.","U","Creature - Crab");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Summoner's Egg","0","4","Imprint - When Summoner's Egg enters the battlefield, you may exile a card from your hand face down.
When Summoner's Egg dies, turn the exiled card face up. If it's a creature card, put it onto the battlefield under your control.","4","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grim Feast","BG","At the beginning of your upkeep, Grim Feast deals 1 damage to you.
Whenever a creature is put into an opponent's graveyard from the battlefield, you gain life equal to its toughness.","1BG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avatar of Fury","6","6","R","If an opponent controls seven or more lands, Avatar of Fury costs {6} less to cast.
Flying
{R}: Avatar of Fury gets +1/+0 until end of turn.","6RR","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellkite Overlord","8","8","BRG","Flying, trample, haste
{R}: Hellkite Overlord gets +1/+0 until end of turn.
{B}{G}: Regenerate Hellkite Overlord.","4BRRG","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thoughtweft Trio","5","5","W","First strike, vigilance
Champion a Kithkin (When this enters the battlefield, sacrifice it unless you exile another Kithkin you control. When this leaves the battlefield, that card returns to the battlefield.)
Thoughtweft Trio can block any number of creatures.","2WW","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Second Sight","U","Choose one - Look at the top five cards of target opponent's library, then put them back in any order; or look at the top five cards of your library, then put them back in any order.
Entwine {U} (Choose both if you pay the entwine cost.)","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gempalm Strider","2","2","G","Cycling {2}{G}{G} ({2}{G}{G}, Discard this card: Draw a card.)
When you cycle Gempalm Strider, Elf creatures get +2/+2 until end of turn.","1G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Living Hive","6","6","G","Trample
Whenever Living Hive deals combat damage to a player, put that many 1/1 green Insect creature tokens onto the battlefield.","6GG","Creature - Elemental Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Command of Unsummoning","U","Cast Command of Unsummoning only during the declare attackers step and only if you've been attacked this step.
Return one or two target attacking creatures to their owner's hand.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rally the Peasants","W","Creatures you control get +2/+0 until end of turn.
Flashback {2}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rockshard Elemental","4","3","R","Double strike (This creature deals both first-strike and regular combat damage.)
Morph {4}{R}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","5RR","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Aeolipile","{1}, {T}, Sacrifice Aeolipile: Aeolipile deals 2 damage to target creature or player.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oath of Mages","R","At the beginning of each player's upkeep, that player chooses target player who has more life than he or she does and is his or her opponent. The first player may have Oath of Mages deal 1 damage to the second player.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Swirling Sandstorm","R","Threshold - Swirling Sandstorm deals 5 damage to each creature without flying if seven or more cards are in your graveyard.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Welkin Guide","2","2","W","Flying
When Welkin Guide enters the battlefield, target creature gets +2/+2 and gains flying until end of turn.","4W","Creature - Bird Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reckless Wurm","4","4","R","Trample
Madness {2}{R} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","3RR","Creature - Wurm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blasted Landscape","{T}: Add {1} to your mana pool.
Cycling {2} ({2}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beacon of Tomorrows","U","Target player takes an extra turn after this one. Shuffle Beacon of Tomorrows into its owner's library.","6UU","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hexplate Golem","5","7",NULL,"7","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Torturer","1","1","B","{B}, {T}: Target creature gets -1/-1 until end of turn.
Threshold - {3}{B}{B}, {T}: Target creature gets -2/-2 until end of turn. Activate this ability only if seven or more cards are in your graveyard.","1BB","Creature - Human Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reinforcements","W","Put up to three target creature cards from your graveyard on top of your library.","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Adventurers' Guildhouse","Green legendary creatures you control have ""bands with other legendary creatures."" (Any legendary creatures can attack in a band as long as at least one has ""bands with other legendary creatures."" Bands are blocked as a group. If at least two legendary creatures you control, one of which has ""bands with other legendary creatures,"" are blocking or being blocked by the same creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kashi-Tribe Reaver","3","2","G","Whenever Kashi-Tribe Reaver deals combat damage to a creature, tap that creature and it doesn't untap during its controller's next untap step.
{1}{G}: Regenerate Kashi-Tribe Reaver.","3G","Creature - Snake Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Acridian","2","4","G","Echo {1}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","1G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jwari Scuttler","2","3","U",NULL,"2U","Creature - Crab");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Laughing Hyena","2","2","G","Gotcha Whenever an opponent laughs, you may say ""Gotcha"" If you do, return Laughing Hyena from your graveyard to your hand.","1G","Creature - Hyena");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crookclaw Transmuter","3","1","U","Flash (You may cast this spell any time you could cast an instant.)
Flying
When Crookclaw Transmuter enters the battlefield, switch target creature's power and toughness until end of turn.","3U","Creature - Bird Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Everlasting Torment","BR","Players can't gain life.
Damage can't be prevented.
All damage is dealt as though its source had wither. (A source with wither deals damage to creatures in the form of -1/-1 counters.)","2(B/R)","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wine of Blood and Iron","{4}: Target creature gets +X/+0 until end of turn, where X is its power. Sacrifice Wine of Blood and Iron at the beginning of the next end step.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grave Defiler","2","1","B","When Grave Defiler enters the battlefield, reveal the top four cards of your library. Put all Zombie cards revealed this way into your hand and the rest on the bottom of your library in any order.
{1}{B}: Regenerate Grave Defiler.","3B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Mobile Fort","0","6","Defender (This creature can't attack.)
{3}: Mobile Fort gets +3/-1 until end of turn and can attack this turn as though it didn't have defender. Activate this ability only once each turn.","4","Artifact Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Folk Medicine","G","You gain 1 life for each creature you control.
Flashback {1}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Noxious Vapors","B","Each player reveals his or her hand and chooses one card of each color from it, then discards all other nonland cards.","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Char","R","Char deals 4 damage to target creature or player and 2 damage to you.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master of the Veil","2","3","U","Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Master of the Veil is turned face up, you may turn target creature with morph face down.","2UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aboshan, Cephalid Emperor","3","3","U","Tap an untapped Cephalid you control: Tap target permanent.
{U}{U}{U}: Tap all creatures without flying.","4UU","Legendary Creature - Cephalid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trigon of Corruption","Trigon of Corruption enters the battlefield with three charge counters on it.
{B}{B}, {T}: Put a charge counter on Trigon of Corruption.
{2}, {T}, Remove a charge counter from Trigon of Corruption: Put a -1/-1 counter on target creature.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Weight of Conscience","W","Enchant creature
Enchanted creature can't attack.
Tap two untapped creatures you control that share a creature type: Exile enchanted creature.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Giant","3","3","R","{R}: Flowstone Giant gets +2/-2 until end of turn.","2RR","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squirrel Farm","G","o1oG: Choose a card in your hand. Covering the artist's name, reveal the card to target player. If that player cannot name the artist, reveal the artist's name and put a Squirrel token into play. Treat this token as a 1/1 green creature.","2G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pearl Medallion","White spells you cast cost {1} less to cast.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream Twist","U","Target player puts the top three cards of his or her library into his or her graveyard.
Flashback {1}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Null Champion","1","1","B","Level up {3} ({3}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-3
4/2
LEVEL 4+
7/3
{B}: Regenerate Null Champion.","1B","Creature - Zombie Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dragon Blood","{3}, {T}: Put a +1/+1 counter on target creature.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Bombardment","R","Sacrifice a creature: Goblin Bombardment deals 1 damage to target creature or player.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sulfurous Blast","R","Sulfurous Blast deals 2 damage to each creature and each player. If you cast this spell during your main phase, Sulfurous Blast deals 3 damage to each creature and each player instead.","2RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Capture of Jingzhou","U","Take an extra turn after this one.","3UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unholy Strength","B","Enchant creature
Enchanted creature gets +2/+1.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Scissors Lizard","4","3","Paper Tigers cannot attack or block.","4","Artifact Creature");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodpyre Elemental","4","1","R","Sacrifice Bloodpyre Elemental: Bloodpyre Elemental deals 4 damage to target creature. Activate this ability only any time you could cast a sorcery.","4R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("AEthersnipe","4","4","U","When Æthersnipe enters the battlefield, return target nonland permanent to its owner's hand.
Evoke {1}{U}{U} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","5U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sever Soul","B","Destroy target nonblack creature. It can't be regenerated. You gain life equal to its toughness.","3BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wrath of God","W","Destroy all creatures. They can't be regenerated.","2WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deadfall","G","Creatures with forestwalk can be blocked as though they didn't have forestwalk.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Peace Strider","3","3","When Peace Strider enters the battlefield, you gain 3 life.","4","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Glimmervoid","At the beginning of the end step, if you control no artifacts, sacrifice Glimmervoid.
{T}: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hamlet Captain","2","2","G","Whenever Hamlet Captain attacks or blocks, other Human creatures you control get +1/+1 until end of turn.","1G","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hoof Skulkin","2","2","{3}: Target green creature gets +1/+1 until end of turn.","3","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bestial Menace","G","Put a 1/1 green Snake creature token, a 2/2 green Wolf creature token, and a 3/3 green Elephant creature token onto the battlefield.","3GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Oath","R","Choose a card type. Target opponent reveals his or her hand. Blood Oath deals 3 damage to that player for each card of the chosen type revealed this way. (Artifact, creature, enchantment, instant, land, planeswalker, sorcery, and tribal are card types.)","3R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serra Bestiary","W","Enchant creature
At the beginning of your upkeep, sacrifice Serra Bestiary unless you pay {W}{W}.
Enchanted creature can't attack or block, and its activated abilities with {T} in their costs can't be activated.","WW","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forbidden Ritual","B","Sacrifice a nontoken permanent. If you do, target opponent loses 2 life unless he or she sacrifices a permanent or discards a card. You may repeat this process any number of times.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mudbrawler Cohort","1","1","R","Haste
Mudbrawler Cohort gets +1/+1 as long as you control another red creature.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ivory Mask","W","You have shroud. (You can't be the target of spells or abilities.)","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pestilent Kathari","1","1","B","Flying
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)
{2}{R}: Pestilent Kathari gains first strike until end of turn.","2B","Creature - Bird Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Granite Gargoyle","2","2","R","Flying
{R}: Granite Gargoyle gets +0/+1 until end of turn.","2R","Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sakiko, Mother of Summer","3","3","G","Whenever a creature you control deals combat damage to a player, add that much {G} to your mana pool. Until end of turn, this mana doesn't empty from your mana pool as steps and phases end.","4GG","Legendary Creature - Snake Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aliban's Tower","R","Target blocking creature gets +3/+1 until end of turn.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Iron Will","W","Target creature gets +0/+4 until end of turn.
Cycling {2} ({2}, Discard this card: Draw a card.)","W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mox Pearl","{T}: Add {W} to your mana pool.","0","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rugged Prairie","{T}: Add {1} to your mana pool.
{(r/w)}, {T}: Add {R}{R}, {R}{W}, or {W}{W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Impelled Giant","3","3","R","Trample
Tap an untapped red creature you control other than Impelled Giant: Impelled Giant gets +X/+0 until end of turn, where X is the power of the creature tapped this way.","4RR","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scavenged Weaponry","B","Enchant creature
When Scavenged Weaponry enters the battlefield, draw a card.
Enchanted creature gets +1/+1.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Librarian","3","3","B","Flying, trample
At the beginning of your upkeep, remove the top card of your library from the game face up and balance it on your body.
When a balanced card falls or touches another balanced card, sacrifice Phyrexian Librarian.","3B","Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Surrender Your Thoughts","When you set this scheme in motion, target opponent chooses self or others. If that player chooses self, he or she discards four cards. If the player chooses others, each of your other opponents discards two cards.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Survivors","4","6","W","When Auriok Survivors enters the battlefield, you may return target Equipment card from your graveyard to the battlefield. If you do, you may attach it to Auriok Survivors.","5W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recuperate","W","Choose one - You gain 6 life; or prevent the next 6 damage that would be dealt to target creature this turn.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blighted Shaman","1","1","B","{T}, Sacrifice a Swamp: Target creature gets +1/+1 until end of turn.
{T}, Sacrifice a creature: Target creature gets +2/+2 until end of turn.","1B","Creature - Human Cleric Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Murderous Redcap","2","2","BR","When Murderous Redcap enters the battlefield, it deals damage equal to its power to target creature or player.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","2(B/R)(B/R)","Creature - Goblin Assassin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Serpent Generator","{4}, {T}: Put a 1/1 colorless Snake artifact creature token onto the battlefield. It has ""Whenever this creature deals damage to a player, that player gets a poison counter."" (A player with ten or more poison counters loses the game.)","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Berserker","4","2","R","Haste","4R","Creature - Ogre Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Metamorphosis","G","As an additional cost to cast Metamorphosis, sacrifice a creature.
Add X mana of any one color to your mana pool, where X is one plus the sacrificed creature's converted mana cost. Spend this mana only to cast creature spells.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crush of Wurms","G","Put three 6/6 green Wurm creature tokens onto the battlefield.
Flashback {9}{G}{G}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","6GGG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost-Lit Raider","2","1","R","{2}{R}, {T}: Ghost-Lit Raider deals 2 damage to target creature.
Channel - {3}{R}, Discard Ghost-Lit Raider: Ghost-Lit Raider deals 4 damage to target creature.","2R","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nephalia Smuggler","1","1","U","{3}{U}, {T}: Exile another target creature you control, then return that card to the battlefield under your control.","U","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primeval Light","G","Destroy all enchantments target player controls.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deranged Outcast","2","1","G","{1}{G}, Sacrifice a Human: Put two +1/+1 counters on target creature.","1G","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planeswalker's Favor","G","{3}{G}: Target opponent reveals a card at random from his or her hand. Target creature gets +X/+X until end of turn, where X is the revealed card's converted mana cost.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kederekt Leviathan","5","5","U","When Kederekt Leviathan enters the battlefield, return all other nonland permanents to their owners' hands.
Unearth {6}{U} ({6}{U}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","6UU","Creature - Leviathan");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Time Bomb","At the beginning of your upkeep, put a time counter on Time Bomb.
{1}, {T}, Sacrifice Time Bomb: Time Bomb deals damage equal to the number of time counters on it to each creature and each player.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battlegate Mimic","2","1","WR","Whenever you cast a spell that's both red and white, Battlegate Mimic becomes 4/2 and gains first strike until end of turn.","1(R/W)","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Simian Grunts","3","4","G","Flash
Echo {2}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","2G","Creature - Ape");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Prototype","2","2","At the beginning of your upkeep, put a +1/+1 counter on Myr Prototype.
Myr Prototype can't attack or block unless you pay {1} for each +1/+1 counter on it.","5","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Worker","0","0","G","Spike Worker enters the battlefield with two +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Worker: Put a +1/+1 counter on target creature.","2G","Creature - Spike");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mycoloth","4","4","G","Devour 2 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with twice that many +1/+1 counters on it.)
At the beginning of your upkeep, put a 1/1 green Saproling creature token onto the battlefield for each +1/+1 counter on Mycoloth.","3GG","Creature - Fungus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Archon","2","2","B","{B}, Sacrifice a Cleric: Target player loses 2 life and you gain 2 life.","2B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sadistic Sacrament","B","Kicker {7} (You may pay an additional {7} as you cast this spell.)
Search target player's library for up to three cards, exile them, then that player shuffles his or her library. If Sadistic Sacrament was kicked, instead search that player's library for up to fifteen cards, exile them, then that player shuffles his or her library.","BBB","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Urza's Engine","1","5","Trample
{3}: Urza's Engine gains banding until end of turn. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
{3}: Attacking creatures banded with Urza's Engine gain trample until end of turn.","5","Artifact Creature - Juggernaut");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sluggishness","R","Enchant creature
Enchanted creature can't block.
When Sluggishness is put into a graveyard from the battlefield, return Sluggishness to its owner's hand.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sun Clasp","W","Enchant creature
Enchanted creature gets +1/+3.
{W}: Return enchanted creature to its owner's hand.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emissary of Hope","2","1","W","Flying
Whenever Emissary of Hope deals combat damage to a player, you gain 1 life for each artifact that player controls.","1WW","Creature - Spirit");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Emblazoned Golem","1","2","Kicker {X} (You may pay an additional {X} as you cast this spell.)
Spend only colored mana on X. No more than one mana of each color may be spent this way.
If Emblazoned Golem was kicked, it enters the battlefield with X +1/+1 counters on it.","2","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Suntail Hawk","1","1","W","Flying (This creature can't be blocked except by creatures with flying or reach.)","W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reckless Waif","1","1","R","At the beginning of each upkeep, if no spells were cast last turn, transform Reckless Waif.","R","Creature - Human Rogue Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ladies' Knight","2","2","W","Flying
Spells that players wearing at least one item of women's clothing play cost {1} less to play. (Women's clothing is designed to be worn exclusively by women.)","3W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inferno Elemental","4","4","R","Whenever Inferno Elemental blocks or becomes blocked by a creature, Inferno Elemental deals 3 damage to that creature.","4RR","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Veldt","Veldt doesn't untap during your untap step if it has a depletion counter on it.
At the beginning of your upkeep, remove a depletion counter from Veldt.
{T}: Add {G} or {W} to your mana pool. Put a depletion counter on Veldt.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serendib Efreet","3","4","U","Flying
At the beginning of your upkeep, Serendib Efreet deals 1 damage to you.","2U","Creature - Efreet");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Snorting Gahr","3","3","G","Whenever Snorting Gahr becomes blocked, it gets +2/+2 until end of turn.","2GG","Creature - Rhino Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reconnaissance","W","{0}: Remove target attacking creature you control from combat and untap it.","W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Volrath's Shapeshifter","0","1","U","As long as the top card of your graveyard is a creature card, Volrath's Shapeshifter has the full text of that card and has the text ""{2}: Discard a card."" (Volrath's Shapeshifter has that card's name, mana cost, color, types, abilities, power, and toughness.)
{2}: Discard a card.","1UU","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wake Thrasher","1","1","U","Whenever a permanent you control becomes untapped, Wake Thrasher gets +1/+1 until end of turn.","2U","Creature - Merfolk Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyromancy","R","{3}, Discard a card at random: Pyromancy deals damage to target creature or player equal to the converted mana cost of the discarded card.","2RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deathgrip","B","{B}{B}: Counter target green spell.","BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("B.F.M.","99","99","B","B.F.M. into play. If either B.F.M. card
more creatures.","BBBBBBBBBBBBBBB","Scariest Creature You'll Ever See");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ratcatcher","4","4","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
At the beginning of your upkeep, you may search your library for a Rat card, reveal it, and put it into your hand. If you do, shuffle your library.","4BB","Creature - Ogre Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reya Dawnbringer","4","6","W","Flying
At the beginning of your upkeep, you may return target creature card from your graveyard to the battlefield.","6WWW","Legendary Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("I Delight in Your Convulsions","When you set this scheme in motion, each opponent loses 3 life. You gain life equal to the life lost this way.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Supreme Inquisitor","1","3","U","Tap five untapped Wizards you control: Search target player's library for up to five cards and exile them. Then that player shuffles his or her library.","3UU","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opal Avenger","W","When you have 10 or less life, if Opal Avenger is an enchantment, Opal Avenger becomes a 3/5 Soldier creature.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tempting Licid","2","2","G","{G}, {T}: Tempting Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {G} to end this effect.
All creatures able to block enchanted creature do so.","2G","Creature - Licid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashen Monstrosity","7","4","R","Haste
Ashen Monstrosity attacks each turn if able.","5RR","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inquisitor Exarch","2","2","W","When Inquisitor Exarch enters the battlefield, choose one - You gain 2 life; or target opponent loses 2 life.","WW","Creature - Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Treetop Bracers","G","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature gets +1/+1 and can't be blocked except by creatures with flying.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Personal Sanctuary","W","During your turn, prevent all damage that would be dealt to you.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightmine Field","W","Whenever one or more creatures attack, Lightmine Field deals damage to each of those creatures equal to the number of attacking creatures.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Savannah Lions","2","1","W",NULL,"W","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stone-Cold Basilisk","2{1/2}","5","G","Whenever Stone-Cold Basilisk blocks or becomes blocked by a creature with fewer letters in its name, destroy that creature at end of combat. (Punctuation and spaces aren't letters.)
Whenever an opponent reads Stone-Cold Basilisk, that player is turned to stone until end of turn. Stoned players can't attack, block, or play spells or abilities.","4G","Creature - Basilisk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Gargoyle","3","3","W","Flying, first strike
Whenever Kjeldoran Gargoyle deals damage, you gain that much life.","5W","Creature - Gargoyle");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Ajani Vengeant","WR","+1: Target permanent doesn't untap during its controller's next untap step.
-2: Ajani Vengeant deals 3 damage to target creature or player and you gain 3 life.
-7: Destroy all lands target player controls.","Planeswalker - Ajani","2RW");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hour of Reckoning","W","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Destroy all nontoken creatures.","4WWW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desert Twister","G","Destroy target permanent.","4GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Standing Troops","1","4","W","Vigilance","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Bane of Hanweir","5","5","Bane of Hanweir attacks each turn if able.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Bane of Hanweir.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Wayfarer","1","5","W",NULL,"2W","Creature - Elephant Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Shark","4","4","U","Giant Shark can't attack unless defending player controls an Island.
Whenever Giant Shark blocks or becomes blocked by a creature that has been dealt damage this turn, Giant Shark gets +2/+0 and gains trample until end of turn.
When you control no Islands, sacrifice Giant Shark.","5U","Creature - Fish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rathi Fiend","2","2","B","When Rathi Fiend enters the battlefield, each player loses 3 life.
{3}, {T}: Search your library for a Mercenary permanent card with converted mana cost 3 or less and put it onto the battlefield. Then shuffle your library.","3B","Creature - Horror Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Doomed Traveler","1","1","W","When Doomed Traveler dies, put a 1/1 white Spirit creature token with flying onto the battlefield.","W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Firespout","RG","Firespout deals 3 damage to each creature without flying if {R} was spent to cast Firespout and 3 damage to each creature with flying if {G} was spent to cast it. (Do both if {R}{G} was spent.)","2(R/G)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Collector Protector","2","5","W","{W}, Give an opponent a nonland card you own from outside the game: Prevent the next 1 damage that would be dealt to you or Collector Protector this turn.","3WW","Creature - Human Gamer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ruinous Minotaur","5","2","R","Whenever Ruinous Minotaur deals damage to an opponent, sacrifice a land.","1RR","Creature - Minotaur Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gossamer Chains","W","Return Gossamer Chains to its owner's hand: Prevent all combat damage that would be dealt by target unblocked creature this turn.","WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Power Sink","U","Counter target spell unless its controller pays {X}. If he or she doesn't, that player taps all lands with mana abilities he or she controls and empties his or her mana pool.","XU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bleak Coven Vampires","4","3","B","Metalcraft - When Bleak Coven Vampires enters the battlefield, if you control three or more artifacts, target player loses 4 life and you gain 4 life.","3BB","Creature - Vampire Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blight","B","Enchant land
When enchanted land becomes tapped, destroy it.","BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divination","U","Draw two cards.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hedron-Field Purists","0","3","W","Level up {2}{W} ({2}{W}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-4
1/4
If a source would deal damage to you or a creature you control, prevent 1 of that damage.
LEVEL 5+
2/5
If a source would deal damage to you or a creature you control, prevent 2 of that damage.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Floodbringer","1","2","U","Flying
{2}, Return a land you control to its owner's hand: Tap target land.","1U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Augur of Skulls","1","1","B","{1}{B}: Regenerate Augur of Skulls.
Sacrifice Augur of Skulls: Target player discards two cards. Activate this ability only during your upkeep.","1B","Creature - Skeleton Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Warden of the Wall","Warden of the Wall enters the battlefield tapped.
{T}: Add {1} to your mana pool.
As long as it's not your turn, Warden of the Wall is a 2/3 Gargoyle artifact creature with flying.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eye of Nowhere","U","Return target permanent to its owner's hand.","UU","Sorcery - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Beacon Behemoth","5","3","G","{1}: Target creature with power 5 or greater gains vigilance until end of turn.","3GG","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Duskmantle, House of Shadow","{T}: Add {1} to your mana pool.
{U}{B}, {T}: Target player puts the top card of his or her library into his or her graveyard.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vanishment","U","Put target nonland permanent on top of its owner's library.
Miracle {U} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","4U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Traveler's Cloak","U","Enchant creature
As Traveler's Cloak enters the battlefield, choose a land type.
When Traveler's Cloak enters the battlefield, draw a card.
Enchanted creature has landwalk of the chosen type.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mire Boa","2","1","G","Swampwalk
{G}: Regenerate Mire Boa.","1G","Creature - Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Smite","W","Destroy target blocked creature.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wind Drake","2","2","U","Flying","2U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Torch Slinger","2","2","R","Kicker {1}{R} (You may pay an additional {1}{R} as you cast this spell.)
When Torch Slinger enters the battlefield, if it was kicked, it deals 2 damage to target creature.","2R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Etched Monstrosity","10","10","Etched Monstrosity enters the battlefield with five -1/-1 counters on it.
{W}{U}{B}{R}{G}, Remove five -1/-1 counters from Etched Monstrosity: Target player draws three cards.","5","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flash Flood","U","Choose one - Destroy target red permanent; or return target Mountain to its owner's hand.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rushing River","U","Kicker-Sacrifice a land. (You may sacrifice a land in addition to any other costs as you cast this spell.)
Return target nonland permanent to its owner's hand. If Rushing River was kicked, return another target nonland permanent to its owner's hand.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plunder","R","Destroy target artifact or land.
Suspend 4-{1}{R} (Rather than cast this card from your hand, you may pay {1}{R} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","4R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Morality Shift","B","Exchange your graveyard and library. Then shuffle your library.","5BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Karona, False God Avatar","At the beginning of your upkeep, exchange control of a permanent you control chosen at random and a permanent target opponent controls chosen at random.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Followed Footsteps","U","Enchant creature
At the beginning of your upkeep, put a token that's a copy of enchanted creature onto the battlefield.","3UU","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ornate Kanzashi","{2}, {T}: Target opponent exiles the top card of his or her library. You may play that card this turn.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Blossoms","0","4","G","Defender (This creature can't attack.)
When Wall of Blossoms enters the battlefield, draw a card.","1G","Creature - Plant Wall");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Selenia","Creatures you control have vigilance.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Midnight Ritual","B","Exile X target creature cards from your graveyard. For each creature card exiled this way, put a 2/2 black Zombie creature token onto the battlefield.","X2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for Renewal","G","Whenever a creature you control becomes tapped, you may put a quest counter on Quest for Renewal.
As long as there are four or more quest counters on Quest for Renewal, untap all creatures you control during each other player's untap step.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Latchkey Faerie","3","1","U","Flying
Prowl {2}{U} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Faerie or Rogue.)
When Latchkey Faerie enters the battlefield, if its prowl cost was paid, draw a card.","3U","Creature - Faerie Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Combust","R","Combust can't be countered by spells or abilities.
Combust deals 5 damage to target white or blue creature. The damage can't be prevented.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Cannibal","0","0","B","Spike Cannibal enters the battlefield with a +1/+1 counter on it.
When Spike Cannibal enters the battlefield, move all +1/+1 counters from all creatures onto it.","1BB","Creature - Spike");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightsoil Kami","6","4","G","Soulshift 5 (When this creature dies, you may return target Spirit card with converted mana cost 5 or less from your graveyard to your hand.)","4GG","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alaborn Musketeer","2","1","W","Reach (This creature can block creatures with flying.)","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Floodchaser","0","0","U","Floodchaser enters the battlefield with six +1/+1 counters on it.
Floodchaser can't attack unless defending player controls an Island.
{U}, Remove a +1/+1 counter from Floodchaser: Target land becomes an Island until end of turn.","5U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning Fields","R","Burning Fields deals 5 damage to target opponent.","4R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unlikely Alliance","W","{1}{W}: Target nonattacking, nonblocking creature gets +0/+2 until end of turn.","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Water Gun Balloon Game","As Water Gun Balloon Game comes into play, each player puts a pop counter on a 0.
Whenever a player plays a spell, move that player's pop counter up 1.
Whenever a player's pop counter hits 5, that player puts a 5/5 pink Giant Teddy Bear creature token into play and resets all pop counters to 0.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kraken Hatchling","0","4","U",NULL,"U","Creature - Kraken");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinder Elemental","2","2","R","{X}{R}, {T}, Sacrifice Cinder Elemental: Cinder Elemental deals X damage to target creature or player.","3R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight Errant","2","2","W",NULL,"1W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Monstrous Hound","4","4","R","Monstrous Hound can't attack unless you control more lands than defending player.
Monstrous Hound can't block unless you control more lands than attacking player.","3R","Creature - Hound");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("War Chariot","{3}, {T}: Target creature gains trample until end of turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonewood Invoker","2","2","G","{7}{G}: Stonewood Invoker gets +5/+5 until end of turn.","1G","Creature - Elf Mutant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crumble","G","Destroy target artifact. It can't be regenerated. That artifact's controller gains life equal to its converted mana cost.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Whelp","2","2","U","When Phantom Whelp attacks or blocks, return it to its owner's hand at end of combat. (Return it only if it's on the battlefield.)","1U","Creature - Illusion Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flaming Sword","R","Flash
Enchant creature
Enchanted creature gets +1/+0 and has first strike.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Telekinesis","U","Tap target creature. Prevent all combat damage that would be dealt by that creature this turn. It doesn't untap during its controller's next two untap steps.","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunted Dragon","6","6","R","Flying, haste
When Hunted Dragon enters the battlefield, put three 2/2 white Knight creature tokens with first strike onto the battlefield under target opponent's control.","3RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghoulcaller's Chant","B","Choose one - Return target creature card from your graveyard to your hand; or return two target Zombie cards from your graveyard to your hand.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skyscribing","U","Each player draws X cards.
Forecast - {2}{U}, Reveal Skyscribing from your hand: Each player draws a card. (Activate this ability only during your upkeep and only once each turn.)","XUU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Karn","Each noncreature artifact you control is an artifact creature with power and toughness each equal to its converted mana cost.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Infused Arrows","Sunburst (This enters the battlefield with a charge counter on it for each color of mana spent to cast it.)
{T}, Remove X charge counters from Infused Arrows: Target creature gets -X/-X until end of turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Looming Hoverguard","3","3","U","Flying
When Looming Hoverguard enters the battlefield, put target artifact on top of its owner's library.","4UU","Creature - Drone");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yew Spirit","3","3","G","{2}{G}{G}: Yew Spirit gets +X/+X until end of turn, where X is its power.","4G","Creature - Spirit Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Contortion","U","Multikicker {1}{U} (You may pay an additional {1}{U} any number of times as you cast this spell.)
Counter target spell unless its controller pays {2}. Draw a card for each time Spell Contortion was kicked.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gamble","R","Search your library for a card, put that card into your hand, discard a card at random, then shuffle your library.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cat Warriors","2","2","G","Forestwalk","1GG","Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Mirror Golem","3","4","Imprint - When Mirror Golem enters the battlefield, you may exile target card from a graveyard.
Mirror Golem has protection from each of the exiled card's card types. (Artifact, creature, enchantment, instant, land, planeswalker, sorcery, and tribal are card types.)","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hoarding Dragon","4","4","R","Flying
When Hoarding Dragon enters the battlefield, you may search your library for an artifact card, exile it, then shuffle your library.
When Hoarding Dragon dies, you may put the exiled card into its owner's hand.","3RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Predict","U","Name a card, then put the top card of target player's library into his or her graveyard. If that card is the named card, you draw two cards. Otherwise, you draw a card.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Into the Void","U","Return up to two target creatures to their owners' hands.","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Preferred Selection","G","At the beginning of your upkeep, look at the top two cards of your library. You may sacrifice Preferred Selection and pay {2}{G}{G}. If you do, put one of those cards into your hand. If you don't, put one of those cards on the bottom of your library.","2GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rites of Refusal","U","Discard any number of cards. Counter target spell unless its controller pays {3} for each card discarded this way.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Repentance","W","Target creature deals damage to itself equal to its power.","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Embolden","W","Prevent the next 4 damage that would be dealt this turn to any number of target creatures and/or players, divided as you choose.
Flashback {1}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Haazda Exonerator","1","1","W","{T}, Sacrifice Haazda Exonerator: Destroy target Aura.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aquamoeba","1","3","U","Discard a card: Switch Aquamoeba's power and toughness until end of turn.","1U","Creature - Elemental Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Titanic Ultimatum","WRG","Until end of turn, creatures you control get +5/+5 and gain first strike, lifelink, and trample.","RRGGGWW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necrologia","B","Cast Necrologia only during your end step.
As an additional cost to cast Necrologia, pay X life.
Draw X cards.","3BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sulfuric Vortex","R","At the beginning of each player's upkeep, Sulfuric Vortex deals 2 damage to that player.
If a player would gain life, that player gains no life instead.","1RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ambush Commander","2","2","G","Forests you control are 1/1 green Elf creatures that are still lands.
{1}{G}, Sacrifice an Elf: Target creature gets +3/+3 until end of turn.","3GG","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Mender","3","3","W","{W}, {T}: Regenerate target artifact.","5W","Creature - Elephant Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sickleslicer","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +2/+2.
Equip {4}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spiteflame Witch","2","1","BR","{B}{R}: Each player loses 1 life.","1(B/R)","Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Douse in Gloom","B","Douse in Gloom deals 2 damage to target creature and you gain 2 life.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Innocent Blood","B","Each player sacrifices a creature.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quicksilver Dagger","UR","Enchant creature
Enchanted creature has ""{T}: This creature deals 1 damage to target player. You draw a card.""","1UR","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demystify","W","Destroy target enchantment.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flash","U","You may put a creature card from your hand onto the battlefield. If you do, sacrifice it unless you pay its mana cost reduced by up to {2}.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magnetic Theft","R","Attach target Equipment to target creature. (Control of the Equipment doesn't change.)","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shambling Remains","4","3","BR","Shambling Remains can't block.
Unearth {B}{R} ({B}{R}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","1BR","Creature - Zombie Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lull","G","Prevent all combat damage that would be dealt this turn.
Cycling {2} ({2}, Discard this card: Draw a card.)","1G","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wall of Shields","0","4","Defender (This creature can't attack.)
Banding (If any creatures with banding you control are blocking a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by.)","3","Artifact Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Path","G","Green creatures have forestwalk.","2GGGG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bringer of the White Dawn","5","5","W","You may pay {W}{U}{B}{R}{G} rather than pay Bringer of the White Dawn's mana cost.
Trample
At the beginning of your upkeep, you may return target artifact card from your graveyard to the battlefield.","7WW","Creature - Bringer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Journeyer's Kite","{3}, {T}: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Exalted Dragon","5","5","W","Flying
Exalted Dragon can't attack unless you sacrifice a land.","4WW","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slingshot Goblin","2","2","R","{R}, {T}: Slingshot Goblin deals 2 damage to target blue creature.","2R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Maraxus","Creatures you control get +1/+0.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ascendant Evincar","3","3","B","Flying (This creature can't be blocked except by creatures with flying or reach.)
Other black creatures get +1/+1.
Nonblack creatures get -1/-1.","4BB","Legendary Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bureaucracy","U","Pursuant to subsection 3.1(4) of Richard's Rules of Order, during the upkeep of each participant in this game of the Magic: The Gathering® trading card game (hereafter known as ""PLAYER""), that PLAYER performs all actions in the sequence of previously added actions (hereafter known as ""ACTION QUEUE""), in the order those actions were added, then adds another action to the end of the ACTION QUEUE. All actions must be simple physical or verbal actions that a player can perform while sitting in a chair, without jeopardizing the health and security of said PLAYER.
If any PLAYER does not perform all the prescribed actions in the correct order, sacrifice Bureaucracy and said PLAYER discards his or her complement of cards in hand (hereafter known as ""HAND"").","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ezuri's Archers","1","2","G","Reach (This creature can block creatures with flying.)
Whenever Ezuri's Archers blocks a creature with flying, Ezuri's Archers gets +3/+0 until end of turn.","G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uktabi Faerie","1","1","G","Flying
{3}{G}, Sacrifice Uktabi Faerie: Destroy target artifact.","1G","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Destructive Force","R","Each player sacrifices five lands. Destructive Force deals 5 damage to each creature.","5RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corrupted Zendikon","B","Enchant land
Enchanted land is a 3/3 black Ooze creature. It's still a land.
When enchanted land dies, return that card to its owner's hand.","1B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Daru Encampment","{T}: Add {1} to your mana pool.
{W}, {T}: Target Soldier creature gets +1/+1 until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avarax","3","3","R","Haste
When Avarax enters the battlefield, you may search your library for a card named Avarax, reveal it, and put it into your hand. If you do, shuffle your library.
{1}{R}: Avarax gets +1/+0 until end of turn.","3RR","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wilderness Elemental","*","3","RG","Trample
Wilderness Elemental's power is equal to the number of nonbasic lands your opponents control.","1RG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Apes of Rath","5","4","G","Whenever Apes of Rath attacks, it doesn't untap during its controller's next untap step.","2GG","Creature - Ape");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cut the Tethers","U","For each Spirit, return it to its owner's hand unless that player pays {3}.","2UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Balduvian Trading Post","If Balduvian Trading Post would enter the battlefield, sacrifice an untapped Mountain instead. If you do, put Balduvian Trading Post onto the battlefield. If you don't, put it into its owner's graveyard.
{T}: Add {1}{R} to your mana pool.
{1}, {T}: Balduvian Trading Post deals 1 damage to target attacking creature.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Augury Owl","1","1","U","Flying
When Augury Owl enters the battlefield, scry 3. (To scry 3, look at the top three cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","1U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duskhunter Bat","1","1","B","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
Flying","1B","Creature - Bat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mulldrifter","2","2","U","Flying
When Mulldrifter enters the battlefield, draw two cards.
Evoke {2}{U} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","4U","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whitemane Lion","2","2","W","Flash (You may cast this spell any time you could cast an instant.)
When Whitemane Lion enters the battlefield, return a creature you control to its owner's hand.","1W","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Uktabi Kong","8","8","G","Trample
When Uktabi Kong comes into play, destroy all artifacts.
Tap two untapped Apes you control: Put a 1/1 green Ape creature token into play.","5GGG","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rabid Wolverines","4","4","G","Whenever Rabid Wolverines becomes blocked by a creature, Rabid Wolverines gets +1/+1 until end of turn.","3GG","Creature - Wolverine");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steadfast Guard","2","2","W","Vigilance (Attacking doesn't cause this creature to tap.)","WW","Creature - Human Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fissure","R","Destroy target creature or land. It can't be regenerated.","3RR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Orzhova, the Church of Deals","{T}: Add {1} to your mana pool.
{3}{W}{B}, {T}: Target player loses 1 life and you gain 1 life.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostly Flame","BR","Black and/or red permanents and spells are colorless sources of damage.","BR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Matsu-Tribe Decoy","1","3","G","{2}{G}: Target creature blocks Matsu-Tribe Decoy this turn if able.
Whenever Matsu-Tribe Decoy deals combat damage to a creature, tap that creature and it doesn't untap during its controller's next untap step.","2G","Creature - Snake Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tectonic Edge","{T}: Add {1} to your mana pool.
{1}, {T}, Sacrifice Tectonic Edge: Destroy target nonbasic land. Activate this ability only if an opponent controls four or more lands.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Pendant","Darksteel Pendant is indestructible. (""Destroy"" effects and lethal damage don't destroy it.)
{1}, {T}: Look at the top card of your library. You may put that card on the bottom of your library.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Porphyry Nodes","W","At the beginning of your upkeep, destroy the creature with the least power. It can't be regenerated. If two or more creatures are tied for least power, you choose one of them.
When there are no creatures on the battlefield, sacrifice Porphyry Nodes.","W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riven Turnbull","5","7","UB","{T}: Add {B} to your mana pool.","5UB","Legendary Creature - Human Advisor");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Erhnam Djinn Avatar","Whenever you cast a creature spell, put a 1/1 green Saproling creature token onto the battlefield.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dual Casting","R","Enchant creature
Enchanted creature has ""{R}, {T}: Copy target instant or sorcery spell you control. You may choose new targets for the copy.""","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mikaeus, the Unhallowed","5","5","B","Intimidate
Whenever a Human deals damage to you, destroy it.
Other non-Human creatures you control get +1/+1 and have undying. (When a creature with undying dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","3BBB","Legendary Creature - Zombie Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chorus of the Conclave","3","8","WG","Forestwalk
As an additional cost to cast creature spells, you may pay any amount of mana. If you do, that creature enters the battlefield with that many additional +1/+1 counters on it.","4GGWW","Legendary Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patrol Signaler","1","1","W","{1}{W}, {Q}: Put a 1/1 white Kithkin Soldier creature token onto the battlefield. ({Q} is the untap symbol.)","1W","Creature - Kithkin Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Snow-Covered Island","U",NULL,"Basic Snow Land - Island");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ouphe Vandals","2","2","G","{G}, Sacrifice Ouphe Vandals: Counter target activated ability from an artifact source and destroy that artifact if it's on the battlefield. (Mana abilities can't be targeted.)","2G","Creature - Ouphe Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Fallout","R","Volcanic Fallout can't be countered.
Volcanic Fallout deals 2 damage to each creature and each player.","1RR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thought Dissector","{X}, {T}: Target opponent reveals cards from the top of his or her library until an artifact card or X cards are revealed, whichever comes first. If an artifact card is revealed this way, put it onto the battlefield under your control and sacrifice Thought Dissector. Put the rest of the revealed cards into that player's graveyard.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrion Howler","2","2","B","Pay 1 life: Carrion Howler gets +2/-1 until end of turn.","3B","Creature - Zombie Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunscape Master","2","2","W","{G}{G}, {T}: Creatures you control get +2/+2 until end of turn.
{U}{U}, {T}: Return target creature to its owner's hand.","2WW","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Festival of the Guildpact","W","Prevent the next X damage that would be dealt to you this turn.
Draw a card.","XW","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Squee","Your opponents play with their hands revealed.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benevolent Ancestor","0","4","W","Defender (This creature can't attack.)
{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","2W","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Amulet of Unmaking","{5}, {T}, Exile Amulet of Unmaking: Exile target artifact, creature, or land. Activate this ability only any time you could cast a sorcery.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Game","R","Each player hides at least one item, then all players reveal them simultaneously. Each player loses life equal to the number of items he or she revealed. The player who revealed the fewest items then loses half his or her life, rounded up. If two or more players are tied for fewest, each loses half his or her life, rounded up.","5RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infinite Reflection","U","Enchant creature
When Infinite Reflection enters the battlefield attached to a creature, each other nontoken creature you control becomes a copy of that creature.
Nontoken creatures you control enter the battlefield as a copy of enchanted creature.","5U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Torpid Moloch","3","2","R","Defender (This creature can't attack.)
Sacrifice three lands: Torpid Moloch loses defender until end of turn.","R","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunting Drake","2","2","U","Flying
When Hunting Drake enters the battlefield, put target red or green creature on top of its owner's library.","4U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gatherer of Graces","1","2","G","Gatherer of Graces gets +1/+1 for each Aura attached to it.
Sacrifice an Aura: Regenerate Gatherer of Graces.","1G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Dragon","8","8","R","When Zodiac Dragon is put into your graveyard from the battlefield, you may return it to your hand.","7RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unquestioned Authority","W","Enchant creature
When Unquestioned Authority enters the battlefield, draw a card.
Enchanted creature has protection from creatures.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Kites","R","{R}: Target creature you control with toughness 2 or less gains flying until end of turn. Flip a coin at the beginning of the next end step. If you lose the flip, sacrifice that creature.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cobra Trap","G","If a noncreature permanent under your control was destroyed this turn by a spell or ability an opponent controlled, you may pay {G} rather than pay Cobra Trap's mana cost.
Put four 1/1 green Snake creature tokens onto the battlefield.","4GG","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lantern Kami","1","1","W","Flying","W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Siege-Gang Commander","2","2","R","When Siege-Gang Commander enters the battlefield, put three 1/1 red Goblin creature tokens onto the battlefield.
{1}{R}, Sacrifice a Goblin: Siege-Gang Commander deals 2 damage to target creature or player.","3RR","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Defiant Falcon","1","1","W","Flying
{4}, {T}: Search your library for a Rebel permanent card with converted mana cost 3 or less and put it onto the battlefield. Then shuffle your library.","1W","Creature - Rebel Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Damping Engine","A player who controls more permanents than each other player can't play lands or cast artifact, creature, or enchantment spells. That player may sacrifice a permanent for that player to ignore this effect until end of turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spinebiter","3","4","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
You may have Spinebiter assign its combat damage as though it weren't blocked.","4GG","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bosh, Iron Golem Avatar","{X}, Sacrifice an artifact with converted mana cost X: Bosh, Iron Golem Avatar deals X damage to target creature or player.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Narcolepsy","U","Enchant creature
At the beginning of each upkeep, if enchanted creature is untapped, tap it.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defensive Maneuvers","W","Creatures of the creature type of your choice get +0/+4 until end of turn.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Petra Sphinx","3","4","W","{T}: Target player names a card, then reveals the top card of his or her library. If that card is the named card, that player puts it into his or her hand. If it isn't, the player puts it into his or her graveyard.","2WWW","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Jubilation","3","3","W","Flying
Other nonblack creatures you control get +1/+1.
Players can't pay life or sacrifice creatures to cast spells or activate abilities.","1WWW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mindlock Orb","U","Players can't search libraries.","3U","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scare Tactics","B","Creatures you control get +1/+0 until end of turn.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cancel","U","Counter target spell.","1UU","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Crumbling Colossus","7","4","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)
When Crumbling Colossus attacks, sacrifice it at end of combat.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodwraith Strangler","2","2","BG","Exile a creature card from your graveyard: Regenerate Woodwraith Strangler.","2BG","Creature - Plant Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soulbright Flamekin","2","1","R","{2}: Target creature gains trample until end of turn. If this is the third time this ability has resolved this turn, you may add {R}{R}{R}{R}{R}{R}{R}{R} to your mana pool.","1R","Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yawning Fissure","R","Each opponent sacrifices a land.","4R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul Sculptor","1","1","W","{1}{W}, {T}: Target creature becomes an enchantment and loses all abilities until a player casts a creature spell.","2W","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bridge from Below","B","Whenever a nontoken creature is put into your graveyard from the battlefield, if Bridge from Below is in your graveyard, put a 2/2 black Zombie creature token onto the battlefield.
When a creature is put into an opponent's graveyard from the battlefield, if Bridge from Below is in your graveyard, exile Bridge from Below.","BBB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gustcloak Sentinel","3","3","W","Whenever Gustcloak Sentinel becomes blocked, you may untap it and remove it from combat.","2WW","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Highborn Ghoul","2","1","B","Intimidate (This creature can't be blocked except by artifact creatures and/or creatures that share a color with it.)","BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Hero","1","1","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flash of Insight","U","Look at the top X cards of your library. Put one of them into your hand and the rest on the bottom of your library in any order.
Flashback-{1}{U}, Exile X blue cards from your graveyard. (You may cast this card from your graveyard for its flashback cost, then exile it. You can't exile Flash of Insight to pay for its own flashback cost.)","X1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shared Trauma","B","Join forces - Starting with you, each player may pay any amount of mana. Each player puts the top X cards of his or her library into his or her graveyard, where X is the total amount of mana paid this way.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reckless Assault","BR","{1}, Pay 2 life: Reckless Assault deals 1 damage to target creature or player.","2BR","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Horn of Ramos","{T}: Add {G} to your mana pool.
Sacrifice Horn of Ramos: Add {G} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nezumi Shadow-Watcher","1","1","B","Sacrifice Nezumi Shadow-Watcher: Destroy target Ninja.","B","Creature - Rat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Palace Guard","1","4","W","Palace Guard can block any number of creatures.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Humble Budoka","2","2","G","Shroud (This permanent can't be the target of spells or abilities.)","1G","Creature - Human Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spikeshot Elder","1","1","R","{1}{R}{R}: Spikeshot Elder deals damage equal to its power to target creature or player.","R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Niveous Wisps","W","Target creature becomes white until end of turn. Tap that creature.
Draw a card.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Waveskimmer Aven","2","4","WUG","Flying
Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","2GWU","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Change of Heart","W","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Target creature can't attack this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Civic Wayfinder","2","2","G","When Civic Wayfinder enters the battlefield, you may search your library for a basic land card, reveal it, and put it into your hand. If you do, shuffle your library.","2G","Creature - Elf Warrior Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Amulet of Vigor","Whenever a permanent enters the battlefield tapped and under your control, untap it.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shrieking Grotesque","2","1","W","Flying
When Shrieking Grotesque enters the battlefield, if {B} was spent to cast Shrieking Grotesque, target player discards a card.","2W","Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greater Basilisk","3","5","G","Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","3GG","Creature - Basilisk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serra Paladin","2","2","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.
{1}{W}{W}, {T}: Target creature gains vigilance until end of turn.","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kabira Vindicator","2","4","W","Level up {2}{W} ({2}{W}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 2-4
3/6
Other creatures you control get +1/+1.
LEVEL 5+
4/8
Other creatures you control get +2/+2.","3W","Creature - Human Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Feldon's Cane","{T}, Exile Feldon's Cane: Shuffle your graveyard into your library.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Endless Whispers","B","Each creature has ""When this creature dies, choose target opponent. That player puts this card from its owner's graveyard onto the battlefield under his or her control at the beginning of the next end step.""","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Javelineer","1","2","W","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
{T}: Kjeldoran Javelineer deals damage equal to the number of age counters on it to target attacking or blocking creature.","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coral Trickster","2","1","U","Morph {U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Coral Trickster is turned face up, you may tap or untap target permanent.","1U","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin War Cry","R","Target opponent chooses a creature he or she controls. Other creatures he or she controls can't block this turn.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darba","5","4","G","At the beginning of your upkeep, sacrifice Darba unless you pay {G}{G}.","3G","Creature - Bird Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bull Elephant","4","4","G","When Bull Elephant enters the battlefield, sacrifice it unless you return two Forests you control to their owner's hand.","3G","Creature - Elephant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clash of Realities","R","All Spirits have ""When this permanent enters the battlefield, you may have it deal 3 damage to target non-Spirit creature.""
Non-Spirit creatures have ""When this creature enters the battlefield, you may have it deal 3 damage to target Spirit creature.""","3R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thran Foundry","{1}, {T}, Exile Thran Foundry: Target player shuffles his or her graveyard into his or her library.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wei Infantry","2","1","B",NULL,"1B","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bone Mask","{2}, {T}: The next time a source of your choice would deal damage to you this turn, prevent that damage. Exile cards from the top of your library equal to the damage prevented this way.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tainted Peak","{T}: Add {1} to your mana pool.
{T}: Add {B} or {R} to your mana pool. Activate this ability only if you control a Swamp.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fyndhorn Druid","2","2","G","When Fyndhorn Druid dies, if it was blocked this turn, you gain 4 life.","2G","Creature - Elf Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ruins of Trokair","Ruins of Trokair enters the battlefield tapped.
{T}: Add {W} to your mana pool.
{T}, Sacrifice Ruins of Trokair: Add {W}{W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guard Dogs","2","2","W","{2}{W}, {T}: Choose a permanent you control. Prevent all combat damage target creature would deal this turn if it shares a color with that permanent.","3W","Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shackles","W","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.
{W}: Return Shackles to its owner's hand.","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Volunteer Reserves","2","4","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Flock","0","0","W","Flying
Phantom Flock enters the battlefield with three +1/+1 counters on it.
If damage would be dealt to Phantom Flock, prevent that damage. Remove a +1/+1 counter from Phantom Flock.","3WW","Creature - Bird Soldier Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wind Shear","G","Attacking creatures with flying get -2/-2 and lose flying until end of turn.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reign of Chaos","R","Choose one - Destroy target Plains and target white creature; or destroy target Island and target blue creature.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fatigue","U","Target player skips his or her next draw step.","1U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fold into AEther","U","Counter target spell. If that spell is countered this way, its controller may put a creature card from his or her hand onto the battlefield.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Windscouter","3","3","U","Flying
Whenever Windscouter attacks or blocks, return it to its owner's hand at end of combat. (Return it only if it's on the battlefield.)","3U","Creature - Human Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Furnace Scamp","1","1","R","Whenever Furnace Scamp deals combat damage to a player, you may sacrifice it. If you do, Furnace Scamp deals 3 damage to that player.","R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necrogenesis","BG","{2}: Exile target creature card from a graveyard. Put a 1/1 green Saproling creature token onto the battlefield.","BG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scrap","R","Destroy target artifact.
Cycling {2} ({2}, Discard this card: Draw a card.)","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Shambler","2","2","R","{R}: Flowstone Shambler gets +1/-1 until end of turn.","2R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eyes of the Watcher","U","Whenever you cast an instant or sorcery spell, you may pay {1}. If you do, scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conqueror's Pledge","W","Kicker {6} (You may pay an additional {6} as you cast this spell.)
Put six 1/1 white Kor Soldier creature tokens onto the battlefield. If Conqueror's Pledge was kicked, put twelve of those tokens onto the battlefield instead.","2WWW","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Assembly-Worker","2","2","{T}: Target Assembly-Worker creature gets +1/+1 until end of turn.","3","Artifact Creature - Assembly-Worker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Honden of Seeing Winds","U","At the beginning of your upkeep, draw a card for each Shrine you control.","4U","Legendary Enchantment - Shrine");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avatar of Hope","4","9","W","If you have 3 or less life, Avatar of Hope costs {6} less to cast.
Flying
Avatar of Hope can block any number of creatures.","6WW","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battering Craghorn","3","1","R","First strike
Morph {1}{R}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2RR","Creature - Goat Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tawnos's Weaponry","You may choose not to untap Tawnos's Weaponry during your untap step.
{2}, {T}: Target creature gets +1/+1 for as long as Tawnos's Weaponry remains tapped.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Snake","2","2","B","Swampwalk","2B","Creature - Snake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Juju Bubble","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When you play a card, sacrifice Juju Bubble.
{2}: You gain 1 life.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riders of Gavony","3","3","W","Vigilance
As Riders of Gavony enters the battlefield, choose a creature type.
Human creatures you control have protection from creatures of the chosen type.","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mesa Falcon","1","1","W","Flying
{1}{W}: Mesa Falcon gets +0/+1 until end of turn.","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Champion's Drake","1","1","U","Flying
Champion's Drake gets +3/+3 as long as you control a creature with three or more level counters on it.","1U","Creature - Drake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hinterland Harbor","Hinterland Harbor enters the battlefield tapped unless you control a Forest or an Island.
{T}: Add {G} or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Extravagant Spirit","4","4","U","Flying
At the beginning of your upkeep, sacrifice Extravagant Spirit unless you pay {1} for each card in your hand.","3U","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Squire","1","1","W","Flying
Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","1W","Creature - Bird Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Distorting Lens","{T}: Target permanent becomes the color of your choice until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argothian Enchantress","0","1","G","Shroud (This permanent can't be the target of spells or abilities.)
Whenever you cast an enchantment spell, draw a card.","1G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grand Melee","R","All creatures attack each turn if able.
All creatures block each turn if able.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("The Mimeoplasm","0","0","UBG","As The Mimeoplasm enters the battlefield, you may exile two creature cards from graveyards. If you do, it enters the battlefield as a copy of one of those cards with a number of additional +1/+1 counters on it equal to the power of the other card.","2GUB","Legendary Creature - Ooze");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Segmented Wurm","5","5","RG","Whenever Segmented Wurm becomes the target of a spell or ability, put a -1/-1 counter on it.","3RG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cold-Eyed Selkie","1","1","UG","Islandwalk
Whenever Cold-Eyed Selkie deals combat damage to a player, you may draw that many cards.","1(G/U)(G/U)","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunter's Insight","G","Choose target creature you control. Whenever that creature deals combat damage to a player or planeswalker this turn, draw that many cards.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cenn's Tactician","1","1","W","{W}, {T}: Put a +1/+1 counter on target Soldier creature.
Each creature you control with a +1/+1 counter on it can block an additional creature.","W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crude Rampart","4","5","W","Defender (This creature can't attack.)
Morph {4}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3W","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recollect","G","Return target card from your graveyard to your hand.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Crumple","U","Counter target spell. If that spell is countered this way, put it on the bottom of its owner's library instead of into that player's graveyard. Put Spell Crumple on the bottom of its owner's library.","1UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dragon Wings","U","Enchant creature
Enchanted creature has flying.
Cycling {1}{U} ({1}{U}, Discard this card: Draw a card.)
When a creature with converted mana cost 6 or greater enters the battlefield, you may return Dragon Wings from your graveyard to the battlefield attached to that creature.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glamer Spinners","2","4","WU","Flash
Flying
When Glamer Spinners enters the battlefield, attach all Auras enchanting target permanent to another permanent with the same controller.","4(W/U)","Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Risky Move","R","At the beginning of each player's upkeep, that player gains control of Risky Move.
When you gain control of Risky Move from another player, choose a creature you control and an opponent. Flip a coin. If you lose the flip, that opponent gains control of that creature.","3RRR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellkite Hatchling","2","2","RG","Devour 1 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with that many +1/+1 counters on it.)
Hellkite Hatchling has flying and trample if it devoured a creature.","2RG","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whimwader","6","4","U","Whimwader can't attack unless defending player controls a blue permanent.","4U","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pianna, Nomad Captain","2","2","W","Whenever Pianna, Nomad Captain attacks, attacking creatures get +1/+1 until end of turn.","1WW","Legendary Creature - Human Nomad");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ward of Bones","Each opponent who controls more creatures than you can't play creature cards. The same is true for artifacts, enchantments, and lands.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sparkspitter","1","3","R","{R}, {T}, Discard a card: Put a 3/1 red Elemental creature token named Spark Elemental onto the battlefield. It has trample, haste, and ""At the beginning of the end step, sacrifice Spark Elemental.""","2R","Creature - Elemental Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fork","R","Copy target instant or sorcery spell, except that the copy is red. You may choose new targets for the copy.","RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scouting Trek","G","Search your library for any number of basic land cards. Reveal those cards, then shuffle your library and put them on top of it.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defy Gravity","U","Target creature gains flying until end of turn.
Flashback {U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Mulch","0","4","G","Defender (This creature can't attack.)
{G}, Sacrifice a Wall: Draw a card.","1G","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avacynian Priest","1","2","W","{1}, {T}: Tap target non-Human creature.","1W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Energy Arc","WU","Untap any number of target creatures. Prevent all combat damage that would be dealt to and dealt by those creatures this turn.","WU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desolation","B","At the beginning of each end step, each player who tapped a land for mana this turn sacrifices a land. Desolation deals 2 damage to each player who sacrificed a Plains this way.","1BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tainted Specter","2","2","B","Flying
{1}{B}{B}, {T}: Target player discards a card unless he or she puts a card from his or her hand on top of his or her library. If that player discards a card this way, Tainted Specter deals 1 damage to each creature and each player. Activate this ability only any time you could cast a sorcery.","3B","Creature - Specter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mogg Infestation","R","Destroy all creatures target player controls. For each creature that died this way, put two 1/1 red Goblin creature tokens onto the battlefield under that player's control.","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bomb Squad","1","1","R","{T}: Put a fuse counter on target creature.
At the beginning of your upkeep, put a fuse counter on each creature with a fuse counter on it.
Whenever a creature has four or more fuse counters on it, remove all fuse counters from it and destroy it. That creature deals 4 damage to its controller.","3R","Creature - Dwarf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ayumi, the Last Visitor","7","3","G","Legendary landwalk","3GG","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mana Leech","1","1","B","You may choose not to untap Mana Leech during your untap step.
{T}: Tap target land. It doesn't untap during its controller's untap step for as long as Mana Leech remains tapped.","2B","Creature - Leech");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Deathraiders","3","1","BR","Trample","BR","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwater Mystic","1","1","U","{1}{U}: Look at the top card of target player's library.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Imperial Edict","B","Target opponent chooses a creature he or she controls. Destroy it.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bonebreaker Giant","4","4","R",NULL,"4R","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flow of Maggots","2","2","B","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Flow of Maggots can't be blocked by non-Wall creatures.","2B","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iizuka the Ruthless","3","3","R","Bushido 2 (When this blocks or becomes blocked, it gets +2/+2 until end of turn.)
{2}{R}, Sacrifice a Samurai: Samurai creatures you control gain double strike until end of turn.","3RR","Legendary Creature - Human Samurai");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Enigma Sphinx Avatar","Whenever you cast a colored artifact spell for the first time in a turn, search your library for a colored artifact card chosen at random whose converted mana cost is less than that spell's converted mana cost. You may play that card without paying its mana cost. If you don't, put that card on the bottom of your library.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Retract","U","Return all artifacts you control to their owner's hand.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kamahl's Desire","R","Enchant creature
Enchanted creature has first strike.
Threshold - Enchanted creature gets +3/+0 as long as seven or more cards are in your graveyard.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grave Bramble","3","4","G","Defender, protection from Zombies","1GG","Creature - Plant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Plateau",NULL,NULL,"Land - Mountain Plains");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rancid Earth","B","Destroy target land.
Threshold - If seven or more cards are in your graveyard, instead destroy that land and Rancid Earth deals 1 damage to each creature and each player.","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Through the Breach","R","You may put a creature card from your hand onto the battlefield. That creature gains haste. Sacrifice that creature at the beginning of the next end step.
Splice onto Arcane {2}{R}{R} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","4R","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Jace Beleren","U","+2: Each player draws a card.
-1: Target player draws a card.
-10: Target player puts the top twenty cards of his or her library into his or her graveyard.","Planeswalker - Jace","1UU");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vengeful Pharaoh","5","4","B","Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)
Whenever combat damage is dealt to you or a planeswalker you control, if Vengeful Pharaoh is in your graveyard, destroy target attacking creature, then put Vengeful Pharaoh on top of your library.","2BBB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Protection: Green","W","{1}: The next time a green source of your choice would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirsdag High Priest","1","2","B","Morbid - {T}, Tap two untapped creatures you control: Put a 5/5 black Demon creature token with flying onto the battlefield. Activate this ability only if a creature died this turn.","1B","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Bowling Team","1","1","R","Whenever Goblin Bowling Team damages any creature or player, roll a six-sided die. Goblin Bowling Team deals to that creature or player additional damage equal to the die roll.","3R","Summon - Goblins");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ambush Viper","2","1","G","Flash (You may cast this spell any time you could cast an instant.)
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","1G","Creature - Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planeswalker's Fury","R","{3}{R}: Target opponent reveals a card at random from his or her hand. Planeswalker's Fury deals damage equal to that card's converted mana cost to that player. Activate this ability only any time you could cast a sorcery.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rathi Trapper","1","2","B","{B}, {T}: Tap target creature.","1B","Creature - Human Rebel Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peer Pressure","U","Choose a creature type. If you control more creatures of that type than each other player, you gain control of all creatures of that type. (This effect lasts indefinitely.)","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dryad's Favor","G","Enchant creature
Enchanted creature has forestwalk. (It's unblockable as long as defending player controls a Forest.)","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Beloved Chaplain","1","1","W","Protection from creatures","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Contagious Nim","2","2","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frost Titan","6","6","U","Whenever Frost Titan becomes the target of a spell or ability an opponent controls, counter that spell or ability unless its controller pays {2}.
Whenever Frost Titan enters the battlefield or attacks, tap target permanent. It doesn't untap during its controller's next untap step.","4UU","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scorched Earth","R","As an additional cost to cast Scorched Earth, discard X land cards.
Destroy X target lands.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Brass Squire","1","3","{T}: Attach target Equipment you control to target creature you control.","3","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Creakwood Ghoul","3","3","B","{(b/g){(b/g)}: Exile target card from a graveyard. You gain 1 life.","4B","Creature - Plant Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Robber Fly","1","1","R","Flying
Whenever Robber Fly becomes blocked, defending player discards all the cards in his or her hand, then draws that many cards.","2R","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kobold Overlord","1","2","R","First strike
Other Kobold creatures you control have first strike.","1R","Creature - Kobold");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reef Pirates","2","2","U","Whenever Reef Pirates deals damage to an opponent, that player puts the top card of his or her library into his or her graveyard.","1UU","Creature - Zombie Pirate");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Terrarion","Terrarion enters the battlefield tapped.
{2}, {T}, Sacrifice Terrarion: Add two mana in any combination of colors to your mana pool.
When Terrarion is put into a graveyard from the battlefield, draw a card.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harmony of Nature","G","Tap any number of untapped creatures you control. You gain 4 life for each creature tapped this way.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frankie Peanuts","2","3","W","At the beginning of your upkeep, you may ask target player a yes-or-no question. If you do, that player answers the question truthfully and abides by that answer if able until end of turn.","2WW","Legendary Creature - Elephant Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Enforcer","4","4","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)","7","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystical Teachings","U","Search your library for an instant card or a card with flash, reveal it, and put it into your hand. Then shuffle your library.
Flashback {5}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("War Elemental","1","1","R","When War Elemental enters the battlefield, sacrifice it unless an opponent was dealt damage this turn.
Whenever an opponent is dealt damage, put that many +1/+1 counters on War Elemental.","RRR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Vorrac","0","0","Trample
Clockwork Vorrac enters the battlefield with four +1/+1 counters on it.
Whenever Clockwork Vorrac attacks or blocks, remove a +1/+1 counter from it at end of combat.
{T}: Put a +1/+1 counter on Clockwork Vorrac.","5","Artifact Creature - Boar Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gilt-Leaf Seer","2","2","G","{G}, {T}: Look at the top two cards of your library, then put them back in any order.","2G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spectral Procession","W","({(2/w)} can be paid with any two mana or with {W}. This card's converted mana cost is 6.)
Put three 1/1 white Spirit creature tokens with flying onto the battlefield.","(2/W)(2/W)(2/W)","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marrow-Gnawer","2","3","B","Rat creatures have fear. (They can't be blocked except by artifact creatures and/or black creatures.)
{T}, Sacrifice a Rat: Put X 1/1 black Rat creature tokens onto the battlefield, where X is the number of Rats you control.","3BB","Legendary Creature - Rat Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Berserkers of Blood Ridge","4","4","R","Berserkers of Blood Ridge attacks each turn if able.","4R","Creature - Human Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wanderguard Sentry","3","3","U","When Wanderguard Sentry enters the battlefield, look at target opponent's hand.","4U","Creature - Drone");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Myojin of Seeing Winds","3","3","U","Myojin of Seeing Winds enters the battlefield with a divinity counter on it if you cast it from your hand.
Myojin of Seeing Winds is indestructible as long as it has a divinity counter on it.
Remove a divinity counter from Myojin of Seeing Winds: Draw a card for each permanent you control.","7UUU","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Repentant Blacksmith","1","2","W","Protection from red","1W","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auratog","1","2","W","Sacrifice an enchantment: Auratog gets +2/+2 until end of turn.","1W","Creature - Atog");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Tangle Golem","5","4","Affinity for Forests (This spell costs {1} less to cast for each Forest you control.)","7","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Avarice Totem","{5}: Exchange control of Avarice Totem and target nonland permanent.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breath of Life","W","Return target creature card from your graveyard to the battlefield.","3W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elephant Ambush","G","Put a 3/3 green Elephant creature token onto the battlefield.
Flashback {6}{G}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glimmerpoint Stag","3","3","W","Vigilance
When Glimmerpoint Stag enters the battlefield, exile another target permanent. Return that card to the battlefield under its owner's control at the beginning of the next end step.","2WW","Creature - Elk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Light from Within","W","Chroma - Each creature you control gets +1/+1 for each white mana symbol in its mana cost.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Death Charmer","2","2","B","Whenever Death Charmer deals combat damage to a creature, that creature's controller loses 2 life unless he or she pays {2}.","2B","Creature - Worm Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call the Skybreaker","UR","Put a 5/5 blue and red Elemental creature token with flying onto the battlefield.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","5(U/R)(U/R)","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Belbe's Portal","As Belbe's Portal enters the battlefield, choose a creature type.
{3}, {T}: You may put a creature card of the chosen type from your hand onto the battlefield.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Juxtapose","U","You and target player exchange control of the creature you each control with the highest converted mana cost. Then exchange control of artifacts the same way. If two or more permanents a player controls are tied for highest cost, their controller chooses one of them.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aerie Ouphes","3","3","G","Sacrifice Aerie Ouphes: Aerie Ouphes deals damage equal to its power to target creature with flying.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","4G","Creature - Ouphe");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg War Marshal","1","1","R","Echo {1}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Mogg War Marshal enters the battlefield or dies, put a 1/1 red Goblin creature token onto the battlefield.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwater Commando","2","2","U","Islandwalk (This creature is unblockable as long as defending player controls an Island.)","2U","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dearly Departed","5","5","W","Flying
As long as Dearly Departed is in your graveyard, each Human creature you control enters the battlefield with an additional +1/+1 counter on it.","4WW","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Warrior","3","3","R",NULL,"3R","Creature - Ogre Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Neverending Torment","B","Search target player's library for X cards, where X is the number of cards in your hand, and exile them. Then that player shuffles his or her library.
Epic (For the rest of the game, you can't cast spells. At the beginning of each of your upkeeps, copy this spell except for its epic ability. You may choose a new target for the copy.)","4BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fugitive Wizard","1","1","U",NULL,"U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necromancer's Covenant","WB","When Necromancer's Covenant enters the battlefield, exile all creature cards from target player's graveyard, then put a 2/2 black Zombie creature token onto the battlefield for each card exiled this way.
Zombies you control have lifelink.","3WBB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rashida Scalebane","3","4","W","{T}: Destroy target attacking or blocking Dragon. It can't be regenerated. You gain life equal to its power.","3WW","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abattoir Ghoul","3","2","B","First strike
Whenever a creature dealt damage by Abattoir Ghoul this turn dies, you gain life equal to that creature's toughness.","3B","Creature - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mizzium Transreliquat","{3}: Mizzium Transreliquat becomes a copy of target artifact until end of turn.
{1}{U}{R}: Mizzium Transreliquat becomes a copy of target artifact and gains this ability.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rocket-Powered Turbo Slug","3","1","R","Super haste (This may attack the turn before you play it. (You may put this card into play from your hand, tapped and attacking, during your declare attackers step. If you do, you lose the game at the end of your next turn unless you pay this card's mana cost during that turn.))","3R","Creature - Slug");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enchantress's Presence","G","Whenever you cast an enchantment spell, draw a card.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Surgeon","2","1","B","{2}{B}{B}, {T}, Exile two cards from your graveyard: Return target creature card from your graveyard to your hand.","2BB","Creature - Human Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fecundity","G","Whenever a creature dies, that creature's controller may draw a card.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Master","2","3","B","Other Zombie creatures have swampwalk.
Other Zombies have ""{B}: Regenerate this permanent.""","1BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sygg, River Cutthroat","1","3","UB","At the beginning of each end step, if an opponent lost 3 or more life this turn, you may draw a card. (Damage causes loss of life.)","(U/B)(U/B)","Legendary Creature - Merfolk Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battlefield Medic","1","1","W","{T}: Prevent the next X damage that would be dealt to target creature this turn, where X is the number of Clerics on the battlefield.","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Weaponsmith","1","1","R","{T}, Sacrifice an artifact: Put a +1/+1 counter on target creature. Activate this ability only during your upkeep.","1R","Creature - Dwarf Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steel Leaf Paladin","4","4","WG","First strike
When Steel Leaf Paladin enters the battlefield, return a green or white creature you control to its owner's hand.","4GW","Creature - Elf Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Canopy Cover","G","Enchant creature
Enchanted creature can't be blocked except by creatures with flying or reach.
Enchanted creature can't be the target of spells or abilities your opponents control.","1G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Craven Knight","2","2","B","Craven Knight can't block.","1B","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scrounge","B","Target opponent chooses an artifact card in his or her graveyard. Put that card onto the battlefield under your control.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cyclops Gladiator","4","4","R","Whenever Cyclops Gladiator attacks, you may have it deal damage equal to its power to target creature defending player controls. If you do, that creature deals damage equal to its power to Cyclops Gladiator.","1RRR","Creature - Cyclops Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coralhelm Commander","2","2","U","Level up {1} ({1}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 2-3
3/3
Flying
LEVEL 4+
4/4
Flying
Other Merfolk creatures you control get +1/+1.","UU","Creature - Merfolk Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Thran Golem","3","3","As long as Thran Golem is enchanted, it gets +2/+2 and has flying, first strike, and trample.","5","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crystal Shard","{3}, {T} or {U}, {T}: Return target creature to its owner's hand unless its controller pays {1}.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Transmute Artifact","U","Sacrifice an artifact. If you do, search your library for an artifact card. If that card's converted mana cost is less than or equal to the sacrificed artifact's converted mana cost, put it onto the battlefield. If it's greater, you may pay {X}, where X is the difference. If you do, put it onto the battlefield. If you don't, put it into its owner's graveyard. Then shuffle your library.","UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spitting Earth","R","Spitting Earth deals damage to target creature equal to the number of Mountains you control.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepwood Wolverine","1","1","G","Whenever Deepwood Wolverine becomes blocked, it gets +2/+0 until end of turn.","G","Creature - Wolverine");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Howlgeist","4","2","G","Creatures with power less than Howlgeist's power can't block it.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","5G","Creature - Spirit Wolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Touch of Darkness","B","Any number of target creatures become black until end of turn.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorilla Berserkers","2","3","G","Trample; rampage 2 (Whenever this creature becomes blocked, it gets +2/+2 until end of turn for each creature blocking it beyond the first.)
Gorilla Berserkers can't be blocked except by three or more creatures.","3GG","Creature - Ape Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planeswalker's Mischief","U","{3}{U}: Target opponent reveals a card at random from his or her hand. If it's an instant or sorcery card, exile it. You may cast it without paying its mana cost for as long as it remains exiled. (If it has X in its mana cost, X is 0.) At the beginning of the next end step, if you haven't cast it, return it to its owner's hand. Activate this ability only any time you could cast a sorcery.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Impatience","R","At the beginning of each player's end step, if that player didn't cast a spell this turn, Impatience deals 2 damage to him or her.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conquer","R","Enchant land
You control enchanted land.","3RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balefire Dragon","6","6","R","Flying
Whenever Balefire Dragon deals combat damage to a player, it deals that much damage to each creature that player controls.","5RR","Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Etched Oracle Avatar","You may pay {W}{U}{B}{R}{G} rather than pay the mana cost for spells that you cast.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dirty Wererat","2","3","B","{B}, Discard a card: Regenerate Dirty Wererat.
Threshold - As long as seven or more cards are in your graveyard, Dirty Wererat gets +2/+2 and can't block.","3B","Creature - Human Rat Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Manta Riders","1","1","U","{U}: Manta Riders gains flying until end of turn.","U","Creature - Merfolk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Geothermal Crevice","Geothermal Crevice enters the battlefield tapped.
{T}: Add {R} to your mana pool.
{T}, Sacrifice Geothermal Crevice: Add {B}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akuta, Born of Ash","3","2","B","Haste
At the beginning of your upkeep, if you have more cards in hand than each opponent, you may sacrifice a Swamp. If you do, return Akuta, Born of Ash from your graveyard to the battlefield.","2BB","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Snow Hound","1","1","W","{1}, {T}: Return Snow Hound and target green or blue creature you control to their owner's hand.","2W","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hoard-Smelter Dragon","5","5","R","Flying
{3}{R}: Destroy target artifact. Hoard-Smelter Dragon gets +X/+0 until end of turn, where X is that artifact's converted mana cost.","4RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Feral Deceiver","3","2","G","{1}: Look at the top card of your library.
{2}: Reveal the top card of your library. If it's a land card, Feral Deceiver gets +2/+2 and gains trample until end of turn. Activate this ability only once each turn.","3G","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nephalia Drownyard","{T}: Add {1} to your mana pool.
{1}{U}{B}, {T}: Target player puts the top three cards of his or her library into his or her graveyard.",NULL,"Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Drill-Skimmer","2","1","Flying
Drill-Skimmer has shroud as long as you control another artifact creature. (It can't be the target of spells or abilities.)","4","Artifact Creature - Thopter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stalking Bloodsucker","4","4","B","Flying
{1}{B}, Discard a card: Stalking Bloodsucker gets +2/+2 until end of turn.","4BB","Creature - Vampire");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Devourer","1","1","When Phyrexian Devourer's power is 7 or greater, sacrifice it.
Exile the top card of your library: Put X +1/+1 counters on Phyrexian Devourer, where X is the exiled card's converted mana cost.","6","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Broodhatch Nantuko","1","1","G","Whenever Broodhatch Nantuko is dealt damage, you may put that many 1/1 green Insect creature tokens onto the battlefield.
Morph {2}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1G","Creature - Insect Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enlightened Tutor","W","Search your library for an artifact or enchantment card and reveal that card. Shuffle your library, then put the card on top of it.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Righteous Aura","W","{W}, Pay 2 life: The next time a source of your choice would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Neurok Stealthsuit","Equipped creature has shroud. (It can't be the target of spells or abilities.)
{U}{U}: Attach Neurok Stealthsuit to target creature you control.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunted Troll","8","4","G","When Hunted Troll enters the battlefield, put four 1/1 blue Faerie creature tokens with flying onto the battlefield under target opponent's control.
{G}: Regenerate Hunted Troll.","2GG","Creature - Troll Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Blood","0","2","B","Defender (This creature can't attack.)
Pay 1 life: Wall of Blood gets +1/+1 until end of turn.","2B","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Protection: Red","W","{1}: The next time a red source of your choice would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Butcher of Malakir","5","4","B","Flying
Whenever Butcher of Malakir or another creature you control dies, each opponent sacrifices a creature.","5BB","Creature - Vampire Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystical Tutor","U","Search your library for an instant or sorcery card and reveal that card. Shuffle your library, then put the card on top of it.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sunken City","U","At the beginning of your upkeep, sacrifice Sunken City unless you pay {U}{U}.
Blue creatures get +1/+1.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Icatian Crier","1","1","W","{1}{W}, {T}, Discard a card: Put two 1/1 white Citizen creature tokens onto the battlefield.","2W","Creature - Human Spellshaper");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Safe Haven","{2}, {T}: Exile target creature you control.
At the beginning of your upkeep, you may sacrifice Safe Haven. If you do, return each card exiled with Safe Haven to the battlefield under its owner's control.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sensei's Divining Top","{1}: Look at the top three cards of your library, then put them back in any order.
{T}: Draw a card, then put Sensei's Divining Top on top of its owner's library.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Horobi's Whisper","B","If you control a Swamp, destroy target nonblack creature.
Splice onto Arcane-Exile four cards from your graveyard. (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1BB","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyroclasm","R","Pyroclasm deals 2 damage to each creature.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tattered Drake","2","2","U","Flying
{B}: Regenerate Tattered Drake.","4U","Creature - Zombie Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raging River","R","Whenever one or more creatures you control attack, each defending player divides all creatures without flying he or she controls into a ""left"" pile and a ""right"" pile. Then, for each attacking creature you control, choose ""left"" or ""right."" That creature can't be blocked this combat except by creatures with flying and creatures in a pile with the chosen label.","RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vent Sentinel","2","4","R","Defender
{1}{R}, {T}: Vent Sentinel deals damage to target player equal to the number of creatures with defender you control.","3R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prismatic Circle","W","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
As Prismatic Circle enters the battlefield, choose a color.
{1}: The next time a source of your choice of the chosen color would deal damage to you this turn, prevent that damage.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mana-Charged Dragon","5","5","R","Flying, trample
Join forces - Whenever Mana-Charged Dragon attacks or blocks, each player starting with you may pay any amount of mana. Mana-Charged Dragon gets +X/+0 until end of turn, where X is the total amount of mana paid this way.","4RR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Clockwork Steed","0","3","Clockwork Steed enters the battlefield with four +1/+0 counters on it.
Clockwork Steed can't be blocked by artifact creatures.
At end of combat, if Clockwork Steed attacked or blocked this combat, remove a +1/+0 counter from it.
{X}, {T}: Put up to X +1/+0 counters on Clockwork Steed. This ability can't cause the total number of +1/+0 counters on Clockwork Steed to be greater than four. Activate this ability only during your upkeep.","4","Artifact Creature - Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Culling the Weak","B","As an additional cost to cast Culling the Weak, sacrifice a creature.
Add {B}{B}{B}{B} to your mana pool.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Halfdane","3","3","WUB","At the beginning of your upkeep, Halfdane's power and toughness become equal to the power and toughness of target creature other than Halfdane until the end of your next upkeep.","1WUB","Legendary Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grixis Illusionist","1","1","U","{T}: Target land you control becomes the basic land type of your choice until end of turn.","U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Togglodyte","4","4","Togglodyte comes into play turned on.
Whenever a player plays a spell, toggle Togglodyte's ON/OFF switch.
As long as Togglodyte is turned off, it can't attack or block, and all damage it would deal is prevented.","3","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Headhunter","1","1","B","Whenever Headhunter deals combat damage to a player, that player discards a card.
Morph {B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deep Water","U","{U}: Until end of turn, if you tap a land you control for mana, it produces {U} instead of any other type.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Wonder","1","5","U","Defender (This creature can't attack.)
{2}{U}{U}: Wall of Wonder gets +4/-4 until end of turn and can attack this turn as though it didn't have defender.","2UU","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of the Mind","1","2","U","{U}, {T}: Choose target opponent who had at least two more cards in hand than you did as you activated this ability. Draw a card.","UU","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Svogthos, the Restless Tomb","{T}: Add {1} to your mana pool.
{3}{B}{G}: Until end of turn, Svogthos, the Restless Tomb becomes a black and green Plant Zombie creature with ""This creature's power and toughness are each equal to the number of creature cards in your graveyard."" It's still a land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kongming, ""Sleeping Dragon""","2","2","W","Other creatures you control get +1/+1.","2WW","Legendary Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spectral Flight","U","Enchant creature
Enchanted creature gets +2/+2 and has flying.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gempalm Polluter","4","3","B","Cycling {B}{B} ({B}{B}, Discard this card: Draw a card.)
When you cycle Gempalm Polluter, you may have target player lose life equal to the number of Zombies on the battlefield.","5B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kemba's Skyguard","2","2","W","Flying
When Kemba's Skyguard enters the battlefield, you gain 2 life.","1WW","Creature - Cat Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heretic's Punishment","R","{3}{R}: Choose target creature or player, then put the top three cards of your library into your graveyard.  Heretic's Punishment deals damage to that creature or player equal to the highest converted mana cost among those cards.","4R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Elite","1","1","G","Kicker {8} (You may pay an additional {8} as you cast this spell.)
Trample
If Llanowar Elite was kicked, it enters the battlefield with five +1/+1 counters on it.","G","Creature - Elf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Angelic Armaments","Equipped creature gets +2/+2, has flying, and is a white Angel in addition to its other colors and types.
Equip {4}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rowan Treefolk","3","4","G",NULL,"3G","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shatter","R","Destroy target artifact.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Snap","B","Remove all counters from all permanents and exile all tokens.","3BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Foresight","U","Search your library for three cards, exile them, then shuffle your library.
Draw a card at the beginning of the next turn's upkeep.","1U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Story Circle","W","As Story Circle enters the battlefield, choose a color.
{W}: The next time a source of your choice of the chosen color would deal damage to you this turn, prevent that damage.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abyssal Specter","2","3","B","Flying
Whenever Abyssal Specter deals damage to a player, that player discards a card.","2BB","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enlisted Wurm","5","5","WG","Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)","4GW","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tunnel Ignus","2","1","R","Whenever a land enters the battlefield under an opponent's control, if that player had another land enter the battlefield under his or her control this turn, Tunnel Ignus deals 3 damage to that player.","1R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Traders","1","2","U","When Merfolk Traders enters the battlefield, draw a card, then discard a card.","1U","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beast Within","G","Destroy target permanent. Its controller puts a 3/3 green Beast creature token onto the battlefield.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Words of Wisdom","U","You draw two cards, then each other player draws a card.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Knight","2","2","R","First strike, protection from white","RR","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Awakener Druid","1","1","G","When Awakener Druid enters the battlefield, target Forest becomes a 4/5 green Treefolk creature for as long as Awakener Druid remains on the battlefield. It's still a land.","2G","Creature - Human Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ghastly Haunting","Enchant creature
You control enchanted creature.",NULL,"Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Once More with Feeling","W","Remove Once More with Feeling from the game as well as all cards in play and in all graveyards. Each player shuffles his or her hand into her or his library, then draws seven cards. Each player's life total is set to 10.
DCI ruling: This card is restricted. (You cannot play with more than one in a deck.)","WWWW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rethink","U","Counter target spell unless its controller pays {X}, where X is its converted mana cost.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thicket Basilisk","2","4","G","Whenever Thicket Basilisk blocks or becomes blocked by a non-Wall creature, destroy that creature at end of combat.","3GG","Creature - Basilisk");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Walking Archive","1","1","Defender (This creature can't attack.)
Walking Archive enters the battlefield with a +1/+1 counter on it.
At the beginning of each player's upkeep, that player draws a card for each +1/+1 counter on Walking Archive.
{2}{W}{U}: Put a +1/+1 counter on Walking Archive.","3","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keldon Vandals","4","1","R","Echo {2}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Keldon Vandals enters the battlefield, destroy target artifact.","2R","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Smokeweaver","2","3","U","Flying, protection from red","2UU","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Waterspout Weavers","3","3","U","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Waterspout Weavers, you may reveal it. If you do, each creature you control gains flying until end of turn.","3UU","Creature - Merfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Runed Arch","Runed Arch enters the battlefield tapped.
{X}, {T}, Sacrifice Runed Arch: X target creatures with power 2 or less are unblockable this turn.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scarab of the Unseen","{T}, Sacrifice Scarab of the Unseen: Return all Auras attached to target permanent you own to their owners' hands. Draw a card at the beginning of the next turn's upkeep.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kuldotha Flamefiend","4","4","R","When Kuldotha Flamefiend enters the battlefield, you may sacrifice an artifact. If you do, Kuldotha Flamefiend deals 4 damage divided as you choose among any number of target creatures and/or players.","4RR","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plague Spores","BR","Destroy target nonblack creature and target land. They can't be regenerated.","4BR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Empty the Warrens","R","Put two 1/1 red Goblin creature tokens onto the battlefield.
Storm (When you cast this spell, copy it for each spell cast before it this turn.)","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burst Lightning","R","Kicker {4} (You may pay an additional {4} as you cast this spell.)
Burst Lightning deals 2 damage to target creature or player. If Burst Lightning was kicked, it deals 4 damage to that creature or player instead.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Oasis","{T}: Prevent the next 1 damage that would be dealt to target creature this turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gutter Grime","G","Whenever a nontoken creature you control dies, put a slime counter on Gutter Grime, then put a green Ooze creature token onto the battlefield with ""This creature's power and toughness are each equal to the number of slime counters on Gutter Grime.""","4G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of the Chosen","{T}: Target legendary creature gets +2/+2 until end of turn.","2","Legendary Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eternity Vessel","Eternity Vessel enters the battlefield with X charge counters on it, where X is your life total.
Landfall - Whenever a land enters the battlefield under your control, you may have your life total become the number of charge counters on Eternity Vessel.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Triumph of Cruelty","B","At the beginning of your upkeep, target opponent discards a card if you control the creature with the greatest power or tied for the greatest power.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Walking Dream","3","3","U","Walking Dream is unblockable.
Walking Dream doesn't untap during your untap step if an opponent controls two or more creatures.","3U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stern Mentor","2","2","U","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Stern Mentor is paired with another creature, each of those creatures has ""{T}: Target player puts the top two cards of his or her library into his or her graveyard.""","3U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Volunteer Militia","1","2","W",NULL,"W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raging Kavu","3","1","RG","Flash
Haste","1RG","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jolting Merfolk","2","2","U","Fading 4 (This creature enters the battlefield with four fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Remove a fade counter from Jolting Merfolk: Tap target creature.","2UU","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jeweled Spirit","3","3","W","Flying
Sacrifice two lands: Jeweled Spirit gains protection from artifacts or from the color of your choice until end of turn.","3WW","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mogg Salvage","R","If an opponent controls an Island and you control a Mountain, you may cast Mogg Salvage without paying its mana cost.
Destroy target artifact.","2R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Meekstone","Creatures with power 3 or greater don't untap during their controllers' untap steps.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Canyon Wildcat","2","1","R","Mountainwalk","1R","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Norin the Wary","2","1","R","When a player casts a spell or a creature attacks, exile Norin the Wary. Return it to the battlefield under its owner's control at the beginning of the next end step.","R","Legendary Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Darigaaz's Charm","BRG","Choose one - Return target creature card from your graveyard to your hand; or Darigaaz's Charm deals 3 damage to target creature or player; or target creature gets +3/+3 until end of turn.","BRG","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hedron Matrix","Equipped creature gets +X/+X, where X is its converted mana cost.
Equip {4}","4","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grounded","G","Enchant creature
Enchanted creature loses flying.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Growth Spasm","G","Search your library for a basic land card, put it onto the battlefield tapped, then shuffle your library. Put a 0/1 colorless Eldrazi Spawn creature token onto the battlefield. It has ""Sacrifice this creature: Add {1} to your mana pool.""","2G","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vivid Meadow","Vivid Meadow enters the battlefield tapped with two charge counters on it.
{T}: Add {W} to your mana pool.
{T}, Remove a charge counter from Vivid Meadow: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benthic Djinn","5","3","UB","Islandwalk
At the beginning of your upkeep, you lose 2 life.","2UB","Creature - Djinn");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Drifting Meadow","Drifting Meadow enters the battlefield tapped.
{T}: Add {W} to your mana pool.
Cycling {2} ({2}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shiv's Embrace","R","Enchant creature
Enchanted creature gets +2/+2 and has flying.
{R}: Enchanted creature gets +1/+0 until end of turn.","2RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Chieftain","2","2","R","Haste (This creature can attack and {T} as soon as it comes under your control.)
Other Goblin creatures you control get +1/+1 and have haste.","1RR","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pyramids","{2}: Choose one - Destroy target Aura attached to a land; or the next time target land would be destroyed this turn, remove all damage marked on it instead.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Floral Spuzzem","2","2","G","Whenever Floral Spuzzem attacks and isn't blocked, you may destroy target artifact defending player controls. If you do, Floral Spuzzem assigns no combat damage this turn.","3G","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quest for Ula's Temple","U","At the beginning of your upkeep, you may look at the top card of your library. If it's a creature card, you may reveal it and put a quest counter on Quest for Ula's Temple.
At the beginning of each end step, if there are three or more quest counters on Quest for Ula's Temple, you may put a Kraken, Leviathan, Octopus, or Serpent creature card from your hand onto the battlefield.","U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thelonite Hermit","1","1","G","Saproling creatures get +1/+1.
Morph {3}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Thelonite Hermit is turned face up, put four 1/1 green Saproling creature tokens onto the battlefield.","3G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Insist","G","The next creature spell you cast this turn can't be countered by spells or abilities.
Draw a card.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Evolution Charm","G","Choose one - Search your library for a basic land card, reveal it, put it into your hand, then shuffle your library; or return target creature card from your graveyard to your hand; or target creature gains flying until end of turn.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hypnox","8","8","B","Flying
When Hypnox enters the battlefield, if you cast it from your hand, exile all cards from target opponent's hand.
When Hypnox leaves the battlefield, return the exiled cards to their owner's hand.","8BBB","Creature - Nightmare Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defense of the Heart","G","At the beginning of your upkeep, if an opponent controls three or more creatures, sacrifice Defense of the Heart, search your library for up to two creature cards, and put those cards onto the battlefield. Then shuffle your library.","3G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sakashima the Impostor","3","1","U","You may have Sakashima the Impostor enter the battlefield as a copy of any creature on the battlefield, except its name is still Sakashima the Impostor, it's legendary in addition to its other types, and it gains ""{2}{U}{U}: Return Sakashima the Impostor to its owner's hand at the beginning of the next end step.""","2UU","Legendary Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soratami Rainshaper","2","1","U","Flying
{3}, Return a land you control to its owner's hand: Target creature you control gains shroud until end of turn. (It can't be the target of spells or abilities.)","2U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slithering Shade","0","1","B","Defender (This creature can't attack.)
{B}: Slithering Shade gets +1/+1 until end of turn.
Hellbent - Slithering Shade can attack as though it didn't have defender as long as you have no cards in hand.","B","Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rimehorn Aurochs","3","3","G","Trample
Whenever Rimehorn Aurochs attacks, it gets +1/+0 until end of turn for each other attacking Aurochs.
{2}{S}i}: Target creature blocks target creature this turn if able. ({S}i} can be paid with one mana from a snow permanent.)","4G","Snow Creature - Aurochs");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pulse of Llanowar","G","If a basic land you control is tapped for mana, it produces mana of a color of your choice instead of any other type.","3G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shu Cavalry","2","2","W","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","2W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hanna's Custody","W","All artifacts have shroud. (They can't be the targets of spells or abilities.)","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Trooper","1","1","W","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever Soltari Trooper attacks, it gets +1/+1 until end of turn.","1W","Creature - Soltari Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Sylvok Replica","1","3","{G}, Sacrifice Sylvok Replica: Destroy target artifact or enchantment.","3","Artifact Creature - Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ulasht, the Hate Seed","0","0","RG","Ulasht, the Hate Seed enters the battlefield with a +1/+1 counter on it for each other red creature you control and a +1/+1 counter on it for each other green creature you control.
{1}, Remove a +1/+1 counter from Ulasht: Choose one - Ulasht deals 1 damage to target creature; or put a 1/1 green Saproling creature token onto the battlefield.","2RG","Legendary Creature - Hellion Hydra");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arnjlot's Ascent","U","Cumulative upkeep {U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
{1}: Target creature gains flying until end of turn.","1UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sinister Strength","B","Enchant creature
Enchanted creature gets +3/+1 and is black.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cloudshift","W","Exile target creature you control, then return that card to the battlefield under your control.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auramancer","2","2","W","When Auramancer enters the battlefield, you may return target enchantment card from your graveyard to your hand.","2W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Multani's Harmony","G","Enchant creature
Enchanted creature has ""{T}: Add one mana of any color to your mana pool.""","G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shadowstorm","R","Shadowstorm deals 2 damage to each creature with shadow.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellhole Rats","2","2","BR","Haste
When Hellhole Rats enters the battlefield, target player discards a card. Hellhole Rats deals damage to that player equal to that card's converted mana cost.","2BR","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Worms of the Earth","B","Players can't play lands.
Lands can't enter the battlefield.
At the beginning of each upkeep, any player may sacrifice two lands or have Worms of the Earth deal 5 damage to him or her. If a player does either, destroy Worms of the Earth.","2BBB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krark-Clan Shaman","1","1","R","Sacrifice an artifact: Krark-Clan Shaman deals 1 damage to each creature without flying.","R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Maniacal Rage","R","Enchant creature
Enchanted creature gets +2/+2 and can't block.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Asmira, Holy Avenger","2","3","WG","Flying
At the beginning of each end step, put a +1/+1 counter on Asmira, Holy Avenger for each creature put into your graveyard from the battlefield this turn.","2GW","Legendary Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nested Ghoul","4","2","B","Whenever a source deals damage to Nested Ghoul, put a 2/2 black Zombie creature token onto the battlefield.","3BB","Creature - Zombie Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nomad Mythmaker","2","2","W","{W}, {T}: Put target Aura card from a graveyard onto the battlefield under your control attached to a creature you control.","2W","Creature - Human Nomad Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parallectric Feedback","R","Parallectric Feedback deals damage to target spell's controller equal to that spell's converted mana cost.","3R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Strandwalker","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +2/+4 and has reach.
Equip {4}","5","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avatar of Might","8","8","G","If an opponent controls at least four more creatures than you, Avatar of Might costs {6} less to cast.
Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","6GG","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ironshell Beetle","1","1","G","When Ironshell Beetle enters the battlefield, put a +1/+1 counter on target creature.","1G","Creature - Insect");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Soldevi Golem","5","3","Soldevi Golem doesn't untap during your untap step.
At the beginning of your upkeep, you may untap target tapped creature an opponent controls. If you do, untap Soldevi Golem.","4","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avatar of Will","5","6","U","If an opponent has no cards in hand, Avatar of Will costs {6} less to cast.
Flying","6UU","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Havoc","R","Whenever an opponent casts a white spell, he or she loses 2 life.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alms","W","{1}, Exile the top card of your graveyard: Prevent the next 1 damage that would be dealt to target creature this turn.","W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wind Zendikon","U","Enchant land
Enchanted land is a 2/2 blue Elemental creature with flying. It's still a land.
When enchanted land dies, return that card to its owner's hand.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Malach of the Dawn","2","4","W","Flying
{W}{W}{W}: Regenerate Malach of the Dawn.","2WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reassembling Skeleton","1","1","B","{1}{B}: Return Reassembling Skeleton from your graveyard to the battlefield tapped.","1B","Creature - Skeleton Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rupture","R","Sacrifice a creature. Rupture deals damage equal to that creature's power to each creature without flying and each player.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Theft of Dreams","U","Draw a card for each tapped creature target opponent controls.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Climber","3","3","G","When Kavu Climber enters the battlefield, draw a card.","3GG","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Marsh Flats","{T}, Pay 1 life, Sacrifice Marsh Flats: Search your library for a Plains or Swamp card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disappear","U","Enchant creature
{U}: Return enchanted creature and Disappear to their owners' hands.","2UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chained Throatseeker","5","5","U","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Chained Throatseeker can't attack unless defending player is poisoned.","5U","Creature - Horror");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Copper-Leaf Angel","2","2","Flying
{T}, Sacrifice X lands: Put X +1/+1 counters on Copper-Leaf Angel.","5","Artifact Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Loxodon Hierarch Avatar","Sacrifice a permanent: Regenerate target creature you control.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mishra's Workshop","{T}: Add {3} to your mana pool. Spend this mana only to cast artifact spells.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gargantuan Gorilla","7","7","G","At the beginning of your upkeep, you may sacrifice a Forest. If you sacrifice a snow Forest this way, Gargantuan Gorilla gains trample until end of turn. If you don't sacrifice a Forest, sacrifice Gargantuan Gorilla and it deals 7 damage to you.
{T}: Gargantuan Gorilla deals damage equal to its power to another target creature. That creature deals damage equal to its power to Gargantuan Gorilla.","4GGG","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lovisa Coldeyes","3","3","R","Warrior, Berserker, and/or other Barbarian creatures get +2/+2 and have haste.","3RR","Legendary Creature - Human Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consecrate Land","W","Enchant land
Enchanted land is indestructible and can't be enchanted by other Auras.","W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sphinx-Bone Wand","Whenever you cast an instant or sorcery spell, you may put a charge counter on Sphinx-Bone Wand. If you do, Sphinx-Bone Wand deals damage equal to the number of charge counters on it to target creature or player.","7","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Briarhorn","3","3","G","Flash
When Briarhorn enters the battlefield, target creature gets +3/+3 until end of turn.
Evoke {1}{G} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","3G","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Revelation","G","Players play with their hands revealed.","G","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oversold Cemetery","B","At the beginning of your upkeep, if you have four or more creature cards in your graveyard, you may return target creature card from your graveyard to your hand.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wandering Eye","1","3","U","Flying
Players play with their hands revealed.","2U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anurid Murkdiver","4","3","B","Swampwalk","4BB","Creature - Zombie Frog Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Electric Eel","1","1","U","When Electric Eel enters the battlefield, it deals 1 damage to you.
{R}{R}: Electric Eel gets +2/+0 until end of turn and deals 1 damage to you.","U","Creature - Fish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Electropotence","R","Whenever a creature enters the battlefield under your control, you may pay {2}{R}. If you do, that creature deals damage equal to its power to target creature or player.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marjhan","8","8","U","Marjhan doesn't untap during your untap step.
{U}{U}, Sacrifice a creature: Untap Marjhan. Activate this ability only during your upkeep.
Marjhan can't attack unless defending player controls an Island.
{U}{U}: Marjhan gets -1/-0 until end of turn and deals 1 damage to target attacking creature without flying.
When you control no Islands, sacrifice Marjhan.","5UU","Creature - Leviathan");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clinging Mists","G","Prevent all combat damage that would be dealt this turn.
Fateful hour - If you have 5 or less life, tap all attacking creatures. Those creatures don't untap during their controller's next untap step.","2G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crevasse","R","Creatures with mountainwalk can be blocked as though they didn't have mountainwalk.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Index","U","Look at the top five cards of your library, then put them back in any order.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nightcreep","B","Until end of turn, all creatures become black and all lands become Swamps.","BB","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gemstone Mine","Gemstone Mine enters the battlefield with three mining counters on it.
{T}, Remove a mining counter from Gemstone Mine: Add one mana of any color to your mana pool. If there are no mining counters on Gemstone Mine, sacrifice it.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yavimaya's Embrace","UG","Enchant creature
You control enchanted creature.
Enchanted creature gets +2/+2 and has trample.","5GUU","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chalice of Life","{T}: You gain 1 life. Then if you have at least 10 life more than your starting life total, transform Chalice of Life.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jabari's Influence","W","Cast Jabari's Influence only after combat.
Gain control of target nonartifact, nonblack creature that attacked you this turn and put a -1/-0 counter on it.","3WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Indomitable Ancients","2","10","W",NULL,"2WW","Creature - Treefolk Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Waterfront Bouncer","1","1","U","{U}, {T}, Discard a card: Return target creature to its owner's hand.","1U","Creature - Merfolk Spellshaper");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Frenetic Efreet Avatar","Each permanent you control has phasing. (It phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)
At the beginning of your end step, flip a coin. If you win the flip, take an extra turn after this one.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mass of Ghouls","5","3","B",NULL,"3BB","Creature - Zombie Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Holistic Wisdom","G","{2}, Exile a card from your hand: Return target card from your graveyard to your hand if it shares a card type with the card exiled this way. (Artifact, creature, enchantment, instant, land, planeswalker, sorcery, and tribal are card types.)","1GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Norwood Ranger","1","2","G",NULL,"G","Creature - Elf Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Traveler's Amulet","{1}, Sacrifice Traveler's Amulet: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kabira Evangel","2","3","W","Whenever Kabira Evangel or another Ally enters the battlefield under your control, you may choose a color. If you do, Allies you control gain protection from the chosen color until end of turn.","2W","Creature - Human Cleric Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Threaten","R","Untap target creature and gain control of it until end of turn. That creature gains haste until end of turn. (It can attack and {T} this turn.)","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blazing Salvo","R","Blazing Salvo deals 3 damage to target creature unless that creature's controller has Blazing Salvo deal 5 damage to him or her.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rats of Rath","2","1","B","{B}: Destroy target artifact, creature, or land you control.","1B","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cycle of Life","G","Return Cycle of Life to its owner's hand: Target creature you cast this turn becomes 0/1 until your next upkeep. At the beginning of your next upkeep, put a +1/+1 counter on that creature.","1GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Citanul Druid","1","1","G","Whenever an opponent casts an artifact spell, put a +1/+1 counter on Citanul Druid.","1G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snowblind","G","Enchant creature
Enchanted creature gets -X/-Y. If that creature is attacking, X is the number of snow lands defending player controls. Otherwise, X is the number of snow lands its controller controls. Y is equal to X or to enchanted creature's toughness minus 1, whichever is smaller.","3G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thoughts of Ruin","R","Each player sacrifices a land for each card in your hand.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Faith's Fetters","W","Enchant permanent
When Faith's Fetters enters the battlefield, you gain 4 life.
Enchanted permanent's activated abilities can't be activated unless they're mana abilities. If enchanted permanent is a creature, it can't attack or block.","3W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mercy Killing","WG","Target creature's controller sacrifices it, then puts X 1/1 green and white Elf Warrior creature tokens onto the battlefield, where X is that creature's power.","2(G/W)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blunt the Assault","G","You gain 1 life for each creature on the battlefield. Prevent all combat damage that would be dealt this turn.","3G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Duplicity","U","When Duplicity enters the battlefield, exile the top five cards of your library face down.
At the beginning of your upkeep, you may exile all cards from your hand face down. If you do, put all other cards you own exiled with Duplicity into your hand.
At the beginning of your end step, discard a card.
When you lose control of Duplicity, put all cards exiled with Duplicity into their owner's graveyard.","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Throat Slitter","2","2","B","Ninjutsu {2}{B} ({2}{B}, Return an unblocked attacker you control to hand: Put this card onto the battlefield from your hand tapped and attacking.)
Whenever Throat Slitter deals combat damage to a player, destroy target nonblack creature that player controls.","4B","Creature - Rat Ninja");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Boggart Shenanigans","R","Whenever another Goblin you control dies, you may have Boggart Shenanigans deal 1 damage to target player.","2R","Tribal Enchantment - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Neurok Transmuter","2","2","U","{U}: Target creature becomes an artifact in addition to its other types until end of turn.
{U}: Until end of turn, target artifact creature becomes blue and isn't an artifact.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skarrgan Pit-Skulk","1","1","G","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
Creatures with power less than Skarrgan Pit-Skulk's power can't block it.","G","Creature - Human Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scroll of Origins","{2}, {T}: Draw a card if you have seven or more cards in hand.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Greater Good","G","Sacrifice a creature: Draw cards equal to the sacrificed creature's power, then discard three cards.","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tolarian Drake","2","4","U","Flying
Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)","2U","Creature - Drake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chalice of Death","{T}: Target player loses 5 life.",NULL,"Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lich's Mirror","If you would lose the game, instead shuffle your hand, your graveyard, and all permanents you own into your library, then draw seven cards and your life total becomes 20.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caustic Hound","4","4","B","When Caustic Hound dies, each player loses 4 life.","5B","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Renegade Troops","4","2","R","Haste","4R","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mage's Guile","U","Target creature gains shroud until end of turn. (It can't be the target of spells or abilities.)
Cycling {U} ({U}, Discard this card: Draw a card.)","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Serum Powder","{T}: Add {1} to your mana pool.
Any time you could mulligan and Serum Powder is in your hand, you may exile all the cards from your hand, then draw that many cards. (You can do this in addition to taking mulligans.)","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thalakos Dreamsower","1","1","U","Shadow (This creature can block or be blocked by only creatures with shadow.)
You may choose not to untap Thalakos Dreamsower during your untap step.
Whenever Thalakos Dreamsower deals damage to an opponent, tap target creature. That creature doesn't untap during its controller's untap step for as long as Thalakos Dreamsower remains tapped.","2U","Creature - Thalakos Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tranquility","G","Destroy all enchantments.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ezuri, Renegade Leader","2","2","G","{G}: Regenerate another target Elf.
{2}{G}{G}{G}: Elf creatures you control get +3/+3 and gain trample until end of turn.","1GG","Legendary Creature - Elf Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Titan Forge","{3}, {T}: Put a charge counter on Titan Forge.
{T}, Remove three charge counters from Titan Forge: Put a 9/9 colorless Golem artifact creature token onto the battlefield.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Takklemaggot","B","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, put a -0/-1 counter on that creature.
When enchanted creature dies, that creature's controller chooses a creature that Takklemaggot could enchant. If he or she does, return Takklemaggot to the battlefield under your control attached to that creature. If he or she doesn't, return Takklemaggot to the battlefield under your control as a non-Aura enchantment. It loses ""enchant creature"" and gains ""At the beginning of that player's upkeep, Takklemaggot deals 1 damage to him or her.""","2BB","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scattershot","R","Scattershot deals 1 damage to target creature.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cryptwailing","B","{1}, Exile two creature cards from your graveyard: Target player discards a card. Activate this ability only any time you could cast a sorcery.","3B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Patron of the Kitsune","5","6","W","Fox offering (You may cast this card any time you could cast an instant by sacrificing a Fox and paying the difference in mana costs between this and the sacrificed Fox. Mana cost includes color.)
Whenever a creature attacks, you may gain 1 life.","4WW","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wrexial, the Risen Deep","5","8","UB","Islandwalk, swampwalk
Whenever Wrexial, the Risen Deep deals combat damage to a player, you may cast target instant or sorcery card from that player's graveyard without paying its mana cost. If that card would be put into a graveyard this turn, exile it instead.","3UUB","Legendary Creature - Kraken");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Altar of Shadows","At the beginning of your precombat main phase, add {B} to your mana pool for each charge counter on Altar of Shadows.
{7}, {T}: Destroy target creature. Then put a charge counter on Altar of Shadows.","7","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Retether","W","Return each Aura card from your graveyard to the battlefield. Only creatures can be enchanted this way. (Aura cards that can't enchant a creature on the battlefield remain in your graveyard.)","3W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gather the Townsfolk","W","Put two 1/1 white Human creature tokens onto the battlefield.
Fateful hour - If you have 5 or less life, put five of those tokens onto the battlefield instead.","1W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shape Anew","U","The controller of target artifact sacrifices it, then reveals cards from the top of his or her library until he or she reveals an artifact card. That player puts that card onto the battlefield, then shuffles all other cards revealed this way into his or her library.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Necratog","1","2","B","Exile the top creature card of your graveyard: Necratog gets +2/+2 until end of turn.","1BB","Creature - Atog");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Giant Fan","o2, ocT: Move target counter from one card to another. If the second card's rules text refers to any type of counters, the moved counter becomes one of those counters. Otherwise, it becomes a +1/+1 counter.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Excruciator","7","7","R","Damage that would be dealt by Excruciator can't be prevented.","6RR","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daru Cavalier","2","2","W","First strike
When Daru Cavalier enters the battlefield, you may search your library for a card named Daru Cavalier, reveal it, and put it into your hand. If you do, shuffle your library.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Crimson Kobolds","0","1",NULL,"0","Creature - Kobold");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bursting Beebles","2","2","U","Bursting Beebles is unblockable as long as defending player controls two or more nonland permanents that share an artist.","2U","Creature - Beeble");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flame-Kin War Scout","2","4","R","When another creature enters the battlefield, sacrifice Flame-Kin War Scout. If you do, Flame-Kin War Scout deals 4 damage to that creature.","3R","Creature - Elemental Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breath of Darigaaz","R","Kicker {2} (You may pay an additional {2} as you cast this spell.)
Breath of Darigaaz deals 1 damage to each creature without flying and each player. If Breath of Darigaaz was kicked, it deals 4 damage to each creature without flying and each player instead.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battletide Alchemist","3","4","W","If a source would deal damage to a player, you may prevent X of that damage, where X is the number of Clerics you control.","3WW","Creature - Kithkin Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slow Motion","U","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, that player sacrifices that creature unless he or she pays {2}.
When Slow Motion is put into a graveyard from the battlefield, return Slow Motion to its owner's hand.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Edric, Spymaster of Trest","2","2","UG","Whenever a creature deals combat damage to one of your opponents, its controller may draw a card.","1GU","Legendary Creature - Elf Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Withstand Death","G","Target creature is indestructible this turn. (Lethal damage and effects that say ""destroy"" don't destroy it. If its toughness is 0 or less, it's still put into its owner's graveyard.)","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Enclave Elite","2","2","U","Multikicker {1}{U} (You may pay an additional {1}{U} any number of times as you cast this spell.)
Islandwalk
Enclave Elite enters the battlefield with a +1/+1 counter on it for each time it was kicked.","2U","Creature - Merfolk Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Awesome Presence","U","Enchant creature
Enchanted creature can't be blocked unless defending player pays {3} for each creature he or she controls that's blocking it.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akroma, Angel of Wrath","6","6","W","Flying, first strike, vigilance, trample, haste, protection from black and from red","5WWW","Legendary Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelic Benediction","W","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
Whenever a creature you control attacks alone, you may tap target creature.","3W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindstab Thrull","2","2","B","Whenever Mindstab Thrull attacks and isn't blocked, you may sacrifice it. If you do, defending player discards three cards.","1BB","Creature - Thrull");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tuktuk the Explorer","1","1","R","Haste
When Tuktuk the Explorer dies, put a legendary 5/5 colorless Goblin Golem artifact creature token named Tuktuk the Returned onto the battlefield.","2R","Legendary Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brood Birthing","R","If you control an Eldrazi Spawn, put three 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool."" Otherwise, put one of those tokens onto the battlefield.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Rot","B","Target player discards two cards.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Furystoke Giant","3","3","R","When Furystoke Giant enters the battlefield, other creatures you control gain ""{T}: This creature deals 2 damage to target creature or player"" until end of turn.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","3RR","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Common Cause","W","Nonartifact creatures get +2/+2 as long as they all share a color.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Words of Wilding","G","{1}: The next time you would draw a card this turn, put a 2/2 green Bear creature token onto the battlefield instead.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mul Daya Channelers","2","2","G","Play with the top card of your library revealed.
As long as the top card of your library is a creature card, Mul Daya Channelers gets +3/+3.
As long as the top card of your library is a land card, Mul Daya Channelers has ""{T}: Add two mana of any one color to your mana pool.""","1GG","Creature - Elf Druid Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chancellor of the Forge","5","5","R","You may reveal this card from your opening hand. If you do, at the beginning of the first upkeep, put a 1/1 red Goblin creature token with haste onto the battlefield.
When Chancellor of the Forge enters the battlefield, put X 1/1 red Goblin creature tokens with haste onto the battlefield, where X is the number of creatures you control.","4RRR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stun Sniper","1","1","WR","{1}, {T}: Stun Sniper deals 1 damage to target creature. Tap that creature.","RW","Creature - Human Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cylian Elf","2","2","G",NULL,"1G","Creature - Elf Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pulse of the Grid","U","Draw two cards, then discard a card. Then if an opponent has more cards in hand than you, return Pulse of the Grid to its owner's hand.","1UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Slayer","2","2","B","Flying
Whenever Phyrexian Slayer becomes blocked by a white creature, destroy that creature. It can't be regenerated.","3B","Creature - Minion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Moorland Haunt","{T}: Add {1} to your mana pool.
{W}{U}, {T}, Exile a creature card from your graveyard: Put a 1/1 white Spirit creature token with flying onto the battlefield.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Into the Earthen Maw","When you set this scheme in motion, exile up to one target creature with flying, up to one target creature without flying, and all cards from up to one target opponent's graveyard.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Skyhunter","2","2","W","Flying","WW","Creature - Cat Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Slate of Ancestry","{4}, {T}, Discard your hand: Draw a card for each creature you control.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lethargy Trap","U","If three or more creatures are attacking, you may pay {U} rather than pay Lethargy Trap's mana cost.
Attacking creatures get -3/-0 until end of turn.","3U","Instant - Trap");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yet Another AEther Vortex","R","All creatures have haste.
Players play with the top card of their libraries revealed.
Noninstant, nonsorcery cards on top of a library are in play under their owner's control in addition to being in that library.","3RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Pet","1","1","B","Sacrifice Blood Pet: Add {B} to your mana pool.","B","Creature - Thrull");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karador, Ghost Chieftain","3","4","WBG","Karador, Ghost Chieftain costs {1} less to cast for each creature card in your graveyard.
During each of your turns, you may cast one creature card from your graveyard.","5BGW","Legendary Creature - Centaur Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bull Rush","R","Target creature gets +2/+0 until end of turn.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Engulfing Flames","R","Engulfing Flames deals 1 damage to target creature. It can't be regenerated this turn.
Flashback {3}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloud Elemental","2","3","U","Flying
Cloud Elemental can block only creatures with flying.","2U","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jeweled Torque","As Jeweled Torque enters the battlefield, choose a color.
Whenever a player casts a spell of the chosen color, you may pay {2}. If you do, you gain 2 life.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Joraga Bard","1","4","G","Whenever Joraga Bard or another Ally enters the battlefield under your control, you may have Ally creatures you control gain vigilance until end of turn.","3G","Creature - Elf Rogue Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Rats","1","1","B","Bog Rats can't be blocked by Walls.","B","Creature - Rat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Herder","1","1","G","{G}: Target creature gains trample until end of turn.","G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Volrath the Fallen","6","4","B","{1}{B}, Discard a creature card: Volrath the Fallen gets +X/+X until end of turn, where X is the discarded card's converted mana cost.","3BBB","Legendary Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Drake","4","3","U","Flying
When Sea Drake enters the battlefield, return two target lands you control to their owner's hand.","2U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Child of Alara","6","6","WUBRG","Trample
When Child of Alara dies, destroy all nonland permanents. They can't be regenerated.","WUBRG","Legendary Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gigantomancer","1","1","G","{1}: Target creature you control becomes 7/7 until end of turn.","7G","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Outrage Shaman","2","2","R","Chroma - When Outrage Shaman enters the battlefield, it deals damage to target creature equal to the number of red mana symbols in the mana costs of permanents you control.","3RR","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunted Wumpus","6","6","G","When Hunted Wumpus enters the battlefield, each other player may put a creature card from his or her hand onto the battlefield.","3G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Splinter","G","Exile target artifact. Search its controller's graveyard, hand, and library for all cards with the same name as that artifact and exile them. Then that player shuffles his or her library.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Palinchron","4","5","U","Flying
When Palinchron enters the battlefield, untap up to seven lands.
{2}{U}{U}: Return Palinchron to its owner's hand.","5UU","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Righteous War","WB","White creatures you control have protection from black.
Black creatures you control have protection from white.","1WB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yixlid Jailer","2","1","B","Cards in graveyards lose all abilities.","1B","Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Loreweaver","2","1","W","{1}{W}: Kitsune Loreweaver gets +0/+X until end of turn, where X is the number of cards in your hand.","1W","Creature - Fox Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Emerald Medallion","Green spells you cast cost {1} less to cast.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jwari Shapeshifter","0","0","U","You may have Jwari Shapeshifter enter the battlefield as a copy of any Ally creature on the battlefield.","1U","Creature - Shapeshifter Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soot Imp","1","2","B","Flying
Whenever a player casts a nonblack spell, that player loses 1 life.","1BB","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Taoist Hermit","2","2","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)","2G","Creature - Human Mystic");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dakmor Bat","1","1","B","Flying","1B","Creature - Bat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Biorhythm","G","Each player's life total becomes the number of creatures he or she controls.","6GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opposition","U","Tap an untapped creature you control: Tap target artifact, creature, or land.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warren Instigator","1","1","R","Double strike
Whenever Warren Instigator deals damage to an opponent, you may put a Goblin creature card from your hand onto the battlefield.","RR","Creature - Goblin Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Imperial Seal","B","Search your library for a card, then shuffle your library and put that card on top of it. You lose 2 life.","B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mountain","R",NULL,"Basic Land - Mountain");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pygmy Hippo","2","2","UG","Whenever Pygmy Hippo attacks and isn't blocked, you may have defending player activate a mana ability of each land he or she controls and empty his or her mana pool. If you do, Pygmy Hippo assigns no combat damage this turn and at the beginning of your postcombat main phase, you add {X} to your mana pool, where X is the amount of mana emptied from defending player's mana pool this way.","GU","Creature - Hippo");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Unstable Frontier","{T}: Add {1} to your mana pool.
{T}: Target land you control becomes the basic land type of your choice until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cateran Overlord","7","5","B","Sacrifice a creature: Regenerate Cateran Overlord.
{6}, {T}: Search your library for a Mercenary permanent card with converted mana cost 6 or less and put it onto the battlefield. Then shuffle your library.","4BBB","Creature - Horror Mercenary");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("My Wish Is Your Command","When you set this scheme in motion, each opponent reveals his or her hand. You may choose a noncreature, nonland card revealed this way and cast it without paying its mana cost.",NULL,"Scheme");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Faerie Conclave","Faerie Conclave enters the battlefield tapped.
{T}: Add {U} to your mana pool.
{1}{U}: Faerie Conclave becomes a 2/1 blue Faerie creature with flying until end of turn. It's still a land. (It can't be blocked except by creatures with flying or reach.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quicksilver Dragon","5","5","U","Flying
{U}: If target spell has only one target and that target is Quicksilver Dragon, change that spell's target to another creature.
Morph {4}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4UU","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Puppeteer Clique","3","2","B","Flying
When Puppeteer Clique enters the battlefield, put target creature card from an opponent's graveyard onto the battlefield under your control. It gains haste. At the beginning of your next end step, exile it.
Persist (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)","3BB","Creature - Faerie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boneknitter","1","1","B","{1}{B}: Regenerate target Zombie.
Morph {2}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1B","Creature - Zombie Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scarred Puma","2","1","R","Scarred Puma can't attack unless a black or green creature also attacks.","R","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Breach","U","Whenever a player casts a spell, that player returns a land he or she controls to its owner's hand.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rimebound Dead","1","1","B","{S}i}: Regenerate Rimebound Dead. ({S}i} can be paid with one mana from a snow permanent.)","B","Snow Creature - Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Damia, Sage of Stone","4","4","UBG","Deathtouch
Skip your draw step.
At the beginning of your upkeep, if you have fewer than seven cards in hand, draw cards equal to the difference.","4GUB","Legendary Creature - Gorgon Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warping Wurm","1","1","UG","Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)
At the beginning of your upkeep, Warping Wurm phases out unless you pay {2}{G}{U}.
When Warping Wurm phases in, put a +1/+1 counter on it.","2GU","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Root-Kin Ally","3","3","G","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Tap two untapped creatures you control: Root-Kin Ally gets +2/+2 until end of turn.","4GG","Creature - Elemental Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Need for Speed","R","Sacrifice a land: Target creature gains haste until end of turn.","R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cutthroat il-Dal","4","1","B","Hellbent - Cutthroat il-Dal has shadow as long as you have no cards in hand. (It can block or be blocked by only creatures with shadow.)","3B","Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Swarmyard","{T}: Add {1} to your mana pool.
{T}: Regenerate target Insect, Rat, Spider, or Squirrel.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kyren Negotiations","R","Tap an untapped creature you control: Kyren Negotiations deals 1 damage to target player.","2RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Meteor Shower","R","Meteor Shower deals X plus 1 damage divided as you choose among any number of target creatures and/or players.","XXR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Urza's Chalice","Whenever a player casts an artifact spell, you may pay {1}. If you do, you gain 1 life.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Manacles of Decay","W","Enchant creature
Enchanted creature can't attack.
{B}: Enchanted creature gets -1/-1 until end of turn.
{R}: Enchanted creature can't block this turn.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jund Battlemage","2","2","R","{B}, {T}: Target player loses 1 life.
{G}, {T}: Put a 1/1 green Saproling creature token onto the battlefield.","2R","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Laccolith Warrior","3","3","R","Whenever Laccolith Warrior becomes blocked, you may have it deal damage equal to its power to target creature. If you do, Laccolith Warrior assigns no combat damage this turn.","2RR","Creature - Beast Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shambling Shell","3","1","BG","Sacrifice Shambling Shell: Put a +1/+1 counter on target creature.
Dredge 3 (If you would draw a card, instead you may put exactly three cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","1BG","Creature - Plant Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barter in Blood","B","Each player sacrifices two creatures.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Trenches","WR","{2}, Sacrifice a land: Put two 1/1 red and white Goblin Soldier creature tokens onto the battlefield.","1RW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heartwood Storyteller Avatar","The first creature spell you cast each turn costs {1} less to cast.
The first noncreature spell each opponent casts each turn costs {1} more to cast.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Voltaic Construct","2","2","{2}: Untap target artifact creature.","4","Artifact Creature - Golem Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Defiance","G","Whenever a creature you control becomes the target of an instant or sorcery spell, that creature gets +3/+3 until end of turn.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Venser, Shaper Savant","2","2","U","Flash (You may cast this spell any time you could cast an instant.)
When Venser, Shaper Savant enters the battlefield, return target spell or permanent to its owner's hand.","2UU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sabertooth Nishoba","5","5","WG","Trample, protection from blue and from red","4GW","Creature - Cat Beast Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mortuary","B","Whenever a creature is put into your graveyard from the battlefield, put that card on top of your library.","3B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vampire's Bite","B","Kicker {2}{B} (You may pay an additional {2}{B} as you cast this spell.)
Target creature gets +3/+0 until end of turn. If Vampire's Bite was kicked, that creature gains lifelink until end of turn. (Damage dealt by the creature also causes its controller to gain that much life.)","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Neck Snap","W","Destroy target attacking or blocking creature.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scuttling Death","4","2","B","Sacrifice Scuttling Death: Target creature gets -1/-1 until end of turn.
Soulshift 4 (When this creature dies, you may return target Spirit card with converted mana cost 4 or less from your graveyard to your hand.)","4B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rummaging Wizard","2","2","U","{2}{U}: Look at the top card of your library. You may put that card into your graveyard.","3U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Banishing","B","Destroy target nonblack creature. It can't be regenerated.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Contaminated Ground","B","Enchant land
Enchanted land is a Swamp.
Whenever enchanted land becomes tapped, its controller loses 2 life.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghastly Demise","B","Destroy target nonblack creature if its toughness is less than or equal to the number of cards in your graveyard.","B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corrupted Roots","B","Enchant Forest or Plains
Whenever enchanted land becomes tapped, its controller loses 2 life.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boggart Loggers","2","1","B","Forestwalk
{2}{B}, Sacrifice Boggart Loggers: Destroy target Treefolk or Forest.","2B","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charging Rhino","4","4","G","Charging Rhino can't be blocked by more than one creature.","3GG","Creature - Rhino");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volrath's Gardens","G","{2}, Tap an untapped creature you control: You gain 2 life. Activate this ability only any time you could cast a sorcery.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Staggershock","R","Staggershock deals 2 damage to target creature or player.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Stair Expedition","B","Landfall - Whenever a land enters the battlefield under your control, you may put a quest counter on Soul Stair Expedition.
Remove three quest counters from Soul Stair Expedition and sacrifice it: Return up to two target creature cards from your graveyard to your hand.","B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Smokespew Invoker","3","1","B","{7}{B}: Target creature gets -3/-3 until end of turn.","2B","Creature - Zombie Mutant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Launch","U","Enchant creature
Enchanted creature has flying.
When Launch is put into a graveyard from the battlefield, return Launch to its owner's hand.","1U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashnod","Whenever a creature deals damage to you, destroy it.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Genju of the Cedars","G","Enchant Forest
{2}: Enchanted Forest becomes a 4/4 green Spirit creature until end of turn. It's still a land.
When enchanted Forest is put into a graveyard, you may return Genju of the Cedars from your graveyard to your hand.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gaddock Teeg","2","2","WG","Noncreature spells with converted mana cost 4 or greater can't be cast.
Noncreature spells with {X} in their mana costs can't be cast.","GW","Legendary Creature - Kithkin Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ihsan's Shade","5","5","B","Protection from white","3BBB","Legendary Creature - Shade Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Wurm","5","4","R","When Wild Wurm enters the battlefield, flip a coin. If you lose the flip, return Wild Wurm to its owner's hand.","3R","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Remembrance","W","Whenever a nontoken creature you control dies, you may search your library for a card with the same name as that creature, reveal it, and put it into your hand. If you do, shuffle your library.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ice Storm","G","Destroy target land.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Anarchy","R","Destroy all white permanents.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deadapult","R","{R}, Sacrifice a Zombie: Deadapult deals 2 damage to target creature or player.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eater of the Dead","3","4","B","{0}: If Eater of the Dead is tapped, exile target creature card from a graveyard and untap Eater of the Dead.","4B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kodama of the North Tree","6","4","G","Trample; shroud (This permanent can't be the target of spells or abilities.)","2GGG","Legendary Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Aladdin's Lamp","{X}, {T}: The next time you would draw a card this turn, instead look at the top X cards of your library, put all but one of them on the bottom of your library in a random order, then draw a card. X can't be 0.","10","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitting Hydra","0","0","R","Spitting Hydra enters the battlefield with four +1/+1 counters on it.
{1}{R}, Remove a +1/+1 counter from Spitting Hydra: Spitting Hydra deals 1 damage to target creature.","3RR","Creature - Hydra");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abbey Griffin","2","2","W","Flying, vigilance","3W","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Heartlash Cinder","1","1","R","Haste
Chroma - When Heartlash Cinder enters the battlefield, it gets +X/+0 until end of turn, where X is the number of red mana symbols in the mana costs of permanents you control.","1R","Creature - Elemental Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sturmgeist","*","*","U","Flying
Sturmgeist's power and toughness are each equal to the number of cards in your hand.
Whenever Sturmgeist deals combat damage to a player, draw a card.","3UU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sygg, River Guide","2","2","WU","Islandwalk
{1}{W}: Target Merfolk you control gains protection from the color of your choice until end of turn.","WU","Legendary Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bosom Buddy","1","4","W","Whenever you play a spell, you may gain {1/2} life for each word in that spell's name.","3W","Creature - Elephant Townsfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phosphorescent Feast","G","Chroma - Reveal any number of cards in your hand. You gain 2 life for each green mana symbol in those cards' mana costs.","2GGG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sacred Knight","3","2","W","Sacred Knight can't be blocked by black and/or red creatures.","3W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lurking Skirge","B","When a creature is put into an opponent's graveyard from the battlefield, if Lurking Skirge is an enchantment, Lurking Skirge becomes a 3/2 Imp creature with flying.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Centaur Safeguard","3","1","WG","({(g/w)} can be paid with either {G} or {W}.)
When Centaur Safeguard dies, you may gain 3 life.","2(G/W)","Creature - Centaur Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bloodstone Cameo","{T}: Add {B} or {R} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cradle Guard","4","4","G","Trample
Echo {1}{G}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","1GG","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Filigree Angel","4","4","WU","Flying
When Filigree Angel enters the battlefield, you gain 3 life for each artifact you control.","5WWU","Artifact Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Pathmage","1","2","U","Cephalid Pathmage is unblockable.
{T}, Sacrifice Cephalid Pathmage: Target creature is unblockable this turn.","2U","Creature - Cephalid Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Fire","0","5","R","Defender (This creature can't attack.)
{R}: Wall of Fire gets +1/+0 until end of turn.","1RR","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Bombers","3","4","R","When another creature enters the battlefield, sacrifice Mogg Bombers and it deals 3 damage to target player.","3R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saber Ants","2","3","G","Whenever Saber Ants is dealt damage, you may put that many 1/1 green Insect creature tokens onto the battlefield.","3G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Beast Walkers","2","2","W","{G}: Beast Walkers gains banding until end of turn. (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","1WW","Creature - Human Beast Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razorclaw Bear","3","3","G","Whenever Razorclaw Bear becomes blocked, it gets +2/+2 until end of turn.","2GG","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbed Sliver","2","2","R","All Sliver creatures have ""{2}: This creature gets +1/+0 until end of turn.""","2R","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Callous Deceiver","1","3","U","{1}: Look at the top card of your library.
{2}: Reveal the top card of your library. If it's a land card, Callous Deceiver gets +1/+0 and gains flying until end of turn. Activate this ability only once each turn.","2U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Power Surge","R","At the beginning of each player's upkeep, Power Surge deals X damage to that player, where X is the number of untapped lands he or she controlled at the beginning of this turn.","RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chime of Night","B","Enchant creature
When Chime of Night is put into a graveyard from the battlefield, destroy target nonblack creature.","1B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Karplusan Forest","{T}: Add {1} to your mana pool.
{T}: Add {R} or {G} to your mana pool. Karplusan Forest deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Reactor","Darksteel Reactor is indestructible. (""Destroy"" effects and lethal damage don't destroy it.)
At the beginning of your upkeep, you may put a charge counter on Darksteel Reactor.
When Darksteel Reactor has twenty or more charge counters on it, you win the game.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Unbound","U","At the beginning of your upkeep, put a lore counter on Mind Unbound, then draw a card for each lore counter on Mind Unbound.","4UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daring Apprentice","1","1","U","{T}, Sacrifice Daring Apprentice: Counter target spell.","1UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sanctuary Cat","1","2","W",NULL,"W","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Searchlight Geist","2","1","B","Flying
{3}{B}: Searchlight Geist gains deathtouch until end of turn. (Any amount of damage it deals to a creature is enough to destroy it.)","2B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coastal Hornclaw","3","3","U","Sacrifice a land: Coastal Hornclaw gains flying until end of turn.","4U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phantom Wings","U","Enchant creature
Enchanted creature has flying.
Sacrifice Phantom Wings: Return enchanted creature to its owner's hand.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Looter","1","1","U","{T}: Draw a card, then discard a card.","1U","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyretic Ritual","R","Add {R}{R}{R} to your mana pool.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thawing Glaciers","Thawing Glaciers enters the battlefield tapped.
{1}, {T}: Search your library for a basic land card, put that card onto the battlefield tapped, then shuffle your library. Return Thawing Glaciers to its owner's hand at the beginning of the next cleanup step.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mnemonic Sliver","2","2","U","All Slivers have ""{2}, Sacrifice this permanent: Draw a card.""","2U","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ritual of Steel","W","Enchant creature
When Ritual of Steel enters the battlefield, draw a card at the beginning of the next turn's upkeep.
Enchanted creature gets +0/+2.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breaking Point","R","Any player may have Breaking Point deal 6 damage to him or her. If no one does, destroy all creatures and they can't be regenerated.","1RR","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tinder Farm","Tinder Farm enters the battlefield tapped.
{T}: Add {G} to your mana pool.
{T}, Sacrifice Tinder Farm: Add {R}{W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reaping the Graves","B","Return target creature card from your graveyard to your hand.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sibilant Spirit","5","6","U","Flying
Whenever Sibilant Spirit attacks, defending player may draw a card.","5U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Canopy Surge","G","Kicker {2} (You may pay an additional {2} as you cast this spell.)
Canopy Surge deals 1 damage to each creature with flying and each player. If Canopy Surge was kicked, it deals 4 damage to each creature with flying and each player instead.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Goblin Replica","2","2","{3}{R}, Sacrifice Goblin Replica: Destroy target artifact.","3","Artifact Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spy Network","U","Look at target player's hand, the top card of that player's library, and any face-down creatures he or she controls. Look at the top four cards of your library, then put them back in any order.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strangling Soot","B","Destroy target creature with toughness 3 or less.
Flashback {5}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hidden Guerrillas","G","When an opponent casts an artifact spell, if Hidden Guerrillas is an enchantment, Hidden Guerrillas becomes a 5/3 Soldier creature with trample.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("In the Eye of Chaos","U","Whenever a player casts an instant spell, counter it unless that player pays {X}, where X is its converted mana cost.","2U","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cornered Market","W","Players can't cast spells with the same name as a nontoken permanent.
Players can't play nonbasic lands with the same name as a nontoken permanent.","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Neko-Te","Whenever equipped creature deals damage to a creature, tap that creature. That creature doesn't untap during its controller's untap step for as long as Neko-Te remains on the battlefield.
Whenever equipped creature deals damage to a player, that player loses 1 life.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Doom Cannon","As Doom Cannon enters the battlefield, choose a creature type.
{3}, {T}, Sacrifice a creature of the chosen type: Doom Cannon deals 3 damage to target creature or player.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abuna Acolyte","1","1","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.
{T}: Prevent the next 2 damage that would be dealt to target artifact creature this turn.","1W","Creature - Cat Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lu Xun, Scholar General","1","3","U","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Whenever Lu Xun, Scholar General deals damage to an opponent, you may draw a card.","2UU","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Esper Charm","WUB","Choose one - Destroy target enchantment; or draw two cards; or target player discards two cards.","WUB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Farseek","G","Search your library for a Plains, Island, Swamp, or Mountain card and put it onto the battlefield tapped. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Falling Timber","G","Kicker-Sacrifice a land. (You may sacrifice a land in addition to any other costs as you cast this spell.)
Prevent all combat damage target creature would deal this turn. If Falling Timber was kicked, prevent all combat damage another target creature would deal this turn.","2G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tahngarth","Creatures you control have haste.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ignorant Bliss","R","Exile all cards from your hand face down. At the beginning of the next end step, return those cards to your hand, then draw a card.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wreath of Geists","G","Enchant creature
Enchanted creature gets +X/+X, where X is the number of creature cards in your graveyard.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ramosian Captain","2","2","W","First strike
{5}, {T}: Search your library for a Rebel permanent card with converted mana cost 4 or less and put it onto the battlefield. Then shuffle your library.","1WW","Creature - Human Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Temporal Mastery","U","Take an extra turn after this one. Exile Temporal Mastery.
Miracle {1}{U} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","5UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Poultice Sliver","2","2","W","All Slivers have ""{2}, {T}: Regenerate target Sliver.""","2W","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stand or Fall","R","At the beginning of combat on your turn, separate all creatures defending player controls into two piles. Only creatures in the pile of that player's choice can block this turn.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Children of Korlis","1","1","W","Sacrifice Children of Korlis: You gain life equal to the life you've lost this turn. (Damage causes loss of life.)","W","Creature - Human Rebel Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avacyn, Angel of Hope","8","8","W","Flying, vigilance
Avacyn, Angel of Hope and other permanents you control are indestructible.","5WWW","Legendary Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gleemax","You choose all targets for all spells and abilities.","1000000","Legendary Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Purge","U","Psychic Purge deals 1 damage to target creature or player.
When a spell or ability an opponent controls causes you to discard Psychic Purge, that player loses 5 life.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cache Raiders","4","4","U","At the beginning of your upkeep, return a permanent you control to its owner's hand.","3UU","Creature - Merfolk Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Trailblazer","2","*","W","Flying
Domain - Aven Trailblazer's toughness is equal to the number of basic land types among lands you control.","2W","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Falling Star","R","Flip Falling Star onto the playing area from a height of at least one foot. Falling Star deals 3 damage to each creature it lands on. Tap all creatures dealt damage by Falling Star. If Falling Star doesn't turn completely over at least once during the flip, it has no effect.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Krovikan Plague","B","Enchant non-Wall creature you control
When Krovikan Plague enters the battlefield, draw a card at the beginning of the next turn's upkeep.
Tap enchanted creature: Krovikan Plague deals 1 damage to target creature or player. Put a -0/-1 counter on enchanted creature. Activate this ability only if enchanted creature is untapped.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grotesque Hybrid","3","3","B","Whenever Grotesque Hybrid deals combat damage to a creature, destroy that creature. It can't be regenerated.
Discard a card: Grotesque Hybrid gains flying and protection from green and from white until end of turn.","4B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hoverguard Sweepers","5","6","U","Flying
When Hoverguard Sweepers enters the battlefield, you may return up to two target creatures to their owners' hands.","6UU","Creature - Drone");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Excommunicate","W","Put target creature on top of its owner's library.","2W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shrieking Drake","1","1","U","Flying
When Shrieking Drake enters the battlefield, return a creature you control to its owner's hand.","U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greenweaver Druid","1","1","G","{T}: Add {G}{G} to your mana pool.","2G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Into Thin Air","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Return target artifact to its owner's hand.","5U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rally the Righteous","WR","Radiance - Untap target creature and each other creature that shares a color with it. Those creatures get +2/+0 until end of turn.","1RW","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Snow Fortress","0","4","Defender (This creature can't attack.)
{1}: Snow Fortress gets +1/+0 until end of turn.
{1}: Snow Fortress gets +0/+1 until end of turn.
{3}: Snow Fortress deals 1 damage to target creature without flying that's attacking you.","5","Artifact Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vulshok Battlemaster","2","2","R","Haste
When Vulshok Battlemaster enters the battlefield, attach all Equipment on the battlefield to it. (Control of the Equipment doesn't change.)","4R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crosis, the Purger","6","6","UBR","Flying
Whenever Crosis, the Purger deals combat damage to a player, you may pay {2}{B}. If you do, choose a color, then that player reveals his or her hand and discards all cards of that color.","3UBR","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trespasser il-Vec","3","1","B","Discard a card: Trespasser il-Vec gains shadow until end of turn. (It can block or be blocked by only creatures with shadow.)","2B","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Duelist","1","1","W","As long as Kor Duelist is equipped, it has double strike. (It deals both first-strike and regular combat damage.)","W","Creature - Kor Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disorder","R","Disorder deals 2 damage to each white creature and each player who controls a white creature.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ethersworn Canonist","2","2","W","Each player who has cast a nonartifact spell this turn can't cast additional nonartifact spells.","1W","Artifact Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Break Open","R","Turn target face-down creature an opponent controls face up.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goliath Beetle","3","1","G","Trample","2G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lantern Spirit","2","1","U","Flying
{U}: Return Lantern Spirit to its owner's hand.","2U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Darkpact","B","Remove Darkpact from your deck before playing if you're not playing for ante.
You own target card in the ante. Exchange that card with the top card of your library.","BBB","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Scarecrow","2","2","{6}, {T}: Prevent all damage that would be dealt to you this turn by creatures with flying.","5","Artifact Creature - Scarecrow");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Rattle","B","Delve (You may exile any number of cards from your graveyard as you cast this spell. It costs {1} less to cast for each card exiled this way.)
Destroy target nongreen creature. It can't be regenerated.","5B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feed the Pack","G","At the beginning of your end step, you may sacrifice a nontoken creature. If you do, put X 2/2 green Wolf creature tokens onto the battlefield, where X is the sacrificed creature's toughness.","5G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Freyalise Supplicant","1","1","G","{T}, Sacrifice a red or white creature: Freyalise Supplicant deals damage to target creature or player equal to half the sacrificed creature's power, rounded down.","1G","Creature - Human Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vesuva","You may have Vesuva enter the battlefield tapped as a copy of any land on the battlefield.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hungry Mist","6","2","G","At the beginning of your upkeep, sacrifice Hungry Mist unless you pay {G}{G}.","2GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Harness","U","Enchant red or green creature
Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
You control enchanted creature.","U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Tibalt, the Fiend-Blooded","R","+1: Draw a card, then discard a card at random.
-4: Tibalt, the Fiend-Blooded deals damage equal to the number of cards in target player's hand to that player.
-6: Gain control of all creatures until end of turn. Untap them. They gain haste until end of turn.","Planeswalker - Tibalt","RR");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Axe","R","As an additional cost to cast Lightning Axe, discard a card or pay {5}.
Lightning Axe deals 5 damage to target creature.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Snow-Covered Forest","G",NULL,"Basic Snow Land - Forest");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gurzigost","6","8","G","At the beginning of your upkeep, sacrifice Gurzigost unless you put two cards from your graveyard on the bottom of your library.
{G}{G}, Discard a card: You may have Gurzigost assign its combat damage this turn as though it weren't blocked.","3GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Esper Cormorants","3","3","WU","Flying","2WU","Artifact Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Bomb","R","At the beginning of your upkeep, you may flip a coin. If you win the flip, put a fuse counter on Goblin Bomb. If you lose the flip, remove a fuse counter from Goblin Bomb.
Remove five fuse counters from Goblin Bomb, Sacrifice Goblin Bomb: Goblin Bomb deals 20 damage to target player.","1R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Extraplanar Lens","Imprint - When Extraplanar Lens enters the battlefield, you may exile target land you control.
Whenever a land with the same name as the exiled card is tapped for mana, its controller adds one mana to his or her mana pool of any type that land produced.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breach","B","Target creature gets +2/+0 and gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Putrid Warrior","2","2","WB","Whenever Putrid Warrior deals damage, choose one - Each player loses 1 life; or each player gains 1 life.","WB","Creature - Zombie Soldier Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spawning Breath","R","Spawning Breath deals 1 damage to target creature or player. Put a 0/1 colorless Eldrazi Spawn creature token onto the battlefield. It has ""Sacrifice this creature: Add {1} to your mana pool.""","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cavern of Souls","As Cavern of Souls enters the battlefield, choose a creature type.
{T}: Add {1} to your mana pool.
{T}: Add one mana of any color to your mana pool. Spend this mana only to cast a creature spell of the chosen type, and that spell can't be countered.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Squad","3","3","R","Mogg Squad gets -1/-1 for each other creature on the battlefield.","1R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thermal Flux","U","Choose one - Target nonsnow permanent becomes snow until end of turn; or target snow permanent isn't snow until end of turn.
Draw a card at the beginning of the next turn's upkeep.","U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Moratorium Stone","{2}, {T}: Exile target card from a graveyard.
{2}{W}{B}, {T}, Sacrifice Moratorium Stone: Exile target nonland card from a graveyard, all other cards from graveyards with the same name as that card, and all permanents with that name.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Walker of the Grove","7","7","G","When Walker of the Grove leaves the battlefield, put a 4/4 green Elemental creature token onto the battlefield.
Evoke {4}{G} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","6GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Train of Thought","U","Replicate {1}{U} (When you cast this spell, copy it for each time you paid its replicate cost.)
Draw a card.","1U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Slippery Karst","Slippery Karst enters the battlefield tapped.
{T}: Add {G} to your mana pool.
Cycling {2} ({2}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread Wight","3","4","B","At end of combat, put a paralyzation counter on each creature blocking or blocked by Dread Wight and tap those creatures. Each of those creatures doesn't untap during its controller's untap step for as long as it has a paralyzation counter on it. Each of those creatures gains ""{4}: Remove a paralyzation counter from this creature.""","3BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dismal Failure","U","Counter target spell. Its controller discards a card.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skeletal Changeling","1","1","B","Changeling (This card is every creature type at all times.)
{1}{B}: Regenerate Skeletal Changeling.","1B","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Lancer","2","2","W","Kicker {2}{W} (You may pay an additional {2}{W} as you cast this spell.)
If Benalish Lancer was kicked, it enters the battlefield with two +1/+1 counters on it and with first strike.","2W","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Field of Dreams","U","Players play with the top card of their libraries revealed.","U","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rag Man","2","1","B","{B}{B}{B}, {T}: Target opponent reveals his or her hand and discards a creature card at random. Activate this ability only during your turn.","2BB","Creature - Human Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strafe","R","Strafe deals 3 damage to target nonred creature.","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oracle's Attendants","1","5","W","{T}: All damage that would be dealt to target creature this turn by a source of your choice is dealt to Oracle's Attendants instead.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kessig Malcontents","3","1","R","When Kessig Malcontents enters the battlefield, it deals damage to target player equal to the number of Humans you control.","2R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ball Lightning","6","1","R","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)
Haste (This creature can attack and {T} as soon as it comes under your control.)
At the beginning of the end step, sacrifice Ball Lightning.","RRR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Daybreak Ranger","2","2","G","{T}: Daybreak Ranger deals 2 damage to target creature with flying.
At the beginning of each upkeep, if no spells were cast last turn, transform Daybreak Ranger.","2G","Creature - Human Archer Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Multani's Acolyte","2","1","G","Echo {G}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Multani's Acolyte enters the battlefield, draw a card.","GG","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Selesnya Evangel","1","2","WG","{1}, {T}, Tap an untapped creature you control: Put a 1/1 green Saproling creature token onto the battlefield.","GW","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Turntimber Basilisk","2","1","G","Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)
Landfall - Whenever a land enters the battlefield under your control, you may have target creature block Turntimber Basilisk this turn if able.","1GG","Creature - Basilisk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Holy Strength","W","Enchant creature
Enchanted creature gets +1/+2.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enchantment Alteration","U","Attach target Aura attached to a creature or land to another permanent of that type.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azure Drake","2","4","U","Flying","3U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stench of Evil","B","Destroy all Plains. For each land destroyed this way, Stench of Evil deals 1 damage to that land's controller unless he or she pays {2}.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rare-B-Gone","BR","Each player sacrifices all rare permanents, then reveals his or her hand and discards all rare cards.","2BR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Furious Assault","R","Whenever you cast a creature spell, Furious Assault deals 1 damage to target player.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mold Demon","6","6","B","When Mold Demon enters the battlefield, sacrifice it unless you sacrifice two Swamps.","5BB","Creature - Fungus Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sporogenesis","G","At the beginning of your upkeep, you may put a fungus counter on target nontoken creature.
Whenever a creature with a fungus counter on it dies, put a 1/1 green Saproling creature token onto the battlefield for each fungus counter on that creature.
When Sporogenesis leaves the battlefield, remove all fungus counters from all creatures.","3G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ghitu Encampment","Ghitu Encampment enters the battlefield tapped.
{T}: Add {R} to your mana pool.
{1}{R}: Ghitu Encampment becomes a 2/1 red Warrior creature with first strike until end of turn. It's still a land. (It deals combat damage before creatures without first strike.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fighting Chance","R","For each blocking creature, flip a coin. If you win the flip, prevent all combat damage that would be dealt by that creature this turn.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parch","R","Choose one - Parch deals 2 damage to target creature or player; or Parch deals 4 damage to target blue creature.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harsh Deceiver","1","4","W","{1}: Look at the top card of your library.
{2}: Reveal the top card of your library. If it's a land card, untap Harsh Deceiver and it gets +1/+1 until end of turn. Activate this ability only once each turn.","3W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Farrel's Mantle","W","Enchant creature
Whenever enchanted creature attacks and isn't blocked, its controller may have it deal damage equal to its power plus 2 to another target creature. If that player does, the attacking creature assigns no combat damage this turn.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Early Harvest","G","Target player untaps all basic lands he or she controls.","1GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ballynock Cohort","2","2","W","First strike
Ballynock Cohort gets +1/+1 as long as you control another white creature.","2W","Creature - Kithkin Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inferno Trap","R","If you've been dealt damage by two or more creatures this turn, you may pay {R} rather than pay Inferno Trap's mana cost.
Inferno Trap deals 4 damage to target creature.","3R","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Python","3","2","B",NULL,"1BB","Creature - Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Viscerid Armor","U","Enchant creature
Enchanted creature gets +1/+1.
{1}{U}: Return Viscerid Armor to its owner's hand.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tuknir Deathlock","2","2","RG","Flying
{R}{G}, {T}: Target creature gets +2/+2 until end of turn.","RRGG","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mudslide","R","Creatures without flying don't untap during their controllers' untap steps.
At the beginning of each player's upkeep, that player may choose any number of tapped creatures without flying he or she controls and pay {2} for each creature chosen this way. If the player does, untap those creatures.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unstable Mutation","U","Enchant creature
Enchanted creature gets +3/+3.
At the beginning of the upkeep of enchanted creature's controller, put a -1/-1 counter on that creature.","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sacellum Godspeaker","2","2","G","{T}: Reveal any number of creature cards with power 5 or greater from your hand. Add {G} to your mana pool for each card revealed this way.","2G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Override","U","Counter target spell unless its controller pays {1} for each artifact you control.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Evil Presence","B","Enchant land
Enchanted land is a Swamp.","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Delusions of Mediocrity","U","When Delusions of Mediocrity enters the battlefield, you gain 10 life.
When Delusions of Mediocrity leaves the battlefield, you lose 10 life.","3U","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Necropotence Avatar","Skip your draw step.
At the beginning of your end step, if it's not the first turn of the game, put a death counter on Necropotence Avatar. You draw X cards and you lose X life, where X is the number of death counters on Necropotence Avatar.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Myojin of Cleansing Fire","4","6","W","Myojin of Cleansing Fire enters the battlefield with a divinity counter on it if you cast it from your hand.
Myojin of Cleansing Fire is indestructible as long as it has a divinity counter on it.
Remove a divinity counter from Myojin of Cleansing Fire: Destroy all other creatures.","5WWW","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Makindi Griffin","2","4","W","Flying","3W","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glaze Fiend","0","1","B","Flying
Whenever another artifact enters the battlefield under your control, Glaze Fiend gets +2/+2 until end of turn.","1B","Artifact Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ironclaw Orcs","2","2","R","Ironclaw Orcs can't block creatures with power 2 or greater.","1R","Creature - Orc");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Treetop Village","Treetop Village enters the battlefield tapped.
{T}: Add {G} to your mana pool.
{1}{G}: Treetop Village becomes a 3/3 green Ape creature with trample until end of turn. It's still a land. (If it would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unburden","B","Target player discards two cards.
Cycling {2} ({2}, Discard this card: Draw a card.)","1BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ordered Migration","WU","Domain - Put a 1/1 blue Bird creature token with flying onto the battlefield for each basic land type among lands you control.","3WU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bogardan Hellkite","5","5","R","Flash (You may cast this spell any time you could cast an instant.)
Flying
When Bogardan Hellkite enters the battlefield, it deals 5 damage divided as you choose among any number of target creatures and/or players.","6RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Tezzeret the Seeker","U","+1: Untap up to two target artifacts.
-X: Search your library for an artifact card with converted mana cost X or less and put it onto the battlefield. Then shuffle your library.
-5: Artifacts you control become 5/5 artifact creatures until end of turn.","Planeswalker - Tezzeret","3UU");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carnassid","5","4","G","Trample
{1}{G}: Regenerate Carnassid.","4GG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kira, Great Glass-Spinner","2","2","U","Flying
Creatures you control have ""Whenever this creature becomes the target of a spell or ability for the first time in a turn, counter that spell or ability.""","1UU","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karmic Guide","2","2","W","Flying, protection from black
Echo {3}{W}{W} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Karmic Guide enters the battlefield, return target creature card from your graveyard to the battlefield.","3WW","Creature - Angel Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skullcage","At the beginning of each opponent's upkeep, Skullcage deals 2 damage to that player unless he or she has exactly three or exactly four cards in hand.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gloom","B","White spells cost {3} more to cast.
Activated abilities of white enchantments cost {3} more to activate.","2B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nocturnal Raid","B","Black creatures get +2/+0 until end of turn.","2BB","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mistveil Plains","({T}: Add {W} to your mana pool.)
Mistveil Plains enters the battlefield tapped.
{W}, {T}: Put target card from your graveyard on the bottom of your library. Activate this ability only if you control two or more white permanents.",NULL,"Land - Plains");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ember Beast","3","4","R","Ember Beast can't attack or block alone.","2R","Creature - Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Soldevi Sentry","1","1","{1}: Choose target opponent. Regenerate Soldevi Sentry. When it regenerates this way, that player may draw a card.","1","Artifact Creature - Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Secrets of the Dead","U","Whenever you cast a spell from your graveyard, draw a card.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbarian Guides","1","2","R","{2}{R}, {T}: Choose a land type. Target creature you control gains snow landwalk of the chosen type until end of turn. Return that creature to its owner's hand at the beginning of the next end step.","2R","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepwood Legate","1","1","B","If an opponent controls a Forest and you control a Swamp, you may cast Deepwood Legate without paying its mana cost.
{B}: Deepwood Legate gets +1/+1 until end of turn.","3B","Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ifh-Bíff Efreet","3","3","G","Flying
{G}: Ifh-Bíff Efreet deals 1 damage to each creature with flying and each player. Any player may activate this ability.","2GG","Creature - Efreet");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Copper Myr","1","1","{T}: Add {G} to your mana pool.","2","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tangle Mantis","3","4","G","Trample","2GG","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stampeding Wildebeests","5","4","G","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)
At the beginning of your upkeep, return a green creature you control to its owner's hand.","2GG","Creature - Antelope Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Volcanic Island",NULL,NULL,"Land - Island Mountain");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Marauder","0","0","Phyrexian Marauder enters the battlefield with X +1/+1 counters on it.
Phyrexian Marauder can't block.
Phyrexian Marauder can't attack unless you pay {1} for each +1/+1 counter on it.","X","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Burst","U","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Counter target spell with converted mana cost X.","XU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seashell Cameo","{T}: Add {W} or {U} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fallowsage","2","2","U","Whenever Fallowsage becomes tapped, you may draw a card.","3U","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of Kookus","1","1","R","{R}: Keeper of Kookus gains protection from red until end of turn.","R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wellwisher","1","1","G","{T}: You gain 1 life for each Elf on the battlefield.","1G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Archive Trap","U","If an opponent searched his or her library this turn, you may pay {0} rather than pay Archive Trap's mana cost.
Target opponent puts the top thirteen cards of his or her library into his or her graveyard.","3UU","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Isperia the Inscrutable","3","6","WU","Flying
Whenever Isperia the Inscrutable deals combat damage to a player, name a card. That player reveals his or her hand. If he or she reveals the named card, search your library for a creature card with flying, reveal it, put it into your hand, then shuffle your library.","1WWUU","Legendary Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ridgeline Rager","1","2","R","{R}: Ridgeline Rager gets +1/+0 until end of turn.","2R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reversal of Fortune","R","Target opponent reveals his or her hand. You may copy an instant or sorcery card in it. If you do, you may cast the copy without paying its mana cost.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pit Keeper","2","1","B","When Pit Keeper enters the battlefield, if you have four or more creature cards in your graveyard, you may return target creature card from your graveyard to your hand.","1B","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Torsten Von Ursus","5","5","WG",NULL,"3GGW","Legendary Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ensnare","U","You may return two Islands you control to their owner's hand rather than pay Ensnare's mana cost.
Tap all creatures.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Flock","2","3","W","Flying (This creature can't be blocked except by creatures with flying or reach.)
{W}: Aven Flock gets +0/+1 until end of turn.","4W","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frenzied Goblin","1","1","R","Whenever Frenzied Goblin attacks, you may pay {R}. If you do, target creature can't block this turn.","R","Creature - Goblin Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Contempt","U","Enchant creature
Whenever enchanted creature attacks, return it and Contempt to their owners' hands at end of combat.","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Armed Response","W","Armed Response deals damage to target attacking creature equal to the number of Equipment you control.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goliath Spider","7","6","G","Reach (This creature can block creatures with flying.)","6GG","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whipkeeper","1","1","R","{T}: Whipkeeper deals damage to target creature equal to the damage already dealt to it this turn.","2RR","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mass Hysteria","R","All creatures have haste.","R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tortoise Formation","U","Creatures you control gain shroud until end of turn. (They can't be the targets of spells or abilities.)","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stingmoggie","0","0","R","Stingmoggie enters the battlefield with two +1/+1 counters on it.
{3}{R}, Remove a +1/+1 counter from Stingmoggie: Destroy target artifact or land.","3R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ritual of Subdual","G","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
If a land is tapped for mana, it produces colorless mana instead of any other type.","4GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pardic Dragon","4","4","R","Flying
{R}: Pardic Dragon gets +1/+0 until end of turn.
Suspend 2-{R}{R} (Rather than cast this card from your hand, you may pay {R}{R} and exile it with two time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Whenever an opponent casts a spell, if Pardic Dragon is suspended, that player may put a time counter on Pardic Dragon.","4RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Artificer's Intuition","U","{U}, Discard an artifact card: Search your library for an artifact card with converted mana cost 1 or less, reveal that card, and put it into your hand. Then shuffle your library.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shatterstorm","R","Destroy all artifacts. They can't be regenerated.","2RR","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Basalt Golem","2","4","Basalt Golem can't be blocked by artifact creatures.
Whenever Basalt Golem becomes blocked by a creature, that creature's controller sacrifices it at end of combat. If the player does, he or she puts a 0/2 colorless Wall artifact creature token with defender onto the battlefield.","5","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Squee's Toy","{T}: Prevent the next 1 damage that would be dealt to target creature this turn.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krovikan Sorcerer","1","1","U","{T}, Discard a nonblack card: Draw a card.
{T}, Discard a black card: Draw two cards, then discard one of them.","2U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sundering Vitae","G","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Destroy target artifact or enchantment.","2G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pendelhaven","{T}: Add {G} to your mana pool.
{T}: Target 1/1 creature gets +1/+2 until end of turn.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sisay's Ingenuity","U","Enchant creature
When Sisay's Ingenuity enters the battlefield, draw a card.
Enchanted creature has ""{2}{U}: Target creature becomes the color of your choice until end of turn.""","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stronghold Biologist","1","1","U","{U}{U}, {T}, Discard a card: Counter target creature spell.","2U","Creature - Human Spellshaper");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Inquisitor's Flail","If equipped creature would deal combat damage, it deals double that damage instead.
If another creature would deal combat damage to equipped creature, it deals double that damage to equipped creature instead.
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Weatherseed Elf","1","1","G","{T}: Target creature gains forestwalk until end of turn.","G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Epic Struggle","G","At the beginning of your upkeep, if you control twenty or more creatures, you win the game.","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Simic Ragworm","3","3","G","{U}: Untap Simic Ragworm.","3G","Creature - Worm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaze of Justice","W","As an additional cost to cast Gaze of Justice, tap three untapped white creatures you control.
Exile target creature.
Flashback {5}{W} (You may cast this card from your graveyard for its flashback cost and any additional costs. Then exile it.)","W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sigil Tracer","2","2","U","{1}{U}, Tap two untapped Wizards you control: Copy target instant or sorcery spell. You may choose new targets for the copy.","1UU","Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hatchet Bully","3","3","R","{2}{R}, {T}, Put a -1/-1 counter on a creature you control: Hatchet Bully deals 2 damage to target creature or player.","3R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Funeral Charm","B","Choose one - Target player discards a card; or target creature gets +2/-1 until end of turn; or target creature gains swampwalk until end of turn.","B","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hovermyr","1","2","Flying, vigilance","2","Artifact Creature - Myr");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Moonring Mirror","Whenever you draw a card, exile the top card of your library face down.
At the beginning of your upkeep, you may exile all cards from your hand face down. If you do, put all other cards you own exiled with Moonring Mirror into your hand.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tine Shrike","2","1","W","Flying
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","3W","Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Whetstone","{3}: Each player puts the top two cards of his or her library into his or her graveyard.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Righteous Fury","W","Destroy all tapped creatures. You gain 2 life for each creature destroyed this way.","4WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voidmage Apprentice","1","1","U","Morph {2}{U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Voidmage Apprentice is turned face up, counter target spell.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Qasali Ambusher","2","3","WG","Reach
If a creature is attacking you and you control a Forest and a Plains, you may cast Qasali Ambusher without paying its mana cost and as though it had flash.","1GW","Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Resistance Fighter","1","1","W","Sacrifice Resistance Fighter: Prevent all combat damage target creature would deal this turn.","W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vampire Nocturnus Avatar","Play with the top card of your library revealed.
As long as the top card of your library is black, black creatures you control get +2/+1.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dromosaur","2","3","R","Whenever Dromosaur blocks or becomes blocked, it gets +2/-2 until end of turn.","2R","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unliving Psychopath","0","4","B","{B}: Unliving Psychopath gets +1/-1 until end of turn.
{B}, {T}: Destroy target creature with power less than Unliving Psychopath's power.","2BB","Creature - Zombie Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Werebear","1","1","G","{T}: Add {G} to your mana pool.
Threshold - Werebear gets +3/+3 as long as seven or more cards are in your graveyard.","1G","Creature - Human Bear Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Melira's Keepers","4","4","G","Melira's Keepers can't have counters placed on it.","4G","Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grixis Charm","UBR","Choose one - Return target permanent to its owner's hand; or target creature gets -4/-4 until end of turn; or creatures you control get +2/+0 until end of turn.","UBR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Synod Artificer","1","2","U","{X}, {T}: Tap X target noncreature artifacts.
{X}, {T}: Untap X target noncreature artifacts.","2U","Creature - Vedalken Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Earthen Goo","2","2","R","Trample
Cumulative upkeep {R} or {G} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Earthen Goo gets +1/+1 for each age counter on it.","2R","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Geyser","R","Volcanic Geyser deals X damage to target creature or player.","XRR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unnatural Predation","G","Target creature gets +1/+1 and gains trample until end of turn.","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Channel","G","Until end of turn, any time you could activate a mana ability, you may pay 1 life. If you do, add {1} to your mana pool.","GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gerrard's Wisdom","W","You gain 2 life for each card in your hand.","2WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Infuser","1","4","U","At the beginning of your upkeep, you may put a charge counter on target artifact.","3U","Creature - Vedalken Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Shockwave","W","Choose one - Tap all Spirits; or tap all non-Spirit creatures.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nacatl Hunt-Pride","5","4","W","Vigilance
{R}, {T}: Target creature can't block this turn.
{G}, {T}: Target creature blocks this turn if able.","5W","Creature - Cat Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fledgling Imp","2","2","B","{B}, Discard a card: Fledgling Imp gains flying until end of turn.","2B","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volrath's Dungeon","B","Pay 5 life: Destroy Volrath's Dungeon. Any player may activate this ability but only during his or her turn.
Discard a card: Target player puts a card from his or her hand on top of his or her library. Activate this ability only any time you could cast a sorcery.","2BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Wardriver","2","2","R","Battle cry (Whenever this creature attacks, each other attacking creature gets +1/+0 until end of turn.)","RR","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fighting Drake","2","4","U","Flying","2UU","Creature - Drake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lava Tubes","Lava Tubes doesn't untap during your untap step if it has a depletion counter on it.
At the beginning of your upkeep, remove a depletion counter from Lava Tubes.
{T}: Add {B} or {R} to your mana pool. Put a depletion counter on Lava Tubes.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treefolk Seedlings","2","*","G","Treefolk Seedlings's toughness is equal to the number of Forests you control.","2G","Creature - Treefolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mine Layer","1","1","R","{1}{R}, {T}: Put a mine counter on target land.
Whenever a land with a mine counter on it becomes tapped, destroy it.
When Mine Layer leaves the battlefield, remove all mine counters from all lands.","3R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crackdown","W","Nonwhite creatures with power 3 or greater don't untap during their controllers' untap steps.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chill Haunting","B","As an additional cost to cast Chill Haunting, exile X creature cards from your graveyard.
Target creature gets -X/-X until end of turn.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Mime","2","2","R","When you speak, sacrifice Goblin Mime.","1R","Creature - Goblin Mime");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coral Reef","U","Coral Reef enters the battlefield with four polyp counters on it.
Sacrifice an Island: Put two polyp counters on Coral Reef.
{U}, Tap an untapped blue creature you control, Remove a polyp counter from Coral Reef: Put a +0/+1 counter on target creature.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magmasaur","0","0","R","Magmasaur enters the battlefield with five +1/+1 counters on it.
At the beginning of your upkeep, you may remove a +1/+1 counter from Magmasaur. If you don't, sacrifice Magmasaur and it deals damage equal to the number of +1/+1 counters on it to each creature without flying and each player.","3RR","Creature - Elemental Lizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Goblin token card",NULL,NULL,NULL);
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Savage Thallid","5","2","G","At the beginning of your upkeep, put a spore counter on Savage Thallid.
Remove three spore counters from Savage Thallid: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: Regenerate target Fungus.","3GG","Creature - Fungus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Instigator","1","1","B","{1}{B}{B}, {T}, Discard a card: Creatures target player controls attack this turn if able.","1B","Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("The Abyss","B","At the beginning of each player's upkeep, destroy target nonartifact creature that player controls of his or her choice. It can't be regenerated.","3B","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEther Burst","U","Return up to X target creatures to their owners' hands, where X is one plus the number of cards named Æther Burst in all graveyards as you cast Æther Burst.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Molten Disaster","R","Kicker {R} (You may pay an additional {R} as you cast this spell.)
If Molten Disaster was kicked, it has split second. (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Molten Disaster deals X damage to each creature without flying and each player.","XRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oxidize","G","Destroy target artifact. It can't be regenerated.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whiptail Wurm","8","5","G",NULL,"6G","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Firebolt","R","Firebolt deals 2 damage to target creature or player.
Flashback {4}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Capricious Sorcerer","1","1","U","{T}: Capricious Sorcerer deals 1 damage to target creature or player. Activate this ability only during your turn, before attackers are declared.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lyzolda, the Blood Witch","3","1","BR","{2}, Sacrifice a creature: Lyzolda, the Blood Witch deals 2 damage to target creature or player if the sacrificed creature was red. Draw a card if the sacrificed creature was black.","1BR","Legendary Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feeling of Dread","W","Tap up to two target creatures.
Flashback {1}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cursed Monstrosity","4","3","B","Flying
Whenever Cursed Monstrosity becomes the target of a spell or ability, sacrifice it unless you discard a land card.","4B","Creature - Horror");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elixir of Vitality","Elixir of Vitality enters the battlefield tapped.
{T}, Sacrifice Elixir of Vitality: You gain 4 life.
{8}, {T}, Sacrifice Elixir of Vitality: You gain 8 life.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mass Polymorph","U","Exile all creatures you control, then reveal cards from the top of your library until you reveal that many creature cards. Put all creature cards revealed this way onto the battlefield, then shuffle the rest of the revealed cards into your library.","5U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blade Splicer","1","1","W","When Blade Splicer enters the battlefield, put a 3/3 colorless Golem artifact creature token onto the battlefield.
Golem creatures you control have first strike.","2W","Creature - Human Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Spring","U","Draw X cards.","XUU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crimson Wisps","R","Target creature becomes red and gains haste until end of turn.
Draw a card.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mausoleum Guard","2","2","W","When Mausoleum Guard dies, put two 1/1 white Spirit creature tokens with flying onto the battlefield.","3W","Creature - Human Scout");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Servitor","1","1","At the beginning of your upkeep, if Myr Servitor is on the battlefield, each player returns all cards named Myr Servitor from his or her graveyard to the battlefield.","1","Artifact Creature - Myr");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thran Dynamo","{T}: Add {3} to your mana pool.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantom Warrior","2","2","U","Phantom Warrior is unblockable.","1UU","Creature - Illusion Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Windreaper Falcon","1","1","RG","Flying, protection from blue","1RG","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unblinking Bleb","1","3","U","Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
Whenever Unblinking Bleb or another permanent is turned face up, you may scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","3U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Transfixer","1","1","W","{W}, {T}: Tap target artifact.","W","Creature - Human Scout");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Coldsteel Heart","Coldsteel Heart enters the battlefield tapped.
As Coldsteel Heart enters the battlefield, choose a color.
{T}: Add one mana of the chosen color to your mana pool.","2","Snow Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hand to Hand","R","During combat, players can't cast instant spells or activate abilities that aren't mana abilities.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Extirpate","B","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Choose target card in a graveyard other than a basic land card. Search its owner's graveyard, hand, and library for all cards with the same name as that card and exile them. Then that player shuffles his or her library.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harvest Mage","1","1","G","{G}, {T}, Discard a card: Until end of turn, if you tap a land for mana, it produces one mana of a color of your choice instead of any other type and amount.","G","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ancient Ooze","*","*","G","Ancient Ooze's power and toughness are each equal to the total converted mana cost of other creatures you control.","5GG","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Headstone","B","Exile target card from a graveyard.
Draw a card at the beginning of the next turn's upkeep.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Glory's Rise","4","6","W","Flying
When Angel of Glory's Rise enters the battlefield, exile all Zombies, then return all Human creature cards from your graveyard to the battlefield.","5WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Darksteel Colossus","11","11","Trample
Darksteel Colossus is indestructible.
If Darksteel Colossus would be put into a graveyard from anywhere, reveal Darksteel Colossus and shuffle it into its owner's library instead.","11","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunted Horror","7","7","B","Trample
When Hunted Horror enters the battlefield, put two 3/3 green Centaur creature tokens with protection from black onto the battlefield under target opponent's control.","BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodrock Cyclops","3","3","R","Bloodrock Cyclops attacks each turn if able.","2R","Creature - Cyclops");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rootwater Hunter","1","1","U","{T}: Rootwater Hunter deals 1 damage to target creature or player.","2U","Creature - Merfolk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Surestrike Trident","Equipped creature has first strike and ""{T}, Unattach Surestrike Trident: This creature deals damage equal to its power to target player.""
Equip {4} ({4}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blighted Agent","1","1","U","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Blighted Agent is unblockable.","1U","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ponder","U","Look at the top three cards of your library, then put them back in any order. You may shuffle your library.
Draw a card.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call to the Kindred","U","Enchant creature
At the beginning of your upkeep, you may look at the top five cards of your library. If you do, you may put a creature card that shares a creature type with enchanted creature from among them onto the battlefield, then you put the rest of those cards on the bottom of your library in any order.","3U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Illusionary Mask","{X}: You may choose a creature card in your hand whose mana cost could be paid by some amount of, or all of, the mana you spent on {X}. If you do, you may cast that card face down as a 2/2 creature spell without paying its mana cost. If the creature that spell becomes as it resolves has not been turned face up and would assign or deal damage, be dealt damage, or become tapped, instead it's turned face up and assigns or deals damage, is dealt damage, or becomes tapped. Activate this ability only any time you could cast a sorcery.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hanabi Blast","R","Hanabi Blast deals 2 damage to target creature or player. Return Hanabi Blast to its owner's hand, then discard a card at random.","1RR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Temple Bell","{T}: Each player draws a card.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Summon the School","W","Put two 1/1 blue Merfolk Wizard creature tokens onto the battlefield.
Tap four untapped Merfolk you control: Return Summon the School from your graveyard to your hand.","3W","Tribal Sorcery - Merfolk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Imi Statue","Players can't untap more than one artifact during their untap steps.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Claim","G","Destroy target artifact or enchantment. Its controller gains 4 life.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ceaseless Searblades","2","4","R","Whenever you activate an ability of an Elemental, Ceaseless Searblades gets +1/+0 until end of turn.","3R","Creature - Elemental Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Congregate","W","Target player gains 2 life for each creature on the battlefield.","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consign to Dream","U","Return target permanent to its owner's hand. If that permanent is red or green, put it on top of its owner's library instead.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thrull Surgeon","1","1","B","{1}{B}, Sacrifice Thrull Surgeon: Look at target player's hand and choose a card from it. That player discards that card. Activate this ability only any time you could cast a sorcery.","1B","Creature - Thrull");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raiding Party","R","Raiding Party can't be the target of white spells or abilities from white sources.
Sacrifice an Orc: Each player may tap any number of untapped white creatures he or she controls. For each creature tapped this way, that player chooses up to two Plains. Then destroy all Plains that weren't chosen this way by any player.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Ravel","B","Target player discards a card.
Draw a card at the beginning of the next turn's upkeep.","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sands of Time","Each player skips his or her untap step.
At the beginning of each player's upkeep, that player simultaneously untaps each tapped artifact, creature, and land he or she controls and taps each untapped artifact, creature, and land he or she controls.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidal Courier","1","2","U","When Tidal Courier enters the battlefield, reveal the top four cards of your library. Put all Merfolk cards revealed this way into your hand and the rest on the bottom of your library in any order.
{3}{U}: Tidal Courier gains flying until end of turn.","3U","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moldgraf Monstrosity","8","8","G","Trample
When Moldgraf Monstrosity dies, exile it, then return two creature cards at random from your graveyard to the battlefield.","4GGG","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirror Entity","1","1","W","Changeling (This card is every creature type at all times.)
{X}: Creatures you control become X/X and gain all creature types until end of turn.","2W","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vigilance","W","Enchant creature
Enchanted creature has vigilance. (Attacking doesn't cause it to tap.)","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ravenous Baboons","2","2","R","When Ravenous Baboons enters the battlefield, destroy target nonbasic land.","3R","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pale Wayfarer","4","4","W","{2}{W}{W}, {Q}: Target creature gains protection from the color of its controller's choice until end of turn. ({Q} is the untap symbol.)","5WW","Creature - Spirit Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Monstrify","G","Target creature gets +4/+4 until end of turn.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodstoke Howler","3","4","R","Morph {6}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Bloodstoke Howler is turned face up, Beast creatures you control get +3/+0 until end of turn.","5R","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tattermunge Witch","2","1","RG","{R}{G}: Each blocked creature gets +1/+0 and gains trample until end of turn.","1(R/G)","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rukh Egg","0","3","R","When Rukh Egg dies, put a 4/4 red Bird creature token with flying onto the battlefield at the beginning of the next end step.","3R","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Order of the Sacred Torch","2","2","W","{T}, Pay 1 life: Counter target black spell.","1WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Protection: Blue","W","{1}: The next time a blue source of your choice would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mishra's Factory","{T}: Add {1} to your mana pool.
{1}: Mishra's Factory becomes a 2/2 Assembly-Worker artifact creature until end of turn. It's still a land.
{T}: Target Assembly-Worker creature gets +1/+1 until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chromium","7","7","WUB","Flying
Rampage 2 (Whenever this creature becomes blocked, it gets +2/+2 until end of turn for each creature blocking it beyond the first.)
At the beginning of your upkeep, sacrifice Chromium unless you pay {W}{U}{B}.","2WWUUBB","Legendary Creature - Elder Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lithophage","7","7","R","At the beginning of your upkeep, sacrifice Lithophage unless you sacrifice a Mountain.","3RR","Creature - Insect");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Sentinel","1","1","{0}: Sentinel's toughness becomes 1 plus the power of target creature blocking or blocked by Sentinel. (This effect lasts indefinitely.)","4","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ballista Squad","2","2","W","{X}{W}, {T}: Ballista Squad deals X damage to target attacking or blocking creature.","3W","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burning Palm Efreet","2","2","R","{1}{R}{R}: Burning Palm Efreet deals 2 damage to target creature with flying and that creature loses flying until end of turn.","2RR","Creature - Efreet");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Black Mana Battery","{2}, {T}: Put a charge counter on Black Mana Battery.
{T}, Remove any number of charge counters from Black Mana Battery: Add {B} to your mana pool, then add an additional {B} to your mana pool for each charge counter removed this way.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bone Dancer","2","2","B","Whenever Bone Dancer attacks and isn't blocked, you may put the top creature card of defending player's graveyard onto the battlefield under your control. If you do, Bone Dancer assigns no combat damage this turn.","1BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Last Gasp","B","Target creature gets -3/-3 until end of turn.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Messenger Falcons","2","2","WUG","Flying
When Messenger Falcons enters the battlefield, draw a card.","2(G/U)W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hollowhenge Beast","5","5","G",NULL,"3GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Animate Land","G","Until end of turn, target land becomes a 3/3 creature that's still a land.","G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Obelisk of Naya","{T}: Add {R}, {G}, or {W} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scrivener","2","2","U","When Scrivener enters the battlefield, you may return target instant card from your graveyard to your hand.","4U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dreamscape Artist","1","1","U","{2}{U}, {T}, Discard a card, Sacrifice a land: Search your library for up to two basic land cards and put them onto the battlefield. Then shuffle your library.","1U","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squirrel Mob","2","2","G","Squirrel Mob gets +1/+1 for each other Squirrel on the battlefield.","1GG","Creature - Squirrel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Iceberg","U","Iceberg enters the battlefield with X ice counters on it.
{3}: Put an ice counter on Iceberg.
Remove an ice counter from Iceberg: Add {1} to your mana pool.","XUU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Haunting Misery","B","As an additional cost to cast Haunting Misery, exile X creature cards from your graveyard.
Haunting Misery deals X damage to target player.","1BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of War and Peace","Equipped creature gets +2/+2 and has protection from red and from white.
Whenever equipped creature deals combat damage to a player, Sword of War and Peace deals damage to that player equal to the number of cards in his or her hand and you gain 1 life for each card in your hand.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gloomdrifter","2","2","B","Flying
Threshold - As long as seven or more cards are in your graveyard, Gloomdrifter has ""When Gloomdrifter enters the battlefield, nonblack creatures get -2/-2 until end of turn.""","3B","Creature - Zombie Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spell Blast","U","Counter target spell with converted mana cost X.","XU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Acorn Catapult","{1}, {T}: Acorn Catapult deals 1 damage to target creature or player. That creature's controller or that player puts a 1/1 green Squirrel creature token onto the battlefield.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Denied","B","Return X target creature cards from your graveyard to your hand.","XBB","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Avatar of Woe","6","5","B","If there are ten or more creature cards total in all graveyards, Avatar of Woe costs {6} less to cast.
Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
{T}: Destroy target creature. It can't be regenerated.","6BB","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dispersing Orb","U","{3}{U}, Sacrifice a permanent: Return target permanent to its owner's hand.","3UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horned Kavu","3","4","RG","When Horned Kavu enters the battlefield, return a red or green creature you control to its owner's hand.","RG","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scrying Glass","{3}, {T}: Choose a number greater than 0 and a color. Target opponent reveals his or her hand. If that opponent reveals exactly the chosen number of cards of the chosen color, you draw a card.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Twinblade Slasher","1","1","G","Wither (This deals damage to creatures in the form of -1/-1 counters.)
{1}{G}: Twinblade Slasher gets +2/+2 until end of turn. Activate this ability only once each turn.","G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Sand Golem","3","3","When a spell or ability an opponent controls causes you to discard Sand Golem, return Sand Golem from your graveyard to the battlefield with a +1/+1 counter on it at the beginning of the next end step.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infectious Horror","2","2","B","Whenever Infectious Horror attacks, each opponent loses 2 life.","3B","Creature - Zombie Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Consuming Vortex","U","Return target creature to its owner's hand.
Splice onto Arcane {3}{U} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1U","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tombfire","B","Target player exiles all cards with flashback from his or her graveyard.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pack's Disdain","B","Choose a creature type. Target creature gets -1/-1 until end of turn for each permanent of the chosen type you control.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guard Gomazoa","1","3","U","Defender, flying
Prevent all combat damage that would be dealt to Guard Gomazoa.","2U","Creature - Jellyfish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maraxus of Keld","*","*","R","Maraxus of Keld's power and toughness are each equal to the number of untapped artifacts, creatures, and lands you control.","4RR","Legendary Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dampen Thought","U","Target player puts the top four cards of his or her library into his or her graveyard.
Splice onto Arcane {1}{U} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1U","Instant - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Aladdin's Ring","{8}, {T}: Aladdin's Ring deals 4 damage to target creature or player.","8","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sisay","Whenever you tap a land for mana, add one mana to your mana pool of any type that land produced.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steam Spitter","1","5","G","Reach (This creature can block creatures with flying.)
{R}: Steam Spitter gets +1/+0 until end of turn.","4G","Creature - Spider");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Goblin","1","1",NULL,NULL,"Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitting Drake","2","2","R","Flying
{R}: Spitting Drake gets +1/+0 until end of turn. Activate this ability only once each turn.","3R","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Honden of Cleansing Fire","W","At the beginning of your upkeep, you gain 2 life for each Shrine you control.","3W","Legendary Enchantment - Shrine");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incinerate","R","Incinerate deals 3 damage to target creature or player. A creature dealt damage this way can't be regenerated this turn.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silverstorm Samurai","3","3","W","Flash
Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)","4WW","Creature - Fox Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primal Forcemage","2","2","G","Whenever another creature enters the battlefield under your control, that creature gets +3/+3 until end of turn.","2G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Momentary Blink","W","Exile target creature you control, then return it to the battlefield under its owner's control.
Flashback {3}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hornet Cannon","{3}, {T}: Put a 1/1 colorless Insect artifact creature token with flying and haste named Hornet onto the battlefield. Destroy it at the beginning of the next end step.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kiyomaro, First to Stand","*","*","W","Kiyomaro, First to Stand's power and toughness are each equal to the number of cards in your hand.
As long as you have four or more cards in hand, Kiyomaro has vigilance.
Whenever Kiyomaro deals damage, if you have seven or more cards in hand, you gain 7 life.","3WW","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Convincing Mirage","U","Enchant land
As Convincing Mirage enters the battlefield, choose a basic land type.
Enchanted land is the chosen type.","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Matsu-Tribe Birdstalker","2","2","G","Whenever Matsu-Tribe Birdstalker deals combat damage to a creature, tap that creature and it doesn't untap during its controller's next untap step.
{G}: Matsu-Tribe Birdstalker gains reach until end of turn. (It can block creatures with flying.)","2GG","Creature - Snake Warrior Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Radiant Essence","2","3","WG","Radiant Essence gets +1/+2 as long as an opponent controls a black permanent.","1GW","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ivory Guardians","3","3","W","Protection from red
Creatures named Ivory Guardians get +1/+1 as long as an opponent controls a nontoken red permanent.","4WW","Creature - Giant Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Walking Dead","1","1","B","{B}: Regenerate Walking Dead.","1B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wyluli Wolf","1","1","G","{T}: Target creature gets +1/+1 until end of turn.","1G","Creature - Wolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Dynamo","4","4","R","{T}: Goblin Dynamo deals 1 damage to target creature or player.
{X}{R}, {T}, Sacrifice Goblin Dynamo: Goblin Dynamo deals X damage to target creature or player.","5RR","Creature - Goblin Mutant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Soratami Cloud Chariot","{2}: Target creature you control gains flying until end of turn.
{2}: Prevent all combat damage that would be dealt to and dealt by target creature you control this turn.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pit Scorpion","1","1","B","Whenever Pit Scorpion deals damage to a player, that player gets a poison counter. (A player with ten or more poison counters loses the game.)","2B","Creature - Scorpion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Flame","R","Whenever a creature without flying attacks you or a planeswalker you control, Circle of Flame deals 1 damage to that creature.","1R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Envoy","0","2","U","Flying","U","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Serpent of the Endless Sea","*","*","U","Serpent of the Endless Sea's power and toughness are each equal to the number of Islands you control.
Serpent of the Endless Sea can't attack unless defending player controls an Island.","4U","Creature - Serpent");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Severance","U","Search your library for any number of land cards and exile them. Then shuffle your library.","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shyft","4","2","U","At the beginning of your upkeep, you may have Shyft become the color or colors of your choice. (This effect lasts indefinitely.)","4U","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Mourning","G","Put target card from your graveyard on top of your library.","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rockcaster Platoon","5","7","W","{4}{G}: Rockcaster Platoon deals 2 damage to each creature with flying and each player.","5WW","Creature - Rhino Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cunning Lethemancer","2","2","B","At the beginning of your upkeep, each player discards a card.","2B","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primordial Hydra","0","0","G","Primordial Hydra enters the battlefield with X +1/+1 counters on it.
At the beginning of your upkeep, double the number of +1/+1 counters on Primordial Hydra.
Primordial Hydra has trample as long as it has ten or more +1/+1 counters on it.","XGG","Creature - Hydra");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Convulsing Licid","2","2","R","{R}, {T}: Convulsing Licid loses this ability and becomes an Aura enchantment with enchant creature. Attach it to target creature. You may pay {R} to end this effect.
Enchanted creature can't block.","2R","Creature - Licid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brimstone Dragon","6","6","R","Flying, haste","6RR","Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Grinning Totem","{2}, {T}, Sacrifice Grinning Totem: Search target opponent's library for a card and exile it. Then that player shuffles his or her library. Until the beginning of your next upkeep, you may play that card. At the beginning of your next upkeep, if you haven't played it, put it into its owner's graveyard.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tolaria West","Tolaria West enters the battlefield tapped.
{T}: Add {U} to your mana pool.
Transmute {1}{U}{U} ({1}{U}{U}, Discard this card: Search your library for a card with converted mana cost 0, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tooth of Ramos","{T}: Add {W} to your mana pool.
Sacrifice Tooth of Ramos: Add {W} to your mana pool.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Red Mana Battery","{2}, {T}: Put a charge counter on Red Mana Battery.
{T}, Remove any number of charge counters from Red Mana Battery: Add {R} to your mana pool, then add an additional {R} to your mana pool for each charge counter removed this way.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skittering Monstrosity","5","5","B","When you cast a creature spell, sacrifice Skittering Monstrosity.","3BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disciple of Kangee","2","2","W","{U}, {T}: Target creature gains flying and becomes blue until end of turn.","2W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Coils of the Medusa","B","Enchant creature
Enchanted creature gets +1/-1.
Sacrifice Coils of the Medusa: Destroy all non-Wall creatures blocking enchanted creature.","1B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Druids' Repository","G","Whenever a creature you control attacks, put a charge counter on Druids' Repository.
Remove a charge counter from Druids' Repository: Add one mana of any color to your mana pool.","1GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning Cinder Fury of Crimson Chaos Fire","R","Whenever any player taps a card, that player gives control of that card to an opponent at end of turn.
If a player does not tap any nonland cards during his or her turn, Burning Cinder Fury of Crimson Chaos Fire deals 3 damage to that player at end of turn.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thelon of Havenwood","2","2","G","Each Fungus creature gets +1/+1 for each spore counter on it.
{B}{G}, Exile a Fungus card from a graveyard: Put a spore counter on each Fungus on the battlefield.","GG","Legendary Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pain's Reward","B","Each player may bid life. You start the bidding with a bid of any number. In turn order, each player may top the high bid. The bidding ends if the high bid stands. The high bidder loses life equal to the high bid and draws four cards.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fleshformer","2","2","B","{W}{U}{B}{R}{G}: Fleshformer gets +2/+2 and gains fear until end of turn. Target creature gets -2/-2 until end of turn. Activate this ability only during your turn. (A creature with fear can't be blocked except by artifact creatures and/or black creatures.)","2B","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thrummingbird","1","1","U","Flying
Whenever Thrummingbird deals combat damage to a player, proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","1U","Creature - Bird Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Essence of the Wild","6","6","G","Creatures you control enter the battlefield as a copy of Essence of the Wild.","3GGG","Creature - Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodline Shaman","1","1","G","{T}: Choose a creature type. Reveal the top card of your library. If that card is a creature card of the chosen type, put it into your hand. Otherwise, put it into your graveyard.","1G","Creature - Elf Wizard Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wild Growth","G","Enchant land
Whenever enchanted land is tapped for mana, its controller adds {G} to his or her mana pool (in addition to the mana the land produces).","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gleancrawler","6","6","BG","({(b/g)} can be paid with either {B} or {G}.)
Trample
At the beginning of your end step, return to your hand all creature cards in your graveyard that were put there from the battlefield this turn.","3(B/G)(B/G)(B/G)","Creature - Insect Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Clot Sliver","1","1","B","All Slivers have ""{2}: Regenerate this permanent.""","1B","Creature - Sliver");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Contagion Engine","When Contagion Engine enters the battlefield, put a -1/-1 counter on each creature target player controls.
{4}, {T}: Proliferate, then proliferate again. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there. Then do it again.)","6","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Isolation Cell","Whenever an opponent casts a creature spell, that player loses 2 life unless he or she pays {2}.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crabapple Cohort","4","4","G","Crabapple Cohort gets +1/+1 as long as you control another green creature.","4G","Creature - Treefolk Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armorer Guildmage","1","1","R","{B}, {T}: Target creature gets +1/+0 until end of turn.
{G}, {T}: Target creature gets +0/+1 until end of turn.","R","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timbermaw Larva","2","2","G","Whenever Timbermaw Larva attacks, it gets +1/+1 until end of turn for each Forest you control.","3G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Floating Shield","W","Enchant creature
As Floating Shield enters the battlefield, choose a color.
Enchanted creature has protection from the chosen color. This effect doesn't remove Floating Shield.
Sacrifice Floating Shield: Target creature gains protection from the chosen color until end of turn.","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glass Asp","2","1","G","Whenever Glass Asp deals combat damage to a player, that player loses 2 life at the beginning of his or her next draw step unless he or she pays {2} before that step.","1GG","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Port Inspector","1","2","U","Whenever Port Inspector becomes blocked, you may look at defending player's hand.","1U","Creature - Human");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sphere of the Suns","Sphere of the Suns enters the battlefield tapped and with three charge counters on it.
{T}, Remove a charge counter from Sphere of the Suns: Add one mana of any color to your mana pool.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Restore Balance","Suspend 6-{W} (Rather than cast this card from your hand, pay {W} and exile it with six time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)
Each player chooses a number of lands he or she controls equal to the number of lands controlled by the player who controls the fewest, then sacrifices the rest. Players sacrifice creatures and discard cards the same way.",NULL,"Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Capricious Efreet","6","4","R","At the beginning of your upkeep, choose target nonland permanent you control and up to two target nonland permanents you don't control. Destroy one of them at random.","4RR","Creature - Efreet");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wydwen, the Biting Gale","3","3","UB","Flash
Flying
{U}{B}, Pay 1 life: Return Wydwen, the Biting Gale to its owner's hand.","2UB","Legendary Creature - Faerie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aurora Griffin","2","2","W","Flying
{W}: Target permanent becomes white until end of turn.","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decree of Pain","B","Destroy all creatures. They can't be regenerated. Draw a card for each creature destroyed this way.
Cycling {3}{B}{B} ({3}{B}{B}, Discard this card: Draw a card.)
When you cycle Decree of Pain, all creatures get -2/-2 until end of turn.","6BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kithkin Armor","W","Enchant creature
Enchanted creature can't be blocked by creatures with power 3 or greater.
Sacrifice Kithkin Armor: The next time a source of your choice would deal damage to enchanted creature this turn, prevent that damage.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Megatherium","4","4","G","Trample
When Megatherium enters the battlefield, sacrifice it unless you pay {1} for each card in your hand.","2G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Transformation","W","Enchant creature
Enchanted creature gets +3/+3.","2WW","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Befoul","B","Destroy target land or nonblack creature. It can't be regenerated.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jagged Lightning","R","Jagged Lightning deals 3 damage to each of two target creatures.","3RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feast or Famine","B","Choose one - Put a 2/2 black Zombie creature token onto the battlefield; or destroy target nonartifact, nonblack creature and it can't be regenerated.","3B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dingus Egg","Whenever a land is put into a graveyard from the battlefield, Dingus Egg deals 2 damage to that land's controller.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Land Tax","W","At the beginning of your upkeep, if an opponent controls more lands than you, you may search your library for up to three basic land cards, reveal them, and put them into your hand. If you do, shuffle your library.","W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thunderblade Charge","R","Thunderblade Charge deals 3 damage to target creature or player.
Whenever one or more creatures you control deal combat damage to a player, if Thunderblade Charge is in your graveyard, you may pay {2}{R}{R}{R}. If you do, you may cast it without paying its mana cost.","1RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scarland Thrinax","2","2","BRG","Sacrifice a creature: Put a +1/+1 counter on Scarland Thrinax.","BRG","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Charm School","W","When Charm School comes into play, choose a color and balance Charm School on your head.
Prevent all damage to you of the chosen color.
If Charm School falls off your head, sacrifice Charm School.","2W","Enchant Player");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steelshaper's Gift","W","Search your library for an Equipment card, reveal that card, and put it into your hand. Then shuffle your library.","W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ramosian Rally","W","If you control a Plains, you may tap an untapped creature you control rather than pay Ramosian Rally's mana cost.
Creatures you control get +1/+1 until end of turn.","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Urborg Uprising","B","Return up to two target creature cards from your graveyard to your hand.
Draw a card.","4B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thoughtlace","U","Target spell or permanent becomes blue. (Mana symbols on that permanent remain unchanged.)","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Rebirth","W","Destroy all creatures, then put an X/X colorless Horror artifact creature token onto the battlefield, where X is the number of creatures destroyed this way.","4WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Mountaineer","1","1","R","Mountainwalk","R","Creature - Goblin Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flood","U","{U}{U}: Tap target creature without flying.","U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jund Hackblade","2","1","BRG","As long as you control another multicolored permanent, Jund Hackblade gets +1/+1 and has haste.","(B/G)R","Creature - Goblin Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Singe-Mind Ogre","3","2","BR","When Singe-Mind Ogre enters the battlefield, target player reveals a card at random from his or her hand, then loses life equal to that card's converted mana cost.","2BR","Creature - Ogre Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squire","1","2","W",NULL,"1W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Siren's Call","U","Cast Siren's Call only during an opponent's turn, before attackers are declared.
Creatures the active player controls attack this turn if able.
At the beginning of the next end step, destroy all non-Wall creatures that player controls that didn't attack this turn. Ignore this effect for each creature the player didn't control continuously since the beginning of the turn.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Weirding Shaman","2","1","B","{3}{B}, Sacrifice a Goblin: Put two 1/1 black Goblin Rogue creature tokens onto the battlefield.","1B","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Mercenary","2","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
{1}{B}: Dauthi Mercenary gets +1/+0 until end of turn.","2B","Creature - Dauthi Knight Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kruin Outlaw","2","2","R","First strike
At the beginning of each upkeep, if no spells were cast last turn, transform Kruin Outlaw.","1RR","Creature - Human Rogue Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Savage Hunger","G","Enchant creature
Enchanted creature gets +1/+0 and has trample.
Cycling {2} ({2}, Discard this card: Draw a card.)","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Radiant's Dragoons","2","5","W","Echo {3}{W} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Radiant's Dragoons enters the battlefield, you gain 5 life.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Syphon-Mage","2","2","B","{2}{B}, {T}, Discard a card: Each other player loses 2 life. You gain life equal to the life lost this way.","2B","Creature - Human Spellshaper");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rites of Spring","G","Discard any number of cards. Search your library for that many basic land cards, reveal those cards, and put them into your hand. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dueling Grounds","WG","No more than one creature can attack each turn.
No more than one creature can block each turn.","1GW","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pillar of the Paruns","{T}: Add one mana of any color to your mana pool. Spend this mana only to cast a multicolored spell.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viscera Seer","1","1","B","Sacrifice a creature: Scry 1. (To scry 1, look at the top card of your library, then you may put that card on the bottom of your library.)","B","Creature - Vampire Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strength of Night","G","Kicker {B} (You may pay an additional {B} as you cast this spell.)
Creatures you control get +1/+1 until end of turn. If Strength of Night was kicked, Zombie creatures you control get an additional +2/+2 until end of turn.","2G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scion of the Wild","*","*","G","Scion of the Wild's power and toughness are each equal to the number of creatures you control.","1GG","Creature - Avatar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gloomlance","B","Destroy target creature. If that creature was green or white, its controller discards a card.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Gardener","2","1","R","When Goblin Gardener dies, destroy target land.","3R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faerie Squadron","1","1","U","Kicker {3}{U} (You may pay an additional {3}{U} as you cast this spell.)
If Faerie Squadron was kicked, it enters the battlefield with two +1/+1 counters on it and with flying.","U","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rabble-Rouser","1","1","R","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
{R}, {T}: Attacking creatures get +X/+0 until end of turn, where X is Rabble-Rouser's power.","3R","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Mirror","4","2","B","{T}, Sacrifice Magus of the Mirror: Exchange life totals with target opponent. Activate this ability only during your upkeep.","4BB","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Faith's Shield","W","Target permanent you control gains protection from the color of your choice until end of turn.
Fateful hour - If you have 5 or less life, instead you and each permanent you control gain protection from the color of your choice until end of turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Royal Falcon","1","1","W","Flying","1W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ruham Djinn","5","5","W","First strike
Ruham Djinn gets -2/-2 as long as white is the most common color among all permanents or is tied for most common.","5W","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shape Stealer","1","1","U","Whenever Shape Stealer blocks or becomes blocked by a creature, change Shape Stealer's power and toughness to that creature's power and toughness until end of turn.","UU","Creature - Shapeshifter Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Numot, the Devastator","6","6","WUR","Flying
Whenever Numot, the Devastator deals combat damage to a player, you may pay {2}{R}. If you do, destroy up to two target lands.","3RWU","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Redeemer","2","2","W","Flying
{T}: Prevent the next 2 damage that would be dealt to target creature or player this turn.","3W","Creature - Bird Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sentry Oak","3","5","W","Defender
At the beginning of combat on your turn, you may clash with an opponent. If you win, Sentry Oak gets +2/+0 and loses defender until end of turn. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","4W","Creature - Treefolk Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glacial Ray","R","Glacial Ray deals 2 damage to target creature or player.
Splice onto Arcane {1}{R} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1R","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drifting Shade","1","1","B","Flying
{B}: Drifting Shade gets +1/+1 until end of turn.","3B","Creature - Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inkwell Leviathan","7","11","U","Islandwalk, trample, shroud","7UU","Artifact Creature - Leviathan");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kessig Cagebreakers","3","4","G","Whenever Kessig Cagebreakers attacks, put a 2/2 green Wolf creature token onto the battlefield tapped and attacking for each creature card in your graveyard.","4G","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flash Counter","U","Counter target instant spell.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eiganjo Free-Riders","3","4","W","Flying
At the beginning of your upkeep, return a white creature you control to its owner's hand.","3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spectral Cloak","U","Enchant creature
Enchanted creature has shroud as long as it's untapped. (It can't be the target of spells or abilities.)","UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Turtleshell Changeling","1","4","U","Changeling (This card is every creature type at all times.)
{1}{U}: Switch Turtleshell Changeling's power and toughness until end of turn.","3U","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Undying Evil","B","Target creature gains undying until end of turn. (When it dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skinshifter","1","1","G","{G}: Choose one - Until end of turn, Skinshifter becomes a 4/4 Rhino and gains trample; or until end of turn, Skinshifter becomes a 2/2 Bird and gains flying; or until end of turn, Skinshifter becomes a 0/8 Plant. Activate this ability only once each turn.","1G","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fat Ass","2","3{1/2}","G","Fat Ass gets +2/+2 and has trample as long as you're eating. (Food is in your mouth and you're chewing, licking, sucking, or swallowing it.)","4G","Creature - Donkey Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Conquering Manticore","5","5","R","Flying
When Conquering Manticore enters the battlefield, gain control of target creature an opponent controls until end of turn. Untap that creature. It gains haste until end of turn.","4RR","Creature - Manticore");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serra's Blessing","W","Creatures you control have vigilance. (Attacking doesn't cause them to tap.)","1W","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Draco","9","9","Domain - Draco costs {2} less to cast for each basic land type among lands you control.
Flying
Domain - At the beginning of your upkeep, sacrifice Draco unless you pay {1}0}. This cost is reduced by {2} for each basic land type among lands you control.","16","Artifact Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flurry of Wings","WUG","Put X 1/1 white Bird Soldier creature tokens with flying onto the battlefield, where X is the number of attacking creatures.","GWU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firemaw Kavu","4","2","R","Echo {5}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Firemaw Kavu enters the battlefield, it deals 2 damage to target creature.
When Firemaw Kavu leaves the battlefield, it deals 4 damage to target creature.","5R","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stensia Bloodhall","{T}: Add {1} to your mana pool.
{3}{B}{R}, {T}: Stensia Bloodhall deals 2 damage to target player.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ashling the Pilgrim Avatar","{2}: Ashling the Pilgrim Avatar deals 1 damage to each creature and each player.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Baneslayer Angel","5","5","W","Flying, first strike, lifelink, protection from Demons and from Dragons","3WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Qumulox","5","4","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Flying","6UU","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Locket of Yesterdays","Spells you cast cost {1} less to cast for each card with the same name as that spell in your graveyard.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Future","2","3","U","Play with the top card of your library revealed.
You may play the top card of your library.","2UUU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leaf Dancer","2","2","G","Forestwalk","1GG","Creature - Centaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Ants","2","1","R","{T}: Fire Ants deals 1 damage to each other creature without flying.","2R","Creature - Insect");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Goblin Dirigible","4","4","Flying
Goblin Dirigible doesn't untap during your untap step.
At the beginning of your upkeep, you may pay {4}. If you do, untap Goblin Dirigible.","6","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Energy Field","U","Prevent all damage that would be dealt to you by sources you don't control.
When a card is put into your graveyard from anywhere, sacrifice Energy Field.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Solemn Offering","W","Destroy target artifact or enchantment. You gain 4 life.","2W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Metathran Aerostat","2","2","U","Flying
{X}{U}: You may put a creature card with converted mana cost X from your hand onto the battlefield. If you do, return Metathran Aerostat to its owner's hand.","2UU","Creature - Metathran");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidal Kraken","6","6","U","Tidal Kraken is unblockable.","5UUU","Creature - Kraken");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Candles' Glow","W","Prevent the next 3 damage that would be dealt to target creature or player this turn. You gain life equal to the damage prevented this way.
Splice onto Arcane {1}{W} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1W","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Spirit","2","3","U","{U}: Sea Spirit gets +1/+0 until end of turn.","4U","Creature - Elemental Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Smite the Monstrous","W","Destroy target creature with power 4 or greater.","3W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Assassin","3","2","B","{T}, Exile two cards from your graveyard and Zombie Assassin: Destroy target nonblack creature. It can't be regenerated.","4B","Creature - Zombie Assassin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Morinfen","5","4","B","Flying
Cumulative upkeep-Pay 1 life. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","3BB","Legendary Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Meltdown","R","Destroy each artifact with converted mana cost X or less.","XR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Afiya Grove","G","Afiya Grove enters the battlefield with three +1/+1 counters on it.
At the beginning of your upkeep, move a +1/+1 counter from Afiya Grove onto target creature.
When Afiya Grove has no +1/+1 counters on it, sacrifice it.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snare the Skies","G","Target creature gets +1/+1 and gains reach until end of turn. (It can block creatures with flying.)","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Run Wild","G","Until end of turn, target creature gains trample and ""{G}: Regenerate this creature.""","G","Instant");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Chandra Ablaze","R","+1: Discard a card. If a red card is discarded this way, Chandra Ablaze deals 4 damage to target creature or player.
-2: Each player discards his or her hand, then draws three cards.
-7: Cast any number of red instant and/or sorcery cards from your graveyard without paying their mana costs.","Planeswalker - Chandra","4RR");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Sentry","3","3","R","Defender","1R","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slinking Giant","4","4","R","Wither (This deals damage to creatures in the form of -1/-1 counters.)
Whenever Slinking Giant blocks or becomes blocked, it gets -3/-0 until end of turn.","2RR","Creature - Giant Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Price of Progress","R","Price of Progress deals damage to each player equal to twice the number of nonbasic lands that player controls.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Druidic Satchel","{2}, {T}: Reveal the top card of your library. If it's a creature card, put a 1/1 green Saproling creature token onto the battlefield. If it's a land card, put that card onto the battlefield under your control. If it's a noncreature, nonland card, you gain 2 life.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Virtue's Ruin","B","Destroy all white creatures.","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crooked Scales","{4}, {T}: Flip a coin. If you win the flip, destroy target creature an opponent controls. If you lose the flip, destroy target creature you control unless you pay {3} and repeat this process.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Legacy Weapon","{W}{U}{B}{R}{G}: Exile target permanent.
If Legacy Weapon would be put into a graveyard from anywhere, reveal Legacy Weapon and shuffle it into its owner's library instead.","7","Legendary Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghitu War Cry","R","{R}: Target creature gets +1/+0 until end of turn.","2R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mindculling","U","You draw two cards and target opponent discards two cards.","5U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eunuchs' Intrigues","R","Target opponent chooses a creature he or she controls. Other creatures he or she controls can't block this turn.","2R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Illusory Demon","4","3","UB","Flying
When you cast a spell, sacrifice Illusory Demon.","1UB","Creature - Demon Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Druid's Call","G","Enchant creature
Whenever enchanted creature is dealt damage, its controller puts that many 1/1 green Squirrel creature tokens onto the battlefield.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prismatic Ward","W","Enchant creature
As Prismatic Ward enters the battlefield, choose a color.
Prevent all damage that would be dealt to enchanted creature by sources of the chosen color.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grasp of Phantoms","U","Put target creature on top of its owner's library.
Flashback {7}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kashi-Tribe Elite","2","3","G","Legendary Snakes you control have shroud. (They can't be the targets of spells or abilities.)
Whenever Kashi-Tribe Elite deals combat damage to a creature, tap that creature and it doesn't untap during its controller's next untap step.","1GG","Creature - Snake Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Flux","U","Other enchantments have ""At the beginning of your upkeep, sacrifice this enchantment unless you pay {2}.""","2U","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Darksteel Gargoyle","3","3","Flying
Darksteel Gargoyle is indestructible. (""Destroy"" effects and lethal damage don't destroy it.)","7","Artifact Creature - Gargoyle");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Veteran's Armaments","Equipped creature has ""Whenever this creature attacks or blocks, it gets +1/+1 until end of turn for each attacking creature.""
Whenever a Soldier creature enters the battlefield, you may attach Veteran's Armaments to it.
Equip {2}","2","Tribal Artifact - Soldier Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shifting Sky","U","As Shifting Sky enters the battlefield, choose a color.
All nonland permanents are the chosen color.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Preemptive Strike","U","Counter target creature spell.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rule of Law","W","Each player can't cast more than one spell each turn.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Recluse","5","4","R","Whenever a player casts a spell, tap Ogre Recluse.","3R","Creature - Ogre Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Turf Wound","R","Target player can't play land cards this turn.
Draw a card.","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Victorious Destruction","R","Destroy target artifact or land. Its controller loses 1 life.","4R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kiln Fiend","1","2","R","Whenever you cast an instant or sorcery spell, Kiln Fiend gets +3/+0 until end of turn.","1R","Creature - Elemental Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fumarole","BR","As an additional cost to cast Fumarole, pay 3 life.
Destroy target creature and target land.","3BR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Winter Blast","G","Tap X target creatures. Winter Blast deals 2 damage to each of those creatures with flying.","XG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pedantic Learning","U","Whenever a land card is put into your graveyard from your library, you may pay {1}. If you do, draw a card.","UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Spy","1","1","R","Play with the top card of your library revealed.","R","Creature - Goblin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Etherium Abomination","4","3","UB","Unearth {1}{U}{B} ({1}{U}{B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","3UB","Artifact Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sky Hussar","4","3","WU","Flying
When Sky Hussar enters the battlefield, untap all creatures you control.
Forecast - Tap two untapped white and/or blue creatures you control, Reveal Sky Hussar from your hand: Draw a card. (Activate this ability only during your upkeep and only once each turn.)","3WU","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foratog","1","2","G","{G}, Sacrifice a Forest: Foratog gets +2/+2 until end of turn.","2G","Creature - Atog");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Treva's Charm","WUG","Choose one - Destroy target enchantment; or exile target attacking creature; or draw a card, then discard a card.","GWU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Citanul Woodreaders","1","4","G","Kicker {2}{G} (You may pay an additional {2}{G} as you cast this spell.)
When Citanul Woodreaders enters the battlefield, if it was kicked, draw two cards.","2G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razing Snidd","3","3","BR","When Razing Snidd enters the battlefield, return a black or red creature you control to its owner's hand.
When Razing Snidd enters the battlefield, each player sacrifices a land.","4BR","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hedge Troll","2","2","G","Hedge Troll gets +1/+1 as long as you control a Plains.
{W}: Regenerate Hedge Troll.","2G","Creature - Troll Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bludgeon Brawl","R","Each noncreature, non-Equipment artifact is an Equipment with equip {X} and ""Equipped creature gets +X/+0,"" where X is that artifact's converted mana cost.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Synapse Sliver","3","3","U","Whenever a Sliver deals combat damage to a player, its controller may draw a card.","4U","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Engineered Plague","B","As Engineered Plague enters the battlefield, choose a creature type.
All creatures of the chosen type get -1/-1.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hanna, Ship's Navigator","1","2","WU","{1}{W}{U}, {T}: Return target artifact or enchantment card from your graveyard to your hand.","1WU","Legendary Creature - Human Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tsabo's Decree","B","Choose a creature type. Target player reveals his or her hand and discards all creature cards of that type. Then destroy all creatures of that type that player controls. They can't be regenerated.","5B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread Reaper","6","5","B","Flying
When Dread Reaper enters the battlefield, you lose 5 life.","3BBB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fog of Gnats","1","1","B","Flying
{B}: Regenerate Fog of Gnats.","BB","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unified Strike","W","Exile target attacking creature if its power is less than or equal to the number of Soldiers on the battlefield.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Oblivion Crown","B","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature has ""Discard a card: This creature gets +1/+1 until end of turn.""","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Pony","1","1","R","{1}{R}, {T}: Target Dwarf creature gains mountainwalk until end of turn.","R","Creature - Horse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Overtaker","1","1","U","{3}{U}, {T}, Discard a card: Untap target creature and gain control of it until end of turn. That creature gains haste until end of turn.","1U","Creature - Merfolk Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of the Skyward Eye","2","2","W","{3}{G}: Knight of the Skyward Eye gets +3/+3 until end of turn. Activate this ability only once each turn.","1W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disciple of Grace","1","2","W","Protection from black
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infernal Medusa","2","4","B","Whenever Infernal Medusa blocks a creature, destroy that creature at end of combat.
Whenever Infernal Medusa becomes blocked by a non-Wall creature, destroy that creature at end of combat.","3BB","Creature - Gorgon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devoted Caretaker","1","2","W","{W}, {T}: Target permanent you control gains protection from instant spells and from sorcery spells until end of turn.","W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Dragon Engine","1","3","{2}: Dragon Engine gets +1/+0 until end of turn.","3","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Volcano Imp","2","2","B","Flying
{1}{R}: Volcano Imp gains first strike until end of turn.","3B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thief of Hope","2","2","B","Whenever you cast a Spirit or Arcane spell, target opponent loses 1 life and you gain 1 life.
Soulshift 2 (When this creature dies, you may return target Spirit card with converted mana cost 2 or less from your graveyard to your hand.)","2B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Brute","5","4","B","Amplify 1 (As this creature enters the battlefield, put a +1/+1 counter on it for each Zombie card you reveal in your hand.)
Trample","6B","Creature - Zombie");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jack-in-the-Mox","ocT: Roll a six-sided die for Jack-in-the-Mox. On a 1, sacrifice Jack-in-the-Mox and lose 5 life. Otherwise, Jack-in-the-Mox has one of the following effects. Treat this ability as a mana source.
2 Add oW to your mana pool.
3 Add oU to your mana pool.
4 Add oB to your mana pool.
5 Add oR to your mana pool.
6 Add oG to your mana pool.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flesh Allergy","B","As an additional cost to cast Flesh Allergy, sacrifice a creature.
Destroy target creature. Its controller loses life equal to the number of creatures that died this turn.","2BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Leonin Scimitar","Equipped creature gets +1/+1.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Duergar Cave-Guard","1","3","R","Wither (This deals damage to creatures in the form of -1/-1 counters.)
{(r/w)}: Duergar Cave-Guard gets +1/+0 until end of turn.","3R","Creature - Dwarf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rogue Elephant","3","3","G","When Rogue Elephant enters the battlefield, sacrifice it unless you sacrifice a Forest.","G","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gutwrencher Oni","5","4","B","Trample
At the beginning of your upkeep, discard a card if you don't control an Ogre.","3BB","Creature - Demon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tombstone Stairwell","B","Cumulative upkeep {1}{B} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of each upkeep, if Tombstone Stairwell is on the battlefield, each player puts a 2/2 black Zombie creature token with haste named Tombspawn onto the battlefield for each creature card in his or her graveyard.
At the beginning of each end step or when Tombstone Stairwell leaves the battlefield, destroy all tokens put onto the battlefield with Tombstone Stairwell. They can't be regenerated.","2BB","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tidal Visionary","1","1","U","{T}: Target creature becomes the color of your choice until end of turn.","U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scepter of Fugue","B","{1}{B}, {T}: Target player discards a card. Activate this ability only during your turn.","BB","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eiganjo Castle","{T}: Add {W} to your mana pool.
{W}, {T}: Prevent the next 2 damage that would be dealt to target legendary creature this turn.",NULL,"Legendary Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pithing Needle","As Pithing Needle enters the battlefield, name a card.
Activated abilities of sources with the chosen name can't be activated unless they're mana abilities.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mischievous Poltergeist","1","1","B","Flying
Pay 1 life: Regenerate Mischievous Poltergeist.","2B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Clambassadors","4","4","U","If Clambassadors damages any player, choose an artifact, creature, or land you control. That player gains control of that artifact, creature, or land.","3U","Summon - Clamfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kataki, War's Wage","2","1","W","All artifacts have ""At the beginning of your upkeep, sacrifice this artifact unless you pay {1}.""","1W","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dream Tides","U","Creatures don't untap during their controllers' untap steps.
At the beginning of each player's upkeep, that player may choose any number of tapped nongreen creatures he or she controls and pay {2} for each creature chosen this way. If the player does, untap those creatures.","2UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wurmweaver Coil","G","Enchant green creature
Enchanted creature gets +6/+6.
{G}{G}{G}, Sacrifice Wurmweaver Coil: Put a 6/6 green Wurm creature token onto the battlefield.","4GG","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Prey Upon","G","Target creature you control fights target creature you don't control. (Each deals damage equal to its power to the other.)","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kuro's Taken","1","1","B","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
{1}{B}: Regenerate Kuro's Taken.","1B","Creature - Rat Samurai");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gravespawn Sovereign","3","3","B","Tap five untapped Zombies you control: Put target creature card from a graveyard onto the battlefield under your control.","4BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Jace, the Mind Sculptor","U","+2: Look at the top card of target player's library. You may put that card on the bottom of that player's library.
0: Draw three cards, then put two cards from your hand on top of your library in any order.
-1: Return target creature to its owner's hand.
-12: Exile all cards from target player's library, then that player shuffles his or her hand into his or her library.","Planeswalker - Jace","2UU");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azorius Herald","2","1","W","Azorius Herald is unblockable.
When Azorius Herald enters the battlefield, you gain 4 life.
When Azorius Herald enters the battlefield, sacrifice it unless {U} was spent to cast it.","2W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rogue Skycaptain","3","4","R","Flying
At the beginning of your upkeep, put a wage counter on Rogue Skycaptain. You may pay {2} for each wage counter on it. If you don't, remove all wage counters from Rogue Skycaptain and an opponent gains control of it.","2R","Creature - Human Rogue Mercenary");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Metrognome","When a spell or ability an opponent controls causes you to discard Metrognome, put four 1/1 colorless Gnome artifact creature tokens onto the battlefield.
{4}, {T}: Put a 1/1 colorless Gnome artifact creature token onto the battlefield.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Salt","R","Destroy two target lands.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bubbling Beebles","3","3","U","Bubbling Beebles is unblockable as long as defending player controls an enchantment.","4U","Creature - Beeble");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Helldozer","6","5","B","{B}{B}{B}, {T}: Destroy target land. If that land was nonbasic, untap Helldozer.","3BBB","Creature - Zombie Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Apex Hawks","2","2","W","Multikicker {1}{W} (You may pay an additional {1}{W} any number of times as you cast this spell.)
Flying
Apex Hawks enters the battlefield with a +1/+1 counter on it for each time it was kicked.","2W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sage of Lat-Nam","1","2","U","{T}, Sacrifice an artifact: Draw a card.","1U","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Looter il-Kor","1","1","U","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever Looter il-Kor deals damage to an opponent, draw a card, then discard a card.","1U","Creature - Kor Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chromatic Sphere","{1}, {T}, Sacrifice Chromatic Sphere: Add one mana of any color to your mana pool. Draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Raider","1","1","R","Sacrifice a Goblin: Target creature gets +1/+1 until end of turn.","R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wei Elite Companions","3","3","B","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","4B","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Koskun Falls","B","At the beginning of your upkeep, sacrifice Koskun Falls unless you tap an untapped creature you control.
Creatures can't attack you unless their controller pays {2} for each creature he or she controls that's attacking you.","2BB","World Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Piranha Marsh","Piranha Marsh enters the battlefield tapped.
When Piranha Marsh enters the battlefield, target player loses 1 life.
{T}: Add {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Swooping Talon","2","6","W","Flying
{1}: Swooping Talon loses flying until end of turn.
Provoke (When this attacks, you may have target creature defending player controls untap and block it if able.)","4WW","Creature - Bird Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mercadian Atlas","At the beginning of your end step, if you didn't play a land this turn, you may draw a card.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Landfill","R","Choose a land type. Remove from play all lands of that type that you control. Drop those cards, one at a time, onto the playing area from a height of at least one foot. Destroy each card in play that is completely covered by those cards. Then return to play, tapped, all lands dropped in this way.","4R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fencer Clique","3","2","U","Flying
{U}: Put Fencer Clique on top of its owner's library.","2UU","Creature - Faerie Soldier");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Treva's Attendant","3","3","{1}, Sacrifice Treva's Attendant: Add {G}{W}{U} to your mana pool.","5","Artifact Creature - Golem");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Worldslayer","Whenever equipped creature deals combat damage to a player, destroy all permanents other than Worldslayer.
Equip {5} ({5}: Attach to target creature you control. Equip only as a sorcery.)","5","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Water Servant","3","4","U","{U}: Water Servant gets +1/-1 until end of turn.
{U}: Water Servant gets -1/+1 until end of turn.","2UU","Creature - Elemental");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Igneous Golem","3","4","{2}: Igneous Golem gains trample until end of turn.","5","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stomp and Howl","G","Destroy target artifact and target enchantment.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Control of the Court","R","Draw four cards, then discard three cards at random.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eyeblight's Ending","B","Destroy target non-Elf creature.","2B","Tribal Instant - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tanglewalker","2","2","G","Each creature you control is unblockable as long as defending player controls an artifact land.","2G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vine Trellis","0","4","G","Defender (This creature can't attack.)
{T}: Add {G} to your mana pool.","1G","Creature - Plant Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("It That Betrays","11","11","Annihilator 2 (Whenever this creature attacks, defending player sacrifices two permanents.)
Whenever an opponent sacrifices a nontoken permanent, put that card onto the battlefield under your control.","12","Creature - Eldrazi");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Salvage Titan","6","4","B","You may sacrifice three artifacts rather than pay Salvage Titan's mana cost.
Exile three artifact cards from your graveyard: Return Salvage Titan from your graveyard to your hand.","4BB","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Get a Life","W","Target player and each of his or her teammates exchange life totals.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Probe","U","Kicker {1}{B} (You may pay an additional {1}{B} as you cast this spell.)
Draw three cards, then discard two cards.
If Probe was kicked, target player discards two cards.","2U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gate to the AEther","At the beginning of each player's upkeep, that player reveals the top card of his or her library. If it's an artifact, creature, enchantment, or land card, the player may put it onto the battlefield.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karplusan Yeti","3","3","R","{T}: Karplusan Yeti deals damage equal to its power to target creature. That creature deals damage equal to its power to Karplusan Yeti.","3RR","Creature - Yeti");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ivory Crane Netsuke","At the beginning of your upkeep, if you have seven or more cards in hand, you gain 4 life.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Feeding Grounds","Red spells cost {1} less to cast.
Green spells cost {1} less to cast.
Whenever you roll {C}, put X +1/+1 counters on target creature, where X is that creature's converted mana cost.",NULL,"Plane - Muraganda");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Smoldering Tar","BR","At the beginning of your upkeep, target player loses 1 life.
Sacrifice Smoldering Tar: Smoldering Tar deals 4 damage to target creature. Activate this ability only any time you could cast a sorcery.","2BR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Storm Shaman","0","4","R","{R}: Storm Shaman gets +1/+0 until end of turn.","2R","Creature - Human Cleric Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Myojin of Night's Reach","5","2","B","Myojin of Night's Reach enters the battlefield with a divinity counter on it if you cast it from your hand.
Myojin of Night's Reach is indestructible as long as it has a divinity counter on it.
Remove a divinity counter from Myojin of Night's Reach: Each opponent discards his or her hand.","5BBB","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pull Under","B","Target creature gets -5/-5 until end of turn.","5B","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Makeshift Mannequin","B","Return target creature card from your graveyard to the battlefield with a mannequin counter on it. For as long as that creature has a mannequin counter on it, it has ""When this creature becomes the target of a spell or ability, sacrifice it.""","3B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flying Men","1","1","U","Flying","U","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Karoo Meerkat","2","1","G","Protection from blue","1G","Creature - Mongoose");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ironclaw Curse","R","Enchant creature
Enchanted creature gets -0/-1.
Enchanted creature can't block creatures with power equal to or greater than the enchanted creature's toughness.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Syncopate","U","Counter target spell unless its controller pays {X}. If that spell is countered this way, exile it instead of putting it into its owner's graveyard.","XU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Morsel Theft","B","Prowl {1}{B} (You may cast this for its prowl cost if you dealt combat damage to a player this turn with a Rogue.)
Target player loses 3 life and you gain 3 life. If Morsel Theft's prowl cost was paid, draw a card.","2BB","Tribal Sorcery - Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Only Blood Ends Your Nightmares","When you set this scheme in motion, each opponent sacrifices a creature. Then each opponent who didn't sacrifice a creature discards two cards.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wren's Run Packmaster","5","5","G","Champion an Elf (When this enters the battlefield, sacrifice it unless you exile another Elf you control. When this leaves the battlefield, that card returns to the battlefield.)
{2}{G}: Put a 2/2 green Wolf creature token onto the battlefield.
Each Wolf you control has deathtouch. (Any amount of damage it deals to a creature is enough to destroy that creature.)","3G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ramosian Sergeant","1","1","W","{3}, {T}: Search your library for a Rebel permanent card with converted mana cost 2 or less and put it onto the battlefield. Then shuffle your library.","W","Creature - Human Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Rogue","0","0","G","Spike Rogue enters the battlefield with two +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Rogue: Put a +1/+1 counter on target creature.
{2}, Remove a +1/+1 counter from a creature you control: Put a +1/+1 counter on Spike Rogue.","1GG","Creature - Spike");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Steamcore Weird","1","3","U","When Steamcore Weird enters the battlefield, if {R} was spent to cast Steamcore Weird, it deals 2 damage to target creature or player.","3U","Creature - Weird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skyshaper","Sacrifice Skyshaper: Creatures you control gain flying until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krark-Clan Engineers","2","2","R","{R}, Sacrifice two artifacts: Destroy target artifact.","3R","Creature - Goblin Artificer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crystal Chimes","{3}, {T}, Sacrifice Crystal Chimes: Return all enchantment cards from your graveyard to your hand.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Springjack Pasture","{T}: Add {1} to your mana pool.
{4}, {T}: Put a 0/1 white Goat creature token onto the battlefield.
{T}, Sacrifice X Goats: Add X mana of any one color to your mana pool. You gain X life.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunted Lammasu","5","5","W","Flying
When Hunted Lammasu enters the battlefield, put a 4/4 black Horror creature token onto the battlefield under target opponent's control.","2WW","Creature - Lammasu");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Carpet of Flowers","G","At the beginning of each of your main phases, if you haven't added mana to your mana pool with this ability this turn, you may add up to X mana of any one color to your mana pool, where X is the number of Islands target opponent controls.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wormwood Dryad","3","1","G","{G}: Wormwood Dryad gains forestwalk until end of turn and deals 1 damage to you.
{B}: Wormwood Dryad gains swampwalk until end of turn and deals 1 damage to you.","2G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Dancing Scimitar","1","5","Flying (This creature can't be blocked except by creatures with flying or reach.)","4","Artifact Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Root Spider","2","2","G","Whenever Root Spider blocks, it gets +1/+0 and gains first strike until end of turn.","3G","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Makindi Shieldmate","0","3","W","Defender
Whenever Makindi Shieldmate or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Makindi Shieldmate.","2W","Creature - Kor Soldier Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deep-Slumber Titan","7","7","R","Deep-Slumber Titan enters the battlefield tapped.
Deep-Slumber Titan doesn't untap during your untap step.
Whenever Deep-Slumber Titan is dealt damage, untap it.","2RR","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chancellor of the Spires","5","7","U","You may reveal this card from your opening hand. If you do, at the beginning of the first upkeep, each opponent puts the top seven cards of his or her library into his or her graveyard.
Flying
When Chancellor of the Spires enters the battlefield, you may cast target instant or sorcery card from an opponent's graveyard without paying its mana cost.","4UUU","Creature - Sphinx");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepwood Tantiv","2","4","G","Whenever Deepwood Tantiv becomes blocked, you gain 2 life.","4G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hissing Miasma","B","Whenever a creature attacks you, its controller loses 1 life.","1BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Back to Nature","G","Destroy all enchantments.","1G","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Proteus Machine","2","2","Morph {0} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Proteus Machine is turned face up, it becomes the creature type of your choice. (This effect lasts indefinitely.)","3","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul of Magma","2","2","R","Whenever you cast a Spirit or Arcane spell, Soul of Magma deals 1 damage to target creature.","3RR","Creature - Spirit");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Darigaaz's Attendant","3","3","{1}, Sacrifice Darigaaz's Attendant: Add {B}{R}{G} to your mana pool.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyshroud Sentinel","1","1","G","When Skyshroud Sentinel enters the battlefield, you may search your library for up to three cards named Skyshroud Sentinel, reveal them, and put them into your hand. If you do, shuffle your library.","2G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squeaking Pie Grubfellows","3","2","B","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Squeaking Pie Grubfellows, you may reveal it. If you do, each opponent discards a card.","3B","Creature - Goblin Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skittering Invasion","Put five 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool.""","7","Tribal Sorcery - Eldrazi");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Festering March","B","Creatures your opponents control get -1/-1 until end of turn. Exile Festering March with three time counters on it.
Suspend 3-{2}{B} (Rather than cast this card from your hand, you may pay {2}{B} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyclaw Thrash","4","4","UR","Whenever Skyclaw Thrash attacks, flip a coin. If you win the flip, Skyclaw Thrash gets +1/+1 and gains flying until end of turn.","3UR","Artifact Creature - Viashino Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Personal Incarnation","6","6","W","{0}: The next 1 damage that would be dealt to Personal Incarnation this turn is dealt to its owner instead. Any player may activate this ability, but only if he or she owns Personal Incarnation.
When Personal Incarnation dies, its owner loses half his or her life, rounded up.","3WWW","Creature - Avatar Incarnation");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ribbons of Night","B","Ribbons of Night deals 4 damage to target creature and you gain 4 life. If {U} was spent to cast Ribbons of Night, draw a card.","4B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gilt-Leaf Archdruid","3","3","G","Whenever you cast a Druid spell, you may draw a card.
Tap seven untapped Druids you control: Gain control of all lands target player controls.","3GG","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mitotic Manipulation","U","Look at the top seven cards of your library. You may put one of those cards onto the battlefield if it has the same name as a permanent. Put the rest on the bottom of your library in any order.","1UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Relic Bind","U","Enchant artifact an opponent controls
Whenever enchanted artifact becomes tapped, choose one - Relic Bind deals 1 damage to target player; or target player gains 1 life.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Neurok Familiar","1","1","U","Flying
When Neurok Familiar enters the battlefield, reveal the top card of your library. If it's an artifact card, put it into your hand. Otherwise, put it into your graveyard.","1U","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Coral Fighters","1","1","U","Whenever Coral Fighters attacks and isn't blocked, look at the top card of defending player's library. You may put that card on the bottom of that player's library.","1U","Creature - Merfolk Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Flunkies","3","3","R","Mogg Flunkies can't attack or block alone.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightning Serpent","2","1","R","Trample, haste
Lightning Serpent enters the battlefield with X +1/+0 counters on it.
At the beginning of the end step, sacrifice Lightning Serpent.","XR","Creature - Elemental Serpent");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strategy, Schmategy","R","Roll a six-sided die for Strategy, Schmategy. On a 1, Strategy, Schmategy has no effect. Otherwise, it has one of the following effects.
2 Destroy all artifacts.
3 Destroy all lands.
4 Strategy, Schmategy deals 3 damage to each creature and player.
5 Each player discards his or her hand and draws seven cards.
6 Roll the die two more times.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Assassin's Blade","B","Cast Assassin's Blade only during the declare attackers step and only if you've been attacked this step.
Destroy target nonblack attacking creature.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eron the Relentless","5","2","R","Haste
{R}{R}{R}: Regenerate Eron the Relentless.","3RR","Legendary Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cheatyface","2","2","U","You may sneak Cheatyface into play at any time without paying for it, but if an opponent catches you right away, that player may remove Cheatyface from the game.
Flying","UUU","Creature - Efreet");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Faerie Tauntings","B","Whenever you cast a spell during an opponent's turn, you may have each opponent lose 1 life.","2B","Tribal Enchantment - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Exuberant Firestoker","1","1","R","At the beginning of your end step, if you control a creature with power 5 or greater, you may have Exuberant Firestoker deal 2 damage to target player.
{T}: Add {1} to your mana pool.","2R","Creature - Human Druid Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fodder Cannon","{4}, {T}, Sacrifice a creature: Fodder Cannon deals 4 damage to target creature.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Goblin Burrows","{T}: Add {1} to your mana pool.
{1}{R}, {T}: Target Goblin creature gets +2/+0 until end of turn.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("No-Dachi","Equipped creature gets +2/+0 and has first strike.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Burning-Tree Bloodscale","2","2","RG","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)
{2}{R}: Target creature can't block Burning-Tree Bloodscale this turn.
{2}{G}: Target creature blocks Burning-Tree Bloodscale this turn if able.","2RG","Creature - Viashino Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Putrid Leech","2","2","BG","Pay 2 life: Putrid Leech gets +2/+2 until end of turn. Activate this ability only once each turn.","BG","Creature - Zombie Leech");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul Shepherd","2","1","W","{W}, Exile a creature card from your graveyard: You gain 1 life.","1W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spore Cloud","G","Tap all blocking creatures. Prevent all combat damage that would be dealt this turn. Each attacking creature and each blocking creature doesn't untap during its controller's next untap step.","1GG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silhana Starfletcher","1","3","G","Reach (This creature can block creatures with flying.)
As Silhana Starfletcher enters the battlefield, choose a color.
{T}: Add one mana of the chosen color to your mana pool.","2G","Creature - Elf Druid Archer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Teysa, Orzhov Scion Avatar","Whenever a nontoken creature dies, put a 1/1 white Spirit creature token with flying onto the battlefield.",NULL,"Vanguard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kjeldoran Outpost","If Kjeldoran Outpost would enter the battlefield, sacrifice a Plains instead. If you do, put Kjeldoran Outpost onto the battlefield. If you don't, put it into its owner's graveyard.
{T}: Add {W} to your mana pool.
{1}{W}, {T}: Put a 1/1 white Soldier creature token onto the battlefield.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Revenant Patriarch","4","3","B","When Revenant Patriarch enters the battlefield, if {W} was spent to cast Revenant Patriarch, target player skips his or her next combat phase.
Revenant Patriarch can't block.","4B","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gaea's Blessing","G","Target player shuffles up to three target cards from his or her graveyard into his or her library.
Draw a card.
When Gaea's Blessing is put into your graveyard from your library, shuffle your graveyard into your library.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Trapper","1","2","W","{W}, {T}: Tap target creature.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Painbringer","1","1","B","{T}, Exile any number of cards from your graveyard: Target creature gets -X/-X until end of turn, where X is the number of cards exiled this way.","2BB","Creature - Human Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Extraction","W","Put target enchantment on top of its owner's library.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Symbiosis","G","Two target creatures each get +2/+2 until end of turn.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Brawler","2","2","R","First strike
Goblin Brawler can't be equipped.","2R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Werewolf Ransacker","5","4","Whenever this creature transforms into Werewolf Ransacker, you may destroy target artifact. If that artifact is put into a graveyard this way, Werewolf Ransacker deals 3 damage to that artifact's controller.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Werewolf Ransacker.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Servant","4","3","R","If a red instant or sorcery spell you control would deal damage, it deals double that damage instead.","3RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crovax the Cursed","0","0","B","Crovax the Cursed enters the battlefield with four +1/+1 counters on it.
At the beginning of your upkeep, you may sacrifice a creature. If you do, put a +1/+1 counter on Crovax. If you don't, remove a +1/+1 counter from Crovax.
{B}: Crovax gains flying until end of turn.","2BB","Legendary Creature - Vampire");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fossil Find","RG","Return a card at random from your graveyard to your hand, then reorder your graveyard as you choose.","(R/G)","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brute Force","R","Target creature gets +3/+3 until end of turn.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Priest","2","1","W","Protection from red
Shadow (This creature can block or be blocked by only creatures with shadow.)","WW","Creature - Soltari Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shrine of Burning Rage","At the beginning of your upkeep or whenever you cast a red spell, put a charge counter on Shrine of Burning Rage.
{3}, {T}, Sacrifice Shrine of Burning Rage: Shrine of Burning Rage deals damage equal to the number of charge counters on it to target creature or player.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elvish Guidance","G","Enchant land
Whenever enchanted land is tapped for mana, its controller adds {G} to his or her mana pool for each Elf on the battlefield (in addition to the mana the land produces).","2G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spider Spawning","G","Put a 1/2 green Spider creature token with reach onto the battlefield for each creature card in your graveyard.
Flashback {6}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","4G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Graxiplon","3","4","U","Graxiplon is unblockable unless defending player controls three or more creatures that share a creature type.","5U","Creature - Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Introductions Are in Order","When you set this scheme in motion, choose one - Search your library for a creature card, reveal it, put it into your hand, then shuffle your library; or you may put a creature card from your hand onto the battlefield.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Fire Fiend","1","1","R","Haste
Goblin Fire Fiend must be blocked if able.
{R}: Goblin Fire Fiend gets +1/+0 until end of turn.","3R","Creature - Goblin Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Onyx Goblet","B","{T}: Target player loses 1 life.","2B","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wings of Aesthir","WU","Enchant creature
Enchanted creature gets +1/+0 and has flying and first strike.","WU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kor Dirge","B","All damage that would be dealt this turn to target creature you control by a source of your choice is dealt to another target creature instead.","2B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wing Puncture","G","Target creature you control deals damage equal to its power to target creature with flying.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Folk of An-Havva","1","1","G","Whenever Folk of An-Havva blocks, it gets +2/+0 until end of turn.","G","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Genju of the Realm","WUBRG","Enchant land
{2}: Enchanted land becomes a legendary 8/12 Spirit creature with trample until end of turn. It's still a land.
When enchanted land is put into a graveyard, you may return Genju of the Realm from your graveyard to your hand.","WUBRG","Legendary Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sage Aven","1","3","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
When Sage Aven enters the battlefield, look at the top four cards of your library, then put them back in any order.","3U","Creature - Bird Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hibernation Sliver","2","2","UB","All Slivers have ""Pay 2 life: Return this permanent to its owner's hand.""","UB","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spectral Sliver","2","2","B","All Sliver creatures have ""{2}: This creature gets +1/+1 until end of turn.""","2B","Creature - Sliver Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Relic-Warder","2","2","W","When Leonin Relic-Warder enters the battlefield, you may exile target artifact or enchantment.
When Leonin Relic-Warder leaves the battlefield, return the exiled card to the battlefield under its owner's control.","WW","Creature - Cat Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Verdant Eidolon","2","2","G","{G}, Sacrifice Verdant Eidolon: Add three mana of any one color to your mana pool.
Whenever you cast a multicolored spell, you may return Verdant Eidolon from your graveyard to your hand.","3G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Troublesome Spirit","3","4","U","Flying
At the beginning of your end step, tap all lands you control.","2UU","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Naar Isle","At the beginning of your upkeep, put a flame counter on Naar Isle, then Naar Isle deals damage to you equal to the number of flame counters on it.
Whenever you roll {C}, Naar Isle deals 3 damage to target player.",NULL,"Plane - Wildfire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrion Wall","3","2","B","Defender (This creature can't attack.)
{1}{B}: Regenerate Carrion Wall.","1BB","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Magus of the Unseen","1","1","U","{1}{U}, {T}: Untap target artifact an opponent controls and gain control of it until end of turn. It gains haste until end of turn. When you lose control of the artifact, tap it.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Varchild's Crusader","3","2","R","{0}: Varchild's Crusader can't be blocked this turn except by Walls. Sacrifice Varchild's Crusader at the beginning of the next end step.","3R","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Feral Hydra","0","0","G","Feral Hydra enters the battlefield with X +1/+1 counters on it.
{3}: Put a +1/+1 counter on Feral Hydra. Any player may activate this ability.","XG","Creature - Hydra Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Ravager of the Fells","4","4","Trample
Whenever this creature transforms into Ravager of the Fells, it deals 2 damage to target opponent and 2 damage to up to one target creature that player controls.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Ravager of the Fells.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mosswort Bridge","Hideaway (This land enters the battlefield tapped. When it does, look at the top four cards of your library, exile one face down, then put the rest on the bottom of your library.)
{T}: Add {G} to your mana pool.
{G}, {T}: You may play the exiled card without paying its mana cost if creatures you control have total power 10 or greater.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ingot Chewer","3","3","R","When Ingot Chewer enters the battlefield, destroy target artifact.
Evoke {R} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","4R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shimmering Barrier","1","3","W","Defender (This creature can't attack.)
First strike
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("One with Nature","G","Enchant creature
Whenever enchanted creature deals combat damage to a player, you may search your library for a basic land card, put that card onto the battlefield tapped, then shuffle your library.","G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rhystic Lightning","R","Rhystic Lightning deals 4 damage to target creature or player unless that creature's controller or that player pays {2}. If he or she does, Rhystic Lightning deals 2 damage to the creature or player.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nova Chaser","10","2","R","Trample
Champion an Elemental (When this enters the battlefield, sacrifice it unless you exile another Elemental you control. When this leaves the battlefield, that card returns to the battlefield.)","3R","Creature - Elemental Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire Bowman","1","1","R","Sacrifice Fire Bowman: Fire Bowman deals 1 damage to target creature or player. Activate this ability only during your turn, before attackers are declared.","R","Creature - Human Soldier Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Guardian's Magemark","W","Flash
Enchant creature
Creatures you control that are enchanted get +1/+1.","2W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plasma Elemental","4","1","U","Plasma Elemental is unblockable.","5U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Invisibility","U","Enchant creature
Enchanted creature can't be blocked except by Walls.","UU","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Doubling Cube","{3}, {T}: Double the amount of each type of mana in your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rushwood Legate","2","1","G","If an opponent controls an Island and you control a Forest, you may cast Rushwood Legate without paying its mana cost.","2G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orzhov Euthanist","2","2","B","Haunt (When this creature dies, exile it haunting target creature.)
When Orzhov Euthanist enters the battlefield or the creature it haunts dies, destroy target creature that was dealt damage this turn.","2B","Creature - Human Assassin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lavalanche","BRG","Lavalanche deals X damage to target player and each creature he or she controls.","XBRG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leap","U","Target creature gains flying until end of turn.
Draw a card.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foriysian Brigade","2","4","W","Foriysian Brigade can block an additional creature.","3W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gorilla Pack","3","3","G","Gorilla Pack can't attack unless defending player controls a Forest.
When you control no Forests, sacrifice Gorilla Pack.","2G","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teremko Griffin","2","2","W","Flying; banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Mutation","BG","Destroy target nonblack creature. It can't be regenerated. Put X 1/1 green Saproling creature tokens onto the battlefield, where X is that creature's converted mana cost.","6BG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Stalking Tiger Avatar","Whenever you cast a creature spell, you may pay {1}. If you do, draw a card.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unsummon","U","Return target creature to its owner's hand.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auriok Champion","1","1","W","Protection from black and from red
Whenever another creature enters the battlefield, you may gain 1 life.","WW","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Protection: White","W","{1}: The next time a white source of your choice would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Pearl Shard","{3}, {T} or {W}, {T}: Prevent the next 2 damage that would be dealt to target creature or player this turn.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Urge to Feed","B","Target creature gets -3/-3 until end of turn. You may tap any number of untapped Vampire creatures you control. If you do, put a +1/+1 counter on each of those Vampires.","BB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inflame","R","Inflame deals 2 damage to each creature dealt damage this turn.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Minion Reflector","Whenever a nontoken creature enters the battlefield under your control, you may pay {2}. If you do, put a token that's a copy of that creature onto the battlefield. That token has haste and ""At the beginning of the end step, sacrifice this permanent.""","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Oyster","0","3","U","You may choose not to untap Giant Oyster during your untap step.
{T}: For as long as Giant Oyster remains tapped, target tapped creature doesn't untap during its controller's untap step, and at the beginning of each of your draw steps, put a -1/-1 counter on that creature. When Giant Oyster leaves the battlefield or becomes untapped, remove all -1/-1 counters from the creature.","2UU","Creature - Oyster");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostly Touch","U","Enchant creature
Enchanted creature has ""Whenever this creature attacks, you may tap or untap target permanent.""","1U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Visions","W","Look at the top five cards of target player's library. You may then have that player shuffle that library.","W","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of Feast and Famine","Equipped creature gets +2/+2 and has protection from black and from green.
Whenever equipped creature deals combat damage to a player, that player discards a card and you untap all lands you control.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Disentomb","B","Return target creature card from your graveyard to your hand.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guardian of the Guildpact","2","3","W","Protection from monocolored","3W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eladamri's Vineyard","G","At the beginning of each player's precombat main phase, add {G}{G} to that player's mana pool.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dense Foliage","G","Creatures can't be the targets of spells.","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Myojin of Infinite Rage","7","4","R","Myojin of Infinite Rage enters the battlefield with a divinity counter on it if you cast it from your hand.
Myojin of Infinite Rage is indestructible as long as it has a divinity counter on it.
Remove a divinity counter from Myojin of Infinite Rage: Destroy all lands.","7RRR","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Regal Force","5","5","G","When Regal Force enters the battlefield, draw a card for each green creature you control.","4GGG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Demonspine Whip","BR","{X}: Equipped creature gets +X/+0 until end of turn.
Equip {1}","BR","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dreg Reaver","4","3","B",NULL,"4B","Creature - Zombie Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Prowess","R","Enchant creature
Enchanted creature has haste and ""{T}: This creature deals 1 damage to target creature or player.""","2R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Outrider","2","2","W","{W}: Kjeldoran Outrider gets +0/+1 until end of turn.","1W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skycloud Expanse","{1}, {T}: Add {W}{U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Squee's Embrace","WR","Enchant creature
Enchanted creature gets +2/+2.
When enchanted creature dies, return that card to its owner's hand.","RW","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kjeldoran War Cry","W","Creatures you control get +X/+X until end of turn, where X is 1 plus the number of cards named Kjeldoran War Cry in all graveyards.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loxodon Convert","4","2","W",NULL,"3W","Creature - Elephant Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Energy Tap","U","Tap target untapped creature you control. If you do, add {X} to your mana pool, where X is that creature's converted mana cost.","U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Keen Sense","G","Enchant creature
Whenever enchanted creature deals damage to an opponent, you may draw a card.","G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rumbling Slum Avatar","At the beginning of your upkeep, Rumbling Slum Avatar deals 1 damage to each opponent.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Auriok Siege Sled","3","5","{1}: Target artifact creature blocks Auriok Siege Sled this turn if able.
{1}: Target artifact creature can't block Auriok Siege Sled this turn.","6","Artifact Creature - Juggernaut");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tolarian Academy","{T}: Add {U} to your mana pool for each artifact you control.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hoofprints of the Stag","W","Whenever you draw a card, you may put a hoofprint counter on Hoofprints of the Stag.
{2}{W}, Remove four hoofprint counters from Hoofprints of the Stag: Put a 4/4 white Elemental creature token with flying onto the battlefield. Activate this ability only during your turn.","1W","Tribal Enchantment - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Falkenrath Noble","2","2","B","Flying
Whenever Falkenrath Noble or another creature dies, target player loses 1 life and you gain 1 life.","3B","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lumengrid Augur","2","2","U","{1}, {T}: Target player draws a card, then discards a card. If that player discards an artifact card this way, untap Lumengrid Augur.","3U","Creature - Vedalken Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sandskin","W","Enchant creature
Prevent all combat damage that would be dealt to and dealt by enchanted creature.","2W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wing Snare","G","Destroy target creature with flying.","2G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dwell on the Past","G","Target player shuffles up to four target cards from his or her graveyard into his or her library.","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fungal Behemoth","*","*","G","Fungal Behemoth's power and toughness are each equal to the number of +1/+1 counters on creatures you control.
Suspend X-{X}{G}{G}. X can't be 0. (Rather than cast this card from your hand, you may pay {X}{G}{G} and exile it with X time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Whenever a time counter is removed from Fungal Behemoth while it's exiled, you may put a +1/+1 counter on target creature.","3G","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Detonate","R","Destroy target artifact with converted mana cost X. It can't be regenerated. Detonate deals X damage to that artifact's controller.","XR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Elven Rite","G","Distribute two +1/+1 counters among one or two target creatures.","1G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fire // Ice","R","Fire deals 2 damage divided as you choose among one or two target creatures and/or players.
---
Tap target permanent.
Draw a card.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riptide Director","2","3","U","{2}{U}{U}, {T}: Draw a card for each Wizard you control.","2UU","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Clearwater Goblet","Sunburst (This enters the battlefield with a charge counter on it for each color of mana spent to cast it.)
At the beginning of your upkeep, you may gain life equal to the number of charge counters on Clearwater Goblet.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keldon Arsonist","1","1","R","{1}, Sacrifice two lands: Destroy target land.","2R","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kalastria Highborn","2","2","B","Whenever Kalastria Highborn or another Vampire you control dies, you may pay {B}. If you do, target player loses 2 life and you gain 2 life.","BB","Creature - Vampire Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Headless Skaab","3","6","U","As an additional cost to cast Headless Skaab, exile a creature card from your graveyard.
Headless Skaab enters the battlefield tapped.","2U","Creature - Zombie Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Emissary","1","4","W","Kicker {1}{G} (You may pay an additional {1}{G} as you cast this spell.)
When Benalish Emissary enters the battlefield, if it was kicked, destroy target land.","2W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fiery Bombardment","R","Chroma - {2}, Sacrifice a creature: Fiery Bombardment deals damage to target creature or player equal to the number of red mana symbols in the sacrificed creature's mana cost.","1R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Agrus Kos, Wojek Veteran","3","3","WR","Whenever Agrus Kos, Wojek Veteran attacks, attacking red creatures get +2/+0 and attacking white creatures get +0/+2 until end of turn.","3RW","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Archer","2","3","G","Reach (This creature can block creatures with flying.)
{G}, Discard a card: Krosan Archer gets +0/+2 until end of turn.","3G","Creature - Centaur Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Winds of Change","R","Each player shuffles the cards from his or her hand into his or her library, then draws that many cards.","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call to Heel","U","Return target creature to its owner's hand. Its controller draws a card.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reaper from the Abyss","6","6","B","Flying
Morbid - At the beginning of each end step, if a creature died this turn, destroy target non-Demon creature.","3BBB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thought Scour","U","Target player puts the top two cards of his or her library into his or her graveyard.
Draw a card.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bone to Ash","U","Counter target creature spell.
Draw a card.","2UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barbed Lightning","R","Choose one - Barbed Lightning deals 3 damage to target creature; or Barbed Lightning deals 3 damage to target player.
Entwine {2} (Choose both if you pay the entwine cost.)","2R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sangrite Backlash","BRG","Enchant creature
Enchanted creature gets +3/-3.","(B/G)R","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Zombie token card",NULL,NULL,NULL);
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Hollow Warrior","4","4","Hollow Warrior can't attack or block unless you tap an untapped creature you control not declared as an attacking or blocking creature this combat.","4","Artifact Creature - Golem Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hungry Spriggan","1","1","G","Trample
Whenever Hungry Spriggan attacks, it gets +3/+3 until end of turn.","2G","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arrogant Bloodlord","4","4","B","Whenever Arrogant Bloodlord blocks or becomes blocked by a creature with power 1 or less, destroy Arrogant Bloodlord at end of combat.","1BB","Creature - Vampire Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Obelisk of Esper","{T}: Add {W}, {U}, or {B} to your mana pool.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dead Ringers","B","Destroy two target nonblack creatures unless either one is a color the other isn't. They can't be regenerated.","4B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloated Toad","2","2","G","Protection from blue
Cycling {2} ({2}, Discard this card: Draw a card.)","2G","Creature - Frog");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pollenbright Wings","WG","Enchant creature
Enchanted creature has flying.
Whenever enchanted creature deals combat damage to a player, put that many 1/1 green Saproling creature tokens onto the battlefield.","4GW","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skill Borrower","1","3","U","Play with the top card of your library revealed.
As long as the top card of your library is an artifact or creature card, Skill Borrower has all activated abilities of that card. (If any of the abilities use that card's name, use this creature's name instead.)","2U","Artifact Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Royal Assassin Avatar","At the beginning of your upkeep, you draw a card and you lose 1 life.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thorn Elemental","7","7","G","You may have Thorn Elemental assign its combat damage as though it weren't blocked.","5GG","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ancient Spring","Ancient Spring enters the battlefield tapped.
{T}: Add {U} to your mana pool.
{T}, Sacrifice Ancient Spring: Add {W}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shields of Velis Vel","W","Changeling (This card is every creature type at all times.)
Creatures target player controls get +0/+1 and gain all creature types until end of turn.","W","Tribal Instant - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodfire Colossus","6","6","R","{R}, Sacrifice Bloodfire Colossus: Bloodfire Colossus deals 6 damage to each creature and each player.","6RR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ursine Fylgja","3","3","W","Ursine Fylgja enters the battlefield with four healing counters on it.
Remove a healing counter from Ursine Fylgja: Prevent the next 1 damage that would be dealt to Ursine Fylgja this turn.
{2}{W}: Put a healing counter on Ursine Fylgja.","4W","Creature - Spirit Bear");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Skyreach Manta","0","0","Sunburst (This enters the battlefield with a +1/+1 counter on it for each color of mana spent to cast it.)
Flying","5","Artifact Creature - Fish");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Culling Dais","{T}, Sacrifice a creature: Put a charge counter on Culling Dais.
{1}, Sacrifice Culling Dais: Draw a card for each charge counter on Culling Dais.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pillory of the Sleepless","WB","Enchant creature
Enchanted creature can't attack or block.
Enchanted creature has ""At the beginning of your upkeep, you lose 1 life.""","1WB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wren's Run Vanquisher","3","3","G","As an additional cost to cast Wren's Run Vanquisher, reveal an Elf card from your hand or pay {3}.
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","1G","Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sphinx of Lost Truths","3","5","U","Kicker {1}{U} (You may pay an additional {1}{U} as you cast this spell.)
Flying
When Sphinx of Lost Truths enters the battlefield, draw three cards. Then if it wasn't kicked, discard three cards.","3UU","Creature - Sphinx");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hint of Insanity","B","Target player reveals his or her hand. That player discards all nonland cards with the same name as another card in his or her hand.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kris Mage","1","1","R","{R}, {T}, Discard a card: Kris Mage deals 1 damage to target creature or player.","R","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woebringer Demon","4","4","B","Flying
At the beginning of each player's upkeep, that player sacrifices a creature. If the player can't, sacrifice Woebringer Demon.","3BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Impulse","U","Look at the top four cards of your library. Put one of them into your hand and the rest on the bottom of your library in any order.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hired Giant","4","4","R","When Hired Giant enters the battlefield, each other player may search his or her library for a land card and put that card onto the battlefield. Then each player who searched his or her library this way shuffles it.","3R","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Favor of the Woods","G","Enchant creature
Whenever enchanted creature blocks, you gain 3 life.","2G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bestial Fury","R","Enchant creature
When Bestial Fury enters the battlefield, draw a card at the beginning of the next turn's upkeep.
Whenever enchanted creature becomes blocked, it gets +4/+0 and gains trample until end of turn.","2R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Telekinetic Bonds","U","Whenever a player discards a card, you may pay {1}{U}. If you do, you may tap or untap target permanent.","2UUU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunder Dragon","5","5","R","Flying
When Thunder Dragon enters the battlefield, it deals 3 damage to each creature without flying.","5RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sauté","R","Sauté deals 3{1/2} damage to target creature or player.","1RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keening Banshee","2","2","B","Flying
When Keening Banshee enters the battlefield, target creature gets -2/-2 until end of turn.","2BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Angel of Flight Alabaster","4","4","W","Flying
At the beginning of your upkeep, return target Spirit card from your graveyard to your hand.","4W","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clarion Ultimatum","WUG","Choose five permanents you control. For each of those permanents, you may search your library for a card with the same name as that permanent. Put those cards onto the battlefield tapped, then shuffle your library.","GGWWWUU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Channeling","B","Enchant creature
Pay 2 life: Regenerate enchanted creature.","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Toxic Iguanar","1","1","R","Toxic Iguanar has deathtouch as long as you control a green permanent. (Any amount of damage a creature with deathtouch deals to a creature is enough to destroy it.)","R","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thelon's Curse","G","Blue creatures don't untap during their controllers' untap steps.
At the beginning of each player's upkeep, that player may choose any number of tapped blue creatures he or she controls and pay {U} for each creature chosen this way. If the player does, untap those creatures.","GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Fyndhorn Bow","{3}, {T}: Target creature gains first strike until end of turn.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nature Shields Its Own","(An ongoing scheme remains face up until it's abandoned.)
Whenever a creature attacks and isn't blocked, if you're the defending player, put a 0/1 green Plant creature token onto the battlefield blocking that creature.
When four or more creatures attack you, abandon this scheme at end of combat.",NULL,"Ongoing Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Harbinger","1","2","G","When Elvish Harbinger enters the battlefield, you may search your library for an Elf card, reveal it, then shuffle your library and put that card on top of it.
{T}: Add one mana of any color to your mana pool.","2G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boggart Sprite-Chaser","1","2","R","As long as you control a Faerie, Boggart Sprite-Chaser gets +1/+1 and has flying.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Devouring Greed","B","As an additional cost to cast Devouring Greed, you may sacrifice any number of Spirits.
Target player loses 2 life plus 2 life for each Spirit sacrificed this way. You gain that much life.","2BB","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gelatinous Genesis","G","Put X X/X green Ooze creature tokens onto the battlefield.","XXG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Reaper","3","3","B","Whenever Phyrexian Reaper becomes blocked by a green creature, destroy that creature. It can't be regenerated.","4B","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Misery Charm","B","Choose one - Destroy target Cleric; or return target Cleric card from your graveyard to your hand; or target player loses 2 life.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trinket Mage","2","2","U","When Trinket Mage enters the battlefield, you may search your library for an artifact card with converted mana cost 1 or less, reveal that card, and put it into your hand. If you do, shuffle your library.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Xira Arien","1","2","BRG","Flying
{B}{R}{G}, {T}: Target player draws a card.","BRG","Legendary Creature - Insect Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vapor Snare","U","Enchant creature
You control enchanted creature.
At the beginning of your upkeep, sacrifice Vapor Snare unless you return a land you control to its owner's hand.","4U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whispers of the Muse","U","Buyback {5} (You may pay an additional {5} as you cast this spell. If you do, put this card into your hand as it resolves.)
Draw a card.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Perplex","UB","Counter target spell unless its controller discards his or her hand.
Transmute {1}{U}{B} ({1}{U}{B}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","1UB","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Candelabra of Tawnos","{X}, {T}: Untap X target lands.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Essence","0","4","W","Defender (This creature can't attack.)
Whenever Wall of Essence is dealt combat damage, you gain that much life.","1W","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mnemonic Wall","0","4","U","Defender
When Mnemonic Wall enters the battlefield, you may return target instant or sorcery card from your graveyard to your hand.","4U","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corpse Harvester","3","3","B","{1}{B}, {T}, Sacrifice a creature: Search your library for a Zombie card and a Swamp card, reveal them, and put them into your hand. Then shuffle your library.","3BB","Creature - Zombie Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Onulet","2","2","When Onulet dies, you gain 2 life.","3","Artifact Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crazed Firecat","4","4","R","When Crazed Firecat enters the battlefield, flip a coin until you lose a flip. Put a +1/+1 counter on Crazed Firecat for each flip you win.","5RR","Creature - Elemental Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Joven's Tools","{4}, {T}: Target creature can't be blocked this turn except by Walls.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goblin Lore","R","Draw four cards, then discard three cards at random.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Molten Ravager","0","4","R","{R}: Molten Ravager gets +1/+0 until end of turn.","2R","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hurr Jackal","1","1","R","{T}: Target creature can't be regenerated this turn.","R","Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Skred","R","Skred deals damage to target creature equal to the number of snow permanents you control.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Village Cannibals","2","2","B","Whenever another Human creature dies, put a +1/+1 counter on Village Cannibals.","2B","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Liliana's Caress","B","Whenever an opponent discards a card, that player loses 2 life.","1B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strip Bare","W","Destroy all Auras and Equipment attached to target creature.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Legionnaire","2","2","WR","{R}, Sacrifice Goblin Legionnaire: Goblin Legionnaire deals 2 damage to target creature or player.
{W}, Sacrifice Goblin Legionnaire: Prevent the next 2 damage that would be dealt to target creature or player this turn.","RW","Creature - Goblin Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thorn of Amethyst","Noncreature spells cost {1} more to cast.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opt","U","Look at the top card of your library. You may put that card on the bottom of your library.
Draw a card.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Two-Headed Giant of Foriys","4","4","R","Trample
Two-Headed Giant of Foriys can block an additional creature.","4R","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Broodmate Dragon","4","4","BRG","Flying
When Broodmate Dragon enters the battlefield, put a 4/4 red Dragon creature token with flying onto the battlefield.","3BRG","Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Malachite Talisman","Whenever a player casts a green spell, you may pay {3}. If you do, untap target permanent.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloud Sprite","1","1","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
Cloud Sprite can block only creatures with flying.","U","Creature - Faerie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Furor of the Bitten","R","Enchant creature
Enchanted creature gets +2/+2 and attacks each turn if able.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shattering Pulse","R","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Destroy target artifact.","1R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Talisman of Progress","{T}: Add {1} to your mana pool.
{T}: Add {W} or {U} to your mana pool. Talisman of Progress deals 1 damage to you.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hammerhead Shark","2","3","U","Hammerhead Shark can't attack unless defending player controls an Island.","1U","Creature - Fish");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kaervek's Torch","R","As long as Kaervek's Torch is on the stack, spells that target it cost {2} more to cast.
Kaervek's Torch deals X damage to target creature or player.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightmare","*","*","B","Flying
Nightmare's power and toughness are each equal to the number of Swamps you control.","5B","Creature - Nightmare Horse");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bramblecrush","G","Destroy target noncreature permanent.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fungusaur","2","2","G","Whenever Fungusaur is dealt damage, put a +1/+1 counter on it.","3G","Creature - Fungus Lizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Turri Island","Creature spells cost {2} less to cast.
Whenever you roll {C}, reveal the top three cards of your library. Put all creature cards revealed this way into your hand and the rest into your graveyard.",NULL,"Plane - Ir");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crushing Pain","R","Crushing Pain deals 6 damage to target creature that was dealt damage this turn.","1R","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Powerstone Minefield","WR","Whenever a creature attacks or blocks, Powerstone Minefield deals 2 damage to it.","2RW","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Shifting Wall","0","0","Defender (This creature can't attack.)
Shifting Wall enters the battlefield with X +1/+1 counters on it.","X","Artifact Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psionic Blast","U","Psionic Blast deals 4 damage to target creature or player and 2 damage to you.","2U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reconstruction","U","Return target artifact card from your graveyard to your hand.","U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devoted Druid","0","2","G","{T}: Add {G} to your mana pool.
Put a -1/-1 counter on Devoted Druid: Untap Devoted Druid.","1G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Incremental Blight","B","Put a -1/-1 counter on target creature, two -1/-1 counters on another target creature, and three -1/-1 counters on a third target creature.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Agent of Stromgald","1","1","R","{R}: Add {B} to your mana pool.","R","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of False Hope","1","1","W","Sacrifice Kami of False Hope: Prevent all combat damage that would be dealt this turn.","W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhang Fei, Fierce Warrior","4","4","W","Vigilance; horsemanship (This creature can't be blocked except by creatures with horsemanship.)","4WW","Legendary Creature - Human Soldier Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kry Shield","{2}, {T}: Prevent all damage that would be dealt this turn by target creature you control. That creature gets +0/+X until end of turn, where X is its converted mana cost.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cabal Coffers","{2}, {T}: Add {B} to your mana pool for each Swamp you control.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Errant Minion","U","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, that player may pay any amount of mana. Errant Minion deals 2 damage to that player. Prevent X of that damage, where X is the amount of mana that player paid this way.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deep-Sea Kraken","6","6","U","Deep-Sea Kraken is unblockable.
Suspend 9-{2}{U} (Rather than cast this card from your hand, you may pay {2}{U} and exile it with nine time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Whenever an opponent casts a spell, if Deep-Sea Kraken is suspended, remove a time counter from it.","7UUU","Creature - Kraken");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("City of Shadows","{T}, Exile a creature you control: Put a storage counter on City of Shadows.
{T}: Add {X} to your mana pool, where X is the number of storage counters on City of Shadows.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phantasmal Bear","2","2","U","When Phantasmal Bear becomes the target of a spell or ability, sacrifice it.","U","Creature - Bear Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Urborg Elf","1","1","G","{T}: Add {G}, {U}, or {B} to your mana pool.","1G","Creature - Elf Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Your Puny Minds Cannot Fathom","When you set this scheme in motion, draw four cards. You have no maximum hand size until your next turn.",NULL,"Scheme");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Midsummer Revel","G","At the beginning of your upkeep, you may put a verse counter on Midsummer Revel.
{G}, Sacrifice Midsummer Revel: Put X 3/3 green Beast creature tokens onto the battlefield, where X is the number of verse counters on Midsummer Revel.","3GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Decomposition","G","Enchant black creature
Enchanted creature has ""Cumulative upkeep-Pay 1 life."" (At the beginning of its controller's upkeep, that player puts an age counter on it, then sacrifices it unless he or she pays its upkeep cost for each age counter on it.)
When enchanted creature dies, its controller loses 2 life.","1G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kamahl's Sledge","R","Kamahl's Sledge deals 4 damage to target creature.
Threshold - If seven or more cards are in your graveyard, instead Kamahl's Sledge deals 4 damage to that creature and 4 damage to that creature's controller.","5RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cheap Ass","1","3{1/2}","W","Spells you play cost {o{1/2}} less to play.","1W","Creature - Donkey Townsfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Martyr of Sands","1","1","W","{1}, Reveal X white cards from your hand, Sacrifice Martyr of Sands: You gain three times X life.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dizzy Spell","U","Target creature gets -3/-0 until end of turn.
Transmute {1}{U}{U} ({1}{U}{U}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Perish the Thought","B","Target opponent reveals his or her hand. You choose a card from it. That player shuffles that card into his or her library.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fit of Rage","R","Target creature gets +3/+3 and gains first strike until end of turn.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lys Alana Huntmaster","3","3","G","Whenever you cast an Elf spell, you may put a 1/1 green Elf Warrior creature token onto the battlefield.","2GG","Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spirit Link","W","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Whenever enchanted creature deals damage, you gain that much life.","W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Juggernaut","5","3","Juggernaut attacks each turn if able.
Juggernaut can't be blocked by Walls.","4","Artifact Creature - Juggernaut");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ebonblade Reaper","1","1","B","Whenever Ebonblade Reaper attacks, you lose half your life, rounded up.
Whenever Ebonblade Reaper deals combat damage to a player, that player loses half his or her life, rounded up.
Morph {3}{B}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2B","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razorjaw Oni","4","5","B","Black creatures can't block.","3B","Creature - Demon Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ethereal Haze","W","Prevent all damage that would be dealt by creatures this turn.","W","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loamdragger Giant","7","6","RG",NULL,"4(R/G)(R/G)(R/G)","Creature - Giant Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Timberland Ruins","Timberland Ruins enters the battlefield tapped.
{T}: Add {G} to your mana pool.
{T}, Sacrifice Timberland Ruins: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emberwilde Augur","2","1","R","Sacrifice Emberwilde Augur: Emberwilde Augur deals 3 damage to target player. Activate this ability only during your upkeep.","1R","Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Abundance","G","If you would draw a card, you may instead choose land or nonland and reveal cards from the top of your library until you reveal a card of the chosen kind. Put that card into your hand and put all other cards revealed this way on the bottom of your library in any order.","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Tinkerer","1","2","R","{R}, {T}: Destroy target artifact. That artifact deals damage equal to its converted mana cost to Goblin Tinkerer.","1R","Creature - Goblin Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Empty City Ruse","W","Target opponent skips all combat phases of his or her next turn.","W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Martyr's Bond","W","Whenever Martyr's Bond or another nonland permanent you control is put into a graveyard from the battlefield, each opponent sacrifices a permanent that shares a card type with it.","4WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dark Confidant","2","1","B","At the beginning of your upkeep, reveal the top card of your library and put that card into your hand. You lose life equal to its converted mana cost.","1B","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elven Riders","3","3","G","Elven Riders can't be blocked except by Walls and/or creatures with flying.","3GG","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teferi's Honor Guard","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{U}{U}: Teferi's Honor Guard phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before you untap during your next untap step.)","2W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hagra Diabolist","3","2","B","Whenever Hagra Diabolist or another Ally enters the battlefield under your control, you may have target player lose life equal to the number of Allies you control.","4B","Creature - Ogre Shaman Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enshrined Memories","G","Reveal the top X cards of your library. Put all creature cards revealed this way into your hand and the rest on the bottom of your library in any order.","XG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura of Dominion","U","Enchant creature
{1}, Tap an untapped creature you control: Untap enchanted creature.","UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veteran Swordsmith","3","2","W","Other Soldier creatures you control get +1/+0.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seedguide Ash","4","4","G","When Seedguide Ash dies, you may search your library for up to three Forest cards and put them onto the battlefield tapped. If you do, shuffle your library.","4G","Creature - Treefolk Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Sandstalker","4","2","R","Haste (This creature can attack the turn it comes under your control.)
At the beginning of the end step, return Viashino Sandstalker to its owner's hand. (Return it only if it's on the battlefield.)","1RR","Creature - Viashino Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kismet","W","Artifacts, creatures, and lands played by your opponents enter the battlefield tapped.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Memoricide","B","Name a nonland card. Search target player's graveyard, hand, and library for any number of cards with that name and exile them. Then that player shuffles his or her library.","3B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mox Lotus","{T}: Add {∞} to your mana pool.
{100}: Add one mana of any color to your mana pool.
You don't lose life due to mana burn.","15","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Still Life","G","{G}{G}: Still Life becomes a 4/3 Centaur creature until end of turn. It's still an enchantment.","1GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fresh Volunteers","2","2","W",NULL,"1W","Creature - Human Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mass Calcify","W","Destroy all nonwhite creatures.","5WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cryptic Annelid","1","4","U","When Cryptic Annelid enters the battlefield, scry 1, then scry 2, then scry 3. (To scry X, look at the top X cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","3U","Creature - Worm Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mossdog","1","1","G","Whenever Mossdog becomes the target of a spell or ability an opponent controls, put a +1/+1 counter on Mossdog.","G","Creature - Plant Hound");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Sorin, Lord of Innistrad","WB","+1: Put a 1/1 black Vampire creature token with lifelink onto the battlefield.
-2: You get an emblem with ""Creatures you control get +1/+0.""
-6: Destroy up to three target creatures and/or other planeswalkers. Return each card put into a graveyard this way to the battlefield under your control.","Planeswalker - Sorin","2WB");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian War Beast","3","4","When Phyrexian War Beast leaves the battlefield, sacrifice a land and Phyrexian War Beast deals 1 damage to you.","3","Artifact Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Twisted Image","U","Switch target creature's power and toughness until end of turn.
Draw a card.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nikko-Onna","2","2","W","When Nikko-Onna enters the battlefield, destroy target enchantment.
Whenever you cast a Spirit or Arcane spell, you may return Nikko-Onna to its owner's hand.","2W","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lost in the Woods","G","Whenever a creature attacks you or a planeswalker you control, reveal the top card of your library. If it's a Forest card, remove that creature from combat. Then put the revealed card on the bottom of your library.","3GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Predator's Gambit","B","Enchant creature
Enchanted creature gets +2/+1.
Enchanted creature has intimidate as long as its controller controls no other creatures. (It can't be blocked except by artifact creatures and/or creatures that share a color with it.)","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rotted Hystrix","3","6","G",NULL,"4G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Infernal Contract","B","Draw four cards. You lose half your life, rounded up.","BBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slobad, Goblin Tinkerer","1","2","R","Sacrifice an artifact: Target artifact is indestructible this turn. (""Destroy"" effects and lethal damage don't destroy that artifact.)","1R","Legendary Creature - Goblin Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dichotomancy","U","For each tapped nonland permanent target opponent controls, search that player's library for a card with the same name as that permanent and put it onto the battlefield under your control. Then that player shuffles his or her library.
Suspend 3-{1}{U}{U} (Rather than cast this card from your hand, you may pay {1}{U}{U} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","7UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spring of Eternal Peace","G","You gain 8 life.","3GG","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("All Is Dust","Each player sacrifices all colored permanents he or she controls.","7","Tribal Sorcery - Eldrazi");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overwhelming Forces","B","Destroy all creatures target opponent controls. Draw a card for each creature destroyed this way.","6BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Scavengers","3","1","B","Exile the top creature card of your graveyard: Regenerate Zombie Scavengers.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Masumaro, First to Live","*","*","G","Masumaro, First to Live's power and toughness are each equal to twice the number of cards in your hand.","3GGG","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Uncontrollable Anger","R","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature gets +2/+2 and attacks each turn if able.","2RR","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sacred Guide","1","1","W","{1}{W}, Sacrifice Sacred Guide: Reveal cards from the top of your library until you reveal a white card. Put that card into your hand and exile all other cards revealed this way.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hush","G","Destroy all enchantments.
Cycling {2} ({2}, Discard this card: Draw a card.)","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Favor of the Mighty","W","Each creature with the highest converted mana cost has protection from all colors.","1W","Tribal Enchantment - Giant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Horn of Plenty","Whenever a player casts a spell, he or she may pay {1}. If that player does, he or she draws a card at the beginning of the next end step.","6","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Archaeological Dig","{T}: Add {1} to your mana pool.
{T}, Sacrifice Archaeological Dig: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Voltaic Key","{1}, {T}: Untap target artifact.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lord Magnus","4","3","WG","First strike
Creatures with plainswalk can be blocked as though they didn't have plainswalk.
Creatures with forestwalk can be blocked as though they didn't have forestwalk.","3GWW","Legendary Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Charging Bandits","3","3","B","Whenever Charging Bandits attacks, it gets +2/+0 until end of turn.","4B","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloud Djinn","5","4","U","Flying
Cloud Djinn can block only creatures with flying.","5U","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Subterranean Shambler","2","3","R","Echo {3}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Subterranean Shambler enters the battlefield or leaves the battlefield, it deals 1 damage to each creature without flying.","3R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Enrage","R","Target creature gets +X/+0 until end of turn.","XR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghitu Slinger","2","2","R","Echo {2}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Ghitu Slinger enters the battlefield, it deals 2 damage to target creature or player.","2R","Creature - Human Nomad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Instigator Gang","2","3","R","Attacking creatures you control get +1/+0.
At the beginning of each upkeep, if no spells were cast last turn, transform Instigator Gang.","3R","Creature - Human Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Syphon Soul","B","Syphon Soul deals 2 damage to each other player. You gain life equal to the damage dealt this way.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodmark Mentor","1","1","R","Red creatures you control have first strike.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crookclaw Elder","3","2","U","Flying
Tap two untapped Birds you control: Draw a card.
Tap two untapped Wizards you control: Target creature gains flying until end of turn.","5U","Creature - Bird Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thornweald Archer","2","1","G","Reach (This creature can block creatures with flying.)
Deathtouch (Any amount of damage this deals to a creature is enough to destroy it.)","1G","Creature - Elf Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volcanic Submersion","R","Destroy target artifact or land.
Cycling {2} ({2}, Discard this card: Draw a card.)","4R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Disciple of Law","1","2","W","Protection from red
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sylvan Safekeeper","1","1","G","Sacrifice a land: Target creature you control gains shroud until end of turn. (It can't be the target of spells or abilities.)","G","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Permafrost Trap","U","If an opponent had a green creature enter the battlefield under his or her control this turn, you may pay {U} rather than pay Permafrost Trap's mana cost.
Tap up to two target creatures. Those creatures don't untap during their controller's next untap step.","2UU","Instant - Trap");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reality Twist","U","Cumulative upkeep {1}{U}{U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
If tapped for mana, Plains produce {R}, Swamps produce {G}, Mountains produce {W}, and Forests produce {B} instead of any other type.","UUU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sylvan Tutor","G","Search your library for a creature card and reveal that card. Shuffle your library, then put the card on top of it.","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Storm Crow","1","2","U","Flying (This creature can't be blocked except by creatures with flying or reach.)","1U","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyromancer Ascension","R","Whenever you cast an instant or sorcery spell that has the same name as a card in your graveyard, you may put a quest counter on Pyromancer Ascension.
Whenever you cast an instant or sorcery spell while Pyromancer Ascension has two or more quest counters on it, you may copy that spell. You may choose new targets for the copy.","1R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Samite Healer","1","1","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","1W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Forge Devil","1","1","R","When Forge Devil enters the battlefield, it deals 1 damage to target creature and 1 damage to you.","R","Creature - Devil");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brackwater Elemental","4","4","U","When Brackwater Elemental attacks or blocks, sacrifice it at the beginning of the next end step.
Unearth {2}{U} ({2}{U}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","2U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bound // Determined","BG","Sacrifice a creature. Return up to X cards from your graveyard to your hand, where X is the number of colors that creature was. Exile this card.
---
Other spells you control can't be countered by spells or abilities this turn.
Draw a card.","3BG","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("AEthertow","WU","Put target attacking or blocking creature on top of its owner's library.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose a new target for the copy.)","3(W/U)","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thrashing Wumpus","3","3","B","{B}: Thrashing Wumpus deals 1 damage to each creature and each player.","3BB","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tangle Angler","1","5","G","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
{G}: Target creature blocks Tangle Angler this turn if able.","3G","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Sprite","1","1","U","Flying, protection from red","1U","Creature - Faerie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wort, the Raidmother","3","3","RG","When Wort, the Raidmother enters the battlefield, put two 1/1 red and green Goblin Warrior creature tokens onto the battlefield.
Each red or green instant or sorcery spell you cast has conspire. (As you cast the spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose new targets for the copy.)","4(R/G)(R/G)","Legendary Creature - Goblin Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whirlpool Whelm","U","Clash with an opponent, then return target creature to its owner's hand. If you win, you may put that creature on top of its owner's library instead. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Winding Wurm","6","6","G","Echo {4}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","4G","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ghostly Flicker","U","Exile two target artifacts, creatures, and/or lands you control, then return those cards to the battlefield under your control.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krazy Kow","3","3","R","During your upkeep, roll a six-sided die. On a 1, sacrifice Krazy Kow and it deals 3 damage to each creature and player.","3R","Summon - Cow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dormant Sliver","2","2","UG","All Sliver creatures have defender.
All Slivers have ""When this permanent enters the battlefield, draw a card.""","2GU","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Quiet Disrepair","G","Enchant artifact or enchantment
At the beginning of your upkeep, choose one - Destroy enchanted permanent; or you gain 2 life.","1G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pristine Angel","4","4","W","Flying
As long as Pristine Angel is untapped, it has protection from artifacts and from all colors.
Whenever you cast a spell, you may untap Pristine Angel.","4WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Jovial Evil","B","Jovial Evil deals X damage to target opponent, where X is twice the number of white creatures that player controls.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Assquatch","3{1/2}","3{1/2}","R","Each other Donkey gets +1{1/2}/+1{1/2}.
Whenever another Donkey comes into play, untap target creature and gain control of it until end of turn. That creature gains haste until end of turn.","4R","Creature - Donkey Lord");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Firebug","2","2","R","When Goblin Firebug leaves the battlefield, sacrifice a land.","1R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Blast","W","Destroy target enchantment.
Draw a card.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fiery Gambit","R","Flip a coin until you lose a flip or choose to stop flipping. If you lose a flip, Fiery Gambit has no effect. If you win one or more flips, Fiery Gambit deals 3 damage to target creature. If you win two or more flips, Fiery Gambit deals 6 damage to each opponent. If you win three or more flips, draw nine cards and untap all lands you control.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Field of Reality","U","Enchant creature
Enchanted creature can't be blocked by Spirits.
{1}{U}: Return Field of Reality to its owner's hand.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Petravark","2","2","R","When Petravark enters the battlefield, exile target land.
When Petravark leaves the battlefield, return the exiled card to the battlefield under its owner's control.","3R","Creature - Nightmare Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Roc Egg","0","3","W","Defender (This creature can't attack.)
When Roc Egg dies, put a 3/3 white Bird creature token with flying onto the battlefield.","2W","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Line-Slinger","0","1","W","{T}: Tap target creature with power 3 or less.","1W","Creature - Kor Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stronghold Taskmaster","4","3","B","Other black creatures get -1/-1.","2BB","Creature - Giant Minion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Echoing Calm","W","Destroy target enchantment and all other enchantments with the same name as that enchantment.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Absorb","WU","Counter target spell. You gain 3 life.","WUU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kabira Crossroads","Kabira Crossroads enters the battlefield tapped.
When Kabira Crossroads enters the battlefield, you gain 2 life.
{T}: Add {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chant of Vitu-Ghazi","W","Convoke (Each creature you tap while casting this spell reduces its cost by {1} or by one mana of that creature's color.)
Prevent all damage that would be dealt by creatures this turn. You gain life equal to the damage prevented this way.","6WW","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Embrace My Diabolical Vision","When you set this scheme in motion, each player shuffles his or her hand and graveyard into his or her library. You draw seven cards, then each other player draws four cards.",NULL,"Scheme");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jeweled Bird","Remove Jeweled Bird from your deck before playing if you're not playing for ante.
{T}: Put Jeweled Bird into the ante. If you do, put all other cards you own from the ante into your graveyard, then draw a card.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Goblin Cannon","{2}: Goblin Cannon deals 1 damage to target creature or player. Sacrifice Goblin Cannon.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sickening Dreams","B","As an additional cost to cast Sickening Dreams, discard X cards.
Sickening Dreams deals X damage to each creature and each player.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Colos","2","2","R","Haste","2R","Creature - Goat Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Manor Skeleton","1","1","B","Haste
{1}{B}: Regenerate Manor Skeleton.","1B","Creature - Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dakmor Ghoul","2","2","B","When Dakmor Ghoul enters the battlefield, target opponent loses 2 life and you gain 2 life.","2BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hokori, Dust Drinker","2","2","W","Lands don't untap during their controllers' untap steps.
At the beginning of each player's upkeep, that player untaps a land he or she controls.","2WW","Legendary Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rolling Temblor","R","Rolling Temblor deals 2 damage to each creature without flying.
Flashback {4}{R}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Malicious Intent","R","Enchant creature
Enchanted creature has ""{T}: Target creature can't block this turn.""","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Overload","U","Enchant permanent
When Psychic Overload enters the battlefield, tap enchanted permanent.
Enchanted permanent doesn't untap during its controller's untap step.
Enchanted permanent has ""Discard two artifact cards: Untap this permanent.""","3U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyrider Trainee","3","3","W","Skyrider Trainee has flying as long as it's enchanted.","4W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Awe Strike","W","The next time target creature would deal damage this turn, prevent that damage. You gain life equal to the damage prevented this way.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rolling Spoil","G","Destroy target land. If {B} was spent to cast Rolling Spoil, all creatures get -1/-1 until end of turn.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lurking Informant","1","2","UB","({(u/b)} can be paid with either {U} or {B}.)
{2}, {T}: Look at the top card of target player's library. You may put that card into that player's graveyard.","1(U/B)","Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Otherworld Atlas","{T}: Put a charge counter on Otherworld Atlas.
{T}: Each player draws a card for each charge counter on Otherworld Atlas.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Venomous Vines","G","Destroy target enchanted permanent.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crossway Vampire","3","2","R","When Crossway Vampire enters the battlefield, target creature can't block this turn.","1RR","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Clearcutter","3","3","R","{T}, Sacrifice a Forest: Add three mana in any combination of {R} and/or {G} to your mana pool.","3R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Havengul Vampire","2","2","R","Whenever Havengul Vampire deals combat damage to a player, put a +1/+1 counter on it.
Whenever another creature dies, put a +1/+1 counter on Havengul Vampire.","3R","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slimy Kavu","2","2","R","{T}: Target land becomes a Swamp until end of turn.","2R","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ivory Cup","Whenever a player casts a white spell, you may pay {1}. If you do, you gain 1 life.","1","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Barrin","Sacrifice a permanent: Return target creature to its owner's hand.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Springjack Shepherd","1","2","W","Chroma - When Springjack Shepherd enters the battlefield, put a 0/1 white Goat creature token onto the battlefield for each white mana symbol in the mana costs of permanents you control.","3W","Creature - Kithkin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cateran Slaver","5","5","B","Swampwalk
{5}, {T}: Search your library for a Mercenary permanent card with converted mana cost 5 or less and put it onto the battlefield. Then shuffle your library.","4BB","Creature - Horror Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sadistic Hypnotist","2","2","B","Sacrifice a creature: Target player discards two cards. Activate this ability only any time you could cast a sorcery.","3BB","Creature - Human Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Willow Satyr","1","1","G","You may choose not to untap Willow Satyr during your untap step.
{T}: Gain control of target legendary creature for as long as you control Willow Satyr and Willow Satyr remains tapped.","2GG","Creature - Satyr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Panic","R","Cast Panic only during combat before blockers are declared.
Target creature can't block this turn.
Draw a card at the beginning of the next turn's upkeep.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Intet, the Dreamer","6","6","URG","Flying
Whenever Intet, the Dreamer deals combat damage to a player, you may pay {2}{U}. If you do, exile the top card of your library face down. You may look at that card for as long as it remains exiled. You may play that card without paying its mana cost for as long as Intet remains on the battlefield.","3URG","Legendary Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("New Benalia","New Benalia enters the battlefield tapped.
When New Benalia enters the battlefield, scry 1. (To scry 1, look at the top card of your library, then you may put that card on the bottom of your library.)
{T}: Add {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tawnos's Wand","{2}, {T}: Target creature with power 2 or less is unblockable this turn.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ravenous Trap","B","If an opponent had three or more cards put into his or her graveyard from anywhere this turn, you may pay {0} rather than pay Ravenous Trap's mana cost.
Exile all cards from target player's graveyard.","2BB","Instant - Trap");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("High Seas","U","Red creature spells and green creature spells cost {1} more to cast.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dawn of the Dead","B","At the beginning of your upkeep, you lose 1 life.
At the beginning of your upkeep, you may return target creature card from your graveyard to the battlefield. That creature gains haste until end of turn. Exile it at the beginning of the next end step.","2BBB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Careful Consideration","U","Target player draws four cards, then discards three cards. If you cast this spell during your main phase, instead that player draws four cards, then discards two cards.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Indomitable Archangel","4","4","W","Flying
Metalcraft - Artifacts you control have shroud as long as you control three or more artifacts.","2WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Shards","1","8","W","Defender, flying
Cumulative upkeep-An opponent gains 1 life. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","1W","Snow Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Screeching Buzzard","2","2","B","Flying
When Screeching Buzzard dies, each opponent discards a card.","3B","Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Implode","R","Destroy target land.
Draw a card.","4R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Diregraf Ghoul","2","2","B","Diregraf Ghoul enters the battlefield tapped.","B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plagued Rusalka","1","1","B","{B}, Sacrifice a creature: Target creature gets -1/-1 until end of turn.","B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squealing Devil","2","1","R","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
When Squealing Devil enters the battlefield, you may pay {X}. If you do, target creature gets +X/+0 until end of turn.
When Squealing Devil enters the battlefield, sacrifice it unless {B} was spent to cast it.","1R","Creature - Devil");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plow Through Reito","W","Sweep - Return any number of Plains you control to their owner's hand. Target creature gets +1/+1 until end of turn for each Plains returned this way.","1W","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sangrite Surge","RG","Target creature gets +3/+3 and gains double strike until end of turn.","4RG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Dryad","2","1","G","Forestwalk
When Yavimaya Dryad enters the battlefield, you may search your library for a Forest card and put it onto the battlefield tapped under target player's control. If you do, shuffle your library.","1GG","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chronatog","1","2","U","{0}: Chronatog gets +3/+3 until end of turn. You skip your next turn. Activate this ability only once each turn.","1U","Creature - Atog");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Calciform Pools","{T}: Add {1} to your mana pool.
{1}, {T}: Put a storage counter on Calciform Pools.
{1}, Remove X storage counters from Calciform Pools: Add X mana in any combination of {W} and/or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shizuko, Caller of Autumn","2","3","G","At the beginning of each player's upkeep, that player adds {G}{G}{G} to his or her mana pool. Until end of turn, this mana doesn't empty from that player's mana pool as steps and phases end.","1GG","Legendary Creature - Snake Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lowland Oaf","3","3","R","{T}: Target Goblin creature you control gets +1/+0 and gains flying until end of turn. Sacrifice that creature at the beginning of the next end step.","3R","Creature - Giant Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pit Imp","0","1","B","Flying
{B}: Pit Imp gets +1/+0 until end of turn. Activate this ability no more than twice each turn.","B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scourge Devil","3","3","R","When Scourge Devil enters the battlefield, creatures you control get +1/+0 until end of turn.
Unearth {2}{R} ({2}{R}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","4R","Creature - Devil");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zombie Infestation","B","Discard two cards: Put a 2/2 black Zombie creature token onto the battlefield.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Village Bell-Ringer","1","4","W","Flash (You may cast this spell any time you could cast an instant.)
When Village Bell-Ringer enters the battlefield, untap all creatures you control.","2W","Creature - Human Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Abolish","W","You may discard a Plains card rather than pay Abolish's mana cost.
Destroy target artifact or enchantment.","1WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Hellion","3","3","R","Haste
{0}: Flowstone Hellion gets +1/-1 until end of turn.","4R","Creature - Hellion Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forbidden Lore","G","Enchant land
Enchanted land has ""{T}: Target creature gets +2/+1 until end of turn.""","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Twilight Drover","1","1","W","Whenever a creature token leaves the battlefield, put a +1/+1 counter on Twilight Drover.
{2}{W}, Remove a +1/+1 counter from Twilight Drover: Put two 1/1 white Spirit creature tokens with flying onto the battlefield.","2W","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Centaur Garden","{T}: Add {G} to your mana pool. Centaur Garden deals 1 damage to you.
Threshold - {G}, {T}, Sacrifice Centaur Garden: Target creature gets +3/+3 until end of turn. Activate this ability only if seven or more cards are in your graveyard.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Runeboggle","U","Counter target spell unless its controller pays {1}.
Draw a card.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gavony Ironwright","1","4","W","Fateful hour - As long as you have 5 or less life, other creatures you control get +1/+4.","2W","Creature - Human Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Snake Basket","{X}, Sacrifice Snake Basket: Put X 1/1 green Snake creature tokens onto the battlefield. Activate this ability only any time you could cast a sorcery.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keldon Halberdier","4","1","R","First strike
Suspend 4-{R} (Rather than cast this card from your hand, you may pay {R} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","4R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Mishra's War Machine","5","5","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)
At the beginning of your upkeep, unless you discard a card, tap Mishra's War Machine and it deals 3 damage to you.","7","Artifact Creature - Juggernaut");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plaguebearer","1","1","B","{X}{X}{B}: Destroy target nonblack creature with converted mana cost X.","1B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shaleskin Bruiser","4","4","R","Trample
Whenever Shaleskin Bruiser attacks, it gets +3/+0 until end of turn for each other attacking Beast.","6R","Creature - Beast");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rock Lobster","4","3","Scissors Lizards cannot attack or block.","4","Artifact Creature");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Threads of Disloyalty","U","Enchant creature with converted mana cost 2 or less
You control enchanted creature.","1UU","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seal of Removal","U","Sacrifice Seal of Removal: Return target creature to its owner's hand.","U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Benalish Knight","2","2","W","Flash (You may cast this spell any time you could cast an instant.)
First strike (This creature deals combat damage before creatures without first strike.)","2W","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mirror-Mad Phantasm","5","1","U","Flying
{1}{U}: Mirror-Mad Phantasm's owner shuffles it into his or her library. If that player does, he or she reveals cards from the top of that library until a card named Mirror-Mad Phantasm is revealed. The player puts that card onto the battlefield and all other cards revealed this way into his or her graveyard.","3UU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin War Buggy","2","2","R","Haste
Echo {1}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Seastalkers","2","3","U","Islandwalk
{2}{U}: Tap target creature without flying.","3U","Creature - Merfolk Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dawntreader Elk","2","2","G","{G}, Sacrifice Dawntreader Elk: Search your library for a basic land card, put it onto the battlefield tapped, then shuffle your library.","1G","Creature - Elk");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hammerheim","{T}: Add {R} to your mana pool.
{T}: Target creature loses all landwalk abilities until end of turn.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seal of Strength","G","Sacrifice Seal of Strength: Target creature gets +3/+3 until end of turn.","G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frostwind Invoker","3","3","U","Flying
{8}: Creatures you control gain flying until end of turn.","4U","Creature - Merfolk Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scrubland",NULL,NULL,"Land - Plains Swamp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fire Tempest","R","Fire Tempest deals 6 damage to each creature and each player.","5RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Echoes","R","Whenever a creature enters the battlefield, you may add {X} to your mana pool, where X is the number of creatures you control that share a creature type with it.","2RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fists of the Anvil","R","Target creature gets +4/+0 until end of turn.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mournwhelk","3","3","B","When Mournwhelk enters the battlefield, target player discards two cards.
Evoke {3}{B} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","6B","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argothian Pixies","2","1","G","Argothian Pixies can't be blocked by artifact creatures.
Prevent all damage that would be dealt to Argothian Pixies by artifact creatures.","1G","Creature - Faerie");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Landshaper","1","1","{T}: Target land becomes an artifact in addition to its other types until end of turn.","3","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Checks and Balances","U","Whenever any spell is played, counter that spell if each player, other than the caster and his or her teammates, agrees to choose and discard a card. Those players must discard those cards after agreeing.
Checks and Balances may be played only in a game with three or more players.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Alaborn Cavalier","2","2","W","Whenever Alaborn Cavalier attacks, you may tap target creature.","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oran-Rief Survivalist","1","1","G","Whenever Oran-Rief Survivalist or another Ally enters the battlefield under your control, you may put a +1/+1 counter on Oran-Rief Survivalist.","1G","Creature - Human Warrior Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Twisted Abomination","5","3","B","{B}: Regenerate Twisted Abomination.
Swampcycling {2} ({2}, Discard this card: Search your library for a Swamp card, reveal it, and put it into your hand. Then shuffle your library.)","5B","Creature - Zombie Mutant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Street Savvy","G","Enchant creature
Enchanted creature gets +0/+2 and can block creatures with landwalk abilities as though they didn't have those abilities.","G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hot Springs","G","Enchant land you control
Enchanted land has ""{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.""","1G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Orochi Hatchery","Orochi Hatchery enters the battlefield with X charge counters on it.
{5}, {T}: Put a 1/1 green Snake creature token onto the battlefield for each charge counter on Orochi Hatchery.","XX","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rock Jockey","3","3","R","You can't cast Rock Jockey if you've played a land this turn.
You can't play lands if you've cast Rock Jockey this turn.","2R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Intervene","U","Counter target spell that targets a creature.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flusterstorm","U","Counter target instant or sorcery spell unless its controller pays {1}.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Freyalise's Winds","G","Whenever a permanent becomes tapped, put a wind counter on it.
If a permanent with a wind counter on it would untap during its controller's untap step, remove all wind counters from it instead.","2GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Well of Discovery","At the beginning of your end step, if you control no untapped lands, draw a card.","6","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Yavimaya Hollow","{T}: Add {1} to your mana pool.
{G}, {T}: Regenerate target creature.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foul Spirit","3","2","B","Flying
When Foul Spirit enters the battlefield, sacrifice a land.","2B","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Asceticism","G","Creatures you control have hexproof. (They can't be the targets of spells or abilities your opponents control.)
{1}{G}: Regenerate target creature.","3GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish House Party","*","*","G","Elvish House Party's power and toughness are each equal to the current hour, using the twelve-hour system.","4GG","Creature - Elf Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Manabond","G","At the beginning of your end step, you may reveal your hand and put all land cards from it onto the battlefield. If you do, discard your hand.","G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Golem-Skin Gauntlets","Equipped creature gets +1/+0 for each Equipment attached to it.
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","1","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Frostbeast","2","4","WG","At end of combat, destroy all creatures blocking or blocked by Kjeldoran Frostbeast.","3GW","Creature - Elemental Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sword of Light and Shadow","Equipped creature gets +2/+2 and has protection from white and from black.
Whenever equipped creature deals combat damage to a player, you gain 3 life and you may return up to one target creature card from your graveyard to your hand.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death Watch","B","Enchant creature
When enchanted creature dies, its controller loses life equal to its power and you gain life equal to its toughness.","B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("AEther Figment","1","1","U","Kicker {3} (You may pay an additional {3} as you cast this spell.)
Æther Figment is unblockable.
If Æther Figment was kicked, it enters the battlefield with two +1/+1 counters on it.","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slagstorm","R","Choose one - Slagstorm deals 3 damage to each creature; or Slagstorm deals 3 damage to each player.","1RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Asha's Favor","W","Enchant creature
Enchanted creature has flying, first strike, and vigilance.","2W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Elfhame Palace","Elfhame Palace enters the battlefield tapped.
{T}: Add {G} or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vedalken Shackles","You may choose not to untap Vedalken Shackles during your untap step.
{2}, {T}: Gain control of target creature with power less than or equal to the number of Islands you control for as long as Vedalken Shackles remains tapped.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Massacre Wurm","6","5","B","When Massacre Wurm enters the battlefield, creatures your opponents control get -2/-2 until end of turn.
Whenever a creature an opponent controls dies, that player loses 2 life.","3BBB","Creature - Wurm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tranquil Thicket","Tranquil Thicket enters the battlefield tapped.
{T}: Add {G} to your mana pool.
Cycling {G} ({G}, Discard this card: Draw a card.)",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Contamination","B","At the beginning of your upkeep, sacrifice Contamination unless you sacrifice a creature.
If a land is tapped for mana, it produces {B} instead of any other type and amount.","2B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planar Collapse","W","At the beginning of your upkeep, if there are four or more creatures on the battlefield, sacrifice Planar Collapse and destroy all creatures. They can't be regenerated.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Visions of Beyond","U","Draw a card. If a graveyard has twenty or more cards in it, draw three cards instead.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reduce to Dreams","U","Return all artifacts and enchantments to their owners' hands.","3UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Azorius Ploy","WU","Prevent all combat damage target creature would deal this turn.
Prevent all combat damage that would be dealt to target creature this turn.","1WWU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Earthquake","R","Earthquake deals X damage to each creature without flying and each player.","XR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blazing Specter","2","2","BR","Flying, haste
Whenever Blazing Specter deals combat damage to a player, that player discards a card.","2BR","Creature - Specter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ribbon Snake","2","3","U","Flying
{2}: Ribbon Snake loses flying until end of turn. Any player may activate this ability.","1UU","Creature - Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fyndhorn Pollen","G","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
All creatures get -1/-0.
{1}{G}: All creatures get -1/-0 until end of turn.","2G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Baku Altar","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Baku Altar.
{2}, {T}, Remove a ki counter from Baku Altar: Put a 1/1 colorless Spirit creature token onto the battlefield.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Martyrdom","W","Until end of turn, target creature you control gains ""{0}: The next 1 damage that would be dealt to target creature or player this turn is dealt to this creature instead."" Only you may activate this ability.","1WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pouncing Kavu","1","1","R","Kicker {2}{R} (You may pay an additional {2}{R} as you cast this spell.)
First strike
If Pouncing Kavu was kicked, it enters the battlefield with two +1/+1 counters on it and with haste.","1R","Creature - Kavu");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Meteor Crater","{T}: Choose a color of a permanent you control. Add one mana of that color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Crusader","2","1","W","Shadow (This creature can block or be blocked by only creatures with shadow.)
{1}{W}: Soltari Crusader gets +1/+0 until end of turn.","2W","Creature - Soltari Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pull from Eternity","W","Put target face-up exiled card into its owner's graveyard.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Stone","0","8","R","Defender (This creature can't attack.)","1RR","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Novijen Sages","0","0","U","Graft 4 (This creature enters the battlefield with four +1/+1 counters on it. Whenever another creature enters the battlefield, you may move a +1/+1 counter from this creature onto it.)
{1}, Remove two +1/+1 counters from among creatures you control: Draw a card.","4UU","Creature - Human Advisor Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Weaver","0","0","G","Spike Weaver enters the battlefield with three +1/+1 counters on it.
{2}, Remove a +1/+1 counter from Spike Weaver: Put a +1/+1 counter on target creature.
{1}, Remove a +1/+1 counter from Spike Weaver: Prevent all combat damage that would be dealt this turn.","2GG","Creature - Spike");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ancient Kavu","3","3","R","{2}: Ancient Kavu becomes colorless until end of turn.","3R","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tsabo Tavoc","7","4","BR","First strike, protection from legendary creatures
{B}{B}, {T}: Destroy target legendary creature. It can't be regenerated.","5BR","Legendary Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moriok Reaver","3","2","B",NULL,"2B","Creature - Human Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chaos Orb","{1}, {T}: If Chaos Orb is on the battlefield, flip Chaos Orb onto the battlefield from a height of at least one foot. If Chaos Orb turns over completely at least once during the flip, destroy all permanents it touches. Then destroy Chaos Orb.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Helvault","{1}, {T}: Exile target creature you control.
{7}, {T}: Exile target creature you don't control.
When Helvault is put into a graveyard from the battlefield, return all cards exiled with it to the battlefield under their owners' control.","3","Legendary Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thirst","U","Enchant creature
When Thirst enters the battlefield, tap enchanted creature.
Enchanted creature doesn't untap during its controller's untap step.
At the beginning of your upkeep, sacrifice Thirst unless you pay {U}.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Szadek, Lord of Secrets","5","5","UB","Flying
If Szadek, Lord of Secrets would deal combat damage to a player, instead put that many +1/+1 counters on Szadek and that player puts that many cards from the top of his or her library into his or her graveyard.","3UUBB","Legendary Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gwendlyn Di Corci","3","5","UBR","{T}: Target player discards a card at random. Activate this ability only during your turn.","UBBR","Legendary Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vivid Grove","Vivid Grove enters the battlefield tapped with two charge counters on it.
{T}: Add {G} to your mana pool.
{T}, Remove a charge counter from Vivid Grove: Add one mana of any color to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woodcloaker","3","3","G","Morph {2}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Woodcloaker is turned face up, target creature gains trample until end of turn.","5G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Taurean Mauler","2","2","R","Changeling (This card is every creature type at all times.)
Whenever an opponent casts a spell, you may put a +1/+1 counter on Taurean Mauler.","2R","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Savage Twister","RG","Savage Twister deals X damage to each creature.","XRG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Total War","R","Whenever a player attacks with one or more creatures, destroy all untapped non-Wall creatures that player controls that didn't attack, except for creatures the player hasn't controlled continuously since the beginning of the turn.","3R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Paperfin Rascal","2","2","U","When Paperfin Rascal enters the battlefield, clash with an opponent. If you win, put a +1/+1 counter on Paperfin Rascal. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","2U","Creature - Merfolk Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Doom Blade","B","Destroy target nonblack creature.","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alluring Scent","G","All creatures able to block target creature this turn do so.","1GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Healer","1","2","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn. If that creature is green, prevent the next 2 damage instead.","2W","Creature - Elf Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Descendant of Soramaro","2","3","U","{1}{U}: Look at the top X cards of your library, where X is the number of cards in your hand, then put them back in any order.","3U","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Helm of Awakening","Spells cost {1} less to cast.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cloud Cover","WU","Whenever another permanent you control becomes the target of a spell or ability an opponent controls, you may return that permanent to its owner's hand.","2WU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slith Strider","1","1","U","Whenever Slith Strider becomes blocked, draw a card.
Whenever Slith Strider deals combat damage to a player, put a +1/+1 counter on it.","1UU","Creature - Slith");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Creakwood Liege","2","2","BG","Other black creatures you control get +1/+1.
Other green creatures you control get +1/+1.
At the beginning of your upkeep, you may put a 1/1 black and green Worm creature token onto the battlefield.","1(B/G)(B/G)(B/G)","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drogskol Reaver","3","5","WU","Flying, double strike, lifelink
Whenever you gain life, draw a card.","5WU","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Scorcher","1","1","R","Sacrifice Dwarven Scorcher: Dwarven Scorcher deals 1 damage to target creature unless that creature's controller has Dwarven Scorcher deal 2 damage to him or her.","R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tribal Flames","R","Domain - Tribal Flames deals X damage to target creature or player, where X is the number of basic land types among lands you control.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silverback Ape","5","5","G",NULL,"3GG","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Holy Justiciar","2","1","W","{2}{W}, {T}: Tap target creature. If that creature is a Zombie, exile it.","3W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("River Boa","2","1","G","Islandwalk
{G}: Regenerate River Boa.","1G","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vectis Agents","4","3","UB","{U}{B}: Vectis Agents gets -2/-0 until end of turn and is unblockable this turn.","3UB","Artifact Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Vector Asp","1","1","{B}: Vector Asp gains infect until end of turn. (It deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)","1","Artifact Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hollowborn Barghest","7","6","B","At the beginning of your upkeep, if you have no cards in hand, each opponent loses 2 life.
At the beginning of each opponent's upkeep, if that player has no cards in hand, he or she loses 2 life.","5BB","Creature - Demon Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lava Flow","R","Destroy target creature or land.","3RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunters' Feast","G","Any number of target players each gain 6 life.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Robe of Mirrors","U","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature has shroud. (It can't be the target of spells or abilities.)","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathless Angel","5","7","W","Flying
{W}{W}: Target creature is indestructible this turn.","4WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("To Arms!","W","Untap all creatures you control.
Draw a card.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stronghold Gambit","R","Each player chooses a card in his or her hand. Then each player reveals his or her chosen card. The owner of each creature card revealed this way with the lowest converted mana cost puts it onto the battlefield.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Puncture Bolt","R","Puncture Bolt deals 1 damage to target creature. Put a -1/-1 counter on that creature.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zuran Spellcaster","1","1","U","{T}: Zuran Spellcaster deals 1 damage to target creature or player.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moltensteel Dragon","4","4","R","({(r/p)} can be paid with either {R} or 2 life.)
Flying
{(r/p)}: Moltensteel Dragon gets +1/+0 until end of turn.","4(R/P)(R/P)","Artifact Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Fallen","3","5","B","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Whenever Balduvian Fallen's cumulative upkeep is paid, it gets +1/+0 until end of turn for each {B} or {R} spent this way.","3B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loyal Retainers","1","1","W","Sacrifice Loyal Retainers: Return target legendary creature card from your graveyard to the battlefield. Activate this ability only during your turn, before attackers are declared.","2W","Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goldmeadow Dodger","1","1","W","Goldmeadow Dodger can't be blocked by creatures with power 4 or greater.","W","Creature - Kithkin Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Witch-Maw Nephilim","1","1","WUBG","Whenever you cast a spell, you may put two +1/+1 counters on Witch-Maw Nephilim.
Whenever Witch-Maw Nephilim attacks, it gains trample until end of turn if its power is 10 or greater.","GWUB","Creature - Nephilim");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Living Airship","2","3","U","Flying
{2}{G}: Regenerate Living Airship.","3U","Creature - Metathran");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fasting","W","At the beginning of your upkeep, put a hunger counter on Fasting. Then destroy Fasting if it has five or more hunger counters on it.
If you would begin your draw step, you may skip that step instead. If you do, you gain 2 life.
When you draw a card, destroy Fasting.","W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chimeric Idol","{0}: Tap all lands you control. Chimeric Idol becomes a 3/3 Turtle artifact creature until end of turn.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opportunity","U","Target player draws four cards.","4UU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Glasses of Urza","{T}: Look at target player's hand.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Might","U","Enchant land you control
Cumulative upkeep {1}{U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Enchanted land has ""{T}: Target creature gets +2/+2 until end of turn.""","U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tomb of Urami","{T}: Add {B} to your mana pool. Tomb of Urami deals 1 damage to you if you don't control an Ogre.
{2}{B}{B}, {T}, Sacrifice all lands you control: Put a legendary 5/5 black Demon Spirit creature token with flying named Urami onto the battlefield.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Joven","3","3","R","{R}{R}{R}, {T}: Destroy target noncreature artifact.","3RR","Legendary Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fiery Hellhound","2","2","R","{R}: Fiery Hellhound gets +1/+0 until end of turn.","1RR","Creature - Elemental Hound");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Tabernacle at Pendrell Vale","All creatures have ""At the beginning of your upkeep, destroy this creature unless you pay {1}.""",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call to Mind","U","Return target instant or sorcery card from your graveyard to your hand.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Into the North","G","Search your library for a snow land card and put it onto the battlefield tapped. Then shuffle your library.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyward Eye Prophets","3","3","WUG","Vigilance
{T}: Reveal the top card of your library. If it's a land card, put it onto the battlefield. Otherwise, put it into your hand.","3GWU","Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wirewood Lodge","{T}: Add {1} to your mana pool.
{G}, {T}: Untap target Elf.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Handcuffs","B","Target player keeps both hands in contact with each other. If he or she does not, sacrifice Handcuffs and that player sacrifices three cards in play.","3BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timid Drake","3","3","U","Flying
When another creature enters the battlefield, return Timid Drake to its owner's hand.","2U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Despise","B","Target opponent reveals his or her hand. You choose a creature or planeswalker card from it. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fleetfoot Panther","3","4","WG","Flash
When Fleetfoot Panther enters the battlefield, return a green or white creature you control to its owner's hand.","1GW","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sky Swallower","8","8","U","Flying
When Sky Swallower enters the battlefield, target opponent gains control of all other permanents you control.","3UU","Creature - Leviathan");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Liliana of the Veil","B","+1: Each player discards a card.
-2: Target player sacrifices a creature.
-6: Separate all permanents target player controls into two piles. That player sacrifices all permanents in the pile of his or her choice.","Planeswalker - Liliana","1BB");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tsabo's Web","When Tsabo's Web enters the battlefield, draw a card.
Each land with an activated ability that isn't a mana ability doesn't untap during its controller's untap step.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tundra",NULL,NULL,"Land - Plains Island");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ancestral Vision","Suspend 4-{U} (Rather than cast this card from your hand, pay {U} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)
Target player draws three cards.",NULL,"Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Exotic Orchard","{T}: Add to your mana pool one mana of any color that a land an opponent controls could produce.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Beguiler of Wills","1","1","U","{T}: Gain control of target creature with power less than or equal to the number of creatures you control.","3UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Inkfathom Divers","3","3","U","Islandwalk
When Inkfathom Divers enters the battlefield, look at the top four cards of your library, then put them back in any order.","3UU","Creature - Merfolk Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhalfirin Crusader","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{1}{W}: The next 1 damage that would be dealt to Zhalfirin Crusader this turn is dealt to target creature or player instead.","1WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Mystic","4","5","W","{1}: Attach target Aura attached to a creature to another creature.","3W","Legendary Creature - Fox Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Den-Guard","1","3","W","As long as Leonin Den-Guard is equipped, it gets +1/+1 and has vigilance.","1W","Creature - Cat Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zirilan of the Claw","3","4","R","{1}{R}{R}, {T}: Search your library for a Dragon permanent card and put that card onto the battlefield. Then shuffle your library. That Dragon gains haste until end of turn. Exile it at the beginning of the next end step.","3RR","Legendary Creature - Viashino Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ivory Charm","W","Choose one - All creatures get -2/-0 until end of turn; or tap target creature; or prevent the next 1 damage that would be dealt to target creature or player this turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crovax, Ascendant Hero","4","4","W","Other white creatures get +1/+1.
Nonwhite creatures get -1/-1.
Pay 2 life: Return Crovax, Ascendant Hero to its owner's hand.","4WW","Legendary Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ondu Cleric","1","1","W","Whenever Ondu Cleric or another Ally enters the battlefield under your control, you may gain life equal to the number of Allies you control.","1W","Creature - Kor Cleric Ally");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spare from Evil","W","Creatures you control gain protection from non-Human creatures until end of turn.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken Engineer","1","1","U","{T}: Add two mana of any one color to your mana pool. Spend this mana only to cast artifact spells or activate abilities of artifacts.","1U","Creature - Vedalken Artificer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Windborne Charge","W","Two target creatures you control each get +2/+2 and gain flying until end of turn.","2WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timber Wolves","1","1","G","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","G","Creature - Wolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forked Bolt","R","Forked Bolt deals 2 damage divided as you choose among one or two target creatures and/or players.","R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Force Void","U","Counter target spell unless its controller pays {1}.
Draw a card at the beginning of the next turn's upkeep.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Selhoff Occultist","2","3","U","Whenever Selhoff Occultist or another creature dies, target player puts the top card of his or her library into his or her graveyard.","2U","Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Glacial Chasm","Cumulative upkeep-Pay 2 life. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When Glacial Chasm enters the battlefield, sacrifice a land.
Creatures you control can't attack.
Prevent all damage that would be dealt to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krovikan Scoundrel","2","1","B",NULL,"1B","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skitter of Lizards","1","1","R","Multikicker {1}{R} (You may pay an additional {1}{R} any number of times as you cast this spell.)
Haste
Skitter of Lizards enters the battlefield with a +1/+1 counter on it for each time it was kicked.","R","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wormfang Manta","6","1","U","Flying
When Wormfang Manta enters the battlefield, you skip your next turn.
When Wormfang Manta leaves the battlefield, you take an extra turn after this one.","5UU","Creature - Nightmare Fish Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voice of Grace","2","2","W","Flying, protection from black","3W","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Moonbow Illusionist","2","1","U","Flying
{2}, Return a land you control to its owner's hand: Target land becomes the basic land type of your choice until end of turn.","2U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Everglove Courier","2","1","G","You may choose not to untap Everglove Courier during your untap step.
{2}{G}, {T}: Target Elf creature gets +2/+2 and has trample for as long as Everglove Courier remains tapped.","2G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyhunter Skirmisher","1","1","W","Flying, double strike (This creature can't be blocked except by creatures with flying or reach, and it deals both first-strike and regular combat damage.)","1WW","Creature - Cat Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sawtooth Ogre","3","3","R","Whenever Sawtooth Ogre blocks or becomes blocked by a creature, Sawtooth Ogre deals 1 damage to that creature at end of combat.","2RR","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Halberdier","3","1","R","First strike","3R","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Requiem Angel","5","5","W","Flying
Whenever another non-Spirit creature you control dies, put a 1/1 white Spirit creature token with flying onto the battlefield.","5W","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Knighthood","W","Creatures you control have first strike.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormfront Pegasus","2","1","W","Flying","1W","Creature - Pegasus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wild Mongrel","2","2","G","Discard a card: Wild Mongrel gets +1/+1 and becomes the color of your choice until end of turn.","1G","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leaf-Crowned Elder","3","5","G","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Leaf-Crowned Elder, you may reveal it. If you do, you may play that card without paying its mana cost.","2GG","Creature - Treefolk Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Ruin","B","Destroy all green creatures.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tainted Sigil","WB","{T}, Sacrifice Tainted Sigil: You gain life equal to the total life lost by all players this turn. (Damage causes loss of life.)","1WB","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Warrior's Oath","R","Take an extra turn after this one. At the beginning of that turn's end step, you lose the game.","RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Furrier","2","2","R","Prevent all damage that Goblin Furrier would deal to snow creatures.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silklash Spider","2","7","G","Reach (This creature can block creatures with flying.)
{X}{G}{G}: Silklash Spider deals X damage to each creature with flying.","3GG","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lim-Dûl's Paladin","0","3","BR","Trample
At the beginning of your upkeep, you may discard a card. If you don't, sacrifice Lim-Dûl's Paladin and draw a card.
Whenever Lim-Dûl's Paladin becomes blocked, it gets +6/+3 until end of turn.
Whenever Lim-Dûl's Paladin attacks and isn't blocked, it assigns no combat damage to defending player this turn and that player loses 4 life.","2BR","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Valakut Fireboar","1","7","R","Whenever Valakut Fireboar attacks, switch its power and toughness until end of turn.","4R","Creature - Elemental Boar");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Citadel of Pain","R","At the beginning of each player's end step, Citadel of Pain deals X damage to that player, where X is the number of untapped lands he or she controls.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Telim'Tor","2","2","R","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
Whenever Telim'Tor attacks, all attacking creatures with flanking get +1/+1 until end of turn.","4R","Legendary Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Energy Storm","W","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Prevent all damage that would be dealt by instant and sorcery spells.
Creatures with flying don't untap during their controllers' untap steps.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Yawgmoth's Agenda","B","You can't cast more than one spell each turn.
You may play cards from your graveyard.
If a card would be put into your graveyard from anywhere, exile it instead.","3BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jhessian Lookout","2","1","U",NULL,"1U","Creature - Human Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hinterland Hermit","2","1","R","At the beginning of each upkeep, if no spells were cast last turn, transform Hinterland Hermit.","1R","Creature - Human Werewolf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Troll-Horn Cameo","{T}: Add {R} or {G} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of the Mists","2","2","U","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
When Knight of the Mists enters the battlefield, you may pay {U}. If you don't, destroy target Knight and it can't be regenerated.","2U","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pallid Mycoderm","2","4","W","At the beginning of your upkeep, put a spore counter on Pallid Mycoderm.
Remove three spore counters from Pallid Mycoderm: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: Fungus and/or Saproling creatures you control get +1/+1 until end of turn.","3W","Creature - Fungus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Driver","1","1","B","When Phyrexian Driver enters the battlefield, other Mercenary creatures get +1/+1 until end of turn.","2B","Creature - Zombie Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Predator's Strike","G","Target creature gets +3/+3 and gains trample until end of turn.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stone Idol Trap","R","Stone Idol Trap costs {1} less to cast for each attacking creature.
Put a 6/12 colorless Construct artifact creature token with trample onto the battlefield. Exile it at the beginning of your next end step.","5R","Instant - Trap");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Steamclaw","{3}, {T}: Exile target card from a graveyard.
{1}, Sacrifice Steamclaw: Exile target card from a graveyard.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Roar of the Wurm","G","Put a 6/6 green Wurm creature token onto the battlefield.
Flashback {3}{G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","6G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Murkfiend Liege","4","4","UG","Other green creatures you control get +1/+1.
Other blue creatures you control get +1/+1.
Untap all green and/or blue creatures you control during each other player's untap step.","2(G/U)(G/U)(G/U)","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Vapor","0","1","U","Defender (This creature can't attack.)
Prevent all damage that would be dealt to Wall of Vapor by creatures it's blocking.","3U","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gloomhunter","2","1","B","Flying","2B","Creature - Bat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Caged Sun","As Caged Sun enters the battlefield, choose a color.
Creatures you control of the chosen color get +1/+1.
Whenever a land's ability adds one or more mana of the chosen color to your mana pool, add one additional mana of that color to your mana pool.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scornful AEther-Lich","2","4","U","{W}{B}: Scornful Æther-Lich gains fear and vigilance until end of turn. (Attacking doesn't cause it to tap, and it can't be blocked except by artifact creatures and/or black creatures.)","3U","Artifact Creature - Zombie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Braid of Fire","R","Cumulative upkeep-Add {R} to your mana pool. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","1R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loch Korrigan","1","1","B","{(u/b)}: Loch Korrigan gets +1/+1 until end of turn.","3B","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Cutthroat","3","4","B","Morph-Pay 5 life. (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","3BB","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grixis Sojourners","4","3","UBR","When you cycle Grixis Sojourners or it dies, you may exile target card from a graveyard.
Cycling {2}{B} ({2}{B}, Discard this card: Draw a card.)","1UBR","Creature - Zombie Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kumano's Pupils","3","3","R","If a creature dealt damage by Kumano's Pupils this turn would die, exile it instead.","4R","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("AEtherplasm","1","1","U","Whenever Ætherplasm blocks a creature, you may return Ætherplasm to its owner's hand. If you do, you may put a creature card from your hand onto the battlefield blocking that creature.","2UU","Creature - Illusion");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Wall of Spears","2","3","Defender (This creature can't attack.)
First strike","3","Artifact Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Springjack Knight","2","1","W","Whenever Springjack Knight attacks, clash with an opponent. If you win, target creature gains double strike until end of turn. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","2W","Creature - Kithkin Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scroll of Griselbrand","{1}, Sacrifice Scroll of Griselbrand: Target opponent discards a card. If you control a Demon, that player loses 3 life.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Selfless Cathar","1","1","W","{1}{W}, Sacrifice Selfless Cathar: Creatures you control get +1/+1 until end of turn.","W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("White Ward","W","Enchant creature
Enchanted creature has protection from white. This effect doesn't remove White Ward.","W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Increasing Devotion","W","Put five 1/1 white Human creature tokens onto the battlefield. If Increasing Devotion was cast from a graveyard, put ten of those tokens onto the battlefield instead.
Flashback {7}{W}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","3WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iron-Barb Hellion","5","4","R","Haste
Iron-Barb Hellion can't block.","5R","Creature - Hellion Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wirefly Hive","{3}, {T}: Flip a coin. If you win the flip, put a 2/2 colorless Insect artifact creature token with flying named Wirefly onto the battlefield. If you lose the flip, destroy all permanents named Wirefly.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wake of Destruction","R","Destroy target land and all other lands with the same name as that land.","3RRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balefire Liege","2","4","WR","Other red creatures you control get +1/+1.
Other white creatures you control get +1/+1.
Whenever you cast a red spell, Balefire Liege deals 3 damage to target player.
Whenever you cast a white spell, you gain 3 life.","2(R/W)(R/W)(R/W)","Creature - Spirit Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Superior Numbers","G","Superior Numbers deals damage to target creature equal to the number of creatures you control in excess of the number of creatures target opponent controls.","GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Final Judgment","W","Exile all creatures.","4WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orgg","6","6","R","Trample
Orgg can't attack if defending player controls an untapped creature with power 3 or greater.
Orgg can't block creatures with power 3 or greater.","3RR","Creature - Orgg");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tyrant of Discord","7","7","R","When Tyrant of Discord enters the battlefield, target opponent chooses a permanent he or she controls at random and sacrifices it. If a nonland permanent is sacrificed this way, repeat this process.","4RRR","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vorrac Battlehorns","Equipped creature has trample and can't be blocked by more than one creature.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Siege Mastodon","3","5","W",NULL,"4W","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iron Tusk Elephant","3","3","W","Trample","4W","Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zealous Conscripts","3","3","R","Haste
When Zealous Conscripts enters the battlefield, gain control of target permanent until end of turn. Untap that permanent. It gains haste until end of turn.","4R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Gremlins","1","1","B","You may choose not to untap Phyrexian Gremlins during your untap step.
{T}: Tap target artifact. It doesn't untap during its controller's untap step for as long as Phyrexian Gremlins remains tapped.","2B","Creature - Gremlin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thoughtcutter Agent","1","1","UB","{U}{B}, {T}: Target player loses 1 life and reveals his or her hand.","UB","Artifact Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Druid Lyrist","1","1","G","{G}, {T}, Sacrifice Druid Lyrist: Destroy target enchantment.","G","Creature - Human Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rain of Filth","B","Until end of turn, lands you control gain ""Sacrifice this land: Add {B} to your mana pool.""","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Royal Trooper","2","2","W","Whenever Royal Trooper blocks, it gets +2/+2 until end of turn.","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Living Tsunami","4","4","U","Flying
At the beginning of your upkeep, sacrifice Living Tsunami unless you return a land you control to its owner's hand.","2UU","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Force of Nature","8","8","G","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)
At the beginning of your upkeep, Force of Nature deals 8 damage to you unless you pay {G}{G}{G}{G}.","2GGGG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shoreline Salvager","3","3","B","Whenever Shoreline Salvager deals combat damage to a player, if you control an Island, you may draw a card.","3B","Creature - Surrakar");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Waterveil Cavern","{T}: Add {1} to your mana pool.
{T}: Add {U} or {B} to your mana pool. Waterveil Cavern doesn't untap during your next untap step.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Measure of Wickedness","B","At the beginning of your end step, sacrifice Measure of Wickedness and you lose 8 life.
Whenever another card is put into your graveyard from anywhere, target opponent gains control of Measure of Wickedness.","3B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wolf-Skull Shaman","2","2","G","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Wolf-Skull Shaman, you may reveal it. If you do, put a 2/2 green Wolf creature token onto the battlefield.","1G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bear Cub","2","2","G",NULL,"1G","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Embermage Goblin","1","1","R","When Embermage Goblin enters the battlefield, you may search your library for a card named Embermage Goblin, reveal it, and put it into your hand. If you do, shuffle your library.
{T}: Embermage Goblin deals 1 damage to target creature or player.","3R","Creature - Goblin Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sewerdreg","3","3","B","Swampwalk
Sacrifice Sewerdreg: Exile target card from a graveyard.","3BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thistledown Liege","1","3","WU","Flash
Other white creatures you control get +1/+1.
Other blue creatures you control get +1/+1.","1(W/U)(W/U)(W/U)","Creature - Kithkin Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thallid Germinator","2","2","G","At the beginning of your upkeep, put a spore counter on Thallid Germinator.
Remove three spore counters from Thallid Germinator: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: Target creature gets +1/+1 until end of turn.","2G","Creature - Fungus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Winter's Chill","U","Cast Winter's Chill only during combat before blockers are declared.
X can't be greater than the number of snow lands you control.
Choose X target attacking creatures. For each of those creatures, its controller may pay {1} or {2}. If that player doesn't, destroy that creature at end of combat. If that player pays only {1}, prevent all combat damage that would be dealt to and dealt by that creature this combat.","XU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undercity Shade","1","1","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
{B}: Undercity Shade gets +1/+1 until end of turn.","4B","Creature - Shade");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Misers' Cage","At the beginning of each opponent's upkeep, if that player has five or more cards in hand, Misers' Cage deals 2 damage to him or her.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Canopy Claws","G","Target creature loses flying until end of turn.
Flashback {G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sivvi's Valor","W","If you control a Plains, you may tap an untapped creature you control rather than pay Sivvi's Valor's mana cost.
All damage that would be dealt to target creature this turn is dealt to you instead.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rhystic Scrying","U","Draw three cards. Then, if any player pays {2}, discard three cards.","2UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shinen of Life's Roar","1","2","G","All creatures able to block Shinen of Life's Roar do so.
Channel - {2}{G}{G}, Discard Shinen of Life's Roar: All creatures able to block target creature this turn do so.","1G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carven Caryatid","2","5","G","Defender (This creature can't attack.)
When Carven Caryatid enters the battlefield, draw a card.","1GG","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tinker","U","As an additional cost to cast Tinker, sacrifice an artifact.
Search your library for an artifact card and put that card onto the battlefield. Then shuffle your library.","2U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Think Tank","U","At the beginning of your upkeep, look at the top card of your library. You may put that card into your graveyard.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bouncing Beebles","2","2","U","Bouncing Beebles is unblockable as long as defending player controls an artifact.","2U","Creature - Beeble");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aysen Crusader","2+*","2+*","W","Aysen Crusader's power and toughness are each equal to 2 plus the number of Soldiers and Warriors you control.","2WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infernal Spawn of Evil","7","7","B","Flying, first strike
o1oB, Reveal Infernal Spawn of Evil from your hand, Say ""It's coming"": Infernal Spawn of Evil deals 1 damage to target opponent. Use this ability only during your upkeep and only once each upkeep.","6BBB","Summon - Demon Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dangerous Wager","R","Discard your hand, then draw two cards.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Visionary","2","2","W","Shadow (This creature can block or be blocked by only creatures with shadow.)
Whenever Soltari Visionary deals damage to a player, destroy target enchantment that player controls.","1WW","Creature - Soltari Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Angel's Tomb","Whenever a creature enters the battlefield under your control, you may have Angel's Tomb become a 3/3 white Angel artifact creature with flying until end of turn.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ebony Owl Netsuke","At the beginning of each opponent's upkeep, if that player has seven or more cards in hand, Ebony Owl Netsuke deals 4 damage to him or her.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hound of Griselbrand","2","2","R","Double strike
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","2RR","Creature - Elemental Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cream of the Crop","G","Whenever a creature enters the battlefield under your control, you may look at the top X cards of your library, where X is that creature's power. If you do, put one of those cards on top of your library and the rest on the bottom of your library in any order.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anurid Barkripper","2","2","G","Threshold - Anurid Barkripper gets +2/+2 as long as seven or more cards are in your graveyard.","1GG","Creature - Frog Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pulverize","R","You may sacrifice two Mountains rather than pay Pulverize's mana cost.
Destroy all artifacts.","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Geralf's Mindcrusher","5","5","U","When Geralf's Mindcrusher enters the battlefield, target player puts the top five cards of his or her library into his or her graveyard.
Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)","4UU","Creature - Zombie Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Volcanist","3","1","R","Morph-Sacrifice two Mountains. (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Skirk Volcanist is turned face up, it deals 3 damage divided as you choose among one, two, or three target creatures.","3R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Double Negative","UR","Counter up to two target spells.","UUR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Golgari Guildmage","2","2","BG","{4}{B}, Sacrifice a creature: Return target creature card from your graveyard to your hand.
{4}{G}: Put a +1/+1 counter on target creature.","(B/G)(B/G)","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deftblade Elite","1","1","W","Provoke (When this attacks, you may have target creature defending player controls untap and block it if able.)
{1}{W}: Prevent all combat damage that would be dealt to and dealt by Deftblade Elite this turn.","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Axelrod Gunnarson","5","5","BR","Trample
Whenever a creature dealt damage by Axelrod Gunnarson this turn dies, you gain 1 life and Axelrod deals 1 damage to target player.","4BBRR","Legendary Creature - Giant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jar of Eyeballs","Whenever a creature you control dies, put two eyeball counters on Jar of Eyeballs.
{3}, {T}, Remove all eyeball counters from Jar of Eyeballs: Look at the top X cards of your library, where X is the number of eyeball counters removed this way. Put one of them into your hand and the rest on the bottom of your library in any order.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blaster Mage","2","2","R","{R}, {T}, Discard a card: Destroy target Wall.","2R","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flameblast Dragon","5","5","R","Flying
Whenever Flameblast Dragon attacks, you may pay {X}{R}. If you do, Flameblast Dragon deals X damage to target creature or player.","4RR","Creature - Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("General's Regalia","{3}: The next time a source of your choice would deal damage to you this turn, that damage is dealt to target creature you control instead.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slaughter Cry","R","Target creature gets +3/+0 and gains first strike until end of turn. (It deals combat damage before creatures without first strike.)","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trenching Steed","2","3","W","Sacrifice a land: Trenching Steed gets +0/+3 until end of turn.","3W","Creature - Horse Rebel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ixidor, Reality Sculptor","3","4","U","Face-down creatures get +1/+1.
{2}{U}: Turn target face-down creature face up.","3UU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Everbark Shaman","3","5","G","{T}, Exile a Treefolk card from your graveyard: Search your library for two Forest cards and put them onto the battlefield tapped. Then shuffle your library.","4G","Creature - Treefolk Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sawtooth Loon","2","2","WU","Flying
When Sawtooth Loon enters the battlefield, return a white or blue creature you control to its owner's hand.
When Sawtooth Loon enters the battlefield, draw two cards, then put two cards from your hand on the bottom of your library.","2WU","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Looter","2","1","U","{T}: Target player draws a card, then discards a card.","2U","Creature - Cephalid Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crystal Quarry","{T}: Add {1} to your mana pool.
{5}, {T}: Add {W}{U}{B}{R}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Essence Bottle","{3}, {T}: Put an elixir counter on Essence Bottle.
{T}, Remove all elixir counters from Essence Bottle: You gain 2 life for each elixir counter removed this way.","2","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nightmare Lash","Equipped creature gets +1/+1 for each Swamp you control.
Equip-Pay 3 life. (Pay 3 life: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","4","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Petals of Insight","U","Look at the top three cards of your library. You may put those cards on the bottom of your library in any order. If you do, return Petals of Insight to its owner's hand. Otherwise, draw three cards.","4U","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grave Pact","B","Whenever a creature you control dies, each other player sacrifices a creature.","1BBB","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Timberline Ridge","Timberline Ridge doesn't untap during your untap step if it has a depletion counter on it.
At the beginning of your upkeep, remove a depletion counter from Timberline Ridge.
{T}: Add {R} or {G} to your mana pool. Put a depletion counter on Timberline Ridge.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hellfire","B","Destroy all nonblack creatures. Hellfire deals X plus 3 damage to you, where X is the number of creatures that died this way.","2BBB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reverberate","R","Copy target instant or sorcery spell. You may choose new targets for the copy.","RR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Simplify","G","Each player sacrifices an enchantment.","G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mycologist","0","2","W","At the beginning of your upkeep, put a spore counter on Mycologist.
Remove three spore counters from Mycologist: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: You gain 2 life.","1W","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathmark Prelate","2","3","B","{2}{B}, {T}, Sacrifice a Zombie: Destroy target non-Zombie creature. It can't be regenerated. Activate this ability only any time you could cast a sorcery.","3B","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tower Gargoyle","4","4","WUB","Flying","1WUB","Artifact Creature - Gargoyle");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kaysa","2","3","G","Green creatures you control get +1/+1.","3GG","Legendary Creature - Elf Druid");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tarnished Citadel","{T}: Add {1} to your mana pool.
{T}: Add one mana of any color to your mana pool. Tarnished Citadel deals 3 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fiddlehead Kami","3","3","G","Whenever you cast a Spirit or Arcane spell, regenerate Fiddlehead Kami.","4G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sapphire Charm","U","Choose one - Target player draws a card at the beginning of the next turn's upkeep; or target creature gains flying until end of turn; or target creature an opponent controls phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before its controller untaps during his or her next untap step.)","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wind Spirit","3","2","U","Flying
Wind Spirit can't be blocked except by two or more creatures.","4U","Creature - Elemental Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Emeria, the Sky Ruin","Emeria, the Sky Ruin enters the battlefield tapped.
At the beginning of your upkeep, if you control seven or more Plains, you may return target creature card from your graveyard to the battlefield.
{T}: Add {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Smoldering Spires","Smoldering Spires enters the battlefield tapped.
When Smoldering Spires enters the battlefield, target creature can't block this turn.
{T}: Add {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistmoon Griffin","2","2","W","Flying
When Mistmoon Griffin dies, exile Mistmoon Griffin, then return the top creature card of your graveyard to the battlefield.","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blast from the Past","R","Madness {R}, cycling {1R}, kicker {2R}, flashback {3R}, buyback {4R}
Blast from the Past deals 2 damage to target creature or player.
If the kicker cost was paid, put a 1/1 red Goblin creature token into play.","2R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skullclamp","Equipped creature gets +1/-1.
Whenever equipped creature dies, draw two cards.
Equip {1}","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Call for Blood","B","As an additional cost to cast Call for Blood, sacrifice a creature.
Target creature gets -X/-X until end of turn, where X is the sacrificed creature's power.","4B","Instant - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Mountain Valley","Mountain Valley enters the battlefield tapped.
{T}, Sacrifice Mountain Valley: Search your library for a Mountain or Forest card and put it onto the battlefield. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Narcissism","G","{G}, Discard a card: Target creature gets +2/+2 until end of turn.
{G}, Sacrifice Narcissism: Target creature gets +2/+2 until end of turn.","2G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mimeofacture","U","Replicate {3}{U} (When you cast this spell, copy it for each time you paid its replicate cost. You may choose new targets for the copies.)
Choose target permanent an opponent controls. Search that player's library for a card with the same name and put it onto the battlefield under your control. Then that player shuffles his or her library.","3U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Armadillo Cloak","WG","Enchant creature
Enchanted creature gets +2/+2 and has trample.
Whenever enchanted creature deals damage, you gain that much life.","1GW","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lightning Rift","R","Whenever a player cycles a card, you may pay {1}. If you do, Lightning Rift deals 2 damage to target creature or player.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dispersal Shield","U","Counter target spell if its converted mana cost is less than or equal to the highest converted mana cost among permanents you control.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spark Fiend","5","6","R","When Spark Fiend comes into play, roll two six-sided dice. On a total of 2, 3, or 12, sacrifice Spark Fiend. On a total of 7 or 11, do not roll dice for Spark Fiend during any of your following upkeep phases. If you roll any other total, note it.
During your upkeep, roll two six-sided dice. On a total of 7, sacrifice Spark Fiend. If you roll the noted total, do not roll dice for Spark Fiend during any of your following upkeep phases. On any other roll, there is no effect.","4R","Summon - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knacksaw Clique","1","4","U","Flying
{1}{U}, {Q}: Target opponent exiles the top card of his or her library. Until end of turn, you may play that card. ({Q} is the untap symbol.)","3U","Creature - Faerie Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chosen of Markov","2","2","B","{T}, Tap an untapped Vampire you control: Transform Chosen of Markov.","2B","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Herd Gnarr","2","2","G","Whenever another creature enters the battlefield under your control, Herd Gnarr gets +2/+2 until end of turn.","3G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thermokarst","G","Destroy target land. If that land was a snow land, you gain 1 life.","1GG","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Riot Spikes","BR","({(b/r)} can be paid with either {B} or {R}.)
Enchant creature
Enchanted creature gets +2/-1.","(B/R)","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Harmonize","G","Draw three cards.","2GG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thorn-Thrash Viashino","2","2","R","Devour 2 (As this enters the battlefield, you may sacrifice any number of creatures. This creature enters the battlefield with twice that many +1/+1 counters on it.)
{G}: Thorn-Thrash Viashino gains trample until end of turn.","3R","Creature - Viashino Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tariel, Reckoner of Souls","4","7","WBR","Flying, vigilance
{T}: Choose a creature card at random from target opponent's graveyard. Put that card onto the battlefield under your control.","4WBR","Legendary Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flayer of the Hatebound","4","2","R","Undying (When this creature dies, if it had no +1/+1 counters on it, return it to the battlefield under its owner's control with a +1/+1 counter on it.)
Whenever Flayer of the Hatebound or another creature enters the battlefield from your graveyard, that creature deals damage equal to its power to target creature or player.","5R","Creature - Devil");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Vigilantes","2","2","R","Whenever Dwarven Vigilantes attacks and isn't blocked, you may have it deal damage equal to its power to target creature. If you do, Dwarven Vigilantes assigns no combat damage this turn.","2R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arcane Denial","U","Counter target spell. Its controller may draw up to two cards at the beginning of the next turn's upkeep.
You draw a card at the beginning of the next turn's upkeep.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soul Snuffers","3","3","B","When Soul Snuffers enters the battlefield, put a -1/-1 counter on each creature.","2BB","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Swirling Spriggan","3","3","G","{(g/u){(g/u)}: Target creature you control becomes the color or colors of your choice until end of turn.","2GG","Creature - Goblin Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cinderhaze Wretch","3","2","B","{T}: Target player discards a card. Activate this ability only during your turn.
Put a -1/-1 counter on Cinderhaze Wretch: Untap Cinderhaze Wretch.","4B","Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Telling Time","U","Look at the top three cards of your library. Put one of those cards into your hand, one on top of your library, and one on the bottom of your library.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tower of Coireall","{T}: Target creature can't be blocked by Walls this turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Junktroller","0","6","Defender (This creature can't attack.)
{T}: Put target card from a graveyard on the bottom of its owner's library.","4","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Library of Lat-Nam","U","An opponent chooses one - You draw three cards at the beginning of the next turn's upkeep; or you search your library for a card, put that card into your hand, then shuffle your library.","4U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scent of Brine","U","Reveal any number of blue cards in your hand. Counter target spell unless its controller pays {1} for each card revealed this way.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Absolute Law","W","All creatures have protection from red.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyrostatic Pillar","R","Whenever a player casts a spell with converted mana cost 3 or less, Pyrostatic Pillar deals 2 damage to that player.","1R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Arrow Volley Trap","W","If four or more creatures are attacking, you may pay {1}{W} rather than pay Arrow Volley Trap's mana cost.
Arrow Volley Trap deals 5 damage divided as you choose among any number of target attacking creatures.","3WW","Instant - Trap");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horobi, Death's Wail","4","4","B","Flying
Whenever a creature becomes the target of a spell or ability, destroy that creature.","2BB","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Timebender","1","1","U","Morph {U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Timebender is turned face up, choose one - Remove two time counters from target permanent or suspended card; or put two time counters on target permanent with a time counter on it or suspended card.","U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Goryo's Vengeance","B","Return target legendary creature card from your graveyard to the battlefield. That creature gains haste. Exile it at the beginning of the next end step.
Splice onto Arcane {2}{B} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","1B","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Muscle Burst","G","Target creature gets +X/+X until end of turn, where X is 3 plus the number of cards named Muscle Burst in all graveyards.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Game-Trail Changeling","4","4","G","Changeling (This card is every creature type at all times.)
Trample","3GG","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kavu Recluse","2","2","R","{T}: Target land becomes a Forest until end of turn.","2R","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Mesmerist","1","2","U","{U}, {T}: Target player puts the top two cards of his or her library into his or her graveyard.","1U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sex Appeal","W","Prevent up to 3 damage total to any number of creatures and/or players. If there are more players in the room of the opposite sex, prevent up to 3 additional damage total to any number of creatures and/or players.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Striker","1","1","R","First strike, haste","1R","Creature - Goblin Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seismic Spike","R","Destroy target land. Add {R}{R} to your mana pool.","2RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talon Sliver","1","1","W","All Sliver creatures have first strike.","1W","Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Living Destiny","G","As an additional cost to cast Living Destiny, reveal a creature card from your hand.
You gain life equal to the revealed card's converted mana cost.","3G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Overload","R","Kicker {2} (You may pay an additional {2} as you cast this spell.)
Destroy target artifact if its converted mana cost is 2 or less. If Overload was kicked, destroy that artifact if its converted mana cost is 5 or less instead.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viashino Slaughtermaster","1","1","R","Double strike
{B}{G}: Viashino Slaughtermaster gets +1/+1 until end of turn. Activate this ability only once each turn.","1R","Creature - Viashino Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deconstruct","G","Destroy target artifact. Add {G}{G}{G} to your mana pool.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viscid Lemures","4","3","B","{0}: Viscid Lemures gets -1/-0 and gains swampwalk until end of turn.","4B","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mystic Remora","U","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Whenever an opponent casts a noncreature spell, you may draw a card unless that player pays {4}.","U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crusade","W","White creatures get +1/+1.","WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spark Spray","R","Spark Spray deals 1 damage to target creature or player.
Cycling {R} ({R}, Discard this card: Draw a card.)","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shrine of Limitless Power","At the beginning of your upkeep or whenever you cast a black spell, put a charge counter on Shrine of Limitless Power.
{4}, {T}, Sacrifice Shrine of Limitless Power: Target player discards a card for each charge counter on Shrine of Limitless Power.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Culling Sun","WB","Destroy each creature with converted mana cost 3 or less.","2WWB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost Warden","1","1","W","{T}: Target creature gets +1/+1 until end of turn.","1W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tilling Treefolk","1","3","G","When Tilling Treefolk enters the battlefield, you may return up to two target land cards from your graveyard to your hand.","2G","Creature - Treefolk Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ranger en-Vec","2","2","WG","First strike
{G}: Regenerate Ranger en-Vec.","1GW","Creature - Human Soldier Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Instill Infection","B","Put a -1/-1 counter on target creature.
Draw a card.","3B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hellfire Mongrel","2","2","R","At the beginning of each opponent's upkeep, if that player has two or fewer cards in hand, Hellfire Mongrel deals 2 damage to him or her.","2R","Creature - Elemental Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Moan of the Unhallowed","B","Put two 2/2 black Zombie creature tokens onto the battlefield.
Flashback {5}{B}{B} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Patrician's Scorn","W","If you've cast another white spell this turn, you may cast Patrician's Scorn without paying its mana cost.
Destroy all enchantments.","3W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vanquish","W","Destroy target blocking creature.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Carrion Ants","0","1","B","{1}: Carrion Ants gets +1/+1 until end of turn.","2BB","Creature - Insect");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Phyrexian Walker","0","3",NULL,"0","Artifact Creature - Construct");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Toxic Stench","B","Target nonblack creature gets -1/-1 until end of turn.
Threshold - If seven or more cards are in your graveyard, instead destroy that creature. It can't be regenerated.","1B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonewright","1","1","R","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Stonewright is paired with another creature, each of those creatures has ""{R}: This creature gets +1/+0 until end of turn.""","R","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rofellos's Gift","G","Reveal any number of green cards in your hand. Return an enchantment card from your graveyard to your hand for each card revealed this way.","G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Needle Drop","R","Needle Drop deals 1 damage to target creature or player that was dealt damage this turn.
Draw a card.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Rites","R","{1}{R}, Sacrifice a creature: Blood Rites deals 2 damage to target creature or player.","3RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wings of Hope","WU","Enchant creature
Enchanted creature gets +1/+3 and has flying.","WU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Caustic Wasps","1","1","G","Flying
Whenever Caustic Wasps deals combat damage to a player, you may destroy target artifact that player controls.","2G","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vodalian Mage","1","1","U","{U}, {T}: Counter target spell unless its controller pays {1}.","2U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glyph of Doom","B","At end of combat, destroy all creatures that were blocked by target Wall this turn.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pardic Miner","1","1","R","Sacrifice Pardic Miner: Target player can't play lands this turn.","1R","Creature - Dwarf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horned Troll","2","2","G","{G}: Regenerate Horned Troll.","2G","Creature - Troll");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stern Judge","2","2","W","{T}: Each player loses 1 life for each Swamp he or she controls.","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Exhumer Thrull","3","3","B","Haunt (When this creature dies, exile it haunting target creature.)
When Exhumer Thrull enters the battlefield or the creature it haunts dies, return target creature card from your graveyard to your hand.","5B","Creature - Thrull");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trusted Forcemage","2","2","G","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Trusted Forcemage is paired with another creature, each of those creatures gets +1/+1.","2G","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Canopy Crawler","2","2","G","Amplify 1 (As this creature enters the battlefield, put a +1/+1 counter on it for each Beast card you reveal in your hand.)
{T}: Target creature gets +1/+1 until end of turn for each +1/+1 counter on Canopy Crawler.","3G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chill to the Bone","B","Destroy target nonsnow creature.","3B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Delay","U","Counter target spell. If the spell is countered this way, exile it with three time counters on it instead of putting it into its owner's graveyard. If it doesn't have suspend, it gains suspend. (At the beginning of its owner's upkeep, remove a counter from that card. When the last is removed, the player plays it without paying its mana cost. If it's a creature, it has haste.)","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pathbreaker Wurm","6","4","G","Soulbond (You may pair this creature with another unpaired creature when either enters the battlefield. They remain paired for as long as you control both of them.)
As long as Pathbreaker Wurm is paired with another creature, both creatures have trample.","4GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Centaur Chieftain","3","3","G","Haste
Threshold - As long as seven or more cards are in your graveyard, Centaur Chieftain has ""When Centaur Chieftain enters the battlefield, creatures you control get +1/+1 and gain trample until end of turn.""","3G","Creature - Centaur");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vigor Mortis","B","Return target creature card from your graveyard to the battlefield. If {G} was spent to cast Vigor Mortis, that creature enters the battlefield with an additional +1/+1 counter on it.","2BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ley Line","G","At the beginning of each player's upkeep, that player may put a +1/+1 counter on target creature of his or her choice.","3G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sliver Legion","7","7","WUBRG","All Sliver creatures get +1/+1 for each other Sliver on the battlefield.","WUBRG","Legendary Creature - Sliver");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Howl of the Night Pack","G","Put a 2/2 green Wolf creature token onto the battlefield for each Forest you control.","6G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kazuul, Tyrant of the Cliffs","5","4","R","Whenever a creature an opponent controls attacks, if you're the defending player, put a 3/3 red Ogre creature token onto the battlefield unless that creature's controller pays {3}.","3RR","Legendary Creature - Ogre Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thousand-Year Elixir","You may activate abilities of creatures you control as though those creatures had haste.
{1}, {T}: Untap target creature.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Every Hope Shall Vanish","When you set this scheme in motion, each opponent reveals his or her hand. Choose a nonland card from each of those hands. Those players discard those cards.",NULL,"Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Paladin of Prahv","3","4","W","Whenever Paladin of Prahv deals damage, you gain that much life.
Forecast - {1}{W}, Reveal Paladin of Prahv from your hand: Whenever target creature deals damage this turn, you gain that much life. (Activate this ability only during your upkeep and only once each turn.)","4WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Helix Pinnacle","G","Shroud
{X}: Put X tower counters on Helix Pinnacle.
At the beginning of your upkeep, if there are 100 or more tower counters on Helix Pinnacle, you win the game.","G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mageta's Boon","W","Flash
Enchant creature
Enchanted creature gets +1/+2.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sachi, Daughter of Seshiro","1","3","G","Other Snake creatures you control get +0/+1.
Shamans you control have ""{T}: Add {G}{G} to your mana pool.""","2GG","Legendary Creature - Snake Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pyre Charger","1","1","R","Haste
{R}: Pyre Charger gets +1/+0 until end of turn.","RR","Creature - Elemental Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Illusionary Forces","4","4","U","Flying
Cumulative upkeep {U} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","3U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lord of Shatterskull Pass","3","3","R","Level up {1}{R} ({1}{R}: Put a level counter on this. Level up only as a sorcery.)
LEVEL 1-5
6/6
LEVEL 6+
6/6
Whenever Lord of Shatterskull Pass attacks, it deals 6 damage to each creature defending player controls.","3R","Creature - Minotaur Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unnatural Selection","U","{1}: Choose a creature type other than Wall. Target creature becomes that type until end of turn.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tivadar's Crusade","W","Destroy all Goblins.","1WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Malakir Bloodwitch","4","4","B","Flying, protection from white
When Malakir Bloodwitch enters the battlefield, each opponent loses life equal to the number of Vampires you control. You gain life equal to the life lost this way.","3BB","Creature - Vampire Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Whispersilk Cloak","Equipped creature is unblockable.
Equipped creature has shroud. (It can't be the target of spells or abilities.)
Equip {2} ({2}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Riftstone Portal","{T}: Add {1} to your mana pool.
As long as Riftstone Portal is in your graveyard, lands you control have ""{T}: Add {G} or {W} to your mana pool.""",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Degavolver","1","1","W","Kicker {1}{B} and/or {R} (You may pay an additional {1}{B} and/or {R} as you cast this spell.)
If Degavolver was kicked with its {1}{B} kicker, it enters the battlefield with two +1/+1 counters on it and with ""Pay 3 life: Regenerate Degavolver.""
If Degavolver was kicked with its {R} kicker, it enters the battlefield with a +1/+1 counter on it and with first strike.","1W","Creature - Volver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gossamer Phantasm","2","1","U","Flying
When Gossamer Phantasm becomes the target of a spell or ability, sacrifice it.","1U","Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dance of Shadows","B","Creatures you control get +1/+0 and gain fear until end of turn. (They can't be blocked except by artifact creatures and/or black creatures.)","3BB","Sorcery - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Veteran's Reflexes","W","Target creature gets +1/+1 until end of turn. Untap that creature.","W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Tangle Hulk","5","3","{2}{G}: Regenerate Tangle Hulk.","5","Artifact Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodscale Prowler","3","1","R","Bloodthirst 1 (If an opponent was dealt damage this turn, this creature enters the battlefield with a +1/+1 counter on it.)","2R","Creature - Viashino Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian War-Makers","3","3","R","Haste
Rampage 1 (Whenever this creature becomes blocked, it gets +1/+1 until end of turn for each creature blocking it beyond the first.)","4R","Creature - Human Barbarian");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nim Grotesque","3","6","B","Nim Grotesque gets +1/+0 for each artifact you control.","6B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Novablast Wurm","7","7","WG","Whenever Novablast Wurm attacks, destroy all other creatures.","3GGWW","Creature - Wurm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seachrome Coast","Seachrome Coast enters the battlefield tapped unless you control two or fewer other lands.
{T}: Add {W} or {U} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Smogsteed Rider","2","2","B","Whenever Smogsteed Rider attacks, each other attacking creature gains fear until end of turn. (They can't be blocked except by artifact creatures and/or black creatures.)","2BB","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spire Monitor","3","3","U","Flash (You may cast this spell any time you could cast an instant.)
Flying","4U","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spectral Lynx","2","1","W","Protection from green
{B}: Regenerate Spectral Lynx.","1W","Creature - Cat Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Miren, the Moaning Well","{T}: Add {1} to your mana pool.
{3}, {T}, Sacrifice a creature: You gain life equal to the sacrificed creature's toughness.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gustcloak Harrier","2","2","W","Flying
Whenever Gustcloak Harrier becomes blocked, you may untap it and remove it from combat.","1WW","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Varchild's War-Riders","3","4","R","Cumulative upkeep-Put a 1/1 red Survivor creature token onto the battlefield under an opponent's control. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Trample; rampage 1 (Whenever this creature becomes blocked, it gets +1/+1 until end of turn for each creature blocking it beyond the first.)","1R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Souldrinker","2","2","B","Pay 3 life: Put a +1/+1 counter on Souldrinker.","3B","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dust Bowl","{T}: Add {1} to your mana pool.
{3}, {T}, Sacrifice a land: Destroy target nonbasic land.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stigma Lasher","2","2","R","Wither (This deals damage to creatures in the form of -1/-1 counters.)
Whenever Stigma Lasher deals damage to a player, that player can't gain life for the rest of the game.","RR","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Auriok Replica","2","2","{W}, Sacrifice Auriok Replica: Prevent all damage a source of your choice would deal to you this turn.","3","Artifact Creature - Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rhystic Deluge","U","{U}: Tap target creature unless its controller pays {1}.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flowstone Mauler","4","5","R","Trample
{R}: Flowstone Mauler gets +1/-1 until end of turn.","4RR","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Resounding Scream","B","Target player discards a card at random.
Cycling {5}{U}{B}{R} ({5}{U}{B}{R}, Discard this card: Draw a card.)
When you cycle Resounding Scream, target player discards two cards at random.","2B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whispering Shade","1","1","B","Swampwalk
{B}: Whispering Shade gets +1/+1 until end of turn.","3B","Creature - Shade");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Urza's Guilt","UB","Each player draws two cards, then discards three cards, then loses 4 life.","2UB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balloon Peddler","2","2","U","{U}, {T}, Discard a card: Target creature gains flying until end of turn.","2U","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sokenzan Spellblade","2","3","R","Bushido 1 (When this blocks or becomes blocked, it gets +1/+1 until end of turn.)
{1}{R}: Sokenzan Spellblade gets +X/+0 until end of turn, where X is the number of cards in your hand.","4R","Creature - Ogre Samurai Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dromar's Cavern","When Dromar's Cavern enters the battlefield, sacrifice it unless you return a non-Lair land you control to its owner's hand.
{T}: Add {W}, {U}, or {B} to your mana pool.",NULL,"Land - Lair");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Celestial Dawn","W","Lands you control are Plains.
Nonland cards you own that aren't on the battlefield, spells you control, and nonland permanents you control are white.
You may spend white mana as though it were mana of any color. You may spend other mana only as though it were colorless mana.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cruel Deceiver","2","1","B","{1}: Look at the top card of your library.
{2}: Reveal the top card of your library. If it's a land card, Cruel Deceiver gains ""Whenever Cruel Deceiver deals damage to a creature, destroy that creature"" until end of turn. Activate this ability only once each turn.","1B","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Conservator","{3}, {T}: Prevent the next 2 damage that would be dealt to you this turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trained Cheetah","2","2","G","Whenever Trained Cheetah becomes blocked, it gets +1/+1 until end of turn.","2G","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wei Strike Force","2","1","B","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","2B","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Puresteel Paladin","2","2","W","Whenever an Equipment enters the battlefield under your control, you may draw a card.
Metalcraft - Equipment you control have equip {0} as long as you control three or more artifacts.","WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zzzyxas's Abyss","B","At the beginning of your upkeep, destroy all nonland permanents with the first name alphabetically among nonland permanents in play.","1BB","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Adarkar Wastes","{T}: Add {1} to your mana pool.
{T}: Add {W} or {U} to your mana pool. Adarkar Wastes deals 1 damage to you.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krark-Clan Grunt","2","2","R","Sacrifice an artifact: Krark-Clan Grunt gets +1/+0 and gains first strike until end of turn.","2R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mistblade Shinobi","1","1","U","Ninjutsu {U} ({U}, Return an unblocked attacker you control to hand: Put this card onto the battlefield from your hand tapped and attacking.)
Whenever Mistblade Shinobi deals combat damage to a player, you may return target creature that player controls to its owner's hand.","2U","Creature - Human Ninja");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blacker Lotus","ocT: Tear Blacker Lotus into pieces. Add four mana of any one color to your mana pool. Play this ability as a mana source. Remove the pieces from the game afterwards.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Douse","U","{1}{U}: Counter target red spell.","2U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Obsidian Fireheart","4","4","R","{1}{R}{R}: Put a blaze counter on target land without a blaze counter on it. For as long as that land has a blaze counter on it, it has ""At the beginning of your upkeep, this land deals 1 damage to you."" (The land continues to burn after Obsidian Fireheart has left the battlefield.)","1RRR","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Force Spike","U","Counter target spell unless its controller pays {1}.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spreading Plague","B","Whenever a creature enters the battlefield, destroy all other creatures that share a color with it. They can't be regenerated.","4B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Samite Alchemist","0","2","W","{W}{W}, {T}: Prevent the next 4 damage that would be dealt this turn to target creature you control. Tap that creature. It doesn't untap during your next untap step.","3W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tariff","W","Each player sacrifices the creature he or she controls with the highest converted mana cost unless he or she pays that creature's mana cost. If two or more creatures a player controls are tied for highest cost, that player chooses one.","1W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anathemancer","2","2","BR","When Anathemancer enters the battlefield, it deals damage to target player equal to the number of nonbasic lands that player controls.
Unearth {5}{B}{R} ({5}{B}{R}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","1BR","Creature - Zombie Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Standing Stones","{1}, {T}, Pay 1 life: Add one mana of any color to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ebon Dragon","5","4","B","Flying
When Ebon Dragon enters the battlefield, you may have target opponent discard a card.","5BB","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crab Umbra","U","Enchant creature
{2}{U}: Untap enchanted creature.
Totem armor (If enchanted creature would be destroyed, instead remove all damage from it and destroy this Aura.)","U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harvester Druid","1","1","G","{T}: Add to your mana pool one mana of any color that a land you control could produce.","1G","Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master Apothecary","2","2","W","Tap an untapped Cleric you control: Prevent the next 2 damage that would be dealt to target creature or player this turn.","WWW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodcurdler","1","1","B","Flying
At the beginning of your upkeep, put the top card of your library into your graveyard.
Threshold - As long as seven or more cards are in your graveyard, Bloodcurdler gets +1/+1 and has ""At the beginning of your end step, exile two cards from your graveyard.""","1B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunrise Sovereign","5","5","R","Other Giant creatures you control get +2/+2 and have trample.","5R","Creature - Giant Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rending Vines","G","Destroy target artifact or enchantment if its converted mana cost is less than or equal to the number of cards in your hand.
Draw a card.","1GG","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Talara's Bane","B","Target opponent reveals his or her hand. You choose a green or white creature card from it. You gain life equal that creature card's toughness, then that player discards that card.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forge Armor","R","As an additional cost to cast Forge Armor, sacrifice an artifact.
Put X +1/+1 counters on target creature, where X is the sacrificed artifact's converted mana cost.","4R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thallid Devourer","2","2","G","At the beginning of your upkeep, put a spore counter on Thallid Devourer.
Remove three spore counters from Thallid Devourer: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: Thallid Devourer gets +1/+2 until end of turn.","1GG","Creature - Fungus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Griffin Sentinel","1","3","W","Flying
Vigilance (Attacking doesn't cause this creature to tap.)","2W","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fleshwrither","3","3","B","Transfigure {1}{B}{B} ({1}{B}{B}, Sacrifice this creature: Search your library for a creature card with the same converted mana cost as this creature and put that card onto the battlefield. Then shuffle your library. Transfigure only as a sorcery.)","2BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Venser's Sliver","3","3",NULL,"5","Artifact Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Madrush Cyclops","3","4","BRG","Creatures you control have haste.","1BRG","Creature - Cyclops Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Deep Wood","G","Cast Deep Wood only during the declare attackers step and only if you've been attacked this step.
Prevent all damage that would be dealt to you this turn by attacking creatures.","1G","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Oran-Rief, the Vastwood","Oran-Rief, the Vastwood enters the battlefield tapped.
{T}: Add {G} to your mana pool.
{T}: Put a +1/+1 counter on each green creature that entered the battlefield this turn.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aura Mutation","WG","Destroy target enchantment. Put X 1/1 green Saproling creature tokens onto the battlefield, where X is that enchantment's converted mana cost.","GW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Feud","R","Target creature fights another target creature. (Each deals damage equal to its power to the other.)","4RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fire-Field Ogre","4","2","UBR","First strike
Unearth {U}{B}{R} ({U}{B}{R}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","1UBR","Creature - Ogre Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wandering Mage","0","3","WUB","{W}, Pay 1 life: Prevent the next 2 damage that would be dealt to target creature this turn.
{U}: Prevent the next 1 damage that would be dealt to target Cleric or Wizard creature this turn.
{B}, Put a -1/-1 counter on a creature you control: Prevent the next 2 damage that would be dealt to target player this turn.","WUB","Creature - Human Cleric Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stone-Tongue Basilisk","4","5","G","Whenever Stone-Tongue Basilisk deals combat damage to a creature, destroy that creature at end of combat.
Threshold - As long as seven or more cards are in your graveyard, all creatures able to block Stone-Tongue Basilisk do so.","4GGG","Creature - Basilisk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Recurring Nightmare","B","Sacrifice a creature, Return Recurring Nightmare to its owner's hand: Return target creature card from your graveyard to the battlefield. Activate this ability only any time you could cast a sorcery.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dawning Purist","2","2","W","Whenever Dawning Purist deals combat damage to a player, you may destroy target enchantment that player controls.
Morph {1}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","2W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grizzled Wolverine","2","2","R","Whenever Grizzled Wolverine becomes blocked, you may pay {R}. If you do, Grizzled Wolverine gets +2/+0 until end of turn.","1RR","Creature - Wolverine");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Molten Psyche","R","Each player shuffles the cards from his or her hand into his or her library, then draws that many cards.
Metalcraft - If you control three or more artifacts, Molten Psyche deals damage to each opponent equal to the number of cards that player has drawn this turn.","1RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mire Blight","B","Enchant creature
When enchanted creature is dealt damage, destroy it.","B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gather Specimens","U","If a creature would enter the battlefield under an opponent's control this turn, it enters the battlefield under your control instead.","3UUU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lava Dart","R","Lava Dart deals 1 damage to target creature or player.
Flashback-Sacrifice a Mountain. (You may cast this card from your graveyard for its flashback cost. Then exile it.)","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Paralyzing Grasp","U","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Norritt","1","1","B","{T}: Untap target blue creature.
{T}: Choose target non-Wall creature the active player has controlled continuously since the beginning of the turn. That creature attacks this turn if able. If it doesn't, destroy it at the beginning of the next end step. Activate this ability only before attackers are declared.","3B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zhou Yu, Chief Commander","8","8","U","Zhou Yu, Chief Commander can't attack unless defending player controls an Island.","5UU","Legendary Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Marsh Flitter","1","1","B","Flying
When Marsh Flitter enters the battlefield, put two 1/1 black Goblin Rogue creature tokens onto the battlefield.
Sacrifice a Goblin: Marsh Flitter becomes 3/3 until end of turn.","3B","Creature - Faerie Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ethersworn Adjudicator","4","4","U","Flying
{1}{W}{B}, {T}: Destroy target creature or enchantment.
{2}{U}: Untap Ethersworn Adjudicator.","4U","Artifact Creature - Vedalken Knight");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nine-Ringed Bo","{T}: Nine-Ringed Bo deals 1 damage to target Spirit creature. If that creature would die this turn, exile it instead.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hickory Woodlot","Hickory Woodlot enters the battlefield tapped with two depletion counters on it.
{T}, Remove a depletion counter from Hickory Woodlot: Add {G}{G} to your mana pool. If there are no depletion counters on Hickory Woodlot, sacrifice it.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Plummet","G","Destroy target creature with flying.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Impaler Shrike","3","1","U","Flying
Whenever Impaler Shrike deals combat damage to a player, you may sacrifice it. If you do, draw three cards.","2UU","Creature - Bird");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Disrupting Scepter","{3}, {T}: Target player discards a card. Activate this ability only during your turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Quill-Slinger Boggart","3","2","B","Whenever a player casts a Kithkin spell, you may have target player lose 1 life.","3B","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bullwhip","{2}, {T}: Bullwhip deals 1 damage to target creature. That creature attacks this turn if able.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Snuff Out","B","If you control a Swamp, you may pay 4 life rather than pay Snuff Out's mana cost.
Destroy target nonblack creature. It can't be regenerated.","3B","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gremlin Mine","{1}, {T}, Sacrifice Gremlin Mine: Gremlin Mine deals 4 damage to target artifact creature.
{1}, {T}, Sacrifice Gremlin Mine: Remove up to four charge counters from target noncreature artifact.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("False Demise","U","Enchant creature
When enchanted creature dies, return that card to the battlefield under your control.","2U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Relentless Rats","2","2","B","Relentless Rats gets +1/+1 for each other creature on the battlefield named Relentless Rats.
A deck can have any number of cards named Relentless Rats.","1BB","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pure Reflection","W","Whenever a player casts a creature spell, destroy all Reflections. Then that player puts an X/X white Reflection creature token onto the battlefield, where X is the converted mana cost of that spell.","2W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Durkwood Baloth","5","5","G","Suspend 5-{G} (Rather than cast this card from your hand, you may pay {G} and exile it with five time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","4GG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Krovikan Rot","B","Destroy target creature with power 2 or less.
Recover {1}{B}{B} (When a creature is put into your graveyard from the battlefield, you may pay {1}{B}{B}. If you do, return this card from your graveyard to your hand. Otherwise, exile this card.)","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightscape Master","2","2","B","{U}{U}, {T}: Return target creature to its owner's hand.
{R}{R}, {T}: Nightscape Master deals 2 damage to target creature.","2BB","Creature - Zombie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vengeful Rebirth","RG","Return target card from your graveyard to your hand. If you return a nonland card to your hand this way, Vengeful Rebirth deals damage equal to that card's converted mana cost to target creature or player.
Exile Vengeful Rebirth.","4RG","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Straw Golem","2","3","When an opponent casts a creature spell, sacrifice Straw Golem.","1","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trained Pronghorn","1","1","W","Discard a card: Prevent all damage that would be dealt to Trained Pronghorn this turn.","1W","Creature - Antelope");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sophic Centaur","1","1","G","{2}{G}{G}, {T}, Discard a card: You gain 2 life for each card in your hand.","3G","Creature - Centaur Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ancient Hellkite","6","6","R","Flying
{R}: Ancient Hellkite deals 1 damage to target creature defending player controls. Activate this ability only if Ancient Hellkite is attacking.","4RRR","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Prospector","1","1","R","Sacrifice a Goblin: Add {R} to your mana pool.","R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Mindcensor","2","1","W","Flash (You may cast this spell any time you could cast an instant.)
Flying
If an opponent would search a library, that player searches the top four cards of that library instead.","2W","Creature - Bird Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viscerid Deepwalker","2","3","U","{U}: Viscerid Deepwalker gets +1/+0 until end of turn.
Suspend 4-{U} (Rather than cast this card from your hand, you may pay {U} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","4U","Creature - Homarid Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oran-Rief Recluse","1","3","G","Kicker {2}{G} (You may pay an additional {2}{G} as you cast this spell.)
Reach (This creature can block creatures with flying.)
When Oran-Rief Recluse enters the battlefield, if it was kicked, destroy target creature with flying.","2G","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sokenzan Bruiser","3","3","R","Mountainwalk","4R","Creature - Ogre Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Living Inferno","8","5","R","{T}: Living Inferno deals damage equal to its power divided as you choose among any number of target creatures. Each of those creatures deals damage equal to its power to Living Inferno.","6RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Galvanic Juggernaut","5","5","Galvanic Juggernaut attacks each turn if able.
Galvanic Juggernaut doesn't untap during your untap step.
Whenever another creature dies, untap Galvanic Juggernaut.","4","Artifact Creature - Juggernaut");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Boa Constrictor","3","3","G","{T}: Boa Constrictor gets +3/+3 until end of turn.","4G","Creature - Snake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rith's Grove","When Rith's Grove enters the battlefield, sacrifice it unless you return a non-Lair land you control to its owner's hand.
{T}: Add {R}, {G}, or {W} to your mana pool.",NULL,"Land - Lair");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Favorable Winds","U","Creatures you control with flying get +1/+1.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Outmaneuver","R","X target blocked creatures assign their combat damage this turn as though they weren't blocked.","XR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stormbind","RG","{2}, Discard a card at random: Stormbind deals 2 damage to target creature or player.","1RG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bonds of Quicksilver","U","Flash (You may cast this spell any time you could cast an instant.)
Enchant creature
Enchanted creature doesn't untap during its controller's untap step.","3U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drinker of Sorrow","5","3","B","Drinker of Sorrow can't block.
Whenever Drinker of Sorrow deals combat damage, sacrifice a permanent.","2B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Underworld Dreams","B","Whenever an opponent draws a card, Underworld Dreams deals 1 damage to him or her.","BBB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thirst for Knowledge","U","Draw three cards. Then discard two cards unless you discard an artifact card.","2U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hazezon Tamar","2","4","WRG","When Hazezon Tamar enters the battlefield, put X 1/1 Sand Warrior creature tokens that are red, green, and white onto the battlefield at the beginning of your next upkeep, where X is the number of lands you control at that time.
When Hazezon leaves the battlefield, exile all Sand Warriors.","4RGW","Legendary Creature - Human Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Skinwing","Living weapon (When this Equipment enters the battlefield, put a 0/0 black Germ creature token onto the battlefield, then attach this to it.)
Equipped creature gets +2/+2 and has flying.
Equip {6}","4","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kiss of the Amesha","WU","Target player gains 7 life and draws two cards.","4WU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightshade Assassin","2","1","B","First strike
When Nightshade Assassin enters the battlefield, you may reveal X black cards in your hand. If you do, target creature gets -X/-X until end of turn.
Madness {1}{B} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","2BB","Creature - Human Assassin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eight-and-a-Half-Tails Avatar","{1}: Until end of turn, target permanent you control gains protection from a color chosen at random from colors it doesn't have protection from.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crackling Club","R","Enchant creature
Enchanted creature gets +1/+0.
Sacrifice Crackling Club: Crackling Club deals 1 damage to target creature.","R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reverse Damage","W","The next time a source of your choice would deal damage to you this turn, prevent that damage. You gain life equal to the damage prevented this way.","1WW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Doubling Chant","G","For each creature you control, you may search your library for a creature card with the same name as that creature. Put those cards onto the battlefield, then shuffle your library.","5G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Razia's Purification","WR","Each player chooses three permanents he or she controls, then sacrifices the rest.","4RW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Renegade Demon","5","3","B",NULL,"3BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reiterate","R","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Copy target instant or sorcery spell. You may choose new targets for the copy.","1RR","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rust Tick","1","3","You may choose not to untap Rust Tick during your untap step.
{1}, {T}: Tap target artifact. It doesn't untap during its controller's untap step for as long as Rust Tick remains tapped.","3","Artifact Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychotic Fury","R","Target multicolored creature gains double strike until end of turn.
Draw a card.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lynx","2","1","G","Forestwalk","1G","Creature - Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Return to Battle","B","Return target creature card from your graveyard to your hand.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unruly Mob","1","1","W","Whenever another creature you control dies, put a +1/+1 counter on Unruly Mob.","1W","Creature - Human");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Peel","B","Buyback {2}{B}{B} (You may pay an additional {2}{B}{B} as you cast this spell. If you do, put this card into your hand as it resolves.)
Target player discards a card.","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mindmoil","R","Whenever you cast a spell, put the cards in your hand on the bottom of your library in any order, then draw that many cards.","4R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Smart Ass","2{1/2}","1","U","Whenever Smart Ass attacks, name a card. Defending player may reveal his or her hand and show you that the named card isn't there. If that player doesn't, Smart Ass is unblockable this turn.","2U","Creature - Donkey Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Precursor Golem","3","3","When Precursor Golem enters the battlefield, put two 3/3 colorless Golem artifact creature tokens onto the battlefield.
Whenever a player casts an instant or sorcery spell that targets only a single Golem, that player copies that spell for each other Golem that spell could target. Each copy targets a different one of those Golems.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Atog","1","2","R","Sacrifice an artifact: Atog gets +2/+2 until end of turn.","1R","Creature - Atog");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Master of the Wild Hunt","3","3","G","At the beginning of your upkeep, put a 2/2 green Wolf creature token onto the battlefield.
{T}: Tap all untapped Wolf creatures you control. Each Wolf tapped this way deals damage equal to its power to target creature. That creature deals damage equal to its power divided as its controller chooses among any number of those Wolves.","2GG","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raid Bombardment","R","Whenever a creature you control with power 2 or less attacks, Raid Bombardment deals 1 damage to defending player.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wizard Mentor","2","2","U","{T}: Return Wizard Mentor and target creature you control to their owner's hand.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fusion Elemental","8","8","WUBRG",NULL,"WUBRG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spore Burst","G","Domain - Put a 1/1 green Saproling creature token onto the battlefield for each basic land type among lands you control.","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tidings","U","Draw four cards.","3UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Courier's Capsule","U","{1}{U}, {T}, Sacrifice Courier's Capsule: Draw two cards.","1U","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blind Hunter","2","2","WB","Flying
Haunt (When this creature dies, exile it haunting target creature.)
When Blind Hunter enters the battlefield or the creature it haunts dies, target player loses 2 life and you gain 2 life.","2WB","Creature - Bat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Protection: Artifacts","W","{2}: The next time an artifact source of your choice would deal damage to you this turn, prevent that damage.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Sanctifiers","2","3","W","Kicker {W} (You may pay an additional {W} as you cast this spell.)
When Kor Sanctifiers enters the battlefield, if it was kicked, destroy target artifact or enchantment.","2W","Creature - Kor Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blinding Mage","1","2","W","{W}, {T}: Tap target creature.","1W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Condemn","W","Put target attacking creature on the bottom of its owner's library. Its controller gains life equal to its toughness.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dominaria's Judgment","W","Until end of turn, creatures you control gain protection from white if you control a Plains, from blue if you control an Island, from black if you control a Swamp, from red if you control a Mountain, and from green if you control a Forest.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Alliance of Arms","W","Join forces - Starting with you, each player may pay any amount of mana. Each player puts X 1/1 white Soldier creature tokens onto the battlefield, where X is the total amount of mana paid this way.","W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rip-Clan Crasher","2","2","RG","Haste","RG","Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Argivian Find","W","Return target artifact or enchantment card from your graveyard to your hand.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Silhouette","U","Prevent all damage that would be dealt to target creature this turn by spells or abilities that target it.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Ox","3","3","G","Swampwalk","3G","Creature - Ox");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Centaur Omenreader","3","3","G","As long as Centaur Omenreader is tapped, creature spells you cast cost {2} less to cast.","3G","Snow Creature - Centaur Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jungle Wurm","5","5","G","Whenever Jungle Wurm becomes blocked, it gets -1/-1 until end of turn for each creature blocking it beyond the first.","3GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Granger","2","2","G","Echo {2}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Yavimaya Granger enters the battlefield, you may search your library for a basic land card, put that card onto the battlefield tapped, then shuffle your library.","2G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Muck Rats","1","1","B",NULL,"B","Creature - Rat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Three Wishes","U","Exile the top three cards of your library face down. You may look at those cards for as long as they remain exiled. Until your next turn, you may play those cards. At the beginning of your next upkeep, put any of those cards you didn't play into your graveyard.","1UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Roc of Kher Ridges","3","3","R","Flying","3R","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sisters of Stone Death","7","5","BG","{G}: Target creature blocks Sisters of Stone Death this turn if able.
{B}{G}: Exile target creature blocking or blocked by Sisters of Stone Death.
{2}{B}: Put a creature card exiled with Sisters of Stone Death onto the battlefield under your control.","4BBGG","Legendary Creature - Gorgon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rolling Thunder","R","Rolling Thunder deals X damage divided as you choose among any number of target creatures and/or players.","XRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Redeem the Lost","W","Target creature you control gains protection from the color of your choice until end of turn. Clash with an opponent. If you win, return Redeem the Lost to its owner's hand. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cackling Counterpart","U","Put a token onto the battlefield that's a copy of target creature you control.
Flashback {5}{U}{U} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1UU","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Quadropod","1","4","{3}: Switch Myr Quadropod's power and toughness until end of turn.","4","Artifact Creature - Myr");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stun","R","Target creature can't block this turn.
Draw a card.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harvest Gwyllion","2","4","WB","Wither (This deals damage to creatures in the form of -1/-1 counters.)","2(W/B)(W/B)","Creature - Hag");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Echoing Courage","G","Target creature and all other creatures with the same name as that creature get +2/+2 until end of turn.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cosi's Ravager","2","2","R","Landfall - Whenever a land enters the battlefield under your control, you may have Cosi's Ravager deal 1 damage to target player.","3R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mask of Riddles","UB","Equipped creature has fear. (It can't be blocked except by artifact creatures and/or black creatures.)
Whenever equipped creature deals combat damage to a player, you may draw a card.
Equip {2}","UB","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Adarkar Windform","3","3","U","Flying
{1}{S}i}: Target creature loses flying until end of turn. ({S}i} can be paid with one mana from a snow permanent.)","4U","Snow Creature - Illusion");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bravado","R","Enchant creature
Enchanted creature gets +1/+1 for each other creature you control.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Redwood Treefolk","3","6","G",NULL,"4G","Creature - Treefolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Assert Authority","U","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Counter target spell. If that spell is countered this way, exile it instead of putting it into its owner's graveyard.","5UU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Molten Slagheap","{T}: Add {1} to your mana pool.
{1}, {T}: Put a storage counter on Molten Slagheap.
{1}, Remove X storage counters from Molten Slagheap: Add X mana in any combination of {B} and/or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burn at the Stake","R","As an additional cost to cast Burn at the Stake, tap any number of untapped creatures you control.
Burn at the Stake deals damage to target creature or player equal to three times the number of creatures tapped this way.","2RRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vastwood Zendikon","G","Enchant land
Enchanted land is a 6/4 green Elemental creature. It's still a land.
When enchanted land dies, return that card to its owner's hand.","4G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ancient Tomb","{T}: Add {2} to your mana pool. Ancient Tomb deals 2 damage to you.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kyren Toy","{1}, {T}: Put a charge counter on Kyren Toy.
{T}, Remove X charge counters from Kyren Toy: Add {X}{1} to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Morkrut Banshee","4","4","B","Morbid - When Morkrut Banshee enters the battlefield, if a creature died this turn, target creature gets -4/-4 until end of turn.","3BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wu Spy","1","1","U","When Wu Spy enters the battlefield, look at the top two cards of target player's library. Put one of them into his or her graveyard.","1U","Creature - Human Soldier Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Xun Yu, Wei Advisor","1","1","B","{T}: Target creature you control gets +2/+0 until end of turn. Activate this ability only during your turn, before attackers are declared.","1BB","Legendary Creature - Human Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Molder","G","Destroy target artifact or enchantment with converted mana cost X. It can't be regenerated. You gain X life.","XG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zuran Enchanter","1","1","U","{2}{B}, {T}: Target player discards a card. Activate this ability only during your turn.","1U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argivian Blacksmith","2","2","W","{T}: Prevent the next 2 damage that would be dealt to target artifact creature this turn.","1WW","Creature - Human Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cuombajj Witches","1","3","B","{T}: Cuombajj Witches deals 1 damage to target creature or player and 1 damage to target creature or player of an opponent's choice.","BB","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Corrupted Harvester","6","3","B","{B}, Sacrifice a creature: Regenerate Corrupted Harvester.","4BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chain of Silence","W","Prevent all damage target creature would deal this turn. That creature's controller may sacrifice a land. If the player does, he or she may copy this spell and may choose a new target for that copy.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Fireslinger","1","1","R","{T}: Goblin Fireslinger deals 1 damage to target player.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bloodfire Dwarf","1","1","R","{R}, Sacrifice Bloodfire Dwarf: Bloodfire Dwarf deals 1 damage to each creature without flying.","R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Screaming Fury","R","Target creature gets +5/+0 and gains haste until end of turn.","2R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fight or Flight","W","At the beginning of each opponent's combat, separate all creatures that player controls into two piles. Only creatures in the pile of his or her choice can attack this turn.","3W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Opal Archangel","W","When an opponent casts a creature spell, if Opal Archangel is an enchantment, Opal Archangel becomes a 5/5 Angel creature with flying and vigilance.","4W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Sappers","1","1","R","{R}{R}, {T}: Target creature you control is unblockable this turn. Destroy it and Goblin Sappers at end of combat.
{R}{R}{R}{R}, {T}: Target creature you control is unblockable this turn. Destroy it at end of combat.","1R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gruul Signet","{1}, {T}: Add {R}{G} to your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Body Double","0","0","U","You may have Body Double enter the battlefield as a copy of any creature card in a graveyard.","4U","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guile","6","6","U","Guile can't be blocked except by three or more creatures.
If a spell or ability you control would counter a spell, instead exile that spell and you may play that card without paying its mana cost.
When Guile is put into a graveyard from anywhere, shuffle it into its owner's library.","3UUU","Creature - Elemental Incarnation");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wandering Ones","1","1","U",NULL,"U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Energy Flux","U","All artifacts have ""At the beginning of your upkeep, sacrifice this artifact unless you pay {2}.""","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Teferi's Response","U","Counter target spell or ability an opponent controls that targets a land you control. If a permanent's ability is countered this way, destroy that permanent.
Draw two cards.","1U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chrome Mox","Imprint - When Chrome Mox enters the battlefield, you may exile a nonartifact, nonland card from your hand.
{T}: Add one mana of any of the exiled card's colors to your mana pool.","0","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Icatian Town","W","Put four 1/1 white Citizen creature tokens onto the battlefield.","5W","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Trigon of Rage","Trigon of Rage enters the battlefield with three charge counters on it.
{R}{R}, {T}: Put a charge counter on Trigon of Rage.
{2}, {T}, Remove a charge counter from Trigon of Rage: Target creature gets +3/+0 until end of turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Fogbringer","2","1","U","Flying
When Aven Fogbringer enters the battlefield, return target land to its owner's hand.","3U","Creature - Bird Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angel's Mercy","W","You gain 7 life.","2WW","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Copperline Gorge","Copperline Gorge enters the battlefield tapped unless you control two or fewer other lands.
{T}: Add {R} or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Dustwasp","3","3","G","Flying
Suspend 4-{1}{G} (Rather than cast this card from your hand, you may pay {1}{G} and exile it with four time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","3GG","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("""Ach! Hans, Run!""","RG","At the beginning of your upkeep, you may say ""Ach Hans, run It's the . . ."" and name a creature card. If you do, search your library for the named card, put it into play, then shuffle your library. That creature has haste. Remove it from the game at end of turn.","2RRGG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shower of Sparks","R","Shower of Sparks deals 1 damage to target creature and 1 damage to target player.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Llanowar Mentor","1","1","G","{G}, {T}, Discard a card: Put a 1/1 green Elf Druid creature token named Llanowar Elves onto the battlefield. It has ""{T}: Add {G} to your mana pool.""","G","Creature - Elf Spellshaper");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Greatbow Doyen","2","4","G","Other Archer creatures you control get +1/+1.
Whenever an Archer you control deals damage to a creature, that Archer deals that much damage to that creature's controller.","4G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Elemental","7","7","Trample",NULL,"Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tel-Jilad Stylus","{T}: Put target permanent you own on the bottom of your library.","1","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pulse of the Forge","R","Pulse of the Forge deals 4 damage to target player. Then if that player has more life than you, return Pulse of the Forge to its owner's hand.","1RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Multani, Maro-Sorcerer","*","*","G","Shroud (This permanent can't be the target of spells or abilities.)
Multani, Maro-Sorcerer's power and toughness are each equal to the total number of cards in all players' hands.","4GG","Legendary Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Severed Legion","2","2","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)","1BB","Creature - Zombie");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Liberate","W","Exile target creature you control. Return that card to the battlefield under its owner's control at the beginning of the next end step.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Appetite for Brains","B","Target opponent reveals his or her hand. You choose a card from it with converted mana cost 4 or greater and exile that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Raven Guild Master","1","1","U","Whenever Raven Guild Master deals combat damage to a player, that player exiles the top ten cards of his or her library.
Morph {2}{U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","1UU","Creature - Human Wizard Mutant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Loxodon Warhammer","Equipped creature gets +3/+0 and has trample and lifelink. (If the creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker. Damage dealt by the creature also causes its controller to gain that much life.)
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ior Ruin Expedition","U","Landfall - Whenever a land enters the battlefield under your control, you may put a quest counter on Ior Ruin Expedition.
Remove three quest counters from Ior Ruin Expedition and sacrifice it: Draw two cards.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Safewright Quest","WG","Search your library for a Forest or Plains card, reveal it, and put it into your hand. Then shuffle your library.","(G/W)","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Krosan Verge","Krosan Verge enters the battlefield tapped.
{T}: Add {1} to your mana pool.
{2}, {T}, Sacrifice Krosan Verge: Search your library for a Forest card and a Plains card and put them onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fists of the Demigod","BR","Enchant creature
As long as enchanted creature is black, it gets +1/+1 and has wither. (It deals damage to creatures in the form of -1/-1 counters.)
As long as enchanted creature is red, it gets +1/+1 and has first strike.","1(B/R)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thundering Giant","4","3","R","Haste (This creature can attack and {T} as soon as it comes under your control.)","3RR","Creature - Giant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Bosh, Iron Golem","6","7","Trample
{3}{R}, Sacrifice an artifact: Bosh, Iron Golem deals damage equal to the sacrificed artifact's converted mana cost to target creature or player.","8","Legendary Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Evangelize","W","Buyback {2}{W}{W} (You may pay an additional {2}{W}{W} as you cast this spell. If you do, put this card into your hand as it resolves.)
Gain control of target creature of an opponent's choice that he or she controls.","4W","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Time Machine","{T}: Remove Time Machine and target nontoken creature you own from the game. Return both cards to play at the beginning of your upkeep on your turn X of the next game you play with the same opponent, where X is the removed creature's converted mana cost.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Death's-Head Buzzard","2","1","B","Flying
When Death's-Head Buzzard dies, all creatures get -1/-1 until end of turn.","1BB","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stalwart Shield-Bearers","0","3","W","Defender
Other creatures you control with defender get +0/+2.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Geist of Saint Traft","2","2","WU","Hexproof (This creature can't be the target of spells or abilities your opponents control.)
Whenever Geist of Saint Traft attacks, put a 4/4 white Angel creature token with flying onto the battlefield tapped and attacking. Exile that token at end of combat.","1WU","Legendary Creature - Spirit Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Noble Vestige","1","2","W","Flying
{T}: Prevent the next 1 damage that would be dealt to target player this turn.","2W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mosquito Guard","1","1","W","First strike
Reinforce 1-{1}{W} ({1}{W}, Discard this card: Put a +1/+1 counter on target creature.)","W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akrasan Squire","1","1","W","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arbor Elf","1","1","G","{T}: Untap target Forest.","G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cruel Bargain","B","Draw four cards. You lose half your life, rounded up.","BBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sootfeather Flock","3","2","B","Flying
Morph {3}{B} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4B","Creature - Bird");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhox Brute","4","4","RG",NULL,"2RG","Creature - Rhino Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stalking Yeti","3","3","R","When Stalking Yeti enters the battlefield, if it's on the battlefield, it deals damage equal to its power to target creature an opponent controls and that creature deals damage equal to its power to Stalking Yeti.
{2}{S}i}: Return Stalking Yeti to its owner's hand. Activate this ability only any time you could cast a sorcery. ({S}i} can be paid with one mana from a snow permanent.)","2RR","Snow Creature - Yeti");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kusari-Gama","Equipped creature has ""{2}: This creature gets +1/+0 until end of turn.""
Whenever equipped creature deals damage to a blocking creature, Kusari-Gama deals that much damage to each other creature defending player controls.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery.)","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Octopus","3","3","U",NULL,"3U","Creature - Octopus");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Archangel","5","5","W","Flying, vigilance","5WW","Creature - Angel");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Cursed Scroll","{3}, {T}: Name a card. Reveal a card at random from your hand. If it's the named card, Cursed Scroll deals 2 damage to target creature or player.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darkslick Drake","2","4","U","Flying
When Darkslick Drake dies, draw a card.","2UU","Creature - Drake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Monkey","2","1","G","Forestwalk (This creature is unblockable as long as defending player controls a Forest.)","1G","Creature - Ape");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Roots","G","Enchant creature without flying
When Roots enters the battlefield, tap enchanted creature.
Enchanted creature doesn't untap during its controller's untap step.","3G","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eightfold Maze","W","Cast Eightfold Maze only during the declare attackers step and only if you've been attacked this step.
Destroy target attacking creature.","2W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Quicksilver Fountain","At the beginning of each player's upkeep, that player puts a flood counter on target non-Island land he or she controls of his or her choice. That land is an Island for as long as it has a flood counter on it.
At the beginning of each end step, if all lands on the battlefield are Islands, remove all flood counters from them.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pearl Dragon","4","4","W","Flying
{1}{W}: Pearl Dragon gets +0/+1 until end of turn.","4WW","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Simian Spirit Guide","2","2","R","Exile Simian Spirit Guide from your hand: Add {R} to your mana pool.","2R","Creature - Ape Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Naya Charm","WRG","Choose one - Naya Charm deals 3 damage to target creature; or return target card from a graveyard to its owner's hand; or tap all creatures target player controls.","RGW","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dread Charge","B","Until end of turn, black creatures you control can be blocked only by black creatures.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Witherscale Wurm","9","9","G","Whenever Witherscale Wurm blocks or becomes blocked by a creature, that creature gains wither until end of turn. (It deals damage to creatures in the form of -1/-1 counters.)
Whenever Witherscale Wurm deals damage to an opponent, remove all -1/-1 counters from it.","4GG","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blanchwood Armor","G","Enchant creature (Target a creature as you cast this. This card enters the battlefield attached to that creature.)
Enchanted creature gets +1/+1 for each Forest you control.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glowing Anemone","1","3","U","When Glowing Anemone enters the battlefield, you may return target land to its owner's hand.","3U","Creature - Jellyfish Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Temple Elder","1","2","W","{T}: You gain 1 life. Activate this ability only during your turn, before attackers are declared.","2W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fabricate","U","Search your library for an artifact card, reveal it, and put it into your hand. Then shuffle your library.","2U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gavony Township","{T}: Add {1} to your mana pool.
{2}{G}{W}, {T}: Put a +1/+1 counter on each creature you control.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vicious Hunger","B","Vicious Hunger deals 2 damage to target creature and you gain 2 life.","BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burning Wish","R","You may choose a sorcery card you own from outside the game, reveal that card, and put it into your hand. Exile Burning Wish.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jade Mage","2","1","G","{2}{G}: Put a 1/1 green Saproling creature token onto the battlefield.","1G","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Priest of Gix","2","1","B","When Priest of Gix enters the battlefield, add {B}{B}{B} to your mana pool.","2B","Creature - Human Cleric Minion");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("The Very Soil Shall Shake","(An ongoing scheme remains face up until it's abandoned.)
Creatures you control get +2/+2 and have trample.
When a creature you control dies, abandon this scheme.",NULL,"Ongoing Scheme");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grollub","3","3","B","Whenever Grollub is dealt damage, each opponent gains that much life.","2B","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heart of Bogardan","R","Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
When a player doesn't pay Heart of Bogardan's cumulative upkeep, Heart of Bogardan deals X damage to target player and each creature he or she controls, where X is twice the number of age counters on Heart of Bogardan minus 2.","2RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Abbey Gargoyles","3","4","W","Flying, protection from red","2WWW","Creature - Gargoyle");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Roots of Life","G","As Roots of Life enters the battlefield, choose Islands or Swamps.
Whenever a land of the chosen type an opponent controls becomes tapped, you gain 1 life.","1GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Skyscout","2","3","U","Flying
Whenever Merfolk Skyscout attacks or blocks, untap target permanent.","2UU","Creature - Merfolk Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Netherborn Phalanx","2","4","B","When Netherborn Phalanx enters the battlefield, each opponent loses 1 life for each creature he or she controls.
Transmute {1}{B}{B} ({1}{B}{B}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","5B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emissary of Despair","2","1","B","Flying
Whenever Emissary of Despair deals combat damage to a player, that player loses 1 life for each artifact he or she controls.","1BB","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Putrefax","5","3","G","Trample, haste
Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
At the beginning of the end step, sacrifice Putrefax.","3GG","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Caltrops","2","1","W","Defender (This creature can't attack.)
Whenever Wall of Caltrops blocks a creature, if no non-Wall creatures are blocking that creature, Wall of Caltrops gains banding until end of turn. (If any creatures with banding you control are blocking a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by.)","1W","Creature - Wall");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Urgent Exorcism","W","Destroy target Spirit or enchantment.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Island Sanctuary","W","If you would draw a card during your draw step, instead you may skip that draw. If you do, until your next turn, you can't be attacked except by creatures with flying and/or islandwalk.","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Specter's Shroud","Equipped creature gets +1/+0.
Whenever equipped creature deals combat damage to a player, that player discards a card.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery. This card enters the battlefield unattached and stays on the battlefield if the creature leaves.)","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soratami Cloudskater","1","1","U","Flying
{2}, Return a land you control to its owner's hand: Draw a card, then discard a card.","1U","Creature - Moonfolk Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loam Lion","1","1","W","Loam Lion gets +1/+2 as long as you control a Forest.","W","Creature - Cat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scythe of the Wretched","Equipped creature gets +2/+2.
Whenever a creature dealt damage by equipped creature this turn dies, return that card to the battlefield under your control. Attach Scythe of the Wretched to that creature.
Equip {4}","2","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spellbreaker Behemoth","5","5","RG","Spellbreaker Behemoth can't be countered.
Creature spells you control with power 5 or greater can't be countered.","1RGG","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Backlash","BR","Tap target untapped creature. That creature deals damage equal to its power to its controller.","1BR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Ranger","4","1","G",NULL,"2G","Creature - Elf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heroic Defiance","W","Enchant creature
Enchanted creature gets +3/+3 unless it shares a color with the most common color among all permanents or a color tied for most common.","1W","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rats' Feast","B","Exile X target cards from a single graveyard.","XB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rivals' Duel","R","Choose two target creatures that share no creature types. Those creatures fight each other. (Each deals damage equal to its power to the other.)","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Harpoon Sniper","2","2","W","{W}, {T}: Harpoon Sniper deals X damage to target attacking or blocking creature, where X is the number of Merfolk you control.","2W","Creature - Merfolk Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ib Halfheart, Goblin Tactician","3","2","R","Whenever another Goblin you control becomes blocked, sacrifice it. If you do, it deals 4 damage to each creature blocking it.
Sacrifice two Mountains: Put two 1/1 red Goblin creature tokens onto the battlefield.","3R","Legendary Creature - Goblin Advisor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Massacre","B","If an opponent controls a Plains and you control a Swamp, you may cast Massacre without paying its mana cost.
All creatures get -2/-2 until end of turn.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flensermite","1","1","B","Infect (This creature deals damage to creatures in the form of -1/-1 counters and to players in the form of poison counters.)
Lifelink (Damage dealt by this creature also causes you to gain that much life.)","1B","Creature - Gremlin");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Spire Golem","2","4","Affinity for Islands (This spell costs {1} less to cast for each Island you control.)
Flying","6","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Terror of Kruin Pass","3","3","Double strike
Each Werewolf you control can't be blocked except by two or more creatures.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Terror of Kruin Pass.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Ring of Renewal","{5}, {T}: Discard a card at random, then draw two cards.","5","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Farmer","0","2","G","At the beginning of your upkeep, put a spore counter on Elvish Farmer.
Remove three spore counters from Elvish Farmer: Put a 1/1 green Saproling creature token onto the battlefield.
Sacrifice a Saproling: You gain 2 life.","1G","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bone Shaman","3","3","R","{B}: Until end of turn, Bone Shaman gains ""Creatures dealt damage by Bone Shaman this turn can't be regenerated this turn.""","2RR","Creature - Giant Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Viridian Revel","G","Whenever an artifact is put into an opponent's graveyard from the battlefield, you may draw a card.","1GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vertigo Spawn","0","3","U","Defender (This creature can't attack.)
Whenever Vertigo Spawn blocks a creature, tap that creature. That creature doesn't untap during its controller's next untap step.","1U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mercenary Knight","4","4","B","When Mercenary Knight enters the battlefield, sacrifice it unless you discard a creature card.","2B","Creature - Human Mercenary Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shock","R","Shock deals 2 damage to target creature or player.","R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Karn's Touch","U","Target noncreature artifact becomes an artifact creature with power and toughness each equal to its converted mana cost until end of turn. (It retains its abilities.)","UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flamekin Harbinger","1","1","R","When Flamekin Harbinger enters the battlefield, you may search your library for an Elemental card, reveal it, then shuffle your library and put that card on top of it.","R","Creature - Elemental Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sudden Impact","R","Sudden Impact deals damage to target player equal to the number of cards in that player's hand.","3R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Roiling Terrain","R","Destroy target land, then Roiling Terrain deals damage to that land's controller equal to the number of land cards in that player's graveyard.","2RR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Song of Blood","R","Put the top four cards of your library into your graveyard.
Whenever a creature attacks this turn, it gets +1/+0 until end of turn for each creature card put into your graveyard this way.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Argivian Archaeologist","1","1","W","{W}{W}, {T}: Return target artifact card from your graveyard to your hand.","1WW","Creature - Human Artificer");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wand of Ith","{3}, {T}: Target player reveals a card at random from his or her hand. If it's a land card, that player discards it unless he or she pays 1 life. If it isn't a land card, the player discards it unless he or she pays life equal to its converted mana cost. Activate this ability only during your turn.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Symbiotic Wurm","7","7","G","When Symbiotic Wurm dies, put seven 1/1 green Insect creature tokens onto the battlefield.","5GGG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cephalid Illusionist","1","1","U","Whenever Cephalid Illusionist becomes the target of a spell or ability, put the top three cards of your library into your graveyard.
{2}{U}, {T}: Prevent all combat damage that would be dealt to and dealt by target creature you control this turn.","1U","Creature - Cephalid Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primordial Sage","4","5","G","Whenever you cast a creature spell, you may draw a card.","4GG","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sewer Nemesis","*","*","B","As Sewer Nemesis enters the battlefield, choose a player.
Sewer Nemesis's power and toughness are each equal to the number of cards in the chosen player's graveyard.
Whenever the chosen player casts a spell, that player puts the top card of his or her library into his or her graveyard.","3B","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kudzu","G","Enchant land
When enchanted land becomes tapped, destroy it. That land's controller attaches Kudzu to a land of his or her choice.","1GG","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Helm of the Ghastlord","UB","Enchant creature
As long as enchanted creature is blue, it gets +1/+1 and has ""Whenever this creature deals damage to an opponent, draw a card.""
As long as enchanted creature is black, it gets +1/+1 and has ""Whenever this creature deals damage to an opponent, that player discards a card.""","3(U/B)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mortivore","*","*","B","Mortivore's power and toughness are each equal to the number of creature cards in all graveyards.
{B}: Regenerate Mortivore.","2BB","Creature - Lhurgoyf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Memory's Journey","U","Target player shuffles up to three target cards from his or her graveyard into his or her library.
Flashback {G} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Saprazzan Breaker","3","3","U","{U}: Put the top card of your library into your graveyard. If that card is a land card, Saprazzan Breaker is unblockable this turn.","4U","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Royal Decree","W","Cumulative upkeep {W} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Whenever a Swamp, Mountain, black permanent, or red permanent becomes tapped, Royal Decree deals 1 damage to that permanent's controller.","2WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gravepurge","B","Put any number of target creature cards from your graveyard on top of your library.
Draw a card.","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tallowisp","1","3","W","Whenever you cast a Spirit or Arcane spell, you may search your library for an Aura card with enchant creature, reveal it, and put it into your hand. If you do, shuffle your library.","1W","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Empyrial Archangel","5","8","WUG","Flying, shroud
All damage that would be dealt to you is dealt to Empyrial Archangel instead.","4GWWU","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brutal Deceiver","2","2","R","{1}: Look at the top card of your library.
{2}: Reveal the top card of your library. If it's a land card, Brutal Deceiver gets +1/+0 and gains first strike until end of turn. Activate this ability only once each turn.","2R","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stone Spirit","4","3","R","Stone Spirit can't be blocked by creatures with flying.","4R","Creature - Elemental Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mass Appeal","U","Draw a card for each Human you control.","2U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Monk Realist","1","1","W","When Monk Realist enters the battlefield, destroy target enchantment.","1W","Creature - Human Monk Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Carom","W","The next 1 damage that would be dealt to target creature this turn is dealt to another target creature instead.
Draw a card.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Runed Halo","W","As Runed Halo enters the battlefield, name a card.
You have protection from the chosen name. (You can't be targeted, dealt damage, or enchanted by anything with that name.)","WW","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Crenellated Wall","0","4","Defender (This creature can't attack.)
{T}: Target creature gets +0/+4 until end of turn.","4","Artifact Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thistledown Duo","2","2","WU","Whenever you cast a white spell, Thistledown Duo gets +1/+1 until end of turn.
Whenever you cast a blue spell, Thistledown Duo gains flying until end of turn.","2(W/U)","Creature - Kithkin Soldier Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Great Wall","W","Creatures with plainswalk can be blocked as though they didn't have plainswalk.","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Copper Carapace","Equipped creature gets +2/+2 and can't block.
Equip {3} ({3}: Attach to target creature you control. Equip only as a sorcery.)","1","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desperate Research","B","Name a card other than a basic land card. Then reveal the top seven cards of your library and put all of them with that name into your hand. Exile the rest.","1B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Provoke","G","Untap target creature you don't control. That creature blocks this turn if able.
Draw a card.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blockbuster","R","{1}{R}, Sacrifice Blockbuster: Blockbuster deals 3 damage to each tapped creature and each player.","3RR","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jolrael's Centaur","2","2","G","Shroud (This permanent can't be the target of spells or abilities.)
Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)","1GG","Creature - Centaur Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Errantry","R","Enchant creature
Enchanted creature gets +3/+0 and can only attack alone.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Irini Sengir","2","2","B","Green enchantment spells and white enchantment spells cost {2} more to cast.","2BB","Legendary Creature - Vampire Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Latulla's Orders","R","Flash
Enchant creature
Whenever enchanted creature deals combat damage to defending player, you may destroy target artifact that player controls.","1R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Foul Familiar","3","1","B","Foul Familiar can't block.
{B}, Pay 1 life: Return Foul Familiar to its owner's hand.","2B","Creature - Spirit");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seat of the Synod","(Seat of the Synod isn't a spell.)
{T}: Add {U} to your mana pool.",NULL,"Artifact Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pale Bears","2","2","G","Islandwalk","2G","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Masked Gorgon","5","5","B","Green creatures and white creatures have protection from Gorgons.
Threshold - Masked Gorgon has protection from green and from white as long as seven or more cards are in your graveyard.","4B","Creature - Gorgon");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Shapeshifter","*","7-*","As Shapeshifter enters the battlefield, choose a number between 0 and 7.
At the beginning of your upkeep, you may choose a number between 0 and 7.
Shapeshifter's power is equal to the last chosen number and its toughness is equal to 7 minus that number.","6","Artifact Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Scattershot Archer","1","2","G","{T}: Scattershot Archer deals 1 damage to each creature with flying.","G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tahngarth, Talruum Hero","4","4","R","Vigilance
{1}{R}, {T}: Tahngarth, Talruum Hero deals damage equal to its power to target creature. That creature deals damage equal to its power to Tahngarth.","3RR","Legendary Creature - Minotaur Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Child of Thorns","1","1","G","Sacrifice Child of Thorns: Target creature gets +1/+1 until end of turn.","G","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kelsinko Ranger","1","1","W","{1}{W}: Target green creature gains first strike until end of turn.","W","Creature - Human");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wei Night Raiders","2","2","B","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Whenever Wei Night Raiders deals damage to an opponent, that player discards a card.","2BB","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Extractor Demon","5","5","B","Flying
Whenever another creature leaves the battlefield, you may have target player put the top two cards of his or her library into his or her graveyard.
Unearth {2}{B} ({2}{B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","4BB","Creature - Demon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rough // Tumble","R","Rough deals 2 damage to each creature without flying.
---
Tumble deals 6 damage to each creature with flying.","1R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dross Harvester","4","4","B","Protection from white
At the beginning of your end step, you lose 4 life.
Whenever a creature dies, you gain 2 life.","1BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raka Sanctuary","R","At the beginning of your upkeep, if you control a white or blue permanent, Raka Sanctuary deals 1 damage to target creature. If you control a white permanent and a blue permanent, Raka Sanctuary deals 3 damage to that creature instead.","2R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of the Honored Dead","3","5","W","Flying
Whenever Kami of the Honored Dead is dealt damage, you gain that much life.
Soulshift 6 (When this creature dies, you may return target Spirit card with converted mana cost 6 or less from your graveyard to your hand.)","5WW","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Voices from the Void","B","Domain - Target player discards a card for each basic land type among lands you control.","4B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cloud Pirates","1","1","U","Flying
Cloud Pirates can block only creatures with flying.","U","Creature - Human Pirate");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Keeper of Progenitus","1","3","G","Whenever a player taps a Mountain, Forest, or Plains for mana, that player adds one mana to his or her mana pool of any type that land produced.","3G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rowen","G","Reveal the first card you draw each turn. Whenever you reveal a basic land card this way, draw a card.","2GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jade Leech","5","5","G","Green spells you cast cost {G} more to cast.","2GG","Creature - Leech");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Crosis's Attendant","3","3","{1}, Sacrifice Crosis's Attendant: Add {U}{B}{R} to your mana pool.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Darting Merfolk","1","1","U","{U}: Return Darting Merfolk to its owner's hand.","1U","Creature - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Secret Agent","2","2","R","First strike
At the beginning of your upkeep, reveal a card from your hand at random.","2R","Creature - Goblin Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Debt of Loyalty","W","Regenerate target creature. You gain control of that creature if it regenerates this way.","1WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Weathered Wayfarer","1","1","W","{W}, {T}: Search your library for a land card, reveal it, and put it into your hand. Then shuffle your library. Activate this ability only if an opponent controls more lands than you.","W","Creature - Human Nomad Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Puppet Strings","{2}, {T}: You may tap or untap target creature.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunseed Nurturer","1","1","W","At the beginning of your end step, if you control a creature with power 5 or greater, you may gain 2 life.
{T}: Add {1} to your mana pool.","2W","Creature - Human Druid Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Elephant","3","3",NULL,NULL,"Creature - Elephant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Furnace Whelp","2","2","R","Flying
{R}: Furnace Whelp gets +1/+0 until end of turn.","2RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("See Beyond","U","Draw two cards, then shuffle a card from your hand into your library.","1U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skeletal Kathari","3","2","B","Flying
{B}, Sacrifice a creature: Regenerate Skeletal Kathari.","4B","Creature - Bird Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Worldgorger Dragon","7","7","R","Flying, trample
When Worldgorger Dragon enters the battlefield, exile all other permanents you control.
When Worldgorger Dragon leaves the battlefield, return the exiled cards to the battlefield under their owners' control.","3RRR","Creature - Nightmare Dragon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chromatic Star","{1}, {T}, Sacrifice Chromatic Star: Add one mana of any color to your mana pool.
When Chromatic Star is put into a graveyard from the battlefield, draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Auratouched Mage","3","3","W","When Auratouched Mage enters the battlefield, search your library for an Aura card that could enchant it. If Auratouched Mage is still on the battlefield, put that Aura card onto the battlefield attached to it. Otherwise, reveal the Aura card and put it into your hand. Then shuffle your library.","5W","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Escape Artist","1","1","U","Escape Artist is unblockable.
{U}, Discard a card: Return Escape Artist to its owner's hand.","1U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shield of the Righteous","WU","Equipped creature gets +0/+2 and has vigilance.
Whenever equipped creature blocks a creature, that creature doesn't untap during its controller's next untap step.
Equip {2}","WU","Artifact - Equipment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Psychogenic Probe","Whenever a spell or ability causes a player to shuffle his or her library, Psychogenic Probe deals 2 damage to him or her.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Imperiosaur","5","5","G","Spend only mana produced by basic lands to cast Imperiosaur.","2GG","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Traproot Kami","0","*","G","Defender; reach (This creature can block creatures with flying.)
Traproot Kami's toughness is equal to the number of Forests on the battlefield.","G","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul's Fire","R","Target creature you control on the battlefield deals damage equal to its power to target creature or player.","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ali Baba","1","1","R","{R}: Tap target Wall.","R","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Vanguard","1","1","G","Whenever another Elf enters the battlefield, put a +1/+1 counter on Elvish Vanguard.","1G","Creature - Elf Warrior");
INSERT INTO cards(name,color,cardtext,type,manacost) VALUES("Garruk Wildspeaker","G","+1: Untap two target lands.
-1: Put a 3/3 green Beast creature token onto the battlefield.
-4: Creatures you control get +3/+3 and gain trample until end of turn.","Planeswalker - Garruk","2GG");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Curfew","U","Each player returns a creature he or she controls to its owner's hand.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stronghold Discipline","B","Each player loses 1 life for each creature he or she controls.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Opaline Sliver","2","2","WU","All Slivers have ""Whenever this permanent becomes the target of a spell an opponent controls, you may draw a card.""","1WU","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rune-Cervin Rider","2","2","W","Flying
{(g/w){(g/w)}: Rune-Cervin Rider gets +1/+1 until end of turn.","3W","Creature - Elf Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ravenous Skirge","1","1","B","Flying
Whenever Ravenous Skirge attacks, it gets +2/+0 until end of turn.","2B","Creature - Imp");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teardrop Kami","1","1","U","Sacrifice Teardrop Kami: You may tap or untap target creature.","U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gravity Well","G","Whenever a creature with flying attacks, it loses flying until end of turn.","1GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Insurrection","R","Untap all creatures and gain control of them until end of turn. They gain haste until end of turn.","5RRR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Citanul Centaurs","6","3","G","Shroud (This permanent can't be the target of spells or abilities.)
Echo {3}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","3G","Creature - Centaur");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ferrovore","2","2","R","{R}, Sacrifice an artifact: Ferrovore gets +3/+0 until end of turn.","2R","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Spiral","G","Return target permanent card from your graveyard to your hand. (A permanent card is an artifact, creature, enchantment, land, or planeswalker card.)","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sea Snidd","3","3","U","{T}: Target land becomes the basic land type of your choice until end of turn.","4U","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Resuscitate","G","Until end of turn, creatures you control gain ""{1}: Regenerate this creature.""","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heroes Remembered","W","You gain 20 life.
Suspend 10-{W} (Rather than cast this card from your hand, you may pay {W} and exile it with ten time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","6WWW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Razia, Boros Archangel","6","3","WR","Flying, vigilance, haste
{T}: The next 3 damage that would be dealt to target creature you control this turn is dealt to another target creature instead.","4RRWW","Legendary Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Morphling","3","3","U","{U}: Untap Morphling.
{U}: Morphling gains flying until end of turn.
{U}: Morphling gains shroud until end of turn. (It can't be the target of spells or abilities.)
{1}: Morphling gets +1/-1 until end of turn.
{1}: Morphling gets -1/+1 until end of turn.","3UU","Creature - Shapeshifter");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drudge Skeletons","1","1","B","{B}: Regenerate Drudge Skeletons. (The next time this creature would be destroyed this turn, it isn't. Instead tap it, remove all damage from it, and remove it from combat.)","1B","Creature - Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unworthy Dead","1","1","B","{B}: Regenerate Unworthy Dead.","1B","Creature - Skeleton");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Talisman of Impulse","{T}: Add {1} to your mana pool.
{T}: Add {R} or {G} to your mana pool. Talisman of Impulse deals 1 damage to you.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rampart Crawler","1","1","B","Rampart Crawler can't be blocked by Walls.","B","Creature - Lizard Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lurking Predators","G","Whenever an opponent casts a spell, reveal the top card of your library. If it's a creature card, put it onto the battlefield. Otherwise, you may put that card on the bottom of your library.","4GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Death's Duet","B","Return two target creature cards from your graveyard to your hand.","2B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eon Hub","Players skip their upkeep steps.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rally the Troops","W","Cast Rally the Troops only during the declare attackers step and only if you've been attacked this step.
Untap all creatures you control.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Markov Warlord","4","4","R","Haste
When Markov Warlord enters the battlefield, up to two target creatures can't block this turn.","5R","Creature - Vampire Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Scroll Rack","{1}, {T}: Exile any number of cards from your hand face down. Put that many cards from the top of your library into your hand. Then look at the exiled cards and put them on top of your library in any order.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primeval Titan","6","6","G","Trample
Whenever Primeval Titan enters the battlefield or attacks, you may search your library for up to two land cards, put them onto the battlefield tapped, then shuffle your library.","4GG","Creature - Giant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unstable Footing","R","Kicker {3}{R} (You may pay an additional {3}{R} as you cast this spell.)
Damage can't be prevented this turn. If Unstable Footing was kicked, it deals 5 damage to target player.","R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Exiled Doomsayer","1","2","W","All morph costs cost {2} more. (This doesn't affect the cost to cast creature spells face down.)","1W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fanning the Flames","R","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Fanning the Flames deals X damage to target creature or player.","XRR","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Marrow Shards","W","({(w/p)} can be paid with either {W} or 2 life.)
Marrow Shards deals 1 damage to each attacking creature.","(W/P)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Planar Void","B","Whenever another card is put into a graveyard from anywhere, exile that card.","B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Denizen of the Deep","11","11","U","When Denizen of the Deep enters the battlefield, return each other creature you control to its owner's hand.","6UU","Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mudbrawler Raiders","3","3","RG","Mudbrawler Raiders can't be blocked by blue creatures.","2(R/G)(R/G)","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Opposition","0","6","R","Defender (This creature can't attack.)
{1}: Wall of Opposition gets +1/+0 until end of turn.","3RR","Creature - Wall");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Iron Myr","1","1","{T}: Add {R} to your mana pool.","2","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bazaar Trader","1","1","R","{T}: Target player gains control of target artifact, creature, or land you control.","1R","Creature - Goblin");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Clifftop Retreat","Clifftop Retreat enters the battlefield tapped unless you control a Mountain or a Plains.
{T}: Add {R} or {W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sisters of the Flame","2","2","R","{T}: Add {R} to your mana pool.","1RR","Creature - Human Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primal Plasma","*","*","U","As Primal Plasma enters the battlefield, it becomes your choice of a 3/3 creature, a 2/2 creature with flying, or a 1/6 creature with defender.","3U","Creature - Elemental Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strength of Isolation","W","Enchant creature
Enchanted creature gets +1/+2 and has protection from black.
Madness {W} (If you discard this card, you may cast it for its madness cost instead of putting it into your graveyard.)","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Giant Ambush Beetle","4","3","BRG","Haste
When Giant Ambush Beetle enters the battlefield, you may have target creature block it this turn if able.","3(B/G)R","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Metamorphose","U","Put target permanent an opponent controls on top of its owner's library. That opponent may put an artifact, creature, enchantment, or land card from his or her hand onto the battlefield.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surreal Memoir","R","Return an instant card at random from your graveyard to your hand.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Volt Charge","R","Volt Charge deals 3 damage to target creature or player. Proliferate. (You choose any number of permanents and/or players with counters on them, then give each another counter of a kind already there.)","2R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Armored Pegasus","1","2","W","Flying","1W","Creature - Pegasus");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Proper Burial","W","Whenever a creature you control dies, you gain life equal to that creature's toughness.","3W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tower of the Magistrate","{T}: Add {1} to your mana pool.
{1}, {T}: Target creature gains protection from artifacts until end of turn.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silver Knight","2","2","W","First strike, protection from red","WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Breathstealer's Crypt","UB","If a player would draw a card, instead he or she draws a card and reveals it. If it's a creature card, that player discards it unless he or she pays 3 life.","2UB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Lead-Belly Chimera","2","2","Trample
Sacrifice Lead-Belly Chimera: Put a +2/+2 counter on target Chimera creature. It gains trample. (This effect lasts indefinitely.)","4","Artifact Creature - Chimera");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Submerge","U","If an opponent controls a Forest and you control an Island, you may cast Submerge without paying its mana cost.
Put target creature on top of its owner's library.","4U","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Isolated Chapel","Isolated Chapel enters the battlefield tapped unless you control a Plains or a Swamp.
{T}: Add {W} or {B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Savage Firecat","0","0","R","Trample
Savage Firecat enters the battlefield with seven +1/+1 counters on it.
Whenever you tap a land for mana, remove a +1/+1 counter from Savage Firecat.","3RR","Creature - Elemental Cat");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Marshaling Cry","W","Creatures you control get +1/+1 and gain vigilance until end of turn.
Cycling {2} ({2}, Discard this card: Draw a card.)
Flashback {3}{W} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","1WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Márton Stromgald","1","1","R","Whenever Márton Stromgald attacks, other attacking creatures get +1/+1 until end of turn for each attacking creature other than Márton Stromgald.
Whenever Márton Stromgald blocks, other blocking creatures get +1/+1 until end of turn for each blocking creature other than Márton Stromgald.","2RR","Legendary Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mirari's Wake","WG","Creatures you control get +1/+1.
Whenever you tap a land for mana, add one mana to your mana pool of any type that land produced.","3GW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Legerdemain","U","Exchange control of target artifact or creature and another target permanent that shares one of those types with it. (This effect lasts indefinitely.)","2UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Phyrexian Monitor","2","2","B","{B}: Regenerate Phyrexian Monitor.","3B","Creature - Skeleton");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Trade Caravan","1","1","W","At the beginning of your upkeep, put a currency counter on Trade Caravan.
Remove two currency counters from Trade Caravan: Untap target basic land. Activate this ability only during an opponent's upkeep.","W","Creature - Human Nomad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thorntooth Witch","3","4","B","Whenever you cast a Treefolk spell, you may have target creature get +3/-3 until end of turn.","5B","Creature - Treefolk Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nin, the Pain Artist","1","1","UR","{X}{U}{R}, {T}: Nin, the Pain Artist deals X damage to target creature. That creature's controller draws X cards.","UR","Legendary Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Shade","2","1","B","{B}: Nantuko Shade gets +1/+1 until end of turn.","BB","Creature - Insect Shade");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Summit Apes","5","2","G","As long as you control a Mountain, Summit Apes can't be blocked except by two or more creatures.","3G","Creature - Ape");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Emissary","2","1","G","When Viridian Emissary dies, you may search your library for a basic land card, put it onto the battlefield tapped, then shuffle your library.","1G","Creature - Elf Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Part Water","U","X target creatures gain islandwalk until end of turn.","XXU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ekundu Cyclops","3","4","R","If a creature you control attacks, Ekundu Cyclops also attacks if able.","3R","Creature - Cyclops");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Choking Fumes","W","Put a -1/-1 counter on each attacking creature.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bramblesnap","1","1","G","Trample
Tap an untapped creature you control: Bramblesnap gets +1/+1 until end of turn.","1G","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Veteran of the Depths","2","2","W","Whenever Veteran of the Depths becomes tapped, you may put a +1/+1 counter on it.","3W","Creature - Merfolk Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Piledriver","1","2","R","Protection from blue
Whenever Goblin Piledriver attacks, it gets +2/+0 until end of turn for each other attacking Goblin.","1R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shield Dancer","1","3","W","{2}{W}: The next time target attacking creature would deal combat damage to Shield Dancer this turn, that creature deals that damage to itself instead.","2W","Creature - Human Rebel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Miasma","B","Target player discards a card. If a land card is discarded this way, return Psychic Miasma to its owner's hand.","1B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mischievous Quanar","3","3","U","{3}{U}{U}: Turn Mischievous Quanar face down.
Morph {1}{U}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Mischievous Quanar is turned face up, copy target instant or sorcery spell. You may choose new targets for that copy.","4U","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Outrider en-Kor","2","2","W","Flanking (Whenever a creature without flanking blocks this creature, the blocking creature gets -1/-1 until end of turn.)
{0}: The next 1 damage that would be dealt to Outrider en-Kor this turn is dealt to target creature you control instead.","2W","Creature - Kor Rebel Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Withering Wisps","B","At the beginning of the end step, if no creatures are on the battlefield, sacrifice Withering Wisps.
{B}: Withering Wisps deals 1 damage to each creature and each player. Activate this ability no more times each turn than the number of snow Swamps you control.","1BB","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wojek Siren","W","Radiance - Target creature and each other creature that shares a color with it get +1/+1 until end of turn.","W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lithatog","1","2","RG","Sacrifice an artifact: Lithatog gets +1/+1 until end of turn.
Sacrifice a land: Lithatog gets +1/+1 until end of turn.","1RG","Creature - Atog");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Agadeem Occultist","0","2","B","{T}: Put target creature card from an opponent's graveyard onto the battlefield under your control if its converted mana cost is less than or equal to the number of Allies you control.","2B","Creature - Human Shaman Ally");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tetsuo Umezawa","3","3","UBR","Tetsuo Umezawa can't be the target of Aura spells.
{U}{B}{B}{R}, {T}: Destroy target tapped or blocking creature.","UBR","Legendary Creature - Human Archer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Orim's Chant","W","Kicker {W} (You may pay an additional {W} as you cast this spell.)
Target player can't cast spells this turn.
If Orim's Chant was kicked, creatures can't attack this turn.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Malignant Growth","UG","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of your upkeep, put a growth counter on Malignant Growth.
At the beginning of each opponent's draw step, that player draws an additional card for each growth counter on Malignant Growth, then Malignant Growth deals damage to the player equal to the number of cards he or she drew this way.","3GU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Amrou Kithkin","1","1","W","Amrou Kithkin can't be blocked by creatures with power 3 or greater.","WW","Creature - Kithkin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Absorb Vis","B","Target player loses 4 life and you gain 4 life.
Basic landcycling {1}{B} ({1}{B}, Discard this card: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.)","6B","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Phyrexian Lens","{T}, Pay 1 life: Add one mana of any color to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Haunting Apparition","1+*","2","UB","Flying
As Haunting Apparition enters the battlefield, choose an opponent.
Haunting Apparition's power is equal to 1 plus the number of green creature cards in the chosen player's graveyard.","1UB","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Withstand","W","Prevent the next 3 damage that would be dealt to target creature or player this turn.
Draw a card.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unseen Walker","1","1","G","Forestwalk
{1}{G}{G}: Target creature gains forestwalk until end of turn.","1G","Creature - Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Firewalker","2","2","W","Protection from red
Whenever a player casts a red spell, you may gain 1 life.","WW","Creature - Kor Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sheoldred, Whispering One","6","6","B","Swampwalk
At the beginning of your upkeep, return target creature card from your graveyard to the battlefield.
At the beginning of each opponent's upkeep, that player sacrifices a creature.","5BB","Legendary Creature - Praetor");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Chaos Charm","R","Choose one - Destroy target Wall; or Chaos Charm deals 1 damage to target creature; or target creature gains haste until end of turn.","R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Strider Harness","Equipped creature gets +1/+1 and has haste.
Equip {1}","3","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blessing of Leeches","B","Flash
Enchant creature
At the beginning of your upkeep, you lose 1 life.
{0}: Regenerate enchanted creature.","2B","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Into the Fray","R","Target creature attacks this turn if able.
Splice onto Arcane {R} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","R","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crossbow Ambush","G","Creatures you control gain reach until end of turn. (They can block creatures with flying.)","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shepherd of Rot","1","1","B","{T}: Each player loses 1 life for each Zombie on the battlefield.","1B","Creature - Zombie Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Torture Chamber","At the beginning of your upkeep, put a pain counter on Torture Chamber.
At the beginning of your end step, Torture Chamber deals damage to you equal to the number of pain counters on it.
{1}, {T}, Remove all pain counters from Torture Chamber: Torture Chamber deals damage to target creature equal to the number of pain counters removed this way.","3","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dreamstone Hedron","{T}: Add {3} to your mana pool.
{3}, {T}, Sacrifice Dreamstone Hedron: Draw three cards.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Incandescent Soulstoke","2","2","R","Other Elemental creatures you control get +1/+1.
{1}{R}, {T}: You may put an Elemental creature card from your hand onto the battlefield. That creature gains haste until end of turn. Sacrifice it at the beginning of the next end step.","2R","Creature - Elemental Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Balduvian Barbarians","3","2","R",NULL,"1RR","Creature - Human Barbarian");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scourge of the Nobilis","WR","Enchant creature
As long as enchanted creature is red, it gets +1/+1 and has ""{(r/w)}: This creature gets +1/+0 until end of turn.""
As long as enchanted creature is white, it gets +1/+1 and has lifelink. (Damage dealt by the creature also causes its controller to gain that much life.)","2(R/W)","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voice of the Woods","2","2","G","Tap five untapped Elves you control: Put a 7/7 green Elemental creature token with trample onto the battlefield.","3GG","Creature - Elf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kodama of the Center Tree","*","*","G","Kodama of the Center Tree's power and toughness are each equal to the number of Spirits you control.
Kodama of the Center Tree has soulshift X, where X is the number of Spirits you control. (When this creature dies, you may return target Spirit card with converted mana cost X or less from your graveyard to your hand.)","4G","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tattermunge Duo","2","3","RG","Whenever you cast a red spell, Tattermunge Duo gets +1/+1 until end of turn.
Whenever you cast a green spell, Tattermunge Duo gains forestwalk until end of turn.","2(R/G)","Creature - Goblin Warrior Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feudkiller's Verdict","W","You gain 10 life. Then if you have more life than an opponent, put a 5/5 white Giant Warrior creature token onto the battlefield.","4WW","Tribal Sorcery - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dwarven Sea Clan","1","1","R","{T}: Choose target attacking or blocking creature whose controller controls an Island. Dwarven Sea Clan deals 2 damage to that creature at end of combat. Activate this ability only before the end of combat step.","2R","Creature - Dwarf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bar the Door","W","Creatures you control get +0/+4 until end of turn.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crack the Earth","R","Each player sacrifices a permanent.","R","Sorcery - Arcane");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Vessel of Endless Rest","When Vessel of Endless Rest enters the battlefield, put target card from a graveyard on the bottom of its owner's library.
{T}: Add one mana of any color to your mana pool.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Norwood Warrior","2","2","G","Whenever Norwood Warrior becomes blocked, it gets +1/+1 until end of turn.","2G","Creature - Elf Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Blinkmoth Well","{T}: Add {1} to your mana pool.
{2}, {T}: Tap target noncreature artifact.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hell's Caretaker","1","1","B","{T}, Sacrifice a creature: Return target creature card from your graveyard to the battlefield. Activate this ability only during your upkeep.","3B","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orzhov Pontiff","1","1","WB","Haunt (When this creature dies, exile it haunting target creature.)
When Orzhov Pontiff enters the battlefield or the creature it haunts dies, choose one - Creatures you control get +1/+1 until end of turn; or creatures you don't control get -1/-1 until end of turn.","1WB","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Myojin of Life's Web","8","8","G","Myojin of Life's Web enters the battlefield with a divinity counter on it if you cast it from your hand.
Myojin of Life's Web is indestructible as long as it has a divinity counter on it.
Remove a divinity counter from Myojin of Life's Web: Put any number of creature cards from your hand onto the battlefield.","6GGG","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nema Siltlurker","3","5","G",NULL,"4G","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grozoth","9","9","U","Defender (This creature can't attack.)
When Grozoth enters the battlefield, you may search your library for any number of cards that have converted mana cost 9, reveal them, and put them into your hand. If you do, shuffle your library.
{4}: Grozoth loses defender until end of turn.
Transmute {1}{U}{U} ({1}{U}{U}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","6UUU","Creature - Leviathan");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sylvan Bounty","G","Target player gains 8 life.
Basic landcycling {1}{G} ({1}{G}, Discard this card: Search your library for a basic land card, reveal it, and put it into your hand. Then shuffle your library.)","5G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Verdant Embrace","G","Enchant creature
Enchanted creature gets +3/+3 and has ""At the beginning of each upkeep, put a 1/1 green Saproling creature token onto the battlefield.""","3GG","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sudden Death","B","Split second (As long as this spell is on the stack, players can't cast spells or activate abilities that aren't mana abilities.)
Target creature gets -4/-4 until end of turn.","1BB","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spike Drone","0","0","G","Spike Drone enters the battlefield with a +1/+1 counter on it.
{2}, Remove a +1/+1 counter from Spike Drone: Put a +1/+1 counter on target creature.","G","Creature - Spike Drone");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Shaman","2","2","G","When Viridian Shaman enters the battlefield, destroy target artifact.","2G","Creature - Elf Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dissipate","U","Counter target spell. If that spell is countered this way, exile it instead of putting it into its owner's graveyard.","1UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Baki's Curse","U","Baki's Curse deals 2 damage to each creature for each Aura attached to that creature.","2UU","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Test of Faith","W","Prevent the next 3 damage that would be dealt to target creature this turn, and put a +1/+1 counter on that creature for each 1 damage prevented this way.","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ashcoat Bear","2","2","G","Flash (You may cast this spell any time you could cast an instant.)","1G","Creature - Bear");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Iona, Shield of Emeria","7","7","W","Flying
As Iona, Shield of Emeria enters the battlefield, choose a color.
Your opponents can't cast spells of the chosen color.","6WWW","Legendary Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Standstill","U","When a player casts a spell, sacrifice Standstill. If you do, each of that player's opponents draws three cards.","1U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wet Willie of the Damned","B","Wet Willie of the Damned deals 2{1/2} damage to target creature or player and you gain 2{1/2} life.","2BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Storm Cauldron","Each player may play an additional land during each of his or her turns.
Whenever a land is tapped for mana, return it to its owner's hand.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Momentum","G","Enchant creature
At the beginning of your upkeep, you may put a growth counter on Momentum.
Enchanted creature gets +1/+1 for each growth counter on Momentum.","2G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Akki Avalanchers","1","1","R","Sacrifice a land: Akki Avalanchers gets +2/+0 until end of turn. Activate this ability only once each turn.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crown of Convergence","Play with the top card of your library revealed.
As long as the top card of your library is a creature card, creatures you control that share a color with that card get +1/+1.
{G}{W}: Put the top card of your library on the bottom of your library.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bladewing the Risen","4","4","BR","Flying
When Bladewing the Risen enters the battlefield, you may return target Dragon permanent card from your graveyard to the battlefield.
{B}{R}: Dragon creatures get +1/+1 until end of turn.","3BBRR","Legendary Creature - Zombie Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grand Abolisher","2","2","W","During your turn, your opponents can't cast spells or activate abilities of artifacts, creatures, or enchantments.","WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhox War Monk","3","4","WUG","Lifelink","GWU","Creature - Rhino Monk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slith Firewalker","1","1","R","Haste
Whenever Slith Firewalker deals combat damage to a player, put a +1/+1 counter on it.","RR","Creature - Slith");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Valakut, the Molten Pinnacle","Valakut, the Molten Pinnacle enters the battlefield tapped.
Whenever a Mountain enters the battlefield under your control, if you control at least five other Mountains, you may have Valakut, the Molten Pinnacle deal 3 damage to target creature or player.
{T}: Add {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Black Market","B","Whenever a creature dies, put a charge counter on Black Market.
At the beginning of your precombat main phase, add {B} to your mana pool for each charge counter on Black Market.","3BB","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Isle of Vesuva","Whenever a nontoken creature enters the battlefield, its controller puts a token onto the battlefield that's a copy of that creature.
Whenever you roll {C}, destroy target creature and all other creatures with the same name as that creature.",NULL,"Plane - Dominaria");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cenn's Enlistment","W","Put two 1/1 white Kithkin Soldier creature tokens onto the battlefield.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","3W","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Captain of the Mists","2","3","U","Whenever another Human enters the battlefield under your control, untap Captain of the Mists.
{1}{U}, {T}: You may tap or untap target permanent.","2U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mindwrack Liege","4","4","UR","Other blue creatures you control get +1/+1.
Other red creatures you control get +1/+1.
{(u/r){(u/r){(u/r){(u/r)}: You may put a blue or red creature card from your hand onto the battlefield.","3(U/R)(U/R)(U/R)","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vulshok War Boar","5","5","R","When Vulshok War Boar enters the battlefield, sacrifice it unless you sacrifice an artifact.","2RR","Creature - Boar Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Conversion","W","At the beginning of your upkeep, sacrifice Conversion unless you pay {W}{W}.
All Mountains are Plains.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hikari, Twilight Guardian","4","4","W","Flying
Whenever you cast a Spirit or Arcane spell, you may exile Hikari, Twilight Guardian. If you do, return it to the battlefield under its owner's control at the beginning of the next end step.","3WW","Legendary Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sower of Temptation","2","2","U","Flying
When Sower of Temptation enters the battlefield, gain control of target creature for as long as Sower of Temptation remains on the battlefield.","2UU","Creature - Faerie Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunting Cheetah","2","3","G","Whenever Hunting Cheetah deals damage to an opponent, you may search your library for a Forest card, reveal that card, put it into your hand, then shuffle your library.","2G","Creature - Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aven Liberator","2","3","W","Flying
Morph {3}{W} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Aven Liberator is turned face up, target creature you control gains protection from the color of your choice until end of turn.","2WW","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mana Vortex","U","When you cast Mana Vortex, counter it unless you sacrifice a land.
At the beginning of each player's upkeep, that player sacrifices a land.
When there are no lands on the battlefield, sacrifice Mana Vortex.","1UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zebra Unicorn","2","2","WG","Whenever Zebra Unicorn deals damage, you gain that much life.","2GW","Creature - Unicorn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Savage Beating","R","Cast Savage Beating only during your turn and only during combat.
Choose one - Creatures you control gain double strike until end of turn; or untap all creatures you control and after this phase, there is an additional combat phase.
Entwine {1}{R} (Choose both if you pay the entwine cost.)","3RR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Altar Of Dementia","Sacrifice a creature: Target player puts a number of cards equal to the sacrificed creature's power from the top of his or her library into his or her graveyard.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sanctum Gargoyle","2","3","W","Flying
When Sanctum Gargoyle enters the battlefield, you may return target artifact card from your graveyard to your hand.","3W","Artifact Creature - Gargoyle");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shivan Gorge","{T}: Add {1} to your mana pool.
{2}{R}, {T}: Shivan Gorge deals 1 damage to each opponent.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Dead","3","1","B","When Kjeldoran Dead enters the battlefield, sacrifice a creature.
{B}: Regenerate Kjeldoran Dead.","B","Creature - Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Unnerving Assault","UR","Creatures your opponents control get -1/-0 until end of turn if {U} was spent to cast Unnerving Assault, and creatures you control get +1/+0 until end of turn if {R} was spent to cast it. (Do both if {U}{R} was spent.)","2(U/R)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Final Punishment","B","Target player loses life equal to the damage already dealt to him or her this turn.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Constrictor","2","2","G","Swampwalk
{T}: Target black creature gets -2/-0 until end of turn.","3G","Creature - Snake");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Martyr of Spores","1","1","G","{1}, Reveal X green cards from your hand, Sacrifice Martyr of Spores: Target creature gets +X/+X until end of turn.","G","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lure of Prey","G","Cast Lure of Prey only if an opponent cast a creature spell this turn.
You may put a green creature card from your hand onto the battlefield.","2GG","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sun Droplet","Whenever you're dealt damage, put that many charge counters on Sun Droplet.
At the beginning of each upkeep, you may remove a charge counter from Sun Droplet. If you do, you gain 1 life.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fungal Bloom","G","{G}{G}: Put a spore counter on target Fungus.","GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dakkon Blackblade","*","*","WUB","Dakkon Blackblade's power and toughness are each equal to the number of lands you control.","2WUUB","Legendary Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stonehorn Dignitary","1","4","W","When Stonehorn Dignitary enters the battlefield, target opponent skips his or her next combat phase.","3W","Creature - Rhino Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gobhobbler Rats","2","2","BR","Hellbent - As long as you have no cards in hand, Gobhobbler Rats gets +1/+0 and has ""{B}: Regenerate Gobhobbler Rats.""","BR","Creature - Rat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Dimir Aqueduct","Dimir Aqueduct enters the battlefield tapped.
When Dimir Aqueduct enters the battlefield, return a land you control to its owner's hand.
{T}: Add {U}{B} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Azure Mage","2","1","U","{3}{U}: Draw a card.","1U","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rebuke","W","Destroy target attacking creature.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rejuvenate","G","You gain 6 life.
Cycling {2} ({2}, Discard this card: Draw a card.)","3G","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dreadwaters","U","Target player puts the top X cards of his or her library into his or her graveyard, where X is the number of lands you control.","3U","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sacred Foundry","({T}: Add {R} or {W} to your mana pool.)
As Sacred Foundry enters the battlefield, you may pay 2 life. If you don't, Sacred Foundry enters the battlefield tapped.",NULL,"Land - Mountain Plains");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Energy Vortex","U","As Energy Vortex enters the battlefield, choose an opponent.
At the beginning of your upkeep, remove all energy counters from Energy Vortex.
At the beginning of the chosen player's upkeep, Energy Vortex deals 3 damage to that player unless he or she pays {1} for each energy counter on Energy Vortex.
{X}: Put X energy counters on Energy Vortex. Activate this ability only during your upkeep.","3UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scourglass","W","{T}, Sacrifice Scourglass: Destroy all permanents except for artifacts and lands. Activate this ability only during your upkeep.","3WW","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grim Harvest","B","Return target creature card from your graveyard to your hand.
Recover {2}{B} (When a creature is put into your graveyard from the battlefield, you may pay {2}{B}. If you do, return this card from your graveyard to your hand. Otherwise, exile this card.)","1B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Survival Cache","W","You gain 2 life. Then if you have more life than an opponent, draw a card.
Rebound (If you cast this spell from your hand, exile it as it resolves. At the beginning of your next upkeep, you may cast this card from exile without paying its mana cost.)","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Seismic Assault","R","Discard a land card: Seismic Assault deals 2 damage to target creature or player.","RRR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Tithe","B","Each opponent loses 3 life. You gain life equal to the life lost this way.","3B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Escort","2","3","W","Banding (Any creatures with banding, and up to one without, can attack in a band. Bands are blocked as a group. If any creatures with banding you control are blocking or being blocked by a creature, you divide that creature's combat damage, not its controller, among any of the creatures it's being blocked by or is blocking.)","2WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Divine Light","W","Prevent all damage that would be dealt this turn to creatures you control.","W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hero's Resolve","W","Enchant creature
Enchanted creature gets +1/+5.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aerial Caravan","4","3","U","Flying
{1}{U}{U}: Exile the top card of your library. Until end of turn, you may play that card. (Reveal the card as you exile it.)","4UU","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cateran Persuader","2","1","B","{1}, {T}: Search your library for a Mercenary permanent card with converted mana cost 1 or less and put it onto the battlefield. Then shuffle your library.","BB","Creature - Human Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Evacuation","U","Return all creatures to their owners' hands.","3UU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Akroma's Memorial","Creatures you control have flying, first strike, vigilance, trample, haste, and protection from black and from red.","7","Legendary Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of Vitality","G","If Leyline of Vitality is in your opening hand, you may begin the game with it on the battlefield.
Creatures you control get +0/+1.
Whenever a creature enters the battlefield under your control, you may gain 1 life.","2GG","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Chimeric Egg","Whenever an opponent casts a nonartifact spell, put a charge counter on Chimeric Egg.
Remove three charge counters from Chimeric Egg: Chimeric Egg becomes a 6/6 Construct artifact creature with trample until end of turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Krosan Cloudscraper","13","13","G","At the beginning of your upkeep, sacrifice Krosan Cloudscraper unless you pay {G}{G}.
Morph {7}{G}{G} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","7GGG","Creature - Beast Mutant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Idyllic Tutor","W","Search your library for an enchantment card, reveal it, and put it into your hand. Then shuffle your library.","2W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Beacon of Unrest","B","Put target artifact or creature card from a graveyard onto the battlefield under your control. Shuffle Beacon of Unrest into its owner's library.","3BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kongming's Contraptions","2","4","W","{T}: Kongming's Contraptions deals 2 damage to target attacking creature. Activate this ability only during the declare attackers step and only if you've been attacked this step.","3W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bloodcurdling Scream","B","Target creature gets +X/+0 until end of turn.","XB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Howl from Beyond","B","Target creature gets +X/+0 until end of turn.","XB","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Inspiration","U","Target player draws two cards.","3U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mercenaries","3","3","W","{3}: The next time Mercenaries would deal damage to you this turn, prevent that damage. Any player may activate this ability.","3W","Creature - Human Mercenary");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leap of Flame","UR","Replicate {U}{R} (When you cast this spell, copy it for each time you paid its replicate cost. You may choose new targets for the copies.)
Target creature gets +1/+0 and gains flying and first strike until end of turn.","UR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dismember","B","({(b/p)} can be paid with either {B} or 2 life.)
Target creature gets -5/-5 until end of turn.","1(B/P)(B/P)","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sideswipe","R","You may change any targets of target Arcane spell.","1R","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Magma Jet","R","Magma Jet deals 2 damage to target creature or player.
Scry 2. (To scry 2, look at the top two cards of your library, then put any number of them on the bottom of your library and the rest on top in any order.)","1R","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Sparring Golem","2","2","Whenever Sparring Golem becomes blocked, it gets +1/+1 until end of turn for each creature blocking it.","3","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Baru, Fist of Krosa","4","4","G","Whenever a Forest enters the battlefield, green creatures you control get +1/+1 and gain trample until end of turn.
Grandeur - Discard another card named Baru, Fist of Krosa: Put an X/X green Wurm creature token onto the battlefield, where X is the number of lands you control.","3GG","Legendary Creature - Human Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Jailer","2","2","R","Mogg Jailer can't attack if defending player controls an untapped creature with power 2 or less.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Storm Spirit","3","3","WUG","Flying
{T}: Storm Spirit deals 2 damage to target creature.","3GWU","Creature - Elemental Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Furnace Dragon","5","5","R","Affinity for artifacts (This spell costs {1} less to cast for each artifact you control.)
Flying
When Furnace Dragon enters the battlefield, if you cast it from your hand, exile all artifacts.","6RRR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Heat of Battle","R","Whenever a creature blocks, Heat of Battle deals 1 damage to that creature's controller.","1R","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Berserker","2","2","R","First strike, haste","3R","Creature - Goblin Berserker");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clout of the Dominus","UR","Enchant creature
As long as enchanted creature is blue, it gets +1/+1 and has shroud. (It can't be the target of spells or abilities.)
As long as enchanted creature is red, it gets +1/+1 and has haste.","(U/R)","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blizzard","G","Cast Blizzard only if you control a snow land.
Cumulative upkeep {2} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Creatures with flying don't untap during their controllers' untap steps.","GG","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thornscape Familiar","2","1","G","Red spells and white spells you cast cost {1} less to cast.","1G","Creature - Insect");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Green Sun's Zenith","G","Search your library for a green creature card with converted mana cost X or less, put it onto the battlefield, then shuffle your library. Shuffle Green Sun's Zenith into its owner's library.","XG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chieftain en-Dal","2","2","W","Whenever Chieftain en-Dal attacks, attacking creatures gain first strike until end of turn.","1WW","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Unyaro Griffin","2","2","W","Flying
Sacrifice Unyaro Griffin: Counter target red instant or sorcery spell.","3W","Creature - Griffin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Feral Instinct","G","Target creature gets +1/+1 until end of turn.
Draw a card at the beginning of the next turn's upkeep.","1G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blind Seer","3","3","U","{1}{U}: Target spell or permanent becomes the color of your choice until end of turn.","2UU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Barren Glory","W","At the beginning of your upkeep, if you control no permanents other than Barren Glory and have no cards in hand, you win the game.","4WW","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hornet Sting","G","Hornet Sting deals 1 damage to target creature or player.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Horde of Notions","5","5","WUBRG","Vigilance, trample, haste
{W}{U}{B}{R}{G}: You may play target Elemental card from your graveyard without paying its mana cost.","WUBRG","Legendary Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cat Burglar","2","2","B","{2}{B}, {T}: Target player discards a card. Activate this ability only any time you could cast a sorcery.","3B","Creature - Kor Rogue Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wall of Frost","0","7","U","Defender (This creature can't attack.)
Whenever Wall of Frost blocks a creature, that creature doesn't untap during its controller's next untap step.","1UU","Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pendelhaven Elder","1","1","G","{T}: Each 1/1 creature you control gets +1/+2 until end of turn.","1G","Creature - Elf Shaman");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sengir Autocrat","2","2","B","When Sengir Autocrat enters the battlefield, put three 0/1 black Serf creature tokens onto the battlefield.
When Sengir Autocrat leaves the battlefield, exile all Serf tokens.","3B","Creature - Human");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Moonscarred Werewolf","2","2","Vigilance
{T}: Add {G}{G} to your mana pool.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Moonscarred Werewolf.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Anger","2","2","R","Haste
As long as Anger is in your graveyard and you control a Mountain, creatures you control have haste.","3R","Creature - Incarnation");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loaming Shaman","3","2","G","When Loaming Shaman enters the battlefield, target player shuffles any number of target cards from his or her graveyard into his or her library.","2G","Creature - Centaur Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Profane Prayers","B","Profane Prayers deals X damage to target creature or player and you gain X life, where X is the number of Clerics on the battlefield.","2BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fortune Thief","0","1","R","Damage that would reduce your life total to less than 1 reduces it to 1 instead.
Morph {R}{R} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)","4R","Creature - Human Rogue");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Oona's Gatewarden","2","1","UB","Defender, flying
Wither (This deals damage to creatures in the form of -1/-1 counters.)","(U/B)","Creature - Faerie Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Eldrazi Conscription","Enchant creature
Enchanted creature gets +10/+10 and has trample and annihilator 2. (Whenever it attacks, defending player sacrifices two permanents.)","8","Tribal Enchantment - Eldrazi Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("A Display of My Dark Power","When you set this scheme in motion, until your next turn, whenever a player taps a land for mana, that player adds one mana to his or her mana pool of any type that land produced.",NULL,"Scheme");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Venser's Journal","You have no maximum hand size.
At the beginning of your upkeep, you gain 1 life for each card in your hand.","5","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Orim's Touch","W","Kicker {1} (You may pay an additional {1} as you cast this spell.)
Prevent the next 2 damage that would be dealt to target creature or player this turn. If Orim's Touch was kicked, prevent the next 4 damage that would be dealt to that creature or player this turn instead.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kamahl's Summons","G","Each player may reveal any number of creature cards from his or her hand. Then each player puts a 2/2 green Bear creature token onto the battlefield for each card he or she revealed this way.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Rattleblaze Scarecrow","5","3","Rattleblaze Scarecrow has persist as long as you control a black creature. (When this creature dies, if it had no -1/-1 counters on it, return it to the battlefield under its owner's control with a -1/-1 counter on it.)
Rattleblaze Scarecrow has haste as long as you control a red creature.","6","Artifact Creature - Scarecrow");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bog Wraith","3","3","B","Swampwalk (This creature is unblockable as long as defending player controls a Swamp.)","3B","Creature - Wraith");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Scent of Cinder","R","Reveal any number of red cards in your hand. Scent of Cinder deals X damage to target creature or player, where X is the number of cards revealed this way.","1R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Steal Enchantment","U","Enchant enchantment
You control enchanted enchantment.","UU","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Renegade Doppelganger","0","1","U","Whenever another creature enters the battlefield under your control, you may have Renegade Doppelganger become a copy of that creature until end of turn. (If it does, it loses this ability for the rest of the turn.)","1U","Creature - Shapeshifter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Minamo","Whenever a player casts a spell, that player may draw a card.
Whenever you roll {C}, each player may return a blue card from his or her graveyard to his or her hand.",NULL,"Plane - Kamigawa");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Funeral Pyre","W","Exile target card from a graveyard. Its owner puts a 1/1 white Spirit creature token with flying onto the battlefield.","W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ooze Garden","G","{1}{G}, Sacrifice a non-Ooze creature: Put an X/X green Ooze creature token onto the battlefield, where X is the sacrificed creature's power. Activate this ability only any time you could cast a sorcery.","1G","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Lich's Tomb","You don't lose the game for having 0 or less life.
Whenever you lose life, sacrifice a permanent for each 1 life you lost. (Damage causes loss of life.)","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Impulsive Maneuvers","R","Whenever a creature attacks, flip a coin. If you win the flip, the next time that creature would deal combat damage this turn, it deals double that damage instead. If you lose the flip, the next time that creature would deal combat damage this turn, prevent that damage.","2RR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mulch","G","Reveal the top four cards of your library. Put all land cards revealed this way into your hand and the rest into your graveyard.","1G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cerulean Wyvern","3","3","U","Flying, protection from green","4U","Creature - Drake");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Flight Spellbomb","{T}, Sacrifice Flight Spellbomb: Target creature gains flying until end of turn.
When Flight Spellbomb is put into a graveyard from the battlefield, you may pay {U}. If you do, draw a card.","1","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hired Muscle","2","2","B","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Hired Muscle.
At the beginning of the end step, if there are two or more ki counters on Hired Muscle, you may flip it.
----
Scarmaker
Legendary Creature - Spirit
4/4
Remove a ki counter from Scarmaker: Target creature gains fear until end of turn. (It can't be blocked except by artifact creatures and/or black creatures.)","1BB","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sootwalkers","3","3","BR","Sootwalkers can't be blocked by white creatures.","2(B/R)(B/R)","Creature - Elemental Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hunger of the Howlpack","G","Put a +1/+1 counter on target creature.
Morbid - Put three +1/+1 counters on that creature instead if a creature died this turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dusk Imp","2","1","B","Flying","2B","Creature - Imp");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Endless Scream","B","Enchant creature
Endless Scream enters the battlefield with X scream counters on it.
Enchanted creature gets +1/+0 for each scream counter on Endless Scream.","XB","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Smokebraider","1","1","R","{T}: Add two mana in any combination of colors to your mana pool. Spend this mana only to cast Elemental spells or activate abilities of Elementals.","1R","Creature - Elemental Shaman");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Heartwood Shard","{3}, {T} or {G}, {T}: Target creature gains trample until end of turn.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sunscape Apprentice","1","1","W","{G}, {T}: Target creature gets +1/+1 until end of turn.
{U}, {T}: Put target creature you control on top of its owner's library.","W","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Primal Frenzy","G","Enchant creature
Enchanted creature has trample.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tornado Elemental","6","6","G","When Tornado Elemental enters the battlefield, it deals 6 damage to each creature with flying.
You may have Tornado Elemental assign its combat damage as though it weren't blocked.","5GG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Maelstrom Pulse","BG","Destroy target nonland permanent and all other permanents with the same name as that permanent.","1BG","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Maelstrom Djinn","5","6","U","Flying
Morph {2}{U} (You may cast this face down as a 2/2 creature for {3}. Turn it face up any time for its morph cost.)
When Maelstrom Djinn is turned face up, put two time counters on it and it gains vanishing. (At the beginning of your upkeep, remove a time counter from it. When the last is removed, sacrifice it.)","7U","Creature - Djinn");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nature's Revolt","G","All lands are 2/2 creatures that are still lands.","3GG","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Smash","R","Destroy target artifact.
Draw a card.","2R","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Plague Boiler","At the beginning of your upkeep, put a plague counter on Plague Boiler.
{1}{B}{G}: Put a plague counter on Plague Boiler or remove a plague counter from it.
When Plague Boiler has three or more plague counters on it, sacrifice it. If you do, destroy all nonland permanents.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gruesome Encore","B","Put target creature card from an opponent's graveyard onto the battlefield under your control. It gains haste. Exile it at the beginning of the next end step. If that creature would leave the battlefield, exile it instead of putting it anywhere else.","2B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Anthem of Rakdos","BR","Whenever a creature you control attacks, it gets +2/+0 until end of turn and Anthem of Rakdos deals 1 damage to you.
Hellbent - As long as you have no cards in hand, if a source you control would deal damage to a creature or player, it deals double that damage to that creature or player instead.","2BRR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Doomsday","B","Search your library and graveyard for five cards and exile the rest. Put the chosen cards on top of your library in any order. You lose half your life, rounded up.","BBB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rewind","U","Counter target spell. Untap up to four lands.","2UU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Terminal Moraine","{T}: Add {1} to your mana pool.
{2}, {T}, Sacrifice Terminal Moraine: Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Birthing Pod","G","({(g/p)} can be paid with either {G} or 2 life.)
{1}{(g/p)}, {T}, Sacrifice a creature: Search your library for a creature card with converted mana cost equal to 1 plus the sacrificed creature's converted mana cost, put that card onto the battlefield, then shuffle your library. Activate this ability only any time you could cast a sorcery.","3(G/P)","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blood Funnel","B","Noncreature spells you cast cost {2} less to cast.
Whenever you cast a noncreature spell, counter that spell unless you sacrifice a creature.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Primalcrux","*","*","G","Trample
Chroma - Primalcrux's power and toughness are each equal to the number of green mana symbols in the mana costs of permanents you control.","GGGGGG","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necrogen Mists","B","At the beginning of each player's upkeep, that player discards a card.","2B","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Forge","Artifacts you control are indestructible. (""Destroy"" effects and lethal damage don't destroy them.)","9","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sheltering Word","G","Target creature you control gains hexproof until end of turn. You gain life equal to that creature's toughness. (A creature with hexproof can't be the target of spells or abilities opponents control.)","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Perilous Research","U","Draw two cards, then sacrifice a permanent.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drake Familiar","2","1","U","Flying
When Drake Familiar enters the battlefield, sacrifice it unless you return an enchantment to its owner's hand.","1U","Creature - Drake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pollen Lullaby","W","Prevent all combat damage that would be dealt this turn. Clash with an opponent. If you win, creatures that player controls don't untap during the player's next untap step. (Each clashing player reveals the top card of his or her library, then puts that card on the top or bottom. A player wins if his or her card had a higher converted mana cost.)","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ghost Hounds","1","1","B","Vigilance
Whenever Ghost Hounds blocks or becomes blocked by a white creature, Ghost Hounds gains first strike until end of turn.","1B","Creature - Hound Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Leyline of Singularity","U","If Leyline of Singularity is in your opening hand, you may begin the game with it on the battlefield.
All nonland permanents are legendary.","2UU","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Counterlash","U","Counter target spell. You may cast a nonland card in your hand that shares a card type with that spell without paying its mana cost.","4UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squall Drifter","1","1","W","Flying
{W}, {T}: Tap target creature.","1W","Snow Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grandmother Sengir","3","3","B","{1}{B}, {T}: Target creature gets -1/-1 until end of turn.","4B","Legendary Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skyfire Kirin","3","3","R","Flying
Whenever you cast a Spirit or Arcane spell, you may gain control of target creature with that spell's converted mana cost until end of turn.","2RR","Legendary Creature - Kirin Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Defy Death","W","Return target creature card from your graveyard to the battlefield. If it's an Angel, put two +1/+1 counters on it.","3WW","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blessed Breath","W","Target creature you control gains protection from the color of your choice until end of turn.
Splice onto Arcane {W} (As you cast an Arcane spell, you may reveal this card from your hand and pay its splice cost. If you do, add this card's effects to that spell.)","W","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Soul Shred","B","Soul Shred deals 3 damage to target nonblack creature. You gain 3 life.","3BB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dark Favor","B","Enchant creature
When Dark Favor enters the battlefield, you lose 1 life.
Enchanted creature gets +3/+1.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Volcanic Dragon","4","4","R","Flying
Haste (This creature can attack and {T} as soon as it comes under your control.)","4RR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Terashi's Verdict","W","Destroy target attacking creature with power 3 or less.","1W","Instant - Arcane");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Grave Servitude","B","You may cast Grave Servitude as though it had flash. If you cast it any time a sorcery couldn't have been cast, the controller of the permanent it becomes sacrifices it at the beginning of the next cleanup step.
Enchant creature
Enchanted creature gets +3/-1 and is black.","1B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Woeleecher","3","5","W","{W}, {T}: Remove a -1/-1 counter from target creature. If you do, you gain 2 life.","5W","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Keening Stone","{5}, {T}: Target player puts the top X cards of his or her library into his or her graveyard, where X is the number of cards in that player's graveyard.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Caverns of Despair","R","No more than two creatures can attack each combat.
No more than two creatures can block each combat.","2RR","World Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parasitic Bond","B","Enchant creature
At the beginning of the upkeep of enchanted creature's controller, Parasitic Bond deals 2 damage to that player.","3B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dream Fighter","1","1","U","Whenever Dream Fighter blocks or becomes blocked by a creature, Dream Fighter and that creature phase out. (While they're phased out, they're treated as though they don't exist. Each one phases in before its controller untaps during his or her next untap step.)","2U","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Entreat the Angels","W","Put X 4/4 white Angel creature tokens with flying onto the battlefield.
Miracle {X}{W}{W} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","XXWWW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Taniwha","7","7","U","Trample
Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)
At the beginning of your upkeep, all lands you control phase out. (They phase in before you untap during your next untap step.)","3UU","Legendary Creature - Serpent");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dogged Hunter","1","1","W","{T}: Destroy target creature token.","2W","Creature - Human Nomad");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Marsh Gas","B","All creatures get -2/-0 until end of turn.","B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shade of Trokair","1","2","W","{W}: Shade of Trokair gets +1/+1 until end of turn.
Suspend 3-{W} (Rather than cast this card from your hand, you may pay {W} and exile it with three time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)","3W","Creature - Shade");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelsong","W","Prevent all combat damage that would be dealt this turn.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Instant");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Henge Guardian","3","4","{2}: Henge Guardian gains trample until end of turn.","5","Artifact Creature - Dragon Wurm");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plated Wurm","4","5","G",NULL,"4G","Creature - Wurm");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ruination","R","Destroy all nonbasic lands.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mudbutton Clanger","1","1","R","Kinship - At the beginning of your upkeep, you may look at the top card of your library. If it shares a creature type with Mudbutton Clanger, you may reveal it. If you do, Mudbutton Clanger gets +1/+1 until end of turn.","R","Creature - Goblin Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pious Kitsune","1","2","W","At the beginning of your upkeep, put a devotion counter on Pious Kitsune. Then if a creature named Eight-and-a-Half-Tails is on the battlefield, you gain 1 life for each devotion counter on Pious Kitsune.
{T}, Remove a devotion counter from Pious Kitsune: You gain 1 life.","2W","Creature - Fox Cleric");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Serra","Creatures you control get +0/+2.",NULL,"Vanguard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Teferi's Care","W","{W}, Sacrifice an enchantment: Destroy target enchantment.
{3}{U}{U}: Counter target enchantment spell.","2W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Banishing Knack","U","Until end of turn, target creature gains ""{T}: Return target nonland permanent to its owner's hand.""","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fountain Watch","2","4","W","Artifacts and enchantments you control have shroud. (They can't be the targets of spells or abilities.)","3WW","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Binding Grasp","U","Enchant creature
At the beginning of your upkeep, sacrifice Binding Grasp unless you pay {1}{U}.
You control enchanted creature.
Enchanted creature gets +0/+1.","3U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silt Crawler","3","3","G","When Silt Crawler enters the battlefield, tap all lands you control.","2G","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jokulmorder","12","12","U","Trample
Jokulmorder enters the battlefield tapped.
When Jokulmorder enters the battlefield, sacrifice it unless you sacrifice five lands.
Jokulmorder doesn't untap during your untap step.
Whenever you play an Island, you may untap Jokulmorder.","4UUU","Creature - Leviathan");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Sins of the Past","B","Until end of turn, you may cast target instant or sorcery card from your graveyard without paying its mana cost. If that card would be put into your graveyard this turn, exile it instead. Exile Sins of the Past.","4BB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Tainted Field","{T}: Add {1} to your mana pool.
{T}: Add {W} or {B} to your mana pool. Activate this ability only if you control a Swamp.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plated Geopede","1","1","R","First strike
Landfall - Whenever a land enters the battlefield under your control, Plated Geopede gets +2/+2 until end of turn.","1R","Creature - Insect");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knotvine Mystic","2","2","WRG","{1}, {T}: Add {R}{G}{W} to your mana pool.","RGW","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mist Dragon","4","4","U","{0}: Mist Dragon gains flying. (This effect lasts indefinitely.)
{0}: Mist Dragon loses flying. (This effect lasts indefinitely.)
{3}{U}{U}: Mist Dragon phases out. (While it's phased out, it's treated as though it doesn't exist. It phases in before you untap during your next untap step.)","4UU","Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skirk Fire Marshal","2","2","R","Protection from red
Tap five untapped Goblins you control: Skirk Fire Marshal deals 10 damage to each creature and each player.","3RR","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Reckless Ogre","3","2","R","Whenever Reckless Ogre attacks alone, it gets +3/+0 until end of turn.","3R","Creature - Ogre");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elvish Archdruid","2","2","G","Other Elf creatures you control get +1/+1.
{T}: Add {G} to your mana pool for each Elf you control.","1GG","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Increasing Vengeance","R","Copy target instant or sorcery spell you control. If Increasing Vengeance was cast from a graveyard, copy that spell twice instead. You may choose new targets for the copies.
Flashback {3}{R}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","RR","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emberwilde Djinn","5","4","R","Flying
At the beginning of each player's upkeep, that player may pay {R}{R} or 2 life. If he or she does, the player gains control of Emberwilde Djinn.","2RR","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deepwood Drummer","1","1","G","{G}, {T}, Discard a card: Target creature gets +2/+2 until end of turn.","1G","Creature - Human Spellshaper");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Necropolis","0","1","Defender (This creature can't attack.)
Exile a creature card from your graveyard: Put X +0/+1 counters on Necropolis, where X is the exiled card's converted mana cost.","5","Artifact Creature - Wall");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stangg","3","4","RG","When Stangg enters the battlefield, put a legendary 3/4 red and green Human Warrior creature token named Stangg Twin onto the battlefield. When Stangg leaves the battlefield, exile that token. When that token leaves the battlefield, sacrifice Stangg.","4RG","Legendary Creature - Human Warrior");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Forbidden Crypt","B","If you would draw a card, return a card from your graveyard to your hand instead. If you can't, you lose the game.
If a card would be put into your graveyard from anywhere, exile that card instead.","3BB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ronom Hulk","5","6","G","Protection from snow
Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","4G","Creature - Beast");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aurification","W","Whenever a creature deals damage to you, put a gold counter on it.
Each creature with a gold counter on it is a Wall in addition to its other creature types and has defender. (Those creatures can't attack.)
When Aurification leaves the battlefield, remove all gold counters from all creatures.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Battered Golem","3","2","Battered Golem doesn't untap during your untap step.
Whenever an artifact enters the battlefield, you may untap Battered Golem.","3","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pygmy Allosaurus","2","2","G","Swampwalk","2G","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Pygmy Giant","0","2","R","{R}, {T}, Sacrifice a creature: Pygmy Giant deals X damage to target creature, where X is a number in the sacrificed creature's text box.","1RR","Creature - Giant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Djinn of Wishes","4","4","U","Flying
Djinn of Wishes enters the battlefield with three wish counters on it.
{2}{U}{U}, Remove a wish counter from Djinn of Wishes: Reveal the top card of your library. You may play that card without paying its mana cost. If you don't, exile it.","3UU","Creature - Djinn");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ertai, the Corrupted","3","4","WUB","{U}, {T}, Sacrifice a creature or enchantment: Counter target spell.","2WUB","Legendary Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Maro Avatar","Tap an untapped creature you control, Discard a card: Target creature you control gets +X/+X until end of turn, where X is the number of cards in your hand.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blood Celebrant","1","1","B","{B}, Pay 1 life: Add one mana of any color to your mana pool.","B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gravestorm","B","At the beginning of your upkeep, target opponent may exile a card from his or her graveyard. If that player doesn't, you may draw a card.","BBB","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ma Chao, Western Warrior","3","3","R","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)
Whenever Ma Chao, Western Warrior attacks alone, it's unblockable this combat.","3RR","Legendary Creature - Human Soldier Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Joiner Adept","2","1","G","Lands you control have ""{T}: Add one mana of any color to your mana pool.""","1G","Creature - Elf Druid");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Lay Waste","R","Destroy target land.
Cycling {2} ({2}, Discard this card: Draw a card.)","3R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Library of Alexandria","{T}: Add {1} to your mana pool.
{T}: Draw a card. Activate this ability only if you have exactly seven cards in hand.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Sakashima the Impostor Avatar","{2}: Choose a creature you control. It becomes a copy of target creature except for its name.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Merfolk Seer","2","2","U","When Merfolk Seer dies, you may pay {1}{U}. If you do, draw a card.","2U","Creature - Merfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Just Fate","W","Cast Just Fate only during the declare attackers step and only if you've been attacked this step.
Destroy target attacking creature.","2W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunting Moa","3","2","G","Echo {2}{G} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)
When Hunting Moa enters the battlefield or dies, put a +1/+1 counter on target creature.","2G","Creature - Bird Beast");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Decimator Web","{4}, {T}: Target opponent loses 2 life, gets a poison counter, then puts the top six cards of his or her library into his or her graveyard.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Justice","W","At the beginning of your upkeep, sacrifice Justice unless you pay {W}{W}.
Whenever a red creature or spell deals damage, Justice deals that much damage to that creature's or spell's controller.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Agent of Masks","2","3","WB","At the beginning of your upkeep, each opponent loses 1 life. You gain life equal to the life lost this way.","3WB","Creature - Human Advisor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faultgrinder","4","4","R","Trample
When Faultgrinder enters the battlefield, destroy target land.
Evoke {4}{R} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","6R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Double Deal","R","Choose another player. Double Deal deals 3 damage to that player now and deals an additional 3 damage to the player at the beginning of the next game with the player.","4R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Riverfall Mimic","2","1","UR","Whenever you cast a spell that's both blue and red, Riverfall Mimic becomes 3/3 and is unblockable until end of turn.","1(U/R)","Creature - Shapeshifter");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mine, Mine, Mine!","G","When Mine, Mine, Mine comes into play, each player puts his or her library into his or her hand.
Each player skips his or her discard phase and does not lose as a result of being unable to draw a card.
Each player cannot play more than one spell each turn.
If Mine, Mine, Mine leaves play, each player shuffles his or her hand and graveyard into his or her library.","4GG","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Dryad Arbor","1","1","(Dryad Arbor isn't a spell, it's affected by summoning sickness, and it has ""{T}: Add {G} to your mana pool."")",NULL,"Land Creature - Forest Dryad");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devouring Deep","1","2","U","Islandwalk","2U","Creature - Fish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sylvan Basilisk","2","4","G","Whenever Sylvan Basilisk becomes blocked by a creature, destroy that creature.","3GG","Creature - Basilisk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Protection: Art","W","As Circle of Protection: Art comes into play, choose an artist.
{1W}: The next time a source of your choice by the chosen artist would deal damage to you this turn, prevent that damage.
{1W}: Return Circle of Protection: Art to its owner's hand.","1W","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Winter's Night","WRG","Whenever a player taps a snow land for mana, that player adds one mana to his or her mana pool of any type that land produced. That land doesn't untap during its controller's next untap step.","RGW","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dawnglare Invoker","2","1","W","Flying
{8}: Tap all creatures target player controls.","2W","Creature - Kor Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ground Seal","G","When Ground Seal enters the battlefield, draw a card.
Cards in graveyards can't be the targets of spells or abilities.","1G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Filigree Sages","2","3","U","{2}{U}: Untap target artifact.","3U","Artifact Creature - Vedalken Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Odds // Ends","WR","Target player sacrifices two attacking creatures.
---
Flip a coin. If it comes up heads, counter target instant or sorcery spell. If it comes up tails, copy that spell and you may choose new targets for the copy.","3RW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shivan Raptor","3","1","R","First strike, haste
Echo {2}{R} (At the beginning of your upkeep, if this came under your control since the beginning of your last upkeep, sacrifice it unless you pay its echo cost.)","2R","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Whiplash Trap","U","If an opponent had two or more creatures enter the battlefield under his or her control this turn, you may pay {U} rather than pay Whiplash Trap's mana cost.
Return two target creatures to their owners' hands.","3UU","Instant - Trap");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kaleidostone","When Kaleidostone enters the battlefield, draw a card.
{5}, {T}, Sacrifice Kaleidostone: Add {W}{U}{B}{R}{G} to your mana pool.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Immobilizing Ink","U","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.
Enchanted creature has ""{1}, Discard a card: Untap this creature.""","1U","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kithkin Healer","2","2","W","{T}: Prevent the next 1 damage that would be dealt to target creature or player this turn.","2W","Creature - Kithkin Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kashi-Tribe Warriors","2","4","G","Whenever Kashi-Tribe Warriors deals combat damage to a creature, tap that creature and it doesn't untap during its controller's next untap step.","3GG","Creature - Snake Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rotting Rats","1","1","B","When Rotting Rats enters the battlefield, each player discards a card.
Unearth {1}{B} ({1}{B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","1B","Creature - Zombie Rat");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wheel of Torture","At the beginning of each opponent's upkeep, Wheel of Torture deals X damage to that player, where X is 3 minus the number of cards in his or her hand.","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Verduran Emissary","2","3","G","Kicker {1}{R} (You may pay an additional {1}{R} as you cast this spell.)
When Verduran Emissary enters the battlefield, if it was kicked, destroy target artifact. It can't be regenerated.","2G","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gigadrowse","U","Replicate {U} (When you cast this spell, copy it for each time you paid its replicate cost. You may choose new targets for the copies.)
Tap target permanent.","U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Viridian Joiner","1","2","G","{T}: Add to your mana pool an amount of {G} equal to Viridian Joiner's power.","2G","Creature - Elf Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ramosian Revivalist","2","2","W","{6}, {T}: Return target Rebel permanent card with converted mana cost 5 or less from your graveyard to the battlefield.","3W","Creature - Human Rebel Cleric");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Aesthir Glider","2","1","Flying
Aesthir Glider can't block.","3","Artifact Creature - Bird");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Execute","B","Destroy target white creature. It can't be regenerated.
Draw a card.","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mortis Dogs","2","2","B","Whenever Mortis Dogs attacks, it gets +2/+0 until end of turn.
When Mortis Dogs dies, target player loses life equal to its power.","3B","Creature - Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sailmonger","3","3","U","{2}: Target creature gains flying until end of turn. Any player may activate this ability.","3U","Creature - Human Monger");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Castle Raptors","3","3","W","Flying
As long as Castle Raptors is untapped, it gets +0/+2.","4W","Creature - Bird Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Renewed Faith","W","You gain 6 life.
Cycling {1}{W} ({1}{W}, Discard this card: Draw a card.)
When you cycle Renewed Faith, you may gain 2 life.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Nevermore","W","As Nevermore enters the battlefield, name a nonland card.
The named card can't be cast.","1WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Whiptail Moloch","6","3","R","When Whiptail Moloch enters the battlefield, it deals 3 damage to target creature you control.","4R","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Captivating Vampire","2","2","B","Other Vampire creatures you control get +1/+1.
Tap five untapped Vampires you control: Gain control of target creature. It becomes a Vampire in addition to its other types.","1BB","Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Slag Fiend","*","*","R","Slag Fiend's power and toughness are each equal to the number of artifact cards in all graveyards.","R","Creature - Construct");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yawgmoth Demon","6","6","B","Flying (This creature can't be blocked except by creatures with flying or reach.)
First strike (This creature deals combat damage before creatures without first strike.)
At the beginning of your upkeep, you may sacrifice an artifact. If you don't, tap Yawgmoth Demon and it deals 2 damage to you.","4BB","Creature - Demon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Seer's Sundial","Landfall - Whenever a land enters the battlefield under your control, you may pay {2}. If you do, draw a card.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Timecrafting","R","Choose one - Remove X time counters from target permanent or suspended card; or put X time counters on target permanent with a time counter on it or suspended card.","XR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Peek","U","Look at target player's hand.
Draw a card.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reverse Polarity","W","You gain X life, where X is twice the damage dealt to you so far this turn by artifacts.","WW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wakedancer","2","2","B","Morbid - When Wakedancer enters the battlefield, if a creature died this turn, put a 2/2 black Zombie creature token onto the battlefield.","2B","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flashfires","R","Destroy all Plains.","3R","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Chandler","3","3","R","{R}{R}{R}, {T}: Destroy target artifact creature.","4R","Legendary Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hail of Arrows","W","Hail of Arrows deals X damage divided as you choose among any number of target attacking creatures.","XW","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gladecover Scout","1","1","G","Hexproof (This creature can't be the target of spells or abilities your opponents control.)","G","Creature - Elf Scout");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ana Battlemage","2","2","G","Kicker {2}{U} and/or {1}{B} (You may pay an additional {2}{U} and/or {1}{B} as you cast this spell.)
When Ana Battlemage enters the battlefield, if it was kicked with its {2}{U} kicker, target player discards three cards.
When Ana Battlemage enters the battlefield, if it was kicked with its {1}{B} kicker, tap target untapped creature and that creature deals damage equal to its power to its controller.","2G","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ogre Savant","3","2","R","When Ogre Savant enters the battlefield, if {U} was spent to cast Ogre Savant, return target creature to its owner's hand.","4R","Creature - Ogre Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spineless Thug","2","2","B","Spineless Thug can't block.","1B","Creature - Zombie Mercenary");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Esperzoa","4","3","U","Flying
At the beginning of your upkeep, return an artifact you control to its owner's hand.","2U","Artifact Creature - Jellyfish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Golgari Grave-Troll","0","0","G","Golgari Grave-Troll enters the battlefield with a +1/+1 counter on it for each creature card in your graveyard.
{1}, Remove a +1/+1 counter from Golgari Grave-Troll: Regenerate Golgari Grave-Troll.
Dredge 6 (If you would draw a card, instead you may put exactly six cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)","4G","Creature - Skeleton Troll");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bereavement","B","Whenever a green creature dies, its controller discards a card.","1B","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Devoted Hero","1","2","W",NULL,"W","Creature - Elf Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rune-Scarred Demon","6","6","B","Flying
When Rune-Scarred Demon enters the battlefield, search your library for a card, put it into your hand, then shuffle your library.","5BB","Creature - Demon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Dog","2","2","R","Mountainwalk","2R","Creature - Hound");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Zombie Apocalypse","B","Return all Zombie creature cards from your graveyard to the battlefield tapped, then destroy all Humans.","3BBB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Teroh's Faithful","1","4","W","When Teroh's Faithful enters the battlefield, you gain 4 life.","3W","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Jungle Patrol","3","2","G","{1}{G}, {T}:  Put a 0/1 green Wall creature token with defender named Wood onto the battlefield.
Sacrifice a token named Wood: Add {R} to your mana pool.","3G","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hunter Sliver","1","1","R","All Sliver creatures have provoke. (When a Sliver attacks, its controller may have target creature defending player controls untap and block it if able.)","1R","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Barbary Apes","2","2","G",NULL,"1G","Creature - Ape");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tribute to Hunger","B","Target opponent sacrifices a creature. You gain life equal to that creature's toughness.","2B","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Loyal Sentry","1","1","W","When Loyal Sentry blocks a creature, destroy that creature and Loyal Sentry.","W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Gruul Scrapper","3","2","G","When Gruul Scrapper enters the battlefield, if {R} was spent to cast Gruul Scrapper, it gains haste until end of turn.","3G","Creature - Human Berserker");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Immersturm","Whenever a creature enters the battlefield, that creature's controller may have it deal damage equal to its power to target creature or player of his or her choice.
Whenever you roll {C}, exile target creature, then return it to the battlefield under its owner's control.",NULL,"Plane - Valla");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lady Zhurong, Warrior Queen","4","3","G","Horsemanship (This creature can't be blocked except by creatures with horsemanship.)","4G","Legendary Creature - Human Soldier Warrior");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Krallenhorde Killer","2","2","{3}{G}: Krallenhorde Killer gets +4/+4 until end of turn. Activate this ability only once each turn.
At the beginning of each upkeep, if a player cast two or more spells last turn, transform Krallenhorde Killer.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Horse","3","3","G","Islandwalk","3G","Creature - Horse");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aurochs Herd","4","4","G","Trample
When Aurochs Herd enters the battlefield, you may search your library for an Aurochs card, reveal it, and put it into your hand. If you do, shuffle your library.
Whenever Aurochs Herd attacks, it gets +1/+0 until end of turn for each other attacking Aurochs.","5G","Creature - Aurochs");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Grotag Thrasher","3","3","R","Whenever Grotag Thrasher attacks, target creature can't block this turn.","4R","Creature - Lizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Halcyon Glaze","U","Whenever you cast a creature spell, Halcyon Glaze becomes a 4/4 Illusion creature with flying until end of turn. It's still an enchantment.","1UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nantuko Cultivator","2","2","G","When Nantuko Cultivator enters the battlefield, you may discard any number of land cards. Put that many +1/+1 counters on Nantuko Cultivator and draw that many cards.","3G","Creature - Insect Druid");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ballyrush Banneret","2","1","W","Kithkin spells and Soldier spells you cast cost {1} less to cast.","1W","Creature - Kithkin Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nihilith","4","4","B","Fear (This creature can't be blocked except by artifact creatures and/or black creatures.)
Suspend 7-{1}{B} (Rather than cast this card from your hand, you may pay {1}{B} and exile it with seven time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost. It has haste.)
Whenever a card is put into an opponent's graveyard from anywhere, if Nihilith is suspended, you may remove a time counter from Nihilith.","4BB","Creature - Horror");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Circle of Despair","WB","{1}, Sacrifice a creature: The next time a source of your choice would deal damage to target creature or player this turn, prevent that damage.","1WB","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Shivan Oasis","Shivan Oasis enters the battlefield tapped.
{T}: Add {R} or {G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fire Covenant","BR","As an additional cost to cast Fire Covenant, pay X life.
Fire Covenant deals X damage divided as you choose among any number of target creatures.","1BR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gerrymandering","G","Remove all lands from play and shuffle them together. Randomly deal to each player one land card for each land he or she had before. Each player puts those lands into play under his or her control, untapped.","2G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Aegis Angel","5","5","W","Flying
When Aegis Angel enters the battlefield, another target permanent is indestructible for as long as you control Aegis Angel. (Effects that say ""destroy"" don't destroy that permanent. An indestructible creature can't be destroyed by damage.)","4WW","Creature - Angel");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Warmonger","3","3","R","{2}: Warmonger deals 1 damage to each creature without flying and each player. Any player may activate this ability.","3R","Creature - Minotaur Monger");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kjeldoran Guard","1","1","W","{T}: Target creature gets +1/+1 until end of turn. When that creature leaves the battlefield this turn, sacrifice Kjeldoran Guard. Activate this ability only during combat and only if defending player controls no snow lands.","1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Seasinger","0","1","U","When you control no Islands, sacrifice Seasinger.
You may choose not to untap Seasinger during your untap step.
{T}: Gain control of target creature whose controller controls an Island for as long as you control Seasinger and Seasinger remains tapped.","1UU","Creature - Merfolk");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Negate","U","Counter target noncreature spell.","1U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Undo","U","Return two target creatures to their owners' hands.","1UU","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Citadel","Darksteel Citadel is indestructible. (""Destroy"" effects and lethal damage don't destroy it.)
{T}: Add {1} to your mana pool.",NULL,"Artifact Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voice of All","2","2","W","Flying
As Voice of All enters the battlefield, choose a color.
Voice of All has protection from the chosen color.","2WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Dust Corona","R","Enchant creature
Enchanted creature gets +2/+0 and can't be blocked by creatures with flying.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hostility","6","6","R","Haste
If a spell you control would deal damage to an opponent, prevent that damage. Put a 3/1 red Elemental Shaman creature token with haste onto the battlefield for each 1 damage prevented this way.
When Hostility is put into a graveyard from anywhere, shuffle it into its owner's library.","3RRR","Creature - Elemental Incarnation");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Equilibrium","U","Whenever you cast a creature spell, you may pay {1}. If you do, return target creature to its owner's hand.","1UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cabal Slaver","2","1","B","Whenever a Goblin deals combat damage to a player, that player discards a card.","2B","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Crucible of Fire","R","Dragon creatures you control get +3/+3.","3R","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Foul Presence","B","Enchant creature
Enchanted creature gets -1/-1 and has ""{T}: Target creature gets -1/-1 until end of turn.""","2B","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thunderscape Apprentice","1","1","R","{B}, {T}: Target player loses 1 life.
{G}, {T}: Target creature gets +1/+1 until end of turn.","R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stitch in Time","UR","Flip a coin. If you win the flip, take an extra turn after this one.","1UR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Skeletal Crocodile","5","1","B",NULL,"3B","Creature - Crocodile Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stream of Unconsciousness","U","Target creature gets -4/-0 until end of turn. If you control a Wizard, draw a card.","U","Tribal Instant - Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Yavimaya Wurm","6","4","G","Trample (If this creature would assign enough damage to its blockers to destroy them, you may have it assign the rest of its damage to defending player or planeswalker.)","4GG","Creature - Wurm");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Icatian Store","Icatian Store enters the battlefield tapped.
You may choose not to untap Icatian Store during your untap step.
At the beginning of your upkeep, if Icatian Store is tapped, put a storage counter on it.
{T}, Remove any number of storage counters from Icatian Store: Add {W} to your mana pool for each storage counter removed this way.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ertai's Familiar","2","2","U","Phasing (This phases in or out before you untap during each of your untap steps. While it's phased out, it's treated as though it doesn't exist.)
When Ertai's Familiar phases out or leaves the battlefield, put the top three cards of your library into your graveyard.
{U}: Until your next upkeep, Ertai's Familiar can't phase out.","1U","Creature - Illusion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tethered Griffin","2","3","W","Flying
When you control no enchantments, sacrifice Tethered Griffin.","W","Creature - Griffin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Spitebellows","6","1","R","When Spitebellows leaves the battlefield, it deals 6 damage to target creature.
Evoke {1}{R}{R} (You may cast this spell for its evoke cost. If you do, it's sacrificed when it enters the battlefield.)","5R","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Petrified Field","{T}: Add {1} to your mana pool.
{T}, Sacrifice Petrified Field: Return target land card from your graveyard to your hand.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ethersworn Shieldmage","2","2","WU","Flash
When Ethersworn Shieldmage enters the battlefield, prevent all damage that would be dealt to artifact creatures this turn.","1WU","Artifact Creature - Vedalken Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Horror","2","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
Dauthi Horror can't be blocked by white creatures.","1B","Creature - Dauthi Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Stormblood Berserker","1","1","R","Bloodthirst 2 (If an opponent was dealt damage this turn, this creature enters the battlefield with two +1/+1 counters on it.)
Stormblood Berserker can't be blocked except by two or more creatures.","1R","Creature - Human Berserker");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kor Spiritdancer","0","2","W","Kor Spiritdancer gets +2/+2 for each Aura attached to it.
Whenever you cast an Aura spell, you may draw a card.","1W","Creature - Kor Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Silkwing Scout","2","1","U","Flying
{G}, Sacrifice Silkwing Scout: Search your library for a basic land card and put that card onto the battlefield tapped. Then shuffle your library.","2U","Creature - Faerie Scout");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Balduvian Rage","R","Target attacking creature gets +X/+0 until end of turn.
Draw a card at the beginning of the next turn's upkeep.","XR","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Culling Scales","At the beginning of your upkeep, destroy target nonland permanent with the lowest converted mana cost. (If two or more permanents are tied for lowest cost, target any one of them.)","3","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dauthi Jackal","2","1","B","Shadow (This creature can block or be blocked by only creatures with shadow.)
{B}{B}, Sacrifice Dauthi Jackal: Destroy target blocking creature.","2B","Creature - Dauthi Hound");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zombie Boa","3","3","B","{1}{B}: Choose a color. Whenever Zombie Boa becomes blocked by a creature of that color this turn, destroy that creature. Activate this ability only any time you could cast a sorcery.","4B","Creature - Zombie Snake");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Abeyance","W","Until end of turn, target player can't cast instant or sorcery spells, and that player can't activate abilities that aren't mana abilities.
Draw a card.","1W","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Balm of Restoration","{1}, {T}, Sacrifice Balm of Restoration: Choose one - You gain 2 life; or prevent the next 2 damage that would be dealt to target creature or player this turn.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Artillery","1","3","R","{T}: Orcish Artillery deals 2 damage to target creature or player and 3 damage to you.","1RR","Creature - Orc Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Acidic Sliver","2","2","BR","All Slivers have ""{2}, Sacrifice this permanent: This permanent deals 2 damage to target creature or player.""","BR","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Guardian Seraph","3","4","W","Flying
If a source an opponent controls would deal damage to you, prevent 1 of that damage.","2WW","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Path of Peace","W","Destroy target creature. Its owner gains 4 life.","3W","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Parallax Wave","W","Fading 5 (This enchantment enters the battlefield with five fade counters on it. At the beginning of your upkeep, remove a fade counter from it. If you can't, sacrifice it.)
Remove a fade counter from Parallax Wave: Exile target creature.
When Parallax Wave leaves the battlefield, each player returns to the battlefield all cards he or she owns exiled with Parallax Wave.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Braids, Cabal Minion","2","2","B","At the beginning of each player's upkeep, that player sacrifices an artifact, creature, or land.","2BB","Legendary Creature - Human Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mogg Toady","2","2","R","Mogg Toady can't attack unless you control more creatures than defending player.
Mogg Toady can't block unless you control more creatures than attacking player.","1R","Creature - Goblin");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Drift of Phantasms","0","5","U","Defender (This creature can't attack.)
Flying
Transmute {1}{U}{U} ({1}{U}{U}, Discard this card: Search your library for a card with the same converted mana cost as this card, reveal it, and put it into your hand. Then shuffle your library. Transmute only as a sorcery.)","2U","Creature - Spirit");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Force Bubble","W","If damage would be dealt to you, put that many depletion counters on Force Bubble instead.
When there are four or more depletion counters on Force Bubble, sacrifice it.
At the beginning of each end step, remove all depletion counters from Force Bubble.","2WW","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Brutalizer Exarch","3","3","G","When Brutalizer Exarch enters the battlefield, choose one - Search your library for a creature card, reveal it, then shuffle your library and put that card on top of it; or put target noncreature permanent on the bottom of its owner's library.","5G","Creature - Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firestorm Hellkite","6","6","UR","Flying, trample
Cumulative upkeep {U}{R} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)","4UR","Creature - Dragon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Merrow Commerce","U","At the beginning of your end step, untap all Merfolk you control.","1U","Tribal Enchantment - Merfolk");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kami of Fire's Roar","2","3","R","Whenever you cast a Spirit or Arcane spell, target creature can't block this turn.","3R","Creature - Spirit");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vendilion Clique","3","1","U","Flash
Flying
When Vendilion Clique enters the battlefield, look at target player's hand. You may choose a nonland card from it. If you do, that player reveals the chosen card, puts it on the bottom of his or her library, then draws a card.","1UU","Legendary Creature - Faerie Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aleatory","R","Cast Aleatory only during combat after blockers are declared.
Flip a coin. If you win the flip, target creature gets +1/+1 until end of turn.
Draw a card at the beginning of the next turn's upkeep.","1R","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crested Craghorn","4","1","R","Haste
Provoke (When this attacks, you may have target creature defending player controls untap and block it if able.)","4R","Creature - Goat Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fault Riders","2","2","R","Sacrifice a land: Fault Riders gets +2/+0 and gains first strike until end of turn. Activate this ability only once each turn.","2R","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Glamerdye","U","Change the text of target spell or permanent by replacing all instances of one color word with another.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Firebrand Ranger","2","1","R","{G}, {T}: You may put a basic land card from your hand onto the battlefield.","1R","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Collective Restraint","U","Domain - Creatures can't attack you unless their controller pays {X} for each creature he or she controls that's attacking you, where X is the number of basic land types among lands you control.","3U","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Manta Ray","3","3","U","Manta Ray can't attack unless defending player controls an Island.
Manta Ray can't be blocked except by blue creatures.
When you control no Islands, sacrifice Manta Ray.","1UU","Creature - Fish");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Hero of Bladehold","3","4","W","Battle cry (Whenever this creature attacks, each other attacking creature gets +1/+0 until end of turn.)
Whenever Hero of Bladehold attacks, put two 1/1 white Soldier creature tokens onto the battlefield tapped and attacking.","2WW","Creature - Human Knight");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Hymn to Tourach","B","Target player discards two cards at random.","BB","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rasputin Dreamweaver","4","1","WU","Rasputin Dreamweaver enters the battlefield with seven dream counters on it.
Remove a dream counter from Rasputin: Add {1} to your mana pool.
Remove a dream counter from Rasputin: Prevent the next 1 damage that would be dealt to Rasputin this turn.
At the beginning of your upkeep, if Rasputin started the turn untapped, put a dream counter on it.
Rasputin can't have more than seven dream counters on it.","4WU","Legendary Creature - Human Wizard");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Darksteel Plate","Darksteel Plate is indestructible.
Equipped creature is indestructible.
Equip {2}","3","Artifact - Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Leonin Battlemage","2","3","W","{T}: Target creature gets +1/+1 until end of turn.
Whenever you cast a spell, you may untap Leonin Battlemage.","3W","Creature - Cat Wizard");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Dromar's Attendant","3","3","{1}, Sacrifice Dromar's Attendant: Add {W}{U}{B} to your mana pool.","5","Artifact Creature - Golem");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kathari Remnant","0","1","UB","Flying
{B}: Regenerate Kathari Remnant.
Cascade (When you cast this spell, exile cards from the top of your library until you exile a nonland card that costs less. You may cast it without paying its mana cost. Put the exiled cards on the bottom in a random order.)","2UB","Creature - Bird Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Discombobulate","U","Counter target spell. Look at the top four cards of your library, then put them back in any order.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Academy Rector","1","2","W","When Academy Rector dies, you may exile it. If you do, search your library for an enchantment card, put that card onto the battlefield, then shuffle your library.","3W","Creature - Human Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Black Sun's Zenith","B","Put X -1/-1 counters on each creature. Shuffle Black Sun's Zenith into its owner's library.","XBB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rishadan Port","{T}: Add {1} to your mana pool.
{1}, {T}: Tap target land.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Daring Leap","WU","Target creature gets +1/+1 and gains flying and first strike until end of turn.","1WU","Instant");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Starlit Sanctum","{T}: Add {1} to your mana pool.
{W}, {T}, Sacrifice a Cleric creature: You gain life equal to the sacrificed creature's toughness.
{B}, {T}, Sacrifice a Cleric creature: Target player loses life equal to the sacrificed creature's power.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Russet Wolves","3","3","R",NULL,"3R","Creature - Wolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Psychic Vortex","U","Cumulative upkeep-Draw a card. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of your end step, sacrifice a land and discard your hand.","2UU","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Cunning Bandit","2","2","R","Whenever you cast a Spirit or Arcane spell, you may put a ki counter on Cunning Bandit.
At the beginning of the end step, if there are two or more ki counters on Cunning Bandit, you may flip it.","1RR","Creature - Human Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Keldon Necropolis","{T}: Add {1} to your mana pool.
{4}{R}, {T}, Sacrifice a creature: Keldon Necropolis deals 2 damage to target creature or player.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Cover of Darkness","B","As Cover of Darkness enters the battlefield, choose a creature type.
Creatures of the chosen type have fear. (They can't be blocked except by artifact creatures and/or black creatures.)","1B","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Piracy","U","Until end of turn, you may tap lands you don't control for mana. Spend this mana only to cast spells.","UU","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kruin Striker","2","1","R","Whenever another creature enters the battlefield under your control, Kruin Striker gets +1/+0 and gains trample until end of turn.","1R","Creature - Human Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soratami Mirror-Guard","3","1","U","Flying
{2}, Return a land you control to its owner's hand: Target creature with power 2 or less is unblockable this turn.","3U","Creature - Moonfolk Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Shivan Meteor","R","Shivan Meteor deals 13 damage to target creature.
Suspend 2-{1}{R}{R} (Rather than cast this card from your hand, you may pay {1}{R}{R} and exile it with two time counters on it. At the beginning of your upkeep, remove a time counter. When the last is removed, cast it without paying its mana cost.)","3RR","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bramble Elemental","4","4","G","Whenever an Aura becomes attached to Bramble Elemental or enters the battlefield attached to Bramble Elemental, put two 1/1 green Saproling creature tokens onto the battlefield.","3GG","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jungle Basin","Jungle Basin enters the battlefield tapped.
When Jungle Basin enters the battlefield, sacrifice it unless you return an untapped Forest you control to its owner's hand.
{T}: Add {1}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Talonrend","0","5","U","Flying
{(u/r)}: Talonrend gets +1/-1 until end of turn.","4U","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fool's Demise","U","Enchant creature
When enchanted creature dies, return that card to the battlefield under your control.
When Fool's Demise is put into a graveyard from the battlefield, return Fool's Demise to its owner's hand.","4U","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bubble Matrix","Prevent all damage that would be dealt to creatures.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Selesnya Signet","{1}, {T}: Add {G}{W} to your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Xathrid Demon","7","7","B","Flying, trample
At the beginning of your upkeep, sacrifice a creature other than Xathrid Demon, then each opponent loses life equal to the sacrificed creature's power. If you can't sacrifice a creature, tap Xathrid Demon and you lose 7 life.","3BBB","Creature - Demon");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rakdos Signet","{1}, {T}: Add {B}{R} to your mana pool.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vow of Wildness","G","Enchant creature
Enchanted creature gets +3/+3, has trample, and can't attack you or a planeswalker you control.","2G","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Boreal Shelf","Boreal Shelf enters the battlefield tapped.
{T}: Add {W} or {U} to your mana pool.",NULL,"Snow Land");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Darksteel Myr","0","1","Darksteel Myr is indestructible. (Lethal damage and effects that say ""destroy"" don't destroy it. If its toughness is 0 or less, it's still put into its owner's graveyard.)","3","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Sliver Overlord","7","7","WUBRG","{3}: Search your library for a Sliver card, reveal that card, and put it into your hand. Then shuffle your library.
{3}: Gain control of target Sliver. (This effect lasts indefinitely.)","WUBRG","Legendary Creature - Sliver Mutant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Arachnus Spinner","5","7","G","Reach (This creature can block creatures with flying.)
Tap an untapped Spider you control: Search your graveyard and/or library for a card named Arachnus Web and put it onto the battlefield attached to target creature. If you search your library this way, shuffle it.","5G","Creature - Spider");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shepherd of the Lost","3","3","W","Flying, first strike, vigilance","4W","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Bazaar of Wonders","U","When Bazaar of Wonders enters the battlefield, exile all cards from all graveyards.
Whenever a player casts a spell, counter it if a card with the same name is in a graveyard or a nontoken permanent with the same name is on the battlefield.","3UU","World Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dread Slag","9","9","BR","Trample
Dread Slag gets -4/-4 for each card in your hand.","3BR","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Terra Stomper","8","8","G","Terra Stomper can't be countered.
Trample","3GGG","Creature - Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Plant Elemental","3","4","G","When Plant Elemental enters the battlefield, sacrifice it unless you sacrifice a Forest.","1G","Creature - Plant Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Strength of Cedars","G","Target creature gets +X/+X until end of turn, where X is the number of lands you control.","4G","Instant - Arcane");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Samite Censer-Bearer","1","1","W","{W}, Sacrifice Samite Censer-Bearer: Prevent the next 1 damage that would be dealt to each creature you control this turn.","W","Creature - Human Rebel Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Nightguard Patrol","2","1","W","First strike, vigilance","2W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shifty Doppelganger","1","1","U","{3}{U}, Exile Shifty Doppelganger: You may put a creature card from your hand onto the battlefield. If you do, that creature gains haste until end of turn. At the beginning of the next end step, sacrifice that creature. If you do, return Shifty Doppelganger to the battlefield.","2U","Creature - Shapeshifter");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Hammer of Ruin","Equipped creature gets +2/+0.
Whenever equipped creature deals combat damage to a player, you may destroy target Equipment that player controls.
Equip {2}","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("View from Above","U","Target creature gains flying until end of turn. If you control a white permanent, return View from Above to its owner's hand.","1U","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Shieldmage Elder","2","3","W","Tap two untapped Clerics you control: Prevent all damage target creature would deal this turn.
Tap two untapped Wizards you control: Prevent all damage target spell would deal this turn.","5W","Creature - Human Cleric Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Glory Seeker","2","2","W",NULL,"1W","Creature - Human Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rhox Bodyguard","2","3","WG","Exalted (Whenever a creature you control attacks alone, that creature gets +1/+1 until end of turn.)
When Rhox Bodyguard enters the battlefield, you gain 3 life.","3GW","Creature - Rhino Monk Soldier");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Malfegor Avatar","Whenever a creature enters the battlefield under your control, if it was unearthed, it gets +3/+0.
Whenever a creature you control is exiled, if it was unearthed, shuffle that card into its owner's library.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Obstinate Familiar","1","1","R","If you would draw a card, you may skip that draw instead.","R","Creature - Lizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Faerie Mechanist","2","2","U","Flying
When Faerie Mechanist enters the battlefield, look at the top three cards of your library. You may reveal an artifact card from among them and put it into your hand. Put the rest on the bottom of your library in any order.","3U","Artifact Creature - Faerie Artificer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Manaplasm","1","1","G","Whenever you cast a spell, Manaplasm gets +X/+X until end of turn, where X is that spell's converted mana cost.","2G","Creature - Ooze");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Logic Knot","U","Delve (You may exile any number of cards from your graveyard as you cast this spell. It costs {1} less to cast for each card exiled this way.)
Counter target spell unless its controller pays {X}.","XUU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Haunted Angel","3","3","W","Flying
When Haunted Angel dies, exile Haunted Angel and each other player puts a 3/3 black Angel creature token with flying onto the battlefield.","2W","Creature - Angel");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Misdirection","U","You may exile a blue card from your hand rather than pay Misdirection's mana cost.
Change the target of target spell with a single target.","3UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Voidmage Husher","2","2","U","Flash (You may cast this spell any time you could cast an instant.)
When Voidmage Husher enters the battlefield, counter target activated ability. (Mana abilities can't be targeted.)
Whenever you cast a spell, you may return Voidmage Husher to its owner's hand.","3U","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Puppet Conjurer","1","2","B","{U}, {T}: Put a 0/1 blue Homunculus artifact creature token onto the battlefield.
At the beginning of your upkeep, sacrifice a Homunculus.","1B","Artifact Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("False Prophet","2","2","W","When False Prophet dies, exile all creatures.","2WW","Creature - Human Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Tel-Jilad Archers","2","4","G","Protection from artifacts; reach (This creature can block creatures with flying.)","4G","Creature - Elf Archer");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wormfang Newt","2","2","U","When Wormfang Newt enters the battlefield, exile a land you control.
When Wormfang Newt leaves the battlefield, return the exiled card to the battlefield under its owner's control.","1U","Creature - Nightmare Salamander Beast");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Squee, Goblin Nabob","1","1","R","At the beginning of your upkeep, you may return Squee, Goblin Nabob from your graveyard to your hand.","2R","Legendary Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Awakening Zone","G","At the beginning of your upkeep, you may put a 0/1 colorless Eldrazi Spawn creature token onto the battlefield. It has ""Sacrifice this creature: Add {1} to your mana pool.""","2G","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Embalmed Brawler","2","2","B","Amplify 1 (As this creature enters the battlefield, put a +1/+1 counter on it for each Zombie card you reveal in your hand.)
Whenever Embalmed Brawler attacks or blocks, you lose 1 life for each +1/+1 counter on it.","2B","Creature - Zombie");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vile Bile","2{1/2}","2{1/2}","B","Whenever a player's skin or fingernail touches Vile Bile, that player loses 2 life.","1B","Creature - Ooze");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Rocket Launcher","{2}: Rocket Launcher deals 1 damage to target creature or player. Destroy Rocket Launcher at the beginning of the next end step. Activate this ability only if you've controlled Rocket Launcher continuously since the beginning of your most recent turn.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Reverberation","U","All damage that would be dealt this turn by target sorcery spell is dealt to that spell's controller instead.","2UU","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Infernal Spawn of Infernal Spawn of Evil","8","8","B","Flying, first strike, trample
If you say ""I'm coming, too"" as you search your library, you may pay {1B} and reveal Infernal Spawn of Infernal Spawn of Evil from your library to have it deal 2 damage to a player of your choice. Do this no more than once each turn.","8BB","Creature - Demon Child");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Rage Weaver","2","1","R","{2}: Target black or green creature gains haste until end of turn. (It can attack and {T} this turn.)","1R","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Identity Crisis","WB","Exile all cards from target player's hand and graveyard.","2WWBB","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Implements of Sacrifice","{1}, {T}, Sacrifice Implements of Sacrifice: Add two mana of any one color to your mana pool.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Landslide","R","Sacrifice any number of Mountains. Landslide deals that much damage to target player.","R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nevinyrral's Disk","Nevinyrral's Disk enters the battlefield tapped.
{1}, {T}: Destroy all artifacts, creatures, and enchantments.","4","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Eager Cadet","1","1","W",NULL,"W","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Polymorph","U","Destroy target creature. It can't be regenerated. Its controller reveals cards from the top of his or her library until he or she reveals a creature card. The player puts that card onto the battlefield, then shuffles all other cards revealed this way into his or her library.","3U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frenetic Sliver","2","2","UR","All Slivers have ""{0}: If this permanent is on the battlefield, flip a coin. If you win the flip, exile this permanent and return it to the battlefield under its owner's control at the beginning of the next end step. If you lose the flip, sacrifice it.""","1UR","Creature - Sliver");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Battlewand Oak","1","3","G","Whenever a Forest enters the battlefield under your control, Battlewand Oak gets +2/+2 until end of turn.
Whenever you cast a Treefolk spell, Battlewand Oak gets +2/+2 until end of turn.","2G","Creature - Treefolk Warrior");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Raven's Run","All creatures have wither. (They deal damage to creatures in the form of -1/-1 counters.)
Whenever you roll {C}, put a -1/-1 counter on target creature, two -1/-1 counters on another target creature, and three -1/-1 counters on a third target creature.",NULL,"Plane - Shadowmoor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Orcish Conscripts","2","2","R","Orcish Conscripts can't attack unless at least two other creatures attack.
Orcish Conscripts can't block unless at least two other creatures block.","R","Creature - Orc");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Revelsong Horn","{1}, {T}, Tap an untapped creature you control: Target creature gets +1/+1 until end of turn.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Clockspinning","U","Buyback {3} (You may pay an additional {3} as you cast this spell. If you do, put this card into your hand as it resolves.)
Choose a counter on target permanent or suspended card. Remove that counter from that permanent or card or put another of those counters on it.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Rune of Protection: Green","W","{W}: The next time a green source of your choice would deal damage to you this turn, prevent that damage.
Cycling {2} ({2}, Discard this card: Draw a card.)","1W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Gerrard","At the beginning of your draw step, draw an additional card.",NULL,"Vanguard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Frozen Shade","0","1","B","{B}: Frozen Shade gets +1/+1 until end of turn.","2B","Creature - Shade");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Counterbore","U","Counter target spell. Search its controller's graveyard, hand, and library for all cards with the same name as that spell and exile them. Then that player shuffles his or her library.","3UU","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Syphon Flesh","B","Each other player sacrifices a creature. You put a 2/2 black Zombie creature token onto the battlefield for each creature sacrificed this way.","4B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Verdant Force","7","7","G","At the beginning of each upkeep, put a 1/1 green Saproling creature token onto the battlefield.","5GGG","Creature - Elemental");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ambassador Laquatus","1","3","U","{3}: Target player puts the top three cards of his or her library into his or her graveyard.","1UU","Legendary Creature - Merfolk Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deathcoil Wurm","7","6","G","You may have Deathcoil Wurm assign its combat damage as though it weren't blocked.","6GG","Creature - Wurm");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Razor Golem","3","4","Affinity for Plains (This spell costs {1} less to cast for each Plains you control.)
Vigilance","6","Artifact Creature - Golem");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Winds of Rath","W","Destroy all creatures that aren't enchanted. They can't be regenerated.","3WW","Sorcery");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Stalking Vampire","5","5","At the beginning of your upkeep, you may pay {2}{B}{B}. If you do, transform Stalking Vampire.",NULL,"Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Deity of Scars","7","7","BG","Trample
Deity of Scars enters the battlefield with two -1/-1 counters on it.
{(b/g)}, Remove a -1/-1 counter from Deity of Scars: Regenerate Deity of Scars.","(B/G)(B/G)(B/G)(B/G)(B/G)","Creature - Spirit Avatar");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Fog Elemental","4","4","U","Flying (This creature can't be blocked except by creatures with flying or reach.)
When Fog Elemental attacks or blocks, sacrifice it at end of combat.","2U","Creature - Elemental");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Clock of Omens","Tap two untapped artifacts you control: Untap target artifact.","4","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Honden of Night's Reach","B","At the beginning of your upkeep, target opponent discards a card for each Shrine you control.","3B","Legendary Enchantment - Shrine");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Thornbite Staff","Equipped creature has ""{2}, {T}: This creature deals 1 damage to target creature or player"" and ""Whenever a creature dies, untap this creature.""
Whenever a Shaman creature enters the battlefield, you may attach Thornbite Staff to it.
Equip {4}","2","Tribal Artifact - Shaman Equipment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Blazing Archon","5","6","W","Flying
Creatures can't attack you.","6WWW","Creature - Archon");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blackmail","B","Target player reveals three cards from his or her hand and you choose one of them. That player discards that card.","B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vulshok Sorcerer","1","1","R","Haste
{T}: Vulshok Sorcerer deals 1 damage to target creature or player.","1RR","Creature - Human Shaman");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brainwash","W","Enchant creature
Enchanted creature can't attack unless its controller pays {3}.","W","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Well of Knowledge","{2}: Draw a card. Any player may activate this ability but only during his or her draw step.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Spiritualize","W","Until end of turn, whenever target creature deals damage, you gain that much life.
Draw a card.","2W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Emerald Charm","G","Choose one - Untap target permanent; or destroy target non-Aura enchantment; or target creature loses flying until end of turn.","G","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Demon's Herald","1","1","B","{2}{B}, {T}, Sacrifice a blue creature, a black creature, and a red creature: Search your library for a card named Prince of Thralls and put it onto the battlefield. Then shuffle your library.","B","Creature - Human Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Serene Sunset","G","Prevent all combat damage X target creatures would deal this turn.","XG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vedalken AEthermage","1","2","U","Flash (You may cast this spell any time you could cast an instant.)
When Vedalken Æthermage enters the battlefield, return target Sliver to its owner's hand.
Wizardcycling {3} ({3}, Discard this card: Search your library for a Wizard card, reveal it, and put it into your hand. Then shuffle your library.)","1U","Creature - Vedalken Wizard");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fiery Mantle","R","Enchant creature
{R}: Enchanted creature gets +1/+0 until end of turn.
When Fiery Mantle is put into a graveyard from the battlefield, return Fiery Mantle to its owner's hand.","1R","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Angelic Renewal","W","Whenever a creature is put into your graveyard from the battlefield, you may sacrifice Angelic Renewal. If you do, return that card to the battlefield.","1W","Enchantment");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Knight of Dusk","2","2","B","{B}{B}: Destroy target creature blocking Knight of Dusk.","1BB","Creature - Human Knight");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Flametongue Kavu","4","2","R","When Flametongue Kavu enters the battlefield, it deals 4 damage to target creature.","3R","Creature - Kavu");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Bringer of the Green Dawn","5","5","G","You may pay {W}{U}{B}{R}{G} rather than pay Bringer of the Green Dawn's mana cost.
Trample
At the beginning of your upkeep, you may put a 3/3 green Beast creature token onto the battlefield.","7GG","Creature - Bringer");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Terminus","W","Put all creatures on the bottom of their owners' libraries.
Miracle {W} (You may cast this card for its miracle cost when you draw it if it's the first card you drew this turn.)","4WW","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Radha, Heir to Keld","2","2","RG","Whenever Radha, Heir to Keld attacks, you may add {R}{R} to your mana pool.
{T}: Add {G} to your mana pool.","RG","Legendary Creature - Elf Warrior");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Garza Zol, Plague Queen","5","5","UBR","Flying, haste
Whenever a creature dealt damage by Garza Zol, Plague Queen this turn dies, put a +1/+1 counter on Garza Zol.
Whenever Garza Zol deals combat damage to a player, you may draw a card.","4UBR","Legendary Creature - Vampire");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soltari Monk","2","1","W","Protection from black
Shadow (This creature can block or be blocked by only creatures with shadow.)","WW","Creature - Soltari Monk Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Herald of Leshrac","2","4","B","Flying
Cumulative upkeep-Gain control of a land you don't control. (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
Herald of Leshrac gets +1/+1 for each land you control but don't own.
When Herald of Leshrac leaves the battlefield, each player gains control of each land he or she owns that you control.","6B","Creature - Avatar");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Kraken's Eye","Whenever a player casts a blue spell, you may gain 1 life.","2","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Fertile Ground","G","Enchant land
Whenever enchanted land is tapped for mana, its controller adds one mana of any color to his or her mana pool (in addition to the mana the land produces).","1G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lim-Dûl's High Guard","2","1","B","First strike
{1}{B}: Regenerate Lim-Dûl's High Guard.","1BB","Creature - Skeleton");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Pyromancer's Swath","R","If an instant or sorcery source you control would deal damage to a creature or player, it deals that much damage plus 2 to that creature or player instead.
At the beginning of each end step, discard your hand.","2R","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Bronze Tablet","Remove Bronze Tablet from your deck before playing if you're not playing for ante.
Bronze Tablet enters the battlefield tapped.
{4}, {T}: Exile Bronze Tablet and target nontoken permanent an opponent owns. That player may pay 10 life. If he or she does, put Bronze Tablet into its owner's graveyard. Otherwise, that player owns Bronze Tablet and you own the other exiled card.","6","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Goblin Commando","2","2","R","When Goblin Commando enters the battlefield, it deals 2 damage to target creature.","4R","Creature - Goblin");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Broken Visage","B","Destroy target nonartifact attacking creature. It can't be regenerated. Put a black Spirit creature token with that creature's power and toughness onto the battlefield. Sacrifice the token at the beginning of the next end step.","4B","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Mind Shatter","B","Target player discards X cards at random.","XBB","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brainstorm","U","Draw three cards, then put two cards from your hand on top of your library in any order.","U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Unlife","W","You don't lose the game for having 0 or less life.
As long as you have 0 or less life, all damage is dealt to you as though its source had infect. (Damage is dealt to you in the form of poison counters.)","2W","Enchantment");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Jandor's Ring","{2}, {T}, Discard the last card you drew this turn: Draw a card.","6","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Blessed Reversal","W","You gain 3 life for each creature attacking you.","1W","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Phyrexian Etchings","B","Cumulative upkeep {B} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of your end step, draw a card for each age counter on Phyrexian Etchings.
When Phyrexian Etchings is put into a graveyard from the battlefield, you lose 2 life for each age counter on it.","BBB","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Howlpack of Estwald","4","6","At the beginning of each upkeep, if a player cast two or more spells last turn, transform Howlpack of Estwald.",NULL,"Creature - Werewolf");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Propaganda","U","Creatures can't attack you unless their controller pays {2} for each creature he or she controls that's attacking you.","2U","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Desecrated Earth","B","Destroy target land. Its controller discards a card.","4B","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kathari Screecher","2","2","U","Flying
Unearth {2}{U} ({2}{U}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","2U","Creature - Bird Soldier");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lore Broker","1","2","U","{T}: Each player draws a card, then discards a card.","1U","Creature - Human Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Worn Powerstone","Worn Powerstone enters the battlefield tapped.
{T}: Add {2} to your mana pool.","3","Artifact");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Govern the Guildless","U","Gain control of target monocolored creature.
Forecast - {1}{U}, Reveal Govern the Guildless from your hand: Target creature becomes the color or colors of your choice until end of turn. (Activate this ability only during your upkeep and only once each turn.)","5U","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Essence Feed","B","Target player loses 3 life. You gain 3 life and put three 0/1 colorless Eldrazi Spawn creature tokens onto the battlefield. They have ""Sacrifice this creature: Add {1} to your mana pool.""","5B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Thopter Foundry","WUB","{1}, Sacrifice a nontoken artifact: Put a 1/1 blue Thopter artifact creature token with flying onto the battlefield. You gain 1 life.","(W/B)U","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Boseiju, Who Shelters All","Boseiju, Who Shelters All enters the battlefield tapped.
{T}, Pay 2 life: Add {1} to your mana pool. If that mana is spent on an instant or sorcery spell, that spell can't be countered by spells or abilities.",NULL,"Legendary Land");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Emcee","0","1","W","Whenever another creature comes into play, you may stand up and say in a deep, booming voice ""Presenting . . . "" and that creature's name. If you do, put a +1/+1 counter on that creature.","2W","Creature - Human Rogue");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Corrosion","BR","Cumulative upkeep {1} (At the beginning of your upkeep, put an age counter on this permanent, then sacrifice it unless you pay its upkeep cost for each age counter on it.)
At the beginning of your upkeep, put a rust counter on each artifact target opponent controls. Then destroy each artifact with converted mana cost less than or equal to the number of rust counters on it. Artifacts destroyed this way can't be regenerated.
When Corrosion leaves the battlefield, remove all rust counters from all permanents.","1BR","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ignite Memories","R","Target player reveals a card at random from his or her hand. Ignite Memories deals damage to that player equal to that card's converted mana cost.
Storm (When you cast this spell, copy it for each spell cast before it this turn. You may choose new targets for the copies.)","4R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wooden Stake","Equipped creature gets +1/+0.
Whenever equipped creature blocks or becomes blocked by a Vampire, destroy that creature. It can't be regenerated.
Equip {1} ({1}: Attach to target creature you control. Equip only as a sorcery.)","2","Artifact - Equipment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Ether Well","U","Put target creature on top of its owner's library. If that creature is red, you may put it on the bottom of its owner's library instead.","3U","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Wave of Reckoning","W","Each creature deals damage to itself equal to its power.","4W","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Haunted Fengraf","{T}: Add {1} to your mana pool.
{3}, {T}, Sacrifice Haunted Fengraf: Return a creature card at random from your graveyard to your hand.",NULL,"Land");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Prismatic Lens","{T}: Add {1} to your mana pool.
{1}, {T}: Add one mana of any color to your mana pool.","2","Artifact");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dream Thief","2","1","U","Flying
When Dream Thief enters the battlefield, draw a card if you've cast another blue spell this turn.","2U","Creature - Faerie Rogue");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Wooded Bastion","{T}: Add {1} to your mana pool.
{(g/w)}, {T}: Add {G}{G}, {G}{W}, or {W}{W} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Necra Sanctuary","B","At the beginning of your upkeep, if you control a green or white permanent, target player loses 1 life. If you control a green permanent and a white permanent, that player loses 3 life instead.","2B","Enchantment");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Su-Chi","4","4","When Su-Chi dies, add {4} to your mana pool.","4","Artifact Creature - Construct");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Crumbling Necropolis","Crumbling Necropolis enters the battlefield tapped.
{T}: Add {U}, {B}, or {R} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Slave of Bolas","UBR","Gain control of target creature. Untap that creature. It gains haste until end of turn. Sacrifice it at the beginning of the next end step.","3(U/R)B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Name Dropping","G","Gotcha Whenever an opponent says a word that's in the name of a card in your graveyard, you may say ""Gotcha"" If you do, return that card to your hand.","1G","Enchantment");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Vow of Malice","B","Enchant creature
Enchanted creature gets +2/+2, has intimidate, and can't attack you or a planeswalker you control.","2B","Enchantment - Aura");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Barrin's Codex","At the beginning of your upkeep, you may put a page counter on Barrin's Codex.
{4}, {T}, Sacrifice Barrin's Codex: Draw X cards, where X is the number of page counters on Barrin's Codex.","4","Artifact");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Golgari Rot Farm","Golgari Rot Farm enters the battlefield tapped.
When Golgari Rot Farm enters the battlefield, return a land you control to its owner's hand.
{T}: Add {B}{G} to your mana pool.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Gift of the Woods","G","Enchant creature
Whenever enchanted creature blocks or becomes blocked, it gets +0/+3 until end of turn and you gain 1 life.","G","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Field Marshal","2","2","W","Other Soldier creatures get +1/+1 and have first strike. (They deal combat damage before creatures without first strike.)","1WW","Creature - Human Soldier");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Raven's Crime","B","Target player discards a card.
Retrace (You may cast this card from your graveyard by discarding a land card in addition to paying its other costs.)","B","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Eternal Warrior","R","Enchant creature
Enchanted creature has vigilance.","R","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kuldotha Phoenix","4","4","R","Flying, haste
Metalcraft - {4}: Return Kuldotha Phoenix from your graveyard to the battlefield. Activate this ability only during your upkeep and only if you control three or more artifacts.","2RRR","Creature - Phoenix");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Nimbus Maze","{T}: Add {1} to your mana pool.
{T}: Add {W} to your mana pool. Activate this ability only if you control an Island.
{T}: Add {U} to your mana pool. Activate this ability only if you control a Plains.",NULL,"Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Burn Trail","R","Burn Trail deals 3 damage to target creature or player.
Conspire (As you cast this spell, you may tap two untapped creatures you control that share a color with it. When you do, copy it and you may choose a new target for the copy.)","3R","Sorcery");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Brilliant Plan","U","Draw three cards.","4U","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Vorosh, the Hunter","6","6","UBG","Flying
Whenever Vorosh, the Hunter deals combat damage to a player, you may pay {2}{G}. If you do, put six +1/+1 counters on Vorosh.","3GUB","Legendary Creature - Dragon");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Red-Hot Hottie","2","5","R","Whenever Red-Hot Hottie deals damage to a creature, put a third-degree-burn counter on that creature. It has ""At the end of each turn, sacrifice this creature unless you scream ‘Aaah' at the top of your lungs.""","2RR","Creature - Elemental");
INSERT INTO cards(name,power,toughness,cardtext,manacost,type) VALUES("Myr Galvanizer","2","2","Other Myr creatures you control get +1/+1.
{1}, {T}: Untap each other Myr you control.","3","Artifact Creature - Myr");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Wishmonger","3","3","W","{2}: Target creature gains protection from the color of its controller's choice until end of turn. Any player may activate this ability.","3W","Creature - Unicorn Monger");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lord of the Unreal","2","2","U","Illusion creatures you control get +1/+1 and have hexproof. (They can't be the targets of spells or abilities your opponents control.)","UU","Creature - Human Wizard");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Undead Leotau","3","4","B","{R}: Undead Leotau gets +1/-1 until end of turn.
Unearth {2}{B} ({2}{B}: Return this card from your graveyard to the battlefield. It gains haste. Exile it at the beginning of the next end step or if it would leave the battlefield. Unearth only as a sorcery.)","5B","Creature - Zombie Cat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Zodiac Goat","1","1","R","Mountainwalk","R","Creature - Goat");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Kitsune Bonesetter","0","1","W","{T}: Prevent the next 3 damage that would be dealt to target creature this turn. Activate this ability only if you have more cards in hand than each opponent.","2W","Creature - Fox Cleric");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Dark Hatchling","3","3","B","Flying
When Dark Hatchling enters the battlefield, destroy target nonblack creature. It can't be regenerated.","4BB","Creature - Horror");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Ninja of the Deep Hours","2","2","U","Ninjutsu {1}{U} ({1}{U}, Return an unblocked attacker you control to hand: Put this card onto the battlefield from your hand tapped and attacking.)
Whenever Ninja of the Deep Hours deals combat damage to a player, you may draw a card.","3U","Creature - Human Ninja");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Stasis Cell","U","Enchant creature
Enchanted creature doesn't untap during its controller's untap step.
{3}{U}: Attach Stasis Cell to target creature.","4U","Enchantment - Aura");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Needlebite Trap","B","If an opponent gained life this turn, you may pay {B} rather than pay Needlebite Trap's mana cost.
Target player loses 5 life and you gain 5 life.","5BB","Instant - Trap");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Aggressive Urge","G","Target creature gets +1/+1 until end of turn.
Draw a card.","1G","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Kjeldoran Pride","W","Enchant creature
Enchanted creature gets +1/+2.
{2}{U}: Attach Kjeldoran Pride to target creature other than enchanted creature.","1W","Enchantment - Aura");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Elesh Norn, Grand Cenobite","4","7","W","Vigilance
Other creatures you control get +2/+2.
Creatures your opponents control get -2/-2.","5WW","Legendary Creature - Praetor");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Mesa Chicken","2","2","W","Stand up, Flap your arms, Cluck like a chicken: Mesa Chicken gains flying until end of turn.","WW","Summon - Chicken");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Surge of Strength","RG","As an additional cost to cast Surge of Strength, discard a red or green card.
Target creature gains trample and gets +X/+0 until end of turn, where X is that creature's converted mana cost.","RG","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Treacherous Werewolf","2","2","B","Threshold - As long as seven or more cards are in your graveyard, Treacherous Werewolf gets +2/+2 and has ""When Treacherous Werewolf dies, you lose 4 life.""","2B","Creature - Werewolf Minion");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Thelonite Monk","1","2","G","{T}, Sacrifice a green creature: Target land becomes a Forest. (This effect lasts indefinitely.)","2GG","Creature - Insect Monk Cleric");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Trash for Treasure","R","As an additional cost to cast Trash for Treasure, sacrifice an artifact.
Return target artifact card from your graveyard to the battlefield.","2R","Sorcery");
INSERT INTO cards(name,cardtext,manacost,type) VALUES("Okina, Temple to the Grandfathers","{T}: Add {G} to your mana pool.
{G}, {T}: Target legendary creature gets +1/+1 until end of turn.",NULL,"Legendary Land");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Tsunami","G","Destroy all Islands.","3G","Sorcery");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Crash of Rhinos","8","4","G","Trample","6GG","Creature - Rhino");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Soulsurge Elemental","*","1","R","First strike
Soulsurge Elemental's power is equal to the number of creatures you control.","3R","Creature - Elemental");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Flaming Gambit","R","Flaming Gambit deals X damage to target player. That player may choose a creature he or she controls and have Flaming Gambit deal that damage to it instead.
Flashback {X}{R}{R} (You may cast this card from your graveyard for its flashback cost. Then exile it.)","XR","Instant");
INSERT INTO cards(name,color,cardtext,manacost,type) VALUES("Break of Day","W","Creatures you control get +1/+1 until end of turn.
Fateful hour - If you have 5 or less life, those creatures also are indestructible this turn. (Lethal damage and effects that say ""destroy"" don't destroy them.)","1W","Instant");
INSERT INTO cards(name,power,toughness,color,cardtext,manacost,type) VALUES("Lightning Hounds","3","2","R","First strike","2RR","Creature - Hound");
COMMIT TRANSACTION;

-- CARD TO SET RELATION --
BEGIN TRANSACTION;
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(13,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(14,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=246949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(15,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(16,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(17,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(18,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(19,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(20,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(20,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(21,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(21,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(22,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(23,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(23,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(23,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(24,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(25,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(26,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(27,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(28,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(29,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(30,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(31,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(32,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(33,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(34,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(35,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(36,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(37,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(37,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(37,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(37,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(37,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(37,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(37,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(38,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(38,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(38,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(39,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(39,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(40,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(41,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(42,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(42,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(43,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(44,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(45,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(46,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(47,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(48,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(49,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(50,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(51,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(52,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(53,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(54,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(55,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(56,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(56,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(56,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(57,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(58,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(59,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(60,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(60,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(61,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(62,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(63,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(64,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(64,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(64,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(64,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(64,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(64,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(65,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(66,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(67,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(68,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(69,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(70,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(71,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(72,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(73,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(73,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(74,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(74,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(74,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(74,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(75,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(76,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(77,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(78,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(79,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(80,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(80,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(80,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(81,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(82,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(83,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(84,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(85,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(85,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(86,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(86,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(86,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(86,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(87,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(87,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(88,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(88,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(89,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(90,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(91,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(92,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(92,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(92,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(92,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(93,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(94,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(95,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(96,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(97,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(98,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=279608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(99,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(100,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(100,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(100,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(101,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(102,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(103,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(104,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(105,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(105,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(106,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(107,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(107,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(108,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(109,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(110,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(111,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(111,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(112,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(113,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(113,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(114,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(115,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(116,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(117,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(118,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(119,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(120,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(121,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(121,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(122,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(123,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(123,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(124,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(124,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(125,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(126,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(127,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(127,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(127,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(128,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(129,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(130,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(131,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(132,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(133,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(134,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(135,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(136,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(137,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(138,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(139,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(140,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(141,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(142,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(143,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(143,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(144,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(145,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(146,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(147,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(148,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(149,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(150,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(151,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(152,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(153,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(154,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(155,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(156,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(157,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(157,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(157,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(157,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(157,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(157,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(158,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(159,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(160,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(161,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(162,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(163,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(164,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(165,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(166,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(167,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(168,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(168,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(168,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(169,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(170,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(170,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(170,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(171,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(171,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(172,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(173,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(174,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(175,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(176,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(177,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(178,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(178,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(179,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(180,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(181,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(182,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(182,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(182,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(182,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(183,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(184,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(185,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(186,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(187,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(187,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(188,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(189,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(190,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(191,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(191,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(191,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(191,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(191,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(192,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(193,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(194,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(195,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(196,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(197,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(198,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(199,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(200,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(200,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(201,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(202,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(203,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(204,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(204,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(205,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(206,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(207,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(207,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(208,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(209,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(210,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=229965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(211,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(212,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(213,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(214,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(215,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(215,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(216,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(217,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(218,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(219,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(220,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(221,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(221,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(222,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(223,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(224,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(224,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(225,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(226,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(226,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(226,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(227,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(228,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(228,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(228,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(228,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(228,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(229,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(229,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(230,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(231,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(232,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(233,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(234,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(234,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(235,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(236,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(237,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(238,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(239,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(240,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(241,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(242,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(243,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(244,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(245,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(246,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(247,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(248,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(249,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(249,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(249,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(250,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(251,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(251,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(251,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(251,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(251,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(252,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(253,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(254,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(255,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(256,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(256,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(256,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(256,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(257,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(257,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(257,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(257,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(258,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(259,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(260,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(260,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(261,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(261,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(262,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(262,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(263,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(264,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(265,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(266,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(267,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(268,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(268,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(269,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(270,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(271,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(271,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(272,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(273,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(273,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(273,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(274,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(275,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(276,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(277,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(278,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(279,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(280,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(280,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(280,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(281,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(281,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(282,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(283,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(284,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(284,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(284,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(284,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(285,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(286,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(287,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(288,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(289,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(290,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(291,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(292,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(293,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(293,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(294,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(295,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(296,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(297,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(298,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(299,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(300,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(301,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(302,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(303,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(304,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(305,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(306,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(307,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(308,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(309,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(310,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(310,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(310,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(310,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(310,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(310,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(310,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(311,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(311,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(312,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(313,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(314,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(315,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(316,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(317,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(318,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(319,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(319,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(320,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(320,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(320,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(320,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(321,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(321,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(322,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(322,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(322,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(322,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(322,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(323,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(324,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(325,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(326,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(327,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(328,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(328,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(328,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(328,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(329,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(330,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(331,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(332,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(333,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(334,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(334,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(335,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(336,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(337,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(338,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(339,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(339,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(339,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(339,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(339,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(340,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(341,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(342,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(343,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(343,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(344,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(345,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=246954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(346,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(347,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(347,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(348,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(348,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(349,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(349,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(350,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(351,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(352,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(352,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(352,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(352,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(353,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(354,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(355,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(355,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(356,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(357,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(358,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(358,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(359,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(360,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(360,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(361,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(362,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(362,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(363,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(363,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(364,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(365,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(366,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(367,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(368,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(369,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(369,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(370,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(371,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(372,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(373,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(374,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(375,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(376,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(376,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(377,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(378,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(379,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(380,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(381,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(382,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(383,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(384,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(384,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(385,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(386,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(387,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(388,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(388,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=94910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(388,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(388,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(388,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(389,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(390,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(391,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(392,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(393,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(394,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(395,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(396,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(397,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(398,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(399,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(400,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(401,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(402,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(403,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(404,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(405,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(406,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(407,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(408,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(409,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(410,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(410,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(411,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(412,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(413,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(414,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(415,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(416,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(416,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(417,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(418,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(418,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(418,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(418,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(418,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(419,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(419,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(420,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(421,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(421,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(421,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(422,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(423,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(424,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(425,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(426,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(427,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(428,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(429,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(430,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(431,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(432,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(433,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(434,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(435,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(436,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(437,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(438,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(439,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(440,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(441,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(442,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(443,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(444,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(444,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(445,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(446,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(447,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(448,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(449,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(450,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(450,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(451,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(452,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(453,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(454,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(454,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(455,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(455,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(456,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(457,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(458,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(459,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(459,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(459,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(460,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(461,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(461,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(462,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(463,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(464,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(464,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(464,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(464,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(464,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(464,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(464,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(465,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(466,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(467,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(467,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(468,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(468,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(469,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(470,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(471,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(472,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(472,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(472,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(472,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(473,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(474,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(475,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(476,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(476,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(477,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(478,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(479,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(480,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(480,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(481,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(482,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(483,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(484,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(485,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(486,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(487,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(488,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(489,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(490,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(490,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(491,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(492,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(492,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(492,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(492,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(492,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(492,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(492,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(493,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(494,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(495,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(495,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(496,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(496,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(496,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(496,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(496,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(496,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(497,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(498,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(499,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(499,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(500,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(500,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(501,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(502,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(502,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(503,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(504,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(505,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(506,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(506,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(507,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(508,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(509,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(510,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(511,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(512,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(513,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(514,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(515,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(516,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(517,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(518,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(519,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(520,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(521,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(522,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(523,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(524,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(525,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(526,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(526,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(527,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(528,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(529,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(530,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(531,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(532,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(533,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(533,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(534,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(534,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(535,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(536,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(536,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(537,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(537,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(538,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(539,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(540,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(541,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(542,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(543,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(544,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(544,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(545,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(546,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(547,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(548,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(549,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(550,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(551,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=18287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(552,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(553,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(553,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(554,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(555,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(555,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(555,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(556,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(557,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(558,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(559,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(560,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(561,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(562,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(563,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(563,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(563,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(563,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(564,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(565,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(566,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(566,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(567,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(568,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(568,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(569,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(570,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(570,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(571,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(572,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(573,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(574,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(574,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(575,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(576,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(577,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(577,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(578,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(579,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(580,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(581,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(582,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(583,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(584,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(585,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(586,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(587,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(588,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(588,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(589,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(590,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(590,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(591,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(592,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(593,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(594,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(595,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(596,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(597,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(598,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(598,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(599,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(599,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(600,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(600,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(600,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(600,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(600,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(601,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(601,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(601,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(601,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(601,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(601,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(601,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(602,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(603,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(604,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(605,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(605,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(605,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(606,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(607,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(607,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(608,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(609,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(609,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(610,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(610,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(610,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(611,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(611,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(612,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(613,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(614,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(615,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(615,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(616,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(616,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(617,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(618,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(619,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(620,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(621,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(621,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(622,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(623,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(624,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(624,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(625,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(626,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(627,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(628,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(629,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(629,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(629,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(629,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(630,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(631,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(632,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(633,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(634,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(635,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(636,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=169971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(637,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(638,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(639,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(640,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(641,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(642,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(643,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(644,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(645,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(646,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(647,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(648,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(649,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(650,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(650,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(650,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(650,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(651,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(652,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(652,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(653,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(654,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(655,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(655,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(656,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(656,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(657,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(658,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(659,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(660,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(661,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(662,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(663,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(663,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(664,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(665,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(665,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(666,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(666,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(666,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(667,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(667,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(667,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(668,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(669,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(670,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(671,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(672,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(673,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(673,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(674,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(674,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(674,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(675,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(676,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(677,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(678,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(679,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(680,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(681,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(681,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(681,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(681,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(681,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(682,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(683,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(684,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(685,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(686,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(687,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(687,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(688,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(688,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(688,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(689,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(690,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(691,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(692,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(693,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(694,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(695,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(696,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(697,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(698,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(699,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(700,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(700,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(700,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(700,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(700,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(701,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(702,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(703,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(704,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(704,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(705,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(706,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(706,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(707,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(708,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(709,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(710,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(711,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(711,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(711,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(711,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(711,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(712,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(713,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(714,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(715,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(716,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=37120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(717,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(718,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(719,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(720,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(720,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(720,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(721,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(722,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(723,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(724,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(725,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(726,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(727,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(728,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(729,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(730,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(730,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(730,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(730,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(730,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(730,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(730,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(731,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(732,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(733,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(734,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(735,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(736,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(736,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(737,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(738,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=266305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(738,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(739,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(740,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(741,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(742,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(743,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(744,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(745,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(745,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(745,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(746,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(746,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(746,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(746,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(746,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(746,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(747,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(748,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(749,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(750,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(751,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(752,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(752,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(753,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(754,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(754,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(754,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(754,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(755,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(756,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(757,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(758,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(758,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(758,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(758,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(758,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(759,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(760,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(760,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(760,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(761,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(762,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(763,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(764,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(765,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(765,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(766,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(767,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(768,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(769,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(770,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(770,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(771,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(771,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(771,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(772,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(772,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(773,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(774,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(775,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(775,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(776,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(777,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(778,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(779,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(780,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(781,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(782,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(783,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(784,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(785,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(786,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(786,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(786,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(787,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(788,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(789,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(790,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(791,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(792,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(793,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(794,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(794,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(795,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(796,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(797,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(797,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(797,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(798,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(799,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(799,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(799,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(800,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(801,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(801,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(802,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(803,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(804,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(805,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(806,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(807,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(808,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(808,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(809,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(810,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(810,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(811,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(812,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(813,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(814,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(814,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(814,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(814,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(814,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(815,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(816,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(817,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(818,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(818,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(819,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(819,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(820,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(821,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(821,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(822,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(823,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(824,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(825,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(825,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(825,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(826,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(827,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(828,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(828,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(828,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(829,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(830,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(831,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(832,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(833,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(834,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(835,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(836,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(837,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(838,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(839,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(839,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(840,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(841,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(842,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(843,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(844,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(845,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(846,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(846,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(847,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(848,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(849,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(850,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(851,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(852,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(853,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(854,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(855,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(855,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(855,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(856,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(856,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(856,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(856,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(857,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(858,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(858,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(859,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(859,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(860,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(860,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(861,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(862,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(863,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(864,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(865,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(866,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(866,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(867,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(868,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(869,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(869,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(869,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(869,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(869,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(869,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(870,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(871,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(872,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(873,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(874,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(875,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(876,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(877,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(878,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(879,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(880,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(880,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(880,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(880,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(880,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(880,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(881,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(881,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(881,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(881,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(882,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(883,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(884,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(885,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(886,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(887,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(888,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(889,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(890,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=253426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(891,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(891,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(891,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(891,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(892,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(892,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(893,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(894,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(895,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(896,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(896,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(897,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(898,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(899,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(900,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(901,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(902,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(903,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(903,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(904,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(905,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(906,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(907,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(907,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(907,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(907,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(907,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(907,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(908,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(909,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(910,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(911,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(912,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(913,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(914,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(915,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(916,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(917,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(918,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(918,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(919,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(919,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(919,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(920,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(920,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(920,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(920,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(920,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(920,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(920,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(921,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(921,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(922,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(923,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(923,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(924,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(925,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(925,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(926,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(926,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(927,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(927,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(927,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(927,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(928,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(929,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(930,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(930,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(931,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(931,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(932,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(933,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(934,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(935,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(936,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(936,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(937,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(937,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(938,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(939,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(939,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(940,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(941,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(942,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(942,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(942,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(943,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(944,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(945,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(945,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(945,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(945,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(946,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(947,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(948,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(949,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(950,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(950,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(951,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(952,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(952,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(952,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(952,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(953,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(954,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(955,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(956,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(957,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(958,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=279611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(959,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(960,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(961,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(961,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(961,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(962,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(963,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(964,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(965,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(966,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(967,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(968,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(969,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(970,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(971,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(972,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(973,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(974,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(975,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(975,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(975,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(976,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(976,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(976,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(977,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(977,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(978,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(978,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(979,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(980,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=171012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(981,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(981,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(982,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(983,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(984,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(985,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(986,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(986,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(987,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(987,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(988,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(989,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(990,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(991,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(992,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(993,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(993,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(993,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(994,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(995,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(996,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(997,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(998,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(999,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1000,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1000,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1001,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1002,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1003,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1004,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1004,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1005,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1006,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1007,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1008,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1009,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1010,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1011,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1012,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1013,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1014,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1015,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1015,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1016,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1017,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1018,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1018,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1019,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1020,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1021,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1022,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1023,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1024,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1024,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1025,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1026,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1027,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1028,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1029,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1030,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1030,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1030,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1030,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1031,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1032,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1033,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1034,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1035,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1036,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1037,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1038,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1039,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1040,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1040,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1041,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1042,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1042,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1042,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1043,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1044,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1045,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1046,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1047,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1047,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1048,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1049,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1049,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1049,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1050,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1051,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1052,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1052,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1052,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1052,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1052,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1052,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1053,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1054,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1055,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1056,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1056,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1057,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1058,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1059,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1059,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1059,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1060,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1060,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1060,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1061,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1062,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1062,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1063,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1064,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1064,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1065,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1066,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1067,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1067,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1068,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1069,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1070,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1070,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1071,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1072,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1073,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1074,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1074,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1075,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1076,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1076,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1077,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1078,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1079,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1079,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1079,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1080,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1081,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1082,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1083,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1084,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1085,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1086,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1086,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1086,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1087,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1087,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1087,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1087,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1087,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1087,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1087,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1088,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1089,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1090,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1090,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1090,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1090,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1091,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1092,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1093,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1093,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1094,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1095,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1096,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1097,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1097,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1097,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1098,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1099,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1100,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=171007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1101,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1102,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1103,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1103,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1103,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1104,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1105,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1106,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1107,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1107,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1108,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1109,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1109,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1110,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1110,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1110,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1110,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1110,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1111,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1112,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1112,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1113,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1114,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1115,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1116,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1116,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1116,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1117,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1118,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1119,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1120,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1121,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1122,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1122,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1122,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1123,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1124,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1125,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1126,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1127,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1128,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1129,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1130,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1131,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1132,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1133,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1134,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1135,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1136,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1137,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1138,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1139,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1139,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1139,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1140,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1141,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1142,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1143,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1144,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1145,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1145,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1145,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1146,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1146,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1147,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1148,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1149,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1150,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1150,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1151,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1152,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1153,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1154,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1155,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1156,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1157,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1157,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1157,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1158,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1159,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1160,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1161,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1162,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1163,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1163,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1164,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1165,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1166,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1167,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1168,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1169,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1170,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1170,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1171,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1171,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1171,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1171,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1172,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1173,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1174,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1175,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1176,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1177,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1177,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1178,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1178,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1178,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1179,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1180,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1181,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1181,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1182,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1183,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1184,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1185,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1186,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1187,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1187,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1188,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1188,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1188,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1188,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1188,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1188,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1188,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1188,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1189,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1190,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1191,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1192,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1193,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1194,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1195,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1196,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1197,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1198,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1198,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1199,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1200,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1201,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1202,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1203,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1204,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1205,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1206,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1207,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1208,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1209,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1210,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1211,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1211,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1212,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1212,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1213,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1214,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1215,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1215,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1215,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1216,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1217,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1218,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1218,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1218,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1219,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1220,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1221,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1222,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1222,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1223,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1224,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1224,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1224,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1225,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1225,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1225,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1225,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1225,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1226,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1227,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1228,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1229,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1229,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1230,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1230,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1231,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1232,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1233,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1234,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1234,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1235,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1236,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1237,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1238,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1239,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1239,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1240,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1241,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1242,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1243,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1244,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1244,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1245,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1246,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1246,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1247,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1247,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1248,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1249,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1250,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1251,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1251,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1251,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1251,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1252,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1253,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1253,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1254,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1255,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1256,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1257,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1257,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1257,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1257,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1257,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1258,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1258,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1258,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1259,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1260,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1260,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1261,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1262,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1263,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1264,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1264,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1265,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1265,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1266,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1266,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1267,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1268,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1269,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1270,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1271,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1272,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1273,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1274,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1274,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1274,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1274,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1274,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1274,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1274,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1275,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1276,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1276,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1277,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1278,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1279,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1280,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1281,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1282,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1283,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1284,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1285,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1286,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1287,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1287,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1288,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1289,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1290,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1290,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1290,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1291,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1292,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1292,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1293,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1294,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1295,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1295,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1296,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1296,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1297,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1298,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1299,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1300,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1301,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1302,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1303,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1304,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1305,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1306,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1306,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1306,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1306,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1306,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1306,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1306,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1307,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1308,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1308,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1308,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1309,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1310,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1310,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1311,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1312,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1312,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1313,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1314,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1315,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1316,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1317,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1318,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1318,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1319,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1320,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1321,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1322,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1323,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1323,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1323,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1324,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1324,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1325,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1326,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1327,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1328,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1328,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1329,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1330,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1330,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1330,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1331,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1332,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1333,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1334,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1335,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1336,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1337,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1338,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1339,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1340,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1341,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1342,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1343,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1343,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1343,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1343,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1343,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1343,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1343,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1344,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1345,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1346,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1347,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=85&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1347,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1347,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1347,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1348,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1349,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1350,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1351,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1352,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1353,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1354,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1355,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1356,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1356,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1357,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1358,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1359,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1360,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1361,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1362,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1363,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1363,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1363,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1363,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1364,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1365,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1366,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1367,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1367,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1368,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1369,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1370,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1371,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1371,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1371,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1371,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1371,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1371,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1371,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1372,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1373,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1374,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1375,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1376,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1377,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1378,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1378,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1379,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1380,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1381,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1382,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1383,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1383,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1384,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1385,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1385,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1386,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1386,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1387,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1388,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1389,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1390,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1391,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1392,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1393,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1394,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1395,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1396,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1397,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1397,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1397,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1398,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1399,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1400,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1401,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1402,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1403,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1404,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1405,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1406,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1407,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1407,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1408,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1408,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1409,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1410,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1411,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1412,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1412,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1413,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1413,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1413,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1413,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1413,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1413,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1413,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1413,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1414,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1414,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1414,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1414,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1414,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1414,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1415,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1416,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1417,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1418,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1419,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1420,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1421,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1422,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1423,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1424,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1425,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1426,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1427,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1428,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1429,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1430,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1430,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1430,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1431,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1431,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1432,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1432,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1432,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1433,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1434,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1435,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1436,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1436,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1436,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1436,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1436,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1436,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1437,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1437,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1437,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1437,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1437,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1438,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1439,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1440,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1441,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1442,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1443,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1444,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1445,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1445,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1446,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1447,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1448,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1449,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1450,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1450,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1451,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1452,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1453,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1453,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1454,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1455,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1456,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1456,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1457,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1458,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1458,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1458,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1459,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1460,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1460,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1461,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1461,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1462,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1462,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1463,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1464,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1465,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1466,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1467,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1468,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1469,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1470,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1471,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1472,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1473,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1474,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1475,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1476,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1476,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1477,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1478,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1479,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1480,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1480,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1481,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1481,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1482,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1483,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1483,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1484,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1485,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1485,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1486,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1487,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1487,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1487,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1487,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1487,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1487,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1487,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1488,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1489,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1490,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1491,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1492,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1493,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1494,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1495,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1496,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1496,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1496,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1497,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1498,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1499,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1500,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1501,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1501,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1502,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1503,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1504,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1505,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1506,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1507,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1508,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1509,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1510,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1511,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1512,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1512,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1513,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1514,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1515,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1515,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1515,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1515,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1515,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1515,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1516,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1517,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1518,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1519,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1520,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1521,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1522,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1523,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1524,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1524,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1524,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1525,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1526,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1527,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1528,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1529,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1529,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1529,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1529,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1529,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1529,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1529,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1530,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1531,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1532,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1533,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1534,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1534,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1535,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1536,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1536,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1537,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1537,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1538,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1539,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1540,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1541,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1542,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1543,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1544,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1544,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1544,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1545,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1546,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1547,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1548,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1548,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1548,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1549,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1550,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1551,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1552,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1552,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1552,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1552,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1553,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1554,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1555,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1556,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1557,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1558,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1558,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1558,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1559,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1559,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1559,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1559,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1559,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1559,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1559,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1560,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1561,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1561,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1562,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1563,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1563,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1563,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1564,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1565,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1566,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1566,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1566,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1566,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1566,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1566,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1567,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1568,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1569,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1570,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1571,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1572,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1572,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1572,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1573,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1574,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1575,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1575,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1575,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1575,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1576,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1577,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1578,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1579,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1580,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1581,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1582,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1583,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1584,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1585,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1586,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1587,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1588,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1589,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1589,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1590,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1591,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1592,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1592,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1593,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1594,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1595,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1596,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1597,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1598,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1599,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1600,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1600,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1601,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1602,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1603,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1603,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1604,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1605,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1606,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1607,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1608,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1608,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1608,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1608,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1609,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1610,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1611,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1612,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1613,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1614,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1615,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1615,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1616,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1617,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1618,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1619,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1620,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1620,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1621,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1622,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1622,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1623,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1623,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1623,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1624,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1625,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1626,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1627,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1628,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1628,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1629,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1630,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1630,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1631,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1632,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1633,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1634,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1635,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1636,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1636,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1637,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1637,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1637,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1637,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1637,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1638,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1639,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1640,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1641,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1642,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1642,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1642,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1642,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1642,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1642,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1642,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1643,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1643,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1644,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1644,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1645,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1646,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1647,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1648,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1648,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1649,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1650,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1651,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1652,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1653,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1654,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1655,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1656,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1657,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1658,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1659,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1660,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1661,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1662,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1663,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1664,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1665,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1666,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1667,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1667,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1668,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1669,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1670,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1671,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1672,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1672,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1673,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1674,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1675,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1676,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1676,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1677,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1678,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1678,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1678,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1679,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1680,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1680,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1680,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1681,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1682,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1683,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1684,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1685,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1686,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1687,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1688,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1688,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1689,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1689,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1689,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1689,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1690,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1691,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1691,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1692,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1693,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1694,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1695,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1696,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1697,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1698,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1699,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1699,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1699,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1700,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1701,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1702,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1703,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1703,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1704,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1704,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1704,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1704,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1704,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1705,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1705,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1706,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1707,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1708,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1709,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1710,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1710,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1710,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1711,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1712,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1713,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1713,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1714,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1715,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1715,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1716,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1717,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1718,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1719,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1720,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1720,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1720,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1721,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1721,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1722,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1722,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1723,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1723,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1724,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1724,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1725,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1726,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1727,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1727,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1728,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1729,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1729,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1729,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1729,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1729,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1729,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1729,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1730,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1731,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1732,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1732,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1733,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1734,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1735,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1736,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1736,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1737,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1737,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1738,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1739,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1740,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1740,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1741,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1742,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1743,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1744,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1745,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1746,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1747,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1748,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1748,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1749,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1750,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1750,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1751,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1752,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1753,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1753,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1753,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1753,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1754,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1755,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1755,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1755,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1755,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1755,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1756,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1757,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1758,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1759,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1760,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1761,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1762,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1763,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1764,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1765,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1766,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1767,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1767,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1767,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1768,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1769,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1769,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1770,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1770,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1770,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1770,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1770,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1770,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1770,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1770,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1771,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1772,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1773,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1773,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1774,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1775,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1776,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1776,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1776,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1776,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1777,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1777,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1778,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1779,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1780,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1781,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1782,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1782,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1782,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1783,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1783,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1784,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1785,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1786,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1787,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1787,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1788,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1789,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1789,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1790,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1790,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1791,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1792,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1793,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1794,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1795,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1795,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1795,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1796,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1797,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1798,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1798,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1799,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1800,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1801,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1802,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1803,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1803,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1804,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1804,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1805,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1806,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1807,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1808,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1809,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1810,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1810,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1811,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1811,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1811,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1811,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1812,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1812,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1812,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1813,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1814,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1814,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1815,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1816,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1817,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1818,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1819,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1820,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1821,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1822,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1823,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1824,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1825,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1825,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1825,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1826,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1827,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1828,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1828,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1829,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1829,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1830,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1831,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1832,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1833,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1834,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1835,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1836,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1837,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1837,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1837,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1837,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1838,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1839,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1840,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1841,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1842,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1843,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1844,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1845,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1846,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1847,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1848,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1849,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1850,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1851,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1852,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1853,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1854,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1855,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1855,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1855,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1855,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1855,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1856,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1857,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1858,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1858,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1859,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1860,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1860,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1860,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1861,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1862,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1863,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1864,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1864,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1865,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1866,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1867,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1868,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1869,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1870,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1870,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1871,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1871,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1872,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1873,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1874,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1875,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1876,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1877,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1878,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1879,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1880,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1881,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1882,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1883,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1884,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1885,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1886,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1886,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1886,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1886,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1887,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1887,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1888,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1888,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1889,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1890,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1890,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1890,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1890,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1891,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1892,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1892,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1893,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1894,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1894,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1894,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1894,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1894,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1895,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1896,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1897,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1897,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1898,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1898,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1898,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1899,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1899,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1899,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1899,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1900,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1901,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1902,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1903,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1904,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1905,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1906,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1907,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1907,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1908,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1908,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1909,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1910,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1911,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1912,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1913,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1913,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1914,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1915,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1915,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1915,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1916,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1916,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1917,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1918,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1919,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1920,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1921,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1922,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1923,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1924,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1925,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1926,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1926,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1926,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1926,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1926,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1927,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1927,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1927,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1928,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1929,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1930,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1931,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1932,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1932,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1933,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1934,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1935,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1936,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1936,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1936,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1937,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1938,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1938,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1939,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1940,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1941,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1942,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1943,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1944,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1945,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1946,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1946,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1946,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1947,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1948,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1949,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1950,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1951,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1951,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1951,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1951,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1951,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1951,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1951,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1952,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1953,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1954,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1954,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1955,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1956,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1957,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=37909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1958,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1959,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1960,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1961,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1962,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1963,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1963,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1963,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1964,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1965,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=161292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1966,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1966,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1967,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1967,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1968,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1969,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1970,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1971,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1971,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1972,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1973,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1974,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1975,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1975,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1975,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1976,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1977,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1977,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1978,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1979,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=269631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1980,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1981,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1982,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1983,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1984,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1985,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1986,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1987,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1987,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1988,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1989,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1989,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1990,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1991,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1992,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1993,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1994,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1995,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1996,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1997,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1997,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1998,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1998,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1998,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(1999,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2000,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2001,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2002,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2003,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2004,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2005,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2006,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2006,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2007,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2008,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2009,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2010,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2010,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2011,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2011,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2012,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2013,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2014,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2015,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2016,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2017,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2018,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2019,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2019,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2019,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2020,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2021,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2021,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2021,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2022,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2023,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2024,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2025,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2026,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2026,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2027,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2028,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2029,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2029,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2030,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2031,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2032,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2032,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2033,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2034,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2034,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2035,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2035,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2036,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2037,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2038,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2039,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2040,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2040,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2040,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2041,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2042,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2043,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2043,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2044,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2044,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2045,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2046,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2046,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2047,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2048,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2049,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2049,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2050,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2051,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2052,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2052,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2053,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2054,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2054,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2055,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2056,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2057,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2058,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2059,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2059,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2059,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2060,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2061,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2062,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2063,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2064,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2064,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2065,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2066,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2067,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2068,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2068,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2068,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2069,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2070,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2071,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2072,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2072,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2073,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2074,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2075,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2076,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2077,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2078,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2079,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2080,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2081,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2081,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2081,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2082,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2082,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2083,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2084,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2084,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2085,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2085,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2086,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2087,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2088,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2088,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2088,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2088,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2088,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2088,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2089,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2090,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2090,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2091,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2092,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2093,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2094,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2095,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2096,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2097,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2098,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2098,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2098,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2098,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2099,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2100,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2101,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2101,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2102,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2103,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2104,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2105,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2105,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2105,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2105,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2106,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2107,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2108,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2109,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2110,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2111,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2111,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2112,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2113,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2114,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2114,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2115,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2115,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2115,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2115,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2116,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2117,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2118,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2119,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2120,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2121,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2122,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2122,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2123,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2124,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2125,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2125,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2126,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2127,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2128,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2128,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2128,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2128,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2128,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2129,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2129,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2130,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2131,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2132,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2133,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2134,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2135,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2135,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2136,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2137,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2138,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2139,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2140,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2141,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2142,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2143,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2143,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2143,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2144,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2145,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2145,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2146,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2147,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2148,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2149,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2150,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2151,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2151,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2152,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2152,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2153,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2153,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2154,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2154,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2155,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2156,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2157,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2158,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2159,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2160,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2161,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2162,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2163,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2163,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2164,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2165,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2166,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2166,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2166,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2167,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2168,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2169,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2170,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2171,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2171,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2171,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2172,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2173,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2174,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2175,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2176,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2177,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2178,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2179,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2180,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2181,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2182,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2183,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2183,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2184,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2185,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2185,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2186,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2186,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2186,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2186,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2187,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2188,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2189,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2189,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2190,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2191,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2192,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2193,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2193,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2194,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2194,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2195,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2196,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2197,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2198,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2199,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2200,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2201,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2202,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2203,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2204,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2205,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2206,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2206,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2207,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2208,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2208,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=94911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2208,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2208,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2208,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2209,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2210,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2210,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2210,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2211,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2211,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2212,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2213,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2214,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2215,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2216,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2217,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2218,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2219,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2220,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2221,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2222,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2223,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2224,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2224,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2224,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2225,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2226,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2226,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2227,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2228,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2228,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2228,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2228,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2229,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2230,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2231,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2232,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2233,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2234,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2234,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2235,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2236,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2237,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2238,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2239,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2240,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2241,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2242,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2243,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2244,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2245,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2246,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2247,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2248,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2249,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2250,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2251,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2251,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2252,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2253,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2254,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2254,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2254,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2255,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2256,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2257,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2258,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2258,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2258,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2259,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2260,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2261,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2262,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2262,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2262,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2262,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2262,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2262,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2263,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2263,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2264,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2265,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2266,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2266,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2266,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2267,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2268,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2269,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2269,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2269,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2269,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2269,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2269,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2270,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2271,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2271,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2272,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2273,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2273,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2274,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2275,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2276,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2277,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2278,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2279,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2280,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2281,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2282,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2283,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2283,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2284,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2285,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2285,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2285,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2285,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2285,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2285,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2286,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2287,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2288,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2288,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2288,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2289,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2290,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2291,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2292,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2293,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2293,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2294,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2295,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2296,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2297,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2298,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2299,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2300,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2301,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2302,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2303,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2304,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2304,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2304,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2304,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2304,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2304,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2304,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2304,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2305,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2305,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2306,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2307,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2308,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2309,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2310,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2311,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2312,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2313,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2314,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2314,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2314,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2315,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2316,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2317,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2318,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2319,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2320,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2321,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2321,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2321,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2322,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2322,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2323,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2324,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2325,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2326,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2327,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2328,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2329,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2329,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2330,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2330,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2331,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2332,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2333,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2333,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2333,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2334,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2335,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2336,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2337,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2337,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2337,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2337,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2337,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2338,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2339,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2340,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2341,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2341,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2341,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2341,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2342,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2343,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2344,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2344,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2344,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2345,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2346,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2347,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2348,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2349,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2350,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2351,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2351,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2352,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2353,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2353,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2354,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2354,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2355,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2356,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2357,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2357,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2358,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2359,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2359,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2360,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=37113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2361,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2362,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2362,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2363,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2364,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2365,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2366,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2367,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2368,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2369,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2370,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2370,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2371,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2372,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2373,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2374,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2375,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2376,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2377,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2378,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2379,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2379,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2380,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2381,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2382,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2383,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2384,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2385,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2385,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2385,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2386,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2387,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2388,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2389,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2389,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2390,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2391,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2391,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2391,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2392,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2393,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2394,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2395,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2395,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2396,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2396,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2397,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2398,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2399,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2400,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2401,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2402,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2403,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2404,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2404,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2405,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2406,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2407,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2408,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2409,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2410,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2411,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2411,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2412,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2413,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2414,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2415,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2416,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2417,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2418,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2419,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2420,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2421,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2422,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2423,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2424,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2424,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2424,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2425,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2426,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=144249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2426,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2427,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2427,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2427,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=38&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2427,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2427,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2427,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2427,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2427,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2428,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2429,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2430,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2431,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2432,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2433,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2433,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2434,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2435,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2435,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2436,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2436,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2437,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2437,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2437,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2437,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2438,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2439,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2440,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2440,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2441,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2442,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2443,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2444,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2444,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2444,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2445,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2446,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2447,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2448,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2449,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2450,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2451,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2452,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2453,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2453,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2454,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2455,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2456,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=99&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2457,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2458,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2459,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2460,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2461,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2462,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2462,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2463,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2464,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2465,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2465,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2466,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2467,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2467,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2468,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2469,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2470,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2470,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2470,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2471,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2471,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2471,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2472,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2473,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2474,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2475,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2476,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2477,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2478,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2479,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2480,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2481,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2481,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2481,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2482,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2483,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2483,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2483,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2484,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2485,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2486,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2486,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2486,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2486,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2487,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2487,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2488,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2489,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2490,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2491,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2492,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2493,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2494,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2494,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2495,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=137900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2496,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2497,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2498,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2498,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2498,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2498,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2498,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2498,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2499,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2500,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2501,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2501,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2502,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2502,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2503,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2504,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2505,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2506,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2507,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2508,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2509,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2509,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2510,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2511,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2512,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2513,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2514,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2515,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2516,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2516,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2517,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2518,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2519,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2520,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2521,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2522,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2523,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2524,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2524,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2525,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2526,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2526,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2527,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2528,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2529,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2530,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2530,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2530,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2531,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2532,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2533,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2534,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2535,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2536,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2536,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2536,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2536,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2536,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2537,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2537,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2537,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2537,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2538,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2539,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2540,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2541,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2542,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2543,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2543,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2544,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2545,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2546,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2547,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2548,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2549,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2549,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2550,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2551,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2552,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2552,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2552,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2552,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2553,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2554,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2555,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2555,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2555,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2556,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2557,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2557,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2558,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2559,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2560,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2560,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2560,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2560,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2560,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2560,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2560,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2561,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2561,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2562,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2563,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2563,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2563,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2563,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2564,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2565,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2566,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2567,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2568,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2569,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2570,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2571,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2572,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2573,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2574,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2575,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2575,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2576,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2577,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2577,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2578,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2579,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2580,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2580,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2581,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2582,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2583,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2584,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2585,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2586,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2587,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2588,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2588,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2588,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2588,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2588,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2588,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2588,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2589,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2590,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2591,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2592,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2593,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2594,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2595,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2595,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2595,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2596,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2597,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2598,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2599,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2599,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2599,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2599,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2599,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2599,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2600,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2601,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2601,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2602,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2603,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2603,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2604,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2604,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2604,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2605,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2606,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2607,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2608,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2608,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2609,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2610,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2611,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2612,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2612,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2612,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2613,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2613,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2614,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2615,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2616,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2616,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2617,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2618,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2619,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2620,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2620,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2620,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2621,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2622,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2623,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2623,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2624,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2625,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2626,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2627,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2628,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2629,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2630,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2631,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2632,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2632,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2633,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2633,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2634,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2635,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2636,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2637,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2638,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2639,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2640,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2640,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2641,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2642,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2643,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2644,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2645,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2646,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2647,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2648,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2648,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2649,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2649,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2649,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2650,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2650,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2650,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2651,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2652,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2653,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2654,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2655,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2656,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2657,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2658,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2659,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2660,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2661,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2661,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2661,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2661,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2661,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2661,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2662,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2663,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2663,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2663,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2663,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2664,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2665,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2665,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2666,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2666,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2667,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2668,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2669,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2670,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2670,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2670,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2670,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2670,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2671,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2672,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2673,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2674,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2675,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2675,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2676,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2677,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2678,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2679,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=98&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2679,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2679,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2679,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2679,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2680,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2681,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2682,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2683,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2684,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2685,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2686,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2687,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2687,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2687,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2688,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2688,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2689,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2689,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2690,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2690,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2690,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2690,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2691,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2692,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2693,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2694,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2694,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2694,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2695,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2696,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2697,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2698,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2699,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2699,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2699,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2700,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2700,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2701,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2702,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2703,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2704,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2704,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2705,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2706,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2707,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2708,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2709,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2710,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2710,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2711,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2712,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2713,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2714,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2715,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2716,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2717,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2718,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2718,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2719,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2720,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2721,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2722,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2723,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2723,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2724,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2725,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2726,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2727,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2728,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2728,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2728,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2728,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2729,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2729,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2729,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2730,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2731,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2732,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2733,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2734,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2734,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2734,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2734,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2735,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2736,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2737,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2737,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2737,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2737,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2738,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2739,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2740,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2741,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2742,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2743,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2743,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2744,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2745,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2746,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2747,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2747,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2747,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2748,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2748,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2749,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2750,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2751,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2752,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2753,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2754,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2755,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2755,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2756,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2757,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2758,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2759,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2760,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2761,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2761,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2762,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2763,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2764,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2764,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2765,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2765,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2766,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2767,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2768,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2769,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2770,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2770,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2771,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2772,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2773,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2774,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2775,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2776,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2777,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2778,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2779,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2780,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2781,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2782,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2782,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2783,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2784,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2785,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2785,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2786,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2786,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2787,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2788,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2789,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2790,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2791,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2792,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2793,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2794,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2795,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2796,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2797,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2798,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2799,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2800,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2801,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2802,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2803,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2803,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=68&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2803,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2803,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2803,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2803,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2803,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2803,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2804,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2805,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2806,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2807,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2807,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2807,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2808,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2808,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2809,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2810,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2811,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2812,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2813,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2814,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2815,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2815,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2815,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2816,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2817,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2818,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2819,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2820,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2821,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2821,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2821,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2822,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2823,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2823,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2823,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2824,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2825,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2826,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2826,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2827,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2827,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2828,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2829,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2830,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2831,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2832,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2833,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2834,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2834,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2835,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2836,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2837,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2838,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2839,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2840,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2841,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2842,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2843,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2844,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2845,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2846,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2846,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2846,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2846,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2846,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2847,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2848,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2848,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2849,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2850,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2851,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2852,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2853,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2853,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2854,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2854,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2854,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2854,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2855,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2856,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2857,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2857,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2857,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2858,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2859,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2860,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2861,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2862,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2863,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2863,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2863,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2863,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2864,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2865,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2866,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2866,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2867,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2868,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2869,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2869,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2870,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2871,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2872,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2873,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2874,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2874,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2874,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2874,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2874,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2875,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2876,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2877,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2878,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2879,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2880,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2880,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2880,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2880,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2880,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2880,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2881,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2882,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2883,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2884,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2885,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2886,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2887,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2888,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2889,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2890,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2891,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2892,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2893,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2894,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2894,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2895,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2896,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2897,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2898,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2899,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2900,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2901,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2902,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2903,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2903,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2903,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2904,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2905,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2906,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2907,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2907,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2908,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2909,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2910,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2911,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2911,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2911,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2911,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2911,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2911,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2911,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2912,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2913,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2914,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2914,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2915,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2916,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2917,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2918,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2919,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2919,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=91&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2919,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2919,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2919,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2919,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2920,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2920,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2920,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2921,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2921,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2922,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=279613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2923,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2924,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2925,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2926,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2927,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2928,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2929,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2929,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2929,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2929,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2930,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2930,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2930,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2930,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2931,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2932,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2933,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2934,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2935,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2936,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2937,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2937,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2938,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2939,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2940,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2941,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2942,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2943,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2943,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2943,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2944,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2945,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2946,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2946,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2946,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2946,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2946,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2946,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2946,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2947,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2948,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2949,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2950,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2951,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2952,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2952,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2952,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2953,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2953,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2954,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2955,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2956,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2957,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2957,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2958,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2958,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2959,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2959,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2960,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2961,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2962,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2963,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2964,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2965,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2966,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2966,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2966,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2967,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2967,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2967,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2968,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2969,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2969,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2969,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2970,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2971,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2972,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2973,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2974,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2975,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2976,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2976,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2977,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2978,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2979,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2979,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2980,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2981,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2982,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2983,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2984,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2985,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2986,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2986,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2986,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2986,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2986,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2987,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2988,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2988,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2988,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2988,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2989,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2990,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2991,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2992,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2993,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2993,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2993,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2993,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2994,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2995,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2996,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2997,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2998,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(2999,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3000,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3001,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3002,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3003,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3004,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3004,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3004,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3004,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3005,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3005,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3006,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3007,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3008,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3009,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3009,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3010,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3010,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3010,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3010,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3011,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3012,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3013,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3014,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3015,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3015,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3015,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3015,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3016,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3017,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3018,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3019,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3020,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3021,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3022,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3023,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3024,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3025,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3025,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3025,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3025,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3025,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3025,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3026,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3027,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3027,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3027,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3027,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3027,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3027,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3028,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3029,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3030,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3031,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3032,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3033,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3034,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3035,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3036,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3037,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3038,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3039,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3040,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3041,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3041,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3042,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3043,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3044,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3045,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3046,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3047,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3048,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3049,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3050,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3051,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3052,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3053,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3054,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3054,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3054,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3054,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3055,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3056,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3057,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3058,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3059,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3059,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3059,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3059,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3059,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3059,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3059,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3059,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3060,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=269637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3061,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3062,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3062,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3063,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3064,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3064,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3065,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3066,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3066,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3067,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3067,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3068,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3069,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3069,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3069,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3070,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3070,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3071,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3072,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3072,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3072,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3072,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3073,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3074,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3075,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3076,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3077,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3078,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3079,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3080,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3080,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3081,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3082,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3083,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3084,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3085,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3086,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3087,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3088,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3089,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3090,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3091,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3092,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3093,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3094,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3095,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3096,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3097,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3098,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3099,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3099,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3100,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3101,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3101,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3101,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3101,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3101,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3102,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3103,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3104,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3104,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3105,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3106,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3107,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3108,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3109,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3110,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3111,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3112,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3113,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3114,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3115,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3115,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3116,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3117,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3118,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3119,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3120,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3121,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3122,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3123,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3124,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3125,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3126,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3127,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3128,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3129,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3129,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3130,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3131,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3131,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3131,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3132,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3132,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3132,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=7168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3133,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3133,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3134,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3135,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3135,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3135,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3136,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3137,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3137,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3138,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3139,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3139,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3139,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3139,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3139,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3140,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3141,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3142,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3143,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3144,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3144,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3144,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3144,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3145,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3146,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3147,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3147,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3148,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3148,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3148,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3149,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3150,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3151,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3152,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3153,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3153,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3154,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3155,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3155,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3156,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3156,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3157,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3158,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3159,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3159,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3160,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3160,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3161,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3162,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3163,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3164,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3164,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3165,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3166,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3167,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3168,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3168,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3169,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3169,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3169,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3169,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3170,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3171,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3172,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3172,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3172,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3173,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3173,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3173,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3174,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3175,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3176,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3177,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3178,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3178,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3179,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3179,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3179,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3179,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3180,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=261321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3180,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3181,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3182,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3183,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3183,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3184,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3185,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3186,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3187,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3188,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3189,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3190,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3190,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3191,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3192,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3192,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3193,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3194,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3195,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3196,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3197,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3198,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3199,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3200,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3201,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3202,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3203,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3204,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3205,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3206,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3206,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3207,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3207,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3208,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3208,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3209,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=128941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3210,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3211,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3212,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3213,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3213,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3214,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3214,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3215,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3216,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3217,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3217,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3218,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3218,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3219,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3220,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3220,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3220,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3220,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3221,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3222,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3223,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3224,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3225,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3226,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3227,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3228,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3229,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3230,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3231,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3232,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3233,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3233,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3234,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3235,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3236,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3237,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3238,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3238,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3239,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3240,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3241,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3242,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3243,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3244,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3245,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3246,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3246,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3246,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3247,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3247,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3248,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3249,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3249,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3249,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3249,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3249,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3249,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3249,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3250,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3250,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3251,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3252,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3253,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3254,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3255,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3256,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3257,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3258,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3259,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3260,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3260,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3261,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3262,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3263,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3263,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3264,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3264,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3265,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3265,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3266,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3267,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3268,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3269,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3269,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3270,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3271,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3272,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3273,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3274,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3275,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3276,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3277,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3278,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3279,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3279,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3279,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3279,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3280,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3281,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3282,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3283,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3284,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3285,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3286,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3286,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3287,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3288,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3289,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3290,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3290,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3290,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3291,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3292,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3293,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3294,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3295,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3296,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3297,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3297,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3298,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3299,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3300,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3301,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3301,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3302,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3303,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3304,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3305,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3306,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3307,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3308,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3309,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3310,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3311,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3311,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3312,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3312,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=265166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3313,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3314,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3314,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3315,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3316,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3317,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3318,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3319,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3320,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3320,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3320,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3320,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3321,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3322,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3323,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3324,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3324,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3325,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3326,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3327,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3328,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3329,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3330,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3331,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3332,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3333,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3334,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3335,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3336,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3337,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3338,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3339,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3340,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3341,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3341,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3342,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3342,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3343,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3344,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3344,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=269636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3345,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3346,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3346,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3346,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3347,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3347,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3347,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3348,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3349,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3350,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3351,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3352,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3352,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3352,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3352,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3352,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3353,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3354,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3354,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3354,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3354,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3355,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3356,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3357,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3358,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3359,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3360,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=128944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3361,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3361,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3361,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3362,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3363,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3364,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3365,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3366,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3367,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3368,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3369,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3370,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3371,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3371,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3371,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3372,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3372,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3373,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3374,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3375,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3376,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3377,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3378,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3379,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3379,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3380,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3380,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3381,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3382,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3383,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3384,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3385,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3386,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3387,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=137931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3388,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3389,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3390,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3391,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3392,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3392,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3392,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3392,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3393,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3394,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3395,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3396,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3396,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3396,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3396,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3396,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3397,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3398,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3399,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3400,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3401,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3401,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3402,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3403,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3403,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3403,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3403,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3403,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3404,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3405,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3405,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3406,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3407,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3408,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3409,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3410,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3410,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3411,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3411,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3412,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3412,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3413,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3413,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3413,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3413,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3414,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3415,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3416,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3417,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3417,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3418,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3418,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3418,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3418,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3418,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3418,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3419,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3420,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3421,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3422,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3423,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3423,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3424,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3425,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3426,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3427,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3427,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3428,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3429,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3430,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3431,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3432,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3433,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3434,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3435,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3436,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3436,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3436,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3437,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3438,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3439,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3440,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3441,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3442,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3443,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3444,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3445,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3446,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3447,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3448,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3449,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3449,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3450,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3451,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3451,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3452,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3453,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3454,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3455,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3456,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3457,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3458,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3459,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3460,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3461,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3462,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3463,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3464,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3465,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3465,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3466,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3467,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3468,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3468,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3469,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3470,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3471,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3472,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3473,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3474,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3475,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3475,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3475,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3476,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3477,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3478,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3479,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3479,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3480,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3481,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3482,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3483,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3483,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3484,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3485,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3486,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3486,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3487,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3487,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3488,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3489,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3490,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3490,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3490,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3491,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3492,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3493,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3493,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3494,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3495,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3496,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3497,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3498,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3499,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3500,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3501,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3502,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3502,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3503,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3504,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3505,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3506,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3506,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3506,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3506,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3507,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3508,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3509,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3509,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3510,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3511,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3512,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3512,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3513,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3514,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3514,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3515,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3516,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3517,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3517,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3518,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3519,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3520,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3521,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3521,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3522,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3523,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3523,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3524,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3525,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3526,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3527,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3528,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3529,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3530,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3531,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3532,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3533,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3534,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3534,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3534,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3535,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3536,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3536,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3536,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3536,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3536,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3536,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3536,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3536,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3537,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3538,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3539,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3540,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3541,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3542,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3543,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3543,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3544,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3544,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3544,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3545,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3546,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3547,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3548,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3549,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3550,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3551,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3552,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3552,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3553,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3554,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3555,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3555,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3556,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3557,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3558,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3559,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3560,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3561,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3562,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3563,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3564,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3564,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3565,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3565,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3566,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3566,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3567,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3567,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3568,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3568,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3569,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3570,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3571,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3572,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3573,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3573,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3573,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3574,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3575,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3576,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3577,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3578,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3579,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3579,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3580,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3581,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3582,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3582,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3583,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3584,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3584,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3585,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3586,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3587,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3588,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3589,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3590,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3591,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3591,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3591,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3591,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3591,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3592,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3593,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3594,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3595,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3595,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3596,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3597,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3598,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3598,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3598,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3599,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3600,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3600,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3601,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3602,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3602,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3602,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3603,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3603,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3604,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3605,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3605,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3606,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3607,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3607,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3607,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3608,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3608,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3609,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3610,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3611,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3611,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3611,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3611,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3611,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3612,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3613,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3614,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3614,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3615,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3615,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3616,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3617,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3618,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3619,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3620,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3621,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3622,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3623,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3624,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3624,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3625,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3626,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3627,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3628,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3628,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3629,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3630,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3631,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3631,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3632,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3632,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3633,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3633,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3634,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3635,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3636,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3637,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3637,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3637,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3638,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3638,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3638,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3639,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3640,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3640,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3641,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3642,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3643,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3643,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3644,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3644,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=71&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3644,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3644,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3644,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3644,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3645,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3646,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3647,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3648,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3648,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3649,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3650,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3651,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3652,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3653,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3653,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3653,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3653,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3653,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3653,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3653,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3654,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3655,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3656,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3656,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3656,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3656,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3657,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3658,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3659,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3660,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3661,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3662,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3663,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3664,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3665,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3665,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3665,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3666,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3667,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3668,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3668,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3668,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3668,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3668,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3669,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3669,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3669,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3670,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3670,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3671,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3672,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3672,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3672,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3673,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3674,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3674,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3675,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3676,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3677,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3678,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3679,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3679,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3680,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3681,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3681,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3681,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3681,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3682,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3683,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3684,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3685,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3686,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3687,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3688,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3689,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3690,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3691,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3692,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3693,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3693,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3693,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3693,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3694,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3695,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3696,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3697,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3698,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3699,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3700,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3701,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3702,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3702,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3703,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3704,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3705,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3706,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3707,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3708,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3708,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3708,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3709,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3710,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3711,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3712,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3713,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3714,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3715,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3715,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3716,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3717,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3718,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3719,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3719,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3719,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3720,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3721,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3722,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3723,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3724,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3724,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3724,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3725,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3726,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3727,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3727,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3728,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3729,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3730,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3731,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3732,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3733,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3734,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3735,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3736,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3737,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3738,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3738,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3739,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3740,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3741,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3742,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3743,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3743,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3743,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3743,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3744,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3744,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3745,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3745,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3746,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3746,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3747,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3748,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3749,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3749,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3749,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3750,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3751,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3752,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3753,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3754,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3755,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3756,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3757,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3758,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3759,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3760,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3761,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3762,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3763,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3764,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3765,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3766,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3767,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3768,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3769,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3770,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3770,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3771,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3772,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3773,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3774,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3775,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3776,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3777,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3778,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3779,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3780,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3781,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3782,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3782,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=266154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3783,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3784,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3785,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3786,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3787,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3788,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=246952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3789,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3790,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3791,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3792,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3793,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3794,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3795,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3796,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3796,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3796,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3797,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3798,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3799,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3800,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3800,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3801,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3802,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3803,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3803,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3804,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3805,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3806,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3806,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3807,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3808,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3808,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3808,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3808,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3808,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3809,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3810,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3811,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3812,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=92&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3812,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3812,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3812,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3813,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3813,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3813,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3813,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3813,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3813,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3814,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3814,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3814,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3815,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3816,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3817,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3817,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3818,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3818,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3818,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3819,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3820,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3820,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3820,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3821,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3822,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3823,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3824,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3825,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3826,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3827,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3828,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3829,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3830,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3831,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3832,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3833,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3834,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3835,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3836,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3837,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3837,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3838,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3839,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3840,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3840,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3841,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3842,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3843,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3844,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3845,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3846,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3846,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3847,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3848,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3849,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3850,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3851,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3852,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3853,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3854,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3855,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3856,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3857,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3858,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3859,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3860,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3861,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3862,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3862,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3863,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3864,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3865,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3866,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3867,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3868,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3868,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3869,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3870,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3871,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3872,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3872,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3872,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3873,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3873,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3873,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3874,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3875,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3876,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3877,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3878,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3879,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3880,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3881,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3881,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3881,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3881,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3881,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3882,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3882,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3882,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3882,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3883,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3884,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3885,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3885,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3886,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3887,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3888,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3889,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3890,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3891,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3891,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3892,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3893,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3894,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3895,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3896,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3897,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3897,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3897,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3897,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3898,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3899,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3899,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3899,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3899,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3899,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3899,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3899,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3899,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3900,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3901,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3902,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3903,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3903,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3904,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3905,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3906,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3906,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3907,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3908,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3909,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3909,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3910,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3911,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3912,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3913,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3914,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3915,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3916,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3917,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3918,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3919,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3920,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3921,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3922,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3923,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3924,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3925,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3926,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3927,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3927,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3927,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3928,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3929,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3930,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3931,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3932,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3933,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3934,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3935,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3936,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3937,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3937,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3937,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3938,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3938,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3939,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3939,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3939,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3940,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3941,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3942,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3943,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3944,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3944,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3944,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3944,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3945,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3945,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3945,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3946,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3947,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3948,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3949,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3950,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3951,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3952,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3952,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3953,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3953,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3954,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3955,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3956,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3957,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3958,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3959,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3959,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3959,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3959,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3959,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3959,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3959,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3960,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3961,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3962,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3963,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3964,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3965,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3966,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3967,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3967,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3968,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3969,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3969,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3969,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3970,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3971,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3972,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3972,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3972,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3973,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3974,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3975,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3975,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3975,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3975,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3975,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3976,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3977,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3978,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3979,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3980,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3980,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3980,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3980,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3980,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3980,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3980,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3981,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3981,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3981,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3982,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3983,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3984,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3984,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3984,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3984,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3984,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3984,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3985,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3986,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3986,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3986,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3986,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3986,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3986,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3987,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3987,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3988,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3989,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3990,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3990,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3990,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3990,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3990,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3990,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3990,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3990,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3991,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3992,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3993,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3994,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3995,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3996,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3997,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3998,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(3999,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4000,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4001,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4002,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4003,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4004,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4005,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4006,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4006,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4007,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4007,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4008,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4009,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4010,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4011,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4012,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4013,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4014,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4015,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4016,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4017,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4017,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4018,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4019,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4020,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4021,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4022,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4023,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4024,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4025,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4026,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4027,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4028,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4028,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4028,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4028,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4029,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4030,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4030,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4031,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4032,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4033,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4034,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4035,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4036,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4037,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4038,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4039,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4039,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4040,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4041,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4042,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4043,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4043,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4044,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4045,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4045,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4045,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4046,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4046,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4047,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4047,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4047,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4047,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4047,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4048,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4049,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4050,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4051,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4052,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4052,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4053,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4054,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4054,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4055,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4056,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4057,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4058,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4058,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4058,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4058,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4059,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4059,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4060,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4061,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4062,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4063,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4064,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4065,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4065,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4066,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4067,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4067,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4068,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4069,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4069,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4069,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4069,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=61&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4069,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4069,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4069,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4069,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4070,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4071,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4072,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4073,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4074,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4075,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4076,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4077,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4078,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4079,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4080,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4080,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4080,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4080,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4081,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4082,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4083,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4084,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4085,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4086,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4087,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4088,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4089,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4089,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4090,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4090,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4091,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4092,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4093,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4094,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4094,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4094,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4094,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4094,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4094,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4095,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4095,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4096,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4097,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4098,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4099,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4099,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4100,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4101,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4102,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4103,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4104,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4105,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4105,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4106,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4106,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4107,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4108,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4109,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4110,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4111,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4112,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4113,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4114,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4115,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4116,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4117,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4118,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4119,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4120,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4121,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4122,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4122,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4123,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4123,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4123,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4124,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4125,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4126,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4127,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4128,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4129,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4129,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4130,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4131,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4132,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4133,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4133,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4133,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4133,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4133,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4134,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4135,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4136,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4137,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4137,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4137,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4137,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4138,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4138,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4139,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4140,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4141,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4142,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4143,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=279607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4144,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4145,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4145,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4145,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4145,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4146,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4146,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4147,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4148,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4149,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4149,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4149,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4149,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4149,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4149,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4149,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4150,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4151,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4152,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4153,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4154,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4155,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4155,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4156,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4156,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4156,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4156,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4157,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4158,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4159,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4160,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4161,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4162,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4163,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4164,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4164,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4165,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4165,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4166,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4166,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4167,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4168,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4169,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4170,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4171,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4172,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4173,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4173,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4174,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4175,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4176,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4177,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4178,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4179,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4180,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4181,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4182,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4183,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4183,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4183,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4183,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4184,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4185,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4186,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4187,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4187,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4188,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4188,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4189,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4189,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4190,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4190,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4191,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4192,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4193,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4194,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4195,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4196,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4196,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4197,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4198,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4199,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4200,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4200,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4200,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4201,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4202,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4203,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4204,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4205,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4205,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4205,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4206,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4207,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4208,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4209,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4210,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4211,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4212,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4212,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4213,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4214,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4214,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4215,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4215,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4216,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4217,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4218,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4219,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4219,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4220,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4221,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4222,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4222,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4223,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4223,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4224,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4225,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4226,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4226,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4227,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4228,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4229,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4229,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4229,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4230,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4231,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4232,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4233,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4234,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4235,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4236,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4237,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4238,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4239,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4240,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4241,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4242,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4243,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4243,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4244,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4245,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4246,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4247,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4247,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4247,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4248,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4249,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4250,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4250,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4250,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4250,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4250,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4251,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4252,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4253,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4254,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4255,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4256,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4257,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4258,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4259,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4259,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4260,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4261,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4261,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4262,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4263,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4264,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4264,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4265,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4266,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4267,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4267,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4267,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4267,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4268,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4269,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4269,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4270,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4270,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4271,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4271,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4272,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4273,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4274,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4275,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4276,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4277,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4277,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4277,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4277,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4278,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4279,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4280,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4280,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4281,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4282,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4283,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4283,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4284,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4284,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4285,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4286,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4287,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4288,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4289,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4290,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4291,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4291,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4291,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4291,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4292,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4293,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4294,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4295,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4295,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4295,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4295,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4295,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4296,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4297,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4298,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4299,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4300,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4301,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4302,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4303,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4304,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4304,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4305,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4306,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4307,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4307,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4307,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4307,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4307,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4308,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4309,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4310,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4311,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4312,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4312,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4313,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4314,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4315,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4315,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4316,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4317,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4318,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4319,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4320,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4321,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4322,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4323,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4324,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4324,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4325,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4326,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4327,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4328,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4329,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4330,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4330,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4331,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4331,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4332,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4333,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4334,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4335,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4336,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4337,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4338,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4339,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4340,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4341,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4341,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4341,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4341,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4341,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4342,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4343,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4344,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4344,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4344,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4345,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4346,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4347,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4348,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4349,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4350,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4350,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4351,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4351,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4352,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4353,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4354,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4355,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4355,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4355,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4355,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4356,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4357,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4358,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4358,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4358,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4358,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4358,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4358,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4358,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4358,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4359,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4360,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4361,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4361,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4361,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4361,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4361,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4361,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4361,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4362,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4362,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4363,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4363,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4363,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4363,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4364,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4364,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4365,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4366,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4367,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4368,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4369,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4370,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4371,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4371,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4372,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4372,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4373,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4373,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4373,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4373,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4373,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4374,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4375,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4376,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4377,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4377,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4378,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4379,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4380,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4381,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4381,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4382,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=18&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4383,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4384,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4385,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4386,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4386,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4387,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4388,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4388,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4389,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4390,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4391,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4392,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4393,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4394,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4395,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4396,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4397,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4398,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4399,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4400,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4401,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4401,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4401,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4402,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4403,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4403,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4404,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4405,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4406,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4407,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4408,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4409,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4410,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4410,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4410,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4411,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4412,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4413,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4413,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4414,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4414,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4415,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4416,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4417,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4418,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4419,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4420,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4421,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4422,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4423,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4424,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4425,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4426,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4426,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4426,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4427,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4428,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4429,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4429,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4430,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4431,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4432,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4433,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4434,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4435,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4436,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4437,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4437,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4438,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4439,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4440,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4441,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4442,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4443,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4443,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4444,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4444,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4445,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4446,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4446,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4446,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4446,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4447,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4447,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4448,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4449,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4450,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4451,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4452,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4452,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4453,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4453,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4454,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4454,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4455,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4456,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4457,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4458,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4459,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4460,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4461,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4462,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4463,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4463,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4464,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4464,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4465,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4466,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4466,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4467,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4468,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4469,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4470,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4470,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4471,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4472,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4473,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4473,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4474,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4475,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4476,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4477,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4477,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4477,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4477,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4478,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4479,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4480,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4481,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4482,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=131008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4483,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4483,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4484,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4485,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4486,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4486,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4486,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4487,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4488,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4489,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4490,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4491,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4492,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4493,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4494,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4495,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4496,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4497,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4497,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4498,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4499,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4500,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4501,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4502,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4503,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4504,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4504,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4504,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4505,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4506,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4507,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4508,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4509,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4510,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4510,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4511,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4512,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4512,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4513,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4513,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4513,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4513,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4513,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4514,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4515,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4516,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4516,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4516,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4516,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4517,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4518,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4519,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4520,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4521,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4522,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4523,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4524,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4525,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4526,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4527,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4528,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4529,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4530,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4531,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4531,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4532,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4533,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4533,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4533,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4533,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4533,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4534,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4535,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4536,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4537,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4538,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4539,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4540,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4541,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4542,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4542,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4542,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4543,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4543,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4544,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4545,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4545,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4546,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4547,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4547,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4547,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4547,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4547,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4547,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4548,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4549,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4549,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4550,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4550,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4551,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4552,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4553,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4554,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4555,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4556,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4557,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4558,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4559,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4560,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4561,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4561,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4562,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4563,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4564,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4565,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4566,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4567,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4567,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4568,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4568,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4569,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4570,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4571,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4571,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4572,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4573,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4574,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4575,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4575,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4576,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4577,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4577,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4578,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4579,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4580,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4580,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4580,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4581,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4582,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4583,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4584,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4585,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4586,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4587,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4588,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4588,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4588,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4589,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4589,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4590,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4591,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4591,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4592,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4593,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4594,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4595,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4596,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4597,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4598,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4599,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4600,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4601,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4602,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4602,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4602,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4603,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4604,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4604,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4604,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4605,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4605,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4606,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4607,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4608,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4609,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=144282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4610,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4611,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4612,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4612,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4613,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4614,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4615,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4616,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4617,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4618,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4619,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4619,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4619,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4619,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4619,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4620,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4621,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4622,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4623,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4623,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4624,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4624,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4624,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4625,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4625,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4625,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4626,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4627,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4628,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4629,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4630,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4630,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4631,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4632,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4632,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4633,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4634,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4635,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4636,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4636,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4637,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4638,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4639,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4639,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4639,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4639,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4639,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4639,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4639,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4640,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4641,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4641,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4642,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4643,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4644,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4645,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4645,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4646,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4647,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4648,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4649,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4649,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4650,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4651,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4652,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4653,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4654,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4654,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4655,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4656,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4657,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4658,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4659,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4660,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4660,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4661,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4661,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4661,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4661,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4662,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4663,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=94&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4664,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4665,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4666,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4666,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4667,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4667,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4668,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4669,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4669,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4670,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4670,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4670,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4671,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4671,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4671,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4672,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4673,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4673,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4674,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4674,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4675,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4676,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4676,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4676,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4676,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4677,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4678,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4678,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4679,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4680,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4680,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4680,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4681,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4681,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4681,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4682,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4682,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4683,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4684,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4685,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4686,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4687,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4688,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4689,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4690,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4691,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4692,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4692,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4692,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4693,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4693,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4693,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4694,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4695,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4696,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4696,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4696,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4697,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4698,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4699,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4700,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4701,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4702,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4703,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4704,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4705,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4706,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4707,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4708,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4709,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4710,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4711,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4711,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4712,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4713,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4714,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4715,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4716,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4717,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4718,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4719,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4720,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4721,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4722,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4723,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4723,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4723,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4723,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4723,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4723,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4724,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=254058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4725,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4726,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4727,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4728,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4729,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4730,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4731,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4732,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4733,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4734,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4735,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4735,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4735,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4736,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4737,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4738,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4739,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4740,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4741,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4742,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4743,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4744,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4745,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4746,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4747,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4748,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4749,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4749,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4749,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4749,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4750,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4750,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4751,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4751,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4752,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4752,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4753,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4754,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4755,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4756,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4757,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4758,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4759,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4760,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4760,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4761,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4761,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4761,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4761,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4762,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4763,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4763,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4764,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4765,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4766,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4767,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4768,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4768,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4769,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4770,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4770,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4771,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4772,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4773,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4774,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4775,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4776,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4777,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4778,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4778,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4778,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4778,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4779,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4780,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4781,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4782,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4783,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4784,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4784,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4785,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4786,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4787,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4788,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4788,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4789,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4789,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4790,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4791,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4791,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4792,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4792,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4792,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4792,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4793,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4794,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4794,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4795,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4796,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4796,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4797,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4797,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4798,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4799,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4800,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4801,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4802,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4803,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4803,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4804,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4805,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4805,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4805,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4805,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4806,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4807,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4808,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4809,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4810,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4811,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4812,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4812,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4812,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4812,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4812,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4812,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4812,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4813,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4813,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4814,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4815,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4816,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4817,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4818,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4819,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4820,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4821,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4822,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4823,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4824,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4825,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4826,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4827,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4828,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4829,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4830,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4831,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4832,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4833,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4834,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4835,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4836,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4836,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4837,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4838,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4839,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4840,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4840,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4840,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4841,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4842,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4843,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4843,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4843,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4843,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4843,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4843,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4843,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4844,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4845,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4845,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4846,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4847,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4848,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4849,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4849,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4850,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4851,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4852,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4852,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4853,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4854,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4855,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4855,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4855,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4855,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4856,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4857,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4857,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4858,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4859,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4860,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4861,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4862,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4862,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4862,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4862,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4862,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4862,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4863,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4864,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4865,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4866,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4867,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4868,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4869,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4870,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4871,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4871,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4872,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4873,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4874,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4875,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4875,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4875,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4876,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4877,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4878,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4879,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4880,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4880,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4880,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4880,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4881,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4881,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4881,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4881,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4882,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4883,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4884,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4885,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4885,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4886,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4887,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4888,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4888,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4889,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4890,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4891,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4892,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4893,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4894,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4895,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4896,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4897,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4898,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4899,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4900,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4900,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4901,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4902,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4903,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4904,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4904,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4905,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4906,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4907,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4907,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4908,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4908,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4909,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4910,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4911,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4912,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4913,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4914,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4915,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4916,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4917,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4918,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4918,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4918,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4918,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4918,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4918,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4918,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4918,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4919,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4920,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4921,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4921,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4921,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4921,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4921,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4921,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4921,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4922,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4923,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4923,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4923,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4923,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4924,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4925,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4926,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4927,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4928,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4928,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4929,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4929,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4930,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4931,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4932,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4932,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4933,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4933,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4934,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4935,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4936,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4937,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4938,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4938,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4939,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4940,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4941,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4942,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4943,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4944,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4945,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4945,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4945,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=17&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4945,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4945,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4945,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4946,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4947,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4948,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4949,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4950,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4951,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4952,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4953,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4954,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4955,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4956,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4957,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4958,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4959,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4960,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4961,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4962,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4963,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4964,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4965,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4966,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4966,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4966,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4966,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4966,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4966,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4967,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4968,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4969,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4970,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4971,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4972,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4972,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4973,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4973,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4974,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4975,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4976,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4976,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4976,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4977,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4978,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4978,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4979,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4980,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4981,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4982,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4983,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4984,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4984,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4985,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4986,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4987,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4988,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4989,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4990,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4991,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4991,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4992,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4993,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4993,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4994,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4995,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4996,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4997,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4997,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4998,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(4999,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5000,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5001,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5002,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5003,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5004,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5005,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5005,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5006,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5006,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5007,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5008,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5009,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5009,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5009,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5009,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5009,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5009,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5010,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5011,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5011,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5012,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5013,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5013,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5014,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5015,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5016,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5017,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5018,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5019,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5020,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5021,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5022,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5022,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5022,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5022,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=7&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5022,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5022,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5022,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5022,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5023,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5023,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5023,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5024,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5024,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5025,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5026,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5027,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5028,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5029,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5030,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5031,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5032,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5033,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5034,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5035,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5036,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5036,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5037,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5038,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5038,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5039,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5040,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5041,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5042,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5043,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5044,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5045,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5046,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5047,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5048,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5049,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5050,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5051,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5052,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5053,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5054,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5055,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5056,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5056,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5057,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5057,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5057,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5057,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5057,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5057,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5058,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5058,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5059,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5060,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5060,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5061,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5062,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5063,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5064,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5065,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5066,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5067,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5067,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5067,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5067,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5067,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5067,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5068,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5068,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5069,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5070,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5071,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5071,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5072,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5072,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5072,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5072,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5072,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5073,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5074,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5075,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5076,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5077,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5078,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5078,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5079,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5080,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5081,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5082,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5082,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5083,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5084,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5085,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5085,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5085,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5085,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5085,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5086,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5087,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5088,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5089,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5089,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5090,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5090,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5091,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5091,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5092,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5092,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5092,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5093,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5094,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5095,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5096,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5097,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5098,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5099,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5100,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5101,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5102,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5103,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5104,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5105,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5106,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5107,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5108,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5109,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5110,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5111,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5112,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5112,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5112,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5112,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5112,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5112,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5112,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5113,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5113,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5114,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5115,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5115,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5116,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5117,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5117,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5117,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5117,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5118,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5119,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5120,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=131004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5121,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5121,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5122,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5123,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5123,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5123,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5123,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5123,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5123,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5124,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5124,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5125,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5126,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5127,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5128,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5128,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5129,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5130,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5130,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5131,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5132,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5133,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5134,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5134,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5134,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5134,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5135,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5136,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5136,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5137,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5138,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5139,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5140,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5140,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5140,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5140,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5140,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5140,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5140,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5140,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5141,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5141,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5142,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5142,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5142,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5143,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5144,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5145,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5146,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5147,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5148,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5149,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5150,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5151,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5152,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5153,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5154,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5155,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5156,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5156,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5156,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5157,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5157,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5157,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5157,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5157,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5158,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5159,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5160,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5161,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5161,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5162,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5163,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5164,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5165,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5166,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5167,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5168,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5169,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5170,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5171,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5172,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5173,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5174,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5175,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5176,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5176,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5177,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5177,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5177,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5177,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5178,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5179,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5180,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5181,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5182,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5182,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5182,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5182,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5182,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5183,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5184,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5185,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5185,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5185,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5186,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5186,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5186,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5186,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5187,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5188,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5188,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5189,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5190,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5191,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5192,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5192,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5192,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5193,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5194,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5195,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5196,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5197,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5198,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5199,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5200,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5201,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5202,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5203,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5203,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5203,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5204,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5205,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5206,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5206,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5206,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5207,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5208,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5209,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5209,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5210,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5210,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5210,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5210,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5210,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5211,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5212,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5213,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5214,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5215,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5216,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5217,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5218,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5219,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5220,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5221,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5222,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5223,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5224,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5224,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5224,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5225,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5226,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5226,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5227,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5227,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5227,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5228,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5229,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5230,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5231,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5232,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5233,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5234,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5235,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5236,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5236,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5236,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5236,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5237,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5238,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5238,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5238,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5239,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5240,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5240,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5241,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5242,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5243,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5244,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5245,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5246,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5246,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5247,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5247,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5248,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5249,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5250,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5251,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5251,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5251,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5251,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5251,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5252,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5252,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5252,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5253,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5254,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5255,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5255,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5256,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5257,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5258,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5258,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5259,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5260,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5261,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5262,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5263,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5264,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5264,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5265,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5265,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5266,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5266,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5266,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5266,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5266,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5267,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5268,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5269,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5269,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5270,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5271,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5272,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5273,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5273,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5274,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5274,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5274,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5275,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5276,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5277,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5278,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5279,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5279,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5279,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5280,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5280,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5281,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=144276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5281,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5282,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5283,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5284,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5285,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5286,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5286,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5287,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5288,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5289,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5290,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5291,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5292,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5293,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5294,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5294,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5294,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5295,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5296,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5297,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5298,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5299,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5300,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5301,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5302,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5303,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5304,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5305,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5306,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5307,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5307,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5307,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5308,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5309,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5310,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5311,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5312,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5312,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5313,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5313,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5314,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5314,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5315,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5316,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5317,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5318,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5319,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5320,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5320,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5321,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5322,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5322,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5322,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5322,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5323,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5324,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5325,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5325,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5326,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5326,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5327,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5327,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5327,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=53&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5327,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5327,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5327,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5328,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5329,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5329,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5329,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5329,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5329,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5330,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5330,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5330,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5331,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5331,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5332,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5333,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5334,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5335,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5336,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5337,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5338,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5339,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5340,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5341,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5341,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5342,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5343,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5344,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5345,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5346,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5346,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5346,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5347,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5348,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5348,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5349,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5350,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5351,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5351,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5351,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5352,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5353,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5354,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5355,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5356,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5357,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5358,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5359,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5359,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5360,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5361,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5362,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5363,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5364,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5365,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5365,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5365,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5366,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5367,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5368,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5369,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5369,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5370,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5370,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5371,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5372,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5372,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5373,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5374,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5375,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5375,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5376,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5376,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5377,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5378,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5378,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5378,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5378,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5379,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5380,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5381,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5381,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5382,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5383,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5384,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5384,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5385,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5386,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5386,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5387,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5388,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5389,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5390,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5390,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5391,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5392,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5393,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5394,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5395,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5396,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5397,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5398,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5399,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5400,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5401,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5402,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5402,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5403,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5404,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5405,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5406,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5407,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5407,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5407,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5408,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5409,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5410,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5411,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5412,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5412,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5412,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5413,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5414,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5415,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5416,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5417,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5418,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5418,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5419,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5420,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5420,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5420,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5420,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5421,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5421,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5422,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5423,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5424,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5424,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5425,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5426,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5426,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5427,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5428,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5429,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5429,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5430,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5431,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5432,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5433,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5433,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5434,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5435,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5436,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5437,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5437,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5438,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5439,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5439,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5440,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5441,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5442,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5443,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5444,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5445,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5446,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5446,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5447,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5448,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5449,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5450,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5451,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5452,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5453,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5454,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5455,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5456,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5457,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5457,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5458,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5458,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5459,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5460,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5461,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5462,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5462,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5463,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5464,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5465,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5466,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5466,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5467,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5468,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5469,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5470,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5471,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5471,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5472,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5472,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5473,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5474,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5474,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5474,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5475,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5476,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5477,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5478,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5478,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5479,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5479,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5480,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5481,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5482,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5483,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5484,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5484,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5485,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5486,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5487,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5488,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5488,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5488,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5489,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5490,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5491,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5492,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5493,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5493,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5494,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5494,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5495,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5496,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5497,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5498,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5499,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5499,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5499,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5499,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5500,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5501,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5502,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5502,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5503,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5504,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5505,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5506,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5507,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5507,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5507,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5507,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5508,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5508,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5508,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5508,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5508,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5508,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5509,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5510,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5510,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5511,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5512,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5513,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5513,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5514,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5515,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5516,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5517,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5517,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5518,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5518,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5519,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=269639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5520,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5521,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5522,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5523,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5524,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5525,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5526,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5526,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5527,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5528,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5528,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5529,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5529,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5530,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5531,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5531,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5531,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5531,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=70&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5531,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5531,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5531,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5531,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5532,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5532,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5532,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5533,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5534,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5534,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5535,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5536,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5537,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5538,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5539,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5540,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5541,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5542,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5543,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5543,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5544,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5545,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5546,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5547,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5547,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5548,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5549,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5550,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5551,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5551,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5552,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5553,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5553,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5553,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5553,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5554,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5554,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5555,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5556,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5557,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5557,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5557,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5558,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5559,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5559,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5559,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5559,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5559,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5559,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5559,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5560,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5561,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5562,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5563,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5563,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5564,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5565,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5566,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5567,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5568,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5569,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5570,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5571,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5572,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5572,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5573,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5574,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5575,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5576,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5577,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5577,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5577,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5578,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5579,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5580,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5581,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5582,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5582,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5583,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5583,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5583,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5583,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5584,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5585,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5586,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5587,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5588,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5588,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5589,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5590,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5591,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5592,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5593,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5594,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5595,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5596,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5597,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5598,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5599,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5599,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5600,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5601,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5602,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5602,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5603,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5604,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5605,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5605,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5605,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5606,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5606,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5607,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5608,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5609,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5610,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5610,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5611,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5612,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5613,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5614,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5615,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5616,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5617,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5618,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5618,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5619,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5620,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5620,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5620,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5621,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5622,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5623,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5623,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5624,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5625,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5626,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5627,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5628,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5629,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5630,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5631,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5632,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5632,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5633,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5634,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5635,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5636,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5637,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5638,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5639,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5640,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5641,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5642,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5643,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5644,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5644,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5645,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5646,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5646,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5646,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5647,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5647,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5648,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5648,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5649,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5650,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5651,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5652,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5653,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5654,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5655,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5656,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5657,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5658,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5659,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5660,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5660,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5661,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5662,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5663,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5664,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5665,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5665,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5666,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5667,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5668,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5669,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5670,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5670,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5671,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5672,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5672,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5673,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5674,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5675,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5676,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5677,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5678,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5678,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5678,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5678,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5678,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5678,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5678,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5678,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5679,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5680,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5681,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5682,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5683,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5684,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5685,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5686,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5687,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5688,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5689,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5690,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5691,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5691,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5692,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5692,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5693,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5693,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5694,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=69&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5694,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5694,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5694,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5695,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=128940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5696,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5697,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5698,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5698,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5699,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5699,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5699,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5700,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5701,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5701,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5702,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5703,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5704,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5705,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5706,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5707,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5708,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5709,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5710,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5711,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5711,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5712,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5713,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5714,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5715,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5715,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5716,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5716,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5717,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5717,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5718,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5719,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5720,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5721,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5722,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5723,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5724,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5725,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5726,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5726,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5727,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5728,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5729,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5730,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5731,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5732,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5733,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5734,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5734,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5735,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5736,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5736,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5736,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5737,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5738,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5738,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5739,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5740,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5741,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5741,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5742,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5742,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5742,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5742,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5742,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5743,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5743,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5744,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5745,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5746,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5746,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5746,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5746,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5746,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5746,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5747,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5747,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5747,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5748,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5749,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5749,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5749,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5749,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5749,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5750,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5751,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5752,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5753,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5753,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5753,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5754,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5754,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5755,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5756,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5756,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5756,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5756,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5757,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5758,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5759,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5760,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5761,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5762,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5762,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5763,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5764,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5764,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5764,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5764,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5765,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5766,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5767,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5767,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5767,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5767,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5767,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5767,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5768,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5769,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5770,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5770,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5770,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5770,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5771,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5772,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5773,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5774,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5775,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5776,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5777,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5778,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5779,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5780,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5781,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5782,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5783,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5784,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5785,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5785,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5786,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5787,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5788,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5789,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5790,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5791,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5791,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5792,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5792,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5792,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5793,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5793,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5794,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5795,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5796,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5796,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5797,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5798,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5799,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5800,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5801,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5802,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5802,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5803,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5804,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5805,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5805,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5806,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5807,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5808,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5809,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5810,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5810,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5810,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5811,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5812,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5813,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5813,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5813,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5814,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5815,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5816,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5817,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5817,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5817,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5818,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5819,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5819,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5820,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5821,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5822,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5823,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5824,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5824,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5824,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5825,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5826,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5827,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5828,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5829,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5830,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=131002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5831,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5832,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5833,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5834,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5835,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5836,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5836,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5837,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5838,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5839,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5840,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5840,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5840,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5841,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5842,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5843,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5844,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5845,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5846,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5846,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5846,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5847,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5848,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5849,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5849,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5850,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5851,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5852,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5853,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5854,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5855,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5856,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5856,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5857,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5858,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5859,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5860,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5861,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5862,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5863,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5864,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5865,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5866,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5867,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5868,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5869,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5870,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5871,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5871,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5871,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5871,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5872,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5872,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5873,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5874,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5875,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5875,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5875,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5876,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5877,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5877,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5878,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5879,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5880,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5881,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5882,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5883,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=161276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5883,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5884,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5885,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5885,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5885,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5885,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5886,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5887,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5888,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5889,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5890,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5891,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5892,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5893,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5894,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5895,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5896,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5896,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5897,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5898,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5899,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5899,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5900,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5901,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5901,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5902,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5903,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=229988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5904,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5904,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5904,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5904,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5904,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5904,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5904,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5905,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5906,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5906,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5907,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5908,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5909,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5910,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5911,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5911,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5912,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5912,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5912,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5912,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5913,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5914,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5915,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5916,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5917,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5918,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5919,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5919,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5919,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5919,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5920,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5921,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5922,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5923,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5924,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5924,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5925,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5926,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5927,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5928,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5929,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5929,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5930,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5931,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5932,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5933,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5933,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5933,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5933,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5933,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5933,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5934,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5935,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5936,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5937,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5938,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5939,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5940,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5941,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5942,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5943,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5944,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5945,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5946,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5946,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5946,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5947,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5948,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5948,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5949,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5950,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5950,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5950,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5951,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5952,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5953,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5954,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5955,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5956,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5957,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5958,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5959,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5959,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5959,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5959,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5959,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5959,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5959,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5960,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5961,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5962,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5963,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5964,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5965,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5966,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5967,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5968,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5968,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5969,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5970,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5970,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5971,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5971,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5971,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5972,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5972,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5972,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5972,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5973,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5974,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5975,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5976,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5977,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5978,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5979,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5979,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5979,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5979,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5979,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5979,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5979,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5980,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5981,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5981,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5981,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5982,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5983,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5984,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5985,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5986,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5986,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5986,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5987,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5987,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5988,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5989,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5989,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5989,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5990,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5991,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5992,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5993,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5994,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5994,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5994,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5994,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5994,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5995,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5996,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5997,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5998,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(5999,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6000,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=52&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6000,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6000,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6000,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6001,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6001,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6002,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6002,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6003,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6003,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6004,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6005,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6005,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6006,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6007,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6007,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6008,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6009,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6010,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6011,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6012,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6013,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6014,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6015,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6016,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6017,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6018,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6019,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6019,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6019,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6019,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6020,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6021,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6022,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6023,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6023,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6024,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6025,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6026,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6027,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6028,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6028,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6028,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6028,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6029,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6030,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6031,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6032,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6033,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6034,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6034,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6035,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6036,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6037,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6037,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6037,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6037,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6037,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6038,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6038,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6039,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6040,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6041,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6042,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6042,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6042,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6043,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6044,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6044,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6045,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6046,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6047,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6048,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6049,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6050,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6051,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6052,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6053,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6053,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6053,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6053,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6053,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6054,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6055,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6056,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6057,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6058,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6059,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6060,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6061,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6062,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6063,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6064,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6065,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6066,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6066,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6067,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6067,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6068,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6069,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6070,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6071,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6072,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6073,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6074,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6075,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6076,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6077,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6078,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6079,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6079,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6080,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6081,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6081,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6082,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6083,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6084,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6085,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6086,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6087,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6088,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6089,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6090,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6091,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6092,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6093,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6094,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6094,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6094,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6094,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6095,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6096,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6097,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6098,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6099,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6100,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6101,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6101,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6101,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6101,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6101,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6101,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6101,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6102,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6103,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6104,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6105,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6106,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6107,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6108,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6108,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6109,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6110,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6111,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6112,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6113,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6114,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6115,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6116,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6117,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6117,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6118,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6119,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6120,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6120,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6120,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6120,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6121,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6122,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6123,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6124,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6125,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6126,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6127,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6127,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6128,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=144286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6128,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6129,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6130,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6131,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6132,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6133,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6134,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6135,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6136,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6137,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6137,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6137,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6138,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6139,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6140,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6141,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6142,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6143,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6144,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6145,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6146,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6147,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6148,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6149,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6149,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6150,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6151,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6151,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6152,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6153,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6154,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6155,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6156,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6157,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6158,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6159,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6160,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6161,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6162,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6163,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6163,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6164,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6165,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6166,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6166,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6167,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6168,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6169,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6169,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6170,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6171,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6171,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6172,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6172,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6172,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6172,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6172,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6172,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6172,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6172,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6173,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6174,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6175,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6176,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6177,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6178,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6179,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6180,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6181,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6182,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6183,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6184,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6184,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6184,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6184,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6184,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6184,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6184,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6185,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6186,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6187,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6187,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6187,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6187,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6187,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6188,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6189,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6190,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6191,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6192,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6193,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6194,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6195,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6195,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6196,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6196,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6197,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6198,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6199,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6200,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6200,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6200,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6200,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6200,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6201,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6202,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6203,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6204,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6205,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6206,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6207,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6208,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6209,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6210,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6211,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6212,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6213,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6214,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6215,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6216,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6217,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6218,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6219,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6220,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6221,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6221,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6222,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6223,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6224,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6225,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6226,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6227,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6228,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6229,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6230,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6230,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6230,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6230,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6231,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6232,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6233,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6234,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6235,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6236,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6237,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6238,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6239,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6240,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6240,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6240,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6240,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6240,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6240,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6241,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6241,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6242,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6243,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6244,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6244,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6245,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6245,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6245,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6246,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6247,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6248,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6249,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6250,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6251,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6251,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6252,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6253,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6254,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6255,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6255,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6256,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6257,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6258,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6259,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6260,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=253433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6261,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6261,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6261,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6262,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6263,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6264,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6265,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6266,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6266,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6266,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6267,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6268,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6269,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6270,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6271,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6272,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6272,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6272,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6272,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6273,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6273,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6274,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6275,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6276,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6277,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6278,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6279,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6280,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6281,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6282,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6283,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6284,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6285,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6286,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6286,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6286,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6286,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6286,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6286,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6287,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6288,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6289,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6289,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6290,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6291,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6292,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6293,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6294,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6294,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6295,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6296,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6297,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6297,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6298,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6299,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6299,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6300,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6301,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6302,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6303,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6304,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6304,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6305,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6306,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6307,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6307,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6308,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6309,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6310,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6311,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6312,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6313,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6314,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6315,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6315,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6316,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6317,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6318,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6319,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6320,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6320,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6320,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6321,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6322,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6323,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6323,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6324,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6325,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6326,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6326,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6327,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6327,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6328,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6329,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6329,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6329,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6330,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6330,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6331,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6332,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6332,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6333,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6334,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6334,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6335,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6335,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6335,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6336,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6337,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6338,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6339,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6340,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6341,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6341,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6342,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6343,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6344,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6345,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6346,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6346,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6346,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6346,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6346,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6346,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6346,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6347,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6348,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6348,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6349,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6350,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6351,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6352,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6353,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6354,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6355,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6355,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6356,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6357,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6357,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6358,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=128948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6359,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6359,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6360,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6361,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6361,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6362,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6362,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6362,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6363,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6364,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6365,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6366,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6367,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6368,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6369,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6370,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6370,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6371,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6372,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6373,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6374,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6374,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6375,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6376,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6377,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6378,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6379,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6380,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6380,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6381,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6381,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6382,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6383,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6384,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6384,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6385,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6386,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6387,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6388,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6388,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6388,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6388,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6389,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6389,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6389,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6390,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6391,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6392,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6393,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6393,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6394,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6395,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6396,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6397,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6398,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6398,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6399,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6400,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6400,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6400,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6401,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6402,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6403,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6404,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6405,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6406,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6407,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6408,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6409,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6410,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6411,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6411,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6412,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6412,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6413,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6414,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6415,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6416,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6417,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6417,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6418,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6419,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6420,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6421,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6421,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6421,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6422,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6423,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6424,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6425,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6425,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6425,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6425,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6425,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6425,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6425,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6426,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6426,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6426,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6426,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6426,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6427,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6428,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6429,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6430,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6431,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6432,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6433,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6434,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6435,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6436,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6437,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6437,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6437,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6438,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6439,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6440,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6441,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6442,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6443,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6444,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6445,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6446,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6447,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6447,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6447,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6448,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6449,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6449,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6450,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6451,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6451,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6451,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6451,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6451,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6451,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6452,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6452,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6453,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6454,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6455,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6456,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6456,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6457,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6457,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6457,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6457,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6458,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6459,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6460,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6460,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6460,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6460,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6460,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6460,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6460,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6461,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6462,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6463,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6464,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6465,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6466,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6467,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6467,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6468,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6469,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6469,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6470,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6471,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6471,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6472,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6473,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6474,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6475,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6476,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6477,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6477,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6478,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6479,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6480,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6481,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6482,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6483,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6483,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6484,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6484,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6485,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6486,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6486,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6486,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6486,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6486,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6486,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6486,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6486,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6487,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6488,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=59&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6488,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6488,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6488,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6488,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6489,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6489,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6490,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6490,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6490,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6490,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6490,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6490,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6491,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6492,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6493,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6493,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6494,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6494,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6494,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6494,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6494,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6495,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6496,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6497,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6498,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6499,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6500,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6501,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6502,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6503,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6504,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6505,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6506,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6507,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6507,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6508,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6509,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6510,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6511,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6512,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6513,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6514,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6515,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6516,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6517,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6518,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6518,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6518,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6518,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6518,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6518,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6518,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6518,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6519,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6520,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6521,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6522,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6523,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6523,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6523,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6524,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6525,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6526,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6527,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6528,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6529,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6530,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6530,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6531,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6532,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6533,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6534,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6534,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6535,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6536,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6537,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6538,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6539,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6540,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6541,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6541,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6542,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6543,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6543,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6543,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6543,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6544,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6545,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6546,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6546,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6547,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6547,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6548,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6549,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6549,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6550,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6551,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6552,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6552,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6552,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6553,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6554,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6554,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6555,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6556,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6557,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6558,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6559,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6560,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6561,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6562,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6563,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6564,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6565,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6566,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6567,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6567,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6568,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6569,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6569,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6570,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6571,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6572,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6573,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6574,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6575,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6575,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6576,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6577,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6577,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6577,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6578,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6579,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6580,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6581,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6582,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6583,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6584,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6584,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6585,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6586,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6587,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6587,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6588,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6589,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6590,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6591,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6591,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6592,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6593,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6593,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6594,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6595,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6596,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6597,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6598,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6598,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6599,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6600,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6601,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6601,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6602,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6603,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6604,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6604,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6604,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6604,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6604,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6605,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6606,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6607,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6608,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6609,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6609,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6610,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6611,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6611,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6612,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6613,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6614,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6615,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6615,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6615,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6616,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6617,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6618,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6619,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6620,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6621,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6622,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6623,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6624,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6624,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6625,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6626,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6627,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6628,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6629,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6630,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6631,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6632,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6633,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6634,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6635,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6636,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6637,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6638,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6639,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6640,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=7167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6641,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6642,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6643,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6644,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6644,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6645,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6646,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6647,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6647,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6648,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6648,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6649,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6649,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6650,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6650,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6651,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6652,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6653,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6654,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6654,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6654,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6655,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6655,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6655,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6655,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6656,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6657,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6658,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6659,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6659,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6659,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6660,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6661,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6662,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6663,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6663,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6664,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6664,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6665,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6666,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6666,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6666,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6666,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6666,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6667,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6668,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6668,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6668,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6669,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6669,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6669,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6669,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6670,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6671,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6672,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6673,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6674,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6675,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6676,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6676,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6677,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6678,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6679,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6680,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6680,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6680,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6680,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6681,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6681,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6681,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6682,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6683,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6684,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6685,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6686,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6687,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6687,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6688,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6689,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6689,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6690,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6691,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6692,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6693,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6694,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6695,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6696,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6697,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6698,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6698,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6699,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6700,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6701,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6702,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6703,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6704,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6705,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6706,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6707,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6708,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6709,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6710,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6711,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6711,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6712,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6712,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6713,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6714,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6715,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6716,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6717,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6718,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6718,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6718,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6719,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6720,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6721,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6722,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6723,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6723,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6723,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6723,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6724,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6725,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6726,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6726,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=7169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6727,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6728,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=161291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6729,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6730,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6731,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6732,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6733,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6734,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6735,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6736,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6737,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6738,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6739,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6740,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6740,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6741,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6742,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6743,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6744,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6745,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6746,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6747,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6748,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6748,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6749,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6749,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6750,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6751,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6752,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6753,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6754,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6755,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6756,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6757,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6757,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6758,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6759,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6760,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6761,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6762,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6763,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6763,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6764,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6765,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6766,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6767,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6768,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6769,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6770,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6771,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6772,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6773,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6774,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6775,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6776,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6777,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6778,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6779,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6780,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6780,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6781,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6782,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6783,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6784,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6785,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6786,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6787,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6788,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6789,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6790,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6791,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6792,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6792,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6793,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6794,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6795,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6795,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6796,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6796,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6797,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6798,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6799,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6800,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6801,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6802,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6803,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6804,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6805,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6806,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6807,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6808,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6809,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6810,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6810,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6811,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6812,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6812,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6812,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6813,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6813,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6814,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6815,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6816,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6817,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6817,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6817,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6817,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6818,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6819,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6820,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6820,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6821,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6821,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6821,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6821,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6821,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6822,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6823,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6824,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6825,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6825,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6825,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6826,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6827,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6827,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6828,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6829,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6830,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6831,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6831,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6831,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6832,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6833,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6833,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6834,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6835,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6836,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6837,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6838,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6839,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6840,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6841,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6842,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6842,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6843,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6844,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6845,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6846,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6847,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6848,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6849,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6849,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6850,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6850,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6851,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6852,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6853,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6854,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6855,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6855,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6855,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6856,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6857,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6857,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6857,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6858,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6859,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6860,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6861,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6862,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6863,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6863,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6864,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6865,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6865,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6866,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6866,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6867,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6868,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6869,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6870,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6870,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6870,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6870,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6871,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6872,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6872,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6872,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6872,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6873,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6874,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6875,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6876,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6877,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6878,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6879,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6880,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6881,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6881,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6881,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6881,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6882,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6883,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6884,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6885,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6885,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6886,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6887,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6887,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6887,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6888,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6889,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6890,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6891,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6892,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6893,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6893,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6894,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6894,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6894,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6895,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6895,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6896,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6897,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6898,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6899,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6899,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6900,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6900,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6900,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6900,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6900,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6901,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6902,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6903,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6904,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6905,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6906,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6906,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6906,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6907,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6908,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6909,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6909,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6909,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6910,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6910,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6910,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6910,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6910,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6910,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6911,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6912,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6913,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6913,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6914,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6914,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6915,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6915,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6916,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6917,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6917,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6917,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6917,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6918,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6919,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6920,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6921,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6922,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6923,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6924,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6925,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6926,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6926,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6927,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6927,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6928,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6929,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6930,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6931,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6932,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6933,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6934,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6935,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6936,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6937,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6938,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6939,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6940,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6941,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6942,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6943,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6944,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6945,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6946,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6947,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6947,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6947,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6947,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6947,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6947,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6948,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6949,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6950,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6950,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6951,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6952,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6952,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6953,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6954,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6954,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6955,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6956,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6957,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6958,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6959,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6960,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6961,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6962,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6962,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6962,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6962,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6962,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6962,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6963,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6964,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6965,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6965,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6966,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6967,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6968,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6969,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6970,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6971,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6972,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6973,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6974,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6974,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6975,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6976,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6977,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6978,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6978,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6978,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6979,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6980,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6980,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6981,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6982,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6982,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6982,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6983,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6984,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6985,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6986,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6987,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6988,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6989,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6990,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6991,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6992,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6992,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6993,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6994,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6995,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6996,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6997,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6998,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6999,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(6999,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7000,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7001,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7002,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7003,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7004,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7005,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7006,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7007,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7007,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7008,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7009,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7010,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7011,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7011,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7011,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7011,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7011,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7012,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7012,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7013,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7014,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7014,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7015,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7016,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7017,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7018,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7018,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7018,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7018,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7019,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7020,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7021,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7022,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7023,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7024,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7025,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7025,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7025,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7025,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7026,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7026,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7027,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7027,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7027,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7027,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7027,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7028,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7029,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7029,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7030,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7031,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7032,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7033,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7034,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7035,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7035,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7035,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7036,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7036,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7036,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7037,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7038,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7039,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7040,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7040,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7040,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7040,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7041,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7042,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7042,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7042,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7042,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7042,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7042,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7043,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7044,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7044,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7045,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7046,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7047,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7048,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7048,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7048,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7048,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7048,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7048,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7048,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7049,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7050,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7050,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7051,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7052,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7052,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7053,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7054,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7055,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=279609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7056,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7057,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7058,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7059,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7060,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7060,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7061,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7062,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7063,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7064,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7065,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7066,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7067,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7068,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7069,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7070,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7071,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7071,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7071,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7072,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7073,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7073,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7074,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7075,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7076,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7077,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7077,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7078,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7078,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7078,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7079,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7080,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7081,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7082,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7082,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7082,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7083,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=54&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7084,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7085,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7086,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7086,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7087,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7088,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7088,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7088,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7088,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7089,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7089,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7090,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7090,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7091,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7092,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7092,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7092,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7093,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7093,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7093,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7094,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7094,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7095,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7096,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7097,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7098,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7098,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7099,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7100,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7101,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7102,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7102,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7103,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7103,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7103,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7104,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7105,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7106,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7107,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7108,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7109,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7110,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7110,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7111,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7112,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7113,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7113,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7113,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7113,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7113,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7113,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7114,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7115,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7116,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7117,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7118,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7118,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7118,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7118,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7119,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7120,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7121,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7122,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7123,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7124,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7125,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7126,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7127,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7128,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7129,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7130,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7131,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7132,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7132,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7132,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7133,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7134,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7135,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7136,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7136,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7136,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7136,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7136,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7136,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7137,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7138,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7139,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7140,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7141,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7141,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7142,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7143,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7144,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7144,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7145,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7146,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7146,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7146,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7147,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7148,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7149,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7150,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7151,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7152,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7153,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7154,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7155,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7156,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7156,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7157,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7158,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7159,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7160,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7161,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7162,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7163,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7164,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7165,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7166,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7167,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7168,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7168,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7169,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7170,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7170,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7171,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7171,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7172,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7173,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7174,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7175,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7176,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7177,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7178,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7179,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7180,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7181,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7181,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7182,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7183,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7183,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7183,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7183,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7184,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7185,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7186,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7187,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7188,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7188,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7189,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7190,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7191,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7192,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7193,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7194,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7195,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7196,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7197,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7198,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7199,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7200,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7201,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7202,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7203,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7204,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7204,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7205,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7206,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7206,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7207,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7208,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7209,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7209,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7210,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7211,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7211,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7211,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7211,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7212,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7213,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7214,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7215,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7215,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7215,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7215,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7215,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7215,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7216,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7217,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7218,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7218,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7219,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7220,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7220,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7221,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7221,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7222,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7223,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7223,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7224,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7225,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7226,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7226,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7227,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7228,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7229,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7230,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7231,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7232,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7233,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7233,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7234,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7234,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7235,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7236,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7236,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7236,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7236,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7237,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7238,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7238,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=131011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7239,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7240,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7241,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7241,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7242,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7243,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7244,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7245,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7245,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7246,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7247,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7248,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7249,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7249,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7250,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7250,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7251,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7252,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7253,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7254,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7255,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7256,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7256,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7257,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7257,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7257,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7257,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7257,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7258,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7259,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7260,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7261,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7261,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7262,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7262,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7262,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7262,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7263,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7263,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7263,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7263,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7263,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7263,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7263,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7263,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7264,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7264,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7265,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7266,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7267,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7268,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7269,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7269,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7270,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7271,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7272,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7273,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7274,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7275,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7276,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7277,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7278,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7279,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7280,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=229952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7281,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7281,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7282,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7283,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7284,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7285,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7286,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7287,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7288,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7289,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7289,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7290,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7291,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7292,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7292,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7293,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7294,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7295,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7296,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7297,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7298,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7298,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7299,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7300,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7301,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7301,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7302,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7303,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7304,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7305,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7306,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7307,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7307,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7308,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7308,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7308,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7308,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7309,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7309,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7310,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7311,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7311,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7311,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7312,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7313,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7314,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7315,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7315,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7315,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7315,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7315,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7315,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7316,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7317,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7318,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7319,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7320,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7321,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7321,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7321,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7322,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7323,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7324,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7325,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7326,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7326,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7327,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7328,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7328,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7329,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7329,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7330,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7330,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7331,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7331,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7332,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7333,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7334,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7335,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7335,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7335,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7336,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7337,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7338,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7339,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7340,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7341,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7342,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7343,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7344,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7344,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7344,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7344,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7345,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7346,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7347,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7347,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7348,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7349,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7350,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7351,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7351,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7352,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7353,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7354,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7355,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7356,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7357,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7358,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7359,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7359,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7360,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7361,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7361,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7362,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7363,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7364,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7365,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7365,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7365,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7365,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7365,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7366,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7366,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7367,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7367,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7368,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7368,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7368,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7369,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7369,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7369,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7369,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7369,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7369,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7370,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7370,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7370,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7371,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7372,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7373,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7374,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7375,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7376,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7377,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7378,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7379,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7380,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7381,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7381,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7382,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7383,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7384,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7385,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7385,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7385,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7386,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7387,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7388,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7388,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7389,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7390,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7391,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7392,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7392,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7393,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7394,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7395,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7395,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7396,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7396,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7397,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7398,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7399,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7400,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7401,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7402,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7403,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7404,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7405,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7406,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7407,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7408,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7409,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7410,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7411,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7412,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7413,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7413,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7414,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7415,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7416,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7417,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7418,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7419,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7420,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7421,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7422,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7423,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7423,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7424,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7424,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7425,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7425,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7426,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7426,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7427,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7428,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7428,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7429,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7430,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7431,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7432,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7433,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7434,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7434,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7435,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7436,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7437,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7438,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7439,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7440,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7441,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7442,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7443,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7444,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7445,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7446,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7446,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7446,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7446,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7447,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7448,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7448,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7449,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7450,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7450,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7451,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7452,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7453,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7454,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7454,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7455,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7455,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7456,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7457,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7458,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7458,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7459,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7460,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7460,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7461,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7462,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7463,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7463,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7464,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7464,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7465,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7465,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7466,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7467,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7467,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7468,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7469,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7470,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7470,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7470,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7470,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7470,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7471,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7471,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7471,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7472,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7472,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7473,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7473,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7474,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7474,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7475,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7476,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7477,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7478,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7479,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7480,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7481,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7481,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7482,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7483,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7484,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7484,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7484,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7484,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7484,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7485,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7486,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7487,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7488,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7489,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7489,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7490,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7491,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7492,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7493,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7494,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7495,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7496,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7497,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7498,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7499,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7500,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7500,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7500,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7500,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7500,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7501,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7502,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7503,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7504,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7505,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7506,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7507,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7507,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7508,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7509,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7509,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7509,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7510,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7511,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7512,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7513,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7514,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7515,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7516,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7517,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7518,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7519,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7520,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7521,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7522,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7523,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7523,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7523,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7523,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7524,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7525,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7525,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7526,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7527,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7528,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7528,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7528,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7528,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7529,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7530,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7531,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7532,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7533,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7534,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7535,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7536,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7537,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7538,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7538,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7539,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7540,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7540,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7540,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7540,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7541,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7542,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7543,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7544,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7545,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7545,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7546,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7547,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7548,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7549,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7550,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7551,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7552,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7552,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7552,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7553,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7553,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7553,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7554,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7555,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7556,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7557,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7558,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7558,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7559,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7560,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7561,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7562,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7563,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7563,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7563,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7563,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7563,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7563,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7564,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7565,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7565,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7566,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7567,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7567,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7568,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7569,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7569,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7570,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7571,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7572,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7572,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7573,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7573,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7574,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7575,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7576,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7576,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7577,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7578,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7579,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7580,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7581,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7582,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7583,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7583,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7584,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7585,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7585,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7586,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7587,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7588,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7588,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7589,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7590,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7591,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7592,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7592,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7593,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7594,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7594,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7595,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7596,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7597,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7598,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7599,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7600,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7601,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7601,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7602,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7603,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7604,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7605,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7606,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7607,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7607,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7607,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7607,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7607,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7608,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7609,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7609,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7609,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7610,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7611,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7612,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7613,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7614,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7615,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7616,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7617,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7618,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7619,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7620,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7621,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7622,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7623,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7623,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7624,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7625,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7626,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7626,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7627,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7628,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7628,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7628,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7629,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7630,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7631,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7632,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7633,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7633,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7634,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7635,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7635,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7636,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7636,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7636,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7636,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7636,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7636,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7637,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7638,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7639,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7639,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7640,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7640,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7640,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7640,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7641,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7642,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7643,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7644,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7645,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7646,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7647,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7648,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7649,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7649,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7650,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7651,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7651,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7651,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7651,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7652,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7653,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7654,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7655,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7656,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7657,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7657,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7658,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7659,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7660,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7660,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7660,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7660,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7661,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7662,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7663,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7664,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7665,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7666,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7666,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7667,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7668,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7669,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=58&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7669,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7669,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7669,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7670,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7671,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7672,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7673,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7673,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7674,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7675,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7676,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7676,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7677,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7678,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7678,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7679,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7680,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7681,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7682,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7683,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7684,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7685,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7685,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7686,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7687,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7688,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7689,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7690,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7691,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7692,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7693,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7694,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7694,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7694,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7695,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7696,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7697,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7698,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7699,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7699,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7700,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7700,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7700,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7701,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7701,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7701,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7702,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7702,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7703,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7704,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7705,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7706,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7707,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7708,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7709,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7710,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7711,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7712,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7713,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7714,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7714,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7715,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7716,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7716,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7717,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7718,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7719,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7720,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=144260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7720,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7721,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7722,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7723,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7724,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7725,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7726,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7726,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7727,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7728,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7728,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7729,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7730,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7731,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7731,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7732,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7733,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7733,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7734,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7735,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7736,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7737,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7738,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7739,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7740,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7740,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7741,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7742,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7743,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7744,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7744,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7744,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7745,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7746,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7747,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7747,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7748,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7749,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7749,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7749,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7750,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7751,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7752,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7753,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7754,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7755,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7756,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7757,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7758,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7758,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7759,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7760,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7761,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7762,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7762,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7763,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7763,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7764,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7765,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7765,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7766,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7766,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7767,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7768,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7769,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7770,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7770,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7771,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7772,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7773,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7774,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7775,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7776,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7776,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7776,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7776,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7776,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7776,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7777,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7778,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7779,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7780,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7781,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7781,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7782,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7782,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7783,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7783,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7784,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7785,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7786,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7786,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7786,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7787,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7788,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7789,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7789,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7789,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7790,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7790,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7790,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7790,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7790,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7790,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7790,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7791,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7792,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7793,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7793,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7794,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7794,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7794,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7795,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7796,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7797,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7798,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7799,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7800,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7801,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7802,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7803,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7804,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7805,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7806,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=169977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7807,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7808,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7809,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7809,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7809,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7810,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7810,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7810,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7811,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7812,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7812,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7813,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7814,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7815,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7816,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7817,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7818,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7819,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7820,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7821,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7821,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7822,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7823,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7824,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7825,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7826,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7827,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7828,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7829,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7830,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7831,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7832,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7833,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7834,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7835,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7836,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7836,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7836,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7837,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7838,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7839,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7840,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7841,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7842,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7843,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7844,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7845,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7845,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7845,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7845,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7846,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7846,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7847,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7848,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7849,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7850,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7850,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7851,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7851,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7852,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7853,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7854,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7855,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7856,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7857,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7857,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7858,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=246945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7859,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7860,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7861,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7862,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7863,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7864,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7865,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7866,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7866,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7867,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7868,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7869,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7870,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7870,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7871,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7872,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7873,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7874,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7874,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7875,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7876,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7877,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7878,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7879,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7879,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7879,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7880,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7880,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7881,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7882,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7882,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7883,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7884,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7884,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7884,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7884,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7884,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7885,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7886,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7887,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7888,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7889,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7890,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7891,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7892,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7893,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7894,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7895,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7895,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7896,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7897,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7898,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7899,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7899,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7900,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7900,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7900,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7901,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7901,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7901,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7901,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7901,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7902,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7902,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7903,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7904,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7905,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7906,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7907,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7908,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7909,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7910,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7911,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7911,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7912,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7913,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7913,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7913,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7914,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7915,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7915,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7916,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7917,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7917,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7918,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7918,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7918,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7919,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7920,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7921,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7922,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7923,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7924,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7925,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7925,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7925,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7925,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7925,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7925,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7926,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7927,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7928,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7928,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7928,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7929,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7930,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7931,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7932,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7933,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=137936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7933,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7933,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7934,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7935,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7936,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7936,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7936,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7936,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7937,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7938,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7939,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7940,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7941,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7942,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7942,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7943,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7944,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7944,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7944,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7944,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7944,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7945,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7946,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7947,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7948,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7949,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7950,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7950,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7950,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7950,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7950,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7951,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7952,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7953,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7953,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7954,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7955,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7956,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7957,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7957,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7958,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7959,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7960,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7961,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7961,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7961,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7962,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7963,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7964,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7965,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7966,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7967,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7968,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7968,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7969,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7969,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7969,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7970,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7971,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7972,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7972,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7973,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7974,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7975,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7976,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7976,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7977,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7977,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7978,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7979,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7980,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7981,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7982,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7982,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7983,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7984,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7985,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7986,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7987,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7988,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7989,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7990,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7991,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7992,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7993,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7994,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7995,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7996,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7996,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7997,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7998,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(7999,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8000,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8001,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8002,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8002,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8003,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8004,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8004,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8005,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8006,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8006,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8006,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8007,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8008,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8009,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8009,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8009,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8010,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8010,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8010,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8011,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8012,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8013,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8014,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8015,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8015,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8016,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8017,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8018,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8019,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8020,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8021,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8022,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8023,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8023,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8024,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8025,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8025,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8026,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8027,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8027,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8028,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8029,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8030,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8030,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8031,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8032,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8033,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8034,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8035,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8036,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8037,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8038,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8039,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8040,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8041,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8042,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8043,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8043,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8043,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8043,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8043,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8043,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8044,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8045,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8045,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8046,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8047,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8048,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8049,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8050,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8051,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8052,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8053,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8054,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8055,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8056,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8057,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8058,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8059,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8060,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8061,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8062,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8062,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8063,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8064,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8064,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8065,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8066,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8066,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8067,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8067,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8068,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8068,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8068,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8069,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8070,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8071,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8072,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8073,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8074,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8075,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8076,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8077,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8077,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8077,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8078,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8079,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8080,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8080,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8081,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8082,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8083,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8084,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8085,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8086,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8087,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8088,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8089,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8090,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8090,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8090,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8091,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8092,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8093,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8093,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8093,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8094,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8095,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8096,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8097,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8097,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8098,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8098,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8099,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8100,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8101,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8101,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8102,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8103,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8103,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8104,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8105,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8106,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8107,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8108,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8109,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8110,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8110,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8110,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8110,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8110,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8111,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8112,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8113,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8114,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8115,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8116,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8117,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8117,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=60&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8117,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8117,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8117,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8117,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8118,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8119,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8120,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8121,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8122,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8123,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8124,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8125,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8126,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8127,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8128,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8129,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8130,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8131,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8132,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8133,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8133,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8134,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8135,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8136,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8136,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8136,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8137,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8138,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8138,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8138,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8138,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8139,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8139,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8140,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8141,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8142,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8142,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8143,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8144,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8144,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8144,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8144,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8144,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8144,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8144,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8145,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8146,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8146,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8147,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8148,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8148,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8148,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8149,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8150,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8151,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8152,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8153,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8154,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8155,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8156,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8157,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8158,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8159,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8160,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8161,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8162,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8163,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8164,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8165,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8166,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8166,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8167,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8167,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8168,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8169,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8169,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8170,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8171,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8172,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8173,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8173,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8173,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8173,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8173,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8173,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8174,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8175,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8176,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8176,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8177,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8177,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8177,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8177,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8178,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8179,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8180,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8181,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8182,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8183,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8184,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8185,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8185,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8186,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8187,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8188,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8189,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8189,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8190,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8191,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8192,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8193,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8194,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8195,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8196,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8197,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8197,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8197,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8197,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8197,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8197,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8198,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8198,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8199,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8199,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8200,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8201,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8201,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8202,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8203,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8204,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8205,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8206,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8206,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8207,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8207,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8208,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8209,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8210,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8210,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8210,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8211,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8212,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8213,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8214,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8214,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8215,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8216,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8217,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8217,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8217,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8218,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8219,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8220,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8220,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8220,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8221,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8222,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8223,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8224,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8225,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8225,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8226,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8227,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8228,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8229,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8230,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8231,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8231,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8232,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8232,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8233,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8233,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8234,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8235,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8235,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8236,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8237,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8238,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8239,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8240,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8241,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8242,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8242,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8242,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8243,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8244,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8245,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8246,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8246,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8247,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8248,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8249,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8250,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8251,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8252,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8253,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8254,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8255,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8256,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8256,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8257,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8258,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8259,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8260,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8261,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8261,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8262,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8263,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8264,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8265,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8266,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8267,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8268,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8268,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8268,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8269,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8270,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8271,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8271,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8271,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8272,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8273,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8274,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8275,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8276,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8276,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8277,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8278,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8279,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8280,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8280,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8281,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8282,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8282,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8283,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8283,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8284,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8284,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8285,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8286,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8287,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8288,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8289,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8289,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8289,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8289,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8290,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8291,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8292,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8292,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8292,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8293,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8294,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8295,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8296,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8296,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8296,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8296,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8297,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8297,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8298,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8298,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8298,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8299,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8300,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8301,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8302,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8303,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8304,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8305,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8306,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8307,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8308,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8309,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8310,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8311,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8312,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8312,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8313,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8314,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8315,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8315,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8316,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8317,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8317,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8318,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8319,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8320,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8320,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8321,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8322,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8323,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8324,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8324,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8325,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8326,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8327,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8328,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8329,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8330,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8331,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8332,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8332,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8333,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8333,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8333,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8334,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8335,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8336,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8336,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8336,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8336,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8336,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8336,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8336,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8336,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8337,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8338,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8339,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=138217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8339,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8340,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8341,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8341,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8342,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8342,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8343,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8344,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8345,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8345,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8346,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8347,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8348,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8349,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8350,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8350,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8351,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8352,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8353,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8354,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8354,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8354,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=94914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8354,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8354,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8354,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8354,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8354,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8355,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8356,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8357,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8358,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8359,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8360,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8360,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8361,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8362,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8363,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8364,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=169958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8365,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8366,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8367,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8368,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8368,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8369,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8369,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8370,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8371,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8372,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8373,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8374,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8375,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8376,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8377,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8377,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8378,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8379,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8380,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8381,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8382,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8383,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8384,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8385,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8386,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8387,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8388,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8389,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8390,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8390,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8391,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8392,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8393,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8393,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8394,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8395,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=90&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8396,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8397,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8398,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8398,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8399,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8400,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8401,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8402,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8403,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8404,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8405,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8406,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8407,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8408,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8408,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8409,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8410,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8411,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8412,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8413,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8413,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8413,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8414,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8415,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8415,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8415,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8416,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8417,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8417,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8418,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8418,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8419,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8420,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8421,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8422,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8423,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8424,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8425,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8425,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8425,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8425,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8426,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8427,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8427,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8428,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8428,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8428,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8428,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8428,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8428,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8428,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8429,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8429,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8430,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8431,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8432,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8433,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8433,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8434,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8435,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8436,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8437,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8438,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8438,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8438,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8438,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8439,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8440,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8441,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8442,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8443,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8444,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8445,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8445,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8446,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8447,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8448,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8449,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8450,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8451,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8452,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8453,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8454,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8455,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8456,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8457,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8458,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8459,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8460,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8461,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8462,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8462,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8462,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8463,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8464,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8465,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8466,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8467,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8468,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8469,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8470,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8471,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8472,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8473,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8473,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8474,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8475,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8476,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8476,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8476,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8476,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8476,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8476,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8477,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8477,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8478,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8479,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8480,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8481,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8482,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8483,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8484,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8485,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8485,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8485,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8486,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8487,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8487,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8488,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8489,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8489,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8490,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8490,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8491,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8492,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8493,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8494,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8494,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8495,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8496,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8496,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8497,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8498,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8499,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8499,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8500,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8501,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8502,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8503,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8504,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8505,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8505,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8506,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8507,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8508,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8509,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8510,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8511,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8512,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8513,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8514,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8515,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8515,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8516,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8517,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8518,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8519,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8520,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8520,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8521,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8522,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8523,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8524,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8525,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8526,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8527,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8528,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8529,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8530,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8531,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8532,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8533,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8533,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8534,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8534,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8535,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8536,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8537,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8537,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8538,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8539,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8540,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8541,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8541,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8541,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8541,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8541,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8541,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8542,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8543,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8543,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8544,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8545,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8545,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8546,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8546,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8546,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8546,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8547,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8548,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8548,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8549,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8550,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8550,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8551,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8551,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8552,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8553,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8554,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8555,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8556,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8557,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8558,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8559,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8560,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8561,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8562,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8563,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8564,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8564,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8565,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8566,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8567,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8567,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8568,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8569,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8570,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8571,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8572,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8572,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8572,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8572,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8573,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8574,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8575,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8576,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8577,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8577,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8578,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8579,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8580,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8581,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8582,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8583,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8583,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8584,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8585,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8585,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8586,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8587,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8587,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8587,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8587,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8587,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8588,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8589,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8589,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8589,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8589,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8590,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8591,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8592,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8593,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8594,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8595,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8596,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8597,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8598,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8599,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8600,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8601,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8601,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8602,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8603,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8603,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8603,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8604,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8605,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8606,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8607,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8608,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=85106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8609,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8610,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8611,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8612,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8613,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8613,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8613,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8613,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8613,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8613,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8613,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8614,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8615,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8615,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8615,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8616,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8617,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8618,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8619,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8619,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8619,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8619,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8619,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8619,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8619,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8620,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8620,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8620,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8621,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8622,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8623,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8624,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8625,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8625,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8626,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8627,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8627,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8628,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8629,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8629,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8629,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8630,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8631,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8632,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8633,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8634,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8635,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8636,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8636,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8637,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8637,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8637,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8637,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8637,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8637,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8638,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8638,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8638,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8638,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8638,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8638,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8639,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8639,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8640,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8640,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8641,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8641,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8642,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8643,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8643,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8644,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8645,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8645,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8646,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8646,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8646,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8647,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8648,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8648,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8649,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8649,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=57&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8649,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8649,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8649,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8650,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8651,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8651,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8652,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8652,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8652,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8652,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8652,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8653,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8653,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8654,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8654,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8655,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8655,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8655,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8656,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8656,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8657,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8658,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8659,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8660,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8661,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8662,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8663,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8664,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8665,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=253429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8666,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8667,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8667,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8667,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8667,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8667,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8668,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8669,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8670,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8671,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8672,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8673,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8673,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8674,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8675,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8676,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8677,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8677,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8677,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8678,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8679,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8680,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8680,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8680,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8680,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8680,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8680,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8681,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8682,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8682,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8683,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8684,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8685,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8686,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8687,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8688,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8689,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8690,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8691,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8691,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8692,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8692,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8693,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8694,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8694,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8695,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8696,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8697,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8698,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8699,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8700,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8700,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8701,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8701,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8702,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8703,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8704,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8705,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8706,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8706,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8706,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8707,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8708,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8708,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8708,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8709,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8710,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8711,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8712,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8713,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8713,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8713,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8713,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8714,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8715,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8716,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8717,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8717,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8717,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8717,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8717,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8718,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8718,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8718,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8719,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8719,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8720,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8721,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8722,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8723,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8724,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8724,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8725,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8726,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8727,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8728,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8728,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8728,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8729,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8730,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8731,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8732,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8733,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8733,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8733,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8733,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8733,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8734,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8735,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8736,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8737,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8738,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8739,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8740,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8741,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8742,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8743,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8743,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8743,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8743,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8744,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8745,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8746,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8746,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8747,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8748,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8749,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=229960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8750,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8750,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8751,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8752,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8752,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8753,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8754,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8755,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8756,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8757,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8758,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8759,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8760,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8761,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8762,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8763,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8764,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8765,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8765,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8765,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8766,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8767,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8768,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8769,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8770,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8771,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8772,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8773,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8774,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8775,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8776,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8777,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8778,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8779,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8780,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8781,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8782,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8783,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8783,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8783,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8784,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8785,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8786,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8787,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8788,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8789,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8790,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8791,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8792,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8793,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8794,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8795,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8796,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8796,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8797,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8798,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8799,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8799,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8800,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8800,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8801,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8801,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8802,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8802,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8803,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8804,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8805,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8805,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8806,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8807,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8808,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8809,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8810,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8811,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8811,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8812,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8813,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8814,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8815,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8816,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8816,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8816,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8817,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8817,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8817,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8818,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8819,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8820,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=86&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8821,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8822,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8823,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8824,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8825,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8826,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8827,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8828,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8829,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8829,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8829,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8829,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8829,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8830,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8831,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8832,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8832,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8832,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8832,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8833,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8834,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8835,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8836,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8837,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8837,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8838,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8839,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8840,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8841,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8842,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8842,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8842,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8842,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8842,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8842,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8842,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8842,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8843,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8844,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8845,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8845,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8846,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8847,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8848,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8849,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8850,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8850,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8850,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8850,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8850,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8850,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8850,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8850,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8851,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8852,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8853,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8854,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8855,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8855,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8856,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8857,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8858,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8858,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8858,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8859,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8860,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8861,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8862,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8863,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8863,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8864,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8864,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8864,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8864,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8864,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8864,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8864,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8865,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8866,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8867,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8868,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8869,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8870,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8871,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8871,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8872,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8872,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8872,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8873,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8874,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8875,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8876,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8876,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8876,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8877,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8878,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8879,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8880,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8881,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8882,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8882,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8882,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8883,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8884,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8885,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8885,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8886,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8887,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8887,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8887,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8887,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8888,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8889,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8890,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8891,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8891,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8891,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8891,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8891,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8892,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8893,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8893,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8894,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8895,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8896,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8897,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8898,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8899,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8900,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8901,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8901,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8902,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8902,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8902,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8903,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8903,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8904,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8904,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8904,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8904,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8904,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8905,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8905,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8906,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8907,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8908,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8909,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8909,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8909,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8909,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8909,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8909,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8910,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8911,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8912,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8913,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8914,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8915,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8916,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8917,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8917,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8918,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8919,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8920,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8920,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8921,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8922,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8922,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8922,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8922,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8922,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8922,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8923,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8924,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8925,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8926,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8927,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8928,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8928,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8928,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8928,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8928,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8928,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8928,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8928,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8929,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8930,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8931,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8931,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8931,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8931,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8932,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8933,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8934,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8935,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8935,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8935,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8935,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8935,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8936,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8936,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8936,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8936,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8936,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8936,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8937,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8937,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8937,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8938,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8939,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8940,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8941,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8941,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8942,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8943,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8944,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8945,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8945,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8946,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8947,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8948,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8949,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8950,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8951,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8952,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8953,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8953,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8954,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8955,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8955,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8956,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8957,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8957,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8957,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8958,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8959,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8960,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8961,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8962,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8963,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8963,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8963,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8963,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8964,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=229968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8965,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8966,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8966,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8967,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8968,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8969,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8970,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8971,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8972,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8973,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8974,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8975,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8976,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8977,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8978,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8978,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8979,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8980,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8981,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8982,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8983,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8984,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8985,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8986,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8986,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8986,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8987,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8987,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8988,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8989,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8990,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8991,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8992,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8993,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8994,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8995,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8996,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8997,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8997,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8998,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(8999,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9000,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9000,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9001,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9002,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9002,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9003,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9003,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9004,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9005,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9006,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9007,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9008,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9009,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9009,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9009,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9010,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9010,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9011,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9012,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9013,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9014,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9015,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9016,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9017,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9017,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9018,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9018,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9019,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9020,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9021,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9021,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9022,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9023,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9024,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9025,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9026,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9027,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9028,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9029,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9030,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9030,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9031,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9032,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9033,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9034,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9035,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9035,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9036,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9036,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9037,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9037,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9038,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9039,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9039,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9039,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9039,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9039,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9040,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9040,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9041,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9042,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9043,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9044,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9045,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9045,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=64&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9046,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9047,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9048,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9049,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9050,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9051,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9052,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9053,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9054,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9055,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9056,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9056,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9057,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9058,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9059,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9059,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9060,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9061,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9062,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9063,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9063,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9064,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9065,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9066,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9067,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9068,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9069,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9069,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9070,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9071,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9071,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9072,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9073,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9074,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9074,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9074,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9074,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9074,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9074,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9074,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9075,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9076,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9077,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9078,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9079,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9080,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9081,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9082,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9083,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9084,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9085,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9086,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9086,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9086,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9087,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9087,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9088,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9089,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9090,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9090,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9091,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9092,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9093,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9094,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9095,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9096,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9097,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9097,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9097,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9097,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9097,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9097,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9098,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9099,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9099,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9100,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9101,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9102,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9102,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9103,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9104,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9105,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9106,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9107,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9108,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9108,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9108,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9109,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9110,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9111,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9112,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9113,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9114,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9114,(SELECT setid FROM sets WHERE sets.name = 'FVL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9115,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9115,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9115,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9116,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9117,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9118,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9119,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9119,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9120,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9121,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9122,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9123,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9124,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9125,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9126,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9127,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9128,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9129,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9130,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9131,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9132,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9132,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9133,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9134,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9135,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9135,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9136,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9137,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9138,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9139,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=131005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9140,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3872&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9140,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9140,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9141,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9142,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9142,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9142,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9142,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9142,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9143,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9144,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9144,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9144,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9145,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9145,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9146,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9147,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9147,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9148,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9148,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9148,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9149,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9150,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9151,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9151,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9152,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9153,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9153,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9153,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9154,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9155,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9156,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9156,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9156,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9156,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9157,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9158,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9158,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9159,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9160,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9161,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9161,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9162,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9163,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9164,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9165,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9166,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9166,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9166,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9166,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9167,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9168,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9169,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=246947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9170,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9171,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9172,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9173,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9173,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9174,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9175,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9175,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9175,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9175,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9176,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9177,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9178,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9179,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9180,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9181,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9181,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9182,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9183,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9183,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9184,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9185,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9186,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9187,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9187,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9187,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9187,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9187,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9187,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9188,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9189,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9190,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9191,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9192,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9193,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9193,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9193,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9194,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9195,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9196,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9196,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9196,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9197,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9198,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9198,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9198,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9198,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9198,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9198,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9198,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9199,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9200,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9201,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9202,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34934&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9203,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9203,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9203,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9203,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9203,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9204,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9204,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9205,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9206,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9207,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9208,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9209,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9209,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9210,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9211,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9212,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9213,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9214,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9215,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9216,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9217,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9218,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9219,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9220,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9221,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9221,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9221,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9221,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9222,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9223,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9224,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9225,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9226,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9227,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9227,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9228,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9229,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9230,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9231,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9232,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9233,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9234,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9234,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9234,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9235,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9235,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9236,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9237,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9238,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9239,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9239,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9240,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9241,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9242,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9243,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9244,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9245,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9246,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9247,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9247,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9247,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9248,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9249,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9250,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9251,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9251,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9251,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9251,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9251,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9252,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9253,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9254,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9255,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9255,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9256,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9257,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9258,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9258,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9259,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9260,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9261,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9261,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9262,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9263,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9264,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9264,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9264,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9264,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9264,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9264,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9265,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9265,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9265,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9265,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9265,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9266,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9266,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9267,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9267,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9268,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9269,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9270,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9271,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9271,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9271,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9271,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9271,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9271,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=265167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9271,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9271,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9272,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9273,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9274,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9274,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9275,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9276,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9277,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9278,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9279,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9280,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9280,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9281,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9281,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9281,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9282,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9282,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9283,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9284,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9285,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9286,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9287,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9287,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9288,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9288,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9288,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9288,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9288,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9289,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9290,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9290,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9290,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9290,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9291,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9292,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9293,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9294,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9295,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9296,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9296,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9297,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9298,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9299,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9300,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9301,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9302,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9303,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9304,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9305,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9305,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9306,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9307,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9308,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9308,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9308,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9309,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9310,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9311,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9312,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9313,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9314,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9315,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9316,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9317,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9317,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9317,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9317,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9317,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9317,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9317,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9318,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9319,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9320,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9321,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9322,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9322,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9323,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9324,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9325,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9325,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9326,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9326,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9326,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9326,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9326,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9327,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9328,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9329,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9330,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9331,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9332,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9332,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9332,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9333,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9334,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9334,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9334,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9334,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9334,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9334,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9335,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9336,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9337,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9338,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9339,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9340,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9341,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9342,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9343,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9344,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9345,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9346,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9347,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9348,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9349,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9350,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9350,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9350,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9351,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9352,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9353,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9354,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9355,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9356,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9357,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9358,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9359,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9360,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9361,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9362,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9362,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9362,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9362,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9362,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9362,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9363,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9364,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9365,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9366,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9367,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9368,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9369,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9370,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9371,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240034&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9372,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9373,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9374,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9375,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9375,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9376,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9377,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9378,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9379,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9380,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9381,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9381,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9381,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9381,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9382,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9382,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9382,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9382,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9382,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9382,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9382,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9383,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9384,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9385,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9385,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9386,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9386,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9386,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9386,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9386,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9387,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9387,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9388,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9388,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9389,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9390,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9391,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9392,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9393,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9393,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9394,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9395,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9396,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9397,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9398,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9399,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9400,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9401,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9402,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9403,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9404,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9405,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9405,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9405,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9405,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9406,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9406,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9407,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9408,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9409,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9410,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9411,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9412,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9413,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9414,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9415,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9416,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9416,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9417,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9418,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9419,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9420,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9421,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9422,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9423,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9424,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9425,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9425,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9425,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9426,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9427,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9427,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9428,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9428,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9429,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9430,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9430,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9431,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9431,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9431,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9432,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9433,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9434,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9435,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9435,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9436,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9437,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9438,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9438,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9439,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9440,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9440,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9441,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9442,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9443,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9443,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9443,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9443,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9443,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9443,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9444,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9445,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9445,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9445,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9445,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9445,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9445,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9446,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9446,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9447,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9448,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9448,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9448,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9448,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9448,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9448,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9448,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9449,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9450,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9450,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9451,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9452,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9453,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9454,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9454,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9455,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9456,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9457,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9458,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9459,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9460,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9461,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9462,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9462,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9463,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9463,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9464,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9465,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9466,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9466,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9467,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9467,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9468,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9469,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9470,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9471,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9471,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9472,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9473,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9474,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9475,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9476,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9477,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9478,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9479,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9480,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9480,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9481,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9482,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9483,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9484,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9485,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9486,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9487,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9488,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9489,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9490,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9491,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9492,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9493,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9494,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9495,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9496,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9497,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9498,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9499,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9500,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9501,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9502,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9503,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9503,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9504,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9505,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9505,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9506,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9506,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9507,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9508,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9509,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9510,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9510,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9510,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9511,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9511,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9512,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9513,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9514,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9515,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9516,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9517,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9518,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9518,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9519,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9520,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9520,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9520,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9521,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9522,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9523,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9523,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9523,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9524,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9525,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9525,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9525,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9525,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9525,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9526,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9527,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9528,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9529,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9529,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9529,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9530,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9531,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9532,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9533,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9534,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9535,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9536,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9537,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9537,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9538,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9539,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9539,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9539,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9539,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9540,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9541,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9541,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9542,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9543,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9544,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9545,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9546,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9547,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9548,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9549,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9549,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9549,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9550,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9550,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9551,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9552,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9553,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9553,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9554,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9555,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9556,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9557,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9558,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9559,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9559,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9560,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9561,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9562,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9562,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9562,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9562,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9563,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9564,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9565,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9566,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9567,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9567,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9567,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9567,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9568,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9569,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9570,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9571,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9572,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9573,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9573,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9573,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=56&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9573,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9573,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9573,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9573,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9574,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9575,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9576,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9576,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9576,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9577,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9578,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9579,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9580,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9581,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9582,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9583,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9583,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9583,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9584,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9585,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9586,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9586,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9586,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9586,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9586,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9586,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9586,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9586,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9587,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9588,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9589,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=266299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9589,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9590,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9590,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9591,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9591,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9591,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9591,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9591,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9591,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9592,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9592,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9592,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9592,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9593,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9594,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9595,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9596,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9597,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9598,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9598,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9599,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9600,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9601,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9602,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9603,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9604,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9604,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9604,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9604,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9604,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9605,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9605,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9605,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9606,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9607,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9608,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9609,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9610,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9611,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9612,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9613,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9614,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22987&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9615,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9616,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9616,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21124&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9617,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9618,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9619,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9620,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9620,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9620,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9621,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9622,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9623,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9624,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9625,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9626,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9627,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9628,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9629,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9629,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9630,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9631,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9632,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9632,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9633,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9634,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9635,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9635,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9636,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9637,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9638,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9639,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9640,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9641,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9641,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9642,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9642,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13099&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9642,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9642,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9643,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9643,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9644,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9645,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9645,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9646,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9646,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9646,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9646,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9646,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9647,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9648,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9649,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9649,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9650,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9651,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9652,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9652,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9653,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9654,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9655,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9656,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9657,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9658,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9659,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9659,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9659,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9659,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9660,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9661,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9662,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9663,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9664,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9665,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9666,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9667,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9668,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9669,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9670,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9670,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9671,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9671,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9672,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9673,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9673,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9674,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9675,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9676,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9677,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9678,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9678,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9679,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9680,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9680,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9680,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9680,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9680,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9680,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9681,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9681,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9682,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9683,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9684,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9685,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9686,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9687,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9688,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9689,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9690,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9691,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9692,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9692,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9692,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9693,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9694,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9695,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9696,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9697,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9698,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9698,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9699,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9700,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9701,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9702,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9703,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9703,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9704,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9705,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9706,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9706,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9706,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9707,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9708,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9709,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9710,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9710,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9711,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9711,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9711,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9712,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9713,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9714,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9715,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197855&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9716,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9717,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9718,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9718,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9719,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9720,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9721,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9722,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9722,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9723,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9724,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9724,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9724,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9725,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9726,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9726,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9727,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9728,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9729,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9730,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9731,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9732,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9733,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9734,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9735,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9736,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9737,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9738,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9739,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9739,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9740,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9741,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9742,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9742,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9743,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9744,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9744,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9744,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9744,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9745,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9746,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9747,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9748,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9749,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9750,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9751,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9751,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9751,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9752,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9753,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9754,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9755,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9755,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9755,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9755,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9755,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9755,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9755,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9756,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9757,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9758,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9758,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9759,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9760,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9761,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9762,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9763,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9764,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9764,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9764,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9764,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9764,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9764,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9764,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9765,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9766,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9767,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9768,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9769,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9770,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9770,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9771,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9772,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9773,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9774,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9775,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9776,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9776,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9777,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9778,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9779,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9779,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9780,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9780,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9781,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9781,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9782,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9782,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9782,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9782,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9783,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9784,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9785,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9786,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9787,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9788,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9789,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9790,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9791,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9791,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9792,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9793,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9794,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9795,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9795,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9795,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9795,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9796,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9797,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9797,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9798,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9799,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9800,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9801,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9802,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9803,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9804,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9805,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9806,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9806,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9807,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9808,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9808,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9809,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9810,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9811,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9811,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9812,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9813,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9813,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9813,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9814,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9814,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9815,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9815,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9815,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9815,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9816,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9817,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9818,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9819,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9819,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9819,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9820,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9821,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9822,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9823,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9824,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9824,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9825,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9826,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9827,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9828,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9828,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9829,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9830,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9831,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9832,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9833,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9833,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9834,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9835,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9836,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9837,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9838,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9839,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9840,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9841,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9842,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9843,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9843,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9843,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9843,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9843,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9844,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9845,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9846,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9847,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9848,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9848,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9849,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9850,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9851,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9852,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9853,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9854,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9855,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9856,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9857,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9858,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9859,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9860,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9861,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9862,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9863,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9864,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=229964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9865,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9865,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9865,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9865,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9865,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9865,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9866,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9867,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9868,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9868,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9868,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9868,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9869,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9870,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9871,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9872,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9873,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9874,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9875,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9876,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9877,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9878,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9879,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9880,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9880,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9881,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9882,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9883,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9884,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9885,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9885,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9886,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9886,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9887,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9888,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9889,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9890,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9890,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9891,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9892,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9893,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9894,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9895,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9896,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9897,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9897,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9898,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9898,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9899,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9900,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9900,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9900,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9901,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9901,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9902,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9903,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9904,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9904,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9904,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9904,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9904,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9905,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9906,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9907,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9908,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9909,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9909,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9910,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9911,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9911,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9911,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9911,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9911,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9912,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9913,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9913,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9914,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9914,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9915,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9916,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9916,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9916,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9917,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9918,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9918,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9918,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9918,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9919,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9920,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9921,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9922,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9923,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9924,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9925,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9926,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9926,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9926,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9927,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9927,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9927,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9927,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=93&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9927,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9927,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9927,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9927,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9928,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9929,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9930,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9930,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9931,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9932,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9933,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9934,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9935,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9936,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9937,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9937,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9937,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9938,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9939,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9939,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9940,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9941,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9942,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9943,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9944,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9945,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9946,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9946,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9947,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9947,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9947,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9947,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9948,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9949,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9950,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9951,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9951,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9952,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9953,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9954,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9955,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9956,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9957,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9958,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9959,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9960,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9960,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9961,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9962,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9962,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9963,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9963,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9964,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9965,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9966,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9967,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9968,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9968,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9969,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15367&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9969,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9969,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9970,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9970,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9970,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9971,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9972,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9972,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9972,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9973,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9973,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9974,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9975,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9975,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9975,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9975,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9975,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9975,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9975,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9975,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9976,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9977,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9977,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9977,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9978,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9979,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9980,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9981,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9981,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9981,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9982,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9983,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9983,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9984,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9985,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9986,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9987,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9988,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9988,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9988,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9989,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9990,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9990,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9991,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9991,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9992,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230775&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9993,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9994,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9994,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9995,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9996,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9997,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9998,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1316&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(9999,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10000,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10001,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10002,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10002,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10002,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10002,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10003,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10003,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10003,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10003,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10003,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10003,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10003,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10004,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10005,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10006,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10007,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10008,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10009,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10010,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10010,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10011,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10012,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10013,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10014,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10014,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10014,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10014,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10014,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10014,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10014,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10014,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10015,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10016,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10017,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10018,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10019,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10020,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10021,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10022,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10023,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10024,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10025,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10025,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10025,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10026,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10027,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10028,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10029,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10030,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10031,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10032,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10032,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10033,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10033,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10034,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10035,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10036,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10037,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10037,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10037,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10037,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10037,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10037,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10037,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10037,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10038,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10038,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10038,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10039,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10040,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10040,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10041,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10042,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10043,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10044,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10044,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10045,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10046,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10046,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10047,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10047,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10047,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10048,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10048,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10048,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10049,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10050,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10051,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10052,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10052,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10053,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10054,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10055,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10056,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10057,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10058,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10059,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10060,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10061,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10062,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10062,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10063,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10064,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10065,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10066,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10067,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10068,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10069,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10070,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10070,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10071,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10072,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10073,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10074,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1745&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10075,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10076,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10076,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10076,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10076,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10077,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10078,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10079,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10079,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10080,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10081,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213790&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10082,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10082,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10083,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10084,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10084,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10085,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10086,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10087,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10088,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10089,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10090,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10090,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10090,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10091,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10092,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10092,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10092,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10092,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10093,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10093,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10093,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10094,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10095,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10096,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10097,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10098,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10099,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10099,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10100,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10100,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10101,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10102,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10102,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10102,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10102,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10103,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10104,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10104,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10104,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10105,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10105,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10106,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10107,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10108,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10109,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10110,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10111,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10111,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10112,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10113,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10114,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10114,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10114,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=94912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10115,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10116,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10116,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10117,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10118,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10119,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10120,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10121,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10122,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10123,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10123,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10124,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10125,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10125,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10125,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10125,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10126,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10127,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10128,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10128,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10128,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10129,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10130,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10131,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10132,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10132,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10133,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10134,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10135,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10135,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10136,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10137,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10138,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10139,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10140,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10140,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10141,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10142,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10143,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88811&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10144,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10144,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10145,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10146,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10147,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10148,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10149,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10150,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10151,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10152,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10153,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10154,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10155,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10156,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10156,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10156,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10157,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10158,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10159,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10160,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10160,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10161,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10162,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=131007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10163,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10164,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10165,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10166,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10167,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10168,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10169,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10169,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10170,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10171,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10172,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10172,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10172,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10172,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10173,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10174,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10175,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10176,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10177,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10178,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10179,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10179,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10179,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10180,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10180,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10181,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10181,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10181,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10182,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10183,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10184,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10185,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10186,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10187,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10188,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10189,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10190,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10191,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10192,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10193,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10194,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10195,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10196,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10196,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10196,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10196,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10197,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10197,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10197,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10198,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10199,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10200,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10200,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10200,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10201,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10202,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10203,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10204,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10204,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10204,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10204,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10205,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10206,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10207,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10207,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10207,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10207,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10208,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5738&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10209,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10210,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10210,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10211,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10212,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10213,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10214,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10214,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10214,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10214,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10215,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10215,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10216,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10217,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10218,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10219,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10220,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10221,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10222,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10222,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10223,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10224,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10225,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10225,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10226,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10226,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10227,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10228,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10228,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=269632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10229,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10230,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10231,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10232,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10233,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212607&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10234,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10234,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10235,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10235,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10236,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10237,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10238,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10239,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10239,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10240,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10241,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10242,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10243,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10244,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10245,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10246,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10247,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10248,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10248,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10249,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10249,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10249,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10250,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10251,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10252,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10253,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10254,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10254,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10254,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10255,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10256,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10257,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10258,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10259,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10260,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10261,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10262,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10263,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10264,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10265,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10266,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197023&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10266,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10266,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10266,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10266,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10266,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10266,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10266,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10267,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10268,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10269,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10270,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10271,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10271,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10272,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10273,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10274,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10275,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10276,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10277,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12329&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10278,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10278,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10279,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10280,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10280,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10281,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10282,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10283,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10283,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10283,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10283,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10284,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10284,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10285,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10286,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10286,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10287,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10288,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10288,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10288,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10289,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10290,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10291,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10292,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=246950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10293,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10294,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10295,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10295,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10296,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10297,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10298,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10299,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10300,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10301,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10302,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=524&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10303,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10304,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10305,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10306,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10307,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10308,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10308,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10309,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10310,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10310,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10310,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10310,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10310,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10311,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10312,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10312,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10312,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10312,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10312,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10312,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10313,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10314,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10315,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10315,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10315,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10315,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10315,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10316,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10317,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10318,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10318,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10319,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10319,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10320,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10320,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10320,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10320,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10320,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10321,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10322,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10322,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10323,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10324,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10325,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10326,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10327,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10328,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10328,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10328,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10328,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184665&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10329,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10330,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10331,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10332,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10333,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10334,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10334,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10335,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10336,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10337,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10338,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10339,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10339,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10340,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=137913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10341,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10342,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10343,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10344,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10345,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10346,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10347,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10348,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10349,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10350,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10351,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10352,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10353,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10353,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10354,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10355,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10356,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10357,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10358,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10359,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10360,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10360,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10360,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10361,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10362,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10363,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10364,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10365,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10365,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10366,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10367,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10367,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10368,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10369,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10370,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10370,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10371,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10372,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10373,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10374,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10375,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=55&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10375,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10375,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10375,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10376,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10376,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10377,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10378,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10379,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10380,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10381,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10381,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10381,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10382,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10383,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10384,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113567&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10385,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10385,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10385,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10386,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10387,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10388,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10389,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10390,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10391,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10392,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10393,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10394,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10395,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10396,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10397,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10398,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10399,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10399,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10399,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10399,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10400,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10400,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10401,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10402,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10403,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10403,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10404,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10405,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10405,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10405,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10405,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10405,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10406,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10407,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10407,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10407,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10408,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10409,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10409,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10409,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10409,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10409,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10409,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10409,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10410,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10411,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10412,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10412,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10413,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129594&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10414,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10415,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10415,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10415,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10416,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10416,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10416,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3894&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10416,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10416,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204984&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10416,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10417,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10418,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10419,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10420,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10421,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10422,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10422,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10422,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10423,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10424,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10425,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10426,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10426,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10427,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10427,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10427,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10427,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10428,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10429,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10429,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10430,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10430,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10430,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25499&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10430,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10431,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10431,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10432,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1694&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10432,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10433,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10433,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10434,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10434,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10434,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10434,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10434,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10435,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10436,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10437,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10437,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10437,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=63&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10437,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=358&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10437,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10437,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10437,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10437,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10438,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10438,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10439,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10440,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10441,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10442,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10443,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10443,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10443,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10443,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10443,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10443,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10444,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10444,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10444,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10444,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10445,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10446,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10447,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10447,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10447,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10447,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10448,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10448,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10449,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10450,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10451,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10452,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10452,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10453,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10453,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10453,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=66&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10453,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10453,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10453,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10453,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10454,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10454,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10455,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10456,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10457,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10458,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10459,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10460,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10461,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10461,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10462,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10462,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10462,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10463,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10464,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10464,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10464,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3678&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10465,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10465,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10465,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10465,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10465,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10466,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10467,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10468,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10469,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10470,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10471,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10471,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10472,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10473,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10474,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10474,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10475,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10476,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10477,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10477,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10478,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10479,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10479,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10479,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10480,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10481,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10482,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10483,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10484,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10485,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10486,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10487,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10488,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10488,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10489,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10490,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10491,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10491,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10491,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10491,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10491,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10492,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10493,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10494,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10494,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10494,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10495,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10496,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10496,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10497,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10498,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10499,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10500,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10501,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10502,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10503,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10504,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10505,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10506,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10506,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10507,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10508,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10509,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10510,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=7247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10511,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10511,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10511,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10512,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10512,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10513,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10514,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10514,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10515,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10516,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10517,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10518,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10519,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10520,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10520,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=137927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10521,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10522,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10523,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10523,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10524,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10525,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10525,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194967&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10525,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10525,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10525,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10525,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10525,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10526,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10526,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10526,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10527,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10528,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10529,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=138571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10529,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10530,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10531,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10532,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10533,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10533,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10533,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10534,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10535,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10535,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10536,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10537,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10538,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10539,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10540,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10541,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10542,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10543,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10544,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10545,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10545,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201167&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10546,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10547,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10548,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10549,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10549,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10549,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10550,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10550,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10551,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10552,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107363&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10553,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10554,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10555,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10556,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10557,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10558,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10558,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10559,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10560,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10561,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10562,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10562,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10563,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10564,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10565,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10566,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10567,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10568,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10569,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10570,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10571,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10572,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10573,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10574,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10574,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10575,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10576,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10576,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10576,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10577,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10578,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10579,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10580,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10580,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10580,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10581,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10582,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10582,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10582,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10582,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10583,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10584,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10585,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10586,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10587,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10588,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10589,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10589,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10589,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10590,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10591,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10591,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10592,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1969&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10593,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10593,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10594,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10595,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10596,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10597,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10598,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10599,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10600,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10601,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10602,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10603,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10604,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10604,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10604,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10605,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10605,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10605,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10605,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10606,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10607,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10607,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10607,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10607,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10608,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10608,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10608,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10609,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10610,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10610,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10611,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19781&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10611,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10611,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129477&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10612,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10613,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10613,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10614,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10615,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10616,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10617,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10618,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10619,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10620,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10620,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10621,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10622,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10623,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10623,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10624,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10625,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10626,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10627,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10627,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10627,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10627,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10627,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10627,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10627,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10627,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10628,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10629,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10630,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10630,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10630,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10630,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10630,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10631,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10632,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10633,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10634,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10635,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10636,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10636,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10637,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10638,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10639,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10640,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10640,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10641,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10642,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10643,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10643,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10643,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10643,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10643,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10643,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82989&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10643,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10643,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10644,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10645,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10646,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10647,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10647,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10648,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10649,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10649,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10649,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10649,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10649,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10649,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10649,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10650,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10651,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10652,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10653,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10653,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10654,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10655,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10656,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10657,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10658,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10658,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10658,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10658,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10659,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10660,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=954&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10660,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10660,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10660,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10661,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10662,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10663,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10663,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10663,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10664,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10665,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10666,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10667,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10668,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10668,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10669,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10670,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10671,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10672,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10673,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10674,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10674,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10675,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10675,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10675,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10676,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10677,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10678,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10678,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10679,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10680,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10681,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10682,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10683,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10684,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10685,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10686,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10687,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10688,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=254133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10689,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10690,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10691,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10692,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10692,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10693,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10694,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10695,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10696,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10697,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10698,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10699,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10700,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10701,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10702,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10703,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10704,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10705,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10706,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10707,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10708,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10709,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10709,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10709,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10709,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10709,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10710,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10710,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10710,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10711,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10711,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10711,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10712,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10712,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10713,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10714,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10714,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10714,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10714,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10715,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10716,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10717,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10718,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10719,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10720,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26802&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10721,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10721,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10721,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10721,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10721,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10722,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10723,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10723,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10723,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10723,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10724,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10724,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10724,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10724,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10725,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10726,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10727,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10727,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10728,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10728,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10728,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10728,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10728,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10729,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10730,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10731,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10732,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10733,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10734,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10734,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10734,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10735,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10736,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10737,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10738,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10739,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10740,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10740,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10741,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10742,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10742,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10743,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10744,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10745,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10746,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10747,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10747,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25459&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10748,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10749,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10750,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10750,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10751,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10752,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10753,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10754,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10755,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10755,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10756,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10757,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10757,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10758,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10759,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10760,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10761,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10762,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10763,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10764,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10765,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10765,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10765,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10766,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10766,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10767,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10768,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10769,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10770,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10771,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10771,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10772,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10772,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10772,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10773,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10774,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10775,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10776,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10776,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10777,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48077&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10778,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10779,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189875&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10779,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10780,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10781,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10781,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10781,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10782,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10783,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10783,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10783,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10784,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10785,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10786,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10787,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10788,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10789,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10790,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10791,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10792,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10793,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10794,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10794,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10795,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10796,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10797,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10798,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10798,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10799,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10800,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10801,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10802,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10803,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10804,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10804,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10805,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10806,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10807,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12943&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10808,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10809,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10810,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10810,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10811,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10812,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10812,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10813,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10814,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10815,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10816,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10817,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10817,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10818,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10818,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10818,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10818,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10819,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10820,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10821,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10822,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10823,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10823,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10824,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209156&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10824,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10825,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10826,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10827,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10828,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10829,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10830,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10830,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10831,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10832,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10833,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10833,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10833,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10834,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10835,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10836,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10837,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10838,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19666&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10838,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10839,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10840,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10840,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10841,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10842,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10843,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10843,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10843,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10843,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10843,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159841&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10843,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10843,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10844,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10845,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=77920&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10846,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10847,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10848,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10849,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10849,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10850,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10851,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10852,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10853,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3318&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10854,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10855,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10856,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10857,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10857,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10857,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10858,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10858,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10859,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10860,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10861,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10862,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10863,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10864,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10865,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10866,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10867,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10868,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10868,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10869,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8862&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10870,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10870,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10871,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15783&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10872,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10872,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10873,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10874,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10874,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10875,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10876,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10876,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10877,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10878,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10878,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10879,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2927&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10880,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=169975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10881,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10882,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10883,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10884,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10885,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10886,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10886,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10887,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10888,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10889,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10890,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10891,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10892,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10893,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10893,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10893,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10894,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10895,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10896,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10897,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10898,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46721&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10899,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10899,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=82955&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10900,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10901,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10902,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10903,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10903,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10904,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10905,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10906,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10906,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10907,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10907,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10908,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10909,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10909,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10910,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10911,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10912,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10913,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10914,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10915,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10916,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10916,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10917,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10918,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10919,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10920,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10921,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10922,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10923,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10924,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10924,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14773&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10924,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10924,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10924,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10924,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10925,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10926,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=195628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10927,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10928,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10929,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10930,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10931,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208284&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10932,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10933,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10934,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10935,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10936,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10937,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10938,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10939,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10940,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10940,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10940,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10940,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10940,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10940,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10940,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10941,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10942,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10943,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10944,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10944,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10945,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10946,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3330&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10947,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10947,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10948,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10949,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10950,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10951,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3002&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10951,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10952,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10953,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10954,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10955,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10955,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10956,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10956,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10957,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10958,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10959,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10960,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10961,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10961,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184710&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10962,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10963,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10964,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10964,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184555&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10965,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10966,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10966,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4517&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10967,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22937&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10968,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10969,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10970,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10970,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10971,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10972,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262698&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10973,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10973,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10974,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10975,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10976,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10977,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10977,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108856&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10978,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218018&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10979,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34763&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10980,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10981,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10982,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10983,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87965&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10984,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10985,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10985,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10986,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=131006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10987,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10988,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10989,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10990,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10990,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10991,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10991,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10992,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10992,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10993,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10994,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10995,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10996,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10997,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10997,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10998,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10998,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10998,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10999,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(10999,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11000,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11000,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11001,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262699&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11002,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11002,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11003,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11003,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11004,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11005,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11006,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11007,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11008,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11009,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11010,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11011,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11011,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11011,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11011,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11012,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11012,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11013,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11014,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96893&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11015,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11016,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11017,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11018,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11019,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11020,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11020,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11021,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11022,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11023,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4869&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11024,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=842&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11025,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11026,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11027,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11027,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11028,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11029,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11029,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11030,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=278260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11031,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11031,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11032,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11033,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11033,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11033,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11034,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11035,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11036,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11036,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11037,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11038,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11039,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11040,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11041,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11042,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121188&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11043,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11044,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11045,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11046,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11047,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11047,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11048,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11049,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11050,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11051,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11052,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11053,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11054,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11055,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11056,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11056,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11056,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11057,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11058,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11059,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11059,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11059,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11059,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11059,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11060,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5144&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11061,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11061,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247159&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11062,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11063,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11064,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11065,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11066,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11067,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22948&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11068,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11069,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11070,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11071,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11071,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11071,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11071,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11071,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11072,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11073,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11074,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11075,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11076,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11077,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11078,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11079,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11080,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11081,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11082,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11082,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11083,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11084,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11085,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11086,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11087,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11088,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11088,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11088,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11088,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11089,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11090,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11091,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11091,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11091,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11092,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11093,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11094,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11095,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11096,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11096,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11097,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11098,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11099,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11100,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11101,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11101,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11102,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11103,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11104,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11104,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11105,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11106,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11106,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11106,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11106,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11107,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11108,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11108,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83599&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11109,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11110,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11111,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11112,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11113,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11114,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11115,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11116,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11116,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11117,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11118,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11119,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11120,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11121,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11122,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11122,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11122,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209040&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11123,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11123,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11123,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201208&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11124,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11125,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11125,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11126,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11127,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11127,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11128,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11129,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11130,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11130,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11131,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11131,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11131,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11131,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11131,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11132,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11133,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11133,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11134,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6602&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11134,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11134,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11135,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191359&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11136,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11136,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11137,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11138,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11139,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11140,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11141,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11142,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11143,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11144,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11144,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11144,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11144,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11145,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11146,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11147,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11147,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11148,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11149,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11150,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11151,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11152,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83186&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11153,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11154,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11155,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11155,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11155,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11155,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11155,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11155,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11155,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=748&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11156,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198085&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11157,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11158,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11159,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11160,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11160,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11160,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11160,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11161,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11161,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11161,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11162,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11163,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11163,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11163,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11164,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11165,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11166,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201312&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11166,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11167,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11168,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205421&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11168,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11169,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11170,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11171,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201252&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11171,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11172,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11173,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11174,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11175,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11176,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11177,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11178,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11179,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11180,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11181,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11182,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16622&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11182,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11183,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11183,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11184,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11184,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11184,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11184,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11184,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11184,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11184,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11184,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129744&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11185,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11186,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39644&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11187,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11187,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11188,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11189,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11190,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11191,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11191,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11192,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11192,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11193,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11194,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11195,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11196,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11197,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11197,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3974&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11197,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11197,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11197,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11197,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11198,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11199,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11200,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11201,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11202,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11203,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11204,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11204,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11204,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3707&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11205,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11205,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11205,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11205,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11205,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11206,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11207,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11208,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11209,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11210,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11211,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11212,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11213,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11214,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11214,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11215,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11216,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11217,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242539&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11218,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11219,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11220,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14604&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11220,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11220,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11221,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11222,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11223,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11224,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193658&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11225,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11225,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11226,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11227,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11228,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11228,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11229,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11230,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5820&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11231,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11232,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11233,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11234,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11234,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11234,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11235,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11236,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11237,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11237,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11238,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11239,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11239,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11239,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11240,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11241,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11242,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11242,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11242,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11242,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11242,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11243,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11243,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11244,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11245,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11246,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11247,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11248,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11249,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11250,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11251,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11251,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4320&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11252,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11252,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11252,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11252,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11252,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11252,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13172&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11252,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11253,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11478&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25533&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11254,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11255,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11256,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11256,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11257,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107373&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11258,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11259,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11260,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205485&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11261,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11261,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11261,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11262,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11263,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11263,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11264,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5850&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11265,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240101&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11266,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11267,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11268,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11269,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11270,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11271,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11272,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11273,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11274,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11275,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11276,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11277,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11277,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11278,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11279,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11280,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11281,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11282,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11282,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=169963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11283,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11284,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11285,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11285,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11286,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11287,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270460&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11287,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11288,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11289,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11290,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11291,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74647&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11292,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11293,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=279615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11294,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11295,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11296,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11297,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11298,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11299,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11300,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11301,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11302,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11303,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11304,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11305,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11306,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11307,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11308,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11309,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11310,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11310,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11311,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11311,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11311,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11312,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11312,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11313,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11314,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11314,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11314,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11315,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11316,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11317,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11318,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11319,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11320,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11321,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11322,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11323,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11324,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11325,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11326,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11327,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11328,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11328,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11329,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11330,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11331,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74436&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11332,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11333,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4677&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11334,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11335,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11335,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11335,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11335,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11336,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11337,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11337,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11338,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11339,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11339,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11340,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88973&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11341,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11342,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11343,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262685&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11344,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11344,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11345,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11346,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1013&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11346,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11346,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11347,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5200&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11348,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11349,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11350,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11351,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11352,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11352,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11353,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11354,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11355,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11356,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=242532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11357,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1700&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11357,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11358,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11359,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11360,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=283&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11360,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11360,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11360,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11360,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11360,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=882&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11361,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11361,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4339&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11361,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11362,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11363,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11363,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11364,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11365,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11365,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2824&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11365,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11366,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11367,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11368,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11368,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11369,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11370,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11370,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11370,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11370,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11370,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11371,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11372,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11373,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11374,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11375,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11376,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228255&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11377,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11377,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11378,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11379,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11380,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11380,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11381,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11382,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73961&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11383,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11384,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11385,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11385,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11386,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11387,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11388,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170993&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11389,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11390,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11391,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11391,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11391,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11392,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11393,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11394,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11394,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11394,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11394,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11395,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5768&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11396,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11397,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11398,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11399,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107427&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11400,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=791&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11401,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259274&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11401,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23192&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11402,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11403,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11403,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11404,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11405,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11406,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11407,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11408,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11409,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11410,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11410,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11410,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2314&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11410,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11410,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11410,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11410,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=827&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11411,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11412,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11412,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11413,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23069&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11414,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11415,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11416,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11416,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11416,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=603&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11417,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11418,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11419,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11419,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11420,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11420,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11421,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11421,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11421,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11422,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11423,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220453&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11423,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11423,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11424,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11424,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11424,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11424,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11424,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205355&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11425,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11426,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145814&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11427,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190535&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11427,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205088&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11427,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11427,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11428,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11428,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11428,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11428,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11429,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11430,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11431,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11432,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11433,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11434,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11435,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11436,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36116&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11437,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11437,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11437,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11438,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11438,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11439,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240025&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11440,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11440,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11440,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3675&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11440,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197405&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11441,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11442,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11443,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11444,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11444,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11445,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11446,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11446,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11447,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11448,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11449,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11450,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11451,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11451,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184683&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11452,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11453,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11454,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11454,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11455,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142356&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11456,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11457,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26795&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11458,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11459,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11460,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11460,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11460,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11461,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11461,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3786&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11461,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11461,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11461,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11461,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11461,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11461,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11462,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2522&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11463,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11464,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11465,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11465,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11465,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11466,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1690&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11466,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11467,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11468,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11469,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11470,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158111&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11470,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11471,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11472,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11472,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11473,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233043&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11474,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11474,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11475,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11476,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=235597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11477,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11478,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11478,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11478,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11478,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202424&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11478,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11478,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11479,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189244&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11479,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11480,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11481,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11482,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139419&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11483,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11484,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11485,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45958&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11486,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11487,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11488,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11489,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192210&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11490,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11491,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11491,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11492,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11493,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11493,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11493,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=470&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11493,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11493,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11494,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193512&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11495,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11496,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11497,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11497,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11498,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121226&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11499,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11500,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11501,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11502,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11503,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11504,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11504,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11505,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4560&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11506,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29751&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11507,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11508,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11508,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11509,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11509,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11509,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11510,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201826&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11510,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11511,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152682&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11512,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11513,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11514,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11515,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121185&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11516,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11516,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11517,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11518,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11519,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11520,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3468&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11521,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11521,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11522,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11523,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176428&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11524,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11525,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11526,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3643&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11527,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11528,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11529,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11530,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11531,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11532,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11533,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11534,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11534,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11535,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11536,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11537,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11538,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11539,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11540,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11541,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11542,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11543,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11544,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11544,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11544,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3819&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11544,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11544,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11564&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11545,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143016&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11546,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11547,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11548,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11548,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11548,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11548,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11548,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11549,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11550,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11551,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11552,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11553,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11553,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11554,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11555,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11556,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11556,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11556,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11557,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=276194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11558,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11559,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11559,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11559,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11560,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11561,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11562,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1042&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11563,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11564,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29778&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11565,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11566,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184758&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11566,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11567,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2212&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11568,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11569,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11570,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79191&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11571,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11572,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11573,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11574,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11575,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11576,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=147409&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11577,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=116381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11578,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11579,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11580,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3259&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11581,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11582,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22302&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11583,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11584,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11584,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11585,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11586,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11586,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12383&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11587,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11588,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11589,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11589,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11590,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11591,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11592,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11593,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11594,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11595,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11596,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11597,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11598,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11599,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262839&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11600,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11601,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11602,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11602,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83838&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11603,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11604,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11604,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11604,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11605,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11606,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11607,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11607,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11607,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11608,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11609,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230762&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11609,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11610,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11611,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11612,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11613,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25933&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11614,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11615,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11616,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11617,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11618,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11619,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11619,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11619,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11619,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11619,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11619,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11620,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2760&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11621,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11621,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11622,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11622,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11622,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11623,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11624,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11625,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11626,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11627,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11627,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11628,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11629,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11630,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11631,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11631,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11631,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11631,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11632,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11633,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11634,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4583&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11635,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11636,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11636,(SELECT setid FROM sets WHERE sets.name = 'FVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11636,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247201&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11637,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11638,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11639,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11640,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11641,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11641,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207890&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11641,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11642,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11643,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11644,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11645,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11646,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11647,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11648,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11648,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11649,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11650,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11650,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11650,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11650,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11650,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11651,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11652,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11653,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11654,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11655,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11655,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11656,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11657,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11658,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11659,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11659,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132071&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11660,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11661,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11662,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11662,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11663,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11664,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8437&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11665,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11666,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11667,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11668,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122370&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11669,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74475&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11670,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11671,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11672,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11673,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11674,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11675,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11676,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11677,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11678,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11679,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11680,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11681,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11682,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11683,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11683,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1341&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11684,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11685,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11686,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11687,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11688,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96901&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11688,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11689,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11690,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11691,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194349&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11692,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11692,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11693,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11694,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11695,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11696,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11696,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11696,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11696,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201125&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11697,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11698,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11699,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11700,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15877&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11701,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11702,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50509&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11703,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24118&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11704,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19865&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11705,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11706,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11707,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29962&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11708,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45404&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11708,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11709,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11710,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11711,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11712,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45849&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11713,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11714,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11715,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11716,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11717,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11718,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19867&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11719,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11720,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11720,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153996&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11721,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11722,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11723,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212593&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11724,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11724,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11725,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11726,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11727,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11728,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11728,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11729,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11729,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159232&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11730,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11731,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11732,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11732,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11733,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11734,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11734,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11734,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11734,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11734,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11735,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11736,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11737,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11738,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11739,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11740,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11741,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11742,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11743,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11744,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11745,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11746,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11747,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11748,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11749,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11750,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39432&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11751,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11751,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159742&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11752,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11752,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4687&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11753,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11754,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11755,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11756,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24576&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11757,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5154&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11758,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11759,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11760,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25918&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11761,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11762,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11763,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11764,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11764,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11764,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106639&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11765,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11766,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2032&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11766,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11766,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11766,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11766,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11767,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11768,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11768,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11769,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11770,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11771,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14727&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11771,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11771,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11771,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11771,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11771,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11772,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11773,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11774,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11775,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5723&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11776,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11777,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3909&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11777,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11777,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11777,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11778,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11779,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11780,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11780,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11781,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11782,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1722&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11783,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11783,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11783,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11784,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11785,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36411&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11786,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29911&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11787,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29950&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11788,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11789,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11790,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151142&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11791,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11792,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11792,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11793,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11793,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11794,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132220&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11795,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=125866&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11796,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11797,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11797,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11798,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11799,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11800,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244731&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11801,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11801,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176447&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11802,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11802,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11803,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29983&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11804,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6103&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11805,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11806,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11807,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11808,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11809,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11810,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11811,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11812,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2479&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11813,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11814,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139705&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11815,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175035&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11816,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50413&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11817,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11818,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11818,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11818,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11819,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217988&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11240&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2039&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15868&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11820,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=610&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11821,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11822,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11823,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201794&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11823,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19595&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11824,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11825,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11825,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11826,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11826,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11826,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11826,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11827,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11828,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11829,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11830,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11831,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11831,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11832,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11833,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11834,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11835,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=204980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11836,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159051&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11836,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11836,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11837,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11838,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11839,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185696&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11840,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11841,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96874&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11842,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11843,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11844,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11845,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11846,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8899&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11847,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11847,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109674&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11848,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11849,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11850,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11850,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11850,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11850,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11850,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129779&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11851,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11851,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45978&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11851,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11852,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11852,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11853,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11854,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11855,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11856,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11857,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15799&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11858,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11859,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220152&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11860,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89115&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11861,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11862,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11863,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11864,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11865,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11865,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11866,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11867,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11868,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5187&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11869,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88982&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11870,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11871,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11872,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11872,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4030&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11872,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11872,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46106&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11872,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1280&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11873,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11874,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11875,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8907&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11876,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11877,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185829&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11878,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83036&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11878,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11878,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11879,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11880,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11881,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1067&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11881,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11881,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11881,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11882,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11882,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11883,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11883,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11884,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11884,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11884,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130528&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11885,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11886,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11887,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11888,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11889,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1503&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11890,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11891,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11892,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11893,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11894,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11894,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11894,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11895,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3652&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11896,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11896,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11896,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11896,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11896,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11897,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83746&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11898,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11898,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11898,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11899,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152616&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11900,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11901,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50176&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11902,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11902,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11902,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134755&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11903,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152089&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11904,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11905,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11906,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11907,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11908,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5848&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11909,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11909,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11909,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11909,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11910,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49062&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11911,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11911,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11912,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=271120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11913,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11914,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4636&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11915,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11916,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230777&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11917,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11918,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11919,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11920,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2547&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11921,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1065&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11921,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11922,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=914&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11922,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11922,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159123&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11923,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11924,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11925,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=228959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11926,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11927,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11928,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22970&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11929,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11930,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11931,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220504&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11931,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96859&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11931,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11932,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11933,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11933,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140190&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11934,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11935,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11935,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11935,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11935,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11935,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19806&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11935,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11936,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11937,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11938,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11938,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11938,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11938,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11939,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11940,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11941,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11941,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11941,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11942,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11943,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11944,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11945,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11945,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11946,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11947,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153163&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11948,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11949,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11950,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39410&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11951,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12440&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11952,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11952,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13204&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11952,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11953,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22953&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11954,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=279612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11955,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11956,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243448&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11956,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11956,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83435&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11956,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11956,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129630&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11957,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11958,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142038&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11959,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11959,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11960,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158753&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11961,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11961,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11961,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5759&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11962,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11962,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11963,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11964,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11965,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47223&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11966,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11967,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11968,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=171001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11969,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11969,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11970,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174963&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11971,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11972,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11973,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11974,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11975,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11976,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75269&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11977,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47785&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11977,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11977,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4263&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11977,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30170&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11977,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11977,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11978,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11978,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11978,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4371&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11978,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6514&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11978,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12825&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11978,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11979,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11980,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194217&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11981,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11981,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11982,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11982,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11983,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11983,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10553&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11984,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11985,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11985,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11986,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124474&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11987,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249385&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11987,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=137905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11988,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11989,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11990,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11990,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11990,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11990,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11990,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11991,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11992,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11993,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11993,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11993,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205348&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11994,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11995,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11995,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45300&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11995,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11996,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11997,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11997,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11998,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11998,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(11999,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12000,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12001,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12002,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12002,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12002,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184585&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12003,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12004,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193505&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12005,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88972&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12006,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12007,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12008,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1640&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12009,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12010,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12010,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12010,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12010,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12010,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12010,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1351&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12010,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=854&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12011,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49094&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12012,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12013,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249377&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12013,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197873&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12014,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12015,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12016,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23171&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12017,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12017,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184691&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12017,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12018,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12019,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12020,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12021,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12022,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157923&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12022,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12023,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12024,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12025,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12025,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12026,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227090&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12027,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1835&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12027,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12028,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12028,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12029,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2612&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12030,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12031,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96860&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12032,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26645&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83261&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12033,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129732&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12034,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12035,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139463&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12035,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12036,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12036,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12036,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12036,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12036,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12036,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129750&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12037,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12038,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3701&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12039,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12039,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12040,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12041,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247354&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12041,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12042,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12043,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12044,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12045,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12045,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12045,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12045,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1254&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12045,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12046,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12047,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12047,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12047,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29946&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12047,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12048,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=254134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12049,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12050,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12050,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12051,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12052,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74412&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12053,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175104&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12054,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=76638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12055,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12055,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12055,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12056,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240095&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12057,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12057,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12058,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12059,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154005&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12060,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12061,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153301&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12062,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1615&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12063,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12064,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12065,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12066,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12067,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3391&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12068,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12068,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12068,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184741&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12069,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2926&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12070,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24632&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12071,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12071,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184625&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12072,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12072,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12072,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12073,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12074,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12075,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12075,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12075,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12075,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12076,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12077,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29291&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12078,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12079,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12080,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201306&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12080,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12081,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12081,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12081,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247178&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12082,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126420&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12083,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50537&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12084,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12085,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=81977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12086,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150847&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12087,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4253&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12088,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12089,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12089,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25591&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12089,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3676&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12090,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12091,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12092,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12093,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12094,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12095,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12095,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39515&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12096,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12096,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4624&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12097,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175122&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12098,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201843&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12099,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220483&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12099,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189227&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12099,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12099,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247141&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12099,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130386&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12100,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193577&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12101,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12102,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35056&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12103,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135279&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12103,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118891&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12104,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87952&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12105,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12106,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12107,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12108,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12109,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12110,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174951&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12111,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=956&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12111,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12112,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5656&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12113,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12113,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140205&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12113,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12113,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12113,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247323&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12114,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12114,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=7817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12115,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12115,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135197&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12116,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109711&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12117,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=142003&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12118,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12119,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74458&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12120,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194919&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12121,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12121,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12121,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205369&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12122,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12123,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12124,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189917&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12124,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207338&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12125,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12126,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6140&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12127,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122438&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12128,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12128,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12129,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5863&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14569&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = 'DDD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45311&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3840&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2100&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=62&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1157&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12130,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12131,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12132,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39600&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12133,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12134,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12135,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6087&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12136,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12137,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6511&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12137,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12138,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262693&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12139,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12140,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205027&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12140,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12141,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=192209&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12142,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12143,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5148&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12144,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12145,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12146,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6546&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12146,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20182&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12146,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12147,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141992&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12148,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12148,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1597&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12149,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12149,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205285&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12149,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194168&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12150,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12151,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12152,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2304&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12152,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1800&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12153,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12154,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34782&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12155,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12156,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43733&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12157,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12158,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12159,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21120&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12159,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12159,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12159,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25498&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12159,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4861&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12160,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12161,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12162,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12162,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12163,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12164,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12165,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21296&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12166,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241979&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12167,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12168,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130689&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12169,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2635&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12169,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159237&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12170,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12171,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12172,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12173,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3033&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12174,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=143015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12175,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=236471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12176,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12176,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35053&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12177,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12178,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213728&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12179,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12180,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12181,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213808&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12182,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12183,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12184,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12185,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12186,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12187,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44403&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12188,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118902&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12189,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12189,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12190,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87896&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12191,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12192,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12193,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12193,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12194,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12195,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12196,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2317&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12196,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1601&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12196,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12197,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179513&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12198,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12199,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12200,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97203&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12201,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12201,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12202,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194708&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12203,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214382&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12204,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3434&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12205,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12206,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12207,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88813&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12208,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12209,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39464&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12209,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12210,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4633&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12211,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220554&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12211,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12211,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12212,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139730&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12213,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12213,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12213,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12213,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12213,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12214,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152015&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12215,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159029&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12215,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40174&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12216,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12217,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12218,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152649&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12219,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2999&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12220,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262858&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12221,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74480&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12222,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240149&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12223,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6620&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12224,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12225,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12225,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1444&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12225,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84590&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12226,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12227,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=78976&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12228,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193628&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12229,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12230,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12230,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12231,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12232,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=108804&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12233,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12234,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12234,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12234,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129792&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12235,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=241985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12235,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12236,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12237,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12237,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12238,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=114905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12239,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12240,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12241,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12242,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3063&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12242,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15402&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12242,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12243,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15235&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12244,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12245,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12246,(SELECT setid FROM sets WHERE sets.name = 'DRB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12246,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247401&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12246,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=42024&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12247,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221898&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12248,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174957&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12249,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12249,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12250,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190400&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12251,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19831&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12252,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198096&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12253,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153469&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12254,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=275713&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12255,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151132&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12256,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12257,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12257,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12257,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12257,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12257,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12257,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=844&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12258,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12259,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12260,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201407&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12260,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10572&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12261,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12262,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12262,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12263,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12264,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43506&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12265,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4596&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12266,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220462&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12266,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174925&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12267,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12268,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20179&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12268,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12268,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2461&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12268,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184641&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12269,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12270,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12270,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12271,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12272,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12273,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3396&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12274,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12274,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48931&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12275,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1916&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12275,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184771&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12276,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2870&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12276,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1651&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12276,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159298&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12277,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12278,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12279,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247376&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12279,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87929&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12280,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220268&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12281,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12282,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5818&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12283,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240008&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12284,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89066&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12285,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3335&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12286,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174853&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12287,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=122114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12288,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193531&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12289,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12289,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12289,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11361&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12289,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129884&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12290,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205054&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12291,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12292,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12293,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12293,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25536&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12293,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12294,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19667&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12295,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19580&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12296,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45281&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12296,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12296,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12296,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5147&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12296,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129541&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12297,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136150&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12298,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205031&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12299,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49097&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12300,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=35166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12300,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12301,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152938&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12302,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220482&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12302,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51611&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12302,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205418&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12302,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12303,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10495&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12304,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6573&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2456&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=67&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12305,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12306,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12306,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12306,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13169&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12306,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12306,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12307,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2704&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12308,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107680&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12309,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12310,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12311,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12311,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12312,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23225&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12313,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136155&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12314,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26275&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12315,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2739&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12315,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184672&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12316,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12317,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12318,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12319,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152092&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12320,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2551&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12321,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25618&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12322,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221559&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12323,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12324,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15357&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12324,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12325,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3664&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12326,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12327,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136048&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12328,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205050&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12329,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12330,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6080&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12331,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190160&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12331,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12332,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12333,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11489&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12333,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12333,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12333,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106638&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12334,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12335,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=33717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12335,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12336,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12337,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12338,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12339,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12340,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193492&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12341,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12342,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=215079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12343,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22945&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12344,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12345,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45308&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=346&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190165&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4211&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11181&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20374&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129491&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12346,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12347,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12348,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12349,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193582&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12350,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198078&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12351,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12352,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174886&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12353,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46134&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12354,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12355,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12355,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12356,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12357,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210233&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12358,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12359,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12360,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243257&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12361,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=196995&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12361,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45288&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12361,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12361,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129490&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12362,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12363,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139465&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12363,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205398&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12364,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46059&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12365,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12366,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12367,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51161&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12367,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12368,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180613&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12369,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12370,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25592&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12370,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12371,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12371,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83646&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12371,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130532&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12372,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83556&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12373,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12374,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12375,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15397&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12375,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12376,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45264&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12376,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12376,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5670&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12377,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25980&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12378,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12379,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=89102&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12380,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12381,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12382,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12382,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205337&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12383,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12384,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121238&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12384,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247183&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12385,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12386,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145815&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12386,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247303&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12387,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12388,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107697&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12389,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262830&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12390,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12391,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2922&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12391,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184588&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12392,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88793&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12393,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=239977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12394,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50451&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12395,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202466&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12395,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4241&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12396,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12397,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=14660&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12397,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12397,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12397,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4365&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12397,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21020&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12398,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74481&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12399,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3295&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12400,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153985&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12401,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193441&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12402,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12403,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8816&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12404,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3334&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12405,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12406,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12407,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31789&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12408,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12408,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12409,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126408&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12410,(SELECT setid FROM sets WHERE sets.name = 'DDC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12410,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12411,(SELECT setid FROM sets WHERE sets.name = 'DDG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=243473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12411,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19770&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12412,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12413,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12413,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12414,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12415,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79256&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12416,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12417,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22959&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12418,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=154350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12419,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12420,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3895&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12420,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12420,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184605&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12421,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12422,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12423,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83648&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12424,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12425,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=170977&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12426,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12427,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3350&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12428,(SELECT setid FROM sets WHERE sets.name = 'EVG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157936&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12428,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40194&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12429,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12430,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12430,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205091&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12430,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=237007&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12431,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262661&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12432,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3443&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12433,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19621&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12434,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1717&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12435,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2881&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12435,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12435,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201231&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12436,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12436,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12437,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121270&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12438,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39900&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12439,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12440,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2581&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12441,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74333&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12442,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190198&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12442,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12443,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29292&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12444,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12445,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12446,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29935&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12447,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10729&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12448,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=72928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12448,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12449,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12449,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5817&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12450,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=990&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12450,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202429&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12451,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12452,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3347&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12453,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202545&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12453,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6519&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12454,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12454,(SELECT setid FROM sets WHERE sets.name = 'TSB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107282&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12454,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15219&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12455,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214055&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12456,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12456,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2692&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12457,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96892&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12458,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12458,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12459,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12460,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=151068&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12461,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9787&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12462,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12463,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1476&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12464,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8905&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12464,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12464,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12465,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73940&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12466,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3230&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12466,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184584&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12467,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12468,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29991&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12469,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174888&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12470,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12471,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5562&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12472,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205343&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12472,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197526&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12473,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12474,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12475,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=145986&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12476,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=50342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12477,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=150833&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12477,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205344&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12478,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12479,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12479,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12480,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96864&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12481,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12482,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12483,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12483,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202510&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12484,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45315&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12484,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83079&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12484,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30558&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12485,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233072&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12486,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12487,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=113614&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12488,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=41153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12489,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226878&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12490,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12491,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205041&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12492,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218010&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12493,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12493,(SELECT setid FROM sets WHERE sets.name = 'CH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12493,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84579&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12494,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201561&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12495,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12496,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30686&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12496,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12497,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1070&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12497,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1362&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12498,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16446&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45372&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4052&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2278&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=199&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4340&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1293&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12499,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=796&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12500,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12501,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12501,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12502,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220082&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12503,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124342&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12504,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=249387&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12504,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97236&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12505,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45287&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12505,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21310&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12505,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12505,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15883&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12505,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129743&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12506,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=180158&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12506,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226715&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12507,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12508,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25629&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12508,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5617&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12509,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4381&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12509,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12510,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220130&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12511,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10534&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12512,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262669&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12513,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12345&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12514,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12515,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44772&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12516,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1516&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12517,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=227064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12518,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222706&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12518,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20390&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12518,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129798&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12519,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12520,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198084&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12521,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10609&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12522,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=253431&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12523,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10598&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12524,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121260&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12525,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191548&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12526,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83764&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12527,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34229&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12528,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12529,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136206&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12530,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3530&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12531,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12531,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45494&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12531,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12531,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205406&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12532,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12532,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12533,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=9761&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12534,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220086&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12535,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12536,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2695&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12537,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1889&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12537,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3942&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12537,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159845&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12538,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=190164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12538,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=208222&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12538,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=230757&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12538,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12539,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12539,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12539,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3654&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12540,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45415&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12540,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222734&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12541,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247146&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12541,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12541,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12542,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=110501&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12543,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189214&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12543,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=140164&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12544,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6075&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12544,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25568&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12545,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39467&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12546,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12547,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=28805&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12548,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12549,(SELECT setid FROM sets WHERE sets.name = 'GP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=96928&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12550,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4239&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12551,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152720&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12552,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26631&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12552,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205221&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12552,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12553,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2009&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12553,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4184&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12554,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4484&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12555,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15769&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12556,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12556,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152887&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12557,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=30014&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12558,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=179578&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12558,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220520&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12559,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4650&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12560,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220277&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12561,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193544&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12562,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12563,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121248&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12564,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12565,(SELECT setid FROM sets WHERE sets.name = 'EX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12566,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26801&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12567,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4563&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12568,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1828&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=11423&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4074&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2297&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=508&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1309&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12569,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=810&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12570,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12570,(SELECT setid FROM sets WHERE sets.name = 'SH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5108&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12571,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193766&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12572,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4389&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12572,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12572,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12572,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5684&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12573,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22028&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12574,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29947&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12575,(SELECT setid FROM sets WHERE sets.name = 'NE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=22299&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12576,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87994&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12577,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12578,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=217998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12579,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3737&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12580,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12581,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79195&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12582,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152549&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12583,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3426&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12584,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12585,(SELECT setid FROM sets WHERE sets.name = 'PY'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24626&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12586,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153439&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12587,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12588,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23001&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12589,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12590,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12591,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1851&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12591,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159180&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12592,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1679&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12592,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201224&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12593,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=213749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12594,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=44307&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12595,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25836&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12596,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=185060&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12597,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39442&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12597,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134749&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12598,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15138&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12599,(SELECT setid FROM sets WHERE sets.name = 'MBS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12600,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19767&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12601,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25822&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12602,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40542&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12603,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=244726&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12604,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4496&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12605,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74671&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12606,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25930&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12607,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=40129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12608,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6550&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12608,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12609,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=240093&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12610,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80414&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12611,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=134740&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12612,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87964&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12613,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3752&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12614,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12615,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126294&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12616,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4430&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12617,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247388&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12617,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=95538&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12618,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189885&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12619,(SELECT setid FROM sets WHERE sets.name = 'AE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220543&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12619,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12619,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247193&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12620,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238135&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12621,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121189&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12622,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209000&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12623,(SELECT setid FROM sets WHERE sets.name = 'H09'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=207915&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12623,(SELECT setid FROM sets WHERE sets.name = 'SC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45166&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12624,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220242&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12625,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193399&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12626,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12627,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107322&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12628,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=186321&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12629,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4313&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12630,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12631,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130327&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12632,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87975&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12633,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=31823&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12634,(SELECT setid FROM sets WHERE sets.name = 'WWK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197871&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12635,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12636,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39879&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12637,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222736&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12637,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45173&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12637,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83107&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12637,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39712&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12637,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193557&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12638,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177924&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12639,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198627&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12640,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29968&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12641,(SELECT setid FROM sets WHERE sets.name = 'CFX'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=182608&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12641,(SELECT setid FROM sets WHERE sets.name = 'DDF'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12642,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175006&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12643,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126151&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12644,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12645,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19681&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12646,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=111076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12647,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175012&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12648,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247523&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12648,(SELECT setid FROM sets WHERE sets.name = 'PR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=97044&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12648,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15765&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12649,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46004&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12650,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34809&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12651,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19716&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12651,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=106473&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12652,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247393&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12652,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193507&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12653,(SELECT setid FROM sets WHERE sets.name = 'LE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=43724&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12654,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74319&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12655,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1019&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12655,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1133&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12656,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1500&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12657,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73981&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12658,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23081&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12658,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130997&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12659,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175109&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12660,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1834&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12661,(SELECT setid FROM sets WHERE sets.name = 'UD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13821&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12662,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12662,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2061&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12662,(SELECT setid FROM sets WHERE sets.name = 'V10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=212637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12662,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=37&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12662,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=332&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12662,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159266&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12662,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1128&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12662,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=634&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12663,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=47784&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12663,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83064&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12663,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15780&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12663,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21021&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12663,(SELECT setid FROM sets WHERE sets.name = 'ST2K'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=25455&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12664,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=15445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12664,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12664,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3353&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12664,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=84565&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12665,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=126011&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12666,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=141857&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12667,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=198076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12668,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4076&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12668,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2642&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12668,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184653&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12669,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146047&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12670,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109673&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12671,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5812&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12672,(SELECT setid FROM sets WHERE sets.name = 'VG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4960&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12673,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3846&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12673,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2105&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12673,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=65&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12673,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=360&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12673,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1162&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12673,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=662&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12674,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153966&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12675,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=233218&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12676,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83384&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12676,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205272&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12676,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270450&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12676,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4797&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12676,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129788&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12677,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129913&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12677,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=34378&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12678,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221518&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12678,(SELECT setid FROM sets WHERE sets.name = 'P2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=6637&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12679,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=46143&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12680,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4910&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12681,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221215&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12682,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153488&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12683,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26657&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12683,(SELECT setid FROM sets WHERE sets.name = '6E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=16433&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12683,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=132073&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12683,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4486&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12684,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51129&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12685,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=79098&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12686,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=152904&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12687,(SELECT setid FROM sets WHERE sets.name = 'PD3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=270449&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12687,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83754&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12688,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83471&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12688,(SELECT setid FROM sets WHERE sets.name = 'ON'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=39709&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12689,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=51177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12689,(SELECT setid FROM sets WHERE sets.name = 'PD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=234719&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12690,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4110&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12690,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12690,(SELECT setid FROM sets WHERE sets.name = 'DK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1803&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12691,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4445&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12692,(SELECT setid FROM sets WHERE sets.name = 'OD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=29807&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12693,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3663&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12694,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175243&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12695,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36454&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12696,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130325&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12697,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10425&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12698,(SELECT setid FROM sets WHERE sets.name = 'WL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4566&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12699,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4668&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12699,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129574&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12700,(SELECT setid FROM sets WHERE sets.name = 'DD2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=189234&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12700,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247368&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12700,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205392&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12700,(SELECT setid FROM sets WHERE sets.name = 'PS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=26262&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12701,(SELECT setid FROM sets WHERE sets.name = 'FD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73552&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12702,(SELECT setid FROM sets WHERE sets.name = 'AVR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262703&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12703,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=128949&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12704,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121126&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12705,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4903&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12706,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121250&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12707,(SELECT setid FROM sets WHERE sets.name = 'DS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=49049&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12707,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12707,(SELECT setid FROM sets WHERE sets.name = 'M11'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=206328&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12707,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=221521&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12707,(SELECT setid FROM sets WHERE sets.name = '9E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=83493&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12707,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=129619&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12708,(SELECT setid FROM sets WHERE sets.name = 'BR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=21175&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12708,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=209113&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12708,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45416&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12708,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=23114&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12708,(SELECT setid FROM sets WHERE sets.name = 'LRW'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=139502&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12708,(SELECT setid FROM sets WHERE sets.name = 'PCH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=205394&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12708,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5655&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12709,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3083&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12709,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184714&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12710,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12711,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=998&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12711,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2026&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12712,(SELECT setid FROM sets WHERE sets.name = 'ST'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=20196&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12713,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3832&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12713,(SELECT setid FROM sets WHERE sets.name = 'HL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2912&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12713,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184589&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12714,(SELECT setid FROM sets WHERE sets.name = 'M10'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=191324&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12714,(SELECT setid FROM sets WHERE sets.name = 'MOR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157422&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12715,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27246&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12715,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3897&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12715,(SELECT setid FROM sets WHERE sets.name = 'IA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2497&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12715,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247331&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12715,(SELECT setid FROM sets WHERE sets.name = 'ME2'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=184606&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12715,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19571&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12716,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=218058&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12717,(SELECT setid FROM sets WHERE sets.name = 'AN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=908&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12717,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1117&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12718,(SELECT setid FROM sets WHERE sets.name = '8E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=45216&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12718,(SELECT setid FROM sets WHERE sets.name = 'PT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4375&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12718,(SELECT setid FROM sets WHERE sets.name = '7E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=13153&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12718,(SELECT setid FROM sets WHERE sets.name = 'UL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=12366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12719,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121121&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12720,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=222906&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12721,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247417&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12721,(SELECT setid FROM sets WHERE sets.name = 'TE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4718&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12722,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=178137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12723,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=174837&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12724,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87941&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12725,(SELECT setid FROM sets WHERE sets.name = 'DDE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=210137&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12725,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=8876&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12726,(SELECT setid FROM sets WHERE sets.name = 'DIS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=107395&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12727,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12728,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=183017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12729,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=75305&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12730,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=73944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12731,(SELECT setid FROM sets WHERE sets.name = 'VI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12732,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=109756&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12733,(SELECT setid FROM sets WHERE sets.name = 'ISD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=226880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12734,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3336&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12735,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19659&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12736,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=245289&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12737,(SELECT setid FROM sets WHERE sets.name = 'TSP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=118880&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12738,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=157207&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12739,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=146747&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12740,(SELECT setid FROM sets WHERE sets.name = 'AP'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27136&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12741,(SELECT setid FROM sets WHERE sets.name = 'AQ'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1022&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12741,(SELECT setid FROM sets WHERE sets.name = 'MED'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=159119&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12742,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12742,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=175112&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12743,(SELECT setid FROM sets WHERE sets.name = 'ARB'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=188971&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12743,(SELECT setid FROM sets WHERE sets.name = 'DDH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=259276&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12744,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74286&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12745,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=238145&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12746,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5735&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12747,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247380&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12747,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=87939&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12748,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3139&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12749,(SELECT setid FROM sets WHERE sets.name = 'CS'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=121265&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12749,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=135258&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12750,(SELECT setid FROM sets WHERE sets.name = 'EVE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=153487&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12751,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12751,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2273&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12751,(SELECT setid FROM sets WHERE sets.name = 'LG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1570&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12752,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=194326&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12753,(SELECT setid FROM sets WHERE sets.name = 'FUT'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=136045&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12754,(SELECT setid FROM sets WHERE sets.name = 'SHM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=158688&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12755,(SELECT setid FROM sets WHERE sets.name = 'ME3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=201290&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12755,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10623&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12756,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247366&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12756,(SELECT setid FROM sets WHERE sets.name = 'PLC'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=124057&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12757,(SELECT setid FROM sets WHERE sets.name = 'UNH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74267&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12758,(SELECT setid FROM sets WHERE sets.name = 'SOM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220364&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12759,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19776&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12760,(SELECT setid FROM sets WHERE sets.name = 'M12'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=220131&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12761,(SELECT setid FROM sets WHERE sets.name = 'ALA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=176452&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12762,(SELECT setid FROM sets WHERE sets.name = 'P3'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=10540&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12763,(SELECT setid FROM sets WHERE sets.name = 'SOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74046&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12764,(SELECT setid FROM sets WHERE sets.name = 'COM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=247527&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12764,(SELECT setid FROM sets WHERE sets.name = 'US'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5852&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12765,(SELECT setid FROM sets WHERE sets.name = 'BOK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=74587&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12766,(SELECT setid FROM sets WHERE sets.name = 'RAV'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=88944&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12767,(SELECT setid FROM sets WHERE sets.name = 'ZEN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=197529&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12768,(SELECT setid FROM sets WHERE sets.name = 'IN'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=24127&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12768,(SELECT setid FROM sets WHERE sets.name = '10E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=130525&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12769,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3202&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12770,(SELECT setid FROM sets WHERE sets.name = 'NPH'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=214352&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12771,(SELECT setid FROM sets WHERE sets.name = 'UG'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=5725&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12772,(SELECT setid FROM sets WHERE sets.name = 'AL'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3228&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12773,(SELECT setid FROM sets WHERE sets.name = 'JU'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=36037&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12774,(SELECT setid FROM sets WHERE sets.name = 'FE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1932&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12775,(SELECT setid FROM sets WHERE sets.name = 'MR'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=48586&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12776,(SELECT setid FROM sets WHERE sets.name = 'CHK'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=80251&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12777,(SELECT setid FROM sets WHERE sets.name = '5E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=4017&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12777,(SELECT setid FROM sets WHERE sets.name = '4E'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=2245&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12777,(SELECT setid FROM sets WHERE sets.name = 'A'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=177&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12777,(SELECT setid FROM sets WHERE sets.name = 'B'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=472&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12777,(SELECT setid FROM sets WHERE sets.name = 'ME4'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=202457&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12777,(SELECT setid FROM sets WHERE sets.name = 'R'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=1271&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12777,(SELECT setid FROM sets WHERE sets.name = 'U'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=774&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12778,(SELECT setid FROM sets WHERE sets.name = 'BD'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=27249&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12778,(SELECT setid FROM sets WHERE sets.name = 'MI'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=3379&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12779,(SELECT setid FROM sets WHERE sets.name = 'ROE'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=193575&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12780,(SELECT setid FROM sets WHERE sets.name = 'TO'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=32921&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12781,(SELECT setid FROM sets WHERE sets.name = 'DKA'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=262702&type=card");
INSERT INTO setlist(cardid,setid,lqurl) VALUES(12782,(SELECT setid FROM sets WHERE sets.name = 'MM'),"http://gatherer.wizards.com/Handlers/Image.ashx?multiverseid=19618&type=card");

COMMIT TRANSACTION;
